module app.view {

    import TabMenuItemBuilder = api.ui.tab.TabMenuItemBuilder;
    import DetailsPanel = app.view.detail.DetailsPanel;
    import ViewItem = api.app.view.ViewItem;
    import ContentSummary = api.content.ContentSummary;
    import StringHelper = api.util.StringHelper;
    import ResponsiveManager = api.ui.responsive.ResponsiveManager;
    import ResponsiveItem = api.ui.responsive.ResponsiveItem;
    import MobileContentTreeGridActions = app.browse.action.MobileContentTreeGridActions;
    import MobileContentBrowseToolbar = app.browse.MobileContentBrowseToolbar;

    export class MobileContentItemStatisticsPanel extends api.app.view.ItemStatisticsPanel<api.content.ContentSummary> {

        private itemHeader: api.dom.DivEl = new api.dom.DivEl("mobile-content-item-statistics-header");
        private headerLabel: api.dom.SpanEl = new api.dom.SpanEl();

        private previewPanel: ContentItemPreviewPanel;
        private detailsPanel: DetailsPanel = DetailsPanel.create().setUseSplitter(false).setUseNameAndIconView(false).setSlideFrom(app.view.detail.SLIDE_FROM.BOTTOM).build();
        private detailsToggleButton: app.view.detail.MobileDetailsPanelToggleButton;

        private mobileBrowseActions: MobileContentTreeGridActions;
        private toolbar: MobileContentBrowseToolbar;

        constructor(mobileBrowseActions: MobileContentTreeGridActions) {
            super("mobile-content-item-statistics-panel");

            this.mobileBrowseActions = mobileBrowseActions;

            this.setDoOffset(false);

            this.initHeader();

            this.initPreviewPanel();

            this.initDetailsPanel();

            this.initToolbar();

            this.onRendered(() => {
                this.slideAllOut();
            });

            ResponsiveManager.onAvailableSizeChanged(this, (item: ResponsiveItem) => {
                this.slideAllOut();
            });
        }

        private initToolbar() {
            this.toolbar = new MobileContentBrowseToolbar(this.mobileBrowseActions);
            this.appendChild(this.toolbar);
        }

        private initHeader() {
            this.itemHeader.appendChild(this.headerLabel);
            this.detailsToggleButton = new app.view.detail.MobileDetailsPanelToggleButton(this.detailsPanel);
            var backButton = new api.dom.DivEl("back-button");
            backButton.onClicked((event) => {
                this.slideAllOut();
            });
            this.itemHeader.appendChild(backButton);
            this.itemHeader.appendChild(this.detailsToggleButton);

            this.appendChild(this.itemHeader);

        }

        private initDetailsPanel() {
            this.appendChild(this.detailsPanel);
        }

        private initPreviewPanel() {
            this.previewPanel = new ContentItemPreviewPanel();
            this.previewPanel.setDoOffset(false);
            this.appendChild(this.previewPanel);
        }

        setItem(item: ViewItem<ContentSummary>) {
            if (!this.getItem() || !this.getItem().equals(item)) {
                super.setItem(item);
                this.previewPanel.setItem(item);
                this.detailsPanel.setItem(item);
                if (item) {
                    this.setName(this.makeDisplayName(item));
                }
            }
            this.slideIn();
        }

        private makeDisplayName(item: ViewItem<ContentSummary>): string {
            return StringHelper.isEmpty(item.getDisplayName()) ? StringHelper.escapeHtml("<Unnamed " +
                                                                                         this.convertName(item.getModel().getType().getLocalName() +
                                                                                         "") + ">") : item.getDisplayName();
        }

        private convertName(name: string): string {
            return StringHelper.capitalize(name.replace(/-/g, " ").trim());
        }

        getDetailsPanel(): DetailsPanel {
            return this.detailsPanel;
        }

        setName(name: string) {
            this.headerLabel.setHtml(name);
        }

        slideAllOut() {
            this.slideOut();
            this.detailsPanel.slideOut();
            this.detailsToggleButton.removeClass("expanded");
        }


        slideOut() {
            this.getEl().setRightPx(-this.getEl().getWidthWithBorder());
        }

        slideIn() {
            this.getEl().setRightPx(0);
        }
    }

}