var api;
(function (api) {
    var Class = (function () {
        function Class(name, fn) {
            this.name = name;
            this.fn = fn;
        }
        Class.prototype.getName = function () {
            return this.name;
        };
        Class.prototype.newInstance = function (constructorParams) {
            var newInstance = Object.create(this.fn.prototype);
            newInstance.constructor.call(newInstance, constructorParams);
            return newInstance;
        };
        return Class;
    })();
    api.Class = Class;
})(api || (api = {}));
var api;
(function (api) {
    var ObjectHelper = (function () {
        function ObjectHelper() {
        }
        ObjectHelper.create = function (constructor) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            var factory = constructor.bind.apply(constructor, arguments);
            return new factory();
        };
        ObjectHelper.iFrameSafeInstanceOf = function (obj, fn) {
            if (!fn) {
                console.warn('Undefined fn passed to iFrameSafeInstanceOf, returning false', obj, fn);
                return false;
            }
            if (!obj) {
                return false;
            }
            if (obj instanceof fn) {
                return true;
            }
            if (obj.constructor.name && obj.constructor.name === fn.name) {
                return true;
            }
            else if (Object.prototype.toString.call(obj) === fn.name) {
                return true;
            }
            if (!(typeof obj === 'object')) {
                return false;
            }
            var prototype = Object.getPrototypeOf(obj);
            if (prototype.constructor.name) {
                do {
                    prototype = Object.getPrototypeOf(prototype);
                    if (!prototype) {
                        return false;
                    }
                } while (prototype.constructor.name !== fn.name);
            }
            else {
                do {
                    prototype = Object.getPrototypeOf(prototype);
                    if (!prototype) {
                        return false;
                    }
                } while (Object.prototype.toString.call(prototype) !== fn.name);
            }
            return true;
        };
        ObjectHelper.equals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            else if (!a && b) {
                return false;
            }
            else if (a && !b) {
                return false;
            }
            return a.equals(b);
        };
        ObjectHelper.arrayEquals = function (arrayA, arrayB) {
            if (!arrayA && !arrayB) {
                return true;
            }
            else if (!arrayA && arrayB) {
                return false;
            }
            else if (arrayA && !arrayB) {
                return false;
            }
            if (arrayA.length != arrayB.length) {
                return false;
            }
            for (var i = 0; i < arrayA.length; i++) {
                if (!ObjectHelper.equals(arrayA[i], arrayB[i])) {
                    return false;
                }
            }
            return true;
        };
        ObjectHelper.anyArrayEquals = function (arrayA, arrayB) {
            if (!arrayA && !arrayB) {
                return true;
            }
            else if (!arrayA && arrayB) {
                return false;
            }
            else if (arrayA && !arrayB) {
                return false;
            }
            if (arrayA.length != arrayB.length) {
                return false;
            }
            for (var i = 0; i < arrayA.length; i++) {
                if (!ObjectHelper.objectEquals(arrayA[i], arrayB[i])) {
                    return false;
                }
            }
            return true;
        };
        ObjectHelper.mapEquals = function (mapA, mapB) {
            if (!mapA && !mapB) {
                return true;
            }
            else if (!mapA && mapB) {
                return false;
            }
            else if (mapA && !mapB) {
                return false;
            }
            var keysA = [];
            for (var keyA in mapA) {
                if (mapA.hasOwnProperty(keyA)) {
                    keysA.push(keyA);
                }
            }
            var keysB = [];
            for (var keyB in mapB) {
                if (mapB.hasOwnProperty(keyB)) {
                    keysB.push(keyB);
                }
            }
            if (!ObjectHelper.stringArrayEquals(keysA, keysB)) {
                return false;
            }
            for (var keyA in keysA) {
                var valueA = mapA[keysA[keyA]];
                var valueB = mapB[keysA[keyA]];
                if (!ObjectHelper.equals(valueA, valueB)) {
                    return false;
                }
            }
            return true;
        };
        ObjectHelper.stringEquals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            else if (!a && b) {
                return false;
            }
            else if (a && !b) {
                return false;
            }
            return a.toString() === b.toString();
        };
        ObjectHelper.stringArrayEquals = function (arrayA, arrayB) {
            if (!arrayA && !arrayB) {
                return true;
            }
            else if (!arrayA && arrayB) {
                return false;
            }
            else if (arrayA && !arrayB) {
                return false;
            }
            if (arrayA.length != arrayB.length) {
                return false;
            }
            for (var i = 0; i < arrayA.length; i++) {
                if (!ObjectHelper.stringEquals(arrayA[i], arrayB[i])) {
                    return false;
                }
            }
            return true;
        };
        ObjectHelper.booleanEquals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            else if (!a && b) {
                return false;
            }
            else if (a && !b) {
                return false;
            }
            return a == b;
        };
        ObjectHelper.numberEquals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            else if (!a && b) {
                return false;
            }
            else if (a && !b) {
                return false;
            }
            return a == b;
        };
        ObjectHelper.dateEquals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            else if (!a && b) {
                return false;
            }
            else if (a && !b) {
                return false;
            }
            return a.toISOString() == b.toISOString();
        };
        ObjectHelper.anyEquals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            else if (!a && b) {
                return false;
            }
            else if (a && !b) {
                return false;
            }
            var aString = JSON.stringify(a);
            var bString = JSON.stringify(b);
            return aString == bString;
        };
        ObjectHelper.objectEquals = function (a, b) {
            if (!a && !b) {
                return true;
            }
            else if (!a && b) {
                return false;
            }
            else if (a && !b) {
                return false;
            }
            var aString = JSON.stringify(a, function (key, value) {
                return (!!key && a == value) ? undefined : value;
            });
            var bString = JSON.stringify(b, function (key, value) {
                return (!!key && b == value) ? undefined : value;
            });
            return aString == bString;
        };
        ObjectHelper.objectPropertyIterator = function (object, callback) {
            var index = 0;
            for (var name in object) {
                if (object.hasOwnProperty(name)) {
                    var property = object[name];
                    callback(name, property, index++);
                }
            }
        };
        return ObjectHelper;
    })();
    api.ObjectHelper = ObjectHelper;
})(api || (api = {}));
var api;
(function (api) {
    var ClassHelper = (function () {
        function ClassHelper() {
        }
        ClassHelper.getFunctionName = function (func) {
            if (func.name) {
                return func.name;
            }
            else {
                var funcNameRegex = /function (.+)\(/;
                var results = (funcNameRegex).exec(func.toString());
                return (results && results.length > 1) ? results[1] : "";
            }
        };
        ClassHelper.getClassName = function (instance) {
            return ClassHelper.getFunctionName(instance["constructor"]);
        };
        ClassHelper.getClass = function (instance) {
            return instance["constructor"];
        };
        ClassHelper.getModuleName = function (instance) {
            var fullName = ClassHelper.getFullName(instance);
            return fullName ? fullName.substr(0, fullName.lastIndexOf(".")) : "";
        };
        ClassHelper.getFullName = function (instance) {
            var constructor = (typeof instance === 'function') ? instance : instance["constructor"];
            return ClassHelper.findPath(window, constructor) || "";
        };
        ClassHelper.findPath = function (obj, constructor, nestLevel) {
            var value, path, nestLevel = nestLevel || 1;
            if (nestLevel > ClassHelper.MAX_NEST_LEVEL) {
                return null;
            }
            for (var key in obj) {
                if (obj.hasOwnProperty(key)) {
                    if (nestLevel == 1 && ClassHelper.ALLOWED_PACKAGES.indexOf(key) < 0) {
                        continue;
                    }
                    value = obj[key];
                    if (!value || value == obj) {
                        continue;
                    }
                    if (typeof value === 'object') {
                        path = ClassHelper.findPath(value, constructor, nestLevel + 1);
                        if (path) {
                            return key + "." + path;
                        }
                    }
                    else if (typeof value === 'function') {
                        if (value == constructor) {
                            return ClassHelper.getFunctionName(constructor);
                        }
                    }
                }
            }
            return path;
        };
        ClassHelper.distanceTo = function (instance, clazz) {
            if (instance.constructor.name === clazz.name) {
                return 0;
            }
            var distance = 0;
            var prototype = Object.getPrototypeOf(instance);
            do {
                prototype = Object.getPrototypeOf(prototype);
                if (!prototype) {
                    return distance;
                }
                distance++;
            } while (prototype.constructor.name !== clazz.name);
            return distance;
        };
        ClassHelper.MAX_NEST_LEVEL = 7;
        ClassHelper.ALLOWED_PACKAGES = ['api', 'app', 'LiveEdit'];
        return ClassHelper;
    })();
    api.ClassHelper = ClassHelper;
})(api || (api = {}));
var api;
(function (api) {
    var BrowserName;
    (function (BrowserName) {
        BrowserName[BrowserName["CHROME"] = 0] = "CHROME";
        BrowserName[BrowserName["FIREFOX"] = 1] = "FIREFOX";
        BrowserName[BrowserName["OPERA"] = 2] = "OPERA";
        BrowserName[BrowserName["SAFARI"] = 3] = "SAFARI";
        BrowserName[BrowserName["MSIE"] = 4] = "MSIE";
        BrowserName[BrowserName["TRIDENT"] = 5] = "TRIDENT";
    })(BrowserName || (BrowserName = {}));
    var BrowserHelper = (function () {
        function BrowserHelper() {
        }
        BrowserHelper.isAvailableBrowser = function () {
            if (!BrowserHelper.BROWSER_NAME) {
                this.init();
            }
            switch (BrowserHelper.BROWSER_NAME) {
                case 0 /* CHROME */:
                    if (BrowserHelper.BROWSER_VERSION >= BrowserHelper.AVAILABLE_VERSIONS[BrowserHelper.BROWSER_NAME]) {
                        return true;
                    }
                    break;
                case 1 /* FIREFOX */:
                    if (BrowserHelper.BROWSER_VERSION >= BrowserHelper.AVAILABLE_VERSIONS[BrowserHelper.BROWSER_NAME]) {
                        return true;
                    }
                    break;
            }
            return false;
        };
        BrowserHelper.isOldBrowser = function () {
            if (!BrowserHelper.BROWSER_NAME) {
                this.init();
            }
            if (BrowserHelper.AVAILABLE_VERSIONS[BrowserHelper.BROWSER_NAME] > BrowserHelper.BROWSER_VERSION) {
                return true;
            }
            return false;
        };
        BrowserHelper.init = function () {
            var M = navigator.userAgent.match(/(opera|chrome|safari|firefox|msie|trident(?=\/))\/?\s*(\d+)/i) || [];
            BrowserHelper.BROWSER_NAME = BrowserName[M[1].toLocaleUpperCase()];
            BrowserHelper.BROWSER_VERSION = M[2];
            BrowserHelper.AVAILABLE_VERSIONS[0 /* CHROME */] = "39";
            BrowserHelper.AVAILABLE_VERSIONS[1 /* FIREFOX */] = "27";
        };
        BrowserHelper.AVAILABLE_VERSIONS = {};
        return BrowserHelper;
    })();
    api.BrowserHelper = BrowserHelper;
})(api || (api = {}));
var api;
(function (api) {
    var Name = (function () {
        function Name(name) {
            api.util.assertNotNull(name, "Name cannot be null");
            api.util.assert(!api.util.StringHelper.isEmpty(name), "Name cannot be empty");
            this.value = name;
        }
        Name.prototype.getValue = function () {
            return this.value;
        };
        Name.prototype.toString = function () {
            return this.value;
        };
        Name.prototype.equals = function (o) {
            if (!(api.ObjectHelper.iFrameSafeInstanceOf(o, Name))) {
                return false;
            }
            var other = o;
            if (this.value != other.value) {
                return false;
            }
            return true;
        };
        Name.FORBIDDEN_CHARS = /[^a-z0-9\-]+/ig;
        Name.SIMPLIFIED_FORBIDDEN_CHARS = /[\/!?\\]/g;
        return Name;
    })();
    api.Name = Name;
})(api || (api = {}));
var api;
(function (api) {
    (function (ExceptionType) {
        ExceptionType[ExceptionType["INFO"] = 0] = "INFO";
        ExceptionType[ExceptionType["ERROR"] = 1] = "ERROR";
        ExceptionType[ExceptionType["WARNING"] = 2] = "WARNING";
    })(api.ExceptionType || (api.ExceptionType = {}));
    var ExceptionType = api.ExceptionType;
    var Exception = (function () {
        function Exception(message, type) {
            if (type === void 0) { type = 1 /* ERROR */; }
            this.message = message;
            this.type = type;
        }
        Exception.prototype.getMessage = function () {
            return this.message;
        };
        Exception.prototype.getType = function () {
            return this.type;
        };
        return Exception;
    })();
    api.Exception = Exception;
})(api || (api = {}));
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var api;
(function (api) {
    var AccessDeniedException = (function (_super) {
        __extends(AccessDeniedException, _super);
        function AccessDeniedException(message) {
            _super.call(this, message, 2 /* WARNING */);
        }
        return AccessDeniedException;
    })(api.Exception);
    api.AccessDeniedException = AccessDeniedException;
})(api || (api = {}));
var api;
(function (api) {
    var DefaultErrorHandler = (function () {
        function DefaultErrorHandler() {
        }
        DefaultErrorHandler.handle = function (error) {
            if (api.ObjectHelper.iFrameSafeInstanceOf(error, Error)) {
                console.error(error);
                throw error;
            }
            else if (api.ObjectHelper.iFrameSafeInstanceOf(error, api.AccessDeniedException)) {
                var application = api.app.Application.getApplication();
                var wnd = application.getWindow();
                new api.app.ShowAppLauncherEvent(application, true).fire(wnd.parent);
                new api.app.ShowAppLauncherEvent(application, true).fire(wnd);
            }
            else if (api.ObjectHelper.iFrameSafeInstanceOf(error, api.Exception)) {
                var message = error.getMessage();
                switch (error.getType()) {
                    case 1 /* ERROR */:
                        console.error(message);
                        api.notify.showError(message);
                        break;
                    case 2 /* WARNING */:
                        console.warn(message);
                        api.notify.showWarning(message);
                        break;
                    case 0 /* INFO */:
                        console.info(message);
                        api.notify.showFeedback(message);
                        break;
                }
            }
            else {
                console.error(error);
                api.notify.showError(error.toString());
                throw error;
            }
        };
        return DefaultErrorHandler;
    })();
    api.DefaultErrorHandler = DefaultErrorHandler;
})(api || (api = {}));
var api;
(function (api) {
    var PropertyChangedEvent = (function () {
        function PropertyChangedEvent(propertyName, oldValue, newValue, source) {
            this.propertyName = propertyName;
            this.oldValue = oldValue;
            this.newValue = newValue;
            this.source = source;
        }
        PropertyChangedEvent.prototype.getPropertyName = function () {
            return this.propertyName;
        };
        PropertyChangedEvent.prototype.getOldValue = function () {
            return this.oldValue;
        };
        PropertyChangedEvent.prototype.getNewValue = function () {
            return this.newValue;
        };
        PropertyChangedEvent.prototype.getSource = function () {
            return this.source;
        };
        return PropertyChangedEvent;
    })();
    api.PropertyChangedEvent = PropertyChangedEvent;
})(api || (api = {}));
var api;
(function (api) {
    var ValidityChangedEvent = (function () {
        function ValidityChangedEvent(valid) {
            this.valid = valid;
        }
        ValidityChangedEvent.prototype.isValid = function () {
            return this.valid;
        };
        return ValidityChangedEvent;
    })();
    api.ValidityChangedEvent = ValidityChangedEvent;
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        function assert(expression, message) {
            if (!expression) {
                console.error(message);
                throw new Error(message || 'Assertion failed');
            }
        }
        util.assert = assert;
        function assertState(expression, message) {
            assert(expression, 'Illegal state: ' + message);
        }
        util.assertState = assertState;
        function assertNotNull(value, message) {
            assert(value != null, message || 'Value may not be null');
            return value;
        }
        util.assertNotNull = assertNotNull;
        function assertNull(value, message) {
            assert(value == null, message);
        }
        util.assertNull = assertNull;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var AppHelper = (function () {
            function AppHelper() {
            }
            AppHelper.debounce = function (func, wait, immediate) {
                var timeout;
                return function () {
                    var context = this, args = arguments;
                    var later = function () {
                        timeout = null;
                        if (!immediate) {
                            func.apply(context, args);
                        }
                    };
                    var callNow = immediate && !timeout;
                    clearTimeout(timeout);
                    timeout = setTimeout(later, wait);
                    if (callNow) {
                        func.apply(context, args);
                    }
                };
            };
            AppHelper.preventDragRedirect = function (message, element) {
                if (message === void 0) { message = ""; }
                element = element || api.dom.Body.get();
                var window = api.dom.WindowDOM.get();
                var timeout = null;
                var beforeUnloadHandler = function (event) {
                    (event || window.asWindow().event)['returnValue'] = message;
                    event.preventDefault();
                    return message;
                };
                var unBeforeUnload = function () {
                    timeout = null;
                    window.unBeforeUnload(beforeUnloadHandler);
                };
                element.onDragOver(function () {
                    if (!timeout) {
                        window.onBeforeUnload(beforeUnloadHandler);
                    }
                    clearTimeout(timeout);
                    timeout = setTimeout(unBeforeUnload, 100);
                });
            };
            return AppHelper;
        })();
        util.AppHelper = AppHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var ArrayHelper = (function () {
            function ArrayHelper() {
            }
            ArrayHelper.moveElement = function (oldIndex, newIndex, array) {
                var element = array.splice(oldIndex, 1)[0];
                if (newIndex >= array.length) {
                    array[newIndex] = element;
                }
                else {
                    array.splice(newIndex, 0, element);
                }
            };
            ArrayHelper.addUnique = function (value, array) {
                if (array.indexOf(value) === -1) {
                    array.push(value);
                }
            };
            ArrayHelper.removeValue = function (value, array) {
                var index = array.indexOf(value);
                if (index > -1) {
                    array.splice(index, 1);
                }
            };
            ArrayHelper.removeDuplicates = function (array, keyFunction) {
                var seen = {};
                return array.filter(function (item) {
                    var key = keyFunction(item);
                    if (!seen.hasOwnProperty(key)) {
                        seen[key] = true;
                        return true;
                    }
                    return false;
                });
            };
            return ArrayHelper;
        })();
        util.ArrayHelper = ArrayHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var BasePath = (function () {
            function BasePath(elements, elementDivider, absolute) {
                this.elementDivider = elementDivider != null ? elementDivider : BasePath.DEFAULT_ELEMENT_DIVIDER;
                this.absolute = absolute == undefined ? true : absolute;
                elements.forEach(function (element, index) {
                    if (element == null) {
                        throw new Error("Path element was null at index: " + index);
                    }
                    else if (element.length == 0) {
                        throw new Error("Path element was empty string at index: " + index);
                    }
                });
                this.elements = elements;
                this.refString = (this.absolute ? this.elementDivider : "") + this.elements.join(this.elementDivider);
            }
            BasePath.prototype.isAbsolute = function () {
                return this.absolute;
            };
            BasePath.prototype.getElements = function () {
                return this.elements;
            };
            BasePath.prototype.getElement = function (index) {
                return this.elements[index];
            };
            BasePath.prototype.hasParent = function () {
                return this.elements.length > 0;
            };
            BasePath.prototype.getParentPath = function () {
                var _this = this;
                if (this.elements.length < 1) {
                    return null;
                }
                var parentElemements = [];
                this.elements.forEach(function (element, index) {
                    if (index < _this.elements.length - 1) {
                        parentElemements.push(element);
                    }
                });
                return this.newInstance(parentElemements, this.absolute);
            };
            BasePath.prototype.newInstance = function (elements, absolute) {
                throw new Error("Must be implemented by inheritor");
            };
            BasePath.prototype.toString = function () {
                return this.refString;
            };
            BasePath.removeEmptyElements = function (elements) {
                var filteredElements = [];
                elements.forEach(function (element) {
                    if (element.length > 0) {
                        filteredElements.push(element);
                    }
                });
                return filteredElements;
            };
            BasePath.DEFAULT_ELEMENT_DIVIDER = "/";
            return BasePath;
        })();
        util.BasePath = BasePath;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var UriHelper = (function () {
            function UriHelper() {
            }
            UriHelper.getUri = function (path) {
                var basePath = window['CONFIG'] && window['CONFIG']['baseUri'] || UriHelper.DEFAULT_URI;
                return UriHelper.joinPath(basePath, UriHelper.relativePath(path));
            };
            UriHelper.getAdminUri = function (path) {
                return UriHelper.getUri(UriHelper.joinPath('admin', UriHelper.relativePath(path)));
            };
            UriHelper.getRestUri = function (path) {
                return UriHelper.getAdminUri(UriHelper.joinPath('rest', UriHelper.relativePath(path)));
            };
            UriHelper.getPortalUri = function (path) {
                return UriHelper.getUri(UriHelper.joinPath('admin', '/portal', UriHelper.relativePath(path)));
            };
            UriHelper.relativePath = function (path) {
                if (util.StringHelper.isBlank(path)) {
                    return util.StringHelper.EMPTY_STRING;
                }
                return path.charAt(0) == '/' ? path.substring(1) : path;
            };
            UriHelper.joinPath = function () {
                var paths = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    paths[_i - 0] = arguments[_i];
                }
                return util.StringHelper.removeEmptyStrings(paths).join('/').replace(/(^|[^:])\/{2,}/g, '$1/');
            };
            UriHelper.getUrlLocation = function (url) {
                return util.StringHelper.isBlank(url) ? util.StringHelper.EMPTY_STRING : url.split(/\?|&/i)[0];
            };
            UriHelper.decodeUrlParams = function (url) {
                if (util.StringHelper.isBlank(url)) {
                    return {};
                }
                var array = url.split(/\?|&/i);
                var params = {};
                var param;
                if (array.length > 1) {
                    for (var i = 1; i < array.length; i++) {
                        param = array[i].split('=');
                        params[param[0]] = param.length > 1 ? decodeURIComponent(param[1]) : undefined;
                    }
                }
                return params;
            };
            UriHelper.encodeUrlParams = function (params, prefix) {
                if (!params) {
                    return util.StringHelper.EMPTY_STRING;
                }
                var urlArray = [];
                for (var key in params) {
                    if (params.hasOwnProperty(key) && params[key] != undefined) {
                        var value = params[key];
                        var prefixedKey = prefix ? prefix + "[" + key + "]" : key;
                        if (typeof value == "object") {
                            urlArray.push(this.encodeUrlParams(value, prefixedKey));
                        }
                        else {
                            urlArray.push(encodeURIComponent(prefixedKey) + "=" + encodeURIComponent(value));
                        }
                    }
                }
                return urlArray.join("&");
            };
            UriHelper.DEFAULT_URI = '/';
            return UriHelper;
        })();
        util.UriHelper = UriHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var StringHelper = (function () {
            function StringHelper() {
            }
            StringHelper.limit = function (str, length, ending) {
                if (ending === void 0) { ending = "\u2026"; }
                return StringHelper.isEmpty(str) ? StringHelper.EMPTY_STRING : str.substring(0, length) + ending;
            };
            StringHelper.capitalize = function (str) {
                return StringHelper.isEmpty(str) ? StringHelper.EMPTY_STRING : str.charAt(0).toUpperCase() + str.slice(1);
            };
            StringHelper.capitalizeAll = function (str) {
                return StringHelper.isEmpty(str) ? StringHelper.EMPTY_STRING : str.replace(/(?:^|\s)\S/g, function (ch) {
                    return ch.toUpperCase();
                });
            };
            StringHelper.escapeHtml = function (str) {
                return StringHelper.isEmpty(str) ? StringHelper.EMPTY_STRING : str.replace(/[&<>"'\/]/g, (function (char) {
                    return StringHelper.SAVE_CHAR_CODES[char];
                }));
            };
            StringHelper.isUpperCase = function (str) {
                if (StringHelper.isEmpty(str)) {
                    return false;
                }
                return str.toUpperCase() == str;
            };
            StringHelper.isLowerCase = function (str) {
                if (StringHelper.isEmpty(str)) {
                    return false;
                }
                return str.toLowerCase() == str;
            };
            StringHelper.isMixedCase = function (str) {
                if (StringHelper.isEmpty(str)) {
                    return false;
                }
                return !StringHelper.isLowerCase(str) && !StringHelper.isUpperCase(str);
            };
            StringHelper.isEmpty = function (str) {
                return !str || str.length == 0;
            };
            StringHelper.isBlank = function (str) {
                return StringHelper.isEmpty(str) || str.trim().length == 0;
            };
            StringHelper.removeCarriageChars = function (str) {
                return StringHelper.isEmpty(str) ? StringHelper.EMPTY_STRING : str.replace(/\r/g, "");
            };
            StringHelper.removeWhitespaces = function (str) {
                return StringHelper.isEmpty(str) ? StringHelper.EMPTY_STRING : str.replace(/\s/g, "");
            };
            StringHelper.removeEmptyStrings = function (elements) {
                return !elements ? [] : elements.filter(function (element) {
                    return !StringHelper.isEmpty(element);
                });
            };
            StringHelper.substringBetween = function (str, left, right) {
                if (StringHelper.isEmpty(str) || StringHelper.isEmpty(left) || StringHelper.isEmpty(right)) {
                    return StringHelper.EMPTY_STRING;
                }
                var start = str.indexOf(left);
                if (start !== -1) {
                    var end = str.indexOf(right, start + left.length);
                    if (end !== -1) {
                        return str.substring(start + left.length, end);
                    }
                }
                return StringHelper.EMPTY_STRING;
            };
            StringHelper.format = function (str) {
                var tokens = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    tokens[_i - 1] = arguments[_i];
                }
                return StringHelper.isEmpty(str) ? StringHelper.EMPTY_STRING : str.replace(/\{\{|\}\}|\{(\d+)\}/g, function (m, n) {
                    if (m == "{{") {
                        return "{";
                    }
                    if (m == "}}") {
                        return "}";
                    }
                    return tokens[n];
                });
            };
            StringHelper.EMPTY_STRING = "";
            StringHelper.SAVE_CHAR_CODES = {
                "&": "&amp;",
                "<": "&lt;",
                ">": "&gt;",
                '"': '&quot;',
                "'": '&#39;',
                "/": '&#x2F;'
            };
            return StringHelper;
        })();
        util.StringHelper = StringHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var Animation = (function () {
            function Animation(duration, interval) {
                if (interval === void 0) { interval = Animation.DEFAULT_INTERVAL; }
                this.running = false;
                this.duration = duration;
                this.interval = interval;
            }
            Animation.prototype.onStep = function (doStep) {
                this.doStep = doStep;
            };
            Animation.prototype.start = function () {
                var _this = this;
                var startTime = this.getCurrentTime();
                this.id = setInterval(function () {
                    var progress = Math.min((_this.getCurrentTime() - startTime) / _this.duration, 1);
                    if (_this.doStep) {
                        _this.doStep(progress);
                    }
                    if (progress == 1) {
                        _this.stop();
                    }
                }, this.interval);
                this.running = true;
            };
            Animation.prototype.stop = function () {
                clearInterval(this.id);
                this.running = false;
            };
            Animation.prototype.isRunning = function () {
                return this.running;
            };
            Animation.prototype.getCurrentTime = function () {
                return new Date().getTime();
            };
            Animation.DEFAULT_INTERVAL = 10;
            return Animation;
        })();
        util.Animation = Animation;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var CookieHelper = (function () {
            function CookieHelper() {
            }
            CookieHelper.setCookie = function (name, value, days) {
                if (days === void 0) { days = 1; }
                if (days) {
                    var date = new Date();
                    date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
                    var expires = '; expires=' + date.toUTCString();
                }
                else {
                    var expires = '';
                }
                document.cookie = CookieHelper.escape(name) + '=' + CookieHelper.escape(value) + expires + '; path=/';
            };
            CookieHelper.getCookie = function (name) {
                var nameEQ = CookieHelper.escape(name) + '=';
                var ca = document.cookie.split(';');
                for (var i = 0; i < ca.length; i++) {
                    var c = ca[i];
                    while (c.charAt(0) === ' ') {
                        c = c.substring(1, c.length);
                    }
                    if (c.indexOf(nameEQ) === 0) {
                        return CookieHelper.unescape(c.substring(nameEQ.length, c.length));
                    }
                }
                return null;
            };
            CookieHelper.removeCookie = function (name) {
                CookieHelper.setCookie(name, '', -1);
            };
            CookieHelper.escape = function (value) {
                return encodeURIComponent(value);
            };
            CookieHelper.unescape = function (value) {
                return decodeURIComponent(value);
            };
            return CookieHelper;
        })();
        util.CookieHelper = CookieHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var DateHelper = (function () {
            function DateHelper() {
            }
            DateHelper.isInvalidDate = function (value) {
                return isNaN(value.getTime());
            };
            DateHelper.getTZOffset = function () {
                return (new Date().getTimezoneOffset() / 60) * -1;
            };
            DateHelper.getTZOffsetFromDate = function (s) {
                return (s.getTimezoneOffset() / 60) * -1;
            };
            DateHelper.parseTimeToUTC = function (localTime) {
                var values = localTime.split(':');
                var date = new Date();
                var localHours = Number(values[0]);
                if (values.length == 3) {
                    date.setHours(localHours, Number(values[1]), Number(values[2]));
                }
                else {
                    date.setHours(localHours, Number(values[1]));
                }
                var hoursAsString = "" + date.getUTCHours();
                var minutesAsString = "" + date.getUTCMinutes();
                return hoursAsString + ":" + minutesAsString;
            };
            DateHelper.parseUTCDate = function (value) {
                var parsedYear = Number(value.substring(0, 4));
                var parsedMonth = Number(value.substring(5, 7));
                var parsedDayOfMonth = Number(value.substring(8, 10));
                return api.util.DateHelper.newUTCDate(parsedYear, parsedMonth - 1, parsedDayOfMonth);
            };
            DateHelper.parseUTCDateTime = function (value, ignoreLocalTimezone) {
                if (ignoreLocalTimezone === void 0) { ignoreLocalTimezone = false; }
                var parsedYear = Number(value.substring(0, 4));
                var parsedMonth = Number(value.substring(5, 7));
                var parsedDayOfMonth = Number(value.substring(8, 10));
                var parsedHours = Number(value.substring(11, 13));
                var parsedMinutes = Number(value.substring(14, 16));
                var parsedSeconds = Number(value.substring(17, 19));
                if (ignoreLocalTimezone)
                    return DateHelper.newDateTime(parsedYear, parsedMonth - 1, parsedDayOfMonth, parsedHours, parsedMinutes, parsedSeconds);
                else
                    return DateHelper.newUTCDateTime(parsedYear, parsedMonth - 1, parsedDayOfMonth, parsedHours, parsedMinutes, parsedSeconds);
            };
            DateHelper.newUTCDate = function (year, month, date) {
                return new Date(Date.UTC(year, month, date));
            };
            DateHelper.newUTCDateTime = function (year, month, date, hours, minutes, seconds) {
                if (seconds === void 0) { seconds = 0; }
                return new Date(Date.UTC(year, month, date, hours, minutes, seconds));
            };
            DateHelper.newDateTime = function (year, month, date, hours, minutes, seconds) {
                if (seconds === void 0) { seconds = 0; }
                return new Date(year, month, date, hours, minutes, seconds);
            };
            DateHelper.formatUTCDate = function (date) {
                var yearAsString = "" + date.getUTCFullYear();
                return yearAsString + "-" + this.padNumber(date.getUTCMonth() + 1) + "-" + this.padNumber(date.getUTCDate());
            };
            DateHelper.formatDate = function (date) {
                var yearAsString = "" + date.getFullYear();
                return yearAsString + "-" + this.padNumber(date.getMonth() + 1) + "-" + this.padNumber(date.getDate());
            };
            DateHelper.formatTime = function (date) {
                return this.padNumber(date.getHours()) + ":" + this.padNumber(date.getMinutes()) + ":" + this.padNumber(date.getSeconds());
            };
            DateHelper.padNumber = function (num) {
                return (num < 10 ? '0' : '') + num;
            };
            DateHelper.formatUTCDateTime = function (date) {
                var dateAsString = DateHelper.formatUTCDate(date);
                return dateAsString + "T" + this.padNumber(date.getUTCHours()) + ":" + this.padNumber(date.getUTCMinutes()) + ":" + this.padNumber(date.getUTCSeconds());
            };
            DateHelper.formatDateTime = function (date) {
                var dateAsString = DateHelper.formatDate(date);
                var timeAsString = DateHelper.formatTime(date);
                return dateAsString + "T" + timeAsString;
            };
            DateHelper.formatUTCDateTimeForInstant = function (date) {
                return this.formatUTCDateTime(date) + "Z";
            };
            DateHelper.parseDate = function (value, dateSeparator) {
                if (dateSeparator === void 0) { dateSeparator = "-"; }
                var dateStr = (value || '').trim();
                if (dateStr.length < 8 || dateStr.length > 10) {
                    return null;
                }
                var parts = dateStr.split(dateSeparator);
                if (parts.length !== 3 || parts[0].length !== 4) {
                    return null;
                }
                var parsedYear = Number(parts[0]);
                var parsedMonth = Number(parts[1]);
                var parsedDayOfMonth = Number(parts[2]);
                var date = new Date(parsedYear, parsedMonth - 1, parsedDayOfMonth);
                return date.getFullYear() === parsedYear && date.getMonth() === (parsedMonth - 1) && date.getDate() === parsedDayOfMonth ? date : null;
            };
            DateHelper.parseTime = function (value) {
                var dateStr = (value || '').trim();
                if (dateStr.length != 5) {
                    return null;
                }
                var parts = dateStr.split(':');
                if (parts.length !== 2) {
                    return null;
                }
                var hour = Number(parts[0]);
                var minute = Number(parts[1]);
                if (isNaN(hour) || isNaN(minute) || hour < 0 || hour > 23 || minute < 0 || minute > 59) {
                    return null;
                }
                return { hour: hour, minute: minute };
            };
            DateHelper.parseDateTime = function (value) {
                var dateStr = (value || '').trim();
                if (dateStr.length < 14 || dateStr.length > 16) {
                    return null;
                }
                var parts = dateStr.split(' ');
                if (parts.length !== 2) {
                    return null;
                }
                var datePart = parts[0];
                var timePart = parts[1];
                var date = DateHelper.parseDate(datePart);
                if (!date) {
                    return null;
                }
                var time = DateHelper.parseTime(timePart);
                if (!time) {
                    return null;
                }
                date.setHours(time.hour, time.minute, 0, 0);
                return date;
            };
            DateHelper.parseLongTime = function (value, timeSeparator, fractionSeparator) {
                if (timeSeparator === void 0) { timeSeparator = ":"; }
                if (fractionSeparator === void 0) { fractionSeparator = "."; }
                var timeStr = (value || '').trim();
                if (timeStr.length < 5 || timeStr.length > 12) {
                    return null;
                }
                var time = timeStr.split(timeSeparator);
                if (time.length < 2 || time.length > 3) {
                    return null;
                }
                var hours = Number(time[0]);
                var minutes = Number(time[1]);
                var seconds = 0;
                var fractions = 0;
                if (time[2]) {
                    var secondArr = time[2].split(fractionSeparator);
                    seconds = Number(secondArr[0]);
                    if (secondArr[1]) {
                        fractions = Number(secondArr[1]);
                    }
                }
                if (isNaN(hours) || isNaN(minutes) || isNaN(seconds) || isNaN(fractions) || hours < 0 || hours > 23 || minutes < 0 || minutes > 59 || seconds < 0 || seconds > 59 || fractions < 0) {
                    return null;
                }
                return {
                    hours: hours,
                    minutes: minutes,
                    seconds: seconds,
                    fractions: fractions
                };
            };
            DateHelper.parseLongDateTime = function (value, dateTimeSeparator, dateSeparator, timeSeparator, fractionSeparator) {
                if (dateTimeSeparator === void 0) { dateTimeSeparator = "-"; }
                if (dateSeparator === void 0) { dateSeparator = "-"; }
                if (timeSeparator === void 0) { timeSeparator = ":"; }
                if (fractionSeparator === void 0) { fractionSeparator = "."; }
                var dateStr = (value || '').trim();
                var parts = dateStr.split(dateTimeSeparator);
                if (parts.length !== 2) {
                    return null;
                }
                var datePart = parts[0];
                var timePart = parts[1];
                var date = DateHelper.parseDate(datePart, dateSeparator);
                if (!date) {
                    return null;
                }
                var time = DateHelper.parseLongTime(timePart, timeSeparator, fractionSeparator);
                if (!time) {
                    return null;
                }
                date.setHours(time.hours, time.minutes, time.seconds, time.fractions);
                return date;
            };
            DateHelper.isUTCdate = function (value) {
                if (value != null && (value[value.length - 1] == "Z" || value[value.length - 1] == "z")) {
                    return true;
                }
                return false;
            };
            DateHelper.numDaysInMonth = function (year, month) {
                return new Date(year, month + 1, 0).getDate();
            };
            return DateHelper;
        })();
        util.DateHelper = DateHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var NumberHelper = (function () {
            function NumberHelper() {
            }
            NumberHelper.isWholeNumber = function (value) {
                return NumberHelper.isNumber(value) && value % 1 == 0;
            };
            NumberHelper.isNumber = function (value) {
                return typeof value === 'number' && !isNaN(value);
            };
            NumberHelper.randomBetween = function (from, to) {
                return from + Math.round(Math.random() * (to - from));
            };
            return NumberHelper;
        })();
        util.NumberHelper = NumberHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var Flow = (function () {
            function Flow(thisOfProducer) {
                this.thisOfProducer = thisOfProducer;
            }
            Flow.prototype.getThisOfProducer = function () {
                return this.thisOfProducer;
            };
            Flow.prototype.doExecute = function (context) {
                return this.doExecuteNext(context);
            };
            Flow.prototype.doExecuteNext = function (context) {
                throw new Error("Must be implemented by inheritor");
            };
            return Flow;
        })();
        util.Flow = Flow;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var Link = (function () {
            function Link(value) {
                this.path = value;
            }
            Link.prototype.getPath = function () {
                return this.path;
            };
            Link.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Link)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.path, other.path)) {
                    return false;
                }
            };
            Link.prototype.toString = function () {
                return this.path;
            };
            return Link;
        })();
        util.Link = Link;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var DelayedFunctionCall = (function () {
            function DelayedFunctionCall(functionToCall, context, delay) {
                if (delay === void 0) { delay = 500; }
                this.functionToCall = functionToCall;
                this.delay = delay;
                this.context = context;
            }
            DelayedFunctionCall.prototype.delayCall = function () {
                var _this = this;
                if (this.timerId) {
                    window.clearTimeout(this.timerId);
                }
                this.timerId = window.setTimeout(function () {
                    _this.functionToCall.call(_this.context);
                    _this.timerId = null;
                }, this.delay);
            };
            return DelayedFunctionCall;
        })();
        util.DelayedFunctionCall = DelayedFunctionCall;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var GeoPoint = (function () {
            function GeoPoint(latitude, longitude) {
                this.latitude = latitude;
                this.longitude = longitude;
            }
            GeoPoint.prototype.getLatitude = function () {
                return this.latitude;
            };
            GeoPoint.prototype.getLongitude = function () {
                return this.longitude;
            };
            GeoPoint.prototype.toString = function () {
                return "" + this.latitude + "," + this.longitude;
            };
            GeoPoint.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, GeoPoint)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.numberEquals(this.latitude, other.latitude)) {
                    return false;
                }
                if (!api.ObjectHelper.numberEquals(this.longitude, other.longitude)) {
                    return false;
                }
                return true;
            };
            GeoPoint.isValidString = function (s) {
                if (util.StringHelper.isBlank(s)) {
                    return false;
                }
                var indexOfComma = s.indexOf(',');
                if (indexOfComma < 1 || s.split(',').length != 2) {
                    return false;
                }
                else if (indexOfComma == s.length - 1) {
                    return false;
                }
                var coordinates = s.split(',');
                var latitude = Number(coordinates[0]);
                var longitude = Number(coordinates[1]);
                var isNumber = typeof latitude === 'number' && !isNaN(latitude) && typeof longitude === 'number' && !isNaN(longitude);
                if (!isNumber) {
                    return false;
                }
                return (-90 <= latitude && latitude <= 90) && (-180 <= longitude && longitude <= 180);
            };
            GeoPoint.fromString = function (s) {
                if (!GeoPoint.isValidString(s)) {
                    throw new Error("Cannot parse GeoPoint from string: " + s);
                }
                var coordinates = s.split(',');
                var latitude = Number(coordinates[0]);
                var longitude = Number(coordinates[1]);
                return new GeoPoint(latitude, longitude);
            };
            return GeoPoint;
        })();
        util.GeoPoint = GeoPoint;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var LocalTime = (function () {
            function LocalTime(builder) {
                this.hours = builder.hours;
                this.minutes = builder.minutes;
                this.seconds = builder.seconds;
            }
            LocalTime.prototype.getHours = function () {
                return this.hours;
            };
            LocalTime.prototype.getMinutes = function () {
                return this.minutes;
            };
            LocalTime.prototype.getSeconds = function () {
                return this.seconds || 0;
            };
            LocalTime.prototype.toString = function () {
                var strSeconds = this.seconds ? LocalTime.TIME_SEPARATOR + this.padNumber(this.seconds) : util.StringHelper.EMPTY_STRING;
                return this.padNumber(this.hours) + LocalTime.TIME_SEPARATOR + this.padNumber(this.minutes) + strSeconds;
            };
            LocalTime.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, LocalTime)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.numberEquals(this.getHours(), other.getHours())) {
                    return false;
                }
                if (!api.ObjectHelper.numberEquals(this.getMinutes(), other.getMinutes())) {
                    return false;
                }
                if (!api.ObjectHelper.numberEquals(this.getSeconds(), other.getSeconds())) {
                    return false;
                }
                return true;
            };
            LocalTime.prototype.padNumber = function (num) {
                return (num < 10 ? '0' : '') + num;
            };
            LocalTime.isValidString = function (s) {
                if (util.StringHelper.isBlank(s)) {
                    return false;
                }
                var re = /^(\d{1}|[0-1]{1}\d{1}|[2]{1}[0-3]{1})((?::)(\d{1}|[0-5]{1}\d{1}))?((?::)(\d{1}|[0-5]{1}\d{1}))?$/;
                return re.test(s);
            };
            LocalTime.fromString = function (s) {
                if (!LocalTime.isValidString(s)) {
                    throw new Error("Cannot parse LocalTime from string: " + s);
                }
                var localTime = s.split(':');
                var hours = Number(localTime[0]);
                var minutes = Number(localTime[1]);
                var seconds = localTime.length > 2 ? Number(localTime[2]) : 0;
                return LocalTime.create().setHours(hours).setMinutes(minutes).setSeconds(seconds).build();
            };
            LocalTime.prototype.getAdjustedTime = function () {
                var date = new Date();
                date.setHours(this.getHours(), this.getMinutes(), this.getSeconds());
                return {
                    hour: date.getHours(),
                    minute: date.getMinutes(),
                    seconds: date.getSeconds()
                };
            };
            LocalTime.create = function () {
                return new LocalTimeBuilder();
            };
            LocalTime.TIME_SEPARATOR = ":";
            return LocalTime;
        })();
        util.LocalTime = LocalTime;
        var LocalTimeBuilder = (function () {
            function LocalTimeBuilder() {
            }
            LocalTimeBuilder.prototype.setHours = function (value) {
                this.hours = value;
                return this;
            };
            LocalTimeBuilder.prototype.setMinutes = function (value) {
                this.minutes = value;
                return this;
            };
            LocalTimeBuilder.prototype.setSeconds = function (value) {
                this.seconds = value;
                return this;
            };
            LocalTimeBuilder.prototype.build = function () {
                return new LocalTime(this);
            };
            return LocalTimeBuilder;
        })();
        util.LocalTimeBuilder = LocalTimeBuilder;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var LocalDate = (function () {
            function LocalDate(builder) {
                this.year = builder.year;
                this.month = builder.month;
                this.day = builder.day;
            }
            LocalDate.prototype.getYear = function () {
                return this.year;
            };
            LocalDate.prototype.getMonth = function () {
                return this.month;
            };
            LocalDate.prototype.getDay = function () {
                return this.day;
            };
            LocalDate.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, LocalDate)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.toString(), other.toString())) {
                    return false;
                }
                return true;
            };
            LocalDate.prototype.toString = function () {
                return this.year + LocalDate.DATE_SEPARATOR + this.padNumber(this.month) + LocalDate.DATE_SEPARATOR + this.padNumber(this.day);
            };
            LocalDate.prototype.padNumber = function (num, length) {
                if (length === void 0) { length = 2; }
                var numAsString = String(num);
                while (numAsString.length < length) {
                    numAsString = "0" + numAsString;
                }
                return numAsString;
            };
            LocalDate.isValidDate = function (s) {
                return !!api.util.DateHelper.parseDate(s);
            };
            LocalDate.fromDate = function (date) {
                if (date) {
                    return LocalDate.parseDate(date.getFullYear() + LocalDate.DATE_SEPARATOR + (date.getMonth() + 1) + LocalDate.DATE_SEPARATOR + date.getDate(), false);
                }
            };
            LocalDate.parseDate = function (s, isNeedToCheck) {
                if (isNeedToCheck === void 0) { isNeedToCheck = true; }
                if (isNeedToCheck && !LocalDate.isValidDate(s)) {
                    throw new Error("Cannot parse LocalDate from string: " + s);
                }
                var date = s.split(LocalDate.DATE_SEPARATOR);
                return LocalDate.create().setYear(Number(date[0])).setMonth(Number(date[1])).setDay(Number(date[2])).build();
            };
            LocalDate.create = function () {
                return new LocalDateBuilder();
            };
            LocalDate.DATE_SEPARATOR = "-";
            return LocalDate;
        })();
        util.LocalDate = LocalDate;
        var LocalDateBuilder = (function () {
            function LocalDateBuilder() {
            }
            LocalDateBuilder.prototype.setYear = function (value) {
                this.year = value;
                return this;
            };
            LocalDateBuilder.prototype.setMonth = function (value) {
                this.month = value;
                return this;
            };
            LocalDateBuilder.prototype.setDay = function (value) {
                this.day = value;
                return this;
            };
            LocalDateBuilder.prototype.validate = function () {
                if (!this.year) {
                    throw new Error("Invalid parameter. Year should be set");
                }
                else if (!this.month) {
                    throw new Error("Invalid parameter. Month should be set");
                }
                else if (!this.day) {
                    throw new Error("Invalid parameter. Day should be set");
                }
            };
            LocalDateBuilder.prototype.build = function () {
                this.validate();
                return new LocalDate(this);
            };
            return LocalDateBuilder;
        })();
        util.LocalDateBuilder = LocalDateBuilder;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var BinaryReference = (function () {
            function BinaryReference(value) {
                this.value = value;
            }
            BinaryReference.prototype.getValue = function () {
                return this.value;
            };
            BinaryReference.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, BinaryReference)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.value, other.value)) {
                    return false;
                }
                return true;
            };
            BinaryReference.prototype.toString = function () {
                return this.value;
            };
            return BinaryReference;
        })();
        util.BinaryReference = BinaryReference;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var Reference = (function () {
            function Reference(value) {
                this.referenceId = value;
            }
            Reference.from = function (value) {
                return new Reference(value.toString());
            };
            Reference.prototype.getNodeId = function () {
                return this.referenceId;
            };
            Reference.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Reference)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.referenceId, other.referenceId)) {
                    return false;
                }
                return true;
            };
            Reference.prototype.toString = function () {
                return this.referenceId;
            };
            return Reference;
        })();
        util.Reference = Reference;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var PromiseHelper = (function () {
            function PromiseHelper() {
            }
            PromiseHelper.newResolvedVoidPromise = function () {
                var deferred = wemQ.defer();
                deferred.resolve(null);
                return deferred.promise;
            };
            return PromiseHelper;
        })();
        util.PromiseHelper = PromiseHelper;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var LocalDateTime = (function () {
            function LocalDateTime(builder) {
                this.year = builder.year;
                this.month = builder.month;
                this.day = builder.day;
                this.hours = builder.hours;
                this.minutes = builder.minutes;
                this.seconds = builder.seconds;
                this.fractions = builder.fractions;
            }
            LocalDateTime.prototype.getYear = function () {
                return this.year;
            };
            LocalDateTime.prototype.getMonth = function () {
                return this.month;
            };
            LocalDateTime.prototype.getDay = function () {
                return this.day;
            };
            LocalDateTime.prototype.getHours = function () {
                return this.hours;
            };
            LocalDateTime.prototype.getMinutes = function () {
                return this.minutes;
            };
            LocalDateTime.prototype.getSeconds = function () {
                return this.seconds || 0;
            };
            LocalDateTime.prototype.getFractions = function () {
                return this.fractions || 0;
            };
            LocalDateTime.prototype.dateToString = function () {
                return this.year + LocalDateTime.DATE_SEPARATOR + this.padNumber(this.month) + LocalDateTime.DATE_SEPARATOR + this.padNumber(this.day);
            };
            LocalDateTime.prototype.timeToString = function () {
                var seconds = this.seconds ? LocalDateTime.TIME_SEPARATOR + this.padNumber(this.seconds) : util.StringHelper.EMPTY_STRING;
                var fractions = this.fractions ? LocalDateTime.FRACTION_SEPARATOR + this.padNumber(this.fractions, 3) : util.StringHelper.EMPTY_STRING;
                return this.padNumber(this.hours) + LocalDateTime.TIME_SEPARATOR + this.padNumber(this.minutes) + seconds + fractions;
            };
            LocalDateTime.prototype.toString = function () {
                return this.dateToString() + LocalDateTime.DATE_TIME_SEPARATOR + this.timeToString();
            };
            LocalDateTime.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, LocalDateTime)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.toString(), other.toString())) {
                    return false;
                }
                return true;
            };
            LocalDateTime.prototype.padNumber = function (num, length) {
                if (length === void 0) { length = 2; }
                var numAsString = String(num);
                while (numAsString.length < length) {
                    numAsString = "0" + numAsString;
                }
                return numAsString;
            };
            LocalDateTime.isValidDateTime = function (s) {
                if (util.StringHelper.isBlank(s)) {
                    return false;
                }
                var re = /^(\d{2}|\d{4})(?:\-)?([0]{1}\d{1}|[1]{1}[0-2]{1})(?:\-)?([0-2]{1}\d{1}|[3]{1}[0-1]{1})(T)([0-1]{1}\d{1}|[2]{1}[0-3]{1})(?::)?([0-5]{1}\d{1})(?::)?([0-5]{1}\d{1})?(?:.)?(\d{3})?/;
                return re.test(s);
            };
            LocalDateTime.fromString = function (s) {
                if (!LocalDateTime.isValidDateTime(s)) {
                    throw new Error("Cannot parse LocalDateTime from string: " + s);
                }
                var date = util.DateHelper.parseLongDateTime(s, LocalDateTime.DATE_TIME_SEPARATOR, LocalDateTime.DATE_SEPARATOR, LocalDateTime.TIME_SEPARATOR, LocalDateTime.FRACTION_SEPARATOR);
                if (!date) {
                    throw new Error("Cannot parse LocalDateTime from string: " + s);
                }
                return LocalDateTime.create().setYear(date.getFullYear()).setMonth(date.getMonth()).setDay(date.getDate()).setHours(date.getHours()).setMinutes(date.getMinutes()).setSeconds(date.getSeconds()).setFractions(date.getMilliseconds()).build();
            };
            LocalDateTime.create = function () {
                return new LocalDateTimeBuilder();
            };
            LocalDateTime.DATE_TIME_SEPARATOR = "T";
            LocalDateTime.DATE_SEPARATOR = "-";
            LocalDateTime.TIME_SEPARATOR = ":";
            LocalDateTime.FRACTION_SEPARATOR = ".";
            return LocalDateTime;
        })();
        util.LocalDateTime = LocalDateTime;
        var LocalDateTimeBuilder = (function () {
            function LocalDateTimeBuilder() {
            }
            LocalDateTimeBuilder.prototype.setYear = function (value) {
                this.year = value;
                return this;
            };
            LocalDateTimeBuilder.prototype.setMonth = function (value) {
                this.month = value;
                return this;
            };
            LocalDateTimeBuilder.prototype.setDay = function (value) {
                this.day = value;
                return this;
            };
            LocalDateTimeBuilder.prototype.setHours = function (value) {
                this.hours = value;
                return this;
            };
            LocalDateTimeBuilder.prototype.setMinutes = function (value) {
                this.minutes = value;
                return this;
            };
            LocalDateTimeBuilder.prototype.setSeconds = function (value) {
                this.seconds = value;
                return this;
            };
            LocalDateTimeBuilder.prototype.setFractions = function (value) {
                if (this.seconds && value > 0)
                    this.fractions = value;
                return this;
            };
            LocalDateTimeBuilder.prototype.build = function () {
                return new LocalDateTime(this);
            };
            return LocalDateTimeBuilder;
        })();
        util.LocalDateTimeBuilder = LocalDateTimeBuilder;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var StringHelper = api.util.StringHelper;
        var DateTime = (function () {
            function DateTime(builder) {
                this.year = builder.year;
                this.month = builder.month;
                this.day = builder.day;
                this.hours = builder.hours;
                this.minutes = builder.minutes;
                this.seconds = builder.seconds;
                this.fractions = builder.fractions;
                this.timezone = builder.timezone;
            }
            DateTime.prototype.getYear = function () {
                return this.year;
            };
            DateTime.prototype.getMonth = function () {
                return this.month;
            };
            DateTime.prototype.getDay = function () {
                return this.day;
            };
            DateTime.prototype.getHours = function () {
                return this.hours;
            };
            DateTime.prototype.getMinutes = function () {
                return this.minutes;
            };
            DateTime.prototype.getSeconds = function () {
                return this.seconds || 0;
            };
            DateTime.prototype.getFractions = function () {
                return this.fractions || 0;
            };
            DateTime.prototype.getTimezone = function () {
                return this.timezone;
            };
            DateTime.prototype.dateToString = function () {
                return this.year + DateTime.DATE_SEPARATOR + this.padNumber(this.month + 1) + DateTime.DATE_SEPARATOR + this.padNumber(this.day);
            };
            DateTime.prototype.timeToString = function () {
                var fractions = this.fractions ? DateTime.FRACTION_SEPARATOR + this.padNumber(this.fractions, 3) : StringHelper.EMPTY_STRING;
                return this.padNumber(this.hours) + DateTime.TIME_SEPARATOR + this.padNumber(this.minutes) + DateTime.TIME_SEPARATOR + this.padNumber(this.seconds ? this.seconds : 0) + fractions;
            };
            DateTime.prototype.toString = function () {
                return this.dateToString() + DateTime.DATE_TIME_SEPARATOR + this.timeToString() + (this.timezone ? this.timezone.toString() : DateTime.DEFAULT_TIMEZONE);
            };
            DateTime.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, DateTime)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.toString(), other.toString())) {
                    return false;
                }
                return true;
            };
            DateTime.prototype.toDate = function () {
                return util.DateHelper.parseLongDateTime(DateTime.trimTZ(this.toString()), DateTime.DATE_TIME_SEPARATOR, DateTime.DATE_SEPARATOR, DateTime.TIME_SEPARATOR, DateTime.FRACTION_SEPARATOR);
            };
            DateTime.prototype.padNumber = function (num, length) {
                if (length === void 0) { length = 2; }
                var numAsString = String(num);
                while (numAsString.length < length) {
                    numAsString = "0" + numAsString;
                }
                return numAsString;
            };
            DateTime.isValidDateTime = function (s) {
                if (StringHelper.isBlank(s)) {
                    return false;
                }
                var re = /^(\d{2}|\d{4})(?:\-)?([0]{1}\d{1}|[1]{1}[0-2]{1})(?:\-)?([0-2]{1}\d{1}|[3]{1}[0-1]{1})(T)([0-1]{1}\d{1}|[2]{1}[0-3]{1})(?::)?([0-5]{1}\d{1})((:[0-5]{1}\d{1})(\.\d{3})?)?((\+|\-)([0-1]{1}\d{1}|[2]{1}[0-3]{1})(:)([0-5]{1}\d{1})|(z|Z)|$)$/;
                return re.test(s);
            };
            DateTime.fromString = function (s) {
                if (!DateTime.isValidDateTime(s)) {
                    throw new Error("Cannot parse DateTime from string: " + s);
                }
                var date, timezone;
                if (util.DateHelper.isUTCdate(s)) {
                    date = util.DateHelper.parseUTCDateTime(s);
                    timezone = util.Timezone.getLocalTimezone();
                }
                else {
                    var withoutTZ = DateTime.trimTZ(s);
                    date = util.DateHelper.parseLongDateTime(withoutTZ, DateTime.DATE_TIME_SEPARATOR, DateTime.DATE_SEPARATOR, DateTime.TIME_SEPARATOR, DateTime.FRACTION_SEPARATOR);
                    var offset = DateTime.parseOffset(s);
                    if (offset != null) {
                        timezone = util.Timezone.fromOffset(offset);
                    }
                    else {
                        timezone = util.Timezone.getZeroOffsetTimezone();
                    }
                }
                if (!date) {
                    throw new Error("Cannot parse DateTime from string: " + s);
                }
                return DateTime.create().setYear(date.getFullYear()).setMonth(date.getMonth()).setDay(date.getDate()).setHours(date.getHours()).setMinutes(date.getMinutes()).setSeconds(date.getSeconds()).setFractions(date.getMilliseconds()).setTimezone(timezone).build();
            };
            DateTime.fromDate = function (s) {
                return DateTime.create().setYear(s.getFullYear()).setMonth(s.getMonth()).setDay(s.getDate()).setHours(s.getHours()).setMinutes(s.getMinutes()).setSeconds(s.getSeconds()).setFractions(s.getMilliseconds()).setTimezone(util.Timezone.fromOffset(util.DateHelper.getTZOffsetFromDate(s))).build();
            };
            DateTime.create = function () {
                return new DateTimeBuilder();
            };
            DateTime.parseOffset = function (value) {
                if (util.DateHelper.isUTCdate(value)) {
                    return 0;
                }
                else {
                    var dateStr = (value || '').trim();
                    if (dateStr.indexOf("+") > 0) {
                        var parts = dateStr.split("+");
                        if (parts.length == 2) {
                            var offsetPart = parts[1];
                            var offset = parseFloat(offsetPart);
                            if (isNaN(offset)) {
                                return 0;
                            }
                            return offset;
                        }
                        else {
                            return 0;
                        }
                    }
                    else if (dateStr.split("-").length == 4) {
                        var parts = dateStr.split("-");
                        var offsetPart = parts[3];
                        var offset = parseFloat(offsetPart);
                        if (isNaN(offset)) {
                            return 0;
                        }
                        return offset * -1;
                    }
                    else {
                        return 0;
                    }
                }
            };
            DateTime.trimTZ = function (dateString) {
                var tzStartIndex = dateString.indexOf("+");
                if (tzStartIndex > 0) {
                    return dateString.substr(0, tzStartIndex);
                }
                else if (dateString.split("-").length == 4) {
                    tzStartIndex = dateString.lastIndexOf("-");
                    return dateString.substr(0, tzStartIndex);
                }
                else {
                    tzStartIndex = dateString.toLowerCase().indexOf("z");
                    if (tzStartIndex > 0)
                        return dateString.substr(0, tzStartIndex);
                }
                return dateString;
            };
            DateTime.DATE_TIME_SEPARATOR = "T";
            DateTime.DATE_SEPARATOR = "-";
            DateTime.TIME_SEPARATOR = ":";
            DateTime.FRACTION_SEPARATOR = ".";
            DateTime.DEFAULT_TIMEZONE = "+00:00";
            return DateTime;
        })();
        util.DateTime = DateTime;
        var DateTimeBuilder = (function () {
            function DateTimeBuilder() {
            }
            DateTimeBuilder.prototype.setYear = function (value) {
                this.year = value;
                return this;
            };
            DateTimeBuilder.prototype.setMonth = function (value) {
                this.month = value;
                return this;
            };
            DateTimeBuilder.prototype.setDay = function (value) {
                this.day = value;
                return this;
            };
            DateTimeBuilder.prototype.setHours = function (value) {
                this.hours = value;
                return this;
            };
            DateTimeBuilder.prototype.setMinutes = function (value) {
                this.minutes = value;
                return this;
            };
            DateTimeBuilder.prototype.setSeconds = function (value) {
                this.seconds = value;
                return this;
            };
            DateTimeBuilder.prototype.setFractions = function (value) {
                if (this.seconds && value > 0)
                    this.fractions = value;
                return this;
            };
            DateTimeBuilder.prototype.setTimezone = function (value) {
                this.timezone = value;
                return this;
            };
            DateTimeBuilder.prototype.build = function () {
                return new DateTime(this);
            };
            return DateTimeBuilder;
        })();
        util.DateTimeBuilder = DateTimeBuilder;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var Timezone = (function () {
            function Timezone(builder) {
                this.offset = builder.offset;
                this.location = builder.location;
            }
            Timezone.prototype.getOffset = function () {
                return this.offset;
            };
            Timezone.prototype.getLocation = function () {
                return this.location;
            };
            Timezone.prototype.offsetToString = function () {
                if (this.offset < 0) {
                    return "-" + this.padOffset(Math.abs(this.offset));
                }
                return "+" + this.padOffset(Math.abs(this.offset));
            };
            Timezone.prototype.toString = function () {
                return this.offsetToString();
            };
            Timezone.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Timezone)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.toString(), other.toString())) {
                    return false;
                }
                return true;
            };
            Timezone.prototype.padOffset = function (num, length) {
                if (length === void 0) { length = 2; }
                var numAsString = String(num);
                while (numAsString.length < length) {
                    numAsString = "0" + numAsString;
                }
                return numAsString + ":00";
            };
            Timezone.isValidTimezone = function (s) {
                if (util.StringHelper.isBlank(s)) {
                    return false;
                }
                return true;
            };
            Timezone.isValidOffset = function (s) {
                if (s > -13 && s < 13) {
                    return true;
                }
                return false;
            };
            Timezone.fromOffset = function (s) {
                if (!Timezone.isValidOffset(s)) {
                    throw new Error("Passed Timezone ofsset is invalid: " + s);
                }
                return Timezone.create().setOffset(s).build();
            };
            Timezone.getLocalTimezone = function () {
                return Timezone.fromOffset(util.DateHelper.getTZOffset());
            };
            Timezone.getZeroOffsetTimezone = function () {
                return Timezone.create().setOffset(0).build();
            };
            Timezone.create = function () {
                return new TimezoneBuilder();
            };
            return Timezone;
        })();
        util.Timezone = Timezone;
        var TimezoneBuilder = (function () {
            function TimezoneBuilder() {
            }
            TimezoneBuilder.prototype.setOffset = function (value) {
                this.offset = value;
                return this;
            };
            TimezoneBuilder.prototype.setLocation = function (value) {
                this.location = value;
                return this;
            };
            TimezoneBuilder.prototype.build = function () {
                return new Timezone(this);
            };
            TimezoneBuilder.prototype.buildDefault = function () {
                this.offset = 0;
                return new Timezone(this);
            };
            return TimezoneBuilder;
        })();
        util.TimezoneBuilder = TimezoneBuilder;
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var loader;
        (function (loader) {
            var event;
            (function (event) {
                var LoaderEvent = (function () {
                    function LoaderEvent() {
                    }
                    return LoaderEvent;
                })();
                event.LoaderEvent = LoaderEvent;
            })(event = loader.event || (loader.event = {}));
        })(loader = util.loader || (util.loader = {}));
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var loader;
        (function (loader) {
            var event;
            (function (event) {
                var LoadedDataEvent = (function (_super) {
                    __extends(LoadedDataEvent, _super);
                    function LoadedDataEvent(data) {
                        _super.call(this);
                        this.data = data;
                    }
                    LoadedDataEvent.prototype.getData = function () {
                        return this.data;
                    };
                    return LoadedDataEvent;
                })(event.LoaderEvent);
                event.LoadedDataEvent = LoadedDataEvent;
            })(event = loader.event || (loader.event = {}));
        })(loader = util.loader || (util.loader = {}));
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var loader;
        (function (loader) {
            var event;
            (function (event) {
                var LoadingDataEvent = (function (_super) {
                    __extends(LoadingDataEvent, _super);
                    function LoadingDataEvent() {
                        _super.call(this);
                    }
                    return LoadingDataEvent;
                })(event.LoaderEvent);
                event.LoadingDataEvent = LoadingDataEvent;
            })(event = loader.event || (loader.event = {}));
        })(loader = util.loader || (util.loader = {}));
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var loader;
        (function (loader) {
            var event;
            (function (event) {
                (function (LoaderEvents) {
                    LoaderEvents[LoaderEvents["LoadedData"] = 0] = "LoadedData";
                    LoaderEvents[LoaderEvents["LoadingData"] = 1] = "LoadingData";
                })(event.LoaderEvents || (event.LoaderEvents = {}));
                var LoaderEvents = event.LoaderEvents;
            })(event = loader.event || (loader.event = {}));
        })(loader = util.loader || (util.loader = {}));
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var loader;
        (function (loader) {
            var LoadedDataEvent = api.util.loader.event.LoadedDataEvent;
            var LoadingDataEvent = api.util.loader.event.LoadingDataEvent;
            var LoaderStatus;
            (function (LoaderStatus) {
                LoaderStatus[LoaderStatus["NOT_STARTED"] = 0] = "NOT_STARTED";
                LoaderStatus[LoaderStatus["LOADING"] = 1] = "LOADING";
                LoaderStatus[LoaderStatus["LOADED"] = 2] = "LOADED";
            })(LoaderStatus || (LoaderStatus = {}));
            var BaseLoader = (function () {
                function BaseLoader(request) {
                    this.status = 0 /* NOT_STARTED */;
                    this.loadedDataListeners = [];
                    this.loadingDataListeners = [];
                    this.setRequest(request);
                }
                BaseLoader.prototype.sendRequest = function () {
                    return this.request.sendAndParse();
                };
                BaseLoader.prototype.load = function () {
                    var _this = this;
                    this.notifyLoadingData();
                    return this.sendRequest().then(function (results) {
                        _this.results = results;
                        if (_this.comparator) {
                            _this.results = results.sort(_this.comparator.compare);
                        }
                        _this.notifyLoadedData(results);
                        return _this.results;
                    });
                };
                BaseLoader.prototype.isLoading = function () {
                    return this.status == 1 /* LOADING */;
                };
                BaseLoader.prototype.isLoaded = function () {
                    return this.status == 2 /* LOADED */;
                };
                BaseLoader.prototype.isNotStarted = function () {
                    return this.status == 0 /* NOT_STARTED */;
                };
                BaseLoader.prototype.setComparator = function (comparator) {
                    this.comparator = comparator;
                    return this;
                };
                BaseLoader.prototype.setRequest = function (request) {
                    this.request = request;
                };
                BaseLoader.prototype.getRequest = function () {
                    return this.request;
                };
                BaseLoader.prototype.search = function (searchString) {
                    var deferred = wemQ.defer();
                    this.searchString = searchString;
                    if (this.results) {
                        var filtered = this.results.filter(this.filterFn, this);
                        this.notifyLoadedData(filtered);
                        deferred.resolve(this.results);
                    }
                    else {
                        deferred.resolve(null);
                    }
                    return deferred.promise;
                };
                BaseLoader.prototype.getSearchString = function () {
                    return this.searchString;
                };
                BaseLoader.prototype.filterFn = function (result) {
                    throw Error("must be implemented");
                };
                BaseLoader.prototype.notifyLoadedData = function (results) {
                    var _this = this;
                    this.status = 2 /* LOADED */;
                    this.loadedDataListeners.forEach(function (listener) {
                        listener.call(_this, new LoadedDataEvent(results));
                    });
                };
                BaseLoader.prototype.notifyLoadingData = function () {
                    var _this = this;
                    this.status = 1 /* LOADING */;
                    this.loadingDataListeners.forEach(function (listener) {
                        listener.call(_this, new LoadingDataEvent());
                    });
                };
                BaseLoader.prototype.onLoadedData = function (listener) {
                    this.loadedDataListeners.push(listener);
                };
                BaseLoader.prototype.onLoadingData = function (listener) {
                    this.loadingDataListeners.push(listener);
                };
                BaseLoader.prototype.unLoadedData = function (listener) {
                    this.loadedDataListeners = this.loadedDataListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                BaseLoader.prototype.unLoadingData = function (listener) {
                    this.loadingDataListeners = this.loadingDataListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                return BaseLoader;
            })();
            loader.BaseLoader = BaseLoader;
        })(loader = util.loader || (util.loader = {}));
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var util;
    (function (util) {
        var loader;
        (function (loader) {
            var ImageLoader = (function () {
                function ImageLoader() {
                }
                ImageLoader.get = function (url, width, height) {
                    var imageFound = false;
                    var returnImage;
                    url = encodeURI(url);
                    for (var i in ImageLoader.cachedImages) {
                        if (ImageLoader.cachedImages[i].src == url) {
                            imageFound = true;
                            returnImage = ImageLoader.cachedImages[i];
                        }
                    }
                    if (!imageFound) {
                        var image = new Image(width, height);
                        image.src = url;
                        ImageLoader.cachedImages[ImageLoader.cachedImages.length + 1] = image;
                        returnImage = image;
                    }
                    return returnImage;
                };
                ImageLoader.cachedImages = [];
                return ImageLoader;
            })();
            loader.ImageLoader = ImageLoader;
        })(loader = util.loader || (util.loader = {}));
    })(util = api.util || (api.util = {}));
})(api || (api = {}));
var api;
(function (api) {
    var i18n;
    (function (i18n) {
        var messages = {};
        var currentLocale = 'en';
        function setLocale(locale) {
            currentLocale = locale;
        }
        i18n.setLocale = setLocale;
        function addBundle(locale, bundle) {
            if (messages[locale] == undefined) {
                messages[locale] = bundle;
            }
            else {
                wemjq.extend(messages[locale], messages[locale], bundle);
            }
        }
        i18n.addBundle = addBundle;
        function message(key, args) {
            var message = key;
            var current = messages[currentLocale];
            if ((current != undefined) && (current[key] != undefined)) {
                message = current[key];
            }
            return message.replace(/\$(\d+)/g, function () {
                return args[arguments[1] - 1];
            });
        }
        i18n.message = message;
    })(i18n = api.i18n || (api.i18n = {}));
})(api || (api = {}));
var _i18n = function (key) {
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    return api.i18n.message(key, args);
};
var api;
(function (api) {
    var i18n;
    (function (i18n) {
        var messages = {};
        i18n.addBundle('no', messages);
    })(i18n = api.i18n || (api.i18n = {}));
})(api || (api = {}));
var api;
(function (api) {
    var cache;
    (function (cache) {
        var Cache = (function () {
            function Cache() {
                this.objectsByKey = {};
            }
            Cache.prototype.getAll = function () {
                var all = [];
                for (var key in this.objectsByKey) {
                    if (this.objectsByKey.hasOwnProperty(key)) {
                        all.push(this.objectsByKey[key]);
                    }
                }
                return all;
            };
            Cache.prototype.copy = function (object) {
                throw new Error("Must be implemented by inheritor");
            };
            Cache.prototype.getKeyFromObject = function (object) {
                throw new Error("Must be implemented by inheritor");
            };
            Cache.prototype.getKeyAsString = function (object) {
                throw new Error("Must be implemented by inheritor");
            };
            Cache.prototype.put = function (object) {
                var copy = this.copy(object);
                var keyAsString = this.getKeyAsString(this.getKeyFromObject(object));
                this.objectsByKey[keyAsString] = copy;
            };
            Cache.prototype.deleteByKey = function (key) {
                var keyAsString = this.getKeyAsString(key);
                delete this.objectsByKey[keyAsString];
            };
            Cache.prototype.getByKey = function (key) {
                var keyAsString = this.getKeyAsString(key);
                var object = this.objectsByKey[keyAsString];
                if (!object) {
                    return null;
                }
                return this.copy(object);
            };
            return Cache;
        })();
        cache.Cache = Cache;
    })(cache = api.cache || (api.cache = {}));
})(api || (api = {}));
var api;
(function (api) {
    var event;
    (function (_event) {
        var Event = (function () {
            function Event(name) {
                this.name = name || api.ClassHelper.getFullName(this);
            }
            Event.prototype.getName = function () {
                return this.name;
            };
            Event.prototype.fire = function (contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                _event.EventBus.fireEvent(this, contextWindow);
            };
            Event.bind = function (name, handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                _event.EventBus.onEvent(name, handler, contextWindow);
            };
            Event.unbind = function (name, handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                _event.EventBus.unEvent(name, handler, contextWindow);
            };
            return Event;
        })();
        _event.Event = Event;
    })(event = api.event || (api.event = {}));
})(api || (api = {}));
var api;
(function (api) {
    var event;
    (function (_event) {
        var EventBus = (function () {
            function EventBus() {
            }
            EventBus.onEvent = function (eventName, handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                var customEventHandler = function (customEvent) { return handler(customEvent['apiEventObj']); };
                if (!EventBus.handlersMap[eventName]) {
                    EventBus.handlersMap[eventName] = [];
                }
                EventBus.handlersMap[eventName].push({
                    customEventHandler: customEventHandler,
                    apiEventHandler: handler
                });
                contextWindow.addEventListener(eventName, customEventHandler);
            };
            EventBus.unEvent = function (eventName, handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                if (handler) {
                    var customEventHandler;
                    EventBus.handlersMap[eventName] = EventBus.handlersMap[eventName].filter(function (entry) {
                        if (entry.apiEventHandler === handler) {
                            customEventHandler = entry.customEventHandler;
                        }
                        return entry.apiEventHandler != handler;
                    });
                    contextWindow.removeEventListener(eventName, customEventHandler);
                }
                else {
                    (EventBus.handlersMap[eventName] || []).forEach(function (entry) {
                        contextWindow.removeEventListener(eventName, entry.customEventHandler);
                    });
                    EventBus.handlersMap[eventName] = [];
                }
            };
            EventBus.fireEvent = function (apiEventObj, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                var customEvent = contextWindow.document.createEvent('Event');
                customEvent.initEvent(apiEventObj.getName(), true, true);
                customEvent['apiEventObj'] = apiEventObj;
                contextWindow.dispatchEvent(customEvent);
            };
            EventBus.handlersMap = {};
            return EventBus;
        })();
        _event.EventBus = EventBus;
    })(event = api.event || (api.event = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var WindowDOM = (function () {
            function WindowDOM() {
                var _this = this;
                this.onBeforeUnloadListeners = [];
                this.onUnloadListeners = [];
                this.el = window;
                this.el.onbeforeunload = function (event) {
                    _this.onBeforeUnloadListeners.forEach(function (listener) { return listener(event); });
                };
                this.el.onunload = function (event) {
                    _this.onUnloadListeners.forEach(function (listener) { return listener(event); });
                };
            }
            WindowDOM.get = function () {
                return WindowDOM.instance;
            };
            WindowDOM.prototype.asWindow = function () {
                return this.el;
            };
            WindowDOM.prototype.getTopParent = function () {
                var parent = this.getParent();
                if (!parent) {
                    return null;
                }
                var i = 0;
                do {
                    var next = parent.getParent();
                    if (!next) {
                        return parent;
                    }
                    parent = next;
                    i++;
                } while (i < 10);
                return null;
            };
            WindowDOM.prototype.getParent = function () {
                var parent = this.el.parent;
                if (parent === this.el) {
                    return null;
                }
                return parent.api.dom.WindowDOM.get();
            };
            WindowDOM.prototype.getHTMLElement = function () {
                return this.el;
            };
            WindowDOM.prototype.getScrollTop = function () {
                return wemjq(this.el).scrollTop();
            };
            WindowDOM.prototype.onResized = function (listener, element) {
                var _this = this;
                this.el.addEventListener("resize", listener);
                if (element) {
                    element.onRemoved(function () { return _this.unResized(listener); });
                }
            };
            WindowDOM.prototype.unResized = function (listener) {
                this.el.removeEventListener("resize", listener);
            };
            WindowDOM.prototype.getWidth = function () {
                return wemjq(this.el).innerWidth();
            };
            WindowDOM.prototype.getHeight = function () {
                return wemjq(this.el).innerHeight();
            };
            WindowDOM.prototype.onScroll = function (listener, element) {
                var _this = this;
                this.el.addEventListener('scroll', listener);
                if (element) {
                    element.onRemoved(function () { return _this.unScroll(listener); });
                }
            };
            WindowDOM.prototype.unScroll = function (listener) {
                this.el.removeEventListener('scroll', listener);
            };
            WindowDOM.prototype.onBeforeUnload = function (listener) {
                this.onBeforeUnloadListeners.push(listener);
            };
            WindowDOM.prototype.unBeforeUnload = function (listener) {
                this.onBeforeUnloadListeners = this.onBeforeUnloadListeners.filter(function (curr) {
                    return curr != listener;
                });
                return this;
            };
            WindowDOM.prototype.onUnload = function (listener) {
                this.onUnloadListeners.push(listener);
            };
            WindowDOM.instance = new WindowDOM();
            return WindowDOM;
        })();
        dom.WindowDOM = WindowDOM;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementHelper = (function () {
            function ElementHelper(element) {
                api.util.assertNotNull(element, 'Element cannot be null');
                this.el = element;
            }
            ElementHelper.fromName = function (name) {
                api.util.assert(!api.util.StringHelper.isEmpty(name), 'Tag name cannot be empty');
                return new ElementHelper(document.createElement(name));
            };
            ElementHelper.prototype.getHTMLElement = function () {
                return this.el;
            };
            ElementHelper.prototype.insertBefore = function (newEl, existingEl) {
                api.util.assertNotNull(newEl, 'New element cannot be null');
                api.util.assertNotNull(existingEl, 'Existing element cannot be null');
                this.el.insertBefore(newEl.getHTMLElement(), existingEl ? existingEl.getHTMLElement() : null);
            };
            ElementHelper.prototype.insertBeforeEl = function (existingEl) {
                existingEl.el.parentElement.insertBefore(this.el, existingEl.el);
            };
            ElementHelper.prototype.insertAfterEl = function (existingEl) {
                api.util.assertNotNull(existingEl, 'Existing element cannot be null');
                api.util.assertNotNull(existingEl.el.parentElement, 'Existing element\'s parentElement cannot be null');
                existingEl.el.parentElement.insertBefore(this.el, existingEl.el.nextElementSibling);
            };
            ElementHelper.prototype.insertAfterThisEl = function (toInsert) {
                api.util.assertNotNull(toInsert, 'Existing element cannot be null');
                this.el.parentElement.insertBefore(toInsert.el, this.el.nextElementSibling);
            };
            ElementHelper.prototype.getPrevious = function () {
                var previous = this.el.previousSibling;
                while (previous && previous.nodeType != Node.ELEMENT_NODE) {
                    previous = previous.previousSibling;
                }
                return previous ? new ElementHelper(previous) : null;
            };
            ElementHelper.prototype.getNext = function () {
                var next = this.el.nextSibling;
                while (next && next.nodeType != Node.ELEMENT_NODE) {
                    next = next.nextSibling;
                }
                return next ? new ElementHelper(next) : null;
            };
            ElementHelper.prototype.getParent = function () {
                var parent = this.el.parentElement;
                return parent ? new ElementHelper(parent) : null;
            };
            ElementHelper.prototype.setDisabled = function (value) {
                this.el.disabled = value;
                return this;
            };
            ElementHelper.prototype.isDisabled = function () {
                return this.el.disabled;
            };
            ElementHelper.prototype.getId = function () {
                return this.el.id;
            };
            ElementHelper.prototype.setId = function (value) {
                this.el.id = value;
                return this;
            };
            ElementHelper.prototype.simulate = function (value) {
                wemjq(this.el).simulate(value);
                return this;
            };
            ElementHelper.prototype.setInnerHtml = function (value, escapeHtml) {
                if (escapeHtml === void 0) { escapeHtml = false; }
                wemjq(this.el).html(escapeHtml ? api.util.StringHelper.escapeHtml(value) : value);
                return this;
            };
            ElementHelper.prototype.getInnerHtml = function () {
                return this.el.innerHTML;
            };
            ElementHelper.prototype.setText = function (value) {
                wemjq(this.el).text(value);
                return this;
            };
            ElementHelper.prototype.getText = function () {
                return this.el.innerText || this.el.textContent;
            };
            ElementHelper.prototype.setAttribute = function (name, value) {
                this.el.setAttribute(name, value);
                return this;
            };
            ElementHelper.prototype.getAttribute = function (name) {
                return this.el.getAttribute(name);
            };
            ElementHelper.prototype.hasAttribute = function (name) {
                return this.el.hasAttribute(name);
            };
            ElementHelper.prototype.removeAttribute = function (name) {
                this.el.removeAttribute(name);
                return this;
            };
            ElementHelper.prototype.setData = function (name, value) {
                api.util.assert(!api.util.StringHelper.isEmpty(name), 'Name cannot be empty');
                api.util.assert(!api.util.StringHelper.isEmpty(value), 'Value cannot be empty');
                this.el.setAttribute('data-' + name, value);
                wemjq(this.el).data(name, value);
                return this;
            };
            ElementHelper.prototype.getData = function (name) {
                return wemjq(this.el).data(name);
            };
            ElementHelper.prototype.getValue = function () {
                return this.el['value'];
            };
            ElementHelper.prototype.setValue = function (value) {
                this.el['value'] = value;
                return this;
            };
            ElementHelper.prototype.addClass = function (clsName) {
                var _this = this;
                api.util.assert(!api.util.StringHelper.isEmpty(clsName), 'Class name cannot be empty');
                var classList = clsName.split(" ");
                classList.forEach(function (classItem) {
                    if (_this.el.classList && !_this.hasClass(classItem)) {
                        _this.el.classList.add(classItem);
                    }
                });
                return this;
            };
            ElementHelper.prototype.setClass = function (value) {
                this.el.className = value;
                return this;
            };
            ElementHelper.prototype.getClass = function () {
                return this.el.className;
            };
            ElementHelper.prototype.setTitle = function (value) {
                this.el.title = value;
                return this;
            };
            ElementHelper.prototype.getTitle = function () {
                return this.el.title;
            };
            ElementHelper.prototype.hasClass = function (clsName) {
                api.util.assert(!api.util.StringHelper.isEmpty(clsName), 'Class name cannot be empty');
                var classList = clsName.split(" ");
                for (var i = 0; i < classList.length; i++) {
                    var classItem = classList[i];
                    if (!this.el.classList || !this.el.classList.contains(classItem)) {
                        return false;
                    }
                }
                return true;
            };
            ElementHelper.prototype.removeClass = function (clsName) {
                var _this = this;
                api.util.assert(!api.util.StringHelper.isEmpty(clsName), 'Class name cannot be empty');
                var classList = clsName.split(" ");
                classList.forEach(function (classItem) {
                    if (_this.el.classList) {
                        _this.el.classList.remove(classItem);
                    }
                });
                return this;
            };
            ElementHelper.prototype.addEventListener = function (eventName, f) {
                this.el.addEventListener(eventName, f);
                return this;
            };
            ElementHelper.prototype.removeEventListener = function (eventName, f) {
                this.el.removeEventListener(eventName, f);
                return this;
            };
            ElementHelper.prototype.appendChild = function (child) {
                return this.insertChild(child, this.el.children.length);
            };
            ElementHelper.prototype.appendChildren = function (children) {
                var _this = this;
                children.forEach(function (child) {
                    _this.el.appendChild(child);
                });
                return this;
            };
            ElementHelper.prototype.insertChild = function (child, index) {
                if (index > this.el.children.length - 1) {
                    this.el.appendChild(child);
                }
                else {
                    this.el.insertBefore(child, this.el.children.item(index));
                }
                return this;
            };
            ElementHelper.prototype.getTagName = function () {
                return this.el.tagName;
            };
            ElementHelper.prototype.getDisplay = function () {
                return this.el.style.display;
            };
            ElementHelper.prototype.setDisplay = function (value) {
                this.el.style.display = value;
                return this;
            };
            ElementHelper.prototype.getVisibility = function () {
                return this.el.style.visibility;
            };
            ElementHelper.prototype.setVisibility = function (value) {
                this.el.style.visibility = value;
                return this;
            };
            ElementHelper.prototype.getPosition = function () {
                return this.getComputedProperty('position');
            };
            ElementHelper.prototype.setPosition = function (value) {
                this.el.style.position = value;
                return this;
            };
            ElementHelper.prototype.setWidth = function (value) {
                this.el.style.width = value;
                return this;
            };
            ElementHelper.prototype.setWidthPx = function (value) {
                this.setWidth(value + "px");
                return this;
            };
            ElementHelper.prototype.setMaxWidth = function (value) {
                this.el.style.maxWidth = value;
                return this;
            };
            ElementHelper.prototype.setMaxWidthPx = function (value) {
                this.setMaxWidth(value + "px");
                return this;
            };
            ElementHelper.prototype.getWidth = function () {
                return wemjq(this.el).innerWidth();
            };
            ElementHelper.prototype.getWidthWithoutPadding = function () {
                return wemjq(this.el).width();
            };
            ElementHelper.prototype.getWidthWithBorder = function () {
                return wemjq(this.el).outerWidth();
            };
            ElementHelper.prototype.getWidthWithMargin = function () {
                return wemjq(this.el).outerWidth(true);
            };
            ElementHelper.prototype.getMinWidth = function () {
                return parseFloat(this.getComputedProperty('min-width')) || 0;
            };
            ElementHelper.prototype.setHeight = function (value) {
                this.el.style.height = value;
                return this;
            };
            ElementHelper.prototype.setHeightPx = function (value) {
                this.setHeight(value + "px");
                return this;
            };
            ElementHelper.prototype.getHeight = function () {
                return wemjq(this.el).innerHeight();
            };
            ElementHelper.prototype.setMaxHeight = function (value) {
                this.el.style.maxHeight = value;
                return this;
            };
            ElementHelper.prototype.setMaxHeightPx = function (value) {
                this.setMaxHeight(value + "px");
                return this;
            };
            ElementHelper.prototype.getHeightWithoutPadding = function () {
                return wemjq(this.el).height();
            };
            ElementHelper.prototype.getHeightWithBorder = function () {
                return wemjq(this.el).outerHeight();
            };
            ElementHelper.prototype.getHeightWithMargin = function () {
                return wemjq(this.el).outerHeight(true);
            };
            ElementHelper.prototype.setTop = function (value) {
                this.el.style.top = value;
                return this;
            };
            ElementHelper.prototype.setTopPx = function (value) {
                return this.setTop(value + "px");
            };
            ElementHelper.prototype.getTopPx = function () {
                return parseFloat(this.getTop());
            };
            ElementHelper.prototype.getTop = function () {
                return this.el.style.top;
            };
            ElementHelper.prototype.setBottom = function (value) {
                this.el.style.bottom = value;
                return this;
            };
            ElementHelper.prototype.setBottomPx = function (value) {
                return this.setBottom(value + "px");
            };
            ElementHelper.prototype.getLeft = function () {
                return this.el.style.left;
            };
            ElementHelper.prototype.getLeftPx = function () {
                return parseFloat(this.getLeft());
            };
            ElementHelper.prototype.setLeftPx = function (value) {
                return this.setLeft(value + "px");
            };
            ElementHelper.prototype.setLeft = function (value) {
                this.el.style.left = value;
                return this;
            };
            ElementHelper.prototype.setRight = function (value) {
                this.el.style.right = value;
                return this;
            };
            ElementHelper.prototype.setRightPx = function (value) {
                return this.setRight(value + "px");
            };
            ElementHelper.prototype.getMarginLeft = function () {
                return parseFloat(this.getComputedProperty('margin-left')) || 0;
            };
            ElementHelper.prototype.setMarginLeft = function (value) {
                this.el.style.marginLeft = value;
                return this;
            };
            ElementHelper.prototype.getMarginRight = function () {
                return parseFloat(this.getComputedProperty('margin-right'));
            };
            ElementHelper.prototype.setMarginRight = function (value) {
                this.el.style.marginRight = value;
                return this;
            };
            ElementHelper.prototype.getMarginTop = function () {
                return parseFloat(this.getComputedProperty('margin-top'));
            };
            ElementHelper.prototype.setMarginTop = function (value) {
                this.el.style.marginTop = value;
                return this;
            };
            ElementHelper.prototype.getMarginBottom = function () {
                return parseFloat(this.getComputedProperty('margin-bottom'));
            };
            ElementHelper.prototype.setMarginBottom = function (value) {
                this.el.style.marginBottom = value;
                return this;
            };
            ElementHelper.prototype.setStroke = function (value) {
                this.el.style.stroke = value;
                return this;
            };
            ElementHelper.prototype.getStroke = function () {
                return this.getComputedProperty('stroke');
            };
            ElementHelper.prototype.setStrokeDasharray = function (value) {
                this.el.style.strokeDasharray = value;
                return this;
            };
            ElementHelper.prototype.getStrokeDasharray = function () {
                return this.getComputedProperty('stroke-dasharray');
            };
            ElementHelper.prototype.setFill = function (value) {
                this.el.style.fill = value;
                return this;
            };
            ElementHelper.prototype.getFill = function () {
                return this.getComputedProperty('fill');
            };
            ElementHelper.prototype.getPaddingLeft = function () {
                return parseFloat(this.getComputedProperty('padding-left')) || 0;
            };
            ElementHelper.prototype.setPaddingLeft = function (value) {
                this.el.style.paddingLeft = value;
                return this;
            };
            ElementHelper.prototype.getPaddingRight = function () {
                return parseFloat(this.getComputedProperty('padding-right'));
            };
            ElementHelper.prototype.setPaddingRight = function (value) {
                this.el.style.paddingRight = value;
                return this;
            };
            ElementHelper.prototype.getPaddingTop = function () {
                return parseFloat(this.getComputedProperty('padding-top'));
            };
            ElementHelper.prototype.setPaddingTop = function (value) {
                this.el.style.paddingTop = value;
                return this;
            };
            ElementHelper.prototype.getPaddingBottom = function () {
                return parseFloat(this.getComputedProperty('padding-bottom'));
            };
            ElementHelper.prototype.setPaddingBottom = function (value) {
                this.el.style.paddingBottom = value;
                return this;
            };
            ElementHelper.prototype.getBorderTopWidth = function () {
                return parseFloat(this.getComputedProperty('border-top-width'));
            };
            ElementHelper.prototype.getBorderBottomWidth = function () {
                return parseFloat(this.getComputedProperty('border-bottom-width'));
            };
            ElementHelper.prototype.getBorderRightWidth = function () {
                return parseFloat(this.getComputedProperty('border-right-width'));
            };
            ElementHelper.prototype.getBorderLeftWidth = function () {
                return parseFloat(this.getComputedProperty('border-left-width'));
            };
            ElementHelper.prototype.setZindex = function (value) {
                this.el.style.zIndex = value.toString();
                return this;
            };
            ElementHelper.prototype.getBoundingClientRect = function () {
                return this.el.getBoundingClientRect();
            };
            ElementHelper.prototype.scrollIntoView = function (top) {
                this.el.scrollIntoView(top);
                return this;
            };
            ElementHelper.prototype.getScrollTop = function () {
                return this.el.scrollTop;
            };
            ElementHelper.prototype.setScrollTop = function (top) {
                this.el.scrollTop = top;
                return this;
            };
            ElementHelper.prototype.getTabIndex = function () {
                return this.el.tabIndex;
            };
            ElementHelper.prototype.setTabIndex = function (tabIndex) {
                this.el.tabIndex = tabIndex;
                return this;
            };
            ElementHelper.prototype.getFontSize = function () {
                return this.getComputedProperty('font-size');
            };
            ElementHelper.prototype.setFontSize = function (value) {
                this.el.style.fontSize = value;
                return this;
            };
            ElementHelper.prototype.setBackgroundImage = function (value) {
                this.el.style.backgroundImage = value;
                return this;
            };
            ElementHelper.prototype.setCursor = function (value) {
                this.el.style.cursor = value;
                return this;
            };
            ElementHelper.prototype.getCursor = function () {
                return this.el.style.cursor;
            };
            ElementHelper.prototype.getElementsByClassName = function (className) {
                var items = [];
                if (className) {
                    var nodeList = this.el.getElementsByClassName(className);
                    for (var i = 0; i < nodeList.length; i++) {
                        items.push(new ElementHelper(nodeList.item(i)));
                    }
                }
                return items;
            };
            ElementHelper.prototype.remove = function () {
                var parent = this.el.parentElement;
                if (parent) {
                    parent.removeChild(this.el);
                }
            };
            ElementHelper.prototype.contains = function (element) {
                return this.el.contains ? this.el.contains(element) : !!(this.el.compareDocumentPosition(element) & 16);
            };
            ElementHelper.prototype.getOffset = function () {
                return wemjq(this.el).offset();
            };
            ElementHelper.prototype.setOffset = function (offset) {
                wemjq(this.el).offset(offset);
                return this;
            };
            ElementHelper.prototype.getDimensions = function () {
                var offset = this.getOffset();
                return {
                    top: offset.top,
                    left: offset.left,
                    width: this.getWidthWithBorder(),
                    height: this.getHeightWithBorder()
                };
            };
            ElementHelper.prototype.getOffsetParent = function () {
                return wemjq(this.el).offsetParent()[0];
            };
            ElementHelper.prototype.getOffsetToParent = function () {
                return wemjq(this.el).position();
            };
            ElementHelper.prototype.getOffsetTop = function () {
                return this.getOffset().top;
            };
            ElementHelper.prototype.getOffsetTopRelativeToParent = function () {
                return this.el.offsetTop;
            };
            ElementHelper.prototype.getOffsetLeft = function () {
                return this.getOffset().left;
            };
            ElementHelper.prototype.getOffsetLeftRelativeToParent = function () {
                return this.el.offsetLeft;
            };
            ElementHelper.prototype.getComputedProperty = function (name, pseudoElement) {
                if (pseudoElement === void 0) { pseudoElement = null; }
                return window.getComputedStyle(this.el, pseudoElement).getPropertyValue(name);
            };
            ElementHelper.prototype.focus = function () {
                this.el.focus();
            };
            ElementHelper.prototype.blur = function () {
                this.el.blur();
            };
            ElementHelper.prototype.getSiblingIndex = function () {
                var i = 0;
                var prev = this.el;
                while ((prev = prev.previousElementSibling) != null) {
                    i++;
                }
                return i;
            };
            ElementHelper.prototype.isVisible = function () {
                return wemjq(this.el).is(':visible');
            };
            return ElementHelper;
        })();
        dom.ElementHelper = ElementHelper;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ImgHelper = (function (_super) {
            __extends(ImgHelper, _super);
            function ImgHelper(element) {
                _super.call(this, element);
                this.imgEl = element;
            }
            ImgHelper.create = function () {
                return new ImgHelper(document.createElement("img"));
            };
            ImgHelper.prototype.getHTMLElement = function () {
                return this.imgEl;
            };
            ImgHelper.prototype.setSrc = function (value) {
                this.imgEl.src = value;
                return this;
            };
            ImgHelper.prototype.getSrc = function () {
                return this.imgEl.src;
            };
            ImgHelper.prototype.getCurrentSrc = function () {
                return this.imgEl['currentSrc'];
            };
            ImgHelper.prototype.getNaturalWidth = function () {
                return this.imgEl.naturalWidth;
            };
            ImgHelper.prototype.getNaturalHeight = function () {
                return this.imgEl.naturalHeight;
            };
            return ImgHelper;
        })(dom.ElementHelper);
        dom.ImgHelper = ImgHelper;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementBuilder = (function () {
            function ElementBuilder() {
            }
            ElementBuilder.prototype.setGenerateId = function (value) {
                this.generateId = value;
                return this;
            };
            ElementBuilder.prototype.setClassName = function (name) {
                if (name) {
                    name = name.trim().split(/\s+/).filter(function (elem, index, arr) {
                        return arr.indexOf(elem) === index;
                    }).join(" ");
                }
                this.className = name;
                return this;
            };
            ElementBuilder.prototype.setParentElement = function (element) {
                this.parentElement = element;
                return this;
            };
            return ElementBuilder;
        })();
        dom.ElementBuilder = ElementBuilder;
        var ElementFromElementBuilder = (function (_super) {
            __extends(ElementFromElementBuilder, _super);
            function ElementFromElementBuilder() {
                _super.apply(this, arguments);
            }
            ElementFromElementBuilder.prototype.setElement = function (element) {
                this.element = element;
                return this;
            };
            return ElementFromElementBuilder;
        })(ElementBuilder);
        dom.ElementFromElementBuilder = ElementFromElementBuilder;
        var ElementFromHelperBuilder = (function (_super) {
            __extends(ElementFromHelperBuilder, _super);
            function ElementFromHelperBuilder() {
                _super.apply(this, arguments);
            }
            ElementFromHelperBuilder.prototype.setHelper = function (helper) {
                this.helper = helper;
                return this;
            };
            ElementFromHelperBuilder.prototype.setLoadExistingChildren = function (value) {
                this.loadExistingChildren = value;
                return this;
            };
            return ElementFromHelperBuilder;
        })(ElementBuilder);
        dom.ElementFromHelperBuilder = ElementFromHelperBuilder;
        var NewElementBuilder = (function (_super) {
            __extends(NewElementBuilder, _super);
            function NewElementBuilder() {
                _super.apply(this, arguments);
            }
            NewElementBuilder.prototype.setTagName = function (name) {
                api.util.assert(!api.util.StringHelper.isEmpty(name), 'Tag name cannot be empty');
                this.tagName = name;
                return this;
            };
            NewElementBuilder.prototype.setHelper = function (helper) {
                this.helper = helper;
                return this;
            };
            return NewElementBuilder;
        })(ElementBuilder);
        dom.NewElementBuilder = NewElementBuilder;
        var Element = (function () {
            function Element(builder) {
                var _this = this;
                this.addedListeners = [];
                this.removedListeners = [];
                this.renderedListeners = [];
                this.shownListeners = [];
                this.hiddenListeners = [];
                this.resizedListeners = [];
                this.mouseEnterByHandler = {};
                this.mouseLeaveByHandler = {};
                this.children = [];
                this.rendered = false;
                if (api.ObjectHelper.iFrameSafeInstanceOf(builder, ElementFromElementBuilder)) {
                    var fromElementBuilder = builder;
                    var sourceElement = fromElementBuilder.element;
                    if (sourceElement) {
                        this.parentElement = fromElementBuilder.parentElement ? fromElementBuilder.parentElement : sourceElement.parentElement;
                        if (this.parentElement) {
                            this.parentElement.replaceChildElement(this, sourceElement);
                        }
                        this.children = sourceElement.children;
                        this.el = sourceElement.el;
                    }
                }
                else if (api.ObjectHelper.iFrameSafeInstanceOf(builder, ElementFromHelperBuilder)) {
                    var fromHelperBuilder = builder;
                    this.el = fromHelperBuilder.helper;
                    if (fromHelperBuilder.loadExistingChildren) {
                        this.loadExistingChildren();
                    }
                    if (fromHelperBuilder.parentElement) {
                        this.parentElement = fromHelperBuilder.parentElement;
                    }
                }
                else if (api.ObjectHelper.iFrameSafeInstanceOf(builder, NewElementBuilder)) {
                    var newElementBuilder = builder;
                    if (!newElementBuilder.tagName) {
                        throw new Error("tagName cannot be null");
                    }
                    if (newElementBuilder.helper) {
                        this.el = newElementBuilder.helper;
                    }
                    else {
                        this.el = dom.ElementHelper.fromName(newElementBuilder.tagName);
                    }
                    if (newElementBuilder.parentElement) {
                        this.parentElement = newElementBuilder.parentElement;
                    }
                }
                else {
                    throw new Error("Unsupported builder: " + api.ClassHelper.getClassName(builder));
                }
                if (this.parentElement && this.el.getHTMLElement().parentElement) {
                    if (!(this.parentElement.getHTMLElement() === this.el.getHTMLElement().parentElement)) {
                        throw new Error("Illegal state: HTMLElement in parent Element is not the as the HTMLElement parent to this HTMLElement");
                    }
                }
                var distance = api.ClassHelper.distanceTo(this, Element);
                if (builder.generateId || distance > 1) {
                    var id = dom.ElementRegistry.registerElement(this);
                    this.setId(id);
                }
                if (builder.className) {
                    this.setClass(builder.className);
                }
                this.onRemoved(function (event) {
                    if (_this.getId()) {
                        dom.ElementRegistry.unregisterElement(_this);
                    }
                });
            }
            Element.prototype.replaceChildElement = function (replacementChild, existingChild) {
                var index = this.children.indexOf(existingChild);
                this.children[index] = replacementChild;
            };
            Element.prototype.loadExistingChildren = function () {
                var children = this.getHTMLElement().children;
                for (var i = 0; i < children.length; i++) {
                    var child = children[i];
                    var childAsElement = Element.fromHtmlElement(child, true, this);
                    this.children.push(childAsElement);
                }
                return this;
            };
            Element.prototype.findChildById = function (id, deep) {
                if (deep === void 0) { deep = false; }
                for (var i = 0; i < this.children.length; i++) {
                    var child = this.children[i];
                    if (child.getId() == id) {
                        return child;
                    }
                    else if (deep) {
                        var found = child.findChildById(id, deep);
                        if (found) {
                            return found;
                        }
                    }
                }
                return null;
            };
            Element.prototype.init = function () {
                this.children.forEach(function (child) {
                    child.init();
                });
                if (!this.isRendered()) {
                    this.render(false);
                }
                if (this.isVisible()) {
                    this.notifyShown();
                }
            };
            Element.prototype.render = function (deep) {
                if (deep === void 0) { deep = true; }
                if (deep) {
                    this.children.forEach(function (child) {
                        child.render();
                    });
                }
                this.rendered = this.doRender();
                this.notifyRendered();
            };
            Element.prototype.isRendered = function () {
                return this.rendered;
            };
            Element.prototype.doRender = function () {
                return true;
            };
            Element.prototype.show = function () {
                wemjq(this.el.getHTMLElement()).show();
                this.notifyShown(this);
            };
            Element.prototype.hide = function () {
                wemjq(this.el.getHTMLElement()).hide();
                this.notifyHidden(this);
            };
            Element.prototype.setVisible = function (value) {
                if (value) {
                    this.show();
                }
                else {
                    this.hide();
                }
                return this;
            };
            Element.prototype.isVisible = function () {
                return this.el.isVisible();
            };
            Element.prototype.setClass = function (className) {
                api.util.assert(!api.util.StringHelper.isEmpty(className), 'Class name cannot be empty');
                this.el.setClass(className);
                return this;
            };
            Element.prototype.addClass = function (className) {
                api.util.assert(!api.util.StringHelper.isEmpty(className), 'Class name cannot be empty');
                this.el.addClass(className);
                return this;
            };
            Element.prototype.toggleClass = function (className, condition) {
                if (condition == false || condition == undefined && this.hasClass(className)) {
                    this.removeClass(className);
                }
                else {
                    this.addClass(className);
                }
                return this;
            };
            Element.prototype.hasClass = function (className) {
                return this.el.hasClass(className);
            };
            Element.prototype.removeClass = function (className) {
                api.util.assert(!api.util.StringHelper.isEmpty(className), 'Class name cannot be empty');
                this.el.removeClass(className);
                return this;
            };
            Element.prototype.getClass = function () {
                return this.el.getClass();
            };
            Element.prototype.getId = function () {
                return this.el.getId();
            };
            Element.prototype.setId = function (value) {
                this.el.setId(value);
                return this;
            };
            Element.prototype.getEl = function () {
                return this.el;
            };
            Element.prototype.traverse = function (handler) {
                this.getChildren().forEach(function (el) {
                    handler(el);
                    el.traverse(handler);
                });
            };
            Element.prototype.setDraggable = function (value) {
                if (value) {
                    this.getEl().setAttribute("draggable", value.toString());
                }
                else {
                    this.getEl().removeAttribute("draggable");
                }
            };
            Element.prototype.isDraggable = function () {
                return this.getEl().getAttribute('draggable') == 'true';
            };
            Element.prototype.setContentEditable = function (flag) {
                this.getEl().setAttribute('contenteditable', flag ? 'true' : 'false');
                return this;
            };
            Element.prototype.isContentEditable = function () {
                return this.getEl().getAttribute('contenteditable') == 'true';
            };
            Element.prototype.giveFocus = function () {
                if (!this.isVisible()) {
                    return false;
                }
                if (this.el.isDisabled()) {
                    return false;
                }
                this.el.focus();
                var gotFocus = document.activeElement == this.el.getHTMLElement();
                if (!gotFocus) {
                    console.log("Element.giveFocus(): Failed to give focus to Element: class = " + api.ClassHelper.getClassName(this) + ", id = " + this.getId());
                }
                return gotFocus;
            };
            Element.prototype.giveBlur = function () {
                if (!this.isVisible()) {
                    return false;
                }
                if (this.el.isDisabled()) {
                    return false;
                }
                this.el.blur();
                var gotBlur = document.activeElement != this.el.getHTMLElement();
                if (!gotBlur) {
                    console.log("Element.giveBlur(): Failed to give blur to Element: class = " + api.ClassHelper.getClassName(this) + ", id = " + this.getId());
                }
                return gotBlur;
            };
            Element.prototype.getHTMLElement = function () {
                return this.el.getHTMLElement();
            };
            Element.prototype.insertChild = function (child, index) {
                api.util.assertNotNull(child, 'Child cannot be null');
                this.el.insertChild(child.getEl().getHTMLElement(), index);
                this.insertChildElement(child, this, index);
                return this;
            };
            Element.prototype.appendChild = function (child) {
                return this.insertChild(child, this.children.length);
            };
            Element.prototype.appendChildren = function () {
                var _this = this;
                var children = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    children[_i - 0] = arguments[_i];
                }
                children.forEach(function (child) {
                    _this.appendChild(child);
                });
                return this;
            };
            Element.prototype.prependChild = function (child) {
                return this.insertChild(child, 0);
            };
            Element.prototype.insertAfterEl = function (existing) {
                api.util.assertNotNull(existing, 'Existing element cannot be null');
                var existingIndex = existing.getSiblingIndex();
                this.el.insertAfterEl(existing.el);
                return this.insertChildElement(this, existing.parentElement, existingIndex + 1);
            };
            Element.prototype.insertBeforeEl = function (existing) {
                api.util.assertNotNull(existing, 'Existing element cannot be null');
                var existingIndex = existing.getSiblingIndex();
                this.el.insertBeforeEl(existing.el);
                return this.insertChildElement(this, existing.getParentElement(), existingIndex);
            };
            Element.prototype.hasChild = function (child) {
                return this.children.indexOf(child) > -1;
            };
            Element.prototype.removeChild = function (child) {
                api.util.assertNotNull(child, "Child element to remove cannot be null");
                child.getEl().remove();
                this.removeChildElement(child);
                return this;
            };
            Element.prototype.removeChildren = function () {
                this.children.slice(0).forEach(function (child) {
                    child.remove();
                });
                this.el.setInnerHtml('');
                return this;
            };
            Element.prototype.insertChildElement = function (child, parent, index) {
                api.util.assertNotNull(child, 'Child element to insert cannot be null');
                api.util.assertNotNull(parent, 'Parent element cannot be null');
                parent.registerChildElement(child, index);
                if (parent.isRendered()) {
                    child.init();
                }
                child.notifyAdded();
                return this;
            };
            Element.prototype.removeChildElement = function (child) {
                api.util.assertNotNull(child, 'Child element to insert cannot be null');
                this.unregisterChildElement(child);
                child.notifyRemoved(this);
                return this;
            };
            Element.prototype.registerChildElement = function (child, index) {
                if (child.parentElement) {
                    if (child.parentElement != this) {
                        child.parentElement.unregisterChildElement(child);
                    }
                    else if (this.children.indexOf(child) > -1) {
                        return;
                    }
                }
                var parentNode = child.getHTMLElement().parentNode;
                if (parentNode && parentNode !== this.getHTMLElement()) {
                    throw new Error("Given child must be a child of this Element in DOM before it can be registered");
                }
                if (!index) {
                    index = child.el.getSiblingIndex();
                }
                this.children.splice(index, 0, child);
                child.parentElement = this;
            };
            Element.prototype.unregisterChildElement = function (child) {
                var childIndex = this.children.indexOf(child);
                if (childIndex < 0) {
                    throw new Error("Child element to remove not found");
                }
                this.children.splice(childIndex, 1);
                child.parentElement = null;
                return childIndex;
            };
            Element.prototype.contains = function (element) {
                return this.getEl().contains(element.getHTMLElement());
            };
            Element.prototype.remove = function () {
                if (this.parentElement) {
                    this.parentElement.removeChild(this);
                }
                else {
                    this.getEl().remove();
                    this.notifyRemoved(null);
                }
                return this;
            };
            Element.prototype.replaceWith = function (replacement) {
                api.util.assertNotNull(replacement, 'replacement element cannot be null');
                replacement.el.insertAfterEl(this.el);
                replacement.notifyAdded();
                var parent = this.parentElement;
                var index = parent.unregisterChildElement(this);
                parent.registerChildElement(replacement, index);
                if (parent.isRendered()) {
                    replacement.init();
                }
                this.getEl().remove();
                this.notifyRemoved(parent, this);
            };
            Element.prototype.wrapWithElement = function (wrapperElement) {
                api.util.assertNotNull(wrapperElement, 'wrapperElement cannot be null');
                var parent = this.parentElement;
                if (!parent) {
                    return;
                }
                var childPos = parent.children.indexOf(this);
                parent.removeChild(this);
                wrapperElement.appendChild(this);
                parent.el.appendChild(wrapperElement.getEl().getHTMLElement());
                parent.insertChildElement(this, wrapperElement, childPos);
            };
            Element.prototype.getParentElement = function () {
                return this.parentElement;
            };
            Element.prototype.getChildren = function () {
                return this.children;
            };
            Element.prototype.getLastChild = function () {
                return this.children[this.children.length - 1];
            };
            Element.prototype.getFirstChild = function () {
                return this.children[0];
            };
            Element.prototype.getNextElement = function () {
                var nextSiblingHtmlElement = this.getHTMLElement().nextElementSibling;
                if (!nextSiblingHtmlElement) {
                    return null;
                }
                return Element.fromHtmlElement(nextSiblingHtmlElement);
            };
            Element.prototype.getPreviousElement = function () {
                var previousSiblingHtmlElement = this.getHTMLElement().previousElementSibling;
                if (!previousSiblingHtmlElement) {
                    return null;
                }
                return Element.fromHtmlElement(previousSiblingHtmlElement);
            };
            Element.prototype.getSiblingIndex = function () {
                var indexFromDOM = this.el.getSiblingIndex();
                if (this.parentElement) {
                    var indexFromElement = this.parentElement.children.indexOf(this);
                    api.util.assertState(indexFromElement == indexFromDOM, "index of Element in parentElement.children" + " [" + indexFromElement + "] does not correspond with" + " the actual index [" + indexFromDOM + "] of the HTMLElement in DOM");
                }
                return indexFromDOM;
            };
            Element.prototype.toString = function () {
                return wemjq('<div>').append(wemjq(this.getHTMLElement()).clone()).html();
            };
            Element.prototype.getHtml = function () {
                return this.getEl().getInnerHtml();
            };
            Element.prototype.setHtml = function (value, escapeHtml) {
                this.getEl().setInnerHtml(value, escapeHtml);
                return this;
            };
            Element.prototype.onMouseEnter = function (handler) {
                var _this = this;
                if (typeof this.getHTMLElement().onmouseenter != "undefined") {
                    this.getEl().addEventListener('mouseenter', handler);
                }
                else {
                    this.mouseEnterByHandler[handler] = function (e) {
                        if (!_this.getEl().contains((e.relatedTarget || e.fromElement))) {
                            handler(e);
                        }
                    };
                    this.getEl().addEventListener('mouseover', this.mouseEnterByHandler[handler]);
                }
            };
            Element.prototype.unMouseEnter = function (handler) {
                if (typeof this.getHTMLElement().onmouseenter != "undefined") {
                    this.getEl().removeEventListener('mouseenter', handler);
                }
                else {
                    this.getEl().removeEventListener('mouseover', this.mouseEnterByHandler[handler]);
                }
            };
            Element.prototype.onMouseLeave = function (handler) {
                var _this = this;
                if (typeof this.getHTMLElement().onmouseleave != "undefined") {
                    this.getEl().addEventListener('mouseleave', handler);
                }
                else {
                    this.mouseLeaveByHandler[handler] = function (e) {
                        if (!_this.getEl().contains((e.relatedTarget || e.toElement))) {
                            handler(e);
                        }
                    };
                    this.getEl().addEventListener('mouseout', this.mouseLeaveByHandler[handler]);
                }
            };
            Element.prototype.unMouseLeave = function (handler) {
                if (typeof this.getHTMLElement().onmouseleave != "undefined") {
                    this.getEl().removeEventListener('mouseleave', handler);
                }
                else {
                    this.getEl().removeEventListener('mouseout', this.mouseLeaveByHandler[handler]);
                }
            };
            Element.prototype.onMouseOver = function (listener) {
                this.getEl().addEventListener('mouseover', listener);
            };
            Element.prototype.unMouseOver = function (listener) {
                this.getEl().removeEventListener("mouseover", listener);
            };
            Element.prototype.onMouseOut = function (listener) {
                this.getEl().addEventListener('mouseout', listener);
            };
            Element.prototype.unMouseOut = function (listener) {
                this.getEl().removeEventListener("mouseout", listener);
            };
            Element.prototype.onAdded = function (listener) {
                this.addedListeners.push(listener);
            };
            Element.prototype.unAdded = function (listener) {
                this.addedListeners = this.addedListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            Element.prototype.notifyAdded = function () {
                var addedEvent = new dom.ElementAddedEvent(this);
                this.addedListeners.forEach(function (listener) {
                    listener(addedEvent);
                });
            };
            Element.prototype.onRemoved = function (listener) {
                this.removedListeners.push(listener);
            };
            Element.prototype.unRemoved = function (listener) {
                this.removedListeners = this.removedListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            Element.prototype.notifyRemoved = function (parent, target) {
                var removedEvent = new dom.ElementRemovedEvent(this, parent, target);
                this.removedListeners.forEach(function (listener) {
                    listener(removedEvent);
                });
                this.children.forEach(function (child) {
                    child.notifyRemoved(removedEvent.getParent(), removedEvent.getTarget());
                });
            };
            Element.prototype.onRendered = function (listener) {
                this.renderedListeners.push(listener);
            };
            Element.prototype.unRendered = function (listener) {
                this.renderedListeners = this.renderedListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            Element.prototype.notifyRendered = function () {
                var renderedEvent = new dom.ElementRenderedEvent(this);
                this.renderedListeners.forEach(function (listener) {
                    listener(renderedEvent);
                });
            };
            Element.prototype.onShown = function (listener) {
                this.shownListeners.push(listener);
            };
            Element.prototype.unShown = function (listener) {
                this.shownListeners = this.shownListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            Element.prototype.notifyShown = function (target) {
                var shownEvent = new dom.ElementShownEvent(this, target);
                this.shownListeners.forEach(function (listener) {
                    listener(shownEvent);
                });
                this.children.forEach(function (child) {
                    if (child.isVisible()) {
                        child.notifyShown(shownEvent.getTarget());
                    }
                });
            };
            Element.prototype.onHidden = function (listener) {
                this.hiddenListeners.push(listener);
            };
            Element.prototype.unHidden = function (listener) {
                this.hiddenListeners = this.hiddenListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            Element.prototype.notifyHidden = function (target) {
                var hiddenEvent = new dom.ElementHiddenEvent(this, target);
                this.hiddenListeners.forEach(function (listener) {
                    listener(hiddenEvent);
                });
                this.children.forEach(function (child) {
                    child.notifyHidden(hiddenEvent.getTarget());
                });
            };
            Element.prototype.onScrolled = function (listener) {
                this.getEl().addEventListener("mousewheel", listener);
                this.getEl().addEventListener("DOMMouseScroll", listener);
            };
            Element.prototype.unScrolled = function (listener) {
                this.getEl().removeEventListener("mousewheel", listener);
                this.getEl().removeEventListener("DOMMouseScroll", listener);
            };
            Element.prototype.onClicked = function (listener) {
                this.getEl().addEventListener("click", listener);
            };
            Element.prototype.unClicked = function (listener) {
                this.getEl().removeEventListener("click", listener);
            };
            Element.prototype.onDblClicked = function (listener) {
                this.getEl().addEventListener("dblclick", listener);
            };
            Element.prototype.unDblClicked = function (listener) {
                this.getEl().removeEventListener("dblclick", listener);
            };
            Element.prototype.onContextMenu = function (listener) {
                this.getEl().addEventListener("contextmenu", listener);
            };
            Element.prototype.unContextMenu = function (listener) {
                this.getEl().removeEventListener("contextmenu", listener);
            };
            Element.prototype.onMouseDown = function (listener) {
                this.getEl().addEventListener("mousedown", listener);
            };
            Element.prototype.unMouseDown = function (listener) {
                this.getEl().removeEventListener("mousedown", listener);
            };
            Element.prototype.onMouseUp = function (listener) {
                this.getEl().addEventListener("mouseup", listener);
            };
            Element.prototype.unMouseUp = function (listener) {
                this.getEl().removeEventListener("mouseup", listener);
            };
            Element.prototype.onMouseMove = function (listener) {
                this.getEl().addEventListener("mousemove", listener);
            };
            Element.prototype.unMouseMove = function (listener) {
                this.getEl().removeEventListener("mousemove", listener);
            };
            Element.prototype.onMouseWheel = function (listener) {
                var eventName = (/Firefox/i.test(navigator.userAgent)) ? "wheel" : "mousewheel";
                this.getEl().addEventListener(eventName, listener);
            };
            Element.prototype.unMouseWheel = function (listener) {
                var eventName = (/Firefox/i.test(navigator.userAgent)) ? "wheel" : "mousewheel";
                this.getEl().removeEventListener(eventName, listener);
            };
            Element.prototype.onTouchStart = function (listener) {
                this.getEl().addEventListener("touchstart", listener);
            };
            Element.prototype.unTouchStart = function (listener) {
                this.getEl().removeEventListener("touchstart", listener);
            };
            Element.prototype.onKeyUp = function (listener) {
                this.getEl().addEventListener("keyup", listener);
            };
            Element.prototype.unKeyUp = function (listener) {
                this.getEl().removeEventListener("keyup", listener);
            };
            Element.prototype.onKeyDown = function (listener) {
                this.getEl().addEventListener("keydown", listener);
            };
            Element.prototype.unKeyDown = function (listener) {
                this.getEl().removeEventListener("keydown", listener);
            };
            Element.prototype.onKeyPressed = function (listener) {
                this.getEl().addEventListener("keypress", listener);
            };
            Element.prototype.unKeyPressed = function (listener) {
                this.getEl().removeEventListener("keypress", listener);
            };
            Element.prototype.onFocus = function (listener) {
                this.getEl().addEventListener("focus", listener);
            };
            Element.prototype.unFocus = function (listener) {
                this.getEl().removeEventListener("focus", listener);
            };
            Element.prototype.onBlur = function (listener) {
                this.getEl().addEventListener("blur", listener);
            };
            Element.prototype.unBlur = function (listener) {
                this.getEl().removeEventListener("blur", listener);
            };
            Element.prototype.onScroll = function (listener) {
                this.getEl().addEventListener("scroll", listener);
            };
            Element.prototype.unScroll = function (listener) {
                this.getEl().removeEventListener("scroll", listener);
            };
            Element.prototype.onDrag = function (listener) {
                this.getEl().addEventListener("drag", listener);
            };
            Element.prototype.unDrag = function (listener) {
                this.getEl().removeEventListener("drag", listener);
            };
            Element.prototype.onDragStart = function (listener) {
                this.getEl().addEventListener("dragstart", listener);
            };
            Element.prototype.unDragStart = function (listener) {
                this.getEl().removeEventListener("dragstart", listener);
            };
            Element.prototype.onDragEnter = function (listener) {
                this.getEl().addEventListener("dragenter", listener);
            };
            Element.prototype.unDragEnter = function (listener) {
                this.getEl().removeEventListener("dragenter", listener);
            };
            Element.prototype.onDragOver = function (listener) {
                this.getEl().addEventListener("dragover", listener);
            };
            Element.prototype.unDragOver = function (listener) {
                this.getEl().removeEventListener("dragover", listener);
            };
            Element.prototype.onDragOut = function (listener) {
                this.getEl().addEventListener("dragout", listener);
            };
            Element.prototype.unDragOut = function (listener) {
                this.getEl().removeEventListener("dragout", listener);
            };
            Element.prototype.onDragLeave = function (listener) {
                this.getEl().addEventListener("dragleave", listener);
            };
            Element.prototype.unDragLeave = function (listener) {
                this.getEl().removeEventListener("dragleave", listener);
            };
            Element.prototype.onDrop = function (listener) {
                this.getEl().addEventListener("drop", listener);
            };
            Element.prototype.unDrop = function (listener) {
                this.getEl().removeEventListener("drop", listener);
            };
            Element.prototype.onDragEnd = function (listener) {
                this.getEl().addEventListener("dragend", listener);
            };
            Element.prototype.unDragEnd = function (listener) {
                this.getEl().removeEventListener("dragend", listener);
            };
            Element.fromHtmlElement = function (element, loadExistingChildren, parent) {
                if (loadExistingChildren === void 0) { loadExistingChildren = false; }
                return new Element(new ElementFromHelperBuilder().setHelper(new dom.ElementHelper(element)).setLoadExistingChildren(loadExistingChildren).setParentElement(parent));
            };
            Element.fromString = function (s, loadExistingChildren) {
                if (loadExistingChildren === void 0) { loadExistingChildren = true; }
                var htmlEl = wemjq(s).get(0);
                var parentEl;
                if (htmlEl && htmlEl.parentElement) {
                    parentEl = Element.fromHtmlElement(htmlEl.parentElement);
                }
                return this.fromHtmlElement(htmlEl, loadExistingChildren, parentEl);
            };
            Element.fromSelector = function (s, loadExistingChildren) {
                if (loadExistingChildren === void 0) { loadExistingChildren = true; }
                return wemjq(s).map(function (index, elem) {
                    var htmlEl = elem, parentEl;
                    if (htmlEl && htmlEl.parentElement) {
                        parentEl = Element.fromHtmlElement(htmlEl.parentElement);
                    }
                    return Element.fromHtmlElement(htmlEl, loadExistingChildren, parentEl);
                }).get();
            };
            Element.debug = true;
            return Element;
        })();
        dom.Element = Element;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementRegistry = (function () {
            function ElementRegistry() {
            }
            ElementRegistry.registerElement = function (el) {
                var fullName, id = el.getId();
                if (!id) {
                    id = fullName = api.ClassHelper.getFullName(el);
                }
                else {
                    fullName = id;
                }
                var count = ElementRegistry.counters[fullName];
                if (count >= 0) {
                    id += '-' + (++count);
                }
                ElementRegistry.counters[fullName] = count || 0;
                ElementRegistry.elements[id] = el;
                return id;
            };
            ElementRegistry.unregisterElement = function (el) {
                if (el) {
                    delete ElementRegistry.elements[el.getId()];
                }
            };
            ElementRegistry.getElementById = function (id) {
                return ElementRegistry.elements[id];
            };
            ElementRegistry.counters = {};
            ElementRegistry.elements = {};
            return ElementRegistry;
        })();
        dom.ElementRegistry = ElementRegistry;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementEvent = (function (_super) {
            __extends(ElementEvent, _super);
            function ElementEvent(name, element, target) {
                _super.call(this, name);
                this.element = element;
                this.target = target || element;
            }
            ElementEvent.prototype.getElement = function () {
                return this.element;
            };
            ElementEvent.prototype.getTarget = function () {
                return this.target;
            };
            return ElementEvent;
        })(api.event.Event);
        dom.ElementEvent = ElementEvent;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementAddedEvent = (function (_super) {
            __extends(ElementAddedEvent, _super);
            function ElementAddedEvent(element, target) {
                _super.call(this, "added", element, target);
            }
            return ElementAddedEvent;
        })(dom.ElementEvent);
        dom.ElementAddedEvent = ElementAddedEvent;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementRenderedEvent = (function (_super) {
            __extends(ElementRenderedEvent, _super);
            function ElementRenderedEvent(element, target) {
                _super.call(this, "rendered", element, target);
            }
            return ElementRenderedEvent;
        })(dom.ElementEvent);
        dom.ElementRenderedEvent = ElementRenderedEvent;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementShownEvent = (function (_super) {
            __extends(ElementShownEvent, _super);
            function ElementShownEvent(element, target) {
                _super.call(this, "shown", element, target);
            }
            return ElementShownEvent;
        })(dom.ElementEvent);
        dom.ElementShownEvent = ElementShownEvent;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementHiddenEvent = (function (_super) {
            __extends(ElementHiddenEvent, _super);
            function ElementHiddenEvent(element, target) {
                _super.call(this, "hidden", element, target);
            }
            return ElementHiddenEvent;
        })(dom.ElementEvent);
        dom.ElementHiddenEvent = ElementHiddenEvent;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementRemovedEvent = (function (_super) {
            __extends(ElementRemovedEvent, _super);
            function ElementRemovedEvent(element, parent, target) {
                _super.call(this, "removed", element, target);
                this.parent = parent;
            }
            ElementRemovedEvent.prototype.getParent = function () {
                return this.parent;
            };
            return ElementRemovedEvent;
        })(dom.ElementEvent);
        dom.ElementRemovedEvent = ElementRemovedEvent;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ElementResizedEvent = (function (_super) {
            __extends(ElementResizedEvent, _super);
            function ElementResizedEvent(newWidth, newHeight, element, target) {
                _super.call(this, "resized", element, target);
                this.newWidth = newWidth;
                this.newHeight = newHeight;
            }
            ElementResizedEvent.prototype.getNewWidth = function () {
                return this.newWidth;
            };
            ElementResizedEvent.prototype.getNewHeight = function () {
                return this.newHeight;
            };
            return ElementResizedEvent;
        })(dom.ElementEvent);
        dom.ElementResizedEvent = ElementResizedEvent;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var AEl = (function (_super) {
            __extends(AEl, _super);
            function AEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("a").setClassName(className));
                this.setUrl('#');
            }
            AEl.prototype.setUrl = function (value, target) {
                this.getEl().setAttribute('href', value);
                if (target) {
                    this.getEl().setAttribute('target', target);
                }
                return this;
            };
            AEl.prototype.setTitle = function (value) {
                this.getEl().setTitle(value);
                return this;
            };
            AEl.prototype.getTitle = function () {
                return this.getEl().getTitle();
            };
            AEl.prototype.getHref = function () {
                return this.getEl().getAttribute('href');
            };
            AEl.prototype.getTarget = function () {
                return this.getEl().getAttribute('target');
            };
            AEl.prototype.getText = function () {
                return this.getEl().getText();
            };
            return AEl;
        })(dom.Element);
        dom.AEl = AEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var BrEl = (function (_super) {
            __extends(BrEl, _super);
            function BrEl() {
                _super.call(this, new dom.NewElementBuilder().setTagName("br"));
            }
            return BrEl;
        })(dom.Element);
        dom.BrEl = BrEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var IEl = (function (_super) {
            __extends(IEl, _super);
            function IEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("i").setClassName(className));
            }
            return IEl;
        })(dom.Element);
        dom.IEl = IEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var Body = (function (_super) {
            __extends(Body, _super);
            function Body(loadExistingChildren, body) {
                if (loadExistingChildren === void 0) { loadExistingChildren = false; }
                if (!body) {
                    body = document.body;
                }
                var html = dom.Element.fromHtmlElement(body.parentElement);
                _super.call(this, new dom.ElementFromHelperBuilder().setHelper(new dom.ElementHelper(body)).setLoadExistingChildren(loadExistingChildren));
                html.appendChild(this);
                this.init();
                this.childrenLoaded = loadExistingChildren;
            }
            Body.get = function () {
                if (!Body.instance) {
                    Body.instance = new Body();
                }
                return Body.instance;
            };
            Body.prototype.isChildrenLoaded = function () {
                return this.childrenLoaded;
            };
            Body.prototype.loadExistingChildren = function () {
                if (!this.isChildrenLoaded()) {
                    _super.prototype.loadExistingChildren.call(this);
                    this.childrenLoaded = true;
                }
                return this;
            };
            return Body;
        })(dom.Element);
        dom.Body = Body;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var DivEl = (function (_super) {
            __extends(DivEl, _super);
            function DivEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("div").setClassName(className));
            }
            return DivEl;
        })(dom.Element);
        dom.DivEl = DivEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var H1El = (function (_super) {
            __extends(H1El, _super);
            function H1El(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("h1").setClassName(className));
            }
            return H1El;
        })(dom.Element);
        dom.H1El = H1El;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var H2El = (function (_super) {
            __extends(H2El, _super);
            function H2El(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("h2").setClassName(className));
            }
            return H2El;
        })(dom.Element);
        dom.H2El = H2El;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var H3El = (function (_super) {
            __extends(H3El, _super);
            function H3El(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("h3").setClassName(className));
            }
            return H3El;
        })(dom.Element);
        dom.H3El = H3El;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var H4El = (function (_super) {
            __extends(H4El, _super);
            function H4El(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("h4").setClassName(className));
            }
            return H4El;
        })(dom.Element);
        dom.H4El = H4El;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var H5El = (function (_super) {
            __extends(H5El, _super);
            function H5El(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("h5").setClassName(className));
            }
            return H5El;
        })(dom.Element);
        dom.H5El = H5El;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var H6El = (function (_super) {
            __extends(H6El, _super);
            function H6El(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("h6").setClassName(className));
            }
            return H6El;
        })(dom.Element);
        dom.H6El = H6El;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var UlEl = (function (_super) {
            __extends(UlEl, _super);
            function UlEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("ul").setClassName(className));
            }
            return UlEl;
        })(dom.Element);
        dom.UlEl = UlEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var LiEl = (function (_super) {
            __extends(LiEl, _super);
            function LiEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("li").setClassName(className));
            }
            return LiEl;
        })(dom.Element);
        dom.LiEl = LiEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var EmEl = (function (_super) {
            __extends(EmEl, _super);
            function EmEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("em").setClassName(className));
            }
            return EmEl;
        })(dom.Element);
        dom.EmEl = EmEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ImgEl = (function (_super) {
            __extends(ImgEl, _super);
            function ImgEl(src, className) {
                var _this = this;
                _super.call(this, new dom.NewElementBuilder().setTagName("img").setHelper(dom.ImgHelper.create()).setClassName(className));
                this.loadedListeners = [];
                this.getEl().setSrc(src ? src : ImgEl.PLACEHOLDER);
                this.onImgElLoaded(function (event) {
                    _this.loaded = true;
                    if (ImgEl.debug) {
                        console.log('ImgEl.onLoaded', _this.getSrc(), _this.loaded.toString());
                    }
                    _this.notifyLoaded(event);
                });
            }
            ImgEl.prototype.refresh = function () {
                this.setSrc(this.getSrc());
            };
            ImgEl.prototype.getSrc = function () {
                return this.getEl().getSrc();
            };
            ImgEl.prototype.getCurrentSrc = function () {
                return this.getEl().getCurrentSrc();
            };
            ImgEl.prototype.setSrc = function (source) {
                this.loaded = false;
                if (ImgEl.debug) {
                    console.log('ImgEl.setSrc', this.getSrc(), source, this.loaded.toString());
                }
                this.getEl().setSrc(source);
            };
            ImgEl.prototype.getEl = function () {
                return _super.prototype.getEl.call(this);
            };
            ImgEl.prototype.onLoaded = function (listener) {
                this.loadedListeners.push(listener);
            };
            ImgEl.prototype.unLoaded = function (listener) {
                this.loadedListeners = this.loadedListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            ImgEl.prototype.notifyLoaded = function (event) {
                this.loadedListeners.forEach(function (listener) { return listener(event); });
            };
            ImgEl.prototype.onImgElLoaded = function (listener) {
                this.getEl().addEventListener("load", listener);
            };
            ImgEl.prototype.isLoaded = function () {
                return this.loaded;
            };
            ImgEl.prototype.isPlaceholder = function () {
                return this.getCurrentSrc() == ImgEl.PLACEHOLDER;
            };
            ImgEl.debug = false;
            ImgEl.PLACEHOLDER = "data:image/gif;base64,R0lGODlhAQABAIAAAP///////yH5BAEKAAEALAAAAAABAAEAAAICTAEAOw==";
            return ImgEl;
        })(dom.Element);
        dom.ImgEl = ImgEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var SpanEl = (function (_super) {
            __extends(SpanEl, _super);
            function SpanEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("span").setClassName(className));
            }
            return SpanEl;
        })(dom.Element);
        dom.SpanEl = SpanEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var FormItemEl = (function (_super) {
            __extends(FormItemEl, _super);
            function FormItemEl(tagName, className) {
                _super.call(this, new dom.NewElementBuilder().setTagName(tagName).setClassName(className));
                this.validityChangedListeners = [];
            }
            FormItemEl.prototype.getName = function () {
                return this.getEl().getAttribute("name");
            };
            FormItemEl.prototype.setName = function (name) {
                this.getEl().setAttribute("name", name);
                return this;
            };
            FormItemEl.prototype.onValidityChanged = function (listener) {
                this.validityChangedListeners.push(listener);
            };
            FormItemEl.prototype.unValidityChanged = function (listener) {
                this.validityChangedListeners = this.validityChangedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            FormItemEl.prototype.notifyValidityChanged = function (valid) {
                var _this = this;
                this.validityChangedListeners.forEach(function (listener) {
                    listener.call(_this, new api.ValidityChangedEvent(valid));
                });
            };
            return FormItemEl;
        })(dom.Element);
        dom.FormItemEl = FormItemEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ButtonEl = (function (_super) {
            __extends(ButtonEl, _super);
            function ButtonEl(className) {
                _super.call(this, "button", className);
            }
            return ButtonEl;
        })(dom.FormItemEl);
        dom.ButtonEl = ButtonEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var PEl = (function (_super) {
            __extends(PEl, _super);
            function PEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("p").setClassName(className));
            }
            return PEl;
        })(dom.Element);
        dom.PEl = PEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var FormInputEl = (function (_super) {
            __extends(FormInputEl, _super);
            function FormInputEl(tagName, className) {
                _super.call(this, tagName, className);
                this.addClass('form-input');
            }
            FormInputEl.prototype.getValue = function () {
                return this.getEl().getValue();
            };
            FormInputEl.prototype.setValue = function (value) {
                this.getEl().setValue(value);
                return this;
            };
            FormInputEl.prototype.onChange = function (listener) {
                this.getEl().addEventListener("change", listener);
            };
            FormInputEl.prototype.unChange = function (listener) {
                this.getEl().removeEventListener("change", listener);
            };
            FormInputEl.prototype.onInput = function (listener) {
                this.getEl().addEventListener("input", listener);
            };
            FormInputEl.prototype.unInput = function (listener) {
                this.getEl().removeEventListener("input", listener);
            };
            return FormInputEl;
        })(dom.FormItemEl);
        dom.FormInputEl = FormInputEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var CompositeFormInputEl = (function (_super) {
            __extends(CompositeFormInputEl, _super);
            function CompositeFormInputEl(wrappedInput) {
                var _this = this;
                var additionalElements = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    additionalElements[_i - 1] = arguments[_i];
                }
                _super.call(this, "div");
                this.additionalElements = [];
                this.addClass("composite-input");
                this.wrappedInput = wrappedInput;
                this.appendChild(this.wrappedInput);
                if (additionalElements) {
                    additionalElements.forEach(function (element) {
                        _this.addAdditionalElement(element);
                    });
                }
            }
            CompositeFormInputEl.prototype.getValue = function () {
                return this.wrappedInput.getValue();
            };
            CompositeFormInputEl.prototype.getName = function () {
                return this.wrappedInput.getName();
            };
            CompositeFormInputEl.prototype.setValue = function (value) {
                this.wrappedInput.setValue(value);
                return this;
            };
            CompositeFormInputEl.prototype.addAdditionalElement = function (element) {
                this.appendChild(element);
                this.additionalElements.push(element);
            };
            return CompositeFormInputEl;
        })(api.dom.FormInputEl);
        dom.CompositeFormInputEl = CompositeFormInputEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var InputEl = (function (_super) {
            __extends(InputEl, _super);
            function InputEl(className, type) {
                _super.call(this, "input", className);
                this.setType(type || 'text');
            }
            InputEl.prototype.getValue = function () {
                return this.getEl().getValue();
            };
            InputEl.prototype.setValue = function (value) {
                this.getEl().setValue(value);
                return this;
            };
            InputEl.prototype.getName = function () {
                return this.getEl().getAttribute('name');
            };
            InputEl.prototype.setName = function (value) {
                this.getEl().setAttribute('name', value);
                return this;
            };
            InputEl.prototype.getType = function () {
                return this.getEl().getAttribute('type');
            };
            InputEl.prototype.setType = function (type) {
                this.getEl().setAttribute('type', type);
                return this;
            };
            InputEl.prototype.setPlaceholder = function (value) {
                this.getEl().setAttribute('placeholder', value);
                return this;
            };
            InputEl.prototype.getPlaceholder = function () {
                return this.getEl().getAttribute('placeholder');
            };
            InputEl.prototype.getPattern = function () {
                return this.getEl().getAttribute('pattern');
            };
            InputEl.prototype.setPattern = function (pattern) {
                this.getEl().setAttribute('pattern', pattern);
                return this;
            };
            InputEl.prototype.reset = function () {
                this.getEl().setValue("");
            };
            InputEl.prototype.isValid = function () {
                var validity = this.getHTMLElement().validity;
                return validity && validity.valid;
            };
            InputEl.prototype.validate = function () {
                return this.getHTMLElement().checkValidity();
            };
            InputEl.prototype.setRequired = function (required) {
                if (required) {
                    this.getEl().setAttribute('required', 'required');
                }
                else {
                    this.getEl().removeAttribute('required');
                }
                return this;
            };
            InputEl.prototype.isRequired = function () {
                return this.getEl().hasAttribute('required');
            };
            InputEl.prototype.setReadOnly = function (readOnly) {
                if (readOnly) {
                    this.getEl().setAttribute('readonly', 'readonly');
                }
                else {
                    this.getEl().removeAttribute('readonly');
                }
                return this;
            };
            InputEl.prototype.isReadOnly = function () {
                return this.getEl().hasAttribute('readonly');
            };
            return InputEl;
        })(dom.FormInputEl);
        dom.InputEl = InputEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var LabelEl = (function (_super) {
            __extends(LabelEl, _super);
            function LabelEl(value, forElement, className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("label").setClassName(className));
                this.setValue(value);
                if (forElement) {
                    this.getEl().setAttribute("for", forElement.getId());
                }
            }
            LabelEl.prototype.setValue = function (value) {
                this.getEl().setInnerHtml(value);
            };
            LabelEl.prototype.getValue = function () {
                return this.getEl().getInnerHtml();
            };
            return LabelEl;
        })(dom.Element);
        dom.LabelEl = LabelEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var SelectEl = (function (_super) {
            __extends(SelectEl, _super);
            function SelectEl(className) {
                _super.call(this, "select", className);
            }
            SelectEl.prototype.setName = function (value) {
                this.getEl().setAttribute('name', value);
                return this;
            };
            return SelectEl;
        })(dom.FormInputEl);
        dom.SelectEl = SelectEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var OptionEl = (function (_super) {
            __extends(OptionEl, _super);
            function OptionEl(value, displayName) {
                _super.call(this, new dom.NewElementBuilder().setTagName("option"));
                this.getEl().setValue(value);
                this.getEl().setInnerHtml(displayName);
            }
            return OptionEl;
        })(dom.Element);
        dom.OptionEl = OptionEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var IFrameEl = (function (_super) {
            __extends(IFrameEl, _super);
            function IFrameEl(className) {
                var _this = this;
                _super.call(this, new dom.NewElementBuilder().setTagName("iframe").setClassName(className));
                this.loaded = false;
                this.onLoaded(function (event) { return _this.loaded = true; });
            }
            IFrameEl.prototype.setSrc = function (src) {
                this.getEl().setAttribute("src", src);
                return this;
            };
            IFrameEl.prototype.isLoaded = function () {
                return this.loaded;
            };
            IFrameEl.prototype.postMessage = function (data, targetOrigin) {
                if (targetOrigin === void 0) { targetOrigin = "*"; }
                var thisIFrameElement = this.getHTMLElement();
                thisIFrameElement.contentWindow.postMessage(data, targetOrigin);
            };
            IFrameEl.prototype.onLoaded = function (listener) {
                this.getEl().addEventListener("load", listener);
            };
            IFrameEl.prototype.unLoaded = function (listener) {
                this.getEl().removeEventListener("load", listener);
            };
            return IFrameEl;
        })(api.dom.Element);
        dom.IFrameEl = IFrameEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var FieldsetEl = (function (_super) {
            __extends(FieldsetEl, _super);
            function FieldsetEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("fieldset").setClassName(className));
            }
            return FieldsetEl;
        })(dom.Element);
        dom.FieldsetEl = FieldsetEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var LegendEl = (function (_super) {
            __extends(LegendEl, _super);
            function LegendEl(legend, className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("legend").setClassName(className));
                this.getEl().setInnerHtml(legend);
            }
            return LegendEl;
        })(dom.Element);
        dom.LegendEl = LegendEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var FormEl = (function (_super) {
            __extends(FormEl, _super);
            function FormEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("form").setClassName(className));
            }
            FormEl.prototype.preventSubmit = function () {
                this.onSubmit(function (event) {
                    event.preventDefault();
                });
            };
            FormEl.prototype.onSubmit = function (listener) {
                this.getEl().addEventListener("submit", listener);
            };
            FormEl.prototype.unSubmit = function (listener) {
                this.getEl().removeEventListener("submit", listener);
            };
            FormEl.moveFocusToNextFocusable = function (input, focusableSelector) {
                var focusableElements = document.querySelectorAll(focusableSelector ? focusableSelector : "input, button, select");
                var index = FormEl.getIndexOfInput(focusableElements, input);
                if (index < 0) {
                    return;
                }
                for (var i = index + 1; i < focusableElements.length; i++) {
                    var nextFocusable = api.dom.Element.fromHtmlElement(focusableElements.item(i));
                    if (nextFocusable.getEl().getTabIndex() && nextFocusable.getEl().getTabIndex() < 0) {
                        continue;
                    }
                    else {
                        nextFocusable.giveFocus();
                        return;
                    }
                }
            };
            FormEl.moveFocusToPrevFocusable = function (input, focusableSelector) {
                var focusableElements = document.querySelectorAll(focusableSelector ? focusableSelector : "input, button, select");
                var index = FormEl.getIndexOfInput(focusableElements, input);
                do {
                    index = index - 1;
                    if (0 <= index) {
                        var nextFocusable = api.dom.Element.fromHtmlElement(focusableElements.item(index));
                    }
                } while (nextFocusable.getEl().getTabIndex() && nextFocusable.getEl().getTabIndex() < 0);
                nextFocusable.giveFocus();
                return;
            };
            FormEl.getIndexOfInput = function (elements, el) {
                var index = -1;
                var inputHTMLElement = el.getHTMLElement();
                for (var i = 0; i < elements.length; i++) {
                    if (inputHTMLElement == elements.item(i)) {
                        index = i;
                        break;
                    }
                }
                return index;
            };
            return FormEl;
        })(dom.Element);
        dom.FormEl = FormEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var AsideEl = (function (_super) {
            __extends(AsideEl, _super);
            function AsideEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("aside").setClassName(className));
            }
            return AsideEl;
        })(dom.Element);
        dom.AsideEl = AsideEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var SectionEl = (function (_super) {
            __extends(SectionEl, _super);
            function SectionEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("section").setClassName(className));
            }
            return SectionEl;
        })(dom.Element);
        dom.SectionEl = SectionEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var ArticleEl = (function (_super) {
            __extends(ArticleEl, _super);
            function ArticleEl(className, contentEditable) {
                _super.call(this, new dom.NewElementBuilder().setTagName("article").setClassName(className));
                this.setContentEditable(contentEditable);
            }
            return ArticleEl;
        })(dom.Element);
        dom.ArticleEl = ArticleEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var DlEl = (function (_super) {
            __extends(DlEl, _super);
            function DlEl(className) {
                _super.call(this, new dom.NewElementBuilder().setTagName("dl").setClassName(className));
            }
            return DlEl;
        })(dom.Element);
        dom.DlEl = DlEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var dom;
    (function (dom) {
        var DdDtEl = (function (_super) {
            __extends(DdDtEl, _super);
            function DdDtEl(tag, className) {
                _super.call(this, this.getElementBuilder(tag).setClassName(className));
            }
            DdDtEl.prototype.getElementBuilder = function (tag) {
                var builder = new dom.NewElementBuilder();
                if (tag == "dt" || tag == "dd") {
                    builder.setTagName(tag);
                }
                return builder;
            };
            return DdDtEl;
        })(dom.Element);
        dom.DdDtEl = DdDtEl;
    })(dom = api.dom || (api.dom = {}));
})(api || (api = {}));
var api;
(function (api) {
    var notify;
    (function (notify) {
        (function (Type) {
            Type[Type["INFO"] = 0] = "INFO";
            Type[Type["ERROR"] = 1] = "ERROR";
            Type[Type["WARNING"] = 2] = "WARNING";
            Type[Type["ACTION"] = 3] = "ACTION";
            Type[Type["SUCCESS"] = 4] = "SUCCESS";
        })(notify.Type || (notify.Type = {}));
        var Type = notify.Type;
        var Action = (function () {
            function Action(name, handler) {
                this.name = name;
                this.handler = handler;
            }
            Action.prototype.getName = function () {
                return this.name;
            };
            Action.prototype.getHandler = function () {
                return this.handler;
            };
            return Action;
        })();
        notify.Action = Action;
        var Message = (function () {
            function Message(type, text, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                this.type = type;
                this.text = text;
                this.actions = [];
                this.autoHide = autoHide;
            }
            Message.prototype.getType = function () {
                return this.type;
            };
            Message.prototype.getText = function () {
                return this.text;
            };
            Message.prototype.getActions = function () {
                return this.actions;
            };
            Message.prototype.getAutoHide = function () {
                return this.autoHide;
            };
            Message.prototype.addAction = function (name, handler) {
                this.actions.push(new Action(name, handler));
            };
            Message.newSuccess = function (text, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                return new Message(4 /* SUCCESS */, text, autoHide);
            };
            Message.newInfo = function (text, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                return new Message(0 /* INFO */, text, autoHide);
            };
            Message.newError = function (text, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                return new Message(1 /* ERROR */, text, autoHide);
            };
            Message.newWarning = function (text, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                return new Message(2 /* WARNING */, text, autoHide);
            };
            Message.newAction = function (text, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                return new Message(3 /* ACTION */, text, autoHide);
            };
            return Message;
        })();
        notify.Message = Message;
    })(notify = api.notify || (api.notify = {}));
})(api || (api = {}));
var api;
(function (api) {
    var notify;
    (function (notify) {
        function showSuccess(message) {
            return notify.NotifyManager.get().showSuccess(message);
        }
        notify.showSuccess = showSuccess;
        function showFeedback(message) {
            return notify.NotifyManager.get().showFeedback(message);
        }
        notify.showFeedback = showFeedback;
        function showError(message, autoHide) {
            if (autoHide === void 0) { autoHide = true; }
            return notify.NotifyManager.get().showError(message, autoHide);
        }
        notify.showError = showError;
        function showWarning(message) {
            return notify.NotifyManager.get().showWarning(message);
        }
        notify.showWarning = showWarning;
    })(notify = api.notify || (api.notify = {}));
})(api || (api = {}));
var api;
(function (api) {
    var notify;
    (function (notify) {
        var NotifyOpts = (function () {
            function NotifyOpts() {
            }
            NotifyOpts.prototype.addListeners = function (message) {
                this.listeners = [];
                var actions = message.getActions();
                for (var i = 0; i < actions.length; i++) {
                    this.listeners.push(actions[i].getHandler());
                }
            };
            NotifyOpts.prototype.createHtmlMessage = function (message) {
                var actions = message.getActions();
                this.message = '<span>' + message.getText() + '</span>';
                if (actions.length > 0) {
                    var linkHtml = '<span style="float: right; margin-left: 30px;">';
                    for (var i = 0; i < actions.length; i++) {
                        if ((i > 0) && (i == (actions.length - 1))) {
                            linkHtml += ' or ';
                        }
                        else if (i > 0) {
                            linkHtml += ', ';
                        }
                        linkHtml += '<a href="#" class="notify.action_"' + i + '">';
                        linkHtml += actions[i].getName() + "</a>";
                    }
                    linkHtml += '</span>';
                    this.message = linkHtml + this.message;
                }
            };
            NotifyOpts.buildOpts = function (message) {
                var opts = new NotifyOpts();
                opts.autoHide = message.getAutoHide();
                if (message.getType() == 1 /* ERROR */) {
                    opts.type = 'error';
                }
                else if (message.getType() == 2 /* WARNING */) {
                    opts.type = 'warning';
                }
                else if (message.getType() == 3 /* ACTION */) {
                    opts.type = 'action';
                }
                else if (message.getType() == 4 /* SUCCESS */) {
                    opts.type = 'success';
                }
                opts.createHtmlMessage(message);
                opts.addListeners(message);
                return opts;
            };
            return NotifyOpts;
        })();
        notify.NotifyOpts = NotifyOpts;
    })(notify = api.notify || (api.notify = {}));
})(api || (api = {}));
var api;
(function (api) {
    var notify;
    (function (notify) {
        var NotifyManager = (function () {
            function NotifyManager() {
                this.space = 3;
                this.lifetime = 5000;
                this.slideDuration = 1000;
                this.timers = {};
                this.registry = {};
                this.el = new notify.NotificationContainer();
                api.dom.Body.get().appendChild(this.el);
                this.el.getEl().setBottomPx(0);
            }
            NotifyManager.prototype.showFeedback = function (message, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                var feedback = notify.Message.newInfo(message, autoHide);
                return this.notify(feedback);
            };
            NotifyManager.prototype.showSuccess = function (message, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                var feedback = notify.Message.newSuccess(message, autoHide);
                return this.notify(feedback);
            };
            NotifyManager.prototype.showError = function (message, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                var error = notify.Message.newError(message, autoHide);
                return this.notify(error);
            };
            NotifyManager.prototype.showWarning = function (message, autoHide) {
                if (autoHide === void 0) { autoHide = true; }
                var warning = notify.Message.newWarning(message, autoHide);
                return this.notify(warning);
            };
            NotifyManager.prototype.notify = function (message) {
                var opts = notify.NotifyOpts.buildOpts(message);
                return this.doNotify(opts);
            };
            NotifyManager.prototype.hide = function (messageId) {
                if (this.registry[messageId]) {
                    this.remove(this.registry[messageId]);
                    delete this.registry[messageId];
                }
            };
            NotifyManager.prototype.doNotify = function (opts) {
                var _this = this;
                var notificationEl = this.renderNotification(opts);
                this.registry[notificationEl.getEl().getId()] = notificationEl;
                this.setListeners(notificationEl, opts);
                wemjq(notificationEl.getHTMLElement()).animate({
                    height: 'toggle'
                }, this.slideDuration, function () {
                    if (opts.autoHide) {
                        _this.timers[notificationEl.getEl().getId()] = {
                            remainingTime: _this.lifetime
                        };
                        _this.startTimer(notificationEl);
                    }
                });
                return notificationEl.getEl().getId();
            };
            NotifyManager.prototype.setListeners = function (el, opts) {
                var _this = this;
                el.onClicked(function () {
                    _this.remove(el);
                    return false;
                });
                el.onMouseEnter(function () {
                    _this.stopTimer(el);
                });
                el.onMouseLeave(function () {
                    _this.startTimer(el);
                });
                if (opts.listeners) {
                    opts.listeners.forEach(function (listener) {
                        el.onClicked(listener);
                    });
                }
            };
            NotifyManager.prototype.remove = function (el) {
                var _this = this;
                if (!el) {
                    return;
                }
                wemjq(el.getHTMLElement()).animate({
                    height: 'hide'
                }, this.slideDuration, 'linear', function () {
                    _this.el.getWrapper().removeChild(el);
                });
                delete this.timers[el.getEl().getId()];
            };
            NotifyManager.prototype.startTimer = function (el) {
                var _this = this;
                var timer = this.timers[el.getEl().getId()];
                if (!timer) {
                    return;
                }
                timer.id = setTimeout(function () {
                    _this.remove(el);
                }, timer.remainingTime);
                timer.startTime = Date.now();
            };
            NotifyManager.prototype.stopTimer = function (el) {
                var timer = this.timers[el.getEl().getId()];
                if (!timer || !timer.id) {
                    return;
                }
                clearTimeout(timer.id);
                timer.id = null;
                timer.remainingTime -= Date.now() - timer.startTime;
            };
            NotifyManager.prototype.renderNotification = function (opts) {
                var style = {};
                var notifyDiv = new notify.NotificationMessage(opts.message);
                this.el.getWrapper().appendChild(notifyDiv);
                notifyDiv.hide();
                if (opts.type) {
                    notifyDiv.addClass(opts.type);
                }
                return notifyDiv;
            };
            NotifyManager.get = function () {
                if (!NotifyManager.instance) {
                    NotifyManager.instance = new NotifyManager();
                }
                return NotifyManager.instance;
            };
            return NotifyManager;
        })();
        notify.NotifyManager = NotifyManager;
    })(notify = api.notify || (api.notify = {}));
})(api || (api = {}));
var api;
(function (api) {
    var notify;
    (function (notify) {
        var NotificationMessage = (function (_super) {
            __extends(NotificationMessage, _super);
            function NotificationMessage(message) {
                _super.call(this, "notification");
                this.notificationInner = new api.dom.DivEl("notification-inner");
                var notificationRemove = new api.dom.AEl("notification-remove");
                notificationRemove.setHtml("X");
                notificationRemove.setUrl("#");
                var notificationContent = new api.dom.DivEl("notification-content");
                notificationContent.getEl().setInnerHtml(message);
                this.notificationInner.appendChild(notificationRemove).appendChild(notificationContent);
                this.appendChild(this.notificationInner);
            }
            return NotificationMessage;
        })(api.dom.DivEl);
        notify.NotificationMessage = NotificationMessage;
    })(notify = api.notify || (api.notify = {}));
})(api || (api = {}));
var api;
(function (api) {
    var notify;
    (function (notify) {
        var NotificationContainer = (function (_super) {
            __extends(NotificationContainer, _super);
            function NotificationContainer() {
                _super.call(this, "notification-container");
                this.wrapper = new api.dom.DivEl("notification-wrapper");
                this.appendChild(this.wrapper);
            }
            NotificationContainer.prototype.getWrapper = function () {
                return this.wrapper;
            };
            return NotificationContainer;
        })(api.dom.DivEl);
        notify.NotificationContainer = NotificationContainer;
    })(notify = api.notify || (api.notify = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rest;
    (function (rest) {
        var Path = (function () {
            function Path(elements, elementDivider, absolute) {
                this.elementDivider = elementDivider != null ? elementDivider : Path.DEFAULT_ELEMENT_DIVIDER;
                this.absolute = absolute == undefined ? true : absolute;
                elements.forEach(function (element, index) {
                    if (element == null) {
                        throw new Error("Path element was null at index: " + index);
                    }
                    else if (element.length == 0) {
                        throw new Error("Path element was empty string at index: " + index);
                    }
                });
                this.elements = elements;
                this.refString = (this.absolute ? this.elementDivider : "") + this.elements.join(this.elementDivider);
            }
            Path.fromString = function (s, elementDivider) {
                if (elementDivider == null) {
                    elementDivider = Path.DEFAULT_ELEMENT_DIVIDER;
                }
                var absolute = s.charAt(0) == elementDivider;
                var elements = s.split(elementDivider);
                return new Path(Path.removeEmptyElements(elements), elementDivider, absolute);
            };
            Path.fromParent = function (parent) {
                var childElements = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    childElements[_i - 1] = arguments[_i];
                }
                var elements = parent.elements.slice(0);
                childElements.forEach(function (element) {
                    elements.push(element);
                });
                return new Path(elements, parent.elementDivider, parent.isAbsolute());
            };
            Path.removeEmptyElements = function (elements) {
                var filteredElements = [];
                elements.forEach(function (element) {
                    if (element.length > 0) {
                        filteredElements.push(element);
                    }
                });
                return filteredElements;
            };
            Path.prototype.getElements = function () {
                return this.elements;
            };
            Path.prototype.getElement = function (index) {
                return this.elements[index];
            };
            Path.prototype.hasParent = function () {
                return this.elements.length > 0;
            };
            Path.prototype.getParentPath = function () {
                var _this = this;
                if (this.elements.length < 1) {
                    return null;
                }
                var parentElemements = [];
                this.elements.forEach(function (element, index) {
                    if (index < _this.elements.length - 1) {
                        parentElemements.push(element);
                    }
                });
                return new Path(parentElemements);
            };
            Path.prototype.toString = function () {
                return this.refString;
            };
            Path.prototype.isAbsolute = function () {
                return this.absolute;
            };
            Path.DEFAULT_ELEMENT_DIVIDER = "/";
            return Path;
        })();
        rest.Path = Path;
    })(rest = api.rest || (api.rest = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rest;
    (function (rest) {
        var JsonRequest = (function () {
            function JsonRequest() {
                this.method = "GET";
                this.timeoutMillis = 10000;
            }
            JsonRequest.prototype.setPath = function (value) {
                this.path = value;
                return this;
            };
            JsonRequest.prototype.setMethod = function (value) {
                this.method = value;
                return this;
            };
            JsonRequest.prototype.setParams = function (params) {
                this.params = params;
                return this;
            };
            JsonRequest.prototype.setTimeout = function (timeoutMillis) {
                if (timeoutMillis) {
                    this.timeoutMillis = timeoutMillis;
                }
                return this;
            };
            JsonRequest.prototype.send = function () {
                var deferred = wemQ.defer();
                var request = new XMLHttpRequest();
                request.onreadystatechange = function () {
                    if (request.readyState === 4) {
                        if (request.status === 204) {
                            deferred.resolve(new rest.JsonResponse(null));
                        }
                        else if (request.status >= 200 && request.status < 300) {
                            deferred.resolve(new rest.JsonResponse(request.response));
                        }
                        else if (request.status === 403) {
                            deferred.reject(new api.AccessDeniedException('Access denied'));
                        }
                        else {
                            try {
                                var errorJson = request.response ? JSON.parse(request.response) : null;
                            }
                            catch (error) {
                                deferred.reject(error);
                            }
                            deferred.reject(new rest.RequestError(request.status, errorJson ? errorJson.message : ""));
                        }
                    }
                };
                if ("POST" === this.method.toUpperCase()) {
                    this.preparePOSTRequest(request);
                    var paramString = JSON.stringify(this.params);
                    request.send(paramString);
                }
                else {
                    var request = this.prepareGETRequest(request);
                    request.send();
                }
                return deferred.promise;
            };
            JsonRequest.prototype.prepareGETRequest = function (request) {
                var paramString = api.util.UriHelper.encodeUrlParams(this.params);
                request.open(this.method, api.util.UriHelper.getUri(this.path.toString()) + '?' + paramString, true);
                request.timeout = this.timeoutMillis;
                request.setRequestHeader("Accept", "application/json");
                return request;
            };
            JsonRequest.prototype.preparePOSTRequest = function (request) {
                request.open(this.method, api.util.UriHelper.getUri(this.path.toString()), true);
                request.timeout = this.timeoutMillis;
                request.setRequestHeader("Accept", "application/json");
                request.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
            };
            return JsonRequest;
        })();
        rest.JsonRequest = JsonRequest;
    })(rest = api.rest || (api.rest = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rest;
    (function (rest) {
        var Response = (function () {
            function Response() {
            }
            return Response;
        })();
        rest.Response = Response;
    })(rest = api.rest || (api.rest = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rest;
    (function (rest) {
        var JsonResponse = (function (_super) {
            __extends(JsonResponse, _super);
            function JsonResponse(json) {
                _super.call(this);
                this.json = JSON.parse(json);
            }
            JsonResponse.prototype.isBlank = function () {
                return !this.json;
            };
            JsonResponse.prototype.getJson = function () {
                return this.json;
            };
            JsonResponse.prototype.hasResult = function () {
                if (this.json == undefined || this.json == null) {
                    return false;
                }
                return true;
            };
            JsonResponse.prototype.getResult = function () {
                if (!this.hasResult()) {
                    return null;
                }
                if (this.json.result) {
                    return this.json.result;
                }
                else {
                    return this.json;
                }
            };
            return JsonResponse;
        })(api.rest.Response);
        rest.JsonResponse = JsonResponse;
    })(rest = api.rest || (api.rest = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rest;
    (function (rest) {
        var ExceptionType = api.ExceptionType;
        var RequestError = (function (_super) {
            __extends(RequestError, _super);
            function RequestError(statusCode, errorMsg) {
                var notifyMsg = (statusCode > 0) ? errorMsg : "Unable to connect to server";
                var type = (statusCode >= 400 && statusCode < 500) ? 2 /* WARNING */ : 1 /* ERROR */;
                _super.call(this, notifyMsg, type);
            }
            return RequestError;
        })(api.Exception);
        rest.RequestError = RequestError;
    })(rest = api.rest || (api.rest = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rest;
    (function (rest) {
        var ResourceRequest = (function () {
            function ResourceRequest() {
                this.method = "GET";
                this.restPath = rest.Path.fromString("admin/rest");
            }
            ResourceRequest.prototype.setMethod = function (value) {
                this.method = value;
            };
            ResourceRequest.prototype.getRestPath = function () {
                return this.restPath;
            };
            ResourceRequest.prototype.getRequestPath = function () {
                throw new Error("Must be implemented by inheritors");
            };
            ResourceRequest.prototype.getParams = function () {
                throw new Error("Must be implemented by inheritors");
            };
            ResourceRequest.prototype.setTimeout = function (timeoutMillis) {
                this.timeoutMillis = timeoutMillis;
            };
            ResourceRequest.prototype.validate = function () {
            };
            ResourceRequest.prototype.send = function () {
                this.validate();
                var jsonRequest = new rest.JsonRequest().setMethod(this.method).setParams(this.getParams()).setPath(this.getRequestPath()).setTimeout(this.timeoutMillis);
                return jsonRequest.send();
            };
            ResourceRequest.prototype.sendAndParse = function () {
                throw new Error("sendAndParse method was not implemented");
            };
            return ResourceRequest;
        })();
        rest.ResourceRequest = ResourceRequest;
    })(rest = api.rest || (api.rest = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rest;
    (function (rest) {
        (function (Expand) {
            Expand[Expand["NONE"] = 0] = "NONE";
            Expand[Expand["SUMMARY"] = 1] = "SUMMARY";
            Expand[Expand["FULL"] = 2] = "FULL";
        })(rest.Expand || (rest.Expand = {}));
        var Expand = rest.Expand;
    })(rest = api.rest || (api.rest = {}));
})(api || (api = {}));
var api;
(function (api) {
    var icon;
    (function (icon) {
        var IconUrlResolver = (function () {
            function IconUrlResolver() {
            }
            IconUrlResolver.prototype.appendParam = function (paramName, paramValue, url) {
                var questionIndex = url.indexOf("?");
                if (questionIndex == -1) {
                    url += "?" + paramName + "=" + paramValue;
                    return url;
                }
                else if (url.charAt(url.length - 1) == '?') {
                    url += paramName + "=" + paramValue;
                    return url;
                }
                else {
                    url += "&" + paramName + "=" + paramValue;
                    return url;
                }
            };
            return IconUrlResolver;
        })();
        icon.IconUrlResolver = IconUrlResolver;
    })(icon = api.icon || (api.icon = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        (function (PrincipalType) {
            PrincipalType[PrincipalType["USER"] = 0] = "USER";
            PrincipalType[PrincipalType["GROUP"] = 1] = "GROUP";
            PrincipalType[PrincipalType["ROLE"] = 2] = "ROLE";
        })(security.PrincipalType || (security.PrincipalType = {}));
        var PrincipalType = security.PrincipalType;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UserStoreKey = (function () {
            function UserStoreKey(id) {
                api.util.assert(!api.util.StringHelper.isBlank(id), "UserStoreKey id cannot be null or empty");
                this.id = id;
            }
            UserStoreKey.prototype.isSystem = function () {
                return this.id === UserStoreKey.SYSTEM.id;
            };
            UserStoreKey.prototype.toString = function () {
                return this.id;
            };
            UserStoreKey.prototype.getId = function () {
                return this.id;
            };
            UserStoreKey.fromString = function (value) {
                return new UserStoreKey(value);
            };
            UserStoreKey.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, UserStoreKey)) {
                    return false;
                }
                var other = o;
                return this.id === other.id;
            };
            UserStoreKey.SYSTEM = new UserStoreKey('system');
            return UserStoreKey;
        })();
        security.UserStoreKey = UserStoreKey;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var PrincipalKey = (function () {
            function PrincipalKey(userStore, type, principalId) {
                api.util.assert((type === 2 /* ROLE */) || (!!userStore), "Principal user store cannot be null");
                api.util.assertNotNull(type, "Principal type cannot be null");
                api.util.assert(!api.util.StringHelper.isBlank(principalId), "Principal id cannot be null or empty");
                this.userStore = userStore;
                this.type = type;
                this.principalId = principalId;
                if (type === 2 /* ROLE */) {
                    this.refString = security.PrincipalType[type].toLowerCase() + PrincipalKey.SEPARATOR + principalId;
                }
                else {
                    this.refString = security.PrincipalType[type].toLowerCase() + PrincipalKey.SEPARATOR + userStore.toString() + PrincipalKey.SEPARATOR + principalId;
                }
            }
            PrincipalKey.fromString = function (str) {
                if (str === PrincipalKey.ANONYMOUS_PRINCIPAL.refString) {
                    return PrincipalKey.ANONYMOUS_PRINCIPAL;
                }
                var sepIndex = str.indexOf(PrincipalKey.SEPARATOR);
                if (sepIndex === -1) {
                    throw new Error("Not a valid principal key [" + str + "]");
                }
                var sepIndex2 = str.indexOf(PrincipalKey.SEPARATOR, sepIndex + 1);
                var typeStr = str.substring(0, sepIndex);
                var type = security.PrincipalType[typeStr.toUpperCase()];
                if (type === 2 /* ROLE */) {
                    var principalId = str.substring(sepIndex + 1, str.length) || '';
                    return new PrincipalKey(null, type, principalId);
                }
                else {
                    if (sepIndex2 === -1) {
                        throw new Error("Not a valid principal key [" + str + "]");
                    }
                    var userStore = str.substring(sepIndex + 1, sepIndex2) || '';
                    var principalId = str.substring(sepIndex2 + 1, str.length);
                    return new PrincipalKey(new security.UserStoreKey(userStore), type, principalId);
                }
            };
            PrincipalKey.prototype.getUserStore = function () {
                return this.userStore;
            };
            PrincipalKey.prototype.getType = function () {
                return this.type;
            };
            PrincipalKey.prototype.getId = function () {
                return this.principalId;
            };
            PrincipalKey.prototype.isUser = function () {
                return this.type === 0 /* USER */;
            };
            PrincipalKey.prototype.isGroup = function () {
                return this.type === 1 /* GROUP */;
            };
            PrincipalKey.prototype.isRole = function () {
                return this.type === 2 /* ROLE */;
            };
            PrincipalKey.prototype.isAnonymous = function () {
                return this.refString === PrincipalKey.ANONYMOUS_PRINCIPAL.refString;
            };
            PrincipalKey.prototype.toString = function () {
                return this.refString;
            };
            PrincipalKey.prototype.toPath = function (toParent) {
                if (toParent === void 0) { toParent = false; }
                var path = this.isRole() ? "/roles/" : api.util.StringHelper.format("/{0}/{1}/", this.getUserStore().toString(), security.PrincipalType[this.getType()].toLowerCase().replace(/(group|user)/g, "$&s"));
                if (!toParent) {
                    path += this.getId();
                }
                return path;
            };
            PrincipalKey.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PrincipalKey)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                    return false;
                }
                return true;
            };
            PrincipalKey.ofAnonymous = function () {
                return PrincipalKey.ANONYMOUS_PRINCIPAL;
            };
            PrincipalKey.ofUser = function (userStore, userId) {
                return new PrincipalKey(userStore, 0 /* USER */, userId);
            };
            PrincipalKey.ofGroup = function (userStore, groupId) {
                return new PrincipalKey(userStore, 1 /* GROUP */, groupId);
            };
            PrincipalKey.ofRole = function (roleId) {
                return new PrincipalKey(security.UserStoreKey.SYSTEM, 2 /* ROLE */, roleId);
            };
            PrincipalKey.SEPARATOR = ":";
            PrincipalKey.ANONYMOUS_PRINCIPAL = new PrincipalKey(security.UserStoreKey.SYSTEM, 0 /* USER */, "anonymous");
            return PrincipalKey;
        })();
        security.PrincipalKey = PrincipalKey;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var RoleKeys = (function () {
            function RoleKeys() {
            }
            RoleKeys.EVERYONE = security.PrincipalKey.ofRole('system.everyone');
            RoleKeys.ADMIN = security.PrincipalKey.ofRole('system.admin');
            return RoleKeys;
        })();
        security.RoleKeys = RoleKeys;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var Principal = (function () {
            function Principal(principalKey, displayName, modifiedTime) {
                this.key = principalKey;
                this.displayName = displayName || "";
                this.type = principalKey.getType();
                this.modifiedTime = modifiedTime;
            }
            Principal.fromJson = function (json) {
                var key = security.PrincipalKey.fromString(json.key);
                var date = json.modifiedTime ? new Date(Date.parse(json.modifiedTime)) : null;
                return new Principal(key, json.displayName, date);
            };
            Principal.fromPrincipal = function (principal) {
                return new Principal(principal.getKey(), principal.getDisplayName(), principal.getModifiedTime());
            };
            Principal.prototype.toJson = function () {
                return {
                    displayName: this.displayName,
                    key: this.key.toString()
                };
            };
            Principal.prototype.getKey = function () {
                return this.key;
            };
            Principal.prototype.getDisplayName = function () {
                return this.displayName;
            };
            Principal.prototype.getType = function () {
                return this.type;
            };
            Principal.prototype.getTypeName = function () {
                switch (this.type) {
                    case 1 /* GROUP */:
                        return "Group";
                    case 0 /* USER */:
                        return "User";
                    case 2 /* ROLE */:
                        return "Role";
                    default:
                        return "";
                }
            };
            Principal.prototype.isUser = function () {
                return this.type === 0 /* USER */;
            };
            Principal.prototype.isGroup = function () {
                return this.type === 1 /* GROUP */;
            };
            Principal.prototype.isRole = function () {
                return this.type === 2 /* ROLE */;
            };
            Principal.prototype.asUser = function () {
                return (this instanceof api.security.User) ? this : null;
            };
            Principal.prototype.asGroup = function () {
                return (this instanceof api.security.Group) ? this : null;
            };
            Principal.prototype.asRole = function () {
                return (this instanceof api.security.Role) ? this : null;
            };
            Principal.prototype.getModifiedTime = function () {
                return this.modifiedTime;
            };
            Principal.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Principal)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.key, other.key)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.displayName, other.displayName)) {
                    return false;
                }
                if (!api.ObjectHelper.dateEquals(this.modifiedTime, other.modifiedTime)) {
                    return false;
                }
                return true;
            };
            Principal.prototype.clone = function () {
                return new Principal(this.key, this.displayName, this.modifiedTime);
            };
            return Principal;
        })();
        security.Principal = Principal;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var User = (function (_super) {
            __extends(User, _super);
            function User(builder) {
                _super.call(this, builder.key, builder.displayName, builder.modifiedTime);
                api.util.assert(builder.key.isUser(), 'Expected PrincipalKey of type User');
                this.email = builder.email || "";
                this.login = builder.login || "";
                this.loginDisabled = builder.loginDisabled || false;
                this.memberships = builder.memberships || [];
            }
            User.prototype.getEmail = function () {
                return this.email;
            };
            User.prototype.getLogin = function () {
                return this.login;
            };
            User.prototype.isDisabled = function () {
                return this.loginDisabled;
            };
            User.prototype.getMemberships = function () {
                return this.memberships;
            };
            User.prototype.setMemberships = function (memberships) {
                this.memberships = memberships;
            };
            User.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, User)) {
                    return false;
                }
                var other = o;
                return _super.prototype.equals.call(this, o) && this.loginDisabled === other.isDisabled() && this.email === other.getEmail() && this.login === other.getLogin() && api.ObjectHelper.arrayEquals(this.memberships, other.getMemberships());
            };
            User.prototype.clone = function () {
                return this.newBuilder().build();
            };
            User.prototype.newBuilder = function () {
                return new UserBuilder(this);
            };
            User.create = function () {
                return new UserBuilder();
            };
            User.fromJson = function (json) {
                return new UserBuilder().fromJson(json).build();
            };
            return User;
        })(security.Principal);
        security.User = User;
        var UserBuilder = (function () {
            function UserBuilder(source) {
                if (source) {
                    this.key = source.getKey();
                    this.displayName = source.getDisplayName();
                    this.email = source.getEmail();
                    this.login = source.getLogin();
                    this.loginDisabled = source.isDisabled();
                    this.modifiedTime = source.getModifiedTime();
                    this.memberships = source.getMemberships().slice(0);
                }
                else {
                    this.memberships = [];
                }
            }
            UserBuilder.prototype.fromJson = function (json) {
                this.key = security.PrincipalKey.fromString(json.key);
                this.displayName = json.displayName;
                this.email = json.email;
                this.login = json.login;
                this.loginDisabled = json.loginDisabled;
                this.modifiedTime = json.modifiedTime ? new Date(Date.parse(json.modifiedTime)) : null;
                if (json.memberships) {
                    this.memberships = json.memberships.map(function (principalJson) { return security.Principal.fromJson(principalJson); });
                }
                return this;
            };
            UserBuilder.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            UserBuilder.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            UserBuilder.prototype.setEmail = function (value) {
                this.email = value;
                return this;
            };
            UserBuilder.prototype.setLogin = function (value) {
                this.login = value;
                return this;
            };
            UserBuilder.prototype.setDisabled = function (value) {
                this.loginDisabled = value;
                return this;
            };
            UserBuilder.prototype.setMemberships = function (memberships) {
                this.memberships = memberships || [];
                return this;
            };
            UserBuilder.prototype.build = function () {
                return new User(this);
            };
            return UserBuilder;
        })();
        security.UserBuilder = UserBuilder;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var Group = (function (_super) {
            __extends(Group, _super);
            function Group(builder) {
                _super.call(this, builder.key, builder.displayName, builder.modifiedTime);
                api.util.assert(builder.key.isGroup(), 'Expected PrincipalKey of type Group');
                this.members = builder.members || [];
            }
            Group.prototype.getMembers = function () {
                return this.members;
            };
            Group.prototype.setMembers = function (members) {
                this.members = members || [];
            };
            Group.prototype.addMember = function (member) {
                this.members.push(member);
            };
            Group.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Group)) {
                    return false;
                }
                var other = o;
                return _super.prototype.equals.call(this, o) && api.ObjectHelper.arrayEquals(this.members, other.getMembers());
            };
            Group.prototype.clone = function () {
                return this.newBuilder().build();
            };
            Group.prototype.newBuilder = function () {
                return new GroupBuilder(this);
            };
            Group.create = function () {
                return new GroupBuilder();
            };
            Group.fromJson = function (json) {
                return new GroupBuilder().fromJson(json).build();
            };
            return Group;
        })(security.Principal);
        security.Group = Group;
        var GroupBuilder = (function () {
            function GroupBuilder(source) {
                if (source) {
                    this.key = source.getKey();
                    this.displayName = source.getDisplayName();
                    this.modifiedTime = source.getModifiedTime();
                    this.members = source.getMembers().slice(0);
                }
                else {
                    this.members = [];
                }
            }
            GroupBuilder.prototype.fromJson = function (json) {
                this.key = security.PrincipalKey.fromString(json.key);
                this.displayName = json.displayName;
                this.modifiedTime = json.modifiedTime ? new Date(Date.parse(json.modifiedTime)) : null;
                if (json.members) {
                    this.members = json.members.map(function (memberStr) { return security.PrincipalKey.fromString(memberStr); });
                }
                return this;
            };
            GroupBuilder.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            GroupBuilder.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            GroupBuilder.prototype.setMembers = function (members) {
                this.members = members || [];
                return this;
            };
            GroupBuilder.prototype.build = function () {
                return new Group(this);
            };
            return GroupBuilder;
        })();
        security.GroupBuilder = GroupBuilder;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var Role = (function (_super) {
            __extends(Role, _super);
            function Role(builder) {
                _super.call(this, builder.key, builder.displayName, builder.modifiedTime);
                api.util.assert(builder.key.isRole(), 'Expected PrincipalKey of type Role');
                this.members = builder.members || [];
            }
            Role.prototype.getMembers = function () {
                return this.members;
            };
            Role.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Role)) {
                    return false;
                }
                var other = o;
                return _super.prototype.equals.call(this, o) && api.ObjectHelper.arrayEquals(this.members, other.getMembers());
            };
            Role.prototype.clone = function () {
                return this.newBuilder().build();
            };
            Role.prototype.newBuilder = function () {
                return new RoleBuilder(this);
            };
            Role.create = function () {
                return new RoleBuilder();
            };
            Role.fromJson = function (json) {
                return new RoleBuilder().fromJson(json).build();
            };
            return Role;
        })(security.Principal);
        security.Role = Role;
        var RoleBuilder = (function () {
            function RoleBuilder(source) {
                if (source) {
                    this.key = source.getKey();
                    this.displayName = source.getDisplayName();
                    this.modifiedTime = source.getModifiedTime();
                    this.members = source.getMembers().slice(0);
                }
                else {
                    this.members = [];
                }
            }
            RoleBuilder.prototype.fromJson = function (json) {
                this.key = security.PrincipalKey.fromString(json.key);
                this.displayName = json.displayName;
                this.modifiedTime = json.modifiedTime ? new Date(Date.parse(json.modifiedTime)) : null;
                if (json.members) {
                    this.members = json.members.map(function (memberStr) { return security.PrincipalKey.fromString(memberStr); });
                }
                return this;
            };
            RoleBuilder.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            RoleBuilder.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            RoleBuilder.prototype.setMembers = function (members) {
                this.members = members || [];
                return this;
            };
            RoleBuilder.prototype.build = function () {
                return new Role(this);
            };
            return RoleBuilder;
        })();
        security.RoleBuilder = RoleBuilder;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UserStore = (function () {
            function UserStore(builder) {
                this.displayName = builder.displayName;
                this.key = builder.key;
                this.permissions = builder.permissions || new api.security.acl.UserStoreAccessControlList();
            }
            UserStore.prototype.getDisplayName = function () {
                return this.displayName;
            };
            UserStore.prototype.getKey = function () {
                return this.key;
            };
            UserStore.prototype.getPermissions = function () {
                return this.permissions;
            };
            UserStore.prototype.isDeletable = function () {
                var deferred = wemQ.defer();
                new security.GetPrincipalsByUserStoreRequest(this.key, [0 /* USER */, 1 /* GROUP */]).sendAndParse().then(function (principals) {
                    if (principals.length > 0) {
                        deferred.resolve(false);
                    }
                    else {
                        deferred.resolve(true);
                    }
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                    deferred.resolve(false);
                }).done();
                ;
                return deferred.promise;
            };
            UserStore.checkOnDeletable = function (key) {
                return !!key ? UserStore.create().setKey(key.toString()).build().isDeletable() : null;
            };
            UserStore.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, UserStore)) {
                    return false;
                }
                var other = o;
                return this.key.equals(other.key) && this.displayName === other.displayName && this.permissions.equals(other.permissions);
            };
            UserStore.prototype.clone = function () {
                return UserStore.create().setDisplayName(this.displayName).setKey(this.key.toString()).setPermissions(this.permissions.clone()).build();
            };
            UserStore.create = function () {
                return new UserStoreBuilder();
            };
            UserStore.fromJson = function (json) {
                return new UserStoreBuilder().fromJson(json).build();
            };
            return UserStore;
        })();
        security.UserStore = UserStore;
        var UserStoreBuilder = (function () {
            function UserStoreBuilder() {
            }
            UserStoreBuilder.prototype.fromJson = function (json) {
                this.key = new security.UserStoreKey(json.key);
                this.displayName = json.displayName;
                this.permissions = json.permissions ? api.security.acl.UserStoreAccessControlList.fromJson(json.permissions) : null;
                return this;
            };
            UserStoreBuilder.prototype.setKey = function (key) {
                this.key = new security.UserStoreKey(key);
                return this;
            };
            UserStoreBuilder.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            UserStoreBuilder.prototype.setPermissions = function (permissions) {
                this.permissions = permissions;
                return this;
            };
            UserStoreBuilder.prototype.build = function () {
                return new UserStore(this);
            };
            return UserStoreBuilder;
        })();
        security.UserStoreBuilder = UserStoreBuilder;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UserStoreListResult = (function () {
            function UserStoreListResult() {
            }
            return UserStoreListResult;
        })();
        security.UserStoreListResult = UserStoreListResult;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var Role = api.security.Role;
        var Group = api.security.Group;
        var User = api.security.User;
        var SecurityResourceRequest = (function (_super) {
            __extends(SecurityResourceRequest, _super);
            function SecurityResourceRequest() {
                _super.call(this);
                this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "security");
            }
            SecurityResourceRequest.prototype.getResourcePath = function () {
                return this.resourcePath;
            };
            SecurityResourceRequest.prototype.fromJsonToPrincipal = function (json) {
                var pKey = security.PrincipalKey.fromString(json.key);
                if (pKey.isRole()) {
                    return Role.fromJson(json);
                }
                else if (pKey.isGroup()) {
                    return Group.fromJson(json);
                }
                else if (pKey.isUser()) {
                    return User.fromJson(json);
                }
            };
            return SecurityResourceRequest;
        })(api.rest.ResourceRequest);
        security.SecurityResourceRequest = SecurityResourceRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var ListUserStoresRequest = (function (_super) {
            __extends(ListUserStoresRequest, _super);
            function ListUserStoresRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
            }
            ListUserStoresRequest.prototype.getParams = function () {
                return {};
            };
            ListUserStoresRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'userstore/list');
            };
            ListUserStoresRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return response.getResult().userStores.map(function (userStoreJson) {
                        return _this.fromJsonToUserStore(userStoreJson);
                    });
                });
            };
            ListUserStoresRequest.prototype.fromJsonToUserStore = function (json) {
                return security.UserStore.fromJson(json);
            };
            return ListUserStoresRequest;
        })(security.SecurityResourceRequest);
        security.ListUserStoresRequest = ListUserStoresRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var GetUserStoreByKeyRequest = (function (_super) {
            __extends(GetUserStoreByKeyRequest, _super);
            function GetUserStoreByKeyRequest(userStoreKey) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.userStoreKey = userStoreKey;
            }
            GetUserStoreByKeyRequest.prototype.getParams = function () {
                return {
                    key: this.userStoreKey.toString()
                };
            };
            GetUserStoreByKeyRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'userstore');
            };
            GetUserStoreByKeyRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToUserStore(response.getResult());
                });
            };
            GetUserStoreByKeyRequest.prototype.fromJsonToUserStore = function (json) {
                return security.UserStore.fromJson(json);
            };
            return GetUserStoreByKeyRequest;
        })(security.SecurityResourceRequest);
        security.GetUserStoreByKeyRequest = GetUserStoreByKeyRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var GetDefaultUserStoreRequest = (function (_super) {
            __extends(GetDefaultUserStoreRequest, _super);
            function GetDefaultUserStoreRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
            }
            GetDefaultUserStoreRequest.prototype.getParams = function () {
                return null;
            };
            GetDefaultUserStoreRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'userstore/default');
            };
            GetDefaultUserStoreRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToUserStore(response.getResult());
                });
            };
            GetDefaultUserStoreRequest.prototype.fromJsonToUserStore = function (json) {
                return security.UserStore.fromJson(json);
            };
            return GetDefaultUserStoreRequest;
        })(security.SecurityResourceRequest);
        security.GetDefaultUserStoreRequest = GetDefaultUserStoreRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var PrincipalListJson = (function () {
            function PrincipalListJson() {
            }
            return PrincipalListJson;
        })();
        security.PrincipalListJson = PrincipalListJson;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var FindPrincipalsRequest = (function (_super) {
            __extends(FindPrincipalsRequest, _super);
            function FindPrincipalsRequest() {
                _super.call(this);
            }
            FindPrincipalsRequest.prototype.getParams = function () {
                return {
                    "types": this.enumToStrings(this.allowedTypes).join(','),
                    "query": this.searchQuery,
                    "userStoreKey": this.userStoreKey ? this.userStoreKey.toString() : undefined
                };
            };
            FindPrincipalsRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals');
            };
            FindPrincipalsRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    var principals = response.getResult().principals.map(function (principalJson) {
                        return _this.fromJsonToPrincipal(principalJson);
                    });
                    if (_this.filterPredicate) {
                        principals = principals.filter(_this.filterPredicate);
                    }
                    return principals;
                });
            };
            FindPrincipalsRequest.prototype.enumToStrings = function (types) {
                return types.map(function (type) {
                    return security.PrincipalType[type].toUpperCase();
                });
            };
            FindPrincipalsRequest.prototype.setUserStoreKey = function (key) {
                this.userStoreKey = key;
                return this;
            };
            FindPrincipalsRequest.prototype.setAllowedTypes = function (types) {
                this.allowedTypes = types;
                return this;
            };
            FindPrincipalsRequest.prototype.setSearchQuery = function (query) {
                this.searchQuery = query;
                return this;
            };
            FindPrincipalsRequest.prototype.setResultFilter = function (filterPredicate) {
                this.filterPredicate = filterPredicate;
            };
            return FindPrincipalsRequest;
        })(api.security.SecurityResourceRequest);
        security.FindPrincipalsRequest = FindPrincipalsRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var GetPrincipalByKeyRequest = (function (_super) {
            __extends(GetPrincipalByKeyRequest, _super);
            function GetPrincipalByKeyRequest(principalKey) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.principalKey = principalKey;
                this.includeMemberships = false;
            }
            GetPrincipalByKeyRequest.prototype.includeUserMemberships = function (includeMemberships) {
                this.includeMemberships = includeMemberships;
                return this;
            };
            GetPrincipalByKeyRequest.prototype.getParams = function () {
                return {
                    memberships: this.includeMemberships && this.principalKey.isUser()
                };
            };
            GetPrincipalByKeyRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', this.principalKey.toString());
            };
            GetPrincipalByKeyRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToPrincipal(response.getResult());
                });
            };
            return GetPrincipalByKeyRequest;
        })(security.SecurityResourceRequest);
        security.GetPrincipalByKeyRequest = GetPrincipalByKeyRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var GetPrincipalsByUserStoreRequest = (function (_super) {
            __extends(GetPrincipalsByUserStoreRequest, _super);
            function GetPrincipalsByUserStoreRequest(userStore, principalTypes) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.userStore = userStore;
                this.principalTypes = principalTypes;
            }
            GetPrincipalsByUserStoreRequest.prototype.getParams = function () {
                return {
                    'userStoreKey': this.userStore.getId(),
                    'types': this.getType()
                };
            };
            GetPrincipalsByUserStoreRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals');
            };
            GetPrincipalsByUserStoreRequest.prototype.getType = function () {
                var typeStr = "";
                this.principalTypes.forEach(function (type) {
                    typeStr += security.PrincipalType[type];
                    typeStr += ",";
                });
                return typeStr.substr(0, typeStr.length - 1);
            };
            GetPrincipalsByUserStoreRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return response.getResult().principals.map(function (principalJson) {
                        return _this.fromJsonToPrincipal(principalJson);
                    });
                });
            };
            return GetPrincipalsByUserStoreRequest;
        })(security.SecurityResourceRequest);
        security.GetPrincipalsByUserStoreRequest = GetPrincipalsByUserStoreRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var CheckEmailAvailabilityRequest = (function (_super) {
            __extends(CheckEmailAvailabilityRequest, _super);
            function CheckEmailAvailabilityRequest(email) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.email = email;
            }
            CheckEmailAvailabilityRequest.prototype.setUserStoreKey = function (key) {
                this.userStoreKey = key;
                return this;
            };
            CheckEmailAvailabilityRequest.prototype.getParams = function () {
                return {
                    email: this.email,
                    userStoreKey: this.userStoreKey ? this.userStoreKey.toString() : undefined
                };
            };
            CheckEmailAvailabilityRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'emailAvailable');
            };
            CheckEmailAvailabilityRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return response.getResult().available;
                });
            };
            return CheckEmailAvailabilityRequest;
        })(security.SecurityResourceRequest);
        security.CheckEmailAvailabilityRequest = CheckEmailAvailabilityRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var SetUserPasswordRequest = (function (_super) {
            __extends(SetUserPasswordRequest, _super);
            function SetUserPasswordRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
            }
            SetUserPasswordRequest.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            SetUserPasswordRequest.prototype.setPassword = function (password) {
                this.password = password;
                return this;
            };
            SetUserPasswordRequest.prototype.getParams = function () {
                return {
                    key: this.key.toString(),
                    password: this.password
                };
            };
            SetUserPasswordRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'setPassword');
            };
            SetUserPasswordRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return security.User.fromJson(response.getResult());
                });
            };
            return SetUserPasswordRequest;
        })(security.SecurityResourceRequest);
        security.SetUserPasswordRequest = SetUserPasswordRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var CreateUserRequest = (function (_super) {
            __extends(CreateUserRequest, _super);
            function CreateUserRequest() {
                _super.call(this);
                this.memberships = [];
                _super.prototype.setMethod.call(this, "POST");
            }
            CreateUserRequest.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            CreateUserRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            CreateUserRequest.prototype.setEmail = function (email) {
                this.email = email;
                return this;
            };
            CreateUserRequest.prototype.setLogin = function (login) {
                this.login = login;
                return this;
            };
            CreateUserRequest.prototype.setPassword = function (password) {
                this.password = password;
                return this;
            };
            CreateUserRequest.prototype.setMemberships = function (memberships) {
                this.memberships = memberships.slice(0);
                return this;
            };
            CreateUserRequest.prototype.getParams = function () {
                return {
                    key: this.key.toString(),
                    displayName: this.displayName,
                    email: this.email,
                    login: this.login,
                    password: this.password,
                    memberships: this.memberships.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            CreateUserRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'createUser');
            };
            CreateUserRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return security.User.fromJson(response.getResult());
                });
            };
            return CreateUserRequest;
        })(security.SecurityResourceRequest);
        security.CreateUserRequest = CreateUserRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var CreateGroupRequest = (function (_super) {
            __extends(CreateGroupRequest, _super);
            function CreateGroupRequest() {
                _super.call(this);
                this.members = [];
                _super.prototype.setMethod.call(this, "POST");
            }
            CreateGroupRequest.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            CreateGroupRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            CreateGroupRequest.prototype.setMembers = function (members) {
                this.members = members.slice(0);
                return this;
            };
            CreateGroupRequest.prototype.getParams = function () {
                return {
                    key: this.key.toString(),
                    displayName: this.displayName,
                    members: this.members.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            CreateGroupRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'createGroup');
            };
            CreateGroupRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return security.Group.fromJson(response.getResult());
                });
            };
            return CreateGroupRequest;
        })(security.SecurityResourceRequest);
        security.CreateGroupRequest = CreateGroupRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var CreateRoleRequest = (function (_super) {
            __extends(CreateRoleRequest, _super);
            function CreateRoleRequest() {
                _super.call(this);
                this.members = [];
                _super.prototype.setMethod.call(this, "POST");
            }
            CreateRoleRequest.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            CreateRoleRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            CreateRoleRequest.prototype.setMembers = function (members) {
                this.members = members.slice(0);
                return this;
            };
            CreateRoleRequest.prototype.getParams = function () {
                return {
                    key: this.key.toString(),
                    displayName: this.displayName,
                    members: this.members.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            CreateRoleRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'createRole');
            };
            CreateRoleRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return security.Role.fromJson(response.getResult());
                });
            };
            return CreateRoleRequest;
        })(security.SecurityResourceRequest);
        security.CreateRoleRequest = CreateRoleRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UpdateUserRequest = (function (_super) {
            __extends(UpdateUserRequest, _super);
            function UpdateUserRequest() {
                _super.call(this);
                this.membershipsToAdd = [];
                this.membershipsToRemove = [];
                _super.prototype.setMethod.call(this, "POST");
            }
            UpdateUserRequest.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            UpdateUserRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            UpdateUserRequest.prototype.setEmail = function (email) {
                this.email = email;
                return this;
            };
            UpdateUserRequest.prototype.setLogin = function (login) {
                this.login = login;
                return this;
            };
            UpdateUserRequest.prototype.addMemberships = function (memberships) {
                this.membershipsToAdd = memberships.slice(0);
                return this;
            };
            UpdateUserRequest.prototype.removeMemberships = function (memberships) {
                this.membershipsToRemove = memberships.slice(0);
                return this;
            };
            UpdateUserRequest.prototype.getParams = function () {
                return {
                    key: this.key.toString(),
                    displayName: this.displayName,
                    email: this.email,
                    login: this.login,
                    addMemberships: this.membershipsToAdd.map(function (memberKey) { return memberKey.toString(); }),
                    removeMemberships: this.membershipsToRemove.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            UpdateUserRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'updateUser');
            };
            UpdateUserRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return security.User.fromJson(response.getResult());
                });
            };
            return UpdateUserRequest;
        })(security.SecurityResourceRequest);
        security.UpdateUserRequest = UpdateUserRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UpdateGroupRequest = (function (_super) {
            __extends(UpdateGroupRequest, _super);
            function UpdateGroupRequest() {
                _super.call(this);
                this.membersToAdd = [];
                this.membersToRemove = [];
                _super.prototype.setMethod.call(this, "POST");
            }
            UpdateGroupRequest.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            UpdateGroupRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            UpdateGroupRequest.prototype.addMembers = function (members) {
                this.membersToAdd = members.slice(0);
                return this;
            };
            UpdateGroupRequest.prototype.removeMembers = function (members) {
                this.membersToRemove = members.slice(0);
                return this;
            };
            UpdateGroupRequest.prototype.getParams = function () {
                return {
                    key: this.key.toString(),
                    displayName: this.displayName,
                    addMembers: this.membersToAdd.map(function (memberKey) { return memberKey.toString(); }),
                    removeMembers: this.membersToRemove.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            UpdateGroupRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'updateGroup');
            };
            UpdateGroupRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return security.Group.fromJson(response.getResult());
                });
            };
            return UpdateGroupRequest;
        })(security.SecurityResourceRequest);
        security.UpdateGroupRequest = UpdateGroupRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UpdateRoleRequest = (function (_super) {
            __extends(UpdateRoleRequest, _super);
            function UpdateRoleRequest() {
                _super.call(this);
                this.membersToAdd = [];
                this.membersToRemove = [];
                _super.prototype.setMethod.call(this, "POST");
            }
            UpdateRoleRequest.prototype.setKey = function (key) {
                this.key = key;
                return this;
            };
            UpdateRoleRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            UpdateRoleRequest.prototype.addMembers = function (members) {
                this.membersToAdd = members.slice(0);
                return this;
            };
            UpdateRoleRequest.prototype.removeMembers = function (members) {
                this.membersToRemove = members.slice(0);
                return this;
            };
            UpdateRoleRequest.prototype.getParams = function () {
                return {
                    key: this.key.toString(),
                    displayName: this.displayName,
                    addMembers: this.membersToAdd.map(function (memberKey) { return memberKey.toString(); }),
                    removeMembers: this.membersToRemove.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            UpdateRoleRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'updateRole');
            };
            UpdateRoleRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return security.Role.fromJson(response.getResult());
                });
            };
            return UpdateRoleRequest;
        })(security.SecurityResourceRequest);
        security.UpdateRoleRequest = UpdateRoleRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var DeletePrincipalResult = (function () {
            function DeletePrincipalResult() {
            }
            DeletePrincipalResult.prototype.getPrincipalKey = function () {
                return this.principalKey;
            };
            DeletePrincipalResult.prototype.isDeleted = function () {
                return this.deleted;
            };
            DeletePrincipalResult.prototype.getReason = function () {
                return this.reason;
            };
            DeletePrincipalResult.fromJson = function (json) {
                var result = new DeletePrincipalResult();
                result.principalKey = security.PrincipalKey.fromString(json.principalKey);
                result.deleted = json.deleted;
                result.reason = json.reason;
                return result;
            };
            return DeletePrincipalResult;
        })();
        security.DeletePrincipalResult = DeletePrincipalResult;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var DeleteUserStoreResult = (function () {
            function DeleteUserStoreResult() {
            }
            DeleteUserStoreResult.prototype.getUserStoreKey = function () {
                return this.userStoreKey;
            };
            DeleteUserStoreResult.prototype.isDeleted = function () {
                return this.deleted;
            };
            DeleteUserStoreResult.prototype.getReason = function () {
                return this.reason;
            };
            DeleteUserStoreResult.fromJson = function (json) {
                var result = new DeleteUserStoreResult();
                result.userStoreKey = security.UserStoreKey.fromString(json.userStoreKey);
                result.deleted = json.deleted;
                result.reason = json.reason;
                return result;
            };
            return DeleteUserStoreResult;
        })();
        security.DeleteUserStoreResult = DeleteUserStoreResult;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var DeletePrincipalRequest = (function (_super) {
            __extends(DeletePrincipalRequest, _super);
            function DeletePrincipalRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
            }
            DeletePrincipalRequest.prototype.setKeys = function (keys) {
                this.keys = keys.slice(0);
                return this;
            };
            DeletePrincipalRequest.prototype.getParams = function () {
                return {
                    keys: this.keys.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            DeletePrincipalRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals', 'delete');
            };
            DeletePrincipalRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return response.getResult().results.map(function (resultJson) { return security.DeletePrincipalResult.fromJson(resultJson); });
                });
            };
            return DeletePrincipalRequest;
        })(security.SecurityResourceRequest);
        security.DeletePrincipalRequest = DeletePrincipalRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var DeleteUserStoreRequest = (function (_super) {
            __extends(DeleteUserStoreRequest, _super);
            function DeleteUserStoreRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
            }
            DeleteUserStoreRequest.prototype.setKeys = function (keys) {
                this.keys = keys.slice(0);
                return this;
            };
            DeleteUserStoreRequest.prototype.getParams = function () {
                return {
                    keys: this.keys.map(function (memberKey) { return memberKey.toString(); })
                };
            };
            DeleteUserStoreRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'userstore', 'delete');
            };
            DeleteUserStoreRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return response.getResult().results.map(function (resultJson) { return security.DeleteUserStoreResult.fromJson(resultJson); });
                });
            };
            return DeleteUserStoreRequest;
        })(security.SecurityResourceRequest);
        security.DeleteUserStoreRequest = DeleteUserStoreRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var PrincipalLoader = (function (_super) {
            __extends(PrincipalLoader, _super);
            function PrincipalLoader() {
                this.findRequest = new security.FindPrincipalsRequest();
                this.skipPrincipalKeys = {};
                this.setAllowedTypes([1 /* GROUP */, 0 /* USER */, 2 /* ROLE */]);
                _super.call(this, this.findRequest);
            }
            PrincipalLoader.prototype.setUserStoreKey = function (key) {
                this.findRequest.setUserStoreKey(key);
                return this;
            };
            PrincipalLoader.prototype.setAllowedTypes = function (principalTypes) {
                this.findRequest.setAllowedTypes(principalTypes);
                return this;
            };
            PrincipalLoader.prototype.search = function (searchString) {
                this.findRequest.setSearchQuery(searchString);
                return this.load();
            };
            PrincipalLoader.prototype.skipPrincipals = function (principalKeys) {
                var _this = this;
                this.skipPrincipalKeys = {};
                principalKeys.forEach(function (principalKey) {
                    _this.skipPrincipalKeys[principalKey.toString()] = principalKey;
                });
                this.findRequest.setResultFilter(function (principal) { return !_this.skipPrincipalKeys[principal.getKey().toString()]; });
                return this;
            };
            return PrincipalLoader;
        })(api.util.loader.BaseLoader);
        security.PrincipalLoader = PrincipalLoader;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var PrincipalNamedEvent = (function (_super) {
            __extends(PrincipalNamedEvent, _super);
            function PrincipalNamedEvent(wizard, principal) {
                _super.call(this);
                this.wizard = wizard;
                this.principal = principal;
            }
            PrincipalNamedEvent.prototype.getWizard = function () {
                return this.wizard;
            };
            PrincipalNamedEvent.prototype.getPrincipal = function () {
                return this.principal;
            };
            PrincipalNamedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            PrincipalNamedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return PrincipalNamedEvent;
        })(api.event.Event);
        security.PrincipalNamedEvent = PrincipalNamedEvent;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var CreateUserStoreRequest = (function (_super) {
            __extends(CreateUserStoreRequest, _super);
            function CreateUserStoreRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
            }
            CreateUserStoreRequest.prototype.getParams = function () {
                return {
                    key: this.userStoreKey.toString(),
                    displayName: this.displayName,
                    permissions: this.permissions ? this.permissions.toJson() : []
                };
            };
            CreateUserStoreRequest.prototype.setKey = function (userStoreKey) {
                this.userStoreKey = userStoreKey;
                return this;
            };
            CreateUserStoreRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            CreateUserStoreRequest.prototype.setPermissions = function (permissions) {
                this.permissions = permissions;
                return this;
            };
            CreateUserStoreRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'userstore/create');
            };
            CreateUserStoreRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToUserStore(response.getResult());
                });
            };
            CreateUserStoreRequest.prototype.fromJsonToUserStore = function (json) {
                return security.UserStore.fromJson(json);
            };
            return CreateUserStoreRequest;
        })(security.SecurityResourceRequest);
        security.CreateUserStoreRequest = CreateUserStoreRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UpdateUserStoreRequest = (function (_super) {
            __extends(UpdateUserStoreRequest, _super);
            function UpdateUserStoreRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
            }
            UpdateUserStoreRequest.prototype.getParams = function () {
                return {
                    key: this.userStoreKey.toString(),
                    displayName: this.displayName,
                    permissions: this.permissions ? this.permissions.toJson() : undefined
                };
            };
            UpdateUserStoreRequest.prototype.setKey = function (userStoreKey) {
                this.userStoreKey = userStoreKey;
                return this;
            };
            UpdateUserStoreRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            UpdateUserStoreRequest.prototype.setPermissions = function (permissions) {
                this.permissions = permissions;
                return this;
            };
            UpdateUserStoreRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'userstore/update');
            };
            UpdateUserStoreRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToUserStore(response.getResult());
                });
            };
            UpdateUserStoreRequest.prototype.fromJsonToUserStore = function (json) {
                return security.UserStore.fromJson(json);
            };
            return UpdateUserStoreRequest;
        })(security.SecurityResourceRequest);
        security.UpdateUserStoreRequest = UpdateUserStoreRequest;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UserStoreNamedEvent = (function (_super) {
            __extends(UserStoreNamedEvent, _super);
            function UserStoreNamedEvent(wizard, userStore) {
                _super.call(this);
                this.wizard = wizard;
                this.userStore = userStore;
            }
            UserStoreNamedEvent.prototype.getWizard = function () {
                return this.wizard;
            };
            UserStoreNamedEvent.prototype.getUserStore = function () {
                return this.userStore;
            };
            UserStoreNamedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            UserStoreNamedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return UserStoreNamedEvent;
        })(api.event.Event);
        security.UserStoreNamedEvent = UserStoreNamedEvent;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UserItemCreatedEvent = (function (_super) {
            __extends(UserItemCreatedEvent, _super);
            function UserItemCreatedEvent(principal, userStore, parentOfSameType) {
                _super.call(this);
                this.principal = principal;
                this.userStore = userStore;
                this.parentOfSameType = parentOfSameType;
            }
            UserItemCreatedEvent.prototype.getPrincipal = function () {
                return this.principal;
            };
            UserItemCreatedEvent.prototype.getUserStore = function () {
                return this.userStore;
            };
            UserItemCreatedEvent.prototype.isParentOfSameType = function () {
                return this.parentOfSameType;
            };
            UserItemCreatedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            UserItemCreatedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return UserItemCreatedEvent;
        })(api.event.Event);
        security.UserItemCreatedEvent = UserItemCreatedEvent;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UserItemUpdatedEvent = (function (_super) {
            __extends(UserItemUpdatedEvent, _super);
            function UserItemUpdatedEvent(principal, userStore) {
                _super.call(this);
                this.principal = principal;
                this.userStore = userStore;
            }
            UserItemUpdatedEvent.prototype.getPrincipal = function () {
                return this.principal;
            };
            UserItemUpdatedEvent.prototype.getUserStore = function () {
                return this.userStore;
            };
            UserItemUpdatedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            UserItemUpdatedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return UserItemUpdatedEvent;
        })(api.event.Event);
        security.UserItemUpdatedEvent = UserItemUpdatedEvent;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var UserItemDeletedEvent = (function (_super) {
            __extends(UserItemDeletedEvent, _super);
            function UserItemDeletedEvent(builder) {
                _super.call(this);
                this.principals = builder.principals;
                this.userStores = builder.userStores;
            }
            UserItemDeletedEvent.prototype.getPrincipals = function () {
                return this.principals;
            };
            UserItemDeletedEvent.prototype.getUserStores = function () {
                return this.userStores;
            };
            UserItemDeletedEvent.create = function () {
                return new UserItemDeletedEventBuilder();
            };
            UserItemDeletedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            UserItemDeletedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return UserItemDeletedEvent;
        })(api.event.Event);
        security.UserItemDeletedEvent = UserItemDeletedEvent;
        var UserItemDeletedEventBuilder = (function () {
            function UserItemDeletedEventBuilder() {
            }
            UserItemDeletedEventBuilder.prototype.setPrincipals = function (principals) {
                this.principals = principals;
                return this;
            };
            UserItemDeletedEventBuilder.prototype.setUserStores = function (userStores) {
                this.userStores = userStores;
                return this;
            };
            UserItemDeletedEventBuilder.prototype.build = function () {
                return new UserItemDeletedEvent(this);
            };
            return UserItemDeletedEventBuilder;
        })();
        security.UserItemDeletedEventBuilder = UserItemDeletedEventBuilder;
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (acl) {
            (function (Permission) {
                Permission[Permission["READ"] = 0] = "READ";
                Permission[Permission["CREATE"] = 1] = "CREATE";
                Permission[Permission["MODIFY"] = 2] = "MODIFY";
                Permission[Permission["DELETE"] = 3] = "DELETE";
                Permission[Permission["PUBLISH"] = 4] = "PUBLISH";
                Permission[Permission["READ_PERMISSIONS"] = 5] = "READ_PERMISSIONS";
                Permission[Permission["WRITE_PERMISSIONS"] = 6] = "WRITE_PERMISSIONS";
            })(acl.Permission || (acl.Permission = {}));
            var Permission = acl.Permission;
            (function (PermissionState) {
                PermissionState[PermissionState["ALLOW"] = 0] = "ALLOW";
                PermissionState[PermissionState["DENY"] = 1] = "DENY";
                PermissionState[PermissionState["INHERIT"] = 2] = "INHERIT";
            })(acl.PermissionState || (acl.PermissionState = {}));
            var PermissionState = acl.PermissionState;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (acl) {
            var PermissionHelper = (function () {
                function PermissionHelper() {
                }
                PermissionHelper.hasPermission = function (permission, loginResult, accessControlList) {
                    var _this = this;
                    var result = false;
                    var entries = accessControlList.getEntries();
                    var accessEntriesWithGivenPermissions = entries.filter(function (item) {
                        return item.isAllowed(permission);
                    });
                    loginResult.getPrincipals().some(function (principalKey) {
                        if (api.security.RoleKeys.ADMIN.equals(principalKey) || _this.isPrincipalPresent(principalKey, accessEntriesWithGivenPermissions)) {
                            result = true;
                            return true;
                        }
                    });
                    return result;
                };
                PermissionHelper.isPrincipalPresent = function (principalKey, accessEntriesToCheck) {
                    var result = false;
                    accessEntriesToCheck.some(function (entry) {
                        if (entry.getPrincipalKey().equals(principalKey)) {
                            result = true;
                            return true;
                        }
                    });
                    return result;
                };
                return PermissionHelper;
            })();
            acl.PermissionHelper = PermissionHelper;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (acl) {
            var ArrayHelper = api.util.ArrayHelper;
            var Principal = api.security.Principal;
            var AccessControlEntry = (function () {
                function AccessControlEntry(principal) {
                    this.principal = api.util.assertNotNull(principal);
                    this.allowedPermissions = [];
                    this.deniedPermissions = [];
                }
                AccessControlEntry.prototype.getPrincipal = function () {
                    return this.principal;
                };
                AccessControlEntry.prototype.getPrincipalKey = function () {
                    return this.principal.getKey();
                };
                AccessControlEntry.prototype.getPrincipalDisplayName = function () {
                    return this.principal.getDisplayName();
                };
                AccessControlEntry.prototype.getPrincipalTypeName = function () {
                    return this.principal.getTypeName();
                };
                AccessControlEntry.prototype.getAllowedPermissions = function () {
                    return this.allowedPermissions;
                };
                AccessControlEntry.prototype.getDeniedPermissions = function () {
                    return this.deniedPermissions;
                };
                AccessControlEntry.prototype.setAllowedPermissions = function (permissions) {
                    this.allowedPermissions = permissions;
                };
                AccessControlEntry.prototype.setDeniedPermissions = function (permissions) {
                    this.deniedPermissions = permissions;
                };
                AccessControlEntry.prototype.isAllowed = function (permission) {
                    return (this.allowedPermissions.indexOf(permission) > -1) && (this.deniedPermissions.indexOf(permission) === -1);
                };
                AccessControlEntry.prototype.isDenied = function (permission) {
                    return !this.isAllowed(permission);
                };
                AccessControlEntry.prototype.isSet = function (permission) {
                    return (this.allowedPermissions.indexOf(permission) > -1) || (this.deniedPermissions.indexOf(permission) > -1);
                };
                AccessControlEntry.prototype.allow = function (permission) {
                    ArrayHelper.addUnique(permission, this.allowedPermissions);
                    ArrayHelper.removeValue(permission, this.deniedPermissions);
                    return this;
                };
                AccessControlEntry.prototype.deny = function (permission) {
                    ArrayHelper.addUnique(permission, this.deniedPermissions);
                    ArrayHelper.removeValue(permission, this.allowedPermissions);
                    return this;
                };
                AccessControlEntry.prototype.remove = function (permission) {
                    ArrayHelper.removeValue(permission, this.allowedPermissions);
                    ArrayHelper.removeValue(permission, this.deniedPermissions);
                    return this;
                };
                AccessControlEntry.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, AccessControlEntry)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.equals(this.getPrincipalKey(), other.getPrincipalKey())) {
                        return false;
                    }
                    if (!api.ObjectHelper.anyArrayEquals(this.allowedPermissions, other.allowedPermissions)) {
                        return false;
                    }
                    if (!api.ObjectHelper.anyArrayEquals(this.deniedPermissions, other.deniedPermissions)) {
                        return false;
                    }
                    return true;
                };
                AccessControlEntry.prototype.toString = function () {
                    var _this = this;
                    var values = '';
                    AccessControlEntry.ALL_PERMISSIONS.forEach(function (permission) {
                        if (_this.isSet(permission)) {
                            if (values !== '') {
                                values += ', ';
                            }
                            values += _this.isAllowed(permission) ? '+' : '-';
                            values += acl.Permission[permission].toUpperCase();
                        }
                    });
                    return this.getPrincipalKey().toString() + '[' + values + ']';
                };
                AccessControlEntry.prototype.clone = function () {
                    var ace = new AccessControlEntry(this.principal.clone());
                    ace.allowedPermissions = this.allowedPermissions.slice(0);
                    ace.deniedPermissions = this.deniedPermissions.slice(0);
                    return ace;
                };
                AccessControlEntry.prototype.toJson = function () {
                    return {
                        "principal": this.principal.toJson(),
                        "allow": this.allowedPermissions.map(function (perm) { return acl.Permission[perm]; }),
                        "deny": this.deniedPermissions.map(function (perm) { return acl.Permission[perm]; })
                    };
                };
                AccessControlEntry.fromJson = function (json) {
                    var ace = new AccessControlEntry(Principal.fromJson(json.principal));
                    var allow = json.allow.map(function (permStr) { return acl.Permission[permStr.toUpperCase()]; });
                    var deny = json.deny.map(function (permStr) { return acl.Permission[permStr.toUpperCase()]; });
                    ace.setAllowedPermissions(allow);
                    ace.setDeniedPermissions(deny);
                    return ace;
                };
                AccessControlEntry.ALL_PERMISSIONS = [0 /* READ */, 1 /* CREATE */, 2 /* MODIFY */, 3 /* DELETE */, 4 /* PUBLISH */, 5 /* READ_PERMISSIONS */, 6 /* WRITE_PERMISSIONS */];
                return AccessControlEntry;
            })();
            acl.AccessControlEntry = AccessControlEntry;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (acl) {
            var PrincipalType = api.security.PrincipalType;
            var FindAccessControlEntriesRequest = (function (_super) {
                __extends(FindAccessControlEntriesRequest, _super);
                function FindAccessControlEntriesRequest() {
                    _super.call(this);
                }
                FindAccessControlEntriesRequest.prototype.getParams = function () {
                    return {
                        "types": this.enumToStrings(this.allowedTypes),
                        "query": this.searchQuery,
                        "userStoreKey": this.userStoreKey ? this.userStoreKey.toString() : undefined
                    };
                };
                FindAccessControlEntriesRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals');
                };
                FindAccessControlEntriesRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().principals.map(function (principalJson) {
                            return new acl.AccessControlEntry(_this.fromJsonToPrincipal(principalJson));
                        });
                    });
                };
                FindAccessControlEntriesRequest.prototype.enumToStrings = function (types) {
                    return types.map(function (type) {
                        return PrincipalType[type].toUpperCase();
                    });
                };
                FindAccessControlEntriesRequest.prototype.setUserStoreKey = function (key) {
                    this.userStoreKey = key;
                    return this;
                };
                FindAccessControlEntriesRequest.prototype.setAllowedTypes = function (types) {
                    this.allowedTypes = types;
                    return this;
                };
                FindAccessControlEntriesRequest.prototype.setSearchQuery = function (query) {
                    this.searchQuery = query;
                    return this;
                };
                return FindAccessControlEntriesRequest;
            })(api.security.SecurityResourceRequest);
            acl.FindAccessControlEntriesRequest = FindAccessControlEntriesRequest;
            var AccessControlEntryLoader = (function (_super) {
                __extends(AccessControlEntryLoader, _super);
                function AccessControlEntryLoader() {
                    this.findRequest = new FindAccessControlEntriesRequest();
                    this.setAllowedTypes([1 /* GROUP */, 0 /* USER */, 2 /* ROLE */]);
                    _super.call(this, this.findRequest);
                }
                AccessControlEntryLoader.prototype.setUserStoreKey = function (key) {
                    this.findRequest.setUserStoreKey(key);
                    return this;
                };
                AccessControlEntryLoader.prototype.setAllowedTypes = function (principalTypes) {
                    this.findRequest.setAllowedTypes(principalTypes);
                    return this;
                };
                AccessControlEntryLoader.prototype.search = function (searchString) {
                    this.findRequest.setSearchQuery(searchString);
                    return this.load();
                };
                return AccessControlEntryLoader;
            })(api.util.loader.BaseLoader);
            acl.AccessControlEntryLoader = AccessControlEntryLoader;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (_acl) {
            var AccessControlList = (function () {
                function AccessControlList(entries) {
                    this.entries = {};
                    if (entries) {
                        this.addAll(entries);
                    }
                }
                AccessControlList.prototype.getEntries = function () {
                    var values = [];
                    for (var key in this.entries) {
                        if (this.entries.hasOwnProperty(key)) {
                            values.push(this.entries[key]);
                        }
                    }
                    return values;
                };
                AccessControlList.prototype.getEntry = function (principalKey) {
                    return this.entries[principalKey.toString()];
                };
                AccessControlList.prototype.add = function (entry) {
                    this.entries[entry.getPrincipalKey().toString()] = entry;
                };
                AccessControlList.prototype.addAll = function (entries) {
                    var _this = this;
                    entries.forEach(function (entry) {
                        _this.entries[entry.getPrincipalKey().toString()] = entry;
                    });
                };
                AccessControlList.prototype.contains = function (principalKey) {
                    return this.entries.hasOwnProperty(principalKey.toString());
                };
                AccessControlList.prototype.remove = function (principalKey) {
                    delete this.entries[principalKey.toString()];
                };
                AccessControlList.prototype.toJson = function () {
                    var acl = [];
                    this.getEntries().forEach(function (entry) {
                        var entryJson = entry.toJson();
                        acl.push(entryJson);
                    });
                    return acl;
                };
                AccessControlList.prototype.toString = function () {
                    return '[' + this.getEntries().map(function (ace) { return ace.toString(); }).join(', ') + ']';
                };
                AccessControlList.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, AccessControlList)) {
                        return false;
                    }
                    var other = o;
                    return api.ObjectHelper.arrayEquals(this.getEntries().sort(), other.getEntries().sort());
                };
                AccessControlList.prototype.clone = function () {
                    var entries = [];
                    for (var key in this.entries) {
                        if (this.entries.hasOwnProperty(key)) {
                            entries.push(this.entries[key].clone());
                        }
                    }
                    return new AccessControlList(entries);
                };
                AccessControlList.fromJson = function (json) {
                    var acl = new AccessControlList();
                    json.forEach(function (entryJson) {
                        var entry = _acl.AccessControlEntry.fromJson(entryJson);
                        acl.add(entry);
                    });
                    return acl;
                };
                return AccessControlList;
            })();
            _acl.AccessControlList = AccessControlList;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (acl) {
            (function (UserStoreAccess) {
                UserStoreAccess[UserStoreAccess["READ"] = 0] = "READ";
                UserStoreAccess[UserStoreAccess["CREATE_USERS"] = 1] = "CREATE_USERS";
                UserStoreAccess[UserStoreAccess["WRITE_USERS"] = 2] = "WRITE_USERS";
                UserStoreAccess[UserStoreAccess["USER_STORE_MANAGER"] = 3] = "USER_STORE_MANAGER";
                UserStoreAccess[UserStoreAccess["ADMINISTRATOR"] = 4] = "ADMINISTRATOR";
            })(acl.UserStoreAccess || (acl.UserStoreAccess = {}));
            var UserStoreAccess = acl.UserStoreAccess;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (acl) {
            var UserStoreAccessControlEntry = (function () {
                function UserStoreAccessControlEntry(principal, access) {
                    api.util.assertNotNull(principal, "principal not set");
                    this.principal = principal;
                    this.access = access;
                }
                UserStoreAccessControlEntry.prototype.getPrincipal = function () {
                    return this.principal;
                };
                UserStoreAccessControlEntry.prototype.getAccess = function () {
                    return this.access;
                };
                UserStoreAccessControlEntry.prototype.setAccess = function (value) {
                    this.access = acl.UserStoreAccess[value];
                    return this;
                };
                UserStoreAccessControlEntry.prototype.getPrincipalKey = function () {
                    return this.principal.getKey();
                };
                UserStoreAccessControlEntry.prototype.getPrincipalDisplayName = function () {
                    return this.principal.getDisplayName();
                };
                UserStoreAccessControlEntry.prototype.getPrincipalTypeName = function () {
                    return this.principal.getTypeName();
                };
                UserStoreAccessControlEntry.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, UserStoreAccessControlEntry)) {
                        return false;
                    }
                    var other = o;
                    return this.principal.equals(other.getPrincipal()) && this.access === other.access;
                };
                UserStoreAccessControlEntry.prototype.getId = function () {
                    return this.principal.getKey().toString();
                };
                UserStoreAccessControlEntry.prototype.toString = function () {
                    return this.principal.getKey().toString() + '[' + acl.UserStoreAccess[this.access] + ']';
                };
                UserStoreAccessControlEntry.prototype.toJson = function () {
                    return {
                        "principal": this.principal.toJson(),
                        "access": acl.UserStoreAccess[this.access]
                    };
                };
                UserStoreAccessControlEntry.fromJson = function (json) {
                    return new UserStoreAccessControlEntry(security.Principal.fromJson(json.principal), acl.UserStoreAccess[json.access.toUpperCase()]);
                };
                return UserStoreAccessControlEntry;
            })();
            acl.UserStoreAccessControlEntry = UserStoreAccessControlEntry;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (_acl) {
            var UserStoreAccessControlList = (function () {
                function UserStoreAccessControlList(entries) {
                    this.entries = {};
                    if (entries) {
                        this.addAll(entries);
                    }
                }
                UserStoreAccessControlList.prototype.getEntries = function () {
                    var values = [];
                    for (var key in this.entries) {
                        if (this.entries.hasOwnProperty(key)) {
                            values.push(this.entries[key]);
                        }
                    }
                    return values;
                };
                UserStoreAccessControlList.prototype.getEntry = function (principalKey) {
                    return this.entries[principalKey.toString()];
                };
                UserStoreAccessControlList.prototype.add = function (entry) {
                    this.entries[entry.getPrincipal().getKey().toString()] = entry;
                };
                UserStoreAccessControlList.prototype.addAll = function (entries) {
                    var _this = this;
                    entries.forEach(function (entry) {
                        _this.entries[entry.getPrincipal().getKey().toString()] = entry;
                    });
                };
                UserStoreAccessControlList.prototype.contains = function (principalKey) {
                    return this.entries.hasOwnProperty(principalKey.toString());
                };
                UserStoreAccessControlList.prototype.remove = function (principalKey) {
                    delete this.entries[principalKey.toString()];
                };
                UserStoreAccessControlList.prototype.toJson = function () {
                    var acl = [];
                    this.getEntries().forEach(function (entry) {
                        var entryJson = entry.toJson();
                        acl.push(entryJson);
                    });
                    return acl;
                };
                UserStoreAccessControlList.prototype.toString = function () {
                    return '[' + this.getEntries().map(function (ace) { return ace.toString(); }).join(', ') + ']';
                };
                UserStoreAccessControlList.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, UserStoreAccessControlList)) {
                        return false;
                    }
                    var other = o;
                    return api.ObjectHelper.arrayEquals(this.getEntries().sort(), other.getEntries().sort());
                };
                UserStoreAccessControlList.fromJson = function (json) {
                    var acl = new UserStoreAccessControlList();
                    json.forEach(function (entryJson) {
                        var entry = _acl.UserStoreAccessControlEntry.fromJson(entryJson);
                        acl.add(entry);
                    });
                    return acl;
                };
                UserStoreAccessControlList.prototype.clone = function () {
                    var result = new UserStoreAccessControlList();
                    var clonedEntries = {};
                    this.getEntries().forEach(function (item) {
                        var clonedItem = new _acl.UserStoreAccessControlEntry(item.getPrincipal().clone(), item.getAccess());
                        result.add(clonedItem);
                    });
                    return result;
                };
                return UserStoreAccessControlList;
            })();
            _acl.UserStoreAccessControlList = UserStoreAccessControlList;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var acl;
        (function (acl) {
            var PrincipalType = api.security.PrincipalType;
            var FindUserStoreAccessControlEntriesRequest = (function (_super) {
                __extends(FindUserStoreAccessControlEntriesRequest, _super);
                function FindUserStoreAccessControlEntriesRequest() {
                    _super.call(this);
                }
                FindUserStoreAccessControlEntriesRequest.prototype.getParams = function () {
                    return {
                        "query": this.searchQuery
                    };
                };
                FindUserStoreAccessControlEntriesRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'principals');
                };
                FindUserStoreAccessControlEntriesRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().principals.map(function (principalJson) {
                            return new acl.UserStoreAccessControlEntry(_this.fromJsonToPrincipal(principalJson));
                        });
                    });
                };
                FindUserStoreAccessControlEntriesRequest.prototype.enumToStrings = function (types) {
                    return types.map(function (type) {
                        return PrincipalType[type].toUpperCase();
                    });
                };
                FindUserStoreAccessControlEntriesRequest.prototype.setUserStoreKey = function (key) {
                    this.userStoreKey = key;
                    return this;
                };
                FindUserStoreAccessControlEntriesRequest.prototype.setAllowedTypes = function (types) {
                    this.allowedTypes = types;
                    return this;
                };
                FindUserStoreAccessControlEntriesRequest.prototype.setSearchQuery = function (query) {
                    this.searchQuery = query;
                    return this;
                };
                return FindUserStoreAccessControlEntriesRequest;
            })(api.security.SecurityResourceRequest);
            acl.FindUserStoreAccessControlEntriesRequest = FindUserStoreAccessControlEntriesRequest;
            var UserStoreAccessControlEntryLoader = (function (_super) {
                __extends(UserStoreAccessControlEntryLoader, _super);
                function UserStoreAccessControlEntryLoader() {
                    this.findRequest = new FindUserStoreAccessControlEntriesRequest();
                    this.setAllowedTypes([1 /* GROUP */, 0 /* USER */, 2 /* ROLE */]);
                    _super.call(this, this.findRequest);
                }
                UserStoreAccessControlEntryLoader.prototype.setUserStoreKey = function (key) {
                    this.findRequest.setUserStoreKey(key);
                    return this;
                };
                UserStoreAccessControlEntryLoader.prototype.setAllowedTypes = function (principalTypes) {
                    this.findRequest.setAllowedTypes(principalTypes);
                    return this;
                };
                UserStoreAccessControlEntryLoader.prototype.search = function (searchString) {
                    this.findRequest.setSearchQuery(searchString);
                    return this.load();
                };
                return UserStoreAccessControlEntryLoader;
            })(api.util.loader.BaseLoader);
            acl.UserStoreAccessControlEntryLoader = UserStoreAccessControlEntryLoader;
        })(acl = security.acl || (security.acl = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var auth;
        (function (auth) {
            var LoginCredentials = (function () {
                function LoginCredentials() {
                    this.rememberMe = false;
                }
                LoginCredentials.prototype.getUser = function () {
                    return this.user;
                };
                LoginCredentials.prototype.getPassword = function () {
                    return this.password;
                };
                LoginCredentials.prototype.isRememberMe = function () {
                    return this.rememberMe;
                };
                LoginCredentials.prototype.setUser = function (user) {
                    this.user = user;
                    return this;
                };
                LoginCredentials.prototype.setPassword = function (pwd) {
                    this.password = pwd;
                    return this;
                };
                LoginCredentials.prototype.setRememberMe = function (value) {
                    this.rememberMe = value;
                    return this;
                };
                return LoginCredentials;
            })();
            auth.LoginCredentials = LoginCredentials;
        })(auth = security.auth || (security.auth = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var auth;
        (function (auth) {
            var LoginResult = (function () {
                function LoginResult(json) {
                    this.authenticated = json.authenticated;
                    if (json.user) {
                        this.user = api.security.User.fromJson(json.user);
                    }
                    this.applications = json.applications || [];
                    this.principals = json.principals ? json.principals.map(function (principal) { return api.security.PrincipalKey.fromString(principal); }) : [];
                    this.message = json.message;
                }
                LoginResult.prototype.isAuthenticated = function () {
                    return this.authenticated;
                };
                LoginResult.prototype.getUser = function () {
                    return this.user;
                };
                LoginResult.prototype.getApplications = function () {
                    return this.applications;
                };
                LoginResult.prototype.getPrincipals = function () {
                    return this.principals;
                };
                LoginResult.prototype.getMessage = function () {
                    return this.message;
                };
                return LoginResult;
            })();
            auth.LoginResult = LoginResult;
        })(auth = security.auth || (security.auth = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var auth;
        (function (auth) {
            var AuthResourceRequest = (function (_super) {
                __extends(AuthResourceRequest, _super);
                function AuthResourceRequest() {
                    _super.call(this);
                    this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "auth");
                }
                AuthResourceRequest.prototype.getResourcePath = function () {
                    return this.resourcePath;
                };
                return AuthResourceRequest;
            })(api.rest.ResourceRequest);
            auth.AuthResourceRequest = AuthResourceRequest;
        })(auth = security.auth || (security.auth = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var auth;
        (function (auth) {
            var LoginRequest = (function (_super) {
                __extends(LoginRequest, _super);
                function LoginRequest(loginCredentials) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "POST");
                    this.loginCredentials = loginCredentials;
                }
                LoginRequest.prototype.getParams = function () {
                    return {
                        user: this.loginCredentials.getUser(),
                        password: this.loginCredentials.getPassword(),
                        rememberMe: this.loginCredentials.isRememberMe()
                    };
                };
                LoginRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'login');
                };
                LoginRequest.prototype.sendAndParse = function () {
                    return this.send().then(function (response) {
                        return new auth.LoginResult(response.getResult());
                    });
                };
                return LoginRequest;
            })(auth.AuthResourceRequest);
            auth.LoginRequest = LoginRequest;
        })(auth = security.auth || (security.auth = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var auth;
        (function (auth) {
            var LogoutRequest = (function (_super) {
                __extends(LogoutRequest, _super);
                function LogoutRequest() {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "POST");
                }
                LogoutRequest.prototype.getParams = function () {
                    return {};
                };
                LogoutRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'logout');
                };
                LogoutRequest.prototype.sendAndParse = function () {
                    return this.send().then(function (response) {
                        return;
                    });
                };
                return LogoutRequest;
            })(auth.AuthResourceRequest);
            auth.LogoutRequest = LogoutRequest;
        })(auth = security.auth || (security.auth = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var security;
    (function (security) {
        var auth;
        (function (auth) {
            var IsAuthenticatedRequest = (function (_super) {
                __extends(IsAuthenticatedRequest, _super);
                function IsAuthenticatedRequest() {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                }
                IsAuthenticatedRequest.prototype.getParams = function () {
                    return {};
                };
                IsAuthenticatedRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'authenticated');
                };
                IsAuthenticatedRequest.prototype.sendAndParse = function () {
                    return this.send().then(function (response) {
                        return new auth.LoginResult(response.getResult());
                    });
                };
                return IsAuthenticatedRequest;
            })(auth.AuthResourceRequest);
            auth.IsAuthenticatedRequest = IsAuthenticatedRequest;
        })(auth = security.auth || (security.auth = {}));
    })(security = api.security || (api.security = {}));
})(api || (api = {}));
var api;
(function (api) {
    var locale;
    (function (locale) {
        var Locale = (function () {
            function Locale() {
            }
            Locale.prototype.equals = function (other) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(other, Locale)) {
                    return false;
                }
                var o = other;
                return this.tag == o.tag && this.displayName == o.displayName && this.language == o.language && this.displayLanguage == o.displayLanguage && this.variant == o.variant && this.displayVariant == o.displayVariant && this.country == o.displayCountry && this.displayCountry == o.displayCountry;
            };
            Locale.fromJson = function (json) {
                var l = new Locale();
                l.tag = json.tag;
                l.displayName = json.displayName;
                l.country = json.country;
                l.displayCountry = json.displayCountry;
                l.variant = json.variant;
                l.displayVariant = json.displayVariant;
                l.language = json.language;
                l.displayLanguage = json.displayLanguage;
                return l;
            };
            Locale.prototype.getTag = function () {
                return this.tag;
            };
            Locale.prototype.getDisplayName = function () {
                return this.displayName;
            };
            Locale.prototype.getLanguage = function () {
                return this.language;
            };
            Locale.prototype.getDisplayLanguage = function () {
                return this.displayLanguage;
            };
            Locale.prototype.getVariant = function () {
                return this.variant;
            };
            Locale.prototype.getDisplayVariant = function () {
                return this.displayVariant;
            };
            Locale.prototype.getCountry = function () {
                return this.country;
            };
            Locale.prototype.getDisplayCountry = function () {
                return this.displayCountry;
            };
            return Locale;
        })();
        locale.Locale = Locale;
    })(locale = api.locale || (api.locale = {}));
})(api || (api = {}));
var api;
(function (api) {
    var locale;
    (function (locale) {
        var GetLocalesRequest = (function (_super) {
            __extends(GetLocalesRequest, _super);
            function GetLocalesRequest() {
                _super.call(this);
            }
            GetLocalesRequest.prototype.getParams = function () {
                return {
                    "query": this.searchQuery
                };
            };
            GetLocalesRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), 'content', 'locales');
            };
            GetLocalesRequest.prototype.setSearchQuery = function (query) {
                this.searchQuery = query;
                return this;
            };
            GetLocalesRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return response.getResult().locales.map(function (localeJson) {
                        return locale.Locale.fromJson(localeJson);
                    }).sort(_this.sortFunction);
                });
            };
            GetLocalesRequest.prototype.sortFunction = function (a, b) {
                return a.getDisplayName().localeCompare(b.getDisplayName());
            };
            return GetLocalesRequest;
        })(api.rest.ResourceRequest);
        locale.GetLocalesRequest = GetLocalesRequest;
    })(locale = api.locale || (api.locale = {}));
})(api || (api = {}));
var api;
(function (api) {
    var locale;
    (function (locale) {
        var LocaleLoader = (function (_super) {
            __extends(LocaleLoader, _super);
            function LocaleLoader() {
                this.getLocalesRequest = new locale.GetLocalesRequest();
                _super.call(this, this.getLocalesRequest);
            }
            LocaleLoader.prototype.search = function (searchString) {
                this.getLocalesRequest.setSearchQuery(searchString);
                return this.load();
            };
            LocaleLoader.prototype.load = function () {
                var _this = this;
                this.notifyLoadingData();
                return this.sendRequest().then(function (locales) {
                    _this.notifyLoadedData(locales);
                    if (_this.preservedSearchString) {
                        _this.search(_this.preservedSearchString);
                        _this.preservedSearchString = null;
                    }
                    return locales;
                });
            };
            return LocaleLoader;
        })(api.util.loader.BaseLoader);
        locale.LocaleLoader = LocaleLoader;
    })(locale = api.locale || (api.locale = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var responsive;
        (function (responsive) {
            var ResponsiveRange = (function () {
                function ResponsiveRange(minRange, maxRange, rangeClass) {
                    this.minRange = minRange;
                    this.maxRange = maxRange || 0;
                    this.rangeClass = rangeClass || ("_" + minRange + "-" + maxRange);
                }
                ResponsiveRange.prototype.getMinimumRange = function () {
                    return this.minRange;
                };
                ResponsiveRange.prototype.getMaximumRange = function () {
                    return this.maxRange;
                };
                ResponsiveRange.prototype.getRangeClass = function () {
                    return this.rangeClass;
                };
                ResponsiveRange.prototype.isFit = function (size) {
                    return (this.minRange <= size) && (size <= this.maxRange);
                };
                ResponsiveRange.prototype.isFitOrSmaller = function (size) {
                    return size <= this.maxRange;
                };
                ResponsiveRange.prototype.isFitOrBigger = function (size) {
                    return size > this.minRange;
                };
                return ResponsiveRange;
            })();
            responsive.ResponsiveRange = ResponsiveRange;
        })(responsive = ui.responsive || (ui.responsive = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var responsive;
        (function (responsive) {
            var ResponsiveRanges = (function () {
                function ResponsiveRanges() {
                }
                ResponsiveRanges._0_240 = new responsive.ResponsiveRange(0, 240);
                ResponsiveRanges._240_360 = new responsive.ResponsiveRange(240, 360);
                ResponsiveRanges._360_540 = new responsive.ResponsiveRange(360, 540);
                ResponsiveRanges._540_720 = new responsive.ResponsiveRange(540, 720);
                ResponsiveRanges._720_960 = new responsive.ResponsiveRange(720, 960);
                ResponsiveRanges._960_1200 = new responsive.ResponsiveRange(960, 1200);
                ResponsiveRanges._1200_1380 = new responsive.ResponsiveRange(1200, 1380);
                ResponsiveRanges._1380_1620 = new responsive.ResponsiveRange(1380, 1620);
                ResponsiveRanges._1620_1920 = new responsive.ResponsiveRange(1620, 1920);
                ResponsiveRanges._1920_UP = new responsive.ResponsiveRange(1920, Infinity);
                return ResponsiveRanges;
            })();
            responsive.ResponsiveRanges = ResponsiveRanges;
        })(responsive = ui.responsive || (ui.responsive = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var responsive;
        (function (responsive) {
            var ResponsiveItem = (function () {
                function ResponsiveItem(element, handler) {
                    if (handler === void 0) { handler = (function (item) {
                    }); }
                    this.element = element;
                    this.rangeValue = this.element.getEl().getWidthWithBorder();
                    this.oldRangeValue = this.rangeValue;
                    this.handle = handler;
                    this.fitToRange();
                    this.element.getEl().addClass(this.rangeSize.getRangeClass());
                }
                ResponsiveItem.prototype.fitToRange = function () {
                    for (var key in responsive.ResponsiveRanges) {
                        var range = responsive.ResponsiveRanges[key];
                        if (range && (api.ObjectHelper.iFrameSafeInstanceOf(range, responsive.ResponsiveRange)) && range.isFit(this.rangeValue)) {
                            this.rangeSize = range;
                            break;
                        }
                    }
                };
                ResponsiveItem.prototype.getElement = function () {
                    return this.element;
                };
                ResponsiveItem.prototype.update = function () {
                    var newRangeValue = this.element.getEl().getWidthWithBorder();
                    if (newRangeValue !== this.rangeValue) {
                        this.oldRangeValue = this.rangeValue;
                        this.oldRangeSize = this.rangeSize;
                        this.rangeValue = newRangeValue;
                        this.element.getEl().removeClass(this.rangeSize.getRangeClass());
                        this.fitToRange();
                        this.element.getEl().addClass(this.rangeSize.getRangeClass());
                    }
                    this.handle(this);
                };
                ResponsiveItem.prototype.isRangeSizeChanged = function () {
                    return this.rangeSize !== this.oldRangeSize;
                };
                ResponsiveItem.prototype.setHandler = function (handler) {
                    if (handler === void 0) { handler = (function (item) {
                    }); }
                    this.handle = handler;
                };
                ResponsiveItem.prototype.getRangeValue = function () {
                    return this.rangeValue;
                };
                ResponsiveItem.prototype.getOldRangeValue = function () {
                    return this.oldRangeValue;
                };
                ResponsiveItem.prototype.getRangeSize = function () {
                    return this.rangeSize;
                };
                ResponsiveItem.prototype.getOldRangeSize = function () {
                    return this.oldRangeSize;
                };
                ResponsiveItem.prototype.isInRange = function (range) {
                    return range.isFit(this.rangeValue);
                };
                ResponsiveItem.prototype.isInRangeOrSmaller = function (range) {
                    return range.isFitOrSmaller(this.rangeValue);
                };
                ResponsiveItem.prototype.isInRangeOrBigger = function (range) {
                    return range.isFitOrBigger(this.rangeValue);
                };
                return ResponsiveItem;
            })();
            responsive.ResponsiveItem = ResponsiveItem;
        })(responsive = ui.responsive || (ui.responsive = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var responsive;
        (function (responsive) {
            var ResponsiveListener = (function () {
                function ResponsiveListener(item, listener) {
                    this.item = item;
                    this.listener = listener;
                }
                ResponsiveListener.prototype.getItem = function () {
                    return this.item;
                };
                ResponsiveListener.prototype.getListener = function () {
                    return this.listener;
                };
                return ResponsiveListener;
            })();
            responsive.ResponsiveListener = ResponsiveListener;
        })(responsive = ui.responsive || (ui.responsive = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var responsive;
        (function (responsive) {
            var ResponsiveManager = (function () {
                function ResponsiveManager() {
                }
                ResponsiveManager.onAvailableSizeChanged = function (el, handler) {
                    if (handler === void 0) { handler = function (item) {
                    }; }
                    var responsiveItem = new responsive.ResponsiveItem(el, handler), listener = function () {
                        responsiveItem.update();
                    }, responsiveListener = new responsive.ResponsiveListener(responsiveItem, listener);
                    ResponsiveManager.responsiveListeners.push(responsiveListener);
                    ResponsiveManager.window.getHTMLElement().addEventListener('availablesizechange', listener);
                    ResponsiveManager.window.onResized(listener);
                    return responsiveItem;
                };
                ResponsiveManager.unAvailableSizeChanged = function (el) {
                    ResponsiveManager.responsiveListeners = ResponsiveManager.responsiveListeners.filter(function (curr) {
                        if (curr.getItem().getElement() === el) {
                            ResponsiveManager.window.getHTMLElement().removeEventListener('availablesizechange', curr.getListener());
                            ResponsiveManager.window.unResized(curr.getListener());
                            return false;
                        }
                        else {
                            return true;
                        }
                    });
                };
                ResponsiveManager.fireResizeEvent = function () {
                    var customEvent = document.createEvent('Event');
                    customEvent.initEvent('availablesizechange', false, true);
                    ResponsiveManager.window.getHTMLElement().dispatchEvent(customEvent);
                };
                ResponsiveManager.getWindow = function () {
                    return ResponsiveManager.window;
                };
                ResponsiveManager.window = api.dom.WindowDOM.get();
                ResponsiveManager.responsiveListeners = [];
                return ResponsiveManager;
            })();
            responsive.ResponsiveManager = ResponsiveManager;
            ResponsiveManager.onAvailableSizeChanged(api.dom.Body.get());
        })(responsive = ui.responsive || (ui.responsive = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var ValueChangedEvent = (function () {
            function ValueChangedEvent(oldValue, newValue) {
                this.oldValue = oldValue;
                this.newValue = newValue;
            }
            ValueChangedEvent.prototype.getOldValue = function () {
                return this.oldValue;
            };
            ValueChangedEvent.prototype.getNewValue = function () {
                return this.newValue;
            };
            return ValueChangedEvent;
        })();
        ui.ValueChangedEvent = ValueChangedEvent;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var KeyHelper = (function () {
            function KeyHelper() {
            }
            KeyHelper.isNumber = function (event) {
                return event.keyCode >= 48 && event.keyCode <= 57;
            };
            KeyHelper.isDash = function (event) {
                return event.keyCode === 189;
            };
            KeyHelper.isDel = function (event) {
                return event.keyCode === 46;
            };
            KeyHelper.isBackspace = function (event) {
                return event.keyCode === 8;
            };
            KeyHelper.isColon = function (event) {
                return event.keyCode === 186;
            };
            KeyHelper.isComma = function (event) {
                return event.keyCode === 188;
            };
            KeyHelper.isDot = function (event) {
                return event.keyCode === 190;
            };
            KeyHelper.isArrowKey = function (event) {
                return event.keyCode >= 37 && event.keyCode <= 40;
            };
            KeyHelper.isControlKey = function (event) {
                return event.keyCode === 17;
            };
            KeyHelper.isShiftKey = function (event) {
                return event.keyCode === 16;
            };
            KeyHelper.isAltKey = function (event) {
                return event.keyCode === 18;
            };
            KeyHelper.isMetaKey = function (event) {
                return event.keyCode === 224 || event.keyCode === 17 || event.keyCode === 91 || event.keyCode === 93;
            };
            KeyHelper.isTabKey = function (event) {
                return event.keyCode === 9;
            };
            KeyHelper.isModifierKey = function (event) {
                return KeyHelper.isControlKey(event) || KeyHelper.isShiftKey(event) || KeyHelper.isAltKey(event) || KeyHelper.isMetaKey(event);
            };
            KeyHelper.isEscKey = function (event) {
                return event.keyCode === 27;
            };
            KeyHelper.isEnterKey = function (event) {
                return event.keyCode === 13;
            };
            return KeyHelper;
        })();
        ui.KeyHelper = KeyHelper;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        (function (KeyBindingAction) {
            KeyBindingAction[KeyBindingAction["KEYDOWN"] = 0] = "KEYDOWN";
            KeyBindingAction[KeyBindingAction["KEYUP"] = 1] = "KEYUP";
            KeyBindingAction[KeyBindingAction["KEYPRESS"] = 2] = "KEYPRESS";
        })(ui.KeyBindingAction || (ui.KeyBindingAction = {}));
        var KeyBindingAction = ui.KeyBindingAction;
        var KeyBinding = (function () {
            function KeyBinding(combination, callback, action, global) {
                this.combination = combination;
                this.callback = callback;
                this.action = action;
                this.global = global;
            }
            KeyBinding.prototype.setCallback = function (func) {
                this.callback = func;
                return this;
            };
            KeyBinding.prototype.setAction = function (value) {
                this.action = value;
                return this;
            };
            KeyBinding.prototype.setGlobal = function (global) {
                this.global = global;
                return this;
            };
            KeyBinding.prototype.getCombination = function () {
                return this.combination;
            };
            KeyBinding.prototype.getCallback = function () {
                return this.callback;
            };
            KeyBinding.prototype.getAction = function () {
                return this.action;
            };
            KeyBinding.prototype.isGlobal = function () {
                return this.global;
            };
            KeyBinding.newKeyBinding = function (combination) {
                return new KeyBinding(combination);
            };
            KeyBinding.createMultiple = function (callback) {
                var combinations = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    combinations[_i - 1] = arguments[_i];
                }
                var bindings = [];
                combinations.forEach(function (combination) {
                    bindings.push(new KeyBinding(combination, callback));
                });
                return bindings;
            };
            return KeyBinding;
        })();
        ui.KeyBinding = KeyBinding;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var KeyBindings = (function () {
            function KeyBindings() {
                this.activeBindings = {};
                this.shelves = [];
                this.helpKeyPressedListeners = [];
                KeyBindings.instanceCount++;
                this.instance = KeyBindings.instanceCount;
                if (KeyBindings.debug) {
                    console.log("KeyBindings constructed instance #" + this.instance);
                }
                this.initializeHelpKey();
            }
            KeyBindings.get = function () {
                return KeyBindings.INSTANCE;
            };
            KeyBindings.prototype.bindKeys = function (bindings) {
                var _this = this;
                var logMessage = "Binded keys: [";
                bindings.forEach(function (binding) {
                    _this.bindKey(binding);
                    logMessage += "'" + binding.getCombination() + "' ,";
                });
                logMessage += "]";
                if (KeyBindings.debug) {
                    console.log("KeyBindings[#" + this.instance + "].bindKeys(): " + logMessage);
                }
            };
            KeyBindings.prototype.bindKey = function (binding) {
                if (binding.isGlobal()) {
                    Mousetrap.bindGlobal(binding.getCombination(), binding.getCallback(), binding.getAction() ? ui.KeyBindingAction[binding.getAction()].toLowerCase() : "");
                }
                else {
                    Mousetrap.bind(binding.getCombination(), binding.getCallback(), binding.getAction() ? ui.KeyBindingAction[binding.getAction()].toLowerCase() : "");
                }
                var bindingKey = binding.getAction() ? binding.getCombination() + "-" + binding.getAction() : binding.getCombination();
                this.activeBindings[bindingKey] = binding;
            };
            KeyBindings.prototype.unbindKeys = function (bindings) {
                var _this = this;
                var logMessage = "Binded keys: [";
                bindings.forEach(function (binding) {
                    _this.unbindKey(binding);
                    logMessage += "'" + binding.getCombination() + "' ,";
                });
                if (KeyBindings.debug) {
                    console.log("KeyBindings[#" + this.instance + "].unbindKeys(): " + logMessage);
                }
            };
            KeyBindings.prototype.unbindKey = function (binding) {
                Mousetrap.unbind(binding.getCombination());
                delete this.activeBindings[binding.getCombination()];
            };
            KeyBindings.prototype.trigger = function (combination, action) {
                Mousetrap.trigger(combination, action);
            };
            KeyBindings.prototype.reset = function () {
                if (KeyBindings.debug) {
                    console.log("KeyBindings[#" + this.instance + "].reset()");
                }
                Mousetrap.reset();
                this.activeBindings = {};
                this.shelves = [];
            };
            KeyBindings.prototype.getActiveBindings = function () {
                var _this = this;
                return Object.keys(this.activeBindings).map(function (key) {
                    return _this.activeBindings[key];
                });
            };
            KeyBindings.prototype.shelveBindings = function () {
                if (KeyBindings.debug) {
                    console.log("KeyBindings[#" + this.instance + "].shelveBindings(): ");
                }
                Mousetrap.reset();
                this.shelves.push(this.activeBindings);
                this.activeBindings = {};
                this.notifyHelpKeyPressed(null);
            };
            KeyBindings.prototype.unshelveBindings = function () {
                var previousMousetraps = this.shelves.pop();
                if (previousMousetraps == undefined) {
                    if (KeyBindings.debug) {
                        console.log("KeyBindings[#" + this.instance + "].unshelveBindings(): nothing to unshelve");
                    }
                    return;
                }
                if (KeyBindings.debug) {
                    console.log("KeyBindings[#" + this.instance + "].unshelveBindings(): unshelving... ");
                }
                this.activeBindings = {};
                Mousetrap.reset();
                for (var key in previousMousetraps) {
                    var mousetrap = previousMousetraps[key];
                    this.bindKey(mousetrap);
                }
            };
            KeyBindings.prototype.isActive = function (keyBinding) {
                for (var key in this.activeBindings) {
                    if (this.activeBindings[key] == keyBinding) {
                        return true;
                    }
                }
                return false;
            };
            KeyBindings.prototype.initializeHelpKey = function () {
                var _this = this;
                this.bindKey(new api.ui.KeyBinding("f2", function (e) {
                    _this.notifyHelpKeyPressed(e);
                }).setGlobal(false).setAction(0 /* KEYDOWN */));
                this.bindKey(new api.ui.KeyBinding("f2", function (e) {
                    _this.notifyHelpKeyPressed(e);
                }).setGlobal(false).setAction(1 /* KEYUP */));
            };
            KeyBindings.prototype.onHelpKeyPressed = function (listener) {
                this.helpKeyPressedListeners.push(listener);
            };
            KeyBindings.prototype.unHelpKeyPressed = function (listener) {
                this.helpKeyPressedListeners = this.helpKeyPressedListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            KeyBindings.prototype.notifyHelpKeyPressed = function (e) {
                var _this = this;
                this.helpKeyPressedListeners.forEach(function (listener) {
                    listener.call(_this, e);
                });
            };
            KeyBindings.instanceCount = 0;
            KeyBindings.INSTANCE = new KeyBindings();
            KeyBindings.debug = false;
            return KeyBindings;
        })();
        ui.KeyBindings = KeyBindings;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var Mnemonic = (function () {
            function Mnemonic(value) {
                this.value = value;
            }
            Mnemonic.prototype.getValue = function () {
                return this.value;
            };
            Mnemonic.prototype.toKeyBinding = function (callback) {
                return new ui.KeyBinding("alt+" + this.getValue(), callback);
            };
            Mnemonic.prototype.underlineMnemonic = function (text) {
                var mStart = text.indexOf(this.value);
                if (mStart == -1) {
                    mStart = text.indexOf(this.value.toLowerCase());
                    if (mStart == -1) {
                        mStart = text.indexOf(this.value.toUpperCase());
                    }
                }
                var result = "";
                if (mStart > 0) {
                    result = text.substr(0, mStart);
                }
                result += "<u>" + text.charAt(mStart) + "</u>";
                if (mStart < text.length - 1) {
                    result += text.substr(mStart + 1, text.length);
                }
                return result;
            };
            return Mnemonic;
        })();
        ui.Mnemonic = Mnemonic;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        (function (IconSize) {
            IconSize[IconSize["SMALL"] = 0] = "SMALL";
            IconSize[IconSize["MEDIUM"] = 1] = "MEDIUM";
            IconSize[IconSize["LARGE"] = 2] = "LARGE";
        })(ui.IconSize || (ui.IconSize = {}));
        var IconSize = ui.IconSize;
        var Icon = (function (_super) {
            __extends(Icon, _super);
            function Icon(iconClass, size) {
                _super.call(this, iconClass);
                this.addClass("wem-icon");
                if (size == 0 /* SMALL */) {
                    this.addClass("icon-small");
                }
                else if (size == 1 /* MEDIUM */) {
                    this.addClass("icon-medium");
                }
                else if (size == 2 /* LARGE */) {
                    this.addClass("icon-large");
                }
            }
            return Icon;
        })(api.dom.IEl);
        ui.Icon = Icon;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var Action = (function () {
            function Action(label, shortcut, global) {
                var _this = this;
                this.enabled = true;
                this.visible = true;
                this.forceExecute = false;
                this.executionListeners = [];
                this.propertyChangedListeners = [];
                this.childActions = [];
                this.sortOrder = 10;
                this.beforeExecuteListeners = [];
                this.afterExecuteListeners = [];
                this.label = label;
                if (shortcut) {
                    this.shortcut = new ui.KeyBinding(shortcut).setGlobal(global).setCallback(function (e, combo) {
                        if (e.preventDefault) {
                            e.preventDefault();
                        }
                        else {
                            e.returnValue = false;
                        }
                        _this.execute();
                        return false;
                    });
                }
            }
            Action.prototype.setTitle = function (title) {
                this.title = title;
            };
            Action.prototype.getTitle = function () {
                return this.title;
            };
            Action.prototype.setSortOrder = function (sortOrder) {
                this.sortOrder = sortOrder;
            };
            Action.prototype.getSortOrder = function () {
                return this.sortOrder;
            };
            Action.prototype.setChildActions = function (actions) {
                var _this = this;
                actions.forEach(function (action) {
                    action.parentAction = _this;
                });
                this.childActions = actions;
                return this;
            };
            Action.prototype.hasChildActions = function () {
                return this.childActions.length > 0;
            };
            Action.prototype.hasParentAction = function () {
                return !!this.parentAction;
            };
            Action.prototype.getChildActions = function () {
                return this.childActions;
            };
            Action.prototype.getLabel = function () {
                return this.label;
            };
            Action.prototype.setLabel = function (value) {
                if (value !== this.label) {
                    this.label = value;
                    this.notifyPropertyChanged();
                }
            };
            Action.prototype.isEnabled = function () {
                return this.enabled;
            };
            Action.prototype.setEnabled = function (value) {
                if (value !== this.enabled) {
                    this.enabled = value;
                    this.notifyPropertyChanged();
                }
            };
            Action.prototype.isVisible = function () {
                return this.visible;
            };
            Action.prototype.setVisible = function (value) {
                if (value !== this.visible) {
                    this.visible = value;
                    this.notifyPropertyChanged();
                }
            };
            Action.prototype.getIconClass = function () {
                return this.iconClass;
            };
            Action.prototype.setIconClass = function (value) {
                if (value !== this.iconClass) {
                    this.iconClass = value;
                    this.notifyPropertyChanged();
                }
            };
            Action.prototype.notifyPropertyChanged = function () {
                for (var i in this.propertyChangedListeners) {
                    this.propertyChangedListeners[i](this);
                }
            };
            Action.prototype.hasShortcut = function () {
                return this.shortcut != null;
            };
            Action.prototype.getShortcut = function () {
                return this.shortcut;
            };
            Action.prototype.setMnemonic = function (value) {
                this.mnemonic = new ui.Mnemonic(value);
            };
            Action.prototype.hasMnemonic = function () {
                return this.mnemonic != null;
            };
            Action.prototype.getMnemonic = function () {
                return this.mnemonic;
            };
            Action.prototype.execute = function (forceExecute) {
                if (forceExecute === void 0) { forceExecute = false; }
                if (this.enabled) {
                    this.notifyBeforeExecute();
                    this.forceExecute = forceExecute;
                    for (var i in this.executionListeners) {
                        this.executionListeners[i](this);
                    }
                    this.forceExecute = false;
                    this.notifyAfterExecute();
                }
            };
            Action.prototype.onExecuted = function (listener) {
                this.executionListeners.push(listener);
                return this;
            };
            Action.prototype.unExecuted = function (listener) {
                this.executionListeners = this.executionListeners.filter(function (curr) {
                    return curr != listener;
                });
                return this;
            };
            Action.prototype.onPropertyChanged = function (listener) {
                this.propertyChangedListeners.push(listener);
            };
            Action.prototype.onBeforeExecute = function (listener) {
                this.beforeExecuteListeners.push(listener);
            };
            Action.prototype.unBeforeExecute = function (listener) {
                this.beforeExecuteListeners = this.beforeExecuteListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            Action.prototype.notifyBeforeExecute = function () {
                var _this = this;
                this.beforeExecuteListeners.forEach(function (listener) {
                    listener(_this);
                });
            };
            Action.prototype.onAfterExecute = function (listener) {
                this.afterExecuteListeners.push(listener);
            };
            Action.prototype.unAfterExecute = function (listener) {
                this.afterExecuteListeners = this.afterExecuteListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            Action.prototype.notifyAfterExecute = function () {
                var _this = this;
                this.afterExecuteListeners.forEach(function (listener) {
                    listener(_this);
                });
            };
            Action.prototype.clearListeners = function () {
                this.beforeExecuteListeners = [];
                this.afterExecuteListeners = [];
            };
            Action.prototype.getKeyBindings = function () {
                var _this = this;
                var bindings = [];
                if (this.hasShortcut()) {
                    bindings.push(this.getShortcut());
                }
                if (this.hasMnemonic()) {
                    bindings.push(this.getMnemonic().toKeyBinding(function () {
                        _this.execute();
                    }));
                }
                return bindings;
            };
            Action.getKeyBindings = function (actions) {
                var bindings = [];
                actions.forEach(function (action) {
                    action.getKeyBindings().forEach(function (keyBinding) {
                        bindings.push(keyBinding);
                    });
                });
                return bindings;
            };
            return Action;
        })();
        ui.Action = Action;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var Viewer = (function (_super) {
            __extends(Viewer, _super);
            function Viewer(className) {
                _super.call(this, new api.dom.NewElementBuilder().setTagName("div").setClassName('viewer ' + (className || '')).setGenerateId(false));
            }
            Viewer.prototype.setObject = function (object) {
                this.object = object;
            };
            Viewer.prototype.getObject = function () {
                return this.object;
            };
            Viewer.prototype.getPreferredHeight = function () {
                throw new Error("Must be implemented by inheritors");
            };
            return Viewer;
        })(api.dom.Element);
        ui.Viewer = Viewer;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var Tooltip = (function () {
            function Tooltip(target, text, showDelay, hideTimeout) {
                var _this = this;
                if (showDelay === void 0) { showDelay = 0; }
                if (hideTimeout === void 0) { hideTimeout = 0; }
                this.targetEl = target;
                this.text = text;
                this.showDelay = showDelay;
                this.hideTimeout = hideTimeout;
                this.side = Tooltip.SIDE_BOTTOM;
                this.overListener = function (event) {
                    _this.startShowDelay();
                };
                this.outListener = function (event) {
                    _this.startHideTimeout();
                };
                this.moveListener = function (event) {
                    if (_this.tooltipEl && _this.tooltipEl.isVisible()) {
                        _this.positionAtMouse(event);
                    }
                };
                this.setTrigger(Tooltip.TRIGGER_HOVER);
                this.setMode(Tooltip.MODE_STATIC);
                Tooltip.instances.push(this);
            }
            Tooltip.prototype.show = function () {
                this.stopTimeout();
                if (!this.tooltipEl) {
                    this.tooltipEl = new api.dom.DivEl("tooltip " + this.side);
                    if (this.contentEl) {
                        this.tooltipEl.appendChild(this.contentEl);
                    }
                    else {
                        this.tooltipEl.getEl().setInnerHtml(this.text);
                    }
                    var appendTo;
                    if (this.mode == Tooltip.MODE_STATIC) {
                        appendTo = this.targetEl.getParentElement() || this.targetEl;
                    }
                    else {
                        appendTo = api.dom.Body.get();
                    }
                    appendTo.appendChild(this.tooltipEl);
                    if (!Tooltip.multipleAllowed) {
                        this.hideOtherInstances();
                    }
                    this.tooltipEl.show();
                    if (this.mode == Tooltip.MODE_STATIC) {
                        this.positionByTarget();
                    }
                }
            };
            Tooltip.prototype.hide = function () {
                this.stopTimeout();
                if (this.tooltipEl) {
                    this.tooltipEl.remove();
                    this.tooltipEl = null;
                }
            };
            Tooltip.prototype.isVisible = function () {
                return this.tooltipEl && this.tooltipEl.isVisible();
            };
            Tooltip.prototype.showAfter = function (ms) {
                this.startShowDelay(ms);
                return this;
            };
            Tooltip.prototype.showFor = function (ms) {
                this.show();
                this.startHideTimeout(ms);
                return this;
            };
            Tooltip.prototype.setText = function (text) {
                this.text = text;
                this.contentEl = undefined;
                return this;
            };
            Tooltip.prototype.getText = function () {
                return this.text;
            };
            Tooltip.prototype.setContent = function (content) {
                this.contentEl = content;
                this.text = undefined;
                return this;
            };
            Tooltip.prototype.getContent = function () {
                return this.contentEl;
            };
            Tooltip.prototype.setHideTimeout = function (timeout) {
                this.hideTimeout = timeout;
                return this;
            };
            Tooltip.prototype.getHideTimeout = function () {
                return this.hideTimeout;
            };
            Tooltip.prototype.setShowDelay = function (delay) {
                this.showDelay = delay;
                return this;
            };
            Tooltip.prototype.getShowDelay = function () {
                return this.showDelay;
            };
            Tooltip.prototype.setTrigger = function (trigger) {
                if (trigger == this.trigger) {
                    return this;
                }
                this.targetEl.getEl().removeEventListener(this.getEventName(true), this.overListener).removeEventListener(this.getEventName(false), this.outListener);
                this.trigger = trigger;
                if (trigger != Tooltip.TRIGGER_NONE) {
                    this.targetEl.getEl().addEventListener(this.getEventName(true), this.overListener).addEventListener(this.getEventName(false), this.outListener);
                }
                return this;
            };
            Tooltip.prototype.getTrigger = function () {
                return this.trigger;
            };
            Tooltip.prototype.setSide = function (side) {
                this.side = side;
                return this;
            };
            Tooltip.prototype.getSide = function () {
                return this.side;
            };
            Tooltip.prototype.setMode = function (mode) {
                if (mode == this.mode) {
                    return this;
                }
                else if (mode == Tooltip.MODE_STATIC) {
                    api.dom.Body.get().unMouseMove(this.moveListener);
                }
                else if (mode == Tooltip.MODE_FOLLOW) {
                    api.dom.Body.get().onMouseMove(this.moveListener);
                }
                this.mode = mode;
                return this;
            };
            Tooltip.prototype.getMode = function () {
                return this.mode;
            };
            Tooltip.prototype.positionAtMouse = function (event) {
                var left, top, x = event.clientX, y = event.clientY, el = this.tooltipEl.getEl(), windowEl = api.dom.WindowDOM.get().getHTMLElement(), elProps = {
                    height: el.getHeightWithMargin(),
                    width: el.getWidthWithMargin(),
                    scrollLeft: this.mode == Tooltip.MODE_FOLLOW ? windowEl.scrollX : 0,
                    scrollTop: this.mode == Tooltip.MODE_FOLLOW ? windowEl.scrollY : 0
                };
                switch (this.side) {
                    case Tooltip.SIDE_TOP:
                        left = x - elProps.width / 2 + elProps.scrollLeft;
                        top = y - elProps.height + elProps.scrollTop;
                        break;
                    case Tooltip.SIDE_BOTTOM:
                        left = x - elProps.width / 2 + elProps.scrollLeft;
                        top = y + elProps.scrollTop;
                        break;
                    case Tooltip.SIDE_LEFT:
                        left = x - elProps.width + elProps.scrollLeft;
                        top = y - elProps.height / 2 + elProps.scrollTop;
                        break;
                    case Tooltip.SIDE_RIGHT:
                        left = x + elProps.scrollLeft;
                        top = y - elProps.height / 2 + elProps.scrollTop;
                        break;
                }
                this.tooltipEl.getEl().setLeftPx(left);
                this.tooltipEl.getEl().setTopPx(top);
            };
            Tooltip.prototype.positionByTarget = function () {
                var targetEl = this.targetEl.getHTMLElement(), targetOffset = this.targetEl.getEl().getOffset(), el = this.tooltipEl.getEl(), elProps = {
                    left: el.getMarginLeft() || 0,
                    top: el.getMarginTop() || 0,
                    height: el.getHeight(),
                    width: el.getWidth()
                };
                var offsetLeft, offsetTop;
                switch (this.side) {
                    case Tooltip.SIDE_TOP:
                        offsetLeft = targetOffset.left + (targetEl.offsetWidth - elProps.width) / 2 + elProps.left;
                        offsetTop = targetOffset.top - elProps.height + elProps.top;
                        break;
                    case Tooltip.SIDE_BOTTOM:
                        offsetLeft = targetOffset.left + (targetEl.offsetWidth - elProps.width) / 2 + elProps.left;
                        offsetTop = targetOffset.top + targetEl.offsetHeight + elProps.top;
                        break;
                    case Tooltip.SIDE_LEFT:
                        offsetLeft = targetOffset.left - elProps.width + elProps.left;
                        offsetTop = targetOffset.top + (targetEl.offsetHeight - elProps.height) / 2 + elProps.top;
                        break;
                    case Tooltip.SIDE_RIGHT:
                        offsetLeft = targetOffset.left + targetEl.offsetWidth + elProps.left;
                        offsetTop = targetOffset.top + (targetEl.offsetHeight - elProps.height) / 2 + elProps.top;
                        break;
                }
                if (offsetLeft < 0) {
                    offsetLeft = 0;
                }
                if (offsetTop < 0) {
                    offsetTop = 0;
                }
                el.setOffset({
                    left: offsetLeft,
                    top: offsetTop
                });
            };
            Tooltip.prototype.startHideTimeout = function (ms) {
                var _this = this;
                this.stopTimeout();
                var t = ms || this.hideTimeout;
                if (t > 0) {
                    this.timeoutTimer = setTimeout(function () {
                        _this.hide();
                    }, t);
                }
                else {
                    this.hide();
                }
            };
            Tooltip.prototype.startShowDelay = function (ms) {
                var _this = this;
                this.stopTimeout();
                var t = ms || this.showDelay;
                if (t > 0) {
                    if (this.trigger == Tooltip.TRIGGER_HOVER) {
                    }
                    this.timeoutTimer = setTimeout(function () {
                        _this.show();
                    }, t);
                }
                else {
                    this.show();
                }
            };
            Tooltip.prototype.stopTimeout = function () {
                if (this.timeoutTimer) {
                    clearTimeout(this.timeoutTimer);
                    this.timeoutTimer = undefined;
                }
            };
            Tooltip.prototype.hideOnMouseOut = function () {
                var tooltip = this;
                var mouseMoveListener = function (event) {
                    var tooltipTargetHtmlElement = tooltip.targetEl.getHTMLElement();
                    for (var element = event.target; element; element = element.parentNode) {
                        if (element == tooltipTargetHtmlElement) {
                            return;
                        }
                    }
                    tooltip.startHideTimeout();
                    api.dom.Body.get().unMouseMove(mouseMoveListener);
                };
                api.dom.Body.get().onMouseMove(mouseMoveListener);
            };
            Tooltip.prototype.getEventName = function (enter) {
                switch (this.trigger) {
                    case Tooltip.TRIGGER_FOCUS:
                        return enter ? "focus" : "blur";
                    case Tooltip.TRIGGER_HOVER:
                    default:
                        return enter ? "mouseenter" : "mouseleave";
                        break;
                }
            };
            Tooltip.prototype.hideOtherInstances = function () {
                var _this = this;
                Tooltip.instances.forEach(function (tooltip) {
                    if (tooltip != _this && tooltip.isVisible()) {
                        tooltip.hide();
                    }
                });
            };
            Tooltip.allowMultipleInstances = function (allow) {
                Tooltip.multipleAllowed = allow;
            };
            Tooltip.isMultipleInstancesAllowed = function () {
                return Tooltip.multipleAllowed;
            };
            Tooltip.SIDE_TOP = "top";
            Tooltip.SIDE_RIGHT = "right";
            Tooltip.SIDE_BOTTOM = "bottom";
            Tooltip.SIDE_LEFT = "left";
            Tooltip.TRIGGER_HOVER = "hover";
            Tooltip.TRIGGER_FOCUS = "focus";
            Tooltip.TRIGGER_NONE = "none";
            Tooltip.MODE_STATIC = "static";
            Tooltip.MODE_FOLLOW = "follow";
            Tooltip.multipleAllowed = true;
            Tooltip.instances = [];
            return Tooltip;
        })();
        ui.Tooltip = Tooltip;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var ProgressBar = (function (_super) {
            __extends(ProgressBar, _super);
            function ProgressBar(value) {
                _super.call(this, "progress-bar");
                this.value = value || 0;
                var progress = this.progress = new api.dom.DivEl("progress-indicator");
                this.getEl().appendChild(progress.getHTMLElement());
                this.setValue(this.value);
            }
            ProgressBar.prototype.setValue = function (value) {
                var normalizedValue = value > 0 ? this.normalizeValue(value) : 0;
                this.progress.getEl().setWidth(normalizedValue * 100 + "%");
                this.value = normalizedValue;
            };
            ProgressBar.prototype.getValue = function () {
                return this.value;
            };
            ProgressBar.prototype.normalizeValue = function (value) {
                var integralLength = Math.ceil(Math.log(value) / Math.log(10));
                var maxValue = Math.pow(10, integralLength);
                return value / maxValue;
            };
            return ProgressBar;
        })(api.dom.DivEl);
        ui.ProgressBar = ProgressBar;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var ToggleSlide = (function (_super) {
            __extends(ToggleSlide, _super);
            function ToggleSlide(actions) {
                var _this = this;
                _super.call(this, 'toggle-slide');
                this.enabled = true;
                this.animationDuration = 300;
                this.disabledClass = "disabled";
                this.actions = actions;
                actions.turnOnAction.onPropertyChanged(function (action) {
                    _this.setEnabled(action.isEnabled());
                    _this.setVisible(action.isVisible());
                });
                if (!actions.turnOnAction.isEnabled()) {
                    this.setEnabled(false);
                }
                this.createMarkup();
                this.calculateStyles();
                this.setupAnimation();
                var turnOn = actions.turnOnAction.isEnabled();
                if (turnOn) {
                    this.slideOn();
                }
                else {
                    this.slideOff();
                }
                this.isOn = turnOn;
                this.onClicked(function (event) {
                    if (_this.enabled) {
                        _this.toggle();
                    }
                });
            }
            ToggleSlide.prototype.toggle = function () {
                this.isOn ? this.turnOff() : this.turnOn();
            };
            ToggleSlide.prototype.turnOn = function () {
                this.slideOn();
                this.isOn = true;
                this.actions.turnOnAction.execute();
            };
            ToggleSlide.prototype.slideOn = function () {
                if (this.slideLeft.isRunning()) {
                    this.slideLeft.stop();
                }
                this.slideRight.start();
            };
            ToggleSlide.prototype.turnOff = function () {
                this.slideOff();
                this.isOn = false;
                this.actions.turnOffAction.execute();
            };
            ToggleSlide.prototype.slideOff = function () {
                if (this.slideRight.isRunning()) {
                    this.slideRight.stop();
                }
                this.slideLeft.start();
            };
            ToggleSlide.prototype.isTurnedOn = function () {
                return this.isOn;
            };
            ToggleSlide.prototype.setEnabled = function (enabled) {
                if (enabled != this.enabled) {
                    if (!enabled) {
                        this.addClass(this.disabledClass);
                    }
                    else {
                        this.removeClass(this.disabledClass);
                    }
                    this.enabled = enabled;
                }
            };
            ToggleSlide.prototype.isEnabled = function () {
                return this.enabled;
            };
            ToggleSlide.prototype.createMarkup = function () {
                this.slider = new api.dom.DivEl('slider');
                this.holder = new api.dom.DivEl('holder');
                this.onLabel = new api.dom.DivEl('on');
                this.offLabel = new api.dom.DivEl('off');
                this.appendChild(this.slider);
                this.appendChild(this.holder);
                this.holder.appendChild(this.onLabel);
                this.holder.appendChild(this.offLabel);
                this.onLabel.getEl().setInnerHtml(this.actions.turnOnAction.getLabel());
                this.offLabel.getEl().setInnerHtml(this.actions.turnOffAction.getLabel());
            };
            ToggleSlide.prototype.calculateStyles = function () {
                var sliderEl = this.slider.getEl(), onLabelEl = this.onLabel.getEl(), offLabelEl = this.offLabel.getEl();
                api.dom.Body.get().appendChild(this);
                var labelWidth = Math.max(onLabelEl.getWidth(), offLabelEl.getWidth());
                sliderEl.setWidth((labelWidth + 4) + 'px');
                this.sliderOffset = labelWidth - 4;
                onLabelEl.setWidth(labelWidth + 'px');
                offLabelEl.setWidth(labelWidth + 'px');
            };
            ToggleSlide.prototype.setupAnimation = function () {
                var _this = this;
                this.slideLeft = new api.util.Animation(this.animationDuration);
                this.slideLeft.onStep(function (progress) {
                    _this.slider.getEl().setLeft(_this.sliderOffset * (1 - progress) + 'px');
                });
                this.slideRight = new api.util.Animation(this.animationDuration);
                this.slideRight.onStep(function (progress) {
                    _this.slider.getEl().setLeft(_this.sliderOffset * progress + 'px');
                });
            };
            return ToggleSlide;
        })(api.dom.DivEl);
        ui.ToggleSlide = ToggleSlide;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var NavigatorEvent = (function () {
            function NavigatorEvent(tab) {
                this.tab = tab;
            }
            NavigatorEvent.prototype.getItem = function () {
                return this.tab;
            };
            return NavigatorEvent;
        })();
        ui.NavigatorEvent = NavigatorEvent;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var Dropdown = (function (_super) {
            __extends(Dropdown, _super);
            function Dropdown(name) {
                var _this = this;
                _super.call(this);
                this.oldValue = "";
                this.valueChangedListeners = [];
                this.getEl().setAttribute("name", name);
                this.onChange(function (event) {
                    _this.notifyValueChanged(_this.oldValue, _this.getValue());
                    _this.oldValue = _this.getValue();
                });
            }
            Dropdown.prototype.addOption = function (value, displayName) {
                var option = new DropdownOption(value, displayName);
                this.appendChild(option);
            };
            Dropdown.prototype.setValue = function (value) {
                _super.prototype.setValue.call(this, value);
                this.notifyValueChanged(this.oldValue, value);
                this.oldValue = value;
                return this;
            };
            Dropdown.prototype.onValueChanged = function (listener) {
                this.valueChangedListeners.push(listener);
            };
            Dropdown.prototype.unValueChanged = function (listener) {
                this.valueChangedListeners = this.valueChangedListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            Dropdown.prototype.notifyValueChanged = function (oldValue, newValue) {
                var _this = this;
                this.valueChangedListeners.forEach(function (listener) {
                    listener.call(_this, new ui.ValueChangedEvent(oldValue, newValue));
                });
            };
            return Dropdown;
        })(api.dom.SelectEl);
        ui.Dropdown = Dropdown;
        var DropdownOption = (function (_super) {
            __extends(DropdownOption, _super);
            function DropdownOption(value, displayName) {
                _super.call(this, value, displayName);
            }
            return DropdownOption;
        })(api.dom.OptionEl);
        ui.DropdownOption = DropdownOption;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var RadioGroup = (function (_super) {
            __extends(RadioGroup, _super);
            function RadioGroup(name, orientation) {
                _super.call(this, "div", "radio-group");
                this.options = [];
                this.oldValue = "";
                this.valueChangedListeners = [];
                this.name = name;
                if (RadioGroup.ORIENTATION_VERTICAL == orientation) {
                    this.addClass("vertical");
                }
            }
            RadioGroup.prototype.addOption = function (value, label, checked) {
                var _this = this;
                var radio = new RadioButton(label, value, this.name, checked);
                this.options.push(radio);
                this.appendChild(radio);
                radio.onClicked(function (event) {
                    _this.notifyValueChanged(_this.oldValue, _this.getValue());
                    _this.oldValue = _this.getValue();
                });
            };
            RadioGroup.prototype.setValue = function (value) {
                var option;
                for (var i = 0; i < this.options.length; i++) {
                    option = this.options[i];
                    option.setChecked(option.getValue() == value);
                }
                return this;
            };
            RadioGroup.prototype.getValue = function () {
                var option;
                for (var i = 0; i < this.options.length; i++) {
                    option = this.options[i];
                    if (option.isChecked()) {
                        return option.getValue();
                    }
                }
                return undefined;
            };
            RadioGroup.prototype.getName = function () {
                return this.name;
            };
            RadioGroup.prototype.onValueChanged = function (listener) {
                this.valueChangedListeners.push(listener);
            };
            RadioGroup.prototype.unValueChanged = function (listener) {
                this.valueChangedListeners = this.valueChangedListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            RadioGroup.prototype.notifyValueChanged = function (oldValue, newValue) {
                var _this = this;
                this.valueChangedListeners.forEach(function (listener) {
                    listener.call(_this, new ui.ValueChangedEvent(oldValue, newValue));
                });
            };
            RadioGroup.ORIENTATION_VERTICAL = "vertical";
            RadioGroup.ORIENTATION_HORIZONTAL = "horizontal";
            return RadioGroup;
        })(api.dom.FormInputEl);
        ui.RadioGroup = RadioGroup;
        var RadioButton = (function (_super) {
            __extends(RadioButton, _super);
            function RadioButton(label, value, name, checked) {
                _super.call(this, "span", "radio-button");
                this.radio = new api.dom.InputEl();
                this.radio.getEl().setAttribute('type', 'radio');
                this.radio.setName(name).setValue(value);
                this.appendChild(this.radio);
                this.label = new api.dom.LabelEl(label, this.radio);
                this.appendChild(this.label);
                this.setChecked(checked);
            }
            RadioButton.prototype.setValue = function (value) {
                this.radio.setValue(value);
                return this;
            };
            RadioButton.prototype.getValue = function () {
                return this.radio.getValue();
            };
            RadioButton.prototype.getName = function () {
                return this.radio.getName();
            };
            RadioButton.prototype.isChecked = function () {
                return this.radio.getHTMLElement()['checked'];
            };
            RadioButton.prototype.setChecked = function (checked) {
                if (checked === void 0) { checked = false; }
                this.radio.getHTMLElement()['checked'] = checked;
                return this;
            };
            return RadioButton;
        })(api.dom.FormInputEl);
        ui.RadioButton = RadioButton;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var Checkbox = (function (_super) {
            __extends(Checkbox, _super);
            function Checkbox(text) {
                var _this = this;
                _super.call(this, "div", "checkbox");
                this.oldValue = false;
                this.valueChangedListeners = [];
                this.checkbox = new api.dom.Element(new api.dom.NewElementBuilder().setTagName('input').setGenerateId(true));
                this.checkbox.getEl().setAttribute('type', 'checkbox');
                this.appendChild(this.checkbox);
                this.label = new api.dom.LabelEl(text, this.checkbox);
                this.appendChild(this.label);
                wemjq(this.checkbox.getHTMLElement()).change(function () {
                    var newValue = _this.isChecked();
                    _this.notifyValueChanged(_this.oldValue, newValue);
                    _this.oldValue = newValue;
                });
            }
            Checkbox.prototype.setChecked = function (newValue, suppressEvent) {
                this.checkbox.getHTMLElement()["checked"] = newValue;
                if (!suppressEvent) {
                    this.notifyValueChanged(this.oldValue, newValue);
                }
                this.oldValue = newValue;
                return this;
            };
            Checkbox.prototype.setDisabled = function (value) {
                this.checkbox.getEl().setDisabled(value);
                return this;
            };
            Checkbox.prototype.toggleChecked = function () {
                this.setChecked(!this.isChecked());
            };
            Checkbox.prototype.isChecked = function () {
                return this.checkbox.getHTMLElement()["checked"];
            };
            Checkbox.prototype.setValue = function (value) {
                throw new Error("CheckboxInput does not support method setValue, use setChecked instead");
            };
            Checkbox.prototype.getValue = function () {
                throw new Error("CheckboxInput does not support method setValue, use isChecked instead");
            };
            Checkbox.prototype.giveFocus = function () {
                return this.checkbox.giveFocus();
            };
            Checkbox.prototype.giveBlur = function () {
                return this.checkbox.giveBlur();
            };
            Checkbox.prototype.setName = function (value) {
                this.checkbox.setName(value);
                return this;
            };
            Checkbox.prototype.setLabel = function (text) {
                this.label.setValue(text);
                return this;
            };
            Checkbox.prototype.getLabel = function () {
                return this.label.getValue();
            };
            Checkbox.prototype.setPlaceholder = function (value) {
                this.checkbox.getEl().setAttribute('placeholder', value);
                return this;
            };
            Checkbox.prototype.getPlaceholder = function () {
                return this.checkbox.getEl().getAttribute('placeholder');
            };
            Checkbox.prototype.onValueChanged = function (listener) {
                this.valueChangedListeners.push(listener);
            };
            Checkbox.prototype.unValueChanged = function (listener) {
                this.valueChangedListeners = this.valueChangedListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            Checkbox.prototype.notifyValueChanged = function (oldValue, newValue) {
                var _this = this;
                this.valueChangedListeners.forEach(function (listener) {
                    listener.call(_this, new ui.ValueChangedEvent(oldValue.toString(), newValue.toString()));
                });
            };
            Checkbox.prototype.onFocus = function (listener) {
                this.checkbox.onFocus(listener);
            };
            Checkbox.prototype.unFocus = function (listener) {
                this.checkbox.unFocus(listener);
            };
            Checkbox.prototype.onBlur = function (listener) {
                this.checkbox.onBlur(listener);
            };
            Checkbox.prototype.unBlur = function (listener) {
                this.checkbox.unBlur(listener);
            };
            return Checkbox;
        })(api.dom.FormInputEl);
        ui.Checkbox = Checkbox;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var DragHelper = (function (_super) {
            __extends(DragHelper, _super);
            function DragHelper() {
                _super.call(this, 'drag-helper');
                this.setId('drag-helper');
            }
            DragHelper.get = function () {
                if (!DragHelper.instance) {
                    DragHelper.instance = new DragHelper();
                }
                return DragHelper.instance;
            };
            DragHelper.prototype.setDropAllowed = function (allowed) {
                if (DragHelper.debug) {
                    console.log('DragHelper.setDropAllowed: ' + allowed.toString());
                }
                this.toggleClass('drop-allowed', allowed);
                return this;
            };
            DragHelper.prototype.isDropAllowed = function () {
                return this.hasClass('drop-allowed');
            };
            DragHelper.prototype.reset = function () {
                this.setDropAllowed(false);
                return this;
            };
            DragHelper.CURSOR_AT = { left: -10, top: -15 };
            DragHelper.debug = false;
            return DragHelper;
        })(api.dom.DivEl);
        ui.DragHelper = DragHelper;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var NamesAndIconViewer = (function (_super) {
            __extends(NamesAndIconViewer, _super);
            function NamesAndIconViewer(className, size) {
                if (size === void 0) { size = 2 /* small */; }
                _super.call(this, className);
                this.namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(size).build();
            }
            NamesAndIconViewer.prototype.setObject = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                _super.prototype.setObject.call(this, object);
                var displayName = this.resolveDisplayName(object) || this.normalizeDisplayName(this.resolveUnnamedDisplayName(object)), subName = this.resolveSubName(object, relativePath) || NamesAndIconViewer.EMPTY_SUB_NAME, subTitle = this.resolveSubTitle(object), iconClass = this.resolveIconClass(object), iconUrl = this.resolveIconUrl(object);
                this.namesAndIconView.setMainName(displayName).setSubName(subName, subTitle).setIconClass(iconClass);
                if (!!iconUrl) {
                    this.namesAndIconView.setIconUrl(iconUrl);
                }
                this.render();
            };
            NamesAndIconViewer.prototype.normalizeDisplayName = function (displayName) {
                if (api.util.StringHelper.isEmpty(displayName)) {
                    return NamesAndIconViewer.EMPTY_DISPLAY_NAME;
                }
                else {
                    displayName = api.util.StringHelper.capitalizeAll(displayName.replace(/-/g, " ").trim());
                    return "<Unnamed " + displayName + ">";
                }
            };
            NamesAndIconViewer.prototype.resolveDisplayName = function (object) {
                return "";
            };
            NamesAndIconViewer.prototype.resolveUnnamedDisplayName = function (object) {
                return "";
            };
            NamesAndIconViewer.prototype.resolveSubName = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                return "";
            };
            NamesAndIconViewer.prototype.resolveSubTitle = function (object) {
                return "";
            };
            NamesAndIconViewer.prototype.resolveIconClass = function (object) {
                return "";
            };
            NamesAndIconViewer.prototype.resolveIconUrl = function (object) {
                return "";
            };
            NamesAndIconViewer.prototype.getPreferredHeight = function () {
                return 50;
            };
            NamesAndIconViewer.prototype.doRender = function () {
                this.removeChildren();
                this.appendChild(this.namesAndIconView);
                return true;
            };
            NamesAndIconViewer.EMPTY_DISPLAY_NAME = "<Display Name>";
            NamesAndIconViewer.EMPTY_SUB_NAME = "<unnamed>";
            return NamesAndIconViewer;
        })(api.ui.Viewer);
        ui.NamesAndIconViewer = NamesAndIconViewer;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var ActivatedEvent = (function () {
            function ActivatedEvent(index) {
                this.index = index;
            }
            ActivatedEvent.prototype.getIndex = function () {
                return this.index;
            };
            return ActivatedEvent;
        })();
        ui.ActivatedEvent = ActivatedEvent;
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var button;
        (function (button) {
            var Button = (function (_super) {
                __extends(Button, _super);
                function Button(label) {
                    _super.call(this, "button");
                    this.labelEl = new api.dom.SpanEl();
                    if (label) {
                        this.labelEl.getEl().setInnerHtml(label);
                    }
                    this.appendChild(this.labelEl);
                }
                Button.prototype.setEnabled = function (value) {
                    this.getEl().setDisabled(!value);
                    return this;
                };
                Button.prototype.isEnabled = function () {
                    return !this.getEl().isDisabled();
                };
                Button.prototype.setLabel = function (label) {
                    this.labelEl.getEl().setInnerHtml(label);
                    return this;
                };
                Button.prototype.getLabel = function () {
                    return this.labelEl.getEl().getInnerHtml();
                };
                Button.prototype.setTitle = function (title, forceAction) {
                    if (forceAction === void 0) { forceAction = true; }
                    if (title) {
                        this.getEl().setAttribute('title', title);
                        if (forceAction) {
                            wemjq(this.getEl().getHTMLElement()).trigger("mouseenter");
                        }
                    }
                    else {
                        if (forceAction) {
                            wemjq(this.getEl().getHTMLElement()).trigger("mouseleave");
                        }
                        this.getEl().removeAttribute('title');
                    }
                };
                return Button;
            })(api.dom.ButtonEl);
            button.Button = Button;
        })(button = ui.button || (ui.button = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var button;
        (function (button) {
            var ActionButton = (function (_super) {
                __extends(ActionButton, _super);
                function ActionButton(action, showTooltip) {
                    var _this = this;
                    if (showTooltip === void 0) { showTooltip = true; }
                    this.action = action;
                    var label;
                    if (this.action.hasMnemonic()) {
                        label = this.action.getMnemonic().underlineMnemonic(this.action.getLabel());
                    }
                    else {
                        label = this.action.getLabel();
                    }
                    _super.call(this, label);
                    this.addClass("action-button");
                    this.setEnabled(this.action.isEnabled());
                    if (this.action.getIconClass()) {
                        this.addClass(action.getIconClass());
                    }
                    if (this.action.hasShortcut() && showTooltip) {
                        this.tooltip = new ui.Tooltip(this, this.action.getShortcut().getCombination(), 1000);
                        api.ui.KeyBindings.get().onHelpKeyPressed(function (e) {
                            if (_this.action.isEnabled() && api.ui.KeyBindings.get().isActive(_this.action.getShortcut())) {
                                if (ui.KeyBindingAction[0 /* KEYDOWN */].toLowerCase() == e.type) {
                                    _this.tooltip.show();
                                    return;
                                }
                            }
                            _this.tooltip.hide();
                        });
                    }
                    this.onClicked(function (event) {
                        _this.action.execute();
                    });
                    this.action.onPropertyChanged(function (action) {
                        _this.setEnabled(action.isEnabled());
                        _this.setVisible(action.isVisible());
                    });
                }
                ActionButton.prototype.getTooltip = function () {
                    return this.tooltip;
                };
                return ActionButton;
            })(api.ui.button.Button);
            button.ActionButton = ActionButton;
        })(button = ui.button || (ui.button = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var button;
        (function (button) {
            var CycleButton = (function (_super) {
                __extends(CycleButton, _super);
                function CycleButton(actions) {
                    var _this = this;
                    _super.call(this);
                    this.addClass("cycle-button icon-screen icon-medium");
                    this.actionList = actions;
                    if (this.actionList.length > 0) {
                        this.active = -1;
                        this.updateActive();
                        this.setTitle(this.actionList[this.active].getTitle(), false);
                        this.onClicked(function () {
                            _this.removeAndHdeTitle();
                            _this.doAction();
                            _this.setAndShowTitle();
                        });
                    }
                }
                CycleButton.prototype.doAction = function () {
                    this.actionList[this.active].execute();
                    this.updateActive();
                };
                CycleButton.prototype.removeAndHdeTitle = function () {
                    if (this.actionList[this.active].getTitle()) {
                        this.setTitle("");
                    }
                };
                CycleButton.prototype.setAndShowTitle = function () {
                    var title = this.actionList[this.active].getTitle();
                    if (title) {
                        this.setTitle(title);
                    }
                };
                CycleButton.prototype.updateActive = function () {
                    var name, prevName;
                    prevName = this.actionList[this.active] ? this.actionList[this.active].getLabel().toLowerCase() : "";
                    this.active++;
                    if (this.active >= this.actionList.length) {
                        this.active = 0;
                    }
                    name = this.actionList[this.active] ? this.actionList[this.active].getLabel().toLowerCase() : "";
                    if (prevName) {
                        this.removeClass(prevName);
                    }
                    if (name) {
                        this.addClass(name);
                    }
                };
                CycleButton.prototype.executePrevAction = function () {
                    var prev = this.active - 1;
                    prev = prev < 0 ? this.actionList.length - 1 : prev;
                    if (this.actionList.length > 0) {
                        this.actionList[prev].execute();
                    }
                };
                CycleButton.prototype.selectActiveAction = function (action) {
                    var i, l = this.actionList.length;
                    for (i = 0; i < l; i++) {
                        if (this.actionList[i] === action) {
                            this.active = i;
                            this.updateActive();
                            this.setTitle(this.actionList[this.active].getTitle(), false);
                            return;
                        }
                    }
                    console.warn('Action not found in CycleButton', action);
                };
                return CycleButton;
            })(api.ui.button.Button);
            button.CycleButton = CycleButton;
        })(button = ui.button || (ui.button = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var button;
        (function (button) {
            var CloseButton = (function (_super) {
                __extends(CloseButton, _super);
                function CloseButton(className) {
                    _super.call(this);
                    this.addClass('close-button icon-medium icon-close2');
                    if (className) {
                        this.addClass(className);
                    }
                }
                return CloseButton;
            })(api.ui.button.Button);
            button.CloseButton = CloseButton;
        })(button = ui.button || (ui.button = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var button;
        (function (button) {
            var TogglerButton = (function (_super) {
                __extends(TogglerButton, _super);
                function TogglerButton(className, title) {
                    var _this = this;
                    _super.call(this);
                    this.activeListeners = [];
                    this.addClass("toggle-button icon-medium " + className);
                    this.setActive(false);
                    this.setEnabled(false);
                    if (title) {
                        this.setTitle(title);
                        this.onActiveChanged(function (isActive) {
                            _this.setTitle(isActive ? "" : title, true);
                        });
                    }
                    this.onClicked(function (event) {
                        if (_this.isEnabled()) {
                            _this.setActive(!_this.isActive());
                        }
                    });
                }
                TogglerButton.prototype.setActive = function (value, silent) {
                    if (silent === void 0) { silent = false; }
                    this.toggleClass('active', value);
                    if (!silent) {
                        this.notifyActiveChanged(value);
                    }
                };
                TogglerButton.prototype.isActive = function () {
                    return this.hasClass("active");
                };
                TogglerButton.prototype.onActiveChanged = function (listener) {
                    this.activeListeners.push(listener);
                };
                TogglerButton.prototype.unActiveChanged = function (listener) {
                    this.activeListeners = this.activeListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                TogglerButton.prototype.notifyActiveChanged = function (isActive) {
                    this.activeListeners.forEach(function (listener) {
                        listener(isActive);
                    });
                };
                return TogglerButton;
            })(api.ui.button.Button);
            button.TogglerButton = TogglerButton;
        })(button = ui.button || (ui.button = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (text) {
            var TextInput = (function (_super) {
                __extends(TextInput, _super);
                function TextInput(className, size) {
                    var _this = this;
                    _super.call(this, className);
                    this.allowedKeyCodes = [8, 9, 46, 39, 37];
                    this.oldValue = "";
                    this.valueChangedListeners = [];
                    this.getEl().setAttribute('type', 'text');
                    this.addClass('text-input');
                    if (size) {
                        this.addClass(size);
                    }
                    this.onKeyPressed(function (event) {
                        if (!_this.stripCharsRe) {
                            return;
                        }
                        var symbol = String.fromCharCode(event.charCode);
                        if (_this.containsForbiddenChars(symbol)) {
                            if (!_this.keyCodeAllowed(event.keyCode)) {
                                event.preventDefault();
                                return false;
                            }
                        }
                    });
                    this.onInput(function (event) {
                        _this.notifyValueChanged(_this.oldValue, _this.getValue());
                        _this.oldValue = _this.getValue();
                    });
                }
                TextInput.large = function (className) {
                    return new TextInput(className, TextInput.LARGE);
                };
                TextInput.middle = function (className) {
                    return new TextInput(className, TextInput.MIDDLE);
                };
                TextInput.prototype.getOldValue = function () {
                    return this.oldValue;
                };
                TextInput.prototype.setValue = function (value) {
                    var oldValue = this.getValue();
                    var newValue = this.removeForbiddenChars(value);
                    _super.prototype.setValue.call(this, value);
                    if (oldValue != newValue) {
                        _super.prototype.setValue.call(this, newValue);
                        this.notifyValueChanged(oldValue, newValue);
                        this.oldValue = newValue;
                    }
                    return this;
                };
                TextInput.prototype.setName = function (value) {
                    _super.prototype.setName.call(this, value);
                    return this;
                };
                TextInput.prototype.setPlaceholder = function (value) {
                    _super.prototype.setPlaceholder.call(this, value);
                    return this;
                };
                TextInput.prototype.setForbiddenCharsRe = function (re) {
                    this.stripCharsRe = re;
                    return this;
                };
                TextInput.prototype.selectText = function (from, to) {
                    var htmlEl = this.getHTMLElement();
                    if (!from) {
                        htmlEl.select();
                    }
                    else if (!to) {
                        to = this.getValue().length;
                    }
                    if (htmlEl.createTextRange) {
                        var selRange = htmlEl.createTextRange();
                        selRange.collapse(true);
                        selRange.moveStart('character', from);
                        selRange.moveEnd('character', to);
                        selRange.select();
                    }
                    else if (htmlEl.setSelectionRange) {
                        htmlEl.setSelectionRange(from, to);
                    }
                    else if (htmlEl.selectionStart) {
                        htmlEl.selectionStart = from;
                        htmlEl.selectionEnd = to;
                    }
                    htmlEl.focus();
                };
                TextInput.prototype.disableAutocomplete = function () {
                    this.getEl().setAttribute('autocomplete', 'off');
                    return this;
                };
                TextInput.prototype.moveCaretTo = function (pos) {
                    this.selectText(pos, pos);
                };
                TextInput.prototype.onValueChanged = function (listener) {
                    this.valueChangedListeners.push(listener);
                };
                TextInput.prototype.unValueChanged = function (listener) {
                    this.valueChangedListeners = this.valueChangedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TextInput.prototype.updateValidationStatusOnUserInput = function (isValid) {
                    if (isValid) {
                        this.removeClass("invalid");
                        if (!api.util.StringHelper.isEmpty(this.getValue())) {
                            this.addClass("valid");
                        }
                        else {
                            this.removeClass("valid");
                        }
                    }
                    else {
                        this.removeClass("valid");
                        this.addClass("invalid");
                    }
                };
                TextInput.prototype.notifyValueChanged = function (oldValue, newValue) {
                    var _this = this;
                    this.valueChangedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.ValueChangedEvent(oldValue, newValue));
                    });
                };
                TextInput.prototype.removeForbiddenChars = function (rawValue) {
                    return this.stripCharsRe ? (rawValue || '').replace(this.stripCharsRe, '') : rawValue;
                };
                TextInput.prototype.containsForbiddenChars = function (value) {
                    var forbidden = new RegExp(this.stripCharsRe);
                    return forbidden.test(value);
                };
                TextInput.prototype.keyCodeAllowed = function (keyCode) {
                    for (var i = 0; i < this.allowedKeyCodes.length; i++) {
                        if (keyCode == this.allowedKeyCodes[i]) {
                            return true;
                        }
                    }
                    return false;
                };
                TextInput.MIDDLE = 'middle';
                TextInput.LARGE = 'large';
                return TextInput;
            })(api.dom.InputEl);
            text.TextInput = TextInput;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (text) {
            var AutosizeTextInput = (function (_super) {
                __extends(AutosizeTextInput, _super);
                function AutosizeTextInput(className, size) {
                    var _this = this;
                    if (size === void 0) { size = text.TextInput.MIDDLE; }
                    _super.call(this, className, size);
                    this.addClass('autosize');
                    this.clone = new api.dom.DivEl().addClass('autosize-clone').addClass(this.getEl().getAttribute('class'));
                    this.attendant = new api.dom.DivEl().addClass('autosize-attendant');
                    this.attendant.appendChild(this.clone);
                    this.onShown(function (event) { return _this.updateSize(); });
                    this.onValueChanged(function (event) { return _this.updateSize(); });
                    api.dom.WindowDOM.get().onResized(function (event) { return _this.updateSize(); }, this);
                    api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function (item) { return _this.updateSize(); });
                }
                AutosizeTextInput.large = function (className) {
                    return new AutosizeTextInput(className, text.TextInput.LARGE);
                };
                AutosizeTextInput.middle = function (className) {
                    return new AutosizeTextInput(className, text.TextInput.MIDDLE);
                };
                AutosizeTextInput.prototype.updateSize = function () {
                    var inputEl = this.getEl(), cloneEl = this.clone.getEl();
                    cloneEl.setFontSize(inputEl.getFontSize()).setPaddingLeft(inputEl.getPaddingLeft() + 'px').setPaddingRight(inputEl.getPaddingRight() + 'px');
                    this.attendant.insertAfterEl(this);
                    cloneEl.setInnerHtml(this.getValue(), true);
                    if (cloneEl.getWidthWithBorder() > this.attendant.getEl().getWidth()) {
                        inputEl.setWidth("100%");
                    }
                    else {
                        inputEl.setWidthPx(cloneEl.getWidthWithBorder());
                    }
                    this.attendant.remove();
                };
                return AutosizeTextInput;
            })(text.TextInput);
            text.AutosizeTextInput = AutosizeTextInput;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (text) {
            var PasswordInput = (function (_super) {
                __extends(PasswordInput, _super);
                function PasswordInput(className) {
                    _super.call(this, className, "password");
                    this.addClass('password-input');
                }
                return PasswordInput;
            })(api.dom.InputEl);
            text.PasswordInput = PasswordInput;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (_text) {
            var TextArea = (function (_super) {
                __extends(TextArea, _super);
                function TextArea(name) {
                    var _this = this;
                    _super.call(this, "textarea", "text-area");
                    this.valueChangedListeners = [];
                    this.oldValue = "";
                    this.getEl().setAttribute("name", name);
                    this.onInput(function (event) {
                        _this.notifyValueChanged(_this.oldValue, _this.getValue());
                        _this.oldValue = _this.getValue();
                    });
                    this.clone = new api.dom.DivEl('autosize-clone').addClass(this.getEl().getAttribute('class'));
                    this.attendant = new api.dom.DivEl('autosize-attendant');
                    this.attendant.appendChild(this.clone);
                    this.onAdded(function (event) {
                        _this.attendant.insertAfterEl(_this);
                    });
                    this.onShown(function (event) { return _this.updateSize(); });
                    this.onValueChanged(function (event) { return _this.updateSize(); });
                    api.dom.WindowDOM.get().onResized(function (event) { return _this.updateSize(); }, this);
                }
                TextArea.prototype.setValue = function (text) {
                    if (this.oldValue != text) {
                        _super.prototype.setValue.call(this, text);
                        this.notifyValueChanged(this.oldValue, text);
                        this.oldValue = text;
                    }
                    return this;
                };
                TextArea.prototype.setRows = function (rows) {
                    this.getEl().setAttribute("rows", rows.toString());
                };
                TextArea.prototype.setColumns = function (columns) {
                    this.getEl().setAttribute("cols", columns.toString());
                };
                TextArea.prototype.updateSize = function () {
                    if (this.isRendered()) {
                        this.clone.getEl().setInnerHtml(this.getValue() + " ", true);
                        this.getEl().setHeightPx(this.clone.getEl().getHeightWithBorder());
                    }
                };
                TextArea.prototype.onValueChanged = function (listener) {
                    this.valueChangedListeners.push(listener);
                };
                TextArea.prototype.unValueChanged = function (listener) {
                    this.valueChangedListeners = this.valueChangedListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                TextArea.prototype.notifyValueChanged = function (oldValue, newValue) {
                    var _this = this;
                    this.valueChangedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.ValueChangedEvent(oldValue, newValue));
                    });
                };
                return TextArea;
            })(api.dom.FormInputEl);
            _text.TextArea = TextArea;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (text) {
            var CodeAreaBuilder = (function () {
                function CodeAreaBuilder() {
                }
                CodeAreaBuilder.prototype.setName = function (value) {
                    this.name = value;
                    return this;
                };
                CodeAreaBuilder.prototype.setMode = function (value) {
                    this.mode = value;
                    return this;
                };
                CodeAreaBuilder.prototype.setLineNumbers = function (value) {
                    this.lineNumbers = value;
                    return this;
                };
                CodeAreaBuilder.prototype.build = function () {
                    return new CodeArea(this);
                };
                return CodeAreaBuilder;
            })();
            text.CodeAreaBuilder = CodeAreaBuilder;
            var CodeArea = (function (_super) {
                __extends(CodeArea, _super);
                function CodeArea(builder) {
                    var _this = this;
                    this.textArea = new text.TextArea(builder.name);
                    _super.call(this, this.textArea);
                    this.mode = builder.mode;
                    this.options = {
                        lineNumbers: builder.lineNumbers
                    };
                    CodeMirror.modeURL = api.util.UriHelper.getUri('admin/common/lib/codemirror/mode/%N.js');
                    this.onAdded(function (event) {
                        _this.codeMirror = CodeMirror.fromTextArea(_this.textArea.getHTMLElement(), _this.options);
                        _this.codeMirror.setSize("540px", "350px");
                        _this.codeMirror.setOption("mode", _this.mode);
                        CodeMirror.autoLoadMode(_this.codeMirror, _this.mode);
                        _this.codeMirror.refresh();
                    });
                    this.onShown(function (event) {
                        _this.codeMirror.refresh();
                    });
                }
                CodeArea.prototype.setValue = function (value) {
                    this.codeMirror.setValue(value);
                    return this;
                };
                CodeArea.prototype.getValue = function () {
                    return this.codeMirror.getValue();
                };
                return CodeArea;
            })(api.dom.CompositeFormInputEl);
            text.CodeArea = CodeArea;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (text) {
            var StringHelper = api.util.StringHelper;
            var NumberHelper = api.util.NumberHelper;
            var ArrayHelper = api.util.ArrayHelper;
            var CharType;
            (function (CharType) {
                CharType[CharType["SPECIAL"] = 0] = "SPECIAL";
                CharType[CharType["DIGIT"] = 1] = "DIGIT";
                CharType[CharType["UPPERCASE"] = 2] = "UPPERCASE";
                CharType[CharType["LOWERCASE"] = 3] = "LOWERCASE";
            })(CharType || (CharType = {}));
            var PasswordGenerator = (function (_super) {
                __extends(PasswordGenerator, _super);
                function PasswordGenerator() {
                    var _this = this;
                    _super.call(this, "div", "password-generator");
                    this.focusListeners = [];
                    this.blurListeners = [];
                    this.SPECIAL_CHARS = '!@#$%^&*()_+{}:"<>?|[];\',./`~';
                    this.LOWERCASE_CHARS = 'abcdefghijklmnopqrstuvwxyz';
                    this.UPPERCASE_CHARS = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
                    this.DIGIT_CHARS = '0123456789';
                    var inputWrapper = new api.dom.DivEl('input-wrapper');
                    this.appendChild(inputWrapper);
                    var strengthMeter = new api.dom.DivEl('strength-meter');
                    inputWrapper.appendChild(strengthMeter);
                    this.input = new text.PasswordInput();
                    this.initFocusEvents(this.input);
                    this.input.onInput(function (event) {
                        _this.assessComplexity(_this.input.getValue());
                        _this.notifyValidityChanged(_this.input.isValid());
                    });
                    inputWrapper.appendChild(this.input);
                    this.showLink = new api.dom.AEl('show-link');
                    this.initFocusEvents(this.showLink);
                    this.showLink.onClicked(function (event) {
                        var unlocked = _this.hasClass('unlocked');
                        _this.toggleClass('unlocked', !unlocked);
                        _this.input.setType(unlocked ? 'password' : 'text');
                    });
                    this.appendChild(this.showLink);
                    this.generateLink = new api.dom.AEl();
                    this.generateLink.setHtml('Generate');
                    this.initFocusEvents(this.generateLink);
                    this.generateLink.onClicked(function (event) {
                        _this.generatePassword();
                        _this.assessComplexity(_this.input.getValue());
                        _this.notifyValidityChanged(_this.input.isValid());
                    });
                    this.appendChild(this.generateLink);
                }
                PasswordGenerator.prototype.getValue = function () {
                    return this.input.getValue();
                };
                PasswordGenerator.prototype.setValue = function (value) {
                    this.input.setValue(value);
                    this.assessComplexity(value);
                    return this;
                };
                PasswordGenerator.prototype.getName = function () {
                    return this.input.getName();
                };
                PasswordGenerator.prototype.setName = function (value) {
                    this.input.setName(value);
                    return this;
                };
                PasswordGenerator.prototype.setPlaceholder = function (value) {
                    this.input.setPlaceholder(value);
                    return this;
                };
                PasswordGenerator.prototype.getPlaceholder = function () {
                    return this.input.getPlaceholder();
                };
                PasswordGenerator.prototype.assessComplexity = function (value) {
                    if (this.complexity) {
                        this.removeClass(this.complexity);
                        this.complexity = undefined;
                    }
                    if (this.isExtreme(value)) {
                        this.complexity = 'extreme';
                    }
                    else if (this.isStrong(value)) {
                        this.complexity = 'strong';
                    }
                    else if (this.isGood(value)) {
                        this.complexity = 'good';
                    }
                    else if (this.isWeak(value)) {
                        this.complexity = 'weak';
                    }
                    if (this.complexity) {
                        this.addClass(this.complexity);
                    }
                };
                PasswordGenerator.prototype.generatePassword = function () {
                    var length = NumberHelper.randomBetween(14, 16), maxSpecials = NumberHelper.randomBetween(1, 3), specials = 0, maxDigits = NumberHelper.randomBetween(2, 4), digits = 0, maxUppercase = NumberHelper.randomBetween(2, 4), uppercase = 0, maxLowercase = length - maxSpecials - maxDigits - maxUppercase, lowercase = 0;
                    var result = "";
                    var types = [0 /* SPECIAL */, 1 /* DIGIT */, 2 /* UPPERCASE */, 3 /* LOWERCASE */];
                    for (var i = 0; i < length; i++) {
                        var type = types[NumberHelper.randomBetween(0, types.length - 1)];
                        switch (type) {
                            case 0 /* SPECIAL */:
                                if (specials < maxSpecials) {
                                    result += this.SPECIAL_CHARS.charAt(NumberHelper.randomBetween(0, this.SPECIAL_CHARS.length - 1));
                                    specials++;
                                }
                                else {
                                    i--;
                                    ArrayHelper.removeValue(0 /* SPECIAL */, types);
                                }
                                break;
                            case 1 /* DIGIT */:
                                if (digits < maxDigits) {
                                    result += this.DIGIT_CHARS.charAt(NumberHelper.randomBetween(0, this.DIGIT_CHARS.length - 1));
                                    digits++;
                                }
                                else {
                                    i--;
                                    ArrayHelper.removeValue(1 /* DIGIT */, types);
                                }
                                break;
                            case 2 /* UPPERCASE */:
                                if (uppercase < maxUppercase) {
                                    result += this.UPPERCASE_CHARS.charAt(NumberHelper.randomBetween(0, this.UPPERCASE_CHARS.length - 1));
                                    uppercase++;
                                }
                                else {
                                    i--;
                                    ArrayHelper.removeValue(2 /* UPPERCASE */, types);
                                }
                                break;
                            case 3 /* LOWERCASE */:
                                if (lowercase < maxLowercase) {
                                    result += this.LOWERCASE_CHARS.charAt(NumberHelper.randomBetween(0, this.LOWERCASE_CHARS.length - 1));
                                    lowercase++;
                                }
                                else {
                                    i--;
                                    ArrayHelper.removeValue(3 /* LOWERCASE */, types);
                                }
                                break;
                        }
                    }
                    console.log('Generated ' + length + ' chars password (SP:' + maxSpecials + ', DG:' + maxDigits + ',UC:' + maxUppercase + ', LC:' + maxLowercase + ') = ' + result);
                    this.input.setValue(result);
                };
                PasswordGenerator.prototype.isWeak = function (value) {
                    return !StringHelper.isBlank(value) && (value.length < 8 || StringHelper.isLowerCase(value) || StringHelper.isUpperCase(value));
                };
                PasswordGenerator.prototype.isGood = function (value) {
                    return !StringHelper.isBlank(value) && (value.length >= 8 || (value.length >= 6 && StringHelper.isMixedCase(value) && this.containsNonAlphabetChars(value)));
                };
                PasswordGenerator.prototype.isStrong = function (value) {
                    return !StringHelper.isBlank(value) && value.length >= 10 && StringHelper.isMixedCase(value) && this.containsNonAlphabetChars(value);
                };
                PasswordGenerator.prototype.isExtreme = function (value) {
                    return !StringHelper.isBlank(value) && value.length >= 14 && StringHelper.isMixedCase(value) && this.containsDigits(value) && this.containsSpecialChars(value);
                };
                PasswordGenerator.prototype.containsDigits = function (value) {
                    return /\d/.test(value);
                };
                PasswordGenerator.prototype.containsSpecialChars = function (value) {
                    return /[^a-z0-9\s]/i.test(value);
                };
                PasswordGenerator.prototype.containsNonAlphabetChars = function (value) {
                    return /[^a-z\s]/i.test(value);
                };
                PasswordGenerator.prototype.initFocusEvents = function (el) {
                    var _this = this;
                    el.onFocus(function (event) {
                        _this.notifyFocused(event);
                    });
                    el.onBlur(function (event) {
                        _this.notifyBlurred(event);
                    });
                };
                PasswordGenerator.prototype.onInput = function (listener) {
                    this.input.onInput(listener);
                };
                PasswordGenerator.prototype.unInput = function (listener) {
                    this.input.unInput(listener);
                };
                PasswordGenerator.prototype.onFocus = function (listener) {
                    this.focusListeners.push(listener);
                };
                PasswordGenerator.prototype.unFocus = function (listener) {
                    this.focusListeners = this.focusListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                PasswordGenerator.prototype.onBlur = function (listener) {
                    this.blurListeners.push(listener);
                };
                PasswordGenerator.prototype.unBlur = function (listener) {
                    this.blurListeners = this.blurListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                PasswordGenerator.prototype.notifyFocused = function (event) {
                    this.focusListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                PasswordGenerator.prototype.notifyBlurred = function (event) {
                    this.blurListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return PasswordGenerator;
            })(api.dom.FormInputEl);
            text.PasswordGenerator = PasswordGenerator;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (text) {
            var InputEl = api.dom.InputEl;
            var StringHelper = api.util.StringHelper;
            var CheckEmailAvailabilityRequest = api.security.CheckEmailAvailabilityRequest;
            var EmailInput = (function (_super) {
                __extends(EmailInput, _super);
                function EmailInput() {
                    var _this = this;
                    _super.call(this, "div", "email-input");
                    this.focusListeners = [];
                    this.blurListeners = [];
                    this.input = new InputEl(undefined, 'email');
                    this.input.setPattern("^[a-zA-Z0-9\_\-\.]+@[a-zA-Z0-9\-]+(?:\.[a-zA-Z0-9\-]{2,})*(?:\.[a-zA-Z]{2,4})$");
                    this.input.onFocus(function (event) {
                        _this.notifyFocused(event);
                    });
                    this.input.onBlur(function (event) {
                        _this.notifyBlurred(event);
                    });
                    this.input.onInput(function (event) {
                        if (_this.checkTimeout) {
                            clearTimeout(_this.checkTimeout);
                        }
                        _this.checkTimeout = setTimeout(function (email) { return _this.checkAvailability(email); }, 500, _this.input.getValue());
                    });
                    this.appendChild(this.input);
                }
                EmailInput.prototype.getInput = function () {
                    return this.input;
                };
                EmailInput.prototype.getValue = function () {
                    return this.input.getValue();
                };
                EmailInput.prototype.setValue = function (value) {
                    this.input.setValue(value);
                    this.checkAvailability(value);
                    return this;
                };
                EmailInput.prototype.getOriginEmail = function () {
                    return this.originEmail;
                };
                EmailInput.prototype.setOriginEmail = function (value) {
                    this.originEmail = value;
                    return this;
                };
                EmailInput.prototype.setUserStoreKey = function (userStoreKey) {
                    this.userStoreKey = userStoreKey;
                    return this;
                };
                EmailInput.prototype.getName = function () {
                    return this.input.getName();
                };
                EmailInput.prototype.setName = function (value) {
                    this.input.setName(value);
                    return this;
                };
                EmailInput.prototype.isAvailable = function () {
                    return this.hasClass('available');
                };
                EmailInput.prototype.checkAvailability = function (email) {
                    var _this = this;
                    var status;
                    var isValid = this.input.isValid();
                    this.toggleClass('invalid', !isValid);
                    if (!StringHelper.isEmpty(email) && isValid) {
                        status = 'checking';
                        new CheckEmailAvailabilityRequest(email).setUserStoreKey(this.userStoreKey).sendAndParse().then(function (available) {
                            var availability = available || email === _this.originEmail;
                            _this.updateStatus(availability ? 'available' : 'notavailable');
                            _this.notifyValidityChanged(isValid && availability);
                        }).fail(function (reason) {
                            _this.notifyValidityChanged(false);
                            _this.updateStatus('error');
                        }).done();
                    }
                    else {
                        this.notifyValidityChanged(isValid);
                    }
                    this.updateStatus(status);
                };
                EmailInput.prototype.updateStatus = function (status) {
                    if (!!this.status) {
                        this.removeClass(this.status);
                    }
                    if (!StringHelper.isEmpty(status)) {
                        this.status = status;
                        this.addClass(this.status);
                    }
                };
                EmailInput.prototype.isValid = function () {
                    return this.input.isValid();
                };
                EmailInput.prototype.validate = function () {
                    return this.input.validate();
                };
                EmailInput.prototype.onFocus = function (listener) {
                    this.focusListeners.push(listener);
                };
                EmailInput.prototype.unFocus = function (listener) {
                    this.focusListeners = this.focusListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                EmailInput.prototype.onBlur = function (listener) {
                    this.blurListeners.push(listener);
                };
                EmailInput.prototype.unBlur = function (listener) {
                    this.blurListeners = this.blurListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                EmailInput.prototype.notifyFocused = function (event) {
                    this.focusListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                EmailInput.prototype.notifyBlurred = function (event) {
                    this.blurListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return EmailInput;
            })(api.dom.FormInputEl);
            text.EmailInput = EmailInput;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var text;
        (function (text) {
            var InputEl = api.dom.InputEl;
            var FileInput = (function (_super) {
                __extends(FileInput, _super);
                function FileInput(className) {
                    var _this = this;
                    _super.call(this, "div", "file-input");
                    if (className) {
                        this.addClass(className);
                    }
                    this.textInput = new InputEl("text");
                    this.mediaUploader = new api.content.MediaUploader({
                        operation: 0 /* create */,
                        name: 'file-input-uploader',
                        allowDrop: false,
                        showButtons: false,
                        showResult: false,
                        allowMultiSelection: true,
                        deferred: true
                    });
                    this.mediaUploader.onUploadStarted(function (event) {
                        var names = event.getUploadItems().map(function (uploadItem) {
                            return uploadItem.getName();
                        });
                        _this.textInput.setValue(names.join(', '));
                    });
                    this.appendChildren(this.textInput, this.mediaUploader);
                }
                FileInput.prototype.setUploaderParams = function (params) {
                    this.mediaUploader.setParams(params);
                    return this;
                };
                FileInput.prototype.getUploaderParams = function () {
                    return this.mediaUploader.getParams();
                };
                FileInput.prototype.setPlaceholder = function (placeholder) {
                    this.textInput.setPlaceholder(placeholder);
                    return this;
                };
                FileInput.prototype.getPlaceholder = function () {
                    return this.textInput.getPlaceholder();
                };
                FileInput.prototype.getValue = function () {
                    return this.textInput.getValue();
                };
                FileInput.prototype.setValue = function (value) {
                    this.textInput.setValue(value);
                    return this;
                };
                FileInput.prototype.onChange = function (listener) {
                    this.textInput.onChange(listener);
                };
                FileInput.prototype.unChange = function (listener) {
                    this.textInput.unChange(listener);
                };
                FileInput.prototype.onInput = function (listener) {
                    this.textInput.onInput(listener);
                };
                FileInput.prototype.unInput = function (listener) {
                    this.textInput.unInput(listener);
                };
                FileInput.prototype.giveFocus = function () {
                    return this.textInput.giveFocus();
                };
                FileInput.prototype.giveBlur = function () {
                    return this.textInput.giveBlur();
                };
                FileInput.prototype.reset = function () {
                    this.textInput.reset();
                    this.mediaUploader.reset();
                    return this;
                };
                FileInput.prototype.stop = function () {
                    this.mediaUploader.stop();
                    return this;
                };
                FileInput.prototype.getUploader = function () {
                    return this.mediaUploader;
                };
                FileInput.prototype.onUploadStarted = function (listener) {
                    this.mediaUploader.onUploadStarted(listener);
                };
                FileInput.prototype.unUploadStarted = function (listener) {
                    this.mediaUploader.unUploadStarted(listener);
                };
                FileInput.prototype.onUploadProgress = function (listener) {
                    this.mediaUploader.onUploadProgress(listener);
                };
                FileInput.prototype.unUploadProgress = function (listener) {
                    this.mediaUploader.unUploadProgress(listener);
                };
                FileInput.prototype.onFileUploaded = function (listener) {
                    this.mediaUploader.onFileUploaded(listener);
                };
                FileInput.prototype.unFileUploaded = function (listener) {
                    this.mediaUploader.unFileUploaded(listener);
                };
                FileInput.prototype.onUploadCompleted = function (listener) {
                    this.mediaUploader.onUploadCompleted(listener);
                };
                FileInput.prototype.unUploadCompleted = function (listener) {
                    this.mediaUploader.unUploadCompleted(listener);
                };
                FileInput.prototype.onUploadReset = function (listener) {
                    this.mediaUploader.onUploadReset(listener);
                };
                FileInput.prototype.unUploadReset = function (listener) {
                    this.mediaUploader.unUploadReset(listener);
                };
                FileInput.prototype.onUploadFailed = function (listener) {
                    this.mediaUploader.onUploadFailed(listener);
                };
                FileInput.prototype.unUploadFailed = function (listener) {
                    this.mediaUploader.unUploadFailed(listener);
                };
                return FileInput;
            })(api.dom.FormInputEl);
            text.FileInput = FileInput;
        })(text = ui.text || (ui.text = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (tab) {
            var TabItem = (function (_super) {
                __extends(TabItem, _super);
                function TabItem(builder, classes) {
                    var _this = this;
                    _super.call(this, "tab-item" + (!classes ? "" : " " + classes));
                    this.active = false;
                    this.labelChangedListeners = [];
                    this.closedListeners = [];
                    this.selectedListeners = [];
                    this.labelEl = new api.dom.SpanEl('label');
                    this.appendChild(this.labelEl);
                    this.setLabel(builder.label, builder.markUnnamed, builder.addLabelTitleAttribute);
                    this.markInvalid(builder.markInvalid);
                    this.closeAction = builder.closeAction;
                    if (builder.closeButtonEnabled) {
                        this.createRemoveButton();
                    }
                    this.onClicked(function (event) {
                        _this.notifySelectedListeners();
                    });
                }
                TabItem.prototype.createRemoveButton = function () {
                    if (this.closeAction && !this.removeButton) {
                        this.removeButton = new api.ui.button.ActionButton(this.closeAction);
                        this.removeButton.getTooltip().setSide(api.ui.Tooltip.SIDE_LEFT);
                        this.removeButton.onClicked(function (event) {
                            event.stopPropagation();
                            event.preventDefault();
                        });
                        this.prependChild(this.removeButton);
                    }
                };
                TabItem.prototype.setIndex = function (value) {
                    this.index = value;
                };
                TabItem.prototype.getIndex = function () {
                    return this.index;
                };
                TabItem.prototype.setLabel = function (newValue, markUnnamed, addLabelTitleAttribute) {
                    if (markUnnamed === void 0) { markUnnamed = false; }
                    if (addLabelTitleAttribute === void 0) { addLabelTitleAttribute = true; }
                    if (this.label == newValue) {
                        return;
                    }
                    var oldValue = this.label;
                    this.label = newValue;
                    this.labelEl.setHtml(newValue, true);
                    if (addLabelTitleAttribute) {
                        this.labelEl.getEl().setAttribute('title', newValue);
                    }
                    this.labelEl.toggleClass("unnamed", markUnnamed);
                    this.notifyLabelChangedListeners(newValue, oldValue);
                };
                TabItem.prototype.markInvalid = function (markInvalid) {
                    if (markInvalid === void 0) { markInvalid = false; }
                    this.toggleClass("invalid", markInvalid);
                };
                TabItem.prototype.getLabel = function () {
                    return this.label;
                };
                TabItem.prototype.setActive = function (value) {
                    this.active = value;
                    this.toggleClass("active", value);
                };
                TabItem.prototype.isActive = function () {
                    return this.active;
                };
                TabItem.prototype.getCloseAction = function () {
                    return this.closeAction;
                };
                TabItem.prototype.setCloseAction = function (closeAction) {
                    this.closeAction = closeAction;
                };
                TabItem.prototype.onLabelChanged = function (listener) {
                    this.labelChangedListeners.push(listener);
                };
                TabItem.prototype.onSelected = function (listener) {
                    this.selectedListeners.push(listener);
                };
                TabItem.prototype.onClosed = function (listener) {
                    if (this.closeAction) {
                        throw new Error("Failed to set 'on closed' listener. Close action is already setted.");
                    }
                    else {
                        this.closedListeners.push(listener);
                    }
                };
                TabItem.prototype.unLabelChanged = function (listener) {
                    this.labelChangedListeners = this.labelChangedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabItem.prototype.unSelected = function (listener) {
                    this.selectedListeners = this.selectedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabItem.prototype.unClosed = function (listener) {
                    this.closedListeners = this.closedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabItem.prototype.notifyLabelChangedListeners = function (newValue, oldValue) {
                    var _this = this;
                    this.labelChangedListeners.forEach(function (listener) {
                        listener.call(_this, new tab.TabItemLabelChangedEvent(_this, oldValue, newValue));
                    });
                };
                TabItem.prototype.notifySelectedListeners = function () {
                    var _this = this;
                    this.selectedListeners.forEach(function (listener) {
                        listener.call(_this, new tab.TabItemSelectedEvent(_this));
                    });
                };
                TabItem.prototype.notifyClosedListeners = function () {
                    var _this = this;
                    this.closedListeners.forEach(function (listener) {
                        listener.call(_this, new tab.TabItemClosedEvent(_this));
                    });
                };
                return TabItem;
            })(api.dom.LiEl);
            tab.TabItem = TabItem;
            var TabItemBuilder = (function () {
                function TabItemBuilder() {
                    this.addLabelTitleAttribute = true;
                }
                TabItemBuilder.prototype.setLabel = function (label) {
                    this.label = label;
                    return this;
                };
                TabItemBuilder.prototype.setCloseAction = function (closeAction) {
                    this.closeAction = closeAction;
                    return this;
                };
                TabItemBuilder.prototype.setCloseButtonEnabled = function (enabled) {
                    this.closeButtonEnabled = enabled;
                    return this;
                };
                TabItemBuilder.prototype.setMarkUnnamed = function (markUnnamed) {
                    this.markUnnamed = markUnnamed;
                    return this;
                };
                TabItemBuilder.prototype.setMarkInvalid = function (markInvalid) {
                    this.markInvalid = markInvalid;
                    return this;
                };
                TabItemBuilder.prototype.setAddLabelTitleAttribute = function (addLabelTitleAttribute) {
                    this.addLabelTitleAttribute = addLabelTitleAttribute;
                    return this;
                };
                TabItemBuilder.prototype.build = function () {
                    return new TabItem(this);
                };
                return TabItemBuilder;
            })();
            tab.TabItemBuilder = TabItemBuilder;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (_tab) {
            var TabBar = (function (_super) {
                __extends(TabBar, _super);
                function TabBar(classes) {
                    _super.call(this, "tab-bar" + (!classes ? "" : " " + classes));
                    this.scrollEnabled = true;
                    this.tabs = [];
                    this.selectedIndex = -1;
                    this.navigationItemAddedListeners = [];
                    this.navigationItemSelectedListeners = [];
                    this.navigationItemActivatedListeners = [];
                }
                TabBar.prototype.setScrollEnabled = function (enabled) {
                    this.scrollEnabled = enabled;
                };
                TabBar.prototype.insertNavigationItem = function (tab, index, silent) {
                    var _this = this;
                    this.tabs.splice(index, 0, tab);
                    tab.setIndex(index);
                    this.tabs.slice(index + 1).forEach(function (tab) {
                        tab.setIndex(tab.getIndex() + 1);
                    });
                    this.insertChild(tab, index);
                    tab.onSelected(function (event) {
                        _this.selectNavigationItem(event.getTab().getIndex());
                    });
                    if (!silent) {
                        this.notifyTabAddedListeners(tab);
                    }
                };
                TabBar.prototype.addNavigationItem = function (tab, silent) {
                    this.insertNavigationItem(tab, this.tabs.length, silent);
                };
                TabBar.prototype.removeNavigationItem = function (tab) {
                    var tabIndex = tab.getIndex();
                    this.tabs.splice(tabIndex, 1);
                    for (var i = tabIndex; i < this.tabs.length; i++) {
                        this.tabs[i].setIndex(i);
                    }
                    if (this.isEmpty()) {
                        this.selectedIndex = -1;
                    }
                    else if ((this.getSize() - 1) < this.selectedIndex) {
                        this.selectedIndex = this.getSize() - 1;
                    }
                    else if (tabIndex < this.selectedIndex) {
                        this.selectedIndex--;
                    }
                    tab.remove();
                };
                TabBar.prototype.selectNavigationItem = function (index, silent, forced) {
                    if (index < 0 || index >= this.getSize()) {
                        return;
                    }
                    this.notifyTabActivatedListeners(index);
                    if (!forced && this.selectedIndex == index) {
                        return;
                    }
                    this.deselectNavigationItem();
                    this.selectedIndex = index;
                    var selectedTab = this.getSelectedNavigationItem();
                    selectedTab.setActive(true);
                    if (!silent && this.scrollEnabled) {
                        this.notifyTabSelectedListeners(selectedTab);
                    }
                };
                TabBar.prototype.deselectNavigationItem = function () {
                    if (this.selectedIndex != -1 && this.getSelectedNavigationItem()) {
                        this.getSelectedNavigationItem().setActive(false);
                    }
                    this.selectedIndex = -1;
                };
                TabBar.prototype.getNavigationItem = function (index) {
                    return this.tabs[index];
                };
                TabBar.prototype.getSelectedNavigationItem = function () {
                    return this.tabs[this.selectedIndex];
                };
                TabBar.prototype.getSelectedIndex = function () {
                    return this.selectedIndex;
                };
                TabBar.prototype.getSize = function () {
                    return this.tabs.length;
                };
                TabBar.prototype.isEmpty = function () {
                    return this.tabs.length === 0;
                };
                TabBar.prototype.getNavigationItems = function () {
                    return this.tabs;
                };
                TabBar.prototype.onNavigationItemAdded = function (listener) {
                    this.navigationItemAddedListeners.push(listener);
                };
                TabBar.prototype.onNavigationItemSelected = function (listener) {
                    this.navigationItemSelectedListeners.push(listener);
                };
                TabBar.prototype.onNavigationItemDeselected = function (listener) {
                };
                TabBar.prototype.onNavigationItemActivated = function (listener) {
                    this.navigationItemActivatedListeners.push(listener);
                };
                TabBar.prototype.unNavigationItemAdded = function (listener) {
                    this.navigationItemAddedListeners = this.navigationItemAddedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabBar.prototype.unNavigationItemSelected = function (listener) {
                    this.navigationItemSelectedListeners = this.navigationItemSelectedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabBar.prototype.unNavigationItemDeselected = function (listener) {
                };
                TabBar.prototype.unNavigationItemActivated = function (listener) {
                    this.navigationItemActivatedListeners = this.navigationItemActivatedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabBar.prototype.notifyTabAddedListeners = function (tab) {
                    var _this = this;
                    this.navigationItemAddedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.NavigatorEvent(tab));
                    });
                };
                TabBar.prototype.notifyTabSelectedListeners = function (tab) {
                    var _this = this;
                    this.navigationItemSelectedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.NavigatorEvent(tab));
                    });
                };
                TabBar.prototype.notifyTabActivatedListeners = function (index) {
                    var _this = this;
                    this.navigationItemActivatedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.ActivatedEvent(index));
                    });
                };
                return TabBar;
            })(api.dom.UlEl);
            _tab.TabBar = TabBar;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (tab) {
            var TabBarItem = (function (_super) {
                __extends(TabBarItem, _super);
                function TabBarItem(builder) {
                    _super.call(this, builder, "tab-bar-item");
                }
                return TabBarItem;
            })(tab.TabItem);
            tab.TabBarItem = TabBarItem;
            var TabBarItemBuilder = (function (_super) {
                __extends(TabBarItemBuilder, _super);
                function TabBarItemBuilder() {
                    _super.apply(this, arguments);
                }
                TabBarItemBuilder.prototype.build = function () {
                    return new TabBarItem(this);
                };
                return TabBarItemBuilder;
            })(tab.TabItemBuilder);
            tab.TabBarItemBuilder = TabBarItemBuilder;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (_tab) {
            var TabMenu = (function (_super) {
                __extends(TabMenu, _super);
                function TabMenu(className) {
                    var _this = this;
                    _super.call(this, "tab-menu" + (className ? " " + className : ""));
                    this.menuVisible = false;
                    this.tabs = [];
                    this.hideOnItemClick = true;
                    this.navigationItemAddedListeners = [];
                    this.navigationItemRemovedListeners = [];
                    this.navigationItemSelectedListeners = [];
                    this.navigationItemDeselectedListeners = [];
                    this.enabled = true;
                    this.tabMenuButton = this.createTabMenuButton();
                    this.tabMenuButton.hide();
                    this.tabMenuButton.addClass("tab-menu-button");
                    this.tabMenuButton.onClicked(function (event) {
                        if (_this.enabled) {
                            _this.toggleMenu();
                        }
                    });
                    this.appendChild(this.tabMenuButton);
                    this.menuEl = new api.dom.UlEl("menu");
                    this.appendChild(this.menuEl);
                    api.dom.Body.get().onClicked(function (event) {
                        if (!_this.getEl().contains(event.target)) {
                            _this.hideMenu();
                        }
                    });
                    this.onClicked(function (e) {
                        if (_this.enabled) {
                            e.preventDefault();
                            e.stopPropagation();
                            new _tab.HideTabMenuEvent(_this).fire();
                        }
                    });
                }
                TabMenu.prototype.setEnabled = function (enabled) {
                    this.enabled = enabled;
                    this.toggleClass('disabled', !enabled);
                    return this;
                };
                TabMenu.prototype.createTabMenuButton = function () {
                    return new _tab.TabMenuButton();
                };
                TabMenu.prototype.setButtonLabel = function (value) {
                    this.tabMenuButton.setLabel(value);
                    return this;
                };
                TabMenu.prototype.setButtonClass = function (cls) {
                    this.tabMenuButton.addClass(cls);
                    return this;
                };
                TabMenu.prototype.setHideOnItemClick = function (hide) {
                    this.hideOnItemClick = hide;
                    return this;
                };
                TabMenu.prototype.getTabMenuButtonEl = function () {
                    return this.tabMenuButton;
                };
                TabMenu.prototype.getMenuEl = function () {
                    return this.menuEl;
                };
                TabMenu.prototype.toggleMenu = function () {
                    if (!this.menuVisible) {
                        this.showMenu();
                    }
                    else {
                        this.hideMenu();
                    }
                };
                TabMenu.prototype.hideMenu = function () {
                    this.menuEl.hide();
                    this.menuVisible = false;
                    this.removeClass('expanded');
                };
                TabMenu.prototype.showMenu = function () {
                    this.menuEl.show();
                    this.menuVisible = true;
                    this.addClass('expanded');
                };
                TabMenu.prototype.isMenuVisible = function () {
                    return this.menuVisible;
                };
                TabMenu.prototype.insertNavigationItem = function (tab, index) {
                    this.tabs.splice(index, 0, tab);
                    tab.setIndex(index);
                    this.tabs.slice(index + 1).forEach(function (tab) {
                        tab.setIndex(tab.getIndex() + 1);
                    });
                    if (tab.isVisibleInMenu()) {
                        this.menuEl.insertChild(tab, index);
                        this.tabMenuButton.show();
                    }
                    this.initializeNewItemEvents(tab);
                };
                TabMenu.prototype.addNavigationItem = function (tab) {
                    this.insertNavigationItem(tab, this.tabs.length);
                };
                TabMenu.prototype.prependNavigationItem = function (tab) {
                    this.tabs.unshift(tab);
                    this.tabs.forEach(function (curTab) {
                        curTab.setIndex(curTab.getIndex() + 1);
                    });
                    tab.setIndex(0);
                    if (tab.isVisibleInMenu()) {
                        this.menuEl.prependChild(tab);
                        this.tabMenuButton.show();
                    }
                    this.initializeNewItemEvents(tab);
                };
                TabMenu.prototype.initializeNewItemEvents = function (tab) {
                    var _this = this;
                    tab.onSelected(function (event) {
                        _this.selectNavigationItem(event.getTab().getIndex());
                        if (_this.hideOnItemClick) {
                            _this.hideMenu();
                        }
                    });
                    tab.onLabelChanged(function (event) {
                        _this.setButtonLabel(event.getNewValue());
                    });
                    this.notifyTabAddedListeners(tab);
                };
                TabMenu.prototype.isEmpty = function () {
                    return this.tabs.length == 0;
                };
                TabMenu.prototype.getSize = function () {
                    return this.tabs.length;
                };
                TabMenu.prototype.countVisible = function () {
                    var size = 0;
                    this.tabs.forEach(function (tab) {
                        if (tab.isVisibleInMenu()) {
                            size++;
                        }
                    });
                    return size;
                };
                TabMenu.prototype.getSelectedNavigationItem = function () {
                    return this.tabs[this.selectedTab];
                };
                TabMenu.prototype.getSelectedIndex = function () {
                    return this.selectedTab;
                };
                TabMenu.prototype.getNavigationItem = function (tabIndex) {
                    return this.tabs[tabIndex];
                };
                TabMenu.prototype.getNavigationItems = function () {
                    return this.tabs;
                };
                TabMenu.prototype.removeNavigationItem = function (tab) {
                    tab.remove();
                    this.tabs.splice(tab.getIndex(), 1);
                    if (this.isEmpty()) {
                        this.selectedTab = -1;
                    }
                    else if (tab.getIndex() < this.selectedTab) {
                        this.selectedTab--;
                    }
                    else if (tab.getIndex() > this.getSize() - 1) {
                        this.selectedTab = this.getSize() - 1;
                    }
                    for (var i = tab.getIndex(); i < this.tabs.length; i++) {
                        this.tabs[i].setIndex(i);
                    }
                    if (this.countVisible() == 0) {
                        this.setButtonLabel("");
                        this.tabMenuButton.hide();
                        this.hideMenu();
                    }
                    else {
                        var newTab = this.getSelectedNavigationItem();
                        if (newTab) {
                            this.setButtonLabel(newTab.getLabel());
                            this.updateActiveTab(newTab.getIndex());
                            this.selectNavigationItem(newTab.getIndex());
                        }
                    }
                    this.notifyTabRemovedListeners(tab);
                };
                TabMenu.prototype.removeNavigationItems = function () {
                    var _this = this;
                    this.tabs.forEach(function (tab) {
                        tab.remove();
                        _this.notifyTabRemovedListeners(tab);
                    });
                    if (this.countVisible() == 0) {
                        this.setButtonLabel("");
                        this.tabMenuButton.hide();
                        this.hideMenu();
                    }
                    this.selectedTab = -1;
                    this.tabs = [];
                };
                TabMenu.prototype.updateActiveTab = function (tabIndex) {
                    this.tabs.forEach(function (tab, index) {
                        var activate = (tabIndex == index);
                        tab.setActive(activate);
                    });
                };
                TabMenu.prototype.selectNavigationItem = function (tabIndex) {
                    if (tabIndex < 0 || tabIndex >= this.getSize() || this.selectedTab == tabIndex) {
                        return;
                    }
                    this.selectedTab = tabIndex;
                    var selectedTab = this.getNavigationItem(tabIndex);
                    this.setButtonLabel(selectedTab.getLabel());
                    this.updateActiveTab(tabIndex);
                    this.notifyTabSelectedListeners(selectedTab);
                };
                TabMenu.prototype.deselectNavigationItem = function () {
                    var selectedTab = this.getSelectedNavigationItem();
                    this.selectedTab = -1;
                    this.updateActiveTab(this.selectedTab);
                    this.notifyTabDeselectedListeners(selectedTab);
                };
                TabMenu.prototype.onNavigationItemAdded = function (listener) {
                    this.navigationItemAddedListeners.push(listener);
                };
                TabMenu.prototype.onNavigationItemRemoved = function (listener) {
                    this.navigationItemRemovedListeners.push(listener);
                };
                TabMenu.prototype.onNavigationItemSelected = function (listener) {
                    this.navigationItemSelectedListeners.push(listener);
                };
                TabMenu.prototype.onNavigationItemDeselected = function (listener) {
                    this.navigationItemDeselectedListeners.push(listener);
                };
                TabMenu.prototype.unNavigationItemAdded = function (listener) {
                    this.navigationItemAddedListeners = this.navigationItemAddedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabMenu.prototype.unNavigationItemRemoved = function (listener) {
                    this.navigationItemRemovedListeners = this.navigationItemRemovedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabMenu.prototype.unNavigationItemSelected = function (listener) {
                    this.navigationItemSelectedListeners = this.navigationItemSelectedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabMenu.prototype.unNavigationItemDeselected = function (listener) {
                    this.navigationItemDeselectedListeners = this.navigationItemDeselectedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TabMenu.prototype.notifyTabAddedListeners = function (tab) {
                    var _this = this;
                    this.navigationItemAddedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.NavigatorEvent(tab));
                    });
                };
                TabMenu.prototype.notifyTabRemovedListeners = function (tab) {
                    var _this = this;
                    this.navigationItemRemovedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.NavigatorEvent(tab));
                    });
                };
                TabMenu.prototype.notifyTabSelectedListeners = function (tab) {
                    var _this = this;
                    this.navigationItemSelectedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.NavigatorEvent(tab));
                    });
                };
                TabMenu.prototype.notifyTabDeselectedListeners = function (tab) {
                    var _this = this;
                    this.navigationItemDeselectedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.NavigatorEvent(tab));
                    });
                };
                return TabMenu;
            })(api.dom.DivEl);
            _tab.TabMenu = TabMenu;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (tab) {
            var TabMenuButton = (function (_super) {
                __extends(TabMenuButton, _super);
                function TabMenuButton() {
                    _super.call(this, "tab-menu-button icon-arrow-down2");
                    this.labelEl = new api.dom.SpanEl('label');
                    this.appendChild(this.labelEl);
                }
                TabMenuButton.prototype.setLabel = function (value, addTitle) {
                    if (addTitle === void 0) { addTitle = true; }
                    this.labelEl.setHtml(value, true);
                    if (addTitle) {
                        this.labelEl.getEl().setAttribute('title', value);
                    }
                };
                return TabMenuButton;
            })(api.dom.DivEl);
            tab.TabMenuButton = TabMenuButton;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (tab) {
            var TabMenuItem = (function (_super) {
                __extends(TabMenuItem, _super);
                function TabMenuItem(builder) {
                    _super.call(this, builder.setCloseButtonEnabled(true), "tab-menu-item");
                    this.visibleInMenu = true;
                }
                TabMenuItem.prototype.isVisibleInMenu = function () {
                    return this.visibleInMenu;
                };
                TabMenuItem.prototype.setVisibleInMenu = function (value) {
                    this.visibleInMenu = value;
                    _super.prototype.setVisible.call(this, value);
                };
                return TabMenuItem;
            })(tab.TabItem);
            tab.TabMenuItem = TabMenuItem;
            var TabMenuItemBuilder = (function (_super) {
                __extends(TabMenuItemBuilder, _super);
                function TabMenuItemBuilder() {
                    _super.apply(this, arguments);
                }
                TabMenuItemBuilder.prototype.build = function () {
                    return new TabMenuItem(this);
                };
                return TabMenuItemBuilder;
            })(tab.TabItemBuilder);
            tab.TabMenuItemBuilder = TabMenuItemBuilder;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (_tab) {
            var TabItemEvent = (function () {
                function TabItemEvent(tab) {
                    this.tab = tab;
                }
                TabItemEvent.prototype.getTab = function () {
                    return this.tab;
                };
                return TabItemEvent;
            })();
            _tab.TabItemEvent = TabItemEvent;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (_tab) {
            var TabItemClosedEvent = (function (_super) {
                __extends(TabItemClosedEvent, _super);
                function TabItemClosedEvent(tab) {
                    _super.call(this, tab);
                }
                return TabItemClosedEvent;
            })(_tab.TabItemEvent);
            _tab.TabItemClosedEvent = TabItemClosedEvent;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (_tab) {
            var TabItemLabelChangedEvent = (function (_super) {
                __extends(TabItemLabelChangedEvent, _super);
                function TabItemLabelChangedEvent(tab, oldValue, newValue) {
                    _super.call(this, tab);
                    this.oldValue = oldValue;
                    this.newValue = newValue;
                }
                TabItemLabelChangedEvent.prototype.getOldValue = function () {
                    return this.oldValue;
                };
                TabItemLabelChangedEvent.prototype.getNewValue = function () {
                    return this.newValue;
                };
                return TabItemLabelChangedEvent;
            })(_tab.TabItemEvent);
            _tab.TabItemLabelChangedEvent = TabItemLabelChangedEvent;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (_tab) {
            var TabItemSelectedEvent = (function (_super) {
                __extends(TabItemSelectedEvent, _super);
                function TabItemSelectedEvent(tab) {
                    _super.call(this, tab);
                }
                return TabItemSelectedEvent;
            })(_tab.TabItemEvent);
            _tab.TabItemSelectedEvent = TabItemSelectedEvent;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tab;
        (function (tab) {
            var HideTabMenuEvent = (function (_super) {
                __extends(HideTabMenuEvent, _super);
                function HideTabMenuEvent(tabMenu) {
                    _super.call(this);
                    this.tabMenu = tabMenu;
                }
                HideTabMenuEvent.prototype.getTabMenu = function () {
                    return this.tabMenu;
                };
                HideTabMenuEvent.on = function (handler) {
                    api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
                };
                HideTabMenuEvent.un = function (handler) {
                    api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
                };
                return HideTabMenuEvent;
            })(api.event.Event);
            tab.HideTabMenuEvent = HideTabMenuEvent;
        })(tab = ui.tab || (ui.tab = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var uploader;
        (function (uploader) {
            var UploadItem = (function () {
                function UploadItem(file) {
                    this.failedListeners = [];
                    this.uploadListeners = [];
                    this.progressListeners = [];
                    this.file = file;
                }
                UploadItem.prototype.getId = function () {
                    return this.file.id;
                };
                UploadItem.prototype.setId = function (id) {
                    this.file.id = id;
                    return this;
                };
                UploadItem.prototype.getModel = function () {
                    return this.model;
                };
                UploadItem.prototype.setModel = function (model) {
                    this.model = model;
                    if (model) {
                        this.notifyUploaded(model);
                    }
                    else {
                        this.notifyFailed();
                    }
                    return this;
                };
                UploadItem.prototype.getName = function () {
                    return this.file.name;
                };
                UploadItem.prototype.setName = function (name) {
                    this.file.name = name;
                    return this;
                };
                UploadItem.prototype.getMimeType = function () {
                    return this.file.type;
                };
                UploadItem.prototype.setMimeType = function (type) {
                    this.file.type = type;
                    return this;
                };
                UploadItem.prototype.getSize = function () {
                    return this.file.size;
                };
                UploadItem.prototype.setSize = function (size) {
                    this.file.size = size;
                    return this;
                };
                UploadItem.prototype.getProgress = function () {
                    return this.file.percent;
                };
                UploadItem.prototype.setProgress = function (progress) {
                    this.file.percent = progress;
                    this.notifyProgress(progress);
                    return this;
                };
                UploadItem.prototype.getStatus = function () {
                    return this.file.status;
                };
                UploadItem.prototype.setStatus = function (status) {
                    this.file.status = status;
                    return this;
                };
                UploadItem.prototype.isUploaded = function () {
                    return !!this.model;
                };
                UploadItem.prototype.onProgress = function (listener) {
                    this.progressListeners.push(listener);
                };
                UploadItem.prototype.unProgress = function (listener) {
                    this.progressListeners = this.progressListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                UploadItem.prototype.notifyProgress = function (progress) {
                    this.progressListeners.forEach(function (listener) {
                        listener(progress);
                    });
                };
                UploadItem.prototype.onUploaded = function (listener) {
                    this.uploadListeners.push(listener);
                };
                UploadItem.prototype.unUploaded = function (listener) {
                    this.uploadListeners = this.uploadListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                UploadItem.prototype.notifyUploaded = function (model) {
                    this.uploadListeners.forEach(function (listener) {
                        listener(model);
                    });
                };
                UploadItem.prototype.onFailed = function (listener) {
                    this.failedListeners.push(listener);
                };
                UploadItem.prototype.unFailed = function (listener) {
                    this.failedListeners = this.failedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                UploadItem.prototype.notifyFailed = function () {
                    this.failedListeners.forEach(function (listener) {
                        listener();
                    });
                };
                return UploadItem;
            })();
            uploader.UploadItem = UploadItem;
        })(uploader = ui.uploader || (ui.uploader = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var uploader;
        (function (uploader) {
            var FileUploadStartedEvent = (function () {
                function FileUploadStartedEvent(uploadItems) {
                    this.uploadItems = uploadItems;
                }
                FileUploadStartedEvent.prototype.getUploadItems = function () {
                    return this.uploadItems;
                };
                return FileUploadStartedEvent;
            })();
            uploader.FileUploadStartedEvent = FileUploadStartedEvent;
        })(uploader = ui.uploader || (ui.uploader = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var uploader;
        (function (uploader) {
            var FileUploadProgressEvent = (function () {
                function FileUploadProgressEvent(uploadItem) {
                    this.uploadItem = uploadItem;
                }
                FileUploadProgressEvent.prototype.getUploadItem = function () {
                    return this.uploadItem;
                };
                return FileUploadProgressEvent;
            })();
            uploader.FileUploadProgressEvent = FileUploadProgressEvent;
        })(uploader = ui.uploader || (ui.uploader = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var uploader;
        (function (uploader) {
            var FileUploadedEvent = (function () {
                function FileUploadedEvent(uploadItem) {
                    this.uploadItem = uploadItem;
                }
                FileUploadedEvent.prototype.getUploadItem = function () {
                    return this.uploadItem;
                };
                return FileUploadedEvent;
            })();
            uploader.FileUploadedEvent = FileUploadedEvent;
        })(uploader = ui.uploader || (ui.uploader = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var uploader;
        (function (uploader) {
            var FileUploadCompleteEvent = (function () {
                function FileUploadCompleteEvent(uploadItems) {
                    this.uploadItems = uploadItems;
                }
                FileUploadCompleteEvent.prototype.getUploadItems = function () {
                    return this.uploadItems;
                };
                return FileUploadCompleteEvent;
            })();
            uploader.FileUploadCompleteEvent = FileUploadCompleteEvent;
        })(uploader = ui.uploader || (ui.uploader = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var uploader;
        (function (uploader) {
            var FileUploadFailedEvent = (function () {
                function FileUploadFailedEvent(uploadItem) {
                    this.uploadItem = uploadItem;
                }
                FileUploadFailedEvent.prototype.getUploadItem = function () {
                    return this.uploadItem;
                };
                return FileUploadFailedEvent;
            })();
            uploader.FileUploadFailedEvent = FileUploadFailedEvent;
        })(uploader = ui.uploader || (ui.uploader = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var uploader;
        (function (_uploader) {
            var Button = api.ui.button.Button;
            var CloseButton = api.ui.button.CloseButton;
            var PluploadStatus = (function () {
                function PluploadStatus() {
                }
                PluploadStatus.QUEUED = plupload.QUEUED;
                PluploadStatus.UPLOADING = plupload.UPLOADING;
                PluploadStatus.FAILED = plupload.FAILED;
                PluploadStatus.DONE = plupload.DONE;
                return PluploadStatus;
            })();
            _uploader.PluploadStatus = PluploadStatus;
            var Uploader = (function (_super) {
                __extends(Uploader, _super);
                function Uploader(config) {
                    var _this = this;
                    _super.call(this, "div", "uploader");
                    this.uploadedItems = [];
                    this.uploadStartedListeners = [];
                    this.uploadProgressListeners = [];
                    this.fileUploadedListeners = [];
                    this.uploadCompleteListeners = [];
                    this.uploadFailedListeners = [];
                    this.uploadResetListeners = [];
                    this.initConfig(config);
                    if (config.showInput) {
                        this.input = api.ui.text.TextInput.middle();
                        this.input.setPlaceholder("Paste URL to image here");
                        this.appendChild(this.input);
                    }
                    if (config.value) {
                        this.value = config.value;
                    }
                    this.dropzoneContainer = new api.dom.DivEl('dropzone-container');
                    this.dropzone = new api.dom.AEl("dropzone");
                    this.dropzone.setId('uploader-dropzone-' + new Date().getTime());
                    this.dropzone.getEl().setTabIndex(-1);
                    this.getEl().setTabIndex(0);
                    this.dropzoneContainer.appendChild(this.dropzone);
                    if (config.hideDropZone) {
                        this.dropzoneContainer.getEl().setAttribute("hidden", "true");
                    }
                    this.appendChild(this.dropzoneContainer);
                    this.progress = new api.ui.ProgressBar();
                    this.appendChild(this.progress);
                    this.resultContainer = new api.dom.DivEl('result-container');
                    this.appendChild(this.resultContainer);
                    this.cancelBtn = new Button("Cancel");
                    this.cancelBtn.setVisible(this.config.showCancel);
                    this.cancelBtn.onClicked(function (event) {
                        _this.stop();
                        _this.reset();
                    });
                    this.appendChild(this.cancelBtn);
                    this.resetBtn = new CloseButton();
                    this.resetBtn.setVisible(this.config.showReset);
                    this.resetBtn.onClicked(function (event) {
                        _this.reset();
                    });
                    this.appendChild(this.resetBtn);
                    this.onKeyPressed(function (event) {
                        if (_this.dropzoneContainer.isVisible() && event.keyCode == 13) {
                            wemjq(_this.dropzone.getEl().getHTMLElement()).simulate("click");
                        }
                    });
                    var resetHandler = function () {
                        _this.reset();
                        return false;
                    };
                    ui.KeyBindings.get().bindKeys([
                        new ui.KeyBinding('del', resetHandler),
                        new ui.KeyBinding('backspace', resetHandler)
                    ]);
                    if (this.config.deferred) {
                        this.onShown(function (event) { return _this.initHandler.call(_this, event); });
                    }
                    else {
                        this.onRendered(function (event) { return _this.initHandler.call(_this, event); });
                    }
                    this.onRemoved(function (event) { return _this.destroyHandler.call(_this, event); });
                    this.listenToDragEvents();
                }
                Uploader.prototype.listenToDragEvents = function () {
                    var _this = this;
                    var dragOverEl;
                    this.onDragEnter(function (event) {
                        if (_this.config.dropAlwaysAllowed) {
                            var targetEl = event.target;
                            if (!dragOverEl) {
                                if (Uploader.debug) {
                                    console.log('drag entered, adding class');
                                }
                                _this.addClass('dragover');
                            }
                            dragOverEl = targetEl;
                        }
                    });
                    this.onDragLeave(function (event) {
                        if (_this.config.dropAlwaysAllowed) {
                            var targetEl = event.target;
                            if (dragOverEl == targetEl) {
                                if (Uploader.debug) {
                                    console.log('drag left, removing class');
                                }
                                _this.removeClass('dragover');
                                dragOverEl = undefined;
                            }
                        }
                    });
                    this.onDrop(function (event) {
                        if (_this.config.dropAlwaysAllowed) {
                            if (Uploader.debug) {
                                console.log('drag drop removing class');
                            }
                            _this.removeClass('dragover');
                        }
                    });
                };
                Uploader.prototype.setResetVisible = function (visible) {
                    this.config.showReset = visible;
                    this.resetBtn.setVisible(visible);
                };
                Uploader.prototype.getResetButton = function () {
                    return this.resetBtn;
                };
                Uploader.prototype.initHandler = function () {
                    if (this.config.disabled) {
                        if (Uploader.debug) {
                            console.log('Skipping init, because of config.disabled = true', this);
                        }
                    }
                    else {
                        if (Uploader.debug) {
                            console.log('Initing uploader', this);
                        }
                        if (!this.uploader && this.config.url) {
                            this.uploader = this.initUploader(this.dropzone.getId(), this.config.dropAlwaysAllowed ? this.getId() : this.dropzone.getId());
                            if (this.value) {
                                this.setValue(this.value);
                            }
                            else if (!this.config.hideDropZone) {
                                this.setDropzoneVisible();
                            }
                        }
                    }
                };
                Uploader.prototype.destroyHandler = function () {
                    if (Uploader.debug) {
                        console.log('Destroying uploader', this);
                    }
                    if (this.uploader) {
                        this.uploader.destroy();
                        this.uploader = null;
                    }
                };
                Uploader.prototype.initConfig = function (config) {
                    this.config = config;
                    if (this.config.showResult == undefined) {
                        this.config.showResult = true;
                    }
                    if (this.config.allowMultiSelection == undefined) {
                        this.config.allowMultiSelection = false;
                    }
                    if (this.config.showCancel == undefined) {
                        this.config.showCancel = true;
                    }
                    if (this.config.showReset == undefined) {
                        this.config.showReset = true;
                    }
                    if (this.config.maximumOccurrences == undefined) {
                        this.config.maximumOccurrences = 0;
                    }
                    if (this.config.allowBrowse == undefined) {
                        this.config.allowBrowse = true;
                    }
                    if (this.config.allowDrop == undefined) {
                        this.config.allowDrop = true;
                    }
                    if (this.config.dropAlwaysAllowed == undefined) {
                        this.config.dropAlwaysAllowed = false;
                    }
                    if (this.config.dropzoneAlwaysVisible == undefined) {
                        this.config.dropzoneAlwaysVisible = false;
                    }
                    if (this.config.resultAlwaysVisisble == undefined) {
                        this.config.resultAlwaysVisisble = false;
                    }
                    if (this.config.allowTypes == undefined) {
                        this.config.allowTypes = [];
                    }
                    if (this.config.deferred == undefined) {
                        this.config.deferred = false;
                    }
                    if (this.config.disabled == undefined) {
                        this.config.disabled = false;
                    }
                };
                Uploader.prototype.getName = function () {
                    return this.config.name;
                };
                Uploader.prototype.getValue = function () {
                    return this.value;
                };
                Uploader.prototype.setValue = function (value) {
                    var _this = this;
                    if (Uploader.debug) {
                        console.log('Setting uploader value', value, this);
                    }
                    this.value = value;
                    if (value && this.config.showResult) {
                        this.setResultVisible();
                    }
                    else {
                        this.setDropzoneVisible();
                    }
                    var results = this.getResultContainer().removeChildren();
                    this.parseValues(value).forEach(function (val) {
                        if (val) {
                            results.appendChild(_this.createResultItem(val));
                        }
                    });
                    return this;
                };
                Uploader.prototype.parseValues = function (jsonString) {
                    try {
                        var o = JSON.parse(jsonString);
                        if (o && typeof o === "object" && o.length) {
                            return o;
                        }
                    }
                    catch (e) {
                    }
                    return [jsonString];
                };
                Uploader.prototype.createResultItem = function (value) {
                    throw new Error('Should be overridden by inheritors');
                };
                Uploader.prototype.setMaximumOccurrences = function (value) {
                    this.config.maximumOccurrences = value;
                    return this;
                };
                Uploader.prototype.stop = function () {
                    if (this.uploader) {
                        this.uploader.stop();
                    }
                    return this;
                };
                Uploader.prototype.reset = function () {
                    this.setValue(null);
                    this.notifyUploadReset();
                    return this;
                };
                Uploader.prototype.setDropzoneVisible = function (visible) {
                    if (visible === void 0) { visible = true; }
                    if (!visible && this.config.dropzoneAlwaysVisible) {
                        return;
                    }
                    if (visible) {
                        this.setProgressVisible(false);
                        this.setResultVisible(false);
                    }
                    if (this.input) {
                        this.input.setVisible(visible);
                    }
                    this.dropzoneContainer.setVisible(visible);
                };
                Uploader.prototype.setProgressVisible = function (visible) {
                    if (visible === void 0) { visible = true; }
                    if (visible) {
                        this.setDropzoneVisible(false);
                        this.setResultVisible(false);
                    }
                    this.progress.setVisible(visible);
                    this.cancelBtn.setVisible(visible && this.config.showCancel);
                };
                Uploader.prototype.setResultVisible = function (visible) {
                    if (visible === void 0) { visible = true; }
                    if (!visible && this.config.resultAlwaysVisisble) {
                        return;
                    }
                    if (visible) {
                        this.setDropzoneVisible(false);
                        this.setProgressVisible(false);
                    }
                    this.resultContainer.setVisible(visible);
                    this.resetBtn.setVisible(visible && this.config.showReset);
                };
                Uploader.prototype.createModel = function (serverResponse) {
                    throw new Error('Should be overridden by inheritors');
                };
                Uploader.prototype.getModelValue = function (item) {
                    throw new Error('Should be overridden by inheritors');
                };
                Uploader.prototype.setParams = function (params) {
                    if (this.uploader) {
                        this.uploader.setOption('multipart_params', params);
                    }
                    else {
                        this.config.params = params;
                    }
                    return this;
                };
                Uploader.prototype.setEnabled = function (enabled) {
                    var _this = this;
                    this.config.disabled = !enabled;
                    if (!enabled) {
                        this.dropzone.getEl().setAttribute('disabled', 'true');
                    }
                    else {
                        this.dropzone.getEl().removeAttribute('disabled');
                    }
                    if (!enabled && this.uploader) {
                        if (Uploader.debug) {
                            console.log('Disabling uploader', this);
                        }
                        this.destroyHandler();
                    }
                    else if (enabled && !this.uploader) {
                        if (Uploader.debug) {
                            console.log('Enabling uploader', this);
                        }
                        if (this.config.deferred) {
                            if (this.isVisible()) {
                                this.initHandler();
                            }
                            else {
                                if (Uploader.debug) {
                                    console.log('Deferring enabling uploader until it\' shown', this);
                                }
                                this.onShown(function (event) { return _this.initHandler.call(_this, event); });
                            }
                        }
                        else {
                            if (this.isRendered()) {
                                this.initHandler();
                            }
                            else {
                                if (Uploader.debug) {
                                    console.log('Deferring enabling uploader until it\' rendered', this);
                                }
                                this.onRendered(function (event) { return _this.initHandler.call(_this, event); });
                            }
                        }
                    }
                    return this;
                };
                Uploader.prototype.isEnabled = function () {
                    return this.uploader != null;
                };
                Uploader.prototype.getParams = function () {
                    return this.uploader ? this.uploader.getOption('multipart_params') : this.config.params;
                };
                Uploader.prototype.findUploadItemById = function (id) {
                    for (var i = 0; i < this.uploadedItems.length; i++) {
                        var uploadItem = this.uploadedItems[i];
                        if (uploadItem.getId() == id) {
                            return uploadItem;
                        }
                    }
                    return null;
                };
                Uploader.prototype.isFolder = function (file) {
                    return file.size % 4096 == 0 && api.util.StringHelper.isEmpty(file.type);
                };
                Uploader.prototype.validateFiles = function (up, files) {
                    for (var i = 0; i < files.length; i++) {
                        var file = files[i];
                        if (this.isFolder(file)) {
                            if (Uploader.debug) {
                                console.log('Removing folder [' + file.name + '] because it can\'t be uploaded', this);
                            }
                            files.splice(i, 1);
                            up.splice(i, 1);
                            api.notify.NotifyManager.get().showWarning('Folder [' + file.name + '] upload is not supported yet');
                        }
                    }
                    if (this.config.maximumOccurrences > 0 && files.length > this.config.maximumOccurrences) {
                        if (Uploader.debug) {
                            console.log('Max ' + this.config.maximumOccurrences + ' files allowed, removing the rest', this);
                        }
                        files.splice(this.config.maximumOccurrences);
                        up.splice(this.config.maximumOccurrences);
                        api.notify.NotifyManager.get().showWarning('Max ' + this.config.maximumOccurrences + ' files are allowed');
                    }
                    return files.length > 0;
                };
                Uploader.prototype.initUploader = function (browseId, dropId) {
                    var _this = this;
                    if (!plupload) {
                        throw new Error("Uploader: plupload not found, check if it is included in page.");
                    }
                    var uploader = new plupload.Uploader({
                        multipart_params: this.config.params,
                        runtimes: 'html5,flash,silverlight,html4',
                        multi_selection: this.config.allowMultiSelection,
                        browse_button: this.config.allowBrowse ? browseId : undefined,
                        url: this.config.url,
                        multipart: true,
                        drop_element: this.config.allowDrop ? dropId : undefined,
                        flash_swf_url: api.util.UriHelper.getAdminUri('common/js/lib/plupload/js/Moxie.swf'),
                        silverlight_xap_url: api.util.UriHelper.getAdminUri('common/js/lib/plupload/js/Moxie.xap'),
                        filters: {
                            mime_types: this.config.allowTypes
                        }
                    });
                    uploader.bind('Init', function (up, params) {
                        if (Uploader.debug) {
                            console.log('uploader init', up, params);
                        }
                        _this.disableInputFocus();
                    }, this);
                    uploader.bind('FilesAdded', function (up, files) {
                        if (Uploader.debug) {
                            console.log('uploader files added', up, files);
                        }
                        if (!_this.validateFiles(up, files)) {
                            if (Uploader.debug) {
                                console.log('no valid files for upload found', up, files);
                            }
                            return;
                        }
                        files.forEach(function (file) {
                            _this.uploadedItems.push(new _uploader.UploadItem(file));
                        });
                        _this.setProgressVisible();
                        setTimeout(function () {
                            up.start();
                        }, 1);
                        _this.notifyFileUploadStarted(_this.uploadedItems);
                    }, this);
                    uploader.bind('QueueChanged', function (up) {
                        if (Uploader.debug) {
                            console.log('uploader queue changed', up);
                        }
                    }, this);
                    uploader.bind('UploadFile', function (up, file) {
                        if (Uploader.debug) {
                            console.log('uploader upload file', up, file);
                        }
                    }, this);
                    uploader.bind('UploadProgress', function (up, file) {
                        if (Uploader.debug) {
                            console.log('uploader upload progress', up, file);
                        }
                        _this.progress.setValue(file.percent);
                        var uploadItem = _this.findUploadItemById(file.id);
                        if (uploadItem) {
                            uploadItem.setProgress(file.percent);
                            _this.notifyFileUploadProgress(uploadItem);
                        }
                    }, this);
                    uploader.bind('FileUploaded', function (up, file, response) {
                        if (Uploader.debug) {
                            console.log('uploader file uploaded', up, file, response);
                        }
                        if (response && response.status === 200) {
                            try {
                                var uploadItem = _this.findUploadItemById(file.id);
                                if (uploadItem) {
                                    var model = _this.createModel(JSON.parse(response.response));
                                    uploadItem.setModel(model);
                                    _this.notifyFileUploaded(uploadItem);
                                }
                            }
                            catch (e) {
                                console.warn("Failed to parse the response", response, e);
                            }
                        }
                    }, this);
                    uploader.bind('Error', function (up, response) {
                        if (Uploader.debug) {
                            console.log('uploader error', up, response);
                        }
                        try {
                            var responseObj = JSON.parse(response.response);
                            var error = new api.rest.RequestError(responseObj.status, responseObj.message);
                            api.DefaultErrorHandler.handle(error);
                        }
                        catch (e) {
                            console.warn("Failed to parse the response", response, e);
                            var files = up.files.map(function (file) { return file.name; });
                            api.notify.NotifyManager.get().showError("File(s) [" + files.join(', ') + "] were not uploaded");
                        }
                        var uploadItem = _this.findUploadItemById(response.file.id);
                        if (uploadItem) {
                            uploadItem.setModel(null);
                            _this.notifyUploadFailed(uploadItem);
                        }
                    }, this);
                    uploader.bind('UploadComplete', function (up, files) {
                        if (Uploader.debug) {
                            console.log('uploader upload complete', up, files);
                        }
                        var values = [];
                        _this.uploadedItems.forEach(function (item) {
                            if (item.getStatus() == PluploadStatus.DONE) {
                                values.push(_this.getModelValue(item.getModel()));
                            }
                        });
                        if (values.length > 0) {
                            _this.setValue(JSON.stringify(values));
                            _this.notifyUploadCompleted(_this.uploadedItems);
                        }
                        _this.uploadedItems.length = 0;
                        _this.uploader.splice();
                    }, this);
                    uploader.init();
                    return uploader;
                };
                Uploader.prototype.disableInputFocus = function () {
                    var focusableElements = this.getDropzoneContainer().getHTMLElement().getElementsByTagName("input");
                    for (var i = 0; i < focusableElements.length; i++) {
                        var el = focusableElements.item(i);
                        el.tabIndex = -1;
                    }
                };
                Uploader.prototype.getResultContainer = function () {
                    return this.resultContainer;
                };
                Uploader.prototype.getDropzoneContainer = function () {
                    return this.dropzoneContainer;
                };
                Uploader.prototype.getDropzone = function () {
                    return this.dropzone;
                };
                Uploader.prototype.onUploadStarted = function (listener) {
                    this.uploadStartedListeners.push(listener);
                };
                Uploader.prototype.unUploadStarted = function (listener) {
                    this.uploadStartedListeners = this.uploadStartedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                Uploader.prototype.onUploadProgress = function (listener) {
                    this.uploadProgressListeners.push(listener);
                };
                Uploader.prototype.unUploadProgress = function (listener) {
                    this.uploadProgressListeners = this.uploadProgressListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                Uploader.prototype.onFileUploaded = function (listener) {
                    this.fileUploadedListeners.push(listener);
                };
                Uploader.prototype.unFileUploaded = function (listener) {
                    this.fileUploadedListeners = this.fileUploadedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                Uploader.prototype.onUploadCompleted = function (listener) {
                    this.uploadCompleteListeners.push(listener);
                };
                Uploader.prototype.unUploadCompleted = function (listener) {
                    this.uploadCompleteListeners = this.uploadCompleteListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                Uploader.prototype.onUploadReset = function (listener) {
                    this.uploadResetListeners.push(listener);
                };
                Uploader.prototype.unUploadReset = function (listener) {
                    this.uploadResetListeners = this.uploadResetListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                Uploader.prototype.onUploadFailed = function (listener) {
                    this.uploadFailedListeners.push(listener);
                };
                Uploader.prototype.unUploadFailed = function (listener) {
                    this.uploadFailedListeners = this.uploadFailedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                Uploader.prototype.notifyFileUploadStarted = function (uploadItems) {
                    this.uploadStartedListeners.forEach(function (listener) {
                        listener(new _uploader.FileUploadStartedEvent(uploadItems));
                    });
                };
                Uploader.prototype.notifyFileUploadProgress = function (uploadItem) {
                    this.uploadProgressListeners.forEach(function (listener) {
                        listener(new _uploader.FileUploadProgressEvent(uploadItem));
                    });
                };
                Uploader.prototype.notifyFileUploaded = function (uploadItem) {
                    var _this = this;
                    this.fileUploadedListeners.forEach(function (listener) {
                        listener.call(_this, new _uploader.FileUploadedEvent(uploadItem));
                    });
                };
                Uploader.prototype.notifyUploadCompleted = function (uploadItems) {
                    this.uploadCompleteListeners.forEach(function (listener) {
                        listener(new _uploader.FileUploadCompleteEvent(uploadItems));
                    });
                };
                Uploader.prototype.notifyUploadReset = function () {
                    var _this = this;
                    this.uploadResetListeners.forEach(function (listener) {
                        listener.call(_this);
                    });
                };
                Uploader.prototype.notifyUploadFailed = function (uploadItem) {
                    this.uploadFailedListeners.forEach(function (listener) {
                        listener(new _uploader.FileUploadFailedEvent(uploadItem));
                    });
                };
                Uploader.debug = false;
                return Uploader;
            })(api.dom.FormInputEl);
            _uploader.Uploader = Uploader;
        })(uploader = ui.uploader || (ui.uploader = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (panel) {
            var Panel = (function (_super) {
                __extends(Panel, _super);
                function Panel(className) {
                    var _this = this;
                    _super.call(this, "panel" + (className ? " " + className : ""));
                    this.doOffset = true;
                    this.onShown(function (event) {
                        if (_this.doOffset) {
                            _this.calculateOffset();
                        }
                    });
                }
                Panel.prototype.setDoOffset = function (value) {
                    this.doOffset = value;
                    if (value && this.isRendered()) {
                        this.calculateOffset();
                    }
                };
                Panel.prototype.calculateOffset = function () {
                    var previous = this.getEl().getPrevious();
                    var top = previous ? (previous.getOffsetTopRelativeToParent() + previous.getHeightWithMargin()) : 0;
                    this.getEl().setTopPx(top);
                };
                return Panel;
            })(api.dom.DivEl);
            panel.Panel = Panel;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (_panel) {
            var PanelShownEvent = (function () {
                function PanelShownEvent(panel, index, previousPanel) {
                    this.panel = panel;
                    this.index = index;
                    this.previousPanel = previousPanel;
                }
                PanelShownEvent.prototype.getPanel = function () {
                    return this.panel;
                };
                PanelShownEvent.prototype.getIndex = function () {
                    return this.index;
                };
                PanelShownEvent.prototype.getPreviousPanel = function () {
                    return this.previousPanel;
                };
                return PanelShownEvent;
            })();
            _panel.PanelShownEvent = PanelShownEvent;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (_panel) {
            var DeckPanel = (function (_super) {
                __extends(DeckPanel, _super);
                function DeckPanel(className) {
                    _super.call(this, "deck-panel" + (className ? " " + className : ""));
                    this.panels = [];
                    this.panelShown = null;
                    this.panelShownListeners = [];
                }
                DeckPanel.prototype.isEmpty = function () {
                    return this.panels.length == 0;
                };
                DeckPanel.prototype.getSize = function () {
                    return this.panels.length;
                };
                DeckPanel.prototype.addPanel = function (panel) {
                    panel.hide();
                    panel.setDoOffset(false);
                    this.appendChild(panel);
                    return this.panels.push(panel) - 1;
                };
                DeckPanel.prototype.getPanel = function (index) {
                    return this.panels[index];
                };
                DeckPanel.prototype.getLastPanel = function () {
                    return this.getPanel(this.getSize() - 1) || null;
                };
                DeckPanel.prototype.getPanelShown = function () {
                    return this.panelShown;
                };
                DeckPanel.prototype.getPanelShownIndex = function () {
                    return this.getPanelIndex(this.panelShown);
                };
                DeckPanel.prototype.getPanelIndex = function (panel) {
                    var size = this.getSize();
                    for (var i = 0; i < size; i++) {
                        if (this.panels[i] === panel) {
                            return i;
                        }
                    }
                    return -1;
                };
                DeckPanel.prototype.removePanelByIndex = function (index, checkCanRemovePanel) {
                    if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                    var panelToRemove = this.getPanel(index);
                    return this.removePanel(panelToRemove, checkCanRemovePanel) ? panelToRemove : null;
                };
                DeckPanel.prototype.removePanel = function (panelToRemove, checkCanRemovePanel) {
                    if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                    var index = this.getPanelIndex(panelToRemove);
                    if (index < 0 || checkCanRemovePanel && !this.canRemovePanel(panelToRemove)) {
                        return -1;
                    }
                    panelToRemove.remove();
                    this.panels.splice(index, 1);
                    if (this.isEmpty()) {
                        this.panelShown = null;
                    }
                    else if (panelToRemove == this.getPanelShown()) {
                        this.showPanelByIndex(Math.min(index, this.getSize() - 1));
                    }
                    return index;
                };
                DeckPanel.prototype.canRemovePanel = function (panel) {
                    return true;
                };
                DeckPanel.prototype.showPanel = function (panel) {
                    var index = this.getPanelIndex(panel);
                    if (index > -1) {
                        this.showPanelByIndex(index);
                    }
                };
                DeckPanel.prototype.showPanelByIndex = function (index) {
                    var previousPanel = this.getPanelShown();
                    var panelToShow = this.getPanel(index);
                    if (panelToShow == null) {
                        return;
                    }
                    if (this.panelShown != null) {
                        this.panelShown.hide();
                    }
                    panelToShow.show();
                    this.panelShown = panelToShow;
                    this.notifyPanelShown(panelToShow, index, previousPanel);
                };
                DeckPanel.prototype.onPanelShown = function (listener) {
                    this.panelShownListeners.push(listener);
                };
                DeckPanel.prototype.unPanelShown = function (listener) {
                    this.panelShownListeners = this.panelShownListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                DeckPanel.prototype.notifyPanelShown = function (panel, panelIndex, previousPanel) {
                    var _this = this;
                    this.panelShownListeners.forEach(function (listener) {
                        listener.call(_this, new _panel.PanelShownEvent(panel, panelIndex, previousPanel));
                    });
                };
                return DeckPanel;
            })(_panel.Panel);
            _panel.DeckPanel = DeckPanel;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (_panel) {
            var PanelStrip = (function (_super) {
                __extends(PanelStrip, _super);
                function PanelStrip(scrollable, className) {
                    var _this = this;
                    _super.call(this, "panel-strip" + (className ? " " + className : ""));
                    this.panels = [];
                    this.headers = [];
                    this.offset = 0;
                    this.panelShown = null;
                    this.panelShownListeners = [];
                    if (scrollable) {
                        this.scrollable = scrollable;
                        this.scrollable.addClass('panel-strip-scrollable');
                    }
                    else {
                        this.scrollable = this;
                    }
                    this.onShown(function () {
                        _this.updateLastPanelHeight();
                    });
                }
                PanelStrip.prototype.countExistingChildren = function (index) {
                    var count = Math.min(this.panels.length, index);
                    for (var i = 0; i < Math.min(this.headers.length, index); i++) {
                        if (this.headers[i]) {
                            count++;
                        }
                    }
                    return count;
                };
                PanelStrip.prototype.insertPanel = function (panel, index, header) {
                    panel.setDoOffset(false);
                    var previousChildrenIndex = this.countExistingChildren(index);
                    if (header) {
                        var headerEl = new api.dom.H2El("panel-strip-panel-header");
                        headerEl.getEl().setInnerHtml(header);
                        this.insertChild(headerEl, previousChildrenIndex);
                    }
                    this.panels.splice(index, 0, panel);
                    this.headers.splice(index, 0, headerEl);
                    if (header) {
                        this.insertChild(panel, previousChildrenIndex + 1);
                    }
                    else {
                        this.insertChild(panel, previousChildrenIndex);
                    }
                    if (this.isVisible()) {
                        this.updateLastPanelHeight();
                    }
                    return index;
                };
                PanelStrip.prototype.getPanels = function () {
                    return this.panels;
                };
                PanelStrip.prototype.getScrollable = function () {
                    return this.scrollable;
                };
                PanelStrip.prototype.setScrollOffset = function (offset) {
                    this.offset = offset;
                    return this;
                };
                PanelStrip.prototype.getScrollOffset = function () {
                    return this.offset;
                };
                PanelStrip.prototype.updateLastPanelHeight = function () {
                    if (this.getSize() == 0) {
                        return;
                    }
                    if (this.getSize() > 1) {
                        var beforeLastEl = this.getPanel(this.getSize() - 2).getEl();
                        var originalHeight = beforeLastEl.getData("originalHeight");
                        if (originalHeight) {
                            beforeLastEl.setHeight(originalHeight);
                        }
                    }
                    var lastEl = this.getPanel(this.getSize() - 1).getEl();
                    if (!lastEl.getData("originalHeight")) {
                        lastEl.setData("originalHeight", lastEl.getHTMLElement().style.height || "auto");
                    }
                    lastEl.setHeightPx(this.scrollable.getEl().getHeight());
                };
                PanelStrip.prototype.removePanel = function (panelToRemove, checkCanRemovePanel) {
                    if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                    var index = this.getPanelIndex(panelToRemove);
                    if (index < 0 || checkCanRemovePanel && !this.canRemovePanel(panelToRemove)) {
                        return -1;
                    }
                    this.removeChild(panelToRemove);
                    this.removeChild(this.getHeader(index));
                    this.panels.splice(index, 1);
                    this.headers.splice(index, 1);
                    if (this.isEmpty()) {
                        this.panelShown = null;
                    }
                    else if (panelToRemove == this.getPanelShown()) {
                        this.showPanelByIndex(Math.min(index, this.getSize() - 1));
                    }
                    if (this.isVisible() && index == this.getSize() && !this.isEmpty()) {
                        this.updateLastPanelHeight();
                    }
                    return index;
                };
                PanelStrip.prototype.canRemovePanel = function (panel) {
                    return true;
                };
                PanelStrip.prototype.isEmpty = function () {
                    return this.panels.length == 0;
                };
                PanelStrip.prototype.getSize = function () {
                    return this.panels.length;
                };
                PanelStrip.prototype.getPanel = function (index) {
                    return this.panels[index];
                };
                PanelStrip.prototype.getHeader = function (index) {
                    return this.headers[index];
                };
                PanelStrip.prototype.getPanelShown = function () {
                    return this.panelShown;
                };
                PanelStrip.prototype.getPanelShownIndex = function () {
                    return this.getPanelIndex(this.panelShown);
                };
                PanelStrip.prototype.getPanelIndex = function (panel) {
                    var size = this.getSize();
                    for (var i = 0; i < size; i++) {
                        if (this.panels[i] === panel) {
                            return i;
                        }
                    }
                    return -1;
                };
                PanelStrip.prototype.showPanel = function (panel) {
                    var index = this.getPanelIndex(panel);
                    if (index > -1) {
                        this.showPanelByIndex(index);
                    }
                };
                PanelStrip.prototype.showPanelByIndex = function (index) {
                    var _this = this;
                    var panelToShow = this.getPanel(index);
                    if (panelToShow == null) {
                        return;
                    }
                    wemjq(this.scrollable.getHTMLElement()).animate({
                        scrollTop: index == 0 ? 0 : this.getScroll() - this.offset + panelToShow.getEl().getOffsetToParent().top - this.headers[index].getEl().getHeightWithBorder()
                    }, {
                        duration: 500,
                        complete: function () {
                            _this.notifyPanelShown(panelToShow, index, _this.getPanelShown());
                            _this.panelShown = panelToShow;
                        }
                    });
                };
                PanelStrip.prototype.getScroll = function () {
                    return this.scrollable.getHTMLElement().scrollTop;
                };
                PanelStrip.prototype.setScroll = function (scrollTop) {
                    this.scrollable.getHTMLElement().scrollTop = scrollTop;
                };
                PanelStrip.prototype.onPanelShown = function (listener) {
                    this.panelShownListeners.push(listener);
                };
                PanelStrip.prototype.unPanelShown = function (listener) {
                    this.panelShownListeners = this.panelShownListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                PanelStrip.prototype.notifyPanelShown = function (panel, panelIndex, previousPanel) {
                    var _this = this;
                    this.panelShownListeners.forEach(function (listener) {
                        listener.call(_this, new _panel.PanelShownEvent(panel, panelIndex, previousPanel));
                    });
                };
                return PanelStrip;
            })(_panel.Panel);
            _panel.PanelStrip = PanelStrip;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (panel) {
            var ResponsiveManager = api.ui.responsive.ResponsiveManager;
            (function (SplitPanelAlignment) {
                SplitPanelAlignment[SplitPanelAlignment["HORIZONTAL"] = 0] = "HORIZONTAL";
                SplitPanelAlignment[SplitPanelAlignment["VERTICAL"] = 1] = "VERTICAL";
            })(panel.SplitPanelAlignment || (panel.SplitPanelAlignment = {}));
            var SplitPanelAlignment = panel.SplitPanelAlignment;
            (function (SplitPanelUnit) {
                SplitPanelUnit[SplitPanelUnit["PIXEL"] = 0] = "PIXEL";
                SplitPanelUnit[SplitPanelUnit["PERCENT"] = 1] = "PERCENT";
            })(panel.SplitPanelUnit || (panel.SplitPanelUnit = {}));
            var SplitPanelUnit = panel.SplitPanelUnit;
            var SplitPanelBuilder = (function () {
                function SplitPanelBuilder(firstPanel, secondPanel) {
                    this.firstPanelSize = 50;
                    this.firstPanelMinSize = 0;
                    this.firstPanelUnit = 1 /* PERCENT */;
                    this.secondPanelSize = 50;
                    this.secondPanelMinSize = 0;
                    this.secondPanelUnit = 1 /* PERCENT */;
                    this.alignment = 0 /* HORIZONTAL */;
                    this.splitterThickness = 5;
                    this.firstPanelIsDecidingPanel = true;
                    this.firstPanel = firstPanel;
                    this.secondPanel = secondPanel;
                }
                SplitPanelBuilder.prototype.build = function () {
                    return new SplitPanel(this);
                };
                SplitPanelBuilder.prototype.setFirstPanelSize = function (size, unit) {
                    this.firstPanelSize = size;
                    this.firstPanelUnit = unit;
                    this.firstPanelIsDecidingPanel = true;
                    return this;
                };
                SplitPanelBuilder.prototype.setFirstPanelMinSize = function (size, unit) {
                    this.firstPanelMinSize = size;
                    this.firstPanelUnit = unit;
                    this.firstPanelIsDecidingPanel = false;
                    return this;
                };
                SplitPanelBuilder.prototype.setSecondPanelSize = function (size, unit) {
                    this.secondPanelSize = size;
                    this.secondPanelUnit = unit;
                    return this;
                };
                SplitPanelBuilder.prototype.setSecondPanelMinSize = function (size, unit) {
                    this.secondPanelMinSize = size;
                    this.secondPanelUnit = unit;
                    return this;
                };
                SplitPanelBuilder.prototype.setAlignment = function (alignment) {
                    this.alignment = alignment;
                    return this;
                };
                SplitPanelBuilder.prototype.setAlignmentTreshold = function (treshold) {
                    this.alignmentTreshold = treshold;
                    return this;
                };
                SplitPanelBuilder.prototype.setAnimationDelay = function (value) {
                    this.animationDelay = value;
                    return this;
                };
                SplitPanelBuilder.prototype.setSplitterThickness = function (thickness) {
                    this.splitterThickness = thickness;
                    return this;
                };
                SplitPanelBuilder.prototype.getFirstPanel = function () {
                    return this.firstPanel;
                };
                SplitPanelBuilder.prototype.getFirstPanelMinSize = function () {
                    return this.firstPanelMinSize;
                };
                SplitPanelBuilder.prototype.getSecondPanel = function () {
                    return this.secondPanel;
                };
                SplitPanelBuilder.prototype.getSecondPanelMinSize = function () {
                    return this.secondPanelMinSize;
                };
                SplitPanelBuilder.prototype.getFirstPanelSize = function () {
                    return this.firstPanelSize;
                };
                SplitPanelBuilder.prototype.getSecondPanelSize = function () {
                    return this.secondPanelSize;
                };
                SplitPanelBuilder.prototype.getAlignment = function () {
                    return this.alignment;
                };
                SplitPanelBuilder.prototype.getAlignmentTreshold = function () {
                    return this.alignmentTreshold;
                };
                SplitPanelBuilder.prototype.getAnimationDelay = function () {
                    return this.animationDelay;
                };
                SplitPanelBuilder.prototype.getSplitterThickness = function () {
                    return this.splitterThickness;
                };
                SplitPanelBuilder.prototype.getFirstPanelUnit = function () {
                    return this.firstPanelUnit;
                };
                SplitPanelBuilder.prototype.getSecondPanelUnit = function () {
                    return this.secondPanelUnit;
                };
                SplitPanelBuilder.prototype.isFirstPanelDecidingPanel = function () {
                    return this.firstPanelIsDecidingPanel;
                };
                return SplitPanelBuilder;
            })();
            panel.SplitPanelBuilder = SplitPanelBuilder;
            var SplitPanel = (function (_super) {
                __extends(SplitPanel, _super);
                function SplitPanel(builder) {
                    var _this = this;
                    _super.call(this, "split-panel");
                    this.firstPanelSize = -1;
                    this.secondPanelSize = -1;
                    this.firstPanel = builder.getFirstPanel();
                    this.firstPanelMinSize = builder.getFirstPanelMinSize();
                    this.firstPanelUnit = builder.getFirstPanelUnit();
                    this.secondPanel = builder.getSecondPanel();
                    this.secondPanelMinSize = builder.getSecondPanelMinSize();
                    this.secondPanelUnit = builder.getSecondPanelUnit();
                    this.animationDelay = builder.getAnimationDelay();
                    this.firstPanelIsHidden = false;
                    this.secondPanelIsHidden = false;
                    this.firstPanelIsFullScreen = false;
                    this.splitterIsHidden = false;
                    this.saveFirstPanelSize();
                    if (builder.isFirstPanelDecidingPanel()) {
                        this.setFirstPanelSize(builder.getFirstPanelSize(), this.firstPanelUnit);
                    }
                    else {
                        this.setSecondPanelSize(builder.getSecondPanelSize(), this.secondPanelUnit);
                    }
                    this.alignment = builder.getAlignment();
                    this.alignmentTreshold = builder.getAlignmentTreshold();
                    this.splitterThickness = builder.getSplitterThickness();
                    this.splitter = new api.dom.DivEl("splitter");
                    this.firstPanel.setDoOffset(false);
                    this.secondPanel.setDoOffset(false);
                    this.appendChild(this.firstPanel);
                    this.appendChild(this.splitter);
                    this.appendChild(this.secondPanel);
                    this.mask = new api.ui.mask.DragMask(this);
                    this.appendChild(this.mask);
                    this.onRendered(function () { return _this.onRenderedDragHandler(); });
                    if (this.alignmentTreshold) {
                        ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                            _this.runWithAnimationDelayIfPresent(function () {
                                if (_this.requiresAlignment()) {
                                    _this.updateAlignment();
                                }
                            });
                        });
                    }
                    this.onShown(function (event) {
                        var splitPanelSize = _this.isHorizontal() ? _this.getEl().getHeight() : _this.getEl().getWidth();
                        api.util.assert(_this.firstPanelMinSize + _this.secondPanelMinSize <= splitPanelSize, "warning: total sum of first and second panel minimum sizes exceed total split panel size");
                        _this.updateAlignment();
                    });
                    ResponsiveManager.onAvailableSizeChanged(this);
                    ResponsiveManager.onAvailableSizeChanged(this.firstPanel);
                    ResponsiveManager.onAvailableSizeChanged(this.secondPanel);
                    this.onRemoved(function (event) {
                        ResponsiveManager.unAvailableSizeChanged(_this);
                        ResponsiveManager.unAvailableSizeChanged(_this.firstPanel);
                        ResponsiveManager.unAvailableSizeChanged(_this.secondPanel);
                    });
                }
                SplitPanel.prototype.onRenderedDragHandler = function () {
                    var _this = this;
                    var initialPos = 0;
                    this.ghostDragger = new api.dom.DivEl("ghost-dragger");
                    this.dragListener = function (e) {
                        if (_this.isHorizontal()) {
                            if (_this.splitterWithinBoundaries(initialPos - e.clientY)) {
                                _this.splitterPosition = e.clientY;
                                _this.ghostDragger.getEl().setTopPx(e.clientY - _this.getEl().getOffsetTop());
                            }
                        }
                        else {
                            if (_this.splitterWithinBoundaries(initialPos - e.clientX)) {
                                _this.splitterPosition = e.clientX;
                                _this.ghostDragger.getEl().setLeftPx(e.clientX - _this.getEl().getOffsetLeft());
                            }
                        }
                    };
                    this.splitter.onMouseDown(function (e) {
                        e.preventDefault();
                        if (_this.isHorizontal()) {
                            initialPos = e.clientY;
                        }
                        else {
                            initialPos = e.clientX;
                        }
                        _this.ghostDragger.insertBeforeEl(_this.splitter);
                        _this.startDrag();
                    });
                    this.onMouseUp(function (e) {
                        if (_this.ghostDragger.getHTMLElement().parentNode) {
                            _this.stopDrag(e);
                            _this.removeChild(_this.ghostDragger);
                        }
                    });
                };
                SplitPanel.prototype.startDrag = function () {
                    this.mask.show();
                    this.addClass("dragging");
                    this.onMouseMove(this.dragListener);
                    if (this.isHorizontal()) {
                        this.ghostDragger.getEl().setTopPx(this.splitter.getEl().getOffsetTopRelativeToParent()).setLeft(null);
                    }
                    else {
                        this.ghostDragger.getEl().setLeftPx(this.splitter.getEl().getOffsetLeftRelativeToParent()).setTop(null);
                    }
                };
                SplitPanel.prototype.stopDrag = function (e) {
                    this.mask.hide();
                    this.removeClass("dragging");
                    this.unMouseMove(this.dragListener);
                    var splitPanelEl = this.getEl();
                    var dragOffset = this.isHorizontal() ? this.splitterPosition - splitPanelEl.getOffsetTop() : this.splitterPosition - splitPanelEl.getOffsetLeft();
                    var splitPanelSize = this.isHorizontal() ? splitPanelEl.getHeightWithBorder() : splitPanelEl.getWidthWithBorder();
                    if (this.firstPanelUnit == 1 /* PERCENT */) {
                        this.firstPanelSize = (dragOffset / splitPanelSize) * 100;
                        this.setSecondPanelSize(100 - this.firstPanelSize, 1 /* PERCENT */);
                    }
                    else {
                        this.firstPanelSize = dragOffset;
                    }
                    this.distribute();
                };
                SplitPanel.prototype.splitterWithinBoundaries = function (offset) {
                    var firstPanelSize = this.isHorizontal() ? this.firstPanel.getEl().getHeight() : this.firstPanel.getEl().getWidth();
                    var secondPanelSize = this.isHorizontal() ? this.secondPanel.getEl().getHeight() : this.secondPanel.getEl().getWidth();
                    var newFirstPanelWidth = firstPanelSize - offset;
                    var newSecondPanelWidth = secondPanelSize + offset;
                    return (newFirstPanelWidth >= this.firstPanelMinSize) && (newSecondPanelWidth >= this.secondPanelMinSize);
                };
                SplitPanel.prototype.requiresAlignment = function () {
                    if (this.alignmentTreshold) {
                        var splitPanelWidth = this.getEl().getWidthWithMargin();
                        if (splitPanelWidth > this.alignmentTreshold && this.isHorizontal()) {
                            return true;
                        }
                        else if (splitPanelWidth < this.alignmentTreshold && !this.isHorizontal()) {
                            return true;
                        }
                    }
                    return false;
                };
                SplitPanel.prototype.updateAlignment = function () {
                    var splitPanelWidth = this.getEl().getWidthWithMargin();
                    if (splitPanelWidth > this.alignmentTreshold && this.isHorizontal()) {
                        this.alignment = 1 /* VERTICAL */;
                    }
                    else if (splitPanelWidth < this.alignmentTreshold && !this.isHorizontal()) {
                        this.alignment = 0 /* HORIZONTAL */;
                    }
                    if (this.isHorizontal()) {
                        this.removeClass("vertical");
                        this.addClass("horizontal");
                        this.firstPanel.getEl().setWidth(null);
                        this.secondPanel.getEl().setWidth(null);
                        this.splitter.getEl().setHeightPx(this.getSplitterThickness()).setWidth(null).setLeft(null);
                    }
                    else {
                        this.addClass("vertical");
                        this.removeClass("horizontal");
                        this.firstPanel.getEl().setHeight(null);
                        this.secondPanel.getEl().setHeight(null);
                        this.splitter.getEl().setWidthPx(this.getSplitterThickness()).setHeight(null);
                    }
                    this.distribute();
                };
                SplitPanel.prototype.setFirstPanelSize = function (size, unit) {
                    this.firstPanelSize = size;
                    this.secondPanelSize = -1;
                    if (unit) {
                        this.firstPanelUnit = unit;
                    }
                };
                SplitPanel.prototype.setFirstPanelIsFullScreen = function (fullScreen) {
                    this.firstPanelIsFullScreen = fullScreen;
                };
                SplitPanel.prototype.setSecondPanelSize = function (size, unit) {
                    this.secondPanelSize = size;
                    this.firstPanelSize = -1;
                    if (unit) {
                        this.secondPanelUnit = unit;
                    }
                };
                SplitPanel.prototype.saveFirstPanelSize = function () {
                    this.savedFirstPanelSize = this.firstPanelSize;
                    this.savedFirstPanelMinSize = this.firstPanelMinSize;
                    this.savedFirstPanelUnit = this.firstPanelUnit;
                };
                SplitPanel.prototype.loadFirstPanelSize = function () {
                    this.firstPanelSize = this.savedFirstPanelSize;
                    this.firstPanelMinSize = this.savedFirstPanelMinSize;
                    this.firstPanelUnit = this.savedFirstPanelUnit;
                    this.secondPanelSize = -1;
                };
                SplitPanel.prototype.saveFirstPanelSizeAndDistribute = function (size, minSize, unit) {
                    if (minSize === void 0) { minSize = -1; }
                    this.saveFirstPanelSize();
                    this.firstPanelSize = size < 0 ? this.firstPanelSize : size;
                    this.firstPanelUnit = minSize < 0 ? this.firstPanelUnit : minSize;
                    this.firstPanelMinSize = unit ? unit : this.firstPanelMinSize;
                    this.distribute();
                };
                SplitPanel.prototype.loadFirstPanelSizeAndDistribute = function () {
                    this.loadFirstPanelSize();
                    this.distribute();
                };
                SplitPanel.prototype.showSplitter = function () {
                    this.splitter.show();
                };
                SplitPanel.prototype.hideSplitter = function () {
                    this.splitter.hide();
                };
                SplitPanel.prototype.distribute = function () {
                    if (this.isHorizontal()) {
                        this.firstPanel.getEl().setHeight(this.getPanelSizeString(1)).setWidth(null);
                        this.secondPanel.getEl().setHeight(this.getPanelSizeString(2)).setWidth(null);
                        this.splitter.getEl().setHeightPx(this.getSplitterThickness()).setWidth(null).setLeft(null);
                        this.runWithAnimationDelayIfPresent(ResponsiveManager.fireResizeEvent);
                    }
                    else {
                        this.firstPanel.getEl().setWidth(this.getPanelSizeString(1)).setHeight(null);
                        this.secondPanel.getEl().setWidth(this.getPanelSizeString(2)).setHeight(null);
                        this.splitter.getEl().setWidthPx(this.getSplitterThickness()).setHeight(null);
                        this.runWithAnimationDelayIfPresent(ResponsiveManager.fireResizeEvent);
                        if (this.firstPanelUnit == 1 /* PERCENT */ && this.secondPanelUnit == 1 /* PERCENT */) {
                            var positionInPercentage = (this.firstPanelSize != -1) ? this.firstPanelSize : 100 - this.secondPanelSize;
                            this.splitter.getEl().setLeft("calc(" + positionInPercentage + "% - " + (this.getSplitterThickness() / 2) + "px)");
                        }
                        else {
                            this.splitter.getEl().setLeft(this.getPanelSizeString(1));
                        }
                    }
                };
                SplitPanel.prototype.runWithAnimationDelayIfPresent = function (callee) {
                    if (this.animationDelay) {
                        setTimeout(function () {
                            callee();
                        }, this.animationDelay);
                    }
                    else {
                        callee();
                    }
                };
                SplitPanel.prototype.isHorizontal = function () {
                    return this.alignment == 0 /* HORIZONTAL */;
                };
                SplitPanel.prototype.getPanelSizeString = function (panelNumber) {
                    api.util.assert((panelNumber == 1 || panelNumber == 2), "Panel number must be 1 or 2");
                    var size = (panelNumber == 1) ? this.firstPanelSize : this.secondPanelSize;
                    var otherPanelSize = (panelNumber == 1) ? this.secondPanelSize : this.firstPanelSize;
                    var unit = (panelNumber == 1) ? this.firstPanelUnit : this.secondPanelUnit;
                    var otherPanelUnit = (panelNumber == 1) ? this.secondPanelUnit : this.firstPanelUnit;
                    if ((panelNumber == 1 && this.isSecondPanelHidden()) || (panelNumber == 2 && this.isFirstPanelHidden())) {
                        return "100%";
                    }
                    else if ((panelNumber == 1 && this.isFirstPanelHidden()) || (panelNumber == 2 && this.isSecondPanelHidden())) {
                        return "0";
                    }
                    var result;
                    if (size != -1) {
                        if (unit == 0 /* PIXEL */) {
                            result = size - (this.getSplitterThickness() / 2) + "px";
                        }
                        else {
                            result = "calc(" + size + "%" + " - " + (this.getSplitterThickness() / 2) + "px)";
                        }
                    }
                    else {
                        if (otherPanelUnit == 0 /* PIXEL */) {
                            result = "calc(100% - " + (otherPanelSize + (this.getSplitterThickness() / 2)) + "px)";
                        }
                        else {
                            result = "calc(" + (100 - otherPanelSize) + "%" + " - " + (this.getSplitterThickness() / 2) + "px)";
                        }
                    }
                    return result;
                };
                SplitPanel.prototype.showFirstPanel = function () {
                    if (!this.firstPanelIsHidden) {
                        return;
                    }
                    if (this.firstPanelIsFullScreen) {
                        this.firstPanelSize = -1;
                    }
                    else {
                        this.firstPanelSize = this.hiddenFirstPanelPreviousSize;
                        this.splitterIsHidden = false;
                        this.splitter.show();
                    }
                    this.firstPanel.show();
                    this.firstPanelIsHidden = false;
                    this.distribute();
                };
                SplitPanel.prototype.showSecondPanel = function (showSplitter) {
                    if (showSplitter === void 0) { showSplitter = true; }
                    if (!this.secondPanelIsHidden) {
                        return;
                    }
                    this.splitterIsHidden = false;
                    if (showSplitter) {
                        this.splitter.show();
                    }
                    this.secondPanelSize = this.hiddenSecondPanelPreviousSize;
                    this.secondPanel.show();
                    this.secondPanelIsHidden = false;
                    this.distribute();
                };
                SplitPanel.prototype.hideFirstPanel = function () {
                    if (this.firstPanelIsHidden) {
                        return;
                    }
                    this.splitterIsHidden = true;
                    this.splitter.hide();
                    if (!this.firstPanelIsFullScreen) {
                        this.hiddenFirstPanelPreviousSize = this.firstPanelSize;
                    }
                    this.firstPanelSize = 0;
                    this.firstPanel.hide();
                    this.firstPanelIsHidden = true;
                    this.distribute();
                };
                SplitPanel.prototype.hideSecondPanel = function () {
                    if (this.secondPanelIsHidden) {
                        return;
                    }
                    this.secondPanel.hide();
                    this.foldSecondPanel();
                };
                SplitPanel.prototype.foldSecondPanel = function () {
                    this.splitterIsHidden = true;
                    this.splitter.hide();
                    this.hiddenSecondPanelPreviousSize = this.secondPanelSize;
                    this.secondPanelSize = 0;
                    this.secondPanelIsHidden = true;
                    this.distribute();
                };
                SplitPanel.prototype.getActiveWidthPxOfSecondPanel = function () {
                    if (this.secondPanelIsHidden) {
                        return this.hiddenSecondPanelPreviousSize;
                    }
                    else {
                        return this.secondPanel.getEl().getWidthWithBorder();
                    }
                };
                SplitPanel.prototype.setActiveWidthPxOfSecondPanel = function (value) {
                    if (this.secondPanelIsHidden) {
                        this.hiddenSecondPanelPreviousSize = value;
                    }
                    else {
                        this.secondPanelSize = value;
                    }
                    this.secondPanelUnit = 0 /* PIXEL */;
                };
                SplitPanel.prototype.isFirstPanelHidden = function () {
                    return this.firstPanelIsHidden;
                };
                SplitPanel.prototype.isSecondPanelHidden = function () {
                    return this.secondPanelIsHidden;
                };
                SplitPanel.prototype.getUnitString = function (panelNumber) {
                    api.util.assert((panelNumber == 1 || panelNumber == 2), "Panel number must be 1 or 2");
                    var unit = (panelNumber == 1) ? this.firstPanelUnit : this.secondPanelUnit;
                    if (unit == 0 /* PIXEL */) {
                        return "px";
                    }
                    else {
                        return "%";
                    }
                };
                SplitPanel.prototype.getSplitterThickness = function () {
                    return this.splitterIsHidden ? 0 : this.splitterThickness;
                };
                return SplitPanel;
            })(panel.Panel);
            panel.SplitPanel = SplitPanel;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (_panel) {
            var NavigatedDeckPanel = (function (_super) {
                __extends(NavigatedDeckPanel, _super);
                function NavigatedDeckPanel(navigator, className) {
                    var _this = this;
                    _super.call(this, className);
                    this.navigator = navigator;
                    navigator.onNavigationItemSelected(function (event) {
                        _this.showPanelByIndex(event.getItem().getIndex());
                    });
                }
                NavigatedDeckPanel.prototype.getSelectedNavigationItem = function () {
                    return this.navigator.getSelectedNavigationItem();
                };
                NavigatedDeckPanel.prototype.addNavigablePanel = function (item, panel, select) {
                    this.navigator.addNavigationItem(item);
                    var index = this.addPanel(panel);
                    if (select) {
                        this.selectPanelByIndex(index);
                    }
                    return index;
                };
                NavigatedDeckPanel.prototype.selectPanel = function (item) {
                    this.selectPanelByIndex(item.getIndex());
                };
                NavigatedDeckPanel.prototype.selectPanelByIndex = function (index) {
                    this.navigator.selectNavigationItem(index);
                };
                NavigatedDeckPanel.prototype.removeNavigablePanel = function (panel, checkCanRemovePanel) {
                    if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                    var index = this.removePanel(panel, checkCanRemovePanel);
                    if (index > -1) {
                        var navigationItem = this.navigator.getNavigationItem(index);
                        this.navigator.removeNavigationItem(navigationItem);
                    }
                    return index;
                };
                NavigatedDeckPanel.prototype.getNavigator = function () {
                    return this.navigator;
                };
                return NavigatedDeckPanel;
            })(_panel.DeckPanel);
            _panel.NavigatedDeckPanel = NavigatedDeckPanel;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (_panel) {
            var NavigatedPanelStrip = (function (_super) {
                __extends(NavigatedPanelStrip, _super);
                function NavigatedPanelStrip(navigator, scrollable, className) {
                    var _this = this;
                    _super.call(this, scrollable, className);
                    this.scrollIndex = -1;
                    this.focusIndex = -1;
                    this.focusVisible = false;
                    this.listenToScroll = true;
                    this.navigator = navigator;
                    navigator.onNavigationItemSelected(function (event) {
                        _this.listenToScroll = false;
                        _this.showPanelByIndex(event.getItem().getIndex());
                    });
                    this.onPanelShown(function (event) {
                        _this.listenToScroll = true;
                    });
                    this.getScrollable().onScroll(function (event) {
                        if (_this.listenToScroll) {
                            _this.updateScrolledNavigationItem();
                        }
                    });
                }
                NavigatedPanelStrip.prototype.setListenToScroll = function (listen) {
                    if (listen === void 0) { listen = true; }
                    this.listenToScroll = listen;
                };
                NavigatedPanelStrip.prototype.updateScrolledNavigationItem = function () {
                    var scrollTop = this.getScrollable().getHTMLElement().scrollTop;
                    var focusVisible = this.isFocusedPanelVisible(scrollTop);
                    var scrollIndex = this.getScrolledPanelIndex(scrollTop);
                    if (this.scrollIndex != scrollIndex || this.focusVisible != focusVisible) {
                        if (focusVisible) {
                            this.navigator.selectNavigationItem(this.focusIndex, true);
                        }
                        else {
                            this.navigator.selectNavigationItem(scrollIndex, true);
                        }
                        this.focusVisible = focusVisible;
                        this.scrollIndex = scrollIndex;
                    }
                };
                NavigatedPanelStrip.prototype.isFocusedPanelVisible = function (scrollTop) {
                    if (this.focusIndex < 0) {
                        return false;
                    }
                    var totalHeight = this.getScrollable().getEl().getHeight(), headerHeight = this.getFocusedHeaderHeight(this.focusIndex), panelEl = this.getPanel(this.focusIndex).getEl(), panelTop = panelEl.getOffsetToParent().top - this.getScrollOffset() - headerHeight, panelBottom = panelTop + panelEl.getHeight() - headerHeight;
                    return panelEl.isVisible() && ((panelTop <= 0 && panelBottom > 0) || (panelTop <= totalHeight && panelBottom > totalHeight));
                };
                NavigatedPanelStrip.prototype.getScrolledPanelIndex = function (scrollTop) {
                    var panelEl, panelTop, panelBottom;
                    if (scrollTop == 0) {
                        return 0;
                    }
                    for (var i = 0; i < this.getSize(); i++) {
                        panelEl = this.getPanel(i).getEl();
                        var headerHeight = this.getFocusedHeaderHeight(i);
                        if (panelEl.isVisible()) {
                            panelTop = scrollTop + panelEl.getOffsetToParent().top - this.getScrollOffset() - headerHeight;
                            panelBottom = panelTop + panelEl.getHeight();
                            if (scrollTop >= panelTop && scrollTop < panelBottom) {
                                return i;
                            }
                        }
                    }
                    return -1;
                };
                NavigatedPanelStrip.prototype.getSelectedNavigationItem = function () {
                    return this.navigator.getSelectedNavigationItem();
                };
                NavigatedPanelStrip.prototype.insertNavigablePanel = function (item, panel, header, index, select) {
                    var _this = this;
                    this.navigator.insertNavigationItem(item, index);
                    var panelHeader = select ? null : header;
                    _super.prototype.insertPanel.call(this, panel, index, panelHeader);
                    panel.onFocus(function (event) {
                        _this.navigator.selectNavigationItem(item.getIndex(), true);
                        _this.focusIndex = item.getIndex();
                    });
                    panel.onBlur(function (event) {
                        _this.focusIndex = -1;
                        _this.updateScrolledNavigationItem();
                    });
                    if (select) {
                        this.selectPanel(item);
                    }
                    return index;
                };
                NavigatedPanelStrip.prototype.addNavigablePanel = function (item, panel, header, select) {
                    return this.insertNavigablePanel(item, panel, header, this.getPanels().length, select);
                };
                NavigatedPanelStrip.prototype.selectPanel = function (item) {
                    this.selectPanelByIndex(item.getIndex());
                };
                NavigatedPanelStrip.prototype.selectPanelByIndex = function (index) {
                    this.navigator.selectNavigationItem(index);
                };
                NavigatedPanelStrip.prototype.removeNavigablePanel = function (panel, checkCanRemovePanel) {
                    if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                    var removedPanelIndex = _super.prototype.removePanel.call(this, panel, checkCanRemovePanel);
                    if (removedPanelIndex > -1) {
                        var navigationItem = this.navigator.getNavigationItem(removedPanelIndex);
                        this.navigator.removeNavigationItem(navigationItem);
                    }
                    return removedPanelIndex;
                };
                NavigatedPanelStrip.prototype.getFocusedHeaderHeight = function (curStrip) {
                    return this.getHeader(curStrip + 1) ? this.getHeader(curStrip + 1).getEl().getHeightWithBorder() : this.getHeader(curStrip).getEl().getHeightWithBorder();
                };
                return NavigatedPanelStrip;
            })(_panel.PanelStrip);
            _panel.NavigatedPanelStrip = NavigatedPanelStrip;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var panel;
        (function (_panel) {
            var TabBar = api.ui.tab.TabBar;
            var DockedPanel = (function (_super) {
                __extends(DockedPanel, _super);
                function DockedPanel() {
                    _super.call(this, "docked-panel");
                    this.items = [];
                    this.navigator = new TabBar();
                    this.deck = new _panel.NavigatedDeckPanel(this.navigator);
                    this.appendChild(this.navigator);
                    this.appendChild(this.deck);
                    this.setDoOffset(false);
                }
                DockedPanel.prototype.addItem = function (label, addLabelTitleAttribute, panel, select) {
                    var item = new api.ui.tab.TabBarItemBuilder().setLabel(label).setAddLabelTitleAttribute(addLabelTitleAttribute).build();
                    this.addItemArray(item);
                    this.deck.addNavigablePanel(item, panel, select || this.items.length == 1);
                    return this.deck.getPanelIndex(panel);
                };
                DockedPanel.prototype.selectPanel = function (panel) {
                    this.deck.selectPanelByIndex(this.deck.getPanelIndex(panel));
                };
                DockedPanel.prototype.getNavigator = function () {
                    return this.navigator;
                };
                DockedPanel.prototype.getDeck = function () {
                    return this.deck;
                };
                DockedPanel.prototype.addItemArray = function (item) {
                    this.items.push(item);
                };
                return DockedPanel;
            })(_panel.Panel);
            _panel.DockedPanel = DockedPanel;
        })(panel = ui.panel || (ui.panel = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var dialog;
        (function (dialog) {
            var DialogButton = (function (_super) {
                __extends(DialogButton, _super);
                function DialogButton(action) {
                    _super.call(this, action, false);
                    this.addClass("dialog-button");
                }
                return DialogButton;
            })(api.ui.button.ActionButton);
            dialog.DialogButton = DialogButton;
        })(dialog = ui.dialog || (ui.dialog = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var dialog;
        (function (dialog) {
            var ModalDialog = (function (_super) {
                __extends(ModalDialog, _super);
                function ModalDialog(config) {
                    var _this = this;
                    _super.call(this, "modal-dialog");
                    this.actions = [];
                    this.config = config;
                    var wrapper = new api.dom.DivEl("modal-dialog-content-wrapper");
                    this.appendChild(wrapper);
                    this.createDefaultCancelAction();
                    this.cancelButton = this.createCancelButton();
                    wrapper.appendChild(this.cancelButton);
                    this.title = this.config.title;
                    wrapper.appendChild(this.title);
                    this.contentPanel = new ModalDialogContentPanel();
                    wrapper.appendChild(this.contentPanel);
                    var push = new api.dom.DivEl("modal-dialog-content-push");
                    wrapper.appendChild(push);
                    var footer = new api.dom.DivEl("modal-dialog-footer");
                    this.appendChild(footer);
                    this.buttonRow = new ModalDialogButtonRow();
                    footer.appendChild(this.buttonRow);
                    this.mouseClickListener = function (event) {
                        if (_this.isVisible()) {
                            for (var element = event.target; element; element = element.parentNode) {
                                if (element == _this.getHTMLElement()) {
                                    return;
                                }
                            }
                            if (_this.cancelAction) {
                                _this.cancelAction.execute();
                            }
                        }
                    };
                    api.dom.Body.get().onMouseDown(this.mouseClickListener);
                    this.onRemoved(function () { return api.dom.Body.get().unMouseDown(_this.mouseClickListener); });
                    this.onAdded(function () { return api.dom.Body.get().onMouseDown(_this.mouseClickListener); });
                    this.onShown(function () { return api.ui.responsive.ResponsiveManager.fireResizeEvent(); });
                    this.responsiveItem = api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                        if (_this.isVisible()) {
                            _this.centerMyself();
                        }
                    });
                }
                ModalDialog.prototype.createDefaultCancelAction = function () {
                    var _this = this;
                    var cancelAction = new api.ui.Action("Cancel", "esc");
                    cancelAction.setIconClass("cancel-button-top");
                    cancelAction.setLabel("");
                    cancelAction.onExecuted(function () {
                        _this.close();
                    });
                    this.cancelAction = cancelAction;
                    this.actions.push(cancelAction);
                };
                ModalDialog.prototype.getCancelAction = function () {
                    return this.cancelAction;
                };
                ModalDialog.prototype.addCancelButtonToBottom = function () {
                    var _this = this;
                    var cancelAction = new api.ui.Action("Cancel", "esc");
                    cancelAction.setIconClass("cancel-button-bottom");
                    cancelAction.onExecuted(function () {
                        _this.close();
                    });
                    this.addAction(cancelAction);
                };
                ModalDialog.prototype.setTitle = function (value) {
                    this.title.setTitle(value);
                };
                ModalDialog.prototype.appendChildToContentPanel = function (child) {
                    this.contentPanel.appendChild(child);
                };
                ModalDialog.prototype.appendChildToTitle = function (child) {
                    this.title.appendChild(child);
                };
                ModalDialog.prototype.removeChildFromContentPanel = function (child) {
                    this.contentPanel.removeChild(child);
                };
                ModalDialog.prototype.addAction = function (action, useDefault, prepend) {
                    this.actions.push(action);
                    return this.buttonRow.addAction(action, useDefault, prepend);
                };
                ModalDialog.prototype.show = function () {
                    _super.prototype.show.call(this);
                    this.centerMyself();
                    this.buttonRow.focusDefaultAction();
                };
                ModalDialog.prototype.centerMyself = function () {
                    var el = this.getEl();
                    el.setMarginTop("-" + (el.getHeightWithBorder() / 2) + "px");
                    if (this.responsiveItem.isInRangeOrBigger(api.ui.responsive.ResponsiveRanges._540_720)) {
                        el.setMarginLeft("-" + (el.getWidthWithBorder() / 2) + "px");
                        el.addClass("centered_horizontally");
                    }
                    else {
                        el.setMarginLeft("0px");
                        el.removeClass("centered_horizontally");
                    }
                };
                ModalDialog.prototype.createCancelButton = function () {
                    return new dialog.DialogButton(this.getCancelAction());
                };
                ModalDialog.prototype.getResponsiveItem = function () {
                    return this.responsiveItem;
                };
                ModalDialog.prototype.getButtonRow = function () {
                    return this.buttonRow;
                };
                ModalDialog.prototype.getContentPanel = function () {
                    return this.contentPanel;
                };
                ModalDialog.prototype.hide = function () {
                    _super.prototype.hide.call(this);
                };
                ModalDialog.prototype.close = function () {
                    api.ui.mask.BodyMask.get().hide();
                    this.hide();
                    api.ui.KeyBindings.get().unshelveBindings();
                };
                ModalDialog.prototype.open = function () {
                    api.ui.mask.BodyMask.get().show();
                    api.ui.KeyBindings.get().shelveBindings();
                    this.show();
                    api.ui.KeyBindings.get().bindKeys(api.ui.Action.getKeyBindings(this.actions));
                };
                return ModalDialog;
            })(api.dom.DivEl);
            dialog.ModalDialog = ModalDialog;
            var ModalDialogHeader = (function (_super) {
                __extends(ModalDialogHeader, _super);
                function ModalDialogHeader(title) {
                    _super.call(this, "dialog-header");
                    this.titleEl = new api.dom.H2El('title');
                    this.titleEl.setHtml(title);
                    this.appendChild(this.titleEl);
                }
                ModalDialogHeader.prototype.setTitle = function (value) {
                    this.titleEl.setHtml(value);
                };
                return ModalDialogHeader;
            })(api.dom.DivEl);
            dialog.ModalDialogHeader = ModalDialogHeader;
            var ModalDialogContentPanel = (function (_super) {
                __extends(ModalDialogContentPanel, _super);
                function ModalDialogContentPanel() {
                    _super.call(this, "dialog-content");
                }
                return ModalDialogContentPanel;
            })(api.dom.DivEl);
            dialog.ModalDialogContentPanel = ModalDialogContentPanel;
            var ModalDialogButtonRow = (function (_super) {
                __extends(ModalDialogButtonRow, _super);
                function ModalDialogButtonRow() {
                    _super.call(this, "dialog-buttons");
                    this.buttonContainer = new api.dom.DivEl('button-container');
                    this.appendChild(this.buttonContainer);
                }
                ModalDialogButtonRow.prototype.addAction = function (action, useDefault, prepend) {
                    var button = new dialog.DialogButton(action);
                    if (useDefault) {
                        this.defaultButton = button;
                    }
                    if (prepend) {
                        this.buttonContainer.prependChild(button);
                    }
                    else {
                        this.buttonContainer.appendChild(button);
                    }
                    action.onPropertyChanged(function () {
                        button.setLabel(action.getLabel());
                        button.setEnabled(action.isEnabled());
                    });
                    return button;
                };
                ModalDialogButtonRow.prototype.focusDefaultAction = function () {
                    if (this.defaultButton) {
                        this.defaultButton.giveFocus();
                    }
                };
                return ModalDialogButtonRow;
            })(api.dom.DivEl);
            dialog.ModalDialogButtonRow = ModalDialogButtonRow;
        })(dialog = ui.dialog || (ui.dialog = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var dialog;
        (function (dialog) {
            var ConfirmationDialog = (function (_super) {
                __extends(ConfirmationDialog, _super);
                function ConfirmationDialog() {
                    var _this = this;
                    _super.call(this, {
                        title: new api.ui.dialog.ModalDialogHeader("Confirmation")
                    });
                    this.addClass("confirmation-dialog");
                    this.questionEl = new api.dom.DivEl("question");
                    this.appendChildToContentPanel(this.questionEl);
                    this.noAction = new api.ui.Action("No", "esc");
                    this.noAction.onExecuted(function (action) {
                        _this.close();
                        if (_this.noCallback) {
                            _this.noCallback();
                        }
                    });
                    this.yesAction = new api.ui.Action("Yes", "enter");
                    this.yesAction.onExecuted(function (action) {
                        _this.close();
                        if (_this.yesCallback) {
                            _this.yesCallback();
                        }
                    });
                    this.addAction(this.yesAction, true);
                    this.addAction(this.noAction);
                    api.dom.Body.get().appendChild(this);
                }
                ConfirmationDialog.get = function () {
                    if (!ConfirmationDialog.instance) {
                        ConfirmationDialog.instance = new ConfirmationDialog();
                    }
                    return ConfirmationDialog.instance;
                };
                ConfirmationDialog.prototype.setQuestion = function (question) {
                    this.questionEl.getEl().setInnerHtml(question);
                    return this;
                };
                ConfirmationDialog.prototype.setYesCallback = function (callback) {
                    this.yesCallback = callback;
                    return this;
                };
                ConfirmationDialog.prototype.setNoCallback = function (callback) {
                    this.noCallback = callback;
                    return this;
                };
                return ConfirmationDialog;
            })(dialog.ModalDialog);
            dialog.ConfirmationDialog = ConfirmationDialog;
        })(dialog = ui.dialog || (ui.dialog = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var form;
        (function (form) {
            var Fieldset = (function (_super) {
                __extends(Fieldset, _super);
                function Fieldset(legend) {
                    _super.call(this);
                    this.items = [];
                    this.focusListeners = [];
                    this.blurListeners = [];
                    this.validityChangedListeners = [];
                    if (legend) {
                        this.legend = new api.dom.LegendEl(legend);
                        this.appendChild(this.legend);
                    }
                }
                Fieldset.prototype.add = function (formItem) {
                    var _this = this;
                    formItem.onFocus(function (event) {
                        _this.notifyFocused(event);
                    });
                    formItem.onBlur(function (event) {
                        _this.notifyBlurred(event);
                    });
                    formItem.onValidityChanged(function (event) {
                        _this.notifyValidityChanged(event.isValid());
                    });
                    this.items.push(formItem);
                    this.appendChild(formItem);
                };
                Fieldset.prototype.removeItem = function (formItem) {
                    for (var i = 0; i < this.items.length; i++) {
                        if (this.items[i] === formItem) {
                            this.items.splice(i, 1);
                            this.removeChild(formItem);
                        }
                    }
                };
                Fieldset.prototype.validate = function (validationResult, markInvalid) {
                    this.items.forEach(function (item) {
                        item.validate(validationResult, markInvalid);
                    });
                };
                Fieldset.prototype.setFieldsetData = function (data) {
                    var input, inputValue;
                    this.items.forEach(function (item) {
                        input = item.getInput();
                        inputValue = data[input.getName()];
                        if (inputValue) {
                            input.setValue(inputValue);
                        }
                    });
                };
                Fieldset.prototype.getFieldsetData = function () {
                    var input, data = {};
                    this.items.forEach(function (item) {
                        input = item.getInput();
                        data[input.getName()] = input.getValue();
                    });
                    return data;
                };
                Fieldset.prototype.onFocus = function (listener) {
                    this.focusListeners.push(listener);
                };
                Fieldset.prototype.unFocus = function (listener) {
                    this.focusListeners = this.focusListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                Fieldset.prototype.onBlur = function (listener) {
                    this.blurListeners.push(listener);
                };
                Fieldset.prototype.unBlur = function (listener) {
                    this.blurListeners = this.blurListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                Fieldset.prototype.notifyFocused = function (event) {
                    this.focusListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                Fieldset.prototype.notifyBlurred = function (event) {
                    this.blurListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                Fieldset.prototype.onValidityChanged = function (listener) {
                    this.validityChangedListeners.push(listener);
                };
                Fieldset.prototype.unValidityChanged = function (listener) {
                    this.validityChangedListeners = this.validityChangedListeners.filter(function (curr) {
                        return curr != listener;
                    });
                };
                Fieldset.prototype.notifyValidityChanged = function (valid) {
                    var _this = this;
                    this.validityChangedListeners.forEach(function (listener) {
                        listener.call(_this, new api.ValidityChangedEvent(valid));
                    });
                };
                return Fieldset;
            })(api.dom.FieldsetEl);
            form.Fieldset = Fieldset;
        })(form = ui.form || (ui.form = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var form;
        (function (form) {
            var Form = (function (_super) {
                __extends(Form, _super);
                function Form(className) {
                    _super.call(this, className);
                    this.fieldsets = [];
                    this.focusListeners = [];
                    this.blurListeners = [];
                    this.validityChangedListeners = [];
                    this.addClass("form-view");
                    this.preventSubmit();
                }
                Form.prototype.add = function (fieldset) {
                    var _this = this;
                    fieldset.onFocus(function (event) {
                        _this.notifyFocused(event);
                    });
                    fieldset.onBlur(function (event) {
                        _this.notifyBlurred(event);
                    });
                    fieldset.onValidityChanged(function (event) {
                        _this.notifyValidityChanged(event.isValid());
                    });
                    this.fieldsets.push(fieldset);
                    this.appendChild(fieldset);
                    return this;
                };
                Form.prototype.validate = function (markInvalid) {
                    var validationResult = new form.ValidationResult();
                    this.fieldsets.forEach(function (fieldset) {
                        fieldset.validate(validationResult, markInvalid);
                    });
                    return validationResult;
                };
                Form.prototype.setFormData = function (data) {
                    this.fieldsets.forEach(function (fieldset) {
                        fieldset.setFieldsetData(data);
                    });
                };
                Form.prototype.getFormData = function () {
                    var data = {};
                    var fieldsetData;
                    this.fieldsets.forEach(function (fieldset) {
                        fieldsetData = fieldset.getFieldsetData();
                        for (var property in fieldsetData) {
                            if (fieldsetData.hasOwnProperty(property)) {
                                data[property] = fieldsetData[property];
                            }
                        }
                    });
                    return data;
                };
                Form.prototype.onFocus = function (listener) {
                    this.focusListeners.push(listener);
                };
                Form.prototype.unFocus = function (listener) {
                    this.focusListeners = this.focusListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                Form.prototype.onBlur = function (listener) {
                    this.blurListeners.push(listener);
                };
                Form.prototype.unBlur = function (listener) {
                    this.blurListeners = this.blurListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                Form.prototype.notifyFocused = function (event) {
                    this.focusListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                Form.prototype.notifyBlurred = function (event) {
                    this.blurListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                Form.prototype.onValidityChanged = function (listener) {
                    this.validityChangedListeners.push(listener);
                };
                Form.prototype.unValidityChanged = function (listener) {
                    this.validityChangedListeners = this.validityChangedListeners.filter(function (curr) {
                        return curr != listener;
                    });
                };
                Form.prototype.notifyValidityChanged = function (valid) {
                    var _this = this;
                    this.validityChangedListeners.forEach(function (listener) {
                        listener.call(_this, new api.ValidityChangedEvent(valid));
                    });
                };
                return Form;
            })(api.dom.FormEl);
            form.Form = Form;
        })(form = ui.form || (ui.form = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var form;
        (function (form) {
            var FormItem = (function (_super) {
                __extends(FormItem, _super);
                function FormItem(builder) {
                    var _this = this;
                    _super.call(this, "input-view");
                    this.invalidClass = "invalid";
                    this.focusListeners = [];
                    this.blurListeners = [];
                    this.error = new api.dom.SpanEl("error");
                    this.appendChild(this.error);
                    this.input = builder.getInput();
                    this.input.onFocus(function (event) {
                        _this.notifyFocused(event);
                    });
                    this.input.onBlur(function (event) {
                        _this.notifyBlurred(event);
                    });
                    if (builder.getLabel()) {
                        this.label = new api.dom.LabelEl(builder.getLabel(), this.input);
                        if (form.Validators.required == builder.getValidator()) {
                            this.label.addClass("required");
                        }
                        this.appendChild(this.label);
                    }
                    this.appendChild(this.input);
                    if (builder.getValidator()) {
                        this.validator = builder.getValidator();
                    }
                }
                FormItem.prototype.getLabel = function () {
                    return this.label;
                };
                FormItem.prototype.getInput = function () {
                    return this.input;
                };
                FormItem.prototype.getValidator = function () {
                    return this.validator;
                };
                FormItem.prototype.validate = function (validationResult, markInvalid) {
                    if (this.validator) {
                        var validationMessage = this.validator(this.input);
                        if (validationMessage) {
                            validationResult.addError(new form.ValidationError(this, validationMessage));
                        }
                        if (markInvalid) {
                            var validityChanged = false;
                            if (validationMessage) {
                                this.addClass(this.invalidClass);
                                validityChanged = (validationMessage !== this.getError());
                            }
                            else {
                                this.removeClass(this.invalidClass);
                                validityChanged = !api.util.StringHelper.isBlank(this.getError());
                            }
                            this.error.setHtml(validationMessage || api.util.StringHelper.EMPTY_STRING);
                            if (validityChanged) {
                                this.notifyValidityChanged(api.util.StringHelper.isBlank(validationMessage));
                            }
                        }
                    }
                };
                FormItem.prototype.getError = function () {
                    return this.error.getHtml();
                };
                FormItem.prototype.onValidityChanged = function (listener) {
                    this.input.onValidityChanged(listener);
                };
                FormItem.prototype.unValidityChanged = function (listener) {
                    this.input.unValidityChanged(listener);
                };
                FormItem.prototype.notifyValidityChanged = function (valid) {
                    this.input.notifyValidityChanged(valid);
                };
                FormItem.prototype.onFocus = function (listener) {
                    this.focusListeners.push(listener);
                };
                FormItem.prototype.unFocus = function (listener) {
                    this.focusListeners = this.focusListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                FormItem.prototype.onBlur = function (listener) {
                    this.blurListeners.push(listener);
                };
                FormItem.prototype.unBlur = function (listener) {
                    this.blurListeners = this.blurListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                FormItem.prototype.notifyFocused = function (event) {
                    this.focusListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                FormItem.prototype.notifyBlurred = function (event) {
                    this.blurListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return FormItem;
            })(api.dom.DivEl);
            form.FormItem = FormItem;
            var FormItemBuilder = (function () {
                function FormItemBuilder(input) {
                    if (!input) {
                        throw new Error("Input can't be null.");
                    }
                    this.input = input;
                }
                FormItemBuilder.prototype.build = function () {
                    return new FormItem(this);
                };
                FormItemBuilder.prototype.getInput = function () {
                    return this.input;
                };
                FormItemBuilder.prototype.setLabel = function (label) {
                    this.label = label;
                    return this;
                };
                FormItemBuilder.prototype.getLabel = function () {
                    return this.label;
                };
                FormItemBuilder.prototype.setValidator = function (validator) {
                    this.validator = validator;
                    return this;
                };
                FormItemBuilder.prototype.getValidator = function () {
                    return this.validator;
                };
                return FormItemBuilder;
            })();
            form.FormItemBuilder = FormItemBuilder;
        })(form = ui.form || (ui.form = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var form;
        (function (form) {
            var Validators = (function () {
                function Validators() {
                }
                Validators.required = function (input) {
                    var value = input.getValue();
                    return api.util.StringHelper.isBlank(value) ? "This field is required" : undefined;
                };
                Validators.validEmail = function (input) {
                    var regexEmail = /\w+([-+.']\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*/;
                    var value = input.getValue();
                    return !regexEmail.test(value) ? "Invalid email address" : undefined;
                };
                return Validators;
            })();
            form.Validators = Validators;
        })(form = ui.form || (ui.form = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var form;
        (function (form) {
            var ValidationResult = (function () {
                function ValidationResult() {
                    this.valid = true;
                    this.errors = [];
                }
                ValidationResult.prototype.addError = function (error) {
                    this.errors.push(error);
                    if (this.valid) {
                        this.valid = false;
                    }
                };
                ValidationResult.prototype.isValid = function () {
                    return this.valid;
                };
                ValidationResult.prototype.getErrors = function () {
                    return this.errors;
                };
                return ValidationResult;
            })();
            form.ValidationResult = ValidationResult;
            var ValidationError = (function () {
                function ValidationError(formItem, message) {
                    this.formItem = formItem;
                    this.message = message;
                }
                ValidationError.prototype.getFormItem = function () {
                    return this.formItem;
                };
                ValidationError.prototype.getMessage = function () {
                    return this.message;
                };
                return ValidationError;
            })();
            form.ValidationError = ValidationError;
        })(form = ui.form || (ui.form = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var grid;
        (function (grid) {
            var ResponsiveManager = api.ui.responsive.ResponsiveManager;
            var Grid = (function (_super) {
                __extends(Grid, _super);
                function Grid(dataView, columns, options) {
                    var _this = this;
                    _super.call(this, "grid");
                    this.defaultHeight = 400;
                    this.defaultWidth = 800;
                    this.defaultAutoRenderGridOnDataChanges = true;
                    options = new grid.GridOptionsBuilder(options).build();
                    if (options.isHideColumnHeaders()) {
                        this.addClass("no-header");
                    }
                    this.checkboxSelectorPlugin = null;
                    this.checkableRows = options.isCheckableRows() || false;
                    this.dragAndDrop = options.isDragAndDrop() || false;
                    if (this.checkableRows) {
                        this.checkboxSelectorPlugin = new Slick.CheckboxSelectColumn({
                            cssClass: "slick-cell-checkboxsel",
                            width: 40
                        });
                        columns.unshift(this.checkboxSelectorPlugin.getColumnDefinition());
                    }
                    if (this.dragAndDrop) {
                        this.rowManagerPlugin = new Slick.RowMoveManager({
                            cancelEditOnDrag: true
                        });
                    }
                    this.getEl().setHeight((options.getHeight() || this.defaultHeight) + "px");
                    this.getEl().setWidth((options.getWidth() || this.defaultWidth) + "px");
                    this.dataView = dataView;
                    this.slickGrid = new Slick.Grid(this.getHTMLElement(), dataView.slick(), columns, options);
                    if (options.isAutoRenderGridOnDataChanges() || (options.isAutoRenderGridOnDataChanges() == undefined && this.defaultAutoRenderGridOnDataChanges)) {
                        this.autoRenderGridOnDataChanges(this.dataView);
                    }
                    if (this.checkboxSelectorPlugin != null) {
                        this.slickGrid.registerPlugin(this.checkboxSelectorPlugin);
                    }
                    if (this.rowManagerPlugin != null) {
                        this.slickGrid.registerPlugin(this.rowManagerPlugin);
                    }
                    this.onRemoved(function (event) {
                        ResponsiveManager.unAvailableSizeChanged(_this);
                    });
                    this.dataView.setItems([], options.getDataIdProperty());
                }
                Grid.prototype.mask = function () {
                    var _this = this;
                    if (this.isVisible()) {
                        if (this.loadMask) {
                            this.loadMask.show();
                        }
                        else {
                            if (this.getParentElement()) {
                                this.createLoadMask();
                                this.loadMask.show();
                            }
                            else {
                                this.onAdded(function () {
                                    _this.createLoadMask();
                                });
                            }
                        }
                    }
                };
                Grid.prototype.unmask = function () {
                    if (this.loadMask) {
                        this.loadMask.hide();
                    }
                };
                Grid.prototype.autoRenderGridOnDataChanges = function (dataView) {
                    var _this = this;
                    dataView.onRowCountChanged(function (eventData, args) {
                        _this.updateRowCount();
                        _this.render();
                    });
                    dataView.onRowsChanged(function (eventData, args) {
                        _this.invalidateRows(args.rows);
                        _this.render();
                    });
                };
                Grid.prototype.createLoadMask = function () {
                    this.loadMask = new api.ui.mask.LoadMask(this);
                    this.getParentElement().appendChild(this.loadMask);
                };
                Grid.prototype.setSelectionModel = function (selectionModel) {
                    this.slickGrid.setSelectionModel(selectionModel);
                };
                Grid.prototype.getDataView = function () {
                    return this.dataView;
                };
                Grid.prototype.setColumns = function (columns) {
                    if (this.checkboxSelectorPlugin) {
                        columns.unshift(this.checkboxSelectorPlugin.getColumnDefinition());
                    }
                    this.slickGrid.setColumns(columns);
                };
                Grid.prototype.getColumns = function () {
                    return this.slickGrid.getColumns();
                };
                Grid.prototype.setFilter = function (f) {
                    this.dataView.setFilter(f);
                };
                Grid.prototype.setOptions = function (options) {
                    this.slickGrid.setOptions(options);
                };
                Grid.prototype.getOptions = function () {
                    return this.slickGrid.getOptions();
                };
                Grid.prototype.getCheckboxSelectorPlugin = function () {
                    return this.checkboxSelectorPlugin;
                };
                Grid.prototype.registerPlugin = function (plugin) {
                    this.slickGrid.registerPlugin(plugin);
                };
                Grid.prototype.unregisterPlugin = function (plugin) {
                    this.slickGrid.unregisterPlugin(plugin);
                };
                Grid.prototype.render = function () {
                    this.slickGrid.render();
                    _super.prototype.render.call(this);
                };
                Grid.prototype.renderGrid = function () {
                    this.slickGrid.render();
                };
                Grid.prototype.resizeCanvas = function () {
                    this.slickGrid.resizeCanvas();
                };
                Grid.prototype.updateRowCount = function () {
                    this.slickGrid.updateRowCount();
                };
                Grid.prototype.invalidateRows = function (rows) {
                    this.slickGrid.invalidateRows(rows);
                };
                Grid.prototype.invalidate = function () {
                    this.slickGrid.invalidate();
                };
                Grid.prototype.syncGridSelection = function (preserveHidden) {
                    this.dataView.syncGridSelection(this.slickGrid, preserveHidden);
                };
                Grid.prototype.focus = function () {
                    this.slickGrid.focus();
                };
                Grid.prototype.setOnClick = function (callback) {
                    this.slickGrid.onClick.subscribe(function (event, data) {
                        event.stopPropagation();
                        callback(event, data);
                    });
                };
                Grid.prototype.setOnKeyDown = function (callback) {
                    this.slickGrid.onKeyDown.subscribe(function (event) {
                        event.stopPropagation();
                        callback(event);
                    });
                };
                Grid.prototype.getSelectedRows = function () {
                    return this.slickGrid.getSelectedRows();
                };
                Grid.prototype.getSelectedRowItems = function () {
                    var _this = this;
                    var rowItems = [];
                    var rows = this.getSelectedRows();
                    rows.forEach(function (rowIndex) {
                        rowItems.push(_this.dataView.getItem(rowIndex));
                    });
                    return rowItems;
                };
                Grid.prototype.setSelectedRows = function (rows) {
                    this.slickGrid.setSelectedRows(rows);
                };
                Grid.prototype.selectRow = function (row) {
                    var rows = this.getSelectedRows();
                    if (rows.length > 1 || (rows.length < 2 && rows.indexOf(row) < 0)) {
                        this.slickGrid.setSelectedRows([row]);
                    }
                };
                Grid.prototype.addSelectedRow = function (row) {
                    var rows = this.getSelectedRows();
                    if (rows.indexOf(row) < 0) {
                        rows.push(row);
                        this.setSelectedRows(rows);
                    }
                };
                Grid.prototype.addSelectedRows = function (rowsToAdd) {
                    var rows = this.getSelectedRows();
                    rowsToAdd.forEach(function (row) {
                        if (rows.indexOf(row) < 0) {
                            rows.push(row);
                        }
                    });
                    this.setSelectedRows(rows);
                };
                Grid.prototype.toggleRow = function (row) {
                    var rows = this.getSelectedRows(), index = rows.indexOf(row);
                    if (index < 0) {
                        rows.push(row);
                        rows.sort(function (a, b) {
                            return a - b;
                        });
                    }
                    else {
                        rows.splice(index, 1);
                    }
                    this.slickGrid.setSelectedRows(rows);
                    return index;
                };
                Grid.prototype.isRowSelected = function (row) {
                    var rows = this.getSelectedRows(), index = rows.indexOf(row);
                    return index >= 0;
                };
                Grid.prototype.selectAll = function () {
                    var rows = [];
                    for (var i = 0; i < this.slickGrid.getDataLength(); i++) {
                        rows.push(i);
                    }
                    this.setSelectedRows(rows);
                };
                Grid.prototype.clearSelection = function () {
                    this.setSelectedRows([]);
                };
                Grid.prototype.isAllSelected = function () {
                    return this.slickGrid.getDataLength() === this.getSelectedRows().length;
                };
                Grid.prototype.resetActiveCell = function () {
                    if (this.slickGrid.getActiveCell()) {
                        this.slickGrid.resetActiveCell();
                    }
                };
                Grid.prototype.getCellFromEvent = function (e) {
                    return this.slickGrid.getCellFromEvent(e);
                };
                Grid.prototype.moveSelectedUp = function () {
                    if (this.slickGrid.getDataLength() > 0) {
                        var selected = this.getSelectedRows().sort(function (a, b) {
                            return a - b;
                        });
                        var row = selected.length >= 1 ? selected[0] - 1 : -1;
                        if (selected.length === 1) {
                            if (row >= 0) {
                                this.selectRow(row);
                                return row;
                            }
                            else {
                                this.clearSelection();
                                return 0;
                            }
                        }
                        else if (selected.length > 1) {
                            row = Math.max(row, 0);
                            this.selectRow(row);
                            return row;
                        }
                    }
                    return -1;
                };
                Grid.prototype.moveSelectedDown = function () {
                    if (this.slickGrid.getDataLength() > 0) {
                        var selected = this.getSelectedRows().sort(function (a, b) {
                            return a - b;
                        });
                        var row = selected.length >= 1 ? Math.min(selected[selected.length - 1] + 1, this.slickGrid.getDataLength() - 1) : 0;
                        this.selectRow(row);
                        return row;
                    }
                    return -1;
                };
                Grid.prototype.addSelectedUp = function () {
                    if (this.slickGrid.getDataLength() > 0) {
                        var selected = this.getSelectedRows().sort(function (a, b) {
                            return a - b;
                        });
                        if (selected.length > 0 && (selected[0] - 1) >= 0) {
                            var row = selected[0] - 1;
                            selected.push(row);
                            selected = selected.sort(function (a, b) {
                                return a - b;
                            });
                            this.setSelectedRows(selected);
                            return row;
                        }
                    }
                    return -1;
                };
                Grid.prototype.addSelectedDown = function () {
                    if (this.slickGrid.getDataLength() > 0) {
                        var selected = this.getSelectedRows().sort(function (a, b) {
                            return a - b;
                        });
                        if (selected.length > 0 && (selected[selected.length - 1] + 1) < this.slickGrid.getDataLength()) {
                            var row = selected[selected.length - 1] + 1;
                            selected.push(row);
                            this.setSelectedRows(selected);
                            return row;
                        }
                        else if (selected.length === 0) {
                            this.moveSelectedDown();
                            return 0;
                        }
                    }
                    return -1;
                };
                Grid.prototype.navigateUp = function () {
                    this.slickGrid.navigateUp();
                };
                Grid.prototype.navigateDown = function () {
                    this.slickGrid.navigateDown();
                };
                Grid.prototype.getActiveCell = function () {
                    return this.slickGrid.getActiveCell();
                };
                Grid.prototype.setActiveCell = function (row, cell) {
                    this.slickGrid.setActiveCell(row, cell);
                };
                Grid.prototype.setCellCssStyles = function (key, hash) {
                    this.slickGrid.setCellCssStyles(key, hash);
                };
                Grid.prototype.removeCellCssStyles = function (key) {
                    this.slickGrid.removeCellCssStyles(key);
                };
                Grid.prototype.getCellCssStyles = function (key) {
                    return this.slickGrid.getCellCssStyles(key);
                };
                Grid.prototype.getCanvasNode = function () {
                    return this.slickGrid.getCanvasNode();
                };
                Grid.prototype.getGridPosition = function () {
                    return this.slickGrid.getGridPosition();
                };
                Grid.prototype.getRenderedRange = function (viewportTop, viewportLeft) {
                    return this.slickGrid.getRenderedRange(viewportTop, viewportLeft);
                };
                Grid.prototype.getViewport = function (viewportTop, viewportLeft) {
                    return this.slickGrid.getViewport(viewportTop, viewportLeft);
                };
                Grid.prototype.subscribeOnSelectedRowsChanged = function (callback) {
                    this.slickGrid.onSelectedRowsChanged.subscribe(callback);
                };
                Grid.prototype.subscribeOnClick = function (callback) {
                    this.slickGrid.onClick.subscribe(callback);
                };
                Grid.prototype.unsubscribeOnClick = function (callback) {
                    this.slickGrid.onClick.unsubscribe(callback);
                };
                Grid.prototype.subscribeOnDblClick = function (callback) {
                    this.slickGrid.onDblClick.subscribe(callback);
                };
                Grid.prototype.subscribeOnContextMenu = function (callback) {
                    this.slickGrid.onContextMenu.subscribe(callback);
                };
                Grid.prototype.subscribeOnDrag = function (callback) {
                    this.slickGrid.onDrag.subscribe(callback);
                };
                Grid.prototype.subscribeOnDragInit = function (callback) {
                    this.slickGrid.onDragInit.subscribe(callback);
                };
                Grid.prototype.subscribeOnDragEnd = function (callback) {
                    this.slickGrid.onDragEnd.subscribe(callback);
                };
                Grid.prototype.subscribeBeforeMoveRows = function (callback) {
                    if (this.rowManagerPlugin) {
                        this.rowManagerPlugin.onBeforeMoveRows.subscribe(callback);
                    }
                };
                Grid.prototype.subscribeMoveRows = function (callback) {
                    if (this.rowManagerPlugin) {
                        this.rowManagerPlugin.onMoveRows.subscribe(callback);
                    }
                };
                Grid.prototype.subscribeOnScroll = function (callback) {
                    if (this.getHTMLElement().addEventListener) {
                        this.getHTMLElement().addEventListener('DOMMouseScroll', callback, false);
                        this.getHTMLElement().addEventListener('mousewheel', callback, false);
                    }
                    else {
                        this.getHTMLElement().attachEvent("onmousewheel", callback);
                    }
                };
                return Grid;
            })(api.dom.DivEl);
            grid.Grid = Grid;
        })(grid = ui.grid || (ui.grid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var grid;
        (function (grid) {
            var GridColumnBuilder = (function () {
                function GridColumnBuilder(source) {
                    if (source) {
                        this.asyncPostRender = source.getAsyncPostRender();
                        this.behavior = source.getBehavior();
                        this.cannotTriggerInsert = source.isCannotTriggerInsert();
                        this.cssClass = source.getCssClass();
                        this.defaultSortAsc = source.isDefaultSortAsc();
                        this.editor = source.getEditor();
                        this.field = source.getField();
                        this.focusable = source.isFocusable();
                        this.formatter = source.getFormatter();
                        this.headerCssClass = source.getHeaderCssClass();
                        this.id = source.getId();
                        this.maxWidth = source.getMaxWidth();
                        this.minWidth = source.getMinWidth();
                        this.name = source.getName();
                        this.rerenderOnResize = source.isRerenderOnResize();
                        this.resizable = source.isResizable();
                        this.selectable = source.isSelectable();
                        this.sortable = source.isSortable();
                        this.toolTip = source.getToolTip();
                        this.width = source.getWidth();
                    }
                }
                GridColumnBuilder.prototype.setAsyncPostRender = function (asyncPostRender) {
                    this.asyncPostRender = asyncPostRender;
                    return this;
                };
                GridColumnBuilder.prototype.setBehavior = function (behavior) {
                    this.behavior = behavior;
                    return this;
                };
                GridColumnBuilder.prototype.setCannotTriggerInsert = function (cannotTriggerInsert) {
                    this.cannotTriggerInsert = cannotTriggerInsert;
                    return this;
                };
                GridColumnBuilder.prototype.setCssClass = function (cssClass) {
                    this.cssClass = cssClass;
                    return this;
                };
                GridColumnBuilder.prototype.setDefaultSortAsc = function (defaultSortAsc) {
                    this.defaultSortAsc = defaultSortAsc;
                    return this;
                };
                GridColumnBuilder.prototype.setEditor = function (editor) {
                    this.editor = editor;
                    return this;
                };
                GridColumnBuilder.prototype.setField = function (field) {
                    this.field = field;
                    return this;
                };
                GridColumnBuilder.prototype.setFocusable = function (focusable) {
                    this.focusable = focusable;
                    return this;
                };
                GridColumnBuilder.prototype.setFormatter = function (formatter) {
                    this.formatter = formatter;
                    return this;
                };
                GridColumnBuilder.prototype.setHeaderCssClass = function (headerCssClass) {
                    this.headerCssClass = headerCssClass;
                    return this;
                };
                GridColumnBuilder.prototype.setId = function (id) {
                    this.id = id;
                    return this;
                };
                GridColumnBuilder.prototype.setMaxWidth = function (maxWidth) {
                    this.maxWidth = maxWidth;
                    return this;
                };
                GridColumnBuilder.prototype.setMinWidth = function (minWidth) {
                    this.minWidth = minWidth;
                    return this;
                };
                GridColumnBuilder.prototype.setName = function (name) {
                    this.name = name;
                    return this;
                };
                GridColumnBuilder.prototype.setRerenderOnResize = function (rerenderOnResize) {
                    this.rerenderOnResize = rerenderOnResize;
                    return this;
                };
                GridColumnBuilder.prototype.setResizable = function (resizable) {
                    this.resizable = resizable;
                    return this;
                };
                GridColumnBuilder.prototype.setSelectable = function (selectable) {
                    this.selectable = selectable;
                    return this;
                };
                GridColumnBuilder.prototype.setSortable = function (sortable) {
                    this.sortable = sortable;
                    return this;
                };
                GridColumnBuilder.prototype.setToolTip = function (toolTip) {
                    this.toolTip = toolTip;
                    return this;
                };
                GridColumnBuilder.prototype.setWidth = function (width) {
                    this.width = width;
                    return this;
                };
                GridColumnBuilder.prototype.build = function () {
                    return new GridColumn(this);
                };
                return GridColumnBuilder;
            })();
            grid.GridColumnBuilder = GridColumnBuilder;
            var GridColumn = (function () {
                function GridColumn(builder) {
                    this.asyncPostRender = builder.asyncPostRender;
                    this.behavior = builder.behavior;
                    this.cannotTriggerInsert = builder.cannotTriggerInsert;
                    this.cssClass = builder.cssClass;
                    this.defaultSortAsc = builder.defaultSortAsc;
                    this.editor = builder.editor;
                    this.field = builder.field;
                    this.focusable = builder.focusable;
                    this.formatter = builder.formatter;
                    this.headerCssClass = builder.headerCssClass;
                    this.id = builder.id;
                    this.maxWidth = builder.maxWidth;
                    this.minWidth = builder.minWidth;
                    this.name = builder.name;
                    this.rerenderOnResize = builder.rerenderOnResize;
                    this.resizable = builder.resizable;
                    this.selectable = builder.selectable;
                    this.sortable = builder.sortable;
                    this.toolTip = builder.toolTip;
                    this.width = builder.width;
                }
                GridColumn.prototype.getAsyncPostRender = function () {
                    return this.asyncPostRender;
                };
                GridColumn.prototype.getBehavior = function () {
                    return this.behavior;
                };
                GridColumn.prototype.isCannotTriggerInsert = function () {
                    return this.cannotTriggerInsert;
                };
                GridColumn.prototype.getCssClass = function () {
                    return this.cssClass;
                };
                GridColumn.prototype.isDefaultSortAsc = function () {
                    return this.defaultSortAsc;
                };
                GridColumn.prototype.getEditor = function () {
                    return this.editor;
                };
                GridColumn.prototype.getField = function () {
                    return this.field;
                };
                GridColumn.prototype.isFocusable = function () {
                    return this.focusable;
                };
                GridColumn.prototype.getFormatter = function () {
                    return this.formatter;
                };
                GridColumn.prototype.getHeaderCssClass = function () {
                    return this.headerCssClass;
                };
                GridColumn.prototype.getId = function () {
                    return this.id;
                };
                GridColumn.prototype.getMaxWidth = function () {
                    return this.maxWidth;
                };
                GridColumn.prototype.getMinWidth = function () {
                    return this.minWidth;
                };
                GridColumn.prototype.getName = function () {
                    return this.name;
                };
                GridColumn.prototype.isRerenderOnResize = function () {
                    return this.rerenderOnResize;
                };
                GridColumn.prototype.isResizable = function () {
                    return this.resizable;
                };
                GridColumn.prototype.isSelectable = function () {
                    return this.selectable;
                };
                GridColumn.prototype.isSortable = function () {
                    return this.sortable;
                };
                GridColumn.prototype.getToolTip = function () {
                    return this.toolTip;
                };
                GridColumn.prototype.getWidth = function () {
                    return this.width;
                };
                GridColumn.prototype.setAsyncPostRender = function (asyncPostRender) {
                    this.asyncPostRender = asyncPostRender;
                    return this;
                };
                GridColumn.prototype.setBehavior = function (behavior) {
                    this.behavior = behavior;
                    return this;
                };
                GridColumn.prototype.setCannotTriggerInsert = function (cannotTriggerInsert) {
                    this.cannotTriggerInsert = cannotTriggerInsert;
                    return this;
                };
                GridColumn.prototype.setCssClass = function (cssClass) {
                    this.cssClass = cssClass;
                    return this;
                };
                GridColumn.prototype.setDefaultSortAsc = function (defaultSortAsc) {
                    this.defaultSortAsc = defaultSortAsc;
                    return this;
                };
                GridColumn.prototype.setEditor = function (editor) {
                    this.editor = editor;
                    return this;
                };
                GridColumn.prototype.setField = function (field) {
                    this.field = field;
                    return this;
                };
                GridColumn.prototype.setFocusable = function (focusable) {
                    this.focusable = focusable;
                    return this;
                };
                GridColumn.prototype.setFormatter = function (formatter) {
                    this.formatter = formatter;
                    return this;
                };
                GridColumn.prototype.setHeaderCssClass = function (headerCssClass) {
                    this.headerCssClass = headerCssClass;
                    return this;
                };
                GridColumn.prototype.setId = function (id) {
                    this.id = id;
                    return this;
                };
                GridColumn.prototype.setMaxWidth = function (maxWidth) {
                    this.maxWidth = maxWidth;
                    return this;
                };
                GridColumn.prototype.setMinWidth = function (minWidth) {
                    this.minWidth = minWidth;
                    return this;
                };
                GridColumn.prototype.setName = function (name) {
                    this.name = name;
                    return this;
                };
                GridColumn.prototype.setRerenderOnResize = function (rerenderOnResize) {
                    this.rerenderOnResize = rerenderOnResize;
                    return this;
                };
                GridColumn.prototype.setResizable = function (resizable) {
                    this.resizable = resizable;
                    return this;
                };
                GridColumn.prototype.setSelectable = function (selectable) {
                    this.selectable = selectable;
                    return this;
                };
                GridColumn.prototype.setSortable = function (sortable) {
                    this.sortable = sortable;
                    return this;
                };
                GridColumn.prototype.setToolTip = function (toolTip) {
                    this.toolTip = toolTip;
                    return this;
                };
                GridColumn.prototype.setWidth = function (width) {
                    this.width = width;
                    return this;
                };
                return GridColumn;
            })();
            grid.GridColumn = GridColumn;
        })(grid = ui.grid || (ui.grid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var grid;
        (function (grid) {
            var GridOptionsBuilder = (function () {
                function GridOptionsBuilder(source) {
                    if (source) {
                        this.asyncEditorLoading = source.isAsyncEditorLoading();
                        this.asyncEditorLoadDelay = source.getAsyncEditorLoadDelay();
                        this.asyncPostRenderDelay = source.getAsyncPostRenderDelay();
                        this.autoEdit = source.isAutoEdit();
                        this.autoHeight = source.isAutoHeight();
                        this.cellFlashingCssClass = source.getCellFlashingCssClass();
                        this.cellHighlightCssClass = source.getCellHighlightCssClass();
                        this.dataItemColumnValueExtractor = source.getDataItemColumnValueExtractor();
                        this.defaultColumnWidth = source.getDefaultColumnWidth();
                        this.defaultFormatter = source.getDefaultFormatter();
                        this.editable = source.isEditable();
                        this.editCommandHandler = source.getEditCommandHandler();
                        this.editorFactory = source.getEditorFactory();
                        this.editorLock = source.getEditorLock();
                        this.enableAddRow = source.isEnableAddRow();
                        this.enableAsyncPostRender = source.isEnableAsyncPostRender();
                        this.enableCellRangeSelection = source.getEnableCellRangeSelection();
                        this.enableCellNavigation = source.isEnableCellNavigation();
                        this.enableColumnReorder = source.isEnableColumnReorder();
                        this.enableRowReordering = source.getEnableRowReordering();
                        this.enableTextSelectionOnCells = source.isEnableTextSelectionOnCells();
                        this.explicitInitialization = source.isExplicitInitialization();
                        this.forceFitColumns = source.isForceFitColumns();
                        this.forceSyncScrolling = source.isForceSyncScrolling();
                        this.formatterFactory = source.getFormatterFactory();
                        this.fullWidthRows = source.isFullWidthRows();
                        this.headerRowHeight = source.getHeaderRowHeight();
                        this.leaveSpaceForNewRows = source.isLeaveSpaceForNewRows();
                        this.multiColumnSort = source.isMultiColumnSort();
                        this.multiSelect = source.isMultiSelect();
                        this.rowHeight = source.getRowHeight();
                        this.selectedCellCssClass = source.getSelectedCellCssClass();
                        this.showHeaderRow = source.getShowHeaderRow();
                        this.syncColumnCellResize = source.isSyncColumnCellResize();
                        this.topPanelHeight = source.getTopPanelHeight();
                        this.hideColumnHeaders = source.isHideColumnHeaders();
                        this.width = source.getWidth();
                        this.height = source.getHeight();
                        this.dataIdProperty = source.getDataIdProperty();
                        this.autoRenderGridOnDataChanges = source.isAutoRenderGridOnDataChanges();
                        this.checkableRows = source.isCheckableRows();
                        this.disabledMultipleSelection = source.isMultipleSelectionDisabled();
                        this.dragAndDrop = source.isDragAndDrop();
                    }
                }
                GridOptionsBuilder.prototype.setAsyncEditorLoading = function (asyncEditorLoading) {
                    this.asyncEditorLoading = asyncEditorLoading;
                    return this;
                };
                GridOptionsBuilder.prototype.setAsyncEditorLoadDelay = function (asyncEditorLoadDelay) {
                    this.asyncEditorLoadDelay = asyncEditorLoadDelay;
                    return this;
                };
                GridOptionsBuilder.prototype.setAsyncPostRenderDelay = function (asyncPostRenderDelay) {
                    this.asyncPostRenderDelay = asyncPostRenderDelay;
                    return this;
                };
                GridOptionsBuilder.prototype.setAutoEdit = function (autoEdit) {
                    this.autoEdit = autoEdit;
                    return this;
                };
                GridOptionsBuilder.prototype.setAutoHeight = function (autoHeight) {
                    this.autoHeight = autoHeight;
                    return this;
                };
                GridOptionsBuilder.prototype.setCellFlashingCssClass = function (cellFlashingCssClass) {
                    this.cellFlashingCssClass = cellFlashingCssClass;
                    return this;
                };
                GridOptionsBuilder.prototype.setCellHighlightCssClass = function (cellHighlightCssClass) {
                    this.cellHighlightCssClass = cellHighlightCssClass;
                    return this;
                };
                GridOptionsBuilder.prototype.setDataItemColumnValueExtractor = function (dataItemColumnValueExtractor) {
                    this.dataItemColumnValueExtractor = dataItemColumnValueExtractor;
                    return this;
                };
                GridOptionsBuilder.prototype.setDefaultColumnWidth = function (defaultColumnWidth) {
                    this.defaultColumnWidth = defaultColumnWidth;
                    return this;
                };
                GridOptionsBuilder.prototype.setDefaultFormatter = function (defaultFormatter) {
                    this.defaultFormatter = defaultFormatter;
                    return this;
                };
                GridOptionsBuilder.prototype.setEditable = function (editable) {
                    this.editable = editable;
                    return this;
                };
                GridOptionsBuilder.prototype.setEditCommandHandler = function (editCommandHandler) {
                    this.editCommandHandler = editCommandHandler;
                    return this;
                };
                GridOptionsBuilder.prototype.setEditorFactory = function (editorFactory) {
                    this.editorFactory = editorFactory;
                    return this;
                };
                GridOptionsBuilder.prototype.setEditorLock = function (editorLock) {
                    this.editorLock = editorLock;
                    return this;
                };
                GridOptionsBuilder.prototype.setEnableAddRow = function (enableAddRow) {
                    this.enableAddRow = enableAddRow;
                    return this;
                };
                GridOptionsBuilder.prototype.setEnableAsyncPostRender = function (enableAsyncPostRender) {
                    this.enableAsyncPostRender = enableAsyncPostRender;
                    return this;
                };
                GridOptionsBuilder.prototype.setEnableCellRangeSelection = function (enableCellRangeSelection) {
                    this.enableCellRangeSelection = enableCellRangeSelection;
                    return this;
                };
                GridOptionsBuilder.prototype.setEnableCellNavigation = function (enableCellNavigation) {
                    this.enableCellNavigation = enableCellNavigation;
                    return this;
                };
                GridOptionsBuilder.prototype.setEnableColumnReorder = function (enableColumnReorder) {
                    this.enableColumnReorder = enableColumnReorder;
                    return this;
                };
                GridOptionsBuilder.prototype.setEnableRowReordering = function (enableRowReordering) {
                    this.enableRowReordering = enableRowReordering;
                    return this;
                };
                GridOptionsBuilder.prototype.setEnableTextSelectionOnCells = function (enableTextSelectionOnCells) {
                    this.enableTextSelectionOnCells = enableTextSelectionOnCells;
                    return this;
                };
                GridOptionsBuilder.prototype.setExplicitInitialization = function (explicitInitialization) {
                    this.explicitInitialization = explicitInitialization;
                    return this;
                };
                GridOptionsBuilder.prototype.setForceFitColumns = function (forceFitColumns) {
                    this.forceFitColumns = forceFitColumns;
                    return this;
                };
                GridOptionsBuilder.prototype.setForceSyncScrolling = function (forceSyncScrolling) {
                    this.forceSyncScrolling = forceSyncScrolling;
                    return this;
                };
                GridOptionsBuilder.prototype.setFormatterFactory = function (formatterFactory) {
                    this.formatterFactory = formatterFactory;
                    return this;
                };
                GridOptionsBuilder.prototype.setFullWidthRows = function (fullWidthRows) {
                    this.fullWidthRows = fullWidthRows;
                    return this;
                };
                GridOptionsBuilder.prototype.setHeaderRowHeight = function (headerRowHeight) {
                    this.headerRowHeight = headerRowHeight;
                    return this;
                };
                GridOptionsBuilder.prototype.setLeaveSpaceForNewRows = function (leaveSpaceForNewRows) {
                    this.leaveSpaceForNewRows = leaveSpaceForNewRows;
                    return this;
                };
                GridOptionsBuilder.prototype.setMultiColumnSort = function (multiColumnSort) {
                    this.multiColumnSort = multiColumnSort;
                    return this;
                };
                GridOptionsBuilder.prototype.setMultiSelect = function (multiSelect) {
                    this.multiSelect = multiSelect;
                    return this;
                };
                GridOptionsBuilder.prototype.setRowHeight = function (rowHeight) {
                    this.rowHeight = rowHeight;
                    return this;
                };
                GridOptionsBuilder.prototype.setSelectedCellCssClass = function (selectedCellCssClass) {
                    this.selectedCellCssClass = selectedCellCssClass;
                    return this;
                };
                GridOptionsBuilder.prototype.setShowHeaderRow = function (showHeaderRow) {
                    this.showHeaderRow = showHeaderRow;
                    return this;
                };
                GridOptionsBuilder.prototype.setSyncColumnCellResize = function (syncColumnCellResize) {
                    this.syncColumnCellResize = syncColumnCellResize;
                    return this;
                };
                GridOptionsBuilder.prototype.setTopPanelHeight = function (topPanelHeight) {
                    this.topPanelHeight = topPanelHeight;
                    return this;
                };
                GridOptionsBuilder.prototype.setHideColumnHeaders = function (hideColumnHeaders) {
                    this.hideColumnHeaders = hideColumnHeaders;
                    return this;
                };
                GridOptionsBuilder.prototype.setWidth = function (width) {
                    this.width = width;
                    return this;
                };
                GridOptionsBuilder.prototype.setHeight = function (height) {
                    this.height = height;
                    return this;
                };
                GridOptionsBuilder.prototype.setDataIdProperty = function (dataIdProperty) {
                    this.dataIdProperty = dataIdProperty;
                    return this;
                };
                GridOptionsBuilder.prototype.setAutoRenderGridOnDataChanges = function (autoRenderGridOnDataChanges) {
                    this.autoRenderGridOnDataChanges = autoRenderGridOnDataChanges;
                    return this;
                };
                GridOptionsBuilder.prototype.setCheckableRows = function (checkableRows) {
                    this.checkableRows = checkableRows;
                    return this;
                };
                GridOptionsBuilder.prototype.disableMultipleSelection = function (disabledMultipleSelection) {
                    this.disabledMultipleSelection = disabledMultipleSelection;
                    return this;
                };
                GridOptionsBuilder.prototype.setDragAndDrop = function (dragAndDrop) {
                    this.dragAndDrop = dragAndDrop;
                    return this;
                };
                GridOptionsBuilder.prototype.build = function () {
                    return new GridOptions(this);
                };
                return GridOptionsBuilder;
            })();
            grid.GridOptionsBuilder = GridOptionsBuilder;
            var GridOptions = (function () {
                function GridOptions(builder) {
                    this.asyncEditorLoading = builder.asyncEditorLoading;
                    this.asyncEditorLoadDelay = builder.asyncEditorLoadDelay;
                    this.asyncPostRenderDelay = builder.asyncPostRenderDelay;
                    this.autoEdit = builder.autoEdit;
                    this.autoHeight = builder.autoHeight;
                    this.cellFlashingCssClass = builder.cellFlashingCssClass;
                    this.cellHighlightCssClass = builder.cellHighlightCssClass;
                    this.dataItemColumnValueExtractor = builder.dataItemColumnValueExtractor;
                    this.defaultColumnWidth = builder.defaultColumnWidth;
                    this.defaultFormatter = builder.defaultFormatter;
                    this.editable = builder.editable;
                    this.editCommandHandler = builder.editCommandHandler;
                    this.editorFactory = builder.editorFactory;
                    this.editorLock = builder.editorLock;
                    this.enableAddRow = builder.enableAddRow;
                    this.enableAsyncPostRender = builder.enableAsyncPostRender;
                    this.enableCellRangeSelection = builder.enableCellRangeSelection;
                    this.enableCellNavigation = builder.enableCellNavigation;
                    this.enableColumnReorder = builder.enableColumnReorder;
                    this.enableRowReordering = builder.enableRowReordering;
                    this.enableTextSelectionOnCells = builder.enableTextSelectionOnCells;
                    this.explicitInitialization = builder.explicitInitialization;
                    this.forceFitColumns = builder.forceFitColumns;
                    this.forceSyncScrolling = builder.forceSyncScrolling;
                    this.formatterFactory = builder.formatterFactory;
                    this.fullWidthRows = builder.fullWidthRows;
                    this.headerRowHeight = builder.headerRowHeight;
                    this.leaveSpaceForNewRows = builder.leaveSpaceForNewRows;
                    this.multiColumnSort = builder.multiColumnSort;
                    this.multiSelect = builder.multiSelect;
                    this.rowHeight = builder.rowHeight;
                    this.selectedCellCssClass = builder.selectedCellCssClass;
                    this.showHeaderRow = builder.showHeaderRow;
                    this.syncColumnCellResize = builder.syncColumnCellResize;
                    this.topPanelHeight = builder.topPanelHeight;
                    this.hideColumnHeaders = builder.hideColumnHeaders;
                    this.width = builder.width;
                    this.height = builder.height;
                    this.dataIdProperty = builder.dataIdProperty;
                    this.autoRenderGridOnDataChanges = builder.autoRenderGridOnDataChanges;
                    this.checkableRows = builder.checkableRows;
                    this.disabledMultipleSelection = builder.disabledMultipleSelection;
                    this.dragAndDrop = builder.dragAndDrop;
                }
                GridOptions.prototype.isAsyncEditorLoading = function () {
                    return this.asyncEditorLoading;
                };
                GridOptions.prototype.getAsyncEditorLoadDelay = function () {
                    return this.asyncEditorLoadDelay;
                };
                GridOptions.prototype.getAsyncPostRenderDelay = function () {
                    return this.asyncPostRenderDelay;
                };
                GridOptions.prototype.isAutoEdit = function () {
                    return this.autoEdit;
                };
                GridOptions.prototype.isAutoHeight = function () {
                    return this.autoHeight;
                };
                GridOptions.prototype.getCellFlashingCssClass = function () {
                    return this.cellFlashingCssClass;
                };
                GridOptions.prototype.getCellHighlightCssClass = function () {
                    return this.cellHighlightCssClass;
                };
                GridOptions.prototype.getDataItemColumnValueExtractor = function () {
                    return this.dataItemColumnValueExtractor;
                };
                GridOptions.prototype.getDefaultColumnWidth = function () {
                    return this.defaultColumnWidth;
                };
                GridOptions.prototype.getDefaultFormatter = function () {
                    return this.defaultFormatter;
                };
                GridOptions.prototype.isEditable = function () {
                    return this.editable;
                };
                GridOptions.prototype.getEditCommandHandler = function () {
                    return this.editCommandHandler;
                };
                GridOptions.prototype.getEditorFactory = function () {
                    return this.editorFactory;
                };
                GridOptions.prototype.getEditorLock = function () {
                    return this.editorLock;
                };
                GridOptions.prototype.isEnableAddRow = function () {
                    return this.enableAddRow;
                };
                GridOptions.prototype.isEnableAsyncPostRender = function () {
                    return this.enableAsyncPostRender;
                };
                GridOptions.prototype.getEnableCellRangeSelection = function () {
                    return this.enableCellRangeSelection;
                };
                GridOptions.prototype.isEnableCellNavigation = function () {
                    return this.enableCellNavigation;
                };
                GridOptions.prototype.isEnableColumnReorder = function () {
                    return this.enableColumnReorder;
                };
                GridOptions.prototype.getEnableRowReordering = function () {
                    return this.enableRowReordering;
                };
                GridOptions.prototype.isEnableTextSelectionOnCells = function () {
                    return this.enableTextSelectionOnCells;
                };
                GridOptions.prototype.isExplicitInitialization = function () {
                    return this.explicitInitialization;
                };
                GridOptions.prototype.isForceFitColumns = function () {
                    return this.forceFitColumns;
                };
                GridOptions.prototype.isForceSyncScrolling = function () {
                    return this.forceSyncScrolling;
                };
                GridOptions.prototype.getFormatterFactory = function () {
                    return this.formatterFactory;
                };
                GridOptions.prototype.isFullWidthRows = function () {
                    return this.fullWidthRows;
                };
                GridOptions.prototype.getHeaderRowHeight = function () {
                    return this.headerRowHeight;
                };
                GridOptions.prototype.isLeaveSpaceForNewRows = function () {
                    return this.leaveSpaceForNewRows;
                };
                GridOptions.prototype.isMultiColumnSort = function () {
                    return this.multiColumnSort;
                };
                GridOptions.prototype.isMultiSelect = function () {
                    return this.multiSelect;
                };
                GridOptions.prototype.getRowHeight = function () {
                    return this.rowHeight;
                };
                GridOptions.prototype.getSelectedCellCssClass = function () {
                    return this.selectedCellCssClass;
                };
                GridOptions.prototype.getShowHeaderRow = function () {
                    return this.showHeaderRow;
                };
                GridOptions.prototype.isSyncColumnCellResize = function () {
                    return this.syncColumnCellResize;
                };
                GridOptions.prototype.getTopPanelHeight = function () {
                    return this.topPanelHeight;
                };
                GridOptions.prototype.isHideColumnHeaders = function () {
                    return this.hideColumnHeaders;
                };
                GridOptions.prototype.getWidth = function () {
                    return this.width;
                };
                GridOptions.prototype.getHeight = function () {
                    return this.height;
                };
                GridOptions.prototype.getDataIdProperty = function () {
                    return this.dataIdProperty;
                };
                GridOptions.prototype.isAutoRenderGridOnDataChanges = function () {
                    return this.autoRenderGridOnDataChanges;
                };
                GridOptions.prototype.isCheckableRows = function () {
                    return this.checkableRows;
                };
                GridOptions.prototype.isMultipleSelectionDisabled = function () {
                    return this.disabledMultipleSelection;
                };
                GridOptions.prototype.isDragAndDrop = function () {
                    return this.dragAndDrop;
                };
                GridOptions.prototype.setAsyncEditorLoading = function (asyncEditorLoading) {
                    this.asyncEditorLoading = asyncEditorLoading;
                    return this;
                };
                GridOptions.prototype.setAsyncEditorLoadDelay = function (asyncEditorLoadDelay) {
                    this.asyncEditorLoadDelay = asyncEditorLoadDelay;
                    return this;
                };
                GridOptions.prototype.setAsyncPostRenderDelay = function (asyncPostRenderDelay) {
                    this.asyncPostRenderDelay = asyncPostRenderDelay;
                    return this;
                };
                GridOptions.prototype.setAutoEdit = function (autoEdit) {
                    this.autoEdit = autoEdit;
                    return this;
                };
                GridOptions.prototype.setAutoHeight = function (autoHeight) {
                    this.autoHeight = autoHeight;
                    return this;
                };
                GridOptions.prototype.setCellFlashingCssClass = function (cellFlashingCssClass) {
                    this.cellFlashingCssClass = cellFlashingCssClass;
                    return this;
                };
                GridOptions.prototype.setCellHighlightCssClass = function (cellHighlightCssClass) {
                    this.cellHighlightCssClass = cellHighlightCssClass;
                    return this;
                };
                GridOptions.prototype.setDataItemColumnValueExtractor = function (dataItemColumnValueExtractor) {
                    this.dataItemColumnValueExtractor = dataItemColumnValueExtractor;
                    return this;
                };
                GridOptions.prototype.setDefaultColumnWidth = function (defaultColumnWidth) {
                    this.defaultColumnWidth = defaultColumnWidth;
                    return this;
                };
                GridOptions.prototype.setDefaultFormatter = function (defaultFormatter) {
                    this.defaultFormatter = defaultFormatter;
                    return this;
                };
                GridOptions.prototype.setEditable = function (editable) {
                    this.editable = editable;
                    return this;
                };
                GridOptions.prototype.setEditCommandHandler = function (editCommandHandler) {
                    this.editCommandHandler = editCommandHandler;
                    return this;
                };
                GridOptions.prototype.setEditorFactory = function (editorFactory) {
                    this.editorFactory = editorFactory;
                    return this;
                };
                GridOptions.prototype.setEditorLock = function (editorLock) {
                    this.editorLock = editorLock;
                    return this;
                };
                GridOptions.prototype.setEnableAddRow = function (enableAddRow) {
                    this.enableAddRow = enableAddRow;
                    return this;
                };
                GridOptions.prototype.setEnableAsyncPostRender = function (enableAsyncPostRender) {
                    this.enableAsyncPostRender = enableAsyncPostRender;
                    return this;
                };
                GridOptions.prototype.setEnableCellRangeSelection = function (enableCellRangeSelection) {
                    this.enableCellRangeSelection = enableCellRangeSelection;
                    return this;
                };
                GridOptions.prototype.setEnableCellNavigation = function (enableCellNavigation) {
                    this.enableCellNavigation = enableCellNavigation;
                    return this;
                };
                GridOptions.prototype.setEnableColumnReorder = function (enableColumnReorder) {
                    this.enableColumnReorder = enableColumnReorder;
                    return this;
                };
                GridOptions.prototype.setEnableRowReordering = function (enableRowReordering) {
                    this.enableRowReordering = enableRowReordering;
                    return this;
                };
                GridOptions.prototype.setEnableTextSelectionOnCells = function (enableTextSelectionOnCells) {
                    this.enableTextSelectionOnCells = enableTextSelectionOnCells;
                    return this;
                };
                GridOptions.prototype.setExplicitInitialization = function (explicitInitialization) {
                    this.explicitInitialization = explicitInitialization;
                    return this;
                };
                GridOptions.prototype.setForceFitColumns = function (forceFitColumns) {
                    this.forceFitColumns = forceFitColumns;
                    return this;
                };
                GridOptions.prototype.setForceSyncScrolling = function (forceSyncScrolling) {
                    this.forceSyncScrolling = forceSyncScrolling;
                    return this;
                };
                GridOptions.prototype.setFormatterFactory = function (formatterFactory) {
                    this.formatterFactory = formatterFactory;
                    return this;
                };
                GridOptions.prototype.setFullWidthRows = function (fullWidthRows) {
                    this.fullWidthRows = fullWidthRows;
                    return this;
                };
                GridOptions.prototype.setHeaderRowHeight = function (headerRowHeight) {
                    this.headerRowHeight = headerRowHeight;
                    return this;
                };
                GridOptions.prototype.setLeaveSpaceForNewRows = function (leaveSpaceForNewRows) {
                    this.leaveSpaceForNewRows = leaveSpaceForNewRows;
                    return this;
                };
                GridOptions.prototype.setMultiColumnSort = function (multiColumnSort) {
                    this.multiColumnSort = multiColumnSort;
                    return this;
                };
                GridOptions.prototype.setMultiSelect = function (multiSelect) {
                    this.multiSelect = multiSelect;
                    return this;
                };
                GridOptions.prototype.setRowHeight = function (rowHeight) {
                    this.rowHeight = rowHeight;
                    return this;
                };
                GridOptions.prototype.setSelectedCellCssClass = function (selectedCellCssClass) {
                    this.selectedCellCssClass = selectedCellCssClass;
                    return this;
                };
                GridOptions.prototype.setShowHeaderRow = function (showHeaderRow) {
                    this.showHeaderRow = showHeaderRow;
                    return this;
                };
                GridOptions.prototype.setSyncColumnCellResize = function (syncColumnCellResize) {
                    this.syncColumnCellResize = syncColumnCellResize;
                    return this;
                };
                GridOptions.prototype.setTopPanelHeight = function (topPanelHeight) {
                    this.topPanelHeight = topPanelHeight;
                    return this;
                };
                GridOptions.prototype.setHideColumnHeaders = function (hideColumnHeaders) {
                    this.hideColumnHeaders = hideColumnHeaders;
                    return this;
                };
                GridOptions.prototype.setWidth = function (width) {
                    this.width = width;
                    return this;
                };
                GridOptions.prototype.setHeight = function (height) {
                    this.height = height;
                    return this;
                };
                GridOptions.prototype.setDataIdProperty = function (dataIdProperty) {
                    this.dataIdProperty = dataIdProperty;
                    return this;
                };
                GridOptions.prototype.setAutoRenderGridOnDataChanges = function (autoRenderGridOnDataChanges) {
                    this.autoRenderGridOnDataChanges = autoRenderGridOnDataChanges;
                    return this;
                };
                GridOptions.prototype.setCheckableRows = function (checkableRows) {
                    this.checkableRows = checkableRows;
                    return this;
                };
                GridOptions.prototype.disableMultipleSelection = function (disabledMultipleSelection) {
                    this.disabledMultipleSelection = disabledMultipleSelection;
                    return this;
                };
                GridOptions.prototype.setDragAndDrop = function (dragAndDrop) {
                    this.dragAndDrop = dragAndDrop;
                    return this;
                };
                return GridOptions;
            })();
            grid.GridOptions = GridOptions;
        })(grid = ui.grid || (ui.grid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var grid;
        (function (_grid) {
            var GridOnClickDataBuilder = (function () {
                function GridOnClickDataBuilder(source) {
                    if (source) {
                        this.row = source.getRow();
                        this.cell = source.getCell();
                        this.grid = source.getGrid();
                    }
                }
                GridOnClickDataBuilder.prototype.setRow = function (row) {
                    this.row = row;
                    return this;
                };
                GridOnClickDataBuilder.prototype.setCell = function (cell) {
                    this.cell = cell;
                    return this;
                };
                GridOnClickDataBuilder.prototype.setGrid = function (grid) {
                    this.grid = grid;
                    return this;
                };
                GridOnClickDataBuilder.prototype.build = function () {
                    return new GridOnClickData(this);
                };
                return GridOnClickDataBuilder;
            })();
            _grid.GridOnClickDataBuilder = GridOnClickDataBuilder;
            var GridOnClickData = (function () {
                function GridOnClickData(builder) {
                    this.row = builder.row;
                    this.cell = builder.cell;
                    this.grid = builder.grid;
                }
                GridOnClickData.prototype.getRow = function () {
                    return this.row;
                };
                GridOnClickData.prototype.getCell = function () {
                    return this.cell;
                };
                GridOnClickData.prototype.getGrid = function () {
                    return this.grid;
                };
                return GridOnClickData;
            })();
            _grid.GridOnClickData = GridOnClickData;
        })(grid = ui.grid || (ui.grid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var grid;
        (function (_grid) {
            var DataView = (function () {
                function DataView() {
                    this.slickDataView = new Slick.Data.DataView({ inlineFilters: true });
                }
                DataView.prototype.slick = function () {
                    return this.slickDataView;
                };
                DataView.prototype.setFilter = function (f) {
                    this.slickDataView.setFilter(f);
                };
                DataView.prototype.setFilterArgs = function (args) {
                    this.slickDataView.setFilterArgs(args);
                };
                DataView.prototype.beginUpdate = function () {
                    this.slickDataView.beginUpdate();
                };
                DataView.prototype.endUpdate = function () {
                    this.slickDataView.endUpdate();
                };
                DataView.prototype.refresh = function () {
                    this.slickDataView.refresh();
                };
                DataView.prototype.setItems = function (items, objectIdProperty) {
                    this.slickDataView.setItems(items, objectIdProperty);
                };
                DataView.prototype.addItem = function (item) {
                    this.slickDataView.addItem(item);
                };
                DataView.prototype.insertItem = function (insertBefore, item) {
                    this.slickDataView.insertItem(insertBefore, item);
                };
                DataView.prototype.updateItem = function (id, item) {
                    this.slickDataView.updateItem(id, item);
                };
                DataView.prototype.deleteItem = function (id) {
                    this.slickDataView.deleteItem(id);
                };
                DataView.prototype.syncGridSelection = function (grid, preserveHidden) {
                    this.slickDataView.syncGridSelection(grid, preserveHidden);
                };
                DataView.prototype.getItem = function (index) {
                    return this.slickDataView.getItem(index);
                };
                DataView.prototype.getItems = function () {
                    return this.slickDataView.getItems();
                };
                DataView.prototype.getItemById = function (id) {
                    return this.slickDataView.getItemById(id);
                };
                DataView.prototype.getLength = function () {
                    return this.slickDataView.getLength();
                };
                DataView.prototype.getRowById = function (id) {
                    return this.slickDataView.getRowById(id);
                };
                DataView.prototype.onRowsChanged = function (callback) {
                    this.slickDataView.onRowsChanged.subscribe(callback);
                };
                DataView.prototype.onRowCountChanged = function (listener) {
                    this.slickDataView.onRowCountChanged.subscribe(listener);
                };
                return DataView;
            })();
            _grid.DataView = DataView;
        })(grid = ui.grid || (ui.grid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var grid;
        (function (grid) {
            var CollapsingItem = (function () {
                function CollapsingItem(actualItem) {
                    this.actualItem = actualItem;
                }
                return CollapsingItem;
            })();
            grid.CollapsingItem = CollapsingItem;
        })(grid = ui.grid || (ui.grid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var grid;
        (function (grid) {
            var Element = api.dom.Element;
            var GridDragHandler = (function () {
                function GridDragHandler(treeGrid) {
                    this.positionChangedListeners = [];
                    this.contentGrid = treeGrid;
                    this.contentGrid.getGrid().subscribeOnDrag(this.handleDrag.bind(this));
                    this.contentGrid.getGrid().subscribeOnDragInit(this.handleDragInit.bind(this));
                    this.contentGrid.getGrid().subscribeOnDragEnd(this.handleDragEnd.bind(this));
                    this.contentGrid.getGrid().subscribeBeforeMoveRows(this.handleBeforeMoveRows.bind(this));
                    this.contentGrid.getGrid().subscribeMoveRows(this.handleMoveRows.bind(this));
                }
                GridDragHandler.prototype.handleDragInit = function (e, dd) {
                    e.stopImmediatePropagation();
                };
                GridDragHandler.prototype.handleDragStart = function () {
                    var draggableClass = this.contentGrid.getOptions().getSelectedCellCssClass() || "";
                    draggableClass = (" " + draggableClass).replace(/\s/g, ".");
                    var row = Element.fromString(draggableClass).getParentElement();
                    var nodes = this.contentGrid.getRoot().getCurrentRoot().treeToList(), draggedNode = nodes[row.getSiblingIndex()];
                    draggedNode.setExpanded(false);
                    this.contentGrid.refreshNode(draggedNode);
                    row = Element.fromString(draggableClass).getParentElement();
                    this.draggableTop = row.getEl().getTopPx();
                    this.draggableItem = Element.fromString(row.toString());
                    this.draggableItem.addClass("draggable");
                    row.getEl().setDisplay("none");
                    this.rowHeight = row.getEl().getHeight();
                    var proxyEl = Element.fromString(".slick-reorder-proxy").getEl();
                    this.draggableItem.getEl().setTop(proxyEl.getTop()).setPosition("absolute");
                    var gridClasses = (" " + this.contentGrid.getGrid().getEl().getClass()).replace(/\s/g, ".");
                    wemjq(".tree-grid " + gridClasses + " .slick-viewport").append(wemjq(this.draggableItem.getHTMLElement()));
                };
                GridDragHandler.prototype.handleDrag = function (event, data) {
                    if (!this.draggableItem) {
                        this.handleDragStart();
                    }
                    var top = Element.fromString(".slick-reorder-proxy").getEl().getTopPx();
                    this.draggableItem.getEl().setTopPx(top).setZindex(2);
                };
                GridDragHandler.prototype.handleDragEnd = function (event, data) {
                    this.draggableItem.remove();
                    this.draggableItem = null;
                    this.contentGrid.refresh();
                };
                GridDragHandler.prototype.handleBeforeMoveRows = function (event, data) {
                    if (!this.draggableItem) {
                        this.handleDragStart();
                    }
                    var gridClasses = (" " + this.contentGrid.getGrid().getEl().getClass()).replace(/\s/g, "."), children = Element.fromSelector(".tree-grid " + gridClasses + " .grid-canvas .slick-row", false);
                    for (var key in children) {
                        if (data.rows[0] <= data.insertBefore) {
                            if (key > data.rows[0] && key <= data.insertBefore) {
                                children[key].getEl().setMarginTop("-" + this.rowHeight + "px");
                            }
                            else {
                                children[key].getEl().setMarginTop(null);
                            }
                            continue;
                        }
                        if (data.rows[0] >= data.insertBefore) {
                            if (key < data.rows[0] && key >= data.insertBefore) {
                                children[key].getEl().setMarginTop(this.rowHeight + "px");
                            }
                            else {
                                children[key].getEl().setMarginTop(null);
                            }
                        }
                    }
                    this.contentGrid.scrollToRow(data.insertBefore);
                    return true;
                };
                GridDragHandler.prototype.handleMoveRows = function (event, args) {
                    var dataView = this.contentGrid.getGrid().getDataView();
                    var draggableRow = args.rows[0];
                    var rowDataId = this.getModelId(dataView.getItem(draggableRow).getData());
                    var insertTarget = args.insertBefore;
                    var insertBefore = draggableRow < insertTarget ? insertTarget + 1 : insertTarget;
                    var moveBeforeRowDataId = ((dataView.getLength() - 1) <= insertTarget) ? null : this.getModelId(dataView.getItem(insertBefore).getData());
                    var selectedRow = this.makeMovementInNodes(draggableRow, insertTarget);
                    if (selectedRow <= this.contentGrid.getRoot().getCurrentRoot().treeToList().length - 1) {
                        this.contentGrid.getGrid().setSelectedRows([selectedRow]);
                    }
                    this.handleMovements(rowDataId, moveBeforeRowDataId);
                    this.notifyPositionChanged();
                };
                GridDragHandler.prototype.makeMovementInNodes = function (draggableRow, insertBefore) {
                    var root = this.contentGrid.getRoot().getCurrentRoot();
                    var rootChildren = root.treeToList();
                    var item = rootChildren.slice(draggableRow, draggableRow + 1)[0];
                    rootChildren.splice(rootChildren.indexOf(item), 1);
                    rootChildren.splice(insertBefore, 0, item);
                    this.contentGrid.initData(rootChildren);
                    root.setChildren(rootChildren);
                    return rootChildren.indexOf(item);
                };
                GridDragHandler.prototype.getDraggableItem = function () {
                    return this.draggableItem;
                };
                GridDragHandler.prototype.handleMovements = function (rowDataId, moveBeforeRowDataId) {
                    throw new Error("Must be implemented by inheritors");
                };
                GridDragHandler.prototype.getModelId = function (model) {
                    throw new Error("Must be implemented by inheritors");
                };
                GridDragHandler.prototype.onPositionChanged = function (listener) {
                    this.positionChangedListeners.push(listener);
                };
                GridDragHandler.prototype.unPositionChanged = function (listener) {
                    this.positionChangedListeners = this.positionChangedListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                GridDragHandler.prototype.notifyPositionChanged = function () {
                    var _this = this;
                    this.positionChangedListeners.forEach(function (listener) {
                        listener.call(_this);
                    });
                };
                return GridDragHandler;
            })();
            grid.GridDragHandler = GridDragHandler;
        })(grid = ui.grid || (ui.grid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var menu;
        (function (menu) {
            var MenuItem = (function (_super) {
                __extends(MenuItem, _super);
                function MenuItem(action) {
                    var _this = this;
                    _super.call(this, "menu-item");
                    this.action = action;
                    this.getEl().setInnerHtml(this.action.getLabel());
                    this.onClicked(function (event) {
                        if (action.isEnabled()) {
                            _this.action.execute();
                        }
                    });
                    this.setEnabled(action.isEnabled());
                    action.onPropertyChanged(function (action) {
                        _this.setEnabled(action.isEnabled());
                        _this.setVisible(action.isVisible());
                    });
                }
                MenuItem.prototype.getAction = function () {
                    return this.action;
                };
                MenuItem.prototype.setEnabled = function (value) {
                    var el = this.getEl();
                    el.setDisabled(!value);
                    if (value) {
                        el.removeClass("disabled");
                    }
                    else {
                        el.addClass("disabled");
                    }
                };
                return MenuItem;
            })(api.dom.LiEl);
            menu.MenuItem = MenuItem;
        })(menu = ui.menu || (ui.menu = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var menu;
        (function (_menu) {
            var ContextMenu = (function (_super) {
                __extends(ContextMenu, _super);
                function ContextMenu(actions, appendToBody) {
                    var _this = this;
                    if (appendToBody === void 0) { appendToBody = true; }
                    _super.call(this, "context-menu");
                    this.menuItems = [];
                    this.hideOnItemClick = true;
                    this.itemClickListeners = [];
                    if (actions) {
                        for (var i = 0; i < actions.length; i++) {
                            this.addAction(actions[i]);
                        }
                    }
                    if (appendToBody) {
                        api.dom.Body.get().appendChild(this);
                        api.dom.Body.get().onClicked(function (event) { return _this.hideMenuOnOutsideClick(event); });
                    }
                    this.onClicked(function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                    });
                }
                ContextMenu.prototype.addAction = function (action) {
                    var menuItem = this.createMenuItem(action);
                    this.appendChild(menuItem);
                    return this;
                };
                ContextMenu.prototype.addActions = function (actions) {
                    var _this = this;
                    actions.forEach(function (action) {
                        _this.addAction(action);
                    });
                    return this;
                };
                ContextMenu.prototype.removeAction = function (action) {
                    var menuItem = this.getMenuItem(action);
                    if (menuItem) {
                        this.removeMenuItem(menuItem);
                        this.removeChild(menuItem);
                    }
                    return this;
                };
                ContextMenu.prototype.removeActions = function (actions) {
                    var _this = this;
                    actions.forEach(function (action) {
                        _this.removeAction(action);
                    });
                    return this;
                };
                ContextMenu.prototype.setActions = function (actions) {
                    this.menuItems.length = 0;
                    this.removeChildren();
                    this.addActions(actions);
                    return this;
                };
                ContextMenu.prototype.setHideOnItemClick = function (hide) {
                    this.hideOnItemClick = hide;
                    return this;
                };
                ContextMenu.prototype.showAt = function (x, y) {
                    ContextMenu.prototype.doMoveTo(this, x, y);
                    this.show();
                };
                ContextMenu.prototype.moveBy = function (dx, dy) {
                    var offset = this.getEl().getOffsetToParent();
                    ContextMenu.prototype.doMoveTo(this, offset.left + dx, offset.top + dy);
                };
                ContextMenu.prototype.doMoveTo = function (menu, x, y) {
                    menu.getEl().setLeftPx(x).setTopPx(y);
                };
                ContextMenu.prototype.onItemClicked = function (listener) {
                    this.itemClickListeners.push(listener);
                };
                ContextMenu.prototype.unItemClicked = function (listener) {
                    this.itemClickListeners = this.itemClickListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                ContextMenu.prototype.notifyItemClicked = function (item) {
                    this.itemClickListeners.forEach(function (listener) {
                        listener(item);
                    });
                };
                ContextMenu.prototype.createMenuItem = function (action) {
                    var _this = this;
                    var menuItem = new _menu.MenuItem(action);
                    menuItem.onClicked(function (event) {
                        _this.notifyItemClicked(menuItem);
                        if (_this.hideOnItemClick) {
                            _this.hide();
                        }
                        event.preventDefault();
                        event.stopPropagation();
                    });
                    this.menuItems.push(menuItem);
                    return menuItem;
                };
                ContextMenu.prototype.removeMenuItem = function (menuItem) {
                    this.menuItems = this.menuItems.filter(function (item) {
                        return item != menuItem;
                    });
                };
                ContextMenu.prototype.getMenuItem = function (action) {
                    for (var i = 0; i < this.menuItems.length; i++) {
                        var menuItem = this.menuItems[i];
                        if (menuItem.getAction() == action) {
                            return menuItem;
                        }
                    }
                    return undefined;
                };
                ContextMenu.prototype.hideMenuOnOutsideClick = function (evt) {
                    if (!this.getEl().contains(evt.target)) {
                        this.hide();
                    }
                };
                return ContextMenu;
            })(api.dom.UlEl);
            _menu.ContextMenu = ContextMenu;
        })(menu = ui.menu || (ui.menu = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var menu;
        (function (menu) {
            var ActionMenuItem = (function (_super) {
                __extends(ActionMenuItem, _super);
                function ActionMenuItem(action) {
                    var _this = this;
                    _super.call(this, "action");
                    this.action = action;
                    this.getEl().setInnerHtml(this.action.getLabel());
                    this.action.onPropertyChanged(function () {
                        if (_this.action.isEnabled()) {
                            _this.show();
                        }
                        else if (!_this.action.isEnabled()) {
                            _this.hide();
                        }
                    });
                    this.onClicked(function () {
                        _this.action.execute();
                    });
                }
                return ActionMenuItem;
            })(api.dom.LiEl);
            menu.ActionMenuItem = ActionMenuItem;
        })(menu = ui.menu || (ui.menu = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var menu;
        (function (menu) {
            var ActionMenu = (function (_super) {
                __extends(ActionMenu, _super);
                function ActionMenu(label) {
                    var _this = this;
                    var actions = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        actions[_i - 1] = arguments[_i];
                    }
                    _super.call(this, "action-menu");
                    this.labelEl = new api.dom.DivEl("drop-down-button icon-arrow-down");
                    this.labelEl.setHtml(label);
                    this.appendChild(this.labelEl);
                    this.actionListEl = new api.dom.UlEl();
                    this.appendChild(this.actionListEl);
                    if (actions.length > 0) {
                        actions.forEach(function (action) {
                            _this.addAction(action);
                        });
                    }
                    this.labelEl.onClicked(function (event) {
                        if (_this.hasClass("expanded")) {
                            _this.removeClass("expanded");
                        }
                        else {
                            _this.addClass("expanded");
                        }
                    });
                }
                ActionMenu.prototype.setLabel = function (label) {
                    this.labelEl.getEl().setInnerHtml(label);
                };
                ActionMenu.prototype.addAction = function (action) {
                    var _this = this;
                    var actionMenuItem = new menu.ActionMenuItem(action);
                    this.actionListEl.appendChild(actionMenuItem);
                    actionMenuItem.onClicked(function () {
                        _this.removeClass("expanded");
                    });
                };
                return ActionMenu;
            })(api.dom.DivEl);
            menu.ActionMenu = ActionMenu;
        })(menu = ui.menu || (ui.menu = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var menu;
        (function (_menu) {
            var TreeContextMenu = (function (_super) {
                __extends(TreeContextMenu, _super);
                function TreeContextMenu(actions, appendToBody) {
                    var _this = this;
                    if (appendToBody === void 0) { appendToBody = true; }
                    _super.call(this, "context-menu");
                    this.itemClickListeners = [];
                    this.actions = [];
                    if (actions) {
                        actions.sort(function (action1, action2) {
                            return action1.getSortOrder() - action2.getSortOrder();
                        }).forEach(function (action) {
                            _this.addAction(action);
                        });
                    }
                    if (appendToBody) {
                        api.dom.Body.get().appendChild(this);
                        api.dom.Body.get().onClicked(function (event) { return _this.hideMenuOnOutsideClick(event); });
                    }
                    this.onClicked(function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                    });
                }
                TreeContextMenu.prototype.addAction = function (action) {
                    var _this = this;
                    var childActions = action.getChildActions();
                    var menuItem = this.createMenuItem(action);
                    var subItems = [];
                    this.appendChild(menuItem);
                    this.actions.push(action);
                    if (childActions.length > 0) {
                        for (var i = 0; i < childActions.length; i++) {
                            subItems.push(this.addAction(childActions[i]));
                        }
                        menuItem.onClicked(function (event) {
                            for (var i = 0; i < subItems.length; i++) {
                                subItems[i].toggleExpand();
                            }
                        });
                    }
                    else {
                        menuItem.onClicked(function (event) {
                            _this.notifyItemClicked();
                            event.preventDefault();
                            event.stopPropagation();
                        });
                    }
                    return menuItem;
                };
                TreeContextMenu.prototype.addActions = function (actions) {
                    var _this = this;
                    actions.forEach(function (action) {
                        _this.addAction(action);
                    });
                    return this;
                };
                TreeContextMenu.prototype.setActions = function (actions) {
                    this.removeChildren();
                    this.clearActionListeners();
                    this.actions = [];
                    this.addActions(actions);
                    return this;
                };
                TreeContextMenu.prototype.clearActionListeners = function () {
                    this.actions.forEach(function (action) {
                        action.clearListeners();
                    });
                };
                TreeContextMenu.prototype.onItemClicked = function (listener) {
                    this.itemClickListeners.push(listener);
                };
                TreeContextMenu.prototype.unItemClicked = function (listener) {
                    this.itemClickListeners = this.itemClickListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                TreeContextMenu.prototype.notifyItemClicked = function () {
                    this.itemClickListeners.forEach(function (listener) {
                        listener();
                    });
                };
                TreeContextMenu.prototype.onBeforeAction = function (listener) {
                    this.actions.forEach(function (action) {
                        action.onBeforeExecute(listener);
                    });
                };
                TreeContextMenu.prototype.onAfterAction = function (listener) {
                    this.actions.forEach(function (action) {
                        action.onAfterExecute(listener);
                    });
                };
                TreeContextMenu.prototype.showAt = function (x, y) {
                    TreeContextMenu.prototype.doMoveTo(this, x, y);
                    this.show();
                };
                TreeContextMenu.prototype.moveBy = function (dx, dy) {
                    var offset = this.getEl().getOffsetToParent();
                    TreeContextMenu.prototype.doMoveTo(this, offset.left + dx, offset.top + dy);
                };
                TreeContextMenu.prototype.doMoveTo = function (menu, x, y) {
                    menu.getEl().setLeftPx(x).setTopPx(y);
                };
                TreeContextMenu.prototype.createMenuItem = function (action) {
                    return new _menu.TreeMenuItem(action, action.getIconClass());
                };
                TreeContextMenu.prototype.hideMenuOnOutsideClick = function (evt) {
                    if (!this.getEl().contains(evt.target)) {
                        this.hide();
                    }
                };
                return TreeContextMenu;
            })(api.dom.DlEl);
            _menu.TreeContextMenu = TreeContextMenu;
        })(menu = ui.menu || (ui.menu = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var menu;
        (function (menu) {
            var TreeMenuItem = (function (_super) {
                __extends(TreeMenuItem, _super);
                function TreeMenuItem(action, cls, expanded) {
                    var _this = this;
                    if (cls === void 0) { cls = ""; }
                    if (expanded === void 0) { expanded = false; }
                    _super.call(this, this.getTag(action), this.getCls(action, cls, expanded));
                    this.action = action;
                    this.getEl().setInnerHtml(this.action.getLabel());
                    this.onClicked(function (event) {
                        if (action.isEnabled()) {
                            if (action.hasChildActions()) {
                                _this.toggleExpand();
                            }
                            else {
                                _this.action.execute();
                            }
                        }
                    });
                    this.setEnabled(action.isEnabled());
                    action.onPropertyChanged(function (action) {
                        _this.setEnabled(action.isEnabled());
                        _this.setVisible(action.isVisible());
                    });
                }
                TreeMenuItem.prototype.toggleExpand = function () {
                    this.toggleClass("expanded");
                };
                TreeMenuItem.prototype.getTag = function (action) {
                    return action.hasParentAction() ? "dd" : "dt";
                };
                TreeMenuItem.prototype.getCls = function (action, cls, expanded) {
                    if (cls === void 0) { cls = ""; }
                    if (expanded === void 0) { expanded = false; }
                    var fullCls = action.hasChildActions() ? "collapsible " : "";
                    fullCls += expanded ? "expanded " : "";
                    return fullCls + cls;
                };
                TreeMenuItem.prototype.getAction = function () {
                    return this.action;
                };
                TreeMenuItem.prototype.setEnabled = function (value) {
                    var el = this.getEl();
                    el.setDisabled(!value);
                    if (value) {
                        el.removeClass("disabled");
                    }
                    else {
                        el.addClass("disabled");
                    }
                };
                return TreeMenuItem;
            })(api.dom.DdDtEl);
            menu.TreeMenuItem = TreeMenuItem;
        })(menu = ui.menu || (ui.menu = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var toolbar;
        (function (toolbar) {
            var Toolbar = (function (_super) {
                __extends(Toolbar, _super);
                function Toolbar(className) {
                    var _this = this;
                    _super.call(this, !className ? "toolbar" : className + " toolbar");
                    this.actions = [];
                    this.fold = new toolbar.FoldButton();
                    this.fold.hide();
                    this.appendChild(this.fold);
                    api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                        _this.foldOrExpand();
                    });
                    this.onShown(function (event) { return _this.foldOrExpand(); });
                }
                Toolbar.prototype.addAction = function (action) {
                    this.actions.push(action);
                    this.addElement(new api.ui.button.ActionButton(action));
                };
                Toolbar.prototype.addActions = function (actions) {
                    var _this = this;
                    this.actions = this.actions.concat(actions);
                    actions.forEach(function (action) {
                        _this.addElement(new api.ui.button.ActionButton(action));
                    });
                };
                Toolbar.prototype.removeActions = function () {
                    var _this = this;
                    this.actions.forEach(function (action) {
                        _this.getChildren().forEach(function (element) {
                            if (api.ObjectHelper.iFrameSafeInstanceOf(element, api.ui.button.ActionButton)) {
                                if (action.getLabel() == element.getLabel()) {
                                    _this.removeChild(element);
                                }
                            }
                        });
                    });
                    this.actions = [];
                };
                Toolbar.prototype.getActions = function () {
                    return this.actions;
                };
                Toolbar.prototype.addElement = function (element) {
                    if (this.hasGreedySpacer) {
                        element.addClass('pull-right');
                        element.insertAfterEl(this.fold);
                    }
                    else {
                        element.insertBeforeEl(this.fold);
                    }
                };
                Toolbar.prototype.addGreedySpacer = function () {
                    this.hasGreedySpacer = true;
                };
                Toolbar.prototype.removeGreedySpacer = function () {
                    this.hasGreedySpacer = false;
                };
                Toolbar.prototype.foldOrExpand = function () {
                    if (!this.isVisible()) {
                        return;
                    }
                    var toolbarWidth = this.getEl().getWidth();
                    if (toolbarWidth < this.getVisibleButtonsWidth()) {
                        while (toolbarWidth <= this.getVisibleButtonsWidth() && this.getNextFoldableButton()) {
                            var buttonToHide = this.getNextFoldableButton();
                            var buttonWidth = buttonToHide.getEl().getWidthWithBorder();
                            this.removeChild(buttonToHide);
                            this.fold.push(buttonToHide, buttonWidth);
                            if (!this.fold.isVisible()) {
                                this.fold.show();
                            }
                        }
                    }
                    else {
                        while (!this.fold.isEmpty() && (this.getVisibleButtonsWidth(this.fold.getButtonsCount() > 1) + this.fold.getNextButtonWidth() < toolbarWidth)) {
                            var buttonToShow = this.fold.pop();
                            buttonToShow.insertBeforeEl(this.fold);
                            if (this.fold.isEmpty()) {
                                this.fold.hide();
                            }
                        }
                    }
                    this.fold.setLabel(this.getFoldableButtons().length == 0 ? 'Actions' : 'More');
                };
                Toolbar.prototype.getVisibleButtonsWidth = function (includeFold) {
                    var _this = this;
                    if (includeFold === void 0) { includeFold = true; }
                    return this.getChildren().reduce(function (totalWidth, element) {
                        return totalWidth + (element.isVisible() && (includeFold || element != _this.fold) ? element.getEl().getWidthWithMargin() : 0);
                    }, 0);
                };
                Toolbar.prototype.getNextFoldableButton = function () {
                    return this.getChildren()[this.getChildren().indexOf(this.fold) - 1];
                };
                Toolbar.prototype.getFoldableButtons = function () {
                    return this.getChildren().slice(0, this.getChildren().indexOf(this.fold));
                };
                return Toolbar;
            })(api.dom.DivEl);
            toolbar.Toolbar = Toolbar;
        })(toolbar = ui.toolbar || (ui.toolbar = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var toolbar;
        (function (toolbar) {
            var FoldButton = (function (_super) {
                __extends(FoldButton, _super);
                function FoldButton() {
                    _super.call(this, 'button fold-button');
                    this.widthCache = [];
                    this.span = new api.dom.SpanEl();
                    this.span.setHtml("More");
                    this.appendChild(this.span);
                    this.dropdown = new api.dom.DivEl('dropdown');
                    this.appendChild(this.dropdown);
                }
                FoldButton.prototype.push = function (element, width) {
                    this.dropdown.prependChild(element);
                    this.widthCache.unshift(width);
                };
                FoldButton.prototype.pop = function () {
                    var top = this.dropdown.getFirstChild();
                    this.dropdown.removeChild(top);
                    this.widthCache.shift();
                    return top;
                };
                FoldButton.prototype.setLabel = function (label) {
                    this.span.setHtml(label);
                };
                FoldButton.prototype.getNextButtonWidth = function () {
                    return this.widthCache[0];
                };
                FoldButton.prototype.getButtonsCount = function () {
                    return this.dropdown.getChildren().length;
                };
                FoldButton.prototype.isEmpty = function () {
                    return this.dropdown.getChildren().length == 0;
                };
                return FoldButton;
            })(api.dom.DivEl);
            toolbar.FoldButton = FoldButton;
        })(toolbar = ui.toolbar || (ui.toolbar = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var mask;
        (function (mask) {
            var Mask = (function (_super) {
                __extends(Mask, _super);
                function Mask(itemToMask) {
                    var _this = this;
                    _super.call(this, "mask");
                    this.masked = itemToMask;
                    if (this.masked) {
                        this.onMouseWheel(function (event) {
                            var evt = _this.cloneWheelEvent(event);
                            _this.masked.getHTMLElement().dispatchEvent(evt);
                            if (_this.isBrowserFirefox()) {
                                _this.triggerScroll(event);
                            }
                        });
                        this.masked.onHidden(function (event) {
                            if (event.getTarget() == _this.masked) {
                                _this.hide();
                            }
                        });
                        this.masked.onRemoved(function (event) {
                            _this.remove();
                        });
                        api.dom.WindowDOM.get().onResized(function (event) {
                            if (_this.isVisible()) {
                                _this.positionOver(_this.masked);
                            }
                        }, this);
                    }
                    api.dom.Body.get().appendChild(this);
                }
                Mask.prototype.cloneWheelEvent = function (e) {
                    return api.ObjectHelper.create(WheelEvent, e.type, {
                        bubbles: e.bubbles,
                        cancelable: e.cancelable,
                        cancelBubble: e.cancelBubble,
                        view: e.view,
                        detail: e.detail,
                        screenX: e.screenX,
                        screenY: e.screenY,
                        clientX: e.clientX,
                        clientY: e.clientY,
                        layerX: e.layerX,
                        layerY: e.layerY,
                        deltaX: e.deltaX,
                        deltaY: e.deltaY,
                        deltaZ: e.deltaZ,
                        deltaMode: e.deltaMode,
                        ctrlKey: e.ctrlKey,
                        altKey: e.altKey,
                        shiftKey: e.shiftKey,
                        metaKey: e.metaKey,
                        button: e.button,
                        relatedTarget: e.relatedTarget
                    });
                };
                Mask.prototype.show = function () {
                    _super.prototype.show.call(this);
                    if (this.masked) {
                        this.positionOver(this.masked);
                    }
                };
                Mask.prototype.positionOver = function (masked) {
                    var maskedEl = masked.getEl(), maskEl = this.getEl(), maskedOffset, isMaskedPositioned = maskedEl.getPosition() != 'static', maskedDimensions = {
                        width: maskedEl.getWidthWithBorder() + "px",
                        height: maskedEl.getHeightWithBorder() + "px"
                    };
                    if (masked.contains(this) && isMaskedPositioned) {
                        maskedOffset = {
                            top: 0,
                            left: 0
                        };
                        if (maskedEl.getPosition() == 'absolute') {
                            maskedDimensions = {
                                width: '100%',
                                height: '100%'
                            };
                        }
                    }
                    else {
                        var maskedParent = maskedEl.getOffsetParent(), maskParent = maskEl.getOffsetParent();
                        maskedOffset = maskedEl.getOffsetToParent();
                        if (maskedParent != maskParent) {
                            var maskedParentOffset = new api.dom.ElementHelper(maskedParent).getOffset(), maskParentOffset = new api.dom.ElementHelper(maskParent).getOffset();
                            maskedOffset.left = maskedOffset.left + (maskedParentOffset.left - maskParentOffset.left);
                            maskedOffset.top = maskedOffset.top + (maskedParentOffset.top - maskParentOffset.top);
                        }
                        if (!isMaskedPositioned) {
                            maskedOffset.top += maskedEl.getMarginTop();
                            maskedOffset.left += maskedEl.getMarginLeft();
                        }
                    }
                    this.getEl().setTopPx(maskedOffset.top).setLeftPx(maskedOffset.left).setWidth(maskedDimensions.width).setHeight(maskedDimensions.height);
                };
                Mask.prototype.isBrowserFirefox = function () {
                    return /Firefox/i.test(navigator.userAgent);
                };
                Mask.prototype.triggerScroll = function (event) {
                    wemjq(this.masked.getHTMLElement()).stop().animate({
                        scrollTop: this.masked.getHTMLElement().scrollTop + event.deltaY * 25
                    }, 600 / Math.abs(event.deltaY), 'linear');
                };
                return Mask;
            })(api.dom.DivEl);
            mask.Mask = Mask;
        })(mask = ui.mask || (ui.mask = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var mask;
        (function (mask) {
            var BodyMask = (function (_super) {
                __extends(BodyMask, _super);
                function BodyMask() {
                    _super.call(this);
                    this.addClass("body-mask");
                }
                BodyMask.get = function () {
                    if (!BodyMask.instance) {
                        BodyMask.instance = new BodyMask();
                    }
                    return BodyMask.instance;
                };
                return BodyMask;
            })(mask.Mask);
            mask.BodyMask = BodyMask;
        })(mask = ui.mask || (ui.mask = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var mask;
        (function (mask) {
            var LoadMask = (function (_super) {
                __extends(LoadMask, _super);
                function LoadMask(el) {
                    _super.call(this, el);
                    this.addClass("load-mask");
                    this.splash = new api.dom.DivEl("mask-splash");
                    this.spinner = new api.dom.DivEl("spinner");
                    this.splash.appendChild(this.spinner);
                    this.appendChild(this.splash);
                }
                LoadMask.prototype.show = function () {
                    _super.prototype.show.call(this);
                    this.splash.show();
                    this.centerSplash();
                };
                LoadMask.prototype.hide = function () {
                    this.splash.hide();
                    _super.prototype.hide.call(this);
                };
                LoadMask.prototype.setText = function (text) {
                    if (!text) {
                        if (this.text) {
                            this.text.hide();
                        }
                    }
                    else {
                        if (!this.text) {
                            this.text = new api.dom.SpanEl("text");
                            this.splash.appendChild(this.text);
                        }
                        this.text.getEl().setInnerHtml(text);
                    }
                };
                LoadMask.prototype.getText = function () {
                    return this.text.getEl().getInnerHtml();
                };
                LoadMask.prototype.centerSplash = function () {
                    var loaderEl = this.splash.getEl();
                    loaderEl.setMarginLeft("-" + loaderEl.getWidthWithBorder() / 2 + "px");
                    loaderEl.setMarginTop("-" + loaderEl.getHeightWithBorder() / 2 + "px");
                };
                return LoadMask;
            })(mask.Mask);
            mask.LoadMask = LoadMask;
        })(mask = ui.mask || (ui.mask = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var mask;
        (function (mask) {
            var DragMask = (function (_super) {
                __extends(DragMask, _super);
                function DragMask(itemToMask) {
                    _super.call(this, itemToMask);
                    this.addClass("drag-mask");
                }
                return DragMask;
            })(mask.Mask);
            mask.DragMask = DragMask;
        })(mask = ui.mask || (ui.mask = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var OptionFilterInput = (function (_super) {
                __extends(OptionFilterInput, _super);
                function OptionFilterInput() {
                    _super.call(this, "option-filter-input");
                    this.setPlaceholder("Type to search...");
                }
                OptionFilterInput.prototype.openForTypingAndFocus = function () {
                    this.openForTyping();
                    this.getHTMLElement().focus();
                };
                OptionFilterInput.prototype.openForTyping = function () {
                    this.setPlaceholder("Type to search...");
                    this.getEl().setDisabled(false);
                };
                return OptionFilterInput;
            })(api.ui.text.TextInput);
            selector.OptionFilterInput = OptionFilterInput;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var OptionFilterInputValueChangedEvent = (function () {
                function OptionFilterInputValueChangedEvent(oldValue, newValue, grid) {
                    this.oldValue = oldValue;
                    this.newValue = newValue;
                    this.grid = grid;
                }
                OptionFilterInputValueChangedEvent.prototype.getOldValue = function () {
                    return this.oldValue;
                };
                OptionFilterInputValueChangedEvent.prototype.getNewValue = function () {
                    return this.newValue;
                };
                OptionFilterInputValueChangedEvent.prototype.getGrid = function () {
                    return this.grid;
                };
                return OptionFilterInputValueChangedEvent;
            })();
            selector.OptionFilterInputValueChangedEvent = OptionFilterInputValueChangedEvent;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var OptionSelectedEvent = (function () {
                function OptionSelectedEvent(option, index) {
                    if (index === void 0) { index = -1; }
                    this.option = option;
                    this.index = index;
                }
                OptionSelectedEvent.prototype.getOption = function () {
                    return this.option;
                };
                OptionSelectedEvent.prototype.getIndex = function () {
                    return this.index;
                };
                return OptionSelectedEvent;
            })();
            selector.OptionSelectedEvent = OptionSelectedEvent;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var DropdownHandle = (function (_super) {
                __extends(DropdownHandle, _super);
                function DropdownHandle() {
                    _super.call(this, "dropdown-handle");
                    this.setEnabled(true);
                    this.removeClass('down');
                }
                DropdownHandle.prototype.setEnabled = function (value) {
                    if (value) {
                        this.removeClass('disabled');
                    }
                    else {
                        this.addClass('disabled');
                    }
                };
                DropdownHandle.prototype.down = function () {
                    this.addClass('down');
                };
                DropdownHandle.prototype.up = function () {
                    this.removeClass('down');
                };
                return DropdownHandle;
            })(api.dom.DivEl);
            selector.DropdownHandle = DropdownHandle;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var DropdownGridRowSelectedEvent = (function () {
                function DropdownGridRowSelectedEvent(row) {
                    this.row = row;
                }
                DropdownGridRowSelectedEvent.prototype.getRow = function () {
                    return this.row;
                };
                return DropdownGridRowSelectedEvent;
            })();
            selector.DropdownGridRowSelectedEvent = DropdownGridRowSelectedEvent;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var DropdownGridMultipleSelectionEvent = (function () {
                function DropdownGridMultipleSelectionEvent(rows) {
                    this.rows = rows;
                }
                DropdownGridMultipleSelectionEvent.prototype.getRows = function () {
                    return this.rows;
                };
                return DropdownGridMultipleSelectionEvent;
            })();
            selector.DropdownGridMultipleSelectionEvent = DropdownGridMultipleSelectionEvent;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var DefaultOptionDisplayValueViewer = (function (_super) {
                __extends(DefaultOptionDisplayValueViewer, _super);
                function DefaultOptionDisplayValueViewer() {
                    _super.call(this);
                }
                DefaultOptionDisplayValueViewer.prototype.setObject = function (object) {
                    _super.prototype.setObject.call(this, object);
                    this.getEl().setInnerHtml(object.toString());
                };
                DefaultOptionDisplayValueViewer.prototype.getPreferredHeight = function () {
                    return 34;
                };
                return DefaultOptionDisplayValueViewer;
            })(api.ui.Viewer);
            selector.DefaultOptionDisplayValueViewer = DefaultOptionDisplayValueViewer;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var DropdownGrid = (function () {
                function DropdownGrid(config) {
                    var _this = this;
                    this.rowSelectionListeners = [];
                    this.multipleSelectionListeners = [];
                    this.maxHeight = config.maxHeight || 200;
                    this.optionDisplayValueViewer = config.optionDisplayValueViewer ? new config.optionDisplayValueViewer['constructor']() : new selector.DefaultOptionDisplayValueViewer();
                    this.filter = config.filter;
                    this.dataIdProperty = config.dataIdProperty || "value";
                    this.maxHeight = config.maxHeight;
                    this.width = config.width;
                    this.multipleSelections = config.multipleSelections || false;
                    var columnFormatter = function (row, cell, value, columnDef, dataContext) {
                        _this.optionDisplayValueViewer.setObject(value);
                        return _this.optionDisplayValueViewer.toString();
                    };
                    var columns = [
                        new api.ui.grid.GridColumnBuilder().setId("option").setName("Options").setField("displayValue").setFormatter(columnFormatter).build()
                    ];
                    var options = new api.ui.grid.GridOptionsBuilder().setWidth(this.width).setHeight(this.maxHeight).setHideColumnHeaders(true).setEnableColumnReorder(false).setFullWidthRows(true).setForceFitColumns(true).setRowHeight(this.optionDisplayValueViewer.getPreferredHeight()).setCheckableRows(this.multipleSelections).setMultiSelect(this.multipleSelections).setDataIdProperty(this.dataIdProperty).build();
                    this.gridData = new api.ui.grid.DataView();
                    if (this.filter) {
                        this.gridData.setFilter(this.filter);
                    }
                    this.grid = new api.ui.grid.Grid(this.gridData, columns, options);
                    this.grid.addClass("options-container");
                    this.grid.getEl().setPosition("absolute");
                    this.grid.hide();
                    this.grid.setSelectionModel(new Slick.RowSelectionModel({ selectActiveRow: false }));
                    this.grid.subscribeOnClick(function (e, args) {
                        _this.notifyRowSelection(args.row);
                        e.preventDefault();
                        e.stopPropagation();
                        return false;
                    });
                    this.grid.subscribeOnSelectedRowsChanged(function (e, args) {
                        _this.notifyMultipleSelection(args.rows);
                    });
                    this.gridData.onRowsChanged(function (e, args) {
                    });
                    this.gridData.onRowCountChanged(function (e, args) {
                    });
                }
                DropdownGrid.prototype.getElement = function () {
                    return this.grid;
                };
                DropdownGrid.prototype.renderGrid = function () {
                    this.grid.render();
                };
                DropdownGrid.prototype.isVisible = function () {
                    return this.grid.isVisible();
                };
                DropdownGrid.prototype.show = function () {
                    this.grid.show();
                };
                DropdownGrid.prototype.hide = function () {
                    this.grid.hide();
                };
                DropdownGrid.prototype.setOptions = function (options) {
                    this.gridData.setItems(options, this.dataIdProperty);
                };
                DropdownGrid.prototype.removeAllOptions = function () {
                    this.gridData.setItems([]);
                };
                DropdownGrid.prototype.addOption = function (option) {
                    this.gridData.addItem(option);
                };
                DropdownGrid.prototype.hasOptions = function () {
                    return this.gridData.getLength() > 0;
                };
                DropdownGrid.prototype.getSelectedOptionCount = function () {
                    return this.grid.getSelectedRows().length;
                };
                DropdownGrid.prototype.getOptionCount = function () {
                    return this.gridData.getLength();
                };
                DropdownGrid.prototype.getOptions = function () {
                    return this.gridData.getItems();
                };
                DropdownGrid.prototype.getOptionByValue = function (value) {
                    return this.gridData.getItemById(value);
                };
                DropdownGrid.prototype.getOptionByRow = function (rowIndex) {
                    return this.gridData.getItem(rowIndex);
                };
                DropdownGrid.prototype.setFilterArgs = function (args) {
                    this.gridData.setFilterArgs(args);
                    this.gridData.refresh();
                };
                DropdownGrid.prototype.setTopPx = function (value) {
                    this.grid.getEl().setTopPx(value);
                };
                DropdownGrid.prototype.setWidthPx = function (value) {
                    this.grid.getEl().setWidthPx(value);
                };
                DropdownGrid.prototype.adjustGridHeight = function () {
                    var gridEl = this.grid.getEl();
                    var rowsHeight = this.getOptionCount() * this.optionDisplayValueViewer.getPreferredHeight();
                    if (rowsHeight < this.maxHeight) {
                        var borderWidth = gridEl.getBorderTopWidth() + gridEl.getBorderBottomWidth();
                        gridEl.setHeightPx(rowsHeight + borderWidth);
                        this.grid.getOptions().setAutoHeight(true);
                    }
                    else if (gridEl.getHeight() < this.maxHeight) {
                        gridEl.setHeightPx(this.maxHeight);
                        this.grid.getOptions().setAutoHeight(false);
                    }
                    this.grid.resizeCanvas();
                };
                DropdownGrid.prototype.markSelections = function (selectedOptions, ignoreEmpty) {
                    var _this = this;
                    if (ignoreEmpty === void 0) { ignoreEmpty = false; }
                    var stylesHash = {};
                    var rows = [];
                    selectedOptions.forEach(function (selectedOption) {
                        var row = _this.gridData.getRowById(selectedOption.value);
                        rows.push(row);
                        stylesHash[row] = { option: "selected" };
                    });
                    this.grid.setCellCssStyles("selected", stylesHash);
                    if (!(rows.length === 0 && ignoreEmpty)) {
                        this.grid.setSelectedRows(rows);
                    }
                };
                DropdownGrid.prototype.markReadOnly = function (selectedOptions) {
                    var _this = this;
                    var stylesHash = {};
                    var rows = [];
                    selectedOptions.forEach(function (selectedOption) {
                        if (selectedOption.readOnly) {
                            var row = _this.gridData.getRowById(selectedOption.value);
                            rows.push(row);
                            stylesHash[row] = { _checkbox_selector: "readonly", option: "readonly" };
                        }
                    });
                    this.grid.setCellCssStyles("readonly", stylesHash);
                };
                DropdownGrid.prototype.hasActiveRow = function () {
                    var activeCell = this.grid.getActiveCell();
                    if (activeCell) {
                        return true;
                    }
                    else {
                        return false;
                    }
                };
                DropdownGrid.prototype.getActiveRow = function () {
                    var activeCell = this.grid.getActiveCell();
                    if (activeCell) {
                        return activeCell.row;
                    }
                    else {
                        return -1;
                    }
                };
                DropdownGrid.prototype.nagivateToFirstRow = function () {
                    this.grid.setActiveCell(0, 0);
                };
                DropdownGrid.prototype.navigateToFirstRowIfNotActive = function () {
                    if (!this.grid.getActiveCell()) {
                        this.grid.setActiveCell(0, 0);
                    }
                };
                DropdownGrid.prototype.navigateToNextRow = function () {
                    this.grid.navigateDown();
                };
                DropdownGrid.prototype.navigateToPreviousRow = function () {
                    this.grid.navigateUp();
                };
                DropdownGrid.prototype.resetActiveSelection = function () {
                    if (this.grid.getActiveCell()) {
                        this.grid.resetActiveCell();
                    }
                };
                DropdownGrid.prototype.toggleRowSelection = function (row, isMaximumReached) {
                    if (isMaximumReached === void 0) { isMaximumReached = false; }
                    var rows = this.grid.getSelectedRows();
                    var oldRows = rows.join();
                    var index = rows.indexOf(row);
                    if (index >= 0) {
                        rows.splice(index, 1);
                    }
                    else if (!isMaximumReached) {
                        rows.push(row);
                    }
                    if (oldRows !== rows.join()) {
                        this.grid.setSelectedRows(rows);
                    }
                };
                DropdownGrid.prototype.onRowSelection = function (listener) {
                    this.rowSelectionListeners.push(listener);
                };
                DropdownGrid.prototype.unRowSelection = function (listener) {
                    this.rowSelectionListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                DropdownGrid.prototype.onMultipleSelection = function (listener) {
                    this.multipleSelectionListeners.push(listener);
                };
                DropdownGrid.prototype.unMultipleSelection = function (listener) {
                    this.multipleSelectionListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                DropdownGrid.prototype.applyMultipleSelection = function () {
                    var _this = this;
                    this.grid.getSelectedRows().forEach(function (row) {
                        _this.notifyRowSelection(row);
                    });
                };
                DropdownGrid.prototype.notifyRowSelection = function (rowSelected) {
                    var event = new selector.DropdownGridRowSelectedEvent(rowSelected);
                    this.rowSelectionListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                DropdownGrid.prototype.notifyMultipleSelection = function (rowsSelected) {
                    var event = new selector.DropdownGridMultipleSelectionEvent(rowsSelected);
                    this.multipleSelectionListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return DropdownGrid;
            })();
            selector.DropdownGrid = DropdownGrid;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var DropdownList = (function () {
                function DropdownList(config) {
                    this.emptyDropdown = new api.dom.DivEl("empty-options");
                    this.emptyDropdown.getEl().setInnerHtml("No matching items");
                    this.emptyDropdown.hide();
                    this.dropdownGrid = new selector.DropdownGrid(this.assembleGridConfig(config));
                }
                DropdownList.prototype.assembleGridConfig = function (config) {
                    return {
                        maxHeight: config.maxHeight,
                        width: config.width,
                        optionDisplayValueViewer: config.optionDisplayValueViewer,
                        filter: config.filter,
                        dataIdProperty: config.dataIdProperty
                    };
                };
                DropdownList.prototype.getDropdownGrid = function () {
                    return this.dropdownGrid;
                };
                DropdownList.prototype.renderDropdownGrid = function () {
                    this.dropdownGrid.renderGrid();
                };
                DropdownList.prototype.getEmptyDropdown = function () {
                    return this.emptyDropdown;
                };
                DropdownList.prototype.isDropdownShown = function () {
                    return this.emptyDropdown.isVisible() || this.dropdownGrid.isVisible();
                };
                DropdownList.prototype.setOptions = function (options, selectedOptions) {
                    if (selectedOptions === void 0) { selectedOptions = []; }
                    this.dropdownGrid.setOptions(options);
                    if (this.dropdownGrid.isVisible()) {
                        this.showDropdown([]);
                    }
                };
                DropdownList.prototype.removeAllOptions = function () {
                    this.dropdownGrid.removeAllOptions();
                };
                DropdownList.prototype.addOption = function (option) {
                    this.dropdownGrid.addOption(option);
                };
                DropdownList.prototype.hasOptions = function () {
                    return this.dropdownGrid.hasOptions();
                };
                DropdownList.prototype.getOptionCount = function () {
                    return this.dropdownGrid.getOptionCount();
                };
                DropdownList.prototype.getOptions = function () {
                    return this.dropdownGrid.getOptions();
                };
                DropdownList.prototype.getOptionByValue = function (value) {
                    return this.dropdownGrid.getOptionByValue(value);
                };
                DropdownList.prototype.getOptionByRow = function (rowIndex) {
                    return this.dropdownGrid.getOptionByRow(rowIndex);
                };
                DropdownList.prototype.setFilterArgs = function (args) {
                    this.dropdownGrid.setFilterArgs(args);
                };
                DropdownList.prototype.showDropdown = function (selectedOptions) {
                    if (this.hasOptions()) {
                        this.emptyDropdown.hide();
                        this.dropdownGrid.show();
                        this.dropdownGrid.adjustGridHeight();
                        if (!!selectedOptions) {
                            this.dropdownGrid.markSelections(selectedOptions.splice(1));
                        }
                    }
                    else {
                        this.dropdownGrid.hide();
                        this.emptyDropdown.getEl().setInnerHtml("No matching items");
                        this.emptyDropdown.show();
                    }
                };
                DropdownList.prototype.hideDropdown = function () {
                    this.emptyDropdown.hide();
                    this.dropdownGrid.hide();
                };
                DropdownList.prototype.setEmptyDropdownText = function (label) {
                    if (this.isDropdownShown()) {
                        this.dropdownGrid.hide();
                        this.emptyDropdown.getEl().setInnerHtml(label);
                        this.emptyDropdown.show();
                    }
                };
                DropdownList.prototype.setTopPx = function (value) {
                    this.dropdownGrid.setTopPx(value);
                    this.emptyDropdown.getEl().setTopPx(value);
                };
                DropdownList.prototype.setWidth = function (value) {
                    this.dropdownGrid.setWidthPx(value);
                };
                DropdownList.prototype.hasActiveRow = function () {
                    return this.dropdownGrid.hasActiveRow();
                };
                DropdownList.prototype.getActiveRow = function () {
                    return this.dropdownGrid.getActiveRow();
                };
                DropdownList.prototype.nagivateToFirstRow = function () {
                    this.dropdownGrid.nagivateToFirstRow();
                };
                DropdownList.prototype.navigateToFirstRowIfNotActive = function () {
                    this.dropdownGrid.navigateToFirstRowIfNotActive();
                };
                DropdownList.prototype.navigateToNextRow = function () {
                    this.dropdownGrid.navigateToNextRow();
                };
                DropdownList.prototype.navigateToPreviousRow = function () {
                    this.dropdownGrid.navigateToPreviousRow();
                };
                DropdownList.prototype.markSelections = function (selectedOptions, ignoreEmpty) {
                    if (ignoreEmpty === void 0) { ignoreEmpty = false; }
                    this.dropdownGrid.markSelections(selectedOptions, ignoreEmpty);
                };
                DropdownList.prototype.onRowSelection = function (listener) {
                    this.dropdownGrid.onRowSelection(listener);
                };
                DropdownList.prototype.unRowSelection = function (listener) {
                    this.dropdownGrid.unRowSelection(listener);
                };
                return DropdownList;
            })();
            selector.DropdownList = DropdownList;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var DropdownExpandedEvent = (function () {
                function DropdownExpandedEvent(dropDownElement, expanded) {
                    this.dropDownElement = dropDownElement;
                    this.expanded = expanded;
                }
                DropdownExpandedEvent.prototype.isExpanded = function () {
                    return this.expanded;
                };
                DropdownExpandedEvent.prototype.getDropdownElement = function () {
                    return this.dropDownElement;
                };
                return DropdownExpandedEvent;
            })();
            selector.DropdownExpandedEvent = DropdownExpandedEvent;
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var list;
            (function (list) {
                var ListBox = (function (_super) {
                    __extends(ListBox, _super);
                    function ListBox(className) {
                        _super.call(this, className);
                        this.items = [];
                        this.itemViews = {};
                        this.itemsAddedListeners = [];
                        this.itemsRemovedListeners = [];
                    }
                    ListBox.prototype.setItems = function (items) {
                        if (this.items.length > 0) {
                            this.notifyItemsRemoved(this.items);
                        }
                        this.items = items;
                        if (items.length > 0) {
                            this.layoutList(items);
                            this.notifyItemsAdded(items);
                        }
                        else {
                            this.clearItems();
                        }
                    };
                    ListBox.prototype.getItems = function () {
                        return this.items;
                    };
                    ListBox.prototype.getItem = function (id) {
                        for (var i = 0; i < this.items.length; i++) {
                            var item = this.items[i];
                            if (this.getItemId(item) == id) {
                                return item;
                            }
                        }
                        return undefined;
                    };
                    ListBox.prototype.clearItems = function () {
                        if (this.items.length > 0) {
                            this.notifyItemsRemoved(this.items);
                        }
                        this.items.length = 0;
                        this.layoutList(this.items);
                    };
                    ListBox.prototype.addItem = function () {
                        var _this = this;
                        var items = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            items[_i - 0] = arguments[_i];
                        }
                        this.items = this.items.concat(items);
                        items.forEach(function (item) {
                            _this.addItemView(item);
                        });
                        if (items.length > 0) {
                            this.notifyItemsAdded(items);
                        }
                    };
                    ListBox.prototype.addItemReadOnly = function () {
                        var _this = this;
                        var items = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            items[_i - 0] = arguments[_i];
                        }
                        this.items = this.items.concat(items);
                        items.forEach(function (item) {
                            _this.addItemView(item, true);
                        });
                        if (items.length > 0) {
                            this.notifyItemsAdded(items);
                        }
                    };
                    ListBox.prototype.removeItem = function () {
                        var _this = this;
                        var items = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            items[_i - 0] = arguments[_i];
                        }
                        var itemsRemoved = [];
                        this.items = this.items.filter(function (item) {
                            for (var i = 0; i < items.length; i++) {
                                if (_this.getItemId(item) == _this.getItemId(items[i])) {
                                    _this.removeItemView(item);
                                    itemsRemoved.push(item);
                                    return false;
                                }
                            }
                            return true;
                        });
                        if (itemsRemoved.length > 0) {
                            this.notifyItemsRemoved(itemsRemoved);
                        }
                    };
                    ListBox.prototype.getItemCount = function () {
                        return this.items.length;
                    };
                    ListBox.prototype.createItemView = function (item, readOnly) {
                        throw new Error("You must override createListItem to create views for list items");
                    };
                    ListBox.prototype.getItemId = function (item) {
                        throw new Error("You must override getItemId to find item views by items");
                    };
                    ListBox.prototype.getItemView = function (item) {
                        return this.itemViews[this.getItemId(item)];
                    };
                    ListBox.prototype.getItemViews = function () {
                        var _this = this;
                        return this.getItems().map(function (item) { return _this.getItemView(item); });
                    };
                    ListBox.prototype.refreshList = function () {
                        this.layoutList(this.items);
                    };
                    ListBox.prototype.layoutList = function (items) {
                        this.removeChildren();
                        for (var i = 0; i < items.length; i++) {
                            this.addItemView(items[i]);
                        }
                    };
                    ListBox.prototype.removeItemView = function (item) {
                        var itemView = this.itemViews[this.getItemId(item)];
                        if (itemView) {
                            this.removeChild(itemView);
                        }
                    };
                    ListBox.prototype.addItemView = function (item, readOnly) {
                        if (readOnly === void 0) { readOnly = false; }
                        if (readOnly) {
                            var itemView = this.createItemView(item, readOnly);
                        }
                        else {
                            var itemView = this.createItemView(item, false);
                        }
                        this.itemViews[this.getItemId(item)] = itemView;
                        this.appendChild(itemView);
                    };
                    ListBox.prototype.onItemsAdded = function (listener) {
                        this.itemsAddedListeners.push(listener);
                    };
                    ListBox.prototype.unItemsAdded = function (listener) {
                        this.itemsAddedListeners = this.itemsAddedListeners.filter(function (current) {
                            return current !== listener;
                        });
                    };
                    ListBox.prototype.notifyItemsAdded = function (items) {
                        this.itemsAddedListeners.forEach(function (listener) {
                            listener(items);
                        });
                    };
                    ListBox.prototype.onItemsRemoved = function (listener) {
                        this.itemsRemovedListeners.push(listener);
                    };
                    ListBox.prototype.unItemsRemoved = function (listener) {
                        this.itemsRemovedListeners = this.itemsRemovedListeners.filter(function (current) {
                            return current !== listener;
                        });
                    };
                    ListBox.prototype.notifyItemsRemoved = function (items) {
                        this.itemsRemovedListeners.forEach(function (listener) {
                            listener(items);
                        });
                    };
                    return ListBox;
                })(api.dom.UlEl);
                list.ListBox = ListBox;
            })(list = selector.list || (selector.list = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (combobox) {
                var ComboBoxOptionFilterInput = (function (_super) {
                    __extends(ComboBoxOptionFilterInput, _super);
                    function ComboBoxOptionFilterInput() {
                        _super.call(this);
                    }
                    ComboBoxOptionFilterInput.prototype.setMaximumReached = function () {
                        this.setPlaceholder("Maximum reached");
                        this.getEl().setDisabled(true);
                    };
                    ComboBoxOptionFilterInput.prototype.getWidth = function () {
                        return this.getEl().getWidth();
                    };
                    return ComboBoxOptionFilterInput;
                })(api.ui.selector.OptionFilterInput);
                combobox.ComboBoxOptionFilterInput = ComboBoxOptionFilterInput;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (combobox) {
                var ComboBoxDropdown = (function (_super) {
                    __extends(ComboBoxDropdown, _super);
                    function ComboBoxDropdown(config) {
                        _super.call(this, config);
                    }
                    ComboBoxDropdown.prototype.assembleGridConfig = function (config) {
                        return {
                            maxHeight: config.maxHeight,
                            width: config.width,
                            optionDisplayValueViewer: config.optionDisplayValueViewer,
                            filter: config.filter,
                            dataIdProperty: config.dataIdProperty,
                            multipleSelections: config.multipleSelections
                        };
                    };
                    ComboBoxDropdown.prototype.setOptions = function (options, selectedOptions) {
                        if (selectedOptions === void 0) { selectedOptions = []; }
                        selectedOptions.forEach(function (selectedOption) {
                            if (selectedOption.readOnly) {
                                for (var i = 0; i < options.length; i++) {
                                    if (selectedOption.value == options[i].value) {
                                        options[i].readOnly = true;
                                        break;
                                    }
                                }
                            }
                        });
                        this.getDropdownGrid().setOptions(options);
                        if (this.isDropdownShown()) {
                            this.showDropdown(selectedOptions);
                        }
                    };
                    ComboBoxDropdown.prototype.showDropdown = function (selectedOptions) {
                        if (this.hasOptions()) {
                            this.getEmptyDropdown().hide();
                            this.getDropdownGrid().show();
                            this.getDropdownGrid().adjustGridHeight();
                            this.getDropdownGrid().markSelections(selectedOptions);
                            this.getDropdownGrid().markReadOnly(selectedOptions);
                        }
                        else {
                            this.getDropdownGrid().hide();
                            this.getEmptyDropdown().getEl().setInnerHtml("No matching items");
                            this.getEmptyDropdown().show();
                        }
                    };
                    ComboBoxDropdown.prototype.getSelectedOptionCount = function () {
                        return this.getDropdownGrid().getSelectedOptionCount();
                    };
                    ComboBoxDropdown.prototype.toggleRowSelection = function (row, isMaximumReached) {
                        if (isMaximumReached === void 0) { isMaximumReached = false; }
                        this.getDropdownGrid().toggleRowSelection(row, isMaximumReached);
                    };
                    ComboBoxDropdown.prototype.resetActiveSelection = function () {
                        this.getDropdownGrid().resetActiveSelection();
                    };
                    ComboBoxDropdown.prototype.applyMultipleSelection = function () {
                        this.getDropdownGrid().applyMultipleSelection();
                    };
                    ComboBoxDropdown.prototype.onMultipleSelection = function (listener) {
                        this.getDropdownGrid().onMultipleSelection(listener);
                    };
                    ComboBoxDropdown.prototype.unMultipleSelection = function (listener) {
                        this.getDropdownGrid().unMultipleSelection(listener);
                    };
                    return ComboBoxDropdown;
                })(selector.DropdownList);
                combobox.ComboBoxDropdown = ComboBoxDropdown;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (_combobox) {
                var OptionSelectedEvent = api.ui.selector.OptionSelectedEvent;
                var OptionFilterInputValueChangedEvent = api.ui.selector.OptionFilterInputValueChangedEvent;
                var DropdownHandle = api.ui.selector.DropdownHandle;
                var DelayedFunctionCall = api.util.DelayedFunctionCall;
                var Button = api.ui.button.Button;
                var ComboBox = (function (_super) {
                    __extends(ComboBox, _super);
                    function ComboBox(name, config) {
                        var _this = this;
                        _super.call(this, "div", "combobox");
                        this.setNextInputFocusWhenMaxReached = true;
                        this.ignoreNextFocus = false;
                        this.minWidth = -1;
                        this.optionSelectedListeners = [];
                        this.optionFilterInputValueChangedListeners = [];
                        this.expandedListeners = [];
                        this.active = false;
                        this.getEl().setAttribute("name", name);
                        this.hideComboBoxWhenMaxReached = config.hideComboBoxWhenMaxReached;
                        if (config.setNextInputFocusWhenMaxReached !== undefined) {
                            this.setNextInputFocusWhenMaxReached = config.setNextInputFocusWhenMaxReached;
                        }
                        if (config.selectedOptionsView != null) {
                            this.selectedOptionsView = config.selectedOptionsView;
                            this.selectedOptionsView.setMaximumOccurrences(config.maximumOccurrences != null ? config.maximumOccurrences : 0);
                        }
                        if (config.iconUrl) {
                            this.icon = new api.dom.ImgEl(config.iconUrl, "input-icon");
                            this.appendChild(this.icon);
                        }
                        if (config.minWidth) {
                            this.minWidth = config.minWidth;
                        }
                        this.input = new _combobox.ComboBoxOptionFilterInput();
                        this.appendChild(this.input);
                        this.delayedInputValueChangedHandling = config.delayedInputValueChangedHandling || 0;
                        this.delayedHandleInputValueChangedFnCall = new DelayedFunctionCall(this.handleInputValueChanged, this, this.delayedInputValueChangedHandling);
                        this.dropdownHandle = new DropdownHandle();
                        this.appendChild(this.dropdownHandle);
                        if (this.selectedOptionsView && (config.maximumOccurrences != 1)) {
                            this.applySelectionsButton = new Button("Apply");
                            this.applySelectionsButton.addClass('small apply-button');
                            this.applySelectionsButton.hide();
                            this.appendChild(this.applySelectionsButton);
                        }
                        this.comboBoxDropdown = new _combobox.ComboBoxDropdown({
                            maxHeight: 200,
                            width: this.input.getWidth(),
                            optionDisplayValueViewer: config.optionDisplayValueViewer,
                            filter: config.filter,
                            dataIdProperty: config.dataIdProperty,
                            multipleSelections: (this.selectedOptionsView && (config.maximumOccurrences != 1))
                        });
                        this.appendChild(this.comboBoxDropdown.getEmptyDropdown());
                        this.appendChild(this.comboBoxDropdown.getDropdownGrid().getElement());
                        this.setupListeners();
                        this.onRendered(function (event) {
                            _this.doUpdateDropdownTopPositionAndWidth();
                        });
                    }
                    ComboBox.prototype.doUpdateDropdownTopPositionAndWidth = function () {
                        var inputEl = this.input.getEl();
                        this.comboBoxDropdown.setTopPx(inputEl.getHeightWithBorder() - inputEl.getBorderBottomWidth());
                        this.comboBoxDropdown.setWidth(Math.max(inputEl.getWidthWithBorder(), this.minWidth));
                    };
                    ComboBox.prototype.giveFocus = function () {
                        return this.input.giveFocus();
                    };
                    ComboBox.prototype.giveInputFocus = function () {
                        this.input.setReadOnly(false);
                        this.input.giveFocus();
                    };
                    ComboBox.prototype.isDropdownShown = function () {
                        return this.comboBoxDropdown.isDropdownShown();
                    };
                    ComboBox.prototype.showDropdown = function () {
                        this.doUpdateDropdownTopPositionAndWidth();
                        this.notifyExpanded(true);
                        this.comboBoxDropdown.showDropdown(this.getSelectedOptions());
                        this.dropdownHandle.down();
                        this.comboBoxDropdown.renderDropdownGrid();
                        this.input.setReadOnly(true);
                        this.setOnBlurListener();
                    };
                    ComboBox.prototype.setEmptyDropdownText = function (label) {
                        this.comboBoxDropdown.setEmptyDropdownText(label);
                    };
                    ComboBox.prototype.hideDropdown = function () {
                        this.dropdownHandle.up();
                        this.comboBoxDropdown.hideDropdown();
                        if (this.applySelectionsButton) {
                            this.applySelectionsButton.hide();
                        }
                        this.input.setReadOnly(false);
                    };
                    ComboBox.prototype.setOptions = function (options) {
                        this.comboBoxDropdown.setOptions(options, this.getSelectedOptions());
                    };
                    ComboBox.prototype.addOption = function (option) {
                        this.comboBoxDropdown.addOption(option);
                    };
                    ComboBox.prototype.setIgnoreNextFocus = function (value) {
                        this.ignoreNextFocus = value;
                        return this;
                    };
                    ComboBox.prototype.isIgnoreNextFocus = function () {
                        return this.ignoreNextFocus;
                    };
                    ComboBox.prototype.loadOptionsAfterShowDropdown = function () {
                        return api.util.PromiseHelper.newResolvedVoidPromise();
                    };
                    ComboBox.prototype.hasOptions = function () {
                        return this.comboBoxDropdown.hasOptions();
                    };
                    ComboBox.prototype.getOptionCount = function () {
                        return this.comboBoxDropdown.getOptionCount();
                    };
                    ComboBox.prototype.getOptions = function () {
                        return this.comboBoxDropdown.getOptions();
                    };
                    ComboBox.prototype.getOptionByValue = function (value) {
                        return this.comboBoxDropdown.getOptionByValue(value);
                    };
                    ComboBox.prototype.getOptionByRow = function (rowIndex) {
                        return this.comboBoxDropdown.getOptionByRow(rowIndex);
                    };
                    ComboBox.prototype.setFilterArgs = function (args) {
                        this.comboBoxDropdown.setFilterArgs(args);
                    };
                    ComboBox.prototype.setValue = function (value, silent) {
                        if (silent === void 0) { silent = false; }
                        var option = this.getOptionByValue(value);
                        if (option != null) {
                            this.selectOption(option, silent);
                        }
                        return this;
                    };
                    ComboBox.prototype.setValues = function (values, silent) {
                        var _this = this;
                        if (silent === void 0) { silent = false; }
                        values.forEach(function (value) {
                            _this.setValue(value, silent);
                        });
                    };
                    ComboBox.prototype.handleRowSelected = function (index) {
                        var option = this.getOptionByRow(index);
                        if (option != null && !option.readOnly) {
                            if (!this.isOptionSelected(option)) {
                                this.selectOption(option);
                            }
                            else {
                                this.deselectOption(option);
                            }
                        }
                    };
                    ComboBox.prototype.isSelectionChanged = function () {
                        var _this = this;
                        var optionsMap = this.getDisplayedOptions().map(function (x) {
                            return x.value;
                        }).join();
                        var selectedOptions = this.getSelectedOptions();
                        var filteredOption = [], gridOptions = [];
                        for (var k in selectedOptions) {
                            if (optionsMap.search(selectedOptions[k].value) >= 0) {
                                filteredOption.push(selectedOptions[k].value);
                            }
                        }
                        this.comboBoxDropdown.getDropdownGrid().getElement().getSelectedRows().forEach(function (row) {
                            gridOptions.push(_this.comboBoxDropdown.getDropdownGrid().getOptionByRow(row).value);
                        });
                        return (filteredOption.length !== gridOptions.length) || (filteredOption.sort().join() !== gridOptions.sort().join());
                    };
                    ComboBox.prototype.selectRowOrApplySelection = function (index) {
                        if (this.applySelectionsButton && this.applySelectionsButton.isVisible()) {
                            this.clearSelection(true);
                            this.comboBoxDropdown.applyMultipleSelection();
                            this.input.setValue("");
                            this.hideDropdown();
                        }
                        else {
                            this.handleRowSelected(index);
                            this.input.setValue("");
                        }
                    };
                    ComboBox.prototype.selectOption = function (option, silent) {
                        if (silent === void 0) { silent = false; }
                        api.util.assertNotNull(option, "option cannot be null");
                        if (this.isOptionSelected(option)) {
                            return;
                        }
                        var added = this.selectedOptionsView.addOption(option);
                        if (!added) {
                            return;
                        }
                        var index = this.countSelectedOptions() - 1;
                        this.comboBoxDropdown.markSelections(this.getSelectedOptions());
                        this.hideDropdown();
                        this.addClass("followed-by-options");
                        if (this.maximumOccurrencesReached()) {
                            this.input.setMaximumReached();
                            if (this.setNextInputFocusWhenMaxReached && !this.ignoreNextFocus) {
                                api.dom.FormEl.moveFocusToNextFocusable(this.input);
                            }
                            this.dropdownHandle.setEnabled(false);
                        }
                        if (!silent) {
                            this.notifyOptionSelected(option, index);
                        }
                        if (this.maximumOccurrencesReached() && this.hideComboBoxWhenMaxReached) {
                            this.hide();
                        }
                        this.ignoreNextFocus = false;
                    };
                    ComboBox.prototype.isOptionSelected = function (option) {
                        return this.selectedOptionsView.isSelected(option);
                    };
                    ComboBox.prototype.deselectOption = function (option, silent) {
                        if (silent === void 0) { silent = false; }
                        api.util.assertNotNull(option, "option cannot be null");
                        if (!this.isOptionSelected(option)) {
                            return;
                        }
                        this.selectedOptionsView.removeOption(option, silent);
                        this.comboBoxDropdown.markSelections(this.getSelectedOptions());
                        this.hideDropdown();
                        this.input.openForTypingAndFocus();
                        this.dropdownHandle.setEnabled(true);
                        if (!this.maximumOccurrencesReached() && this.hideComboBoxWhenMaxReached) {
                            this.show();
                        }
                    };
                    ComboBox.prototype.clearSelection = function (ignoreEmpty, giveInputFocus, forceClear) {
                        var _this = this;
                        if (ignoreEmpty === void 0) { ignoreEmpty = false; }
                        if (giveInputFocus === void 0) { giveInputFocus = true; }
                        if (forceClear === void 0) { forceClear = false; }
                        var optionsMap = this.getDisplayedOptions().map(function (x) { return x.value; }).join();
                        var selectedOptions = this.getSelectedOptions();
                        selectedOptions.forEach(function (option) {
                            if (forceClear) {
                                _this.selectedOptionsView.removeOption(option, true);
                            }
                            else {
                                var filteredOption = optionsMap.search(option.value) >= 0 ? option : undefined;
                                if (filteredOption && !filteredOption.readOnly) {
                                    _this.selectedOptionsView.removeOption(option, true);
                                }
                            }
                        });
                        this.comboBoxDropdown.markSelections([], ignoreEmpty);
                        if (giveInputFocus) {
                            this.input.openForTypingAndFocus();
                        }
                        else {
                            this.input.openForTyping();
                        }
                        this.dropdownHandle.setEnabled(true);
                        if (this.hideComboBoxWhenMaxReached) {
                            this.show();
                        }
                    };
                    ComboBox.prototype.getSelectedOptions = function () {
                        if (this.selectedOptionsView) {
                            return this.selectedOptionsView.getSelectedOptions().map(function (selectedOption) {
                                return selectedOption.getOption();
                            });
                        }
                        else {
                            throw new Error("Not supported yet");
                        }
                    };
                    ComboBox.prototype.getDisplayedOptions = function () {
                        var displayedOptions = [];
                        for (var row = 0; row < this.comboBoxDropdown.getOptionCount(); row++) {
                            var option = this.getOptionByRow(row);
                            if (option) {
                                displayedOptions.push(option);
                            }
                        }
                        return displayedOptions;
                    };
                    ComboBox.prototype.getValue = function () {
                        if (this.selectedOptionsView) {
                            return this.getSelectedOptions().map(function (item) { return item.value; }).join(';');
                        }
                        else {
                            throw new Error("Not supported yet");
                        }
                    };
                    ComboBox.prototype.countSelectedOptions = function () {
                        if (this.selectedOptionsView) {
                            return this.selectedOptionsView.count();
                        }
                        else {
                            throw new Error("Not supported yet");
                        }
                    };
                    ComboBox.prototype.maximumOccurrencesReached = function () {
                        api.util.assert(this.selectedOptionsView != null, "No point of calling maximumOccurrencesReached when no multiple selections are enabled");
                        return this.selectedOptionsView.maximumOccurrencesReached();
                    };
                    ComboBox.prototype.maximumSelectionsReached = function () {
                        if (this.selectedOptionsView && this.selectedOptionsView.getMaximumOccurrences() !== 0) {
                            var totalSelected = this.comboBoxDropdown.getSelectedOptionCount();
                            var optionsMap = this.getDisplayedOptions().map(function (x) { return x.value; }).join();
                            totalSelected += this.getSelectedOptions().filter(function (option) { return (optionsMap.search(option.value) < 0); }).length;
                            return this.selectedOptionsView.getMaximumOccurrences() <= totalSelected;
                        }
                        else {
                            return false;
                        }
                    };
                    ComboBox.prototype.setInputIconUrl = function (iconUrl) {
                        if (!this.icon) {
                            this.icon = new api.dom.ImgEl();
                            this.icon.addClass("input-icon");
                            this.icon.insertBeforeEl(this.input);
                        }
                        this.icon.getEl().setSrc(iconUrl);
                    };
                    ComboBox.prototype.getInput = function () {
                        return this.input;
                    };
                    ComboBox.prototype.setupListeners = function () {
                        var _this = this;
                        this.onClicked(function (event) {
                            _this.setOnBlurListener();
                        });
                        this.onScroll(function (event) {
                            event.stopPropagation();
                        });
                        this.input.onClicked(function (event) {
                            _this.input.setReadOnly(false);
                        });
                        this.comboBoxDropdown.onRowSelection(function (event) {
                            _this.handleRowSelected(event.getRow());
                        });
                        this.dropdownHandle.onClicked(function (event) {
                            event.preventDefault();
                            event.stopPropagation();
                            if (!_this.maximumOccurrencesReached()) {
                                if (_this.isDropdownShown()) {
                                    _this.hideDropdown();
                                    _this.giveInputFocus();
                                }
                                else {
                                    _this.showDropdown();
                                    _this.loadOptionsAfterShowDropdown().then(function () {
                                        _this.giveInputFocus();
                                    }).catch(function (reason) {
                                        api.DefaultErrorHandler.handle(reason);
                                    }).done();
                                }
                            }
                        });
                        if (this.applySelectionsButton) {
                            this.applySelectionsButton.onClicked(this.selectRowOrApplySelection.bind(this, -1));
                            this.comboBoxDropdown.onMultipleSelection(this.handleMultipleSelectionChanged.bind(this));
                        }
                        this.input.onValueChanged(function (event) {
                            _this.preservedInputValueChangedEvent = event;
                            if (_this.delayedInputValueChangedHandling == 0) {
                                _this.handleInputValueChanged();
                            }
                            else {
                                _this.setEmptyDropdownText("Just keep on typing...");
                                _this.delayedHandleInputValueChangedFnCall.delayCall();
                            }
                        });
                        this.input.onDblClicked(function (event) {
                            event.preventDefault();
                            event.stopPropagation();
                            if (!_this.isDropdownShown()) {
                                _this.showDropdown();
                                _this.loadOptionsAfterShowDropdown().then(function () {
                                    _this.comboBoxDropdown.navigateToFirstRowIfNotActive();
                                }).catch(function (reason) {
                                    api.DefaultErrorHandler.handle(reason);
                                }).done();
                                _this.input.setReadOnly(false);
                            }
                        });
                        this.onKeyDown(this.handleKeyDown.bind(this));
                        if (this.selectedOptionsView) {
                            this.selectedOptionsView.onOptionDeselected(function (removedOption) {
                                _this.handleSelectedOptionRemoved(removedOption);
                            });
                        }
                    };
                    ComboBox.prototype.handleInputValueChanged = function () {
                        if (this.preservedInputValueChangedEvent) {
                            this.notifyOptionFilterInputValueChanged(this.preservedInputValueChangedEvent.getOldValue(), this.preservedInputValueChangedEvent.getNewValue());
                            this.comboBoxDropdown.resetActiveSelection();
                            this.showDropdown();
                            this.input.setReadOnly(false);
                        }
                    };
                    ComboBox.prototype.handleKeyDown = function (event) {
                        var _this = this;
                        if (event.which == 9) {
                            this.hideDropdown();
                            return;
                        }
                        else if (event.which == 16 || event.which == 17 || event.which == 18 || event.which == 91) {
                            return;
                        }
                        if (!this.isDropdownShown()) {
                            if (event.which === 40) {
                                this.showDropdown();
                                this.loadOptionsAfterShowDropdown().then(function () {
                                    _this.comboBoxDropdown.navigateToFirstRowIfNotActive();
                                    _this.input.setReadOnly(true);
                                }).catch(function (reason) {
                                    api.DefaultErrorHandler.handle(reason);
                                }).done();
                            }
                            else {
                                this.showDropdown();
                                this.input.setReadOnly(false);
                            }
                            return;
                        }
                        switch (event.which) {
                            case 38:
                                if (this.comboBoxDropdown.hasActiveRow()) {
                                    if (this.comboBoxDropdown.getActiveRow() === 0) {
                                        this.comboBoxDropdown.resetActiveSelection();
                                        this.input.setReadOnly(false);
                                        this.input.giveFocus();
                                    }
                                    else {
                                        this.comboBoxDropdown.navigateToPreviousRow();
                                        this.input.setReadOnly(true);
                                    }
                                }
                                event.stopPropagation();
                                event.preventDefault();
                                break;
                            case 40:
                                if (this.comboBoxDropdown.hasActiveRow()) {
                                    this.comboBoxDropdown.navigateToNextRow();
                                }
                                else {
                                    this.comboBoxDropdown.nagivateToFirstRow();
                                }
                                this.input.setReadOnly(true);
                                event.stopPropagation();
                                event.preventDefault();
                                break;
                            case 13:
                                this.selectRowOrApplySelection(this.comboBoxDropdown.getActiveRow());
                                event.stopPropagation();
                                event.preventDefault();
                                break;
                            case 32:
                                if (this.input.isReadOnly() && this.applySelectionsButton) {
                                    if (!this.isSelectedRowReadOnly())
                                        this.comboBoxDropdown.toggleRowSelection(this.comboBoxDropdown.getActiveRow(), this.maximumSelectionsReached());
                                    event.stopPropagation();
                                    event.preventDefault();
                                }
                                break;
                            case 8:
                                if (this.input.isReadOnly()) {
                                    event.stopPropagation();
                                    event.preventDefault();
                                }
                                break;
                            case 27:
                                this.hideDropdown();
                                break;
                        }
                        this.input.giveFocus();
                    };
                    ComboBox.prototype.isSelectedRowReadOnly = function () {
                        return this.getOptionByRow(this.comboBoxDropdown.getActiveRow()).readOnly;
                    };
                    ComboBox.prototype.handleSelectedOptionRemoved = function (removedSelectedOption) {
                        this.comboBoxDropdown.markSelections(this.getSelectedOptions());
                        this.dropdownHandle.setEnabled(true);
                        if (this.hideComboBoxWhenMaxReached) {
                            this.show();
                        }
                        if (this.countSelectedOptions() == 0) {
                            this.removeClass("followed-by-options");
                        }
                        this.input.openForTypingAndFocus();
                    };
                    ComboBox.prototype.handleMultipleSelectionChanged = function (event) {
                        this.input.giveFocus();
                        if (this.isSelectionChanged()) {
                            this.applySelectionsButton.show();
                        }
                        else {
                            this.applySelectionsButton.hide();
                        }
                    };
                    ComboBox.prototype.setOnBlurListener = function () {
                        var combobox = this;
                        var hideDropdownOnBlur = function (event) {
                            var comboboxHtmlElement = combobox.getHTMLElement();
                            for (var element = event.target; element; element = element.parentNode) {
                                if (element == comboboxHtmlElement) {
                                    return;
                                }
                            }
                            combobox.hideDropdown();
                            combobox.active = false;
                            api.dom.Body.get().getEl().removeEventListener('click', hideDropdownOnBlur);
                        };
                        if (!this.active) {
                            this.active = true;
                            api.dom.Body.get().onClicked(hideDropdownOnBlur);
                        }
                    };
                    ComboBox.prototype.onOptionSelected = function (listener) {
                        this.optionSelectedListeners.push(listener);
                    };
                    ComboBox.prototype.unOptionSelected = function (listener) {
                        this.optionSelectedListeners.filter(function (currentListener) {
                            return listener != currentListener;
                        });
                    };
                    ComboBox.prototype.notifyOptionSelected = function (item, index) {
                        var event = new OptionSelectedEvent(item, index);
                        this.optionSelectedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    ComboBox.prototype.onOptionFilterInputValueChanged = function (listener) {
                        this.optionFilterInputValueChangedListeners.push(listener);
                    };
                    ComboBox.prototype.unOptionFilterInputValueChanged = function (listener) {
                        this.optionFilterInputValueChangedListeners.filter(function (currentListener) {
                            return listener != currentListener;
                        });
                    };
                    ComboBox.prototype.notifyOptionFilterInputValueChanged = function (oldValue, newValue) {
                        var event = new OptionFilterInputValueChangedEvent(oldValue, newValue, this.comboBoxDropdown.getDropdownGrid().getElement());
                        this.optionFilterInputValueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    ComboBox.prototype.onExpanded = function (listener) {
                        this.expandedListeners.push(listener);
                    };
                    ComboBox.prototype.notifyExpanded = function (expanded) {
                        var event = new api.ui.selector.DropdownExpandedEvent(this.comboBoxDropdown.getDropdownGrid().getElement(), expanded);
                        this.expandedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    ComboBox.prototype.onOptionDeselected = function (listener) {
                        this.selectedOptionsView.onOptionDeselected(listener);
                    };
                    ComboBox.prototype.unOptionDeselected = function (listener) {
                        this.selectedOptionsView.unOptionDeselected(listener);
                    };
                    ComboBox.prototype.onFocus = function (listener) {
                        this.input.onFocus(listener);
                    };
                    ComboBox.prototype.unFocus = function (listener) {
                        this.input.unFocus(listener);
                    };
                    ComboBox.prototype.onBlur = function (listener) {
                        this.input.onBlur(listener);
                    };
                    ComboBox.prototype.unBlur = function (listener) {
                        this.input.unBlur(listener);
                    };
                    ComboBox.prototype.onScroll = function (listener) {
                        this.comboBoxDropdown.getDropdownGrid().getElement().subscribeOnScroll(listener);
                    };
                    return ComboBox;
                })(api.dom.FormInputEl);
                _combobox.ComboBox = ComboBox;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (combobox) {
                var BaseSelectedOptionView = (function (_super) {
                    __extends(BaseSelectedOptionView, _super);
                    function BaseSelectedOptionView(option) {
                        _super.call(this, "selected-option");
                        this.selectedOptionToBeRemovedListeners = [];
                        this.layout();
                        this.setOption(option);
                    }
                    BaseSelectedOptionView.prototype.setOption = function (option) {
                        this.option = option;
                        if (this.optionValueEl) {
                            this.optionValueEl.getEl().setInnerHtml(this.option.displayValue.toString());
                        }
                    };
                    BaseSelectedOptionView.prototype.getOption = function () {
                        return this.option;
                    };
                    BaseSelectedOptionView.prototype.layout = function () {
                        var _this = this;
                        var removeButtonEl = new api.dom.AEl("remove");
                        this.optionValueEl = new api.dom.DivEl('option-value');
                        removeButtonEl.onClicked(function (event) {
                            _this.notifySelectedOptionRemoveRequested();
                            event.stopPropagation();
                            event.preventDefault();
                            return false;
                        });
                        this.appendChild(removeButtonEl);
                        this.appendChild(this.optionValueEl);
                    };
                    BaseSelectedOptionView.prototype.notifySelectedOptionRemoveRequested = function () {
                        this.selectedOptionToBeRemovedListeners.forEach(function (listener) {
                            listener();
                        });
                    };
                    BaseSelectedOptionView.prototype.onSelectedOptionRemoveRequest = function (listener) {
                        this.selectedOptionToBeRemovedListeners.push(listener);
                    };
                    BaseSelectedOptionView.prototype.unSelectedOptionRemoveRequest = function (listener) {
                        this.selectedOptionToBeRemovedListeners = this.selectedOptionToBeRemovedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    return BaseSelectedOptionView;
                })(api.dom.DivEl);
                combobox.BaseSelectedOptionView = BaseSelectedOptionView;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (combobox) {
                var BaseSelectedOptionsView = (function (_super) {
                    __extends(BaseSelectedOptionsView, _super);
                    function BaseSelectedOptionsView(className) {
                        _super.call(this, "selected-options" + (className ? " " + className : ""));
                        this.list = [];
                        this.selectedOptionRemovedListeners = [];
                    }
                    BaseSelectedOptionsView.prototype.setMaximumOccurrences = function (value) {
                        this.maximumOccurrences = value;
                    };
                    BaseSelectedOptionsView.prototype.getMaximumOccurrences = function () {
                        return this.maximumOccurrences;
                    };
                    BaseSelectedOptionsView.prototype.createSelectedOption = function (option) {
                        return new combobox.SelectedOption(new combobox.BaseSelectedOptionView(option), this.count());
                    };
                    BaseSelectedOptionsView.prototype.addOption = function (option) {
                        var _this = this;
                        if (this.isSelected(option) || this.maximumOccurrencesReached()) {
                            return false;
                        }
                        var selectedOption = this.createSelectedOption(option);
                        selectedOption.getOptionView().onSelectedOptionRemoveRequest(function () {
                            _this.removeOption(option);
                        });
                        this.list.push(selectedOption);
                        this.appendChild(selectedOption.getOptionView());
                        return true;
                    };
                    BaseSelectedOptionsView.prototype.removeOption = function (optionToRemove, silent) {
                        if (silent === void 0) { silent = false; }
                        api.util.assertNotNull(optionToRemove, "optionToRemove cannot be null");
                        var selectedOption = this.getByOption(optionToRemove);
                        api.util.assertNotNull(selectedOption, "Did not find any selected option to remove from option: " + optionToRemove.value);
                        selectedOption.getOptionView().remove();
                        this.list = this.list.filter(function (option) {
                            return option.getOption().value != selectedOption.getOption().value;
                        });
                        if (selectedOption.getIndex() < this.list.length) {
                            for (var i = selectedOption.getIndex(); i < this.list.length; i++) {
                                this.list[i].setIndex(i);
                            }
                        }
                        if (!silent) {
                            this.notifySelectedOptionRemoved(selectedOption);
                        }
                    };
                    BaseSelectedOptionsView.prototype.count = function () {
                        return this.list.length;
                    };
                    BaseSelectedOptionsView.prototype.getSelectedOptions = function () {
                        return this.list;
                    };
                    BaseSelectedOptionsView.prototype.getByIndex = function (index) {
                        return this.list[index];
                    };
                    BaseSelectedOptionsView.prototype.getByOption = function (option) {
                        return this.getById(option.value);
                    };
                    BaseSelectedOptionsView.prototype.getById = function (id) {
                        return this.list.filter(function (selectedOption) {
                            return selectedOption.getOption().value == id;
                        })[0];
                    };
                    BaseSelectedOptionsView.prototype.isSelected = function (option) {
                        return this.getByOption(option) != null;
                    };
                    BaseSelectedOptionsView.prototype.maximumOccurrencesReached = function () {
                        if (this.maximumOccurrences == 0) {
                            return false;
                        }
                        return this.count() >= this.maximumOccurrences;
                    };
                    BaseSelectedOptionsView.prototype.moveOccurrence = function (formIndex, toIndex) {
                        api.util.ArrayHelper.moveElement(formIndex, toIndex, this.list);
                        api.util.ArrayHelper.moveElement(formIndex, toIndex, this.getChildren());
                        this.list.forEach(function (selectedOption, index) { return selectedOption.setIndex(index); });
                    };
                    BaseSelectedOptionsView.prototype.notifySelectedOptionRemoved = function (removed) {
                        this.selectedOptionRemovedListeners.forEach(function (listener) {
                            listener(removed);
                        });
                    };
                    BaseSelectedOptionsView.prototype.onOptionDeselected = function (listener) {
                        this.selectedOptionRemovedListeners.push(listener);
                    };
                    BaseSelectedOptionsView.prototype.unOptionDeselected = function (listener) {
                        this.selectedOptionRemovedListeners = this.selectedOptionRemovedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    return BaseSelectedOptionsView;
                })(api.dom.DivEl);
                combobox.BaseSelectedOptionsView = BaseSelectedOptionsView;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (combobox) {
                var SelectedOption = (function () {
                    function SelectedOption(optionView, index) {
                        api.util.assertNotNull(optionView, "optionView cannot be null");
                        this.optionView = optionView;
                        this.index = index;
                    }
                    SelectedOption.prototype.getOption = function () {
                        return this.optionView.getOption();
                    };
                    SelectedOption.prototype.getOptionView = function () {
                        return this.optionView;
                    };
                    SelectedOption.prototype.getIndex = function () {
                        return this.index;
                    };
                    SelectedOption.prototype.setIndex = function (value) {
                        this.index = value;
                    };
                    return SelectedOption;
                })();
                combobox.SelectedOption = SelectedOption;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (combobox) {
                var RichComboBox = (function (_super) {
                    __extends(RichComboBox, _super);
                    function RichComboBox(builder) {
                        var _this = this;
                        this.loadedListeners = [];
                        this.loadingListeners = [];
                        this.comboBoxName = builder.comboBoxName;
                        this.identifierMethod = builder.identifierMethod;
                        this.comboBoxView = new api.dom.DivEl();
                        this.delayedInputValueChangedHandling = builder.delayedInputValueChangedHandling;
                        this.selectedOptionsView = builder.selectedOptionsView;
                        this.selectedOptionsView.hide();
                        this.maximumOccurrences = builder.maximumOccurrences;
                        this.minWidth = builder.minWidth;
                        this.optionDisplayValueViewer = builder.optionDisplayValueViewer;
                        this.setNextInputFocusWhenMaxReached = builder.nextInputFocusWhenMaxReached;
                        var comboBoxConfig = {
                            maximumOccurrences: this.maximumOccurrences,
                            selectedOptionsView: this.selectedOptionsView,
                            optionDisplayValueViewer: this.optionDisplayValueViewer,
                            hideComboBoxWhenMaxReached: true,
                            setNextInputFocusWhenMaxReached: this.setNextInputFocusWhenMaxReached,
                            delayedInputValueChangedHandling: this.delayedInputValueChangedHandling,
                            minWidth: this.minWidth
                        };
                        this.loader = builder.loader;
                        this.comboBox = new RichComboBoxComboBox(name, comboBoxConfig, this.loader);
                        this.setupLoader();
                        this.comboBox.onClicked(function (event) {
                            _this.comboBox.giveFocus();
                        });
                        _super.call(this, this.comboBox, this.selectedOptionsView);
                        this.addClass('rich-combobox');
                    }
                    RichComboBox.prototype.setIgnoreNextFocus = function (value) {
                        if (value === void 0) { value = true; }
                        this.comboBox.setIgnoreNextFocus(value);
                        return this;
                    };
                    RichComboBox.prototype.isIgnoreNextFocus = function () {
                        return this.comboBox.isIgnoreNextFocus();
                    };
                    RichComboBox.prototype.getSelectedDisplayValues = function () {
                        return this.comboBox.getSelectedOptions().map(function (option) {
                            return option.displayValue;
                        });
                    };
                    RichComboBox.prototype.getSelectedValues = function () {
                        return this.comboBox.getSelectedOptions().map(function (option) {
                            return option.value;
                        });
                    };
                    RichComboBox.prototype.getDisplayValues = function () {
                        return this.comboBox.getOptions().map(function (option) {
                            return option.displayValue;
                        });
                    };
                    RichComboBox.prototype.getValues = function () {
                        return this.comboBox.getOptions().map(function (option) {
                            return option.value;
                        });
                    };
                    RichComboBox.prototype.getSelectedOptions = function () {
                        return this.selectedOptionsView.getSelectedOptions();
                    };
                    RichComboBox.prototype.getSelectedOption = function (option) {
                        return this.selectedOptionsView.getByOption(option);
                    };
                    RichComboBox.prototype.isOptionSelected = function (option) {
                        return this.comboBox.isOptionSelected(option);
                    };
                    RichComboBox.prototype.maximumOccurrencesReached = function () {
                        return this.comboBox.maximumOccurrencesReached();
                    };
                    RichComboBox.prototype.getComboBox = function () {
                        return this.comboBox;
                    };
                    RichComboBox.prototype.addOption = function (option) {
                        this.comboBox.addOption(option);
                    };
                    RichComboBox.prototype.selectOption = function (option, silent) {
                        if (silent === void 0) { silent = false; }
                        this.comboBox.selectOption(option, silent);
                    };
                    RichComboBox.prototype.hasOptions = function () {
                        return this.comboBox.hasOptions();
                    };
                    RichComboBox.prototype.getOptionCount = function () {
                        return this.comboBox.getOptionCount();
                    };
                    RichComboBox.prototype.getOptions = function () {
                        return this.comboBox.getOptions();
                    };
                    RichComboBox.prototype.getOptionByValue = function (value) {
                        return this.comboBox.getOptionByValue(value);
                    };
                    RichComboBox.prototype.getOptionByRow = function (rowIndex) {
                        return this.comboBox.getOptionByRow(rowIndex);
                    };
                    RichComboBox.prototype.countSelected = function () {
                        return this.comboBox.countSelectedOptions();
                    };
                    RichComboBox.prototype.select = function (value) {
                        this.comboBox.selectOption(this.createOption(value));
                    };
                    RichComboBox.prototype.selectReadOnly = function (value) {
                        this.comboBox.selectOption(this.createOptionReadOnly(value));
                    };
                    RichComboBox.prototype.selectByOption = function (option) {
                        this.comboBox.selectOption(option);
                    };
                    RichComboBox.prototype.deselect = function (value) {
                        this.comboBox.deselectOption(this.createOption(value));
                    };
                    RichComboBox.prototype.clearCombobox = function () {
                        this.clearSelection(true);
                        this.comboBox.getInput().getEl().setValue("");
                    };
                    RichComboBox.prototype.clearSelection = function (forceClear) {
                        if (forceClear === void 0) { forceClear = false; }
                        this.comboBox.clearSelection(false, true, forceClear);
                    };
                    RichComboBox.prototype.isSelected = function (value) {
                        var selectedValues = this.getSelectedValues();
                        var valueToFind = this.getDisplayValueId(value);
                        for (var i = 0; i < selectedValues.length; i++) {
                            if (selectedValues[i] == valueToFind) {
                                return true;
                            }
                        }
                        return false;
                    };
                    RichComboBox.prototype.getDisplayValueId = function (value) {
                        var val = value[this.identifierMethod]();
                        return typeof val == 'object' && val['toString'] ? val.toString() : val;
                    };
                    RichComboBox.prototype.createOption = function (value) {
                        return {
                            value: this.getDisplayValueId(value),
                            displayValue: value
                        };
                    };
                    RichComboBox.prototype.createOptionReadOnly = function (value) {
                        return {
                            value: this.getDisplayValueId(value),
                            displayValue: value,
                            readOnly: true
                        };
                    };
                    RichComboBox.prototype.setupLoader = function () {
                        var _this = this;
                        this.comboBox.onOptionFilterInputValueChanged(function (event) {
                            _this.loader.search(event.getNewValue()).then(function (result) {
                                return result;
                            }).catch(function (reason) {
                                api.DefaultErrorHandler.handle(reason);
                            }).done();
                        });
                        this.comboBox.onOptionSelected(function (event) {
                            _this.selectedOptionsView.show();
                        });
                        this.loader.onLoadingData(function (event) {
                            _this.comboBox.setEmptyDropdownText("Searching...");
                            _this.notifyLoading();
                        });
                        this.loader.onLoadedData(function (event) {
                            var options = _this.createOptions(event.getData());
                            _this.comboBox.setOptions(options);
                            _this.notifyLoaded(event.getData());
                        });
                    };
                    RichComboBox.prototype.createOptions = function (items) {
                        var _this = this;
                        var options = [];
                        items.forEach(function (itemInst) {
                            options.push(_this.createOption(itemInst));
                        });
                        return options;
                    };
                    RichComboBox.prototype.getLoader = function () {
                        return this.loader;
                    };
                    RichComboBox.prototype.setInputIconUrl = function (url) {
                        this.comboBox.setInputIconUrl(url);
                    };
                    RichComboBox.prototype.onOptionDeselected = function (listener) {
                        this.comboBox.onOptionDeselected(listener);
                    };
                    RichComboBox.prototype.unOptionDeselected = function (listener) {
                        this.comboBox.unOptionDeselected(listener);
                    };
                    RichComboBox.prototype.onOptionSelected = function (listener) {
                        this.comboBox.onOptionSelected(listener);
                    };
                    RichComboBox.prototype.unOptionSelected = function (listener) {
                        this.comboBox.unOptionSelected(listener);
                    };
                    RichComboBox.prototype.notifyLoading = function () {
                        this.loadingListeners.forEach(function (listener) {
                            listener();
                        });
                    };
                    RichComboBox.prototype.onLoading = function (listener) {
                        this.loadingListeners.push(listener);
                    };
                    RichComboBox.prototype.unLoading = function (listener) {
                        var index = this.loadedListeners.indexOf(listener);
                        this.loadedListeners.splice(index, 1);
                    };
                    RichComboBox.prototype.onLoaded = function (listener) {
                        this.loadedListeners.push(listener);
                    };
                    RichComboBox.prototype.unLoaded = function (listenerToBeRemoved) {
                        var index = this.loadedListeners.indexOf(listenerToBeRemoved);
                        this.loadedListeners.splice(index, 1);
                    };
                    RichComboBox.prototype.notifyLoaded = function (items) {
                        this.loadedListeners.forEach(function (listener) {
                            listener(items);
                        });
                    };
                    RichComboBox.prototype.giveFocus = function () {
                        return this.comboBox.giveFocus();
                    };
                    RichComboBox.prototype.onFocus = function (listener) {
                        this.comboBox.onFocus(listener);
                    };
                    RichComboBox.prototype.unFocus = function (listener) {
                        this.comboBox.unFocus(listener);
                    };
                    RichComboBox.prototype.onBlur = function (listener) {
                        this.comboBox.onBlur(listener);
                    };
                    RichComboBox.prototype.unBlur = function (listener) {
                        this.comboBox.unBlur(listener);
                    };
                    RichComboBox.prototype.setValue = function (value) {
                        var _this = this;
                        if (this.loader.isLoaded()) {
                            _super.prototype.setValue.call(this, value);
                        }
                        else {
                            var singleLoadListener = function (event) {
                                _super.prototype.setValue.call(_this, value);
                                _this.loader.unLoadedData(singleLoadListener);
                            };
                            this.loader.onLoadedData(singleLoadListener);
                            if (this.loader.isNotStarted()) {
                                this.loader.load();
                            }
                        }
                        return this;
                    };
                    return RichComboBox;
                })(api.dom.CompositeFormInputEl);
                combobox.RichComboBox = RichComboBox;
                var RichComboBoxComboBox = (function (_super) {
                    __extends(RichComboBoxComboBox, _super);
                    function RichComboBoxComboBox(name, config, loader) {
                        _super.call(this, name, config);
                        this.loader = loader;
                    }
                    RichComboBoxComboBox.prototype.loadOptionsAfterShowDropdown = function () {
                        var deferred = wemQ.defer();
                        this.loader.load().then(function () {
                            deferred.resolve(null);
                        }).catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).done();
                        return deferred.promise;
                    };
                    return RichComboBoxComboBox;
                })(combobox.ComboBox);
                var RichComboBoxBuilder = (function () {
                    function RichComboBoxBuilder() {
                        this.identifierMethod = "getId";
                        this.maximumOccurrences = 0;
                        this.nextInputFocusWhenMaxReached = true;
                    }
                    RichComboBoxBuilder.prototype.setComboBoxName = function (comboBoxName) {
                        this.comboBoxName = comboBoxName;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setIdentifierMethod = function (identifierMethod) {
                        this.identifierMethod = identifierMethod;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setLoader = function (loader) {
                        this.loader = loader;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setSelectedOptionsView = function (selectedOptionsView) {
                        this.selectedOptionsView = selectedOptionsView;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setMaximumOccurrences = function (maximumOccurrences) {
                        this.maximumOccurrences = maximumOccurrences;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setOptionDisplayValueViewer = function (value) {
                        this.optionDisplayValueViewer = value;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setDelayedInputValueChangedHandling = function (value) {
                        this.delayedInputValueChangedHandling = value;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setNextInputFocusWhenMaxReached = function (value) {
                        this.nextInputFocusWhenMaxReached = value;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.setMinWidth = function (value) {
                        this.minWidth = value;
                        return this;
                    };
                    RichComboBoxBuilder.prototype.build = function () {
                        return new RichComboBox(this);
                    };
                    return RichComboBoxBuilder;
                })();
                combobox.RichComboBoxBuilder = RichComboBoxBuilder;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var combobox;
            (function (combobox) {
                var RichSelectedOptionView = (function (_super) {
                    __extends(RichSelectedOptionView, _super);
                    function RichSelectedOptionView(option, size) {
                        if (size === void 0) { size = 2 /* small */; }
                        this.optionDisplayValue = option.displayValue;
                        this.size = size;
                        _super.call(this, option);
                    }
                    RichSelectedOptionView.prototype.resolveIconUrl = function (content) {
                        return "";
                    };
                    RichSelectedOptionView.prototype.resolveTitle = function (content) {
                        return "";
                    };
                    RichSelectedOptionView.prototype.resolveSubTitle = function (content) {
                        return "";
                    };
                    RichSelectedOptionView.prototype.resolveIconClass = function (content) {
                        return "";
                    };
                    RichSelectedOptionView.prototype.layout = function () {
                        var _this = this;
                        var namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(this.size).build();
                        namesAndIconView.setMainName(this.resolveTitle(this.optionDisplayValue)).setSubName(this.resolveSubTitle(this.optionDisplayValue));
                        var url = this.resolveIconUrl(this.optionDisplayValue);
                        if (!api.util.StringHelper.isBlank(url)) {
                            namesAndIconView.setIconUrl(this.resolveIconUrl(this.optionDisplayValue) + '?crop=false');
                        }
                        else {
                            namesAndIconView.setIconClass(this.resolveIconClass(this.optionDisplayValue));
                        }
                        var removeButton = new api.dom.AEl("remove");
                        removeButton.onClicked(function (event) {
                            _this.notifySelectedOptionRemoveRequested();
                            event.stopPropagation();
                            event.preventDefault();
                            return false;
                        });
                        this.appendChild(new api.dom.DivEl("drag-control"));
                        this.appendChild(removeButton);
                        this.appendChild(namesAndIconView);
                    };
                    return RichSelectedOptionView;
                })(api.ui.selector.combobox.BaseSelectedOptionView);
                combobox.RichSelectedOptionView = RichSelectedOptionView;
            })(combobox = selector.combobox || (selector.combobox = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var dropdown;
            (function (dropdown) {
                var DropdownOptionFilterInput = (function (_super) {
                    __extends(DropdownOptionFilterInput, _super);
                    function DropdownOptionFilterInput() {
                        _super.call(this);
                    }
                    return DropdownOptionFilterInput;
                })(api.ui.selector.OptionFilterInput);
                dropdown.DropdownOptionFilterInput = DropdownOptionFilterInput;
            })(dropdown = selector.dropdown || (selector.dropdown = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var dropdown;
            (function (dropdown) {
                var DropdownHandle = api.ui.selector.DropdownHandle;
                var SelectedOptionView = (function (_super) {
                    __extends(SelectedOptionView, _super);
                    function SelectedOptionView(objectViewer) {
                        var _this = this;
                        _super.call(this, "selected-option");
                        this.openDropdownListeners = [];
                        this.objectViewer = objectViewer;
                        this.dropdownHandle = new DropdownHandle();
                        this.optionValueEl = new api.dom.DivEl('option-value');
                        this.appendChild(this.optionValueEl);
                        this.optionValueEl.appendChild(this.objectViewer);
                        this.appendChild(this.dropdownHandle);
                        this.dropdownHandle.onClicked(function () {
                            _this.notifyOpenDropdown();
                        });
                        this.onClicked(function (event) {
                            if (document.selection && document.selection.empty) {
                                document.selection.empty();
                            }
                            else if (window.getSelection) {
                                var sel = window.getSelection();
                                sel.removeAllRanges();
                            }
                            _this.notifyOpenDropdown();
                        });
                        this.onKeyPressed(function (event) {
                            if (event.which == 32 || event.which == 13) {
                                _this.notifyOpenDropdown();
                            }
                        });
                    }
                    SelectedOptionView.prototype.setOption = function (option) {
                        this.option = option;
                        this.objectViewer.setObject(option.displayValue);
                    };
                    SelectedOptionView.prototype.getOption = function () {
                        return this.option;
                    };
                    SelectedOptionView.prototype.notifyOpenDropdown = function () {
                        this.openDropdownListeners.forEach(function (listener) {
                            listener();
                        });
                    };
                    SelectedOptionView.prototype.resetOption = function () {
                        this.option = null;
                    };
                    SelectedOptionView.prototype.onOpenDropdown = function (listener) {
                        this.openDropdownListeners.push(listener);
                    };
                    SelectedOptionView.prototype.unOpenDropdown = function (listener) {
                        this.openDropdownListeners = this.openDropdownListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    return SelectedOptionView;
                })(api.dom.ButtonEl);
                dropdown.SelectedOptionView = SelectedOptionView;
            })(dropdown = selector.dropdown || (selector.dropdown = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var selector;
        (function (selector) {
            var dropdown;
            (function (dropdown) {
                var OptionSelectedEvent = api.ui.selector.OptionSelectedEvent;
                var OptionFilterInputValueChangedEvent = api.ui.selector.OptionFilterInputValueChangedEvent;
                var DropdownHandle = api.ui.selector.DropdownHandle;
                var DefaultOptionDisplayValueViewer = api.ui.selector.DefaultOptionDisplayValueViewer;
                var Dropdown = (function (_super) {
                    __extends(Dropdown, _super);
                    function Dropdown(name, config) {
                        var _this = this;
                        _super.call(this, "div", "dropdown");
                        this.optionSelectedListeners = [];
                        this.optionFilterInputValueChangedListeners = [];
                        this.expandedListeners = [];
                        this.active = false;
                        this.getEl().setAttribute("name", name);
                        this.optionDisplayValueViewer = config.optionDisplayValueViewer || new DefaultOptionDisplayValueViewer();
                        if (config.iconUrl) {
                            this.icon = new api.dom.ImgEl(config.iconUrl, "input-icon");
                            this.appendChild(this.icon);
                        }
                        this.input = new dropdown.DropdownOptionFilterInput();
                        this.appendChild(this.input);
                        this.dropdownHandle = new DropdownHandle();
                        this.appendChild(this.dropdownHandle);
                        var filter = this.defaultFilter;
                        if (config.filter) {
                            filter = config.filter;
                        }
                        this.dropdownList = new selector.DropdownList({
                            maxHeight: 200,
                            width: this.input.getEl().getWidth(),
                            optionDisplayValueViewer: config.optionDisplayValueViewer,
                            filter: filter,
                            dataIdProperty: config.dataIdProperty
                        });
                        if (filter) {
                            this.dropdownList.setFilterArgs({ searchString: "" });
                        }
                        this.dropdownList.onRowSelection(function (event) {
                            _this.selectRow(event.getRow());
                        });
                        this.appendChild(this.dropdownList.getEmptyDropdown());
                        this.appendChild(this.dropdownList.getDropdownGrid().getElement());
                        this.selectedOptionView = new dropdown.SelectedOptionView(this.optionDisplayValueViewer);
                        this.selectedOptionView.hide();
                        this.appendChild(this.selectedOptionView);
                        this.selectedOptionView.onOpenDropdown(function () {
                            _this.showDropdown();
                            _this.dropdownList.navigateToFirstRowIfNotActive();
                            _this.input.giveFocus();
                        });
                        this.setupListeners();
                        this.onRendered(function (event) {
                            _this.doUpdateDropdownTopPositionAndWidth();
                        });
                    }
                    Dropdown.prototype.getInput = function () {
                        return this.input;
                    };
                    Dropdown.prototype.reset = function () {
                        this.input.show();
                        this.selectedOptionView.hide();
                        this.selectedOptionView.resetOption();
                        this.dropdownHandle.show();
                    };
                    Dropdown.prototype.defaultFilter = function (option, args) {
                        if (!args.searchString || api.util.StringHelper.isEmpty(args.searchString)) {
                            return true;
                        }
                        var lowerCasedSearchString = args.searchString.toLowerCase();
                        if (option.value.toLowerCase().indexOf(lowerCasedSearchString) > -1) {
                            return true;
                        }
                        var displayVaueAsString = option.displayValue.toString();
                        if (displayVaueAsString.toLowerCase().indexOf(lowerCasedSearchString) > -1) {
                            return true;
                        }
                        var indices = option.indices;
                        if (indices && indices.length > 0) {
                            for (var i = 0; i < indices.length; i++) {
                                var index = indices[i];
                                if (index) {
                                    if (index.toLocaleLowerCase().indexOf(lowerCasedSearchString) > -1) {
                                        return true;
                                    }
                                }
                            }
                        }
                        return false;
                    };
                    Dropdown.prototype.doUpdateDropdownTopPositionAndWidth = function () {
                        var inputEl = this.input.getEl();
                        this.dropdownList.setTopPx(inputEl.getHeightWithBorder() - inputEl.getBorderBottomWidth());
                        this.dropdownList.setWidth(inputEl.getWidthWithBorder());
                    };
                    Dropdown.prototype.giveFocus = function () {
                        return this.input.giveFocus();
                    };
                    Dropdown.prototype.isDropdownShown = function () {
                        return this.dropdownList.isDropdownShown();
                    };
                    Dropdown.prototype.showDropdown = function () {
                        this.selectedOptionView.hide();
                        this.input.show();
                        this.dropdownHandle.show();
                        this.doUpdateDropdownTopPositionAndWidth();
                        this.dropdownList.showDropdown([this.getSelectedOption()]);
                        this.dropdownHandle.down();
                        this.dropdownList.renderDropdownGrid();
                        this.notifyExpanded();
                    };
                    Dropdown.prototype.hideDropdown = function () {
                        if (this.selectedOptionView.getOption()) {
                            this.input.hide();
                            this.dropdownHandle.hide();
                            this.selectedOptionView.show();
                        }
                        else {
                            this.selectedOptionView.hide();
                            this.input.show();
                            this.dropdownHandle.up();
                            this.dropdownHandle.show();
                        }
                        this.dropdownList.hideDropdown();
                    };
                    Dropdown.prototype.setOptions = function (options) {
                        this.dropdownList.setOptions(options);
                    };
                    Dropdown.prototype.removeAllOptions = function () {
                        this.dropdownList.removeAllOptions();
                    };
                    Dropdown.prototype.addOption = function (option) {
                        this.dropdownList.addOption(option);
                    };
                    Dropdown.prototype.hasOptions = function () {
                        return this.dropdownList.hasOptions();
                    };
                    Dropdown.prototype.getOptionCount = function () {
                        return this.dropdownList.getOptionCount();
                    };
                    Dropdown.prototype.getOptions = function () {
                        return this.dropdownList.getOptions();
                    };
                    Dropdown.prototype.getOptionByValue = function (value) {
                        return this.dropdownList.getOptionByValue(value);
                    };
                    Dropdown.prototype.getOptionByRow = function (rowIndex) {
                        return this.dropdownList.getOptionByRow(rowIndex);
                    };
                    Dropdown.prototype.setValue = function (value) {
                        var option = this.getOptionByValue(value);
                        if (option != null) {
                            this.selectOption(option);
                        }
                        return this;
                    };
                    Dropdown.prototype.selectRow = function (index, silent) {
                        if (silent === void 0) { silent = false; }
                        var option = this.getOptionByRow(index);
                        if (option != null) {
                            this.selectOption(option, silent);
                            api.dom.FormEl.moveFocusToNextFocusable(this.input);
                        }
                    };
                    Dropdown.prototype.selectOption = function (option, silent) {
                        if (silent === void 0) { silent = false; }
                        this.dropdownList.markSelections([option]);
                        if (!silent) {
                            this.notifyOptionSelected(option);
                        }
                        this.selectedOptionView.setOption(option);
                        this.hideDropdown();
                    };
                    Dropdown.prototype.getSelectedOption = function () {
                        return this.selectedOptionView.getOption();
                    };
                    Dropdown.prototype.getValue = function () {
                        var selectedOption = this.getSelectedOption();
                        if (!selectedOption) {
                            return null;
                        }
                        return selectedOption.value;
                    };
                    Dropdown.prototype.setInputIconUrl = function (iconUrl) {
                        if (!this.icon) {
                            this.icon = new api.dom.ImgEl();
                            this.icon.addClass("input-icon");
                            this.icon.insertBeforeEl(this.input);
                        }
                        this.icon.getEl().setSrc(iconUrl);
                    };
                    Dropdown.prototype.setupListeners = function () {
                        var _this = this;
                        this.onClicked(function () {
                            _this.setOnBlurListener();
                        });
                        this.dropdownHandle.onClicked(function (event) {
                            _this.dropdownList.navigateToFirstRowIfNotActive();
                            if (_this.isDropdownShown()) {
                                _this.hideDropdown();
                                _this.giveFocus();
                            }
                            else {
                                _this.showDropdown();
                                _this.giveFocus();
                            }
                        });
                        this.input.onValueChanged(function (event) {
                            _this.notifyOptionFilterInputValueChanged(event.getOldValue(), event.getNewValue());
                            _this.dropdownList.setFilterArgs({ searchString: event.getNewValue() });
                            _this.showDropdown();
                            _this.dropdownList.nagivateToFirstRow();
                        });
                        this.input.onDblClicked(function (event) {
                            _this.dropdownList.navigateToFirstRowIfNotActive();
                            if (!_this.isDropdownShown()) {
                                _this.showDropdown();
                            }
                        });
                        this.input.onKeyDown(function (event) {
                            if (event.which == 9) {
                                _this.hideDropdown();
                                return;
                            }
                            else if (event.which == 16 || event.which == 17 || event.which == 18) {
                                return;
                            }
                            _this.dropdownList.navigateToFirstRowIfNotActive();
                            if (!_this.isDropdownShown()) {
                                _this.showDropdown();
                                return;
                            }
                            if (event.which == 38) {
                                _this.dropdownList.navigateToPreviousRow();
                            }
                            else if (event.which == 40) {
                                _this.dropdownList.navigateToNextRow();
                            }
                            else if (event.which == 13) {
                                _this.selectRow(_this.dropdownList.getActiveRow());
                                _this.input.getEl().setValue("");
                                event.preventDefault();
                                event.stopPropagation();
                            }
                            else if (event.which == 27) {
                                _this.hideDropdown();
                            }
                            _this.input.getHTMLElement().focus();
                        });
                    };
                    Dropdown.prototype.setOnBlurListener = function () {
                        var combobox = this;
                        var hideDropdownOnBlur = function (event) {
                            var comboboxHtmlElement = combobox.getHTMLElement();
                            for (var element = event.target; element; element = element.parentNode) {
                                if (element == comboboxHtmlElement) {
                                    return;
                                }
                            }
                            combobox.hideDropdown();
                            combobox.active = false;
                            api.dom.Body.get().getEl().removeEventListener('click', hideDropdownOnBlur);
                        };
                        if (!this.active) {
                            this.active = true;
                            api.dom.Body.get().onClicked(hideDropdownOnBlur);
                        }
                    };
                    Dropdown.prototype.onOptionSelected = function (listener) {
                        this.optionSelectedListeners.push(listener);
                    };
                    Dropdown.prototype.unOptionSelected = function (listener) {
                        this.optionSelectedListeners.filter(function (currentListener) {
                            return listener != currentListener;
                        });
                    };
                    Dropdown.prototype.notifyOptionSelected = function (item) {
                        var event = new OptionSelectedEvent(item);
                        this.optionSelectedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Dropdown.prototype.onValueChanged = function (listener) {
                        this.optionFilterInputValueChangedListeners.push(listener);
                    };
                    Dropdown.prototype.unValueChanged = function (listener) {
                        this.optionFilterInputValueChangedListeners.filter(function (currentListener) {
                            return listener != currentListener;
                        });
                    };
                    Dropdown.prototype.notifyOptionFilterInputValueChanged = function (oldValue, newValue) {
                        var event = new OptionFilterInputValueChangedEvent(oldValue, newValue, this.dropdownList.getDropdownGrid().getElement());
                        this.optionFilterInputValueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Dropdown.prototype.onExpanded = function (listener) {
                        this.expandedListeners.push(listener);
                    };
                    Dropdown.prototype.notifyExpanded = function () {
                        var event = new api.ui.selector.DropdownExpandedEvent(this.dropdownList.getDropdownGrid().getElement(), true);
                        this.expandedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    return Dropdown;
                })(api.dom.FormInputEl);
                dropdown.Dropdown = Dropdown;
            })(dropdown = selector.dropdown || (selector.dropdown = {}));
        })(selector = ui.selector || (ui.selector = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tags;
        (function (tags) {
            var TagBuilder = (function () {
                function TagBuilder() {
                }
                TagBuilder.prototype.setValue = function (value) {
                    this.value = value;
                    return this;
                };
                TagBuilder.prototype.setRemovable = function (value) {
                    this.removable = value;
                    return this;
                };
                TagBuilder.prototype.build = function () {
                    return new Tag(this);
                };
                return TagBuilder;
            })();
            tags.TagBuilder = TagBuilder;
            var Tag = (function (_super) {
                __extends(Tag, _super);
                function Tag(builder) {
                    var _this = this;
                    _super.call(this, "tag");
                    this.tagRemoveListeners = [];
                    this.value = builder.value;
                    this.valueHolderEl = new api.dom.SpanEl();
                    this.valueHolderEl.setHtml(this.value);
                    this.appendChild(this.valueHolderEl);
                    this.removable = builder.removable;
                    if (this.removable) {
                        this.removeButtonEl = new api.dom.AEl("remove-button");
                        this.appendChild(this.removeButtonEl);
                        this.removeButtonEl.onClicked(function () {
                            _this.notifyTagRemoved();
                        });
                    }
                }
                Tag.prototype.getValue = function () {
                    return this.value;
                };
                Tag.prototype.onTagRemove = function (listener) {
                    this.tagRemoveListeners.push(listener);
                };
                Tag.prototype.unTagRemoved = function (listener) {
                    this.tagRemoveListeners.push(listener);
                };
                Tag.prototype.notifyTagRemoved = function () {
                    this.tagRemoveListeners.forEach(function (listener) {
                        listener();
                    });
                };
                return Tag;
            })(api.dom.LiEl);
            tags.Tag = Tag;
        })(tags = ui.tags || (ui.tags = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tags;
        (function (_tags) {
            var TagSuggestions = (function (_super) {
                __extends(TagSuggestions, _super);
                function TagSuggestions() {
                    var _this = this;
                    _super.call(this, 'tag-suggestions');
                    this.selectedIndex = null;
                    this.selectedListeners = [];
                    this.onMouseMove(function (event) {
                        var htmlEl = event.target;
                        if (htmlEl.tagName == 'LI') {
                            _this.notifySelected(htmlEl.innerText || htmlEl.textContent);
                        }
                    });
                }
                TagSuggestions.prototype.setTags = function (values) {
                    var _this = this;
                    this.removeChildren();
                    values.forEach(function (value) {
                        _this.appendChild(new api.dom.LiEl().setHtml(value));
                    });
                    this.selectedIndex = null;
                };
                TagSuggestions.prototype.moveDown = function () {
                    var nextIndex;
                    if (this.selectedIndex == null) {
                        nextIndex = 0;
                    }
                    else if (this.selectedIndex == this.getChildren().length - 1) {
                        nextIndex = null;
                    }
                    else {
                        nextIndex = this.selectedIndex + 1;
                    }
                    this.select(nextIndex);
                };
                TagSuggestions.prototype.moveUp = function () {
                    var nextIndex;
                    if (this.selectedIndex == null) {
                        nextIndex = this.getChildren().length - 1;
                    }
                    else if (this.selectedIndex == 0) {
                        nextIndex = null;
                    }
                    else {
                        nextIndex = this.selectedIndex - 1;
                    }
                    this.select(nextIndex);
                };
                TagSuggestions.prototype.select = function (index) {
                    var tags = this.getChildren();
                    var tag = tags[this.selectedIndex];
                    if (tag) {
                        tag.removeClass('selected');
                    }
                    this.selectedIndex = index;
                    tag = tags[this.selectedIndex];
                    if (tag) {
                        tag.addClass('selected');
                        this.notifySelected(tag.getEl().getText());
                    }
                    else {
                        this.notifySelected(null);
                    }
                };
                TagSuggestions.prototype.onSelected = function (listener) {
                    this.selectedListeners.push(listener);
                };
                TagSuggestions.prototype.unSelected = function (listener) {
                    this.selectedListeners.push(listener);
                };
                TagSuggestions.prototype.notifySelected = function (value) {
                    this.selectedListeners.forEach(function (listener) { return listener(value); });
                };
                return TagSuggestions;
            })(api.dom.UlEl);
            _tags.TagSuggestions = TagSuggestions;
        })(tags = ui.tags || (ui.tags = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tags;
        (function (tags) {
            var TagAddedEvent = (function () {
                function TagAddedEvent(value) {
                    this.value = value;
                }
                TagAddedEvent.prototype.getValue = function () {
                    return this.value;
                };
                return TagAddedEvent;
            })();
            tags.TagAddedEvent = TagAddedEvent;
        })(tags = ui.tags || (ui.tags = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tags;
        (function (tags) {
            var TagRemovedEvent = (function () {
                function TagRemovedEvent(value, index) {
                    this.value = value;
                    this.index = index;
                }
                TagRemovedEvent.prototype.getValue = function () {
                    return this.value;
                };
                TagRemovedEvent.prototype.getIndex = function () {
                    return this.index;
                };
                return TagRemovedEvent;
            })();
            tags.TagRemovedEvent = TagRemovedEvent;
        })(tags = ui.tags || (ui.tags = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var tags;
        (function (tags) {
            var TagsBuilder = (function () {
                function TagsBuilder() {
                    this.tags = [];
                    this.maxTags = 0;
                }
                TagsBuilder.prototype.setTagSuggester = function (value) {
                    this.tagSuggester = value;
                    return this;
                };
                TagsBuilder.prototype.addTag = function (value) {
                    this.tags.push(value);
                    return this;
                };
                TagsBuilder.prototype.setMaxTags = function (value) {
                    this.maxTags = value;
                    return this;
                };
                TagsBuilder.prototype.build = function () {
                    return new Tags(this);
                };
                return TagsBuilder;
            })();
            tags.TagsBuilder = TagsBuilder;
            var Tags = (function (_super) {
                __extends(Tags, _super);
                function Tags(builder) {
                    var _this = this;
                    _super.call(this, "tags");
                    this.tags = [];
                    this.tagAddedListeners = [];
                    this.tagRemovedListeners = [];
                    this.tagSuggester = builder.tagSuggester;
                    builder.tags.forEach(function (value) {
                        _this.doAddTag(value);
                    });
                    this.maxTags = builder.maxTags;
                    this.textInput = new api.ui.text.TextInput();
                    this.textInput.disableAutocomplete();
                    this.appendChild(this.textInput);
                    this.tagSuggestions = new tags.TagSuggestions();
                    this.tagSuggestions.hide();
                    this.appendChild(this.tagSuggestions);
                    this.textInput.onKeyDown(function (event) {
                        if (event.keyCode == 188 || event.keyCode == 13) {
                            _this.handleWordCompleted();
                            event.preventDefault();
                        }
                        else if (event.keyCode == 8) {
                            if (!_this.textInput.getValue() && _this.countTags() > 0) {
                                _this.removeTag(_this.tags[_this.countTags() - 1]);
                            }
                        }
                        else if (event.keyCode == 38) {
                            if (_this.tagSuggestions.isVisible()) {
                                _this.tagSuggestions.moveUp();
                                event.preventDefault();
                            }
                        }
                        else if (event.keyCode == 40) {
                            if (_this.tagSuggestions.isVisible()) {
                                _this.tagSuggestions.moveDown();
                                event.preventDefault();
                            }
                        }
                    });
                    this.tagSuggestions.onSelected(function (value) {
                        _this.textInput.getEl().setValue(value || _this.preservedValue);
                        if (value) {
                            _this.textInput.selectText(_this.preservedValue.length, value.length);
                        }
                    });
                    this.textInput.onBlur(function (event) {
                        _this.handleWordCompleted();
                        _this.textInput.getEl().setValue('').setWidthPx(1);
                    });
                    this.textInput.onFocus(function (event) {
                        _this.textInput.getEl().setWidth('');
                    });
                    this.textInput.onValueChanged(function (event) {
                        if (_this.searchTimeout) {
                            clearTimeout(_this.searchTimeout);
                            _this.searchTimeout = undefined;
                        }
                        var searchString = event.getNewValue();
                        if (api.util.StringHelper.isBlank(searchString)) {
                            _this.tagSuggestions.hide();
                        }
                        else {
                            _this.searchTimeout = setTimeout(function () { return _this.searchSuggestions(searchString); }, 100);
                        }
                    });
                    this.onClicked(function () {
                        _this.textInput.getEl().setWidth('');
                        _this.textInput.giveFocus();
                    });
                }
                Tags.prototype.searchSuggestions = function (searchString) {
                    var _this = this;
                    this.tagSuggester.suggest(searchString).then(function (values) {
                        if (searchString !== _this.textInput.getValue()) {
                            return;
                        }
                        var existingValues = _this.getTags().concat(searchString);
                        values = values.filter(function (value) { return (existingValues.indexOf(value) < 0); });
                        if (values.length == 0) {
                            _this.tagSuggestions.hide();
                        }
                        else {
                            _this.tagSuggestions.setTags(values);
                            _this.tagSuggestions.getEl().setTopPx(_this.textInput.getEl().getOffsetToParent().top + _this.textInput.getEl().getHeightWithMargin()).setLeftPx(_this.textInput.getEl().getOffsetToParent().left);
                            _this.tagSuggestions.show();
                            _this.preservedValue = searchString;
                        }
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                        return [];
                    }).done();
                };
                Tags.prototype.handleWordCompleted = function () {
                    var inputValue = this.textInput.getValue();
                    var word = inputValue.trim();
                    var tag = this.doAddTag(word);
                    if (tag) {
                        this.notifyTagAdded(new tags.TagAddedEvent(tag.getValue()));
                        this.textInput.setValue("");
                        if (this.isMaxTagsReached()) {
                            this.textInput.hide();
                        }
                        this.tagSuggestions.hide();
                    }
                };
                Tags.prototype.clearTags = function () {
                    this.tags.forEach(function (tag) { return tag.remove(); });
                    this.tags = [];
                };
                Tags.prototype.addTag = function (value) {
                    var tag = this.doAddTag(value);
                    if (tag) {
                        if (this.isMaxTagsReached()) {
                            this.textInput.hide();
                        }
                    }
                };
                Tags.prototype.doAddTag = function (value) {
                    var _this = this;
                    if (this.hasTag(value) || !value) {
                        return null;
                    }
                    var tag = new tags.TagBuilder().setValue(value).setRemovable(true).build();
                    this.tags.push(tag);
                    tag.insertBeforeEl(this.textInput);
                    tag.onTagRemove(function () { return _this.removeTag(tag); });
                    return tag;
                };
                Tags.prototype.removeTag = function (tag) {
                    var index = this.tags.indexOf(tag);
                    if (index >= 0) {
                        tag.remove();
                        this.tags.splice(index, 1);
                        if (!this.textInput.isVisible() && !this.isMaxTagsReached()) {
                            this.textInput.setVisible(true);
                        }
                        this.textInput.giveFocus();
                        this.notifyTagRemoved(new tags.TagRemovedEvent(tag.getValue(), index));
                    }
                };
                Tags.prototype.hasTag = function (value) {
                    var match = false;
                    this.tags.forEach(function (tag) {
                        if (value == tag.getValue()) {
                            match = true;
                        }
                    });
                    return match;
                };
                Tags.prototype.countTags = function () {
                    return this.tags.length;
                };
                Tags.prototype.getTags = function () {
                    return this.tags.map(function (tag) { return tag.getValue(); });
                };
                Tags.prototype.isMaxTagsReached = function () {
                    if (this.maxTags == 0) {
                        return false;
                    }
                    return this.countTags() >= this.maxTags;
                };
                Tags.prototype.onTagAdded = function (listener) {
                    this.tagAddedListeners.push(listener);
                };
                Tags.prototype.unTagAdded = function (listener) {
                    this.tagAddedListeners.push(listener);
                };
                Tags.prototype.notifyTagAdded = function (event) {
                    this.tagAddedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                Tags.prototype.onTagRemoved = function (listener) {
                    this.tagRemovedListeners.push(listener);
                };
                Tags.prototype.unTagRemoved = function (listener) {
                    this.tagRemovedListeners.push(listener);
                };
                Tags.prototype.giveFocus = function () {
                    if (this.isMaxTagsReached()) {
                        return this.tags[0].giveFocus();
                    }
                    else {
                        this.textInput.getEl().setWidth('');
                        return this.textInput.giveFocus();
                    }
                };
                Tags.prototype.notifyTagRemoved = function (event) {
                    this.tagRemovedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return Tags;
            })(api.dom.UlEl);
            tags.Tags = Tags;
        })(tags = ui.tags || (ui.tags = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var geo;
        (function (geo) {
            var GeoPoint = (function (_super) {
                __extends(GeoPoint, _super);
                function GeoPoint() {
                    var _this = this;
                    _super.call(this, "geo-point");
                    this.valueChangedListeners = [];
                    this.validUserInput = true;
                    this.geoLocationInput = new api.ui.text.TextInput();
                    this.geoLocationInput.getEl().setAttribute("title", "latitude,longitude").addClass("geo-point-input");
                    this.layoutItems();
                    this.geoLocationInput.onKeyUp(function (event) {
                        var typedGeoPoint = _this.geoLocationInput.getValue();
                        _this.validUserInput = api.util.StringHelper.isEmpty(typedGeoPoint) || api.util.GeoPoint.isValidString(typedGeoPoint);
                        _this.updateInputStyling();
                        _this.notifyValueChanged();
                    });
                }
                GeoPoint.prototype.layoutItems = function () {
                    this.removeChildren();
                    this.appendChild(this.geoLocationInput);
                    this.geoLocationInput.setPlaceholder(_i18n('latitude,longitude'));
                    return this;
                };
                GeoPoint.prototype.onValueChanged = function (listener) {
                    this.valueChangedListeners.push(listener);
                };
                GeoPoint.prototype.notifyValueChanged = function () {
                    var _this = this;
                    var newValue = this.validUserInput ? this.geoLocationInput.getValue() : null;
                    var oldValue = this.geoLocationInput.getOldValue();
                    this.valueChangedListeners.forEach(function (listener) {
                        listener.call(_this, new ui.ValueChangedEvent(oldValue, newValue));
                    });
                };
                GeoPoint.prototype.setGeoPoint = function (value) {
                    this.geoLocationInput.setValue("" + value.getLatitude() + "," + value.getLongitude());
                    return this;
                };
                GeoPoint.prototype.getGeoPoint = function () {
                    if (api.util.StringHelper.isEmpty(this.geoLocationInput.getValue())) {
                        return null;
                    }
                    return api.util.GeoPoint.fromString(this.geoLocationInput.getValue());
                };
                GeoPoint.prototype.giveFocus = function () {
                    return this.geoLocationInput.giveFocus();
                };
                GeoPoint.prototype.hasValidUserInput = function () {
                    return this.validUserInput;
                };
                GeoPoint.prototype.updateInputStyling = function () {
                    this.geoLocationInput.updateValidationStatusOnUserInput(this.validUserInput);
                };
                return GeoPoint;
            })(api.dom.DivEl);
            geo.GeoPoint = GeoPoint;
        })(geo = ui.geo || (ui.geo = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var DayOfWeek = (function () {
                function DayOfWeek(numberCode, oneLetterName, shortName, fullName) {
                    this.numberCode = numberCode;
                    this.oneLetterName = oneLetterName;
                    this.shortName = shortName;
                    this.fullName = fullName;
                }
                DayOfWeek.prototype.getNumberCode = function () {
                    return this.numberCode;
                };
                DayOfWeek.prototype.getOneLetterName = function () {
                    return this.oneLetterName;
                };
                DayOfWeek.prototype.getShortName = function () {
                    return this.shortName;
                };
                DayOfWeek.prototype.getFullName = function () {
                    return this.fullName;
                };
                DayOfWeek.prototype.getPrevioius = function () {
                    return this.previous;
                };
                DayOfWeek.prototype.getNext = function () {
                    return this.next;
                };
                DayOfWeek.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, DayOfWeek)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.numberEquals(this.numberCode, other.numberCode)) {
                        return false;
                    }
                    return true;
                };
                return DayOfWeek;
            })();
            time.DayOfWeek = DayOfWeek;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var DaysOfWeek = (function () {
                function DaysOfWeek() {
                }
                DaysOfWeek.getByNumberCode = function (value) {
                    var match = null;
                    DaysOfWeek.ALL.forEach(function (dayOfWeek) {
                        if (dayOfWeek.getNumberCode() == value) {
                            match = dayOfWeek;
                        }
                    });
                    return match;
                };
                DaysOfWeek.MONDAY = new time.DayOfWeek(1, "M", "Mon", "Monday");
                DaysOfWeek.TUESDAY = new time.DayOfWeek(2, "T", "Tue", "Tuesday");
                DaysOfWeek.WEDNESDAY = new time.DayOfWeek(3, "W", "Wed", "Wednesday");
                DaysOfWeek.THURSDAY = new time.DayOfWeek(4, "T", "Thu", "Thursday");
                DaysOfWeek.FRIDAY = new time.DayOfWeek(5, "F", "Fri", "Friday");
                DaysOfWeek.SATURDAY = new time.DayOfWeek(6, "S", "Sat", "Saturday");
                DaysOfWeek.SUNDAY = new time.DayOfWeek(0, "S", "Sun", "Sunday");
                DaysOfWeek.ALL = [DaysOfWeek.MONDAY, DaysOfWeek.TUESDAY, DaysOfWeek.WEDNESDAY, DaysOfWeek.THURSDAY, DaysOfWeek.FRIDAY, DaysOfWeek.SATURDAY, DaysOfWeek.SUNDAY];
                return DaysOfWeek;
            })();
            time.DaysOfWeek = DaysOfWeek;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var CalendarDayBuilder = (function () {
                function CalendarDayBuilder() {
                }
                CalendarDayBuilder.prototype.setDate = function (value) {
                    this.date = value;
                    return this;
                };
                CalendarDayBuilder.prototype.setMonth = function (value) {
                    this.month = value;
                    return this;
                };
                CalendarDayBuilder.prototype.setPreviousDay = function (value) {
                    this.previousDay = value;
                    return this;
                };
                CalendarDayBuilder.prototype.setNextDay = function (value) {
                    this.nextDay = value;
                    return this;
                };
                CalendarDayBuilder.prototype.build = function () {
                    return new CalendarDay(this);
                };
                return CalendarDayBuilder;
            })();
            time.CalendarDayBuilder = CalendarDayBuilder;
            var CalendarDay = (function (_super) {
                __extends(CalendarDay, _super);
                function CalendarDay(builder) {
                    var _this = this;
                    _super.call(this, "calendar-day");
                    this.selectedDay = false;
                    this.calendarDayClickedListeners = [];
                    this.date = builder.date;
                    this.month = builder.month;
                    this.previousDay = builder.previousDay;
                    if (this.previousDay) {
                        this.previousDay.nextDay = this;
                    }
                    this.nextDay = builder.nextDay;
                    if (this.nextDay) {
                        this.nextDay.previousDay = this;
                    }
                    if (new Date().toDateString() == builder.date.toDateString()) {
                        this.addClass('today');
                    }
                    this.dayOfWeek = time.DaysOfWeek.getByNumberCode(this.date.getDay());
                    this.setHtml("" + this.date.getDate());
                    if (this.isBeforeMonth()) {
                        this.addClass("before-month");
                    }
                    if (this.isAfterMonth()) {
                        this.addClass("after-month");
                    }
                    this.onClicked(function (event) {
                        _this.notifyCalendarDayClicked();
                    });
                }
                CalendarDay.prototype.getDate = function () {
                    return this.date;
                };
                CalendarDay.prototype.getDayOfMonth = function () {
                    return this.date.getDate();
                };
                CalendarDay.prototype.setSelectedDay = function (value) {
                    this.selectedDay = value;
                    this.refreshSelectedDay();
                };
                CalendarDay.prototype.refreshSelectedDay = function () {
                    if (this.selectedDay && !this.hasClass("selected-day")) {
                        this.addClass("selected-day");
                    }
                    else if (!this.selectedDay && this.hasClass("selected-day")) {
                        this.removeClass("selected-day");
                    }
                };
                CalendarDay.prototype.isInMonth = function () {
                    return !this.isBeforeMonth() || !this.isAfterMonth();
                };
                CalendarDay.prototype.isBeforeMonth = function () {
                    return this.date.getMonth() < this.month || (this.month == 0 && this.date.getMonth() == 11);
                };
                CalendarDay.prototype.isAfterMonth = function () {
                    return this.date.getMonth() > this.month || (this.month == 11 && this.date.getMonth() == 0);
                };
                CalendarDay.prototype.isLastDayOfMonth = function (month) {
                    var lastDateOfMonth = new Date(this.date.getFullYear(), month + 1, 0);
                    return month == this.date.getMonth() && this.date.getDate() == lastDateOfMonth.getDate();
                };
                CalendarDay.prototype.getDayOfWeek = function () {
                    return this.dayOfWeek;
                };
                CalendarDay.prototype.getPrevious = function () {
                    if (this.previousDay) {
                        return this.previousDay;
                    }
                    else {
                        var prevDate = new Date(this.date.getFullYear(), this.date.getMonth(), this.date.getDate() - 1);
                        this.previousDay = new CalendarDayBuilder().setDate(prevDate).setMonth(this.month).setNextDay(this).build();
                        return this.previousDay;
                    }
                };
                CalendarDay.prototype.getNext = function () {
                    if (this.nextDay) {
                        return this.nextDay;
                    }
                    else {
                        var nextDate = new Date(this.date.getFullYear(), this.date.getMonth(), this.date.getDate() + 1);
                        this.nextDay = new CalendarDayBuilder().setDate(nextDate).setMonth(this.month).setPreviousDay(this).build();
                        return this.nextDay;
                    }
                };
                CalendarDay.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, CalendarDay)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.dateEquals(this.date, other.date)) {
                        return false;
                    }
                    if (!api.ObjectHelper.numberEquals(this.month, other.month)) {
                        return false;
                    }
                    return true;
                };
                CalendarDay.prototype.onCalendarDayClicked = function (listener) {
                    this.calendarDayClickedListeners.push(listener);
                };
                CalendarDay.prototype.unCalendarDayClicked = function (listener) {
                    this.calendarDayClickedListeners = this.calendarDayClickedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                CalendarDay.prototype.notifyCalendarDayClicked = function () {
                    var event = new time.CalendarDayClickedEvent(this);
                    this.calendarDayClickedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return CalendarDay;
            })(api.dom.LiEl);
            time.CalendarDay = CalendarDay;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var CalendarDayClickedEvent = (function () {
                function CalendarDayClickedEvent(calendarDay) {
                    this.calendarDay = calendarDay;
                }
                CalendarDayClickedEvent.prototype.getCalendarDay = function () {
                    return this.calendarDay;
                };
                return CalendarDayClickedEvent;
            })();
            time.CalendarDayClickedEvent = CalendarDayClickedEvent;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var CalendarWeekBuilder = (function () {
                function CalendarWeekBuilder() {
                    this.calendarDays = [];
                }
                CalendarWeekBuilder.prototype.addDay = function (value) {
                    this.calendarDays.push(value);
                    return this;
                };
                CalendarWeekBuilder.prototype.build = function () {
                    return new CalendarWeek(this);
                };
                return CalendarWeekBuilder;
            })();
            time.CalendarWeekBuilder = CalendarWeekBuilder;
            var CalendarWeek = (function (_super) {
                __extends(CalendarWeek, _super);
                function CalendarWeek(builder) {
                    var _this = this;
                    this.calendarDays = builder.calendarDays;
                    _super.call(this, "calendar-week");
                    this.calendarDays.forEach(function (day) {
                        _this.appendChild(day);
                    });
                }
                CalendarWeek.prototype.hasLastDayOfMonth = function (month) {
                    var match = false;
                    this.calendarDays.forEach(function (day) {
                        if (day.isLastDayOfMonth(month)) {
                            match = true;
                        }
                    });
                    return match;
                };
                CalendarWeek.prototype.getNextWeeksFirstDay = function () {
                    return this.calendarDays[6].getNext();
                };
                return CalendarWeek;
            })(api.dom.UlEl);
            time.CalendarWeek = CalendarWeek;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var SelectedDateChangedEvent = (function () {
                function SelectedDateChangedEvent(selectedDate) {
                    this.date = selectedDate;
                }
                SelectedDateChangedEvent.prototype.getDate = function () {
                    return this.date;
                };
                return SelectedDateChangedEvent;
            })();
            time.SelectedDateChangedEvent = SelectedDateChangedEvent;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var CalendarBuilder = (function () {
                function CalendarBuilder() {
                    this.interactive = false;
                }
                CalendarBuilder.prototype.setYear = function (value) {
                    this.year = value;
                    return this;
                };
                CalendarBuilder.prototype.setMonth = function (value) {
                    this.month = value;
                    return this;
                };
                CalendarBuilder.prototype.setSelectedDate = function (value) {
                    this.selectedDate = value;
                    return this;
                };
                CalendarBuilder.prototype.setStartingDayOfWeek = function (value) {
                    this.startingDayOfWeek = value;
                    return this;
                };
                CalendarBuilder.prototype.setInteractive = function (value) {
                    this.interactive = value;
                    return this;
                };
                CalendarBuilder.prototype.build = function () {
                    return new Calendar(this);
                };
                return CalendarBuilder;
            })();
            time.CalendarBuilder = CalendarBuilder;
            var Calendar = (function (_super) {
                __extends(Calendar, _super);
                function Calendar(builder) {
                    _super.call(this, "calendar");
                    this.selectedDateChangedListeners = [];
                    this.shownMonthChangedListeners = [];
                    var now = new Date();
                    this.year = builder.year || now.getFullYear();
                    this.month = builder.month != undefined ? builder.month : now.getMonth();
                    this.selectedDate = builder.selectedDate;
                    this.startingDayOfWeek = builder.startingDayOfWeek || time.DaysOfWeek.MONDAY;
                    this.interactive = builder.interactive;
                    this.renderMonth();
                }
                Calendar.prototype.selectDate = function (value) {
                    if (value) {
                        this.year = value.getFullYear();
                        this.month = value.getMonth();
                        this.selectedDate = value;
                        this.removeChildren();
                        if (api.util.DateHelper.isInvalidDate(value)) {
                            var spanEl = new api.dom.SpanEl().setHtml("Invalid date");
                            this.appendChild(spanEl);
                        }
                        else {
                            this.renderMonth();
                        }
                    }
                    else {
                        this.selectedDate = null;
                        var now = new Date();
                        this.year = now.getFullYear();
                        this.month = now.getMonth();
                    }
                };
                Calendar.prototype.nextMonth = function () {
                    this.month++;
                    if (this.month > 11) {
                        this.month = 0;
                        this.year++;
                    }
                    this.removeChildren();
                    this.renderMonth();
                };
                Calendar.prototype.previousMonth = function () {
                    this.month--;
                    if (this.month < 0) {
                        this.month = 11;
                        this.year--;
                    }
                    this.removeChildren();
                    this.renderMonth();
                };
                Calendar.prototype.nextYear = function () {
                    this.year++;
                    this.removeChildren();
                    this.renderMonth();
                };
                Calendar.prototype.previousYear = function () {
                    this.year--;
                    this.removeChildren();
                    this.renderMonth();
                };
                Calendar.prototype.renderMonth = function () {
                    var _this = this;
                    this.calendarDays = this.resolveDaysInMonth();
                    var firstDay = this.resolveFirstDayOfCalendar();
                    this.weeks = this.createCalendarWeeks(firstDay);
                    this.weeks.forEach(function (week) {
                        _this.appendChild(week);
                    });
                    this.notifyShownMonthChanged(this.month, this.year);
                };
                Calendar.prototype.resolveDaysInMonth = function () {
                    var calendarDays = [];
                    var daysInMonth = api.util.DateHelper.numDaysInMonth(this.year, this.month);
                    var previousDay = null;
                    for (var i = 1; i <= daysInMonth; i++) {
                        var calendarDay = this.createCalendarDay(i, previousDay);
                        calendarDays.push(calendarDay);
                        previousDay = calendarDay;
                    }
                    return calendarDays;
                };
                Calendar.prototype.resolveFirstDayOfCalendar = function () {
                    var firstDay = null;
                    if (this.startingDayOfWeek.equals(this.calendarDays[0].getDayOfWeek())) {
                        firstDay = this.calendarDays[0];
                    }
                    else {
                        var previousDay = this.calendarDays[0].getPrevious();
                        while (!previousDay.getDayOfWeek().equals(this.startingDayOfWeek)) {
                            previousDay = previousDay.getPrevious();
                        }
                        firstDay = previousDay;
                    }
                    return firstDay;
                };
                Calendar.prototype.createCalendarWeeks = function (firstDay) {
                    var weeks = [];
                    var currWeek = this.createCalendarWeek(firstDay);
                    weeks.push(currWeek);
                    while (!currWeek.hasLastDayOfMonth(this.month)) {
                        var newWeek = this.createCalendarWeek(currWeek.getNextWeeksFirstDay());
                        weeks.push(newWeek);
                        currWeek = newWeek;
                    }
                    return weeks;
                };
                Calendar.prototype.createCalendarWeek = function (firstDayOfWeek) {
                    var weekBuilder = new time.CalendarWeekBuilder();
                    weekBuilder.addDay(firstDayOfWeek);
                    var nextDay = firstDayOfWeek.getNext();
                    for (var i = 0; i < 6; i++) {
                        weekBuilder.addDay(nextDay);
                        nextDay = nextDay.getNext();
                    }
                    return weekBuilder.build();
                };
                Calendar.prototype.createCalendarDay = function (dayOfMonth, previousDay) {
                    var _this = this;
                    var date = new Date(this.year, this.month, dayOfMonth);
                    var calendarDay = new time.CalendarDayBuilder().setDate(date).setMonth(this.month).setPreviousDay(previousDay).build();
                    if (calendarDay.isInMonth()) {
                        if (this.interactive) {
                            calendarDay.onCalendarDayClicked(function (event) { return _this.handleCalendarDayClicked(event); });
                        }
                        if (this.selectedDate && this.selectedDate.toDateString() == date.toDateString()) {
                            calendarDay.setSelectedDay(true);
                        }
                    }
                    return calendarDay;
                };
                Calendar.prototype.handleCalendarDayClicked = function (event) {
                    this.calendarDays.forEach(function (calendarDay) {
                        calendarDay.setSelectedDay(event.getCalendarDay().equals(calendarDay));
                    });
                    this.selectedDate = event.getCalendarDay().getDate();
                    this.notifySelectedDateChanged(this.selectedDate);
                };
                Calendar.prototype.getSelectedDate = function () {
                    return this.selectedDate;
                };
                Calendar.prototype.getMonth = function () {
                    return this.month;
                };
                Calendar.prototype.getYear = function () {
                    return this.year;
                };
                Calendar.prototype.getCalendarDays = function () {
                    return this.calendarDays;
                };
                Calendar.prototype.onSelectedDateChanged = function (listener) {
                    this.selectedDateChangedListeners.push(listener);
                };
                Calendar.prototype.unSelectedDateChanged = function (listener) {
                    this.selectedDateChangedListeners = this.selectedDateChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                Calendar.prototype.notifySelectedDateChanged = function (date) {
                    var event = new time.SelectedDateChangedEvent(date);
                    this.selectedDateChangedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                Calendar.prototype.onShownMonthChanged = function (listener) {
                    this.shownMonthChangedListeners.push(listener);
                };
                Calendar.prototype.unShownMonthChanged = function (listener) {
                    this.shownMonthChangedListeners = this.shownMonthChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                Calendar.prototype.notifyShownMonthChanged = function (month, year) {
                    this.shownMonthChangedListeners.forEach(function (listener) {
                        listener(month, year);
                    });
                };
                return Calendar;
            })(api.dom.DivEl);
            time.Calendar = Calendar;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var DatePickerBuilder = (function () {
                function DatePickerBuilder() {
                    this.startingDayOfWeek = time.DaysOfWeek.MONDAY;
                    this.closeOnSelect = true;
                    this.closeOnOutsideClick = true;
                }
                DatePickerBuilder.prototype.setYear = function (value) {
                    this.year = value;
                    return this;
                };
                DatePickerBuilder.prototype.setMonth = function (value) {
                    this.month = value;
                    return this;
                };
                DatePickerBuilder.prototype.setSelectedDate = function (value) {
                    this.selectedDate = value;
                    return this;
                };
                DatePickerBuilder.prototype.setCalendar = function (value) {
                    this.calendar = value;
                    return this;
                };
                DatePickerBuilder.prototype.setStartingDayOfWeek = function (value) {
                    this.startingDayOfWeek = value;
                    return this;
                };
                DatePickerBuilder.prototype.setCloseOnSelect = function (value) {
                    this.closeOnSelect = value;
                    return this;
                };
                DatePickerBuilder.prototype.setCloseOnOutsideClick = function (value) {
                    this.closeOnOutsideClick = value;
                    return this;
                };
                DatePickerBuilder.prototype.build = function () {
                    return new DatePicker(this);
                };
                return DatePickerBuilder;
            })();
            time.DatePickerBuilder = DatePickerBuilder;
            var DatePicker = (function (_super) {
                __extends(DatePicker, _super);
                function DatePicker(builder) {
                    var _this = this;
                    _super.call(this, 'date-picker');
                    this.selectedDateChangedListeners = [];
                    var onDatePickerShown = this.onDatePickerShown.bind(this);
                    DatePickerShownEvent.on(onDatePickerShown);
                    this.onRemoved(function (event) {
                        DatePickerShownEvent.un(onDatePickerShown);
                    });
                    this.validUserInput = true;
                    this.input = api.ui.text.TextInput.middle();
                    this.input.onClicked(function (e) {
                        e.preventDefault();
                        _this.popup.show();
                    });
                    var wrapper = new api.dom.DivEl('wrapper');
                    wrapper.appendChild(this.input);
                    this.calendar = builder.calendar || new time.CalendarBuilder().setSelectedDate(builder.selectedDate).setMonth(builder.month).setYear(builder.year).setInteractive(true).build();
                    var popupBuilder = new time.DatePickerPopupBuilder().setCalendar(this.calendar).setCloseOnOutsideClick(false);
                    this.popup = popupBuilder.build();
                    this.popup.onShown(function () {
                        new DatePickerShownEvent(_this).fire();
                    });
                    wrapper.appendChild(this.popup);
                    this.popupTrigger = new api.ui.button.Button();
                    this.popupTrigger.addClass('icon-calendar4');
                    wrapper.appendChild(this.popupTrigger);
                    this.appendChild(wrapper);
                    this.popupTrigger.onClicked(function (e) {
                        e.preventDefault();
                        if (_this.popup.isVisible()) {
                            _this.popup.hide();
                        }
                        else {
                            _this.popup.show();
                        }
                    });
                    if (builder.selectedDate) {
                        this.input.setValue(this.formatDate(builder.selectedDate));
                        this.selectedDate = builder.selectedDate;
                    }
                    this.popup.onSelectedDateChanged(function (e) {
                        if (builder.closeOnSelect) {
                            _this.popup.hide();
                        }
                        _this.selectedDate = e.getDate();
                        _this.validUserInput = true;
                        _this.input.setValue(_this.formatDate(e.getDate()));
                        _this.notifySelectedDateChanged(e);
                        _this.updateInputStyling();
                    });
                    this.input.onKeyUp(function (event) {
                        var typedDate = _this.input.getValue();
                        if (api.util.StringHelper.isEmpty(typedDate)) {
                            _this.calendar.selectDate(null);
                            _this.selectedDate = null;
                            _this.validUserInput = true;
                            _this.popup.hide();
                            _this.notifySelectedDateChanged(new time.SelectedDateChangedEvent(null));
                        }
                        else {
                            var date = api.util.DateHelper.parseDate(typedDate);
                            if (date) {
                                _this.selectedDate = date;
                                _this.validUserInput = true;
                                _this.calendar.selectDate(date);
                                _this.notifySelectedDateChanged(new time.SelectedDateChangedEvent(date));
                                if (!_this.popup.isVisible()) {
                                    _this.popup.show();
                                }
                            }
                            else {
                                _this.selectedDate = null;
                                _this.validUserInput = false;
                                _this.notifySelectedDateChanged(new time.SelectedDateChangedEvent(null));
                            }
                        }
                        _this.updateInputStyling();
                    });
                    this.popup.onKeyDown(function (event) {
                        if (api.ui.KeyHelper.isTabKey(event)) {
                            if (!(document.activeElement == _this.input.getEl().getHTMLElement())) {
                                event.preventDefault();
                                event.stopPropagation();
                                _this.popup.hide();
                                _this.popupTrigger.giveFocus();
                            }
                        }
                    });
                    this.input.onKeyDown(function (event) {
                        if (api.ui.KeyHelper.isTabKey(event)) {
                            if (!event.shiftKey) {
                                event.preventDefault();
                                event.stopPropagation();
                                _this.popupTrigger.giveFocus();
                            }
                            else {
                                _this.popup.hide();
                            }
                        }
                    });
                    this.popupTrigger.onKeyDown(function (event) {
                        if (api.ui.KeyHelper.isTabKey(event)) {
                            _this.popup.hide();
                        }
                    });
                    api.dom.Body.get().onKeyDown(function (e) { return _this.popupTabListener(e); });
                    if (builder.closeOnOutsideClick) {
                        api.dom.Body.get().onClicked(function (e) { return _this.outsideClickListener(e); });
                    }
                }
                DatePicker.prototype.popupTabListener = function (e) {
                    if (api.ui.KeyHelper.isTabKey(e) && !this.getEl().contains(e.target)) {
                        if (this.popup.isVisible()) {
                            e.stopPropagation();
                            e.preventDefault();
                            this.popupTrigger.getEl().focus();
                            this.popup.hide();
                        }
                    }
                };
                DatePicker.prototype.outsideClickListener = function (e) {
                    if (!this.getEl().contains(e.target)) {
                        this.popup.hide();
                    }
                };
                DatePicker.prototype.onDatePickerShown = function (event) {
                    if (event.getDatePicker() !== this) {
                        this.popup.hide();
                    }
                };
                DatePicker.prototype.hasValidUserInput = function () {
                    return this.validUserInput;
                };
                DatePicker.prototype.getSelectedDate = function () {
                    return this.selectedDate;
                };
                DatePicker.prototype.onSelectedDateChanged = function (listener) {
                    this.selectedDateChangedListeners.push(listener);
                };
                DatePicker.prototype.unSelectedDateChanged = function (listener) {
                    this.selectedDateChangedListeners = this.selectedDateChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                DatePicker.prototype.notifySelectedDateChanged = function (event) {
                    this.selectedDateChangedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                DatePicker.prototype.formatDate = function (date) {
                    return api.util.DateHelper.formatDate(date);
                };
                DatePicker.prototype.updateInputStyling = function () {
                    this.input.updateValidationStatusOnUserInput(this.validUserInput);
                };
                return DatePicker;
            })(api.dom.DivEl);
            time.DatePicker = DatePicker;
            var DatePickerShownEvent = (function (_super) {
                __extends(DatePickerShownEvent, _super);
                function DatePickerShownEvent(datePicker) {
                    _super.call(this);
                    this.datePicker = datePicker;
                }
                DatePickerShownEvent.prototype.getDatePicker = function () {
                    return this.datePicker;
                };
                DatePickerShownEvent.on = function (handler) {
                    api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
                };
                DatePickerShownEvent.un = function (handler) {
                    api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
                };
                return DatePickerShownEvent;
            })(api.event.Event);
            time.DatePickerShownEvent = DatePickerShownEvent;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var DatePickerPopupBuilder = (function () {
                function DatePickerPopupBuilder() {
                    this.closeOnOutsideClick = true;
                }
                DatePickerPopupBuilder.prototype.setCalendar = function (value) {
                    this.calendar = value;
                    return this;
                };
                DatePickerPopupBuilder.prototype.getCalendar = function () {
                    return this.calendar;
                };
                DatePickerPopupBuilder.prototype.setCloseOnOutsideClick = function (value) {
                    this.closeOnOutsideClick = value;
                    return this;
                };
                DatePickerPopupBuilder.prototype.isCloseOnOutsideClick = function () {
                    return this.closeOnOutsideClick;
                };
                DatePickerPopupBuilder.prototype.build = function () {
                    return new DatePickerPopup(this);
                };
                return DatePickerPopupBuilder;
            })();
            time.DatePickerPopupBuilder = DatePickerPopupBuilder;
            var DatePickerPopup = (function (_super) {
                __extends(DatePickerPopup, _super);
                function DatePickerPopup(builder) {
                    var _this = this;
                    _super.call(this, 'date-picker-dialog');
                    var yearContainer = new api.dom.H2El('year-container');
                    this.appendChild(yearContainer);
                    this.prevYear = new api.dom.AEl('prev');
                    this.prevYear.onClicked(function (e) {
                        _this.calendar.previousYear();
                    });
                    yearContainer.appendChild(this.prevYear);
                    this.year = new api.dom.SpanEl();
                    yearContainer.appendChild(this.year);
                    this.nextYear = new api.dom.AEl('next');
                    this.nextYear.onClicked(function (e) {
                        _this.calendar.nextYear();
                    });
                    yearContainer.appendChild(this.nextYear);
                    var monthContainer = new api.dom.H5El('month-container');
                    this.appendChild(monthContainer);
                    this.prevMonth = new api.dom.AEl('prev');
                    this.prevMonth.onClicked(function (e) {
                        _this.calendar.previousMonth();
                    });
                    monthContainer.appendChild(this.prevMonth);
                    this.month = new api.dom.SpanEl();
                    monthContainer.appendChild(this.month);
                    this.nextMonth = new api.dom.AEl('next');
                    this.nextMonth.onClicked(function (e) {
                        _this.calendar.nextMonth();
                    });
                    monthContainer.appendChild(this.nextMonth);
                    this.calendar = builder.getCalendar() || new time.CalendarBuilder().build();
                    this.year.setHtml(this.calendar.getYear().toString());
                    this.month.setHtml(time.MonthsOfYear.getByNumberCode(this.calendar.getMonth()).getFullName());
                    this.calendar.onShownMonthChanged(function (month, year) {
                        _this.month.setHtml(time.MonthsOfYear.getByNumberCode(month).getFullName());
                        _this.year.setHtml(year.toString());
                    });
                    this.appendChild(this.calendar);
                    if (builder.isCloseOnOutsideClick()) {
                        api.dom.Body.get().onClicked(function (e) { return _this.outsideClickListener(e); });
                    }
                }
                DatePickerPopup.prototype.getSelectedDate = function () {
                    return this.calendar.getSelectedDate();
                };
                DatePickerPopup.prototype.getCalendar = function () {
                    return this.calendar;
                };
                DatePickerPopup.prototype.onSelectedDateChanged = function (listener) {
                    this.calendar.onSelectedDateChanged(listener);
                };
                DatePickerPopup.prototype.unSelectedDateChanged = function (listener) {
                    this.calendar.unSelectedDateChanged(listener);
                };
                DatePickerPopup.prototype.outsideClickListener = function (e) {
                    if (!this.getEl().contains(e.target)) {
                        this.hide();
                    }
                };
                return DatePickerPopup;
            })(api.dom.DivEl);
            time.DatePickerPopup = DatePickerPopup;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var MonthOfYear = (function () {
                function MonthOfYear(numberCode, oneLetterName, shortName, fullName) {
                    this.numberCode = numberCode;
                    this.oneLetterName = oneLetterName;
                    this.shortName = shortName;
                    this.fullName = fullName;
                }
                MonthOfYear.prototype.getNumberCode = function () {
                    return this.numberCode;
                };
                MonthOfYear.prototype.getOneLetterName = function () {
                    return this.oneLetterName;
                };
                MonthOfYear.prototype.getShortName = function () {
                    return this.shortName;
                };
                MonthOfYear.prototype.getFullName = function () {
                    return this.fullName;
                };
                MonthOfYear.prototype.getPrevioius = function () {
                    return this.previous;
                };
                MonthOfYear.prototype.getNext = function () {
                    return this.next;
                };
                MonthOfYear.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, MonthOfYear)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.numberEquals(this.numberCode, other.numberCode)) {
                        return false;
                    }
                    return true;
                };
                return MonthOfYear;
            })();
            time.MonthOfYear = MonthOfYear;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var MonthsOfYear = (function () {
                function MonthsOfYear() {
                }
                MonthsOfYear.getByNumberCode = function (code) {
                    return MonthsOfYear.monthsByCode[code];
                };
                MonthsOfYear.JANUARY = new time.MonthOfYear(0, "J", "Jan", "January");
                MonthsOfYear.FEBRUARY = new time.MonthOfYear(1, "F", "Feb", "February");
                MonthsOfYear.MARCH = new time.MonthOfYear(2, "M", "Mar", "March");
                MonthsOfYear.APRIL = new time.MonthOfYear(3, "A", "Apr", "April");
                MonthsOfYear.MAY = new time.MonthOfYear(4, "M", "May", "May");
                MonthsOfYear.JUNE = new time.MonthOfYear(5, "J", "Jun", "June");
                MonthsOfYear.JULY = new time.MonthOfYear(6, "J", "Jul", "July");
                MonthsOfYear.AUGUST = new time.MonthOfYear(7, "A", "Aug", "August");
                MonthsOfYear.SEPTEMBER = new time.MonthOfYear(8, "S", "Sep", "September");
                MonthsOfYear.OCTOBER = new time.MonthOfYear(9, "O", "Oct", "October");
                MonthsOfYear.NOVEMBER = new time.MonthOfYear(10, "N", "Nov", "November");
                MonthsOfYear.DECEMBER = new time.MonthOfYear(11, "D", "Dec", "December");
                MonthsOfYear.monthsByCode = {
                    0: MonthsOfYear.JANUARY,
                    1: MonthsOfYear.FEBRUARY,
                    2: MonthsOfYear.MARCH,
                    3: MonthsOfYear.APRIL,
                    4: MonthsOfYear.MAY,
                    5: MonthsOfYear.JUNE,
                    6: MonthsOfYear.JULY,
                    7: MonthsOfYear.AUGUST,
                    8: MonthsOfYear.SEPTEMBER,
                    9: MonthsOfYear.OCTOBER,
                    10: MonthsOfYear.NOVEMBER,
                    11: MonthsOfYear.DECEMBER
                };
                return MonthsOfYear;
            })();
            time.MonthsOfYear = MonthsOfYear;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var TimePickerBuilder = (function () {
                function TimePickerBuilder() {
                    this.closeOnOutsideClick = true;
                }
                TimePickerBuilder.prototype.setHours = function (value) {
                    this.hours = value;
                    return this;
                };
                TimePickerBuilder.prototype.setMinutes = function (value) {
                    this.minutes = value;
                    return this;
                };
                TimePickerBuilder.prototype.setCloseOnOutsideClick = function (value) {
                    this.closeOnOutsideClick = value;
                    return this;
                };
                TimePickerBuilder.prototype.build = function () {
                    return new TimePicker(this);
                };
                return TimePickerBuilder;
            })();
            time.TimePickerBuilder = TimePickerBuilder;
            var TimePicker = (function (_super) {
                __extends(TimePicker, _super);
                function TimePicker(builder) {
                    var _this = this;
                    _super.call(this, 'time-picker');
                    this.validUserInput = true;
                    this.input = api.ui.text.TextInput.middle();
                    this.input.onClicked(function (e) {
                        e.preventDefault();
                        _this.popup.show();
                    });
                    this.input.onKeyUp(function (event) {
                        var typedTime = _this.input.getValue();
                        if (api.util.StringHelper.isEmpty(typedTime)) {
                            _this.validUserInput = true;
                            _this.popup.setSelectedTime(null, null);
                            if (_this.popup.isVisible()) {
                                _this.popup.hide();
                            }
                        }
                        else {
                            var parsedTime = typedTime.match(/^[0-2][0-9]:[0-5][0-9]$/);
                            if (parsedTime && parsedTime.length == 1) {
                                var splitTime = parsedTime[0].split(':');
                                _this.validUserInput = true;
                                _this.popup.setSelectedTime(parseInt(splitTime[0]), parseInt(splitTime[1]));
                                if (!_this.popup.isVisible()) {
                                    _this.popup.show();
                                }
                            }
                            else {
                                _this.validUserInput = false;
                                _this.popup.setSelectedTime(null, null);
                            }
                        }
                        _this.updateInputStyling();
                    });
                    var wrapper = new api.dom.DivEl('wrapper');
                    wrapper.appendChild(this.input);
                    var popupBuilder = new time.TimePickerPopupBuilder().setHours(builder.hours).setMinutes(builder.minutes).setCloseOnOutsideClick(false);
                    this.popup = popupBuilder.build();
                    wrapper.appendChild(this.popup);
                    this.popupTrigger = new api.ui.button.Button();
                    this.popupTrigger.addClass('icon-clock');
                    wrapper.appendChild(this.popupTrigger);
                    this.appendChild(wrapper);
                    this.popupTrigger.onClicked(function (e) {
                        e.preventDefault();
                        if (_this.popup.isVisible()) {
                            _this.popup.hide();
                        }
                        else {
                            _this.popup.show();
                        }
                    });
                    if (builder.hours || builder.minutes) {
                        this.input.setValue(this.formatTime(builder.hours, builder.minutes));
                    }
                    this.popup.onSelectedTimeChanged(function (hours, minutes) {
                        if (hours != null && minutes != null) {
                            _this.input.setValue(_this.formatTime(hours, minutes));
                            _this.validUserInput = true;
                        }
                        _this.updateInputStyling();
                    });
                    this.popup.onKeyDown(function (event) {
                        if (api.ui.KeyHelper.isTabKey(event)) {
                            if (!(document.activeElement == _this.input.getEl().getHTMLElement())) {
                                event.preventDefault();
                                event.stopPropagation();
                                _this.popup.hide();
                                _this.popupTrigger.giveFocus();
                            }
                        }
                    });
                    this.input.onKeyDown(function (event) {
                        if (api.ui.KeyHelper.isTabKey(event)) {
                            if (!event.shiftKey) {
                                event.preventDefault();
                                event.stopPropagation();
                                _this.popupTrigger.giveFocus();
                            }
                            else {
                                _this.popup.hide();
                            }
                        }
                    });
                    this.popupTrigger.onKeyDown(function (event) {
                        if (api.ui.KeyHelper.isTabKey(event)) {
                            _this.popup.hide();
                        }
                    });
                    api.dom.Body.get().onKeyDown(function (e) { return _this.popupTabListener(e); });
                    if (builder.closeOnOutsideClick) {
                        api.dom.Body.get().onClicked(function (e) { return _this.outsideClickListener(e); });
                    }
                }
                TimePicker.prototype.popupTabListener = function (e) {
                    if (api.ui.KeyHelper.isTabKey(e) && !this.getEl().contains(e.target)) {
                        if (this.popup.isVisible()) {
                            e.stopPropagation();
                            e.preventDefault();
                            this.popupTrigger.getEl().focus();
                            this.popup.hide();
                        }
                    }
                };
                TimePicker.prototype.outsideClickListener = function (e) {
                    if (!this.getEl().contains(e.target)) {
                        this.popup.hide();
                    }
                };
                TimePicker.prototype.hasValidUserInput = function () {
                    return this.validUserInput;
                };
                TimePicker.prototype.getSelectedTime = function () {
                    return this.popup.getSelectedTime();
                };
                TimePicker.prototype.onSelectedTimeChanged = function (listener) {
                    this.popup.onSelectedTimeChanged(listener);
                };
                TimePicker.prototype.unSelectedTimeChanged = function (listener) {
                    this.popup.unSelectedTimeChanged(listener);
                };
                TimePicker.prototype.formatTime = function (hours, minutes) {
                    return this.padNumber(hours, 2) + ':' + this.padNumber(minutes, 2);
                };
                TimePicker.prototype.padNumber = function (value, pad) {
                    return Array(pad - String(value).length + 1).join('0') + value;
                };
                TimePicker.prototype.updateInputStyling = function () {
                    this.input.updateValidationStatusOnUserInput(this.validUserInput);
                };
                TimePicker.prototype.giveFocus = function () {
                    return this.input.giveFocus();
                };
                return TimePicker;
            })(api.dom.DivEl);
            time.TimePicker = TimePicker;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var Timezone = api.util.Timezone;
            var TimePickerPopupBuilder = (function () {
                function TimePickerPopupBuilder() {
                    this.useLocalTimezoneIfNotPresent = false;
                    this.closeOnOutsideClick = true;
                }
                TimePickerPopupBuilder.prototype.setHours = function (value) {
                    this.hours = value;
                    return this;
                };
                TimePickerPopupBuilder.prototype.getHours = function () {
                    return this.hours;
                };
                TimePickerPopupBuilder.prototype.setMinutes = function (value) {
                    this.minutes = value;
                    return this;
                };
                TimePickerPopupBuilder.prototype.getMinutes = function () {
                    return this.minutes;
                };
                TimePickerPopupBuilder.prototype.setTimezone = function (value) {
                    this.timezone = value;
                    return this;
                };
                TimePickerPopupBuilder.prototype.getTimezone = function () {
                    return this.timezone;
                };
                TimePickerPopupBuilder.prototype.setUseLocalTimezoneIfNotPresent = function (value) {
                    this.useLocalTimezoneIfNotPresent = value;
                    return this;
                };
                TimePickerPopupBuilder.prototype.isUseLocalTimezoneIfNotPresent = function () {
                    return this.useLocalTimezoneIfNotPresent;
                };
                TimePickerPopupBuilder.prototype.setCloseOnOutsideClick = function (value) {
                    this.closeOnOutsideClick = value;
                    return this;
                };
                TimePickerPopupBuilder.prototype.isCloseOnOutsideClick = function () {
                    return this.closeOnOutsideClick;
                };
                TimePickerPopupBuilder.prototype.build = function () {
                    return new TimePickerPopup(this);
                };
                return TimePickerPopupBuilder;
            })();
            time.TimePickerPopupBuilder = TimePickerPopupBuilder;
            var TimePickerPopup = (function (_super) {
                __extends(TimePickerPopup, _super);
                function TimePickerPopup(builder) {
                    var _this = this;
                    _super.call(this, 'time-picker-dialog');
                    this.useLocalTimezoneIfNotPresent = false;
                    this.timeChangedListeners = [];
                    var hourContainer = new api.dom.LiEl();
                    this.appendChild(hourContainer);
                    this.nextHour = new api.dom.AEl('next');
                    this.nextHour.onMouseDown(function (e) {
                        _this.addHour(+1);
                        _this.startInterval(_this.addHour, 1);
                    });
                    api.dom.Body.get().onMouseUp(function (e) {
                        _this.stopInterval();
                    });
                    this.nextHour.appendChild(new api.dom.SpanEl());
                    hourContainer.appendChild(this.nextHour);
                    this.hour = new api.dom.SpanEl();
                    hourContainer.appendChild(this.hour);
                    this.prevHour = new api.dom.AEl('prev');
                    this.prevHour.onMouseDown(function (e) {
                        _this.addHour(-1);
                        _this.startInterval(_this.addHour, -1);
                    });
                    this.prevHour.appendChild(new api.dom.SpanEl());
                    hourContainer.appendChild(this.prevHour);
                    this.appendChild(new api.dom.LiEl('colon'));
                    var minuteContainer = new api.dom.LiEl();
                    this.appendChild(minuteContainer);
                    this.nextMinute = new api.dom.AEl('next');
                    this.nextMinute.onMouseDown(function (e) {
                        _this.addMinute(+1);
                        _this.startInterval(_this.addMinute, 1);
                    });
                    this.nextMinute.appendChild(new api.dom.SpanEl());
                    minuteContainer.appendChild(this.nextMinute);
                    this.minute = new api.dom.SpanEl();
                    minuteContainer.appendChild(this.minute);
                    this.prevMinute = new api.dom.AEl('prev');
                    this.prevMinute.onMouseDown(function (e) {
                        _this.addMinute(-1);
                        _this.startInterval(_this.addMinute, -1);
                    });
                    this.prevMinute.appendChild(new api.dom.SpanEl());
                    minuteContainer.appendChild(this.prevMinute);
                    this.selectedHour = builder.getHours() || null;
                    this.selectedMinute = builder.getMinutes() || null;
                    this.useLocalTimezoneIfNotPresent = builder.useLocalTimezoneIfNotPresent;
                    this.timezone = builder.timezone;
                    if (!this.timezone && this.useLocalTimezoneIfNotPresent) {
                        this.timezone = Timezone.getLocalTimezone();
                    }
                    if (this.timezone) {
                        var timezoneContainer = new api.dom.LiEl("timezone");
                        this.timezoneLocation = new api.dom.SpanEl("timezone-location").setHtml(this.timezone.getLocation());
                        this.timezoneOffset = new api.dom.SpanEl("timezone-offset").setHtml(this.getUTCString(this.timezone.getOffset()));
                        timezoneContainer.appendChild(this.timezoneLocation);
                        timezoneContainer.appendChild(this.timezoneOffset);
                        this.appendChild(timezoneContainer);
                    }
                    this.hour.setHtml(this.padNumber(this.selectedHour || 0, 2));
                    this.minute.setHtml(this.padNumber(this.selectedMinute || 0, 2));
                    if (builder.isCloseOnOutsideClick()) {
                        api.dom.Body.get().onClicked(function (e) { return _this.outsideClickListener(e); });
                    }
                }
                TimePickerPopup.prototype.getSelectedTime = function () {
                    return this.selectedHour != null && this.selectedMinute != null ? {
                        hour: this.selectedHour,
                        minute: this.selectedMinute
                    } : null;
                };
                TimePickerPopup.prototype.onSelectedTimeChanged = function (listener) {
                    this.timeChangedListeners.push(listener);
                };
                TimePickerPopup.prototype.unSelectedTimeChanged = function (listener) {
                    this.timeChangedListeners = this.timeChangedListeners.filter(function (curr) {
                        return curr != listener;
                    });
                };
                TimePickerPopup.prototype.startInterval = function (fn) {
                    var _this = this;
                    var args = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        args[_i - 1] = arguments[_i];
                    }
                    var times = 0;
                    var delay = 400;
                    var intervalFn = function () {
                        fn.apply(_this, args);
                        if (++times % 5 == 0 && delay > 50) {
                            _this.stopInterval();
                            delay /= 2;
                            _this.interval = setInterval(intervalFn, delay);
                        }
                    };
                    this.interval = setInterval(intervalFn, delay);
                };
                TimePickerPopup.prototype.stopInterval = function () {
                    clearInterval(this.interval);
                };
                TimePickerPopup.prototype.getUTCString = function (value) {
                    if (!value && value != 0) {
                        return "";
                    }
                    var result = "UTC";
                    result = value > 0 ? result + "+" : (value == 0 ? result + "-" : result);
                    return result + value;
                };
                TimePickerPopup.prototype.addHour = function (add, silent) {
                    this.selectedHour += add;
                    if (this.selectedHour < 0) {
                        this.selectedHour += 24;
                    }
                    else if (this.selectedHour > 23) {
                        this.selectedHour -= 24;
                    }
                    this.setSelectedTime(this.selectedHour, this.selectedMinute || 0, silent);
                };
                TimePickerPopup.prototype.addMinute = function (add, silent) {
                    this.selectedMinute += add;
                    if (this.selectedMinute < 0) {
                        this.selectedMinute += 60;
                        this.addHour(-1, true);
                    }
                    else if (this.selectedMinute > 59) {
                        this.selectedMinute -= 60;
                        this.addHour(1, true);
                    }
                    this.setSelectedTime(this.selectedHour || 0, this.selectedMinute, silent);
                };
                TimePickerPopup.prototype.notifyTimeChanged = function (hours, minutes) {
                    this.timeChangedListeners.forEach(function (listener) {
                        listener(hours, minutes);
                    });
                };
                TimePickerPopup.prototype.outsideClickListener = function (e) {
                    if (!this.getEl().contains(e.target)) {
                        this.hide();
                    }
                };
                TimePickerPopup.prototype.setSelectedTime = function (hours, minutes, silent) {
                    if (hours == null || hours < 0 || hours > 23 || minutes == null || minutes < 0 || minutes > 59) {
                        this.selectedHour = null;
                        this.selectedMinute = null;
                    }
                    else {
                        this.selectedHour = hours;
                        this.selectedMinute = minutes;
                    }
                    this.hour.setHtml(this.padNumber(this.selectedHour || 0, 2));
                    this.minute.setHtml(this.padNumber(this.selectedMinute || 0, 2));
                    if (!silent) {
                        this.notifyTimeChanged(this.selectedHour, this.selectedMinute);
                    }
                };
                TimePickerPopup.prototype.padNumber = function (value, pad) {
                    return Array(pad - String(value).length + 1).join('0') + value;
                };
                return TimePickerPopup;
            })(api.dom.UlEl);
            time.TimePickerPopup = TimePickerPopup;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var DateTimePickerBuilder = (function () {
                function DateTimePickerBuilder() {
                    this.startingDayOfWeek = time.DaysOfWeek.MONDAY;
                    this.closeOnSelect = false;
                    this.closeOnOutsideClick = true;
                    this.useLocalTimezoneIfNotPresent = false;
                }
                DateTimePickerBuilder.prototype.setYear = function (value) {
                    this.year = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setMonth = function (value) {
                    this.month = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setSelectedDate = function (value) {
                    this.selectedDate = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setHours = function (value) {
                    this.hours = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setMinutes = function (value) {
                    this.minutes = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setStartingDayOfWeek = function (value) {
                    this.startingDayOfWeek = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setTimezone = function (value) {
                    this.timezone = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setCloseOnSelect = function (value) {
                    this.closeOnSelect = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setCloseOnOutsideClick = function (value) {
                    this.closeOnOutsideClick = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.setUseLocalTimezoneIfNotPresent = function (value) {
                    this.useLocalTimezoneIfNotPresent = value;
                    return this;
                };
                DateTimePickerBuilder.prototype.build = function () {
                    return new DateTimePicker(this);
                };
                return DateTimePickerBuilder;
            })();
            time.DateTimePickerBuilder = DateTimePickerBuilder;
            var DateTimePicker = (function (_super) {
                __extends(DateTimePicker, _super);
                function DateTimePicker(builder) {
                    var _this = this;
                    _super.call(this, 'date-time-picker');
                    this.selectedDateTimeChangedListeners = [];
                    var onDatePickerShown = this.onDateTimePickerShown.bind(this);
                    DateTimePickerShownEvent.on(onDatePickerShown);
                    this.onRemoved(function (event) {
                        DateTimePickerShownEvent.un(onDatePickerShown);
                    });
                    this.validUserInput = true;
                    this.input = api.ui.text.TextInput.middle();
                    this.input.onClicked(function (e) {
                        e.preventDefault();
                        _this.popup.show();
                    });
                    var wrapper = new api.dom.DivEl('wrapper');
                    wrapper.appendChild(this.input);
                    this.calendar = new time.CalendarBuilder().setSelectedDate(builder.selectedDate).setMonth(builder.month).setYear(builder.year).setInteractive(true).build();
                    var popupBuilder = new time.DateTimePickerPopupBuilder().setHours(builder.hours).setMinutes(builder.minutes).setCalendar(this.calendar).setTimezone(builder.timezone).setUseLocalTimezoneIfNotPresent(builder.useLocalTimezoneIfNotPresent).setCloseOnOutsideClick(false);
                    this.popup = new time.DateTimePickerPopup(popupBuilder);
                    this.popup.onShown(function () {
                        new DateTimePickerShownEvent(_this).fire();
                    });
                    wrapper.appendChild(this.popup);
                    this.popupTrigger = new api.ui.button.Button();
                    this.popupTrigger.addClass('icon-calendar4');
                    wrapper.appendChild(this.popupTrigger);
                    this.appendChild(wrapper);
                    this.popupTrigger.onClicked(function (e) {
                        e.preventDefault();
                        if (_this.popup.isVisible()) {
                            _this.popup.hide();
                        }
                        else {
                            _this.popup.show();
                        }
                    });
                    if (builder.selectedDate) {
                        this.input.setValue(this.formatDate(builder.selectedDate));
                        this.selectedDate = builder.selectedDate;
                    }
                    if (builder.hours || builder.minutes) {
                        var value = this.input.getValue() || "";
                        this.setTime(builder.hours, builder.minutes);
                        this.input.setValue(value + " " + this.formatTime(builder.hours, builder.minutes));
                    }
                    this.popup.onSelectedDateChanged(function (e) {
                        if (builder.closeOnSelect) {
                            _this.popup.hide();
                        }
                        _this.setDate(e.getDate());
                        _this.validUserInput = true;
                        _this.input.setValue(_this.formatDate(_this.selectedDate) + " " + _this.formatTime(_this.selectedDate.getHours(), _this.selectedDate.getMinutes()));
                        _this.notifySelectedDateTimeChanged(new time.SelectedDateChangedEvent(_this.selectedDate));
                        _this.updateInputStyling();
                    });
                    this.popup.onSelectedTimeChanged(function (hours, minutes) {
                        _this.setTime(hours, minutes);
                        _this.validUserInput = true;
                        _this.input.setValue(_this.formatDate(_this.selectedDate) + " " + _this.formatTime(hours, minutes));
                        _this.notifySelectedDateTimeChanged(new time.SelectedDateChangedEvent(_this.selectedDate));
                        _this.updateInputStyling();
                    });
                    this.input.onKeyUp(function (event) {
                        if (api.ui.KeyHelper.isArrowKey(event) || api.ui.KeyHelper.isModifierKey(event)) {
                            return;
                        }
                        var typedDateTime = _this.input.getValue();
                        if (api.util.StringHelper.isEmpty(typedDateTime)) {
                            _this.calendar.selectDate(null);
                            _this.selectedDate = null;
                            _this.validUserInput = true;
                            _this.notifySelectedDateTimeChanged(new time.SelectedDateChangedEvent(null));
                            _this.popup.hide();
                        }
                        else {
                            var date = api.util.DateHelper.parseDateTime(typedDateTime);
                            var dateLength = date && date.getFullYear().toString().length + 12;
                            if (date && date.toString() != "Invalid Date" && typedDateTime.length == dateLength) {
                                _this.selectedDate = date;
                                _this.validUserInput = true;
                                _this.calendar.selectDate(date);
                                _this.popup.setSelectedTime(date.getHours(), date.getMinutes());
                                _this.notifySelectedDateTimeChanged(new time.SelectedDateChangedEvent(date));
                                if (!_this.popup.isVisible()) {
                                    _this.popup.show();
                                }
                            }
                            else {
                                _this.selectedDate = null;
                                _this.validUserInput = false;
                                _this.notifySelectedDateTimeChanged(new time.SelectedDateChangedEvent(null));
                            }
                        }
                        _this.updateInputStyling();
                    });
                    this.popup.onKeyDown(function (event) {
                        if (api.ui.KeyHelper.isTabKey(event)) {
                            if (!(document.activeElement == _this.input.getEl().getHTMLElement())) {
                                event.preventDefault();
                                event.stopPropagation();
                                _this.popup.hide();
                                _this.popupTrigger.giveFocus();
                            }
                        }
                    });
                    this.input.onKeyDown(function (event) {
                        if (api.ui.KeyHelper.isTabKey(event)) {
                            if (!event.shiftKey) {
                                event.preventDefault();
                                event.stopPropagation();
                                _this.popupTrigger.giveFocus();
                            }
                            else {
                                _this.popup.hide();
                            }
                        }
                    });
                    this.popupTrigger.onKeyDown(function (event) {
                        if (api.ui.KeyHelper.isTabKey(event)) {
                            _this.popup.hide();
                        }
                    });
                    api.dom.Body.get().onKeyDown(function (e) { return _this.popupTabListener(e); });
                    if (builder.closeOnOutsideClick) {
                        api.dom.Body.get().onClicked(function (e) { return _this.outsideClickListener(e); });
                    }
                }
                DateTimePicker.prototype.popupTabListener = function (e) {
                    if (api.ui.KeyHelper.isTabKey(e) && !this.getEl().contains(e.target)) {
                        if (this.popup.isVisible()) {
                            e.stopPropagation();
                            e.preventDefault();
                            this.popupTrigger.getEl().focus();
                            this.popup.hide();
                        }
                    }
                };
                DateTimePicker.prototype.outsideClickListener = function (e) {
                    if (!this.getEl().contains(e.target)) {
                        this.popup.hide();
                    }
                };
                DateTimePicker.prototype.onDateTimePickerShown = function (event) {
                    if (event.getDateTimePicker() !== this) {
                        this.popup.hide();
                    }
                };
                DateTimePicker.prototype.setTime = function (hours, minutes) {
                    if (!this.selectedDate) {
                        var today = new Date();
                        this.selectedDate = new Date(today.getFullYear(), today.getMonth(), today.getDate());
                    }
                    this.selectedDate.setHours(hours);
                    this.selectedDate.setMinutes(minutes);
                };
                DateTimePicker.prototype.setDate = function (date) {
                    var hours = this.selectedDate ? this.selectedDate.getHours() : 0, minutes = this.selectedDate ? this.selectedDate.getMinutes() : 0;
                    this.selectedDate = date;
                    if (hours || minutes) {
                        this.setTime(hours, minutes);
                    }
                };
                DateTimePicker.prototype.hasValidUserInput = function () {
                    return this.validUserInput;
                };
                DateTimePicker.prototype.getSelectedDateTime = function () {
                    return this.selectedDate;
                };
                DateTimePicker.prototype.onSelectedDateTimeChanged = function (listener) {
                    this.selectedDateTimeChangedListeners.push(listener);
                };
                DateTimePicker.prototype.unSelectedDateTimeChanged = function (listener) {
                    this.selectedDateTimeChangedListeners = this.selectedDateTimeChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                DateTimePicker.prototype.notifySelectedDateTimeChanged = function (event) {
                    this.selectedDateTimeChangedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                DateTimePicker.prototype.formatDate = function (date) {
                    return api.util.DateHelper.formatDate(date);
                };
                DateTimePicker.prototype.formatTime = function (hours, minutes) {
                    return this.padNumber(hours, 2) + ':' + this.padNumber(minutes, 2);
                };
                DateTimePicker.prototype.padNumber = function (value, pad) {
                    return Array(pad - String(value).length + 1).join('0') + value;
                };
                DateTimePicker.prototype.updateInputStyling = function () {
                    this.input.updateValidationStatusOnUserInput(this.validUserInput);
                };
                DateTimePicker.prototype.giveFocus = function () {
                    return this.input.giveFocus();
                };
                return DateTimePicker;
            })(api.dom.DivEl);
            time.DateTimePicker = DateTimePicker;
            var DateTimePickerShownEvent = (function (_super) {
                __extends(DateTimePickerShownEvent, _super);
                function DateTimePickerShownEvent(dateTimePicker) {
                    _super.call(this);
                    this.dateTimePicker = dateTimePicker;
                }
                DateTimePickerShownEvent.prototype.getDateTimePicker = function () {
                    return this.dateTimePicker;
                };
                DateTimePickerShownEvent.on = function (handler) {
                    api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
                };
                DateTimePickerShownEvent.un = function (handler) {
                    api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
                };
                return DateTimePickerShownEvent;
            })(api.event.Event);
            time.DateTimePickerShownEvent = DateTimePickerShownEvent;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (_time) {
            var DateTimePickerPopupBuilder = (function () {
                function DateTimePickerPopupBuilder() {
                    this.useLocalTimezoneIfNotPresent = false;
                    this.closeOnOutsideClick = true;
                }
                DateTimePickerPopupBuilder.prototype.setHours = function (value) {
                    this.hours = value;
                    return this;
                };
                DateTimePickerPopupBuilder.prototype.getHours = function () {
                    return this.hours;
                };
                DateTimePickerPopupBuilder.prototype.setMinutes = function (value) {
                    this.minutes = value;
                    return this;
                };
                DateTimePickerPopupBuilder.prototype.getMinutes = function () {
                    return this.minutes;
                };
                DateTimePickerPopupBuilder.prototype.setCalendar = function (value) {
                    this.calendar = value;
                    return this;
                };
                DateTimePickerPopupBuilder.prototype.getCalendar = function () {
                    return this.calendar;
                };
                DateTimePickerPopupBuilder.prototype.setTimezone = function (value) {
                    this.timezone = value;
                    return this;
                };
                DateTimePickerPopupBuilder.prototype.setUseLocalTimezoneIfNotPresent = function (value) {
                    this.useLocalTimezoneIfNotPresent = value;
                    return this;
                };
                DateTimePickerPopupBuilder.prototype.isUseLocalTimezoneIfNotPresent = function () {
                    return this.useLocalTimezoneIfNotPresent;
                };
                DateTimePickerPopupBuilder.prototype.getTimezone = function () {
                    return this.timezone;
                };
                DateTimePickerPopupBuilder.prototype.setCloseOnOutsideClick = function (value) {
                    this.closeOnOutsideClick = value;
                    return this;
                };
                DateTimePickerPopupBuilder.prototype.isCloseOnOutsideClick = function () {
                    return this.closeOnOutsideClick;
                };
                DateTimePickerPopupBuilder.prototype.build = function () {
                    return new DateTimePickerPopup(this);
                };
                return DateTimePickerPopupBuilder;
            })();
            _time.DateTimePickerPopupBuilder = DateTimePickerPopupBuilder;
            var DateTimePickerPopup = (function (_super) {
                __extends(DateTimePickerPopup, _super);
                function DateTimePickerPopup(builder) {
                    var _this = this;
                    _super.call(this, 'date-time-dialog');
                    var closeOnOutsideClick = builder.isCloseOnOutsideClick();
                    builder.setCloseOnOutsideClick(false);
                    this.datePickerPopup = new _time.DatePickerPopupBuilder().setCalendar(builder.getCalendar()).setCloseOnOutsideClick(false).build();
                    this.timePickerPopup = new _time.TimePickerPopupBuilder().setHours(builder.getHours()).setCloseOnOutsideClick(false).setTimezone(builder.timezone).setUseLocalTimezoneIfNotPresent(builder.useLocalTimezoneIfNotPresent).setMinutes(builder.getMinutes()).build();
                    this.appendChildren(this.datePickerPopup, this.timePickerPopup);
                    if (closeOnOutsideClick) {
                        api.dom.Body.get().onClicked(function (e) { return _this.outsideClickListener(e); });
                    }
                }
                DateTimePickerPopup.prototype.getSelectedDate = function () {
                    return this.datePickerPopup.getCalendar().getSelectedDate();
                };
                DateTimePickerPopup.prototype.onSelectedDateChanged = function (listener) {
                    this.datePickerPopup.onSelectedDateChanged(listener);
                };
                DateTimePickerPopup.prototype.unSelectedDateChanged = function (listener) {
                    this.datePickerPopup.unSelectedDateChanged(listener);
                };
                DateTimePickerPopup.prototype.getSelectedTime = function () {
                    return this.timePickerPopup.getSelectedTime();
                };
                DateTimePickerPopup.prototype.onSelectedTimeChanged = function (listener) {
                    this.timePickerPopup.onSelectedTimeChanged(listener);
                };
                DateTimePickerPopup.prototype.unSelectedTimeChanged = function (listener) {
                    this.timePickerPopup.unSelectedTimeChanged(listener);
                };
                DateTimePickerPopup.prototype.getSelectedDateTime = function () {
                    var date = this.getSelectedDate();
                    var time = this.getSelectedTime();
                    if (!date || !time) {
                        return null;
                    }
                    return new Date(date.getFullYear(), date.getMonth(), date.getDate(), time.hour, time.minute);
                };
                DateTimePickerPopup.prototype.setSelectedTime = function (hours, minutes, silent) {
                    this.timePickerPopup.setSelectedTime(hours, minutes, silent);
                };
                DateTimePickerPopup.prototype.outsideClickListener = function (e) {
                    if (!this.getEl().contains(e.target)) {
                        this.hide();
                    }
                };
                return DateTimePickerPopup;
            })(api.dom.DivEl);
            _time.DateTimePickerPopup = DateTimePickerPopup;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var time;
        (function (time) {
            var LocalTime = (function (_super) {
                __extends(LocalTime, _super);
                function LocalTime(timeBuilder) {
                    _super.call(this, "date-picker");
                    if (!timeBuilder) {
                        timeBuilder = new api.ui.time.TimePickerBuilder();
                    }
                    this.timePicker = timeBuilder.build();
                    this.layoutItems();
                }
                LocalTime.prototype.layoutItems = function () {
                    this.removeChildren();
                    this.appendChild(this.timePicker);
                    return this;
                };
                LocalTime.prototype.onTimeChanged = function (listener) {
                    this.timePicker.onSelectedTimeChanged(listener);
                };
                LocalTime.prototype.setTime = function (value) {
                    var localTime = value.getAdjustedTime();
                    this.timePicker = new api.ui.time.TimePickerBuilder().setHours(localTime.hour).setMinutes(localTime.minute).build();
                    this.layoutItems();
                    return this;
                };
                LocalTime.prototype.giveFocus = function () {
                    return this.timePicker.giveFocus();
                };
                LocalTime.prototype.getTimePicker = function () {
                    return this.timePicker;
                };
                return LocalTime;
            })(api.dom.DivEl);
            time.LocalTime = LocalTime;
        })(time = ui.time || (ui.time = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var actions;
            (function (actions) {
                var Action = api.ui.Action;
                var SelectAllAction = (function (_super) {
                    __extends(SelectAllAction, _super);
                    function SelectAllAction(treeGrid) {
                        var allCount = treeGrid.getGrid().getDataView().getLength();
                        var availableCount = allCount - treeGrid.getEmptyNodesCount();
                        _super.call(this, "Select All (" + availableCount + ")");
                        this.setEnabled(true);
                        this.onExecuted(function () {
                            treeGrid.selectAll();
                        });
                    }
                    return SelectAllAction;
                })(Action);
                actions.SelectAllAction = SelectAllAction;
            })(actions = treegrid.actions || (treegrid.actions = {}));
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var actions;
            (function (actions) {
                var Action = api.ui.Action;
                var ClearSelectionAction = (function (_super) {
                    __extends(ClearSelectionAction, _super);
                    function ClearSelectionAction(treeGrid, selectedItemsCount) {
                        var label = "Clear Selection";
                        label += !!selectedItemsCount ? " (" + selectedItemsCount + ")" : "";
                        _super.call(this, label);
                        this.setEnabled(true);
                        this.onExecuted(function () {
                            treeGrid.getRoot().clearStashedSelection();
                            treeGrid.getGrid().clearSelection();
                        });
                    }
                    return ClearSelectionAction;
                })(Action);
                actions.ClearSelectionAction = ClearSelectionAction;
            })(actions = treegrid.actions || (treegrid.actions = {}));
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var actions;
            (function (actions) {
                var TreeGridToolbarActions = (function () {
                    function TreeGridToolbarActions(grid, selectedItemsCount) {
                        this.actions = [];
                        this.SELECT_ALL = new actions.SelectAllAction(grid);
                        this.CLEAR_SELECTION = new actions.ClearSelectionAction(grid, selectedItemsCount);
                        this.actions.push(this.SELECT_ALL, this.CLEAR_SELECTION);
                    }
                    TreeGridToolbarActions.prototype.getAllActions = function () {
                        return this.actions;
                    };
                    TreeGridToolbarActions.prototype.updateActionsEnabledState = function (browseItems) {
                        var deferred = wemQ.defer();
                        deferred.resolve(browseItems);
                        return deferred.promise;
                    };
                    return TreeGridToolbarActions;
                })();
                actions.TreeGridToolbarActions = TreeGridToolbarActions;
            })(actions = treegrid.actions || (treegrid.actions = {}));
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var Element = api.dom.Element;
            var ElementHelper = api.dom.ElementHelper;
            var Grid = api.ui.grid.Grid;
            var DataView = api.ui.grid.DataView;
            var KeyBinding = api.ui.KeyBinding;
            var KeyBindings = api.ui.KeyBindings;
            var TreeGridToolbarActions = api.ui.treegrid.actions.TreeGridToolbarActions;
            var TreeGrid = (function (_super) {
                __extends(TreeGrid, _super);
                function TreeGrid(builder) {
                    var _this = this;
                    _super.call(this, builder.getClasses());
                    this.columns = [];
                    this.loadedListeners = [];
                    this.contextMenuListeners = [];
                    this.selectionChangeListeners = [];
                    this.dataChangeListeners = [];
                    this.expandAll = builder.isExpandAll();
                    this.root = new treegrid.TreeRoot();
                    this.gridData = new DataView();
                    this.gridData.setFilter(function (node) {
                        return node.isVisible();
                    });
                    this.columns = this.updateColumnsFormatter(builder.getColumns());
                    this.gridOptions = builder.getOptions();
                    this.grid = new Grid(this.gridData, this.columns, this.gridOptions);
                    this.grid.setSelectionModel(new Slick.RowSelectionModel({
                        selectActiveRow: false
                    }));
                    var selectorPlugin = this.grid.getCheckboxSelectorPlugin();
                    if (selectorPlugin) {
                        this.grid.unregisterPlugin(this.grid.getCheckboxSelectorPlugin());
                    }
                    this.grid.syncGridSelection(false);
                    this.actions = new TreeGridToolbarActions(this);
                    this.onClicked(function () {
                        _this.grid.focus();
                    });
                    if (builder.getContextMenu()) {
                        this.contextMenu = builder.getContextMenu();
                        this.grid.subscribeOnContextMenu(function (event) {
                            event.preventDefault();
                            _this.active = false;
                            var cell = _this.grid.getCellFromEvent(event);
                            _this.grid.selectRow(cell.row);
                            _this.contextMenu.showAt(event.pageX, event.pageY);
                            _this.notifyContextMenuShown(event.pageX, event.pageY);
                            _this.active = true;
                        });
                    }
                    this.grid.subscribeOnClick(function (event, data) {
                        if (_this.active) {
                            _this.active = false;
                            var elem = new ElementHelper(event.target);
                            if (elem.hasClass("expand")) {
                                elem.removeClass("expand").addClass("collapse");
                                var node = _this.gridData.getItem(data.row);
                                _this.expandNode(node);
                            }
                            else if (elem.hasClass("collapse")) {
                                _this.active = false;
                                elem.removeClass("collapse").addClass("expand");
                                var node = _this.gridData.getItem(data.row);
                                _this.collapseNode(node);
                            }
                            else if (data.cell === 0) {
                                _this.active = true;
                                if (elem.getAttribute("type") === "checkbox") {
                                    _this.grid.toggleRow(data.row);
                                }
                            }
                            else {
                                _this.active = true;
                                _this.root.clearStashedSelection();
                                _this.grid.selectRow(data.row);
                                if (!elem.hasClass("sort-dialog-trigger")) {
                                    new api.content.TreeGridItemClickedEvent().fire();
                                }
                            }
                        }
                        if (_this.contextMenu) {
                            _this.contextMenu.hide();
                        }
                    });
                    if (builder.isShowToolbar()) {
                        this.toolbar = new treegrid.TreeGridToolbar(this.actions, this);
                        this.appendChild(this.toolbar);
                        this.removeClass("no-toolbar");
                    }
                    else {
                        this.addClass("no-toolbar");
                    }
                    this.appendChild(this.grid);
                    if (builder.isAutoLoad()) {
                        this.reload();
                    }
                    if (builder.isPartialLoadEnabled()) {
                        this.loadBufferSize = builder.getLoadBufferSize();
                        var interval;
                        this.onShown(function () {
                            if (interval) {
                                clearInterval(interval);
                            }
                            interval = setInterval(_this.postLoad.bind(_this), 200);
                        });
                    }
                    var keyBindings = [];
                    this.grid.onShown(function () {
                        _this.scrollable = _this.queryScrollable();
                    });
                    this.onShown(function () {
                        _this.grid.resizeCanvas();
                        if (builder.isHotkeysEnabled()) {
                            if (!_this.gridOptions.isMultipleSelectionDisabled()) {
                                keyBindings = [
                                    new KeyBinding('shift+up', function (event) {
                                        if (_this.active) {
                                            _this.scrollToRow(_this.grid.addSelectedUp());
                                        }
                                        event.preventDefault();
                                        event.stopImmediatePropagation();
                                    }),
                                    new KeyBinding('shift+down', function (event) {
                                        if (_this.active) {
                                            _this.scrollToRow(_this.grid.addSelectedDown());
                                        }
                                        event.preventDefault();
                                        event.stopImmediatePropagation();
                                    })
                                ];
                            }
                            keyBindings = keyBindings.concat([
                                new KeyBinding('up', function () {
                                    if (_this.active) {
                                        _this.scrollToRow(_this.grid.moveSelectedUp());
                                    }
                                }),
                                new KeyBinding('down', function () {
                                    if (_this.active) {
                                        _this.scrollToRow(_this.grid.moveSelectedDown());
                                    }
                                }),
                                new KeyBinding('left', function () {
                                    var selected = _this.grid.getSelectedRows();
                                    if (selected.length === 1) {
                                        var node = _this.gridData.getItem(selected[0]);
                                        if (node && _this.active) {
                                            if (node.isExpanded()) {
                                                _this.active = false;
                                                _this.collapseNode(node);
                                            }
                                            else if (node.getParent() !== _this.root.getCurrentRoot()) {
                                                node = node.getParent();
                                                _this.active = false;
                                                var row = _this.gridData.getRowById(node.getId());
                                                _this.grid.selectRow(row);
                                                _this.collapseNode(node);
                                            }
                                        }
                                    }
                                }),
                                new KeyBinding('right', function () {
                                    var selected = _this.grid.getSelectedRows();
                                    if (selected.length === 1) {
                                        var node = _this.gridData.getItem(selected[0]);
                                        if (node && _this.hasChildren(node.getData()) && !node.isExpanded() && _this.active) {
                                            _this.active = false;
                                            _this.invalidate();
                                            _this.expandNode(node);
                                        }
                                    }
                                }),
                                new KeyBinding('space', function () {
                                    _this.deselectAll();
                                })
                            ]);
                            KeyBindings.get().bindKeys(keyBindings);
                        }
                    });
                    this.onRendered(function () {
                        _this.grid.resizeCanvas();
                    });
                    this.onRemoved(function () {
                        if (builder.isHotkeysEnabled()) {
                            KeyBindings.get().unbindKeys(keyBindings);
                        }
                        if (builder.isPartialLoadEnabled() && interval) {
                            clearInterval(interval);
                        }
                    });
                    this.grid.subscribeOnSelectedRowsChanged(function (event, rows) {
                        _this.notifySelectionChanged(event, rows.rows);
                    });
                    this.onLoaded(function () { return _this.grid.unmask(); });
                }
                TreeGrid.prototype.updateColumnsFormatter = function (columns) {
                    var _this = this;
                    if (columns.length > 0) {
                        var formatter = columns[0].getFormatter();
                        var toggleFormatter = function (row, cell, value, columnDef, node) {
                            var toggleSpan = new api.dom.SpanEl("toggle icon");
                            if (_this.hasChildren(node.getData())) {
                                var toggleClass = node.isExpanded() ? "collapse" : "expand";
                                toggleSpan.addClass(toggleClass);
                            }
                            toggleSpan.getEl().setMarginLeft(TreeGrid.LEVEL_STEP_INDENT * (node.calcLevel() - 1) + "px");
                            return toggleSpan.toString() + formatter(row, cell, value, columnDef, node);
                        };
                        columns[0].setFormatter(toggleFormatter);
                    }
                    return columns;
                };
                TreeGrid.prototype.getEmptyNodesCount = function () {
                    var viewportRange = this.grid.getViewport(), lastIndex = this.gridData.getItems().length - 1, firstVisible = viewportRange.top, from = firstVisible;
                    var emptyNodesCount = 0;
                    for (var i = from; i <= lastIndex; i++) {
                        if (!!this.gridData.getItem(i) && this.gridData.getItem(i).getDataId() === "") {
                            emptyNodesCount++;
                        }
                    }
                    return emptyNodesCount;
                };
                TreeGrid.prototype.mask = function () {
                    this.grid.mask();
                };
                TreeGrid.prototype.unmask = function () {
                    this.grid.unmask();
                };
                TreeGrid.prototype.getGrid = function () {
                    return this.grid;
                };
                TreeGrid.prototype.getOptions = function () {
                    return this.gridOptions;
                };
                TreeGrid.prototype.getColumns = function () {
                    return this.grid.getColumns();
                };
                TreeGrid.prototype.getContextMenu = function () {
                    return this.contextMenu;
                };
                TreeGrid.prototype.getRoot = function () {
                    return this.root;
                };
                TreeGrid.prototype.isActive = function () {
                    return this.active;
                };
                TreeGrid.prototype.setActive = function (active) {
                    if (active === void 0) { active = true; }
                    this.active = active;
                };
                TreeGrid.prototype.getToolbar = function () {
                    return this.toolbar;
                };
                TreeGrid.prototype.hasToolbar = function () {
                    return !!this.toolbar;
                };
                TreeGrid.prototype.scrollToRow = function (row) {
                    if (!this.scrollable) {
                        return;
                    }
                    var scrollEl = this.scrollable.getEl();
                    if (row > -1 && this.grid.getSelectedRows().length > 0) {
                        if (scrollEl.getScrollTop() > row * 45) {
                            scrollEl.setScrollTop(row * 45);
                        }
                        else if (scrollEl.getScrollTop() + scrollEl.getHeight() < (row + 1) * 45) {
                            scrollEl.setScrollTop((row + 1) * 45 - scrollEl.getHeight());
                        }
                    }
                };
                TreeGrid.prototype.queryScrollable = function () {
                    var gridClasses = (" " + this.grid.getEl().getClass()).replace(/\s/g, ".");
                    var viewport = Element.fromString(".tree-grid " + gridClasses + " .slick-viewport", false);
                    return viewport;
                };
                TreeGrid.prototype.loadEmptyNode = function (node) {
                    var _this = this;
                    if (!this.getDataId(node.getData())) {
                        this.fetchChildren(node.getParent()).then(function (dataList) {
                            var oldChildren = node.getParent().getChildren();
                            if (oldChildren[oldChildren.length - 1].getDataId() === "") {
                                oldChildren.pop();
                            }
                            var fetchedChildren = _this.dataToTreeNodes(dataList, node.getParent());
                            var needToCheckFetchedChildren = _this.areAllOldChildrenSelected(oldChildren);
                            var newChildren = oldChildren.concat(fetchedChildren.slice(oldChildren.length));
                            node.getParent().setChildren(newChildren);
                            _this.initData(_this.root.getCurrentRoot().treeToList());
                            if (needToCheckFetchedChildren) {
                                _this.select(fetchedChildren);
                            }
                        }).catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).finally(function () {
                        }).done(function () { return _this.notifyLoaded(); });
                    }
                };
                TreeGrid.prototype.select = function (fetchedChildren) {
                    var _this = this;
                    var rowsToSelect = [];
                    fetchedChildren.forEach(function (node) {
                        var row = _this.gridData.getRowById(node.getId());
                        if (row) {
                            rowsToSelect.push(row);
                        }
                    });
                    this.grid.addSelectedRows(rowsToSelect);
                };
                TreeGrid.prototype.areAllOldChildrenSelected = function (oldChildren) {
                    var _this = this;
                    if (oldChildren && oldChildren.length > 0) {
                        return oldChildren.every(function (node) { return _this.grid.isRowSelected(_this.gridData.getRowById(node.getId())); });
                    }
                    else {
                        return false;
                    }
                };
                TreeGrid.prototype.postLoad = function () {
                    var iFrame = api.app.Application.getApplication().getAppFrame(), disabled = !iFrame.isVisible() || !this.isVisible() || !this.isActive();
                    if (disabled) {
                        return;
                    }
                    var viewportRange = this.grid.getViewport(), lastIndex = this.gridData.getItems().length - 1, firstVisible = viewportRange.top, lastVisible = Math.min(viewportRange.bottom, lastIndex), from = firstVisible, to = Math.min(lastVisible + this.loadBufferSize, lastIndex);
                    for (var i = from; i <= to; i++) {
                        if (!!this.gridData.getItem(i) && this.gridData.getItem(i).getDataId() === "") {
                            this.loadEmptyNode(this.gridData.getItem(i));
                            break;
                        }
                    }
                };
                TreeGrid.prototype.hasChildren = function (data) {
                    return false;
                };
                TreeGrid.prototype.getDataId = function (data) {
                    throw new Error("Must be implemented by inheritors");
                };
                TreeGrid.prototype.fetch = function (node, dataId) {
                    var deferred = wemQ.defer();
                    deferred.resolve(null);
                    return deferred.promise;
                };
                TreeGrid.prototype.fetchChildren = function (parentNode) {
                    var deferred = wemQ.defer();
                    deferred.resolve([]);
                    return deferred.promise;
                };
                TreeGrid.prototype.fetchRoot = function () {
                    return this.fetchChildren();
                };
                TreeGrid.prototype.fetchData = function (parentNode) {
                    return parentNode ? this.fetchChildren(parentNode) : this.fetchRoot();
                };
                TreeGrid.prototype.dataToTreeNode = function (data, parent) {
                    return new treegrid.TreeNodeBuilder().setData(data, this.getDataId(data)).setExpanded(this.expandAll).setParent(parent).build();
                };
                TreeGrid.prototype.dataToTreeNodes = function (dataArray, parent) {
                    var _this = this;
                    var nodes = [];
                    dataArray.forEach(function (data) {
                        nodes.push(_this.dataToTreeNode(data, parent));
                    });
                    return nodes;
                };
                TreeGrid.prototype.filter = function (dataList) {
                    this.active = false;
                    this.root.setFiltered(true);
                    this.root.getCurrentRoot().setChildren(this.dataToTreeNodes(dataList, this.root.getCurrentRoot()));
                    this.initData(this.root.getCurrentRoot().treeToList());
                    this.invalidate();
                    this.active = true;
                };
                TreeGrid.prototype.resetFilter = function () {
                    this.active = false;
                    if (this.root.isFiltered()) {
                        this.root.setFiltered(false);
                        this.initData(this.root.getCurrentRoot().treeToList());
                        this.invalidate();
                        this.active = true;
                        this.notifyLoaded();
                    }
                    else {
                        this.reload();
                    }
                };
                TreeGrid.prototype.selectNode = function (dataId) {
                    var root = this.root.getCurrentRoot(), node = root.findNode(dataId);
                    if (node) {
                        var row = this.gridData.getRowById(node.getId());
                        this.grid.selectRow(row);
                    }
                };
                TreeGrid.prototype.selectAll = function () {
                    this.grid.selectAll();
                };
                TreeGrid.prototype.deselectAll = function () {
                    this.grid.clearSelection();
                };
                TreeGrid.prototype.deselectNode = function (dataId) {
                    var oldSelected = this.grid.getSelectedRows(), newSelected = [];
                    for (var i = 0; i < oldSelected.length; i++) {
                        if (dataId !== this.gridData.getItem(oldSelected[i]).getDataId()) {
                            newSelected.push(oldSelected[i]);
                        }
                    }
                    this.root.removeSelection(dataId);
                    if (oldSelected.length !== newSelected.length) {
                        this.grid.setSelectedRows(newSelected);
                    }
                };
                TreeGrid.prototype.getSelectedNodes = function () {
                    return this.grid.getSelectedRowItems();
                };
                TreeGrid.prototype.getSelectedDataList = function () {
                    return this.root.getFullSelection().map(function (node) {
                        return node.getData();
                    });
                };
                TreeGrid.prototype.reload = function (parentNodeData) {
                    var _this = this;
                    var expandedNodesDataId = this.grid.getDataView().getItems().filter(function (item) {
                        return item.isExpanded();
                    }).map(function (item) {
                        return item.getDataId();
                    });
                    var selection = this.root.getCurrentSelection();
                    this.root.resetCurrentRoot(parentNodeData);
                    this.initData([]);
                    this.grid.mask();
                    this.reloadNode(null, expandedNodesDataId).then(function () {
                        _this.root.setCurrentSelection(selection);
                        _this.initData(_this.root.getCurrentRoot().treeToList());
                        _this.updateExpanded();
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).finally(function () {
                        _this.updateExpanded();
                    }).done(function () { return _this.notifyLoaded(); });
                };
                TreeGrid.prototype.reloadNode = function (parentNode, expandedNodesDataId) {
                    var _this = this;
                    var deferred = wemQ.defer(), promises = [];
                    this.fetchData(parentNode).then(function (dataList) {
                        var hasNotEmptyChildren = false;
                        parentNode = parentNode || _this.root.getCurrentRoot();
                        parentNode.getChildren().length = 0;
                        dataList.forEach(function (data) {
                            var child = _this.dataToTreeNode(data, parentNode);
                            var dataId = _this.getDataId(data);
                            child.setExpanded(_this.expandAll || expandedNodesDataId.indexOf(dataId) > -1);
                            parentNode.addChild(child);
                            if (child.isExpanded() && _this.hasChildren(data)) {
                                hasNotEmptyChildren = true;
                                promises.push(_this.reloadNode(child, expandedNodesDataId));
                            }
                        });
                        if (!hasNotEmptyChildren) {
                            deferred.resolve(null);
                        }
                        else {
                            wemQ.all(promises).spread(function () {
                                deferred.resolve(null);
                            }).catch(function (reason) {
                                deferred.reject(reason);
                            }).done();
                        }
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                        deferred.reject(reason);
                    }).done();
                    return deferred.promise;
                };
                TreeGrid.prototype.refreshNode = function (node) {
                    var root = this.root.getCurrentRoot();
                    this.active = false;
                    node = node || root;
                    node.regenerateIds();
                    root.setExpanded(true);
                    this.initData(root.treeToList());
                    this.invalidate();
                    this.active = true;
                    this.notifyLoaded();
                };
                TreeGrid.prototype.refresh = function () {
                    var root = this.root.getCurrentRoot();
                    this.active = false;
                    this.grid.invalidate();
                    root.setExpanded(true);
                    this.initData(root.treeToList());
                    this.invalidate();
                    this.active = true;
                    this.notifyLoaded();
                };
                TreeGrid.prototype.updateNode = function (data, oldDataId) {
                    var dataId = oldDataId || this.getDataId(data), nodeToUpdate = this.root.getCurrentRoot().findNode(dataId);
                    if (!nodeToUpdate) {
                        throw new Error("TreeNode to update not found: " + dataId);
                    }
                    return this.fetchAndUpdateNodes([nodeToUpdate], oldDataId ? this.getDataId(data) : undefined);
                };
                TreeGrid.prototype.updateNodes = function (data, oldDataId) {
                    var dataId = oldDataId || this.getDataId(data), nodesToUpdate = this.root.getCurrentRoot().findNodes(dataId);
                    if (!nodesToUpdate) {
                        throw new Error("TreeNode to update not found: " + dataId);
                    }
                    return this.fetchAndUpdateNodes(nodesToUpdate, oldDataId ? this.getDataId(data) : undefined);
                };
                TreeGrid.prototype.fetchAndUpdateNodes = function (nodesToUpdate, dataId) {
                    var _this = this;
                    return this.fetch(nodesToUpdate[0], dataId).then(function (data) {
                        nodesToUpdate.forEach(function (node) {
                            if (dataId) {
                                node.setDataId(dataId);
                            }
                            node.setData(data);
                            if (_this.expandAll) {
                                node.setExpanded(_this.expandAll);
                            }
                            node.setDataId(_this.getDataId(data));
                            node.clearViewers();
                            if (node.isVisible()) {
                                var selected = _this.grid.isRowSelected(_this.gridData.getRowById(node.getId()));
                                _this.gridData.updateItem(node.getId(), node);
                                if (selected) {
                                    _this.grid.addSelectedRow(_this.gridData.getRowById(node.getId()));
                                }
                            }
                        });
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    });
                };
                TreeGrid.prototype.deleteNode = function (data) {
                    this.deleteRootNode(this.root.getDefaultRoot(), data);
                    if (this.root.isFiltered()) {
                        this.deleteRootNode(this.root.getFilteredRoot(), data);
                    }
                };
                TreeGrid.prototype.deleteRootNode = function (root, data) {
                    var _this = this;
                    var dataId = this.getDataId(data), node;
                    while (node = root.findNode(dataId)) {
                        if (node.hasChildren()) {
                            node.getChildren().forEach(function (child) {
                                _this.deleteNode(child.getData());
                            });
                        }
                        if (this.gridData.getItemById(node.getId())) {
                            this.gridData.deleteItem(node.getId());
                        }
                        var parent = node.getParent();
                        if (node && parent) {
                            parent.removeChild(node);
                            parent.setMaxChildren(parent.getMaxChildren() - 1);
                            this.notifyDataChanged(new treegrid.DataChangedEvent([node], treegrid.DataChangedEvent.DELETED));
                        }
                    }
                    this.root.removeSelection(dataId);
                };
                TreeGrid.prototype.appendNode = function (data, nextToSelection, prepend, stashedParentNode) {
                    if (nextToSelection === void 0) { nextToSelection = false; }
                    if (prepend === void 0) { prepend = true; }
                    var parentNode = this.getParentNode(nextToSelection, stashedParentNode);
                    var index = prepend ? 0 : parentNode.getChildren().length - 1;
                    return this.insertNode(data, nextToSelection, index, stashedParentNode);
                };
                TreeGrid.prototype.getParentNode = function (nextToSelection, stashedParentNode) {
                    if (nextToSelection === void 0) { nextToSelection = false; }
                    var root = stashedParentNode || this.root.getCurrentRoot();
                    var parentNode;
                    if (this.getSelectedNodes() && this.getSelectedNodes().length == 1) {
                        parentNode = root.findNode(this.getSelectedNodes()[0].getDataId());
                        if (nextToSelection) {
                            parentNode = parentNode.getParent() || this.root.getCurrentRoot();
                        }
                    }
                    else {
                        parentNode = root;
                    }
                    return parentNode;
                };
                TreeGrid.prototype.insertNode = function (data, nextToSelection, index, stashedParentNode) {
                    var _this = this;
                    if (nextToSelection === void 0) { nextToSelection = false; }
                    if (index === void 0) { index = 0; }
                    var deferred = wemQ.defer();
                    var root = stashedParentNode || this.root.getCurrentRoot();
                    var parentNode = this.getParentNode(nextToSelection, stashedParentNode);
                    var isRootParentNode = (parentNode == root);
                    if (!parentNode.hasChildren() && !isRootParentNode) {
                        this.fetchData(parentNode).then(function (dataList) {
                            if (parentNode.hasChildren()) {
                                _this.doInsertNodeToParentWithChildren(parentNode, data, root, index, stashedParentNode, isRootParentNode);
                            }
                            else {
                                parentNode.setChildren(_this.dataToTreeNodes(dataList, parentNode));
                                _this.initData(root.treeToList());
                                var node = root.findNode(_this.getDataId(data));
                                if (!node) {
                                    parentNode.insertChild(_this.dataToTreeNode(data, root), index);
                                    node = root.findNode(_this.getDataId(data));
                                }
                                if (node) {
                                    if (!stashedParentNode) {
                                        _this.gridData.setItems(root.treeToList());
                                    }
                                    _this.notifyDataChanged(new treegrid.DataChangedEvent([node], treegrid.DataChangedEvent.ADDED));
                                    if (parentNode != root) {
                                        _this.refreshNodeData(parentNode).then(function (node) {
                                            if (!stashedParentNode) {
                                                _this.updateSelectedNode(node);
                                            }
                                        });
                                    }
                                }
                            }
                            deferred.resolve(null);
                        }).catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                            deferred.reject(reason);
                        });
                    }
                    else {
                        this.doInsertNodeToParentWithChildren(parentNode, data, root, index, stashedParentNode, isRootParentNode);
                        deferred.resolve(null);
                    }
                    return deferred.promise;
                };
                TreeGrid.prototype.doInsertNodeToParentWithChildren = function (parentNode, data, root, index, stashedParentNode, isRootParentNode) {
                    parentNode.insertChild(this.dataToTreeNode(data, root), index);
                    var node = root.findNode(this.getDataId(data));
                    if (node) {
                        if (!stashedParentNode) {
                            this.gridData.setItems(root.treeToList());
                        }
                        if (isRootParentNode) {
                            this.sortNodeChildren(parentNode);
                        }
                        else {
                            if (!stashedParentNode) {
                                this.updateSelectedNode(parentNode);
                            }
                        }
                    }
                };
                TreeGrid.prototype.deleteNodes = function (dataList) {
                    this.deleteRootNodes(this.root.getDefaultRoot(), dataList);
                    if (this.root.isFiltered()) {
                        this.deleteRootNodes(this.root.getFilteredRoot(), dataList);
                    }
                };
                TreeGrid.prototype.deleteRootNodes = function (root, dataList) {
                    var _this = this;
                    var updated = [], deleted = [];
                    dataList.forEach(function (data) {
                        var node = root.findNode(_this.getDataId(data));
                        if (node && node.getParent()) {
                            var parent = node.getParent();
                            _this.deleteRootNode(root, node.getData());
                            updated.push(parent);
                            deleted.push(node);
                            updated.filter(function (el) {
                                return el.getDataId() !== node.getDataId();
                            });
                        }
                    });
                    root.treeToList().forEach(function (child) {
                        _this.refreshNodeData(child);
                    });
                    this.notifyDataChanged(new treegrid.DataChangedEvent(deleted, treegrid.DataChangedEvent.DELETED));
                };
                TreeGrid.prototype.initData = function (nodes) {
                    var selection = [], selectionIds = this.root.getFullSelection().map(function (el) {
                        return el.getDataId();
                    });
                    this.gridData.setItems(nodes, "id");
                    this.notifyDataChanged(new treegrid.DataChangedEvent(nodes, treegrid.DataChangedEvent.ADDED));
                    selectionIds.forEach(function (selectionId) {
                        nodes.forEach(function (node, index) {
                            if (node.getDataId() === selectionId) {
                                selection.push(index);
                            }
                        });
                    });
                    this.grid.setSelectedRows(selection);
                };
                TreeGrid.prototype.expandNode = function (node, expandAll) {
                    var _this = this;
                    if (expandAll === void 0) { expandAll = false; }
                    node = node || this.root.getCurrentRoot();
                    if (node) {
                        node.setExpanded(true);
                        if (node.hasChildren()) {
                            this.initData(this.root.getCurrentRoot().treeToList());
                            this.updateExpanded();
                            if (expandAll) {
                                node.getChildren().forEach(function (child) {
                                    _this.expandNode(child);
                                });
                            }
                        }
                        else {
                            this.grid.mask();
                            this.fetchData(node).then(function (dataList) {
                                node.setChildren(_this.dataToTreeNodes(dataList, node));
                                _this.initData(_this.root.getCurrentRoot().treeToList());
                                _this.updateExpanded();
                                if (expandAll) {
                                    node.getChildren().forEach(function (child) {
                                        _this.expandNode(child);
                                    });
                                }
                            }).catch(function (reason) {
                                api.DefaultErrorHandler.handle(reason);
                            }).finally(function () {
                            }).done(function () { return _this.notifyLoaded(); });
                        }
                    }
                };
                TreeGrid.prototype.updateExpanded = function () {
                    this.invalidate();
                    this.active = true;
                };
                TreeGrid.prototype.updateSelectedNode = function (node) {
                    this.getGrid().clearSelection();
                    this.refreshNode(node);
                    var row = this.gridData.getRowById(node.getId());
                    this.grid.selectRow(row);
                };
                TreeGrid.prototype.collapseNode = function (node) {
                    node.setExpanded(false);
                    this.root.stashSelection();
                    this.gridData.refresh();
                    this.invalidate();
                    this.triggerSelectionChangedListeners();
                    this.active = true;
                };
                TreeGrid.prototype.notifyLoaded = function () {
                    var _this = this;
                    this.loadedListeners.forEach(function (listener) {
                        listener(_this);
                    });
                };
                TreeGrid.prototype.onLoaded = function (listener) {
                    this.loadedListeners.push(listener);
                    return this;
                };
                TreeGrid.prototype.unLoaded = function (listener) {
                    this.loadedListeners = this.loadedListeners.filter(function (curr) {
                        return curr != listener;
                    });
                    return this;
                };
                TreeGrid.prototype.notifySelectionChanged = function (event, rows) {
                    var _this = this;
                    var currentSelection = [];
                    if (rows) {
                        rows.forEach(function (rowIndex) {
                            currentSelection.push(_this.gridData.getItem(rowIndex));
                        });
                    }
                    this.root.setCurrentSelection(currentSelection);
                    this.triggerSelectionChangedListeners();
                };
                TreeGrid.prototype.triggerSelectionChangedListeners = function () {
                    for (var i in this.selectionChangeListeners) {
                        this.selectionChangeListeners[i](this.root.getCurrentSelection(), this.root.getFullSelection());
                    }
                };
                TreeGrid.prototype.onSelectionChanged = function (listener) {
                    this.selectionChangeListeners.push(listener);
                    return this;
                };
                TreeGrid.prototype.unSelectionChanged = function (listener) {
                    this.selectionChangeListeners = this.selectionChangeListeners.filter(function (curr) {
                        return curr != listener;
                    });
                    return this;
                };
                TreeGrid.prototype.notifyContextMenuShown = function (x, y) {
                    var showContextMenuEvent = new treegrid.ContextMenuShownEvent(x, y);
                    this.contextMenuListeners.forEach(function (listener) {
                        listener(showContextMenuEvent);
                    });
                };
                TreeGrid.prototype.onContextMenuShown = function (listener) {
                    this.contextMenuListeners.push(listener);
                    return this;
                };
                TreeGrid.prototype.unContextMenuShown = function (listener) {
                    this.contextMenuListeners = this.contextMenuListeners.filter(function (curr) {
                        return curr != listener;
                    });
                    return this;
                };
                TreeGrid.prototype.notifyDataChanged = function (event) {
                    this.dataChangeListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                TreeGrid.prototype.onDataChanged = function (listener) {
                    this.dataChangeListeners.push(listener);
                    return this;
                };
                TreeGrid.prototype.unDataChanged = function (listener) {
                    this.dataChangeListeners = this.dataChangeListeners.filter(function (curr) {
                        return curr != listener;
                    });
                    return this;
                };
                TreeGrid.prototype.isFiltered = function () {
                    return this.root.isFiltered();
                };
                TreeGrid.prototype.invalidate = function () {
                    this.grid.invalidate();
                };
                TreeGrid.prototype.initAndRender = function () {
                    this.initData(this.getRoot().getCurrentRoot().treeToList());
                    this.invalidate();
                };
                TreeGrid.prototype.refreshNodeData = function (parentNode) {
                    return null;
                };
                TreeGrid.prototype.sortNodeChildren = function (node) {
                };
                TreeGrid.LEVEL_STEP_INDENT = 16;
                return TreeGrid;
            })(api.ui.panel.Panel);
            treegrid.TreeGrid = TreeGrid;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var TreeNode = (function () {
                function TreeNode(builder) {
                    this.viewersByName = {};
                    this.id = Math.random().toString(36).substring(2);
                    this.dataId = builder.getDataId();
                    this.data = builder.getData();
                    this.parent = builder.getParent();
                    this.setChildren(builder.getChildren());
                    this.maxChildren = builder.getMaxChildren();
                    this.expanded = builder.isExpanded();
                    this.selected = builder.isSelected();
                    this.pinned = builder.isPinned();
                    if (this.pinned) {
                        this.pinToRoot();
                    }
                }
                TreeNode.prototype.getId = function () {
                    return this.id;
                };
                TreeNode.prototype.hasData = function () {
                    return !!this.data;
                };
                TreeNode.prototype.getDataId = function () {
                    return this.dataId;
                };
                TreeNode.prototype.isExpanded = function () {
                    return this.expanded;
                };
                TreeNode.prototype.setExpanded = function (expanded) {
                    if (expanded === void 0) { expanded = true; }
                    this.expanded = expanded;
                };
                TreeNode.prototype.isSelected = function () {
                    return this.selected;
                };
                TreeNode.prototype.setSelected = function (selected) {
                    if (selected === void 0) { selected = true; }
                    this.selected = selected;
                };
                TreeNode.prototype.isPinned = function () {
                    return this.pinned;
                };
                TreeNode.prototype.getMaxChildren = function () {
                    return this.maxChildren;
                };
                TreeNode.prototype.setMaxChildren = function (maxChildren) {
                    this.maxChildren = maxChildren;
                };
                TreeNode.prototype.getData = function () {
                    return this.data;
                };
                TreeNode.prototype.setData = function (data) {
                    this.data = data;
                };
                TreeNode.prototype.setDataId = function (dataId) {
                    this.dataId = dataId;
                };
                TreeNode.prototype.setViewer = function (name, viewer) {
                    this.viewersByName[name] = viewer;
                };
                TreeNode.prototype.clearViewers = function () {
                    this.viewersByName = {};
                };
                TreeNode.prototype.getViewer = function (name) {
                    return this.viewersByName[name];
                };
                TreeNode.prototype.getParent = function () {
                    return this.parent;
                };
                TreeNode.prototype.setParent = function (parent) {
                    this.parent = parent;
                    if (this.pinned) {
                        this.pinToRoot();
                    }
                };
                TreeNode.prototype.hasParent = function () {
                    return !!this.parent;
                };
                TreeNode.prototype.getRoot = function () {
                    var root = this, parent = this.getParent();
                    while (parent) {
                        root = parent;
                        parent = parent.getParent();
                    }
                    return root;
                };
                TreeNode.prototype.getChildren = function () {
                    return this.children;
                };
                TreeNode.prototype.setChildren = function (children) {
                    var _this = this;
                    this.children = children;
                    this.children.forEach(function (child) {
                        child.setParent(_this);
                    });
                };
                TreeNode.prototype.hasChildren = function () {
                    return this.children.length > 0;
                };
                TreeNode.prototype.regenerateId = function () {
                    this.id = Math.random().toString(36).substring(2);
                };
                TreeNode.prototype.regenerateIds = function () {
                    this.regenerateId();
                    this.children.forEach(function (elem) {
                        elem.regenerateIds();
                    });
                };
                TreeNode.prototype.insertChild = function (child, index) {
                    if (index === void 0) { index = 0; }
                    this.children = this.children || [];
                    this.children.splice(index, 0, child);
                    this.clearViewers();
                    child.setParent(this);
                };
                TreeNode.prototype.moveChild = function (child, index) {
                    if (index === void 0) { index = 0; }
                    this.removeChild(child);
                    this.insertChild(child, index);
                };
                TreeNode.prototype.addChild = function (child, isToBegin) {
                    this.children = this.children || [];
                    if (isToBegin) {
                        this.children.unshift(child);
                    }
                    else {
                        this.children.push(child);
                    }
                    this.clearViewers();
                    child.setParent(this);
                };
                TreeNode.prototype.removeChild = function (child) {
                    var children = [];
                    for (var i = 0; i < this.children.length; i++) {
                        if (this.children[i].getId() !== child.getId()) {
                            children.push(this.children[i]);
                        }
                    }
                    this.children = children;
                    if (this.children.length === 0 && !!this.getParent()) {
                        this.expanded = false;
                    }
                };
                TreeNode.prototype.removeChildren = function () {
                    this.children.length = 0;
                    this.expanded = false;
                };
                TreeNode.prototype.remove = function () {
                    if (this.parent) {
                        this.parent.removeChild(this);
                        this.parent.clearViewers();
                        this.parent = null;
                    }
                };
                TreeNode.prototype.isVisible = function () {
                    var visible = true;
                    var parent = this.parent;
                    while (parent && visible) {
                        visible = parent.isExpanded();
                        parent = parent.getParent();
                    }
                    return visible;
                };
                TreeNode.prototype.treeToList = function (empty, expanded, selected) {
                    if (empty === void 0) { empty = false; }
                    if (expanded === void 0) { expanded = true; }
                    if (selected === void 0) { selected = false; }
                    var list = [];
                    if (this.selected === true || selected === false) {
                        if (this.getData() || empty === true) {
                            list.push(this);
                        }
                    }
                    if (this.expanded === true || expanded === false) {
                        this.children.forEach(function (child) {
                            list = list.concat(child.treeToList(empty, expanded, selected));
                        });
                    }
                    return list;
                };
                TreeNode.prototype.findNode = function (dataId) {
                    if (this.hasData() && this.getDataId() === dataId) {
                        return this;
                    }
                    for (var i = 0; i < this.children.length; i++) {
                        var child = this.children[i].findNode(dataId);
                        if (child) {
                            return child;
                        }
                    }
                    return null;
                };
                TreeNode.prototype.findNodes = function (dataId) {
                    if (this.hasData() && this.getDataId() === dataId) {
                        return [this];
                    }
                    var nodes = [];
                    this.children.forEach(function (el) {
                        var children = el.findNodes(dataId);
                        if (!!children) {
                            nodes = nodes.concat(children);
                        }
                    });
                    return !nodes ? null : nodes;
                };
                TreeNode.prototype.calcLevel = function () {
                    var parent = this.parent, lvl = 0;
                    while (parent) {
                        parent = parent.getParent();
                        lvl++;
                    }
                    return lvl;
                };
                TreeNode.prototype.pinToRoot = function () {
                    if (this.calcLevel() > 1 && this.data && this.parent) {
                        var duplicated = false;
                        var relatives = this.getRoot().getChildren();
                        for (var i = 0; i < relatives.length; i++) {
                            if (relatives[i].getData() && relatives[i].getDataId() === this.getDataId()) {
                                duplicated = true;
                                break;
                            }
                        }
                        if (!duplicated) {
                            if (this.pinned) {
                                this.parent.removeChild(this);
                                this.getRoot().addChild(this);
                            }
                            else {
                                new treegrid.TreeNodeBuilder(this).setPinned(true).build();
                            }
                        }
                    }
                };
                return TreeNode;
            })();
            treegrid.TreeNode = TreeNode;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var TreeRoot = (function () {
                function TreeRoot() {
                    this.defaultRoot = new treegrid.TreeNodeBuilder().setExpanded(true).build();
                    this.filteredRoot = new treegrid.TreeNodeBuilder().setExpanded(true).build();
                    this.filtered = false;
                    this.currentSelection = [];
                    this.stashedSelection = [];
                }
                TreeRoot.prototype.getDefaultRoot = function () {
                    return this.defaultRoot;
                };
                TreeRoot.prototype.resetDefaultRoot = function (rootData) {
                    this.defaultRoot = new treegrid.TreeNodeBuilder().setExpanded(true).build();
                    if (rootData) {
                        this.defaultRoot.setData(rootData);
                    }
                };
                TreeRoot.prototype.getFilteredRoot = function () {
                    return this.filteredRoot;
                };
                TreeRoot.prototype.resetFilteredRoot = function (rootData) {
                    this.filteredRoot = new treegrid.TreeNodeBuilder().setExpanded(true).build();
                    if (rootData) {
                        this.filteredRoot.setData(rootData);
                    }
                };
                TreeRoot.prototype.resetCurrentRoot = function (rootData) {
                    if (this.isFiltered()) {
                        this.resetFilteredRoot(rootData);
                    }
                    else {
                        this.resetDefaultRoot(rootData);
                    }
                };
                TreeRoot.prototype.getCurrentRoot = function () {
                    return this.filtered ? this.filteredRoot : this.defaultRoot;
                };
                TreeRoot.prototype.isFiltered = function () {
                    return this.filtered;
                };
                TreeRoot.prototype.setFiltered = function (filtered) {
                    if (filtered === void 0) { filtered = true; }
                    if (filtered) {
                        this.filteredRoot = new treegrid.TreeNodeBuilder().setExpanded(true).build();
                        this.stashSelection();
                    }
                    else if (this.filtered && !filtered) {
                        this.stashSelection();
                    }
                    this.filtered = filtered;
                };
                TreeRoot.prototype.getCurrentSelection = function () {
                    return this.currentSelection;
                };
                TreeRoot.prototype.setCurrentSelection = function (selection) {
                    this.currentSelection = selection;
                    this.cleanStashedSelection();
                };
                TreeRoot.prototype.getStashedSelection = function () {
                    return this.stashedSelection;
                };
                TreeRoot.prototype.stashSelection = function () {
                    this.stashedSelection = this.stashedSelection.concat(this.currentSelection);
                    this.currentSelection = [];
                    this.cleanStashedSelection();
                };
                TreeRoot.prototype.getFullSelection = function (uniqueOnly) {
                    if (uniqueOnly === void 0) { uniqueOnly = true; }
                    var fullSelection = this.currentSelection.concat(this.stashedSelection);
                    if (uniqueOnly) {
                        var fullIds = fullSelection.map(function (el) {
                            return el.getDataId();
                        });
                        fullSelection = fullSelection.filter(function (value, index, self) {
                            return fullIds.indexOf(value.getDataId()) === index;
                        });
                    }
                    return fullSelection;
                };
                TreeRoot.prototype.cleanStashedSelection = function () {
                    var currentIds = this.currentSelection.map(function (el) {
                        return el.getDataId();
                    }), stashedIds = this.stashedSelection.map(function (el) {
                        return el.getDataId();
                    });
                    this.stashedSelection = this.stashedSelection.filter(function (value, index, self) {
                        return (currentIds.indexOf(value.getDataId()) < 0) && (stashedIds.indexOf(value.getDataId()) === index);
                    });
                };
                TreeRoot.prototype.clearStashedSelection = function () {
                    this.stashedSelection = [];
                };
                TreeRoot.prototype.removeSelection = function (dataId) {
                    this.currentSelection = this.currentSelection.filter(function (el) {
                        return el.getDataId() !== dataId;
                    });
                    this.stashedSelection = this.stashedSelection.filter(function (el) {
                        return el.getDataId() !== dataId;
                    });
                };
                TreeRoot.prototype.updateSelection = function (dataId, data) {
                    this.currentSelection.forEach(function (el) {
                        if (el.getDataId() === dataId) {
                            el.setData(data);
                        }
                    });
                    this.stashedSelection.forEach(function (el) {
                        if (el.getDataId() === dataId) {
                            el.setData(data);
                        }
                    });
                };
                return TreeRoot;
            })();
            treegrid.TreeRoot = TreeRoot;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var TreeGridToolbar = (function (_super) {
                __extends(TreeGridToolbar, _super);
                function TreeGridToolbar(actions, treeGrid) {
                    _super.call(this);
                    this.addActions(actions.getAllActions());
                    this.addGreedySpacer();
                    this.refreshButton = new api.ui.button.Button().addClass('icon-loop2');
                    this.refreshButton.onClicked(function (event) { return treeGrid.reload(); });
                    this.addElement(this.refreshButton);
                    this.treeGrid = treeGrid;
                }
                TreeGridToolbar.prototype.refresh = function (selectedItemsCount) {
                    this.removeChild(this.refreshButton);
                    this.removeGreedySpacer();
                    this.removeActions();
                    var actions = new api.ui.treegrid.actions.TreeGridToolbarActions(this.treeGrid, selectedItemsCount).getAllActions();
                    this.addActions(actions);
                    this.addGreedySpacer();
                    this.addElement(this.refreshButton);
                };
                return TreeGridToolbar;
            })(api.ui.toolbar.Toolbar);
            treegrid.TreeGridToolbar = TreeGridToolbar;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var DateTimeFormatter = (function () {
                function DateTimeFormatter() {
                }
                DateTimeFormatter.format = function (row, cell, value, columnDef, item) {
                    return DateTimeFormatter.createHtml(value);
                };
                DateTimeFormatter.formatNoTimestamp = function (row, cell, value, columnDef, item) {
                    return DateTimeFormatter.createHtmlNoTimestamp(value);
                };
                DateTimeFormatter.createHtml = function (date) {
                    if (!!date) {
                        var s = DateTimeFormatter.zeroPad(date.getFullYear(), 4) + "-" + DateTimeFormatter.zeroPad(date.getMonth() + 1, 2) + "-" + DateTimeFormatter.zeroPad(date.getDate(), 2) + " " + DateTimeFormatter.zeroPad(date.getHours(), 2) + ":" + DateTimeFormatter.zeroPad(date.getMinutes(), 2) + ":" + DateTimeFormatter.zeroPad(date.getSeconds(), 2);
                        return s;
                    }
                    return "";
                };
                DateTimeFormatter.createHtmlNoTimestamp = function (date) {
                    if (!!date) {
                        var s = DateTimeFormatter.zeroPad(date.getFullYear(), 4) + "-" + DateTimeFormatter.zeroPad(date.getMonth() + 1, 2) + "-" + DateTimeFormatter.zeroPad(date.getDate(), 2);
                        return s;
                    }
                    return "";
                };
                DateTimeFormatter.zeroPad = function (n, width) {
                    var nWidth = n.toString().length;
                    if (nWidth >= width) {
                        return "" + n;
                    }
                    var neededZeroes = width - nWidth;
                    var s = "";
                    for (var i = 0; i < neededZeroes; i++) {
                        s += "0";
                    }
                    return s + n;
                };
                return DateTimeFormatter;
            })();
            treegrid.DateTimeFormatter = DateTimeFormatter;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var GridColumnBuilder = api.ui.grid.GridColumnBuilder;
            var GridOptionsBuilder = api.ui.grid.GridOptionsBuilder;
            var TreeGridBuilder = (function () {
                function TreeGridBuilder(grid) {
                    this.expandAll = false;
                    this.showToolbar = true;
                    this.columns = [];
                    this.classes = "";
                    this.autoLoad = true;
                    this.hotkeysEnabled = true;
                    this.partialLoadEnabled = false;
                    this.loadBufferSize = 0;
                    if (grid) {
                        this.showToolbar = grid.hasToolbar();
                        this.contextMenu = grid.getContextMenu();
                        this.classes = grid.getEl().getClass();
                        this.copyOptions(grid.getOptions()).copyColumns(grid.getColumns());
                    }
                    else {
                        this.options = this.buildDefaultOptions();
                        this.columns = this.buildDefaultColumns();
                    }
                    this.classes = "tree-grid " + this.classes;
                }
                TreeGridBuilder.prototype.nodeExtractor = function (node, column) {
                    var names = column.field.split('.');
                    var val = node["data"][names[0]];
                    for (var i = 1; i < names.length; i++) {
                        if (val && typeof val == 'object' && names[i] in val) {
                            val = val[names[i]];
                        }
                        else {
                            val = '';
                        }
                    }
                    return val;
                };
                TreeGridBuilder.prototype.buildDefaultOptions = function () {
                    return new GridOptionsBuilder().setDataItemColumnValueExtractor(this.nodeExtractor).setEditable(false).setAutoHeight(false).setEnableAsyncPostRender(true).setAutoRenderGridOnDataChanges(true).setEnableCellNavigation(false).setEnableColumnReorder(false).setForceFitColumns(true).setHideColumnHeaders(true).setCheckableRows(true).setRowHeight(45).build();
                };
                TreeGridBuilder.prototype.buildDefaultColumns = function () {
                    return [];
                };
                TreeGridBuilder.prototype.copyOptions = function (options) {
                    this.options = new GridOptionsBuilder().build();
                    return this;
                };
                TreeGridBuilder.prototype.copyColumns = function (columns) {
                    var _this = this;
                    this.columns = [];
                    columns.forEach(function (column) {
                        _this.columns.push(new GridColumnBuilder(column).build());
                    });
                    return this;
                };
                TreeGridBuilder.prototype.isShowToolbar = function () {
                    return this.showToolbar;
                };
                TreeGridBuilder.prototype.getContextMenu = function () {
                    return this.contextMenu;
                };
                TreeGridBuilder.prototype.getOptions = function () {
                    return this.options;
                };
                TreeGridBuilder.prototype.getColumns = function () {
                    return this.columns;
                };
                TreeGridBuilder.prototype.getClasses = function () {
                    return this.classes;
                };
                TreeGridBuilder.prototype.isExpandAll = function () {
                    return this.expandAll;
                };
                TreeGridBuilder.prototype.setExpandAll = function (value) {
                    this.expandAll = value;
                    return this;
                };
                TreeGridBuilder.prototype.setShowToolbar = function (showToolbar) {
                    this.showToolbar = showToolbar;
                    return this;
                };
                TreeGridBuilder.prototype.setShowContextMenu = function (contextMenu) {
                    this.contextMenu = contextMenu;
                    return this;
                };
                TreeGridBuilder.prototype.setOptions = function (options) {
                    this.options = options;
                    return this;
                };
                TreeGridBuilder.prototype.setColumns = function (columns) {
                    this.columns = columns;
                    return this;
                };
                TreeGridBuilder.prototype.setClasses = function (classes) {
                    this.classes = classes;
                    return this;
                };
                TreeGridBuilder.prototype.prependClasses = function (classes) {
                    this.classes = classes + " " + this.classes;
                    return this;
                };
                TreeGridBuilder.prototype.setAutoLoad = function (autoLoad) {
                    this.autoLoad = autoLoad;
                    return this;
                };
                TreeGridBuilder.prototype.isAutoLoad = function () {
                    return this.autoLoad;
                };
                TreeGridBuilder.prototype.setCheckableRows = function (checkable) {
                    this.options.setCheckableRows(checkable);
                    return this;
                };
                TreeGridBuilder.prototype.isCheckableRows = function () {
                    return this.options.isCheckableRows();
                };
                TreeGridBuilder.prototype.setDragAndDrop = function (dragAndDrop) {
                    this.options.setDragAndDrop(dragAndDrop);
                    return this;
                };
                TreeGridBuilder.prototype.isDragAndDrop = function () {
                    return this.options.isDragAndDrop();
                };
                TreeGridBuilder.prototype.setSelectedCellCssClass = function (selectedCellCss) {
                    this.options.setSelectedCellCssClass(selectedCellCss);
                    return this;
                };
                TreeGridBuilder.prototype.getSelectedCellCssClass = function () {
                    return this.options.getSelectedCellCssClass();
                };
                TreeGridBuilder.prototype.disableMultipleSelection = function (disableMultipleSelection) {
                    this.options.disableMultipleSelection(disableMultipleSelection);
                    return this;
                };
                TreeGridBuilder.prototype.isMultipleSelectionDisabled = function () {
                    return this.options.isMultipleSelectionDisabled();
                };
                TreeGridBuilder.prototype.setHotkeysEnabled = function (enabled) {
                    this.hotkeysEnabled = enabled;
                    return this;
                };
                TreeGridBuilder.prototype.isHotkeysEnabled = function () {
                    return this.hotkeysEnabled;
                };
                TreeGridBuilder.prototype.setPartialLoadEnabled = function (enabled) {
                    this.partialLoadEnabled = enabled;
                    return this;
                };
                TreeGridBuilder.prototype.isPartialLoadEnabled = function () {
                    return this.partialLoadEnabled;
                };
                TreeGridBuilder.prototype.setLoadBufferSize = function (loadBufferSize) {
                    this.loadBufferSize = loadBufferSize;
                    return this;
                };
                TreeGridBuilder.prototype.getLoadBufferSize = function () {
                    return this.loadBufferSize;
                };
                TreeGridBuilder.prototype.setRowHeight = function (rowHeight) {
                    this.options.rowHeight = rowHeight;
                    return this;
                };
                TreeGridBuilder.prototype.build = function () {
                    return new treegrid.TreeGrid(this);
                };
                return TreeGridBuilder;
            })();
            treegrid.TreeGridBuilder = TreeGridBuilder;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var TreeNodeBuilder = (function () {
                function TreeNodeBuilder(node) {
                    if (node) {
                        this.data = node.getData();
                        this.parent = node.getParent();
                        this.children = node.getChildren() || [];
                        this.maxChildren = node.getMaxChildren();
                    }
                    else {
                        this.children = [];
                        this.maxChildren = 0;
                    }
                    this.pinned = false;
                    this.expanded = false;
                    this.selected = false;
                }
                TreeNodeBuilder.prototype.isExpanded = function () {
                    return this.expanded;
                };
                TreeNodeBuilder.prototype.setExpanded = function (expanded) {
                    if (expanded === void 0) { expanded = true; }
                    this.expanded = expanded;
                    return this;
                };
                TreeNodeBuilder.prototype.isSelected = function () {
                    return this.selected;
                };
                TreeNodeBuilder.prototype.setSelected = function (selected) {
                    if (selected === void 0) { selected = true; }
                    this.selected = selected;
                    return this;
                };
                TreeNodeBuilder.prototype.isPinned = function () {
                    return this.pinned;
                };
                TreeNodeBuilder.prototype.setPinned = function (pinned) {
                    if (pinned === void 0) { pinned = true; }
                    this.pinned = pinned;
                    return this;
                };
                TreeNodeBuilder.prototype.getMaxChildren = function () {
                    return this.maxChildren;
                };
                TreeNodeBuilder.prototype.setMaxChildren = function (maxChildren) {
                    this.maxChildren = maxChildren;
                };
                TreeNodeBuilder.prototype.getData = function () {
                    return this.data;
                };
                TreeNodeBuilder.prototype.getDataId = function () {
                    return this.dataId;
                };
                TreeNodeBuilder.prototype.setData = function (data, dataId) {
                    this.data = data;
                    this.dataId = dataId;
                    return this;
                };
                TreeNodeBuilder.prototype.getParent = function () {
                    return this.parent;
                };
                TreeNodeBuilder.prototype.setParent = function (parent) {
                    this.parent = parent;
                    return this;
                };
                TreeNodeBuilder.prototype.getChildren = function () {
                    return this.children;
                };
                TreeNodeBuilder.prototype.setChildren = function (children) {
                    this.children = children;
                    return this;
                };
                TreeNodeBuilder.prototype.build = function () {
                    return new treegrid.TreeNode(this);
                };
                return TreeNodeBuilder;
            })();
            treegrid.TreeNodeBuilder = TreeNodeBuilder;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var TreeGridContextMenu = (function (_super) {
                __extends(TreeGridContextMenu, _super);
                function TreeGridContextMenu(actions) {
                    _super.call(this);
                    this.actions = actions;
                    this.addActions(actions.getAllActions());
                }
                TreeGridContextMenu.prototype.getActions = function () {
                    return this.actions;
                };
                return TreeGridContextMenu;
            })(api.ui.menu.ContextMenu);
            treegrid.TreeGridContextMenu = TreeGridContextMenu;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var ContextMenuShownEvent = (function () {
                function ContextMenuShownEvent(x, y) {
                    this.x = x;
                    this.y = y;
                }
                ContextMenuShownEvent.prototype.getX = function () {
                    return this.x;
                };
                ContextMenuShownEvent.prototype.getY = function () {
                    return this.y;
                };
                return ContextMenuShownEvent;
            })();
            treegrid.ContextMenuShownEvent = ContextMenuShownEvent;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var treegrid;
        (function (treegrid) {
            var DataChangedEvent = (function () {
                function DataChangedEvent(treeNode, action) {
                    this.treeNodes = treeNode;
                    this.type = action;
                }
                DataChangedEvent.prototype.getTreeNodes = function () {
                    return this.treeNodes;
                };
                DataChangedEvent.prototype.getType = function () {
                    return this.type;
                };
                DataChangedEvent.ADDED = 'added';
                DataChangedEvent.UPDATED = 'updated';
                DataChangedEvent.DELETED = 'deleted';
                return DataChangedEvent;
            })();
            treegrid.DataChangedEvent = DataChangedEvent;
        })(treegrid = ui.treegrid || (ui.treegrid = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var image;
        (function (_image) {
            var ImageCanvas = (function (_super) {
                __extends(ImageCanvas, _super);
                function ImageCanvas(image) {
                    var _this = this;
                    _super.call(this, 'image-canvas');
                    this.zoom = { x: -1, y: -1, factor: 1, previous: 1 };
                    this.pan = { x: 0, y: 0, overrideZoom: false };
                    this.zoomChangeListeners = [];
                    this.panChangeListeners = [];
                    this.image = image;
                    api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                        console.group('on size changed');
                        if (_this.enabled) {
                            _this.recalculateHeight(true, false);
                            _this.renderCanvas();
                        }
                        console.groupEnd();
                    });
                    image.onLoaded(function (event) {
                        console.group('on image loaded');
                        if (_this.enabled) {
                            _this.recalculateHeight(false, true);
                            _this.renderCanvas();
                        }
                        console.groupEnd();
                    });
                    var prev;
                    image.onMouseDown(function (event) {
                        if (_this.enabled) {
                            event.preventDefault();
                            event.stopPropagation();
                            prev = { x: event.clientX, y: event.clientY };
                            _this.addClass('dragging');
                        }
                    });
                    image.onMouseMove(function (event) {
                        if (_this.enabled && prev && _this.zoom.factor > 1) {
                            _this.setPan(_this.pan.x + (event.clientX - prev.x) / _this.canvasWidth, _this.pan.y + (event.clientY - prev.y) / _this.canvasWidth);
                            prev = { x: event.clientX, y: event.clientY };
                        }
                    });
                    image.onMouseWheel(function (event) {
                        if (_this.enabled) {
                            var delta = (event['wheelDelta'] || -event.detail) > 0 ? 0.05 : -0.05;
                            var offset = _this.getEl().getOffset();
                            _this.setZoom(_this.getZoom() + delta, true, (event.clientX - offset.left) / _this.canvasWidth, (event.clientY - offset.top) / _this.canvasWidth);
                        }
                    });
                    var stopDrag = function (event) {
                        prev = undefined;
                        _this.removeClass('dragging');
                    };
                    image.onMouseLeave(stopDrag);
                    image.onMouseUp(stopDrag);
                }
                ImageCanvas.prototype.setPan = function (x, y, override) {
                    if (override === void 0) { override = true; }
                    this.pan.x = x;
                    this.pan.y = y;
                    if (override) {
                        this.pan.overrideZoom = true;
                    }
                    if (!this.enabled) {
                        return;
                    }
                    console.group('setPan');
                    console.debug('x = ' + x, 'y = ' + y);
                    this.renderCanvas();
                    if (this.pan.x == x && this.pan.y == y) {
                        this.notifyPanChanged(this.pan.x, this.pan.y);
                    }
                    console.groupEnd();
                };
                ImageCanvas.prototype.getPan = function () {
                    return {
                        x: this.pan.x,
                        y: this.pan.y
                    };
                };
                ImageCanvas.prototype.setZoom = function (value, override, x, y) {
                    if (override === void 0) { override = true; }
                    if (this.zoom.factor != value) {
                        this.zoom.previous = this.zoom.factor;
                        this.zoom.factor = +value.toFixed(3);
                    }
                    if (x) {
                        this.zoom.x = x;
                    }
                    if (y) {
                        this.zoom.y = y;
                    }
                    if (override) {
                        this.pan.overrideZoom = false;
                    }
                    if (!this.enabled) {
                        return;
                    }
                    console.group('setZoom');
                    console.debug('zoom = ' + value, 'x = ', x, 'y = ', y);
                    if (this.zoom.factor > 1) {
                        if (this.zoom.x < 0 && this.zoom.y < 0) {
                            this.zoom.x = 0.5;
                            this.zoom.y = 0.5 * this.canvasHeight / this.canvasWidth;
                            console.debug('setting default zoom center to (' + this.zoom.x + ', ' + this.zoom.y + ')');
                        }
                        if (!this.hasClass('draggable')) {
                            this.addClass('draggable');
                        }
                    }
                    else if (this.zoom.factor == 1) {
                        this.pan.overrideZoom = false;
                        if (this.hasClass('draggable')) {
                            this.removeClass('draggable');
                        }
                    }
                    this.renderCanvas();
                    if (this.zoom.factor == value) {
                        this.notifyZoomChanged(this.zoom.factor);
                    }
                    console.groupEnd();
                };
                ImageCanvas.prototype.getZoom = function () {
                    return this.zoom.factor;
                };
                ImageCanvas.prototype.setEnabled = function (isEnabled) {
                    this.enabled = isEnabled;
                    if (isEnabled) {
                        this.image.replaceWith(this);
                        this.appendChild(this.image);
                        this.enableCanvas();
                    }
                    else {
                        this.removeChild(this.image);
                        this.replaceWith(this.image);
                        this.disableCanvas();
                    }
                };
                ImageCanvas.prototype.isEnabled = function () {
                    return this.enabled;
                };
                ImageCanvas.prototype.renderCanvas = function () {
                    if (this.suspendRender || !this.enabled) {
                        return;
                    }
                    console.group('renderCanvas');
                    console.debug('zoom', this.zoom, '\npan', this.pan);
                    var imgEl = this.image.getEl();
                    console.debug('canvas (' + this.canvasWidth + ', ' + this.canvasHeight + ')');
                    var oldZoomFactor = this.zoom.previous, zoomFactor = Math.min(Math.max(this.zoom.factor, 1), 10);
                    console.debug('old zoom factor = ' + oldZoomFactor + ', \nnew zoom factor after restraining = ' + zoomFactor);
                    imgEl.setWidthPx(this.canvasWidth * zoomFactor);
                    var imgHeight = imgEl.getHeight(), imgWidth = imgEl.getWidth();
                    console.debug('image (' + imgWidth + ', ' + imgHeight + ')');
                    var panX, panY;
                    if (!this.pan.overrideZoom) {
                        var zoomWidthPanFactor = (this.zoom.x - this.pan.x) / oldZoomFactor;
                        var zoomHeightPanFactor = (this.zoom.y - this.pan.y) / oldZoomFactor;
                        panX = this.zoom.x - zoomFactor * zoomWidthPanFactor;
                        panY = this.zoom.y - zoomFactor * zoomHeightPanFactor;
                        console.debug('based on zoom center, pan = (' + panX + ', ' + panY + ')');
                    }
                    else {
                        panX = this.pan.x;
                        panY = this.pan.y;
                    }
                    var rightLimit = (this.canvasWidth - imgWidth) / this.canvasWidth;
                    if (imgWidth > this.canvasWidth) {
                        panX = Math.max(Math.min(panX, 0), rightLimit);
                    }
                    else {
                        panX = Math.min(Math.max(panX, 0), rightLimit);
                    }
                    var bottomLimit = (this.canvasHeight - imgHeight) / this.canvasWidth;
                    if (imgHeight > this.canvasHeight) {
                        panY = Math.max(Math.min(panY, 0), bottomLimit);
                    }
                    else {
                        panY = Math.min(Math.max(panY, 0), bottomLimit);
                    }
                    console.debug('after restraining to (0, 0, ' + rightLimit + ', ' + bottomLimit + '), \npan = (' + panX, ', ' + panY + ')');
                    imgEl.setMarginLeft(panX * 100 + '%').setMarginTop(panY * 100 + '%');
                    if (panX != this.pan.x || panY != this.pan.y) {
                        this.pan.x = panX;
                        this.pan.y = panY;
                        this.notifyPanChanged(panX, panY);
                    }
                    if (zoomFactor != this.zoom.factor) {
                        this.zoom.factor = zoomFactor;
                        this.notifyZoomChanged(zoomFactor);
                    }
                    console.groupEnd();
                };
                ImageCanvas.prototype.disableCanvas = function () {
                    this.image.getEl().setWidth('').setMarginLeft('').setMarginTop('');
                };
                ImageCanvas.prototype.enableCanvas = function () {
                    this.recalculateHeight(true, true);
                    this.suspendRender = true;
                    this.setZoom(this.zoom.factor, false);
                    this.setPan(this.pan.x, this.pan.y, false);
                    this.suspendRender = false;
                    this.renderCanvas();
                };
                ImageCanvas.prototype.recalculateHeight = function (updateWidth, updateRatio) {
                    if (updateWidth) {
                        this.canvasWidth = this.getEl().getWidth();
                        console.debug('new width = ' + this.canvasWidth);
                    }
                    if (updateRatio) {
                        this.imageRatio = +(this.image.getEl().getWidth() / this.image.getEl().getHeight()).toFixed(3);
                        console.debug('new ratio = ' + this.imageRatio);
                    }
                    this.canvasHeight = +(this.canvasWidth / this.imageRatio).toFixed(3);
                    console.debug('new height = ' + this.canvasHeight);
                    this.getEl().setHeightPx(this.canvasHeight);
                };
                ImageCanvas.prototype.onZoomChanged = function (listener) {
                    this.zoomChangeListeners.push(listener);
                };
                ImageCanvas.prototype.unZoomChanged = function (listener) {
                    this.zoomChangeListeners = this.zoomChangeListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                ImageCanvas.prototype.notifyZoomChanged = function (zoom) {
                    console.debug('notifyZoomChanged', zoom);
                    this.zoomChangeListeners.forEach(function (listener) {
                        listener(zoom);
                    });
                };
                ImageCanvas.prototype.onPanChanged = function (listener) {
                    this.panChangeListeners.push(listener);
                };
                ImageCanvas.prototype.unPanChanged = function (listener) {
                    this.panChangeListeners = this.panChangeListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                ImageCanvas.prototype.notifyPanChanged = function (x, y) {
                    console.debug('notifyPanChanged', x, y);
                    this.panChangeListeners.forEach(function (listener) {
                        listener(x, y);
                    });
                };
                return ImageCanvas;
            })(api.dom.DivEl);
            _image.ImageCanvas = ImageCanvas;
        })(image = ui.image || (ui.image = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var image;
        (function (_image) {
            var ImgEl = api.dom.ImgEl;
            var DivEl = api.dom.DivEl;
            var Button = api.ui.button.Button;
            var Element = api.dom.Element;
            var ImageEditor = (function (_super) {
                __extends(ImageEditor, _super);
                function ImageEditor(src) {
                    var _this = this;
                    _super.call(this, 'image-editor');
                    this.SCROLLABLE_SELECTOR = '.form-panel';
                    this.WIZARD_TOOLBAR_SELECTOR = '.wizard-step-navigator-and-toolbar';
                    this.focusData = { x: 0, y: 0, r: 0.25, auto: undefined };
                    this.cropData = { x: 0, y: 0, w: 0, h: 0, auto: undefined };
                    this.zoomData = { x: 0, y: 0, w: 0, h: 0 };
                    this.maxZoom = 5;
                    this.editModeListeners = [];
                    this.focusPositionChangedListeners = [];
                    this.autoFocusChangedListeners = [];
                    this.focusRadiusChangedListeners = [];
                    this.cropPositionChangedListeners = [];
                    this.autoCropChangedListeners = [];
                    this.shaderVisibilityChangedListeners = [];
                    this.WHEEL_PIXEL_STEP = 10;
                    this.WHEEL_LINE_HEIGHT = 20;
                    this.WHEEL_PAGE_HEIGHT = 800;
                    this.frame = new DivEl('image-frame');
                    this.canvas = new DivEl('image-canvas');
                    this.image = new ImgEl(null, 'image-bg');
                    this.image.onLoaded(function (event) {
                        if (_this.isImageLoaded()) {
                            _this.updateImageDimensions(true);
                            _this.updateStickyToolbar();
                        }
                    });
                    var myId = this.getId();
                    var clipHtml = '<svg xmlns="http://www.w3.org/2000/svg" version="1.1">' + '    <defs>' + '        <clipPath id="' + myId + '-focusClipPath">' + '            <circle cx="0" cy="0" r="0" class="clip-circle"/>' + '        </clipPath>' + '        <clipPath id="' + myId + '-cropClipPath">' + '            <rect x="0" y="0" width="0" height="0"/>' + '        </clipPath>' + '    </defs>' + '    <image xlink:href="' + ImgEl.PLACEHOLDER + '" width="100%" height="100%"/>' + '    <g class="edit-group focus-group">' + '        <circle cx="0" cy="0" r="0" class="stroke-circle"/>' + '    </g>' + '    <g class="edit-group crop-group">' + '        <svg id="' + myId + '-dragHandle" class="drag-handle">' + '            <defs>' + '                <polygon id="' + myId + '-dragTriangle" class="drag-triangle" points="8,0,16,8,0,8"/>' + '            </defs>' + '            <circle cx="16" cy="16" r="16"/>' + '            <use xlink:href="#' + myId + '-dragTriangle" x="8" y="6"/>' + '            <use xlink:href="#' + myId + '-dragTriangle" x="8" y="18" transform="rotate(180, 16, 22)"/>' + '        </svg>' + '    </g>' + '</svg>';
                    this.clip = Element.fromString(clipHtml);
                    this.dragHandle = this.clip.findChildById(myId + '-dragHandle', true);
                    this.focusClipPath = this.clip.findChildById(myId + '-focusClipPath', true);
                    this.cropClipPath = this.clip.findChildById(myId + '-cropClipPath', true);
                    this.clip.getHTMLElement().querySelector('image')['ondragstart'] = function () {
                        return false;
                    };
                    var imageMask = new api.dom.DivEl("image-bg-mask");
                    this.canvas.appendChildren(imageMask, this.image, this.clip);
                    this.frame.appendChild(this.canvas);
                    this.stickyToolbar = this.createStickyToolbar();
                    this.appendChildren(this.stickyToolbar, this.frame);
                    var scrollListener = function (event) { return _this.updateStickyToolbar(); };
                    var resizeListener = function (item) {
                        if (_this.isVisible()) {
                            _this.updateImageDimensions(false, true);
                            _this.updateStickyToolbar();
                        }
                    };
                    this.onAdded(function (event) {
                        wemjq(_this.getHTMLElement()).closest(_this.SCROLLABLE_SELECTOR).bind("scroll", scrollListener);
                        api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(_this, resizeListener);
                    });
                    this.onRemoved(function (event) {
                        wemjq(event.getParent().getHTMLElement()).closest(_this.SCROLLABLE_SELECTOR).unbind("scroll", scrollListener);
                        api.ui.responsive.ResponsiveManager.unAvailableSizeChanged(_this);
                    });
                    if (src) {
                        this.setSrc(src);
                    }
                    this.setFocusAutoPositioned(true);
                    this.setCropAutoPositioned(true);
                }
                ImageEditor.prototype.isElementInsideButtonsContainer = function (el) {
                    return this.stickyToolbar.getHTMLElement().contains(el);
                };
                ImageEditor.prototype.getLastButtonInContainer = function () {
                    return this.uploadButton;
                };
                ImageEditor.prototype.remove = function () {
                    if (this.isFocusEditMode()) {
                        this.disableFocusEditMode(false);
                    }
                    else if (this.isCropEditMode()) {
                        this.disableCropEditMode(false);
                    }
                    _super.prototype.remove.call(this);
                    return this;
                };
                ImageEditor.prototype.setSrc = function (src) {
                    this.image.setSrc(src);
                    var image = this.clip.getHTMLElement().querySelector('image');
                    image.setAttribute('xlink:href', src);
                };
                ImageEditor.prototype.getSrc = function () {
                    return this.image.getSrc();
                };
                ImageEditor.prototype.getImage = function () {
                    return this.image;
                };
                ImageEditor.prototype.getUploadButton = function () {
                    return this.uploadButton;
                };
                ImageEditor.prototype.setImageClipPath = function (path) {
                    var image = this.clip.getHTMLElement().querySelector('image');
                    image.setAttribute('clip-path', 'url(#' + path.getId() + ')');
                };
                ImageEditor.prototype.normalizePoint = function (point) {
                    return {
                        x: point.x / this.cropData.w,
                        y: point.y / this.cropData.h
                    };
                };
                ImageEditor.prototype.denormalizePoint = function (x, y) {
                    return {
                        x: x * this.cropData.w,
                        y: y * this.cropData.h
                    };
                };
                ImageEditor.prototype.normalizeRect = function (rect) {
                    var minW = Math.min(this.frameW, this.imgW);
                    var minH = Math.min(this.frameH, this.imgH);
                    return {
                        x: rect.x / minW,
                        y: rect.y / minH,
                        w: rect.w / minW,
                        h: rect.h / minH
                    };
                };
                ImageEditor.prototype.denormalizeRect = function (rect) {
                    var minW = Math.min(this.frameW, this.imgW);
                    var minH = Math.min(this.frameH, this.imgH);
                    return {
                        x: rect.x * minW,
                        y: rect.y * minH,
                        w: rect.w * minW,
                        h: rect.h * minH
                    };
                };
                ImageEditor.prototype.normalizeRadius = function (r) {
                    return r / Math.min(this.cropData.w, this.cropData.h);
                };
                ImageEditor.prototype.denormalizeRadius = function (r) {
                    return r * Math.min(this.cropData.w, this.cropData.h);
                };
                ImageEditor.prototype.getOffsetX = function (e) {
                    return e.clientX - this.frame.getEl().getOffset().left;
                };
                ImageEditor.prototype.getOffsetY = function (e) {
                    return e.clientY - this.frame.getEl().getOffset().top;
                };
                ImageEditor.prototype.isImageLoaded = function () {
                    return this.image.isLoaded() && !this.image.isPlaceholder();
                };
                ImageEditor.prototype.updateImageDimensions = function (reset, scale) {
                    if (reset === void 0) { reset = false; }
                    if (scale === void 0) { scale = false; }
                    var imgEl = this.image.getEl(), frameEl = this.frame.getEl();
                    var zoomPct, cropPct, focusPosPct, focusRadPct, revZoomPct, revCropPct, revFocusPosPct, revFocusRadPct;
                    if (scale) {
                        zoomPct = this.normalizeRect(this.getZoomPositionPx());
                        cropPct = this.normalizeRect(this.getCropPositionPx());
                        focusPosPct = this.normalizePoint(this.getFocusPositionPx());
                        focusRadPct = this.normalizeRadius(this.getFocusRadiusPx());
                        if (this.revertZoomData) {
                            revZoomPct = this.normalizeRect(this.revertZoomData);
                        }
                        if (this.revertCropData) {
                            revCropPct = this.normalizeRect(this.revertCropData);
                        }
                        if (this.revertFocusData) {
                            revFocusPosPct = this.normalizePoint(this.revertFocusData);
                            revFocusRadPct = this.normalizeRadius(this.revertFocusData.r);
                        }
                    }
                    this.imgW = imgEl.getNaturalWidth();
                    this.imgH = imgEl.getNaturalHeight();
                    this.frameW = this.getEl().getWidth();
                    this.frameH = (this.frameW * this.imgH) / this.imgW;
                    frameEl.setWidthPx(this.frameW).setHeightPx(this.frameH);
                    if (ImageEditor.debug) {
                        console.group('ImageEditor.updateImageDimensions');
                        console.log('Image loaded: ' + this.imgW + ' x ' + this.imgH + ', frame: ' + this.frameW + ' x ' + this.frameH);
                    }
                    if (reset) {
                        if (this.revertZoomData) {
                            this.setZoomPositionPx(this.denormalizeRect(this.revertZoomData));
                            this.revertZoomData = undefined;
                        }
                        else if (this.cropData.auto) {
                            this.resetZoomPosition();
                        }
                        if (this.revertCropData) {
                            this.setCropPositionPx(this.denormalizeRect(this.revertCropData));
                            this.revertCropData = undefined;
                        }
                        else if (this.cropData.auto) {
                            this.resetCropPosition();
                        }
                        if (this.revertFocusData) {
                            this.setFocusPositionPx(this.denormalizePoint(this.revertFocusData.x, this.revertFocusData.y));
                            this.setFocusRadiusPx(this.denormalizeRadius(this.revertFocusData.r));
                            this.revertFocusData = undefined;
                        }
                        else if (this.focusData.auto) {
                            this.resetFocusPosition();
                            this.resetFocusRadius();
                        }
                        this.updateFocusMaskPosition();
                        this.updateCropMaskPosition();
                        this.updateZoomPosition();
                    }
                    if (scale) {
                        this.setZoomPositionPx(this.denormalizeRect(zoomPct), false);
                        this.setCropPositionPx(this.denormalizeRect(cropPct), false);
                        this.setFocusPositionPx(this.denormalizePoint(focusPosPct.x, focusPosPct.y), false);
                        this.setFocusRadiusPx(this.denormalizeRadius(focusRadPct), false);
                        if (revZoomPct) {
                            this.revertZoomData = this.denormalizeRect(revZoomPct);
                        }
                        if (revCropPct) {
                            var revCrop = this.denormalizeRect(revCropPct);
                            this.revertCropData = {
                                x: revCrop.x,
                                y: revCrop.y,
                                w: revCrop.w,
                                h: revCrop.h,
                                auto: this.revertCropData.auto
                            };
                        }
                        if (revFocusPosPct && revFocusRadPct) {
                            var revFocusPos = this.denormalizePoint(revFocusPosPct.x, revFocusPosPct.y);
                            var revFocusRad = this.denormalizeRadius(revFocusRadPct);
                            this.revertFocusData = {
                                x: revFocusPos.x,
                                y: revFocusPos.y,
                                r: revFocusRad,
                                auto: this.revertFocusData.auto
                            };
                        }
                    }
                    if (ImageEditor.debug) {
                        console.groupEnd();
                    }
                };
                ImageEditor.prototype.updateFrameHeight = function () {
                    this.frame.getEl().setHeightPx(this.cropData.h);
                };
                ImageEditor.prototype.isOutside = function (event) {
                    var el = this.getEl(), offset = el.getOffset(), bottom = offset.top + el.getHeightWithBorder(), right = offset.left + el.getWidthWithBorder(), scrollEl = wemjq(this.getHTMLElement()).closest(this.SCROLLABLE_SELECTOR), scrollOffset = scrollEl.length == 1 ? scrollEl.offset() : {
                        left: 0,
                        top: 0
                    };
                    return event.clientX < Math.max(scrollOffset.left, offset.left) || event.clientX > right || event.clientY < Math.max(scrollOffset.top, offset.top) || event.clientY > bottom;
                };
                ImageEditor.prototype.setShaderVisible = function (visible) {
                    var _this = this;
                    if (ImageEditor.debug) {
                        console.log('setShaderVisible', visible);
                    }
                    var bodyMask = api.ui.mask.BodyMask.get();
                    if (visible) {
                        if (!this.maskClickListener) {
                            this.maskClickListener = function (event) {
                                if (ImageEditor.debug) {
                                    console.log('maskClickListener', event);
                                }
                                if (_this.isOutside(event)) {
                                    event.stopPropagation();
                                    event.preventDefault();
                                    if (_this.skipNextOutsideClick) {
                                        if (ImageEditor.debug) {
                                            console.log('maskClickListener, skipping mask click as requested earlier');
                                        }
                                        _this.skipNextOutsideClick = false;
                                        return;
                                    }
                                    if (_this.isCropEditMode()) {
                                        _this.disableCropEditMode();
                                    }
                                    else if (_this.isFocusEditMode()) {
                                        _this.disableFocusEditMode();
                                    }
                                    bodyMask.hide();
                                }
                            };
                        }
                        api.dom.Body.get().onClicked(this.maskClickListener);
                        if (!this.maskWheelListener) {
                            this.maskWheelListener = function (event) {
                                var el = _this.getEl(), win = api.dom.WindowDOM.get(), myHeight = el.getHeight(), myTop = el.getTopPx(), winHeight = win.getHeight();
                                var newTop = myTop - _this.normalizeWheel(event).pixelY;
                                var newTopLimited;
                                var heightLimit = _this.stickyToolbar.getEl().getHeight() + 100;
                                newTopLimited = Math.min(winHeight - heightLimit, Math.max(heightLimit - myHeight, newTop));
                                var isInsideLimit = newTop == newTopLimited;
                                if (!isInsideLimit && (Math.abs(newTop - newTopLimited) > Math.abs(myTop - newTopLimited))) {
                                    newTop = myTop;
                                }
                                else if (isInsideLimit) {
                                    newTop = newTopLimited;
                                }
                                else {
                                }
                                if (newTop != myTop) {
                                    el.setTopPx(newTop);
                                    _this.updateStickyToolbar();
                                }
                            };
                        }
                        api.dom.Body.get().onMouseWheel(this.maskWheelListener);
                        if (!this.maskHideListener) {
                            this.maskHideListener = function (event) {
                                api.dom.Body.get().unClicked(_this.maskClickListener);
                                api.dom.Body.get().unMouseWheel(_this.maskWheelListener);
                                bodyMask.unHidden(_this.maskHideListener);
                            };
                        }
                        bodyMask.onHidden(this.maskHideListener);
                        bodyMask.addClass('opaque').show();
                    }
                    else {
                        bodyMask.removeClass('opaque').hide();
                    }
                    this.notifyShaderVisibilityChanged(visible);
                };
                ImageEditor.prototype.normalizeWheel = function (event) {
                    var sX = 0, sY = 0, pX = 0, pY = 0;
                    if ('detail' in event) {
                        sY = event.detail;
                    }
                    if ('wheelDelta' in event) {
                        sY = -event.wheelDelta / 120;
                    }
                    if ('wheelDeltaY' in event) {
                        sY = -event.wheelDeltaY / 120;
                    }
                    if ('wheelDeltaX' in event) {
                        sX = -event.wheelDeltaX / 120;
                    }
                    if ('axis' in event && event.axis === event.HORIZONTAL_AXIS) {
                        sX = sY;
                        sY = 0;
                    }
                    pX = sX * this.WHEEL_PIXEL_STEP;
                    pY = sY * this.WHEEL_PIXEL_STEP;
                    if ('deltaY' in event) {
                        pY = event.deltaY;
                    }
                    if ('deltaX' in event) {
                        pX = event.deltaX;
                    }
                    if ((pX || pY) && event.deltaMode) {
                        if (event.deltaMode == 1) {
                            pX *= this.WHEEL_LINE_HEIGHT;
                            pY *= this.WHEEL_LINE_HEIGHT;
                        }
                        else {
                            pX *= this.WHEEL_PAGE_HEIGHT;
                            pY *= this.WHEEL_PAGE_HEIGHT;
                        }
                    }
                    if (pX && !sX) {
                        sX = (pX < 1) ? -1 : 1;
                    }
                    if (pY && !sY) {
                        sY = (pY < 1) ? -1 : 1;
                    }
                    return {
                        spinX: sX,
                        spinY: sY,
                        pixelX: pX,
                        pixelY: pY
                    };
                };
                ImageEditor.prototype.createStickyToolbar = function () {
                    var _this = this;
                    var toolbar = new DivEl('sticky-toolbar');
                    var editContainer = new DivEl('edit-container');
                    this.editResetButton = new Button('Reset');
                    this.editResetButton.setVisible(false).addClass('transparent').onClicked(function (event) {
                        event.stopPropagation();
                        if (_this.isFocusEditMode()) {
                            _this.resetFocusPosition();
                            _this.resetFocusRadius();
                        }
                        else if (_this.isCropEditMode()) {
                            _this.resetZoomPosition();
                            _this.resetCropPosition();
                        }
                    });
                    var applyButton = new Button('Apply');
                    applyButton.addClass('blue').onClicked(function (event) {
                        event.stopPropagation();
                        if (_this.isCropEditMode()) {
                            _this.disableCropEditMode();
                        }
                        else if (_this.isFocusEditMode()) {
                            _this.disableFocusEditMode();
                        }
                    });
                    var cancelButton = new api.ui.button.CloseButton();
                    cancelButton.onClicked(function (event) {
                        event.stopPropagation();
                        if (_this.isCropEditMode()) {
                            _this.disableCropEditMode(false);
                        }
                        else if (_this.isFocusEditMode()) {
                            _this.disableFocusEditMode(false);
                        }
                    });
                    editContainer.appendChildren(this.editResetButton, applyButton, cancelButton);
                    var standbyContainer = new DivEl('standby-container');
                    var resetButton = new Button('Reset');
                    resetButton.addClass('button-reset red').setVisible(false).onClicked(function (event) {
                        event.stopPropagation();
                        _this.resetCropPosition();
                        _this.resetZoomPosition();
                        _this.resetFocusPosition();
                        _this.resetFocusRadius();
                    });
                    this.onFocusAutoPositionedChanged(function (auto) {
                        _this.editResetButton.setVisible(!auto);
                        resetButton.setVisible(!auto || !_this.cropData.auto);
                    });
                    this.onCropAutoPositionedChanged(function (auto) {
                        _this.editResetButton.setVisible(!auto);
                        resetButton.setVisible(!auto || !_this.focusData.auto);
                    });
                    this.uploadButton = new Button();
                    this.uploadButton.addClass('button-upload');
                    standbyContainer.appendChildren(resetButton, this.uploadButton);
                    this.editCropButton = new Button();
                    this.editCropButton.addClass('button-crop transparent icon-crop').onClicked(function (event) {
                        event.stopPropagation();
                        if (_this.isCropEditMode()) {
                            _this.disableCropEditMode();
                        }
                        else {
                            if (_this.isFocusEditMode()) {
                                _this.disableFocusEditMode(true, false);
                                _this.enableCropEditMode(true, false);
                            }
                            else {
                                _this.enableCropEditMode();
                            }
                        }
                    });
                    this.editFocusButton = new Button();
                    this.editFocusButton.addClass('button-focus transparent icon-center_focus_strong').onClicked(function (event) {
                        event.stopPropagation();
                        if (_this.isFocusEditMode()) {
                            _this.disableFocusEditMode();
                        }
                        else {
                            if (_this.isCropEditMode()) {
                                _this.disableCropEditMode(true, false);
                                _this.enableFocusEditMode(true, false);
                            }
                            else {
                                _this.enableFocusEditMode();
                            }
                        }
                    });
                    var rightContainer = new DivEl('right-container');
                    rightContainer.appendChildren(standbyContainer, editContainer);
                    var zoomContainer = this.createZoomContainer();
                    var topContainer = new DivEl('top-container');
                    topContainer.appendChildren(this.editCropButton, this.editFocusButton, rightContainer);
                    toolbar.appendChildren(topContainer, zoomContainer);
                    return toolbar;
                };
                ImageEditor.prototype.updateStickyToolbar = function () {
                    var relativeScrollTop = this.getRelativeScrollTop();
                    if (!this.isTopEdgeVisible(relativeScrollTop) && this.isBottomEdgeVisible(relativeScrollTop)) {
                        this.addClass("sticky-mode");
                        this.stickyToolbar.getEl().setTopPx(-relativeScrollTop);
                    }
                    else {
                        this.stickyToolbar.getEl().setTopPx(0);
                        this.removeClass("sticky-mode");
                    }
                };
                ImageEditor.prototype.createZoomContainer = function () {
                    this.zoomContainer = new DivEl('zoom-container');
                    this.zoomLine = new DivEl('zoom-line');
                    this.zoomKnob = new api.dom.SpanEl('zoom-knob');
                    this.zoomLine.appendChild(this.zoomKnob);
                    var zoomTitle = new api.dom.SpanEl('zoom-title');
                    zoomTitle.setHtml('Zoom');
                    this.zoomContainer.appendChildren(zoomTitle, this.zoomLine);
                    return this.zoomContainer;
                };
                ImageEditor.prototype.isTopEdgeVisible = function (relativeScrollTop) {
                    return relativeScrollTop > 0;
                };
                ImageEditor.prototype.isBottomEdgeVisible = function (relativeScrollTop) {
                    var stickyToolbarHeight = this.stickyToolbar.getEl().getHeight(), frameHeight = this.frame.getEl().getHeight(), totalHeight = this.getEl().getHeight();
                    var toolbarDelta = stickyToolbarHeight - (totalHeight - frameHeight);
                    return (this.getCropPositionPx().h + relativeScrollTop - toolbarDelta) > 0;
                };
                ImageEditor.prototype.getRelativeScrollTop = function () {
                    var scrollEl = wemjq(this.getHTMLElement()).closest(this.SCROLLABLE_SELECTOR), scrollElOffsetTop = scrollEl.length == 1 ? scrollEl.offset().top : 0, wizardToolbarHeight = !this.isEditMode() && scrollEl.length == 1 ? scrollEl.find(this.WIZARD_TOOLBAR_SELECTOR).innerHeight() : 0;
                    return this.getEl().getOffsetTop() - scrollElOffsetTop - wizardToolbarHeight;
                };
                ImageEditor.prototype.setEditMode = function (edit, applyChanges) {
                    if (applyChanges === void 0) { applyChanges = true; }
                    if (ImageEditor.debug) {
                        console.group('setEditMode');
                        console.log('edit=' + edit + ', applyChanges=' + applyChanges);
                    }
                    this.setShaderVisible(edit);
                    this.toggleClass('edit-mode', edit);
                    var crop, zoom, focus, radius;
                    if (edit) {
                        this.updateRevertCropData();
                        this.updateRevertZoomData();
                        this.updateRevertFocusData();
                        if (ImageEditor.debug) {
                            console.log('updated revert data');
                        }
                    }
                    else {
                        if (applyChanges) {
                            crop = this.getCropPosition();
                            zoom = this.getZoomPosition();
                            focus = this.getFocusPosition();
                            radius = this.getFocusRadius();
                            if (ImageEditor.debug) {
                                console.log('Applying changes: \nCrop', crop, '\nZoom', zoom, '\nFocus', focus, '\nRadius', radius);
                            }
                        }
                        else {
                            if (ImageEditor.debug) {
                                console.log('reverting focus to', this.revertFocusData);
                                console.log('reverting crop to', this.revertCropData);
                                console.log('reverting zoom to', this.revertZoomData);
                            }
                            this.setFocusPositionPx({ x: this.revertFocusData.x, y: this.revertFocusData.y }, false);
                            this.setFocusRadiusPx(this.revertFocusData.r, false);
                            this.setFocusAutoPositioned(this.revertFocusData.auto);
                            this.setZoomPositionPx(this.revertZoomData, false);
                            this.setCropPositionPx(this.revertCropData, false);
                            this.setCropAutoPositioned(this.revertCropData.auto);
                        }
                        this.revertFocusData = undefined;
                        this.revertCropData = undefined;
                        this.revertZoomData = undefined;
                    }
                    this.notifyEditModeChanged(edit, crop, zoom, focus);
                    this.updateStickyToolbar();
                    if (ImageEditor.debug) {
                        console.groupEnd();
                    }
                };
                ImageEditor.prototype.isEditMode = function () {
                    return this.hasClass('edit-mode');
                };
                ImageEditor.prototype.enableFocusEditMode = function (applyChanges, enterEditMode) {
                    if (applyChanges === void 0) { applyChanges = true; }
                    if (enterEditMode === void 0) { enterEditMode = true; }
                    if (ImageEditor.debug) {
                        console.log('enableFocusEditMode, applyChanges=' + applyChanges + ', enterEditMode=' + enterEditMode);
                    }
                    this.editResetButton.setLabel('Reset Autofocus').setVisible(!this.focusData.auto);
                    this.setFocusEditMode(true);
                    if (enterEditMode) {
                        this.setEditMode(true, applyChanges);
                    }
                    this.bindFocusMouseListeners();
                    this.updateFocusMaskPosition();
                };
                ImageEditor.prototype.disableFocusEditMode = function (applyChanges, exitEditMode) {
                    if (applyChanges === void 0) { applyChanges = true; }
                    if (exitEditMode === void 0) { exitEditMode = true; }
                    if (ImageEditor.debug) {
                        console.log('disableFocusEditMode, applyChanges=' + applyChanges + ', exitEditMode=' + exitEditMode);
                    }
                    this.unbindFocusMouseListeners();
                    this.setFocusEditMode(false);
                    if (exitEditMode) {
                        this.setEditMode(false, applyChanges);
                    }
                };
                ImageEditor.prototype.setFocusEditMode = function (edit) {
                    if (ImageEditor.debug) {
                        console.log('setFocusEditMode', edit);
                    }
                    this.toggleClass('edit-focus', edit);
                    if (edit) {
                        this.setImageClipPath(this.focusClipPath);
                    }
                };
                ImageEditor.prototype.isFocusEditMode = function () {
                    return this.hasClass('edit-focus');
                };
                ImageEditor.prototype.setFocusPosition = function (x, y) {
                    if (this.isImageLoaded()) {
                        this.setFocusPositionPx(this.denormalizePoint(x, y));
                    }
                    else {
                        this.revertFocusData = {
                            x: x,
                            y: y,
                            r: this.focusData.r,
                            auto: this.focusData.auto
                        };
                    }
                };
                ImageEditor.prototype.setFocusPositionPx = function (position, updateAuto) {
                    if (updateAuto === void 0) { updateAuto = true; }
                    var oldX = this.focusData.x, oldY = this.focusData.y;
                    if (ImageEditor.debug) {
                        console.group('ImageEditor.setFocusPositionPx');
                        console.log('Before restraining', position.x - oldX, position.y - oldY, position);
                    }
                    this.focusData.x = this.restrainFocusX(position.x);
                    this.focusData.y = this.restrainFocusY(position.y);
                    if (updateAuto) {
                        this.setFocusAutoPositioned(this.isFocusNotModified(this.focusData));
                    }
                    if (oldX != this.focusData.x || oldY != this.focusData.y) {
                        this.notifyFocusPositionChanged(this.focusData);
                        if (ImageEditor.debug) {
                            console.log('After restraining', this.focusData.x - oldX, this.focusData.y - oldY, this.focusData);
                        }
                        if (this.isImageLoaded()) {
                            this.updateFocusMaskPosition();
                        }
                    }
                    if (ImageEditor.debug) {
                        console.groupEnd();
                    }
                };
                ImageEditor.prototype.getFocusPosition = function () {
                    return this.normalizePoint(this.getFocusPositionPx());
                };
                ImageEditor.prototype.getFocusPositionPx = function () {
                    return {
                        x: this.focusData.x,
                        y: this.focusData.y
                    };
                };
                ImageEditor.prototype.resetFocusPosition = function () {
                    if (ImageEditor.debug) {
                        console.log('resetFocusPosition');
                    }
                    var denormalizedPoint = this.denormalizePoint(0.5, 0.5);
                    this.setFocusPositionPx({
                        x: denormalizedPoint.x,
                        y: denormalizedPoint.y
                    }, false);
                    this.setFocusAutoPositioned(true);
                };
                ImageEditor.prototype.resetFocusRadius = function () {
                    this.setFocusRadiusPx(this.denormalizeRadius(0.25), false);
                };
                ImageEditor.prototype.setFocusRadius = function (r) {
                    return this.setFocusRadiusPx(this.denormalizeRadius(r));
                };
                ImageEditor.prototype.setFocusRadiusPx = function (r, updateAuto) {
                    if (updateAuto === void 0) { updateAuto = true; }
                    var oldR = this.focusData.r;
                    if (ImageEditor.debug) {
                        console.group('ImageEditor.setFocusRadiusPx');
                        console.log('Before restraining', r - oldR, r);
                    }
                    this.focusData.r = this.restrainFocusRadius(r);
                    if (updateAuto) {
                        this.setFocusAutoPositioned(this.isFocusNotModified(this.focusData));
                    }
                    if (oldR != this.focusData.r) {
                        this.notifyFocusRadiusChanged(this.focusData.r);
                        if (ImageEditor.debug) {
                            console.log('After restraining', this.focusData.r - oldR, this.focusData);
                        }
                        if (this.isImageLoaded()) {
                            this.updateFocusMaskPosition();
                        }
                    }
                    if (ImageEditor.debug) {
                        console.groupEnd();
                    }
                };
                ImageEditor.prototype.getFocusRadius = function () {
                    return this.normalizeRadius(this.getFocusRadiusPx());
                };
                ImageEditor.prototype.getFocusRadiusPx = function () {
                    return this.focusData.r;
                };
                ImageEditor.prototype.setFocusAutoPositioned = function (auto) {
                    if (ImageEditor.debug) {
                        console.log('setFocusAutoPositioned', auto);
                    }
                    var autoChanged = this.focusData.auto != auto;
                    this.focusData.auto = auto;
                    this.toggleClass('focused', !auto);
                    if (autoChanged) {
                        this.notifyFocusAutoPositionedChanged(auto);
                    }
                };
                ImageEditor.prototype.bindFocusMouseListeners = function () {
                    var _this = this;
                    var mouseDown = false;
                    var lastPos;
                    if (ImageEditor.debug) {
                        console.log('ImageEditor.bindFocusMouseListeners');
                    }
                    var mouseDownOriginalTarget;
                    this.mouseDownListener = function (event) {
                        if (ImageEditor.debug) {
                            console.log('ImageEditor.mouseDownListener');
                        }
                        mouseDownOriginalTarget = event['originalTarget'];
                        mouseDown = true;
                        lastPos = {
                            x: _this.getOffsetX(event),
                            y: _this.getOffsetY(event)
                        };
                    };
                    this.clip.onMouseDown(this.mouseDownListener);
                    this.mouseMoveListener = function (event) {
                        if (mouseDown) {
                            if (ImageEditor.debug) {
                                console.log('ImageEditor.mouseMoveListener');
                            }
                            var restrainedPos = {
                                x: _this.restrainFocusX(_this.focusData.x + _this.getOffsetX(event) - lastPos.x),
                                y: _this.restrainFocusY(_this.focusData.y + _this.getOffsetY(event) - lastPos.y)
                            };
                            _this.setFocusPositionPx(restrainedPos);
                            lastPos = restrainedPos;
                        }
                    };
                    api.dom.Body.get().onMouseMove(this.mouseMoveListener);
                    this.mouseUpListener = function (event) {
                        if (mouseDown) {
                            if (ImageEditor.debug) {
                                console.log('ImageEditor.mouseUpListener');
                            }
                            if (_this.isOutside(event) && mouseDownOriginalTarget == event['originalTarget']) {
                                if (ImageEditor.debug) {
                                    console.log('mouseUpListener, set to skip next click');
                                }
                                _this.skipNextOutsideClick = true;
                            }
                            var restrainedPos = {
                                x: _this.restrainFocusX(_this.getOffsetX(event)),
                                y: _this.restrainFocusY(_this.getOffsetY(event))
                            };
                            _this.setFocusPositionPx(restrainedPos);
                            mouseDown = false;
                        }
                    };
                    api.dom.Body.get().onMouseUp(this.mouseUpListener);
                };
                ImageEditor.prototype.unbindFocusMouseListeners = function () {
                    if (ImageEditor.debug) {
                        console.log('ImageEditor.unbindFocusMouseListeners');
                    }
                    api.dom.Body.get().unMouseMove(this.mouseMoveListener);
                    api.dom.Body.get().unMouseUp(this.mouseUpListener);
                };
                ImageEditor.prototype.updateFocusMaskPosition = function () {
                    var clipCircle = this.focusClipPath.getHTMLElement().querySelector('circle'), strokeCircle = this.clip.getHTMLElement().querySelector('.focus-group circle');
                    if (ImageEditor.debug) {
                        console.log('ImageEditor.updateFocusPosition', this.focusData);
                    }
                    var circles = [clipCircle, strokeCircle];
                    for (var i = 0; i < circles.length; i++) {
                        var circle = circles[i];
                        circle.setAttribute('r', this.focusData.r.toString());
                        circle.setAttribute('cx', (this.cropData.x + this.focusData.x).toString());
                        circle.setAttribute('cy', (this.cropData.y + this.focusData.y).toString());
                    }
                };
                ImageEditor.prototype.restrainFocusX = function (x) {
                    return Math.max(0, Math.min(this.cropData.w, x));
                };
                ImageEditor.prototype.restrainFocusY = function (y) {
                    return Math.max(0, Math.min(this.cropData.h, y));
                };
                ImageEditor.prototype.restrainFocusRadius = function (r) {
                    return Math.max(0, Math.min(this.cropData.w / 4, this.cropData.h / 4, r));
                };
                ImageEditor.prototype.isFocusNotModified = function (focus) {
                    return this.isFocusPositionNotModified(focus) && this.isFocusRadiusNotModified(focus.r);
                };
                ImageEditor.prototype.isFocusPositionNotModified = function (focus) {
                    return focus.x == Math.min(this.frameW, this.cropData.w) / 2 && focus.y == Math.min(this.frameH, this.cropData.h) / 2;
                };
                ImageEditor.prototype.isFocusRadiusNotModified = function (r) {
                    return r == Math.min(this.cropData.w, this.cropData.h) / 4;
                };
                ImageEditor.prototype.enableCropEditMode = function (applyChanges, enterEditMode) {
                    if (applyChanges === void 0) { applyChanges = true; }
                    if (enterEditMode === void 0) { enterEditMode = true; }
                    if (ImageEditor.debug) {
                        console.log('enableCropEditMode, applyChanges=' + applyChanges + ', enterEditMode=' + enterEditMode);
                    }
                    this.editResetButton.setLabel('Reset Mask').setVisible(!this.cropData.auto);
                    this.setCropEditMode(true);
                    if (enterEditMode) {
                        this.setEditMode(true, applyChanges);
                    }
                    this.bindCropMouseListeners();
                    this.updateCropMaskPosition();
                    this.updateZoomPosition();
                };
                ImageEditor.prototype.disableCropEditMode = function (applyChanges, exitEditMode) {
                    if (applyChanges === void 0) { applyChanges = true; }
                    if (exitEditMode === void 0) { exitEditMode = true; }
                    if (ImageEditor.debug) {
                        console.log('disableCropEditMode, applyChanges=' + applyChanges + ', exitEditMode=' + exitEditMode);
                    }
                    this.unbindCropMouseListeners();
                    this.setCropEditMode(false);
                    if (exitEditMode) {
                        this.setEditMode(false, applyChanges);
                    }
                };
                ImageEditor.prototype.setCropEditMode = function (edit) {
                    if (ImageEditor.debug) {
                        console.log('setCropEditMode, edit=' + edit);
                    }
                    this.toggleClass('edit-crop', edit);
                    if (edit) {
                        this.setImageClipPath(this.cropClipPath);
                    }
                    else {
                        this.resetFocusRadius();
                        if (this.focusData.auto) {
                            this.resetFocusPosition();
                        }
                        else {
                            this.setFocusPositionPx(this.focusData, false);
                        }
                    }
                };
                ImageEditor.prototype.isCropEditMode = function () {
                    return this.hasClass('edit-crop');
                };
                ImageEditor.prototype.setCropPosition = function (x, y, x2, y2) {
                    var svg = this.rectToSVG(x, y, x2, y2);
                    if (this.isImageLoaded()) {
                        this.setCropPositionPx(this.denormalizeRect(svg));
                    }
                    else {
                        this.revertCropData = {
                            x: svg.x,
                            y: svg.y,
                            w: svg.w,
                            h: svg.h,
                            auto: this.cropData.auto
                        };
                    }
                };
                ImageEditor.prototype.setCropPositionPx = function (crop, updateAuto) {
                    if (updateAuto === void 0) { updateAuto = true; }
                    var oldX = this.cropData.x, oldY = this.cropData.y, oldW = this.cropData.w, oldH = this.cropData.h;
                    if (ImageEditor.debug) {
                        console.group('ImageEditor.setCropPositionPx');
                        console.log('Before restraining', crop.x - oldX, crop.y - oldY, crop);
                    }
                    this.cropData.w = this.restrainCropW(crop.w);
                    this.cropData.h = this.restrainCropH(crop.h);
                    this.cropData.x = this.restrainCropX(crop.x);
                    this.cropData.y = this.restrainCropY(crop.y);
                    this.updateFrameHeight();
                    if (updateAuto) {
                        this.setCropAutoPositioned(this.isCropNotModified(this.cropData));
                    }
                    if (oldX != this.cropData.x || oldY != this.cropData.y || oldW != this.cropData.w || oldH != this.cropData.h) {
                        var dx = this.cropData.x - oldX, dy = this.cropData.y - oldY;
                        if (ImageEditor.debug) {
                            console.log('After restraining', dx, dy, this.cropData);
                        }
                        this.notifyCropPositionChanged(this.cropData);
                        if (this.isImageLoaded() && this.isCropEditMode()) {
                            this.updateCropMaskPosition();
                        }
                    }
                    if (ImageEditor.debug) {
                        console.groupEnd();
                    }
                };
                ImageEditor.prototype.getCropPosition = function () {
                    return this.rectFromSVG(this.normalizeRect(this.getCropPositionPx()));
                };
                ImageEditor.prototype.getCropPositionPx = function () {
                    return {
                        x: this.cropData.x,
                        y: this.cropData.y,
                        w: this.cropData.w,
                        h: this.cropData.h
                    };
                };
                ImageEditor.prototype.resetCropPosition = function () {
                    if (ImageEditor.debug) {
                        console.log('resetCropPosition');
                    }
                    var crop = { x: 0, y: 0, w: 1, h: 1 };
                    this.setCropPositionPx(this.denormalizeRect(crop), false);
                    this.setCropAutoPositioned(true);
                };
                ImageEditor.prototype.setCropAutoPositioned = function (auto) {
                    if (ImageEditor.debug) {
                        console.log('setCropAutoPositioned', auto);
                    }
                    var autoChanged = this.cropData.auto != auto;
                    this.cropData.auto = auto;
                    if (autoChanged) {
                        this.notifyCropAutoPositionedChanged(auto);
                    }
                };
                ImageEditor.prototype.updateCropMaskPosition = function () {
                    var rect = this.cropClipPath.getHTMLElement().querySelector('rect'), drag = this.dragHandle.getHTMLElement();
                    if (ImageEditor.debug) {
                        console.log('ImageEditor.updateCropPosition', this.cropData);
                    }
                    rect.setAttribute('x', this.cropData.x.toString());
                    rect.setAttribute('y', this.cropData.y.toString());
                    rect.setAttribute('width', this.cropData.w.toString());
                    rect.setAttribute('height', this.cropData.h.toString());
                    drag.setAttribute('x', (this.cropData.x + this.cropData.w / 2 - 16).toString());
                    drag.setAttribute('y', (this.cropData.y + this.cropData.h - 16).toString());
                };
                ImageEditor.prototype.isInsideCrop = function (x, y) {
                    return x >= this.zoomData.x + this.cropData.x && x <= (this.zoomData.x + this.cropData.x + this.cropData.w) && y >= this.zoomData.y + this.cropData.y && y <= (this.zoomData.y + this.cropData.y + this.cropData.h);
                };
                ImageEditor.prototype.bindCropMouseListeners = function () {
                    var _this = this;
                    var dragMouseDown = false, zoomMouseDown = false, panMouseDown = false;
                    var lastPos;
                    if (ImageEditor.debug) {
                        console.log('ImageEditor.bindCropMouseListeners');
                    }
                    var mouseDownOriginalTarget;
                    this.dragMouseDownListener = function (event) {
                        event.stopPropagation();
                        event.preventDefault();
                        if (ImageEditor.debug) {
                            console.group('ImageEditor.dragMouseListener');
                            console.log('mouse down', event);
                        }
                        mouseDownOriginalTarget = event['originalTarget'];
                        dragMouseDown = true;
                        lastPos = {
                            x: _this.getOffsetX(event),
                            y: _this.getOffsetY(event)
                        };
                        _this.dragHandle.addClass('active');
                    };
                    this.dragHandle.onMouseDown(this.dragMouseDownListener);
                    this.knobMouseDownListener = function (event) {
                        event.stopPropagation();
                        event.preventDefault();
                        if (ImageEditor.debug) {
                            console.log('ImageEditor.knobMouseListener', event);
                        }
                        mouseDownOriginalTarget = event['originalTarget'];
                        zoomMouseDown = true;
                        lastPos = {
                            x: _this.getOffsetX(event),
                            y: _this.getOffsetY(event)
                        };
                        _this.zoomContainer.addClass('active');
                    };
                    this.zoomKnob.onMouseDown(this.knobMouseDownListener);
                    this.mouseDownListener = function (event) {
                        event.stopPropagation();
                        event.preventDefault();
                        var x = _this.getOffsetX(event), y = _this.getOffsetY(event);
                        if (ImageEditor.debug) {
                            console.group('ImageEditor.mouseDownListener');
                            console.log('mouse down', event);
                        }
                        if (_this.isInsideCrop(x, y)) {
                            if (ImageEditor.debug) {
                                console.log('click inside crop area');
                            }
                            mouseDownOriginalTarget = event['originalTarget'];
                            panMouseDown = true;
                            lastPos = {
                                x: x,
                                y: y
                            };
                        }
                        else {
                            if (ImageEditor.debug) {
                                console.log('click outside crop area');
                            }
                        }
                    };
                    this.clip.onMouseDown(this.mouseDownListener);
                    this.mouseMoveListener = function (event) {
                        var currPos = {
                            x: _this.getOffsetX(event),
                            y: _this.getOffsetY(event)
                        };
                        if (!_this.isInsideZoom(currPos.x, currPos.y)) {
                            return;
                        }
                        if (zoomMouseDown) {
                            _this.moveZoomKnobByPx(currPos.x - lastPos.x);
                        }
                        else if (dragMouseDown) {
                            var deltaY = _this.getOffsetY(event) - lastPos.y, distBetweenCropAndZoomBottoms = _this.zoomData.h - _this.cropData.h - _this.cropData.y, newH = _this.cropData.h + (deltaY > distBetweenCropAndZoomBottoms ? distBetweenCropAndZoomBottoms : deltaY);
                            if (newH > 0 && newH != _this.cropData.h) {
                                _this.setCropPositionPx({
                                    x: _this.cropData.x,
                                    y: _this.cropData.y,
                                    w: _this.cropData.w,
                                    h: newH
                                });
                            }
                        }
                        else if (panMouseDown) {
                            _this.setZoomPositionPx({
                                x: _this.zoomData.x + currPos.x - lastPos.x,
                                y: _this.zoomData.y + currPos.y - lastPos.y,
                                w: _this.zoomData.w,
                                h: _this.zoomData.h
                            });
                        }
                        lastPos = currPos;
                    };
                    api.dom.Body.get().onMouseMove(this.mouseMoveListener);
                    this.mouseUpListener = function (event) {
                        if (ImageEditor.debug) {
                            console.log('mouse up', event);
                        }
                        if (_this.isOutside(event) && mouseDownOriginalTarget == event['originalTarget'] && (dragMouseDown || zoomMouseDown || panMouseDown)) {
                            if (ImageEditor.debug) {
                                console.log('mouseUpListener, set to skip next click');
                            }
                            _this.skipNextOutsideClick = true;
                        }
                        if (dragMouseDown) {
                            dragMouseDown = false;
                            _this.dragHandle.removeClass('active');
                        }
                        else if (zoomMouseDown) {
                            zoomMouseDown = false;
                            _this.zoomContainer.removeClass('active');
                        }
                        else if (panMouseDown) {
                            panMouseDown = false;
                        }
                        if (ImageEditor.debug) {
                            console.groupEnd();
                        }
                    };
                    api.dom.Body.get().onMouseUp(this.mouseUpListener);
                };
                ImageEditor.prototype.unbindCropMouseListeners = function () {
                    if (ImageEditor.debug) {
                        console.log('ImageEditor.unbindCropMouseListeners');
                    }
                    this.dragHandle.unMouseDown(this.dragMouseDownListener);
                    this.zoomKnob.unMouseDown(this.knobMouseDownListener);
                    this.clip.unMouseDown(this.mouseDownListener);
                    api.dom.Body.get().unMouseMove(this.mouseMoveListener);
                    api.dom.Body.get().unMouseUp(this.mouseUpListener);
                };
                ImageEditor.prototype.restrainCropX = function (x) {
                    return Math.max(0, Math.min(this.zoomData.w - this.cropData.w, x));
                };
                ImageEditor.prototype.restrainCropY = function (y) {
                    return Math.max(0, Math.min(this.zoomData.h - this.cropData.h, y));
                };
                ImageEditor.prototype.restrainCropW = function (x) {
                    return Math.max(0, Math.min(this.zoomData.w, x));
                };
                ImageEditor.prototype.restrainCropH = function (y) {
                    return Math.max(0, Math.min(this.zoomData.h, y));
                };
                ImageEditor.prototype.rectFromSVG = function (svg) {
                    return svg ? {
                        x: svg.x,
                        y: svg.y,
                        x2: svg.x + svg.w,
                        y2: svg.y + svg.h
                    } : undefined;
                };
                ImageEditor.prototype.rectToSVG = function (x, y, x2, y2) {
                    return {
                        x: x,
                        y: y,
                        w: x2 - x,
                        h: y2 - y
                    };
                };
                ImageEditor.prototype.isCropNotModified = function (rect) {
                    return rect.x == 0 && rect.y == 0 && rect.w == this.frameW && rect.h == this.frameH;
                };
                ImageEditor.prototype.setZoomPosition = function (x, y, x2, y2) {
                    var zoom = this.rectToSVG(x, y, x2, y2);
                    if (this.isImageLoaded()) {
                        this.setZoomPositionPx(this.denormalizeRect(zoom));
                    }
                    else {
                        this.revertZoomData = {
                            x: zoom.x,
                            y: zoom.y,
                            w: zoom.w,
                            h: zoom.h
                        };
                    }
                };
                ImageEditor.prototype.setZoomPositionPx = function (zoom, updateAuto) {
                    if (updateAuto === void 0) { updateAuto = true; }
                    var oldX = this.zoomData.x, oldY = this.zoomData.y, oldW = this.zoomData.w, oldH = this.zoomData.h;
                    if (ImageEditor.debug) {
                        console.group('ImageEditor.setZoomPositionPx');
                        console.log('Before restraining', zoom.x - oldX, zoom.y - oldY, zoom);
                    }
                    this.zoomData.w = this.restrainZoomW(zoom.w);
                    this.zoomData.h = this.restrainZoomH(zoom.h);
                    this.zoomData.x = this.restrainZoomX(zoom.x);
                    this.zoomData.y = this.restrainZoomY(zoom.y);
                    if (oldX != this.zoomData.x || oldY != this.zoomData.y || oldW != this.zoomData.w || oldH != this.zoomData.h) {
                        var dx = this.zoomData.x - oldX, dy = this.zoomData.y - oldY;
                        if (ImageEditor.debug) {
                            console.log('After restraining', dx, dy, this.zoomData);
                        }
                        if (this.isImageLoaded()) {
                            this.updateZoomPosition();
                        }
                        this.setCropPositionPx({
                            x: this.cropData.x - dx,
                            y: this.cropData.y - dy,
                            w: this.cropData.w,
                            h: this.cropData.h
                        }, updateAuto);
                    }
                    if (ImageEditor.debug) {
                        console.groupEnd();
                    }
                };
                ImageEditor.prototype.getZoomPosition = function () {
                    return this.rectFromSVG(this.normalizeRect(this.getZoomPositionPx()));
                };
                ImageEditor.prototype.getZoomPositionPx = function () {
                    return {
                        x: this.zoomData.x,
                        y: this.zoomData.y,
                        w: this.zoomData.w,
                        h: this.zoomData.h
                    };
                };
                ImageEditor.prototype.resetZoomPosition = function () {
                    if (ImageEditor.debug) {
                        console.log('resetZoomPosition');
                    }
                    var zoom = { x: 0, y: 0, w: 1, h: 1 };
                    this.setZoomPositionPx(this.denormalizeRect(zoom), false);
                    this.setCropAutoPositioned(true);
                };
                ImageEditor.prototype.isInsideZoom = function (x, y) {
                    return x >= this.zoomData.x && x <= (this.zoomData.x + this.zoomData.w) && y >= this.zoomData.y && y <= (this.zoomData.y + this.zoomData.h);
                };
                ImageEditor.prototype.moveZoomKnobByPx = function (delta) {
                    var zoomLineEl = this.zoomLine.getEl(), zoomKnobEl = this.zoomKnob.getEl();
                    var sliderLength = zoomLineEl.getWidth(), knobX = zoomKnobEl.getLeftPx() || 0, knobNewX = Math.max(0, Math.min(sliderLength, knobX + delta));
                    if (knobNewX != knobX) {
                        zoomKnobEl.setLeftPx(knobNewX);
                        var knobPct = knobNewX / sliderLength, zoomCoeff = 1 + knobPct * (this.maxZoom - 1), newW = this.restrainZoomW(this.frameW * zoomCoeff), newH = this.restrainZoomH(this.frameH * zoomCoeff), newX = this.zoomData.x - (newW - this.zoomData.w) / 2, newY = this.zoomData.y - (newH - this.zoomData.h) / 2;
                        this.setZoomPositionPx({
                            x: newX,
                            y: newY,
                            w: newW,
                            h: newH
                        });
                    }
                };
                ImageEditor.prototype.updateZoomPosition = function () {
                    if (ImageEditor.debug) {
                        console.log('ImageEditor.updateZoomPosition', this.zoomData);
                    }
                    this.canvas.getEl().setWidthPx(this.zoomData.w).setHeightPx(this.zoomData.h).setLeftPx(this.zoomData.x).setTopPx(this.zoomData.y);
                    var zoomKnobEl = this.zoomKnob.getEl(), zoomLineEl = this.zoomLine.getEl();
                    var sliderLength = zoomLineEl.getWidth(), knobPct = (this.zoomData.w / this.frameW - 1) / (this.maxZoom - 1), knobNewX = Math.max(0, Math.min(sliderLength, knobPct * sliderLength));
                    zoomKnobEl.setLeftPx(knobNewX);
                };
                ImageEditor.prototype.updateRevertCropData = function () {
                    var cropPosition = this.getCropPositionPx();
                    this.revertCropData = {
                        x: cropPosition.x,
                        y: cropPosition.y,
                        w: cropPosition.w,
                        h: cropPosition.h,
                        auto: this.cropData.auto
                    };
                };
                ImageEditor.prototype.updateRevertZoomData = function () {
                    var zoomPosition = this.getZoomPositionPx();
                    this.revertZoomData = {
                        x: zoomPosition.x,
                        y: zoomPosition.y,
                        w: zoomPosition.w,
                        h: zoomPosition.h
                    };
                };
                ImageEditor.prototype.updateRevertFocusData = function () {
                    var focusPosition = this.getFocusPositionPx();
                    this.revertFocusData = {
                        x: focusPosition.x,
                        y: focusPosition.y,
                        r: this.getFocusRadiusPx(),
                        auto: this.focusData.auto
                    };
                };
                ImageEditor.prototype.restrainZoomX = function (x) {
                    return Math.max(Math.min(this.frameW, this.cropData.w) - this.zoomData.w, Math.min(0, x));
                };
                ImageEditor.prototype.restrainZoomY = function (y) {
                    return Math.max(Math.min(this.frameH, this.cropData.h) - this.zoomData.h, Math.min(0, y));
                };
                ImageEditor.prototype.restrainZoomW = function (x) {
                    return Math.max(Math.min(this.frameW, this.cropData.w), Math.min(this.maxZoom * this.frameW, x));
                };
                ImageEditor.prototype.restrainZoomH = function (y) {
                    return Math.max(Math.min(this.frameH, this.cropData.h), Math.min(this.maxZoom * this.frameH, y));
                };
                ImageEditor.prototype.onEditModeChanged = function (listener) {
                    this.editModeListeners.push(listener);
                };
                ImageEditor.prototype.unEditModeChanged = function (listener) {
                    this.editModeListeners = this.editModeListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                ImageEditor.prototype.notifyEditModeChanged = function (edit, position, zoom, focus) {
                    this.editModeListeners.forEach(function (listener) {
                        listener(edit, position, zoom, focus);
                    });
                };
                ImageEditor.prototype.onFocusAutoPositionedChanged = function (listener) {
                    this.autoFocusChangedListeners.push(listener);
                };
                ImageEditor.prototype.unFocusAutoPositionedChanged = function (listener) {
                    this.autoFocusChangedListeners = this.autoFocusChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                ImageEditor.prototype.notifyFocusAutoPositionedChanged = function (auto) {
                    this.autoFocusChangedListeners.forEach(function (listener) {
                        listener(auto);
                    });
                };
                ImageEditor.prototype.onFocusPositionChanged = function (listener) {
                    this.focusPositionChangedListeners.push(listener);
                };
                ImageEditor.prototype.unFocusPositionChanged = function (listener) {
                    this.focusPositionChangedListeners = this.focusPositionChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                ImageEditor.prototype.notifyFocusPositionChanged = function (position) {
                    var normalizedPosition = this.normalizePoint(position);
                    this.focusPositionChangedListeners.forEach(function (listener) {
                        listener(normalizedPosition);
                    });
                };
                ImageEditor.prototype.onFocusRadiusChanged = function (listener) {
                    this.focusRadiusChangedListeners.push(listener);
                };
                ImageEditor.prototype.unFocusRadiusChanged = function (listener) {
                    this.focusRadiusChangedListeners = this.focusRadiusChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                ImageEditor.prototype.notifyFocusRadiusChanged = function (r) {
                    var normalizedRadius = this.normalizeRadius(r);
                    this.focusRadiusChangedListeners.forEach(function (listener) {
                        listener(normalizedRadius);
                    });
                };
                ImageEditor.prototype.onCropAutoPositionedChanged = function (listener) {
                    this.autoCropChangedListeners.push(listener);
                };
                ImageEditor.prototype.unCropAutoPositionedChanged = function (listener) {
                    this.autoCropChangedListeners = this.autoCropChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                ImageEditor.prototype.notifyCropAutoPositionedChanged = function (auto) {
                    this.autoCropChangedListeners.forEach(function (listener) {
                        listener(auto);
                    });
                };
                ImageEditor.prototype.onCropPositionChanged = function (listener) {
                    this.cropPositionChangedListeners.push(listener);
                };
                ImageEditor.prototype.unCropPositionChanged = function (listener) {
                    this.cropPositionChangedListeners = this.cropPositionChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                ImageEditor.prototype.notifyCropPositionChanged = function (position) {
                    var normalizedPosition = this.rectFromSVG(this.normalizeRect(position));
                    this.cropPositionChangedListeners.forEach(function (listener) {
                        listener(normalizedPosition);
                    });
                };
                ImageEditor.prototype.onShaderVisibilityChanged = function (listener) {
                    this.shaderVisibilityChangedListeners.push(listener);
                };
                ImageEditor.prototype.unShaderVisibilityChanged = function (listener) {
                    this.shaderVisibilityChangedListeners = this.shaderVisibilityChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                ImageEditor.prototype.notifyShaderVisibilityChanged = function (auto) {
                    this.shaderVisibilityChangedListeners.forEach(function (listener) {
                        listener(auto);
                    });
                };
                ImageEditor.debug = false;
                return ImageEditor;
            })(api.dom.DivEl);
            _image.ImageEditor = ImageEditor;
        })(image = ui.image || (ui.image = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var image;
        (function (image) {
            var LazyImage = (function (_super) {
                __extends(LazyImage, _super);
                function LazyImage(src) {
                    var _this = this;
                    _super.call(this, "lazy-image");
                    this.addClass("empty");
                    this.phantomImage = new api.dom.ImgEl(null, "phantom-image");
                    this.phantomImage.onLoaded(function () {
                        _this.getEl().setBackgroundImage("url(" + _this.phantomImage.getSrc() + ")");
                        _this.removeClass("empty");
                    });
                    this.setSrc(src);
                }
                LazyImage.prototype.setSrc = function (src) {
                    if (!this.hasClass("empty")) {
                        this.addClass("empty");
                    }
                    this.phantomImage.setSrc(src);
                };
                return LazyImage;
            })(api.dom.DivEl);
            image.LazyImage = LazyImage;
        })(image = ui.image || (ui.image = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var PrincipalType = api.security.PrincipalType;
            var PrincipalViewer = (function (_super) {
                __extends(PrincipalViewer, _super);
                function PrincipalViewer() {
                    _super.call(this);
                    this.removeClickedListeners = [];
                }
                PrincipalViewer.prototype.resolveDisplayName = function (object) {
                    return object.getDisplayName();
                };
                PrincipalViewer.prototype.resolveUnnamedDisplayName = function (object) {
                    return object.getTypeName();
                };
                PrincipalViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getKey().toPath();
                };
                PrincipalViewer.prototype.resolveIconClass = function (object) {
                    switch (object.getKey().getType()) {
                        case 0 /* USER */:
                            return "icon-user";
                        case 1 /* GROUP */:
                            return "icon-users";
                        case 2 /* ROLE */:
                            return "icon-shield";
                    }
                    return "";
                };
                PrincipalViewer.prototype.onRemoveClicked = function (listener) {
                    this.removeClickedListeners.push(listener);
                };
                PrincipalViewer.prototype.unRemoveClicked = function (listener) {
                    this.removeClickedListeners = this.removeClickedListeners.filter(function (current) {
                        return current !== listener;
                    });
                };
                PrincipalViewer.prototype.notifyRemoveClicked = function (event) {
                    this.removeClickedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return PrincipalViewer;
            })(api.ui.NamesAndIconViewer);
            security.PrincipalViewer = PrincipalViewer;
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var PrincipalLoader = api.security.PrincipalLoader;
            var PrincipalComboBox = (function (_super) {
                __extends(PrincipalComboBox, _super);
                function PrincipalComboBox(loader, max) {
                    var builder = new api.ui.selector.combobox.RichComboBoxBuilder().setMaximumOccurrences(max || 0).setComboBoxName("principalSelector").setIdentifierMethod("getKey").setLoader(loader || new PrincipalLoader()).setSelectedOptionsView(new PrincipalSelectedOptionsView()).setOptionDisplayValueViewer(new security.PrincipalViewer()).setDelayedInputValueChangedHandling(500);
                    _super.call(this, builder);
                }
                return PrincipalComboBox;
            })(api.ui.selector.combobox.RichComboBox);
            security.PrincipalComboBox = PrincipalComboBox;
            var PrincipalSelectedOptionView = (function (_super) {
                __extends(PrincipalSelectedOptionView, _super);
                function PrincipalSelectedOptionView(option) {
                    var _this = this;
                    _super.call(this);
                    this.setOption(option);
                    this.setClass("principal-selected-option-view");
                    var removeButton = new api.dom.AEl("icon-close");
                    removeButton.onClicked(function (event) { return _this.notifyRemoveClicked(event); });
                    this.appendChild(removeButton);
                }
                PrincipalSelectedOptionView.prototype.setOption = function (option) {
                    this.option = option;
                    this.setObject(option.displayValue);
                };
                PrincipalSelectedOptionView.prototype.getOption = function () {
                    return this.option;
                };
                PrincipalSelectedOptionView.prototype.onSelectedOptionRemoveRequest = function (listener) {
                    this.onRemoveClicked(listener);
                };
                PrincipalSelectedOptionView.prototype.unSelectedOptionRemoveRequest = function (listener) {
                    this.unRemoveClicked(listener);
                };
                return PrincipalSelectedOptionView;
            })(security.PrincipalViewer);
            var PrincipalSelectedOptionsView = (function (_super) {
                __extends(PrincipalSelectedOptionsView, _super);
                function PrincipalSelectedOptionsView() {
                    _super.call(this, "principal-selected-options-view");
                }
                PrincipalSelectedOptionsView.prototype.createSelectedOption = function (option) {
                    var optionView = new PrincipalSelectedOptionView(option);
                    return new api.ui.selector.combobox.SelectedOption(optionView, this.count());
                };
                return PrincipalSelectedOptionsView;
            })(api.ui.selector.combobox.BaseSelectedOptionsView);
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var TabMenuItemBuilder = api.ui.tab.TabMenuItemBuilder;
                (function (Access) {
                    Access[Access["READ"] = 0] = "READ";
                    Access[Access["WRITE"] = 1] = "WRITE";
                    Access[Access["PUBLISH"] = 2] = "PUBLISH";
                    Access[Access["FULL"] = 3] = "FULL";
                    Access[Access["CUSTOM"] = 4] = "CUSTOM";
                })(acl.Access || (acl.Access = {}));
                var Access = acl.Access;
                var AccessSelector = (function (_super) {
                    __extends(AccessSelector, _super);
                    function AccessSelector() {
                        var _this = this;
                        _super.call(this, "access-selector");
                        this.valueChangedListeners = [];
                        AccessSelector.OPTIONS.forEach(function (option, index) {
                            var menuItem = new TabMenuItemBuilder().setLabel(option.name).setAddLabelTitleAttribute(false).build();
                            _this.addNavigationItem(menuItem);
                        });
                        this.onNavigationItemSelected(function (event) {
                            var item = event.getItem();
                            _this.setValue(AccessSelector.OPTIONS[item.getIndex()].value);
                        });
                    }
                    AccessSelector.prototype.getValue = function () {
                        return this.value;
                    };
                    AccessSelector.prototype.setValue = function (value, silent) {
                        var option = this.findOptionByValue(value);
                        if (option) {
                            this.selectNavigationItem(AccessSelector.OPTIONS.indexOf(option));
                            if (!silent) {
                                this.notifyValueChanged(new api.ui.ValueChangedEvent(Access[this.value], Access[value]));
                            }
                            this.value = value;
                        }
                        return this;
                    };
                    AccessSelector.prototype.setButtonLabel = function (value) {
                        this.getTabMenuButtonEl().setLabel(value, false);
                        return this;
                    };
                    AccessSelector.prototype.findOptionByValue = function (value) {
                        for (var i = 0; i < AccessSelector.OPTIONS.length; i++) {
                            var option = AccessSelector.OPTIONS[i];
                            if (option.value == value) {
                                return option;
                            }
                        }
                        return undefined;
                    };
                    AccessSelector.prototype.showMenu = function () {
                        var menu = this.getMenuEl(), button = this.getTabMenuButtonEl(), entry = menu.getParentElement().getParentElement(), list = entry.getParentElement(), offset = entry.getEl().getOffsetTopRelativeToParent() - (list.getEl().getOffsetTopRelativeToParent() + list.getEl().getPaddingTop() + list.getEl().getScrollTop()), height = menu.getEl().getHeightWithoutPadding() - button.getEl().getHeight() + 2;
                        if (offset > height) {
                            menu.addClass("upward");
                        }
                        else {
                            menu.removeClass("upward");
                        }
                        _super.prototype.showMenu.call(this);
                    };
                    AccessSelector.prototype.onValueChanged = function (listener) {
                        this.valueChangedListeners.push(listener);
                    };
                    AccessSelector.prototype.unValueChanged = function (listener) {
                        this.valueChangedListeners = this.valueChangedListeners.filter(function (curr) {
                            return curr !== listener;
                        });
                    };
                    AccessSelector.prototype.notifyValueChanged = function (event) {
                        this.valueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    AccessSelector.OPTIONS = [
                        { value: 0 /* READ */, name: 'Can Read' },
                        { value: 1 /* WRITE */, name: 'Can Write' },
                        { value: 2 /* PUBLISH */, name: 'Can Publish' },
                        { value: 3 /* FULL */, name: 'Full Access' },
                        { value: 4 /* CUSTOM */, name: 'Custom...' }
                    ];
                    return AccessSelector;
                })(api.ui.tab.TabMenu);
                acl.AccessSelector = AccessSelector;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var Permission = api.security.acl.Permission;
                var PermissionState = api.security.acl.PermissionState;
                var PermissionSelector = (function (_super) {
                    __extends(PermissionSelector, _super);
                    function PermissionSelector() {
                        var _this = this;
                        _super.call(this, 'permission-selector');
                        this.toggles = [];
                        this.valueChangedListeners = [];
                        this.enabled = true;
                        PermissionSelector.OPTIONS.forEach(function (option) {
                            var toggle = new PermissionToggle(option);
                            toggle.setEnabled(_this.enabled);
                            toggle.onValueChanged(function (event) {
                                var newValue = _this.getValue();
                                _this.notifyValueChanged(new api.ui.ValueChangedEvent(JSON.stringify(_this.oldValue), JSON.stringify(newValue)));
                                _this.oldValue = newValue;
                            });
                            _this.toggles.push(toggle);
                            _this.appendChild(toggle);
                        });
                    }
                    PermissionSelector.prototype.setEnabled = function (enabled) {
                        if (enabled != this.enabled) {
                            this.toggleClass('disabled', !enabled);
                            this.toggles.forEach(function (toggle) {
                                toggle.setEnabled(enabled);
                            });
                            this.enabled = enabled;
                        }
                        return this;
                    };
                    PermissionSelector.prototype.isEnabled = function () {
                        return this.enabled;
                    };
                    PermissionSelector.prototype.getValue = function () {
                        var values = {
                            allow: [],
                            deny: []
                        };
                        this.toggles.forEach(function (toggle) {
                            switch (toggle.getState()) {
                                case 0 /* ALLOW */:
                                    values.allow.push(toggle.getValue());
                                    break;
                                case 1 /* DENY */:
                                    values.deny.push(toggle.getValue());
                                    break;
                            }
                        });
                        return values;
                    };
                    PermissionSelector.prototype.setValue = function (newValue, silent) {
                        this.toggles.forEach(function (toggle) {
                            var value = toggle.getValue();
                            var state;
                            if (newValue.allow.indexOf(value) >= 0) {
                                state = 0 /* ALLOW */;
                            }
                            else if (newValue.deny.indexOf(value) >= 0) {
                                state = 1 /* DENY */;
                            }
                            else {
                                state = 2 /* INHERIT */;
                            }
                            toggle.setState(state, true);
                        });
                        if (!silent) {
                            this.notifyValueChanged(new api.ui.ValueChangedEvent(JSON.stringify(this.oldValue), JSON.stringify(newValue)));
                        }
                        this.oldValue = newValue;
                        return this;
                    };
                    PermissionSelector.prototype.onValueChanged = function (listener) {
                        this.valueChangedListeners.push(listener);
                    };
                    PermissionSelector.prototype.unValueChanged = function (listener) {
                        this.valueChangedListeners = this.valueChangedListeners.filter(function (curr) {
                            return curr !== listener;
                        });
                    };
                    PermissionSelector.prototype.notifyValueChanged = function (event) {
                        this.valueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    PermissionSelector.OPTIONS = [
                        { name: 'Read', value: 0 /* READ */ },
                        { name: 'Create', value: 1 /* CREATE */ },
                        { name: 'Modify', value: 2 /* MODIFY */ },
                        { name: 'Delete', value: 3 /* DELETE */ },
                        { name: 'Publish', value: 4 /* PUBLISH */ },
                        { name: 'Read Permissions', value: 5 /* READ_PERMISSIONS */ },
                        { name: 'Write Permissions', value: 6 /* WRITE_PERMISSIONS */ }
                    ];
                    return PermissionSelector;
                })(api.dom.DivEl);
                acl.PermissionSelector = PermissionSelector;
                var PermissionToggle = (function (_super) {
                    __extends(PermissionToggle, _super);
                    function PermissionToggle(option, state) {
                        var _this = this;
                        if (state === void 0) { state = 2 /* INHERIT */; }
                        _super.call(this, 'permission-toggle ' + state);
                        this.valueChangedListeners = [];
                        this.originalStateIndex = -1;
                        this.stateIndex = -1;
                        this.enabled = true;
                        this.setHtml(option.name);
                        this.value = option.value;
                        if (state) {
                            this.setState(state, true);
                        }
                        this.onClicked(function (event) {
                            if (_this.enabled) {
                                var newIndex = (_this.stateIndex + 1) % PermissionToggle.STATES.length;
                                _this.setState(PermissionToggle.STATES[newIndex]);
                                event.preventDefault();
                                event.stopPropagation();
                            }
                        });
                    }
                    PermissionToggle.prototype.setEnabled = function (enabled) {
                        this.enabled = enabled;
                        this.toggleClass('disabled', !enabled);
                        return this;
                    };
                    PermissionToggle.prototype.isEnabled = function () {
                        return this.enabled;
                    };
                    PermissionToggle.prototype.getValue = function () {
                        return this.value;
                    };
                    PermissionToggle.prototype.getState = function () {
                        return PermissionToggle.STATES[this.stateIndex];
                    };
                    PermissionToggle.prototype.setState = function (newState, silent) {
                        var newStateIndex = PermissionToggle.STATES.indexOf(newState);
                        if (newStateIndex != this.stateIndex) {
                            if (this.originalStateIndex < 0) {
                                this.originalStateIndex = newStateIndex;
                            }
                            this.toggleClass('dirty', this.originalStateIndex >= 0 && this.originalStateIndex != newStateIndex);
                            var oldState = this.getState();
                            if (oldState != undefined) {
                                this.removeClass(PermissionState[oldState].toLowerCase());
                            }
                            this.addClass(PermissionState[newState].toLowerCase());
                            this.stateIndex = newStateIndex;
                            if (!silent) {
                                this.notifyValueChanged(new api.ui.ValueChangedEvent(PermissionState[oldState], PermissionState[newState]));
                            }
                        }
                        return this;
                    };
                    PermissionToggle.prototype.onValueChanged = function (listener) {
                        this.valueChangedListeners.push(listener);
                    };
                    PermissionToggle.prototype.unValueChanged = function (listener) {
                        this.valueChangedListeners = this.valueChangedListeners.filter(function (curr) {
                            return curr !== listener;
                        });
                    };
                    PermissionToggle.prototype.notifyValueChanged = function (event) {
                        this.valueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    PermissionToggle.STATES = [0 /* ALLOW */, 1 /* DENY */, 2 /* INHERIT */];
                    return PermissionToggle;
                })(api.dom.AEl);
                acl.PermissionToggle = PermissionToggle;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var Principal = api.security.Principal;
                var Permission = api.security.acl.Permission;
                var AccessControlEntry = api.security.acl.AccessControlEntry;
                var AccessControlEntryView = (function (_super) {
                    __extends(AccessControlEntryView, _super);
                    function AccessControlEntryView(ace) {
                        var _this = this;
                        _super.call(this);
                        this.valueChangedListeners = [];
                        this.editable = true;
                        this.setClass('access-control-entry');
                        this.ace = ace;
                        this.accessSelector = new acl.AccessSelector();
                        this.removeButton = new api.dom.AEl("icon-close");
                        this.removeButton.onClicked(function (event) { return _this.notifyRemoveClicked(event); });
                        this.permissionSelector = new acl.PermissionSelector();
                        this.permissionSelector.onValueChanged(function (event) {
                            _this.toggleClass("dirty", event.getNewValue() != JSON.stringify({
                                allow: _this.ace.getAllowedPermissions().sort(),
                                deny: _this.ace.getDeniedPermissions().sort()
                            }));
                            _this.notifyValueChanged(_this.getAccessControlEntry());
                        });
                        this.permissionSelector.setValue({ allow: ace.getAllowedPermissions(), deny: ace.getDeniedPermissions() }, true);
                        this.accessSelector.onValueChanged(function (event) {
                            if (acl.Access[event.getNewValue()] == 4 /* CUSTOM */) {
                                _this.permissionSelector.show();
                            }
                            else {
                                if (acl.Access[event.getOldValue()] == 4 /* CUSTOM */) {
                                    _this.permissionSelector.hide();
                                }
                                _this.permissionSelector.setValue(_this.getPermissionsValueFromAccess(acl.Access[event.getNewValue()]));
                            }
                        });
                        this.setAccessControlEntry(this.ace, true);
                    }
                    AccessControlEntryView.prototype.doRender = function () {
                        _super.prototype.doRender.call(this);
                        this.appendChild(this.accessSelector);
                        this.appendChild(this.removeButton);
                        this.appendChild(this.permissionSelector);
                        return true;
                    };
                    AccessControlEntryView.prototype.getPermissionSelector = function () {
                        return this.permissionSelector;
                    };
                    AccessControlEntryView.prototype.getValueChangedListeners = function () {
                        return this.valueChangedListeners;
                    };
                    AccessControlEntryView.prototype.setEditable = function (editable) {
                        if (editable != this.editable) {
                            this.permissionSelector.setEnabled(editable);
                            this.accessSelector.setEnabled(editable);
                            this.editable = editable;
                        }
                    };
                    AccessControlEntryView.prototype.isEditable = function () {
                        return this.editable;
                    };
                    AccessControlEntryView.prototype.onValueChanged = function (listener) {
                        this.valueChangedListeners.push(listener);
                    };
                    AccessControlEntryView.prototype.unValueChanged = function (listener) {
                        this.valueChangedListeners = this.valueChangedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    AccessControlEntryView.prototype.notifyValueChanged = function (item) {
                        this.valueChangedListeners.forEach(function (listener) {
                            listener(item);
                        });
                    };
                    AccessControlEntryView.prototype.setAccessControlEntry = function (ace, silent) {
                        this.ace = ace;
                        var principal = new Principal(ace.getPrincipalKey(), ace.getPrincipalDisplayName());
                        this.setObject(principal);
                        var permissions = {
                            allow: ace.getAllowedPermissions().sort(),
                            deny: ace.getDeniedPermissions().sort()
                        };
                        this.accessSelector.setValue(this.getAccessValueFromPermissions(permissions), silent);
                    };
                    AccessControlEntryView.prototype.getAccessControlEntry = function () {
                        var permissions = this.permissionSelector.getValue();
                        var ace = new AccessControlEntry(this.ace.getPrincipal());
                        ace.setAllowedPermissions(permissions.allow);
                        ace.setDeniedPermissions(permissions.deny);
                        return ace;
                    };
                    AccessControlEntryView.prototype.getPermissionsValueFromAccess = function (access) {
                        var permissions = {
                            allow: [],
                            deny: []
                        };
                        switch (access) {
                            case 3 /* FULL */:
                                permissions.allow.push(5 /* READ_PERMISSIONS */);
                                permissions.allow.push(6 /* WRITE_PERMISSIONS */);
                            case 2 /* PUBLISH */:
                                permissions.allow.push(4 /* PUBLISH */);
                            case 1 /* WRITE */:
                                permissions.allow.push(1 /* CREATE */);
                                permissions.allow.push(2 /* MODIFY */);
                                permissions.allow.push(3 /* DELETE */);
                            case 0 /* READ */:
                                permissions.allow.push(0 /* READ */);
                                break;
                        }
                        permissions.allow.sort();
                        permissions.deny.sort();
                        return permissions;
                    };
                    AccessControlEntryView.prototype.getAccessValueFromPermissions = function (permissions) {
                        if (permissions.deny.length == 0) {
                            if (this.isFullAccess(permissions.allow)) {
                                return 3 /* FULL */;
                            }
                            else if (this.isCanPublish(permissions.allow)) {
                                return 2 /* PUBLISH */;
                            }
                            else if (this.isCanWrite(permissions.allow)) {
                                return 1 /* WRITE */;
                            }
                            else if (this.isCanRead(permissions.allow)) {
                                return 0 /* READ */;
                            }
                        }
                        return 4 /* CUSTOM */;
                    };
                    AccessControlEntryView.prototype.isCanRead = function (allowed) {
                        return allowed.indexOf(0 /* READ */) >= 0 && allowed.length === 1;
                    };
                    AccessControlEntryView.prototype.isCanWrite = function (allowed) {
                        return allowed.indexOf(0 /* READ */) >= 0 && allowed.indexOf(1 /* CREATE */) >= 0 && allowed.indexOf(2 /* MODIFY */) >= 0 && allowed.indexOf(3 /* DELETE */) >= 0 && allowed.length === 4;
                    };
                    AccessControlEntryView.prototype.isCanPublish = function (allowed) {
                        return allowed.indexOf(0 /* READ */) >= 0 && allowed.indexOf(1 /* CREATE */) >= 0 && allowed.indexOf(2 /* MODIFY */) >= 0 && allowed.indexOf(3 /* DELETE */) >= 0 && allowed.indexOf(4 /* PUBLISH */) >= 0 && allowed.length === 5;
                    };
                    AccessControlEntryView.prototype.isFullAccess = function (allowed) {
                        return allowed.indexOf(0 /* READ */) >= 0 && allowed.indexOf(1 /* CREATE */) >= 0 && allowed.indexOf(2 /* MODIFY */) >= 0 && allowed.indexOf(3 /* DELETE */) >= 0 && allowed.indexOf(4 /* PUBLISH */) >= 0 && allowed.indexOf(5 /* READ_PERMISSIONS */) >= 0 && allowed.indexOf(6 /* WRITE_PERMISSIONS */) >= 0 && allowed.length === 7;
                    };
                    return AccessControlEntryView;
                })(api.ui.security.PrincipalViewer);
                acl.AccessControlEntryView = AccessControlEntryView;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var AccessControlListView = (function (_super) {
                    __extends(AccessControlListView, _super);
                    function AccessControlListView(className) {
                        _super.call(this, 'access-control-list' + (className ? " " + className : ""));
                        this.itemValueChangedListeners = [];
                        this.itemsEditable = true;
                    }
                    AccessControlListView.prototype.createItemView = function (entry) {
                        var _this = this;
                        var itemView = new acl.AccessControlEntryView(entry);
                        itemView.setEditable(this.itemsEditable);
                        itemView.onRemoveClicked(function () {
                            _this.removeItem(entry);
                        });
                        itemView.onValueChanged(function (item) {
                            _this.notifyItemValueChanged(item);
                        });
                        return itemView;
                    };
                    AccessControlListView.prototype.getItemId = function (item) {
                        return item.getPrincipalKey().toString();
                    };
                    AccessControlListView.prototype.onItemValueChanged = function (listener) {
                        this.itemValueChangedListeners.push(listener);
                    };
                    AccessControlListView.prototype.unItemValueChanged = function (listener) {
                        this.itemValueChangedListeners = this.itemValueChangedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    AccessControlListView.prototype.notifyItemValueChanged = function (item) {
                        this.itemValueChangedListeners.forEach(function (listener) {
                            listener(item);
                        });
                    };
                    AccessControlListView.prototype.setItemsEditable = function (editable) {
                        if (this.itemsEditable != editable) {
                            this.itemsEditable = editable;
                            this.refreshList();
                        }
                        return this;
                    };
                    AccessControlListView.prototype.isItemsEditable = function () {
                        return this.itemsEditable;
                    };
                    return AccessControlListView;
                })(api.ui.selector.list.ListBox);
                acl.AccessControlListView = AccessControlListView;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var PrincipalType = api.security.PrincipalType;
                var AccessControlEntryViewer = (function (_super) {
                    __extends(AccessControlEntryViewer, _super);
                    function AccessControlEntryViewer() {
                        _super.call(this);
                    }
                    AccessControlEntryViewer.prototype.resolveDisplayName = function (object) {
                        return object.getPrincipalDisplayName();
                    };
                    AccessControlEntryViewer.prototype.resolveUnnamedDisplayName = function (object) {
                        return object.getPrincipalTypeName();
                    };
                    AccessControlEntryViewer.prototype.resolveSubName = function (object, relativePath) {
                        if (relativePath === void 0) { relativePath = false; }
                        return object.getPrincipalKey().toPath();
                    };
                    AccessControlEntryViewer.prototype.resolveIconClass = function (object) {
                        switch (object.getPrincipalKey().getType()) {
                            case 0 /* USER */:
                                return "icon-user";
                            case 1 /* GROUP */:
                                return "icon-users";
                            case 2 /* ROLE */:
                                return "icon-shield";
                        }
                        return "";
                    };
                    return AccessControlEntryViewer;
                })(api.ui.NamesAndIconViewer);
                acl.AccessControlEntryViewer = AccessControlEntryViewer;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var SelectedOption = api.ui.selector.combobox.SelectedOption;
                var Permission = api.security.acl.Permission;
                var AccessControlEntryLoader = api.security.acl.AccessControlEntryLoader;
                var AccessControlComboBox = (function (_super) {
                    __extends(AccessControlComboBox, _super);
                    function AccessControlComboBox() {
                        this.aceSelectedOptionsView = new ACESelectedOptionsView();
                        var builder = new api.ui.selector.combobox.RichComboBoxBuilder().setMaximumOccurrences(0).setComboBoxName("principalSelector").setIdentifierMethod("getPrincipalKey").setLoader(new AccessControlEntryLoader()).setSelectedOptionsView(this.aceSelectedOptionsView).setOptionDisplayValueViewer(new acl.AccessControlEntryViewer()).setDelayedInputValueChangedHandling(500);
                        _super.call(this, builder);
                    }
                    AccessControlComboBox.prototype.setEditable = function (editable) {
                        this.getSelectedOptions().forEach(function (option) {
                            option.getOptionView().setEditable(editable);
                        });
                    };
                    AccessControlComboBox.prototype.onOptionValueChanged = function (listener) {
                        this.aceSelectedOptionsView.onItemValueChanged(listener);
                    };
                    AccessControlComboBox.prototype.unItemValueChanged = function (listener) {
                        this.aceSelectedOptionsView.unItemValueChanged(listener);
                    };
                    return AccessControlComboBox;
                })(api.ui.selector.combobox.RichComboBox);
                acl.AccessControlComboBox = AccessControlComboBox;
                var ACESelectedOptionView = (function (_super) {
                    __extends(ACESelectedOptionView, _super);
                    function ACESelectedOptionView(option) {
                        var ace = option.displayValue;
                        if (ace.getAllowedPermissions().length == 0 && ace.getDeniedPermissions().length == 0) {
                            ace.allow(0 /* READ */);
                        }
                        _super.call(this, ace);
                        this.option = option;
                    }
                    ACESelectedOptionView.prototype.setOption = function (option) {
                        this.option = option;
                        this.setAccessControlEntry(option.displayValue);
                    };
                    ACESelectedOptionView.prototype.getOption = function () {
                        return this.option;
                    };
                    ACESelectedOptionView.prototype.onSelectedOptionRemoveRequest = function (listener) {
                        this.onRemoveClicked(listener);
                    };
                    ACESelectedOptionView.prototype.unSelectedOptionRemoveRequest = function (listener) {
                        this.unRemoveClicked(listener);
                    };
                    return ACESelectedOptionView;
                })(acl.AccessControlEntryView);
                var ACESelectedOptionsView = (function (_super) {
                    __extends(ACESelectedOptionsView, _super);
                    function ACESelectedOptionsView(className) {
                        _super.call(this, className);
                        this.list = [];
                        this.selectedOptionRemovedListeners = [];
                    }
                    ACESelectedOptionsView.prototype.setMaximumOccurrences = function (value) {
                        this.maximumOccurrences = value;
                    };
                    ACESelectedOptionsView.prototype.getMaximumOccurrences = function () {
                        return this.maximumOccurrences;
                    };
                    ACESelectedOptionsView.prototype.createSelectedOption = function (option) {
                        throw new Error('Not supported, use createItemView instead');
                    };
                    ACESelectedOptionsView.prototype.createItemView = function (entry) {
                        var _this = this;
                        var option = {
                            displayValue: entry,
                            value: this.getItemId(entry)
                        };
                        var itemView = new ACESelectedOptionView(option);
                        itemView.onValueChanged(function (item) {
                            var selectedOption = _this.getById(item.getPrincipalKey().toString());
                            if (selectedOption) {
                                selectedOption.getOption().displayValue = item;
                            }
                            _this.notifyItemValueChanged(item);
                        });
                        var selectedOption = new SelectedOption(itemView, this.list.length);
                        itemView.onSelectedOptionRemoveRequest(function () {
                            _this.removeOption(option, false);
                        });
                        this.list.push(selectedOption);
                        return itemView;
                    };
                    ACESelectedOptionsView.prototype.addOption = function (option) {
                        this.addItem(option.displayValue);
                        return true;
                    };
                    ACESelectedOptionsView.prototype.removeOption = function (optionToRemove, silent) {
                        api.util.assertNotNull(optionToRemove, "optionToRemove cannot be null");
                        var selectedOption = this.getByOption(optionToRemove);
                        api.util.assertNotNull(selectedOption, "Did not find any selected option to remove from option: " + optionToRemove.value);
                        this.removeItem(optionToRemove.displayValue);
                        this.list = this.list.filter(function (option) {
                            return option.getOption().value != selectedOption.getOption().value;
                        });
                        if (selectedOption.getIndex() < this.list.length) {
                            for (var i = selectedOption.getIndex(); i < this.list.length; i++) {
                                this.list[i].setIndex(i);
                            }
                        }
                        if (!silent) {
                            this.notifySelectedOptionRemoved(selectedOption);
                        }
                    };
                    ACESelectedOptionsView.prototype.count = function () {
                        return this.list.length;
                    };
                    ACESelectedOptionsView.prototype.getSelectedOptions = function () {
                        return this.list;
                    };
                    ACESelectedOptionsView.prototype.getByIndex = function (index) {
                        return this.list[index];
                    };
                    ACESelectedOptionsView.prototype.getByOption = function (option) {
                        return this.getById(option.value);
                    };
                    ACESelectedOptionsView.prototype.getById = function (id) {
                        return this.list.filter(function (selectedOption) {
                            return selectedOption.getOption().value == id;
                        })[0];
                    };
                    ACESelectedOptionsView.prototype.isSelected = function (option) {
                        return this.getByOption(option) != null;
                    };
                    ACESelectedOptionsView.prototype.maximumOccurrencesReached = function () {
                        if (this.maximumOccurrences == 0) {
                            return false;
                        }
                        return this.count() >= this.maximumOccurrences;
                    };
                    ACESelectedOptionsView.prototype.moveOccurrence = function (formIndex, toIndex) {
                        api.util.ArrayHelper.moveElement(formIndex, toIndex, this.list);
                        api.util.ArrayHelper.moveElement(formIndex, toIndex, this.getChildren());
                        this.list.forEach(function (selectedOption, index) { return selectedOption.setIndex(index); });
                    };
                    ACESelectedOptionsView.prototype.notifySelectedOptionRemoved = function (removed) {
                        this.selectedOptionRemovedListeners.forEach(function (listener) {
                            listener(removed);
                        });
                    };
                    ACESelectedOptionsView.prototype.onOptionDeselected = function (listener) {
                        this.selectedOptionRemovedListeners.push(listener);
                    };
                    ACESelectedOptionsView.prototype.unOptionDeselected = function (listener) {
                        this.selectedOptionRemovedListeners = this.selectedOptionRemovedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    return ACESelectedOptionsView;
                })(acl.AccessControlListView);
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var TabMenuItemBuilder = api.ui.tab.TabMenuItemBuilder;
                var UserStoreAccess = api.security.acl.UserStoreAccess;
                var UserStoreAccessSelector = (function (_super) {
                    __extends(UserStoreAccessSelector, _super);
                    function UserStoreAccessSelector() {
                        var _this = this;
                        _super.call(this, "access-selector");
                        this.valueChangedListeners = [];
                        UserStoreAccessSelector.OPTIONS.forEach(function (option, index) {
                            var menuItem = new TabMenuItemBuilder().setLabel(option.name).build();
                            _this.addNavigationItem(menuItem);
                        });
                        this.onNavigationItemSelected(function (event) {
                            var item = event.getItem();
                            _this.setValue(UserStoreAccessSelector.OPTIONS[item.getIndex()].value);
                        });
                    }
                    UserStoreAccessSelector.prototype.getValue = function () {
                        return this.value;
                    };
                    UserStoreAccessSelector.prototype.setValue = function (value, silent) {
                        var option = this.findOptionByValue(value);
                        if (option) {
                            this.selectNavigationItem(UserStoreAccessSelector.OPTIONS.indexOf(option));
                            if (!silent) {
                                this.notifyValueChanged(new api.ui.ValueChangedEvent(UserStoreAccess[this.value], UserStoreAccess[value]));
                            }
                            this.value = value;
                        }
                        return this;
                    };
                    UserStoreAccessSelector.prototype.findOptionByValue = function (value) {
                        for (var i = 0; i < UserStoreAccessSelector.OPTIONS.length; i++) {
                            var option = UserStoreAccessSelector.OPTIONS[i];
                            if (option.value == value) {
                                return option;
                            }
                        }
                        return undefined;
                    };
                    UserStoreAccessSelector.prototype.onValueChanged = function (listener) {
                        this.valueChangedListeners.push(listener);
                    };
                    UserStoreAccessSelector.prototype.unValueChanged = function (listener) {
                        this.valueChangedListeners = this.valueChangedListeners.filter(function (curr) {
                            return curr !== listener;
                        });
                    };
                    UserStoreAccessSelector.prototype.notifyValueChanged = function (event) {
                        this.valueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    UserStoreAccessSelector.OPTIONS = [
                        { value: 0 /* READ */, name: 'Read' },
                        { value: 1 /* CREATE_USERS */, name: 'Create Users' },
                        { value: 2 /* WRITE_USERS */, name: 'Write Users' },
                        { value: 3 /* USER_STORE_MANAGER */, name: 'User Store Manager' },
                        { value: 4 /* ADMINISTRATOR */, name: 'Administrator' }
                    ];
                    return UserStoreAccessSelector;
                })(api.ui.tab.TabMenu);
                acl.UserStoreAccessSelector = UserStoreAccessSelector;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var Principal = api.security.Principal;
                var UserStoreAccessControlEntry = api.security.acl.UserStoreAccessControlEntry;
                var UserStoreAccess = api.security.acl.UserStoreAccess;
                var UserStoreAccessControlEntryView = (function (_super) {
                    __extends(UserStoreAccessControlEntryView, _super);
                    function UserStoreAccessControlEntryView(ace) {
                        var _this = this;
                        _super.call(this);
                        this.valueChangedListeners = [];
                        this.editable = true;
                        this.setClass('userstore-access-control-entry');
                        this.ace = ace;
                        if (isNaN(this.ace.getAccess())) {
                            this.ace.setAccess(UserStoreAccess[1 /* CREATE_USERS */]);
                        }
                        this.accessSelector = new acl.UserStoreAccessSelector();
                        this.accessSelector.onValueChanged(function (event) {
                            _this.ace.setAccess(event.getNewValue());
                        });
                        this.removeButton = new api.dom.AEl("icon-close");
                        this.removeButton.onClicked(function (event) {
                            if (_this.editable) {
                                _this.notifyRemoveClicked(event);
                            }
                        });
                        this.setUserStoreAccessControlEntry(this.ace, true);
                    }
                    UserStoreAccessControlEntryView.prototype.getValueChangedListeners = function () {
                        return this.valueChangedListeners;
                    };
                    UserStoreAccessControlEntryView.prototype.setEditable = function (editable) {
                        if (editable != this.editable) {
                            this.accessSelector.setEnabled(editable);
                            this.editable = editable;
                        }
                        if (this.editable) {
                            this.removeClass("readonly");
                        }
                        else {
                            this.addClass("readonly");
                        }
                    };
                    UserStoreAccessControlEntryView.prototype.isEditable = function () {
                        return this.editable;
                    };
                    UserStoreAccessControlEntryView.prototype.onValueChanged = function (listener) {
                        this.valueChangedListeners.push(listener);
                    };
                    UserStoreAccessControlEntryView.prototype.unValueChanged = function (listener) {
                        this.valueChangedListeners = this.valueChangedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    UserStoreAccessControlEntryView.prototype.notifyValueChanged = function (item) {
                        this.valueChangedListeners.forEach(function (listener) {
                            listener(item);
                        });
                    };
                    UserStoreAccessControlEntryView.prototype.setUserStoreAccessControlEntry = function (ace, silent) {
                        this.ace = ace;
                        var principal = new Principal(ace.getPrincipal().getKey(), ace.getPrincipal().getDisplayName(), ace.getPrincipal().getModifiedTime());
                        this.setObject(principal);
                        this.accessSelector.setValue(ace.getAccess(), silent);
                    };
                    UserStoreAccessControlEntryView.prototype.getUserStoreAccessControlEntry = function () {
                        var ace = new UserStoreAccessControlEntry(this.ace.getPrincipal(), this.ace.getAccess());
                        return ace;
                    };
                    UserStoreAccessControlEntryView.prototype.doRender = function () {
                        _super.prototype.doRender.call(this);
                        this.appendChild(this.accessSelector);
                        this.appendChild(this.removeButton);
                        return true;
                    };
                    return UserStoreAccessControlEntryView;
                })(api.ui.security.PrincipalViewer);
                acl.UserStoreAccessControlEntryView = UserStoreAccessControlEntryView;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var UserStoreAccessControlListView = (function (_super) {
                    __extends(UserStoreAccessControlListView, _super);
                    function UserStoreAccessControlListView(className) {
                        _super.call(this, 'access-control-list' + (className ? " " + className : ""));
                        this.itemValueChangedListeners = [];
                        this.itemsEditable = true;
                    }
                    UserStoreAccessControlListView.prototype.createItemView = function (entry, readOnly) {
                        var _this = this;
                        var itemView = new acl.UserStoreAccessControlEntryView(entry);
                        itemView.setEditable(this.itemsEditable);
                        itemView.onRemoveClicked(function () {
                            _this.removeItem(entry);
                        });
                        itemView.onValueChanged(function (item) {
                            _this.notifyItemValueChanged(item);
                        });
                        if (readOnly) {
                            itemView.setEditable(false);
                        }
                        return itemView;
                    };
                    UserStoreAccessControlListView.prototype.getItemId = function (item) {
                        return item.getPrincipal().getKey().toString();
                    };
                    UserStoreAccessControlListView.prototype.onItemValueChanged = function (listener) {
                        this.itemValueChangedListeners.push(listener);
                    };
                    UserStoreAccessControlListView.prototype.unItemValueChanged = function (listener) {
                        this.itemValueChangedListeners = this.itemValueChangedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    UserStoreAccessControlListView.prototype.notifyItemValueChanged = function (item) {
                        this.itemValueChangedListeners.forEach(function (listener) {
                            listener(item);
                        });
                    };
                    UserStoreAccessControlListView.prototype.setItemsEditable = function (editable) {
                        if (this.itemsEditable != editable) {
                            this.itemsEditable = editable;
                            this.refreshList();
                        }
                        return this;
                    };
                    UserStoreAccessControlListView.prototype.isItemsEditable = function () {
                        return this.itemsEditable;
                    };
                    return UserStoreAccessControlListView;
                })(api.ui.selector.list.ListBox);
                acl.UserStoreAccessControlListView = UserStoreAccessControlListView;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var UserStoreAccessControlEntryViewer = (function (_super) {
                    __extends(UserStoreAccessControlEntryViewer, _super);
                    function UserStoreAccessControlEntryViewer() {
                        _super.call(this);
                    }
                    UserStoreAccessControlEntryViewer.prototype.resolveDisplayName = function (object) {
                        return object.getPrincipalDisplayName();
                    };
                    UserStoreAccessControlEntryViewer.prototype.resolveUnnamedDisplayName = function (object) {
                        return object.getPrincipalTypeName();
                    };
                    UserStoreAccessControlEntryViewer.prototype.resolveSubName = function (object, relativePath) {
                        if (relativePath === void 0) { relativePath = false; }
                        return object.getPrincipalKey().toPath();
                    };
                    UserStoreAccessControlEntryViewer.prototype.resolveIconClass = function (object) {
                        return "icon-users";
                    };
                    return UserStoreAccessControlEntryViewer;
                })(api.ui.NamesAndIconViewer);
                acl.UserStoreAccessControlEntryViewer = UserStoreAccessControlEntryViewer;
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var security;
        (function (security) {
            var acl;
            (function (acl) {
                var SelectedOption = api.ui.selector.combobox.SelectedOption;
                var UserStoreAccessControlEntryLoader = api.security.acl.UserStoreAccessControlEntryLoader;
                var UserStoreAccessControlComboBox = (function (_super) {
                    __extends(UserStoreAccessControlComboBox, _super);
                    function UserStoreAccessControlComboBox() {
                        this.aceSelectedOptionsView = new UserStoreACESelectedOptionsView();
                        var builder = new api.ui.selector.combobox.RichComboBoxBuilder().setMaximumOccurrences(0).setComboBoxName("principalSelector").setLoader(new UserStoreAccessControlEntryLoader()).setSelectedOptionsView(this.aceSelectedOptionsView).setOptionDisplayValueViewer(new acl.UserStoreAccessControlEntryViewer()).setDelayedInputValueChangedHandling(500);
                        _super.call(this, builder);
                    }
                    UserStoreAccessControlComboBox.prototype.setEditable = function (editable) {
                        this.getSelectedOptions().forEach(function (option) {
                            option.getOptionView().setEditable(editable);
                        });
                    };
                    UserStoreAccessControlComboBox.prototype.onOptionValueChanged = function (listener) {
                        this.aceSelectedOptionsView.onItemValueChanged(listener);
                    };
                    UserStoreAccessControlComboBox.prototype.unItemValueChanged = function (listener) {
                        this.aceSelectedOptionsView.unItemValueChanged(listener);
                    };
                    return UserStoreAccessControlComboBox;
                })(api.ui.selector.combobox.RichComboBox);
                acl.UserStoreAccessControlComboBox = UserStoreAccessControlComboBox;
                var UserStoreACESelectedOptionView = (function (_super) {
                    __extends(UserStoreACESelectedOptionView, _super);
                    function UserStoreACESelectedOptionView(option) {
                        var ace = option.displayValue;
                        _super.call(this, ace);
                        this.option = option;
                    }
                    UserStoreACESelectedOptionView.prototype.setOption = function (option) {
                        this.option = option;
                        this.setUserStoreAccessControlEntry(option.displayValue);
                    };
                    UserStoreACESelectedOptionView.prototype.getOption = function () {
                        return this.option;
                    };
                    UserStoreACESelectedOptionView.prototype.onSelectedOptionRemoveRequest = function (listener) {
                        this.onRemoveClicked(listener);
                    };
                    UserStoreACESelectedOptionView.prototype.unSelectedOptionRemoveRequest = function (listener) {
                        this.unRemoveClicked(listener);
                    };
                    return UserStoreACESelectedOptionView;
                })(acl.UserStoreAccessControlEntryView);
                var UserStoreACESelectedOptionsView = (function (_super) {
                    __extends(UserStoreACESelectedOptionsView, _super);
                    function UserStoreACESelectedOptionsView(className) {
                        _super.call(this, className);
                        this.list = [];
                        this.selectedOptionRemovedListeners = [];
                    }
                    UserStoreACESelectedOptionsView.prototype.setMaximumOccurrences = function (value) {
                        this.maximumOccurrences = value;
                    };
                    UserStoreACESelectedOptionsView.prototype.getMaximumOccurrences = function () {
                        return this.maximumOccurrences;
                    };
                    UserStoreACESelectedOptionsView.prototype.createSelectedOption = function (option) {
                        throw new Error('Not supported, use createItemView instead');
                    };
                    UserStoreACESelectedOptionsView.prototype.createItemView = function (entry, readOnly) {
                        var _this = this;
                        var option = {
                            displayValue: entry,
                            value: this.getItemId(entry),
                            readOnly: readOnly
                        };
                        var itemView = new UserStoreACESelectedOptionView(option);
                        itemView.onValueChanged(function (item) {
                            var selectedOption = _this.getById(item.getPrincipal().getKey().toString());
                            if (selectedOption) {
                                selectedOption.getOption().displayValue = item;
                            }
                            _this.notifyItemValueChanged(item);
                        });
                        var selectedOption = new SelectedOption(itemView, this.list.length);
                        itemView.onSelectedOptionRemoveRequest(function () {
                            _this.removeOption(option, false);
                        });
                        if (readOnly) {
                            itemView.setEditable(false);
                        }
                        this.list.push(selectedOption);
                        return itemView;
                    };
                    UserStoreACESelectedOptionsView.prototype.addOption = function (option) {
                        if (option.readOnly) {
                            this.addItemReadOnly(option.displayValue);
                        }
                        else {
                            this.addItem(option.displayValue);
                        }
                        return true;
                    };
                    UserStoreACESelectedOptionsView.prototype.removeOption = function (optionToRemove, silent) {
                        api.util.assertNotNull(optionToRemove, "optionToRemove cannot be null");
                        var selectedOption = this.getByOption(optionToRemove);
                        api.util.assertNotNull(selectedOption, "Did not find any selected option to remove from option: " + optionToRemove.value);
                        this.removeItem(optionToRemove.displayValue);
                        this.list = this.list.filter(function (option) {
                            return option.getOption().value != selectedOption.getOption().value;
                        });
                        if (selectedOption.getIndex() < this.list.length) {
                            for (var i = selectedOption.getIndex(); i < this.list.length; i++) {
                                this.list[i].setIndex(i);
                            }
                        }
                        if (!silent) {
                            this.notifySelectedOptionRemoved(selectedOption);
                        }
                    };
                    UserStoreACESelectedOptionsView.prototype.count = function () {
                        return this.list.length;
                    };
                    UserStoreACESelectedOptionsView.prototype.getSelectedOptions = function () {
                        return this.list;
                    };
                    UserStoreACESelectedOptionsView.prototype.getByIndex = function (index) {
                        return this.list[index];
                    };
                    UserStoreACESelectedOptionsView.prototype.getByOption = function (option) {
                        return this.getById(option.value);
                    };
                    UserStoreACESelectedOptionsView.prototype.getById = function (id) {
                        return this.list.filter(function (selectedOption) {
                            return selectedOption.getOption().value == id;
                        })[0];
                    };
                    UserStoreACESelectedOptionsView.prototype.isSelected = function (option) {
                        return this.getByOption(option) != null;
                    };
                    UserStoreACESelectedOptionsView.prototype.maximumOccurrencesReached = function () {
                        if (this.maximumOccurrences == 0) {
                            return false;
                        }
                        return this.count() >= this.maximumOccurrences;
                    };
                    UserStoreACESelectedOptionsView.prototype.moveOccurrence = function (formIndex, toIndex) {
                        api.util.ArrayHelper.moveElement(formIndex, toIndex, this.list);
                        api.util.ArrayHelper.moveElement(formIndex, toIndex, this.getChildren());
                        this.list.forEach(function (selectedOption, index) { return selectedOption.setIndex(index); });
                    };
                    UserStoreACESelectedOptionsView.prototype.notifySelectedOptionRemoved = function (removed) {
                        this.selectedOptionRemovedListeners.forEach(function (listener) {
                            listener(removed);
                        });
                    };
                    UserStoreACESelectedOptionsView.prototype.onOptionDeselected = function (listener) {
                        this.selectedOptionRemovedListeners.push(listener);
                    };
                    UserStoreACESelectedOptionsView.prototype.unOptionDeselected = function (listener) {
                        this.selectedOptionRemovedListeners = this.selectedOptionRemovedListeners.filter(function (curr) {
                            return curr != listener;
                        });
                    };
                    return UserStoreACESelectedOptionsView;
                })(acl.UserStoreAccessControlListView);
            })(acl = security.acl || (security.acl = {}));
        })(security = ui.security || (ui.security = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var locale;
        (function (_locale) {
            var LocaleViewer = (function (_super) {
                __extends(LocaleViewer, _super);
                function LocaleViewer() {
                    _super.call(this);
                    this.removeClickedListeners = [];
                    this.displayNamePattern = '{0} ({1})';
                    this.namesView = new api.app.NamesView();
                    this.appendChild(this.namesView);
                }
                LocaleViewer.prototype.setObject = function (locale) {
                    _super.prototype.setObject.call(this, locale);
                    this.namesView.setMainName(api.util.StringHelper.format(this.displayNamePattern, locale.getDisplayName(), locale.getTag()));
                };
                LocaleViewer.prototype.getPreferredHeight = function () {
                    return 30;
                };
                LocaleViewer.prototype.onRemoveClicked = function (listener) {
                    this.removeClickedListeners.push(listener);
                };
                LocaleViewer.prototype.unRemoveClicked = function (listener) {
                    this.removeClickedListeners = this.removeClickedListeners.filter(function (current) {
                        return current !== listener;
                    });
                };
                LocaleViewer.prototype.notifyRemoveClicked = function (event) {
                    this.removeClickedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return LocaleViewer;
            })(api.ui.Viewer);
            _locale.LocaleViewer = LocaleViewer;
        })(locale = ui.locale || (ui.locale = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var ui;
    (function (ui) {
        var locale;
        (function (locale) {
            var LocaleLoader = api.locale.LocaleLoader;
            var LocaleComboBox = (function (_super) {
                __extends(LocaleComboBox, _super);
                function LocaleComboBox(max) {
                    var _this = this;
                    var localeSelectedOptionsView = new LocaleSelectedOptionsView();
                    localeSelectedOptionsView.onOptionDeselected(function () {
                        _this.clearSelection();
                    });
                    var builder = new api.ui.selector.combobox.RichComboBoxBuilder().setMaximumOccurrences(max || 0).setComboBoxName("localeSelector").setIdentifierMethod("getTag").setLoader(new LocaleLoader()).setSelectedOptionsView(localeSelectedOptionsView).setOptionDisplayValueViewer(new locale.LocaleViewer()).setDelayedInputValueChangedHandling(500);
                    _super.call(this, builder);
                }
                LocaleComboBox.prototype.clearSelection = function () {
                    this.getLoader().search("");
                    _super.prototype.clearSelection.call(this);
                };
                return LocaleComboBox;
            })(api.ui.selector.combobox.RichComboBox);
            locale.LocaleComboBox = LocaleComboBox;
            var LocaleSelectedOptionView = (function (_super) {
                __extends(LocaleSelectedOptionView, _super);
                function LocaleSelectedOptionView(option) {
                    var _this = this;
                    _super.call(this);
                    this.setOption(option);
                    this.setClass("locale-selected-option-view");
                    var removeButton = new api.dom.AEl("icon-close");
                    removeButton.onClicked(function (event) { return _this.notifyRemoveClicked(event); });
                    this.appendChild(removeButton);
                }
                LocaleSelectedOptionView.prototype.setOption = function (option) {
                    this.option = option;
                    this.setObject(option.displayValue);
                };
                LocaleSelectedOptionView.prototype.getOption = function () {
                    return this.option;
                };
                LocaleSelectedOptionView.prototype.onSelectedOptionRemoveRequest = function (listener) {
                    this.onRemoveClicked(listener);
                };
                LocaleSelectedOptionView.prototype.unSelectedOptionRemoveRequest = function (listener) {
                    this.unRemoveClicked(listener);
                };
                return LocaleSelectedOptionView;
            })(locale.LocaleViewer);
            var LocaleSelectedOptionsView = (function (_super) {
                __extends(LocaleSelectedOptionsView, _super);
                function LocaleSelectedOptionsView() {
                    _super.call(this, "locale-selected-options-view");
                }
                LocaleSelectedOptionsView.prototype.createSelectedOption = function (option) {
                    var optionView = new LocaleSelectedOptionView(option);
                    return new api.ui.selector.combobox.SelectedOption(optionView, this.count());
                };
                return LocaleSelectedOptionsView;
            })(api.ui.selector.combobox.BaseSelectedOptionsView);
        })(locale = ui.locale || (ui.locale = {}));
    })(ui = api.ui || (api.ui = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueType = (function () {
            function ValueType(name) {
                this.name = name;
            }
            ValueType.prototype.toString = function () {
                return this.name;
            };
            ValueType.prototype.valueToString = function (value) {
                return String(value.getObject());
            };
            ValueType.prototype.valueToBoolean = function (value) {
                return value.getString() == "true";
            };
            ValueType.prototype.valueToNumber = function (value) {
                return Number(value.getObject());
            };
            ValueType.prototype.isValid = function (value) {
                return true;
            };
            ValueType.prototype.isConvertible = function (value) {
                return true;
            };
            ValueType.prototype.newValue = function (value) {
                return new data.Value(value, this);
            };
            ValueType.prototype.newNullValue = function () {
                return new data.Value(null, this);
            };
            ValueType.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ValueType)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.name, other.name)) {
                    return false;
                }
                return true;
            };
            ValueType.prototype.valueEquals = function (a, b) {
                throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
            };
            ValueType.prototype.toJsonValue = function (value) {
                return value.getObject();
            };
            ValueType.prototype.fromJsonValue = function (jsonValue) {
                if (jsonValue) {
                    return this.newValue(jsonValue.toString());
                }
                else if ("" == jsonValue) {
                    return this.newValue(jsonValue);
                }
                else {
                    return this.newNullValue();
                }
            };
            return ValueType;
        })();
        data.ValueType = ValueType;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (_data) {
        var ValueTypePropertySet = (function (_super) {
            __extends(ValueTypePropertySet, _super);
            function ValueTypePropertySet() {
                _super.call(this, "PropertySet");
            }
            ValueTypePropertySet.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, _data.PropertySet)) {
                    return false;
                }
                return true;
            };
            ValueTypePropertySet.prototype.isConvertible = function (value) {
                return false;
            };
            ValueTypePropertySet.prototype.newValue = function (value) {
                throw new Error("A value of type Data cannot be created from a string");
            };
            ValueTypePropertySet.prototype.toJsonValue = function (value) {
                if (value.isNull()) {
                    return null;
                }
                var data = value.getObject();
                return data.toJson();
            };
            ValueTypePropertySet.prototype.fromJsonValue = function (propertyArrayJsonArray) {
                throw new Error("Method not supported!");
            };
            ValueTypePropertySet.prototype.valueToString = function (value) {
                throw new Error("A value of type Data cannot be made into a string");
            };
            ValueTypePropertySet.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.equals(a, b);
            };
            return ValueTypePropertySet;
        })(_data.ValueType);
        _data.ValueTypePropertySet = ValueTypePropertySet;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeString = (function (_super) {
            __extends(ValueTypeString, _super);
            function ValueTypeString() {
                _super.call(this, "String");
            }
            ValueTypeString.prototype.isValid = function (value) {
                return typeof value === 'string';
            };
            ValueTypeString.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.stringEquals(a, b);
            };
            ValueTypeString.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                return new data.Value(value, this);
            };
            ValueTypeString.prototype.toJsonValue = function (value) {
                return value.getString() ? value.getObject() : null;
            };
            return ValueTypeString;
        })(data.ValueType);
        data.ValueTypeString = ValueTypeString;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeXml = (function (_super) {
            __extends(ValueTypeXml, _super);
            function ValueTypeXml() {
                _super.call(this, "Xml");
            }
            ValueTypeXml.prototype.isValid = function (value) {
                return typeof value === 'string';
            };
            ValueTypeXml.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.stringEquals(a, b);
            };
            ValueTypeXml.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                return new data.Value(value, this);
            };
            ValueTypeXml.prototype.toJsonValue = function (value) {
                return value.getString() ? value.getObject() : null;
            };
            return ValueTypeXml;
        })(data.ValueType);
        data.ValueTypeXml = ValueTypeXml;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeBoolean = (function (_super) {
            __extends(ValueTypeBoolean, _super);
            function ValueTypeBoolean() {
                _super.call(this, "Boolean");
            }
            ValueTypeBoolean.prototype.isValid = function (value) {
                return typeof value === 'boolean';
            };
            ValueTypeBoolean.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                if (!(value == "true" || value == "false")) {
                    return false;
                }
                var convertedValue = Boolean(value);
                return this.isValid(convertedValue);
            };
            ValueTypeBoolean.prototype.newValue = function (value) {
                if (!this.isConvertible(value)) {
                    return new data.Value(null, this);
                }
                return new data.Value(this.convertFromString(value), this);
            };
            ValueTypeBoolean.prototype.fromJsonValue = function (jsonValue) {
                return new data.Value(jsonValue, this);
            };
            ValueTypeBoolean.prototype.convertFromString = function (value) {
                if (value == "true") {
                    return true;
                }
                else if (value == "false") {
                    return false;
                }
                else {
                    throw new Error("given string cannot be converted to a Boolean Value: " + value);
                }
            };
            ValueTypeBoolean.prototype.valueToString = function (value) {
                return value.getObject().toString();
            };
            ValueTypeBoolean.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.booleanEquals(a, b);
            };
            ValueTypeBoolean.prototype.newFalse = function () {
                return new data.Value(false, this);
            };
            ValueTypeBoolean.prototype.newTrue = function () {
                return new data.Value(true, this);
            };
            return ValueTypeBoolean;
        })(data.ValueType);
        data.ValueTypeBoolean = ValueTypeBoolean;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeLong = (function (_super) {
            __extends(ValueTypeLong, _super);
            function ValueTypeLong() {
                _super.call(this, "Long");
            }
            ValueTypeLong.prototype.isValid = function (value) {
                return api.util.NumberHelper.isWholeNumber(value);
            };
            ValueTypeLong.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                var convertedValue = Number(value);
                return this.isValid(convertedValue);
            };
            ValueTypeLong.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                if (!this.isConvertible(value)) {
                    return this.newNullValue();
                }
                return new data.Value(this.convertFromString(value), this);
            };
            ValueTypeLong.prototype.fromJsonValue = function (jsonValue) {
                return new data.Value(jsonValue, this);
            };
            ValueTypeLong.prototype.convertFromString = function (value) {
                return Number(value);
            };
            ValueTypeLong.prototype.valueToString = function (value) {
                return value.getObject().toString();
            };
            ValueTypeLong.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.numberEquals(a, b);
            };
            return ValueTypeLong;
        })(data.ValueType);
        data.ValueTypeLong = ValueTypeLong;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeDouble = (function (_super) {
            __extends(ValueTypeDouble, _super);
            function ValueTypeDouble() {
                _super.call(this, "Double");
            }
            ValueTypeDouble.prototype.isValid = function (value) {
                return api.util.NumberHelper.isNumber(value);
            };
            ValueTypeDouble.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                var convertedValue = Number(value);
                return this.isValid(convertedValue);
            };
            ValueTypeDouble.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                if (!this.isConvertible(value)) {
                    return this.newNullValue();
                }
                return new data.Value(this.convertFromString(value), this);
            };
            ValueTypeDouble.prototype.fromJsonValue = function (jsonValue) {
                return new data.Value(jsonValue, this);
            };
            ValueTypeDouble.prototype.convertFromString = function (value) {
                return Number(value);
            };
            ValueTypeDouble.prototype.valueToString = function (value) {
                return value.getObject().toString();
            };
            ValueTypeDouble.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.numberEquals(a, b);
            };
            return ValueTypeDouble;
        })(data.ValueType);
        data.ValueTypeDouble = ValueTypeDouble;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeLocalDate = (function (_super) {
            __extends(ValueTypeLocalDate, _super);
            function ValueTypeLocalDate() {
                _super.call(this, "LocalDate");
            }
            ValueTypeLocalDate.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, Date)) {
                    return false;
                }
                var valueAsDate = value;
                return !api.util.DateHelper.isInvalidDate(valueAsDate);
            };
            ValueTypeLocalDate.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                if (value.length != 10) {
                    return false;
                }
                if (!(value.charAt(4) == '-' && value.charAt(7) == '-')) {
                    return false;
                }
                return this.isValid(new Date(value));
            };
            ValueTypeLocalDate.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                if (!this.isConvertible(value)) {
                    return this.newNullValue();
                }
                var date = api.util.DateHelper.parseDate(value);
                return new data.Value(date, this);
            };
            ValueTypeLocalDate.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : api.util.DateHelper.formatDate(value.getLocalDate());
            };
            ValueTypeLocalDate.prototype.valueToString = function (value) {
                return api.util.DateHelper.formatDate(value.getObject());
            };
            ValueTypeLocalDate.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.dateEquals(a, b);
            };
            return ValueTypeLocalDate;
        })(data.ValueType);
        data.ValueTypeLocalDate = ValueTypeLocalDate;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeLocalDateTime = (function (_super) {
            __extends(ValueTypeLocalDateTime, _super);
            function ValueTypeLocalDateTime() {
                _super.call(this, "LocalDateTime");
            }
            ValueTypeLocalDateTime.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, Date)) {
                    return false;
                }
                var valueAsDate = value;
                return !api.util.DateHelper.isInvalidDate(valueAsDate);
            };
            ValueTypeLocalDateTime.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                if (value.length != 19) {
                    return false;
                }
                var valueAsDate = api.util.DateHelper.parseUTCDateTime(value, true);
                return this.isValid(valueAsDate);
            };
            ValueTypeLocalDateTime.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                if (!this.isConvertible(value)) {
                    return this.newNullValue();
                }
                var date = api.util.DateHelper.parseUTCDateTime(value, true);
                return new data.Value(date, this);
            };
            ValueTypeLocalDateTime.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : api.util.DateHelper.formatDateTime(value.getLocalDateTime());
            };
            ValueTypeLocalDateTime.prototype.valueToString = function (value) {
                return api.util.DateHelper.formatUTCDateTime(value.getObject());
            };
            ValueTypeLocalDateTime.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.dateEquals(a, b);
            };
            return ValueTypeLocalDateTime;
        })(data.ValueType);
        data.ValueTypeLocalDateTime = ValueTypeLocalDateTime;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeLocalTime = (function (_super) {
            __extends(ValueTypeLocalTime, _super);
            function ValueTypeLocalTime() {
                _super.call(this, "LocalTime");
            }
            ValueTypeLocalTime.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, api.util.LocalTime)) {
                    return false;
                }
                return true;
            };
            ValueTypeLocalTime.prototype.isConvertible = function (value) {
                var asString = value;
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                return api.util.LocalTime.isValidString(value);
            };
            ValueTypeLocalTime.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                if (!this.isConvertible(value)) {
                    return this.newNullValue();
                }
                return new data.Value(api.util.LocalTime.fromString(value), this);
            };
            ValueTypeLocalTime.prototype.valueToString = function (value) {
                if (value.isNotNull()) {
                    return value.getLocalTime().toString();
                }
                else {
                    return null;
                }
            };
            ValueTypeLocalTime.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.equals(a, b);
            };
            ValueTypeLocalTime.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : value.getLocalTime().toString();
            };
            return ValueTypeLocalTime;
        })(data.ValueType);
        data.ValueTypeLocalTime = ValueTypeLocalTime;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var DateTime = api.util.DateTime;
        var ValueTypeDateTime = (function (_super) {
            __extends(ValueTypeDateTime, _super);
            function ValueTypeDateTime() {
                _super.call(this, "DateTime");
            }
            ValueTypeDateTime.prototype.isValid = function (value) {
                if (api.ObjectHelper.iFrameSafeInstanceOf(value, api.util.DateTime)) {
                    return true;
                }
                return api.util.DateTime.isValidDateTime(value);
            };
            ValueTypeDateTime.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                if (value.length < 19) {
                    return false;
                }
                return this.isValid(value);
            };
            ValueTypeDateTime.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                if (!this.isConvertible(value)) {
                    return this.newNullValue();
                }
                var date = DateTime.fromString(value);
                return new data.Value(date, this);
            };
            ValueTypeDateTime.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : value.getDateTime().toString();
            };
            ValueTypeDateTime.prototype.valueToString = function (value) {
                return value.getDateTime().toString();
            };
            ValueTypeDateTime.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.equals(a, b);
            };
            return ValueTypeDateTime;
        })(data.ValueType);
        data.ValueTypeDateTime = ValueTypeDateTime;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeGeoPoint = (function (_super) {
            __extends(ValueTypeGeoPoint, _super);
            function ValueTypeGeoPoint() {
                _super.call(this, "GeoPoint");
            }
            ValueTypeGeoPoint.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, api.util.GeoPoint)) {
                    return false;
                }
                return true;
            };
            ValueTypeGeoPoint.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                return api.util.GeoPoint.isValidString(value);
            };
            ValueTypeGeoPoint.prototype.newValue = function (value) {
                if (!value) {
                    return this.newNullValue();
                }
                if (!this.isConvertible(value)) {
                    return this.newNullValue();
                }
                return new data.Value(api.util.GeoPoint.fromString(value), this);
            };
            ValueTypeGeoPoint.prototype.valueToString = function (value) {
                if (value.isNotNull()) {
                    return value.getGeoPoint().toString();
                }
                else {
                    return null;
                }
            };
            ValueTypeGeoPoint.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : value.getGeoPoint().toString();
            };
            ValueTypeGeoPoint.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.equals(a, b);
            };
            return ValueTypeGeoPoint;
        })(data.ValueType);
        data.ValueTypeGeoPoint = ValueTypeGeoPoint;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeBinaryReference = (function (_super) {
            __extends(ValueTypeBinaryReference, _super);
            function ValueTypeBinaryReference() {
                _super.call(this, "BinaryReference");
            }
            ValueTypeBinaryReference.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, api.util.BinaryReference)) {
                    return false;
                }
                return true;
            };
            ValueTypeBinaryReference.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                return true;
            };
            ValueTypeBinaryReference.prototype.newValue = function (value) {
                if (this.isConvertible(value)) {
                    return new data.Value(new api.util.BinaryReference(value), this);
                }
                else {
                    return this.newNullValue();
                }
            };
            ValueTypeBinaryReference.prototype.valueToString = function (value) {
                if (value.isNotNull()) {
                    return value.getBinaryReference().toString();
                }
                else {
                    return null;
                }
            };
            ValueTypeBinaryReference.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : value.getBinaryReference().toString();
            };
            ValueTypeBinaryReference.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.equals(a, b);
            };
            return ValueTypeBinaryReference;
        })(data.ValueType);
        data.ValueTypeBinaryReference = ValueTypeBinaryReference;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeReference = (function (_super) {
            __extends(ValueTypeReference, _super);
            function ValueTypeReference() {
                _super.call(this, "Reference");
            }
            ValueTypeReference.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, api.util.Reference)) {
                    return false;
                }
                return true;
            };
            ValueTypeReference.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                return true;
            };
            ValueTypeReference.prototype.newValue = function (value) {
                if (this.isConvertible(value)) {
                    return new data.Value(new api.util.Reference(value), this);
                }
                else {
                    return this.newNullValue();
                }
            };
            ValueTypeReference.prototype.valueToString = function (value) {
                if (value.isNotNull()) {
                    return value.getReference().toString();
                }
                else {
                    return null;
                }
            };
            ValueTypeReference.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : value.getReference().toString();
            };
            ValueTypeReference.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.equals(a, b);
            };
            return ValueTypeReference;
        })(data.ValueType);
        data.ValueTypeReference = ValueTypeReference;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypeLink = (function (_super) {
            __extends(ValueTypeLink, _super);
            function ValueTypeLink() {
                _super.call(this, "Link");
            }
            ValueTypeLink.prototype.isValid = function (value) {
                if (!(typeof value === 'object')) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(value, api.util.Link)) {
                    return false;
                }
                return true;
            };
            ValueTypeLink.prototype.isConvertible = function (value) {
                if (api.util.StringHelper.isBlank(value)) {
                    return false;
                }
                return true;
            };
            ValueTypeLink.prototype.newValue = function (value) {
                if (this.isConvertible(value)) {
                    return new data.Value(new api.util.Link(value), this);
                }
                else {
                    return this.newNullValue();
                }
            };
            ValueTypeLink.prototype.valueToString = function (value) {
                if (value.isNotNull()) {
                    return value.getLink().toString();
                }
                else {
                    return null;
                }
            };
            ValueTypeLink.prototype.toJsonValue = function (value) {
                return value.isNull() ? null : value.getLink().toString();
            };
            ValueTypeLink.prototype.valueEquals = function (a, b) {
                return api.ObjectHelper.equals(a, b);
            };
            return ValueTypeLink;
        })(data.ValueType);
        data.ValueTypeLink = ValueTypeLink;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var ValueTypes = (function () {
            function ValueTypes() {
            }
            ValueTypes.fromName = function (name) {
                for (var i = 0; i < ValueTypes.ALL.length; i++) {
                    var type = ValueTypes.ALL[i];
                    if (type.toString() == name) {
                        return type;
                    }
                }
                throw ("Unknown ValueType: " + name);
            };
            ValueTypes.DATA = new data.ValueTypePropertySet();
            ValueTypes.STRING = new data.ValueTypeString();
            ValueTypes.XML = new data.ValueTypeXml();
            ValueTypes.LOCAL_DATE = new data.ValueTypeLocalDate();
            ValueTypes.LOCAL_TIME = new data.ValueTypeLocalTime();
            ValueTypes.LOCAL_DATE_TIME = new data.ValueTypeLocalDateTime();
            ValueTypes.DATE_TIME = new data.ValueTypeDateTime();
            ValueTypes.LONG = new data.ValueTypeLong();
            ValueTypes.BOOLEAN = new data.ValueTypeBoolean();
            ValueTypes.DOUBLE = new data.ValueTypeDouble();
            ValueTypes.GEO_POINT = new data.ValueTypeGeoPoint();
            ValueTypes.REFERENCE = new data.ValueTypeReference();
            ValueTypes.BINARY_REFERENCE = new data.ValueTypeBinaryReference();
            ValueTypes.ALL = [
                ValueTypes.DATA,
                ValueTypes.STRING,
                ValueTypes.XML,
                ValueTypes.LOCAL_DATE,
                ValueTypes.LOCAL_TIME,
                ValueTypes.LOCAL_DATE_TIME,
                ValueTypes.DATE_TIME,
                ValueTypes.LONG,
                ValueTypes.BOOLEAN,
                ValueTypes.DOUBLE,
                ValueTypes.GEO_POINT,
                ValueTypes.REFERENCE,
                ValueTypes.BINARY_REFERENCE
            ];
            return ValueTypes;
        })();
        data.ValueTypes = ValueTypes;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var Value = (function () {
            function Value(value, type) {
                this.value = null;
                this.value = value;
                this.type = type;
                if (value) {
                    var isValid = this.type.isValid(value);
                    if (isValid == undefined) {
                        throw new Error(api.ClassHelper.getClassName(this.type) + ".isValid() did not return any value: " + isValid);
                    }
                    if (isValid == false) {
                        throw new Error("Invalid value for type " + type.toString() + ": " + value);
                    }
                }
            }
            Value.prototype.getType = function () {
                return this.type;
            };
            Value.prototype.isNotNull = function () {
                return !this.isNull();
            };
            Value.prototype.isNull = function () {
                return this.value == null || this.value == undefined;
            };
            Value.prototype.getObject = function () {
                return this.value;
            };
            Value.prototype.getString = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.type.valueToString(this);
            };
            Value.prototype.isPropertySet = function () {
                return data.ValueTypes.DATA.toString() == this.type.toString();
            };
            Value.prototype.getPropertySet = function () {
                if (this.isNull()) {
                    return null;
                }
                api.util.assert(api.ObjectHelper.iFrameSafeInstanceOf(this.value, data.PropertySet), "Expected value to be a PropertySet: " + api.ClassHelper.getClassName(this.value));
                return this.value;
            };
            Value.prototype.getBoolean = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.type.valueToBoolean(this);
            };
            Value.prototype.getLong = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.type.valueToNumber(this);
            };
            Value.prototype.getDouble = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.type.valueToNumber(this);
            };
            Value.prototype.getDateTime = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.getLocalDate = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.getLocalDateTime = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.getLocalTime = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.getGeoPoint = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.getBinaryReference = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.getReference = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.getLink = function () {
                if (this.isNull()) {
                    return null;
                }
                return this.value;
            };
            Value.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Value)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.type, other.type)) {
                    return false;
                }
                return this.type.valueEquals(this.value, other.value);
            };
            Value.prototype.clone = function () {
                return new Value(this.value, this.type);
            };
            return Value;
        })();
        data.Value = Value;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyPath = (function () {
            function PropertyPath(elements, absolute) {
                if (absolute === void 0) { absolute = true; }
                this.absolute = absolute;
                elements.forEach(function (element, index) {
                    if (element == null) {
                        throw new Error("Path element was null at index: " + index);
                    }
                    else if (element.getName().length == 0) {
                        throw new Error("Path element was empty string at index: " + index);
                    }
                });
                this.elements = elements;
                this.refString = (this.absolute ? PropertyPath.ELEMENT_DIVIDER : "") + this.elements.join(PropertyPath.ELEMENT_DIVIDER);
            }
            PropertyPath.fromString = function (s) {
                var absolute = s.charAt(0) == PropertyPath.ELEMENT_DIVIDER;
                var dataPathElements = s.split(PropertyPath.ELEMENT_DIVIDER).filter(function (element) { return !!element; }).map(function (element) { return PropertyPathElement.fromString(element); });
                return new PropertyPath(dataPathElements, absolute);
            };
            PropertyPath.fromParent = function (parent) {
                var childElements = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    childElements[_i - 1] = arguments[_i];
                }
                var elements = parent.elements.slice(0).concat(childElements);
                return new PropertyPath(elements, parent.isAbsolute());
            };
            PropertyPath.fromPathElement = function (element) {
                return new PropertyPath([element], true);
            };
            PropertyPath.prototype.removeFirstPathElement = function () {
                api.util.assert(this.elements.length > 1, "Cannot create new path without first path element when path does not contain more than one element");
                return new PropertyPath(this.elements.slice(1), this.absolute);
            };
            PropertyPath.prototype.elementCount = function () {
                return this.getElements().length;
            };
            PropertyPath.prototype.getElements = function () {
                return this.elements;
            };
            PropertyPath.prototype.getElement = function (index) {
                return this.elements[index];
            };
            PropertyPath.prototype.getFirstElement = function () {
                return this.elements[0];
            };
            PropertyPath.prototype.getLastElement = function () {
                return this.elements[this.elements.length - 1];
            };
            PropertyPath.prototype.hasParent = function () {
                return this.elements.length > 0;
            };
            PropertyPath.prototype.getParentPath = function () {
                if (this.elements.length < 1) {
                    return null;
                }
                return new PropertyPath(this.elements.slice(0, -1));
            };
            PropertyPath.prototype.toString = function () {
                return this.refString;
            };
            PropertyPath.prototype.isAbsolute = function () {
                return this.absolute;
            };
            PropertyPath.prototype.asRelative = function () {
                return new PropertyPath(this.elements, false);
            };
            PropertyPath.prototype.isRoot = function () {
                return this.elementCount() == 0;
            };
            PropertyPath.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PropertyPath)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                    return false;
                }
                return true;
            };
            PropertyPath.ELEMENT_DIVIDER = ".";
            PropertyPath.ROOT = new PropertyPath([], true);
            return PropertyPath;
        })();
        data.PropertyPath = PropertyPath;
        var PropertyPathElement = (function () {
            function PropertyPathElement(name, index) {
                this.name = name;
                this.index = index;
            }
            PropertyPathElement.prototype.getName = function () {
                return this.name;
            };
            PropertyPathElement.prototype.getIndex = function () {
                return this.index;
            };
            PropertyPathElement.prototype.toString = function () {
                if (this.index == 0) {
                    return this.name;
                }
                else {
                    return this.name + "[" + this.index + "]";
                }
            };
            PropertyPathElement.fromString = function (str) {
                if (str.indexOf("[") == -1) {
                    return new PropertyPathElement(str, 0);
                }
                var name = str.substring(0, str.indexOf("["));
                var index = parseInt(str.substring(str.indexOf("[") + 1, str.indexOf("]")));
                return new PropertyPathElement(name, index);
            };
            return PropertyPathElement;
        })();
        data.PropertyPathElement = PropertyPathElement;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyEvent = (function () {
            function PropertyEvent(type, property) {
                this.type = type;
                this.property = property;
            }
            PropertyEvent.prototype.getType = function () {
                return this.type;
            };
            PropertyEvent.prototype.getProperty = function () {
                return this.property;
            };
            PropertyEvent.prototype.getPath = function () {
                return this.property.getPath();
            };
            PropertyEvent.prototype.toString = function () {
                return this.getPath().toString();
            };
            return PropertyEvent;
        })();
        data.PropertyEvent = PropertyEvent;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyIndexChangedEvent = (function (_super) {
            __extends(PropertyIndexChangedEvent, _super);
            function PropertyIndexChangedEvent(property, previousIndex, newIndex) {
                _super.call(this, 1 /* INDEX_CHANGED */, property);
                this.previousIndex = previousIndex;
                this.newIndex = newIndex;
            }
            PropertyIndexChangedEvent.prototype.getPreviousIndex = function () {
                return this.previousIndex;
            };
            PropertyIndexChangedEvent.prototype.getNewIndex = function () {
                return this.newIndex;
            };
            PropertyIndexChangedEvent.prototype.toString = function () {
                return "[" + this.previousIndex + "] -> [" + this.newIndex + "]";
            };
            return PropertyIndexChangedEvent;
        })(data.PropertyEvent);
        data.PropertyIndexChangedEvent = PropertyIndexChangedEvent;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyValueChangedEvent = (function (_super) {
            __extends(PropertyValueChangedEvent, _super);
            function PropertyValueChangedEvent(property, previousValue, newValue) {
                _super.call(this, 2 /* VALUE_CHANGED */, property);
                this.previousValue = previousValue;
                this.newValue = newValue;
            }
            PropertyValueChangedEvent.prototype.getPreviousValue = function () {
                return this.previousValue;
            };
            PropertyValueChangedEvent.prototype.getNewValue = function () {
                return this.newValue;
            };
            PropertyValueChangedEvent.prototype.toString = function () {
                return "[" + (this.previousValue ? this.previousValue.getObject() : null) + "] -> [" + (this.newValue ? this.newValue.getObject() : null) + "]";
            };
            return PropertyValueChangedEvent;
        })(data.PropertyEvent);
        data.PropertyValueChangedEvent = PropertyValueChangedEvent;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        (function (PropertyEventType) {
            PropertyEventType[PropertyEventType["ADDED"] = 0] = "ADDED";
            PropertyEventType[PropertyEventType["INDEX_CHANGED"] = 1] = "INDEX_CHANGED";
            PropertyEventType[PropertyEventType["VALUE_CHANGED"] = 2] = "VALUE_CHANGED";
            PropertyEventType[PropertyEventType["REMOVED"] = 3] = "REMOVED";
        })(data.PropertyEventType || (data.PropertyEventType = {}));
        var PropertyEventType = data.PropertyEventType;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyAddedEvent = (function (_super) {
            __extends(PropertyAddedEvent, _super);
            function PropertyAddedEvent(property) {
                _super.call(this, 0 /* ADDED */, property);
            }
            PropertyAddedEvent.prototype.toString = function () {
                var value = this.getProperty().getValue();
                return "" + this.getPath().toString() + " = " + (value.getObject() ? value.getObject().toString() : null);
            };
            return PropertyAddedEvent;
        })(data.PropertyEvent);
        data.PropertyAddedEvent = PropertyAddedEvent;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyRemovedEvent = (function (_super) {
            __extends(PropertyRemovedEvent, _super);
            function PropertyRemovedEvent(property) {
                _super.call(this, 3 /* REMOVED */, property);
            }
            PropertyRemovedEvent.prototype.toString = function () {
                var value = this.getProperty().getValue();
                return "" + this.getPath().toString() + " = " + (value.getObject() ? value.getObject().toString() : null);
            };
            return PropertyRemovedEvent;
        })(data.PropertyEvent);
        data.PropertyRemovedEvent = PropertyRemovedEvent;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var Property = (function () {
            function Property(builder) {
                this.propertyIndexChangedListeners = [];
                this.propertyValueChangedListeners = [];
                api.util.assertNotNull(builder.array, "array of a Property cannot be null");
                api.util.assertNotNull(builder.name, "name of a Property cannot be null");
                api.util.assertNotNull(builder.index, "index of a Property cannot be null");
                api.util.assertNotNull(builder.value, "value of a Property cannot be null");
                this.array = builder.array;
                this.parent = builder.array.getParent();
                this.name = builder.name;
                this.index = builder.index;
                this.value = builder.value;
                if (this.value.getType().equals(data.ValueTypes.DATA) && this.value.isNotNull()) {
                    var valuePropertySet = this.value.getPropertySet();
                    valuePropertySet.setContainerProperty(this);
                }
            }
            Property.prototype.setIndex = function (newIndex) {
                var oldIndex = newIndex;
                this.index = newIndex;
                if (oldIndex != newIndex) {
                    this.notifyPropertyIndexChangedEvent(oldIndex, newIndex);
                }
            };
            Property.prototype.setValue = function (value) {
                api.util.assertNotNull(value, "value of a Property cannot be null");
                var oldValue = this.value;
                this.value = value;
                if (this.value.getType().equals(data.ValueTypes.DATA) && this.value.isNotNull()) {
                    var propertySet = this.value.getPropertySet();
                    propertySet.setContainerProperty(this);
                    this.array.registerPropertySetListeners(propertySet);
                }
                if (oldValue.getType().equals(data.ValueTypes.DATA) && oldValue.isNotNull()) {
                    var removedPropertySet = oldValue.getPropertySet();
                    removedPropertySet.setContainerProperty(null);
                    this.array.unregisterPropertySetListeners(removedPropertySet);
                }
                if (!value.equals(oldValue)) {
                    this.notifyPropertyValueChangedEvent(oldValue, value);
                }
            };
            Property.prototype.detach = function () {
                this.array = null;
                this.parent = null;
                this.propertyIndexChangedListeners = [];
                this.propertyValueChangedListeners = [];
            };
            Property.prototype.getParent = function () {
                return this.parent;
            };
            Property.prototype.hasParentProperty = function () {
                return !!this.getParentProperty();
            };
            Property.prototype.getParentProperty = function () {
                return this.parent.getProperty();
            };
            Property.prototype.getPath = function () {
                if (this.hasParentProperty()) {
                    return data.PropertyPath.fromParent(this.getParentProperty().getPath(), new data.PropertyPathElement(this.name, this.index));
                }
                else {
                    return data.PropertyPath.fromPathElement(new data.PropertyPathElement(this.name, this.index));
                }
            };
            Property.prototype.getName = function () {
                return this.name;
            };
            Property.prototype.getIndex = function () {
                return this.index;
            };
            Property.prototype.getType = function () {
                return this.value.getType();
            };
            Property.prototype.getValue = function () {
                return this.value;
            };
            Property.prototype.hasNullValue = function () {
                return this.value.isNull();
            };
            Property.prototype.hasNonNullValue = function () {
                return !this.value.isNull();
            };
            Property.prototype.getPropertySet = function () {
                return this.value.getPropertySet();
            };
            Property.prototype.getString = function () {
                return this.value.getString();
            };
            Property.prototype.getLong = function () {
                return this.value.getLong();
            };
            Property.prototype.getDouble = function () {
                return this.value.getDouble();
            };
            Property.prototype.getBoolean = function () {
                return this.value.getBoolean();
            };
            Property.prototype.getDateTime = function () {
                return this.value.getDateTime();
            };
            Property.prototype.getLocalDate = function () {
                return this.value.getLocalDate();
            };
            Property.prototype.getLocalDateTime = function () {
                return this.value.getLocalDateTime();
            };
            Property.prototype.getLocalTime = function () {
                return this.value.getLocalTime();
            };
            Property.prototype.getGeoPoint = function () {
                return this.value.getGeoPoint();
            };
            Property.prototype.getReference = function () {
                return this.value.getReference();
            };
            Property.prototype.getBinaryReference = function () {
                return this.value.getBinaryReference();
            };
            Property.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Property)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.name, other.name)) {
                    return false;
                }
                if (!api.ObjectHelper.numberEquals(this.index, other.index)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.value, other.value)) {
                    return false;
                }
                return true;
            };
            Property.prototype.copy = function (destinationPropertyArray) {
                var destinationTree = destinationPropertyArray.getTree();
                var value;
                if (this.value.isPropertySet() && this.value.isNotNull()) {
                    var copiedPropertySet = this.value.getPropertySet().copy(destinationTree);
                    value = new data.Value(copiedPropertySet, data.ValueTypes.DATA);
                }
                else {
                    value = this.value;
                }
                var copy = Property.create().setName(this.name).setValue(value).setIndex(this.index).setArray(destinationPropertyArray).build();
                return copy;
            };
            Property.prototype.onPropertyIndexChanged = function (listener) {
                this.propertyIndexChangedListeners.push(listener);
            };
            Property.prototype.unPropertyIndexChanged = function (listener) {
                this.propertyIndexChangedListeners = this.propertyIndexChangedListeners.filter(function (curr) { return (curr != listener); });
            };
            Property.prototype.notifyPropertyIndexChangedEvent = function (previousIndex, newIndex) {
                var event = new data.PropertyIndexChangedEvent(this, previousIndex, newIndex);
                if (Property.debug) {
                    console.debug("Property[" + this.getPath().toString() + "].notifyPropertyIndexChangedEvent: " + event.toString());
                }
                this.propertyIndexChangedListeners.forEach(function (listener) { return listener(event); });
            };
            Property.prototype.onPropertyValueChanged = function (listener) {
                this.propertyValueChangedListeners.push(listener);
            };
            Property.prototype.unPropertyValueChanged = function (listener) {
                this.propertyValueChangedListeners = this.propertyValueChangedListeners.filter(function (curr) { return (curr != listener); });
            };
            Property.prototype.notifyPropertyValueChangedEvent = function (previousValue, newValue) {
                var event = new data.PropertyValueChangedEvent(this, previousValue, newValue);
                if (Property.debug) {
                    console.debug("Property[" + this.getPath().toString() + "].notifyPropertyValueChangedEvent: " + event.toString());
                }
                this.propertyValueChangedListeners.forEach(function (listener) { return listener(event); });
            };
            Property.checkName = function (name) {
                if (name == null) {
                    throw new Error("Property name cannot be null");
                }
                if (api.util.StringHelper.isBlank(name)) {
                    throw new Error("Property name cannot be blank");
                }
                if (name.indexOf(".") >= 0) {
                    throw new Error("Property name cannot contain .");
                }
                if (name.indexOf("[") >= 0 || name.indexOf("]") >= 0) {
                    throw new Error("Property name cannot contain [ or ]");
                }
            };
            Property.create = function () {
                return new PropertyBuilder();
            };
            Property.debug = false;
            return Property;
        })();
        data.Property = Property;
        var PropertyBuilder = (function () {
            function PropertyBuilder() {
            }
            PropertyBuilder.prototype.setArray = function (value) {
                this.array = value;
                return this;
            };
            PropertyBuilder.prototype.setName = function (value) {
                this.name = value;
                return this;
            };
            PropertyBuilder.prototype.setIndex = function (value) {
                this.index = value;
                return this;
            };
            PropertyBuilder.prototype.setValue = function (value) {
                this.value = value;
                return this;
            };
            PropertyBuilder.prototype.build = function () {
                return new Property(this);
            };
            return PropertyBuilder;
        })();
        data.PropertyBuilder = PropertyBuilder;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyVisitor = (function () {
            function PropertyVisitor() {
            }
            PropertyVisitor.prototype.restrictType = function (value) {
                this.valueType = value;
                return this;
            };
            PropertyVisitor.prototype.traverse = function (propertySet) {
                var _this = this;
                propertySet.forEach(function (property, index) {
                    if (_this.valueType == null || _this.valueType == property.getType()) {
                        _this.visit(property);
                    }
                    if (property.getType().equals(data.ValueTypes.DATA)) {
                        if (property.hasNonNullValue()) {
                            _this.traverse(property.getPropertySet());
                        }
                    }
                });
            };
            PropertyVisitor.prototype.visit = function (property) {
                throw new Error("Must be implemented by inheritor");
            };
            return PropertyVisitor;
        })();
        data.PropertyVisitor = PropertyVisitor;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyTreeComparator = (function () {
            function PropertyTreeComparator() {
            }
            PropertyTreeComparator.prototype.compareTree = function (treeA, treeB) {
                if (!treeA || !treeB) {
                    return;
                }
                this.compareSet(treeA.getRoot(), treeB.getRoot());
            };
            PropertyTreeComparator.prototype.compareSet = function (setA, setB) {
                var _this = this;
                setA.forEach(function (propertyA, index) {
                    var propertyB = setB.getProperty(propertyA.getName(), index);
                    if (!propertyB) {
                        console.log("Property A[" + propertyA.getPath().toString() + "] does not exist in B[" + setB.getPropertyPath().toString() + "]");
                    }
                    if (!propertyA.equals(propertyB)) {
                        console.log("Property A[" + propertyA.getPath().toString() + "] is not equal B[" + propertyB.getPath().toString() + "]");
                    }
                    if (propertyA.getValue().isPropertySet()) {
                        var childSetA = propertyA.getPropertySet();
                        var childSetB = propertyB.getPropertySet();
                        _this.compareSet(childSetA, childSetB);
                    }
                });
                setB.forEach(function (propertyB, index) {
                    var propertyA = setA.getProperty(propertyB.getName(), index);
                    if (!propertyA) {
                        console.log("Property B[" + propertyB.getPath().toString() + "] does not exist in A[" + setA.getPropertyPath().toString() + "]");
                    }
                });
            };
            return PropertyTreeComparator;
        })();
        data.PropertyTreeComparator = PropertyTreeComparator;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyArray = (function () {
            function PropertyArray(builder) {
                var _this = this;
                this.propertyAddedListeners = [];
                this.propertyRemovedListeners = [];
                this.propertyIndexChangedListeners = [];
                this.propertyValueChangedListeners = [];
                this.tree = builder.parent.getTree();
                this.parent = builder.parent;
                this.name = builder.name;
                this.type = builder.type;
                this.array = [];
                this.propertyAddedEventHandler = function (event) {
                    _this.forwardPropertyAddedEvent(event);
                };
                this.propertyRemovedEventHandler = function (event) {
                    _this.forwardPropertyRemovedEvent(event);
                };
                this.propertyIndexChangedEventHandler = function (event) {
                    _this.forwardPropertyIndexChangedEvent(event);
                };
                this.propertyValueChangedEventHandler = function (event) {
                    _this.forwardPropertyValueChangedEvent(event);
                };
            }
            PropertyArray.prototype.forEach = function (callBack) {
                this.array.forEach(function (property, index) {
                    callBack(property, index);
                });
            };
            PropertyArray.prototype.containsValue = function (value) {
                var result = false;
                this.forEach(function (property) {
                    if (api.ObjectHelper.equals(property.getValue(), value)) {
                        result = true;
                    }
                });
                return result;
            };
            PropertyArray.prototype.getTree = function () {
                return this.tree;
            };
            PropertyArray.prototype.getParent = function () {
                return this.parent;
            };
            PropertyArray.prototype.getParentPropertyPath = function () {
                if (this.parent.getProperty() == null) {
                    return data.PropertyPath.ROOT;
                }
                return this.parent.getProperty().getPath();
            };
            PropertyArray.prototype.getName = function () {
                return this.name;
            };
            PropertyArray.prototype.getType = function () {
                return this.type;
            };
            PropertyArray.prototype.checkType = function (type) {
                if (!this.type.equals(type)) {
                    throw new Error("This PropertyArray expects only properties with value of type '" + this.type + "', got: " + type);
                }
            };
            PropertyArray.prototype.checkIndex = function (index) {
                if (!(index <= this.array.length)) {
                    throw new Error("Index out of bounds: index: " + index + ", size: " + this.array.length);
                }
            };
            PropertyArray.prototype.newSet = function () {
                return this.parent.newSet();
            };
            PropertyArray.prototype.addProperty = function (property) {
                api.util.assert(property.getName() == this.name, "Expected name of added Property to be [" + this.name + "], got: " + property.getName());
                api.util.assert(property.getType().equals(this.getType()), "Expected type of added Property to be [" + this.type.toString() + "], got: " + property.getType().toString());
                api.util.assert(property.getIndex() == this.array.length, "Expected index of added Property to be [" + this.array.length + "], got: " + property.getIndex());
                this.array.push(property);
                this.registerPropertyListeners(property);
            };
            PropertyArray.prototype.add = function (value) {
                this.checkType(value.getType());
                var property = data.Property.create().setArray(this).setName(this.name).setIndex(this.array.length).setValue(value).build();
                this.array.push(property);
                if (this.tree) {
                    if (this.type.equals(data.ValueTypes.DATA) && value.isNotNull()) {
                        var addedPropertySet = value.getPropertySet();
                        if (addedPropertySet && addedPropertySet.isDetached()) {
                            addedPropertySet.attachToTree(this.tree);
                        }
                    }
                }
                this.notifyPropertyAdded(property);
                this.registerPropertyListeners(property);
                return property;
            };
            PropertyArray.prototype.addSet = function () {
                var newSet = this.parent.newSet();
                this.add(new data.Value(newSet, data.ValueTypes.DATA));
                return newSet;
            };
            PropertyArray.prototype.set = function (index, value) {
                this.checkType(value.getType());
                this.checkIndex(index);
                var property;
                if (this.get(index) != null) {
                    property = this.array[index];
                    property.setValue(value);
                }
                else {
                    property = data.Property.create().setArray(this).setName(this.name).setIndex(this.array.length).setValue(value).build();
                    this.array[index] = property;
                    this.notifyPropertyAdded(property);
                    this.registerPropertyListeners(property);
                }
                return property;
            };
            PropertyArray.prototype.move = function (index, destinationIndex) {
                var toBeMoved = this.array[index];
                api.util.ArrayHelper.moveElement(index, destinationIndex, this.array);
                toBeMoved.setIndex(destinationIndex);
                this.forEach(function (property, index) {
                    property.setIndex(index);
                });
            };
            PropertyArray.prototype.remove = function (index) {
                var propertyToRemove = this.get(index);
                if (!propertyToRemove) {
                    throw new Error("Property not found: " + data.PropertyPath.fromParent(this.getParentPropertyPath(), new data.PropertyPathElement(name, index)));
                }
                this.array.splice(index, 1);
                this.forEach(function (property, index) {
                    property.setIndex(index);
                });
                this.notifyPropertyRemoved(propertyToRemove);
                this.unregisterPropertyListeners(propertyToRemove);
                propertyToRemove.detach();
            };
            PropertyArray.prototype.exists = function (index) {
                return !!this.get(index);
            };
            PropertyArray.prototype.get = function (index) {
                if (index >= this.array.length) {
                    return null;
                }
                return this.array[index];
            };
            PropertyArray.prototype.getValue = function (index) {
                var property = this.get(index);
                return !property ? null : property.getValue();
            };
            PropertyArray.prototype.getSet = function (index) {
                var property = this.get(index);
                return !property ? null : property.getPropertySet();
            };
            PropertyArray.prototype.getSize = function () {
                return this.array.length;
            };
            PropertyArray.prototype.isEmpty = function () {
                return this.array.length === 0;
            };
            PropertyArray.prototype.getProperties = function () {
                return this.array.slice(0);
            };
            PropertyArray.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PropertyArray)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.name, other.name)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.type, other.type)) {
                    return false;
                }
                if (!api.ObjectHelper.arrayEquals(this.array, other.array)) {
                    return false;
                }
                return true;
            };
            PropertyArray.prototype.copy = function (destinationPropertySet) {
                var copy = PropertyArray.create().setName(this.name).setType(this.type).setParent(destinationPropertySet).build();
                this.array.forEach(function (sourceProperty) {
                    copy.addProperty(sourceProperty.copy(copy));
                });
                return copy;
            };
            PropertyArray.prototype.registerPropertyListeners = function (property) {
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].registerPropertyListeners: " + property.getPath().toString());
                }
                if (this.type.equals(data.ValueTypes.DATA) && property.hasNonNullValue()) {
                    this.registerPropertySetListeners(property.getPropertySet());
                }
                property.onPropertyIndexChanged(this.propertyIndexChangedEventHandler);
                property.onPropertyValueChanged(this.propertyValueChangedEventHandler);
            };
            PropertyArray.prototype.unregisterPropertyListeners = function (property) {
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].unregisterPropertyListeners: " + property.getPath().toString());
                }
                property.unPropertyIndexChanged(this.propertyIndexChangedEventHandler);
                property.unPropertyValueChanged(this.propertyValueChangedEventHandler);
                if (property.hasNonNullValue() && property.getType().equals(data.ValueTypes.DATA)) {
                    var propertySet = property.getPropertySet();
                    this.unregisterPropertySetListeners(propertySet);
                }
            };
            PropertyArray.prototype.registerPropertySetListeners = function (propertySet) {
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].registerPropertySetListeners: " + propertySet.getPropertyPath().toString());
                }
                propertySet.onPropertyAdded(this.propertyAddedEventHandler);
                propertySet.onPropertyRemoved(this.propertyRemovedEventHandler);
                propertySet.onPropertyIndexChanged(this.propertyIndexChangedEventHandler);
                propertySet.onPropertyValueChanged(this.propertyValueChangedEventHandler);
            };
            PropertyArray.prototype.unregisterPropertySetListeners = function (propertySet) {
                propertySet.unPropertyAdded(this.propertyAddedEventHandler);
                propertySet.unPropertyRemoved(this.propertyRemovedEventHandler);
                propertySet.unPropertyIndexChanged(this.propertyIndexChangedEventHandler);
                propertySet.unPropertyValueChanged(this.propertyValueChangedEventHandler);
            };
            PropertyArray.prototype.onPropertyAdded = function (listener) {
                this.propertyAddedListeners.push(listener);
            };
            PropertyArray.prototype.unPropertyAdded = function (listener) {
                this.propertyAddedListeners = this.propertyAddedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertyArray.prototype.notifyPropertyAdded = function (property) {
                var event = new data.PropertyAddedEvent(property);
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].notifyPropertyAdded: " + event.toString());
                }
                this.propertyAddedListeners.forEach(function (listener) { return listener(event); });
            };
            PropertyArray.prototype.forwardPropertyAddedEvent = function (event) {
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].forwardPropertyAddedEvent: " + event.toString());
                }
                this.propertyAddedListeners.forEach(function (listener) { return listener(event); });
            };
            PropertyArray.prototype.onPropertyRemoved = function (listener) {
                this.propertyRemovedListeners.push(listener);
            };
            PropertyArray.prototype.unPropertyRemoved = function (listener) {
                this.propertyRemovedListeners = this.propertyRemovedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertyArray.prototype.notifyPropertyRemoved = function (property) {
                var event = new data.PropertyRemovedEvent(property);
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].notifyPropertyRemoved: " + event.toString());
                }
                this.propertyRemovedListeners.forEach(function (listener) { return listener(event); });
            };
            PropertyArray.prototype.forwardPropertyRemovedEvent = function (event) {
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].forwardPropertyRemovedEvent: " + event.toString());
                }
                this.propertyRemovedListeners.forEach(function (listener) { return listener(event); });
            };
            PropertyArray.prototype.onPropertyIndexChanged = function (listener) {
                this.propertyIndexChangedListeners.push(listener);
            };
            PropertyArray.prototype.unPropertyIndexChanged = function (listener) {
                this.propertyIndexChangedListeners = this.propertyIndexChangedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertyArray.prototype.forwardPropertyIndexChangedEvent = function (event) {
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].forwardPropertyIndexChangedEvent: " + event.toString());
                }
                this.propertyIndexChangedListeners.forEach(function (listener) { return listener(event); });
            };
            PropertyArray.prototype.onPropertyValueChanged = function (listener) {
                this.propertyValueChangedListeners.push(listener);
            };
            PropertyArray.prototype.unPropertyValueChanged = function (listener) {
                this.propertyValueChangedListeners = this.propertyValueChangedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertyArray.prototype.forwardPropertyValueChangedEvent = function (event) {
                if (PropertyArray.debug) {
                    console.debug("PropertyArray[" + this.idForDebug() + "].forwardPropertyValueChangedEvent: : " + event.toString());
                }
                this.propertyValueChangedListeners.forEach(function (listener) { return listener(event); });
            };
            PropertyArray.create = function () {
                return new PropertyArrayBuilder();
            };
            PropertyArray.prototype.toJson = function () {
                var _this = this;
                var valuesJson = [];
                this.array.forEach(function (property) {
                    if (_this.type.equals(data.ValueTypes.DATA)) {
                        var valueSetJson = property.hasNullValue() ? null : property.getPropertySet().toJson();
                        valuesJson.push({
                            set: valueSetJson
                        });
                    }
                    else {
                        var valueJson = _this.type.toJsonValue(property.getValue());
                        valuesJson.push({
                            v: valueJson
                        });
                    }
                });
                return {
                    name: this.name,
                    type: this.type.toString(),
                    values: valuesJson
                };
            };
            PropertyArray.prototype.idForDebug = function () {
                return this.getParentPropertyPath().toString() + "." + this.getName();
            };
            PropertyArray.fromJson = function (json, parentPropertySet, tree) {
                var type = data.ValueTypes.fromName(json.type);
                var array = new PropertyArrayBuilder().setName(json.name).setType(type).setParent(parentPropertySet).build();
                json.values.forEach(function (propertyValueJson, index) {
                    var value;
                    if (type.equals(data.ValueTypes.DATA)) {
                        var valueAsPropertySet = tree.newPropertySet();
                        var propertyArrayJsonArray = propertyValueJson.set;
                        propertyArrayJsonArray.forEach(function (propertyArrayJson) {
                            valueAsPropertySet.addPropertyArray(PropertyArray.fromJson(propertyArrayJson, valueAsPropertySet, tree));
                        });
                        value = new data.Value(valueAsPropertySet, data.ValueTypes.DATA);
                    }
                    else {
                        value = type.fromJsonValue(propertyValueJson.v);
                    }
                    var property = data.Property.create().setArray(array).setName(json.name).setIndex(index).setValue(value).build();
                    array.addProperty(property);
                });
                return array;
            };
            PropertyArray.debug = false;
            return PropertyArray;
        })();
        data.PropertyArray = PropertyArray;
        var PropertyArrayBuilder = (function () {
            function PropertyArrayBuilder() {
            }
            PropertyArrayBuilder.prototype.setParent = function (value) {
                this.parent = value;
                return this;
            };
            PropertyArrayBuilder.prototype.setName = function (value) {
                this.name = value;
                return this;
            };
            PropertyArrayBuilder.prototype.setType = function (value) {
                this.type = value;
                return this;
            };
            PropertyArrayBuilder.prototype.build = function () {
                return new PropertyArray(this);
            };
            return PropertyArrayBuilder;
        })();
        data.PropertyArrayBuilder = PropertyArrayBuilder;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertySet = (function () {
            function PropertySet(tree) {
                var _this = this;
                this.tree = null;
                this.property = null;
                this.propertyArrayByName = {};
                this._ifNotNull = false;
                this.changedListeners = [];
                this.propertyAddedListeners = [];
                this.propertyRemovedListeners = [];
                this.propertyIndexChangedListeners = [];
                this.propertyValueChangedListeners = [];
                this.tree = tree;
                this.propertyAddedEventHandler = function (event) {
                    _this.forwardPropertyAddedEvent(event);
                };
                this.propertyRemovedEventHandler = function (event) {
                    _this.forwardPropertyRemovedEvent(event);
                };
                this.propertyIndexChangedEventHandler = function (event) {
                    _this.forwardPropertyIndexChangedEvent(event);
                };
                this.propertyValueChangedEventHandler = function (event) {
                    _this.forwardPropertyValueChangedEvent(event);
                };
            }
            PropertySet.prototype.setContainerProperty = function (value) {
                this.property = value;
            };
            PropertySet.prototype.isDetached = function () {
                return !this.tree;
            };
            PropertySet.prototype.getTree = function () {
                return this.tree;
            };
            PropertySet.prototype.attachToTree = function (tree) {
                this.tree = tree;
                this.forEach(function (property) {
                    if (property.hasNonNullValue() && property.getType().equals(data.ValueTypes.DATA)) {
                        property.getPropertySet().attachToTree(tree);
                    }
                });
            };
            PropertySet.prototype.addPropertyArray = function (array) {
                api.util.assertState(this.tree === array.getTree(), "Added PropertyArray must be attached to the same PropertyTree as this PropertySet");
                api.util.assert(this == array.getParent(), "propertyArray must have this PropertySet as parent");
                this.propertyArrayByName[array.getName()] = array;
                this.registerPropertyArrayListeners(array);
            };
            PropertySet.prototype.addProperty = function (name, value) {
                if (this._ifNotNull && value.isNull()) {
                    this._ifNotNull = false;
                    return null;
                }
                var array = this.getOrCreatePropertyArray(name, value.getType());
                var property = array.add(value);
                return property;
            };
            PropertySet.prototype.setPropertyByPath = function (path, value) {
                if (api.ObjectHelper.iFrameSafeInstanceOf(path, data.PropertyPath)) {
                    return this.doSetProperty(path, value);
                }
                else {
                    return this.doSetProperty(data.PropertyPath.fromString(path.toString()), value);
                }
            };
            PropertySet.prototype.doSetProperty = function (path, value) {
                var firstPathElement = path.getFirstElement();
                if (path.elementCount() > 1) {
                    var propertySet = this.getOrCreateSet(firstPathElement.getName(), firstPathElement.getIndex());
                    return propertySet.setPropertyByPath(path.removeFirstPathElement(), value);
                }
                else {
                    return this.setProperty(firstPathElement.getName(), firstPathElement.getIndex(), value);
                }
            };
            PropertySet.prototype.getOrCreateSet = function (name, index) {
                var existingProperty = this.getProperty(name, index);
                if (!existingProperty) {
                    var newSet = this.tree ? new PropertySet(this.tree) : new PropertySet();
                    this.setProperty(name, index, new data.Value(newSet, data.ValueTypes.DATA));
                    return newSet;
                }
                else {
                    return existingProperty.getPropertySet();
                }
            };
            PropertySet.prototype.setProperty = function (name, index, value) {
                var array = this.getOrCreatePropertyArray(name, value.getType());
                return array.set(index, value);
            };
            PropertySet.prototype.getOrCreatePropertyArray = function (name, type) {
                var array = this.propertyArrayByName[name];
                if (!array) {
                    array = data.PropertyArray.create().setParent(this).setName(name).setType(type).build();
                    this.propertyArrayByName[name] = array;
                    this.registerPropertyArrayListeners(array);
                }
                return array;
            };
            PropertySet.prototype.removeProperties = function (properties) {
                var _this = this;
                properties.forEach(function (property) {
                    _this.removeProperty(property.getName(), property.getIndex());
                });
            };
            PropertySet.prototype.removeProperty = function (name, index) {
                var array = this.propertyArrayByName[name];
                if (array) {
                    array.remove(index);
                }
                if (!array || array.isEmpty()) {
                    delete this.propertyArrayByName[name];
                }
            };
            PropertySet.prototype.isEmpty = function () {
                var isEmpty = true;
                for (var name in this.propertyArrayByName) {
                    if (!isEmpty) {
                        return isEmpty;
                    }
                    if (this.propertyArrayByName.hasOwnProperty(name)) {
                        var propertyArray = this.propertyArrayByName[name];
                        propertyArray.forEach(function (property) {
                            if (!isEmpty) {
                                return;
                            }
                            var type = property.getType();
                            if (property.hasNullValue()) {
                                return;
                            }
                            if (type.equals(api.data.ValueTypes.STRING) && property.getValue().getString() === '') {
                                return;
                            }
                            if (type.equals(api.data.ValueTypes.BOOLEAN) && property.getValue().getBoolean() === false) {
                                return;
                            }
                            if (type.equals(api.data.ValueTypes.DATA) && !property.getValue().getPropertySet().isEmpty()) {
                                return;
                            }
                            isEmpty = false;
                        });
                    }
                }
                return isEmpty;
            };
            PropertySet.prototype.removeEmptyValues = function () {
                this.doRemoveEmptyValues(this);
            };
            PropertySet.prototype.doRemoveEmptyValues = function (propertySet) {
                var _this = this;
                var toRemove = [];
                propertySet.forEach(function (property) {
                    var type = property.getType();
                    if (property.hasNullValue()) {
                        toRemove.push(property);
                    }
                    else if (type.equals(api.data.ValueTypes.STRING) && (property.getValue().getString() === '')) {
                        toRemove.push(property);
                    }
                    else if (type.equals(api.data.ValueTypes.DATA)) {
                        var propertySetValue = property.getValue().getPropertySet();
                        _this.doRemoveEmptyValues(propertySetValue);
                        if (propertySetValue.isEmpty()) {
                            toRemove.push(property);
                        }
                    }
                    else if (type.equals(api.data.ValueTypes.BOOLEAN) && (property.getValue().getBoolean() == false)) {
                        toRemove.push(property);
                    }
                });
                this.removeProperties(toRemove);
                this.removeEmptyArrays();
            };
            PropertySet.prototype.removeEmptyArrays = function () {
                var _this = this;
                api.ObjectHelper.objectPropertyIterator(this.propertyArrayByName, function (name, propertyArray) {
                    if (propertyArray.isEmpty()) {
                        delete _this.propertyArrayByName[name];
                    }
                });
            };
            PropertySet.prototype.getSize = function () {
                var size = 0;
                api.ObjectHelper.objectPropertyIterator(this.propertyArrayByName, function (name, propertyArray) {
                    size += propertyArray.getSize();
                });
                return size;
            };
            PropertySet.prototype.countProperties = function (name) {
                var array = this.propertyArrayByName[name];
                if (!array) {
                    return 0;
                }
                return array.getSize();
            };
            PropertySet.prototype.getPropertyPath = function () {
                return !this.property ? data.PropertyPath.ROOT : this.property.getPath();
            };
            PropertySet.prototype.getProperty = function (identifier, index) {
                if (identifier == undefined && index == undefined) {
                    return this.property;
                }
                else if (index != undefined) {
                    data.Property.checkName(identifier);
                    var array = this.propertyArrayByName[identifier];
                    if (!array) {
                        return null;
                    }
                    return array.get(index);
                }
                else {
                    return this.getPropertyByPath(identifier);
                }
            };
            PropertySet.prototype.getPropertyByPath = function (path) {
                if (api.ObjectHelper.iFrameSafeInstanceOf(path, data.PropertyPath)) {
                    return this.doGetPropertyByPath(path);
                }
                else {
                    return this.doGetPropertyByPath(data.PropertyPath.fromString(path.toString()));
                }
            };
            PropertySet.prototype.doGetPropertyByPath = function (path) {
                var firstElement = path.getFirstElement();
                if (path.elementCount() > 1) {
                    var property = this.getProperty(firstElement.getName(), firstElement.getIndex());
                    if (!property) {
                        return null;
                    }
                    var propertySet = property.getPropertySet();
                    return propertySet.getPropertyByPath(path.removeFirstPathElement());
                }
                else {
                    return this.getProperty(firstElement.getName(), firstElement.getIndex());
                }
            };
            PropertySet.prototype.getPropertyArray = function (name) {
                return this.propertyArrayByName[name];
            };
            PropertySet.prototype.forEach = function (callback) {
                api.ObjectHelper.objectPropertyIterator(this.propertyArrayByName, function (name, propertyArray) {
                    propertyArray.forEach(function (property, index) {
                        callback(property, index);
                    });
                });
            };
            PropertySet.prototype.forEachProperty = function (propertyName, callback) {
                var array = this.getPropertyArray(propertyName);
                if (array) {
                    array.forEach(callback);
                }
            };
            PropertySet.prototype.isNotNull = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                if (property == null) {
                    return false;
                }
                return !property.hasNullValue();
            };
            PropertySet.prototype.isNull = function (identifier, index) {
                return !this.isNotNull(identifier, index);
            };
            PropertySet.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PropertySet)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.mapEquals(this.propertyArrayByName, other.propertyArrayByName)) {
                    return false;
                }
                return true;
            };
            PropertySet.prototype.copy = function (destinationTree) {
                var copy = new PropertySet(destinationTree);
                api.ObjectHelper.objectPropertyIterator(this.propertyArrayByName, function (name, sourcePropertyArray) {
                    var propertyArrayCopy = sourcePropertyArray.copy(copy);
                    copy.addPropertyArray(propertyArrayCopy);
                });
                return copy;
            };
            PropertySet.prototype.toJson = function () {
                var jsonArray = [];
                api.ObjectHelper.objectPropertyIterator(this.propertyArrayByName, function (name, propertyArray) {
                    jsonArray.push(propertyArray.toJson());
                });
                return jsonArray;
            };
            PropertySet.prototype.registerPropertyArrayListeners = function (array) {
                if (PropertySet.debug) {
                    console.debug("PropertySet[" + this.getPropertyPath().toString() + "].registerPropertyArrayListeners: " + array.getName());
                }
                array.onPropertyAdded(this.propertyAddedEventHandler);
                array.onPropertyRemoved(this.propertyRemovedEventHandler);
                array.onPropertyIndexChanged(this.propertyIndexChangedEventHandler);
                array.onPropertyValueChanged(this.propertyValueChangedEventHandler);
            };
            PropertySet.prototype.unregisterPropertyArrayListeners = function (array) {
                array.unPropertyAdded(this.propertyAddedEventHandler);
                array.unPropertyRemoved(this.propertyRemovedEventHandler);
                array.unPropertyIndexChanged(this.propertyIndexChangedEventHandler);
                array.unPropertyValueChanged(this.propertyValueChangedEventHandler);
            };
            PropertySet.prototype.onChanged = function (listener) {
                this.changedListeners.push(listener);
            };
            PropertySet.prototype.unChanged = function (listener) {
                this.changedListeners = this.changedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertySet.prototype.notifyChangedListeners = function (event) {
                if (PropertySet.debug) {
                    console.debug("PropertySet[" + this.getPropertyPath().toString() + "].notifyChangedListeners: " + event.toString());
                }
                this.changedListeners.forEach(function (listener) { return listener(event); });
            };
            PropertySet.prototype.onPropertyAdded = function (listener) {
                this.propertyAddedListeners.push(listener);
            };
            PropertySet.prototype.unPropertyAdded = function (listener) {
                this.propertyAddedListeners = this.propertyAddedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertySet.prototype.forwardPropertyAddedEvent = function (event) {
                this.propertyAddedListeners.forEach(function (listener) { return listener(event); });
                if (PropertySet.debug) {
                    console.debug("PropertySet[" + this.getPropertyPath().toString() + "].forwardPropertyAddedEvent: " + event.toString());
                }
                this.notifyChangedListeners(event);
            };
            PropertySet.prototype.onPropertyRemoved = function (listener) {
                this.propertyRemovedListeners.push(listener);
            };
            PropertySet.prototype.unPropertyRemoved = function (listener) {
                this.propertyRemovedListeners = this.propertyRemovedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertySet.prototype.forwardPropertyRemovedEvent = function (event) {
                if (PropertySet.debug) {
                    console.debug("PropertySet[" + this.getPropertyPath().toString() + "].forwardPropertyRemovedEvent: " + event.toString());
                }
                this.propertyRemovedListeners.forEach(function (listener) { return listener(event); });
                this.notifyChangedListeners(event);
            };
            PropertySet.prototype.onPropertyIndexChanged = function (listener) {
                this.propertyIndexChangedListeners.push(listener);
            };
            PropertySet.prototype.unPropertyIndexChanged = function (listener) {
                this.propertyIndexChangedListeners = this.propertyIndexChangedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertySet.prototype.forwardPropertyIndexChangedEvent = function (event) {
                if (PropertySet.debug) {
                    console.debug("PropertySet[" + this.getPropertyPath().toString() + "].forwardPropertyIndexChangedEvent: " + event.toString());
                }
                this.propertyIndexChangedListeners.forEach(function (listener) { return listener(event); });
                this.notifyChangedListeners(event);
            };
            PropertySet.prototype.onPropertyValueChanged = function (listener) {
                this.propertyValueChangedListeners.push(listener);
            };
            PropertySet.prototype.unPropertyValueChanged = function (listener) {
                this.propertyValueChangedListeners = this.propertyValueChangedListeners.filter(function (curr) { return (curr != listener); });
            };
            PropertySet.prototype.forwardPropertyValueChangedEvent = function (event) {
                if (PropertySet.debug) {
                    console.debug("PropertySet[" + this.getPropertyPath().toString() + "].forwardPropertyValueChangedEvent: " + event.toString());
                }
                this.propertyValueChangedListeners.forEach(function (listener) { return listener(event); });
                this.notifyChangedListeners(event);
            };
            PropertySet.prototype.newSet = function () {
                if (!this.tree) {
                    throw new Error("The PropertySet must be attached to a PropertyTree before this method can be invoked. Use PropertySet constructor with no arguments instead.");
                }
                return this.tree.newPropertySet();
            };
            PropertySet.prototype.addPropertySet = function (name, value) {
                if (!value) {
                    if (!this.tree) {
                        throw new Error("The PropertySet must be attached to a PropertyTree before this method can be invoked. Use PropertySet constructor with no arguments instead.");
                    }
                    value = this.tree.newPropertySet();
                }
                this.addProperty(name, new data.Value(value, data.ValueTypes.DATA));
                return value;
            };
            PropertySet.prototype.setPropertySet = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.DATA));
            };
            PropertySet.prototype.setPropertySetByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.DATA));
            };
            PropertySet.prototype.getPropertySet = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getPropertySet();
            };
            PropertySet.prototype.getPropertySets = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getPropertySet());
                });
                return values;
            };
            PropertySet.prototype.addString = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.STRING));
            };
            PropertySet.prototype.addStrings = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addString(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setString = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.STRING));
            };
            PropertySet.prototype.setStringByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.STRING));
            };
            PropertySet.prototype.getString = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getString();
            };
            PropertySet.prototype.getStrings = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getString());
                });
                return values;
            };
            PropertySet.prototype.addLong = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.LONG));
            };
            PropertySet.prototype.addLongs = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addLong(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setLong = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.LONG));
            };
            PropertySet.prototype.setLongByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.LONG));
            };
            PropertySet.prototype.getLong = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLong();
            };
            PropertySet.prototype.getLongs = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLong());
                });
                return values;
            };
            PropertySet.prototype.addDouble = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.DOUBLE));
            };
            PropertySet.prototype.addDoubles = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addDouble(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setDouble = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.DOUBLE));
            };
            PropertySet.prototype.setDoubleByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.DOUBLE));
            };
            PropertySet.prototype.getDouble = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getDouble();
            };
            PropertySet.prototype.getDoubles = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getDouble());
                });
                return values;
            };
            PropertySet.prototype.addBoolean = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.BOOLEAN));
            };
            PropertySet.prototype.addBooleans = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addBoolean(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setBoolean = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.BOOLEAN));
            };
            PropertySet.prototype.setBooleanByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.BOOLEAN));
            };
            PropertySet.prototype.getBoolean = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getBoolean();
            };
            PropertySet.prototype.getBooleans = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getBoolean());
                });
                return values;
            };
            PropertySet.prototype.addReference = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.REFERENCE));
            };
            PropertySet.prototype.addReferences = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addReference(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setReference = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.REFERENCE));
            };
            PropertySet.prototype.setReferenceByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.REFERENCE));
            };
            PropertySet.prototype.getReference = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getReference();
            };
            PropertySet.prototype.getReferences = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getReference());
                });
                return values;
            };
            PropertySet.prototype.addBinaryReference = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.BINARY_REFERENCE));
            };
            PropertySet.prototype.addBinaryReferences = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addBinaryReference(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setBinaryReference = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.BINARY_REFERENCE));
            };
            PropertySet.prototype.setBinaryReferenceByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.BINARY_REFERENCE));
            };
            PropertySet.prototype.getBinaryReference = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getBinaryReference();
            };
            PropertySet.prototype.getBinaryReferences = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getBinaryReference());
                });
                return values;
            };
            PropertySet.prototype.addGeoPoint = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.GEO_POINT));
            };
            PropertySet.prototype.addGeoPoints = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addGeoPoint(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setGeoPoint = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.GEO_POINT));
            };
            PropertySet.prototype.setGeoPointByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.GEO_POINT));
            };
            PropertySet.prototype.getGeoPoint = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getGeoPoint();
            };
            PropertySet.prototype.getGeoPoints = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getGeoPoint());
                });
                return values;
            };
            PropertySet.prototype.addLocalDate = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.LOCAL_DATE));
            };
            PropertySet.prototype.addLocalDates = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addLocalDate(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setLocalDate = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.LOCAL_DATE));
            };
            PropertySet.prototype.setLocalDateByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.LOCAL_DATE));
            };
            PropertySet.prototype.getLocalDate = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLocalDate();
            };
            PropertySet.prototype.getLocalDates = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLocalDate());
                });
                return values;
            };
            PropertySet.prototype.addLocalDateTime = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.LOCAL_DATE_TIME));
            };
            PropertySet.prototype.addLocalDateTimes = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addLocalDateTime(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setLocalDateTime = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.LOCAL_DATE_TIME));
            };
            PropertySet.prototype.setLocalDateTimeByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.LOCAL_DATE_TIME));
            };
            PropertySet.prototype.getLocalDateTime = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLocalDateTime();
            };
            PropertySet.prototype.getLocalDateTimes = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLocalDateTime());
                });
                return values;
            };
            PropertySet.prototype.addLocalTime = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.LOCAL_TIME));
            };
            PropertySet.prototype.addLocalTimes = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addLocalTime(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setLocalTime = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.LOCAL_TIME));
            };
            PropertySet.prototype.setLocalTimeByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.LOCAL_TIME));
            };
            PropertySet.prototype.getLocalTime = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLocalTime();
            };
            PropertySet.prototype.getLocalTimes = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLocalTime());
                });
                return values;
            };
            PropertySet.prototype.addDateTime = function (name, value) {
                return this.addProperty(name, new data.Value(value, data.ValueTypes.DATE_TIME));
            };
            PropertySet.prototype.addDateTimes = function (name, values) {
                var _this = this;
                var properties = [];
                values.forEach(function (value) {
                    properties.push(_this.addDateTime(name, value));
                });
                return properties;
            };
            PropertySet.prototype.setDateTime = function (name, index, value) {
                return this.setProperty(name, index, new data.Value(value, data.ValueTypes.DATE_TIME));
            };
            PropertySet.prototype.setDateTimeByPath = function (path, value) {
                return this.setPropertyByPath(path, new data.Value(value, data.ValueTypes.DATE_TIME));
            };
            PropertySet.prototype.getDateTime = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getDateTime();
            };
            PropertySet.prototype.getDateTimes = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getDateTime());
                });
                return values;
            };
            PropertySet.debug = false;
            return PropertySet;
        })();
        data.PropertySet = PropertySet;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var data;
    (function (data) {
        var PropertyTree = (function () {
            function PropertyTree(sourceRoot) {
                if (sourceRoot) {
                    this.root = sourceRoot.copy(this);
                }
                else {
                    this.root = new data.PropertySet(this);
                }
            }
            PropertyTree.prototype.getRoot = function () {
                return this.root;
            };
            PropertyTree.prototype.addProperty = function (name, value) {
                return this.root.addProperty(name, value);
            };
            PropertyTree.prototype.setPropertyByPath = function (path, value) {
                return this.root.setPropertyByPath(path, value);
            };
            PropertyTree.prototype.setProperty = function (name, index, value) {
                return this.root.setProperty(name, index, value);
            };
            PropertyTree.prototype.removeProperties = function (properties) {
                this.root.removeProperties(properties);
            };
            PropertyTree.prototype.removeProperty = function (name, index) {
                this.root.removeProperty(name, index);
            };
            PropertyTree.prototype.getProperty = function (identifier, index) {
                return this.root.getProperty(identifier, index);
            };
            PropertyTree.prototype.getPropertyArray = function (name) {
                return this.root.getPropertyArray(name);
            };
            PropertyTree.prototype.forEachProperty = function (name, callback) {
                this.root.forEachProperty(name, callback);
            };
            PropertyTree.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PropertyTree)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.root, other.root)) {
                    return false;
                }
                return true;
            };
            PropertyTree.prototype.copy = function () {
                return new PropertyTree(this.getRoot());
            };
            PropertyTree.prototype.toJson = function () {
                return this.getRoot().toJson();
            };
            PropertyTree.prototype.onChanged = function (listener) {
                this.root.onChanged(listener);
            };
            PropertyTree.prototype.unChanged = function (listener) {
                this.root.unChanged(listener);
            };
            PropertyTree.prototype.onPropertyAdded = function (listener) {
                this.root.onPropertyAdded(listener);
            };
            PropertyTree.prototype.unPropertyAdded = function (listener) {
                this.root.unPropertyAdded(listener);
            };
            PropertyTree.prototype.onPropertyRemoved = function (listener) {
                this.root.onPropertyRemoved(listener);
            };
            PropertyTree.prototype.unPropertyRemoved = function (listener) {
                this.root.unPropertyRemoved(listener);
            };
            PropertyTree.prototype.onPropertyIndexChanged = function (listener) {
                this.root.onPropertyIndexChanged(listener);
            };
            PropertyTree.prototype.unPropertyIndexChanged = function (listener) {
                this.root.unPropertyIndexChanged(listener);
            };
            PropertyTree.prototype.onPropertyValueChanged = function (listener) {
                this.root.onPropertyValueChanged(listener);
            };
            PropertyTree.prototype.unPropertyValueChanged = function (listener) {
                this.root.unPropertyValueChanged(listener);
            };
            PropertyTree.fromJson = function (json) {
                api.util.assertNotNull(json, "a json is required");
                var tree = new PropertyTree();
                json.forEach(function (propertyArrayJson) {
                    tree.root.addPropertyArray(data.PropertyArray.fromJson(propertyArrayJson, tree.root, tree));
                });
                return tree;
            };
            PropertyTree.prototype.newPropertySet = function () {
                return new data.PropertySet(this);
            };
            PropertyTree.prototype.addPropertySet = function (name, value) {
                return this.root.addPropertySet(name, value);
            };
            PropertyTree.prototype.setPropertySet = function (name, index, value) {
                return this.root.setPropertySet(name, index, value);
            };
            PropertyTree.prototype.setPropertySetByPath = function (path, value) {
                return this.root.setPropertySetByPath(path, value);
            };
            PropertyTree.prototype.getPropertySet = function (identifier, index) {
                return this.root.getPropertySet(identifier, index);
            };
            PropertyTree.prototype.getPropertySets = function (name) {
                return this.root.getPropertySets(name);
            };
            PropertyTree.prototype.addString = function (name, value) {
                return this.root.addString(name, value);
            };
            PropertyTree.prototype.addStrings = function (name, values) {
                return this.root.addStrings(name, values);
            };
            PropertyTree.prototype.setString = function (name, index, value) {
                return this.root.setString(name, index, value);
            };
            PropertyTree.prototype.setStringByPath = function (path, value) {
                return this.root.setStringByPath(path, value);
            };
            PropertyTree.prototype.getString = function (identifier, index) {
                return this.root.getString(identifier, index);
            };
            PropertyTree.prototype.getStrings = function (name) {
                return this.root.getStrings(name);
            };
            PropertyTree.prototype.addLong = function (name, value) {
                return this.root.addLong(name, value);
            };
            PropertyTree.prototype.addLongs = function (name, values) {
                return this.root.addLongs(name, values);
            };
            PropertyTree.prototype.setLong = function (name, index, value) {
                return this.root.setLong(name, index, value);
            };
            PropertyTree.prototype.setLongByPath = function (path, value) {
                return this.root.setLongByPath(path, value);
            };
            PropertyTree.prototype.getLong = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLong();
            };
            PropertyTree.prototype.getLongs = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLong());
                });
                return values;
            };
            PropertyTree.prototype.addDouble = function (name, value) {
                return this.root.addDouble(name, value);
            };
            PropertyTree.prototype.addDoubles = function (name, values) {
                return this.root.addDoubles(name, values);
            };
            PropertyTree.prototype.setDouble = function (name, index, value) {
                return this.root.setDouble(name, index, value);
            };
            PropertyTree.prototype.setDoubleByPath = function (path, value) {
                return this.root.setDoubleByPath(path, value);
            };
            PropertyTree.prototype.getDouble = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getDouble();
            };
            PropertyTree.prototype.getDoubles = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getDouble());
                });
                return values;
            };
            PropertyTree.prototype.addBoolean = function (name, value) {
                return this.root.addBoolean(name, value);
            };
            PropertyTree.prototype.addBooleans = function (name, values) {
                return this.root.addBooleans(name, values);
            };
            PropertyTree.prototype.setBoolean = function (name, index, value) {
                return this.root.setBoolean(name, index, value);
            };
            PropertyTree.prototype.setBooleanByPath = function (path, value) {
                return this.root.setBooleanByPath(path, value);
            };
            PropertyTree.prototype.getBoolean = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getBoolean();
            };
            PropertyTree.prototype.getBooleans = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getBoolean());
                });
                return values;
            };
            PropertyTree.prototype.addReference = function (name, value) {
                return this.root.addReference(name, value);
            };
            PropertyTree.prototype.addReferences = function (name, values) {
                return this.root.addReferences(name, values);
            };
            PropertyTree.prototype.setReference = function (name, index, value) {
                return this.root.setReference(name, index, value);
            };
            PropertyTree.prototype.setReferenceByPath = function (path, value) {
                return this.root.setReferenceByPath(path, value);
            };
            PropertyTree.prototype.getReference = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getReference();
            };
            PropertyTree.prototype.getReferences = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getReference());
                });
                return values;
            };
            PropertyTree.prototype.addBinaryReference = function (name, value) {
                return this.root.addBinaryReference(name, value);
            };
            PropertyTree.prototype.addBinaryReferences = function (name, values) {
                return this.root.addBinaryReferences(name, values);
            };
            PropertyTree.prototype.setBinaryReference = function (name, index, value) {
                return this.root.setBinaryReference(name, index, value);
            };
            PropertyTree.prototype.setBinaryReferenceByPath = function (path, value) {
                return this.root.setBinaryReferenceByPath(path, value);
            };
            PropertyTree.prototype.getBinaryReference = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getBinaryReference();
            };
            PropertyTree.prototype.getBinaryReferences = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getBinaryReference());
                });
                return values;
            };
            PropertyTree.prototype.addGeoPoint = function (name, value) {
                return this.root.addGeoPoint(name, value);
            };
            PropertyTree.prototype.addGeoPoints = function (name, values) {
                return this.root.addGeoPoints(name, values);
            };
            PropertyTree.prototype.setGeoPoint = function (name, index, value) {
                return this.root.setGeoPoint(name, index, value);
            };
            PropertyTree.prototype.setGeoPointByPath = function (path, value) {
                return this.root.setGeoPointByPath(path, value);
            };
            PropertyTree.prototype.getGeoPoint = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getGeoPoint();
            };
            PropertyTree.prototype.getGeoPoints = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getGeoPoint());
                });
                return values;
            };
            PropertyTree.prototype.addLocalDate = function (name, value) {
                return this.root.addLocalDate(name, value);
            };
            PropertyTree.prototype.addLocalDates = function (name, values) {
                return this.root.addLocalDates(name, values);
            };
            PropertyTree.prototype.setLocalDate = function (name, index, value) {
                return this.root.setLocalDate(name, index, value);
            };
            PropertyTree.prototype.setLocalDateByPath = function (path, value) {
                return this.root.setLocalDateByPath(path, value);
            };
            PropertyTree.prototype.getLocalDate = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLocalDate();
            };
            PropertyTree.prototype.getLocalDates = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLocalDate());
                });
                return values;
            };
            PropertyTree.prototype.addLocalDateTime = function (name, value) {
                return this.root.addLocalDateTime(name, value);
            };
            PropertyTree.prototype.addLocalDateTimes = function (name, values) {
                return this.root.addLocalDateTimes(name, values);
            };
            PropertyTree.prototype.setLocalDateTime = function (name, index, value) {
                return this.root.setLocalDateTime(name, index, value);
            };
            PropertyTree.prototype.setLocalDateTimeByPath = function (path, value) {
                return this.root.setLocalDateTimeByPath(path, value);
            };
            PropertyTree.prototype.getLocalDateTime = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLocalDateTime();
            };
            PropertyTree.prototype.getLocalDateTimes = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLocalDateTime());
                });
                return values;
            };
            PropertyTree.prototype.addLocalTime = function (name, value) {
                return this.root.addLocalTime(name, value);
            };
            PropertyTree.prototype.addLocalTimes = function (name, values) {
                return this.root.addLocalTimes(name, values);
            };
            PropertyTree.prototype.setLocalTime = function (name, index, value) {
                return this.root.setLocalTime(name, index, value);
            };
            PropertyTree.prototype.setLocalTimeByPath = function (path, value) {
                return this.root.setLocalTimeByPath(path, value);
            };
            PropertyTree.prototype.getLocalTime = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getLocalTime();
            };
            PropertyTree.prototype.getLocalTimes = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getLocalTime());
                });
                return values;
            };
            PropertyTree.prototype.addDateTime = function (name, value) {
                return this.root.addDateTime(name, value);
            };
            PropertyTree.prototype.addDateTimes = function (name, values) {
                return this.root.addDateTimes(name, values);
            };
            PropertyTree.prototype.setDateTime = function (name, index, value) {
                return this.root.setDateTime(name, index, value);
            };
            PropertyTree.prototype.setDateTimeByPath = function (path, value) {
                return this.root.setDateTimeByPath(path, value);
            };
            PropertyTree.prototype.getDateTime = function (identifier, index) {
                var property = this.getProperty(identifier, index);
                return !property ? null : property.getDateTime();
            };
            PropertyTree.prototype.getDateTimes = function (name) {
                var values = [];
                var array = this.getPropertyArray(name);
                array.forEach(function (property) {
                    values.push(property.getDateTime());
                });
                return values;
            };
            PropertyTree.prototype.isEmpty = function () {
                return this.root.isEmpty();
                ;
            };
            return PropertyTree;
        })();
        data.PropertyTree = PropertyTree;
    })(data = api.data || (api.data = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var FormItemJson = (function () {
                function FormItemJson() {
                }
                return FormItemJson;
            })();
            json.FormItemJson = FormItemJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var OccurrencesJson = (function () {
                function OccurrencesJson() {
                }
                return OccurrencesJson;
            })();
            json.OccurrencesJson = OccurrencesJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var LayoutJson = (function (_super) {
                __extends(LayoutJson, _super);
                function LayoutJson() {
                    _super.apply(this, arguments);
                }
                return LayoutJson;
            })(json.FormItemJson);
            json.LayoutJson = LayoutJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var FieldSetJson = (function (_super) {
                __extends(FieldSetJson, _super);
                function FieldSetJson() {
                    _super.apply(this, arguments);
                }
                return FieldSetJson;
            })(json.LayoutJson);
            json.FieldSetJson = FieldSetJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var FormItemSetJson = (function (_super) {
                __extends(FormItemSetJson, _super);
                function FormItemSetJson() {
                    _super.apply(this, arguments);
                }
                return FormItemSetJson;
            })(json.FormItemJson);
            json.FormItemSetJson = FormItemSetJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var InputJson = (function (_super) {
                __extends(InputJson, _super);
                function InputJson() {
                    _super.apply(this, arguments);
                }
                return InputJson;
            })(json.FormItemJson);
            json.InputJson = InputJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var json;
        (function (json) {
            var FormJson = (function () {
                function FormJson() {
                }
                return FormJson;
            })();
            json.FormJson = FormJson;
        })(json = form.json || (form.json = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemPath = (function () {
            function FormItemPath(elements, elementDivider, absolute) {
                this.elementDivider = elementDivider != null ? elementDivider : FormItemPath.DEFAULT_ELEMENT_DIVIDER;
                this.absolute = absolute == undefined ? true : absolute;
                elements.forEach(function (element, index) {
                    if (element == null) {
                        throw new Error("Path element was null at index: " + index);
                    }
                    else if (element.getName().length == 0) {
                        throw new Error("Path element was empty string at index: " + index);
                    }
                });
                this.elements = elements;
                this.refString = (this.absolute ? this.elementDivider : "") + this.elements.join(this.elementDivider);
            }
            FormItemPath.fromString = function (s, elementDivider) {
                if (elementDivider == null) {
                    elementDivider = FormItemPath.DEFAULT_ELEMENT_DIVIDER;
                }
                var absolute = s.charAt(0) == elementDivider;
                var elements = s.split(elementDivider);
                elements = FormItemPath.removeEmptyElements(elements);
                var pathElements = [];
                elements.forEach(function (s) {
                    pathElements.push(FormItemPathElement.fromString(s));
                });
                return new FormItemPath(pathElements, elementDivider, absolute);
            };
            FormItemPath.fromParent = function (parent) {
                var childElements = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    childElements[_i - 1] = arguments[_i];
                }
                var elements = parent.elements.slice(0);
                childElements.forEach(function (element) {
                    elements.push(element);
                });
                return new FormItemPath(elements, parent.elementDivider, parent.isAbsolute());
            };
            FormItemPath.removeEmptyElements = function (elements) {
                var filteredElements = [];
                elements.forEach(function (element) {
                    if (element.length > 0) {
                        filteredElements.push(element);
                    }
                });
                return filteredElements;
            };
            FormItemPath.prototype.newWithoutFirstElement = function () {
                var arr = this.elements;
                arr.shift();
                return new FormItemPath(arr);
            };
            FormItemPath.prototype.elementCount = function () {
                return this.getElements().length;
            };
            FormItemPath.prototype.getElements = function () {
                return this.elements;
            };
            FormItemPath.prototype.getElement = function (index) {
                return this.elements[index];
            };
            FormItemPath.prototype.getFirstElement = function () {
                return this.elements[0];
            };
            FormItemPath.prototype.getLastElement = function () {
                return this.elements[this.elements.length - 1];
            };
            FormItemPath.prototype.hasParent = function () {
                return this.elements.length > 0;
            };
            FormItemPath.prototype.getParentPath = function () {
                var _this = this;
                if (this.elements.length < 1) {
                    return null;
                }
                var parentElemements = [];
                this.elements.forEach(function (element, index) {
                    if (index < _this.elements.length - 1) {
                        parentElemements.push(element);
                    }
                });
                return new FormItemPath(parentElemements);
            };
            FormItemPath.prototype.toString = function () {
                return this.refString;
            };
            FormItemPath.prototype.isAbsolute = function () {
                return this.absolute;
            };
            FormItemPath.DEFAULT_ELEMENT_DIVIDER = ".";
            FormItemPath.ROOT = new FormItemPath([], FormItemPath.DEFAULT_ELEMENT_DIVIDER, true);
            return FormItemPath;
        })();
        form.FormItemPath = FormItemPath;
        var FormItemPathElement = (function () {
            function FormItemPathElement(name) {
                this.name = name;
            }
            FormItemPathElement.prototype.getName = function () {
                return this.name;
            };
            FormItemPathElement.prototype.toString = function () {
                return this.name;
            };
            FormItemPathElement.fromString = function (str) {
                return new FormItemPathElement(str);
            };
            return FormItemPathElement;
        })();
        form.FormItemPathElement = FormItemPathElement;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItem = (function () {
            function FormItem(name) {
                this.name = name;
            }
            FormItem.prototype.setParent = function (parent) {
                if (!(api.ObjectHelper.iFrameSafeInstanceOf(parent, form.FormItemSet) || api.ObjectHelper.iFrameSafeInstanceOf(parent, form.FieldSet))) {
                    throw new Error("A parent FormItem must either be a FormItemSet or a FieldSet");
                }
                this.parent = parent;
            };
            FormItem.prototype.getName = function () {
                return this.name;
            };
            FormItem.prototype.getPath = function () {
                return this.resolvePath();
            };
            FormItem.prototype.resolvePath = function () {
                return form.FormItemPath.fromParent(this.resolveParentPath(), form.FormItemPathElement.fromString(this.name));
            };
            FormItem.prototype.resolveParentPath = function () {
                if (this.parent == null) {
                    return form.FormItemPath.ROOT;
                }
                else {
                    return this.parent.getPath();
                }
            };
            FormItem.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, FormItem)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.name, other.name)) {
                    return false;
                }
                return true;
            };
            FormItem.prototype.toFormItemJson = function () {
                if (api.ObjectHelper.iFrameSafeInstanceOf(this, form.Input)) {
                    return this.toInputJson();
                }
                else if (api.ObjectHelper.iFrameSafeInstanceOf(this, form.FormItemSet)) {
                    return this.toFormItemSetJson();
                }
                else if (api.ObjectHelper.iFrameSafeInstanceOf(this, form.Layout)) {
                    return this.toLayoutJson();
                }
                else {
                    throw new Error("Unsupported FormItem: " + this);
                }
            };
            FormItem.formItemsToJson = function (formItems) {
                var formItemArray = [];
                formItems.forEach(function (formItem) {
                    formItemArray.push(formItem.toFormItemJson());
                });
                return formItemArray;
            };
            return FormItem;
        })();
        form.FormItem = FormItem;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var OccurrencesBuilder = (function () {
            function OccurrencesBuilder() {
            }
            OccurrencesBuilder.prototype.setMinimum = function (value) {
                this.minimum = value;
                return this;
            };
            OccurrencesBuilder.prototype.setMaximum = function (value) {
                this.maximum = value;
                return this;
            };
            OccurrencesBuilder.prototype.fromJson = function (json) {
                this.minimum = json.minimum;
                this.maximum = json.maximum;
            };
            OccurrencesBuilder.prototype.build = function () {
                return new Occurrences(this);
            };
            return OccurrencesBuilder;
        })();
        form.OccurrencesBuilder = OccurrencesBuilder;
        var Occurrences = (function () {
            function Occurrences(builder) {
                this.minimum = builder.minimum;
                this.maximum = builder.maximum;
            }
            Occurrences.fromJson = function (json) {
                var builder = new OccurrencesBuilder();
                builder.fromJson(json);
                return builder.build();
            };
            Occurrences.prototype.getMaximum = function () {
                return this.maximum;
            };
            Occurrences.prototype.getMinimum = function () {
                return this.minimum;
            };
            Occurrences.prototype.required = function () {
                return this.minimum > 0;
            };
            Occurrences.prototype.multiple = function () {
                return this.maximum > 1 || this.maximum == 0;
            };
            Occurrences.prototype.minimumReached = function (occurrenceCount) {
                return occurrenceCount >= this.minimum;
            };
            Occurrences.prototype.minimumBreached = function (occurrenceCount) {
                return (this.minimum == 0) ? false : (occurrenceCount < this.minimum);
            };
            Occurrences.prototype.maximumReached = function (occurrenceCount) {
                if (this.maximum == 0) {
                    return false;
                }
                return occurrenceCount >= this.maximum;
            };
            Occurrences.prototype.maximumBreached = function (occurrenceCount) {
                if (this.maximum == 0) {
                    return false;
                }
                return occurrenceCount > this.maximum;
            };
            Occurrences.prototype.toJson = function () {
                return {
                    maximum: this.getMaximum(),
                    minimum: this.getMinimum()
                };
            };
            Occurrences.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Occurrences)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.numberEquals(this.minimum, other.minimum)) {
                    return false;
                }
                if (!api.ObjectHelper.numberEquals(this.maximum, other.maximum)) {
                    return false;
                }
                return true;
            };
            return Occurrences;
        })();
        form.Occurrences = Occurrences;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var InputTypeName = (function () {
            function InputTypeName(name, custom) {
                this.name = name;
                this.custom = custom;
                if (this.custom) {
                    this.refString = InputTypeName.CUSTOM_PREFIX + name;
                }
                else {
                    this.refString = name;
                }
            }
            InputTypeName.parseInputTypeName = function (str) {
                if (str.substr(0, InputTypeName.CUSTOM_PREFIX.length) == InputTypeName.CUSTOM_PREFIX) {
                    return new InputTypeName(str.substr(InputTypeName.CUSTOM_PREFIX.length, str.length), true);
                }
                else {
                    return new InputTypeName(str, false);
                }
            };
            InputTypeName.prototype.getName = function () {
                return this.name;
            };
            InputTypeName.prototype.isBuiltIn = function () {
                return !this.custom;
            };
            InputTypeName.prototype.toString = function () {
                return this.refString;
            };
            InputTypeName.prototype.toJson = function () {
                return this.toString();
            };
            InputTypeName.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, InputTypeName)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.booleanEquals(this.custom, other.custom)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.name, other.name)) {
                    return false;
                }
                return true;
            };
            InputTypeName.CUSTOM_PREFIX = "custom:";
            return InputTypeName;
        })();
        form.InputTypeName = InputTypeName;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var InputBuilder = (function () {
            function InputBuilder() {
                this.immutable = false;
                this.indexed = true;
            }
            InputBuilder.prototype.setName = function (value) {
                this.name = value;
                return this;
            };
            InputBuilder.prototype.setInputType = function (value) {
                this.inputType = value;
                return this;
            };
            InputBuilder.prototype.setLabel = function (value) {
                this.label = value;
                return this;
            };
            InputBuilder.prototype.setImmutable = function (value) {
                this.immutable = value;
                return this;
            };
            InputBuilder.prototype.setOccurrences = function (value) {
                this.occurrences = value;
                return this;
            };
            InputBuilder.prototype.setIndexed = function (value) {
                this.indexed = value;
                return this;
            };
            InputBuilder.prototype.setCustomText = function (value) {
                this.customText = value;
                return this;
            };
            InputBuilder.prototype.setValidationRegex = function (value) {
                this.validationRegex = value;
                return this;
            };
            InputBuilder.prototype.setHelpText = function (value) {
                this.helpText = value;
                return this;
            };
            InputBuilder.prototype.setInputTypeConfig = function (value) {
                this.inputTypeConfig = value;
                return this;
            };
            InputBuilder.prototype.fromJson = function (json) {
                this.name = json.name;
                this.inputType = form.InputTypeName.parseInputTypeName(json.inputType);
                this.label = json.label;
                this.immutable = json.immutable;
                this.occurrences = form.Occurrences.fromJson(json.occurrences);
                this.indexed = json.indexed;
                this.customText = json.customText;
                this.validationRegex = json.validationRegexp;
                this.helpText = json.helpText;
                this.inputTypeConfig = json.config;
                this.maximizeUIInputWidth = json.maximizeUIInputWidth;
                return this;
            };
            InputBuilder.prototype.build = function () {
                return new Input(this);
            };
            return InputBuilder;
        })();
        form.InputBuilder = InputBuilder;
        var Input = (function (_super) {
            __extends(Input, _super);
            function Input(builder) {
                _super.call(this, builder.name);
                this.inputType = builder.inputType;
                this.inputTypeConfig = builder.inputTypeConfig;
                this.label = builder.label;
                this.immutable = builder.immutable;
                this.occurrences = builder.occurrences;
                this.indexed = builder.indexed;
                this.customText = builder.customText;
                this.validationRegex = builder.validationRegex;
                this.helpText = builder.helpText;
                this.maximizeUIInputWidth = builder.maximizeUIInputWidth;
            }
            Input.fromJson = function (json) {
                var builder = new InputBuilder();
                builder.fromJson(json);
                return builder.build();
            };
            Input.prototype.getInputType = function () {
                return this.inputType;
            };
            Input.prototype.getLabel = function () {
                return this.label;
            };
            Input.prototype.isImmutable = function () {
                return this.immutable;
            };
            Input.prototype.getOccurrences = function () {
                return this.occurrences;
            };
            Input.prototype.isIndexed = function () {
                return this.indexed;
            };
            Input.prototype.isMaximizeUIInputWidth = function () {
                return this.maximizeUIInputWidth;
            };
            Input.prototype.getCustomText = function () {
                return this.customText;
            };
            Input.prototype.getValidationRegex = function () {
                return this.validationRegex;
            };
            Input.prototype.getHelpText = function () {
                return this.helpText;
            };
            Input.prototype.getInputTypeConfig = function () {
                return this.inputTypeConfig;
            };
            Input.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Input)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.inputType, other.inputType)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.label, other.label)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.immutable, other.immutable)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.occurrences, other.occurrences)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.indexed, other.indexed)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.customText, other.customText)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.validationRegex, other.validationRegex)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.helpText, other.helpText)) {
                    return false;
                }
                if (!api.ObjectHelper.anyEquals(this.inputTypeConfig, other.inputTypeConfig)) {
                    return false;
                }
                return true;
            };
            Input.prototype.toInputJson = function () {
                return { Input: {
                    name: this.getName(),
                    customText: this.getCustomText(),
                    helpText: this.getHelpText(),
                    immutable: this.isImmutable(),
                    indexed: this.isIndexed(),
                    label: this.getLabel(),
                    occurrences: this.getOccurrences().toJson(),
                    validationRegexp: this.getValidationRegex(),
                    inputType: this.getInputType().toJson(),
                    config: this.getInputTypeConfig(),
                    maximizeUIInputWidth: this.isMaximizeUIInputWidth()
                } };
            };
            return Input;
        })(form.FormItem);
        form.Input = Input;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var Layout = (function (_super) {
            __extends(Layout, _super);
            function Layout(name) {
                _super.call(this, name);
            }
            Layout.prototype.toLayoutJson = function () {
                if (api.ObjectHelper.iFrameSafeInstanceOf(this, form.FieldSet)) {
                    return this.toFieldSetJson();
                }
                else {
                    throw new Error("Unsupported Layout: " + this);
                }
            };
            Layout.prototype.equals = function (o) {
                if (!(api.ObjectHelper.iFrameSafeInstanceOf(o, Layout))) {
                    return false;
                }
                return _super.prototype.equals.call(this, o);
            };
            return Layout;
        })(form.FormItem);
        form.Layout = Layout;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FieldSet = (function (_super) {
            __extends(FieldSet, _super);
            function FieldSet(fieldSetJson) {
                var _this = this;
                _super.call(this, fieldSetJson.name);
                this.formItems = [];
                this.label = fieldSetJson.label;
                if (fieldSetJson.items != null) {
                    fieldSetJson.items.forEach(function (formItemJson) {
                        _this.addFormItem(form.FormItemFactory.createFormItem(formItemJson));
                    });
                }
            }
            FieldSet.prototype.addFormItem = function (formItem) {
                this.formItems.push(formItem);
            };
            FieldSet.prototype.getLabel = function () {
                return this.label;
            };
            FieldSet.prototype.getFormItems = function () {
                return this.formItems;
            };
            FieldSet.prototype.toFieldSetJson = function () {
                return { FieldSet: {
                    name: this.getName(),
                    items: form.FormItem.formItemsToJson(this.getFormItems()),
                    label: this.getLabel()
                } };
            };
            FieldSet.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, FieldSet)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.label, other.label)) {
                    return false;
                }
                if (!api.ObjectHelper.arrayEquals(this.formItems, other.formItems)) {
                    return false;
                }
                return true;
            };
            return FieldSet;
        })(form.Layout);
        form.FieldSet = FieldSet;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemSet = (function (_super) {
            __extends(FormItemSet, _super);
            function FormItemSet(formItemSetJson) {
                var _this = this;
                _super.call(this, formItemSetJson.name);
                this.formItems = [];
                this.formItemByName = {};
                this.label = formItemSetJson.label;
                this.immutable = formItemSetJson.immutable;
                this.occurrences = form.Occurrences.fromJson(formItemSetJson.occurrences);
                this.customText = formItemSetJson.customText;
                this.helpText = formItemSetJson.helpText;
                if (formItemSetJson.items != null) {
                    formItemSetJson.items.forEach(function (formItemJson) {
                        _this.addFormItem(form.FormItemFactory.createFormItem(formItemJson));
                    });
                }
            }
            FormItemSet.prototype.addFormItem = function (formItem) {
                if (this.formItemByName[name]) {
                    throw new Error("FormItem already added: " + name);
                }
                formItem.setParent(this);
                this.formItemByName[formItem.getName()] = formItem;
                this.formItems.push(formItem);
            };
            FormItemSet.prototype.getFormItems = function () {
                return this.formItems;
            };
            FormItemSet.prototype.getFormItemByName = function (name) {
                return this.formItemByName[name];
            };
            FormItemSet.prototype.getInputByName = function (name) {
                return this.formItemByName[name];
            };
            FormItemSet.prototype.getLabel = function () {
                return this.label;
            };
            FormItemSet.prototype.isImmutable = function () {
                return this.immutable;
            };
            FormItemSet.prototype.getCustomText = function () {
                return this.customText;
            };
            FormItemSet.prototype.getHelpText = function () {
                return this.helpText;
            };
            FormItemSet.prototype.getOccurrences = function () {
                return this.occurrences;
            };
            FormItemSet.prototype.toFormItemSetJson = function () {
                return { FormItemSet: {
                    name: this.getName(),
                    customText: this.getCustomText(),
                    helpText: this.getHelpText(),
                    immutable: this.isImmutable(),
                    items: form.FormItem.formItemsToJson(this.getFormItems()),
                    label: this.getLabel(),
                    occurrences: this.getOccurrences().toJson(),
                } };
            };
            FormItemSet.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, FormItemSet)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.label, other.label)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.immutable, other.immutable)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.occurrences, other.occurrences)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.customText, other.customText)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.helpText, other.helpText)) {
                    return false;
                }
                if (!api.ObjectHelper.arrayEquals(this.formItems, other.formItems)) {
                    return false;
                }
                return true;
            };
            return FormItemSet;
        })(form.FormItem);
        form.FormItemSet = FormItemSet;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormValidityChangedEvent = (function () {
            function FormValidityChangedEvent(recording, atLeastOneInputValueBroken) {
                if (atLeastOneInputValueBroken === void 0) { atLeastOneInputValueBroken = false; }
                this.recording = recording;
                this.atLeastOneInputValueBroken = atLeastOneInputValueBroken;
            }
            FormValidityChangedEvent.prototype.isValid = function () {
                return this.recording.isValid() && !this.atLeastOneInputValueBroken;
            };
            FormValidityChangedEvent.prototype.getRecording = function () {
                return this.recording;
            };
            return FormValidityChangedEvent;
        })();
        form.FormValidityChangedEvent = FormValidityChangedEvent;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormBuilder = (function () {
            function FormBuilder() {
                this.formItems = [];
            }
            FormBuilder.prototype.addFormItem = function (formItem) {
                this.formItems.push(formItem);
                return this;
            };
            FormBuilder.prototype.addFormItems = function (formItems) {
                var _this = this;
                formItems.forEach(function (formItem) {
                    _this.addFormItem(formItem);
                });
                return this;
            };
            FormBuilder.prototype.fromJson = function (json) {
                var _this = this;
                json.formItems.forEach(function (formItemJson) {
                    _this.addFormItem(form.FormItemFactory.createFormItem(formItemJson));
                });
                return this;
            };
            FormBuilder.prototype.build = function () {
                return new Form(this);
            };
            return FormBuilder;
        })();
        form.FormBuilder = FormBuilder;
        var Form = (function () {
            function Form(builder) {
                var _this = this;
                this.formItems = [];
                this.formItemByName = {};
                builder.formItems.forEach(function (formItem) {
                    _this.addFormItem(formItem);
                });
            }
            Form.fromJson = function (json) {
                var builder = new FormBuilder();
                builder.fromJson(json);
                return builder.build();
            };
            Form.prototype.addFormItem = function (formItem) {
                if (this.formItemByName[name]) {
                    throw new Error("FormItem already added: " + name);
                }
                this.formItemByName[formItem.getName()] = formItem;
                this.formItems.push(formItem);
            };
            Form.prototype.getFormItems = function () {
                return this.formItems;
            };
            Form.prototype.getFormItemByName = function (name) {
                return this.formItemByName[name];
            };
            Form.prototype.getInputByName = function (name) {
                return this.formItemByName[name];
            };
            Form.prototype.toJson = function () {
                return {
                    formItems: form.FormItem.formItemsToJson(this.getFormItems())
                };
            };
            Form.prototype.equals = function (o) {
                if (!(api.ObjectHelper.iFrameSafeInstanceOf(o, Form))) {
                    return false;
                }
                var other = o;
                if (this.formItems.length != other.formItems.length) {
                    return false;
                }
                for (var i = 0; i < this.formItems.length; i++) {
                    if (!this.formItems[i].equals(other.formItems[i])) {
                        return false;
                    }
                }
                return true;
            };
            return Form;
        })();
        form.Form = Form;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemFactory = (function () {
            function FormItemFactory() {
            }
            FormItemFactory.createForm = function (formJson) {
                return form.Form.fromJson(formJson);
            };
            FormItemFactory.createFormItem = function (formItemTypeWrapperJson) {
                if (formItemTypeWrapperJson.Input) {
                    return FormItemFactory.createInput(formItemTypeWrapperJson.Input);
                }
                else if (formItemTypeWrapperJson.FormItemSet) {
                    return FormItemFactory.createFormItemSet(formItemTypeWrapperJson.FormItemSet);
                }
                else if (formItemTypeWrapperJson.FieldSet) {
                    return FormItemFactory.createFieldSetLayout(formItemTypeWrapperJson.FieldSet);
                }
                console.log("Unknown FormItem type: ", formItemTypeWrapperJson);
                throw new Error("Unknown FormItem");
            };
            FormItemFactory.createInput = function (inputJson) {
                return form.Input.fromJson(inputJson);
            };
            FormItemFactory.createFormItemSet = function (formItemSetJson) {
                return new form.FormItemSet(formItemSetJson);
            };
            FormItemFactory.createFieldSetLayout = function (fieldSetJson) {
                return new form.FieldSet(fieldSetJson);
            };
            return FormItemFactory;
        })();
        form.FormItemFactory = FormItemFactory;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var ValidationRecordingPath = (function () {
            function ValidationRecordingPath(parentPropertySet, dataName, min, max) {
                this.parentDataSet = parentPropertySet != null ? parentPropertySet.asRelative() : null;
                this.dataName = dataName;
                this.refString = this.resolveRefString();
                this.min = min;
                this.max = max;
            }
            ValidationRecordingPath.prototype.resolveRefString = function () {
                var s = "";
                if (this.parentDataSet && !this.parentDataSet.isRoot()) {
                    s += this.parentDataSet.toString();
                    if (this.dataName) {
                        s += "." + this.dataName;
                    }
                }
                else {
                    if (this.dataName) {
                        s += this.dataName;
                    }
                }
                return s;
            };
            ValidationRecordingPath.prototype.getParentDataSet = function () {
                return this.parentDataSet;
            };
            ValidationRecordingPath.prototype.getDataName = function () {
                return this.dataName;
            };
            ValidationRecordingPath.prototype.getMin = function () {
                return this.min;
            };
            ValidationRecordingPath.prototype.getMax = function () {
                return this.max;
            };
            ValidationRecordingPath.prototype.toString = function () {
                return this.refString;
            };
            return ValidationRecordingPath;
        })();
        form.ValidationRecordingPath = ValidationRecordingPath;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var RecordingValidityChangedEvent = (function () {
            function RecordingValidityChangedEvent(recording, origin, inputValueBroken) {
                if (inputValueBroken === void 0) { inputValueBroken = false; }
                this.recording = recording;
                this.origin = origin;
                this.inputValueBroken = inputValueBroken;
            }
            RecordingValidityChangedEvent.prototype.getOrigin = function () {
                return this.origin;
            };
            RecordingValidityChangedEvent.prototype.isValid = function () {
                return this.recording.isValid() && !this.inputValueBroken;
            };
            RecordingValidityChangedEvent.prototype.getRecording = function () {
                return this.recording;
            };
            RecordingValidityChangedEvent.prototype.isInputValueBroken = function () {
                return this.inputValueBroken;
            };
            return RecordingValidityChangedEvent;
        })();
        form.RecordingValidityChangedEvent = RecordingValidityChangedEvent;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var ValidationRecording = (function () {
            function ValidationRecording() {
                this.breaksMinimumOccurrencesArray = [];
                this.breaksMaximumOccurrencesArray = [];
            }
            ValidationRecording.prototype.breaksMinimumOccurrences = function (path) {
                if (!this.exists(path, this.breaksMinimumOccurrencesArray)) {
                    this.breaksMinimumOccurrencesArray.push(path);
                }
            };
            ValidationRecording.prototype.breaksMaximumOccurrences = function (path) {
                if (!this.exists(path, this.breaksMaximumOccurrencesArray)) {
                    this.breaksMaximumOccurrencesArray.push(path);
                }
            };
            ValidationRecording.prototype.isValid = function () {
                return this.breaksMinimumOccurrencesArray.length == 0 && this.breaksMaximumOccurrencesArray.length == 0;
            };
            ValidationRecording.prototype.isMinimumOccurrencesValid = function () {
                return this.breaksMinimumOccurrencesArray.length == 0;
            };
            ValidationRecording.prototype.isMaximumOccurrencesValid = function () {
                return this.breaksMaximumOccurrencesArray.length == 0;
            };
            ValidationRecording.prototype.flatten = function (recording) {
                var _this = this;
                recording.breaksMinimumOccurrencesArray.forEach(function (path) {
                    _this.breaksMinimumOccurrences(path);
                });
                recording.breaksMaximumOccurrencesArray.forEach(function (path) {
                    _this.breaksMaximumOccurrences(path);
                });
            };
            ValidationRecording.prototype.subtract = function (recording) {
                var _this = this;
                this.breaksMinimumOccurrencesArray = this.breaksMinimumOccurrencesArray.filter(function (path) {
                    return !_this.exists(path, recording.breaksMinimumOccurrencesArray);
                });
                this.breaksMaximumOccurrencesArray = this.breaksMaximumOccurrencesArray.filter(function (path) {
                    return !_this.exists(path, recording.breaksMaximumOccurrencesArray);
                });
            };
            ValidationRecording.prototype.removeByPath = function (path) {
                this.removeUnreachedMinimumOccurrencesByPath(path);
                this.removeBreachedMaximumOccurrencesByPath(path);
            };
            ValidationRecording.prototype.removeUnreachedMinimumOccurrencesByPath = function (path) {
                for (var i = this.breaksMinimumOccurrencesArray.length - 1; i >= 0; i--) {
                    if (this.breaksMinimumOccurrencesArray[i].toString().indexOf(path.toString()) == 0) {
                        this.breaksMinimumOccurrencesArray.splice(i, 1);
                    }
                }
            };
            ValidationRecording.prototype.removeBreachedMaximumOccurrencesByPath = function (path) {
                for (var i = this.breaksMaximumOccurrencesArray.length - 1; i >= 0; i--) {
                    if (this.breaksMaximumOccurrencesArray[i].toString().indexOf(path.toString()) == 0) {
                        this.breaksMaximumOccurrencesArray.splice(i, 1);
                    }
                }
            };
            ValidationRecording.prototype.equals = function (other) {
                if (this.breaksMinimumOccurrencesArray.length != other.breaksMinimumOccurrencesArray.length) {
                    return false;
                }
                else if (this.breaksMaximumOccurrencesArray.length != other.breaksMaximumOccurrencesArray.length) {
                    return false;
                }
                for (var i = 0; i < this.breaksMinimumOccurrencesArray.length; i++) {
                    if (this.breaksMinimumOccurrencesArray[i].toString() != other.breaksMinimumOccurrencesArray[i].toString()) {
                        return false;
                    }
                }
                for (var i = 0; i < this.breaksMaximumOccurrencesArray.length; i++) {
                    if (this.breaksMaximumOccurrencesArray[i].toString() != other.breaksMaximumOccurrencesArray[i].toString()) {
                        return false;
                    }
                }
                return true;
            };
            ValidationRecording.prototype.validityChanged = function (previous) {
                return previous == undefined || previous == null || !previous.equals(this);
            };
            ValidationRecording.prototype.exists = function (path, array) {
                for (var i = 0; i < array.length; i++) {
                    if (array[i].toString() == path.toString()) {
                        return true;
                    }
                }
                return false;
            };
            return ValidationRecording;
        })();
        form.ValidationRecording = ValidationRecording;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormContext = (function () {
            function FormContext(builder) {
                this.showEmptyFormItemSetOccurrences = builder.showEmptyFormItemSetOccurrences;
            }
            FormContext.prototype.getShowEmptyFormItemSetOccurrences = function () {
                return this.showEmptyFormItemSetOccurrences;
            };
            FormContext.create = function () {
                return new FormContextBuilder();
            };
            FormContext.prototype.createInputTypeViewContext = function (inputTypeConfig, parentPropertyPath, input) {
                return {
                    formContext: this,
                    input: input,
                    inputConfig: inputTypeConfig,
                    parentDataPath: parentPropertyPath
                };
            };
            return FormContext;
        })();
        form.FormContext = FormContext;
        var FormContextBuilder = (function () {
            function FormContextBuilder() {
            }
            FormContextBuilder.prototype.setShowEmptyFormItemSetOccurrences = function (value) {
                this.showEmptyFormItemSetOccurrences = value;
                return this;
            };
            FormContextBuilder.prototype.build = function () {
                return new FormContext(this);
            };
            return FormContextBuilder;
        })();
        form.FormContextBuilder = FormContextBuilder;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemOccurrenceView = (function (_super) {
            __extends(FormItemOccurrenceView, _super);
            function FormItemOccurrenceView(className, formItemOccurrence) {
                _super.call(this, className);
                this.removeButtonClickedListeners = [];
                this.formItemOccurrence = formItemOccurrence;
            }
            FormItemOccurrenceView.prototype.getDataPath = function () {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemOccurrenceView.prototype.layout = function () {
                return wemQ(null);
            };
            FormItemOccurrenceView.prototype.hasValidUserInput = function () {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemOccurrenceView.prototype.onRemoveButtonClicked = function (listener) {
                this.removeButtonClickedListeners.push(listener);
            };
            FormItemOccurrenceView.prototype.unRemoveButtonClicked = function (listener) {
                this.removeButtonClickedListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            FormItemOccurrenceView.prototype.notifyRemoveButtonClicked = function () {
                var _this = this;
                this.removeButtonClickedListeners.forEach(function (listener) {
                    listener.call(_this, new form.RemoveButtonClickedEvent(_this, _this.formItemOccurrence.getIndex()));
                });
            };
            FormItemOccurrenceView.prototype.getIndex = function () {
                return this.formItemOccurrence.getIndex();
            };
            FormItemOccurrenceView.prototype.refresh = function () {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemOccurrenceView.prototype.giveFocus = function () {
                return false;
            };
            return FormItemOccurrenceView;
        })(api.dom.DivEl);
        form.FormItemOccurrenceView = FormItemOccurrenceView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemOccurrences = (function () {
            function FormItemOccurrences(config) {
                this.occurrences = [];
                this.occurrenceViews = [];
                this.occurrenceAddedListeners = [];
                this.occurrenceRenderedListeners = [];
                this.occurrenceRemovedListeners = [];
                this.focusListeners = [];
                this.blurListeners = [];
                this.formItem = config.formItem;
                this.propertyArray = config.propertyArray;
                this.occurrenceViewContainer = config.occurrenceViewContainer;
                this.allowedOccurrences = config.allowedOccurrences;
            }
            FormItemOccurrences.prototype.getAllowedOccurrences = function () {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemOccurrences.prototype.onOccurrenceRendered = function (listener) {
                this.occurrenceRenderedListeners.push(listener);
            };
            FormItemOccurrences.prototype.unOccurrenceRendered = function (listener) {
                this.occurrenceRenderedListeners = this.occurrenceRenderedListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            FormItemOccurrences.prototype.notifyOccurrenceRendered = function (occurrence, occurrenceView) {
                var _this = this;
                this.occurrenceRenderedListeners.forEach(function (listener) {
                    listener.call(_this, new form.OccurrenceRenderedEvent(occurrence, occurrenceView));
                });
            };
            FormItemOccurrences.prototype.onOccurrenceAdded = function (listener) {
                this.occurrenceAddedListeners.push(listener);
            };
            FormItemOccurrences.prototype.unOccurrenceAdded = function (listener) {
                this.occurrenceAddedListeners = this.occurrenceAddedListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            FormItemOccurrences.prototype.notifyOccurrenceAdded = function (occurrence, occurrenceView) {
                var _this = this;
                this.occurrenceAddedListeners.forEach(function (listener) {
                    listener.call(_this, new form.OccurrenceAddedEvent(occurrence, occurrenceView));
                });
            };
            FormItemOccurrences.prototype.onOccurrenceRemoved = function (listener) {
                this.occurrenceRemovedListeners.push(listener);
            };
            FormItemOccurrences.prototype.unOccurrenceRemoved = function (listener) {
                this.occurrenceRemovedListeners = this.occurrenceRemovedListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            FormItemOccurrences.prototype.notifyOccurrenceRemoved = function (occurrence, occurrenceView) {
                var _this = this;
                this.occurrenceRemovedListeners.forEach(function (listener) {
                    listener.call(_this, new form.OccurrenceRemovedEvent(occurrence, occurrenceView));
                });
            };
            FormItemOccurrences.prototype.constructOccurrencesForNoData = function () {
                if (this.getAllowedOccurrences().getMinimum() > 0) {
                    for (var i = 0; i < this.getAllowedOccurrences().getMinimum(); i++) {
                        this.addOccurrence(this.createNewOccurrence(this, i));
                    }
                }
                else {
                    this.addOccurrence(this.createNewOccurrence(this, 0));
                }
            };
            FormItemOccurrences.prototype.addOccurrence = function (occurrence) {
                this.occurrences.push(occurrence);
            };
            FormItemOccurrences.prototype.getFormItem = function () {
                return this.formItem;
            };
            FormItemOccurrences.prototype.maximumOccurrencesReached = function () {
                return this.allowedOccurrences.maximumReached(this.countOccurrences());
            };
            FormItemOccurrences.prototype.layout = function () {
                var _this = this;
                var layoutPromises = [];
                this.occurrences.forEach(function (occurrence) {
                    var occurrenceView = _this.createNewOccurrenceView(occurrence);
                    _this.occurrenceViewContainer.appendChild(occurrenceView);
                    var layoutPromise = occurrenceView.layout();
                    layoutPromises.push(layoutPromise);
                    layoutPromise.then(function () {
                        occurrenceView.onFocus(function (event) { return _this.notifyFocused(event); });
                        occurrenceView.onBlur(function (event) { return _this.notifyBlurred(event); });
                        _this.occurrenceViews.push(occurrenceView);
                    });
                });
                return wemQ.all(layoutPromises).spread(function () { return wemQ(null); });
            };
            FormItemOccurrences.prototype.createNewOccurrenceView = function (occurrence) {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemOccurrences.prototype.createNewOccurrence = function (formItemOccurrences, insertAtIndex) {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemOccurrences.prototype.doRemoveOccurrence = function (occurrenceViewToRemove, indexToRemove) {
                if (!this.canRemove()) {
                    return;
                }
                occurrenceViewToRemove.remove();
                this.occurrenceViews = this.occurrenceViews.filter(function (curr) {
                    return curr != occurrenceViewToRemove;
                });
                var occurrenceToRemove = this.occurrences[indexToRemove];
                this.occurrences = this.occurrences.filter(function (curr) {
                    return curr.getIndex() != indexToRemove;
                });
                this.resetOccurrenceIndexes();
                this.refreshOccurrenceViews();
                this.propertyArray.remove(indexToRemove);
                this.notifyOccurrenceRemoved(occurrenceToRemove, occurrenceViewToRemove);
            };
            FormItemOccurrences.prototype.createAndAddOccurrence = function () {
                var _this = this;
                var insertAtIndex = this.countOccurrences();
                var occurrence = this.createNewOccurrence(this, insertAtIndex);
                this.doAddOccurrence(occurrence);
                var occurrenceView = this.occurrenceViews[occurrence.getIndex()];
                occurrenceView.onFocus(function (event) { return _this.notifyFocused(event); });
                occurrenceView.onBlur(function (event) { return _this.notifyBlurred(event); });
                if (occurrenceView) {
                    occurrenceView.giveFocus();
                }
            };
            FormItemOccurrences.prototype.onFocus = function (listener) {
                this.focusListeners.push(listener);
            };
            FormItemOccurrences.prototype.unFocus = function (listener) {
                this.focusListeners = this.focusListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            FormItemOccurrences.prototype.onBlur = function (listener) {
                this.blurListeners.push(listener);
            };
            FormItemOccurrences.prototype.unBlur = function (listener) {
                this.blurListeners = this.blurListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            FormItemOccurrences.prototype.notifyFocused = function (event) {
                this.focusListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            FormItemOccurrences.prototype.notifyBlurred = function (event) {
                this.blurListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            FormItemOccurrences.prototype.doAddOccurrence = function (occurrence) {
                var _this = this;
                var countOccurrences = this.countOccurrences();
                if (this.allowedOccurrences.maximumReached(countOccurrences)) {
                    return;
                }
                var occurrenceView = this.createNewOccurrenceView(occurrence);
                var insertAtIndex = occurrence.getIndex();
                this.occurrences.splice(insertAtIndex, 0, occurrence);
                if (insertAtIndex == countOccurrences) {
                    this.occurrenceViewContainer.appendChild(occurrenceView);
                }
                else {
                    var occurrenceViewBefore = this.getOccurrenceViewElementBefore(insertAtIndex);
                    occurrenceView.insertAfterEl(occurrenceViewBefore);
                }
                occurrenceView.layout().then(function () {
                    _this.notifyOccurrenceRendered(occurrence, occurrenceView);
                }).done();
                this.occurrenceViews.splice(insertAtIndex, 0, occurrenceView);
                this.resetOccurrenceIndexes();
                this.refreshOccurrenceViews();
                this.notifyOccurrenceAdded(occurrence, occurrenceView);
            };
            FormItemOccurrences.prototype.resetOccurrenceIndexes = function () {
                this.occurrences.forEach(function (currOccurrence, index) {
                    currOccurrence.setIndex(index);
                });
            };
            FormItemOccurrences.prototype.refreshOccurrenceViews = function () {
                this.occurrenceViews.forEach(function (currOccurrenceView) {
                    currOccurrenceView.refresh();
                });
            };
            FormItemOccurrences.prototype.getOccurrenceViewElementBefore = function (index) {
                if (index < 1) {
                    return null;
                }
                return this.occurrenceViews.filter(function (occurrenceView) {
                    return occurrenceView.getIndex() == index - 1;
                })[0];
            };
            FormItemOccurrences.prototype.countOccurrences = function () {
                return this.occurrences.length;
            };
            FormItemOccurrences.prototype.moveOccurrence = function (fromIndex, toIndex) {
                api.util.ArrayHelper.moveElement(fromIndex, toIndex, this.occurrences);
                this.occurrences.forEach(function (occurrence, index) {
                    occurrence.setIndex(index);
                });
                api.util.ArrayHelper.moveElement(fromIndex, toIndex, this.occurrenceViews);
                this.propertyArray.move(fromIndex, toIndex);
            };
            FormItemOccurrences.prototype.getOccurrences = function () {
                return this.occurrences;
            };
            FormItemOccurrences.prototype.canRemove = function () {
                return this.occurrences.length > Math.max(0, this.allowedOccurrences.getMinimum());
            };
            FormItemOccurrences.prototype.getOccurrenceViews = function () {
                return this.occurrenceViews;
            };
            return FormItemOccurrences;
        })();
        form.FormItemOccurrences = FormItemOccurrences;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemOccurrence = (function () {
            function FormItemOccurrence(occurrences, index, allowedOccurrences) {
                this.occurrences = occurrences;
                this.formItem = occurrences.getFormItem();
                this.allowedOccurrences = allowedOccurrences;
                this.index = index;
            }
            FormItemOccurrence.prototype.setIndex = function (value) {
                this.index = value;
            };
            FormItemOccurrence.prototype.getIndex = function () {
                return this.index;
            };
            FormItemOccurrence.prototype.isRemoveButtonRequired = function () {
                return this.moreThanRequiredOccurrences();
            };
            FormItemOccurrence.prototype.isRemoveButtonRequiredStrict = function () {
                return this.occurrences.countOccurrences() == 1 ? false : this.moreThanRequiredOccurrences();
            };
            FormItemOccurrence.prototype.showAddButton = function () {
                if (!this.isLastOccurrence()) {
                    return false;
                }
                return this.lessOccurrencesThanMaximumAllowed();
            };
            FormItemOccurrence.prototype.isMultiple = function () {
                return this.allowedOccurrences.multiple();
            };
            FormItemOccurrence.prototype.oneAndOnly = function () {
                return this.index == 0 && this.occurrences.countOccurrences() == 1;
            };
            FormItemOccurrence.prototype.moreThanRequiredOccurrences = function () {
                return this.occurrences.countOccurrences() > this.allowedOccurrences.getMinimum();
            };
            FormItemOccurrence.prototype.lessOccurrencesThanMaximumAllowed = function () {
                return !this.allowedOccurrences.maximumReached(this.occurrences.countOccurrences());
            };
            FormItemOccurrence.prototype.isLastOccurrence = function () {
                return this.index == this.occurrences.countOccurrences() - 1;
            };
            return FormItemOccurrence;
        })();
        form.FormItemOccurrence = FormItemOccurrence;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemView = (function (_super) {
            __extends(FormItemView, _super);
            function FormItemView(config) {
                _super.call(this, config.className);
                this.editContentRequestListeners = [];
                api.util.assertNotNull(config.context, "context cannot be null");
                api.util.assertNotNull(config.formItem, "formItem cannot be null");
                this.context = config.context;
                this.formItem = config.formItem;
                this.parent = config.parent;
            }
            FormItemView.prototype.broadcastFormSizeChanged = function () {
                throw new Error("Must be implemented by inheritors");
            };
            FormItemView.prototype.getContext = function () {
                return this.context;
            };
            FormItemView.prototype.getFormItem = function () {
                return this.formItem;
            };
            FormItemView.prototype.getParent = function () {
                return this.parent;
            };
            FormItemView.prototype.displayValidationErrors = function (value) {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemView.prototype.hasValidUserInput = function () {
                throw new Error("Must be implemented by inheritor");
            };
            FormItemView.prototype.validate = function (silent) {
                if (silent === void 0) { silent = true; }
                return new form.ValidationRecording();
            };
            FormItemView.prototype.giveFocus = function () {
                return false;
            };
            FormItemView.prototype.onEditContentRequest = function (listener) {
                this.editContentRequestListeners.push(listener);
            };
            FormItemView.prototype.unEditContentRequest = function (listener) {
                this.editContentRequestListeners = this.editContentRequestListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            FormItemView.prototype.notifyEditContentRequested = function (content) {
                this.editContentRequestListeners.forEach(function (listener) {
                    listener(content);
                });
            };
            FormItemView.prototype.onValidityChanged = function (listener) {
            };
            FormItemView.prototype.unValidityChanged = function (listener) {
            };
            return FormItemView;
        })(api.dom.DivEl);
        form.FormItemView = FormItemView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemLayer = (function () {
            function FormItemLayer() {
                this.formItemViews = [];
            }
            FormItemLayer.prototype.setFormContext = function (context) {
                this.context = context;
                return this;
            };
            FormItemLayer.prototype.setFormItems = function (formItems) {
                this.formItems = formItems;
                return this;
            };
            FormItemLayer.prototype.setParentElement = function (parentEl) {
                this.parentEl = parentEl;
                return this;
            };
            FormItemLayer.prototype.setParent = function (value) {
                this.parent = value;
                return this;
            };
            FormItemLayer.prototype.layout = function (propertySet) {
                var _this = this;
                this.formItemViews = [];
                return this.doLayoutPropertySet(propertySet).then(function () {
                    return wemQ(_this.formItemViews);
                });
            };
            FormItemLayer.prototype.doLayoutPropertySet = function (propertySet) {
                var _this = this;
                var layoutPromises = [];
                this.formItems.forEach(function (formItem) {
                    if (api.ObjectHelper.iFrameSafeInstanceOf(formItem, form.FormItemSet)) {
                        var formItemSet = formItem;
                        var propertyArray = propertySet.getPropertyArray(formItemSet.getName());
                        if (propertyArray && propertyArray.getSize() == 0) {
                            _this.context = form.FormContext.create().setShowEmptyFormItemSetOccurrences(false).build();
                        }
                        var formItemSetView = new form.FormItemSetView({
                            context: _this.context,
                            formItemSet: formItemSet,
                            parent: _this.parent,
                            parentDataSet: propertySet
                        });
                        _this.parentEl.appendChild(formItemSetView);
                        _this.formItemViews.push(formItemSetView);
                        layoutPromises.push(formItemSetView.layout());
                    }
                    else if (api.ObjectHelper.iFrameSafeInstanceOf(formItem, form.FieldSet)) {
                        var fieldSet = formItem;
                        var fieldSetView = new form.FieldSetView({
                            context: _this.context,
                            fieldSet: fieldSet,
                            parent: _this.parent,
                            dataSet: propertySet
                        });
                        _this.parentEl.appendChild(fieldSetView);
                        _this.formItemViews.push(fieldSetView);
                        layoutPromises.push(fieldSetView.layout());
                    }
                    else if (api.ObjectHelper.iFrameSafeInstanceOf(formItem, form.Input)) {
                        var input = formItem;
                        var inputView = new form.InputView({
                            context: _this.context,
                            input: input,
                            parent: _this.parent,
                            parentDataSet: propertySet
                        });
                        _this.parentEl.appendChild(inputView);
                        _this.formItemViews.push(inputView);
                        layoutPromises.push(inputView.layout());
                    }
                });
                return wemQ.all(layoutPromises).spread(function () {
                    return wemQ(null);
                });
            };
            return FormItemLayer;
        })();
        form.FormItemLayer = FormItemLayer;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var InputLabel = (function (_super) {
            __extends(InputLabel, _super);
            function InputLabel(input) {
                _super.call(this, "input-label");
                this.input = input;
                var wrapper = new api.dom.DivEl("wrapper");
                var label = new api.dom.DivEl("label");
                label.getEl().setInnerHtml(input.getLabel());
                wrapper.getEl().appendChild(label.getHTMLElement());
                if (input.getOccurrences().required()) {
                    wrapper.addClass("required");
                }
                this.getEl().appendChild(wrapper.getHTMLElement());
            }
            return InputLabel;
        })(api.dom.DivEl);
        form.InputLabel = InputLabel;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var PropertyArray = api.data.PropertyArray;
        var InputView = (function (_super) {
            __extends(InputView, _super);
            function InputView(config) {
                _super.call(this, {
                    className: "input-view",
                    context: config.context,
                    formItem: config.input,
                    parent: config.parent
                });
                this.validityChangedListeners = [];
                api.util.assertNotNull(config.parentDataSet, "parentDataSet not expected to be null");
                api.util.assertNotNull(config.input, "input not expected to be null");
                this.input = config.input;
                this.parentPropertySet = config.parentDataSet;
            }
            InputView.prototype.layout = function () {
                var _this = this;
                if (this.input.getLabel()) {
                    var label = new form.InputLabel(this.input);
                    this.appendChild(label);
                }
                else {
                    this.addClass("no-label");
                }
                if (this.input.isMaximizeUIInputWidth()) {
                    this.addClass("label-over-input");
                }
                this.inputTypeView = this.createInputTypeView();
                this.inputTypeView.onEditContentRequest(function (content) {
                    _this.notifyEditContentRequested(content);
                });
                this.propertyArray = this.parentPropertySet.getPropertyArray(this.input.getName());
                if (!this.propertyArray) {
                    this.propertyArray = PropertyArray.create().setType(this.inputTypeView.getValueType()).setName(this.input.getName()).setParent(this.parentPropertySet).build();
                    this.parentPropertySet.addPropertyArray(this.propertyArray);
                    var initialValue = this.inputTypeView.newInitialValue();
                    if (initialValue) {
                        this.propertyArray.add(initialValue);
                    }
                }
                var inputTypeViewLayoutPromise = this.inputTypeView.layout(this.input, this.propertyArray);
                inputTypeViewLayoutPromise.then(function () {
                    _this.appendChild(_this.inputTypeView.getElement());
                    if (!_this.inputTypeView.isManagingAdd()) {
                        var inputTypeViewNotManagingAdd = _this.inputTypeView;
                        inputTypeViewNotManagingAdd.onOccurrenceAdded(function () {
                            _this.refresh();
                        });
                        inputTypeViewNotManagingAdd.onOccurrenceRemoved(function (event) {
                            _this.refresh();
                            if (api.ObjectHelper.iFrameSafeInstanceOf(event.getOccurrenceView(), api.form.inputtype.support.InputOccurrenceView)) {
                                _this.validate(false);
                            }
                        });
                        _this.addButton = new api.ui.button.Button("Add");
                        _this.addButton.addClass("small");
                        _this.addButton.onClicked(function (event) {
                            inputTypeViewNotManagingAdd.createAndAddOccurrence();
                        });
                        _this.bottomButtonRow = new api.dom.DivEl("bottom-button-row");
                        _this.appendChild(_this.bottomButtonRow);
                        _this.bottomButtonRow.appendChild(_this.addButton);
                    }
                    _this.validationViewer = new form.ValidationRecordingViewer();
                    _this.appendChild(_this.validationViewer);
                    _this.inputTypeView.onValidityChanged(function (event) {
                        _this.handleInputValidationRecording(event.getRecording(), false);
                    });
                    _this.refresh();
                });
                return inputTypeViewLayoutPromise;
            };
            InputView.prototype.createInputTypeView = function () {
                var inputType = this.input.getInputType();
                var inputTypeViewContext = this.getContext().createInputTypeViewContext(this.input.getInputTypeConfig(), this.parentPropertySet.getPropertyPath(), this.input);
                if (form.inputtype.InputTypeManager.isRegistered(inputType.getName())) {
                    return form.inputtype.InputTypeManager.createView(inputType.getName(), inputTypeViewContext);
                }
                else {
                    console.warn("Input type [" + inputType.getName() + "] needs to be registered first.");
                    return form.inputtype.InputTypeManager.createView("NoInputTypeFound", inputTypeViewContext);
                }
            };
            InputView.prototype.broadcastFormSizeChanged = function () {
                if (this.isVisible()) {
                    this.inputTypeView.availableSizeChanged();
                }
            };
            InputView.prototype.refresh = function () {
                if (!this.inputTypeView.isManagingAdd()) {
                    var inputTypeViewNotManagingAdd = this.inputTypeView;
                    this.addButton.setVisible(!inputTypeViewNotManagingAdd.maximumOccurrencesReached());
                }
                this.validate(false);
            };
            InputView.prototype.resolveValidationRecordingPath = function () {
                return new form.ValidationRecordingPath(this.propertyArray.getParentPropertyPath(), this.input.getName(), this.input.getOccurrences().getMinimum(), this.input.getOccurrences().getMaximum());
            };
            InputView.prototype.displayValidationErrors = function (value) {
                this.inputTypeView.displayValidationErrors(value);
            };
            InputView.prototype.hasValidUserInput = function () {
                return this.inputTypeView.hasValidUserInput();
            };
            InputView.prototype.validate = function (silent) {
                if (silent === void 0) { silent = true; }
                var inputRecording = this.inputTypeView.validate(silent);
                return this.handleInputValidationRecording(inputRecording, silent);
            };
            InputView.prototype.handleInputValidationRecording = function (inputRecording, silent) {
                if (silent === void 0) { silent = true; }
                var recording = new form.ValidationRecording();
                var validationRecordingPath = this.resolveValidationRecordingPath();
                var hasValidInput = this.hasValidUserInput();
                if (inputRecording.isMinimumOccurrencesBreached()) {
                    recording.breaksMinimumOccurrences(validationRecordingPath);
                }
                if (inputRecording.isMaximumOccurrencesBreached()) {
                    recording.breaksMaximumOccurrences(validationRecordingPath);
                }
                if (!silent && (recording.validityChanged(this.previousValidityRecording) || this.userInputValidityChanged(hasValidInput))) {
                    this.notifyFormValidityChanged(new form.RecordingValidityChangedEvent(recording, validationRecordingPath, !hasValidInput));
                }
                this.renderValidationErrors(recording, inputRecording.getAdditionalValidationRecord());
                this.previousValidityRecording = recording;
                this.userInputValid = hasValidInput;
                return recording;
            };
            InputView.prototype.userInputValidityChanged = function (currentState) {
                return this.userInputValid == undefined || this.userInputValid == null || !(this.userInputValid == currentState);
            };
            InputView.prototype.giveFocus = function () {
                return this.inputTypeView.giveFocus();
            };
            InputView.prototype.onValidityChanged = function (listener) {
                this.validityChangedListeners.push(listener);
            };
            InputView.prototype.unValidityChanged = function (listener) {
                this.validityChangedListeners.filter(function (currentListener) {
                    return listener == currentListener;
                });
            };
            InputView.prototype.notifyFormValidityChanged = function (event) {
                this.validityChangedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            InputView.prototype.renderValidationErrors = function (recording, additionalValidationRecord) {
                if (recording.isValid()) {
                    this.removeClass("invalid");
                    this.addClass("valid");
                }
                else {
                    this.removeClass("valid");
                    this.addClass("invalid");
                }
                this.validationViewer.setObject(recording);
                if (additionalValidationRecord && additionalValidationRecord.isOverwriteDefault()) {
                    this.validationViewer.appendValidationMessage(additionalValidationRecord.getMessage());
                }
            };
            InputView.prototype.onFocus = function (listener) {
                this.inputTypeView.onFocus(listener);
            };
            InputView.prototype.unFocus = function (listener) {
                this.inputTypeView.unFocus(listener);
            };
            InputView.prototype.onBlur = function (listener) {
                this.inputTypeView.onBlur(listener);
            };
            InputView.prototype.unBlur = function (listener) {
                this.inputTypeView.unBlur(listener);
            };
            return InputView;
        })(api.form.FormItemView);
        form.InputView = InputView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (_form) {
        var FormView = (function (_super) {
            __extends(FormView, _super);
            function FormView(context, form, data) {
                _super.call(this, "form-view");
                this.formItemViews = [];
                this.formValidityChangedListeners = [];
                this.focusListeners = [];
                this.blurListeners = [];
                this.context = context;
                this.form = form;
                this.data = data;
            }
            FormView.prototype.layout = function () {
                var _this = this;
                var deferred = wemQ.defer();
                var formItems = this.form.getFormItems();
                var layoutPromise = new _form.FormItemLayer().setFormContext(this.context).setFormItems(formItems).setParentElement(this).layout(this.data);
                layoutPromise.then(function (formItemViews) {
                    _this.formItemViews = formItemViews;
                    api.util.assert(_this.formItemViews.length == formItems.length, "Not all FormItemView-s was created. Expected " + formItems.length + ", was: " + formItemViews.length);
                    deferred.resolve(null);
                    _this.formItemViews.forEach(function (formItemView) {
                        formItemView.onFocus(function (event) {
                            _this.notifyFocused(event);
                        });
                        formItemView.onBlur(function (event) {
                            _this.notifyBlurred(event);
                        });
                        formItemView.onValidityChanged(function (event) {
                            if (!_this.previousValidationRecording) {
                                _this.previousValidationRecording = event.getRecording();
                                _this.notifyValidityChanged(new _form.FormValidityChangedEvent(_this.previousValidationRecording, event.isInputValueBroken()));
                            }
                            else {
                                var previousValidState = _this.previousValidationRecording.isValid();
                                if (event.isValid()) {
                                    _this.previousValidationRecording.removeByPath(event.getOrigin());
                                }
                                else {
                                    _this.previousValidationRecording.flatten(event.getRecording());
                                }
                                _this.notifyValidityChanged(new _form.FormValidityChangedEvent(_this.previousValidationRecording, event.isInputValueBroken()));
                            }
                            _this.previousValidationRecording = event.getRecording();
                        });
                    });
                    api.dom.WindowDOM.get().onResized(function (event) { return _this.checkSizeChanges(); }, _this);
                    _this.onShown(function () { return _this.checkSizeChanges(); });
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                }).done();
                return deferred.promise;
            };
            FormView.prototype.checkSizeChanges = function () {
                if (this.isVisible() && this.isSizeChanged()) {
                    this.preserveCurrentSize();
                    this.broadcastFormSizeChanged();
                }
            };
            FormView.prototype.preserveCurrentSize = function () {
                this.width = this.getEl().getWidth();
            };
            FormView.prototype.isSizeChanged = function () {
                return this.width != this.getEl().getWidth();
            };
            FormView.prototype.broadcastFormSizeChanged = function () {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.broadcastFormSizeChanged();
                });
            };
            FormView.prototype.hasValidUserInput = function () {
                var result = true;
                this.formItemViews.forEach(function (formItemView) {
                    if (!formItemView.hasValidUserInput()) {
                        result = false;
                    }
                });
                return result;
            };
            FormView.prototype.validate = function (silent, triggerNotifyAnyWay) {
                if (triggerNotifyAnyWay === void 0) { triggerNotifyAnyWay = false; }
                var recording = new _form.ValidationRecording();
                this.formItemViews.forEach(function (formItemView) {
                    recording.flatten(formItemView.validate(silent));
                });
                if (!silent && (recording.validityChanged(this.previousValidationRecording) || triggerNotifyAnyWay)) {
                    this.notifyValidityChanged(new _form.FormValidityChangedEvent(recording));
                }
                this.previousValidationRecording = recording;
                return recording;
            };
            FormView.prototype.isValid = function () {
                if (!this.previousValidationRecording) {
                    this.previousValidationRecording = this.validate(true);
                }
                return this.previousValidationRecording.isValid();
            };
            FormView.prototype.displayValidationErrors = function (value) {
                if (value) {
                    this.addClass("display-validation-errors");
                }
                else {
                    this.removeClass("display-validation-errors");
                }
                for (var i = 0; i < this.formItemViews.length; i++) {
                    this.formItemViews[i].displayValidationErrors(value);
                }
            };
            FormView.prototype.getData = function () {
                return this.data;
            };
            FormView.prototype.giveFocus = function () {
                var focusGiven = false;
                if (this.formItemViews.length > 0) {
                    for (var i = 0; i < this.formItemViews.length; i++) {
                        if (this.formItemViews[i].giveFocus()) {
                            focusGiven = true;
                            break;
                        }
                    }
                }
                return focusGiven;
            };
            FormView.prototype.onEditContentRequest = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.onEditContentRequest(listener);
                });
            };
            FormView.prototype.unEditContentRequest = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.unEditContentRequest(listener);
                });
            };
            FormView.prototype.onValidityChanged = function (listener) {
                this.formValidityChangedListeners.push(listener);
            };
            FormView.prototype.unValidityChanged = function (listener) {
                this.formValidityChangedListeners = this.formValidityChangedListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            FormView.prototype.notifyValidityChanged = function (event) {
                var _this = this;
                this.formValidityChangedListeners.forEach(function (listener) {
                    listener.call(_this, event);
                });
            };
            FormView.prototype.notifyEditContentRequested = function (content) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.notifyEditContentRequested(content);
                });
            };
            FormView.prototype.onFocus = function (listener) {
                this.focusListeners.push(listener);
            };
            FormView.prototype.unFocus = function (listener) {
                this.focusListeners = this.focusListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            FormView.prototype.onBlur = function (listener) {
                this.blurListeners.push(listener);
            };
            FormView.prototype.unBlur = function (listener) {
                this.blurListeners = this.blurListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            FormView.prototype.notifyFocused = function (event) {
                this.focusListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            FormView.prototype.notifyBlurred = function (event) {
                this.blurListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            return FormView;
        })(api.dom.DivEl);
        _form.FormView = FormView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemSetLabel = (function (_super) {
            __extends(FormItemSetLabel, _super);
            function FormItemSetLabel(formItemSet) {
                _super.call(this, "form-item-set-label");
                this.formItemSet = formItemSet;
                var nodes = [];
                var dragHandle = new api.dom.SpanEl("drag-handle");
                dragHandle.setHtml(":::");
                nodes.push(dragHandle.getHTMLElement());
                nodes.push(document.createTextNode(formItemSet.getLabel()));
                if (formItemSet.getOccurrences().required()) {
                    nodes.push(document.createTextNode(" "));
                    var requiredMarker = new api.dom.SpanEl("required");
                    nodes.push(requiredMarker.getHTMLElement());
                }
                nodes.push(document.createTextNode(":"));
                this.getEl().appendChildren(nodes);
            }
            return FormItemSetLabel;
        })(api.dom.DivEl);
        form.FormItemSetLabel = FormItemSetLabel;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemSetOccurrence = (function (_super) {
            __extends(FormItemSetOccurrence, _super);
            function FormItemSetOccurrence(formItemSetOccurrences, index) {
                _super.call(this, formItemSetOccurrences, index, formItemSetOccurrences.getFormItemSet().getOccurrences());
            }
            return FormItemSetOccurrence;
        })(form.FormItemOccurrence);
        form.FormItemSetOccurrence = FormItemSetOccurrence;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemSetOccurrences = (function (_super) {
            __extends(FormItemSetOccurrences, _super);
            function FormItemSetOccurrences(config) {
                this.occurrencesCollapsed = false;
                _super.call(this, {
                    formItem: config.formItemSet,
                    propertyArray: config.propertyArray,
                    occurrenceViewContainer: config.occurrenceViewContainer,
                    allowedOccurrences: config.formItemSet.getOccurrences()
                });
                this.context = config.context;
                this.formItemSet = config.formItemSet;
                this.parent = config.parent;
                var dataSetCount = this.propertyArray.getSize();
                if (dataSetCount > 0) {
                    this.constructOccurrencesForData();
                }
                else {
                    this.constructOccurrencesForNoData();
                }
            }
            FormItemSetOccurrences.prototype.getFormItemSet = function () {
                return this.formItemSet;
            };
            FormItemSetOccurrences.prototype.getAllowedOccurrences = function () {
                return this.formItemSet.getOccurrences();
            };
            FormItemSetOccurrences.prototype.constructOccurrencesForNoData = function () {
                var minimumOccurrences = this.getAllowedOccurrences().getMinimum();
                if (minimumOccurrences > 0) {
                    for (var i = 0; i < minimumOccurrences; i++) {
                        this.addOccurrence(this.createNewOccurrence(this, i));
                    }
                }
                else {
                    if (this.context.getShowEmptyFormItemSetOccurrences()) {
                        this.addOccurrence(this.createNewOccurrence(this, 0));
                    }
                }
            };
            FormItemSetOccurrences.prototype.constructOccurrencesForData = function () {
                var _this = this;
                this.propertyArray.forEach(function (property, index) {
                    _this.addOccurrence(new form.FormItemSetOccurrence(_this, index));
                });
                if (this.countOccurrences() < this.formItemSet.getOccurrences().getMinimum()) {
                    for (var index = this.countOccurrences(); index < this.formItemSet.getOccurrences().getMinimum(); index++) {
                        this.addOccurrence(new form.FormItemSetOccurrence(this, index));
                    }
                }
            };
            FormItemSetOccurrences.prototype.createNewOccurrence = function (formItemOccurrences, insertAtIndex) {
                return new form.FormItemSetOccurrence(formItemOccurrences, insertAtIndex);
            };
            FormItemSetOccurrences.prototype.createNewOccurrenceView = function (occurrence) {
                var _this = this;
                var dataSet = this.propertyArray.getSet(occurrence.getIndex());
                if (!dataSet) {
                    dataSet = this.propertyArray.addSet();
                }
                var newOccurrenceView = new form.FormItemSetOccurrenceView({
                    context: this.context,
                    formItemSetOccurrence: occurrence,
                    formItemSet: this.formItemSet,
                    parent: this.parent,
                    dataSet: dataSet
                });
                newOccurrenceView.onRemoveButtonClicked(function (event) {
                    _this.doRemoveOccurrence(event.getView(), event.getIndex());
                });
                return newOccurrenceView;
            };
            FormItemSetOccurrences.prototype.showOccurrences = function (show) {
                var views = this.getOccurrenceViews();
                this.occurrencesCollapsed = !show;
                views.forEach(function (formItemSetOccurrenceView) {
                    formItemSetOccurrenceView.showContainer(show);
                });
            };
            FormItemSetOccurrences.prototype.isCollapsed = function () {
                return this.occurrencesCollapsed;
            };
            FormItemSetOccurrences.prototype.moveOccurrence = function (index, destinationIndex) {
                _super.prototype.moveOccurrence.call(this, index, destinationIndex);
            };
            return FormItemSetOccurrences;
        })(form.FormItemOccurrences);
        form.FormItemSetOccurrences = FormItemSetOccurrences;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FormItemSetOccurrenceView = (function (_super) {
            __extends(FormItemSetOccurrenceView, _super);
            function FormItemSetOccurrenceView(config) {
                _super.call(this, "form-item-set-occurrence-view", config.formItemSetOccurrence);
                this.formItemViews = [];
                this.validityChangedListeners = [];
                this.context = config.context;
                this.formItemSetOccurrence = config.formItemSetOccurrence;
                this.formItemSet = config.formItemSet;
                this.parent = config.parent;
                this.constructedWithData = config.dataSet != null;
                this.propertySet = config.dataSet;
            }
            FormItemSetOccurrenceView.prototype.getDataPath = function () {
                return this.propertySet.getProperty().getPath();
            };
            FormItemSetOccurrenceView.prototype.layout = function () {
                var _this = this;
                var deferred = wemQ.defer();
                this.removeButton = new api.dom.AEl("remove-button");
                this.appendChild(this.removeButton);
                this.removeButton.onClicked(function (event) {
                    _this.notifyRemoveButtonClicked();
                });
                this.label = new form.FormItemSetLabel(this.formItemSet);
                this.appendChild(this.label);
                this.formItemSetOccurrencesContainer = new api.dom.DivEl("form-item-set-occurrences-container");
                this.appendChild(this.formItemSetOccurrencesContainer);
                var layoutPromise = new form.FormItemLayer().setFormContext(this.context).setFormItems(this.formItemSet.getFormItems()).setParentElement(this.formItemSetOccurrencesContainer).setParent(this).layout(this.propertySet);
                layoutPromise.then(function (formItemViews) {
                    _this.formItemViews = formItemViews;
                    _this.validate(true);
                    _this.formItemViews.forEach(function (formItemView) {
                        formItemView.onValidityChanged(function (event) {
                            var previousValidState = _this.previousValidationRecording.isValid();
                            if (event.isValid()) {
                                _this.previousValidationRecording.removeByPath(event.getOrigin());
                            }
                            else {
                                _this.previousValidationRecording.flatten(event.getRecording());
                            }
                            if (previousValidState != _this.previousValidationRecording.isValid()) {
                                _this.notifyValidityChanged(new form.RecordingValidityChangedEvent(_this.previousValidationRecording, _this.resolveValidationRecordingPath()));
                            }
                        });
                    });
                    _this.refresh();
                    deferred.resolve(null);
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                }).done();
                return deferred.promise;
            };
            FormItemSetOccurrenceView.prototype.getFormItemViews = function () {
                return this.formItemViews;
            };
            FormItemSetOccurrenceView.prototype.giveFocus = function () {
                var focusGiven = false;
                this.getFormItemViews().forEach(function (formItemView) {
                    if (!focusGiven && formItemView.giveFocus()) {
                        focusGiven = true;
                    }
                });
                return focusGiven;
            };
            FormItemSetOccurrenceView.prototype.refresh = function () {
                if (!this.formItemSetOccurrence.oneAndOnly()) {
                    this.label.addClass("drag-control");
                }
                else {
                    this.label.removeClass("drag-control");
                }
                this.removeButton.setVisible(this.formItemSetOccurrence.isRemoveButtonRequired());
            };
            FormItemSetOccurrenceView.prototype.showContainer = function (show) {
                if (show) {
                    this.formItemSetOccurrencesContainer.show();
                }
                else {
                    this.formItemSetOccurrencesContainer.hide();
                }
            };
            FormItemSetOccurrenceView.prototype.resolveValidationRecordingPath = function () {
                return new form.ValidationRecordingPath(this.getDataPath(), null);
            };
            FormItemSetOccurrenceView.prototype.getLastValidationRecording = function () {
                return this.previousValidationRecording;
            };
            FormItemSetOccurrenceView.prototype.displayValidationErrors = function (value) {
                this.formItemViews.forEach(function (view) {
                    view.displayValidationErrors(value);
                });
            };
            FormItemSetOccurrenceView.prototype.hasValidUserInput = function () {
                var result = true;
                this.formItemViews.forEach(function (formItemView) {
                    if (!formItemView.hasValidUserInput()) {
                        result = false;
                    }
                });
                return result;
            };
            FormItemSetOccurrenceView.prototype.validate = function (silent) {
                if (silent === void 0) { silent = true; }
                var allRecordings = new form.ValidationRecording();
                this.formItemViews.forEach(function (formItemView) {
                    var currRecording = formItemView.validate(silent);
                    allRecordings.flatten(currRecording);
                });
                if (!silent) {
                    if (allRecordings.validityChanged(this.previousValidationRecording)) {
                        this.notifyValidityChanged(new form.RecordingValidityChangedEvent(allRecordings, this.resolveValidationRecordingPath()));
                    }
                }
                this.previousValidationRecording = allRecordings;
                return allRecordings;
            };
            FormItemSetOccurrenceView.prototype.onValidityChanged = function (listener) {
                this.validityChangedListeners.push(listener);
            };
            FormItemSetOccurrenceView.prototype.unValidityChanged = function (listener) {
                this.validityChangedListeners.filter(function (currentListener) {
                    return listener == currentListener;
                });
            };
            FormItemSetOccurrenceView.prototype.notifyValidityChanged = function (event) {
                this.validityChangedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            FormItemSetOccurrenceView.prototype.onFocus = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.onFocus(listener);
                });
            };
            FormItemSetOccurrenceView.prototype.unFocus = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.unFocus(listener);
                });
            };
            FormItemSetOccurrenceView.prototype.onBlur = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.onBlur(listener);
                });
            };
            FormItemSetOccurrenceView.prototype.unBlur = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.unBlur(listener);
                });
            };
            return FormItemSetOccurrenceView;
        })(form.FormItemOccurrenceView);
        form.FormItemSetOccurrenceView = FormItemSetOccurrenceView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var PropertyArray = api.data.PropertyArray;
        var ValueTypes = api.data.ValueTypes;
        var FormItemSetView = (function (_super) {
            __extends(FormItemSetView, _super);
            function FormItemSetView(config) {
                _super.call(this, {
                    className: "form-item-set-view",
                    context: config.context,
                    formItem: config.formItemSet,
                    parent: config.parent
                });
                this.validityChangedListeners = [];
                this.parentDataSet = config.parentDataSet;
                this.formItemSet = config.formItemSet;
                this.addClass(this.formItemSet.getPath().getElements().length % 2 ? "even" : "odd");
            }
            FormItemSetView.prototype.layout = function () {
                var _this = this;
                var deferred = wemQ.defer();
                this.occurrenceViewsContainer = new api.dom.DivEl("occurrence-views-container");
                wemjq(this.occurrenceViewsContainer.getHTMLElement()).sortable({
                    revert: false,
                    containment: this.getHTMLElement(),
                    cursor: 'move',
                    cursorAt: { left: 14, top: 14 },
                    distance: 20,
                    tolerance: 'pointer',
                    handle: '.drag-control',
                    placeholder: 'form-item-set-drop-target-placeholder',
                    helper: function (event, ui) { return api.ui.DragHelper.get().getHTMLElement(); },
                    start: function (event, ui) { return _this.handleDnDStart(event, ui); },
                    update: function (event, ui) { return _this.handleDnDUpdate(event, ui); }
                });
                this.appendChild(this.occurrenceViewsContainer);
                var propertyArray = this.parentDataSet.getPropertyArray(this.formItemSet.getName());
                if (!propertyArray) {
                    propertyArray = PropertyArray.create().setType(ValueTypes.DATA).setName(this.formItemSet.getName()).setParent(this.parentDataSet).build();
                    this.parentDataSet.addPropertyArray(propertyArray);
                }
                this.formItemSetOccurrences = new form.FormItemSetOccurrences({
                    context: this.getContext(),
                    occurrenceViewContainer: this.occurrenceViewsContainer,
                    formItemSet: this.formItemSet,
                    parent: this.getParent(),
                    propertyArray: propertyArray
                });
                var layoutPromise = this.formItemSetOccurrences.layout();
                layoutPromise.then(function () {
                    _this.formItemSetOccurrences.onOccurrenceRendered(function () {
                        _this.validate(false);
                    });
                    _this.formItemSetOccurrences.onOccurrenceAdded(function (event) {
                        _this.refresh();
                        wemjq(_this.occurrenceViewsContainer.getHTMLElement()).sortable("refresh");
                        if (api.ObjectHelper.iFrameSafeInstanceOf(event.getOccurrenceView(), form.FormItemSetOccurrenceView)) {
                            var addedFormItemSetOccurrenceView = event.getOccurrenceView();
                            addedFormItemSetOccurrenceView.onValidityChanged(function (event) {
                                _this.handleFormItemSetOccurrenceViewValidityChanged(event);
                            });
                        }
                    });
                    _this.formItemSetOccurrences.onOccurrenceRemoved(function (event) {
                        _this.refresh();
                        if (api.ObjectHelper.iFrameSafeInstanceOf(event.getOccurrenceView(), form.FormItemSetOccurrenceView)) {
                            _this.validate(false);
                        }
                    });
                    _this.formItemSetOccurrences.getOccurrenceViews().forEach(function (formItemSetOccurrenceView) {
                        formItemSetOccurrenceView.onValidityChanged(function (event) {
                            _this.handleFormItemSetOccurrenceViewValidityChanged(event);
                        });
                    });
                    _this.bottomButtonRow = new api.dom.DivEl("bottom-button-row");
                    _this.appendChild(_this.bottomButtonRow);
                    _this.addButton = new api.ui.button.Button("Add " + _this.formItemSet.getLabel());
                    _this.addButton.addClass("small");
                    _this.addButton.onClicked(function (event) {
                        _this.formItemSetOccurrences.createAndAddOccurrence();
                        if (_this.formItemSetOccurrences.isCollapsed()) {
                            _this.collapseButton.getHTMLElement().click();
                        }
                    });
                    _this.collapseButton = new api.dom.AEl("collapse-button");
                    _this.collapseButton.setHtml("Collapse");
                    _this.collapseButton.onClicked(function (event) {
                        if (_this.formItemSetOccurrences.isCollapsed()) {
                            _this.collapseButton.setHtml("Collapse");
                            _this.formItemSetOccurrences.showOccurrences(true);
                        }
                        else {
                            _this.collapseButton.setHtml("Expand");
                            _this.formItemSetOccurrences.showOccurrences(false);
                        }
                    });
                    _this.bottomButtonRow.appendChild(_this.addButton);
                    _this.bottomButtonRow.appendChild(_this.collapseButton);
                    _this.validationViewer = new api.form.ValidationRecordingViewer();
                    _this.appendChild(_this.validationViewer);
                    _this.refresh();
                    _this.validate(true);
                    deferred.resolve(null);
                });
                return deferred.promise;
            };
            FormItemSetView.prototype.handleFormItemSetOccurrenceViewValidityChanged = function (event) {
                if (!this.previousValidationRecording) {
                    return;
                }
                var previousValidState = this.previousValidationRecording.isValid();
                if (event.isValid()) {
                    this.previousValidationRecording.removeByPath(event.getOrigin());
                }
                else {
                    this.previousValidationRecording.flatten(event.getRecording());
                }
                var validationRecordingPath = this.resolveValidationRecordingPath();
                var occurrenceViews = this.formItemSetOccurrences.getOccurrenceViews();
                var occurrenceRecording = new form.ValidationRecording();
                var numberOfValids = 0;
                occurrenceViews.forEach(function (occurrenceView) {
                    var recordingForOccurrence = occurrenceView.getLastValidationRecording();
                    if (recordingForOccurrence && recordingForOccurrence.isValid()) {
                        numberOfValids++;
                    }
                });
                if (numberOfValids < this.formItemSet.getOccurrences().getMinimum()) {
                    this.previousValidationRecording.breaksMinimumOccurrences(validationRecordingPath);
                    occurrenceRecording.breaksMinimumOccurrences(validationRecordingPath);
                }
                else {
                    this.previousValidationRecording.removeUnreachedMinimumOccurrencesByPath(validationRecordingPath);
                }
                if (this.formItemSet.getOccurrences().maximumBreached(numberOfValids)) {
                    this.previousValidationRecording.breaksMaximumOccurrences(validationRecordingPath);
                    occurrenceRecording.breaksMaximumOccurrences(validationRecordingPath);
                }
                else {
                    this.previousValidationRecording.removeBreachedMaximumOccurrencesByPath(validationRecordingPath);
                }
                this.renderValidationErrors(occurrenceRecording);
                if (previousValidState != this.previousValidationRecording.isValid()) {
                    this.notifyValidityChanged(new form.RecordingValidityChangedEvent(this.previousValidationRecording, this.resolveValidationRecordingPath()));
                }
                else if (previousValidState != event.isValid()) {
                    this.notifyValidityChanged(event);
                }
            };
            FormItemSetView.prototype.broadcastFormSizeChanged = function () {
                this.formItemSetOccurrences.getOccurrenceViews().forEach(function (occurrenceView) {
                    occurrenceView.getFormItemViews().forEach(function (formItemView) {
                        formItemView.broadcastFormSizeChanged();
                    });
                });
            };
            FormItemSetView.prototype.refresh = function () {
                this.collapseButton.setVisible(this.formItemSetOccurrences.getOccurrences().length > 0);
                this.addButton.setVisible(!this.formItemSetOccurrences.maximumOccurrencesReached());
            };
            FormItemSetView.prototype.getFormItemSetOccurrenceView = function (index) {
                return this.formItemSetOccurrences.getOccurrenceViews()[index];
            };
            FormItemSetView.prototype.resolveValidationRecordingPath = function () {
                return new form.ValidationRecordingPath(this.parentDataSet.getPropertyPath(), this.formItemSet.getName(), this.formItemSet.getOccurrences().getMinimum(), this.formItemSet.getOccurrences().getMaximum());
            };
            FormItemSetView.prototype.displayValidationErrors = function (value) {
                this.formItemSetOccurrences.getOccurrenceViews().forEach(function (view) {
                    view.displayValidationErrors(value);
                });
            };
            FormItemSetView.prototype.hasValidUserInput = function () {
                var result = true;
                this.formItemSetOccurrences.getOccurrenceViews().forEach(function (formItemOccurrenceView) {
                    if (!formItemOccurrenceView.hasValidUserInput()) {
                        result = false;
                    }
                });
                return result;
            };
            FormItemSetView.prototype.validate = function (silent) {
                if (silent === void 0) { silent = true; }
                var validationRecordingPath = this.resolveValidationRecordingPath();
                var wholeRecording = new form.ValidationRecording();
                var occurrenceViews = this.formItemSetOccurrences.getOccurrenceViews();
                var occurrenceRecording = new form.ValidationRecording();
                var numberOfValids = 0;
                occurrenceViews.forEach(function (occurrenceView) {
                    var recordingForOccurrence = occurrenceView.validate(silent);
                    if (recordingForOccurrence.isValid()) {
                        numberOfValids++;
                    }
                    wholeRecording.flatten(recordingForOccurrence);
                });
                if (numberOfValids < this.formItemSet.getOccurrences().getMinimum()) {
                    wholeRecording.breaksMinimumOccurrences(validationRecordingPath);
                    occurrenceRecording.breaksMinimumOccurrences(validationRecordingPath);
                }
                if (this.formItemSet.getOccurrences().maximumBreached(numberOfValids)) {
                    wholeRecording.breaksMaximumOccurrences(validationRecordingPath);
                    occurrenceRecording.breaksMaximumOccurrences(validationRecordingPath);
                }
                if (!silent && wholeRecording.validityChanged(this.previousValidationRecording)) {
                    this.notifyValidityChanged(new form.RecordingValidityChangedEvent(wholeRecording, validationRecordingPath));
                }
                this.renderValidationErrors(occurrenceRecording);
                this.previousValidationRecording = wholeRecording;
                return wholeRecording;
            };
            FormItemSetView.prototype.onValidityChanged = function (listener) {
                this.validityChangedListeners.push(listener);
            };
            FormItemSetView.prototype.unValidityChanged = function (listener) {
                this.validityChangedListeners.filter(function (currentListener) {
                    return listener == currentListener;
                });
            };
            FormItemSetView.prototype.notifyValidityChanged = function (event) {
                this.validityChangedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            FormItemSetView.prototype.renderValidationErrors = function (recording) {
                if (recording.isValid()) {
                    this.removeClass("invalid");
                    this.addClass("valid");
                }
                else {
                    this.removeClass("valid");
                    this.addClass("invalid");
                }
                this.validationViewer.setObject(recording);
            };
            FormItemSetView.prototype.giveFocus = function () {
                var focusGiven = false;
                if (this.formItemSetOccurrences.getOccurrenceViews().length > 0) {
                    var views = this.formItemSetOccurrences.getOccurrenceViews();
                    for (var i = 0; i < views.length; i++) {
                        if (views[i].giveFocus()) {
                            focusGiven = true;
                            break;
                        }
                    }
                }
                return focusGiven;
            };
            FormItemSetView.prototype.handleDnDStart = function (event, ui) {
                var draggedElement = api.dom.Element.fromHtmlElement(ui.item.context);
                api.util.assert(draggedElement.hasClass("form-item-set-occurrence-view"));
                this.draggingIndex = draggedElement.getSiblingIndex();
                ui.placeholder.html("Drop form item set here");
            };
            FormItemSetView.prototype.handleDnDUpdate = function (event, ui) {
                if (this.draggingIndex >= 0) {
                    var draggedElement = api.dom.Element.fromHtmlElement(ui.item.context);
                    api.util.assert(draggedElement.hasClass("form-item-set-occurrence-view"));
                    var draggedToIndex = draggedElement.getSiblingIndex();
                    this.formItemSetOccurrences.moveOccurrence(this.draggingIndex, draggedToIndex);
                }
                this.draggingIndex = -1;
            };
            FormItemSetView.prototype.onFocus = function (listener) {
                this.formItemSetOccurrences.onFocus(listener);
            };
            FormItemSetView.prototype.unFocus = function (listener) {
                this.formItemSetOccurrences.unFocus(listener);
            };
            FormItemSetView.prototype.onBlur = function (listener) {
                this.formItemSetOccurrences.onBlur(listener);
            };
            FormItemSetView.prototype.unBlur = function (listener) {
                this.formItemSetOccurrences.unBlur(listener);
            };
            return FormItemSetView;
        })(form.FormItemView);
        form.FormItemSetView = FormItemSetView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var LayoutView = (function (_super) {
            __extends(LayoutView, _super);
            function LayoutView(config) {
                _super.call(this, {
                    className: config.className,
                    context: config.context,
                    formItem: config.layout,
                    parent: config.parent
                });
                this._layout = config.layout;
            }
            LayoutView.prototype.layout = function () {
                throw new Error("Must be implemented by inheritors");
            };
            return LayoutView;
        })(form.FormItemView);
        form.LayoutView = LayoutView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FieldSetLabel = (function (_super) {
            __extends(FieldSetLabel, _super);
            function FieldSetLabel(fieldSet) {
                _super.call(this, "field-set-label");
                this.fieldSet = fieldSet;
                this.getEl().setInnerHtml(this.fieldSet.getLabel());
                this.getEl().setAttribute("title", this.fieldSet.getLabel());
            }
            return FieldSetLabel;
        })(api.dom.DivEl);
        form.FieldSetLabel = FieldSetLabel;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var FieldSetView = (function (_super) {
            __extends(FieldSetView, _super);
            function FieldSetView(config) {
                _super.call(this, {
                    context: config.context,
                    layout: config.fieldSet,
                    parent: config.parent,
                    className: "field-set-view"
                });
                this.formItemViews = [];
                this.fieldSet = config.fieldSet;
                this.propertySet = config.dataSet;
            }
            FieldSetView.prototype.broadcastFormSizeChanged = function () {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.broadcastFormSizeChanged();
                });
            };
            FieldSetView.prototype.layout = function () {
                return this.doLayout();
            };
            FieldSetView.prototype.doLayout = function () {
                var _this = this;
                var deferred = wemQ.defer();
                var label = new form.FieldSetLabel(this.fieldSet);
                this.appendChild(label);
                var wrappingDiv = new api.dom.DivEl("field-set-container");
                this.appendChild(wrappingDiv);
                var layoutPromise = new form.FormItemLayer().setFormContext(this.getContext()).setFormItems(this.fieldSet.getFormItems()).setParentElement(wrappingDiv).setParent(this.getParent()).layout(this.propertySet);
                layoutPromise.then(function (formItemViews) {
                    _this.formItemViews = formItemViews;
                    deferred.resolve(null);
                }).catch(function (reason) {
                    var fieldSetValue = _this.fieldSet ? _this.fieldSet.toFieldSetJson() : {};
                    console.error('Could not render FieldSet view: ' + reason + '\r\n FieldSet value:', JSON.stringify(fieldSetValue));
                    api.DefaultErrorHandler.handle(reason);
                }).done();
                return deferred.promise;
            };
            FieldSetView.prototype.onEditContentRequest = function (listener) {
                _super.prototype.onEditContentRequest.call(this, listener);
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.onEditContentRequest(listener);
                });
            };
            FieldSetView.prototype.unEditContentRequest = function (listener) {
                _super.prototype.unEditContentRequest.call(this, listener);
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.unEditContentRequest(listener);
                });
            };
            FieldSetView.prototype.giveFocus = function () {
                var focusGiven = false;
                if (this.formItemViews.length > 0) {
                    for (var i = 0; i < this.formItemViews.length; i++) {
                        if (this.formItemViews[i].giveFocus()) {
                            focusGiven = true;
                            break;
                        }
                    }
                }
                return focusGiven;
            };
            FieldSetView.prototype.displayValidationErrors = function (value) {
                this.formItemViews.forEach(function (view) {
                    view.displayValidationErrors(value);
                });
            };
            FieldSetView.prototype.hasValidUserInput = function () {
                var result = true;
                this.formItemViews.forEach(function (formItemView) {
                    if (!formItemView.hasValidUserInput()) {
                        result = false;
                    }
                });
                return result;
            };
            FieldSetView.prototype.validate = function (silent) {
                if (silent === void 0) { silent = true; }
                var recording = new form.ValidationRecording();
                this.formItemViews.forEach(function (formItemView) {
                    recording.flatten(formItemView.validate(silent));
                });
                return recording;
            };
            FieldSetView.prototype.onValidityChanged = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.onValidityChanged(listener);
                });
            };
            FieldSetView.prototype.unValidityChanged = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.unValidityChanged(listener);
                });
            };
            FieldSetView.prototype.onFocus = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.onFocus(listener);
                });
            };
            FieldSetView.prototype.unFocus = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.unFocus(listener);
                });
            };
            FieldSetView.prototype.onBlur = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.onBlur(listener);
                });
            };
            FieldSetView.prototype.unBlur = function (listener) {
                this.formItemViews.forEach(function (formItemView) {
                    formItemView.unBlur(listener);
                });
            };
            return FieldSetView;
        })(form.LayoutView);
        form.FieldSetView = FieldSetView;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var RemoveButtonClickedEvent = (function () {
            function RemoveButtonClickedEvent(view, index) {
                this.view = view;
                this.index = index;
            }
            RemoveButtonClickedEvent.prototype.getView = function () {
                return this.view;
            };
            RemoveButtonClickedEvent.prototype.getIndex = function () {
                return this.index;
            };
            return RemoveButtonClickedEvent;
        })();
        form.RemoveButtonClickedEvent = RemoveButtonClickedEvent;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var OccurrenceRemovedEvent = (function () {
            function OccurrenceRemovedEvent(occurrence, occurrenceView) {
                this.occurrence = occurrence;
                this.occurrenceView = occurrenceView;
            }
            OccurrenceRemovedEvent.prototype.getOccurrence = function () {
                return this.occurrence;
            };
            OccurrenceRemovedEvent.prototype.getOccurrenceView = function () {
                return this.occurrenceView;
            };
            return OccurrenceRemovedEvent;
        })();
        form.OccurrenceRemovedEvent = OccurrenceRemovedEvent;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var OccurrenceAddedEvent = (function () {
            function OccurrenceAddedEvent(occurrence, occurrenceView) {
                this.occurrence = occurrence;
                this.occurrenceView = occurrenceView;
            }
            OccurrenceAddedEvent.prototype.getOccurrence = function () {
                return this.occurrence;
            };
            OccurrenceAddedEvent.prototype.getOccurrenceView = function () {
                return this.occurrenceView;
            };
            return OccurrenceAddedEvent;
        })();
        form.OccurrenceAddedEvent = OccurrenceAddedEvent;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var OccurrenceRenderedEvent = (function () {
            function OccurrenceRenderedEvent(occurrence, occurrenceView) {
                this.occurrence = occurrence;
                this.occurrenceView = occurrenceView;
            }
            OccurrenceRenderedEvent.prototype.getOccurrence = function () {
                return this.occurrence;
            };
            OccurrenceRenderedEvent.prototype.getOccurrenceView = function () {
                return this.occurrenceView;
            };
            return OccurrenceRenderedEvent;
        })();
        form.OccurrenceRenderedEvent = OccurrenceRenderedEvent;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var ValidationRecordingViewer = (function (_super) {
            __extends(ValidationRecordingViewer, _super);
            function ValidationRecordingViewer() {
                _super.call(this, 'validation-viewer');
                this.minText = "Min {0} occurrences required";
                this.minTextSingle = "This field is required";
                this.maxText = "Max {0} occurrence{1} allowed";
                this.list = new api.dom.UlEl();
                this.appendChild(this.list);
            }
            ValidationRecordingViewer.prototype.setObject = function (recording) {
                var _this = this;
                this.list.removeChildren();
                recording.breaksMinimumOccurrencesArray.forEach(function (path) {
                    _this.list.appendChild(_this.createItemView(path, true));
                });
                recording.breaksMaximumOccurrencesArray.forEach(function (path) {
                    _this.list.appendChild(_this.createItemView(path, false));
                });
            };
            ValidationRecordingViewer.prototype.appendValidationMessage = function (message, removeExisting) {
                if (removeExisting === void 0) { removeExisting = true; }
                if (removeExisting) {
                    this.list.removeChildren();
                }
                this.list.appendChild(new api.dom.LiEl().setHtml(message));
            };
            ValidationRecordingViewer.prototype.setError = function (text) {
                this.list.removeChildren();
                if (text) {
                    this.list.appendChild(new api.dom.LiEl().setHtml(text));
                }
            };
            ValidationRecordingViewer.prototype.createItemView = function (path, breaksMin) {
                var text = breaksMin ? this.resolveMinText(path) : this.resolveMaxText(path);
                return new api.dom.LiEl().setHtml(text);
            };
            ValidationRecordingViewer.prototype.resolveMinText = function (path) {
                return path.getMin() > 1 ? api.util.StringHelper.format(this.minText, path.getMin()) : this.minTextSingle;
            };
            ValidationRecordingViewer.prototype.resolveMaxText = function (path) {
                return api.util.StringHelper.format(this.maxText, path.getMax(), path.getMax() > 1 ? 's' : '');
            };
            return ValidationRecordingViewer;
        })(api.ui.Viewer);
        form.ValidationRecordingViewer = ValidationRecordingViewer;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var AdditionalValidationRecord = (function () {
            function AdditionalValidationRecord(builder) {
                this.overwriteDefault = false;
                this.message = builder.message;
                this.overwriteDefault = builder.overwriteDefault;
            }
            AdditionalValidationRecord.create = function () {
                return new Builder();
            };
            AdditionalValidationRecord.prototype.getMessage = function () {
                return this.message;
            };
            AdditionalValidationRecord.prototype.isOverwriteDefault = function () {
                return this.overwriteDefault;
            };
            AdditionalValidationRecord.prototype.equals = function (that) {
                if (this.message != that.message) {
                    return false;
                }
                if (this.overwriteDefault != that.overwriteDefault) {
                    return false;
                }
                return true;
            };
            return AdditionalValidationRecord;
        })();
        form.AdditionalValidationRecord = AdditionalValidationRecord;
        var Builder = (function () {
            function Builder() {
                this.overwriteDefault = false;
            }
            Builder.prototype.setMessage = function (value) {
                this.message = value;
                return this;
            };
            Builder.prototype.setOverwriteDefault = function (value) {
                this.overwriteDefault = value;
                return this;
            };
            Builder.prototype.build = function () {
                return new AdditionalValidationRecord(this);
            };
            return Builder;
        })();
        form.Builder = Builder;
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var ValueAddedEvent = (function () {
                function ValueAddedEvent(value) {
                    this.value = value;
                }
                ValueAddedEvent.prototype.getValue = function () {
                    return this.value;
                };
                return ValueAddedEvent;
            })();
            inputtype.ValueAddedEvent = ValueAddedEvent;
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var ValueChangedEvent = (function () {
                function ValueChangedEvent(newValue, arrayIndex) {
                    this.newValue = newValue;
                    this.arrayIndex = arrayIndex;
                }
                ValueChangedEvent.prototype.getNewValue = function () {
                    return this.newValue;
                };
                ValueChangedEvent.prototype.getArrayIndex = function () {
                    return this.arrayIndex;
                };
                return ValueChangedEvent;
            })();
            inputtype.ValueChangedEvent = ValueChangedEvent;
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var ValueRemovedEvent = (function () {
                function ValueRemovedEvent(arrayIndex) {
                    this.arrayIndex = arrayIndex;
                }
                ValueRemovedEvent.prototype.getArrayIndex = function () {
                    return this.arrayIndex;
                };
                return ValueRemovedEvent;
            })();
            inputtype.ValueRemovedEvent = ValueRemovedEvent;
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var InputValidationRecording = (function () {
                function InputValidationRecording() {
                    this.breaksMinimumOccurrences = false;
                    this.breaksMaximumOccurrences = false;
                }
                InputValidationRecording.prototype.isValid = function () {
                    return !this.breaksMaximumOccurrences && !this.breaksMinimumOccurrences;
                };
                InputValidationRecording.prototype.setBreaksMinimumOccurrences = function (value) {
                    this.breaksMinimumOccurrences = value;
                };
                InputValidationRecording.prototype.setBreaksMaximumOccurrences = function (value) {
                    this.breaksMaximumOccurrences = value;
                };
                InputValidationRecording.prototype.setAdditionalValidationRecord = function (value) {
                    this.additionalValidationRecord = value;
                };
                InputValidationRecording.prototype.isMinimumOccurrencesBreached = function () {
                    return this.breaksMinimumOccurrences;
                };
                InputValidationRecording.prototype.isMaximumOccurrencesBreached = function () {
                    return this.breaksMaximumOccurrences;
                };
                InputValidationRecording.prototype.getAdditionalValidationRecord = function () {
                    return this.additionalValidationRecord;
                };
                InputValidationRecording.prototype.equals = function (that) {
                    if (this.breaksMinimumOccurrences != that.breaksMinimumOccurrences) {
                        return false;
                    }
                    if (this.breaksMaximumOccurrences != that.breaksMaximumOccurrences) {
                        return false;
                    }
                    if (!api.ObjectHelper.equals(this.additionalValidationRecord, that.additionalValidationRecord)) {
                        return false;
                    }
                    return true;
                };
                InputValidationRecording.prototype.validityChanged = function (other) {
                    return other == undefined || other == null || !other.equals(this);
                };
                return InputValidationRecording;
            })();
            inputtype.InputValidationRecording = InputValidationRecording;
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var InputValidityChangedEvent = (function () {
                function InputValidityChangedEvent(recording, inputName) {
                    this.recording = recording;
                    this.inputName = inputName;
                }
                InputValidityChangedEvent.prototype.getInputName = function () {
                    return this.inputName;
                };
                InputValidityChangedEvent.prototype.isValid = function () {
                    return this.recording.isValid();
                };
                InputValidityChangedEvent.prototype.getRecording = function () {
                    return this.recording;
                };
                return InputValidityChangedEvent;
            })();
            inputtype.InputValidityChangedEvent = InputValidityChangedEvent;
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var InputTypeManager = (function () {
                function InputTypeManager() {
                }
                InputTypeManager.isRegistered = function (inputTypeName) {
                    var name = InputTypeManager.normalize(inputTypeName);
                    return InputTypeManager.inputTypes[name] != undefined;
                };
                InputTypeManager.register = function (inputTypeClass) {
                    var name = InputTypeManager.normalize(inputTypeClass.getName());
                    if (!InputTypeManager.isRegistered(name)) {
                        InputTypeManager.inputTypes[name] = inputTypeClass;
                    }
                    else {
                        throw new Error('Input type [' + name + '] is already registered, unregister it first.');
                    }
                };
                InputTypeManager.unregister = function (inputTypeName) {
                    var name = InputTypeManager.normalize(inputTypeName);
                    if (InputTypeManager.isRegistered(name)) {
                        InputTypeManager.inputTypes[name] = undefined;
                        console.log('Unregistered input type [' + name + "]");
                    }
                    else {
                        throw new Error('Input type [' + name + '] is not registered.');
                    }
                };
                InputTypeManager.createView = function (inputTypeName, context) {
                    var name = InputTypeManager.normalize(inputTypeName);
                    if (InputTypeManager.isRegistered(name)) {
                        var inputTypeClass = InputTypeManager.inputTypes[name];
                        return inputTypeClass.newInstance(context);
                    }
                    else {
                        throw new Error("Input type [" + name + "] need to be registered first.");
                    }
                };
                InputTypeManager.normalize = function (inputTypeName) {
                    return (inputTypeName || '').toLowerCase();
                };
                InputTypeManager.inputTypes = {};
                return InputTypeManager;
            })();
            inputtype.InputTypeManager = InputTypeManager;
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var InputOccurrence = (function (_super) {
                    __extends(InputOccurrence, _super);
                    function InputOccurrence(inputOccurrences, index) {
                        _super.call(this, inputOccurrences, index, inputOccurrences.getInput().getOccurrences());
                    }
                    return InputOccurrence;
                })(api.form.FormItemOccurrence);
                support.InputOccurrence = InputOccurrence;
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var InputOccurrencesBuilder = (function () {
                    function InputOccurrencesBuilder() {
                    }
                    InputOccurrencesBuilder.prototype.setBaseInputTypeView = function (value) {
                        this.baseInputTypeView = value;
                        return this;
                    };
                    InputOccurrencesBuilder.prototype.setInput = function (value) {
                        this.input = value;
                        return this;
                    };
                    InputOccurrencesBuilder.prototype.setPropertyArray = function (value) {
                        this.propertyArray = value;
                        return this;
                    };
                    InputOccurrencesBuilder.prototype.build = function () {
                        return new InputOccurrences(this);
                    };
                    return InputOccurrencesBuilder;
                })();
                support.InputOccurrencesBuilder = InputOccurrencesBuilder;
                var InputOccurrences = (function (_super) {
                    __extends(InputOccurrences, _super);
                    function InputOccurrences(config) {
                        _super.call(this, {
                            formItem: config.input,
                            propertyArray: config.propertyArray,
                            occurrenceViewContainer: config.baseInputTypeView,
                            allowedOccurrences: config.input.getOccurrences()
                        });
                        this.baseInputTypeView = config.baseInputTypeView;
                        this.input = config.input;
                        if (this.propertyArray.getSize() > 0) {
                            this.constructOccurrencesForData();
                        }
                        else {
                            this.constructOccurrencesForNoData();
                        }
                    }
                    InputOccurrences.prototype.hasValidUserInput = function () {
                        var result = true;
                        this.getOccurrenceViews().forEach(function (formItemOccurrenceView) {
                            if (!formItemOccurrenceView.hasValidUserInput()) {
                                result = false;
                            }
                        });
                        return result;
                    };
                    InputOccurrences.prototype.moveOccurrence = function (fromIndex, toIndex) {
                        _super.prototype.moveOccurrence.call(this, fromIndex, toIndex);
                    };
                    InputOccurrences.prototype.getInput = function () {
                        return this.input;
                    };
                    InputOccurrences.prototype.getAllowedOccurrences = function () {
                        return this.input.getOccurrences();
                    };
                    InputOccurrences.prototype.constructOccurrencesForData = function () {
                        var _this = this;
                        this.propertyArray.forEach(function (property, index) {
                            _this.addOccurrence(new support.InputOccurrence(_this, index));
                        });
                        if (this.countOccurrences() < this.input.getOccurrences().getMinimum()) {
                            for (var index = this.countOccurrences(); index < this.input.getOccurrences().getMinimum(); index++) {
                                this.addOccurrence(this.createNewOccurrence(this, index));
                            }
                        }
                    };
                    InputOccurrences.prototype.createNewOccurrence = function (formItemOccurrences, insertAtIndex) {
                        return new support.InputOccurrence(formItemOccurrences, insertAtIndex);
                    };
                    InputOccurrences.prototype.createNewOccurrenceView = function (occurrence) {
                        var newInitialValue = this.baseInputTypeView.newInitialValue();
                        api.util.assertNotNull(newInitialValue, "InputTypeView-s extending BaseInputTypeNotManagingAdd must must return a Value from newInitialValue");
                        var property = this.propertyArray.get(occurrence.getIndex());
                        if (!property) {
                            property = this.propertyArray.add(newInitialValue);
                        }
                        var inputOccurrenceView = new support.InputOccurrenceView(occurrence, this.baseInputTypeView, property);
                        var inputOccurrences = this;
                        inputOccurrenceView.onRemoveButtonClicked(function (event) {
                            inputOccurrences.doRemoveOccurrence(event.getView(), event.getIndex());
                        });
                        return inputOccurrenceView;
                    };
                    InputOccurrences.prototype.giveFocus = function () {
                        var focusGiven = false;
                        var occurrenceViews = this.getOccurrenceViews();
                        if (occurrenceViews.length > 0) {
                            for (var i = 0; i < occurrenceViews.length; i++) {
                                if (occurrenceViews[i].giveFocus()) {
                                    focusGiven = true;
                                    break;
                                }
                            }
                        }
                        return focusGiven;
                    };
                    InputOccurrences.create = function () {
                        return new InputOccurrencesBuilder();
                    };
                    return InputOccurrences;
                })(api.form.FormItemOccurrences);
                support.InputOccurrences = InputOccurrences;
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var InputOccurrenceView = (function (_super) {
                    __extends(InputOccurrenceView, _super);
                    function InputOccurrenceView(inputOccurrence, baseInputTypeView, property) {
                        var _this = this;
                        _super.call(this, "input-occurrence-view", inputOccurrence);
                        this.property = property;
                        this.inputTypeView = baseInputTypeView;
                        this.inputElement = this.inputTypeView.createInputOccurrenceElement(inputOccurrence.getIndex(), property);
                        this.requiredContractBroken = this.inputTypeView.valueBreaksRequiredContract(property != null ? property.getValue() : null);
                        var propertyValueChangedHandler = function (event) {
                            var newStateOfRequiredContractBroken = _this.inputTypeView.valueBreaksRequiredContract(event.getNewValue());
                            if (_this.requiredContractBroken != newStateOfRequiredContractBroken) {
                                _this.requiredContractBroken = newStateOfRequiredContractBroken;
                                _this.inputTypeView.notifyRequiredContractBroken(newStateOfRequiredContractBroken, inputOccurrence.getIndex());
                            }
                        };
                        property.onPropertyValueChanged(propertyValueChangedHandler);
                        this.inputOccurrence = inputOccurrence;
                        this.dragControl = new api.dom.DivEl("drag-control");
                        this.appendChild(this.dragControl);
                        this.removeButtonEl = new api.dom.AEl("remove-button");
                        this.appendChild(this.removeButtonEl);
                        this.removeButtonEl.onClicked(function (event) {
                            _this.notifyRemoveButtonClicked();
                        });
                        var inputWrapper = new api.dom.DivEl("input-wrapper");
                        this.appendChild(inputWrapper);
                        inputWrapper.appendChild(this.inputElement);
                        this.refresh();
                    }
                    InputOccurrenceView.prototype.refresh = function () {
                        if (this.inputOccurrence.oneAndOnly()) {
                            this.addClass("single-occurrence").removeClass("multiple-occurrence");
                        }
                        else {
                            this.addClass("multiple-occurrence").removeClass("single-occurrence");
                        }
                        this.removeButtonEl.setVisible(this.inputOccurrence.isRemoveButtonRequiredStrict());
                    };
                    InputOccurrenceView.prototype.getDataPath = function () {
                        return this.property.getPath();
                    };
                    InputOccurrenceView.prototype.getIndex = function () {
                        return this.inputOccurrence.getIndex();
                    };
                    InputOccurrenceView.prototype.getInputElement = function () {
                        return this.inputElement;
                    };
                    InputOccurrenceView.prototype.hasValidUserInput = function () {
                        return this.inputTypeView.hasInputElementValidUserInput(this.inputElement);
                    };
                    InputOccurrenceView.prototype.giveFocus = function () {
                        return this.inputElement.giveFocus();
                    };
                    InputOccurrenceView.prototype.onFocus = function (listener) {
                        this.inputElement.onFocus(listener);
                    };
                    InputOccurrenceView.prototype.unFocus = function (listener) {
                        this.inputElement.unFocus(listener);
                    };
                    InputOccurrenceView.prototype.onBlur = function (listener) {
                        this.inputElement.onBlur(listener);
                    };
                    InputOccurrenceView.prototype.unBlur = function (listener) {
                        this.inputElement.unBlur(listener);
                    };
                    return InputOccurrenceView;
                })(api.form.FormItemOccurrenceView);
                support.InputOccurrenceView = InputOccurrenceView;
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var Value = api.data.Value;
                var BaseInputTypeNotManagingAdd = (function (_super) {
                    __extends(BaseInputTypeNotManagingAdd, _super);
                    function BaseInputTypeNotManagingAdd(context, className) {
                        var _this = this;
                        _super.call(this, "input-type-view" + (className ? " " + className : ""));
                        this.inputValidityChangedListeners = [];
                        api.util.assertNotNull(context, "context cannot be null");
                        this.context = context;
                        wemjq(this.getHTMLElement()).sortable({
                            axis: "y",
                            containment: 'parent',
                            handle: '.drag-control',
                            tolerance: 'pointer',
                            start: function (event, ui) { return _this.handleDnDStart(event, ui); },
                            stop: function (event, ui) { return _this.handleDnDStop(event, ui); },
                            update: function (event, ui) { return _this.handleDnDUpdate(event, ui); }
                        });
                    }
                    BaseInputTypeNotManagingAdd.prototype.handleDnDStart = function (event, ui) {
                        var draggedElement = api.dom.Element.fromHtmlElement(ui.item.context);
                        this.draggingIndex = draggedElement.getSiblingIndex();
                        ui.placeholder.html("Drop form item set here");
                    };
                    BaseInputTypeNotManagingAdd.prototype.handleDnDStop = function (event, ui) {
                    };
                    BaseInputTypeNotManagingAdd.prototype.handleDnDUpdate = function (event, ui) {
                        if (this.draggingIndex >= 0) {
                            var draggedElement = api.dom.Element.fromHtmlElement(ui.item.context);
                            var draggedToIndex = draggedElement.getSiblingIndex();
                            this.inputOccurrences.moveOccurrence(this.draggingIndex, draggedToIndex);
                        }
                        this.draggingIndex = -1;
                    };
                    BaseInputTypeNotManagingAdd.prototype.availableSizeChanged = function () {
                    };
                    BaseInputTypeNotManagingAdd.prototype.getContext = function () {
                        return this.context;
                    };
                    BaseInputTypeNotManagingAdd.prototype.getElement = function () {
                        return this;
                    };
                    BaseInputTypeNotManagingAdd.prototype.isManagingAdd = function () {
                        return false;
                    };
                    BaseInputTypeNotManagingAdd.prototype.onOccurrenceAdded = function (listener) {
                        this.inputOccurrences.onOccurrenceAdded(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.onOccurrenceRemoved = function (listener) {
                        this.inputOccurrences.onOccurrenceRemoved(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.unOccurrenceAdded = function (listener) {
                        this.inputOccurrences.unOccurrenceAdded(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.unOccurrenceRemoved = function (listener) {
                        this.inputOccurrences.unOccurrenceRemoved(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.onValidityChanged = function (listener) {
                        this.inputValidityChangedListeners.push(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.unValidityChanged = function (listener) {
                        this.inputValidityChangedListeners.filter(function (currentListener) {
                            return listener == currentListener;
                        });
                    };
                    BaseInputTypeNotManagingAdd.prototype.notifyValidityChanged = function (event) {
                        this.inputValidityChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    BaseInputTypeNotManagingAdd.prototype.maximumOccurrencesReached = function () {
                        return this.inputOccurrences.maximumOccurrencesReached();
                    };
                    BaseInputTypeNotManagingAdd.prototype.createAndAddOccurrence = function () {
                        this.inputOccurrences.createAndAddOccurrence();
                    };
                    BaseInputTypeNotManagingAdd.prototype.layout = function (input, propertyArray) {
                        var _this = this;
                        this.input = input;
                        this.propertyArray = propertyArray;
                        this.inputOccurrences = support.InputOccurrences.create().setBaseInputTypeView(this).setInput(this.input).setPropertyArray(propertyArray).build();
                        this.onAdded(function (event) {
                            _this.onOccurrenceAdded(function () {
                                wemjq(_this.getHTMLElement()).sortable("refresh");
                            });
                        });
                        return this.inputOccurrences.layout().then(function () {
                            wemjq(_this.getHTMLElement()).sortable("refresh");
                        });
                    };
                    BaseInputTypeNotManagingAdd.prototype.hasValidUserInput = function () {
                        return this.inputOccurrences.hasValidUserInput();
                    };
                    BaseInputTypeNotManagingAdd.prototype.hasInputElementValidUserInput = function (inputElement) {
                        throw new Error("Must be implemented by inheritor");
                    };
                    BaseInputTypeNotManagingAdd.prototype.onFocus = function (listener) {
                        this.inputOccurrences.onFocus(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.unFocus = function (listener) {
                        this.inputOccurrences.unFocus(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.onBlur = function (listener) {
                        this.inputOccurrences.onBlur(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.unBlur = function (listener) {
                        this.inputOccurrences.unBlur(listener);
                    };
                    BaseInputTypeNotManagingAdd.prototype.displayValidationErrors = function (value) {
                    };
                    BaseInputTypeNotManagingAdd.prototype.validate = function (silent) {
                        var _this = this;
                        if (silent === void 0) { silent = true; }
                        var recording = new api.form.inputtype.InputValidationRecording();
                        var numberOfValids = 0;
                        this.inputOccurrences.getOccurrenceViews().forEach(function (occurrenceView) {
                            var valueFromPropertyArray = _this.propertyArray.getValue(occurrenceView.getIndex());
                            if (valueFromPropertyArray) {
                                var breaksRequiredContract = _this.valueBreaksRequiredContract(valueFromPropertyArray);
                                if (!breaksRequiredContract) {
                                    numberOfValids++;
                                }
                            }
                        });
                        if (numberOfValids < this.input.getOccurrences().getMinimum()) {
                            recording.setBreaksMinimumOccurrences(true);
                        }
                        if (this.input.getOccurrences().maximumBreached(numberOfValids)) {
                            recording.setBreaksMaximumOccurrences(true);
                        }
                        var additionalValidation = this.getSpecialValidation();
                        recording.setAdditionalValidationRecord(additionalValidation);
                        if (!silent) {
                            if (recording.validityChanged(this.previousValidationRecording)) {
                                this.notifyValidityChanged(new api.form.inputtype.InputValidityChangedEvent(recording, this.input.getName()));
                            }
                        }
                        this.previousValidationRecording = recording;
                        return recording;
                    };
                    BaseInputTypeNotManagingAdd.prototype.getSpecialValidation = function () {
                        var result = api.form.AdditionalValidationRecord.create().setOverwriteDefault(false).build();
                        this.inputOccurrences.getOccurrenceViews().forEach(function (occurrenceView) {
                            var picker;
                            if (api.ObjectHelper.iFrameSafeInstanceOf(occurrenceView.getInputElement(), api.ui.time.DatePicker)) {
                                picker = (occurrenceView.getInputElement());
                            }
                            else if (api.ObjectHelper.iFrameSafeInstanceOf(occurrenceView.getInputElement(), api.ui.time.DateTimePicker)) {
                                picker = (occurrenceView.getInputElement());
                            }
                            else if (api.ObjectHelper.iFrameSafeInstanceOf(occurrenceView.getInputElement(), api.ui.time.LocalTime)) {
                                picker = (occurrenceView.getInputElement()).getTimePicker();
                            }
                            if (picker && !picker.hasValidUserInput()) {
                                result = api.form.AdditionalValidationRecord.create().setOverwriteDefault(true).setMessage("Incorrect value entered").build();
                                return;
                            }
                        });
                        return result;
                    };
                    BaseInputTypeNotManagingAdd.prototype.onValueChanged = function (property, value, type) {
                        var newValue = new Value(value, type);
                        property.setValue(newValue);
                        this.validate(false);
                    };
                    BaseInputTypeNotManagingAdd.prototype.notifyRequiredContractBroken = function (state, index) {
                        this.validate(false);
                    };
                    BaseInputTypeNotManagingAdd.prototype.getInput = function () {
                        return this.input;
                    };
                    BaseInputTypeNotManagingAdd.prototype.valueBreaksRequiredContract = function (value) {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeNotManagingAdd.prototype.createInputOccurrenceElement = function (index, property) {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeNotManagingAdd.prototype.getValueType = function () {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeNotManagingAdd.prototype.newInitialValue = function () {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeNotManagingAdd.prototype.giveFocus = function () {
                        if (this.inputOccurrences) {
                            return this.inputOccurrences.giveFocus();
                        }
                        return false;
                    };
                    BaseInputTypeNotManagingAdd.prototype.onEditContentRequest = function (listener) {
                    };
                    BaseInputTypeNotManagingAdd.prototype.unEditContentRequest = function (listener) {
                    };
                    return BaseInputTypeNotManagingAdd;
                })(api.dom.DivEl);
                support.BaseInputTypeNotManagingAdd = BaseInputTypeNotManagingAdd;
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var BaseInputTypeManagingAdd = (function (_super) {
                    __extends(BaseInputTypeManagingAdd, _super);
                    function BaseInputTypeManagingAdd(className) {
                        _super.call(this, "input-type-view" + (className ? " " + className : ""));
                        this.inputValidityChangedListeners = [];
                    }
                    BaseInputTypeManagingAdd.prototype.availableSizeChanged = function () {
                    };
                    BaseInputTypeManagingAdd.prototype.getElement = function () {
                        return this;
                    };
                    BaseInputTypeManagingAdd.prototype.isManagingAdd = function () {
                        return true;
                    };
                    BaseInputTypeManagingAdd.prototype.getValueType = function () {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeManagingAdd.prototype.newInitialValue = function () {
                        throw new Error("Must be overridden by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeManagingAdd.prototype.layout = function (input, propertyArray) {
                        this.input = input;
                        this.layoutInProgress = true;
                        this.propertyArray = propertyArray;
                        return wemQ(null);
                    };
                    BaseInputTypeManagingAdd.prototype.hasValidUserInput = function () {
                        return true;
                    };
                    BaseInputTypeManagingAdd.prototype.displayValidationErrors = function (value) {
                    };
                    BaseInputTypeManagingAdd.prototype.validate = function (silent, rec) {
                        if (silent === void 0) { silent = true; }
                        if (rec === void 0) { rec = null; }
                        var recording = rec || new api.form.inputtype.InputValidationRecording();
                        if (this.layoutInProgress) {
                            this.previousValidationRecording = recording;
                            return recording;
                        }
                        var numberOfValids = this.getNumberOfValids();
                        if (this.input.getOccurrences().minimumBreached(numberOfValids)) {
                            recording.setBreaksMinimumOccurrences(true);
                        }
                        if (this.input.getOccurrences().maximumBreached(numberOfValids)) {
                            recording.setBreaksMaximumOccurrences(true);
                        }
                        if (!silent && recording.validityChanged(this.previousValidationRecording)) {
                            this.notifyValidityChanged(new api.form.inputtype.InputValidityChangedEvent(recording, this.input.getName()));
                        }
                        this.previousValidationRecording = recording;
                        return recording;
                    };
                    BaseInputTypeManagingAdd.prototype.onValidityChanged = function (listener) {
                        this.inputValidityChangedListeners.push(listener);
                    };
                    BaseInputTypeManagingAdd.prototype.unValidityChanged = function (listener) {
                        this.inputValidityChangedListeners.filter(function (currentListener) {
                            return listener == currentListener;
                        });
                    };
                    BaseInputTypeManagingAdd.prototype.notifyValidityChanged = function (event) {
                        this.inputValidityChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    BaseInputTypeManagingAdd.prototype.giveFocus = function () {
                        throw new Error("Must be overridden by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeManagingAdd.prototype.onEditContentRequest = function (listener) {
                    };
                    BaseInputTypeManagingAdd.prototype.unEditContentRequest = function (listener) {
                    };
                    BaseInputTypeManagingAdd.prototype.getInput = function () {
                        return this.input;
                    };
                    BaseInputTypeManagingAdd.prototype.getNumberOfValids = function () {
                        throw new Error("Must be overridden by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeManagingAdd.prototype.isLayoutInProgress = function () {
                        return this.layoutInProgress;
                    };
                    BaseInputTypeManagingAdd.prototype.setLayoutInProgress = function (layoutInProgress) {
                        this.layoutInProgress = layoutInProgress;
                    };
                    BaseInputTypeManagingAdd.prototype.getPropertyArray = function () {
                        return this.propertyArray;
                    };
                    return BaseInputTypeManagingAdd;
                })(api.dom.DivEl);
                support.BaseInputTypeManagingAdd = BaseInputTypeManagingAdd;
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var BaseInputTypeSingleOccurrence = (function (_super) {
                    __extends(BaseInputTypeSingleOccurrence, _super);
                    function BaseInputTypeSingleOccurrence(ctx, className) {
                        _super.call(this, "input-type-view" + (className ? " " + className : ""));
                        this.inputValidityChangedListeners = [];
                        api.util.assertNotNull(ctx, "CONTEXT cannot be null");
                        this.context = ctx;
                    }
                    BaseInputTypeSingleOccurrence.prototype.availableSizeChanged = function () {
                    };
                    BaseInputTypeSingleOccurrence.prototype.getContext = function () {
                        return this.context;
                    };
                    BaseInputTypeSingleOccurrence.prototype.getElement = function () {
                        return this;
                    };
                    BaseInputTypeSingleOccurrence.prototype.isManagingAdd = function () {
                        return true;
                    };
                    BaseInputTypeSingleOccurrence.prototype.layout = function (input, propertyArray) {
                        this.layoutProperty(input, propertyArray.get(0));
                        return wemQ(null);
                    };
                    BaseInputTypeSingleOccurrence.prototype.layoutProperty = function (input, property) {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.getValueType = function () {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.newInitialValue = function () {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.displayValidationErrors = function (value) {
                    };
                    BaseInputTypeSingleOccurrence.prototype.hasValidUserInput = function () {
                        return true;
                    };
                    BaseInputTypeSingleOccurrence.prototype.validate = function (silent) {
                        if (silent === void 0) { silent = true; }
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.notifyValidityChanged = function (event) {
                        this.inputValidityChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    BaseInputTypeSingleOccurrence.prototype.onValidityChanged = function (listener) {
                        this.inputValidityChangedListeners.push(listener);
                    };
                    BaseInputTypeSingleOccurrence.prototype.unValidityChanged = function (listener) {
                        this.inputValidityChangedListeners.filter(function (currentListener) {
                            return listener == currentListener;
                        });
                    };
                    BaseInputTypeSingleOccurrence.prototype.onFocus = function (listener) {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.unFocus = function (listener) {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.onBlur = function (listener) {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.unBlur = function (listener) {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    BaseInputTypeSingleOccurrence.prototype.onEditContentRequest = function (listener) {
                    };
                    BaseInputTypeSingleOccurrence.prototype.unEditContentRequest = function (listener) {
                    };
                    return BaseInputTypeSingleOccurrence;
                })(api.dom.DivEl);
                support.BaseInputTypeSingleOccurrence = BaseInputTypeSingleOccurrence;
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var ValueTypes = api.data.ValueTypes;
                var NoInputTypeFoundView = (function (_super) {
                    __extends(NoInputTypeFoundView, _super);
                    function NoInputTypeFoundView(context) {
                        _super.call(this, context);
                    }
                    NoInputTypeFoundView.prototype.getValueType = function () {
                        return ValueTypes.STRING;
                    };
                    NoInputTypeFoundView.prototype.newInitialValue = function () {
                        return ValueTypes.STRING.newValue("");
                    };
                    NoInputTypeFoundView.prototype.layout = function (input, property) {
                        var divEl = new api.dom.DivEl();
                        divEl.getEl().setInnerHtml("Warning: no input type found: " + input.getInputType().toString());
                        return _super.prototype.layout.call(this, input, property);
                    };
                    NoInputTypeFoundView.prototype.createInputOccurrenceElement = function (index, property) {
                        var inputEl = api.ui.text.TextInput.middle();
                        inputEl.setName(this.getInput().getName());
                        if (property != null) {
                            inputEl.setValue(property.getString());
                        }
                        inputEl.onValueChanged(function (event) {
                            property.setValue(ValueTypes.STRING.newValue(event.getNewValue()));
                        });
                        return inputEl;
                    };
                    NoInputTypeFoundView.prototype.valueBreaksRequiredContract = function (value) {
                        return value.isNull() || !value.getType().equals(ValueTypes.STRING) || api.util.StringHelper.isBlank(value.getString());
                    };
                    NoInputTypeFoundView.prototype.hasInputElementValidUserInput = function (inputElement) {
                        return true;
                    };
                    return NoInputTypeFoundView;
                })(support.BaseInputTypeNotManagingAdd);
                support.NoInputTypeFoundView = NoInputTypeFoundView;
                api.form.inputtype.InputTypeManager.register(new api.Class("NoInputTypeFound", NoInputTypeFoundView));
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var support;
            (function (support) {
                var ValueChangedEvent = (function () {
                    function ValueChangedEvent(newValue) {
                        api.util.assertNotNull(newValue, "sending ValueChangedEvent-s for null values is not allowed");
                        this.newValue = newValue;
                    }
                    ValueChangedEvent.prototype.getNewValue = function () {
                        return this.newValue;
                    };
                    return ValueChangedEvent;
                })();
                support.ValueChangedEvent = ValueChangedEvent;
            })(support = inputtype.support || (inputtype.support = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var combobox;
            (function (combobox) {
                var ComboBoxDisplayValueViewer = (function (_super) {
                    __extends(ComboBoxDisplayValueViewer, _super);
                    function ComboBoxDisplayValueViewer() {
                        _super.call(this);
                    }
                    ComboBoxDisplayValueViewer.prototype.setObject = function (value) {
                        _super.prototype.setObject.call(this, value);
                        this.getEl().setInnerHtml(value);
                    };
                    ComboBoxDisplayValueViewer.prototype.getPreferredHeight = function () {
                        return 34;
                    };
                    return ComboBoxDisplayValueViewer;
                })(api.ui.Viewer);
                combobox.ComboBoxDisplayValueViewer = ComboBoxDisplayValueViewer;
            })(combobox = inputtype.combobox || (inputtype.combobox = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var combobox;
            (function (combobox) {
                var Value = api.data.Value;
                var ValueTypes = api.data.ValueTypes;
                var ComboBox = (function (_super) {
                    __extends(ComboBox, _super);
                    function ComboBox(context) {
                        _super.call(this, "combo-box");
                        this.addClass("input-type-view");
                        this.context = context;
                        this.readConfig(context.inputConfig);
                    }
                    ComboBox.prototype.readConfig = function (inputConfig) {
                        var options = [];
                        var optionValues = inputConfig['option'] || [];
                        var l = optionValues.length, optionValue;
                        for (var i = 0; i < l; i++) {
                            optionValue = optionValues[i];
                            options.push({ label: optionValue['value'], value: optionValue['@value'] });
                        }
                        this.comboBoxOptions = options;
                    };
                    ComboBox.prototype.availableSizeChanged = function () {
                    };
                    ComboBox.prototype.getValueType = function () {
                        return ValueTypes.STRING;
                    };
                    ComboBox.prototype.newInitialValue = function () {
                        return null;
                    };
                    ComboBox.prototype.layout = function (input, propertyArray) {
                        var _this = this;
                        _super.prototype.layout.call(this, input, propertyArray);
                        this.selectedOptionsView = new api.ui.selector.combobox.BaseSelectedOptionsView();
                        this.comboBox = this.createComboBox(input);
                        this.comboBoxOptions.forEach(function (option) {
                            _this.comboBox.addOption({ value: option.value, displayValue: option.label });
                        });
                        var valueArray = [];
                        this.getPropertyArray().forEach(function (property) {
                            valueArray.push(property.getString());
                        });
                        this.comboBox.setValues(valueArray, true);
                        this.appendChild(this.comboBox);
                        this.appendChild(this.selectedOptionsView);
                        this.setLayoutInProgress(false);
                        return wemQ(null);
                    };
                    ComboBox.prototype.createComboBox = function (input) {
                        var _this = this;
                        var comboBox = new api.ui.selector.combobox.ComboBox(name, {
                            filter: this.comboBoxFilter,
                            selectedOptionsView: this.selectedOptionsView,
                            maximumOccurrences: input.getOccurrences().getMaximum(),
                            optionDisplayValueViewer: new combobox.ComboBoxDisplayValueViewer(),
                            hideComboBoxWhenMaxReached: true
                        });
                        comboBox.onOptionFilterInputValueChanged(function (event) {
                            _this.comboBox.setFilterArgs({ searchString: event.getNewValue() });
                        });
                        comboBox.onOptionSelected(function (event) {
                            var value = new Value(event.getOption().value, ValueTypes.STRING);
                            if (event.getIndex() >= 0) {
                                _this.getPropertyArray().set(event.getIndex(), value);
                            }
                            else {
                                _this.getPropertyArray().add(value);
                            }
                            _this.validate(false);
                        });
                        comboBox.onOptionDeselected(function (removed) {
                            _this.getPropertyArray().remove(removed.getIndex());
                            _this.validate(false);
                        });
                        return comboBox;
                    };
                    ComboBox.prototype.giveFocus = function () {
                        if (this.comboBox.maximumOccurrencesReached()) {
                            return false;
                        }
                        return this.comboBox.giveFocus();
                    };
                    ComboBox.prototype.valueBreaksRequiredContract = function (value) {
                        return value.isNull() || !value.getType().equals(ValueTypes.STRING) || !this.isExistingValue(value.getString());
                    };
                    ComboBox.prototype.isExistingValue = function (value) {
                        return this.comboBoxOptions.some(function (option) {
                            return option.value == value;
                        });
                    };
                    ComboBox.prototype.comboBoxFilter = function (item, args) {
                        return !(args && args.searchString && item.displayValue.toUpperCase().indexOf(args.searchString.toUpperCase()) == -1);
                    };
                    ComboBox.prototype.getNumberOfValids = function () {
                        return this.comboBox.countSelectedOptions();
                    };
                    ComboBox.prototype.onFocus = function (listener) {
                        this.comboBox.onFocus(listener);
                    };
                    ComboBox.prototype.unFocus = function (listener) {
                        this.comboBox.unFocus(listener);
                    };
                    ComboBox.prototype.onBlur = function (listener) {
                        this.comboBox.onBlur(listener);
                    };
                    ComboBox.prototype.unBlur = function (listener) {
                        this.comboBox.unBlur(listener);
                    };
                    return ComboBox;
                })(api.form.inputtype.support.BaseInputTypeManagingAdd);
                combobox.ComboBox = ComboBox;
                api.form.inputtype.InputTypeManager.register(new api.Class("ComboBox", ComboBox));
            })(combobox = inputtype.combobox || (inputtype.combobox = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var radiobutton;
            (function (radiobutton) {
                var Value = api.data.Value;
                var ValueTypes = api.data.ValueTypes;
                var RadioButton = (function (_super) {
                    __extends(RadioButton, _super);
                    function RadioButton(config) {
                        _super.call(this, config, "radio-button");
                        this.readConfig(config.inputConfig);
                    }
                    RadioButton.prototype.readConfig = function (inputConfig) {
                        var options = [];
                        var optionValues = inputConfig['option'] || [];
                        var l = optionValues.length, optionValue;
                        for (var i = 0; i < l; i++) {
                            optionValue = optionValues[i];
                            options.push({ label: optionValue['value'], value: optionValue['@value'] });
                        }
                        this.radioButtonOptions = options;
                    };
                    RadioButton.prototype.getValueType = function () {
                        return ValueTypes.STRING;
                    };
                    RadioButton.prototype.newInitialValue = function () {
                        return ValueTypes.STRING.newNullValue();
                    };
                    RadioButton.prototype.layoutProperty = function (input, property) {
                        this.input = input;
                        this.property = property;
                        this.selector = this.createRadioElement(input.getName(), property);
                        this.appendChild(this.selector);
                        return wemQ(null);
                    };
                    RadioButton.prototype.giveFocus = function () {
                        return this.selector.giveFocus();
                    };
                    RadioButton.prototype.validate = function (silent) {
                        if (silent === void 0) { silent = true; }
                        var recording = new api.form.inputtype.InputValidationRecording();
                        var propertyValue = this.property.getValue();
                        if (propertyValue.isNull() && this.input.getOccurrences().getMinimum() > 0) {
                            recording.setBreaksMinimumOccurrences(true);
                        }
                        if (!silent) {
                            if (recording.validityChanged(this.previousValidationRecording)) {
                                this.notifyValidityChanged(new api.form.inputtype.InputValidityChangedEvent(recording, this.input.getName()));
                            }
                        }
                        this.previousValidationRecording = recording;
                        return recording;
                    };
                    RadioButton.prototype.onFocus = function (listener) {
                        this.selector.onFocus(listener);
                    };
                    RadioButton.prototype.unFocus = function (listener) {
                        this.selector.unFocus(listener);
                    };
                    RadioButton.prototype.onBlur = function (listener) {
                        this.selector.onBlur(listener);
                    };
                    RadioButton.prototype.unBlur = function (listener) {
                        this.selector.unBlur(listener);
                    };
                    RadioButton.prototype.createRadioElement = function (name, property) {
                        var _this = this;
                        var radioGroup = new api.ui.RadioGroup(name);
                        var options = this.radioButtonOptions;
                        var l = options.length;
                        for (var i = 0; i < l; i++) {
                            var option = options[i];
                            radioGroup.addOption(option.value, option.label);
                        }
                        if (property.hasNonNullValue()) {
                            radioGroup.setValue(property.getString());
                        }
                        radioGroup.onValueChanged(function (event) {
                            property.setValue(_this.newValue(event.getNewValue()));
                            _this.validate(false);
                        });
                        return radioGroup;
                    };
                    RadioButton.prototype.newValue = function (s) {
                        return new Value(s, ValueTypes.STRING);
                    };
                    return RadioButton;
                })(api.form.inputtype.support.BaseInputTypeSingleOccurrence);
                radiobutton.RadioButton = RadioButton;
                api.form.inputtype.InputTypeManager.register(new api.Class("RadioButton", RadioButton));
            })(radiobutton = inputtype.radiobutton || (inputtype.radiobutton = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (_form) {
        var inputtype;
        (function (inputtype) {
            var text;
            (function (text) {
                var htmlarea;
                (function (htmlarea) {
                    var Form = api.ui.form.Form;
                    var Fieldset = api.ui.form.Fieldset;
                    var FormItemBuilder = api.ui.form.FormItemBuilder;
                    var BaseDialog = api.ui.dialog.ModalDialog;
                    var ModalDialog = (function (_super) {
                        __extends(ModalDialog, _super);
                        function ModalDialog(editor, title) {
                            var _this = this;
                            _super.call(this, {
                                title: title
                            });
                            this.fields = {};
                            this.validated = false;
                            this.editor = editor;
                            this.getEl().addClass("html-area-modal-dialog");
                            this.layout();
                            this.initializeActions();
                            this.keyDownListener = function (e) { return _this.onDialogKeyDown(e); };
                            api.dom.Body.get().onKeyDown(this.keyDownListener);
                        }
                        ModalDialog.prototype.setSubmitAction = function (action) {
                            this.submitAction = action;
                        };
                        ModalDialog.prototype.onDialogKeyDown = function (e) {
                            if (api.ui.KeyHelper.isEscKey(e)) {
                                this.getCancelAction().execute();
                            }
                            if (api.ui.KeyHelper.isEnterKey(e)) {
                                this.submitAction.execute();
                            }
                        };
                        ModalDialog.prototype.getEditor = function () {
                            return this.editor;
                        };
                        ModalDialog.prototype.setValidated = function () {
                            this.validated = true;
                        };
                        ModalDialog.prototype.setFirstFocusField = function (field) {
                            this.firstFocusField = field;
                        };
                        ModalDialog.prototype.focusFirstField = function () {
                            this.firstFocusField.giveFocus();
                        };
                        ModalDialog.prototype.layout = function () {
                            this.appendChildToContentPanel(this.createMainForm());
                        };
                        ModalDialog.prototype.getMainFormItems = function () {
                            return [];
                        };
                        ModalDialog.prototype.createMainForm = function () {
                            return this.mainForm = this.createForm(this.getMainFormItems());
                        };
                        ModalDialog.prototype.validate = function () {
                            this.setValidated();
                            return this.mainForm.validate(true).isValid();
                        };
                        ModalDialog.prototype.show = function () {
                            api.dom.Body.get().appendChild(this);
                            _super.prototype.show.call(this);
                            if (this.firstFocusField) {
                                this.focusFirstField();
                            }
                        };
                        ModalDialog.prototype.createForm = function (formItems) {
                            var _this = this;
                            var form = new Form(), validationCls = "display-validation-errors";
                            formItems.forEach(function (formItem) {
                                form.add(_this.createFieldSet(formItem));
                                if (formItem.getValidator() && validationCls) {
                                    form.addClass(validationCls);
                                    validationCls = "";
                                }
                            });
                            return form;
                        };
                        ModalDialog.prototype.createFormPanel = function (formItems) {
                            var panel = new api.ui.panel.Panel(), form = this.createForm(formItems);
                            panel.appendChild(form);
                            return panel;
                        };
                        ModalDialog.prototype.createFieldSet = function (formItem) {
                            var fieldSet = new Fieldset();
                            fieldSet.addClass("modal-dialog-fieldset");
                            fieldSet.add(formItem);
                            if (formItem.getValidator()) {
                                var validationRecordingViewer = new _form.ValidationRecordingViewer();
                                fieldSet.appendChild(validationRecordingViewer);
                                fieldSet.onValidityChanged(function (event) {
                                    validationRecordingViewer.setError(formItem.getError());
                                });
                            }
                            return fieldSet;
                        };
                        ModalDialog.prototype.onValidatedFieldValueChanged = function (formItem) {
                            if (this.validated) {
                                formItem.validate(new api.ui.form.ValidationResult(), true);
                            }
                        };
                        ModalDialog.prototype.createFormItem = function (id, label, validator, value, inputEl) {
                            var formItemEl = inputEl || new api.ui.text.TextInput(), formItemBuilder = new FormItemBuilder(formItemEl).setLabel(label), required = false, inputWrapper = new api.dom.DivEl("input-wrapper"), formItem;
                            if (this.fields[id]) {
                                throw "Element with id " + id + " already exists";
                            }
                            if (value) {
                                formItemEl.setValue(value);
                            }
                            this.fields[id] = formItemEl;
                            if (validator) {
                                formItemBuilder.setValidator(validator);
                                required = true;
                            }
                            formItem = formItemBuilder.build();
                            formItem.getInput().wrapWithElement(inputWrapper);
                            if (validator) {
                                if (api.ObjectHelper.iFrameSafeInstanceOf(formItemEl, api.ui.text.TextInput)) {
                                    formItemEl.onValueChanged(this.onValidatedFieldValueChanged.bind(this, formItem));
                                }
                                if (api.ObjectHelper.iFrameSafeInstanceOf(formItemEl, api.ui.selector.combobox.RichComboBox)) {
                                    formItemEl.onOptionSelected(this.onValidatedFieldValueChanged.bind(this, formItem));
                                    formItemEl.onOptionDeselected(this.onValidatedFieldValueChanged.bind(this, formItem));
                                }
                            }
                            return formItem;
                        };
                        ModalDialog.prototype.initializeActions = function () {
                            this.addCancelButtonToBottom();
                        };
                        ModalDialog.prototype.getFieldById = function (id) {
                            return this.fields[id];
                        };
                        ModalDialog.prototype.close = function () {
                            _super.prototype.close.call(this);
                            api.dom.Body.get().unKeyDown(this.keyDownListener);
                            this.editor.focus();
                        };
                        return ModalDialog;
                    })(BaseDialog);
                    htmlarea.ModalDialog = ModalDialog;
                })(htmlarea = text.htmlarea || (text.htmlarea = {}));
            })(text = inputtype.text || (inputtype.text = {}));
        })(inputtype = _form.inputtype || (_form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (_form) {
        var inputtype;
        (function (inputtype) {
            var text;
            (function (text) {
                var htmlarea;
                (function (htmlarea) {
                    var DockedPanel = api.ui.panel.DockedPanel;
                    var Validators = api.ui.form.Validators;
                    var Dropdown = api.ui.selector.dropdown.Dropdown;
                    var LinkModalDialog = (function (_super) {
                        __extends(LinkModalDialog, _super);
                        function LinkModalDialog(config) {
                            this.link = config.element;
                            this.linkText = config.text;
                            this.anchorList = config.anchorList;
                            this.onlyTextSelected = config.onlyTextSelected;
                            _super.call(this, config.editor, new api.ui.dialog.ModalDialogHeader("Insert Link"));
                        }
                        LinkModalDialog.prototype.getHref = function () {
                            return this.link ? this.link.getAttribute("href") : api.util.StringHelper.EMPTY_STRING;
                        };
                        LinkModalDialog.prototype.getLinkText = function () {
                            return this.link ? this.link["text"] : this.linkText;
                        };
                        LinkModalDialog.prototype.getToolTip = function () {
                            return this.link ? this.link.getAttribute("title") : api.util.StringHelper.EMPTY_STRING;
                        };
                        LinkModalDialog.prototype.isContentLink = function () {
                            return this.getHref().indexOf(LinkModalDialog.contentPrefix) === 0;
                        };
                        LinkModalDialog.prototype.getContentId = function () {
                            if (this.link && this.isContentLink()) {
                                return this.getHref().replace(LinkModalDialog.contentPrefix, api.util.StringHelper.EMPTY_STRING);
                            }
                            return api.util.StringHelper.EMPTY_STRING;
                        };
                        LinkModalDialog.prototype.isDownloadLink = function () {
                            return this.getHref().indexOf(LinkModalDialog.downloadPrefix) === 0;
                        };
                        LinkModalDialog.prototype.getDownloadId = function () {
                            return this.isDownloadLink() ? this.getHref().replace(LinkModalDialog.downloadPrefix, api.util.StringHelper.EMPTY_STRING) : api.util.StringHelper.EMPTY_STRING;
                        };
                        LinkModalDialog.prototype.isUrl = function () {
                            return this.link ? !(this.isContentLink() || this.isDownloadLink() || this.isEmail()) : false;
                        };
                        LinkModalDialog.prototype.getUrl = function () {
                            return this.isUrl() ? this.getHref() : api.util.StringHelper.EMPTY_STRING;
                        };
                        LinkModalDialog.prototype.isEmail = function () {
                            return this.getHref().indexOf(LinkModalDialog.emailPrefix) === 0;
                        };
                        LinkModalDialog.prototype.getEmail = function () {
                            if (!this.isEmail()) {
                                return api.util.StringHelper.EMPTY_STRING;
                            }
                            var emailArr = this.getHref().split(LinkModalDialog.subjectPrefix);
                            return emailArr[0].replace(LinkModalDialog.emailPrefix, api.util.StringHelper.EMPTY_STRING);
                        };
                        LinkModalDialog.prototype.isAnchor = function () {
                            return this.getHref().indexOf("#") === 0;
                        };
                        LinkModalDialog.prototype.getAnchor = function () {
                            return this.isAnchor() ? this.getHref() : api.util.StringHelper.EMPTY_STRING;
                        };
                        LinkModalDialog.prototype.getSubject = function () {
                            if (!this.isEmail()) {
                                return api.util.StringHelper.EMPTY_STRING;
                            }
                            var emailArr = this.getHref().split(LinkModalDialog.subjectPrefix);
                            return decodeURI(emailArr[1].replace(LinkModalDialog.subjectPrefix, api.util.StringHelper.EMPTY_STRING));
                        };
                        LinkModalDialog.prototype.layout = function () {
                            _super.prototype.layout.call(this);
                            this.appendChildToContentPanel(this.dockedPanel = this.createDockedPanel());
                        };
                        LinkModalDialog.prototype.createContentPanel = function () {
                            return this.createFormPanel([
                                this.createContentSelector("contentId", "Target", this.getContentId()),
                                this.createTargetCheckbox("contentTarget", this.isContentLink())
                            ]);
                        };
                        LinkModalDialog.prototype.createDownloadPanel = function () {
                            return this.createFormPanel([
                                this.createContentSelector("downloadId", "Target", this.getDownloadId(), api.schema.content.ContentTypeName.getMediaTypes())
                            ]);
                        };
                        LinkModalDialog.prototype.createUrlPanel = function () {
                            return this.createFormPanel([
                                this.createFormItem("url", "Url", Validators.required, this.getUrl()),
                                this.createTargetCheckbox("urlTarget", this.isUrl())
                            ]);
                        };
                        LinkModalDialog.prototype.createAnchorPanel = function () {
                            return this.createFormPanel([
                                this.createAnchorDropdown()
                            ]);
                        };
                        LinkModalDialog.prototype.createEmailPanel = function () {
                            var emailFormItem = this.createFormItem("email", "Email", LinkModalDialog.validationRequiredEmail, this.getEmail());
                            emailFormItem.getLabel().addClass("required");
                            return this.createFormPanel([
                                emailFormItem,
                                this.createFormItem("subject", "Subject", null, this.getSubject())
                            ]);
                        };
                        LinkModalDialog.validationRequiredEmail = function (input) {
                            var isValid;
                            if (!(isValid = Validators.required(input))) {
                                isValid = Validators.validEmail(input);
                            }
                            return isValid;
                        };
                        LinkModalDialog.prototype.getTarget = function (isTabSelected) {
                            return isTabSelected ? !api.util.StringHelper.isBlank(this.link.getAttribute("target")) : false;
                        };
                        LinkModalDialog.prototype.createTargetCheckbox = function (id, isTabSelected) {
                            var checkbox = new api.ui.Checkbox().setChecked(this.getTarget(isTabSelected));
                            return this.createFormItem(id, "Open new window", null, null, checkbox);
                        };
                        LinkModalDialog.prototype.getMainFormItems = function () {
                            var items = [];
                            if (this.onlyTextSelected) {
                                var linkTextFormItem = this.createFormItem("linkText", "Text", Validators.required, this.getLinkText());
                                this.setFirstFocusField(linkTextFormItem.getInput());
                                items.push(linkTextFormItem);
                            }
                            items.push(this.createFormItem("toolTip", "Tooltip", null, this.getToolTip()));
                            return items;
                        };
                        LinkModalDialog.prototype.createDockedPanel = function () {
                            var dockedPanel = new DockedPanel();
                            dockedPanel.addItem(LinkModalDialog.tabNames.content, true, this.createContentPanel());
                            dockedPanel.addItem(LinkModalDialog.tabNames.url, true, this.createUrlPanel(), this.isUrl());
                            dockedPanel.addItem(LinkModalDialog.tabNames.download, true, this.createDownloadPanel(), this.isDownloadLink());
                            dockedPanel.addItem(LinkModalDialog.tabNames.email, true, this.createEmailPanel(), this.isEmail());
                            if (this.anchorList.length) {
                                dockedPanel.addItem(LinkModalDialog.tabNames.anchor, true, this.createAnchorPanel(), this.isAnchor());
                            }
                            return dockedPanel;
                        };
                        LinkModalDialog.prototype.initializeActions = function () {
                            var _this = this;
                            var submitAction = new api.ui.Action(this.link ? "Update" : "Insert", "enter");
                            this.setSubmitAction(submitAction);
                            this.addAction(submitAction.onExecuted(function () {
                                if (_this.validate()) {
                                    _this.createLink();
                                    _this.close();
                                }
                            }));
                            _super.prototype.initializeActions.call(this);
                        };
                        LinkModalDialog.prototype.createContentSelector = function (id, label, value, contentTypeNames) {
                            var _this = this;
                            var loader = new api.content.ContentSummaryLoader(), contentSelector = api.content.ContentComboBox.create().setLoader(loader).setMaximumOccurrences(1).build(), contentSelectorComboBox = contentSelector.getComboBox();
                            if (contentTypeNames) {
                                loader.setAllowedContentTypeNames(contentTypeNames);
                            }
                            contentSelectorComboBox.onExpanded(function (event) {
                                _this.adjustDropDown(contentSelectorComboBox.getInput(), event.getDropdownElement().getEl());
                            });
                            contentSelectorComboBox.onKeyDown(function (e) {
                                if (api.ui.KeyHelper.isEscKey(e) && !contentSelectorComboBox.isDropdownShown()) {
                                    e.preventDefault();
                                    e.stopPropagation();
                                }
                            });
                            return this.createFormItem(id, label, Validators.required, value, contentSelector);
                        };
                        LinkModalDialog.prototype.createAnchorDropdown = function () {
                            var _this = this;
                            var dropDown = new Dropdown(name, {});
                            this.anchorList.forEach(function (anchor) {
                                dropDown.addOption({ value: "#" + anchor, displayValue: anchor });
                            });
                            dropDown.onExpanded(function (event) {
                                _this.adjustDropDown(dropDown, event.getDropdownElement().getEl());
                            });
                            if (this.getAnchor()) {
                                dropDown.setValue(this.getAnchor());
                            }
                            return this.createFormItem("anchor", "Anchor", Validators.required, null, dropDown);
                        };
                        LinkModalDialog.prototype.adjustDropDown = function (inputElement, dropDownElement) {
                            var inputPosition = wemjq(inputElement.getHTMLElement()).offset();
                            dropDownElement.setMaxWidthPx(inputElement.getEl().getWidthWithBorder());
                            dropDownElement.setTopPx(inputPosition.top + inputElement.getEl().getHeightWithBorder() - 1);
                            dropDownElement.setLeftPx(inputPosition.left);
                        };
                        LinkModalDialog.prototype.validateDockPanel = function () {
                            var form = this.dockedPanel.getDeck().getPanelShown().getFirstChild();
                            return form.validate(true).isValid();
                        };
                        LinkModalDialog.prototype.validate = function () {
                            var mainFormValid = _super.prototype.validate.call(this);
                            var dockPanelValid = this.validateDockPanel();
                            return mainFormValid && dockPanelValid;
                        };
                        LinkModalDialog.prototype.createContentLink = function () {
                            var contentSelector = this.getFieldById("contentId"), targetCheckbox = this.getFieldById("contentTarget");
                            var linkEl = new api.dom.AEl();
                            linkEl.setUrl(LinkModalDialog.contentPrefix + contentSelector.getValue(), targetCheckbox.isChecked() ? "_blank" : null);
                            return linkEl;
                        };
                        LinkModalDialog.prototype.createDownloadLink = function () {
                            var contentSelector = this.getFieldById("downloadId");
                            var linkEl = new api.dom.AEl();
                            linkEl.setUrl(LinkModalDialog.downloadPrefix + contentSelector.getValue());
                            return linkEl;
                        };
                        LinkModalDialog.prototype.createUrlLink = function () {
                            var url = this.getFieldById("url").getValue(), targetCheckbox = this.getFieldById("urlTarget");
                            var linkEl = new api.dom.AEl();
                            linkEl.setUrl(url, targetCheckbox.isChecked() ? "_blank" : null);
                            return linkEl;
                        };
                        LinkModalDialog.prototype.createAnchor = function () {
                            var anchorName = this.getFieldById("anchor").getValue();
                            var linkEl = new api.dom.AEl();
                            linkEl.setUrl(anchorName);
                            return linkEl;
                        };
                        LinkModalDialog.prototype.createEmailLink = function () {
                            var email = this.getFieldById("email").getValue(), subject = this.getFieldById("subject").getValue();
                            var linkEl = new api.dom.AEl();
                            linkEl.setUrl(LinkModalDialog.emailPrefix + email + (subject ? LinkModalDialog.subjectPrefix + encodeURI(subject) : ""));
                            return linkEl;
                        };
                        LinkModalDialog.prototype.createLink = function () {
                            var linkEl, deck = this.dockedPanel.getDeck(), selectedTab = deck.getSelectedNavigationItem(), linkText = this.onlyTextSelected ? this.getFieldById("linkText").getValue() : "", toolTip = this.getFieldById("toolTip").getValue();
                            switch (selectedTab.getLabel()) {
                                case (LinkModalDialog.tabNames.content):
                                    linkEl = this.createContentLink();
                                    break;
                                case (LinkModalDialog.tabNames.url):
                                    linkEl = this.createUrlLink();
                                    break;
                                case (LinkModalDialog.tabNames.download):
                                    linkEl = this.createDownloadLink();
                                    break;
                                case (LinkModalDialog.tabNames.email):
                                    linkEl = this.createEmailLink();
                                    break;
                                case (LinkModalDialog.tabNames.anchor):
                                    linkEl = this.createAnchor();
                                    break;
                            }
                            linkEl.setHtml(linkText);
                            if (toolTip) {
                                linkEl.setTitle(toolTip);
                            }
                            if (this.link) {
                                this.link.parentElement.replaceChild(linkEl.getHTMLElement(), this.link);
                            }
                            else {
                                if (this.onlyTextSelected) {
                                    this.getEditor().insertContent(linkEl.toString());
                                }
                                else {
                                    var linkAttrs = {
                                        href: linkEl.getHref(),
                                        target: linkEl.getTarget() ? linkEl.getTarget() : null,
                                        rel: null,
                                        "class": null,
                                        title: linkEl.getTitle()
                                    };
                                    this.getEditor().execCommand('mceInsertLink', false, linkAttrs);
                                }
                            }
                        };
                        LinkModalDialog.tabNames = {
                            content: "Content",
                            url: "URL",
                            download: "Download",
                            email: "Email",
                            anchor: "Anchor"
                        };
                        LinkModalDialog.contentPrefix = "content://";
                        LinkModalDialog.downloadPrefix = "media://download/";
                        LinkModalDialog.emailPrefix = "mailto:";
                        LinkModalDialog.subjectPrefix = "?subject=";
                        return LinkModalDialog;
                    })(htmlarea.ModalDialog);
                    htmlarea.LinkModalDialog = LinkModalDialog;
                })(htmlarea = text.htmlarea || (text.htmlarea = {}));
            })(text = inputtype.text || (inputtype.text = {}));
        })(inputtype = _form.inputtype || (_form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var text;
            (function (_text) {
                var htmlarea;
                (function (htmlarea) {
                    var Validators = api.ui.form.Validators;
                    var ImageModalDialog = (function (_super) {
                        __extends(ImageModalDialog, _super);
                        function ImageModalDialog(config, contentId) {
                            this.imageElement = config.element;
                            this.elementContainer = config.container;
                            this.contentId = contentId;
                            this.callback = config.callback;
                            _super.call(this, config.editor, new api.ui.dialog.ModalDialogHeader("Insert Image"));
                        }
                        ImageModalDialog.prototype.getImageId = function (images) {
                            var _this = this;
                            var filteredImages = images.filter(function (image) {
                                return _this.imageElement.src.indexOf(image.getId()) > 0;
                            });
                            return filteredImages.length > 0 ? filteredImages[0].getId() : api.util.StringHelper.EMPTY_STRING;
                        };
                        ImageModalDialog.prototype.createImageSelector = function (id) {
                            var _this = this;
                            var loader = new api.content.ContentSummaryLoader(), imageSelector = api.content.ContentComboBox.create().setLoader(loader).setMaximumOccurrences(1).build(), formItem = this.createFormItem(id, "Image", Validators.required, api.util.StringHelper.EMPTY_STRING, imageSelector), imageSelectorComboBox = imageSelector.getComboBox();
                            this.imageSelector = imageSelector;
                            formItem.addClass("image-selector");
                            loader.setAllowedContentTypeNames([api.schema.content.ContentTypeName.IMAGE]);
                            if (this.imageElement) {
                                var singleLoadListener = function (event) {
                                    var imageId = _this.getImageId(event.getData());
                                    if (imageId) {
                                        imageSelector.setValue(imageId);
                                    }
                                    loader.unLoadedData(singleLoadListener);
                                };
                                loader.onLoadedData(singleLoadListener);
                                loader.load();
                            }
                            imageSelectorComboBox.onOptionSelected(function (event) {
                                var imageContent = event.getOption().displayValue;
                                if (!imageContent.getContentId()) {
                                    return;
                                }
                                formItem.addClass("image-preview");
                                _this.previewImage(imageContent);
                                _this.uploader.hide();
                            });
                            imageSelectorComboBox.onExpanded(function (event) {
                                if (event.isExpanded()) {
                                    _this.adjustSelectorDropDown(imageSelectorComboBox.getInput(), event.getDropdownElement().getEl());
                                }
                            });
                            imageSelectorComboBox.onOptionDeselected(function () {
                                formItem.removeClass("image-preview");
                                _this.removePreview();
                                _this.uploader.show();
                                api.ui.responsive.ResponsiveManager.fireResizeEvent();
                            });
                            imageSelectorComboBox.onKeyDown(function (e) {
                                if (api.ui.KeyHelper.isEscKey(e) && !imageSelectorComboBox.isDropdownShown()) {
                                    e.preventDefault();
                                    e.stopPropagation();
                                }
                            });
                            return formItem;
                        };
                        ImageModalDialog.prototype.adjustSelectorDropDown = function (inputElement, dropDownElement) {
                            var inputPosition = wemjq(inputElement.getHTMLElement()).offset();
                            dropDownElement.setMaxWidthPx(inputElement.getEl().getWidthWithBorder() - 2);
                            dropDownElement.setTopPx(inputPosition.top + inputElement.getEl().getHeightWithBorder() - 1);
                            dropDownElement.setLeftPx(inputPosition.left);
                        };
                        ImageModalDialog.prototype.createImgEl = function (src, alt, contentId) {
                            var imageEl = new api.dom.ImgEl(src);
                            imageEl.getEl().setAttribute("alt", alt);
                            imageEl.getEl().setAttribute("style", "width: 100%;");
                            imageEl.getEl().setAttribute("data-src", _text.HtmlArea.imagePrefix + contentId);
                            return imageEl;
                        };
                        ImageModalDialog.prototype.previewImage = function (imageContent) {
                            var _this = this;
                            var contentId = imageContent.getContentId().toString(), imgUrl = new api.content.ContentImageUrlResolver().setContentId(new api.content.ContentId(contentId)).setScaleWidth(true).setSize(_text.HtmlArea.maxImageWidth).resolve();
                            this.image = this.createImgEl(imgUrl, imageContent.getDisplayName(), contentId);
                            this.image.onLoaded(function () {
                                _this.imagePreviewContainer.removeClass("upload");
                                api.ui.responsive.ResponsiveManager.fireResizeEvent();
                            });
                            this.hideUploadMasks();
                            this.imagePreviewContainer.insertChild(this.image, 0);
                        };
                        ImageModalDialog.prototype.removePreview = function () {
                            this.imagePreviewContainer.removeChild(this.image);
                        };
                        ImageModalDialog.prototype.show = function () {
                            _super.prototype.show.call(this);
                            this.uploader.show();
                        };
                        ImageModalDialog.prototype.getMainFormItems = function () {
                            var imageSelector = this.createImageSelector("imageId");
                            this.addUploaderAndPreviewControls(imageSelector);
                            this.setFirstFocusField(imageSelector.getInput());
                            return [
                                imageSelector
                            ];
                        };
                        ImageModalDialog.prototype.getImagePreviewContainer = function () {
                            var imagePreviewContainer = new api.dom.DivEl("content-item-preview-panel");
                            this.progress = new api.ui.ProgressBar();
                            imagePreviewContainer.appendChild(this.progress);
                            this.error = new api.dom.DivEl("error");
                            imagePreviewContainer.appendChild(this.error);
                            return imagePreviewContainer;
                        };
                        ImageModalDialog.prototype.addUploaderAndPreviewControls = function (imageSelector) {
                            var imageSelectorContainer = imageSelector.getInput().getParentElement();
                            imageSelectorContainer.appendChild(this.uploader = this.createImageUploader());
                            this.imagePreviewContainer = this.getImagePreviewContainer();
                            wemjq(this.imagePreviewContainer.getHTMLElement()).insertAfter(imageSelectorContainer.getHTMLElement());
                        };
                        ImageModalDialog.prototype.createImageUploader = function () {
                            var _this = this;
                            var uploader = new api.content.ImageUploader({
                                params: {
                                    parent: this.contentId.toString()
                                },
                                operation: 0 /* create */,
                                name: 'image-selector-upload-dialog',
                                showButtons: false,
                                showResult: false,
                                maximumOccurrences: 1,
                                allowMultiSelection: false,
                                scaleWidth: false,
                                deferred: true
                            });
                            uploader.addClass("minimized");
                            uploader.hide();
                            uploader.onUploadStarted(function (event) {
                                _this.hideUploadMasks();
                                _this.imagePreviewContainer.addClass("upload");
                                _this.showProgress();
                            });
                            uploader.onUploadProgress(function (event) {
                                var item = event.getUploadItem();
                                _this.setProgress(item.getProgress());
                            });
                            uploader.onFileUploaded(function (event) {
                                var item = event.getUploadItem();
                                var createdContent = item.getModel();
                                new api.content.ContentUpdatedEvent(_this.contentId).fire();
                                _this.imageSelector.setContent(createdContent);
                            });
                            uploader.onUploadFailed(function (event) {
                                _this.showError("Upload failed");
                            });
                            return uploader;
                        };
                        ImageModalDialog.prototype.setProgress = function (value) {
                            this.progress.setValue(value);
                        };
                        ImageModalDialog.prototype.showProgress = function () {
                            this.progress.show();
                        };
                        ImageModalDialog.prototype.hideUploadMasks = function () {
                            this.progress.hide();
                            this.error.hide();
                        };
                        ImageModalDialog.prototype.showError = function (text) {
                            this.progress.hide();
                            this.error.setHtml(text).show();
                            this.error.show();
                        };
                        ImageModalDialog.prototype.initializeActions = function () {
                            var _this = this;
                            var submitAction = new api.ui.Action(this.imageElement ? "Update" : "Insert", "enter");
                            this.setSubmitAction(submitAction);
                            this.addAction(submitAction.onExecuted(function () {
                                if (_this.validate()) {
                                    _this.createImageTag();
                                    _this.close();
                                }
                            }));
                            _super.prototype.initializeActions.call(this);
                        };
                        ImageModalDialog.prototype.generateUUID = function () {
                            var d = new Date().getTime();
                            var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
                                var r = (d + Math.random() * 16) % 16 | 0;
                                d = Math.floor(d / 16);
                                return (c == 'x' ? r : (r & 0x3 | 0x8)).toString(16);
                            });
                            return uuid;
                        };
                        ImageModalDialog.prototype.createFigureElement = function (figCaptionId) {
                            var figure = api.dom.ElementHelper.fromName("figure");
                            var figCaption = api.dom.ElementHelper.fromName("figcaption");
                            figCaption.setId(figCaptionId);
                            figCaption.setInnerHtml("caption");
                            figure.appendChildren([this.image.getEl().getHTMLElement(), figCaption.getHTMLElement()]);
                            return figure;
                        };
                        ImageModalDialog.prototype.createImageTag = function () {
                            var _this = this;
                            var container = this.elementContainer, isProperContainer = function () {
                                return container.nodeName !== "FIGCAPTION" && container.nodeName !== "#text";
                            };
                            if (this.imageElement) {
                                this.imageElement.parentElement.replaceChild(this.image.getEl().getHTMLElement(), this.imageElement);
                                setTimeout(function () {
                                    _this.getEditor().nodeChanged({ selectionChange: true });
                                }, 50);
                            }
                            else {
                                var figCaptionId = this.generateUUID(), figure = this.createFigureElement(figCaptionId);
                                if (!isProperContainer() || container.nodeName === "FIGURE") {
                                    while (!isProperContainer()) {
                                        container = container.parentElement;
                                    }
                                    figure.insertAfterEl(new api.dom.ElementHelper(container));
                                    this.getEditor().nodeChanged();
                                }
                                else {
                                    this.getEditor().insertContent(figure.getHTMLElement().outerHTML);
                                }
                                this.callback(figCaptionId);
                            }
                        };
                        return ImageModalDialog;
                    })(htmlarea.ModalDialog);
                    htmlarea.ImageModalDialog = ImageModalDialog;
                })(htmlarea = _text.htmlarea || (_text.htmlarea = {}));
            })(text = inputtype.text || (inputtype.text = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var text;
            (function (text) {
                var htmlarea;
                (function (htmlarea) {
                    var Validators = api.ui.form.Validators;
                    var AnchorModalDialog = (function (_super) {
                        __extends(AnchorModalDialog, _super);
                        function AnchorModalDialog(editor) {
                            _super.call(this, editor, new api.ui.dialog.ModalDialogHeader("Insert Anchor"));
                        }
                        AnchorModalDialog.prototype.getMainFormItems = function () {
                            var nameField = this.createFormItem("name", "Name", Validators.required);
                            this.setFirstFocusField(nameField.getInput());
                            return [
                                nameField
                            ];
                        };
                        AnchorModalDialog.prototype.initializeActions = function () {
                            var _this = this;
                            var submitAction = new api.ui.Action("Insert", "enter");
                            this.setSubmitAction(submitAction);
                            this.addAction(submitAction.onExecuted(function () {
                                if (_this.validate()) {
                                    _this.insertAnchor();
                                    _this.close();
                                }
                            }));
                            _super.prototype.initializeActions.call(this);
                        };
                        AnchorModalDialog.prototype.createAnchorEl = function () {
                            var anchorEl = new api.dom.AEl();
                            anchorEl.setId(this.getName());
                            anchorEl.getEl().removeAttribute('href');
                            return anchorEl;
                        };
                        AnchorModalDialog.prototype.getName = function () {
                            return this.getFieldById("name").getValue();
                        };
                        AnchorModalDialog.prototype.insertAnchor = function () {
                            var anchorEl = this.createAnchorEl();
                            this.getEditor().insertContent(anchorEl.toString());
                        };
                        return AnchorModalDialog;
                    })(htmlarea.ModalDialog);
                    htmlarea.AnchorModalDialog = AnchorModalDialog;
                })(htmlarea = text.htmlarea || (text.htmlarea = {}));
            })(text = inputtype.text || (inputtype.text = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var text;
            (function (text) {
                var support = api.form.inputtype.support;
                var Value = api.data.Value;
                var ValueTypes = api.data.ValueTypes;
                var TextLine = (function (_super) {
                    __extends(TextLine, _super);
                    function TextLine(config) {
                        _super.call(this, config);
                    }
                    TextLine.prototype.getValueType = function () {
                        return ValueTypes.STRING;
                    };
                    TextLine.prototype.newInitialValue = function () {
                        return new Value("", ValueTypes.STRING);
                    };
                    TextLine.prototype.createInputOccurrenceElement = function (index, property) {
                        var _this = this;
                        var inputEl = api.ui.text.TextInput.middle();
                        if (property.hasNonNullValue()) {
                            inputEl.setName(this.getInput().getName() + "-" + property.getIndex());
                            inputEl.setValue(property.getString());
                        }
                        else {
                            inputEl.setName(this.getInput().getName());
                        }
                        inputEl.onValueChanged(function (event) {
                            property.setValue(_this.newValue(event.getNewValue()));
                        });
                        return inputEl;
                    };
                    TextLine.prototype.availableSizeChanged = function () {
                    };
                    TextLine.prototype.newValue = function (s) {
                        return new Value(s, ValueTypes.STRING);
                    };
                    TextLine.prototype.valueBreaksRequiredContract = function (value) {
                        return value.isNull() || !value.getType().equals(ValueTypes.STRING) || api.util.StringHelper.isBlank(value.getString());
                    };
                    TextLine.prototype.hasInputElementValidUserInput = function (inputElement) {
                        return true;
                    };
                    TextLine.getName = function () {
                        return new api.form.InputTypeName("TextLine", false);
                    };
                    return TextLine;
                })(support.BaseInputTypeNotManagingAdd);
                text.TextLine = TextLine;
                api.form.inputtype.InputTypeManager.register(new api.Class(TextLine.getName().getName(), TextLine));
            })(text = inputtype.text || (inputtype.text = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var text;
            (function (text) {
                var support = api.form.inputtype.support;
                var Value = api.data.Value;
                var ValueTypes = api.data.ValueTypes;
                var TextArea = (function (_super) {
                    __extends(TextArea, _super);
                    function TextArea(config) {
                        _super.call(this, config);
                    }
                    TextArea.prototype.getValueType = function () {
                        return ValueTypes.STRING;
                    };
                    TextArea.prototype.newInitialValue = function () {
                        return new Value("", ValueTypes.STRING);
                    };
                    TextArea.prototype.createInputOccurrenceElement = function (index, property) {
                        var _this = this;
                        var inputEl = new api.ui.text.TextArea(this.getInput().getName() + "-" + index);
                        if (property.hasNonNullValue()) {
                            inputEl.setValue(property.getString());
                        }
                        inputEl.onValueChanged(function (event) {
                            property.setValue(_this.newValue(event.getNewValue()));
                        });
                        return inputEl;
                    };
                    TextArea.prototype.newValue = function (s) {
                        return new Value(s, ValueTypes.STRING);
                    };
                    TextArea.prototype.valueBreaksRequiredContract = function (value) {
                        return value.isNull() || !value.getType().equals(ValueTypes.STRING) || api.util.StringHelper.isBlank(value.getString());
                    };
                    TextArea.prototype.hasInputElementValidUserInput = function (inputElement) {
                        return true;
                    };
                    TextArea.getName = function () {
                        return new api.form.InputTypeName("TextArea", false);
                    };
                    return TextArea;
                })(support.BaseInputTypeNotManagingAdd);
                text.TextArea = TextArea;
                api.form.inputtype.InputTypeManager.register(new api.Class(TextArea.getName().getName(), TextArea));
            })(text = inputtype.text || (inputtype.text = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var form;
    (function (form) {
        var inputtype;
        (function (inputtype) {
            var text;
            (function (text) {
                var support = api.form.inputtype.support;
                var Value = api.data.Value;
                var ValueTypes = api.data.ValueTypes;
                var LinkModalDialog = api.form.inputtype.text.htmlarea.LinkModalDialog;
                var ImageModalDialog = api.form.inputtype.text.htmlarea.ImageModalDialog;
                var AnchorModalDialog = api.form.inputtype.text.htmlarea.AnchorModalDialog;
                var HtmlArea = (function (_super) {
                    __extends(HtmlArea, _super);
                    function HtmlArea(config) {
                        _super.call(this, config);
                        this.previousScrollPos = 0;
                        this.isScrollProhibited = false;
                        this.addClass("html-area");
                        this.editors = [];
                        this.contentId = config.contentId;
                    }
                    HtmlArea.prototype.getValueType = function () {
                        return ValueTypes.STRING;
                    };
                    HtmlArea.prototype.newInitialValue = function () {
                        return ValueTypes.STRING.newValue("");
                    };
                    HtmlArea.prototype.createInputOccurrenceElement = function (index, property) {
                        var _this = this;
                        var textAreaEl = new api.ui.text.TextArea(this.getInput().getName() + "-" + index);
                        var editorId = textAreaEl.getId();
                        var clazz = editorId.replace(/\./g, '_');
                        textAreaEl.addClass(clazz);
                        var textAreaWrapper = new api.dom.DivEl();
                        this.editors.push({ id: editorId, textAreaWrapper: textAreaWrapper, property: property });
                        textAreaEl.onRendered(function () {
                            _this.initEditor(editorId, property, textAreaWrapper);
                        });
                        textAreaWrapper.appendChild(textAreaEl);
                        this.setFocusOnEditorAfterCreate(textAreaWrapper, editorId);
                        return textAreaWrapper;
                    };
                    HtmlArea.prototype.initEditor = function (id, property, textAreaWrapper) {
                        var _this = this;
                        this.previousScrollPos = wemjq(this.getHTMLElement()).closest(".form-panel").scrollTop();
                        var focusedEditorCls = "html-area-focused";
                        var baseUrl = CONFIG.assetsUri;
                        tinymce.init({
                            selector: 'textarea.' + id.replace(/\./g, '_'),
                            document_base_url: baseUrl + '/common/lib/tinymce/',
                            skin_url: baseUrl + '/common/lib/tinymce/skins/lightgray',
                            content_css: baseUrl + '/common/styles/api/form/inputtype/text/tinymce-editor.css',
                            theme_url: 'modern',
                            toolbar: [
                                "styleselect | cut copy pastetext | bullist numlist outdent indent | charmap anchor image link unlink | table | code"
                            ],
                            menubar: false,
                            statusbar: false,
                            paste_as_text: true,
                            plugins: ['autoresize', 'table', 'paste', 'charmap', 'code'],
                            external_plugins: {
                                "link": baseUrl + "/common/js/form/inputtype/text/plugins/link.js",
                                "image": baseUrl + "/common/js/form/inputtype/text/plugins/image.js",
                                "anchor": baseUrl + "/common/js/form/inputtype/text/plugins/anchor.js"
                            },
                            autoresize_min_height: 100,
                            autoresize_bottom_margin: 0,
                            height: 100,
                            setup: function (editor) {
                                editor.addCommand("openLinkDialog", _this.openLinkDialog, _this);
                                editor.addCommand("openImageDialog", _this.openImageDialog, _this);
                                editor.addCommand("openAnchorDialog", _this.openAnchorDialog, _this);
                                editor.on('change', function (e) {
                                    _this.setPropertyValue(id, property);
                                });
                                editor.on('focus', function (e) {
                                    _this.resetInputHeight();
                                    textAreaWrapper.addClass(focusedEditorCls);
                                });
                                editor.on('blur', function (e) {
                                    _this.setStaticInputHeight();
                                    textAreaWrapper.removeClass(focusedEditorCls);
                                });
                                editor.on('keydown', function (e) {
                                    if ((e.metaKey || e.ctrlKey) && e.keyCode === 83) {
                                        e.preventDefault();
                                        _this.setPropertyValue(id, property);
                                        wemjq(_this.getEl().getHTMLElement()).simulate(e.type, {
                                            bubbles: e.bubbles,
                                            cancelable: e.cancelable,
                                            view: parent,
                                            ctrlKey: e.ctrlKey,
                                            altKey: e.altKey,
                                            shiftKey: e.shiftKey,
                                            metaKey: e.metaKey,
                                            keyCode: e.keyCode,
                                            charCode: e.charCode
                                        });
                                    }
                                });
                            },
                            init_instance_callback: function (editor) {
                                _this.setEditorContent(id, property);
                                if (_this.notInLiveEdit()) {
                                    _this.setupStickyEditorToolbarForInputOccurence(textAreaWrapper);
                                }
                                _this.removeTooltipFromEditorArea(textAreaWrapper);
                                _this.temporarilyDisableScrolling();
                            }
                        });
                    };
                    HtmlArea.prototype.setFocusOnEditorAfterCreate = function (inputOccurence, id) {
                        var _this = this;
                        inputOccurence.giveFocus = function () {
                            try {
                                _this.getEditor(id).focus();
                                return true;
                            }
                            catch (e) {
                                console.log("Element.giveFocus(): Failed to give focus to HtmlArea element: id = " + _this.getId());
                                return false;
                            }
                        };
                    };
                    HtmlArea.prototype.setupStickyEditorToolbarForInputOccurence = function (inputOccurence) {
                        var _this = this;
                        wemjq(this.getHTMLElement()).closest(".form-panel").on("scroll", function (event) {
                            _this.updateStickyEditorToolbar(inputOccurence);
                            if (_this.isScrollProhibited) {
                                wemjq(_this.getHTMLElement()).closest(".form-panel").scrollTop(_this.previousScrollPos);
                            }
                        });
                        api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function () {
                            _this.updateEditorToolbarWidth();
                            _this.updateEditorToolbarPos(inputOccurence);
                        });
                        this.onRemoved(function (event) {
                            api.ui.responsive.ResponsiveManager.unAvailableSizeChanged(_this);
                        });
                        this.onOccurrenceAdded(function () {
                            _this.resetInputHeight();
                            _this.updateEditorToolbarWidth();
                        });
                        this.onOccurrenceRemoved(function () {
                            _this.resetInputHeight();
                            _this.updateEditorToolbarWidth();
                        });
                    };
                    HtmlArea.prototype.temporarilyDisableScrolling = function () {
                        var _this = this;
                        this.isScrollProhibited = true;
                        setTimeout(function () {
                            _this.isScrollProhibited = false;
                        }, 300);
                    };
                    HtmlArea.prototype.updateStickyEditorToolbar = function (inputOccurence) {
                        if (!this.editorTopEdgeIsVisible(inputOccurence) && this.editorLowerEdgeIsVisible(inputOccurence)) {
                            inputOccurence.addClass("sticky-toolbar");
                            this.updateEditorToolbarWidth();
                            this.updateEditorToolbarPos(inputOccurence);
                        }
                        else {
                            inputOccurence.removeClass("sticky-toolbar");
                        }
                    };
                    HtmlArea.prototype.updateEditorToolbarWidth = function () {
                        wemjq(this.getHTMLElement()).find(".mce-toolbar-grp").width(wemjq(this.getHTMLElement()).find(".mce-edit-area").innerWidth());
                    };
                    HtmlArea.prototype.updateEditorToolbarPos = function (inputOccurence) {
                        wemjq(inputOccurence.getHTMLElement()).find(".mce-toolbar-grp").css({ top: this.getToolbarOffsetTop(1) });
                    };
                    HtmlArea.prototype.editorTopEdgeIsVisible = function (inputOccurence) {
                        return this.calcDistToTopOfScrlbleArea(inputOccurence) > 0;
                    };
                    HtmlArea.prototype.editorLowerEdgeIsVisible = function (inputOccurence) {
                        var distToTopOfScrlblArea = this.calcDistToTopOfScrlbleArea(inputOccurence);
                        var editorToolbarHeight = wemjq(inputOccurence.getHTMLElement()).find(".mce-toolbar-grp").outerHeight(true);
                        return (inputOccurence.getEl().getHeightWithoutPadding() - editorToolbarHeight + distToTopOfScrlblArea) > 0;
                    };
                    HtmlArea.prototype.calcDistToTopOfScrlbleArea = function (inputOccurence) {
                        return inputOccurence.getEl().getOffsetTop() - this.getToolbarOffsetTop();
                    };
                    HtmlArea.prototype.getToolbarOffsetTop = function (delta) {
                        if (delta === void 0) { delta = 0; }
                        var toolbar = wemjq(this.getHTMLElement()).closest(".form-panel").find(".wizard-step-navigator-and-toolbar"), stickyToolbarHeight = toolbar.outerHeight(true), stickyToolbarOffset = toolbar.offset().top;
                        return stickyToolbarOffset + stickyToolbarHeight + delta;
                    };
                    HtmlArea.prototype.resetInputHeight = function () {
                        wemjq(this.getHTMLElement()).height("auto");
                    };
                    HtmlArea.prototype.setStaticInputHeight = function () {
                        wemjq(this.getHTMLElement()).height(wemjq(this.getHTMLElement()).height());
                    };
                    HtmlArea.prototype.getEditor = function (editorId) {
                        return tinymce.get(editorId);
                    };
                    HtmlArea.prototype.setEditorContent = function (editorId, property) {
                        if (property.hasNonNullValue()) {
                            this.getEditor(editorId).setContent(this.propertyValue2Content(property.getString()));
                        }
                    };
                    HtmlArea.prototype.notInLiveEdit = function () {
                        return !(wemjq(this.getHTMLElement()).parents(".inspection-panel").length > 0);
                    };
                    HtmlArea.prototype.setPropertyValue = function (id, property) {
                        property.setValue(this.editorContent2PropertyValue(id));
                    };
                    HtmlArea.prototype.newValue = function (s) {
                        return new Value(s, ValueTypes.STRING);
                    };
                    HtmlArea.prototype.valueBreaksRequiredContract = function (value) {
                        return value.isNull() || !value.getType().equals(ValueTypes.STRING) || api.util.StringHelper.isBlank(value.getString());
                    };
                    HtmlArea.prototype.hasInputElementValidUserInput = function (inputElement) {
                        return true;
                    };
                    HtmlArea.prototype.openLinkDialog = function (config) {
                        var linkModalDialog = new LinkModalDialog(config);
                        linkModalDialog.open();
                    };
                    HtmlArea.prototype.openImageDialog = function (config) {
                        var imageModalDialog = new ImageModalDialog(config, this.contentId);
                        imageModalDialog.open();
                    };
                    HtmlArea.prototype.openAnchorDialog = function (editor) {
                        var anchorModalDialog = new AnchorModalDialog(editor);
                        anchorModalDialog.open();
                    };
                    HtmlArea.prototype.removeTooltipFromEditorArea = function (inputOccurence) {
                        wemjq(inputOccurence.getHTMLElement()).find("iframe").removeAttr("title");
                    };
                    HtmlArea.prototype.handleDnDStart = function (event, ui) {
                        _super.prototype.handleDnDStart.call(this, event, ui);
                        var editorId = wemjq('textarea', ui.item)[0].id;
                        this.destroyEditor(editorId);
                    };
                    HtmlArea.prototype.handleDnDStop = function (event, ui) {
                        var editorId = wemjq('textarea', ui.item)[0].id;
                        this.reInitEditor(editorId);
                        tinymce.execCommand('mceAddEditor', false, editorId);
                        this.getEditor(editorId).focus();
                    };
                    HtmlArea.prototype.destroyEditor = function (id) {
                        this.getEditor(id).destroy(false);
                    };
                    HtmlArea.prototype.reInitEditor = function (id) {
                        var savedEditor = this.findElementByFieldValue(this.editors, "id", id);
                        this.initEditor(id, savedEditor.property, savedEditor.textAreaWrapper);
                    };
                    HtmlArea.prototype.findElementByFieldValue = function (array, field, value) {
                        var result;
                        array.every(function (element) {
                            if (element[field] == value) {
                                result = element;
                                return false;
                            }
                            return true;
                        });
                        return result;
                    };
                    HtmlArea.prototype.getConvertedImageSrc = function (imgSrc) {
                        var contentId = imgSrc.replace(HtmlArea.imagePrefix, api.util.StringHelper.EMPTY_STRING), imageUrl = new api.content.ContentImageUrlResolver().setContentId(new api.content.ContentId(contentId)).setScaleWidth(true).setSize(HtmlArea.maxImageWidth).resolve();
                        return "src=\"" + imageUrl + "\" data-src=\"" + imgSrc + "\"";
                    };
                    HtmlArea.prototype.propertyValue2Content = function (propertyValue) {
                        var content = propertyValue, processedContent = propertyValue, regex = /<img.*?src="(.*?)"/g, imgSrcs, imgSrc;
                        while ((imgSrcs = regex.exec(content)) != null) {
                            imgSrc = imgSrcs[1];
                            if (imgSrc.indexOf(HtmlArea.imagePrefix) === 0) {
                                processedContent = processedContent.replace("src=\"" + imgSrc + "\"", this.getConvertedImageSrc(imgSrc));
                            }
                        }
                        return processedContent;
                    };
                    HtmlArea.prototype.editorContent2PropertyValue = function (editorId) {
                        var content = this.getEditor(editorId).getContent(), processedContent = this.getEditor(editorId).getContent(), regex = /<img.*?data-src="(.*?)".*?>/g, imgTags, imgTag;
                        while ((imgTags = regex.exec(content)) != null) {
                            imgTag = imgTags[0];
                            if (imgTag.indexOf("<img ") === 0 && imgTag.indexOf(HtmlArea.imagePrefix) > 0) {
                                var dataSrc = /<img.*?data-src="(.*?)".*?>/.exec(imgTag)[1], src = /<img.*?src="(.*?)".*?>/.exec(imgTags[0])[1];
                                var convertedImg = imgTag.replace(src, dataSrc).replace(" data-src=\"" + dataSrc + "\"", api.util.StringHelper.EMPTY_STRING);
                                processedContent = processedContent.replace(imgTag, convertedImg);
                            }
                        }
                        return this.newValue(processedContent);
                    };
                    HtmlArea.imagePrefix = "image://";
                    HtmlArea.maxImageWidth = 640;
                    return HtmlArea;
                })(support.BaseInputTypeNotManagingAdd);
                text.HtmlArea = HtmlArea;
                api.form.inputtype.InputTypeManager.register(new api.Class("HtmlArea", HtmlArea));
            })(text = inputtype.text || (inputtype.text = {}));
        })(inputtype = form.inputtype || (form.inputtype = {}));
    })(form = api.form || (api.form = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var checkbox;
                (function (checkbox) {
                    var ValueTypes = api.data.ValueTypes;
                    var BaseInputTypeSingleOccurrence = api.form.inputtype.support.BaseInputTypeSingleOccurrence;
                    var Checkbox = (function (_super) {
                        __extends(Checkbox, _super);
                        function Checkbox(config) {
                            _super.call(this, config);
                            this.checkbox = new api.ui.Checkbox();
                        }
                        Checkbox.prototype.getValueType = function () {
                            return ValueTypes.BOOLEAN;
                        };
                        Checkbox.prototype.newInitialValue = function () {
                            return ValueTypes.BOOLEAN.newFalse();
                        };
                        Checkbox.prototype.layoutProperty = function (input, property) {
                            if (property.hasNonNullValue()) {
                                this.checkbox.setChecked(property.getBoolean());
                            }
                            else {
                                this.checkbox.setChecked(false);
                            }
                            this.appendChild(this.checkbox);
                            this.checkbox.onValueChanged(function (event) {
                                var newValue = ValueTypes.BOOLEAN.newValue(event.getNewValue());
                                if (newValue) {
                                    property.setValue(newValue);
                                }
                            });
                            return wemQ(null);
                        };
                        Checkbox.prototype.giveFocus = function () {
                            return this.checkbox.giveFocus();
                        };
                        Checkbox.prototype.validate = function (silent) {
                            if (silent === void 0) { silent = true; }
                            return new api.form.inputtype.InputValidationRecording();
                        };
                        Checkbox.prototype.onFocus = function (listener) {
                            this.checkbox.onFocus(listener);
                        };
                        Checkbox.prototype.unFocus = function (listener) {
                            this.checkbox.unFocus(listener);
                        };
                        Checkbox.prototype.onBlur = function (listener) {
                            this.checkbox.onBlur(listener);
                        };
                        Checkbox.prototype.unBlur = function (listener) {
                            this.checkbox.unBlur(listener);
                        };
                        return Checkbox;
                    })(BaseInputTypeSingleOccurrence);
                    checkbox.Checkbox = Checkbox;
                    api.form.inputtype.InputTypeManager.register(new api.Class("Checkbox", Checkbox));
                })(checkbox = inputtype.checkbox || (inputtype.checkbox = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var long;
                (function (long) {
                    var BaseInputTypeNotManagingAdd = api.form.inputtype.support.BaseInputTypeNotManagingAdd;
                    var ValueTypes = api.data.ValueTypes;
                    var Long = (function (_super) {
                        __extends(Long, _super);
                        function Long(config) {
                            _super.call(this, config);
                        }
                        Long.prototype.getValueType = function () {
                            return ValueTypes.LONG;
                        };
                        Long.prototype.newInitialValue = function () {
                            return ValueTypes.LONG.newNullValue();
                        };
                        Long.prototype.createInputOccurrenceElement = function (index, property) {
                            var _this = this;
                            var inputEl = api.ui.text.TextInput.middle();
                            inputEl.setName(this.getInput().getName() + "-" + property.getIndex());
                            inputEl.setValue(!property.hasNullValue() ? property.getString() : "");
                            inputEl.onValueChanged(function (event) {
                                var value = ValueTypes.LONG.newValue(event.getNewValue());
                                property.setValue(value);
                                inputEl.updateValidationStatusOnUserInput(_this.isValid(event.getNewValue()));
                            });
                            return inputEl;
                        };
                        Long.prototype.availableSizeChanged = function () {
                        };
                        Long.prototype.valueBreaksRequiredContract = function (value) {
                            return value.isNull() || !value.getType().equals(ValueTypes.LONG);
                        };
                        Long.prototype.hasInputElementValidUserInput = function (inputElement) {
                            var value = inputElement;
                            return this.isValid(value.getValue());
                        };
                        Long.prototype.isValid = function (value) {
                            var validUserInput = true;
                            if (api.util.StringHelper.isEmpty(value)) {
                                validUserInput = true;
                            }
                            else {
                                if (api.util.NumberHelper.isWholeNumber(+value)) {
                                    validUserInput = true;
                                }
                                else {
                                    validUserInput = false;
                                }
                            }
                            return validUserInput;
                        };
                        return Long;
                    })(BaseInputTypeNotManagingAdd);
                    long.Long = Long;
                    api.form.inputtype.InputTypeManager.register(new api.Class("Long", Long));
                })(long = inputtype.long || (inputtype.long = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var double;
                (function (double) {
                    var BaseInputTypeNotManagingAdd = api.form.inputtype.support.BaseInputTypeNotManagingAdd;
                    var ValueTypes = api.data.ValueTypes;
                    var Double = (function (_super) {
                        __extends(Double, _super);
                        function Double(config) {
                            _super.call(this, config);
                        }
                        Double.prototype.getValueType = function () {
                            return ValueTypes.DOUBLE;
                        };
                        Double.prototype.newInitialValue = function () {
                            return ValueTypes.DOUBLE.newNullValue();
                        };
                        Double.prototype.createInputOccurrenceElement = function (index, property) {
                            var _this = this;
                            var inputEl = api.ui.text.TextInput.middle();
                            inputEl.setName(this.getInput().getName() + "-" + property.getIndex());
                            inputEl.setValue(!property.hasNullValue() ? property.getString() : "");
                            inputEl.onValueChanged(function (event) {
                                var value = ValueTypes.DOUBLE.newValue(event.getNewValue());
                                property.setValue(value);
                                inputEl.updateValidationStatusOnUserInput(_this.isValid(event.getNewValue()));
                            });
                            return inputEl;
                        };
                        Double.prototype.availableSizeChanged = function () {
                        };
                        Double.prototype.valueBreaksRequiredContract = function (value) {
                            return value.isNull() || !value.getType().equals(ValueTypes.DOUBLE);
                        };
                        Double.prototype.hasInputElementValidUserInput = function (inputElement) {
                            var value = inputElement;
                            return this.isValid(value.getValue());
                        };
                        Double.prototype.isValid = function (value) {
                            var validUserInput = true;
                            if (api.util.StringHelper.isEmpty(value)) {
                                validUserInput = true;
                            }
                            else {
                                if (api.util.NumberHelper.isNumber(+value)) {
                                    validUserInput = true;
                                }
                                else {
                                    validUserInput = false;
                                }
                            }
                            return validUserInput;
                        };
                        return Double;
                    })(BaseInputTypeNotManagingAdd);
                    double.Double = Double;
                    api.form.inputtype.InputTypeManager.register(new api.Class("Double", Double));
                })(double = inputtype.double || (inputtype.double = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var time;
                (function (time) {
                    var support = api.form.inputtype.support;
                    var ValueTypes = api.data.ValueTypes;
                    var Time = (function (_super) {
                        __extends(Time, _super);
                        function Time(config) {
                            _super.call(this, config);
                        }
                        Time.prototype.getValueType = function () {
                            return ValueTypes.LOCAL_TIME;
                        };
                        Time.prototype.newInitialValue = function () {
                            return ValueTypes.LOCAL_TIME.newNullValue();
                        };
                        Time.prototype.createInputOccurrenceElement = function (index, property) {
                            var _this = this;
                            var localTimeEl = new api.ui.time.LocalTime();
                            var timeValue = property.getLocalTime();
                            if (timeValue) {
                                localTimeEl.setTime(timeValue);
                            }
                            localTimeEl.onTimeChanged(function (hours, minutes) {
                                var changedValue = ValueTypes.LOCAL_TIME.newValue(hours + ":" + minutes);
                                property.setValue(changedValue);
                                _this.validate(false);
                            });
                            return localTimeEl;
                        };
                        Time.prototype.availableSizeChanged = function () {
                        };
                        Time.prototype.valueBreaksRequiredContract = function (value) {
                            return value.isNull() || !value.getType().equals(ValueTypes.LOCAL_TIME);
                        };
                        Time.prototype.hasInputElementValidUserInput = function (inputElement) {
                            var localTime = inputElement;
                            var timePicker = localTime.getTimePicker();
                            return timePicker.hasValidUserInput();
                        };
                        return Time;
                    })(support.BaseInputTypeNotManagingAdd);
                    time.Time = Time;
                    api.form.inputtype.InputTypeManager.register(new api.Class("Time", Time));
                })(time = inputtype.time || (inputtype.time = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var time;
                (function (time) {
                    var support = api.form.inputtype.support;
                    var ValueTypes = api.data.ValueTypes;
                    var Date = (function (_super) {
                        __extends(Date, _super);
                        function Date(config) {
                            _super.call(this, config);
                        }
                        Date.prototype.getValueType = function () {
                            return ValueTypes.LOCAL_DATE;
                        };
                        Date.prototype.newInitialValue = function () {
                            return ValueTypes.LOCAL_DATE.newNullValue();
                        };
                        Date.prototype.createInputOccurrenceElement = function (index, property) {
                            var _this = this;
                            var datePickerBuilder = new api.ui.time.DatePickerBuilder();
                            if (!property.hasNullValue()) {
                                var date = property.getLocalDate();
                                datePickerBuilder.setSelectedDate(date).setYear(date.getFullYear()).setMonth(date.getMonth());
                            }
                            var datePicker = datePickerBuilder.build();
                            datePicker.onSelectedDateChanged(function (event) {
                                _this.onValueChanged(property, event.getDate(), ValueTypes.LOCAL_DATE);
                            });
                            return datePicker;
                        };
                        Date.prototype.availableSizeChanged = function () {
                        };
                        Date.prototype.valueBreaksRequiredContract = function (value) {
                            return value.isNull() || !value.getType().equals(ValueTypes.LOCAL_DATE);
                        };
                        Date.prototype.hasInputElementValidUserInput = function (inputElement) {
                            var datePicker = inputElement;
                            return datePicker.hasValidUserInput();
                        };
                        return Date;
                    })(support.BaseInputTypeNotManagingAdd);
                    time.Date = Date;
                    api.form.inputtype.InputTypeManager.register(new api.Class("Date", Date));
                })(time = inputtype.time || (inputtype.time = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var time;
                (function (time) {
                    var support = api.form.inputtype.support;
                    var ValueTypes = api.data.ValueTypes;
                    var DateTime = (function (_super) {
                        __extends(DateTime, _super);
                        function DateTime(config) {
                            _super.call(this, config);
                            this.withTimezone = false;
                            this.valueType = ValueTypes.LOCAL_DATE_TIME;
                            this.readConfig(config.inputConfig);
                        }
                        DateTime.prototype.readConfig = function (inputConfig) {
                            var timeZoneConfig = inputConfig['timezone'] && inputConfig['timezone'][0];
                            var timeZone = timeZoneConfig && timeZoneConfig['value'];
                            if (timeZone === "true") {
                                this.withTimezone = true;
                                this.valueType = ValueTypes.DATE_TIME;
                            }
                        };
                        DateTime.prototype.getValueType = function () {
                            return this.valueType;
                        };
                        DateTime.prototype.newInitialValue = function () {
                            return this.valueType.newNullValue();
                        };
                        DateTime.prototype.createInputOccurrenceElement = function (index, property) {
                            if (this.valueType == ValueTypes.DATE_TIME) {
                                return this.createInputAsDateTime(property, this.valueType);
                            }
                            else {
                                return this.createInputAsLocalDateTime(property, this.valueType);
                            }
                        };
                        DateTime.prototype.hasInputElementValidUserInput = function (inputElement) {
                            var dateTimePicker = inputElement;
                            return dateTimePicker.hasValidUserInput();
                        };
                        DateTime.prototype.availableSizeChanged = function () {
                        };
                        DateTime.prototype.valueBreaksRequiredContract = function (value) {
                            return value.isNull() || !(value.getType().equals(ValueTypes.LOCAL_DATE_TIME) || value.getType().equals(ValueTypes.DATE_TIME));
                        };
                        DateTime.prototype.createInputAsLocalDateTime = function (property, valueType) {
                            var _this = this;
                            var dateTimeBuilder = new api.ui.time.DateTimePickerBuilder();
                            if (property.hasNonNullValue()) {
                                var date = property.getLocalDateTime();
                                dateTimeBuilder.setYear(date.getFullYear()).setMonth(date.getMonth()).setSelectedDate(date).setHours(date.getHours()).setMinutes(date.getMinutes());
                            }
                            var dateTimePicker = new api.ui.time.DateTimePicker(dateTimeBuilder);
                            dateTimePicker.onSelectedDateTimeChanged(function (event) {
                                _this.onValueChanged(property, event.getDate(), valueType);
                            });
                            return dateTimePicker;
                        };
                        DateTime.prototype.createInputAsDateTime = function (property, valueType) {
                            var _this = this;
                            var dateTimeBuilder = new api.ui.time.DateTimePickerBuilder();
                            dateTimeBuilder.setUseLocalTimezoneIfNotPresent(true);
                            if (property.hasNonNullValue()) {
                                var date = property.getDateTime();
                                dateTimeBuilder.setYear(date.getYear()).setMonth(date.getMonth()).setSelectedDate(date.toDate()).setHours(date.getHours()).setMinutes(date.getMinutes()).setTimezone(date.getTimezone());
                            }
                            var dateTimePicker = new api.ui.time.DateTimePicker(dateTimeBuilder);
                            dateTimePicker.onSelectedDateTimeChanged(function (event) {
                                _this.onValueChanged(property, event.getDate() == null ? null : api.util.DateTime.fromDate(event.getDate()), valueType);
                            });
                            return dateTimePicker;
                        };
                        return DateTime;
                    })(support.BaseInputTypeNotManagingAdd);
                    time.DateTime = DateTime;
                    api.form.inputtype.InputTypeManager.register(new api.Class("DateTime", DateTime));
                })(time = inputtype.time || (inputtype.time = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var site;
    (function (site) {
        var Form = api.form.Form;
        var SiteDescriptor = (function () {
            function SiteDescriptor(form, mixinNames) {
                this.form = form;
                this.metaSteps = mixinNames;
            }
            SiteDescriptor.prototype.getForm = function () {
                return this.form;
            };
            SiteDescriptor.prototype.getMetaSteps = function () {
                return this.metaSteps;
            };
            SiteDescriptor.fromJson = function (json) {
                return new SiteDescriptor(Form.fromJson(json.form), api.schema.mixin.MixinNames.create().fromStrings(json.metaSteps).build());
            };
            SiteDescriptor.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, SiteDescriptor)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.form, other.form)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.metaSteps, other.metaSteps)) {
                    return false;
                }
                return true;
            };
            return SiteDescriptor;
        })();
        site.SiteDescriptor = SiteDescriptor;
    })(site = api.site || (api.site = {}));
})(api || (api = {}));
var api;
(function (api) {
    var item;
    (function (item) {
        var BaseItem = (function () {
            function BaseItem(builder) {
                this.id = builder.id;
                this.createdTime = builder.createdTime;
                this.modifiedTime = builder.modifiedTime;
                this.deletable = builder.deletable;
                this.editable = builder.editable;
            }
            BaseItem.prototype.getId = function () {
                return this.id;
            };
            BaseItem.prototype.getCreatedTime = function () {
                return this.createdTime;
            };
            BaseItem.prototype.getModifiedTime = function () {
                return this.modifiedTime;
            };
            BaseItem.prototype.isDeletable = function () {
                return this.deletable;
            };
            BaseItem.prototype.isEditable = function () {
                return this.editable;
            };
            BaseItem.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, BaseItem)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.id, other.id)) {
                    return false;
                }
                if (!api.ObjectHelper.dateEquals(this.createdTime, other.createdTime)) {
                    return false;
                }
                if (!api.ObjectHelper.dateEquals(this.modifiedTime, other.modifiedTime)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.deletable, other.deletable)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.editable, other.editable)) {
                    return false;
                }
                return true;
            };
            return BaseItem;
        })();
        item.BaseItem = BaseItem;
        var BaseItemBuilder = (function () {
            function BaseItemBuilder(source) {
                if (source) {
                    this.id = source.getId();
                    this.createdTime = source.getCreatedTime();
                    this.modifiedTime = source.getModifiedTime();
                    this.deletable = source.isDeletable();
                    this.editable = source.isEditable();
                }
            }
            BaseItemBuilder.prototype.fromBaseItemJson = function (json, idProperty) {
                if (idProperty === void 0) { idProperty = 'id'; }
                this.id = json[idProperty];
                this.createdTime = json.createdTime ? new Date(json.createdTime) : null;
                this.modifiedTime = json.modifiedTime ? new Date(json.modifiedTime) : null;
                this.deletable = json.deletable;
                this.editable = json.editable;
                return this;
            };
            BaseItemBuilder.prototype.build = function () {
                return new BaseItem(this);
            };
            return BaseItemBuilder;
        })();
        item.BaseItemBuilder = BaseItemBuilder;
    })(item = api.item || (api.item = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (_expr) {
            var CompareExpr = (function () {
                function CompareExpr(field, operator, values) {
                    this.values = [];
                    this.field = field;
                    this.operator = operator;
                    this.values = values;
                }
                CompareExpr.prototype.getField = function () {
                    return this.field;
                };
                CompareExpr.prototype.getOperator = function () {
                    return this.operator;
                };
                CompareExpr.prototype.getFirstValue = function () {
                    return this.values.length > 0 ? this.values[0] : null;
                };
                CompareExpr.prototype.getValues = function () {
                    return this.values;
                };
                CompareExpr.prototype.toString = function () {
                    var result = this.field.toString();
                    result = result.concat(" ");
                    result = result.concat(this.operatorAsString());
                    result = result.concat(" ");
                    if (this.allowMultipleValues()) {
                        result = result.concat("(");
                        var sub = [];
                        this.values.forEach(function (expr) {
                            sub.push(expr.toString());
                        });
                        result = result.concat(sub.join(", "));
                        result = result.concat(")");
                    }
                    else {
                        result = result.concat(this.getFirstValue().toString());
                    }
                    return result;
                };
                CompareExpr.eq = function (field, value) {
                    return CompareExpr.create(field, 0 /* EQ */, [value]);
                };
                CompareExpr.neq = function (field, value) {
                    return CompareExpr.create(field, 1 /* NEQ */, [value]);
                };
                CompareExpr.gt = function (field, value) {
                    return CompareExpr.create(field, 2 /* GT */, [value]);
                };
                CompareExpr.gte = function (field, value) {
                    return CompareExpr.create(field, 3 /* GTE */, [value]);
                };
                CompareExpr.lt = function (field, value) {
                    return CompareExpr.create(field, 4 /* LT */, [value]);
                };
                CompareExpr.lte = function (field, value) {
                    return CompareExpr.create(field, 5 /* LTE */, [value]);
                };
                CompareExpr.like = function (field, value) {
                    return CompareExpr.create(field, 6 /* LIKE */, [value]);
                };
                CompareExpr.notLike = function (field, value) {
                    return CompareExpr.create(field, 7 /* NOT_LIKE */, [value]);
                };
                CompareExpr.In = function (field, values) {
                    return CompareExpr.create(field, 8 /* IN */, values);
                };
                CompareExpr.notIn = function (field, values) {
                    return CompareExpr.create(field, 9 /* NOT_IN */, values);
                };
                CompareExpr.create = function (field, operator, values) {
                    return new CompareExpr(field, operator, values);
                };
                CompareExpr.prototype.operatorAsString = function () {
                    switch (this.operator) {
                        case 0 /* EQ */:
                            return "=";
                        case 1 /* NEQ */:
                            return "!=";
                        case 2 /* GT */:
                            return ">";
                        case 3 /* GTE */:
                            return ">=";
                        case 4 /* LT */:
                            return "<";
                        case 5 /* LTE */:
                            return "<=";
                        case 6 /* LIKE */:
                            return "LIKE";
                        case 7 /* NOT_LIKE */:
                            return "NOT LIKE";
                        case 8 /* IN */:
                            return "IN";
                        case 9 /* NOT_IN */:
                            return "NOT IN";
                        default:
                            return "";
                    }
                };
                CompareExpr.prototype.allowMultipleValues = function () {
                    return this.operator == 8 /* IN */ || this.operator == 9 /* NOT_IN */;
                };
                return CompareExpr;
            })();
            _expr.CompareExpr = CompareExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            var OrderExpr = (function () {
                function OrderExpr(direction) {
                    this.direction = direction;
                }
                OrderExpr.prototype.getDirection = function () {
                    return this.direction;
                };
                OrderExpr.prototype.directionAsString = function () {
                    switch (this.direction) {
                        case 0 /* ASC */:
                            return "ASC";
                        case 1 /* DESC */:
                            return "DESC";
                        default:
                            return "";
                    }
                };
                return OrderExpr;
            })();
            expr.OrderExpr = OrderExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (_expr) {
            var QueryExpr = (function () {
                function QueryExpr(constraint, orderList) {
                    this.orderList = [];
                    this.constraint = constraint;
                    if (orderList) {
                        this.orderList = orderList;
                    }
                }
                QueryExpr.prototype.getConstraint = function () {
                    return this.constraint;
                };
                QueryExpr.prototype.getOrderList = function () {
                    return this.orderList;
                };
                QueryExpr.prototype.toString = function () {
                    var result = "";
                    if (this.constraint != null) {
                        result = result.concat(this.constraint.toString());
                    }
                    if (this.orderList.length > 0) {
                        result = result.concat(" ORDER BY ");
                        var sub = [];
                        this.orderList.forEach(function (expr) {
                            sub.push(expr.toString());
                        });
                        result = result.concat(sub.join(", "));
                    }
                    return result;
                };
                return QueryExpr;
            })();
            _expr.QueryExpr = QueryExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            var ValueExpr = (function () {
                function ValueExpr(value) {
                    this.value = value;
                }
                ValueExpr.prototype.getValue = function () {
                    return this.value;
                };
                ValueExpr.stringValue = function (value) {
                    return new ValueExpr(new api.data.Value(value, api.data.ValueTypes.STRING));
                };
                ValueExpr.prototype.toString = function () {
                    var type = this.value.getType();
                    if (type == api.data.ValueTypes.DOUBLE) {
                        return this.value.getString();
                    }
                    if (type == api.data.ValueTypes.DATE_TIME) {
                        return this.typecastFunction("dateTime", this.value.getString());
                    }
                    if (type == api.data.ValueTypes.GEO_POINT) {
                        return this.typecastFunction("geoPoint", this.value.getString());
                    }
                    return this.quoteString(this.value.getString());
                };
                ValueExpr.prototype.typecastFunction = function (name, argument) {
                    return name + "(" + this.quoteString(argument) + ")";
                };
                ValueExpr.prototype.quoteString = function (value) {
                    if (value.indexOf("'") > -1) {
                        return "\"" + value + "\"";
                    }
                    else {
                        return "'" + value + "'";
                    }
                };
                ValueExpr.string = function (value) {
                    return new ValueExpr(new api.data.Value(value, api.data.ValueTypes.STRING));
                };
                ValueExpr.number = function (value) {
                    return new ValueExpr(new api.data.Value(value, api.data.ValueTypes.DOUBLE));
                };
                ValueExpr.dateTime = function (value) {
                    return new ValueExpr(new api.data.Value(value, api.data.ValueTypes.DATE_TIME));
                };
                ValueExpr.geoPoint = function (value) {
                    return new ValueExpr(new api.data.Value(value, api.data.ValueTypes.GEO_POINT));
                };
                return ValueExpr;
            })();
            expr.ValueExpr = ValueExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (_expr) {
            var FunctionExpr = (function () {
                function FunctionExpr(name, arguments) {
                    this.arguments = [];
                    this.name = name;
                    this.arguments = arguments;
                }
                FunctionExpr.prototype.getName = function () {
                    return this.name;
                };
                FunctionExpr.prototype.getArguments = function () {
                    return this.arguments;
                };
                FunctionExpr.prototype.toString = function () {
                    var result = this.name;
                    result = result.concat("(");
                    var sub = [];
                    this.arguments.forEach(function (expr) {
                        sub.push(expr.toString());
                    });
                    result = result.concat(sub.join(", "));
                    result = result.concat(")");
                    return result;
                };
                return FunctionExpr;
            })();
            _expr.FunctionExpr = FunctionExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            var FieldExpr = (function () {
                function FieldExpr(name) {
                    this.name = name;
                }
                FieldExpr.prototype.getName = function () {
                    return this.name;
                };
                FieldExpr.prototype.toString = function () {
                    return this.name;
                };
                return FieldExpr;
            })();
            expr.FieldExpr = FieldExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            var DynamicConstraintExpr = (function () {
                function DynamicConstraintExpr(func) {
                    this.func = func;
                }
                DynamicConstraintExpr.prototype.getFunction = function () {
                    return this.func;
                };
                DynamicConstraintExpr.prototype.toString = function () {
                    return this.func.toString();
                };
                return DynamicConstraintExpr;
            })();
            expr.DynamicConstraintExpr = DynamicConstraintExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            var DynamicOrderExpr = (function (_super) {
                __extends(DynamicOrderExpr, _super);
                function DynamicOrderExpr(func, direction) {
                    _super.call(this, direction);
                    this.func = func;
                }
                DynamicOrderExpr.prototype.getFunction = function () {
                    return this.func;
                };
                DynamicOrderExpr.prototype.toString = function () {
                    return this.func.toString() + " " + _super.prototype.directionAsString.call(this);
                };
                return DynamicOrderExpr;
            })(expr.OrderExpr);
            expr.DynamicOrderExpr = DynamicOrderExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            var FieldOrderExpr = (function (_super) {
                __extends(FieldOrderExpr, _super);
                function FieldOrderExpr(field, direction) {
                    _super.call(this, direction);
                    this.field = field;
                }
                FieldOrderExpr.prototype.getField = function () {
                    return this.field;
                };
                FieldOrderExpr.prototype.toString = function () {
                    return this.field.toString() + " " + _super.prototype.directionAsString.call(this);
                };
                return FieldOrderExpr;
            })(expr.OrderExpr);
            expr.FieldOrderExpr = FieldOrderExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            var LogicalExpr = (function () {
                function LogicalExpr(left, operator, right) {
                    this.left = left;
                    this.right = right;
                    this.operator = operator;
                }
                LogicalExpr.prototype.getLeft = function () {
                    return this.left;
                };
                LogicalExpr.prototype.getRight = function () {
                    return this.right;
                };
                LogicalExpr.prototype.getOperator = function () {
                    return this.operator;
                };
                LogicalExpr.prototype.toString = function () {
                    return "(" + this.left.toString() + " " + this.operatorAsString() + " " + this.right.toString() + ")";
                };
                LogicalExpr.and = function (left, right) {
                    return new LogicalExpr(left, 0 /* AND */, right);
                };
                LogicalExpr.or = function (left, right) {
                    return new LogicalExpr(left, 1 /* OR */, right);
                };
                LogicalExpr.prototype.operatorAsString = function () {
                    switch (this.operator) {
                        case 0 /* AND */:
                            return "AND";
                        case 1 /* OR */:
                            return "OR";
                        default:
                            return "";
                    }
                };
                return LogicalExpr;
            })();
            expr.LogicalExpr = LogicalExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (_expr) {
            var NotExpr = (function () {
                function NotExpr(expr) {
                    this.expr = expr;
                }
                NotExpr.prototype.getExpression = function () {
                    return this.expr;
                };
                NotExpr.prototype.toString = function () {
                    return "NOT (" + this.expr.toString() + ")";
                };
                return NotExpr;
            })();
            _expr.NotExpr = NotExpr;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            (function (CompareOperator) {
                CompareOperator[CompareOperator["EQ"] = 0] = "EQ";
                CompareOperator[CompareOperator["NEQ"] = 1] = "NEQ";
                CompareOperator[CompareOperator["GT"] = 2] = "GT";
                CompareOperator[CompareOperator["GTE"] = 3] = "GTE";
                CompareOperator[CompareOperator["LT"] = 4] = "LT";
                CompareOperator[CompareOperator["LTE"] = 5] = "LTE";
                CompareOperator[CompareOperator["LIKE"] = 6] = "LIKE";
                CompareOperator[CompareOperator["NOT_LIKE"] = 7] = "NOT_LIKE";
                CompareOperator[CompareOperator["IN"] = 8] = "IN";
                CompareOperator[CompareOperator["NOT_IN"] = 9] = "NOT_IN";
            })(expr.CompareOperator || (expr.CompareOperator = {}));
            var CompareOperator = expr.CompareOperator;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            (function (LogicalOperator) {
                LogicalOperator[LogicalOperator["AND"] = 0] = "AND";
                LogicalOperator[LogicalOperator["OR"] = 1] = "OR";
            })(expr.LogicalOperator || (expr.LogicalOperator = {}));
            var LogicalOperator = expr.LogicalOperator;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var expr;
        (function (expr) {
            (function (OrderDirection) {
                OrderDirection[OrderDirection["ASC"] = 0] = "ASC";
                OrderDirection[OrderDirection["DESC"] = 1] = "DESC";
            })(expr.OrderDirection || (expr.OrderDirection = {}));
            var OrderDirection = expr.OrderDirection;
        })(expr = query.expr || (query.expr = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var SearchInputValues = (function () {
            function SearchInputValues() {
            }
            SearchInputValues.prototype.setAggregationSelections = function (aggregationSelections) {
                this.aggregationSelections = aggregationSelections;
            };
            SearchInputValues.prototype.setTextSearchFieldValue = function (textSearchFieldValue) {
                this.textSearchFieldValue = textSearchFieldValue;
            };
            SearchInputValues.prototype.getTextSearchFieldValue = function () {
                return this.textSearchFieldValue;
            };
            SearchInputValues.prototype.getSelectedValuesForAggregationName = function (name) {
                for (var i = 0; i < this.aggregationSelections.length; i++) {
                    if (this.aggregationSelections[i].getName() == name) {
                        return this.aggregationSelections[i].getSelectedBuckets();
                    }
                }
                return null;
            };
            return SearchInputValues;
        })();
        query.SearchInputValues = SearchInputValues;
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var ValueExpr = api.query.expr.ValueExpr;
        var FunctionExpr = api.query.expr.FunctionExpr;
        var DynamicConstraintExpr = api.query.expr.DynamicConstraintExpr;
        var LogicalExpr = api.query.expr.LogicalExpr;
        var LogicalOperator = api.query.expr.LogicalOperator;
        var FulltextSearchExpression = (function () {
            function FulltextSearchExpression() {
            }
            FulltextSearchExpression.create = function (searchString, queryFields) {
                if (searchString == null) {
                    return null;
                }
                var arguments = [];
                arguments.push(ValueExpr.stringValue(queryFields.toString()));
                arguments.push(ValueExpr.stringValue(searchString));
                arguments.push(ValueExpr.stringValue("AND"));
                var fulltextExp = new FunctionExpr("fulltext", arguments);
                var fulltextDynamicExpr = new DynamicConstraintExpr(fulltextExp);
                var nGramExpr = new FunctionExpr("ngram", arguments);
                var nGramDynamicExpr = new DynamicConstraintExpr(nGramExpr);
                var booleanExpr = new LogicalExpr(fulltextDynamicExpr, 1 /* OR */, nGramDynamicExpr);
                return booleanExpr;
            };
            return FulltextSearchExpression;
        })();
        query.FulltextSearchExpression = FulltextSearchExpression;
        var FulltextSearchExpressionBuilder = (function () {
            function FulltextSearchExpressionBuilder() {
                this.queryFields = new query.QueryFields();
            }
            FulltextSearchExpressionBuilder.prototype.addField = function (queryField) {
                this.queryFields.add(queryField);
                return this;
            };
            FulltextSearchExpressionBuilder.prototype.setSearchString = function (searchString) {
                this.searchString = searchString;
                return this;
            };
            FulltextSearchExpressionBuilder.prototype.build = function () {
                return FulltextSearchExpression.create(this.searchString, this.queryFields);
            };
            return FulltextSearchExpressionBuilder;
        })();
        query.FulltextSearchExpressionBuilder = FulltextSearchExpressionBuilder;
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var QueryField = (function () {
            function QueryField(name, weight) {
                this.name = name;
                if (weight) {
                    this.weight = weight;
                }
            }
            QueryField.prototype.toString = function () {
                if (this.weight) {
                    return this.name + QueryField.WEIGHT_SEPARATOR + this.weight;
                }
                else {
                    return this.name;
                }
            };
            QueryField.DISPLAY_NAME = "displayName";
            QueryField.NAME = "_name";
            QueryField.ALL = "_alltext";
            QueryField.MODIFIED_TIME = "modifiedTime";
            QueryField.MANUAL_ORDER_VALUE = "_manualOrderValue";
            QueryField.WEIGHT_SEPARATOR = "^";
            QueryField.CONTENT_TYPE = "type";
            return QueryField;
        })();
        query.QueryField = QueryField;
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var QueryFields = (function () {
            function QueryFields() {
                this.queryFields = [];
            }
            QueryFields.prototype.add = function (queryField) {
                this.queryFields.push(queryField);
            };
            QueryFields.prototype.toString = function () {
                if (this.queryFields) {
                    return this.queryFields.join();
                }
            };
            return QueryFields;
        })();
        query.QueryFields = QueryFields;
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var aggregation;
        (function (aggregation) {
            var AggregationQuery = (function () {
                function AggregationQuery(name) {
                    this.name = name;
                }
                AggregationQuery.prototype.toJson = function () {
                    throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                };
                AggregationQuery.prototype.toAggregationQueryJson = function () {
                    return {
                        "name": this.getName()
                    };
                };
                AggregationQuery.prototype.getName = function () {
                    return this.name;
                };
                return AggregationQuery;
            })();
            aggregation.AggregationQuery = AggregationQuery;
        })(aggregation = query.aggregation || (query.aggregation = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var aggregation;
        (function (aggregation) {
            var TermsAggregationQuery = (function (_super) {
                __extends(TermsAggregationQuery, _super);
                function TermsAggregationQuery(name) {
                    _super.call(this, name);
                    this.size = TermsAggregationQuery.TERM_DEFAULT_SIZE;
                    this.orderByDirection = TermsAggregationOrderDirection.ASC;
                    this.orderByType = TermsAggregationOrderType.TERM;
                }
                TermsAggregationQuery.prototype.toJson = function () {
                    var json = _super.prototype.toAggregationQueryJson.call(this);
                    json.fieldName = this.getFieldName();
                    json.size = this.getSize();
                    json.orderByDirection = this.orderByDirection;
                    json.orderByType = this.orderByType;
                    return {
                        TermsAggregationQuery: json
                    };
                };
                TermsAggregationQuery.prototype.setFieldName = function (fieldName) {
                    this.fieldName = fieldName;
                };
                TermsAggregationQuery.prototype.getFieldName = function () {
                    return this.fieldName;
                };
                TermsAggregationQuery.prototype.setSize = function (size) {
                    this.size = size;
                };
                TermsAggregationQuery.prototype.getSize = function () {
                    return this.size;
                };
                TermsAggregationQuery.prototype.setOrderByType = function (type) {
                    this.orderByType = type;
                };
                TermsAggregationQuery.prototype.setOrderByDirection = function (direction) {
                    this.orderByDirection = direction;
                };
                TermsAggregationQuery.TERM_DEFAULT_SIZE = 10;
                return TermsAggregationQuery;
            })(aggregation.AggregationQuery);
            aggregation.TermsAggregationQuery = TermsAggregationQuery;
            var TermsAggregationOrderDirection = (function () {
                function TermsAggregationOrderDirection() {
                }
                TermsAggregationOrderDirection.ASC = "ASC";
                TermsAggregationOrderDirection.DESC = "DESC";
                return TermsAggregationOrderDirection;
            })();
            aggregation.TermsAggregationOrderDirection = TermsAggregationOrderDirection;
            var TermsAggregationOrderType = (function () {
                function TermsAggregationOrderType() {
                }
                TermsAggregationOrderType.DOC_COUNT = "DOC_COUNT";
                TermsAggregationOrderType.TERM = "TERM";
                return TermsAggregationOrderType;
            })();
            aggregation.TermsAggregationOrderType = TermsAggregationOrderType;
        })(aggregation = query.aggregation || (query.aggregation = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var aggregation;
        (function (aggregation) {
            var DateRangeAggregationQuery = (function (_super) {
                __extends(DateRangeAggregationQuery, _super);
                function DateRangeAggregationQuery(name) {
                    _super.call(this, name);
                    this.ranges = [];
                }
                DateRangeAggregationQuery.prototype.setFieldName = function (fieldName) {
                    this.fieldName = fieldName;
                };
                DateRangeAggregationQuery.prototype.getFieldName = function () {
                    return this.fieldName;
                };
                DateRangeAggregationQuery.prototype.addRange = function (range) {
                    this.ranges.push(range);
                };
                DateRangeAggregationQuery.prototype.toJson = function () {
                    var json = _super.prototype.toAggregationQueryJson.call(this);
                    json.fieldName = this.getFieldName();
                    json.ranges = [];
                    this.ranges.forEach(function (range) {
                        json.ranges.push(range.toJson());
                    });
                    return {
                        DateRangeAggregationQuery: json
                    };
                };
                return DateRangeAggregationQuery;
            })(api.query.aggregation.AggregationQuery);
            aggregation.DateRangeAggregationQuery = DateRangeAggregationQuery;
        })(aggregation = query.aggregation || (query.aggregation = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var aggregation;
        (function (aggregation) {
            var Range = (function () {
                function Range(key) {
                    this.key = key;
                }
                Range.prototype.setKey = function (key) {
                    this.key = key;
                };
                Range.prototype.getKey = function () {
                    return this.key;
                };
                Range.prototype.toRangeJson = function () {
                    if (this.getKey() != null) {
                        return {
                            "key": this.getKey()
                        };
                    }
                    else {
                        return {};
                    }
                };
                return Range;
            })();
            aggregation.Range = Range;
        })(aggregation = query.aggregation || (query.aggregation = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var aggregation;
        (function (aggregation) {
            var DateRange = (function (_super) {
                __extends(DateRange, _super);
                function DateRange(from, to, key) {
                    _super.call(this, key);
                    this.from = from;
                    this.to = to;
                }
                DateRange.prototype.setTo = function (to) {
                    this.to = to;
                };
                DateRange.prototype.setFrom = function (from) {
                    this.from = from;
                };
                DateRange.prototype.setToDate = function (to) {
                    this.to = to.toISOString();
                };
                DateRange.prototype.setFromDate = function (from) {
                    this.from = from.toISOString();
                };
                DateRange.prototype.toJson = function () {
                    var json = _super.prototype.toRangeJson.call(this);
                    json.from = this.from;
                    json.to = this.to;
                    return json;
                };
                return DateRange;
            })(aggregation.Range);
            aggregation.DateRange = DateRange;
        })(aggregation = query.aggregation || (query.aggregation = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var filter;
        (function (filter) {
            var Filter = (function () {
                function Filter() {
                }
                Filter.prototype.toJson = function () {
                    throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                };
                return Filter;
            })();
            filter.Filter = Filter;
        })(filter = query.filter || (query.filter = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var filter;
        (function (_filter) {
            var BooleanFilter = (function (_super) {
                __extends(BooleanFilter, _super);
                function BooleanFilter() {
                    _super.apply(this, arguments);
                    this.must = [];
                    this.mustNot = [];
                    this.should = [];
                }
                BooleanFilter.prototype.addMust = function (must) {
                    this.must.push(must);
                };
                BooleanFilter.prototype.addMustNot = function (mustNot) {
                    this.mustNot.push(mustNot);
                };
                BooleanFilter.prototype.addShould = function (should) {
                    this.should.push(should);
                };
                BooleanFilter.prototype.toJson = function () {
                    var json = {
                        must: this.toJsonWrapperElements(this.must),
                        mustNot: this.toJsonWrapperElements(this.mustNot),
                        should: this.toJsonWrapperElements(this.should)
                    };
                    return {
                        BooleanFilter: json
                    };
                };
                BooleanFilter.prototype.toJsonWrapperElements = function (filters) {
                    var wrapperJsons = [];
                    filters.forEach(function (filter) {
                        var filterTypeWrapperJson = filter.toJson();
                        wrapperJsons.push(filterTypeWrapperJson);
                    });
                    return wrapperJsons;
                };
                return BooleanFilter;
            })(_filter.Filter);
            _filter.BooleanFilter = BooleanFilter;
        })(filter = query.filter || (query.filter = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var query;
    (function (query) {
        var filter;
        (function (filter) {
            var RangeFilter = (function (_super) {
                __extends(RangeFilter, _super);
                function RangeFilter(fieldName, from, to) {
                    _super.call(this);
                    this.fieldName = fieldName;
                    this.from = from;
                    this.to = to;
                }
                RangeFilter.prototype.toJson = function () {
                    var json = {
                        fieldName: this.fieldName,
                        from: this.from != null ? this.from.getString() : null,
                        to: this.to != null ? this.to.getString() : null
                    };
                    return {
                        RangeFilter: json
                    };
                };
                return RangeFilter;
            })(api.query.filter.Filter);
            filter.RangeFilter = RangeFilter;
        })(filter = query.filter || (query.filter = {}));
    })(query = api.query || (api.query = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var ResourcePath = (function (_super) {
            __extends(ResourcePath, _super);
            function ResourcePath(elements, absolute) {
                _super.call(this, elements, ResourcePath.ELEMENT_DIVIDER, absolute);
            }
            ResourcePath.fromString = function (s) {
                var absolute = s.charAt(0) == ResourcePath.ELEMENT_DIVIDER;
                var elements = s.split(ResourcePath.ELEMENT_DIVIDER);
                return new ResourcePath(api.util.BasePath.removeEmptyElements(elements), absolute);
            };
            ResourcePath.prototype.newInstance = function (elements, absolute) {
                return new ResourcePath(elements, absolute);
            };
            ResourcePath.ELEMENT_DIVIDER = "/";
            return ResourcePath;
        })(api.util.BasePath);
        application.ResourcePath = ResourcePath;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var ApplicationKey = (function () {
            function ApplicationKey(applicationName) {
                this.name = applicationName;
            }
            ApplicationKey.fromString = function (applicationName) {
                return new ApplicationKey(applicationName);
            };
            ApplicationKey.prototype.getName = function () {
                return this.name;
            };
            ApplicationKey.prototype.isSystemReserved = function () {
                for (var key in ApplicationKey.SYSTEM_RESERVED_APPLICATION_KEYS) {
                    if (ApplicationKey.SYSTEM_RESERVED_APPLICATION_KEYS[key].equals(this)) {
                        return true;
                    }
                }
                return false;
            };
            ApplicationKey.prototype.toString = function () {
                return this.name;
            };
            ApplicationKey.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ApplicationKey)) {
                    return false;
                }
                var other = o;
                return api.ObjectHelper.stringEquals(this.name, other.name);
            };
            ApplicationKey.toStringArray = function (keys) {
                return keys.map(function (key) { return key.toString(); });
            };
            ApplicationKey.fromApplications = function (applications) {
                return applications.map(function (mod) { return mod.getApplicationKey(); });
            };
            ApplicationKey.SYSTEM = ApplicationKey.fromString('system');
            ApplicationKey.BASE = ApplicationKey.fromString('base');
            ApplicationKey.PORTAL = ApplicationKey.fromString('portal');
            ApplicationKey.MEDIA = ApplicationKey.fromString('media');
            ApplicationKey.SYSTEM_RESERVED_APPLICATION_KEYS = [
                ApplicationKey.SYSTEM,
                ApplicationKey.BASE,
                ApplicationKey.PORTAL,
                ApplicationKey.MEDIA
            ];
            return ApplicationKey;
        })();
        application.ApplicationKey = ApplicationKey;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var ApplicationResourceKey = (function () {
            function ApplicationResourceKey(applicationKey, path) {
                this.applicationKey = applicationKey;
                this.path = path;
                this.refString = applicationKey.toString() + ApplicationResourceKey.SEPARATOR + path.toString();
            }
            ApplicationResourceKey.fromString = function (str) {
                var sepIndex = str.indexOf(ApplicationResourceKey.SEPARATOR);
                if (sepIndex == -1) {
                    throw new Error("ApplicationResourceKey must contain separator '" + ApplicationResourceKey.SEPARATOR + "':" + str);
                }
                var applicationKey = str.substring(0, sepIndex);
                var path = str.substring(sepIndex + 1, str.length);
                return new ApplicationResourceKey(application.ApplicationKey.fromString(applicationKey), application.ResourcePath.fromString(path));
            };
            ApplicationResourceKey.prototype.getApplicationKey = function () {
                return this.applicationKey;
            };
            ApplicationResourceKey.prototype.getPath = function () {
                return this.path;
            };
            ApplicationResourceKey.prototype.toString = function () {
                return this.refString;
            };
            ApplicationResourceKey.SEPARATOR = ":";
            return ApplicationResourceKey;
        })();
        application.ApplicationResourceKey = ApplicationResourceKey;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var Application = (function (_super) {
            __extends(Application, _super);
            function Application(builder) {
                _super.call(this, builder);
                this.applicationDependencies = [];
                this.contentTypeDependencies = [];
                this.applicationKey = builder.applicationKey;
                this.displayName = builder.displayName;
                this.vendorName = builder.vendorName;
                this.vendorUrl = builder.vendorUrl;
                this.url = builder.url;
                this.state = builder.state;
                this.version = builder.version;
                this.config = builder.config;
                this.applicationDependencies = builder.applicationDependencies;
                this.contentTypeDependencies = builder.contentTypeDependencies;
                this.metaSteps = builder.metaSteps;
                this.minSystemVersion = builder.minSystemVersion;
                this.maxSystemVersion = builder.maxSystemVersion;
            }
            Application.prototype.getDisplayName = function () {
                return this.displayName;
            };
            Application.prototype.getApplicationKey = function () {
                return this.applicationKey;
            };
            Application.prototype.getVersion = function () {
                return this.version;
            };
            Application.prototype.getName = function () {
                return this.applicationKey.getName();
            };
            Application.prototype.getVendorName = function () {
                return this.vendorName;
            };
            Application.prototype.getVendorUrl = function () {
                return this.vendorUrl;
            };
            Application.prototype.getUrl = function () {
                return this.url;
            };
            Application.prototype.getState = function () {
                return this.state;
            };
            Application.prototype.isStarted = function () {
                return this.state === Application.STATE_STARTED;
            };
            Application.prototype.hasChildren = function () {
                return false;
            };
            Application.prototype.getForm = function () {
                return this.config;
            };
            Application.prototype.getMinSystemVersion = function () {
                return this.minSystemVersion;
            };
            Application.prototype.getMaxSystemVersion = function () {
                return this.maxSystemVersion;
            };
            Application.prototype.getapplicationDependencies = function () {
                return this.applicationDependencies;
            };
            Application.prototype.getContentTypeDependencies = function () {
                return this.contentTypeDependencies;
            };
            Application.prototype.getMetaSteps = function () {
                return this.metaSteps;
            };
            Application.fromJson = function (json) {
                return new ApplicationBuilder().fromJson(json).build();
            };
            Application.fromJsonArray = function (jsonArray) {
                var array = [];
                jsonArray.forEach(function (json) {
                    array.push(Application.fromJson(json));
                });
                return array;
            };
            Application.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Application) || !_super.prototype.equals.call(this, o)) {
                    return false;
                }
                var other = o;
                return this.applicationKey.equals(other.applicationKey) && this.displayName == other.displayName && this.vendorName == other.vendorName && this.vendorUrl == other.vendorUrl && this.url == other.url && this.state == other.state && this.version == other.version && api.ObjectHelper.arrayEquals(this.applicationDependencies, other.applicationDependencies) && api.ObjectHelper.arrayEquals(this.contentTypeDependencies, other.contentTypeDependencies) && api.ObjectHelper.equals(this.metaSteps, other.metaSteps) && this.minSystemVersion == other.minSystemVersion && this.maxSystemVersion == other.maxSystemVersion;
            };
            Application.STATE_STARTED = 'started';
            Application.STATE_STOPPED = 'stopped';
            return Application;
        })(api.item.BaseItem);
        application.Application = Application;
        var ApplicationBuilder = (function (_super) {
            __extends(ApplicationBuilder, _super);
            function ApplicationBuilder(source) {
                this.applicationDependencies = [];
                this.contentTypeDependencies = [];
                this.metaSteps;
                if (source) {
                    _super.call(this, source);
                    this.applicationKey = source.getApplicationKey();
                    this.displayName = source.getDisplayName();
                    this.vendorName = source.getVendorName();
                    this.vendorUrl = source.getVendorUrl();
                    this.url = source.getUrl();
                    this.state = source.getState();
                    this.version = source.getVersion();
                    this.config = source.getForm();
                    this.applicationDependencies = source.getapplicationDependencies();
                    this.contentTypeDependencies = source.getContentTypeDependencies();
                    this.metaSteps = source.getMetaSteps();
                    this.minSystemVersion = source.getMinSystemVersion();
                    this.maxSystemVersion = source.getMaxSystemVersion();
                }
            }
            ApplicationBuilder.prototype.fromJson = function (json) {
                var _this = this;
                _super.prototype.fromBaseItemJson.call(this, json, 'key');
                this.applicationKey = application.ApplicationKey.fromString(json.key);
                this.displayName = json.displayName;
                this.vendorName = json.vendorName;
                this.vendorUrl = json.vendorUrl;
                this.url = json.url;
                this.state = json.state;
                this.version = json.version;
                this.config = json.config != null ? api.form.Form.fromJson(json.config) : null;
                this.minSystemVersion = json.minSystemVersion;
                this.maxSystemVersion = json.maxSystemVersion;
                if (json.applicationDependencies != null) {
                    json.applicationDependencies.forEach(function (dependency) {
                        _this.applicationDependencies.push(api.application.ApplicationKey.fromString(dependency));
                    });
                }
                if (json.contentTypeDependencies != null) {
                    json.contentTypeDependencies.forEach(function (dependency) {
                        _this.contentTypeDependencies.push(new api.schema.content.ContentTypeName(dependency));
                    });
                }
                if (json.metaSteps != null) {
                    this.metaSteps = api.schema.mixin.MixinNames.create().fromStrings(json.metaSteps).build();
                }
                return this;
            };
            ApplicationBuilder.prototype.build = function () {
                return new Application(this);
            };
            return ApplicationBuilder;
        })(api.item.BaseItemBuilder);
        application.ApplicationBuilder = ApplicationBuilder;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var ApplicationResourceRequest = (function (_super) {
            __extends(ApplicationResourceRequest, _super);
            function ApplicationResourceRequest() {
                _super.call(this);
                this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "application");
            }
            ApplicationResourceRequest.prototype.getResourcePath = function () {
                return this.resourcePath;
            };
            ApplicationResourceRequest.prototype.fromJsonToApplication = function (json) {
                return application.Application.fromJson(json);
            };
            return ApplicationResourceRequest;
        })(api.rest.ResourceRequest);
        application.ApplicationResourceRequest = ApplicationResourceRequest;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var GetApplicationRequest = (function (_super) {
            __extends(GetApplicationRequest, _super);
            function GetApplicationRequest(applicationKey, skipCache) {
                if (skipCache === void 0) { skipCache = false; }
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.applicationKey = applicationKey;
                this.skipCache = skipCache;
            }
            GetApplicationRequest.prototype.getParams = function () {
                return {
                    applicationKey: this.applicationKey.toString()
                };
            };
            GetApplicationRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this));
            };
            GetApplicationRequest.prototype.sendAndParse = function () {
                var _this = this;
                var cache = application.ApplicationCache.get();
                var appObj = this.skipCache ? null : cache.getByKey(this.applicationKey);
                if (appObj) {
                    return wemQ(appObj);
                }
                else {
                    return this.send().then(function (response) {
                        appObj = _this.fromJsonToApplication(response.getResult());
                        cache.put(appObj);
                        return appObj;
                    });
                }
            };
            return GetApplicationRequest;
        })(application.ApplicationResourceRequest);
        application.GetApplicationRequest = GetApplicationRequest;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var ListApplicationsRequest = (function (_super) {
            __extends(ListApplicationsRequest, _super);
            function ListApplicationsRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
            }
            ListApplicationsRequest.prototype.getParams = function () {
                return {
                    "query": this.searchQuery
                };
            };
            ListApplicationsRequest.prototype.setSearchQuery = function (query) {
                this.searchQuery = query;
                return this;
            };
            ListApplicationsRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "list");
            };
            ListApplicationsRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return application.Application.fromJsonArray(response.getResult().applications);
                });
            };
            return ListApplicationsRequest;
        })(application.ApplicationResourceRequest);
        application.ListApplicationsRequest = ListApplicationsRequest;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var ApplicationListResult = (function () {
            function ApplicationListResult() {
            }
            return ApplicationListResult;
        })();
        application.ApplicationListResult = ApplicationListResult;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (_application) {
        var ApplicationLoader = (function (_super) {
            __extends(ApplicationLoader, _super);
            function ApplicationLoader(delay, filterObject) {
                if (delay === void 0) { delay = 500; }
                if (filterObject === void 0) { filterObject = null; }
                _super.call(this, this.listApplicationsRequest = new _application.ListApplicationsRequest());
                if (filterObject) {
                    this.filterObject = filterObject;
                }
            }
            ApplicationLoader.prototype.search = function (searchString) {
                this.listApplicationsRequest.setSearchQuery(searchString);
                return this.load();
            };
            ApplicationLoader.prototype.load = function () {
                var me = this;
                me.notifyLoadingData();
                return me.sendRequest().then(function (applications) {
                    if (me.filterObject) {
                        applications = applications.filter(me.filterResults, me);
                    }
                    me.notifyLoadedData(applications);
                    return applications;
                });
            };
            ApplicationLoader.prototype.filterResults = function (application) {
                if (!this.filterObject) {
                    return true;
                }
                var result = true;
                for (var name in this.filterObject) {
                    if (this.filterObject.hasOwnProperty(name)) {
                        if (!application.hasOwnProperty(name) || this.filterObject[name] != application[name]) {
                            result = false;
                        }
                    }
                }
                return result;
            };
            return ApplicationLoader;
        })(api.util.loader.BaseLoader);
        _application.ApplicationLoader = ApplicationLoader;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var ApplicationLoaderListener = (function () {
            function ApplicationLoaderListener() {
            }
            return ApplicationLoaderListener;
        })();
        application.ApplicationLoaderListener = ApplicationLoaderListener;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var ApplicationViewer = (function (_super) {
            __extends(ApplicationViewer, _super);
            function ApplicationViewer() {
                _super.call(this);
            }
            ApplicationViewer.prototype.resolveDisplayName = function (object) {
                return object.getDisplayName();
            };
            ApplicationViewer.prototype.resolveSubName = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                return object.getName();
            };
            ApplicationViewer.prototype.resolveIconClass = function (object) {
                return "icon-puzzle icon-large";
            };
            return ApplicationViewer;
        })(api.ui.NamesAndIconViewer);
        application.ApplicationViewer = ApplicationViewer;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var ApplicationComboBox = (function (_super) {
            __extends(ApplicationComboBox, _super);
            function ApplicationComboBox(maximumOccurrences) {
                var builder = new api.ui.selector.combobox.RichComboBoxBuilder();
                builder.setMaximumOccurrences(maximumOccurrences || 0).setComboBoxName("applicationSelector").setLoader(new api.application.ApplicationLoader()).setSelectedOptionsView(new ApplicationSelectedOptionsView()).setOptionDisplayValueViewer(new application.ApplicationViewer()).setDelayedInputValueChangedHandling(500);
                _super.call(this, builder);
            }
            return ApplicationComboBox;
        })(api.ui.selector.combobox.RichComboBox);
        application.ApplicationComboBox = ApplicationComboBox;
        var ApplicationSelectedOptionsView = (function (_super) {
            __extends(ApplicationSelectedOptionsView, _super);
            function ApplicationSelectedOptionsView() {
                _super.apply(this, arguments);
            }
            ApplicationSelectedOptionsView.prototype.createSelectedOption = function (option) {
                var optionView = new ApplicationSelectedOptionView(option);
                return new api.ui.selector.combobox.SelectedOption(optionView, this.count());
            };
            return ApplicationSelectedOptionsView;
        })(api.ui.selector.combobox.BaseSelectedOptionsView);
        application.ApplicationSelectedOptionsView = ApplicationSelectedOptionsView;
        var ApplicationSelectedOptionView = (function (_super) {
            __extends(ApplicationSelectedOptionView, _super);
            function ApplicationSelectedOptionView(option) {
                _super.call(this, option);
            }
            ApplicationSelectedOptionView.prototype.resolveIconUrl = function (content) {
                return api.util.UriHelper.getAdminUri("common/images/icons/icoMoon/128x128/puzzle.png");
            };
            ApplicationSelectedOptionView.prototype.resolveTitle = function (content) {
                return content.getDisplayName().toString();
            };
            ApplicationSelectedOptionView.prototype.resolveSubTitle = function (content) {
                return content.getApplicationKey().toString();
            };
            return ApplicationSelectedOptionView;
        })(api.ui.selector.combobox.RichSelectedOptionView);
        application.ApplicationSelectedOptionView = ApplicationSelectedOptionView;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var StopApplicationRequest = (function (_super) {
            __extends(StopApplicationRequest, _super);
            function StopApplicationRequest(applicationKeys) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.applicationKeys = applicationKeys;
            }
            StopApplicationRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "stop");
            };
            StopApplicationRequest.prototype.getParams = function () {
                return {
                    key: application.ApplicationKey.toStringArray(this.applicationKeys)
                };
            };
            StopApplicationRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                });
            };
            return StopApplicationRequest;
        })(application.ApplicationResourceRequest);
        application.StopApplicationRequest = StopApplicationRequest;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var StartApplicationRequest = (function (_super) {
            __extends(StartApplicationRequest, _super);
            function StartApplicationRequest(applicationKeys) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.applicationKeys = applicationKeys;
            }
            StartApplicationRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "start");
            };
            StartApplicationRequest.prototype.getParams = function () {
                return {
                    key: application.ApplicationKey.toStringArray(this.applicationKeys)
                };
            };
            StartApplicationRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                });
            };
            return StartApplicationRequest;
        })(application.ApplicationResourceRequest);
        application.StartApplicationRequest = StartApplicationRequest;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var InstallApplicationRequest = (function (_super) {
            __extends(InstallApplicationRequest, _super);
            function InstallApplicationRequest(applicationUrl) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.applicationUrl = applicationUrl;
            }
            InstallApplicationRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "install");
            };
            InstallApplicationRequest.prototype.getParams = function () {
                return {
                    url: this.applicationUrl
                };
            };
            InstallApplicationRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                });
            };
            return InstallApplicationRequest;
        })(application.ApplicationResourceRequest);
        application.InstallApplicationRequest = InstallApplicationRequest;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var UninstallApplicationRequest = (function (_super) {
            __extends(UninstallApplicationRequest, _super);
            function UninstallApplicationRequest(applicationKeys) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.applicationKeys = applicationKeys;
            }
            UninstallApplicationRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "uninstall");
            };
            UninstallApplicationRequest.prototype.getParams = function () {
                return {
                    key: application.ApplicationKey.toStringArray(this.applicationKeys)
                };
            };
            UninstallApplicationRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                });
            };
            return UninstallApplicationRequest;
        })(application.ApplicationResourceRequest);
        application.UninstallApplicationRequest = UninstallApplicationRequest;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var UpdateApplicationRequest = (function (_super) {
            __extends(UpdateApplicationRequest, _super);
            function UpdateApplicationRequest(applicationKeys) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.applicationKeys = applicationKeys;
            }
            UpdateApplicationRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "update");
            };
            UpdateApplicationRequest.prototype.getParams = function () {
                return {
                    key: application.ApplicationKey.toStringArray(this.applicationKeys)
                };
            };
            UpdateApplicationRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                });
            };
            return UpdateApplicationRequest;
        })(application.ApplicationResourceRequest);
        application.UpdateApplicationRequest = UpdateApplicationRequest;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        (function (ApplicationUpdatedEventType) {
            ApplicationUpdatedEventType[ApplicationUpdatedEventType["INSTALLED"] = 0] = "INSTALLED";
            ApplicationUpdatedEventType[ApplicationUpdatedEventType["UNINSTALLED"] = 1] = "UNINSTALLED";
            ApplicationUpdatedEventType[ApplicationUpdatedEventType["RESOLVED"] = 2] = "RESOLVED";
            ApplicationUpdatedEventType[ApplicationUpdatedEventType["STARTING"] = 3] = "STARTING";
            ApplicationUpdatedEventType[ApplicationUpdatedEventType["STARTED"] = 4] = "STARTED";
            ApplicationUpdatedEventType[ApplicationUpdatedEventType["UPDATED"] = 5] = "UPDATED";
            ApplicationUpdatedEventType[ApplicationUpdatedEventType["STOPPING"] = 6] = "STOPPING";
            ApplicationUpdatedEventType[ApplicationUpdatedEventType["STOPPED"] = 7] = "STOPPED";
            ApplicationUpdatedEventType[ApplicationUpdatedEventType["UNRESOLVED"] = 8] = "UNRESOLVED";
        })(application.ApplicationUpdatedEventType || (application.ApplicationUpdatedEventType = {}));
        var ApplicationUpdatedEventType = application.ApplicationUpdatedEventType;
        var ApplicationUpdatedEvent = (function (_super) {
            __extends(ApplicationUpdatedEvent, _super);
            function ApplicationUpdatedEvent(applicationKey, eventType) {
                _super.call(this);
                this.applicationKey = applicationKey;
                this.eventType = eventType;
            }
            ApplicationUpdatedEvent.prototype.getApplicationKey = function () {
                return this.applicationKey;
            };
            ApplicationUpdatedEvent.prototype.getEventType = function () {
                return this.eventType;
            };
            ApplicationUpdatedEvent.prototype.isNeedToUpdateApplication = function () {
                return 2 /* RESOLVED */ != this.eventType && 3 /* STARTING */ != this.eventType && 8 /* UNRESOLVED */ != this.eventType && 6 /* STOPPING */ != this.eventType;
            };
            ApplicationUpdatedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ApplicationUpdatedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            ApplicationUpdatedEvent.fromJson = function (json) {
                var applicationKey = api.application.ApplicationKey.fromString(json.applicationKey);
                var eventType = ApplicationUpdatedEventType[json.eventType];
                return new ApplicationUpdatedEvent(applicationKey, eventType);
            };
            return ApplicationUpdatedEvent;
        })(api.event.Event);
        application.ApplicationUpdatedEvent = ApplicationUpdatedEvent;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var ApplicationCache = (function (_super) {
            __extends(ApplicationCache, _super);
            function ApplicationCache() {
                var _this = this;
                _super.call(this);
                application.ApplicationUpdatedEvent.on(function (event) {
                    console.log("ApplicationCache on ApplicationUpdatedEvent, deleting: " + event.getApplicationKey().toString());
                    _this.deleteByKey(event.getApplicationKey());
                });
            }
            ApplicationCache.prototype.copy = function (object) {
                return new application.ApplicationBuilder(object).build();
            };
            ApplicationCache.prototype.getKeyFromObject = function (object) {
                return object.getApplicationKey();
            };
            ApplicationCache.prototype.getKeyAsString = function (key) {
                return key.toString();
            };
            ApplicationCache.get = function () {
                if (!ApplicationCache.instance) {
                    ApplicationCache.instance = new ApplicationCache();
                }
                return ApplicationCache.instance;
            };
            return ApplicationCache;
        })(api.cache.Cache);
        application.ApplicationCache = ApplicationCache;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var ApplicationBasedName = (function () {
            function ApplicationBasedName(applicationKey, localName) {
                this.applicationKey = applicationKey;
                this.localName = localName;
                this.refString = applicationKey.toString() ? applicationKey.toString() + ApplicationBasedName.SEPARATOR + localName : localName;
            }
            ApplicationBasedName.prototype.getLocalName = function () {
                return this.localName;
            };
            ApplicationBasedName.prototype.getApplicationKey = function () {
                return this.applicationKey;
            };
            ApplicationBasedName.prototype.toString = function () {
                return this.refString;
            };
            ApplicationBasedName.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, api.ClassHelper.getClass(this))) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                    return false;
                }
                return true;
            };
            ApplicationBasedName.SEPARATOR = ":";
            return ApplicationBasedName;
        })();
        application.ApplicationBasedName = ApplicationBasedName;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var ApplicationCaches = (function () {
            function ApplicationCaches() {
                this.cacheByApplicationKey = {};
            }
            ApplicationCaches.prototype.put = function (key, cache) {
                this.cacheByApplicationKey[key.toString()] = cache;
            };
            ApplicationCaches.prototype.getByKey = function (key) {
                return this.cacheByApplicationKey[key.toString()];
            };
            ApplicationCaches.prototype.removeByKey = function (key) {
                delete this.cacheByApplicationKey[key.toString()];
            };
            return ApplicationCaches;
        })();
        application.ApplicationCaches = ApplicationCaches;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var application;
    (function (application) {
        var ApplicationBasedCache = (function () {
            function ApplicationBasedCache() {
                var _this = this;
                this.applicationCaches = new application.ApplicationCaches();
                application.ApplicationUpdatedEvent.on(function (event) {
                    if (4 /* STARTED */ == event.getEventType()) {
                        console.log(api.ClassHelper.getClassName(_this) + " received ApplicationUpdatedEvent STARTED, calling - loadByApplication.. " + event.getApplicationKey().toString());
                        _this.loadByApplication(event.getApplicationKey());
                    }
                    else if (7 /* STOPPED */ == event.getEventType()) {
                        console.log(api.ClassHelper.getClassName(_this) + " received ApplicationUpdatedEvent STOPPED - calling deleteByApplicationKey.. " + event.getApplicationKey().toString());
                        _this.deleteByApplicationKey(event.getApplicationKey());
                    }
                });
            }
            ApplicationBasedCache.prototype.loadByApplication = function (applicationKey) {
                throw new Error("Must be implemented by inheritor");
            };
            ApplicationBasedCache.prototype.getByApplication = function (applicationKey) {
                api.util.assertNotNull(applicationKey, "applicationKey not given");
                var cache = this.applicationCaches.getByKey(applicationKey);
                if (!cache) {
                    return null;
                }
                return cache.getAll();
            };
            ApplicationBasedCache.prototype.getByKey = function (key, applicationKey) {
                api.util.assertNotNull(key, "key not given");
                var cache = this.applicationCaches.getByKey(applicationKey);
                if (!cache) {
                    return null;
                }
                return cache.getByKey(key);
            };
            ApplicationBasedCache.prototype.put = function (object, applicationKey) {
                api.util.assertNotNull(object, "a object to cache must be given");
                var cache = this.applicationCaches.getByKey(applicationKey);
                if (!cache) {
                    cache = this.createApplicationCache();
                    this.applicationCaches.put(applicationKey, cache);
                }
                cache.put(object);
            };
            ApplicationBasedCache.prototype.createApplicationCache = function () {
                throw new Error("Must be implemented by inheritor");
            };
            ApplicationBasedCache.prototype.deleteByApplicationKey = function (applicationKey) {
                this.applicationCaches.removeByKey(applicationKey);
            };
            return ApplicationBasedCache;
        })();
        application.ApplicationBasedCache = ApplicationBasedCache;
    })(application = api.application || (api.application = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var Schema = (function (_super) {
            __extends(Schema, _super);
            function Schema(builder) {
                _super.call(this, builder);
                this.name = builder.name;
                this.displayName = builder.displayName;
                this.description = builder.description;
                this.iconUrl = builder.iconUrl;
            }
            Schema.prototype.getName = function () {
                return this.name;
            };
            Schema.prototype.getDisplayName = function () {
                return this.displayName;
            };
            Schema.prototype.getDescription = function () {
                return this.description;
            };
            Schema.prototype.getIconUrl = function () {
                return this.iconUrl;
            };
            Schema.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Schema)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.name, other.name)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.displayName, other.displayName)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.iconUrl, other.iconUrl)) {
                    return false;
                }
                return true;
            };
            Schema.fromJson = function (json) {
                return new SchemaBuilder().fromSchemaJson(json).build();
            };
            return Schema;
        })(api.item.BaseItem);
        schema.Schema = Schema;
        var SchemaBuilder = (function (_super) {
            __extends(SchemaBuilder, _super);
            function SchemaBuilder(source) {
                if (source) {
                    _super.call(this, source);
                    this.name = source.getName();
                    this.displayName = source.getDisplayName();
                    this.description = source.getDescription();
                    this.iconUrl = source.getIconUrl();
                }
            }
            SchemaBuilder.prototype.fromSchemaJson = function (json) {
                _super.prototype.fromBaseItemJson.call(this, json, "name");
                this.name = json.name;
                this.displayName = json.displayName;
                this.description = json.description;
                this.iconUrl = json.iconUrl;
                return this;
            };
            SchemaBuilder.prototype.setName = function (value) {
                this.name = value;
                return this;
            };
            SchemaBuilder.prototype.build = function () {
                return new Schema(this);
            };
            return SchemaBuilder;
        })(api.item.BaseItemBuilder);
        schema.SchemaBuilder = SchemaBuilder;
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (_schema) {
        var SchemaIconUrlResolver = (function (_super) {
            __extends(SchemaIconUrlResolver, _super);
            function SchemaIconUrlResolver() {
                _super.apply(this, arguments);
            }
            SchemaIconUrlResolver.prototype.resolve = function (schema) {
                return schema.getIconUrl();
            };
            SchemaIconUrlResolver.getResourcePath = function () {
                return api.rest.Path.fromString("schema/icon");
            };
            return SchemaIconUrlResolver;
        })(api.icon.IconUrlResolver);
        _schema.SchemaIconUrlResolver = SchemaIconUrlResolver;
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var mixin;
        (function (mixin) {
            var ApplicationKey = api.application.ApplicationKey;
            var MixinName = (function (_super) {
                __extends(MixinName, _super);
                function MixinName(name) {
                    api.util.assertNotNull(name, "Mixin name can't be null");
                    var parts = name.split(api.application.ApplicationBasedName.SEPARATOR);
                    _super.call(this, ApplicationKey.fromString(parts[0]), parts[1]);
                }
                MixinName.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, MixinName)) {
                        return false;
                    }
                    return _super.prototype.equals.call(this, o);
                };
                return MixinName;
            })(api.application.ApplicationBasedName);
            mixin.MixinName = MixinName;
        })(mixin = schema.mixin || (schema.mixin = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var mixin;
        (function (_mixin) {
            var MixinNames = (function () {
                function MixinNames(array) {
                    var _this = this;
                    this.array = [];
                    array.forEach(function (mixinName) {
                        var duplicate = _this.array.some(function (possibleDuplicate) {
                            return mixinName.equals(possibleDuplicate);
                        });
                        if (!duplicate) {
                            _this.array.push(mixinName);
                        }
                        else {
                            throw Error("MixinNames do not allow duplicates, found: '" + mixinName.toString() + "'");
                        }
                    });
                }
                MixinNames.prototype.forEach = function (callback) {
                    this.array.forEach(function (mixinName, index) {
                        callback(mixinName, index);
                    });
                };
                MixinNames.prototype.contains = function (mixinName) {
                    var containName = this.array.some(function (curMixin) {
                        return curMixin.equals(mixinName);
                    });
                    return !!containName;
                };
                MixinNames.prototype.filter = function (callbackfn) {
                    return new MixinNames(this.array.filter(function (value, index) {
                        return callbackfn(value, index);
                    }));
                };
                MixinNames.prototype.map = function (callbackfn) {
                    return this.array.map(function (value, index) {
                        return callbackfn(value, index);
                    });
                };
                MixinNames.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, MixinNames)) {
                        return false;
                    }
                    var other = o;
                    return api.ObjectHelper.arrayEquals(this.array, other.array);
                };
                MixinNames.create = function () {
                    return new MixinNamesBuilder();
                };
                return MixinNames;
            })();
            _mixin.MixinNames = MixinNames;
            var MixinNamesBuilder = (function () {
                function MixinNamesBuilder() {
                    this.array = [];
                }
                MixinNamesBuilder.prototype.fromStrings = function (values) {
                    var _this = this;
                    if (!!values) {
                        values.forEach(function (value) {
                            _this.addMixinName(new _mixin.MixinName(value));
                        });
                    }
                    return this;
                };
                MixinNamesBuilder.prototype.fromMixins = function (mixins) {
                    var _this = this;
                    if (!!mixins) {
                        mixins.forEach(function (mixin) {
                            _this.addMixinName(mixin.getMixinName());
                        });
                    }
                    return this;
                };
                MixinNamesBuilder.prototype.addMixinName = function (value) {
                    this.array.push(value);
                    return this;
                };
                MixinNamesBuilder.prototype.build = function () {
                    return new MixinNames(this.array);
                };
                return MixinNamesBuilder;
            })();
            _mixin.MixinNamesBuilder = MixinNamesBuilder;
        })(mixin = schema.mixin || (schema.mixin = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var mixin;
        (function (mixin) {
            var Mixin = (function (_super) {
                __extends(Mixin, _super);
                function Mixin(builder) {
                    _super.call(this, builder);
                    this.formItems = builder.formItems;
                    this.schemaKey = builder.schemaKey;
                }
                Mixin.prototype.getMixinName = function () {
                    return new mixin.MixinName(this.getName());
                };
                Mixin.prototype.getFormItems = function () {
                    return this.formItems;
                };
                Mixin.prototype.getSchemaKey = function () {
                    return this.schemaKey;
                };
                Mixin.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Mixin)) {
                        return false;
                    }
                    if (!_super.prototype.equals.call(this, o)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.stringEquals(this.schemaKey, other.schemaKey)) {
                        return false;
                    }
                    if (!api.ObjectHelper.arrayEquals(this.formItems, other.formItems)) {
                        return false;
                    }
                    return true;
                };
                Mixin.prototype.toForm = function () {
                    return new api.form.FormBuilder().addFormItems(this.formItems).build();
                };
                Mixin.fromJson = function (json) {
                    return new MixinBuilder().fromMixinJson(json).build();
                };
                return Mixin;
            })(api.schema.Schema);
            mixin.Mixin = Mixin;
            var MixinBuilder = (function (_super) {
                __extends(MixinBuilder, _super);
                function MixinBuilder(source) {
                    _super.call(this, source);
                    if (source) {
                        this.schemaKey = source.getSchemaKey();
                        this.formItems = source.getFormItems();
                    }
                }
                MixinBuilder.prototype.fromMixinJson = function (mixinJson) {
                    var _this = this;
                    _super.prototype.fromSchemaJson.call(this, mixinJson);
                    this.formItems = [];
                    mixinJson.items.forEach(function (formItemJson) {
                        _this.formItems.push(api.form.FormItemFactory.createFormItem(formItemJson));
                    });
                    this.schemaKey = "mixin:" + this.name;
                    return this;
                };
                MixinBuilder.prototype.build = function () {
                    return new Mixin(this);
                };
                return MixinBuilder;
            })(api.schema.SchemaBuilder);
            mixin.MixinBuilder = MixinBuilder;
        })(mixin = schema.mixin || (schema.mixin = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var mixin;
        (function (mixin) {
            var MixinResourceRequest = (function (_super) {
                __extends(MixinResourceRequest, _super);
                function MixinResourceRequest() {
                    _super.call(this);
                    this.resourceUrl = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "schema/mixin");
                }
                MixinResourceRequest.prototype.getResourcePath = function () {
                    return this.resourceUrl;
                };
                MixinResourceRequest.prototype.fromJsonToMixin = function (json) {
                    return mixin.Mixin.fromJson(json);
                };
                return MixinResourceRequest;
            })(api.rest.ResourceRequest);
            mixin.MixinResourceRequest = MixinResourceRequest;
        })(mixin = schema.mixin || (schema.mixin = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var mixin;
        (function (mixin) {
            var GetMixinByQualifiedNameRequest = (function (_super) {
                __extends(GetMixinByQualifiedNameRequest, _super);
                function GetMixinByQualifiedNameRequest(name) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.name = name;
                }
                GetMixinByQualifiedNameRequest.prototype.getParams = function () {
                    return {
                        name: this.name.toString()
                    };
                };
                GetMixinByQualifiedNameRequest.prototype.getRequestPath = function () {
                    return _super.prototype.getResourcePath.call(this);
                };
                GetMixinByQualifiedNameRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return _this.fromJsonToMixin(response.getResult());
                    });
                };
                return GetMixinByQualifiedNameRequest;
            })(mixin.MixinResourceRequest);
            mixin.GetMixinByQualifiedNameRequest = GetMixinByQualifiedNameRequest;
        })(mixin = schema.mixin || (schema.mixin = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var mixin;
        (function (mixin) {
            var GetMixinsByApplicationRequest = (function (_super) {
                __extends(GetMixinsByApplicationRequest, _super);
                function GetMixinsByApplicationRequest(applicationKey) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.applicationKey = applicationKey;
                }
                GetMixinsByApplicationRequest.prototype.getParams = function () {
                    return {
                        applicationKey: this.applicationKey.toString()
                    };
                };
                GetMixinsByApplicationRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "byApplication");
                };
                GetMixinsByApplicationRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().mixins.map(function (mixinJson) {
                            return _this.fromJsonToMixin(mixinJson);
                        });
                    });
                };
                return GetMixinsByApplicationRequest;
            })(mixin.MixinResourceRequest);
            mixin.GetMixinsByApplicationRequest = GetMixinsByApplicationRequest;
        })(mixin = schema.mixin || (schema.mixin = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var mixin;
        (function (mixin) {
            var InlineMixinJson = (function (_super) {
                __extends(InlineMixinJson, _super);
                function InlineMixinJson() {
                    _super.apply(this, arguments);
                }
                return InlineMixinJson;
            })(api.form.json.FormItemJson);
            mixin.InlineMixinJson = InlineMixinJson;
        })(mixin = schema.mixin || (schema.mixin = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ApplicationKey = api.application.ApplicationKey;
            var ContentTypeName = (function (_super) {
                __extends(ContentTypeName, _super);
                function ContentTypeName(name) {
                    api.util.assertNotNull(name, "Content type name can't be null");
                    var parts = name.split(api.application.ApplicationBasedName.SEPARATOR);
                    _super.call(this, ApplicationKey.fromString(parts[0]), parts[1]);
                }
                ContentTypeName.from = function (applicationKey, localName) {
                    return new ContentTypeName(applicationKey.toString() + ":" + localName);
                };
                ContentTypeName.prototype.isSite = function () {
                    return ContentTypeName.SITE.equals(this);
                };
                ContentTypeName.prototype.isPageTemplate = function () {
                    return ContentTypeName.PAGE_TEMPLATE.equals(this);
                };
                ContentTypeName.prototype.isTemplateFolder = function () {
                    return ContentTypeName.TEMPLATE_FOLDER.equals(this);
                };
                ContentTypeName.prototype.isImage = function () {
                    return ContentTypeName.IMAGE.equals(this);
                };
                ContentTypeName.prototype.isMedia = function () {
                    return ContentTypeName.MEDIA.equals(this);
                };
                ContentTypeName.getMediaTypes = function () {
                    return [
                        ContentTypeName.MEDIA_ARCHIVE,
                        ContentTypeName.MEDIA_AUDIO,
                        ContentTypeName.MEDIA_VIDEO,
                        ContentTypeName.MEDIA_CODE,
                        ContentTypeName.MEDIA_DATA,
                        ContentTypeName.MEDIA_DOCUMENT,
                        ContentTypeName.MEDIA_EXECUTABLE,
                        ContentTypeName.MEDIA_IMAGE,
                        ContentTypeName.MEDIA_SPREADSHEET,
                        ContentTypeName.MEDIA_PRESENTATION,
                        ContentTypeName.MEDIA_VECTOR,
                        ContentTypeName.MEDIA_TEXT,
                        ContentTypeName.MEDIA_UNKNOWN
                    ];
                };
                ContentTypeName.prototype.isDescendantOfMedia = function () {
                    var _this = this;
                    return ContentTypeName.getMediaTypes().some(function (contentTypeName) {
                        return contentTypeName.equals(_this);
                    });
                };
                ContentTypeName.UNSTRUCTURED = ContentTypeName.from(ApplicationKey.BASE, "unstructured");
                ContentTypeName.FOLDER = ContentTypeName.from(ApplicationKey.BASE, "folder");
                ContentTypeName.SHORTCUT = ContentTypeName.from(ApplicationKey.BASE, "shortcut");
                ContentTypeName.MEDIA = ContentTypeName.from(ApplicationKey.BASE, "media");
                ContentTypeName.MEDIA_TEXT = ContentTypeName.from(ApplicationKey.MEDIA, "text");
                ContentTypeName.MEDIA_DATA = ContentTypeName.from(ApplicationKey.MEDIA, "data");
                ContentTypeName.MEDIA_AUDIO = ContentTypeName.from(ApplicationKey.MEDIA, "audio");
                ContentTypeName.MEDIA_VIDEO = ContentTypeName.from(ApplicationKey.MEDIA, "video");
                ContentTypeName.MEDIA_IMAGE = ContentTypeName.from(ApplicationKey.MEDIA, "image");
                ContentTypeName.MEDIA_VECTOR = ContentTypeName.from(ApplicationKey.MEDIA, "vector");
                ContentTypeName.MEDIA_ARCHIVE = ContentTypeName.from(ApplicationKey.MEDIA, "archive");
                ContentTypeName.MEDIA_DOCUMENT = ContentTypeName.from(ApplicationKey.MEDIA, "document");
                ContentTypeName.MEDIA_SPREADSHEET = ContentTypeName.from(ApplicationKey.MEDIA, "spreadsheet");
                ContentTypeName.MEDIA_PRESENTATION = ContentTypeName.from(ApplicationKey.MEDIA, "presentation");
                ContentTypeName.MEDIA_CODE = ContentTypeName.from(ApplicationKey.MEDIA, "code");
                ContentTypeName.MEDIA_EXECUTABLE = ContentTypeName.from(ApplicationKey.MEDIA, "executable");
                ContentTypeName.MEDIA_UNKNOWN = ContentTypeName.from(ApplicationKey.MEDIA, "unknown");
                ContentTypeName.SITE = ContentTypeName.from(ApplicationKey.PORTAL, "site");
                ContentTypeName.PAGE_TEMPLATE = ContentTypeName.from(ApplicationKey.PORTAL, "page-template");
                ContentTypeName.TEMPLATE_FOLDER = ContentTypeName.from(ApplicationKey.PORTAL, "template-folder");
                ContentTypeName.IMAGE = ContentTypeName.from(ApplicationKey.MEDIA, "image");
                return ContentTypeName;
            })(api.application.ApplicationBasedName);
            content.ContentTypeName = ContentTypeName;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var MixinNames = api.schema.mixin.MixinNames;
            var ContentTypeSummary = (function (_super) {
                __extends(ContentTypeSummary, _super);
                function ContentTypeSummary(builder) {
                    _super.call(this, builder);
                    this.allowChildContent = builder.allowChildContent;
                    this.final = builder.final;
                    this.abstract = builder.abstract;
                    this.superType = builder.superType;
                    this.contentDisplayNameScript = builder.contentDisplayNameScript;
                    this.owner = builder.owner;
                    this.modifier = builder.modifier;
                    this.metadata = builder.metadata;
                }
                ContentTypeSummary.prototype.getContentTypeName = function () {
                    return new api.schema.content.ContentTypeName(this.getName());
                };
                ContentTypeSummary.prototype.isSite = function () {
                    return this.getContentTypeName().isSite();
                };
                ContentTypeSummary.prototype.isPageTemplate = function () {
                    return this.getContentTypeName().isPageTemplate();
                };
                ContentTypeSummary.prototype.isImage = function () {
                    return this.getContentTypeName().isImage();
                };
                ContentTypeSummary.prototype.isFinal = function () {
                    return this.final;
                };
                ContentTypeSummary.prototype.isAbstract = function () {
                    return this.abstract;
                };
                ContentTypeSummary.prototype.isAllowChildContent = function () {
                    return this.allowChildContent;
                };
                ContentTypeSummary.prototype.getSuperType = function () {
                    return this.superType;
                };
                ContentTypeSummary.prototype.hasContentDisplayNameScript = function () {
                    return !api.util.StringHelper.isBlank(this.contentDisplayNameScript);
                };
                ContentTypeSummary.prototype.getContentDisplayNameScript = function () {
                    return this.contentDisplayNameScript;
                };
                ContentTypeSummary.prototype.getOwner = function () {
                    return this.owner;
                };
                ContentTypeSummary.prototype.getModifier = function () {
                    return this.modifier;
                };
                ContentTypeSummary.prototype.getMetadata = function () {
                    return this.metadata;
                };
                ContentTypeSummary.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentTypeSummary)) {
                        return false;
                    }
                    if (!_super.prototype.equals.call(this, o)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.booleanEquals(this.allowChildContent, other.allowChildContent)) {
                        return false;
                    }
                    if (!api.ObjectHelper.booleanEquals(this.abstract, other.abstract)) {
                        return false;
                    }
                    if (!api.ObjectHelper.equals(this.superType, other.superType)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringEquals(this.contentDisplayNameScript, other.contentDisplayNameScript)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringEquals(this.modifier, other.modifier)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringEquals(this.owner, other.owner)) {
                        return false;
                    }
                    if (!api.ObjectHelper.equals(this.metadata, other.metadata)) {
                        return false;
                    }
                    return true;
                };
                ContentTypeSummary.fromJsonArray = function (jsonArray) {
                    var array = [];
                    jsonArray.forEach(function (summaryJson) {
                        array.push(ContentTypeSummary.fromJson(summaryJson));
                    });
                    return array;
                };
                ContentTypeSummary.fromJson = function (json) {
                    return new ContentTypeSummaryBuilder().fromContentTypeSummaryJson(json).build();
                };
                return ContentTypeSummary;
            })(api.schema.Schema);
            content.ContentTypeSummary = ContentTypeSummary;
            var ContentTypeSummaryBuilder = (function (_super) {
                __extends(ContentTypeSummaryBuilder, _super);
                function ContentTypeSummaryBuilder(source) {
                    if (source) {
                        _super.call(this, source);
                        this.allowChildContent = source.isAllowChildContent();
                        this.abstract = source.isAbstract();
                        this.final = source.isFinal();
                        this.superType = source.getSuperType();
                        this.contentDisplayNameScript = source.getContentDisplayNameScript();
                        this.modifier = source.getModifier();
                        this.owner = source.getOwner();
                        this.metadata = source.getMetadata();
                    }
                }
                ContentTypeSummaryBuilder.prototype.fromContentTypeSummaryJson = function (json) {
                    _super.prototype.fromSchemaJson.call(this, json);
                    this.allowChildContent = json.allowChildContent;
                    this.final = json.final;
                    this.abstract = json.abstract;
                    this.superType = json.superType ? new api.schema.content.ContentTypeName(json.superType) : null;
                    this.contentDisplayNameScript = json.contentDisplayNameScript;
                    this.owner = json.owner;
                    this.modifier = json.modifier;
                    this.metadata = MixinNames.create().fromStrings(json.metadata).build();
                    return this;
                };
                ContentTypeSummaryBuilder.prototype.build = function () {
                    return new ContentTypeSummary(this);
                };
                return ContentTypeSummaryBuilder;
            })(api.schema.SchemaBuilder);
            content.ContentTypeSummaryBuilder = ContentTypeSummaryBuilder;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ContentType = (function (_super) {
                __extends(ContentType, _super);
                function ContentType(builder) {
                    _super.call(this, builder);
                    this.form = builder.form;
                }
                ContentType.prototype.getForm = function () {
                    return this.form;
                };
                ContentType.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentType)) {
                        return false;
                    }
                    if (!_super.prototype.equals.call(this, o)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.equals(this.form, other.form)) {
                        return false;
                    }
                    return true;
                };
                ContentType.fromJson = function (json) {
                    return new ContentTypeBuilder().fromContentTypeJson(json).build();
                };
                return ContentType;
            })(content.ContentTypeSummary);
            content.ContentType = ContentType;
            var ContentTypeBuilder = (function (_super) {
                __extends(ContentTypeBuilder, _super);
                function ContentTypeBuilder(source) {
                    if (source) {
                        _super.call(this, source);
                        this.form = source.getForm();
                    }
                }
                ContentTypeBuilder.prototype.fromContentTypeJson = function (json) {
                    _super.prototype.fromContentTypeSummaryJson.call(this, json);
                    this.form = api.form.FormItemFactory.createForm(json.form);
                    return this;
                };
                ContentTypeBuilder.prototype.setForm = function (value) {
                    this.form = value;
                    return this;
                };
                ContentTypeBuilder.prototype.build = function () {
                    return new ContentType(this);
                };
                return ContentTypeBuilder;
            })(content.ContentTypeSummaryBuilder);
            content.ContentTypeBuilder = ContentTypeBuilder;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ContentTypeIconUrlResolver = (function (_super) {
                __extends(ContentTypeIconUrlResolver, _super);
                function ContentTypeIconUrlResolver() {
                    _super.apply(this, arguments);
                }
                ContentTypeIconUrlResolver.default = function () {
                    return api.util.UriHelper.getRestUri(api.rest.Path.fromParent(api.schema.SchemaIconUrlResolver.getResourcePath(), "base:structured").toString());
                };
                return ContentTypeIconUrlResolver;
            })(api.schema.SchemaIconUrlResolver);
            content.ContentTypeIconUrlResolver = ContentTypeIconUrlResolver;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ContentTypeSummaryViewer = (function (_super) {
                __extends(ContentTypeSummaryViewer, _super);
                function ContentTypeSummaryViewer() {
                    _super.call(this);
                    this.contentTypeIconUrlResolver = new content.ContentTypeIconUrlResolver();
                }
                ContentTypeSummaryViewer.prototype.resolveDisplayName = function (object) {
                    return object.getDisplayName();
                };
                ContentTypeSummaryViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getName();
                };
                ContentTypeSummaryViewer.prototype.resolveIconUrl = function (object) {
                    return this.contentTypeIconUrlResolver.resolve(object);
                };
                return ContentTypeSummaryViewer;
            })(api.ui.NamesAndIconViewer);
            content.ContentTypeSummaryViewer = ContentTypeSummaryViewer;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (_content) {
            var ContentTypeComboBox = (function (_super) {
                __extends(ContentTypeComboBox, _super);
                function ContentTypeComboBox(maximumOccurrences, loader) {
                    if (maximumOccurrences === void 0) { maximumOccurrences = 0; }
                    var loader = loader || new _content.ContentTypeSummaryLoader();
                    _super.call(this, new api.ui.selector.combobox.RichComboBoxBuilder().setLoader(loader).setSelectedOptionsView(new ContentTypeSelectedOptionsView()).setOptionDisplayValueViewer(new _content.ContentTypeSummaryViewer()).setMaximumOccurrences(maximumOccurrences));
                    loader.load();
                }
                return ContentTypeComboBox;
            })(api.ui.selector.combobox.RichComboBox);
            _content.ContentTypeComboBox = ContentTypeComboBox;
            var ContentTypeSelectedOptionsView = (function (_super) {
                __extends(ContentTypeSelectedOptionsView, _super);
                function ContentTypeSelectedOptionsView() {
                    _super.apply(this, arguments);
                }
                ContentTypeSelectedOptionsView.prototype.createSelectedOption = function (option) {
                    var optionView = new ContentTypeSelectedOptionView(option);
                    return new api.ui.selector.combobox.SelectedOption(optionView, this.count());
                };
                return ContentTypeSelectedOptionsView;
            })(api.ui.selector.combobox.BaseSelectedOptionsView);
            _content.ContentTypeSelectedOptionsView = ContentTypeSelectedOptionsView;
            var ContentTypeSelectedOptionView = (function (_super) {
                __extends(ContentTypeSelectedOptionView, _super);
                function ContentTypeSelectedOptionView(option) {
                    _super.call(this, option);
                }
                ContentTypeSelectedOptionView.prototype.resolveIconUrl = function (content) {
                    return content.getIconUrl();
                };
                ContentTypeSelectedOptionView.prototype.resolveTitle = function (content) {
                    return content.getDisplayName().toString();
                };
                ContentTypeSelectedOptionView.prototype.resolveSubTitle = function (content) {
                    return content.getName();
                };
                return ContentTypeSelectedOptionView;
            })(api.ui.selector.combobox.RichSelectedOptionView);
            _content.ContentTypeSelectedOptionView = ContentTypeSelectedOptionView;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ContentTypeSummaryLoader = (function (_super) {
                __extends(ContentTypeSummaryLoader, _super);
                function ContentTypeSummaryLoader() {
                    _super.call(this, new content.GetAllContentTypesRequest());
                }
                ContentTypeSummaryLoader.prototype.filterFn = function (contentType) {
                    return contentType.getContentTypeName().toString().indexOf(this.getSearchString().toLowerCase()) != -1;
                };
                return ContentTypeSummaryLoader;
            })(api.util.loader.BaseLoader);
            content.ContentTypeSummaryLoader = ContentTypeSummaryLoader;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var PageTemplateContentTypeLoader = (function (_super) {
                __extends(PageTemplateContentTypeLoader, _super);
                function PageTemplateContentTypeLoader(contentId) {
                    _super.call(this, new content.GetAllContentTypesRequest());
                    this.contentId = contentId;
                }
                PageTemplateContentTypeLoader.prototype.filterFn = function (contentType) {
                    return contentType.getContentTypeName().toString().indexOf(this.getSearchString().toLowerCase()) != -1;
                };
                PageTemplateContentTypeLoader.prototype.sendRequest = function () {
                    var _this = this;
                    return new content.GetAllContentTypesRequest().sendAndParse().then(function (contentTypeArray) {
                        return new api.content.GetNearestSiteRequest(_this.contentId).sendAndParse().then(function (parentSite) {
                            var typesAllowedEverywhere = {};
                            [content.ContentTypeName.UNSTRUCTURED, content.ContentTypeName.FOLDER, content.ContentTypeName.SITE, content.ContentTypeName.SHORTCUT].forEach(function (contentTypeName) {
                                typesAllowedEverywhere[contentTypeName.toString()] = contentTypeName;
                            });
                            var siteApplications = {};
                            parentSite.getApplicationKeys().forEach(function (applicationKey) {
                                siteApplications[applicationKey.toString()] = applicationKey;
                            });
                            var results = contentTypeArray.filter(function (item) {
                                var contentTypeName = item.getContentTypeName();
                                if (item.isAbstract()) {
                                    return false;
                                }
                                else if (contentTypeName.isDescendantOfMedia()) {
                                    return true;
                                }
                                else if (typesAllowedEverywhere[contentTypeName.toString()]) {
                                    return true;
                                }
                                else if (siteApplications[contentTypeName.getApplicationKey().toString()]) {
                                    return true;
                                }
                                else {
                                    return false;
                                }
                            });
                            return results;
                        });
                    });
                };
                return PageTemplateContentTypeLoader;
            })(api.util.loader.BaseLoader);
            content.PageTemplateContentTypeLoader = PageTemplateContentTypeLoader;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            (function (ContentStateEnum) {
                ContentStateEnum[ContentStateEnum["PENDING_DELETE"] = 0] = "PENDING_DELETE";
                ContentStateEnum[ContentStateEnum["DEFAULT"] = 1] = "DEFAULT";
            })(content.ContentStateEnum || (content.ContentStateEnum = {}));
            var ContentStateEnum = content.ContentStateEnum;
            var ContentState = (function () {
                function ContentState(state) {
                    if (state === void 0) { state = 1 /* DEFAULT */; }
                    this.state = state;
                }
                ContentState.prototype.getState = function () {
                    return this.state;
                };
                ContentState.prototype.getStateAsString = function () {
                    return ContentStateEnum[this.state];
                };
                ContentState.fromString = function (value) {
                    if (value === ContentStateEnum[0 /* PENDING_DELETE */]) {
                        return new ContentState(0 /* PENDING_DELETE */);
                    }
                    else {
                        return new ContentState(1 /* DEFAULT */);
                    }
                };
                ContentState.prototype.isDefault = function () {
                    return this.state === 1 /* DEFAULT */;
                };
                ContentState.prototype.isPendingDelete = function () {
                    return this.state === 0 /* PENDING_DELETE */;
                };
                return ContentState;
            })();
            content.ContentState = ContentState;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ContentTypeResourceRequest = (function (_super) {
                __extends(ContentTypeResourceRequest, _super);
                function ContentTypeResourceRequest() {
                    _super.call(this);
                    this.resourceUrl = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "schema/content");
                }
                ContentTypeResourceRequest.prototype.getResourcePath = function () {
                    return this.resourceUrl;
                };
                ContentTypeResourceRequest.prototype.fromJsonToContentType = function (json) {
                    return content.ContentType.fromJson(json);
                };
                ContentTypeResourceRequest.prototype.fromJsonToContentTypeSummary = function (json) {
                    return content.ContentTypeSummary.fromJson(json);
                };
                return ContentTypeResourceRequest;
            })(api.rest.ResourceRequest);
            content.ContentTypeResourceRequest = ContentTypeResourceRequest;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var GetAllContentTypesRequest = (function (_super) {
                __extends(GetAllContentTypesRequest, _super);
                function GetAllContentTypesRequest() {
                    _super.call(this);
                    this.inlineMixinsToFormItems = true;
                    _super.prototype.setMethod.call(this, "GET");
                }
                GetAllContentTypesRequest.prototype.getParams = function () {
                    return {
                        inlineMixinsToFormItems: this.inlineMixinsToFormItems
                    };
                };
                GetAllContentTypesRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "all");
                };
                GetAllContentTypesRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().contentTypes.map(function (contentTypeJson) {
                            return _this.fromJsonToContentTypeSummary(contentTypeJson);
                        });
                    });
                };
                return GetAllContentTypesRequest;
            })(content.ContentTypeResourceRequest);
            content.GetAllContentTypesRequest = GetAllContentTypesRequest;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var GetContentTypeByNameRequest = (function (_super) {
                __extends(GetContentTypeByNameRequest, _super);
                function GetContentTypeByNameRequest(name) {
                    _super.call(this);
                    this.inlineMixinsToFormItems = true;
                    _super.prototype.setMethod.call(this, "GET");
                    this.name = name;
                }
                GetContentTypeByNameRequest.prototype.getParams = function () {
                    return {
                        name: this.name.toString(),
                        inlineMixinsToFormItems: this.inlineMixinsToFormItems
                    };
                };
                GetContentTypeByNameRequest.prototype.getRequestPath = function () {
                    return _super.prototype.getResourcePath.call(this);
                };
                GetContentTypeByNameRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    var contentTypeCache = content.ContentTypeCache.get();
                    var contentType = contentTypeCache.getByKey(this.name);
                    if (contentType) {
                        return wemQ(contentType);
                    }
                    else {
                        return this.send().then(function (response) {
                            contentType = _this.fromJsonToContentType(response.getResult());
                            contentTypeCache.put(contentType);
                            return contentType;
                        });
                    }
                };
                return GetContentTypeByNameRequest;
            })(content.ContentTypeResourceRequest);
            content.GetContentTypeByNameRequest = GetContentTypeByNameRequest;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var GetContentTypesByApplicationRequest = (function (_super) {
                __extends(GetContentTypesByApplicationRequest, _super);
                function GetContentTypesByApplicationRequest(applicationKey) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.applicationKey = applicationKey;
                }
                GetContentTypesByApplicationRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "byApplication");
                };
                GetContentTypesByApplicationRequest.prototype.getParams = function () {
                    return {
                        applicationKey: this.applicationKey.toString()
                    };
                };
                GetContentTypesByApplicationRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().contentTypes.map(function (contentTypeJson) {
                            return _this.fromJsonToContentTypeSummary(contentTypeJson);
                        });
                    });
                };
                return GetContentTypesByApplicationRequest;
            })(content.ContentTypeResourceRequest);
            content.GetContentTypesByApplicationRequest = GetContentTypesByApplicationRequest;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var ApplicationUpdatedEvent = api.application.ApplicationUpdatedEvent;
            var ApplicationUpdatedEventType = api.application.ApplicationUpdatedEventType;
            var ContentTypeCache = (function (_super) {
                __extends(ContentTypeCache, _super);
                function ContentTypeCache() {
                    var _this = this;
                    _super.call(this);
                    ApplicationUpdatedEvent.on(function (event) {
                        if (4 /* STARTED */ == event.getEventType() || 7 /* STOPPED */ == event.getEventType() || 5 /* UPDATED */ == event.getEventType()) {
                            console.log(api.ClassHelper.getClassName(_this) + " received ApplicationUpdatedEvent - removing cached content types... " + event.getApplicationKey().toString());
                            _this.getCachedByApplicationKey(event.getApplicationKey()).forEach(function (contentType) {
                                _this.deleteByKey(_this.getKeyFromObject(contentType));
                                console.log("Removed cached content type: " + contentType.getName());
                            });
                        }
                    });
                }
                ContentTypeCache.prototype.copy = function (object) {
                    return new content.ContentTypeBuilder(object).build();
                };
                ContentTypeCache.prototype.getKeyFromObject = function (object) {
                    return new content.ContentTypeName(object.getName());
                };
                ContentTypeCache.prototype.getKeyAsString = function (key) {
                    return key.toString();
                };
                ContentTypeCache.prototype.getCachedByApplicationKey = function (applicationKey) {
                    var _this = this;
                    var result = [];
                    this.getAll().forEach(function (contentType) {
                        if (applicationKey.equals(_this.getKeyFromObject(contentType).getApplicationKey())) {
                            result.push(contentType);
                        }
                    });
                    return result;
                };
                ContentTypeCache.get = function () {
                    if (!ContentTypeCache.instance) {
                        ContentTypeCache.instance = new ContentTypeCache();
                    }
                    return ContentTypeCache.instance;
                };
                return ContentTypeCache;
            })(api.cache.Cache);
            content.ContentTypeCache = ContentTypeCache;
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var content;
        (function (content) {
            var inputtype;
            (function (inputtype) {
                var Value = api.data.Value;
                var ValueTypes = api.data.ValueTypes;
                var ContentTypeComboBox = api.schema.content.ContentTypeComboBox;
                var ContentTypeFilter = (function (_super) {
                    __extends(ContentTypeFilter, _super);
                    function ContentTypeFilter(context) {
                        _super.call(this, 'content-type-filter');
                        this.context = context;
                    }
                    ContentTypeFilter.prototype.getValueType = function () {
                        return ValueTypes.STRING;
                    };
                    ContentTypeFilter.prototype.newInitialValue = function () {
                        return null;
                    };
                    ContentTypeFilter.prototype.createPageTemplateLoader = function () {
                        var contentId = this.context.site.getContentId(), loader = new api.schema.content.PageTemplateContentTypeLoader(contentId);
                        loader.setComparator(new api.content.ContentSummaryByDisplayNameComparator());
                        return loader;
                    };
                    ContentTypeFilter.prototype.createComboBox = function () {
                        var _this = this;
                        var loader = this.context.formContext.getContentTypeName().isPageTemplate() ? this.createPageTemplateLoader() : null, comboBox = new ContentTypeComboBox(this.getInput().getOccurrences().getMaximum(), loader);
                        comboBox.onLoaded(function (contentTypeArray) { return _this.onContentTypesLoaded(contentTypeArray); });
                        comboBox.onOptionSelected(function (event) { return _this.onContentTypeSelected(event); });
                        comboBox.onOptionDeselected(function (option) { return _this.onContentTypeDeselected(option); });
                        return comboBox;
                    };
                    ContentTypeFilter.prototype.onContentTypesLoaded = function (contentTypeArray) {
                        var contentTypes = [];
                        this.getPropertyArray().forEach(function (property) {
                            contentTypes.push(property.getString());
                        });
                        this.combobox.getComboBox().setValues(contentTypes);
                        this.setLayoutInProgress(false);
                        this.combobox.unLoaded(this.onContentTypesLoaded);
                        this.validate(false);
                    };
                    ContentTypeFilter.prototype.onContentTypeSelected = function (event) {
                        if (this.isLayoutInProgress()) {
                            return;
                        }
                        var value = new Value(event.getOption().displayValue.getContentTypeName().toString(), ValueTypes.STRING);
                        if (this.combobox.countSelected() == 1) {
                            this.getPropertyArray().set(0, value);
                        }
                        else {
                            this.getPropertyArray().add(value);
                        }
                        this.validate(false);
                    };
                    ContentTypeFilter.prototype.onContentTypeDeselected = function (option) {
                        this.getPropertyArray().remove(option.getIndex());
                        this.validate(false);
                    };
                    ContentTypeFilter.prototype.layout = function (input, propertyArray) {
                        _super.prototype.layout.call(this, input, propertyArray);
                        this.appendChild(this.combobox = this.createComboBox());
                        return wemQ(null);
                    };
                    ContentTypeFilter.prototype.getValues = function () {
                        return this.combobox.getSelectedDisplayValues().map(function (contentType) {
                            return new Value(contentType.getContentTypeName().toString(), ValueTypes.STRING);
                        });
                    };
                    ContentTypeFilter.prototype.getNumberOfValids = function () {
                        return this.getValues().length;
                    };
                    ContentTypeFilter.prototype.giveFocus = function () {
                        return this.combobox.maximumOccurrencesReached() ? false : this.combobox.giveFocus();
                    };
                    ContentTypeFilter.prototype.onFocus = function (listener) {
                        this.combobox.onFocus(listener);
                    };
                    ContentTypeFilter.prototype.unFocus = function (listener) {
                        this.combobox.unFocus(listener);
                    };
                    ContentTypeFilter.prototype.onBlur = function (listener) {
                        this.combobox.onBlur(listener);
                    };
                    ContentTypeFilter.prototype.unBlur = function (listener) {
                        this.combobox.unBlur(listener);
                    };
                    return ContentTypeFilter;
                })(api.form.inputtype.support.BaseInputTypeManagingAdd);
                inputtype.ContentTypeFilter = ContentTypeFilter;
                api.form.inputtype.InputTypeManager.register(new api.Class("ContentTypeFilter", ContentTypeFilter));
            })(inputtype = content.inputtype || (content.inputtype = {}));
        })(content = schema.content || (schema.content = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var relationshiptype;
        (function (relationshiptype) {
            var ApplicationKey = api.application.ApplicationKey;
            var RelationshipTypeName = (function (_super) {
                __extends(RelationshipTypeName, _super);
                function RelationshipTypeName(name) {
                    api.util.assertNotNull(name, "RelationshipType name can't be null");
                    var parts = name.split(api.application.ApplicationBasedName.SEPARATOR);
                    _super.call(this, ApplicationKey.fromString(parts[0]), parts[1]);
                }
                RelationshipTypeName.REFERENCE = new RelationshipTypeName(ApplicationKey.SYSTEM + api.application.ApplicationBasedName.SEPARATOR + "reference");
                return RelationshipTypeName;
            })(api.application.ApplicationBasedName);
            relationshiptype.RelationshipTypeName = RelationshipTypeName;
        })(relationshiptype = schema.relationshiptype || (schema.relationshiptype = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var relationshiptype;
        (function (relationshiptype) {
            var RelationshipType = (function (_super) {
                __extends(RelationshipType, _super);
                function RelationshipType(builder) {
                    _super.call(this, builder);
                    this.fromSemantic = builder.fromSemantic;
                    this.toSemantic = builder.toSemantic;
                    this.allowedFromTypes = builder.allowedFromTypes;
                    this.allowedToTypes = builder.allowedToTypes;
                }
                RelationshipType.prototype.getRelationshiptypeName = function () {
                    return new relationshiptype.RelationshipTypeName(this.getName());
                };
                RelationshipType.prototype.getFromSemantic = function () {
                    return this.fromSemantic;
                };
                RelationshipType.prototype.getToSemantic = function () {
                    return this.toSemantic;
                };
                RelationshipType.prototype.getAllowedFromTypes = function () {
                    return this.allowedFromTypes;
                };
                RelationshipType.prototype.getAllowedToTypes = function () {
                    return this.allowedToTypes;
                };
                RelationshipType.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, RelationshipType)) {
                        return false;
                    }
                    if (!_super.prototype.equals.call(this, o)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.stringEquals(this.fromSemantic, other.fromSemantic)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringEquals(this.toSemantic, other.toSemantic)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringArrayEquals(this.allowedFromTypes, other.allowedFromTypes)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringArrayEquals(this.allowedToTypes, other.allowedToTypes)) {
                        return false;
                    }
                    return true;
                };
                RelationshipType.fromJson = function (json) {
                    return new RelationshipTypeBuilder().fromRelationshipTypeJson(json).build();
                };
                return RelationshipType;
            })(api.schema.Schema);
            relationshiptype.RelationshipType = RelationshipType;
            var RelationshipTypeBuilder = (function (_super) {
                __extends(RelationshipTypeBuilder, _super);
                function RelationshipTypeBuilder(source) {
                    if (source) {
                        _super.call(this, source);
                        this.fromSemantic = source.getFromSemantic();
                        this.toSemantic = source.getToSemantic();
                        this.allowedFromTypes = source.getAllowedFromTypes();
                        this.allowedToTypes = source.getAllowedToTypes();
                    }
                }
                RelationshipTypeBuilder.prototype.fromRelationshipTypeJson = function (relationshipTypeJson) {
                    _super.prototype.fromSchemaJson.call(this, relationshipTypeJson);
                    this.fromSemantic = relationshipTypeJson.fromSemantic;
                    this.toSemantic = relationshipTypeJson.toSemantic;
                    this.allowedFromTypes = relationshipTypeJson.allowedFromTypes;
                    this.allowedToTypes = relationshipTypeJson.allowedToTypes;
                    return this;
                };
                RelationshipTypeBuilder.prototype.build = function () {
                    return new RelationshipType(this);
                };
                return RelationshipTypeBuilder;
            })(api.schema.SchemaBuilder);
            relationshiptype.RelationshipTypeBuilder = RelationshipTypeBuilder;
        })(relationshiptype = schema.relationshiptype || (schema.relationshiptype = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var relationshiptype;
        (function (relationshiptype) {
            var RelationshipTypeResourceRequest = (function (_super) {
                __extends(RelationshipTypeResourceRequest, _super);
                function RelationshipTypeResourceRequest() {
                    _super.call(this);
                    this.resourceUrl = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "schema/relationship");
                }
                RelationshipTypeResourceRequest.prototype.getResourcePath = function () {
                    return this.resourceUrl;
                };
                RelationshipTypeResourceRequest.prototype.fromJsonToReleationshipType = function (json) {
                    return relationshiptype.RelationshipType.fromJson(json);
                };
                return RelationshipTypeResourceRequest;
            })(api.rest.ResourceRequest);
            relationshiptype.RelationshipTypeResourceRequest = RelationshipTypeResourceRequest;
        })(relationshiptype = schema.relationshiptype || (schema.relationshiptype = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var relationshiptype;
        (function (relationshiptype) {
            var GetRelationshipTypeByNameRequest = (function (_super) {
                __extends(GetRelationshipTypeByNameRequest, _super);
                function GetRelationshipTypeByNameRequest(name) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.name = name;
                }
                GetRelationshipTypeByNameRequest.prototype.getParams = function () {
                    return {
                        name: this.name.toString()
                    };
                };
                GetRelationshipTypeByNameRequest.prototype.getRequestPath = function () {
                    return _super.prototype.getResourcePath.call(this);
                };
                GetRelationshipTypeByNameRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return _this.fromJsonToReleationshipType(response.getResult());
                    });
                };
                return GetRelationshipTypeByNameRequest;
            })(relationshiptype.RelationshipTypeResourceRequest);
            relationshiptype.GetRelationshipTypeByNameRequest = GetRelationshipTypeByNameRequest;
        })(relationshiptype = schema.relationshiptype || (schema.relationshiptype = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var schema;
    (function (schema) {
        var relationshiptype;
        (function (relationshiptype) {
            var GetRelationshipTypesByApplicationRequest = (function (_super) {
                __extends(GetRelationshipTypesByApplicationRequest, _super);
                function GetRelationshipTypesByApplicationRequest(applicationKey) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.applicationKey = applicationKey;
                }
                GetRelationshipTypesByApplicationRequest.prototype.getParams = function () {
                    return {
                        applicationKey: this.applicationKey.toString()
                    };
                };
                GetRelationshipTypesByApplicationRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "byApplication");
                };
                GetRelationshipTypesByApplicationRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().relationshipTypes.map(function (json) { return _this.fromJsonToReleationshipType(json); });
                    });
                };
                return GetRelationshipTypesByApplicationRequest;
            })(relationshiptype.RelationshipTypeResourceRequest);
            relationshiptype.GetRelationshipTypesByApplicationRequest = GetRelationshipTypesByApplicationRequest;
        })(relationshiptype = schema.relationshiptype || (schema.relationshiptype = {}));
    })(schema = api.schema || (api.schema = {}));
})(api || (api = {}));
var api;
(function (api) {
    var thumb;
    (function (thumb) {
        var Thumbnail = (function () {
            function Thumbnail(builder) {
                this.binaryReference = builder.binaryReference;
                this.mimeType = builder.mimeType;
                this.size = builder.size;
            }
            Thumbnail.prototype.getBinaryReference = function () {
                return this.binaryReference;
            };
            Thumbnail.prototype.getMimeType = function () {
                return this.mimeType;
            };
            Thumbnail.prototype.getSize = function () {
                return this.size;
            };
            Thumbnail.prototype.toJson = function () {
                return {
                    "binaryReference": this.getBinaryReference().toString(),
                    "mimeType": this.getMimeType(),
                    "size": this.getSize()
                };
            };
            Thumbnail.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Thumbnail)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.binaryReference, other.binaryReference)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.mimeType, other.mimeType)) {
                    return false;
                }
                if (!api.ObjectHelper.numberEquals(this.size, other.size)) {
                    return false;
                }
                return true;
            };
            Thumbnail.create = function () {
                return new ThumbnailBuilder();
            };
            return Thumbnail;
        })();
        thumb.Thumbnail = Thumbnail;
        var ThumbnailBuilder = (function () {
            function ThumbnailBuilder() {
            }
            ThumbnailBuilder.prototype.fromJson = function (json) {
                this.binaryReference = new api.util.BinaryReference(json.binaryReference);
                this.mimeType = json.mimeType;
                this.size = json.size;
                return this;
            };
            ThumbnailBuilder.prototype.setBinaryReference = function (value) {
                this.binaryReference = value;
                return this;
            };
            ThumbnailBuilder.prototype.setMimeType = function (value) {
                this.mimeType = value;
                return this;
            };
            ThumbnailBuilder.prototype.setSize = function (value) {
                this.size = value;
                return this;
            };
            ThumbnailBuilder.prototype.build = function () {
                return new Thumbnail(this);
            };
            return ThumbnailBuilder;
        })();
        thumb.ThumbnailBuilder = ThumbnailBuilder;
    })(thumb = api.thumb || (api.thumb = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentId = (function () {
            function ContentId(value) {
                if (!ContentId.isValidContentId(value)) {
                    throw new Error("Invalid content id: " + value);
                }
                this.value = value;
            }
            ContentId.prototype.toString = function () {
                return this.value;
            };
            ContentId.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentId)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.value, other.value)) {
                    return false;
                }
                return true;
            };
            ContentId.isValidContentId = function (id) {
                return !api.util.StringHelper.isEmpty(id) && !api.util.StringHelper.isBlank(id);
            };
            ContentId.fromReference = function (reference) {
                return new ContentId(reference.getNodeId());
            };
            return ContentId;
        })();
        content.ContentId = ContentId;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var ContentIds = (function () {
            function ContentIds(array) {
                var _this = this;
                this.array = [];
                array.forEach(function (contentId) {
                    var duplicate = _this.array.some(function (possibleDuplicate) {
                        return contentId.equals(possibleDuplicate);
                    });
                    if (!duplicate) {
                        _this.array.push(contentId);
                    }
                    else {
                        throw Error("ContentIds do not allow duplicates, found: '" + contentId.toString() + "'");
                    }
                });
            }
            ContentIds.prototype.map = function (callbackfn) {
                return this.array.map(function (value, index) {
                    return callbackfn(value, index);
                });
            };
            ContentIds.from = function (contentIds) {
                return ContentIds.create().fromContentIds(contentIds).build();
            };
            ContentIds.fromContents = function (contents) {
                var builder = ContentIds.create();
                contents.forEach(function (content) {
                    builder.addContentId(content.getContentId());
                });
                return builder.build();
            };
            ContentIds.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentIds)) {
                    return false;
                }
                var other = o;
                return api.ObjectHelper.arrayEquals(this.array, other.array);
            };
            ContentIds.create = function () {
                return new ContentIdsBuilder();
            };
            return ContentIds;
        })();
        _content.ContentIds = ContentIds;
        var ContentIdsBuilder = (function () {
            function ContentIdsBuilder() {
                this.array = [];
            }
            ContentIdsBuilder.prototype.fromStrings = function (values) {
                var _this = this;
                if (!!values) {
                    values.forEach(function (value) {
                        _this.addContentId(new _content.ContentId(value));
                    });
                }
                return this;
            };
            ContentIdsBuilder.prototype.fromContentIds = function (contentIds) {
                var _this = this;
                if (!!contentIds) {
                    contentIds.forEach(function (contentId) { return _this.addContentId(contentId); });
                }
                return this;
            };
            ContentIdsBuilder.prototype.addContentId = function (value) {
                this.array.push(value);
                return this;
            };
            ContentIdsBuilder.prototype.build = function () {
                return new ContentIds(this.array);
            };
            return ContentIdsBuilder;
        })();
        _content.ContentIdsBuilder = ContentIdsBuilder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentName = (function (_super) {
            __extends(ContentName, _super);
            function ContentName(name) {
                _super.call(this, name);
            }
            ContentName.prototype.isUnnamed = function () {
                return false;
            };
            ContentName.prototype.toUnnamed = function () {
                api.util.assert(api.ObjectHelper.iFrameSafeInstanceOf(this, content.ContentUnnamed), "this is not a ContentUnnamed");
                return this;
            };
            ContentName.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentName)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                return true;
            };
            ContentName.fromString = function (str) {
                api.util.assert(str != null, "name cannot be null");
                if (str.indexOf(ContentName.UNNAMED_PREFIX) == 0) {
                    return new content.ContentUnnamed(str);
                }
                else {
                    return new ContentName(str);
                }
            };
            ContentName.UNNAMED_PREFIX = "__unnamed__";
            return ContentName;
        })(api.Name);
        content.ContentName = ContentName;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentUnnamed = (function (_super) {
            __extends(ContentUnnamed, _super);
            function ContentUnnamed(name) {
                _super.call(this, name);
                api.util.assert(name.indexOf(content.ContentName.UNNAMED_PREFIX) == 0, "An UnnamedContent must start with [" + content.ContentName.UNNAMED_PREFIX + "]: " + name);
            }
            ContentUnnamed.prototype.isUnnamed = function () {
                return true;
            };
            ContentUnnamed.prototype.toString = function () {
                return "";
            };
            ContentUnnamed.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentUnnamed)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                return true;
            };
            ContentUnnamed.prototype.toStringIncludingHidden = function () {
                return _super.prototype.toString.call(this);
            };
            ContentUnnamed.newUnnamed = function () {
                return new ContentUnnamed(content.ContentName.UNNAMED_PREFIX);
            };
            ContentUnnamed.PRETTY_UNNAMED = "unnamed";
            return ContentUnnamed;
        })(content.ContentName);
        content.ContentUnnamed = ContentUnnamed;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentPath = (function () {
            function ContentPath(elements) {
                this.elements = elements;
                if (elements.length == 0) {
                    this.refString = ContentPath.ELEMENT_DIVIDER;
                }
                else {
                    this.refString = ContentPath.ELEMENT_DIVIDER + this.elements.join(ContentPath.ELEMENT_DIVIDER);
                }
            }
            ContentPath.fromParent = function (parent, name) {
                var elements = parent.elements;
                elements.push(name);
                return new ContentPath(elements);
            };
            ContentPath.fromString = function (path) {
                var elements;
                if (path.indexOf("/") == 0 && path.length > 1) {
                    path = path.substr(1);
                    elements = path.split(ContentPath.ELEMENT_DIVIDER);
                }
                else if (path == "/") {
                    elements = [];
                }
                return new ContentPath(elements);
            };
            ContentPath.prototype.getElements = function () {
                return this.elements;
            };
            ContentPath.prototype.getName = function () {
                return this.elements[this.elements.length - 1];
            };
            ContentPath.prototype.getLevel = function () {
                return this.elements.length;
            };
            ContentPath.prototype.hasParentContent = function () {
                return this.elements.length > 1;
            };
            ContentPath.prototype.getLastElement = function () {
                return (this.elements[this.elements.length - 1] || "");
            };
            ContentPath.prototype.getParentPath = function () {
                var _this = this;
                if (this.elements.length < 1) {
                    return null;
                }
                var parentElements = [];
                this.elements.forEach(function (element, index) {
                    if (index < _this.elements.length - 1) {
                        parentElements.push(element);
                    }
                });
                return new ContentPath(parentElements);
            };
            ContentPath.prototype.isRoot = function () {
                return this.equals(ContentPath.ROOT);
            };
            ContentPath.prototype.isNotRoot = function () {
                return !this.equals(ContentPath.ROOT);
            };
            ContentPath.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentPath)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                    return false;
                }
                return true;
            };
            ContentPath.prototype.isDescendantOf = function (path) {
                return (this.refString.indexOf(path.toString()) === 0) && (this.getLevel() > path.getLevel());
            };
            ContentPath.prototype.isChildOf = function (path) {
                return (this.refString.indexOf(path.toString()) === 0) && (this.getLevel() === path.getLevel() + 1);
            };
            ContentPath.prototype.prettifyUnnamedPathElements = function () {
                var prettyElements = [];
                this.elements.forEach(function (element) {
                    if (content.ContentName.fromString(element).isUnnamed()) {
                        prettyElements.push("<" + content.ContentUnnamed.PRETTY_UNNAMED + ">");
                    }
                    else {
                        prettyElements.push(element);
                    }
                });
                return new ContentPath(prettyElements);
            };
            ContentPath.prototype.toString = function () {
                return this.refString;
            };
            ContentPath.ELEMENT_DIVIDER = "/";
            ContentPath.ROOT = ContentPath.fromString("/");
            return ContentPath;
        })();
        content.ContentPath = ContentPath;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentIdBaseItem = (function () {
            function ContentIdBaseItem(builder) {
                this.contentId = builder.contentId;
            }
            ContentIdBaseItem.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentIdBaseItem.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, content.ContentSummary)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.contentId, other.contentId)) {
                    return false;
                }
                return true;
            };
            ContentIdBaseItem.fromJson = function (json) {
                return new ContentIdBaseItemBuilder().fromContentIdBaseItemJson(json).build();
            };
            ContentIdBaseItem.fromJsonArray = function (jsonArray) {
                var array = [];
                jsonArray.forEach(function (json) {
                    array.push(ContentIdBaseItem.fromJson(json));
                });
                return array;
            };
            return ContentIdBaseItem;
        })();
        content.ContentIdBaseItem = ContentIdBaseItem;
        var ContentIdBaseItemBuilder = (function () {
            function ContentIdBaseItemBuilder(source) {
                if (source) {
                    this.contentId = source.getContentId();
                }
            }
            ContentIdBaseItemBuilder.prototype.fromContentIdBaseItemJson = function (json) {
                this.contentId = new content.ContentId(json.id);
                return this;
            };
            ContentIdBaseItemBuilder.prototype.build = function () {
                return new ContentIdBaseItem(this);
            };
            return ContentIdBaseItemBuilder;
        })();
        content.ContentIdBaseItemBuilder = ContentIdBaseItemBuilder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var Thumbnail = api.thumb.Thumbnail;
        var ContentState = api.schema.content.ContentState;
        var ContentSummary = (function (_super) {
            __extends(ContentSummary, _super);
            function ContentSummary(builder) {
                _super.call(this, builder);
                this.name = builder.name;
                this.displayName = builder.displayName;
                this.path = builder.path;
                this.root = builder.root;
                this.children = builder.children;
                this.type = builder.type;
                this.iconUrl = builder.iconUrl;
                this.thumbnail = builder.thumbnail;
                this.modifier = builder.modifier;
                this.owner = builder.owner;
                this.page = builder.page;
                this.valid = builder.valid;
                this.requireValid = builder.requireValid;
                this.id = builder.id;
                this.createdTime = builder.createdTime;
                this.modifiedTime = builder.modifiedTime;
                this.deletable = builder.deletable;
                this.editable = builder.editable;
                this.childOrder = builder.childOrder;
                this.language = builder.language;
                this.contentState = builder.contentState;
            }
            ContentSummary.prototype.getName = function () {
                return this.name;
            };
            ContentSummary.prototype.getDisplayName = function () {
                return this.displayName;
            };
            ContentSummary.prototype.hasParent = function () {
                return this.path.hasParentContent();
            };
            ContentSummary.prototype.getPath = function () {
                return this.path;
            };
            ContentSummary.prototype.isRoot = function () {
                return this.root;
            };
            ContentSummary.prototype.hasChildren = function () {
                return this.children;
            };
            ContentSummary.prototype.getType = function () {
                return this.type;
            };
            ContentSummary.prototype.getIconUrl = function () {
                return this.iconUrl;
            };
            ContentSummary.prototype.hasThumbnail = function () {
                return !!this.thumbnail;
            };
            ContentSummary.prototype.getThumbnail = function () {
                return this.thumbnail;
            };
            ContentSummary.prototype.getOwner = function () {
                return this.owner;
            };
            ContentSummary.prototype.getModifier = function () {
                return this.modifier;
            };
            ContentSummary.prototype.isSite = function () {
                return this.type.isSite();
            };
            ContentSummary.prototype.isPage = function () {
                return this.page;
            };
            ContentSummary.prototype.isPageTemplate = function () {
                return this.type.isPageTemplate();
            };
            ContentSummary.prototype.isImage = function () {
                return this.type.isImage();
            };
            ContentSummary.prototype.isValid = function () {
                return this.valid;
            };
            ContentSummary.prototype.isRequireValid = function () {
                return this.requireValid;
            };
            ContentSummary.prototype.getId = function () {
                return this.id;
            };
            ContentSummary.prototype.getCreatedTime = function () {
                return this.createdTime;
            };
            ContentSummary.prototype.getModifiedTime = function () {
                return this.modifiedTime;
            };
            ContentSummary.prototype.isDeletable = function () {
                return this.deletable;
            };
            ContentSummary.prototype.isEditable = function () {
                return this.editable;
            };
            ContentSummary.prototype.getChildOrder = function () {
                return this.childOrder;
            };
            ContentSummary.prototype.getLanguage = function () {
                return this.language;
            };
            ContentSummary.prototype.getContentState = function () {
                return this.contentState;
            };
            ContentSummary.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentSummary)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.id, other.id)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.name, other.name)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.displayName, other.displayName)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.path, other.path)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.root, other.root)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.children, other.children)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.type, other.type)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.iconUrl, other.iconUrl)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.thumbnail, other.thumbnail)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.modifier, other.modifier)) {
                    return false;
                }
                if (!api.ObjectHelper.objectEquals(this.owner, other.owner)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.page, other.page)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.valid, other.valid)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.requireValid, other.requireValid)) {
                    return false;
                }
                if (!api.ObjectHelper.dateEquals(this.createdTime, other.createdTime)) {
                    return false;
                }
                if (!api.ObjectHelper.dateEquals(this.modifiedTime, other.modifiedTime)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.deletable, other.deletable)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.editable, other.editable)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.language, other.language)) {
                    return false;
                }
                if (!api.ObjectHelper.objectEquals(this.contentState, other.contentState)) {
                    return false;
                }
                return true;
            };
            ContentSummary.fromJson = function (json) {
                return new ContentSummaryBuilder().fromContentSummaryJson(json).build();
            };
            ContentSummary.fromJsonArray = function (jsonArray) {
                var array = [];
                jsonArray.forEach(function (json) {
                    array.push(ContentSummary.fromJson(json));
                });
                return array;
            };
            return ContentSummary;
        })(content.ContentIdBaseItem);
        content.ContentSummary = ContentSummary;
        var ContentSummaryBuilder = (function (_super) {
            __extends(ContentSummaryBuilder, _super);
            function ContentSummaryBuilder(source) {
                _super.call(this, source);
                if (source) {
                    this.id = source.getId();
                    this.name = source.getName();
                    this.displayName = source.getDisplayName();
                    this.path = source.getPath();
                    this.root = source.isRoot();
                    this.children = source.hasChildren();
                    this.type = source.getType();
                    this.iconUrl = source.getIconUrl();
                    this.thumbnail = source.getThumbnail();
                    this.modifier = source.getModifier();
                    this.owner = source.getOwner();
                    this.page = source.isPage();
                    this.valid = source.isValid();
                    this.requireValid = source.isRequireValid();
                    this.createdTime = source.getCreatedTime();
                    this.modifiedTime = source.getModifiedTime();
                    this.deletable = source.isDeletable();
                    this.editable = source.isEditable();
                    this.childOrder = source.getChildOrder();
                    this.language = source.getLanguage();
                    this.contentState = source.getContentState();
                }
            }
            ContentSummaryBuilder.prototype.fromContentSummaryJson = function (json) {
                _super.prototype.fromContentIdBaseItemJson.call(this, json);
                this.name = content.ContentName.fromString(json.name);
                this.displayName = json.displayName;
                this.path = content.ContentPath.fromString(json.path);
                this.root = json.isRoot;
                this.children = json.hasChildren;
                this.type = new api.schema.content.ContentTypeName(json.type);
                this.iconUrl = json.iconUrl;
                this.thumbnail = json.thumbnail ? Thumbnail.create().fromJson(json.thumbnail).build() : null;
                this.modifier = json.modifier;
                this.owner = json.owner ? api.security.PrincipalKey.fromString(json.owner) : null;
                this.page = json.isPage;
                this.valid = json.isValid;
                this.requireValid = json.requireValid;
                this.language = json.language;
                this.id = json.id;
                this.createdTime = json.createdTime ? new Date(Date.parse(json.createdTime)) : null;
                this.modifiedTime = json.modifiedTime ? new Date(Date.parse(json.modifiedTime)) : null;
                this.deletable = json.deletable;
                this.editable = json.editable;
                this.childOrder = content.ChildOrder.fromJson(json.childOrder);
                this.contentState = ContentState.fromString(json.contentState);
                return this;
            };
            ContentSummaryBuilder.prototype.setId = function (value) {
                this.id = value;
                return this;
            };
            ContentSummaryBuilder.prototype.setValid = function (value) {
                this.valid = value;
                return this;
            };
            ContentSummaryBuilder.prototype.setRequireValid = function (value) {
                this.valid = value;
                return this;
            };
            ContentSummaryBuilder.prototype.setName = function (value) {
                this.name = value;
                return this;
            };
            ContentSummaryBuilder.prototype.setPath = function (path) {
                this.path = path;
                return this;
            };
            ContentSummaryBuilder.prototype.setType = function (value) {
                this.type = value;
                return this;
            };
            ContentSummaryBuilder.prototype.setDisplayName = function (value) {
                this.displayName = value;
                return this;
            };
            ContentSummaryBuilder.prototype.setHasChildren = function (value) {
                this.children = value;
                return this;
            };
            ContentSummaryBuilder.prototype.setDeletable = function (value) {
                this.deletable = value;
                return this;
            };
            ContentSummaryBuilder.prototype.build = function () {
                return new ContentSummary(this);
            };
            return ContentSummaryBuilder;
        })(content.ContentIdBaseItemBuilder);
        content.ContentSummaryBuilder = ContentSummaryBuilder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentSummaryViewer = (function (_super) {
            __extends(ContentSummaryViewer, _super);
            function ContentSummaryViewer() {
                _super.call(this, "content-summary-viewer");
            }
            ContentSummaryViewer.prototype.resolveDisplayName = function (object) {
                var contentName = object.getName(), invalid = !object.isValid() || !object.getDisplayName() || contentName.isUnnamed(), pendingDelete = object.getContentState().isPendingDelete();
                this.toggleClass("invalid", invalid);
                this.toggleClass("pending-delete", pendingDelete);
                return object.getDisplayName();
            };
            ContentSummaryViewer.prototype.resolveUnnamedDisplayName = function (object) {
                return object.getType() ? object.getType().getLocalName() : "";
            };
            ContentSummaryViewer.prototype.resolveSubName = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                var contentName = object.getName();
                if (relativePath) {
                    return !contentName.isUnnamed() ? object.getName().toString() : api.ui.NamesAndIconViewer.EMPTY_SUB_NAME;
                }
                else {
                    return !contentName.isUnnamed() ? object.getPath().toString() : content.ContentPath.fromParent(object.getPath().getParentPath(), api.ui.NamesAndIconViewer.EMPTY_SUB_NAME).toString();
                }
            };
            ContentSummaryViewer.prototype.resolveSubTitle = function (object) {
                return object.getPath().toString();
            };
            ContentSummaryViewer.prototype.resolveIconUrl = function (object) {
                return new content.ContentIconUrlResolver().setContent(object).resolve();
            };
            return ContentSummaryViewer;
        })(api.ui.NamesAndIconViewer);
        content.ContentSummaryViewer = ContentSummaryViewer;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var QueryField = api.query.QueryField;
        var QueryExpr = api.query.expr.QueryExpr;
        var ContentSummaryRequest = (function (_super) {
            __extends(ContentSummaryRequest, _super);
            function ContentSummaryRequest() {
                _super.call(this);
                this.contentQuery = new content.query.ContentQuery();
                this.request = new content.ContentQueryRequest(this.contentQuery).setExpand(1 /* SUMMARY */);
            }
            ContentSummaryRequest.prototype.getRestPath = function () {
                return this.request.getRestPath();
            };
            ContentSummaryRequest.prototype.getRequestPath = function () {
                return this.request.getRequestPath();
            };
            ContentSummaryRequest.prototype.getParams = function () {
                return this.request.getParams();
            };
            ContentSummaryRequest.prototype.send = function () {
                return this.request.send();
            };
            ContentSummaryRequest.prototype.sendAndParse = function () {
                return this.request.sendAndParse().then(function (queryResult) {
                    return queryResult.getContents();
                });
            };
            ContentSummaryRequest.prototype.setAllowedContentTypes = function (contentTypes) {
                this.contentQuery.setContentTypeNames(this.createContentTypeNames(contentTypes));
            };
            ContentSummaryRequest.prototype.setAllowedContentTypeNames = function (contentTypeNames) {
                this.contentQuery.setContentTypeNames(contentTypeNames);
            };
            ContentSummaryRequest.prototype.setSize = function (size) {
                this.contentQuery.setSize(size);
            };
            ContentSummaryRequest.prototype.setQueryExpr = function (searchString, orderList) {
                var fulltextExpression = new api.query.FulltextSearchExpressionBuilder().setSearchString(searchString).addField(new QueryField(QueryField.DISPLAY_NAME, 5)).addField(new QueryField(QueryField.NAME, 3)).addField(new QueryField(QueryField.ALL)).build();
                var queryExpr = new QueryExpr(fulltextExpression, orderList);
                this.contentQuery.setQueryExpr(queryExpr);
            };
            ContentSummaryRequest.prototype.createContentTypeNames = function (names) {
                return (names || []).map(function (name) { return new api.schema.content.ContentTypeName(name); });
            };
            return ContentSummaryRequest;
        })(api.rest.ResourceRequest);
        content.ContentSummaryRequest = ContentSummaryRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var FieldOrderExpr = api.query.expr.FieldOrderExpr;
        var OrderDirection = api.query.expr.OrderDirection;
        var FieldExpr = api.query.expr.FieldExpr;
        var ContentSummaryLoader = (function (_super) {
            __extends(ContentSummaryLoader, _super);
            function ContentSummaryLoader() {
                this.contentSummaryRequest = new content.ContentSummaryRequest();
                this.order = ContentSummaryLoader.ORDER_BY_MODIFIED_TIME_DESC;
                _super.call(this, this.contentSummaryRequest);
            }
            ContentSummaryLoader.prototype.setOrder = function (orderList) {
                this.order = orderList;
            };
            ContentSummaryLoader.prototype.setAllowedContentTypes = function (contentTypes) {
                this.contentSummaryRequest.setAllowedContentTypes(contentTypes);
            };
            ContentSummaryLoader.prototype.setAllowedContentTypeNames = function (contentTypeNames) {
                this.contentSummaryRequest.setAllowedContentTypeNames(contentTypeNames);
            };
            ContentSummaryLoader.prototype.setSize = function (size) {
                this.contentSummaryRequest.setSize(size);
            };
            ContentSummaryLoader.prototype.search = function (searchString) {
                this.contentSummaryRequest.setQueryExpr(searchString, this.order);
                return this.load();
            };
            ContentSummaryLoader.prototype.sendRequest = function () {
                return this.contentSummaryRequest.sendAndParse();
            };
            ContentSummaryLoader.MODIFIED_TIME_DESC = new FieldOrderExpr(new FieldExpr("_modifiedTime"), 1 /* DESC */);
            ContentSummaryLoader.ORDER_BY_MODIFIED_TIME_DESC = [ContentSummaryLoader.MODIFIED_TIME_DESC];
            return ContentSummaryLoader;
        })(api.util.loader.BaseLoader);
        content.ContentSummaryLoader = ContentSummaryLoader;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentIconUrlResolver = (function (_super) {
            __extends(ContentIconUrlResolver, _super);
            function ContentIconUrlResolver() {
                _super.apply(this, arguments);
            }
            ContentIconUrlResolver.prototype.setContent = function (value) {
                this.content = value;
                return this;
            };
            ContentIconUrlResolver.prototype.resolve = function () {
                var url = this.content.getIconUrl();
                if (!url) {
                    return null;
                }
                if (this.crop == undefined) {
                    this.crop = !this.content.isImage();
                }
                url = this.appendParam("crop", this.crop.toString(), url);
                return url;
            };
            ContentIconUrlResolver.default = function () {
                return api.util.UriHelper.getAdminUri("common/images/default_content.png");
            };
            return ContentIconUrlResolver;
        })(api.icon.IconUrlResolver);
        content.ContentIconUrlResolver = ContentIconUrlResolver;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentImageUrlResolver = (function (_super) {
            __extends(ContentImageUrlResolver, _super);
            function ContentImageUrlResolver() {
                _super.apply(this, arguments);
                this.size = "";
                this.ts = null;
                this.scaleWidth = null;
                this.source = null;
            }
            ContentImageUrlResolver.prototype.setContentId = function (value) {
                this.contentId = value;
                return this;
            };
            ContentImageUrlResolver.prototype.setSize = function (value) {
                this.size = "" + value;
                return this;
            };
            ContentImageUrlResolver.prototype.setTimestamp = function (value) {
                this.ts = "" + value.getTime();
                return this;
            };
            ContentImageUrlResolver.prototype.setScaleWidth = function (value) {
                this.scaleWidth = "" + value;
                return this;
            };
            ContentImageUrlResolver.prototype.setSource = function (value) {
                this.source = "" + value;
                return this;
            };
            ContentImageUrlResolver.prototype.resolve = function () {
                var url = "content/image/" + this.contentId.toString();
                if (this.size.length > 0) {
                    url = this.appendParam("size", this.size, url);
                }
                if (this.ts) {
                    url = this.appendParam("ts", this.ts, url);
                }
                if (this.scaleWidth) {
                    url = this.appendParam("scaleWidth", this.scaleWidth, url);
                }
                if (this.source) {
                    url = this.appendParam("source", this.source, url);
                }
                return api.util.UriHelper.getRestUri(url);
            };
            return ContentImageUrlResolver;
        })(api.icon.IconUrlResolver);
        content.ContentImageUrlResolver = ContentImageUrlResolver;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentMetadata = (function () {
            function ContentMetadata(hits, totalHits) {
                this.hits = hits;
                this.totalHits = totalHits;
            }
            ContentMetadata.prototype.getHits = function () {
                return this.hits;
            };
            ContentMetadata.prototype.getTotalHits = function () {
                return this.totalHits;
            };
            ContentMetadata.prototype.setHits = function (hits) {
                this.hits = hits;
            };
            ContentMetadata.prototype.setTotalHits = function (totalHits) {
                this.totalHits = totalHits;
            };
            ContentMetadata.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentMetadata)) {
                    return false;
                }
                var other = o;
                if (this.hits !== other.hits || this.totalHits !== other.totalHits) {
                    return false;
                }
                return true;
            };
            return ContentMetadata;
        })();
        content.ContentMetadata = ContentMetadata;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var AccessControlList = api.security.acl.AccessControlList;
        var PropertyTree = api.data.PropertyTree;
        var Content = (function (_super) {
            __extends(Content, _super);
            function Content(builder) {
                _super.call(this, builder);
                this.extraData = [];
                api.util.assertNotNull(builder.data, "data is required for Content");
                this.data = builder.data;
                this.attachments = builder.attachments;
                this.extraData = builder.extraData || [];
                this.pageObj = builder.pageObj;
                this.permissions = builder.permissions || new AccessControlList();
                this.inheritPermissions = builder.inheritPermissions;
            }
            Content.prototype.getContentData = function () {
                return this.data;
            };
            Content.prototype.getAttachments = function () {
                return this.attachments;
            };
            Content.prototype.getExtraData = function (name) {
                return this.extraData.filter(function (item) { return item.getName().equals(name); })[0];
            };
            Content.prototype.getAllExtraData = function () {
                return this.extraData;
            };
            Content.prototype.getProperty = function (propertyName) {
                return !!propertyName ? this.data.getProperty(propertyName) : null;
            };
            Content.prototype.getPage = function () {
                return this.pageObj;
            };
            Content.prototype.getPermissions = function () {
                return this.permissions;
            };
            Content.prototype.isInheritPermissionsEnabled = function () {
                return this.inheritPermissions;
            };
            Content.prototype.trimPropertyTree = function (data) {
                var copy = data.copy();
                copy.getRoot().removeEmptyValues();
                return copy;
            };
            Content.prototype.trimExtraData = function (extraData) {
                var copy = extraData.clone();
                copy.getData().getRoot().removeEmptyValues();
                return copy;
            };
            Content.prototype.extraDataEqualsIgnoreEmpty = function (extraData, otherMeta) {
                var _this = this;
                extraData = extraData.map(function (m) { return _this.trimExtraData(m); }).filter(function (m) { return !m.getData().isEmpty(); });
                otherMeta = otherMeta.map(function (m) { return _this.trimExtraData(m); }).filter(function (m) { return !m.getData().isEmpty(); });
                var comparator = new api.content.ExtraDataByMixinNameComparator();
                return api.ObjectHelper.arrayEquals(extraData.sort(comparator.compare), otherMeta.sort(comparator.compare));
            };
            Content.prototype.equals = function (o, ignoreEmptyValues) {
                if (ignoreEmptyValues === void 0) { ignoreEmptyValues = false; }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Content)) {
                    return false;
                }
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                var other = o;
                if (ignoreEmptyValues) {
                    if (!api.ObjectHelper.equals(this.trimPropertyTree(this.data), this.trimPropertyTree(other.data))) {
                        return false;
                    }
                }
                else {
                    if (!api.ObjectHelper.equals(this.data, other.data)) {
                        return false;
                    }
                }
                if (ignoreEmptyValues) {
                    if (!this.extraDataEqualsIgnoreEmpty(this.extraData, other.extraData)) {
                        return false;
                    }
                }
                else {
                    var comparator = new api.content.ExtraDataByMixinNameComparator();
                    if (!api.ObjectHelper.arrayEquals(this.extraData.sort(comparator.compare), other.extraData.sort(comparator.compare))) {
                        return false;
                    }
                }
                if (!api.ObjectHelper.equals(this.pageObj, other.pageObj)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.permissions, other.permissions)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.attachments, other.attachments)) {
                    return false;
                }
                if (this.inheritPermissions !== other.inheritPermissions) {
                    return false;
                }
                return true;
            };
            Content.prototype.clone = function () {
                return this.newBuilder().build();
            };
            Content.prototype.newBuilder = function () {
                return new ContentBuilder(this);
            };
            Content.fromJson = function (json) {
                var type = new api.schema.content.ContentTypeName(json.type);
                if (type.isSite()) {
                    return new content.site.SiteBuilder().fromContentJson(json).build();
                }
                else if (type.isPageTemplate()) {
                    return new content.page.PageTemplateBuilder().fromContentJson(json).build();
                }
                return new ContentBuilder().fromContentJson(json).build();
            };
            return Content;
        })(content.ContentSummary);
        content.Content = Content;
        var ContentBuilder = (function (_super) {
            __extends(ContentBuilder, _super);
            function ContentBuilder(source) {
                _super.call(this, source);
                this.inheritPermissions = true;
                if (source) {
                    this.data = source.getContentData() ? source.getContentData().copy() : null;
                    this.attachments = source.getAttachments();
                    this.extraData = source.getAllExtraData() ? source.getAllExtraData().map(function (extraData) { return extraData.clone(); }) : [];
                    this.pageObj = source.getPage() ? source.getPage().clone() : null;
                    this.permissions = source.getPermissions();
                    this.inheritPermissions = source.isInheritPermissionsEnabled();
                }
            }
            ContentBuilder.prototype.fromContentJson = function (json) {
                var _this = this;
                _super.prototype.fromContentSummaryJson.call(this, json);
                this.data = PropertyTree.fromJson(json.data);
                this.attachments = new api.content.attachment.AttachmentsBuilder().fromJson(json.attachments).build();
                this.extraData = [];
                json.meta.forEach(function (extraDataJson) {
                    _this.extraData.push(content.ExtraData.fromJson(extraDataJson));
                });
                if (this.page) {
                    this.pageObj = new api.content.page.PageBuilder().fromJson(json.page).build();
                    this.page = true;
                }
                if (json.permissions) {
                    this.permissions = AccessControlList.fromJson(json.permissions);
                }
                if (typeof json.inheritPermissions !== "undefined") {
                    this.inheritPermissions = json.inheritPermissions;
                }
                return this;
            };
            ContentBuilder.prototype.setData = function (value) {
                this.data = value;
                return this;
            };
            ContentBuilder.prototype.setAttachments = function (value) {
                this.attachments = value;
                return this;
            };
            ContentBuilder.prototype.setPage = function (value) {
                this.pageObj = value;
                this.page = value ? true : false;
                return this;
            };
            ContentBuilder.prototype.setExtraData = function (extraData) {
                this.extraData = extraData;
                return this;
            };
            ContentBuilder.prototype.setPermissions = function (value) {
                this.permissions = value;
                return this;
            };
            ContentBuilder.prototype.setInheritPermissionsEnabled = function (value) {
                this.inheritPermissions = value;
                return this;
            };
            ContentBuilder.prototype.build = function () {
                return new Content(this);
            };
            return ContentBuilder;
        })(content.ContentSummaryBuilder);
        content.ContentBuilder = ContentBuilder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentResponse = (function () {
            function ContentResponse(contents, metadata) {
                this.contents = contents;
                this.metadata = metadata;
            }
            ContentResponse.prototype.getContents = function () {
                return this.contents;
            };
            ContentResponse.prototype.getMetadata = function () {
                return this.metadata;
            };
            ContentResponse.prototype.setContents = function (contents) {
                this.contents = contents;
                return this;
            };
            ContentResponse.prototype.setMetadata = function (metadata) {
                this.metadata = metadata;
                return this;
            };
            return ContentResponse;
        })();
        content.ContentResponse = ContentResponse;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentNodeByDisplayNameComparator = (function () {
            function ContentNodeByDisplayNameComparator() {
            }
            ContentNodeByDisplayNameComparator.prototype.compare = function (a, b) {
                if (!a.getData().getContentSummary()) {
                    return 1;
                }
                else {
                    var firstName = a.getData().getContentSummary().getDisplayName();
                }
                if (!b.getData().getContentSummary()) {
                    return -1;
                }
                else {
                    var secondName = b.getData().getContentSummary().getDisplayName();
                }
                return firstName.localeCompare(secondName);
            };
            return ContentNodeByDisplayNameComparator;
        })();
        content.ContentNodeByDisplayNameComparator = ContentNodeByDisplayNameComparator;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentSummaryByDisplayNameComparator = (function () {
            function ContentSummaryByDisplayNameComparator() {
            }
            ContentSummaryByDisplayNameComparator.prototype.compare = function (item1, item2) {
                if (item1.getDisplayName().toLowerCase() > item2.getDisplayName().toLowerCase()) {
                    return 1;
                }
                else if (item1.getDisplayName().toLowerCase() < item2.getDisplayName().toLowerCase()) {
                    return -1;
                }
                else if (item1.getName() > item2.getName()) {
                    return 1;
                }
                else if (item1.getName() < item2.getName()) {
                    return -1;
                }
                else {
                    return 0;
                }
            };
            return ContentSummaryByDisplayNameComparator;
        })();
        content.ContentSummaryByDisplayNameComparator = ContentSummaryByDisplayNameComparator;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentNodeByModifiedTimeComparator = (function () {
            function ContentNodeByModifiedTimeComparator() {
            }
            ContentNodeByModifiedTimeComparator.prototype.compare = function (a, b) {
                var firstDate = !a.getData().getContentSummary() ? null : a.getData().getContentSummary().getModifiedTime(), secondDate = !b.getData().getContentSummary() ? null : b.getData().getContentSummary().getModifiedTime();
                return firstDate < secondDate ? 1 : (firstDate > secondDate) ? -1 : 0;
            };
            return ContentNodeByModifiedTimeComparator;
        })();
        content.ContentNodeByModifiedTimeComparator = ContentNodeByModifiedTimeComparator;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentByPathComparator = (function () {
            function ContentByPathComparator() {
            }
            ContentByPathComparator.prototype.compare = function (a, b) {
                if (!a) {
                    return 1;
                }
                else {
                    var firstName = a.getPath().toString();
                }
                if (!b) {
                    return -1;
                }
                else {
                    var secondName = b.getPath().toString();
                }
                return firstName.localeCompare(secondName);
            };
            return ContentByPathComparator;
        })();
        content.ContentByPathComparator = ContentByPathComparator;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var GetContentTypeByNameRequest = api.schema.content.GetContentTypeByNameRequest;
        var MoveContentSummaryLoader = (function (_super) {
            __extends(MoveContentSummaryLoader, _super);
            function MoveContentSummaryLoader() {
                this.contentSummaryRequest = new _content.ContentSummaryRequest();
                _super.call(this, this.contentSummaryRequest);
            }
            MoveContentSummaryLoader.prototype.setSize = function (size) {
                this.contentSummaryRequest.setSize(size);
            };
            MoveContentSummaryLoader.prototype.setFilterContentPath = function (filterContentPath) {
                this.filterContentPath = filterContentPath;
            };
            MoveContentSummaryLoader.prototype.setFilterSourceContentType = function (filterSourceContentType) {
                this.filterSourceContentType = filterSourceContentType;
            };
            MoveContentSummaryLoader.prototype.search = function (searchString) {
                this.contentSummaryRequest.setQueryExpr(searchString);
                return this.load();
            };
            MoveContentSummaryLoader.prototype.load = function () {
                var _this = this;
                this.notifyLoadingData();
                return this.sendRequest().then(function (contents) {
                    var deferred = wemQ.defer();
                    var allContentTypes = contents.map(function (content) { return content.getType(); });
                    var contentTypes = api.util.ArrayHelper.removeDuplicates(allContentTypes, function (ct) { return ct.toString(); });
                    var contentTypeRequests = contentTypes.map(function (contentType) { return new GetContentTypeByNameRequest(contentType).sendAndParse(); });
                    wemQ.all(contentTypeRequests).spread(function () {
                        var contentTypes = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            contentTypes[_i - 0] = arguments[_i];
                        }
                        if (_this.filterContentPath) {
                            contents = _this.filterContent(contents, contentTypes);
                        }
                        if (contents && contents.length > 0) {
                            contents.sort(new _content.ContentByPathComparator().compare);
                            _this.notifyLoadedData(contents);
                        }
                        else {
                            _this.notifyLoadedData([]);
                        }
                        if (_this.preservedSearchString) {
                            _this.search(_this.preservedSearchString);
                            _this.preservedSearchString = null;
                        }
                        deferred.resolve(contents);
                    }).catch(function (reason) { return deferred.reject(reason); }).done();
                    return deferred.promise;
                });
            };
            MoveContentSummaryLoader.prototype.filterContent = function (contents, contentTypes) {
                var _this = this;
                var contentTypeAllowsChild = {};
                contentTypes.forEach(function (contentType) { return contentTypeAllowsChild[contentType.getName()] = contentType.isAllowChildContent(); });
                var createContentFilter = new api.content.CreateContentFilter();
                return contents.filter(function (content) {
                    return !content.getPath().isDescendantOf(_this.filterContentPath) && !_this.filterContentPath.isChildOf(content.getPath()) && !_this.filterContentPath.equals(content.getPath()) && contentTypeAllowsChild[content.getType().toString()] && createContentFilter.isCreateContentAllowed(content, _this.filterSourceContentType);
                });
            };
            MoveContentSummaryLoader.prototype.sendRequest = function () {
                return this.contentSummaryRequest.sendAndParse();
            };
            return MoveContentSummaryLoader;
        })(api.util.loader.BaseLoader);
        _content.MoveContentSummaryLoader = MoveContentSummaryLoader;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var MixinName = api.schema.mixin.MixinName;
        var PropertyTree = api.data.PropertyTree;
        var ExtraData = (function () {
            function ExtraData(name, data) {
                this.name = name;
                this.data = data;
            }
            ExtraData.prototype.getName = function () {
                return this.name;
            };
            ExtraData.prototype.getData = function () {
                return this.data;
            };
            ExtraData.prototype.clone = function () {
                return new ExtraData(this.name, this.data.copy());
            };
            ExtraData.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ExtraData)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.equals(this.name, other.name)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.data, other.data)) {
                    return false;
                }
                return true;
            };
            ExtraData.prototype.toJson = function () {
                return {
                    name: this.name.toString(),
                    data: this.data.toJson()
                };
            };
            ExtraData.fromJson = function (metadataJson) {
                return new ExtraData(new MixinName(metadataJson.name), PropertyTree.fromJson(metadataJson.data));
            };
            return ExtraData;
        })();
        content.ExtraData = ExtraData;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ExtraDataByMixinNameComparator = (function () {
            function ExtraDataByMixinNameComparator() {
            }
            ExtraDataByMixinNameComparator.prototype.compare = function (a, b) {
                if (!a.getName()) {
                    return 1;
                }
                else {
                    var firstName = a.getName().getLocalName();
                }
                if (!b.getName()) {
                    return -1;
                }
                else {
                    var secondName = b.getName().getLocalName();
                }
                return firstName.localeCompare(secondName);
            };
            return ExtraDataByMixinNameComparator;
        })();
        content.ExtraDataByMixinNameComparator = ExtraDataByMixinNameComparator;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var TreeNodesOfContentPath = (function () {
            function TreeNodesOfContentPath(path, altPath) {
                this.path = path;
                this.altPath = altPath;
                this.nodes = [];
            }
            TreeNodesOfContentPath.prototype.getPath = function () {
                return this.path;
            };
            TreeNodesOfContentPath.prototype.getAltPath = function () {
                return this.altPath;
            };
            TreeNodesOfContentPath.prototype.getNodes = function () {
                return this.nodes;
            };
            TreeNodesOfContentPath.prototype.hasNodes = function () {
                return this.nodes.length > 0;
            };
            TreeNodesOfContentPath.prototype.getId = function () {
                return (this.hasNodes() && this.nodes[0].getData()) ? this.nodes[0].getData().getId() : "";
            };
            TreeNodesOfContentPath.prototype.updateNodeData = function (data) {
                this.nodes.forEach(function (node) {
                    node.setData(data);
                    node.clearViewers();
                });
            };
            return TreeNodesOfContentPath;
        })();
        content.TreeNodesOfContentPath = TreeNodesOfContentPath;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var TreeNodeParentOfContent = (function () {
            function TreeNodeParentOfContent(data, node) {
                this.data = data;
                this.node = node;
            }
            TreeNodeParentOfContent.prototype.getData = function () {
                return this.data;
            };
            TreeNodeParentOfContent.prototype.getNode = function () {
                return this.node;
            };
            return TreeNodeParentOfContent;
        })();
        content.TreeNodeParentOfContent = TreeNodeParentOfContent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        (function (ContentServerChangeType) {
            ContentServerChangeType[ContentServerChangeType["UNKNOWN"] = 0] = "UNKNOWN";
            ContentServerChangeType[ContentServerChangeType["PUBLISH"] = 1] = "PUBLISH";
            ContentServerChangeType[ContentServerChangeType["DUPLICATE"] = 2] = "DUPLICATE";
            ContentServerChangeType[ContentServerChangeType["CREATE"] = 3] = "CREATE";
            ContentServerChangeType[ContentServerChangeType["UPDATE"] = 4] = "UPDATE";
            ContentServerChangeType[ContentServerChangeType["DELETE"] = 5] = "DELETE";
            ContentServerChangeType[ContentServerChangeType["PENDING"] = 6] = "PENDING";
            ContentServerChangeType[ContentServerChangeType["RENAME"] = 7] = "RENAME";
            ContentServerChangeType[ContentServerChangeType["SORT"] = 8] = "SORT";
        })(content.ContentServerChangeType || (content.ContentServerChangeType = {}));
        var ContentServerChangeType = content.ContentServerChangeType;
        var ContentServerChange = (function () {
            function ContentServerChange(contentPaths, type) {
                this.contentPaths = contentPaths;
                this.type = type;
            }
            ContentServerChange.prototype.getContentPaths = function () {
                return this.contentPaths;
            };
            ContentServerChange.prototype.getChangeType = function () {
                return this.type;
            };
            ContentServerChange.prototype.toString = function () {
                return ContentServerChangeType[this.type] + ": <" + this.contentPaths.map(function (contentPath) { return contentPath.toString(); }).join(", ") + ">";
            };
            ContentServerChange.fromJson = function (json) {
                var contentEventType;
                switch (json.t) {
                    case 'P':
                        contentEventType = 1 /* PUBLISH */;
                        break;
                    case 'C':
                        contentEventType = 3 /* CREATE */;
                        break;
                    case 'U':
                        contentEventType = 4 /* UPDATE */;
                        break;
                    case 'X':
                        contentEventType = 5 /* DELETE */;
                        break;
                    case 'D':
                        contentEventType = 2 /* DUPLICATE */;
                        break;
                    case 'A':
                        contentEventType = 6 /* PENDING */;
                        break;
                    case 'R':
                        contentEventType = 7 /* RENAME */;
                        break;
                    case 'S':
                        contentEventType = 8 /* SORT */;
                        break;
                    default:
                        contentEventType = 0 /* UNKNOWN */;
                }
                var contentPaths = json.p.map(function (contentPath) { return api.content.ContentPath.fromString(contentPath); });
                return new ContentServerChange(contentPaths, contentEventType);
            };
            return ContentServerChange;
        })();
        content.ContentServerChange = ContentServerChange;
        var ContentServerEvent = (function (_super) {
            __extends(ContentServerEvent, _super);
            function ContentServerEvent(changes) {
                _super.call(this);
                this.changes = changes || [];
            }
            ContentServerEvent.prototype.getContentChanges = function () {
                return this.changes;
            };
            ContentServerEvent.prototype.toString = function () {
                return "ContentServerEvent: [" + this.changes.map(function (change) { return change.toString(); }).join(", ") + "]";
            };
            ContentServerEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentServerEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            ContentServerEvent.fromJson = function (json) {
                var changes = json.changes.map(function (changeJson) { return ContentServerChange.fromJson(changeJson); });
                return new ContentServerEvent(changes);
            };
            return ContentServerEvent;
        })(api.event.Event);
        content.ContentServerEvent = ContentServerEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentChangeResult = (function () {
            function ContentChangeResult(type, result) {
                this.type = type;
                this.result = result;
            }
            ContentChangeResult.prototype.getResult = function () {
                return this.result;
            };
            ContentChangeResult.prototype.getChangeType = function () {
                return this.type;
            };
            return ContentChangeResult;
        })();
        content.ContentChangeResult = ContentChangeResult;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentResourceRequest = (function (_super) {
            __extends(ContentResourceRequest, _super);
            function ContentResourceRequest() {
                _super.call(this);
                this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "content");
            }
            ContentResourceRequest.prototype.getResourcePath = function () {
                return this.resourcePath;
            };
            ContentResourceRequest.prototype.fromJsonToContent = function (json) {
                return content.Content.fromJson(json);
            };
            ContentResourceRequest.prototype.fromJsonToContentArray = function (json) {
                var _this = this;
                var array = [];
                json.forEach(function (itemJson) {
                    array.push(_this.fromJsonToContent(itemJson));
                });
                return array;
            };
            ContentResourceRequest.prototype.fromJsonToContentSummary = function (json) {
                return content.ContentSummary.fromJson(json);
            };
            ContentResourceRequest.prototype.fromJsonToContentSummaryArray = function (json) {
                var _this = this;
                var array = [];
                json.forEach(function (itemJson) {
                    array.push(_this.fromJsonToContentSummary(itemJson));
                });
                return array;
            };
            ContentResourceRequest.prototype.fromJsonToContentIdBaseItem = function (json) {
                return content.ContentIdBaseItem.fromJson(json);
            };
            ContentResourceRequest.prototype.fromJsonToContentIdBaseItemArray = function (jsonArray) {
                return content.ContentIdBaseItem.fromJsonArray(jsonArray);
            };
            ContentResourceRequest.EXPAND_NONE = 'none';
            ContentResourceRequest.EXPAND_SUMMARY = 'summary';
            ContentResourceRequest.EXPAND_FULL = 'full';
            return ContentResourceRequest;
        })(api.rest.ResourceRequest);
        content.ContentResourceRequest = ContentResourceRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ApplyContentPermissionsRequest = (function (_super) {
            __extends(ApplyContentPermissionsRequest, _super);
            function ApplyContentPermissionsRequest() {
                _super.call(this);
                this.inheritPermissions = true;
                this.overwriteChildPermissions = false;
                this.setMethod("POST");
            }
            ApplyContentPermissionsRequest.prototype.setId = function (id) {
                this.id = id;
                return this;
            };
            ApplyContentPermissionsRequest.prototype.setPermissions = function (permissions) {
                this.permissions = permissions;
                return this;
            };
            ApplyContentPermissionsRequest.prototype.setInheritPermissions = function (inheritPermissions) {
                this.inheritPermissions = inheritPermissions;
                return this;
            };
            ApplyContentPermissionsRequest.prototype.setOverwriteChildPermissions = function (overwriteChildPermissions) {
                this.overwriteChildPermissions = overwriteChildPermissions;
                return this;
            };
            ApplyContentPermissionsRequest.prototype.getParams = function () {
                return {
                    contentId: this.id,
                    permissions: this.permissions ? this.permissions.toJson() : undefined,
                    inheritPermissions: this.inheritPermissions,
                    overwriteChildPermissions: this.overwriteChildPermissions
                };
            };
            ApplyContentPermissionsRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "applyPermissions");
            };
            ApplyContentPermissionsRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return ApplyContentPermissionsRequest;
        })(content.ContentResourceRequest);
        content.ApplyContentPermissionsRequest = ApplyContentPermissionsRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetContentByIdRequest = (function (_super) {
            __extends(GetContentByIdRequest, _super);
            function GetContentByIdRequest(id) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.id = id;
            }
            GetContentByIdRequest.prototype.setExpand = function (expand) {
                this.expand = expand;
                return this;
            };
            GetContentByIdRequest.prototype.getParams = function () {
                return {
                    id: this.id.toString(),
                    expand: this.expand
                };
            };
            GetContentByIdRequest.prototype.getRequestPath = function () {
                return _super.prototype.getResourcePath.call(this);
            };
            GetContentByIdRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return GetContentByIdRequest;
        })(content.ContentResourceRequest);
        content.GetContentByIdRequest = GetContentByIdRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetContentSummaryByIdRequest = (function (_super) {
            __extends(GetContentSummaryByIdRequest, _super);
            function GetContentSummaryByIdRequest(id) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.id = id;
                this.expand = content.ContentResourceRequest.EXPAND_SUMMARY;
            }
            GetContentSummaryByIdRequest.prototype.getParams = function () {
                return {
                    id: this.id.toString(),
                    expand: this.expand
                };
            };
            GetContentSummaryByIdRequest.prototype.getRequestPath = function () {
                return _super.prototype.getResourcePath.call(this);
            };
            GetContentSummaryByIdRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContentSummary(response.getResult());
                });
            };
            return GetContentSummaryByIdRequest;
        })(content.ContentResourceRequest);
        content.GetContentSummaryByIdRequest = GetContentSummaryByIdRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetContentSummaryByIds = (function () {
            function GetContentSummaryByIds(ids) {
                this.ids = ids;
            }
            GetContentSummaryByIds.prototype.get = function () {
                var deferred = wemQ.defer();
                var allPromises = (this.ids || []).map(function (contentId) {
                    return new api.content.GetContentSummaryByIdRequest(contentId).sendAndParse();
                });
                var settledPromises = wemQ.allSettled(allPromises);
                settledPromises.spread(function () {
                    var contentResults = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        contentResults[_i - 0] = arguments[_i];
                    }
                    if (!contentResults) {
                        deferred.resolve([]);
                        return;
                    }
                    var contents = contentResults.filter(function (contentResult) { return contentResult.state === 'fulfilled'; }).map(function (contentResult) { return contentResult.value; });
                    deferred.resolve(contents);
                }).done();
                return deferred.promise;
            };
            return GetContentSummaryByIds;
        })();
        content.GetContentSummaryByIds = GetContentSummaryByIds;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetContentByPathRequest = (function (_super) {
            __extends(GetContentByPathRequest, _super);
            function GetContentByPathRequest(path) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.contentPath = path;
            }
            GetContentByPathRequest.prototype.getParams = function () {
                return {
                    path: this.contentPath.toString()
                };
            };
            GetContentByPathRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "bypath");
            };
            GetContentByPathRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return GetContentByPathRequest;
        })(content.ContentResourceRequest);
        content.GetContentByPathRequest = GetContentByPathRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var AccessControlList = api.security.acl.AccessControlList;
        var GetContentPermissionsByIdRequest = (function (_super) {
            __extends(GetContentPermissionsByIdRequest, _super);
            function GetContentPermissionsByIdRequest(contentId) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.contentId = contentId;
            }
            GetContentPermissionsByIdRequest.prototype.getParams = function () {
                return {
                    id: this.contentId.toString()
                };
            };
            GetContentPermissionsByIdRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "contentPermissions");
            };
            GetContentPermissionsByIdRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return AccessControlList.fromJson(response.getResult().permissions);
                });
            };
            return GetContentPermissionsByIdRequest;
        })(content.ContentResourceRequest);
        content.GetContentPermissionsByIdRequest = GetContentPermissionsByIdRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var AccessControlList = api.security.acl.AccessControlList;
        var GetContentRootPermissionsRequest = (function (_super) {
            __extends(GetContentRootPermissionsRequest, _super);
            function GetContentRootPermissionsRequest() {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
            }
            GetContentRootPermissionsRequest.prototype.getParams = function () {
                return {};
            };
            GetContentRootPermissionsRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "rootPermissions");
            };
            GetContentRootPermissionsRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return AccessControlList.fromJson(response.getResult().permissions);
                });
            };
            return GetContentRootPermissionsRequest;
        })(content.ContentResourceRequest);
        content.GetContentRootPermissionsRequest = GetContentRootPermissionsRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var CreateContentRequest = (function (_super) {
            __extends(CreateContentRequest, _super);
            function CreateContentRequest() {
                _super.call(this);
                this.meta = [];
                this.valid = false;
                this.requireValid = false;
                _super.prototype.setMethod.call(this, "POST");
            }
            CreateContentRequest.prototype.setValid = function (value) {
                this.valid = value;
                return this;
            };
            CreateContentRequest.prototype.setRequireValid = function (value) {
                this.requireValid = value;
                return this;
            };
            CreateContentRequest.prototype.setName = function (value) {
                this.name = value;
                return this;
            };
            CreateContentRequest.prototype.setParent = function (value) {
                this.parent = value;
                return this;
            };
            CreateContentRequest.prototype.setContentType = function (value) {
                this.contentType = value;
                return this;
            };
            CreateContentRequest.prototype.setData = function (data) {
                this.data = data;
                return this;
            };
            CreateContentRequest.prototype.setExtraData = function (extraData) {
                this.meta = extraData;
                return this;
            };
            CreateContentRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            CreateContentRequest.prototype.getParams = function () {
                return {
                    valid: this.valid,
                    requireValid: this.requireValid,
                    name: this.name.isUnnamed() ? this.name.toUnnamed().toStringIncludingHidden() : this.name.toString(),
                    parent: this.parent.toString(),
                    contentType: this.contentType.toString(),
                    data: this.data.toJson(),
                    meta: this.extraDataToJson(),
                    displayName: this.displayName
                };
            };
            CreateContentRequest.prototype.extraDataToJson = function () {
                return this.meta ? this.meta.map(function (extraData) { return extraData.toJson(); }) : null;
            };
            CreateContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "create");
            };
            CreateContentRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return CreateContentRequest;
        })(content.ContentResourceRequest);
        content.CreateContentRequest = CreateContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var OrderContentRequest = (function (_super) {
            __extends(OrderContentRequest, _super);
            function OrderContentRequest() {
                _super.call(this);
                this.silent = false;
                _super.prototype.setMethod.call(this, "POST");
            }
            OrderContentRequest.prototype.setContentId = function (value) {
                this.contentId = value;
                return this;
            };
            OrderContentRequest.prototype.setChildOrder = function (value) {
                this.childOrder = value;
                return this;
            };
            OrderContentRequest.prototype.setSilent = function (silent) {
                this.silent = silent;
                return this;
            };
            OrderContentRequest.prototype.getParams = function () {
                return this.contentToJson();
            };
            OrderContentRequest.prototype.contentToJson = function () {
                return content.ChildOrder.toSetChildOrderJson(this.contentId, this.childOrder, this.silent);
            };
            OrderContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "setChildOrder");
            };
            OrderContentRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return OrderContentRequest;
        })(content.ContentResourceRequest);
        content.OrderContentRequest = OrderContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var OrderChildContentRequest = (function (_super) {
            __extends(OrderChildContentRequest, _super);
            function OrderChildContentRequest() {
                _super.call(this);
                this.silent = false;
                this.manualOrder = false;
                _super.prototype.setMethod.call(this, "POST");
            }
            OrderChildContentRequest.prototype.setSilent = function (silent) {
                this.silent = silent;
                return this;
            };
            OrderChildContentRequest.prototype.setManualOrder = function (manualOrder) {
                this.manualOrder = manualOrder;
                return this;
            };
            OrderChildContentRequest.prototype.setContentId = function (value) {
                this.contentId = value;
                return this;
            };
            OrderChildContentRequest.prototype.setChildOrder = function (value) {
                this.childOrder = value;
                return this;
            };
            OrderChildContentRequest.prototype.setContentMovements = function (value) {
                this.contentMovements = value;
                return this;
            };
            OrderChildContentRequest.prototype.getParams = function () {
                return {
                    "silent": this.silent,
                    "manualOrder": this.manualOrder,
                    "contentId": this.contentId.toString(),
                    "childOrder": this.childOrder ? this.childOrder.toJson() : undefined,
                    "reorderChildren": this.contentMovements.toArrayJson()
                };
            };
            OrderChildContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "reorderChildren");
            };
            OrderChildContentRequest.prototype.sendAndParse = function () {
                return this.send();
            };
            return OrderChildContentRequest;
        })(content.ContentResourceRequest);
        content.OrderChildContentRequest = OrderChildContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var UpdateContentRequest = (function (_super) {
            __extends(UpdateContentRequest, _super);
            function UpdateContentRequest(id) {
                _super.call(this);
                this.id = id;
                this.requireValid = false;
                this.setMethod("POST");
            }
            UpdateContentRequest.prototype.setId = function (id) {
                this.id = id;
                return this;
            };
            UpdateContentRequest.prototype.setContentName = function (value) {
                this.name = value;
                return this;
            };
            UpdateContentRequest.prototype.setData = function (contentData) {
                this.data = contentData;
                return this;
            };
            UpdateContentRequest.prototype.setExtraData = function (extraData) {
                this.meta = extraData;
                return this;
            };
            UpdateContentRequest.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                return this;
            };
            UpdateContentRequest.prototype.setRequireValid = function (requireValid) {
                this.requireValid = requireValid;
                return this;
            };
            UpdateContentRequest.prototype.setLanguage = function (language) {
                this.language = language;
                return this;
            };
            UpdateContentRequest.prototype.setOwner = function (owner) {
                this.owner = owner;
                return this;
            };
            UpdateContentRequest.prototype.getParams = function () {
                return {
                    contentId: this.id,
                    requireValid: this.requireValid,
                    contentName: this.name.isUnnamed() ? this.name.toUnnamed().toStringIncludingHidden() : this.name.toString(),
                    data: this.data.toJson(),
                    meta: (this.meta || []).map(function (extraData) { return extraData.toJson(); }),
                    displayName: this.displayName,
                    language: this.language,
                    owner: this.owner ? this.owner.toString() : undefined
                };
            };
            UpdateContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "update");
            };
            UpdateContentRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return UpdateContentRequest;
        })(content.ContentResourceRequest);
        content.UpdateContentRequest = UpdateContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var DuplicateContentRequest = (function (_super) {
            __extends(DuplicateContentRequest, _super);
            function DuplicateContentRequest(id) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.id = id;
            }
            DuplicateContentRequest.prototype.getParams = function () {
                return {
                    contentId: this.id.toString()
                };
            };
            DuplicateContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "duplicate");
            };
            DuplicateContentRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContent(response.getResult());
                });
            };
            return DuplicateContentRequest;
        })(content.ContentResourceRequest);
        content.DuplicateContentRequest = DuplicateContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var MoveContentRequest = (function (_super) {
            __extends(MoveContentRequest, _super);
            function MoveContentRequest(id, parentPath) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.ids = id;
                this.parentPath = parentPath;
            }
            MoveContentRequest.prototype.getParams = function () {
                var fn = function (contentId) {
                    return contentId.toString();
                };
                return {
                    contentIds: this.ids.map(fn),
                    parentContentPath: !!this.parentPath ? this.parentPath.toString() : ""
                };
            };
            MoveContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "move");
            };
            MoveContentRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return content.MoveContentResult.fromJson(response.getResult());
                });
            };
            return MoveContentRequest;
        })(content.ContentResourceRequest);
        content.MoveContentRequest = MoveContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ListContentByIdRequest = (function (_super) {
            __extends(ListContentByIdRequest, _super);
            function ListContentByIdRequest(parentId) {
                _super.call(this);
                this.expand = 1 /* SUMMARY */;
                _super.prototype.setMethod.call(this, "GET");
                this.parentId = parentId;
            }
            ListContentByIdRequest.prototype.setExpand = function (value) {
                this.expand = value;
                return this;
            };
            ListContentByIdRequest.prototype.setFrom = function (value) {
                this.from = value;
                return this;
            };
            ListContentByIdRequest.prototype.setSize = function (value) {
                this.size = value;
                return this;
            };
            ListContentByIdRequest.prototype.setOrder = function (value) {
                this.order = value;
                return this;
            };
            ListContentByIdRequest.prototype.getParams = function () {
                return {
                    parentId: this.parentId ? this.parentId.toString() : null,
                    expand: this.expand,
                    from: this.from,
                    size: this.size,
                    childOrder: !!this.order ? this.order.toString() : ""
                };
            };
            ListContentByIdRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "list");
            };
            ListContentByIdRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return new content.ContentResponse(content.ContentSummary.fromJsonArray(response.getResult().contents), new content.ContentMetadata(response.getResult().metadata["hits"], response.getResult().metadata["totalHits"]));
                });
            };
            return ListContentByIdRequest;
        })(content.ContentResourceRequest);
        content.ListContentByIdRequest = ListContentByIdRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ListContentByPathRequest = (function (_super) {
            __extends(ListContentByPathRequest, _super);
            function ListContentByPathRequest(parentPath) {
                _super.call(this);
                this.expand = 1 /* SUMMARY */;
                _super.prototype.setMethod.call(this, "GET");
                this.parentPath = parentPath;
            }
            ListContentByPathRequest.prototype.setExpand = function (value) {
                this.expand = value;
                return this;
            };
            ListContentByPathRequest.prototype.setFrom = function (value) {
                this.from = value;
                return this;
            };
            ListContentByPathRequest.prototype.setSize = function (value) {
                this.size = value;
                return this;
            };
            ListContentByPathRequest.prototype.getParams = function () {
                return {
                    parentPath: this.parentPath.toString(),
                    expand: this.expand,
                    from: this.from,
                    size: this.size
                };
            };
            ListContentByPathRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "list", "bypath");
            };
            ListContentByPathRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return new content.ContentResponse(content.ContentSummary.fromJsonArray(response.getResult().contents), new content.ContentMetadata(response.getResult().metadata["hits"], response.getResult().metadata["totalHits"]));
                });
            };
            return ListContentByPathRequest;
        })(content.ContentResourceRequest);
        content.ListContentByPathRequest = ListContentByPathRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var DeleteContentRequest = (function (_super) {
            __extends(DeleteContentRequest, _super);
            function DeleteContentRequest(contentPath) {
                _super.call(this);
                this.contentPaths = [];
                _super.prototype.setMethod.call(this, "POST");
                if (contentPath) {
                    this.addContentPath(contentPath);
                }
            }
            DeleteContentRequest.prototype.setContentPaths = function (contentPaths) {
                this.contentPaths = contentPaths;
                return this;
            };
            DeleteContentRequest.prototype.addContentPath = function (contentPath) {
                this.contentPaths.push(contentPath);
                return this;
            };
            DeleteContentRequest.prototype.getParams = function () {
                var fn = function (contentPath) {
                    return contentPath.toString();
                };
                return {
                    contentPaths: this.contentPaths.map(fn)
                };
            };
            DeleteContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "delete");
            };
            DeleteContentRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return content.DeleteContentResult.fromJson(response.getResult());
                });
            };
            return DeleteContentRequest;
        })(content.ContentResourceRequest);
        content.DeleteContentRequest = DeleteContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var DeleteContentResult = (function () {
            function DeleteContentResult(success, pending, failures) {
                this.deleteSuccess = !!success ? success : [];
                this.deleteFailures = !!failures ? failures : [];
                this.deletePending = !!pending ? pending : [];
            }
            DeleteContentResult.prototype.getDeleted = function () {
                return this.deleteSuccess;
            };
            DeleteContentResult.prototype.getPendings = function () {
                return this.deletePending;
            };
            DeleteContentResult.prototype.getDeleteFailures = function () {
                return this.deleteFailures;
            };
            DeleteContentResult.fromJson = function (json) {
                if (json.successes) {
                    var success = json.successes.map(function (success) { return new DeleteContentResultSuccess(success.id, success.name); });
                }
                if (json.pendings) {
                    var pending = json.pendings.map(function (pending) { return pending.name; });
                }
                if (json.failures) {
                    var failure = json.failures.map(function (failure) { return new DeleteContentResultFailure(failure.id, failure.name, failure.reason); });
                }
                return new DeleteContentResult(success, pending, failure);
            };
            return DeleteContentResult;
        })();
        content.DeleteContentResult = DeleteContentResult;
        var DeleteContentResultSuccess = (function () {
            function DeleteContentResultSuccess(id, name) {
                this.id = id;
                this.name = name;
            }
            DeleteContentResultSuccess.prototype.getId = function () {
                return this.id;
            };
            DeleteContentResultSuccess.prototype.getName = function () {
                return this.name;
            };
            return DeleteContentResultSuccess;
        })();
        content.DeleteContentResultSuccess = DeleteContentResultSuccess;
        var DeleteContentResultFailure = (function (_super) {
            __extends(DeleteContentResultFailure, _super);
            function DeleteContentResultFailure(id, name, reason) {
                _super.call(this, id, name);
                this.reason = reason;
            }
            DeleteContentResultFailure.prototype.getReason = function () {
                return this.reason;
            };
            return DeleteContentResultFailure;
        })(DeleteContentResultSuccess);
        content.DeleteContentResultFailure = DeleteContentResultFailure;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var BatchContentRequest = (function (_super) {
            __extends(BatchContentRequest, _super);
            function BatchContentRequest(contentPath) {
                _super.call(this);
                this.contentPaths = [];
                _super.prototype.setMethod.call(this, "POST");
                if (contentPath) {
                    this.addContentPath(contentPath);
                }
            }
            BatchContentRequest.prototype.setContentPaths = function (contentPaths) {
                this.contentPaths = contentPaths;
                return this;
            };
            BatchContentRequest.prototype.addContentPath = function (contentPath) {
                this.contentPaths.push(contentPath);
                return this;
            };
            BatchContentRequest.prototype.getParams = function () {
                var fn = function (contentPath) {
                    return contentPath.toString();
                };
                return {
                    contentPaths: this.contentPaths.map(fn)
                };
            };
            BatchContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "batch");
            };
            BatchContentRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return new content.ContentResponse(content.ContentSummary.fromJsonArray(response.getResult().contents), new content.ContentMetadata(response.getResult().metadata["hits"], response.getResult().metadata["totalHits"]));
                });
            };
            return BatchContentRequest;
        })(content.ContentResourceRequest);
        content.BatchContentRequest = BatchContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentCreatedEvent = (function (_super) {
            __extends(ContentCreatedEvent, _super);
            function ContentCreatedEvent(contentId) {
                _super.call(this);
                this.contentId = contentId;
            }
            ContentCreatedEvent.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentCreatedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentCreatedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            ContentCreatedEvent.fromJson = function (json) {
                return new ContentCreatedEvent(new content.ContentId(json.contentId));
            };
            return ContentCreatedEvent;
        })(api.event.Event);
        content.ContentCreatedEvent = ContentCreatedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentUpdatedEvent = (function (_super) {
            __extends(ContentUpdatedEvent, _super);
            function ContentUpdatedEvent(contentId) {
                _super.call(this);
                this.contentId = contentId;
            }
            ContentUpdatedEvent.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentUpdatedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentUpdatedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            ContentUpdatedEvent.fromJson = function (json) {
                return new ContentUpdatedEvent(new content.ContentId(json.contentId));
            };
            return ContentUpdatedEvent;
        })(api.event.Event);
        content.ContentUpdatedEvent = ContentUpdatedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentPublishedEvent = (function (_super) {
            __extends(ContentPublishedEvent, _super);
            function ContentPublishedEvent(contentId) {
                _super.call(this);
                this.contentId = contentId;
            }
            ContentPublishedEvent.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentPublishedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentPublishedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            ContentPublishedEvent.fromJson = function (json) {
                return new ContentPublishedEvent(new content.ContentId(json.contentId));
            };
            return ContentPublishedEvent;
        })(api.event.Event);
        content.ContentPublishedEvent = ContentPublishedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentsPublishedEvent = (function (_super) {
            __extends(ContentsPublishedEvent, _super);
            function ContentsPublishedEvent(contentIds) {
                _super.call(this);
                this.contentIds = contentIds;
            }
            ContentsPublishedEvent.prototype.getContentIds = function () {
                return this.contentIds;
            };
            ContentsPublishedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentsPublishedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ContentsPublishedEvent;
        })(api.event.Event);
        content.ContentsPublishedEvent = ContentsPublishedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentDeletedEvent = (function (_super) {
            __extends(ContentDeletedEvent, _super);
            function ContentDeletedEvent(contentId) {
                _super.call(this);
                this.contentId = contentId;
            }
            ContentDeletedEvent.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentDeletedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentDeletedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            ContentDeletedEvent.fromJson = function (json) {
                return new ContentDeletedEvent(new content.ContentId(json.contentId));
            };
            return ContentDeletedEvent;
        })(api.event.Event);
        content.ContentDeletedEvent = ContentDeletedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentChildOrderUpdatedEvent = (function (_super) {
            __extends(ContentChildOrderUpdatedEvent, _super);
            function ContentChildOrderUpdatedEvent(contentId) {
                _super.call(this);
                this.contentId = contentId;
            }
            ContentChildOrderUpdatedEvent.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentChildOrderUpdatedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentChildOrderUpdatedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ContentChildOrderUpdatedEvent;
        })(api.event.Event);
        content.ContentChildOrderUpdatedEvent = ContentChildOrderUpdatedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var SelectedOption = api.ui.selector.combobox.SelectedOption;
        var RichComboBox = api.ui.selector.combobox.RichComboBox;
        var RichComboBoxBuilder = api.ui.selector.combobox.RichComboBoxBuilder;
        var ContentComboBox = (function (_super) {
            __extends(ContentComboBox, _super);
            function ContentComboBox(builder) {
                var loader = builder.loader ? builder.loader : new _content.ContentSummaryLoader();
                loader.setAllowedContentTypes(builder.allowedContentTypes);
                var richComboBoxBuilder = new RichComboBoxBuilder().setComboBoxName(builder.name ? builder.name : 'contentSelector').setLoader(loader).setSelectedOptionsView(new ContentSelectedOptionsView()).setMaximumOccurrences(builder.maximumOccurrences).setOptionDisplayValueViewer(new api.content.ContentSummaryViewer()).setDelayedInputValueChangedHandling(750).setMinWidth(builder.minWidth);
                _super.call(this, richComboBoxBuilder);
            }
            ContentComboBox.prototype.getContent = function (contentId) {
                var option = this.getOptionByValue(contentId.toString());
                if (option) {
                    return option.displayValue;
                }
                return null;
            };
            ContentComboBox.prototype.setContent = function (content) {
                this.clearSelection();
                if (content) {
                    var optionToSelect = this.getOptionByValue(content.getContentId().toString());
                    if (!optionToSelect) {
                        optionToSelect = {
                            value: content.getContentId().toString(),
                            displayValue: content
                        };
                        this.addOption(optionToSelect);
                    }
                    this.selectOption(optionToSelect);
                }
            };
            ContentComboBox.create = function () {
                return new ContentComboBoxBuilder();
            };
            return ContentComboBox;
        })(RichComboBox);
        _content.ContentComboBox = ContentComboBox;
        var ContentSelectedOptionsView = (function (_super) {
            __extends(ContentSelectedOptionsView, _super);
            function ContentSelectedOptionsView() {
                _super.apply(this, arguments);
            }
            ContentSelectedOptionsView.prototype.createSelectedOption = function (option) {
                var optionView = new ContentSelectedOptionView(option);
                return new SelectedOption(optionView, this.count());
            };
            return ContentSelectedOptionsView;
        })(api.ui.selector.combobox.BaseSelectedOptionsView);
        _content.ContentSelectedOptionsView = ContentSelectedOptionsView;
        var ContentSelectedOptionView = (function (_super) {
            __extends(ContentSelectedOptionView, _super);
            function ContentSelectedOptionView(option) {
                _super.call(this, option);
            }
            ContentSelectedOptionView.prototype.resolveIconUrl = function (content) {
                return content.getIconUrl();
            };
            ContentSelectedOptionView.prototype.resolveTitle = function (content) {
                return content.getDisplayName().toString();
            };
            ContentSelectedOptionView.prototype.resolveSubTitle = function (content) {
                return content.getPath().toString();
            };
            return ContentSelectedOptionView;
        })(api.ui.selector.combobox.RichSelectedOptionView);
        _content.ContentSelectedOptionView = ContentSelectedOptionView;
        var ContentComboBoxBuilder = (function () {
            function ContentComboBoxBuilder() {
                this.maximumOccurrences = 0;
            }
            ContentComboBoxBuilder.prototype.setName = function (value) {
                this.name = value;
                return this;
            };
            ContentComboBoxBuilder.prototype.setMaximumOccurrences = function (maximumOccurrences) {
                this.maximumOccurrences = maximumOccurrences;
                return this;
            };
            ContentComboBoxBuilder.prototype.setLoader = function (loader) {
                this.loader = loader;
                return this;
            };
            ContentComboBoxBuilder.prototype.setAllowedContentTypes = function (allowedTypes) {
                this.allowedContentTypes = allowedTypes;
                return this;
            };
            ContentComboBoxBuilder.prototype.setMinWidth = function (value) {
                this.minWidth = value;
                return this;
            };
            ContentComboBoxBuilder.prototype.build = function () {
                return new ContentComboBox(this);
            };
            return ContentComboBoxBuilder;
        })();
        _content.ContentComboBoxBuilder = ContentComboBoxBuilder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentMovedEvent = (function (_super) {
            __extends(ContentMovedEvent, _super);
            function ContentMovedEvent(contentIds) {
                _super.call(this);
                this.names = contentIds;
            }
            ContentMovedEvent.prototype.getNames = function () {
                return this.names;
            };
            ContentMovedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentMovedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ContentMovedEvent;
        })(api.event.Event);
        content.ContentMovedEvent = ContentMovedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var MoveContentResult = (function () {
            function MoveContentResult(success, failures) {
                this.moveSuccess = success;
                this.moveFailures = failures;
            }
            MoveContentResult.prototype.getMoved = function () {
                return this.moveSuccess;
            };
            MoveContentResult.prototype.getMoveFailures = function () {
                return this.moveFailures;
            };
            MoveContentResult.fromJson = function (json) {
                var success = json.successes.map(function (success) { return success.name; });
                var failure = json.failures.map(function (failure) { return new MoveContentResultFailure(failure.name, failure.reason); });
                return new MoveContentResult(success, failure);
            };
            return MoveContentResult;
        })();
        content.MoveContentResult = MoveContentResult;
        var MoveContentResultFailure = (function () {
            function MoveContentResultFailure(name, reason) {
                this.name = name;
                this.reason = reason;
            }
            MoveContentResultFailure.prototype.getName = function () {
                return this.name;
            };
            MoveContentResultFailure.prototype.getReason = function () {
                return this.reason;
            };
            return MoveContentResultFailure;
        })();
        content.MoveContentResultFailure = MoveContentResultFailure;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentMoveComboBox = (function (_super) {
            __extends(ContentMoveComboBox, _super);
            function ContentMoveComboBox() {
                this.contentLoader = new content.MoveContentSummaryLoader();
                this.contentLoader.setSize(-1);
                var richComboBoxBuilder = new api.ui.selector.combobox.RichComboBoxBuilder();
                richComboBoxBuilder.setMaximumOccurrences(1).setComboBoxName("contentSelector").setLoader(this.contentLoader).setSelectedOptionsView(new content.ContentSelectedOptionsView()).setOptionDisplayValueViewer(new api.content.ContentSummaryViewer()).setDelayedInputValueChangedHandling(500);
                _super.call(this, richComboBoxBuilder);
            }
            ContentMoveComboBox.prototype.setFilterContentPath = function (contentPath) {
                this.contentLoader.setFilterContentPath(contentPath);
            };
            ContentMoveComboBox.prototype.setFilterSourceContentType = function (contentType) {
                this.contentLoader.setFilterSourceContentType(contentType);
            };
            return ContentMoveComboBox;
        })(api.ui.selector.combobox.RichComboBox);
        content.ContentMoveComboBox = ContentMoveComboBox;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentQueryRequest = (function (_super) {
            __extends(ContentQueryRequest, _super);
            function ContentQueryRequest(contentQuery) {
                _super.call(this);
                this.expand = 1 /* SUMMARY */;
                _super.prototype.setMethod.call(this, "POST");
                this.contentQuery = contentQuery;
            }
            ContentQueryRequest.prototype.setContentQuery = function (contentQuery) {
                this.contentQuery = contentQuery;
            };
            ContentQueryRequest.prototype.setExpand = function (expand) {
                this.expand = expand;
                return this;
            };
            ContentQueryRequest.prototype.getParams = function () {
                var queryExprAsString = this.contentQuery.getQueryExpr() ? this.contentQuery.getQueryExpr().toString() : "";
                return {
                    queryExpr: queryExprAsString,
                    from: this.contentQuery.getFrom(),
                    size: this.contentQuery.getSize(),
                    contentTypeNames: this.contentTypeNamesAsString(this.contentQuery.getContentTypes()),
                    expand: this.expandAsString(),
                    aggregationQueries: this.aggregationQueriesToJson(this.contentQuery.getAggregationQueries()),
                    queryFilters: this.queryFiltersToJson(this.contentQuery.getQueryFilters())
                };
            };
            ContentQueryRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    var responseResult = response.getResult();
                    var aggregations = api.aggregation.Aggregation.fromJsonArray(responseResult.aggregations);
                    var contentsAsJson = responseResult.contents;
                    var contentQueryResult;
                    var metadata = new content.ContentMetadata(response.getResult().metadata["hits"], response.getResult().metadata["totalHits"]);
                    if (_this.expand == 0 /* NONE */) {
                        var contentIdBaseItems = _this.fromJsonToContentIdBaseItemArray(contentsAsJson);
                        contentQueryResult = new content.ContentQueryResult(contentIdBaseItems, aggregations, contentsAsJson, metadata);
                    }
                    else if (_this.expand == 1 /* SUMMARY */) {
                        var contentSummaries = _this.fromJsonToContentSummaryArray(contentsAsJson);
                        contentQueryResult = new content.ContentQueryResult(contentSummaries, aggregations, contentsAsJson, metadata);
                    }
                    else {
                        var contents = _this.fromJsonToContentArray(contentsAsJson);
                        contentQueryResult = new content.ContentQueryResult(contents, aggregations, contentsAsJson, metadata);
                    }
                    return contentQueryResult;
                });
            };
            ContentQueryRequest.prototype.aggregationQueriesToJson = function (aggregationQueries) {
                var aggregationQueryJsons = [];
                if (aggregationQueries == null) {
                    return aggregationQueryJsons;
                }
                aggregationQueries.forEach(function (aggregationQuery) {
                    aggregationQueryJsons.push(aggregationQuery.toJson());
                });
                return aggregationQueryJsons;
            };
            ContentQueryRequest.prototype.queryFiltersToJson = function (queryFilters) {
                var queryFilterJsons = [];
                if (queryFilters == null || queryFilters.length == 0) {
                    return queryFilterJsons;
                }
                queryFilters.forEach(function (queryFilter) {
                    queryFilterJsons.push(queryFilter.toJson());
                });
                return queryFilterJsons;
            };
            ContentQueryRequest.prototype.expandAsString = function () {
                switch (this.expand) {
                    case 2 /* FULL */:
                        return "full";
                    case 1 /* SUMMARY */:
                        return "summary";
                    case 0 /* NONE */:
                        return "none";
                    default:
                        return "summary";
                }
            };
            ContentQueryRequest.prototype.contentTypeNamesAsString = function (names) {
                var result = [];
                names.forEach(function (name) {
                    result.push(name.toString());
                });
                return result;
            };
            ContentQueryRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "query");
            };
            return ContentQueryRequest;
        })(content.ContentResourceRequest);
        content.ContentQueryRequest = ContentQueryRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentQueryResult = (function () {
            function ContentQueryResult(contents, aggreations, contentsAsJson, metadata) {
                this.contents = contents;
                this.aggregations = aggreations;
                this.contentsAsJson = contentsAsJson;
                this.metadata = metadata;
            }
            ContentQueryResult.prototype.getContents = function () {
                return this.contents;
            };
            ContentQueryResult.prototype.getContentsAsJson = function () {
                return this.contentsAsJson;
            };
            ContentQueryResult.prototype.getAggregations = function () {
                return this.aggregations;
            };
            ContentQueryResult.prototype.getMetadata = function () {
                return this.metadata;
            };
            return ContentQueryResult;
        })();
        content.ContentQueryResult = ContentQueryResult;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var PublishContentRequest = (function (_super) {
            __extends(PublishContentRequest, _super);
            function PublishContentRequest(contentId) {
                _super.call(this);
                this.ids = [];
                _super.prototype.setMethod.call(this, "POST");
                if (contentId) {
                    this.addId(contentId);
                }
            }
            PublishContentRequest.prototype.setIds = function (contentIds) {
                this.ids = contentIds;
                return this;
            };
            PublishContentRequest.prototype.setIncludeChildren = function (includeChildren) {
                this.includeChildren = includeChildren;
                return this;
            };
            PublishContentRequest.prototype.addId = function (contentId) {
                this.ids.push(contentId);
                return this;
            };
            PublishContentRequest.prototype.getParams = function () {
                return {
                    ids: this.ids.map(function (el) {
                        return el.toString();
                    }),
                    includeChildren: this.includeChildren
                };
            };
            PublishContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "publish");
            };
            PublishContentRequest.feedback = function (jsonResponse) {
                var result = jsonResponse.getResult(), succeeded = result.successes.length, failed = result.failures.length, deleted = result.deleted.length, total = succeeded + failed + deleted;
                switch (total) {
                    case 0:
                        api.notify.showFeedback('Nothing to publish.');
                        break;
                    case 1:
                        if (succeeded === 1) {
                            api.notify.showSuccess('\"' + result.successes[0].name + '\" published');
                        }
                        else if (failed === 1) {
                            api.notify.showError('\"' + result.failures[0].name + '\" failed, reason: ' + result.failures[0].reason);
                        }
                        else {
                            api.notify.showSuccess('\"' + result.deleted[0].name + '\" deleted');
                        }
                        break;
                    default:
                        if (succeeded > 0) {
                            api.notify.showSuccess(succeeded + ' items were published');
                        }
                        if (deleted > 0) {
                            api.notify.showSuccess(deleted + ' pending items were deleted');
                        }
                        if (failed > 0) {
                            api.notify.showError(failed + ' items failed to publish');
                        }
                }
            };
            return PublishContentRequest;
        })(content.ContentResourceRequest);
        content.PublishContentRequest = PublishContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ResolvePublishRequestedContentsRequest = (function (_super) {
            __extends(ResolvePublishRequestedContentsRequest, _super);
            function ResolvePublishRequestedContentsRequest(contentIds, includeChildren) {
                _super.call(this);
                this.ids = [];
                _super.prototype.setMethod.call(this, "POST");
                this.ids = contentIds;
                this.includeChildren = includeChildren;
            }
            ResolvePublishRequestedContentsRequest.prototype.getParams = function () {
                return {
                    ids: this.ids.map(function (el) {
                        return el.toString();
                    }),
                    includeChildren: this.includeChildren
                };
            };
            ResolvePublishRequestedContentsRequest.prototype.getRequestPath = function () {
                console.log("aa");
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "resolvePublishContent");
            };
            return ResolvePublishRequestedContentsRequest;
        })(content.ContentResourceRequest);
        content.ResolvePublishRequestedContentsRequest = ResolvePublishRequestedContentsRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ResolvePublishDependenciesRequest = (function (_super) {
            __extends(ResolvePublishDependenciesRequest, _super);
            function ResolvePublishDependenciesRequest(contentIds, includeChildren) {
                _super.call(this);
                this.ids = [];
                _super.prototype.setMethod.call(this, "POST");
                this.ids = contentIds;
                this.includeChildren = includeChildren;
            }
            ResolvePublishDependenciesRequest.prototype.getParams = function () {
                return {
                    ids: this.ids.map(function (el) {
                        return el.toString();
                    }),
                    includeChildren: this.includeChildren
                };
            };
            ResolvePublishDependenciesRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "resolvePublishContent");
            };
            return ResolvePublishDependenciesRequest;
        })(content.ContentResourceRequest);
        content.ResolvePublishDependenciesRequest = ResolvePublishDependenciesRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var CompareContentRequest = (function (_super) {
            __extends(CompareContentRequest, _super);
            function CompareContentRequest(ids) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.ids = ids;
            }
            CompareContentRequest.fromContentSummaries = function (contentSummaries) {
                var ids = [];
                contentSummaries.forEach(function (contentSummary) {
                    ids.push(contentSummary.getContentId().toString());
                });
                return new CompareContentRequest(ids);
            };
            CompareContentRequest.prototype.getParams = function () {
                return {
                    ids: this.ids
                };
            };
            CompareContentRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "compare");
            };
            CompareContentRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToCompareResults(response.getResult());
                });
            };
            CompareContentRequest.prototype.fromJsonToCompareResults = function (json) {
                return content.CompareContentResults.fromJson(json);
            };
            return CompareContentRequest;
        })(content.ContentResourceRequest);
        content.CompareContentRequest = CompareContentRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var CompareContentResults = (function () {
            function CompareContentResults(compareContentResults) {
                this.compareContentResults = [];
                this.compareContentResults = compareContentResults;
            }
            CompareContentResults.prototype.get = function (contentId) {
                var compareContentResult = null;
                this.compareContentResults.forEach(function (result) {
                    if (result.getId() == contentId) {
                        compareContentResult = result;
                    }
                });
                return compareContentResult;
            };
            CompareContentResults.fromJson = function (json) {
                var list = [];
                json.compareContentResults.forEach(function (compareContentResult) {
                    list.push(content.CompareContentResult.fromJson(compareContentResult));
                });
                return new CompareContentResults(list);
            };
            return CompareContentResults;
        })();
        content.CompareContentResults = CompareContentResults;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var CompareContentResult = (function () {
            function CompareContentResult(id, compareStatus) {
                this.compareStatus = compareStatus;
                this.id = id;
            }
            CompareContentResult.prototype.getId = function () {
                return this.id;
            };
            CompareContentResult.prototype.getCompareStatus = function () {
                return this.compareStatus;
            };
            CompareContentResult.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, CompareContentResult)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.id.toString(), other.id.toString())) {
                    return false;
                }
                return true;
            };
            CompareContentResult.fromJson = function (json) {
                var status = content.CompareStatus[json.compareStatus];
                return new CompareContentResult(json.id, status);
            };
            return CompareContentResult;
        })();
        content.CompareContentResult = CompareContentResult;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        (function (CompareStatus) {
            CompareStatus[CompareStatus["NEW"] = 0] = "NEW";
            CompareStatus[CompareStatus["NEW_TARGET"] = 1] = "NEW_TARGET";
            CompareStatus[CompareStatus["NEWER"] = 2] = "NEWER";
            CompareStatus[CompareStatus["OLDER"] = 3] = "OLDER";
            CompareStatus[CompareStatus["PENDING_DELETE"] = 4] = "PENDING_DELETE";
            CompareStatus[CompareStatus["PENDING_DELETE_TARGET"] = 5] = "PENDING_DELETE_TARGET";
            CompareStatus[CompareStatus["EQUAL"] = 6] = "EQUAL";
            CompareStatus[CompareStatus["MOVED"] = 7] = "MOVED";
            CompareStatus[CompareStatus["CONFLICT_PATH_EXISTS"] = 8] = "CONFLICT_PATH_EXISTS";
            CompareStatus[CompareStatus["CONFLICT_VERSION_BRANCH_DIVERGS"] = 9] = "CONFLICT_VERSION_BRANCH_DIVERGS";
            CompareStatus[CompareStatus["UNKNOWN"] = 10] = "UNKNOWN";
        })(content.CompareStatus || (content.CompareStatus = {}));
        var CompareStatus = content.CompareStatus;
        var CompareStatusFormatter = (function () {
            function CompareStatusFormatter() {
            }
            CompareStatusFormatter.formatStatus = function (compareStatus) {
                var status;
                switch (compareStatus) {
                    case 0 /* NEW */:
                        status = "Offline";
                        break;
                    case 2 /* NEWER */:
                        status = "Modified";
                        break;
                    case 3 /* OLDER */:
                        status = "Behind";
                        break;
                    case 10 /* UNKNOWN */:
                        status = "Unknown";
                        break;
                    case 4 /* PENDING_DELETE */:
                        status = "Pending delete";
                        break;
                    case 6 /* EQUAL */:
                        status = "Online";
                        break;
                    case 7 /* MOVED */:
                        status = "Moved";
                        break;
                    case 5 /* PENDING_DELETE_TARGET */:
                        status = "Deleted in prod";
                        break;
                    case 1 /* NEW_TARGET */:
                        status = "New in prod";
                        break;
                    case 8 /* CONFLICT_PATH_EXISTS */:
                        status = "Conflict";
                        break;
                    default:
                        status = "Unknown";
                }
                if (!!CompareStatus[status]) {
                    return "Unknown";
                }
                return status;
            };
            return CompareStatusFormatter;
        })();
        content.CompareStatusFormatter = CompareStatusFormatter;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        (function (Branch) {
            Branch[Branch["DRAFT"] = 0] = "DRAFT";
            Branch[Branch["MASTER"] = 1] = "MASTER";
        })(content.Branch || (content.Branch = {}));
        var Branch = content.Branch;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentSummaryAndCompareStatus = (function () {
            function ContentSummaryAndCompareStatus() {
            }
            ContentSummaryAndCompareStatus.fromContentSummary = function (contentSummary) {
                return new ContentSummaryAndCompareStatus().setContentSummary(contentSummary);
            };
            ContentSummaryAndCompareStatus.fromContentAndCompareStatus = function (contentSummary, compareStatus) {
                return new ContentSummaryAndCompareStatus().setContentSummary(contentSummary).setCompareStatus(compareStatus);
            };
            ContentSummaryAndCompareStatus.fromUploadItem = function (item) {
                return new ContentSummaryAndCompareStatus().setUploadItem(item);
            };
            ContentSummaryAndCompareStatus.prototype.getContentSummary = function () {
                return this.contentSummary;
            };
            ContentSummaryAndCompareStatus.prototype.setContentSummary = function (contentSummary) {
                this.contentSummary = contentSummary;
                return this;
            };
            ContentSummaryAndCompareStatus.prototype.getCompareStatus = function () {
                return this.compareStatus;
            };
            ContentSummaryAndCompareStatus.prototype.setCompareStatus = function (status) {
                this.compareStatus = status;
                return this;
            };
            ContentSummaryAndCompareStatus.prototype.getUploadItem = function () {
                return this.uploadItem;
            };
            ContentSummaryAndCompareStatus.prototype.setUploadItem = function (item) {
                var _this = this;
                this.uploadItem = item;
                if (item.isUploaded()) {
                    this.contentSummary = item.getModel();
                }
                else {
                    item.onUploaded(function (contentSummary) {
                        _this.contentSummary = contentSummary;
                    });
                }
                return this;
            };
            ContentSummaryAndCompareStatus.prototype.getContentId = function () {
                return this.contentSummary ? this.contentSummary.getContentId() : null;
            };
            ContentSummaryAndCompareStatus.prototype.getId = function () {
                return (this.contentSummary && this.contentSummary.getId()) || (this.uploadItem && this.uploadItem.getId()) || "";
            };
            ContentSummaryAndCompareStatus.prototype.hasChildren = function () {
                return !!this.contentSummary ? this.contentSummary.hasChildren() : false;
            };
            return ContentSummaryAndCompareStatus;
        })();
        content.ContentSummaryAndCompareStatus = ContentSummaryAndCompareStatus;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var ContentSummaryAndCompareStatusFetcher = (function () {
            function ContentSummaryAndCompareStatusFetcher() {
            }
            ContentSummaryAndCompareStatusFetcher.fetchChildren = function (parentContentId, from, size, childOrder) {
                if (from === void 0) { from = 0; }
                if (size === void 0) { size = -1; }
                var deferred = wemQ.defer();
                new _content.ListContentByIdRequest(parentContentId).setFrom(from).setSize(size).setOrder(childOrder).sendAndParse().then(function (response) {
                    _content.CompareContentRequest.fromContentSummaries(response.getContents()).sendAndParse().then(function (compareResults) {
                        var result = new _content.ContentResponse(ContentSummaryAndCompareStatusFetcher.updateCompareStatus(response.getContents(), compareResults), response.getMetadata());
                        deferred.resolve(result);
                    });
                });
                return deferred.promise;
            };
            ContentSummaryAndCompareStatusFetcher.fetch = function (contentId) {
                var deferred = wemQ.defer();
                new _content.GetContentByIdRequest(contentId).sendAndParse().then(function (content) {
                    _content.CompareContentRequest.fromContentSummaries([content]).sendAndParse().then(function (compareResults) {
                        deferred.resolve(ContentSummaryAndCompareStatusFetcher.updateCompareStatus([content], compareResults)[0]);
                    });
                });
                return deferred.promise;
            };
            ContentSummaryAndCompareStatusFetcher.fetchByContent = function (content) {
                var deferred = wemQ.defer();
                _content.CompareContentRequest.fromContentSummaries([content]).sendAndParse().then(function (compareResults) {
                    deferred.resolve(ContentSummaryAndCompareStatusFetcher.updateCompareStatus([content], compareResults)[0]);
                });
                return deferred.promise;
            };
            ContentSummaryAndCompareStatusFetcher.fetchByPaths = function (paths) {
                var deferred = wemQ.defer();
                new _content.BatchContentRequest().setContentPaths(paths).sendAndParse().then(function (response) {
                    var contentSummaries = response.getContents();
                    _content.CompareContentRequest.fromContentSummaries(contentSummaries).sendAndParse().then(function (compareResults) {
                        deferred.resolve(ContentSummaryAndCompareStatusFetcher.updateCompareStatus(contentSummaries, compareResults));
                    });
                });
                return deferred.promise;
            };
            ContentSummaryAndCompareStatusFetcher.fetchByIds = function (ids) {
                var deferred = wemQ.defer();
                new _content.GetContentSummaryByIds(ids).get().then(function (contentSummaries) {
                    _content.CompareContentRequest.fromContentSummaries(contentSummaries).sendAndParse().then(function (compareResults) {
                        deferred.resolve(ContentSummaryAndCompareStatusFetcher.updateCompareStatus(contentSummaries, compareResults));
                    });
                });
                return deferred.promise;
            };
            ContentSummaryAndCompareStatusFetcher.fetchStatus = function (contentSummaries) {
                var deferred = wemQ.defer();
                _content.CompareContentRequest.fromContentSummaries(contentSummaries).sendAndParse().then(function (compareResults) {
                    deferred.resolve(ContentSummaryAndCompareStatusFetcher.updateCompareStatus(contentSummaries, compareResults));
                });
                return deferred.promise;
            };
            ContentSummaryAndCompareStatusFetcher.fetchChildrenIds = function (parentContentId, from, size) {
                if (from === void 0) { from = 0; }
                if (size === void 0) { size = -1; }
                var deferred = wemQ.defer();
                new _content.ListContentByIdRequest(parentContentId).setFrom(from).setSize(size).setExpand(0 /* NONE */).sendAndParse().then(function (response) {
                    deferred.resolve(response);
                });
                return deferred.promise;
            };
            ContentSummaryAndCompareStatusFetcher.updateCompareStatus = function (contentSummaries, compareResults) {
                var list = [];
                contentSummaries.forEach(function (contentSummary) {
                    var compareResult = compareResults.get(contentSummary.getId());
                    var newEntry = _content.ContentSummaryAndCompareStatus.fromContentAndCompareStatus(contentSummary, compareResult.getCompareStatus());
                    list.push(newEntry);
                });
                return list;
            };
            return ContentSummaryAndCompareStatusFetcher;
        })();
        _content.ContentSummaryAndCompareStatusFetcher = ContentSummaryAndCompareStatusFetcher;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentSummaryAndCompareStatusViewer = (function (_super) {
            __extends(ContentSummaryAndCompareStatusViewer, _super);
            function ContentSummaryAndCompareStatusViewer() {
                _super.call(this, "content-summary-and-compare-status-viewer");
            }
            ContentSummaryAndCompareStatusViewer.prototype.resolveDisplayName = function (object) {
                var contentSummary = object.getContentSummary(), uploadItem = object.getUploadItem();
                if (contentSummary) {
                    return contentSummary.getDisplayName();
                }
                else if (uploadItem) {
                    return uploadItem.getName();
                }
                return "";
            };
            ContentSummaryAndCompareStatusViewer.prototype.resolveUnnamedDisplayName = function (object) {
                var contentSummary = object.getContentSummary();
                return (contentSummary && contentSummary.getType()) ? contentSummary.getType().getLocalName() : "";
            };
            ContentSummaryAndCompareStatusViewer.prototype.resolveSubName = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                var contentSummary = object.getContentSummary(), uploadItem = object.getUploadItem();
                if (contentSummary) {
                    var contentName = contentSummary.getName(), invalid = !contentSummary.isValid() || !contentSummary.getDisplayName() || contentName.isUnnamed(), pendingDelete = contentSummary.getContentState().isPendingDelete();
                    this.toggleClass("invalid", invalid);
                    this.toggleClass("pending-delete", pendingDelete);
                    if (relativePath) {
                        return !contentName.isUnnamed() ? contentName.toString() : api.ui.NamesAndIconViewer.EMPTY_SUB_NAME;
                    }
                    else {
                        return !contentName.isUnnamed() ? contentSummary.getPath().toString() : content.ContentPath.fromParent(contentSummary.getPath().getParentPath(), api.ui.NamesAndIconViewer.EMPTY_SUB_NAME).toString();
                    }
                }
                else if (uploadItem) {
                    return uploadItem.getName();
                }
                return "";
            };
            ContentSummaryAndCompareStatusViewer.prototype.resolveSubTitle = function (object) {
                var contentSummary = object.getContentSummary();
                return !!contentSummary ? contentSummary.getPath().toString() : "";
            };
            ContentSummaryAndCompareStatusViewer.prototype.resolveIconClass = function (object) {
                return !!object.getUploadItem() ? "icon-file-upload2" : "";
            };
            ContentSummaryAndCompareStatusViewer.prototype.resolveIconUrl = function (object) {
                var contentSummary = object.getContentSummary();
                return !!contentSummary ? new content.ContentIconUrlResolver().setContent(contentSummary).resolve() : "";
            };
            return ContentSummaryAndCompareStatusViewer;
        })(api.ui.NamesAndIconViewer);
        content.ContentSummaryAndCompareStatusViewer = ContentSummaryAndCompareStatusViewer;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentVersion = (function () {
            function ContentVersion() {
            }
            ContentVersion.fromJson = function (contentVersionJson, workspaces) {
                var contentVersion = new ContentVersion();
                contentVersion.modifier = contentVersionJson.modifier;
                contentVersion.displayName = contentVersionJson.displayName;
                contentVersion.modified = new Date(contentVersionJson.modified);
                contentVersion.modifierDisplayName = contentVersionJson.modifierDisplayName;
                contentVersion.comment = contentVersionJson.comment;
                contentVersion.id = contentVersionJson.id;
                contentVersion.workspaces = workspaces || [];
                return contentVersion;
            };
            return ContentVersion;
        })();
        content.ContentVersion = ContentVersion;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentVersionViewer = (function (_super) {
            __extends(ContentVersionViewer, _super);
            function ContentVersionViewer() {
                _super.call(this);
                this.namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(2 /* small */).build();
                this.appendChild(this.namesAndIconView);
            }
            ContentVersionViewer.prototype.getModifiedString = function (modified) {
                var timeDiff = Math.abs(Date.now() - modified.getTime());
                var secInMs = 1000;
                var minInMs = secInMs * 60;
                var hrInMs = minInMs * 60;
                var dayInMs = hrInMs * 24;
                var monInMs = dayInMs * 31;
                var yrInMs = dayInMs * 365;
                if (timeDiff < minInMs) {
                    return "less than a minute ago";
                }
                else if (timeDiff < 2 * minInMs) {
                    return "a minute ago";
                }
                else if (timeDiff < hrInMs) {
                    return ~~(timeDiff / minInMs) + " minutes ago";
                }
                else if (timeDiff < 2 * hrInMs) {
                    return "over an hour ago";
                }
                else if (timeDiff < dayInMs) {
                    return "over " + ~~(timeDiff / hrInMs) + " hours ago";
                }
                else if (timeDiff < 2 * dayInMs) {
                    return "over a day ago";
                }
                else if (timeDiff < monInMs) {
                    return "over " + ~~(timeDiff / dayInMs) + " days ago";
                }
                else if (timeDiff < 2 * monInMs) {
                    return "over a month ago";
                }
                else if (timeDiff < yrInMs) {
                    return "over " + ~~(timeDiff / monInMs) + " months ago";
                }
                else if (timeDiff < 2 * yrInMs) {
                    return "over a year ago";
                }
                return "over " + ~~(timeDiff / yrInMs) + " years ago";
            };
            ContentVersionViewer.prototype.getModifierSpan = function (contentVersion) {
                var span = new api.dom.SpanEl("version-modifier");
                span.setHtml(this.getModifiedString(contentVersion.modified));
                return span;
            };
            ContentVersionViewer.prototype.getCommentSpan = function (contentVersion) {
                if (contentVersion.comment.length = 0) {
                    return null;
                }
                var span = new api.dom.SpanEl("version-comment");
                span.setHtml(contentVersion.comment);
                return span;
            };
            ContentVersionViewer.prototype.getSubNameElements = function (contentVersion) {
                var elements = [this.getModifierSpan(contentVersion)];
                return elements;
            };
            ContentVersionViewer.prototype.setObject = function (contentVersion, row) {
                _super.prototype.setObject.call(this, contentVersion);
                this.namesAndIconView.setMainName(contentVersion.modifierDisplayName).setSubNameElements(this.getSubNameElements(contentVersion)).setIconClass("icon-user");
            };
            return ContentVersionViewer;
        })(api.ui.Viewer);
        content.ContentVersionViewer = ContentVersionViewer;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetActiveContentVersionsRequest = (function (_super) {
            __extends(GetActiveContentVersionsRequest, _super);
            function GetActiveContentVersionsRequest(id) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.id = id;
            }
            GetActiveContentVersionsRequest.prototype.getParams = function () {
                return {
                    id: this.id.toString()
                };
            };
            GetActiveContentVersionsRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'getActiveVersions');
            };
            GetActiveContentVersionsRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContentVersions(response.getResult().activeContentVersions);
                });
            };
            GetActiveContentVersionsRequest.prototype.fromJsonToContentVersions = function (json) {
                var contentVersionJson;
                var contentVersion;
                var contentVersionsMap = {};
                json.forEach(function (activeContentVersion) {
                    contentVersionJson = activeContentVersion.contentVersion;
                    contentVersion = contentVersionsMap[contentVersionJson.id];
                    if (!contentVersion) {
                        contentVersion = content.ContentVersion.fromJson(contentVersionJson, [activeContentVersion.branch]);
                        contentVersionsMap[contentVersion.id] = contentVersion;
                    }
                    else {
                        contentVersion.workspaces.push(activeContentVersion.branch);
                    }
                });
                return Object.keys(contentVersionsMap).map(function (key) {
                    return contentVersionsMap[key];
                });
            };
            return GetActiveContentVersionsRequest;
        })(content.ContentResourceRequest);
        content.GetActiveContentVersionsRequest = GetActiveContentVersionsRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetContentVersionsRequest = (function (_super) {
            __extends(GetContentVersionsRequest, _super);
            function GetContentVersionsRequest(contentId) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.contentId = contentId;
            }
            GetContentVersionsRequest.prototype.setFrom = function (from) {
                this.from = from;
                return this;
            };
            GetContentVersionsRequest.prototype.setSize = function (size) {
                this.size = size;
                return this;
            };
            GetContentVersionsRequest.prototype.getParams = function () {
                return {
                    contentId: this.contentId.toString(),
                    from: this.from,
                    size: this.size
                };
            };
            GetContentVersionsRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), 'getVersions');
            };
            GetContentVersionsRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return _this.fromJsonToContentVersions(response.getResult().contentVersions);
                });
            };
            GetContentVersionsRequest.prototype.fromJsonToContentVersions = function (json) {
                var contentVersions = [];
                json.forEach(function (contentVersionJson) {
                    contentVersions.push(content.ContentVersion.fromJson(contentVersionJson));
                });
                return contentVersions;
            };
            return GetContentVersionsRequest;
        })(content.ContentResourceRequest);
        content.GetContentVersionsRequest = GetContentVersionsRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetNearestSiteRequest = (function (_super) {
            __extends(GetNearestSiteRequest, _super);
            function GetNearestSiteRequest(contentId) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "POST");
                this.contentId = contentId;
            }
            GetNearestSiteRequest.prototype.getParams = function () {
                return {
                    contentId: this.contentId.toString()
                };
            };
            GetNearestSiteRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "nearestSite");
            };
            GetNearestSiteRequest.prototype.sendAndParse = function () {
                var _this = this;
                return this.send().then(function (response) {
                    return response.isBlank() ? null : _this.fromJsonToContent(response.getResult());
                });
            };
            return GetNearestSiteRequest;
        })(content.ContentResourceRequest);
        content.GetNearestSiteRequest = GetNearestSiteRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var ContentNamedEvent = (function (_super) {
            __extends(ContentNamedEvent, _super);
            function ContentNamedEvent(wizard, content) {
                _super.call(this);
                this.wizard = wizard;
                this.content = content;
            }
            ContentNamedEvent.prototype.getWizard = function () {
                return this.wizard;
            };
            ContentNamedEvent.prototype.getContent = function () {
                return this.content;
            };
            ContentNamedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentNamedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ContentNamedEvent;
        })(api.event.Event);
        _content.ContentNamedEvent = ContentNamedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var OrderChildMovement = (function () {
            function OrderChildMovement(id, moveBeforeId) {
                this.contentId = id;
                this.moveBefore = moveBeforeId;
            }
            OrderChildMovement.prototype.getContentId = function () {
                return this.contentId;
            };
            OrderChildMovement.prototype.getMoveBefore = function () {
                return this.moveBefore;
            };
            OrderChildMovement.prototype.toJson = function () {
                return {
                    "contentId": this.contentId.toString(),
                    "moveBefore": !!this.moveBefore ? this.moveBefore.toString() : ""
                };
            };
            return OrderChildMovement;
        })();
        content.OrderChildMovement = OrderChildMovement;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var OrderChildMovements = (function () {
            function OrderChildMovements() {
                this.reorderChildren = [];
            }
            OrderChildMovements.prototype.getReorderChildren = function () {
                return this.reorderChildren;
            };
            OrderChildMovements.prototype.addChildMovement = function (movement) {
                this.reorderChildren.push(movement);
            };
            OrderChildMovements.prototype.toArrayJson = function () {
                var result = [];
                this.reorderChildren.forEach(function (movement) {
                    result.push(movement.toJson());
                });
                return result;
            };
            return OrderChildMovements;
        })();
        content.OrderChildMovements = OrderChildMovements;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ContentTypeName = api.schema.content.ContentTypeName;
        var TYPES_ALLOWED_EVERYWHERE = {};
        [ContentTypeName.UNSTRUCTURED, ContentTypeName.FOLDER, ContentTypeName.SITE, ContentTypeName.SHORTCUT].forEach(function (contentTypeName) {
            TYPES_ALLOWED_EVERYWHERE[contentTypeName.toString()] = contentTypeName;
        });
        var CreateContentFilter = (function () {
            function CreateContentFilter() {
                this.siteApplicationsAllowed = null;
            }
            CreateContentFilter.prototype.siteApplicationsFilter = function (siteApplicationKeys) {
                var _this = this;
                if (siteApplicationKeys == null) {
                    return this;
                }
                this.siteApplicationsAllowed = {};
                siteApplicationKeys.forEach(function (applicationKey) {
                    _this.siteApplicationsAllowed[applicationKey.toString()] = applicationKey;
                });
                return this;
            };
            CreateContentFilter.prototype.isCreateContentAllowed = function (parentContent, contentType) {
                var parentContentIsTemplateFolder = parentContent && parentContent.getType().isTemplateFolder();
                var parentContentIsSite = parentContent && parentContent.getType().isSite();
                var parentContentIsPageTemplate = parentContent && parentContent.getType().isPageTemplate();
                var contentTypeName = contentType.getContentTypeName();
                if (contentType.isAbstract()) {
                    return false;
                }
                else if (parentContentIsPageTemplate) {
                    return false;
                }
                else if (contentTypeName.isTemplateFolder()) {
                    return parentContentIsSite;
                }
                else if (contentTypeName.isPageTemplate()) {
                    return parentContentIsTemplateFolder;
                }
                else if (parentContentIsTemplateFolder) {
                    return contentTypeName.isPageTemplate();
                }
                else if (TYPES_ALLOWED_EVERYWHERE[contentTypeName.toString()]) {
                    return true;
                }
                else if ((!this.siteApplicationsAllowed) || this.siteApplicationsAllowed[contentTypeName.getApplicationKey().toString()]) {
                    return true;
                }
                else {
                    return false;
                }
            };
            return CreateContentFilter;
        })();
        content.CreateContentFilter = CreateContentFilter;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var CountContentsWithDescendantsRequest = (function (_super) {
            __extends(CountContentsWithDescendantsRequest, _super);
            function CountContentsWithDescendantsRequest(contentPath) {
                _super.call(this);
                this.contentPaths = [];
                _super.prototype.setMethod.call(this, "POST");
                if (contentPath) {
                    this.addContentPath(contentPath);
                }
            }
            CountContentsWithDescendantsRequest.prototype.setContentPaths = function (contentPaths) {
                this.contentPaths = contentPaths;
                return this;
            };
            CountContentsWithDescendantsRequest.prototype.addContentPath = function (contentPath) {
                this.contentPaths.push(contentPath);
                return this;
            };
            CountContentsWithDescendantsRequest.prototype.getParams = function () {
                var fn = function (contentPath) {
                    return contentPath.toString();
                };
                return {
                    contentPaths: this.contentPaths.map(fn)
                };
            };
            CountContentsWithDescendantsRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "countContentsWithDescendants");
            };
            CountContentsWithDescendantsRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return response.getResult();
                });
            };
            return CountContentsWithDescendantsRequest;
        })(content.ContentResourceRequest);
        content.CountContentsWithDescendantsRequest = CountContentsWithDescendantsRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var OrderExpr = (function () {
            function OrderExpr(builder) {
                this.direction = builder.direction;
            }
            OrderExpr.prototype.getDirection = function () {
                return this.direction;
            };
            OrderExpr.prototype.toJson = function () {
                throw new Error("Must be implemented by inheritors");
            };
            OrderExpr.prototype.toString = function () {
                throw new Error("Must be implemented by inheritors");
            };
            OrderExpr.toArrayJson = function (expressions) {
                var wrappers = [];
                expressions.forEach(function (expr) {
                    if (api.ObjectHelper.iFrameSafeInstanceOf(expr, content.FieldOrderExpr)) {
                        wrappers.push({ "FieldOrderExpr": expr.toJson() });
                    }
                    else if (api.ObjectHelper.iFrameSafeInstanceOf(expr, content.DynamicOrderExpr)) {
                        wrappers.push({ "DynamicOrderExpr": expr.toJson() });
                    }
                });
                return wrappers;
            };
            OrderExpr.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, OrderExpr)) {
                    return false;
                }
                var other = o;
                if (this.direction.toLowerCase() != other.getDirection().toLowerCase()) {
                    return false;
                }
                return true;
            };
            return OrderExpr;
        })();
        content.OrderExpr = OrderExpr;
        var OrderExprBuilder = (function () {
            function OrderExprBuilder(json) {
                if (json) {
                    this.direction = json.direction;
                }
            }
            OrderExprBuilder.prototype.setDirection = function (value) {
                this.direction = value;
                return this;
            };
            OrderExprBuilder.prototype.build = function () {
                return new OrderExpr(this);
            };
            return OrderExprBuilder;
        })();
        content.OrderExprBuilder = OrderExprBuilder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var DynamicOrderExpr = (function (_super) {
            __extends(DynamicOrderExpr, _super);
            function DynamicOrderExpr(builder) {
                _super.call(this, builder);
                this.function = builder.function;
            }
            DynamicOrderExpr.prototype.getFunction = function () {
                return this.function;
            };
            DynamicOrderExpr.prototype.toString = function () {
                return this.function + " " + _super.prototype.getDirection.call(this);
            };
            DynamicOrderExpr.prototype.toJson = function () {
                return {
                    "function": this.function,
                    "direction": this.getDirection()
                };
            };
            return DynamicOrderExpr;
        })(content.OrderExpr);
        content.DynamicOrderExpr = DynamicOrderExpr;
        var DynamicOrderExprBuilder = (function (_super) {
            __extends(DynamicOrderExprBuilder, _super);
            function DynamicOrderExprBuilder(json) {
                _super.call(this, json);
                this.function = json.function;
            }
            DynamicOrderExprBuilder.prototype.setFunction = function (value) {
                this.function = value;
                return this;
            };
            DynamicOrderExprBuilder.prototype.build = function () {
                return new DynamicOrderExpr(this);
            };
            return DynamicOrderExprBuilder;
        })(content.OrderExprBuilder);
        content.DynamicOrderExprBuilder = DynamicOrderExprBuilder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var FieldOrderExpr = (function (_super) {
            __extends(FieldOrderExpr, _super);
            function FieldOrderExpr(builder) {
                _super.call(this, builder);
                this.fieldName = builder.fieldName;
            }
            FieldOrderExpr.prototype.getFieldName = function () {
                return this.fieldName;
            };
            FieldOrderExpr.prototype.toJson = function () {
                return {
                    "fieldName": this.fieldName,
                    "direction": this.getDirection()
                };
            };
            FieldOrderExpr.prototype.toString = function () {
                return this.fieldName + " " + _super.prototype.getDirection.call(this);
            };
            FieldOrderExpr.prototype.equals = function (o) {
                if (!_super.prototype.equals.call(this, o)) {
                    return false;
                }
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, FieldOrderExpr)) {
                    return false;
                }
                var other = o;
                if (this.fieldName.toLowerCase() != other.getFieldName().toLowerCase()) {
                    return false;
                }
                return true;
            };
            return FieldOrderExpr;
        })(content.OrderExpr);
        content.FieldOrderExpr = FieldOrderExpr;
        var FieldOrderExprBuilder = (function (_super) {
            __extends(FieldOrderExprBuilder, _super);
            function FieldOrderExprBuilder(json) {
                _super.call(this, json);
                if (json) {
                    this.fieldName = json.fieldName;
                }
            }
            FieldOrderExprBuilder.prototype.setFieldName = function (value) {
                this.fieldName = value;
                return this;
            };
            FieldOrderExprBuilder.prototype.build = function () {
                return new FieldOrderExpr(this);
            };
            return FieldOrderExprBuilder;
        })(content.OrderExprBuilder);
        content.FieldOrderExprBuilder = FieldOrderExprBuilder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ChildOrder = (function () {
            function ChildOrder() {
                this.DEFAULT_ORDER_DIRECTION_VALUE = "DESC";
                this.orderExpressions = [];
            }
            ChildOrder.prototype.getOrderExpressions = function () {
                return this.orderExpressions;
            };
            ChildOrder.prototype.addOrderExpr = function (expr) {
                this.orderExpressions.push(expr);
            };
            ChildOrder.prototype.addOrderExpressions = function (expressions) {
                var _this = this;
                expressions.forEach(function (expr) {
                    _this.orderExpressions.push(expr);
                });
            };
            ChildOrder.fromJson = function (childOrderJson) {
                var childOrder = new ChildOrder();
                childOrderJson.orderExpressions.forEach(function (orderExprJson) {
                    if (orderExprJson.FieldOrderExpr) {
                        childOrder.orderExpressions.push(new content.FieldOrderExprBuilder(orderExprJson.FieldOrderExpr).build());
                    }
                    else if (orderExprJson.DynamicOrderExpr) {
                        childOrder.orderExpressions.push(new content.DynamicOrderExprBuilder(orderExprJson.DynamicOrderExpr).build());
                    }
                });
                return childOrder;
            };
            ChildOrder.prototype.isManual = function () {
                if (this.orderExpressions.length == 0) {
                    return false;
                }
                var order = this.orderExpressions[0];
                if (api.ObjectHelper.iFrameSafeInstanceOf(order, content.FieldOrderExpr)) {
                    return api.ObjectHelper.stringEquals(ChildOrder.MANUAL_ORDER_VALUE_KEY.toLowerCase(), order.getFieldName().toLowerCase());
                }
                return false;
            };
            ChildOrder.prototype.isDesc = function () {
                if (this.orderExpressions.length == 0) {
                    return this.DEFAULT_ORDER_DIRECTION_VALUE == ChildOrder.DESC_ORDER_DIRECTION_VALUE;
                }
                var order = this.orderExpressions[0];
                return api.ObjectHelper.stringEquals(ChildOrder.DESC_ORDER_DIRECTION_VALUE.toLowerCase(), order.getDirection().toLowerCase());
            };
            ChildOrder.prototype.isDefault = function () {
                var order = this.orderExpressions[0];
                if (api.ObjectHelper.iFrameSafeInstanceOf(order, content.FieldOrderExpr)) {
                    var fieldOrder = order;
                    if (api.ObjectHelper.stringEquals(this.DEFAULT_ORDER_DIRECTION_VALUE.toLowerCase(), fieldOrder.getDirection().toLowerCase()) && api.ObjectHelper.stringEquals(ChildOrder.DEFAULT_ORDER_FIELD_VALUE.toLowerCase(), fieldOrder.getFieldName().toLowerCase())) {
                        return true;
                    }
                }
                return false;
            };
            ChildOrder.prototype.toJson = function () {
                return {
                    "orderExpressions": content.OrderExpr.toArrayJson(this.getOrderExpressions())
                };
            };
            ChildOrder.prototype.toString = function () {
                var result = "";
                this.orderExpressions.forEach(function (expr) {
                    result = result.concat(" ", expr.toString());
                });
                return result;
            };
            ChildOrder.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ChildOrder)) {
                    return false;
                }
                var other = o;
                if (this.orderExpressions.length != other.getOrderExpressions().length) {
                    return false;
                }
                for (var count in this.orderExpressions) {
                    if (!this.orderExpressions[count].equals(other.getOrderExpressions()[count])) {
                        return false;
                    }
                }
                return true;
            };
            ChildOrder.toSetChildOrderJson = function (contentId, childOrder, silent) {
                if (contentId && childOrder) {
                    return {
                        "silent": silent,
                        "childOrder": childOrder.toJson(),
                        "contentId": contentId.toString()
                    };
                }
            };
            ChildOrder.DEFAULT_ORDER_FIELD_VALUE = api.query.QueryField.MODIFIED_TIME;
            ChildOrder.ASC_ORDER_DIRECTION_VALUE = "ASC";
            ChildOrder.DESC_ORDER_DIRECTION_VALUE = "DESC";
            ChildOrder.MANUAL_ORDER_VALUE_KEY = api.query.QueryField.MANUAL_ORDER_VALUE;
            return ChildOrder;
        })();
        content.ChildOrder = ChildOrder;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ValueTypes = api.data.ValueTypes;
        (function (MediaUploaderOperation) {
            MediaUploaderOperation[MediaUploaderOperation["create"] = 0] = "create";
            MediaUploaderOperation[MediaUploaderOperation["update"] = 1] = "update";
        })(content.MediaUploaderOperation || (content.MediaUploaderOperation = {}));
        var MediaUploaderOperation = content.MediaUploaderOperation;
        var MediaUploader = (function (_super) {
            __extends(MediaUploader, _super);
            function MediaUploader(config) {
                if (config.url == undefined) {
                    config.url = api.util.UriHelper.getRestUri("content/" + MediaUploaderOperation[config.operation] + "Media");
                }
                _super.call(this, config);
                this.addClass('media-uploader');
            }
            MediaUploader.prototype.createModel = function (serverResponse) {
                if (serverResponse) {
                    return new api.content.ContentBuilder().fromContentJson(serverResponse).build();
                }
                else {
                    return null;
                }
            };
            MediaUploader.prototype.getModelValue = function (item) {
                return item.getId();
            };
            MediaUploader.prototype.getMediaValue = function (item) {
                var mediaProperty = item.getContentData().getProperty("media");
                var mediaValue;
                switch (mediaProperty.getType()) {
                    case ValueTypes.DATA:
                        mediaValue = mediaProperty.getPropertySet().getProperty('attachment').getValue();
                        break;
                    case ValueTypes.STRING:
                        mediaValue = mediaProperty.getValue();
                        break;
                }
                return mediaValue;
            };
            MediaUploader.prototype.setFileName = function (name) {
                this.fileName = name;
            };
            MediaUploader.prototype.createResultItem = function (value) {
                var link = new api.dom.AEl().setUrl(api.util.UriHelper.getRestUri('content/media/' + value), "_blank");
                link.setHtml(this.fileName != null && this.fileName != "" ? this.fileName : value);
                return link;
            };
            return MediaUploader;
        })(api.ui.uploader.Uploader);
        content.MediaUploader = MediaUploader;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var ImageEditor = api.ui.image.ImageEditor;
        var ImageUploader = (function (_super) {
            __extends(ImageUploader, _super);
            function ImageUploader(config) {
                var _this = this;
                if (config.allowTypes == undefined) {
                    config.allowTypes = [
                        { title: 'Image files', extensions: 'jpg,jpeg,gif,png' }
                    ];
                }
                if (config.dropAlwaysAllowed == undefined) {
                    config.dropAlwaysAllowed = true;
                }
                if (config.dropzoneAlwaysVisible == undefined) {
                    config.dropzoneAlwaysVisible = true;
                }
                _super.call(this, config);
                this.scaleWidth = false;
                this.imageEditors = [];
                this.editModeListeners = [];
                this.focusAutoPositionedListeners = [];
                this.cropAutoPositionedListeners = [];
                if (config.scaleWidth != undefined) {
                    this.scaleWidth = config.scaleWidth;
                }
                if (config.allowTypes == undefined) {
                    config.allowTypes = [
                        { title: 'Image files', extensions: 'jpg,gif,png' }
                    ];
                }
                this.addClass('image-uploader');
                this.initialWidth = 0;
                this.onShown(function () {
                    _this.setResetVisible(false);
                    if (_this.getEl().getWidth() == 0) {
                        _this.initialWidth = Math.max(_this.getParentElement().getEl().getWidth(), _this.initialWidth);
                        _this.getEl().setMaxWidthPx(_this.initialWidth);
                    }
                });
                this.onUploadStarted(function () {
                    _this.imageEditors.forEach(function (imageEditor) {
                        imageEditor.remove();
                    });
                    _this.imageEditors = [];
                });
                this.onFocus(function () {
                    setTimeout(function () {
                        if (_this.imageEditors.length && !_this.imageEditors[0].hasClass(ImageUploader.SELECTED_CLASS)) {
                            _this.toggleSelected(_this.imageEditors[0]);
                        }
                    }, 150);
                });
                this.onBlur(function (event) {
                    _this.imageEditors.forEach(function (imageEditor) {
                        if (event.relatedTarget && !imageEditor.isElementInsideButtonsContainer(event.relatedTarget)) {
                            _this.toggleSelected(imageEditor);
                        }
                    });
                });
                this.onClicked(function (event) {
                    _this.imageEditors.forEach(function (imageEditor) {
                        if (event.target && !imageEditor.isElementInsideButtonsContainer(event.target)) {
                            _this.toggleSelected(imageEditor);
                        }
                    });
                });
                api.dom.Body.get().onClicked(function (event) {
                    _this.imageEditors.forEach(function (imageEditor) {
                        if (imageEditor.hasClass(ImageUploader.SELECTED_CLASS) && imageEditor.getImage().getHTMLElement() !== event.target) {
                            imageEditor.removeClass(ImageUploader.SELECTED_CLASS);
                        }
                    });
                });
            }
            ImageUploader.prototype.getSizeValue = function (content, propertyName) {
                var value = 0, metaData = content.getContentData().getProperty('metadata');
                if (metaData && api.data.ValueTypes.DATA.equals(metaData.getType())) {
                    value = parseInt(metaData.getPropertySet().getProperty(propertyName).getString());
                }
                else {
                    var allExtraData = content.getAllExtraData();
                    allExtraData.forEach(function (extraData) {
                        if (!value && extraData.getData().getProperty(propertyName)) {
                            value = parseInt(extraData.getData().getProperty(propertyName).getValue().getString());
                        }
                    });
                }
                return value;
            };
            ImageUploader.prototype.setOriginalDimensions = function (content) {
                this.originalWidth = this.getSizeValue(content, "imageWidth") || this.initialWidth;
                this.originalHeight = this.getSizeValue(content, "imageHeight");
            };
            ImageUploader.prototype.getProportionalHeight = function () {
                if (!this.originalHeight || !this.originalWidth) {
                    return 0;
                }
                return Math.round(this.initialWidth * this.originalHeight / this.originalWidth);
            };
            ImageUploader.prototype.togglePlaceholder = function (flag) {
                var resultEl = this.getResultContainer().toggleClass('placeholder', flag).getEl();
                if (flag) {
                    resultEl.setHeightPx(resultEl.getHeight() || this.getProportionalHeight());
                }
                else {
                    resultEl.setHeight('auto');
                }
            };
            ImageUploader.prototype.createImageEditor = function (imgUrl) {
                var _this = this;
                this.togglePlaceholder(true);
                var imageEditor = new ImageEditor(imgUrl);
                imageEditor.onEditModeChanged(function (edit, crop, zoom, focus) {
                    _this.notifyEditModeChanged(edit, crop, zoom, focus);
                });
                imageEditor.onFocusAutoPositionedChanged(function (auto) { return _this.notifyFocusAutoPositionedChanged(auto); });
                imageEditor.onCropAutoPositionedChanged(function (auto) { return _this.notifyCropAutoPositionedChanged(auto); });
                imageEditor.getImage().onLoaded(function (event) {
                    _this.togglePlaceholder(false);
                });
                imageEditor.getUploadButton().onClicked(function () {
                    wemjq(_this.getDropzone().getEl().getHTMLElement()).simulate("click");
                });
                imageEditor.getLastButtonInContainer().onBlur(function () {
                    _this.toggleSelected(imageEditor);
                });
                var index = -1;
                imageEditor.onEditModeChanged(function (edit, position, zoom, focus) {
                    _this.togglePlaceholder(edit);
                    if (edit) {
                        index = imageEditor.getSiblingIndex();
                        api.dom.Body.get().appendChild(imageEditor.addClass(ImageUploader.STANDOUT_CLASS));
                        _this.positionImageEditor(imageEditor);
                    }
                    else {
                        _this.getResultContainer().insertChild(imageEditor.removeClass(ImageUploader.STANDOUT_CLASS), index);
                    }
                });
                return imageEditor;
            };
            ImageUploader.prototype.positionImageEditor = function (imageEditor) {
                var resultOffset = this.getResultContainer().getEl().getOffset();
                imageEditor.getEl().setTopPx(resultOffset.top).setLeftPx(resultOffset.left);
            };
            ImageUploader.prototype.createResultItem = function (value) {
                if (!this.initialWidth) {
                    this.initialWidth = this.getParentElement().getEl().getWidth();
                }
                var contentId = new api.content.ContentId(value), imgUrl = new _content.ContentImageUrlResolver().setContentId(contentId).setTimestamp(new Date()).setSource(true).resolve();
                var imageEditor = this.createImageEditor(imgUrl);
                imageEditor.onShaderVisibilityChanged(function (visible) {
                    new api.app.wizard.MaskContentWizardPanelEvent(contentId, visible).fire();
                });
                this.imageEditors.push(imageEditor);
                return imageEditor;
            };
            ImageUploader.prototype.toggleSelected = function (imageEditor) {
                imageEditor.toggleClass(ImageUploader.SELECTED_CLASS);
            };
            ImageUploader.prototype.setFocalPoint = function (x, y) {
                this.imageEditors.forEach(function (editor) {
                    editor.setFocusPosition(x, y);
                });
            };
            ImageUploader.prototype.setCrop = function (crop) {
                this.imageEditors.forEach(function (editor) {
                    editor.setCropPosition(crop.x, crop.y, crop.x2, crop.y2);
                });
            };
            ImageUploader.prototype.setZoom = function (zoom) {
                this.imageEditors.forEach(function (editor) {
                    editor.setZoomPosition(zoom.x, zoom.y, zoom.x2, zoom.y2);
                });
            };
            ImageUploader.prototype.isFocalPointEditMode = function () {
                return this.imageEditors.some(function (editor) {
                    return editor.isFocusEditMode();
                });
            };
            ImageUploader.prototype.isCropEditMode = function () {
                return this.imageEditors.some(function (editor) {
                    return editor.isCropEditMode();
                });
            };
            ImageUploader.prototype.onEditModeChanged = function (listener) {
                this.editModeListeners.push(listener);
            };
            ImageUploader.prototype.unEditModeChanged = function (listener) {
                this.editModeListeners = this.editModeListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            ImageUploader.prototype.notifyEditModeChanged = function (edit, crop, zoom, focus) {
                this.editModeListeners.forEach(function (listener) {
                    listener(edit, crop, zoom, focus);
                });
            };
            ImageUploader.prototype.onCropAutoPositionedChanged = function (listener) {
                this.cropAutoPositionedListeners.push(listener);
            };
            ImageUploader.prototype.unCropAutoPositionedChanged = function (listener) {
                this.cropAutoPositionedListeners = this.cropAutoPositionedListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            ImageUploader.prototype.notifyCropAutoPositionedChanged = function (auto) {
                this.cropAutoPositionedListeners.forEach(function (listener) { return listener(auto); });
            };
            ImageUploader.prototype.onFocusAutoPositionedChanged = function (listener) {
                this.focusAutoPositionedListeners.push(listener);
            };
            ImageUploader.prototype.unFocusAutoPositionedChanged = function (listener) {
                this.focusAutoPositionedListeners = this.focusAutoPositionedListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            ImageUploader.prototype.notifyFocusAutoPositionedChanged = function (auto) {
                this.focusAutoPositionedListeners.forEach(function (listener) { return listener(auto); });
            };
            ImageUploader.SELECTED_CLASS = 'selected';
            ImageUploader.STANDOUT_CLASS = 'standout';
            return ImageUploader;
        })(_content.MediaUploader);
        _content.ImageUploader = ImageUploader;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ThumbnailUploader = (function (_super) {
            __extends(ThumbnailUploader, _super);
            function ThumbnailUploader(config) {
                if (config.url == undefined) {
                    config.url = api.util.UriHelper.getRestUri("content/updateThumbnail");
                }
                if (config.showCancel == undefined) {
                    config.showCancel = false;
                }
                if (config.showReset == undefined) {
                    config.showReset = false;
                }
                if (config.dropzoneAlwaysVisible == undefined) {
                    config.dropzoneAlwaysVisible = true;
                }
                if (config.resultAlwaysVisisble == undefined) {
                    config.resultAlwaysVisisble = true;
                }
                if (config.allowTypes == undefined) {
                    config.allowTypes = [
                        { title: 'Image files', extensions: 'jpg,gif,png' }
                    ];
                }
                if (config.allowMultiSelection == undefined) {
                    config.allowMultiSelection = false;
                }
                _super.call(this, config);
                this.addClass('thumbnail-uploader');
                this.iconUrlResolver = new content.ContentIconUrlResolver();
            }
            ThumbnailUploader.prototype.createModel = function (serverResponse) {
                if (serverResponse) {
                    return new api.content.ContentBuilder().fromContentJson(serverResponse).build();
                }
                else {
                    return null;
                }
            };
            ThumbnailUploader.prototype.getModelValue = function (item) {
                return this.iconUrlResolver.setContent(item).resolve();
            };
            ThumbnailUploader.prototype.createResultItem = function (value) {
                return new api.dom.ImgEl(value);
            };
            return ThumbnailUploader;
        })(api.ui.uploader.Uploader);
        content.ThumbnailUploader = ThumbnailUploader;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var EditContentEvent = (function (_super) {
            __extends(EditContentEvent, _super);
            function EditContentEvent(model) {
                this.model = model;
                _super.call(this);
            }
            EditContentEvent.prototype.getModels = function () {
                return this.model;
            };
            EditContentEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            EditContentEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return EditContentEvent;
        })(api.event.Event);
        content.EditContentEvent = EditContentEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var TreeGridItemClickedEvent = (function (_super) {
            __extends(TreeGridItemClickedEvent, _super);
            function TreeGridItemClickedEvent() {
                _super.apply(this, arguments);
            }
            TreeGridItemClickedEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            TreeGridItemClickedEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return TreeGridItemClickedEvent;
        })(api.event.Event);
        content.TreeGridItemClickedEvent = TreeGridItemClickedEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var ContentRequiresSaveEvent = (function (_super) {
            __extends(ContentRequiresSaveEvent, _super);
            function ContentRequiresSaveEvent(content) {
                _super.call(this);
                this.content = content;
            }
            ContentRequiresSaveEvent.prototype.getContent = function () {
                return this.content;
            };
            ContentRequiresSaveEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentRequiresSaveEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ContentRequiresSaveEvent;
        })(api.event.Event);
        _content.ContentRequiresSaveEvent = ContentRequiresSaveEvent;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var ApplicationKey = api.application.ApplicationKey;
        var Widget = (function () {
            function Widget(name, displayName, interfaces, applicationKey, descriptorKeyName) {
                this.name = name;
                this.displayName = displayName;
                this.interfaces = interfaces;
                this.widgetDescriptorKey = new WidgetDescriptorKey(ApplicationKey.fromString(applicationKey), descriptorKeyName);
            }
            Widget.prototype.getName = function () {
                return this.name;
            };
            Widget.prototype.getDisplayName = function () {
                return this.displayName;
            };
            Widget.prototype.getInterfaces = function () {
                return this.interfaces;
            };
            Widget.prototype.getWidgetDescriptorKey = function () {
                return this.widgetDescriptorKey;
            };
            return Widget;
        })();
        content.Widget = Widget;
        var WidgetDescriptorKey = (function () {
            function WidgetDescriptorKey(applicationKey, name) {
                this.applicationKey = applicationKey;
                this.name = name;
                this.refString = applicationKey.toString() + WidgetDescriptorKey.SEPARATOR + name.toString();
            }
            WidgetDescriptorKey.fromString = function (str) {
                var sepIndex = str.indexOf(WidgetDescriptorKey.SEPARATOR);
                if (sepIndex == -1) {
                    throw new Error("WidgetDescriptorKey must contain separator '" + WidgetDescriptorKey.SEPARATOR + "':" + str);
                }
                var applicationKey = str.substring(0, sepIndex);
                var name = str.substring(sepIndex + 1, str.length);
                return new WidgetDescriptorKey(ApplicationKey.fromString(applicationKey), name);
            };
            WidgetDescriptorKey.prototype.getApplicationKey = function () {
                return this.applicationKey;
            };
            WidgetDescriptorKey.prototype.getName = function () {
                return this.name;
            };
            WidgetDescriptorKey.prototype.toString = function () {
                return this.refString;
            };
            WidgetDescriptorKey.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, WidgetDescriptorKey)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                    return false;
                }
                return true;
            };
            WidgetDescriptorKey.SEPARATOR = ":";
            return WidgetDescriptorKey;
        })();
        content.WidgetDescriptorKey = WidgetDescriptorKey;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var WidgetDescriptorResourceRequest = (function (_super) {
            __extends(WidgetDescriptorResourceRequest, _super);
            function WidgetDescriptorResourceRequest() {
                _super.call(this);
                this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "widget/descriptor");
            }
            WidgetDescriptorResourceRequest.prototype.getResourcePath = function () {
                return this.resourcePath;
            };
            WidgetDescriptorResourceRequest.fromJson = function (json) {
                var result = [];
                json.widgetDescriptors.forEach(function (widgetDescriptor) {
                    result.push(new content.Widget(widgetDescriptor.name, widgetDescriptor.displayName, widgetDescriptor.interfaces, widgetDescriptor.key.applicationKey.name, widgetDescriptor.key.name));
                });
                return result;
            };
            return WidgetDescriptorResourceRequest;
        })(api.rest.ResourceRequest);
        content.WidgetDescriptorResourceRequest = WidgetDescriptorResourceRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var GetWidgetsByInterfaceRequest = (function (_super) {
            __extends(GetWidgetsByInterfaceRequest, _super);
            function GetWidgetsByInterfaceRequest(widgetInterface) {
                _super.call(this);
                _super.prototype.setMethod.call(this, "GET");
                this.widgetInterface = widgetInterface;
            }
            GetWidgetsByInterfaceRequest.prototype.getParams = function () {
                return {
                    "interface": this.widgetInterface
                };
            };
            GetWidgetsByInterfaceRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "byInterface");
            };
            GetWidgetsByInterfaceRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return content.WidgetDescriptorResourceRequest.fromJson(response.getResult());
                });
            };
            return GetWidgetsByInterfaceRequest;
        })(content.WidgetDescriptorResourceRequest);
        content.GetWidgetsByInterfaceRequest = GetWidgetsByInterfaceRequest;
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var query;
        (function (query) {
            var ContentQuery = (function () {
                function ContentQuery() {
                    this.contentTypeNames = [];
                    this.aggregationQueries = [];
                    this.queryFilters = [];
                    this.from = 0;
                    this.size = ContentQuery.DEFAULT_SIZE;
                }
                ContentQuery.prototype.setQueryExpr = function (queryExpr) {
                    this.queryExpr = queryExpr;
                };
                ContentQuery.prototype.getQueryExpr = function () {
                    return this.queryExpr;
                };
                ContentQuery.prototype.setContentTypeNames = function (contentTypeNames) {
                    this.contentTypeNames = contentTypeNames;
                };
                ContentQuery.prototype.getContentTypes = function () {
                    return this.contentTypeNames;
                };
                ContentQuery.prototype.setFrom = function (from) {
                    this.from = from;
                };
                ContentQuery.prototype.getFrom = function () {
                    return this.from;
                };
                ContentQuery.prototype.setSize = function (size) {
                    this.size = size;
                };
                ContentQuery.prototype.getSize = function () {
                    return this.size;
                };
                ContentQuery.prototype.addAggregationQuery = function (aggregationQuery) {
                    this.aggregationQueries.push(aggregationQuery);
                };
                ContentQuery.prototype.getAggregationQueries = function () {
                    return this.aggregationQueries;
                };
                ContentQuery.prototype.addQueryFilter = function (queryFilter) {
                    this.queryFilters.push(queryFilter);
                };
                ContentQuery.prototype.getQueryFilters = function () {
                    return this.queryFilters;
                };
                ContentQuery.POSTLOAD_SIZE = 10;
                ContentQuery.DEFAULT_SIZE = 100;
                return ContentQuery;
            })();
            query.ContentQuery = ContentQuery;
        })(query = content.query || (content.query = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var attachment;
        (function (attachment) {
            var AttachmentName = (function () {
                function AttachmentName(fileName) {
                    this.fileName = fileName;
                }
                AttachmentName.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, AttachmentName)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.stringEquals(this.fileName, other.fileName)) {
                        return false;
                    }
                    return true;
                };
                AttachmentName.prototype.toString = function () {
                    return this.fileName;
                };
                return AttachmentName;
            })();
            attachment.AttachmentName = AttachmentName;
        })(attachment = content.attachment || (content.attachment = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var attachment;
        (function (attachment) {
            var Attachment = (function () {
                function Attachment(builder) {
                    this.name = builder.name;
                    this.label = builder.label;
                    this.mimeType = builder.mimeType;
                    this.size = builder.size;
                }
                Attachment.prototype.getBinaryReference = function () {
                    return new api.util.BinaryReference(this.name.toString());
                };
                Attachment.prototype.getName = function () {
                    return this.name;
                };
                Attachment.prototype.getLabel = function () {
                    return this.label;
                };
                Attachment.prototype.getMimeType = function () {
                    return this.mimeType;
                };
                Attachment.prototype.getSize = function () {
                    return this.size;
                };
                Attachment.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Attachment)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.equals(this.name, other.name)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringEquals(this.label, other.label)) {
                        return false;
                    }
                    if (!api.ObjectHelper.stringEquals(this.mimeType, other.mimeType)) {
                        return false;
                    }
                    if (!api.ObjectHelper.numberEquals(this.size, other.size)) {
                        return false;
                    }
                    return true;
                };
                Attachment.prototype.toJson = function () {
                    return {
                        "name": this.getName().toString(),
                        "label": this.getLabel(),
                        "mimeType": this.getMimeType(),
                        "size": this.getSize()
                    };
                };
                Attachment.create = function () {
                    return new AttachmentBuilder();
                };
                return Attachment;
            })();
            attachment.Attachment = Attachment;
            var AttachmentBuilder = (function () {
                function AttachmentBuilder() {
                }
                AttachmentBuilder.prototype.fromJson = function (json) {
                    this.setName(new attachment.AttachmentName(json.name)).setLabel(json.label).setSize(json.size).setMimeType(json.mimeType);
                    return this;
                };
                AttachmentBuilder.prototype.setName = function (value) {
                    this.name = value;
                    return this;
                };
                AttachmentBuilder.prototype.setLabel = function (value) {
                    this.label = value;
                    return this;
                };
                AttachmentBuilder.prototype.setMimeType = function (value) {
                    this.mimeType = value;
                    return this;
                };
                AttachmentBuilder.prototype.setSize = function (value) {
                    this.size = value;
                    return this;
                };
                AttachmentBuilder.prototype.build = function () {
                    return new Attachment(this);
                };
                return AttachmentBuilder;
            })();
            attachment.AttachmentBuilder = AttachmentBuilder;
        })(attachment = content.attachment || (content.attachment = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var attachment;
        (function (_attachment) {
            var Attachments = (function () {
                function Attachments(builder) {
                    var _this = this;
                    this.attachmentByName = {};
                    this.attachmentByLabel = {};
                    this.attachments = [];
                    var count = 0;
                    builder.attachments.forEach(function (attachment) {
                        _this.attachmentByName[attachment.getName().toString()] = attachment;
                        _this.attachmentByLabel[attachment.getLabel()] = attachment;
                        count++;
                        _this.attachments.push(attachment);
                    });
                    this.size = count;
                }
                Attachments.prototype.forEach = function (callBack) {
                    this.attachments.forEach(function (attachment, index) {
                        callBack(attachment, index);
                    });
                };
                Attachments.prototype.getAttachmentByName = function (name) {
                    return this.attachmentByName[name];
                };
                Attachments.prototype.getAttachmentByLabel = function (label) {
                    return this.attachmentByLabel[name];
                };
                Attachments.prototype.getAttachment = function (index) {
                    return this.attachments[index];
                };
                Attachments.prototype.getSize = function () {
                    return this.size;
                };
                Attachments.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Attachments)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.arrayEquals(this.attachments, other.attachments)) {
                        return false;
                    }
                    return true;
                };
                return Attachments;
            })();
            _attachment.Attachments = Attachments;
            var AttachmentsBuilder = (function () {
                function AttachmentsBuilder() {
                    this.attachments = [];
                }
                AttachmentsBuilder.prototype.fromJson = function (jsons) {
                    var _this = this;
                    jsons.forEach(function (json) {
                        _this.attachments.push(new _attachment.AttachmentBuilder().fromJson(json).build());
                    });
                    return this;
                };
                AttachmentsBuilder.prototype.add = function (value) {
                    this.attachments.push(value);
                    return this;
                };
                AttachmentsBuilder.prototype.addAll = function (value) {
                    var _this = this;
                    value.forEach(function (attachment) {
                        _this.attachments.push(attachment);
                    });
                    return this;
                };
                AttachmentsBuilder.prototype.build = function () {
                    return new Attachments(this);
                };
                return AttachmentsBuilder;
            })();
            _attachment.AttachmentsBuilder = AttachmentsBuilder;
        })(attachment = content.attachment || (content.attachment = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var image;
        (function (image) {
            var PropertyTree = api.data.PropertyTree;
            var ImageContentDataFactory = (function () {
                function ImageContentDataFactory() {
                }
                ImageContentDataFactory.prototype.setImage = function (value) {
                    this.image = value;
                    return this;
                };
                ImageContentDataFactory.prototype.setMimeType = function (value) {
                    this.mimeType = value;
                    return this;
                };
                ImageContentDataFactory.prototype.create = function () {
                    var data = new PropertyTree();
                    data.addString("image", this.image.toString());
                    data.addString("mimeType", this.mimeType);
                    return data;
                };
                return ImageContentDataFactory;
            })();
            image.ImageContentDataFactory = ImageContentDataFactory;
        })(image = content.image || (content.image = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var DescriptorName = (function () {
                function DescriptorName(name) {
                    if (name == null) {
                        throw new Error("name cannot be null");
                    }
                    this.name = name;
                }
                DescriptorName.prototype.toString = function () {
                    return this.name;
                };
                return DescriptorName;
            })();
            page.DescriptorName = DescriptorName;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var Descriptor = (function () {
                function Descriptor(builder) {
                    this.name = builder.name;
                    this.key = builder.key;
                    this.displayName = builder.displayName;
                    this.config = builder.config;
                }
                Descriptor.prototype.getKey = function () {
                    return this.key;
                };
                Descriptor.prototype.getName = function () {
                    return this.name;
                };
                Descriptor.prototype.getDisplayName = function () {
                    return this.displayName;
                };
                Descriptor.prototype.getConfig = function () {
                    return this.config;
                };
                Descriptor.prototype.clone = function () {
                    throw new Error("Must be implemented in inheritor");
                };
                return Descriptor;
            })();
            page.Descriptor = Descriptor;
            var DescriptorBuilder = (function () {
                function DescriptorBuilder(source) {
                    if (source) {
                        this.key = source.getKey();
                        this.name = source.getName();
                        this.displayName = source.getDisplayName();
                        this.config = source.getConfig();
                    }
                }
                return DescriptorBuilder;
            })();
            page.DescriptorBuilder = DescriptorBuilder;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var DescriptorKey = (function () {
                function DescriptorKey(applicationKey, name) {
                    this.applicationKey = applicationKey;
                    this.name = name;
                    this.refString = applicationKey.toString() + DescriptorKey.SEPARATOR + name.toString();
                }
                DescriptorKey.fromString = function (str) {
                    var sepIndex = str.indexOf(DescriptorKey.SEPARATOR);
                    if (sepIndex == -1) {
                        throw new Error("DescriptorKey must contain separator '" + DescriptorKey.SEPARATOR + "':" + str);
                    }
                    var applicationKey = str.substring(0, sepIndex);
                    var name = str.substring(sepIndex + 1, str.length);
                    return new DescriptorKey(api.application.ApplicationKey.fromString(applicationKey), new page.DescriptorName(name));
                };
                DescriptorKey.prototype.getApplicationKey = function () {
                    return this.applicationKey;
                };
                DescriptorKey.prototype.getName = function () {
                    return this.name;
                };
                DescriptorKey.prototype.toString = function () {
                    return this.refString;
                };
                DescriptorKey.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, DescriptorKey)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                        return false;
                    }
                    return true;
                };
                DescriptorKey.SEPARATOR = ":";
                return DescriptorKey;
            })();
            page.DescriptorKey = DescriptorKey;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageDescriptor = (function (_super) {
                __extends(PageDescriptor, _super);
                function PageDescriptor(builder) {
                    _super.call(this, builder);
                    this.regions = builder.regions;
                }
                PageDescriptor.prototype.getRegions = function () {
                    return this.regions;
                };
                PageDescriptor.prototype.clone = function () {
                    return new PageDescriptorBuilder(this).build();
                };
                return PageDescriptor;
            })(page.Descriptor);
            page.PageDescriptor = PageDescriptor;
            var PageDescriptorBuilder = (function (_super) {
                __extends(PageDescriptorBuilder, _super);
                function PageDescriptorBuilder(source) {
                    this.regions = [];
                    if (source) {
                        _super.call(this, source);
                        this.regions = source.getRegions();
                    }
                }
                PageDescriptorBuilder.prototype.fromJson = function (json) {
                    this.setName(new page.DescriptorName(json.name));
                    this.setDisplayName(json.displayName);
                    this.setConfig(json.config != null ? api.form.Form.fromJson(json.config) : null);
                    this.setKey(page.DescriptorKey.fromString(json.key));
                    for (var i = 0; i < json.regions.length; i++) {
                        var region = new api.content.page.region.RegionDescriptorBuilder().fromJson(json.regions[i]).build();
                        this.regions.push(region);
                    }
                    return this;
                };
                PageDescriptorBuilder.prototype.setKey = function (key) {
                    this.key = key;
                    return this;
                };
                PageDescriptorBuilder.prototype.setName = function (value) {
                    this.name = value;
                    return this;
                };
                PageDescriptorBuilder.prototype.setDisplayName = function (value) {
                    this.displayName = value;
                    return this;
                };
                PageDescriptorBuilder.prototype.setConfig = function (value) {
                    this.config = value;
                    return this;
                };
                PageDescriptorBuilder.prototype.addRegion = function (value) {
                    this.regions.push(value);
                    return this;
                };
                PageDescriptorBuilder.prototype.build = function () {
                    return new PageDescriptor(this);
                };
                return PageDescriptorBuilder;
            })(page.DescriptorBuilder);
            page.PageDescriptorBuilder = PageDescriptorBuilder;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var ApplicationBasedCache = api.application.ApplicationBasedCache;
            var PageDescriptorCache = (function (_super) {
                __extends(PageDescriptorCache, _super);
                function PageDescriptorCache() {
                    if (PageDescriptorCache.instance) {
                        throw new Error("Instantiation failed: Use PageDescriptorCache.get() instead!");
                    }
                    _super.call(this);
                }
                PageDescriptorCache.get = function () {
                    var w = api.dom.WindowDOM.get();
                    var topWindow = w.getTopParent().asWindow();
                    if (!topWindow.api.content.page.PageDescriptorCache.instance) {
                        topWindow.api.content.page.PageDescriptorCache.instance = new PageDescriptorCache();
                    }
                    return topWindow.api.content.page.PageDescriptorCache.instance;
                };
                PageDescriptorCache.prototype.loadByApplication = function (applicationKey) {
                    new page.GetPageDescriptorsByApplicationRequest(applicationKey).sendAndParse().catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).done();
                };
                PageDescriptorCache.prototype.put = function (descriptor) {
                    api.util.assertNotNull(descriptor, "a PageDescriptor must be given");
                    _super.prototype.put.call(this, descriptor, descriptor.getKey().getApplicationKey());
                };
                PageDescriptorCache.prototype.getByKey = function (key) {
                    return _super.prototype.getByKey.call(this, key, key.getApplicationKey());
                };
                PageDescriptorCache.prototype.createApplicationCache = function () {
                    return new PageDescriptorApplicationCache();
                };
                return PageDescriptorCache;
            })(ApplicationBasedCache);
            page.PageDescriptorCache = PageDescriptorCache;
            var PageDescriptorApplicationCache = (function (_super) {
                __extends(PageDescriptorApplicationCache, _super);
                function PageDescriptorApplicationCache() {
                    _super.apply(this, arguments);
                }
                PageDescriptorApplicationCache.prototype.copy = function (object) {
                    return object.clone();
                };
                PageDescriptorApplicationCache.prototype.getKeyFromObject = function (object) {
                    return object.getKey();
                };
                PageDescriptorApplicationCache.prototype.getKeyAsString = function (key) {
                    return key.toString();
                };
                return PageDescriptorApplicationCache;
            })(api.cache.Cache);
            page.PageDescriptorApplicationCache = PageDescriptorApplicationCache;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageDescriptorResourceRequest = (function (_super) {
                __extends(PageDescriptorResourceRequest, _super);
                function PageDescriptorResourceRequest() {
                    _super.call(this);
                    this.cache = page.PageDescriptorCache.get();
                    this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "content", "page", "descriptor");
                }
                PageDescriptorResourceRequest.prototype.getResourcePath = function () {
                    return this.resourcePath;
                };
                PageDescriptorResourceRequest.prototype.fromJsonToPageDescriptor = function (json, ignoreCache) {
                    if (ignoreCache === void 0) { ignoreCache = false; }
                    var pageDescriptor = new api.content.page.PageDescriptorBuilder().fromJson(json).build();
                    if (!ignoreCache) {
                        this.cache.put(pageDescriptor);
                    }
                    return pageDescriptor;
                };
                PageDescriptorResourceRequest.prototype.fromJsonToPageDescriptors = function (json) {
                    var _this = this;
                    return json.descriptors.map(function (descriptorJson) {
                        return _this.fromJsonToPageDescriptor(descriptorJson);
                    });
                };
                return PageDescriptorResourceRequest;
            })(api.rest.ResourceRequest);
            page.PageDescriptorResourceRequest = PageDescriptorResourceRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var GetPageDescriptorByKeyRequest = (function (_super) {
                __extends(GetPageDescriptorByKeyRequest, _super);
                function GetPageDescriptorByKeyRequest(key) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.key = key;
                }
                GetPageDescriptorByKeyRequest.prototype.getParams = function () {
                    return {
                        key: this.key.toString()
                    };
                };
                GetPageDescriptorByKeyRequest.prototype.getRequestPath = function () {
                    return _super.prototype.getResourcePath.call(this);
                };
                GetPageDescriptorByKeyRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    var pageDescriptor = this.cache.getByKey(this.key);
                    if (pageDescriptor) {
                        return wemQ(pageDescriptor);
                    }
                    else {
                        return this.send().then(function (response) {
                            pageDescriptor = _this.fromJsonToPageDescriptor(response.getResult(), true);
                            return pageDescriptor;
                        });
                    }
                };
                return GetPageDescriptorByKeyRequest;
            })(page.PageDescriptorResourceRequest);
            page.GetPageDescriptorByKeyRequest = GetPageDescriptorByKeyRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var GetPageDescriptorsByApplicationRequest = (function (_super) {
                __extends(GetPageDescriptorsByApplicationRequest, _super);
                function GetPageDescriptorsByApplicationRequest(applicationKey) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.applicationKey = applicationKey;
                }
                GetPageDescriptorsByApplicationRequest.prototype.getParams = function () {
                    return {
                        applicationKey: this.applicationKey.toString()
                    };
                };
                GetPageDescriptorsByApplicationRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "list", "by_application");
                };
                GetPageDescriptorsByApplicationRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    var cached = this.cache.getByApplication(this.applicationKey);
                    if (cached) {
                        return wemQ(cached);
                    }
                    else {
                        return this.send().then(function (response) {
                            return _this.fromJsonToPageDescriptors(response.getResult());
                        });
                    }
                };
                return GetPageDescriptorsByApplicationRequest;
            })(page.PageDescriptorResourceRequest);
            page.GetPageDescriptorsByApplicationRequest = GetPageDescriptorsByApplicationRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var GetPageDescriptorsByApplicationsRequest = (function (_super) {
                __extends(GetPageDescriptorsByApplicationsRequest, _super);
                function GetPageDescriptorsByApplicationsRequest(applicationKeys) {
                    _super.call(this);
                    this.applicationKeys = applicationKeys;
                }
                GetPageDescriptorsByApplicationsRequest.prototype.setApplicationKeys = function (applicationKeys) {
                    this.applicationKeys = applicationKeys;
                };
                GetPageDescriptorsByApplicationsRequest.prototype.getParams = function () {
                    throw new Error("Unexpected call");
                };
                GetPageDescriptorsByApplicationsRequest.prototype.getRequestPath = function () {
                    throw new Error("Unexpected call");
                };
                GetPageDescriptorsByApplicationsRequest.prototype.sendAndParse = function () {
                    var promises = this.applicationKeys.map(function (applicationKey) { return new page.GetPageDescriptorsByApplicationRequest(applicationKey).sendAndParse(); });
                    return wemQ.all(promises).then(function (results) {
                        var all = [];
                        results.forEach(function (descriptors) {
                            Array.prototype.push.apply(all, descriptors);
                        });
                        return all;
                    });
                };
                return GetPageDescriptorsByApplicationsRequest;
            })(page.PageDescriptorResourceRequest);
            page.GetPageDescriptorsByApplicationsRequest = GetPageDescriptorsByApplicationsRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var Dropdown = api.ui.selector.dropdown.Dropdown;
            var PageDescriptorDropdown = (function (_super) {
                __extends(PageDescriptorDropdown, _super);
                function PageDescriptorDropdown(model) {
                    var _this = this;
                    _super.call(this, 'page-controller', {
                        optionDisplayValueViewer: new page.PageDescriptorViewer(),
                        dataIdProperty: 'value'
                    });
                    this.loadedDataListeners = [];
                    this.siteModel = model.getSiteModel();
                    this.initLoader();
                    this.onOptionSelected(function (event) {
                        var pageDescriptor = event.getOption().displayValue;
                        var setController = new page.SetController(_this).setDescriptor(pageDescriptor);
                        model.getPageModel().setController(setController);
                    });
                    this.siteModel.onApplicationAdded(function (event) {
                        _this.getPageDescriptorsByApplicationsRequest.setApplicationKeys(_this.siteModel.getApplicationKeys());
                        _this.removeAllOptions();
                        _this.load();
                    });
                    this.siteModel.onApplicationRemoved(function (event) {
                        _this.getPageDescriptorsByApplicationsRequest.setApplicationKeys(_this.siteModel.getApplicationKeys());
                        _this.removeAllOptions();
                        _this.load();
                    });
                }
                PageDescriptorDropdown.prototype.initLoader = function () {
                    var _this = this;
                    this.getPageDescriptorsByApplicationsRequest = new page.GetPageDescriptorsByApplicationsRequest(this.siteModel.getApplicationKeys());
                    this.loader = new api.util.loader.BaseLoader(this.getPageDescriptorsByApplicationsRequest).setComparator(new api.content.page.DescriptorByDisplayNameComparator());
                    this.loader.onLoadedData(function (event) {
                        event.getData().forEach(function (descriptor) {
                            var option = {
                                value: descriptor.getKey().toString(),
                                displayValue: descriptor,
                                indices: [descriptor.getDisplayName(), descriptor.getName().toString()]
                            };
                            _this.addOption(option);
                        });
                        _this.notifyLoadedData(event);
                    });
                };
                PageDescriptorDropdown.prototype.load = function () {
                    this.loader.load();
                };
                PageDescriptorDropdown.prototype.onLoadedData = function (listener) {
                    this.loadedDataListeners.push(listener);
                };
                PageDescriptorDropdown.prototype.unLoadedData = function (listener) {
                    this.loadedDataListeners = this.loadedDataListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                PageDescriptorDropdown.prototype.notifyLoadedData = function (event) {
                    var _this = this;
                    this.loadedDataListeners.forEach(function (listener) {
                        listener.call(_this, event);
                    });
                };
                return PageDescriptorDropdown;
            })(Dropdown);
            page.PageDescriptorDropdown = PageDescriptorDropdown;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageDescriptorViewer = (function (_super) {
                __extends(PageDescriptorViewer, _super);
                function PageDescriptorViewer() {
                    _super.call(this);
                }
                PageDescriptorViewer.prototype.resolveDisplayName = function (object) {
                    return object.getDisplayName();
                };
                PageDescriptorViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getKey().toString();
                };
                PageDescriptorViewer.prototype.resolveIconClass = function (object) {
                    return "icon-file icon-large";
                };
                return PageDescriptorViewer;
            })(api.ui.NamesAndIconViewer);
            page.PageDescriptorViewer = PageDescriptorViewer;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageTemplateKey = (function (_super) {
                __extends(PageTemplateKey, _super);
                function PageTemplateKey(s) {
                    _super.call(this, s);
                }
                PageTemplateKey.fromContentId = function (id) {
                    return new PageTemplateKey(id.toString());
                };
                PageTemplateKey.fromString = function (s) {
                    return new PageTemplateKey(s);
                };
                PageTemplateKey.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PageTemplateKey)) {
                        return false;
                    }
                    var other = o;
                    return _super.prototype.equals.call(this, other);
                };
                return PageTemplateKey;
            })(api.content.ContentId);
            page.PageTemplateKey = PageTemplateKey;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageTemplateName = (function () {
                function PageTemplateName(name) {
                    if (name == null) {
                        throw new Error("name cannot be null");
                    }
                    this.name = name;
                }
                PageTemplateName.prototype.toString = function () {
                    return this.name;
                };
                return PageTemplateName;
            })();
            page.PageTemplateName = PageTemplateName;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var ContentTypeName = api.schema.content.ContentTypeName;
            var PageTemplate = (function (_super) {
                __extends(PageTemplate, _super);
                function PageTemplate(builder) {
                    var _this = this;
                    _super.call(this, builder);
                    this.canRender = [];
                    this.getContentData().forEachProperty("supports", function (property) {
                        _this.canRender.push(new ContentTypeName(property.getString()));
                    });
                }
                PageTemplate.prototype.getKey = function () {
                    return this.getContentId();
                };
                PageTemplate.prototype.getPageMode = function () {
                    if (this.isPage()) {
                        if (this.getPage().hasController()) {
                            return 2 /* FORCED_CONTROLLER */;
                        }
                        else {
                            throw new Error("Illegal state: A PageTemplate's Page must a controller set");
                        }
                    }
                    else {
                        return 3 /* NO_CONTROLLER */;
                    }
                };
                PageTemplate.prototype.getController = function () {
                    return this.getPage().getController();
                };
                PageTemplate.prototype.isCanRender = function (pattern) {
                    return this.getCanRender().some(function (name) {
                        return name.equals(pattern);
                    });
                };
                PageTemplate.prototype.getCanRender = function () {
                    return this.canRender;
                };
                PageTemplate.prototype.hasRegions = function () {
                    if (!this.isPage()) {
                        return false;
                    }
                    return this.getPage().hasRegions();
                };
                PageTemplate.prototype.getRegions = function () {
                    if (!this.isPage()) {
                        return null;
                    }
                    return this.getPage().getRegions();
                };
                PageTemplate.prototype.hasConfig = function () {
                    if (!this.isPage()) {
                        return false;
                    }
                    return this.getPage().hasConfig();
                };
                PageTemplate.prototype.getConfig = function () {
                    if (!this.isPage()) {
                        return null;
                    }
                    return this.getPage().getConfig();
                };
                PageTemplate.prototype.equals = function (o, ignoreEmptyValues) {
                    if (ignoreEmptyValues === void 0) { ignoreEmptyValues = false; }
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PageTemplate)) {
                        return false;
                    }
                    return _super.prototype.equals.call(this, o, ignoreEmptyValues);
                };
                PageTemplate.prototype.clone = function () {
                    return this.newBuilder().build();
                };
                PageTemplate.prototype.newBuilder = function () {
                    return new PageTemplateBuilder(this);
                };
                return PageTemplate;
            })(api.content.Content);
            page.PageTemplate = PageTemplate;
            var PageTemplateBuilder = (function (_super) {
                __extends(PageTemplateBuilder, _super);
                function PageTemplateBuilder(source) {
                    _super.call(this, source);
                }
                PageTemplateBuilder.prototype.fromContentJson = function (contentJson) {
                    _super.prototype.fromContentJson.call(this, contentJson);
                    return this;
                };
                PageTemplateBuilder.prototype.build = function () {
                    return new PageTemplate(this);
                };
                return PageTemplateBuilder;
            })(api.content.ContentBuilder);
            page.PageTemplateBuilder = PageTemplateBuilder;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageTemplateViewer = (function (_super) {
                __extends(PageTemplateViewer, _super);
                function PageTemplateViewer() {
                    _super.call(this);
                }
                PageTemplateViewer.prototype.resolveDisplayName = function (object) {
                    return object.getDisplayName();
                };
                PageTemplateViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getController().toString();
                };
                PageTemplateViewer.prototype.resolveIconClass = function (object) {
                    return "icon-newspaper icon-large";
                };
                return PageTemplateViewer;
            })(api.ui.NamesAndIconViewer);
            page.PageTemplateViewer = PageTemplateViewer;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageTemplateComboBox = (function (_super) {
                __extends(PageTemplateComboBox, _super);
                function PageTemplateComboBox() {
                    _super.call(this, new api.ui.selector.combobox.RichComboBoxBuilder().setSelectedOptionsView(new PageTemplateSelectedOptionsView()).setIdentifierMethod("getKey").setMaximumOccurrences(1).setOptionDisplayValueViewer(new page.PageTemplateViewer));
                }
                return PageTemplateComboBox;
            })(api.ui.selector.combobox.RichComboBox);
            page.PageTemplateComboBox = PageTemplateComboBox;
            var PageTemplateSelectedOptionsView = (function (_super) {
                __extends(PageTemplateSelectedOptionsView, _super);
                function PageTemplateSelectedOptionsView() {
                    _super.apply(this, arguments);
                }
                PageTemplateSelectedOptionsView.prototype.createSelectedOption = function (option) {
                    return new api.ui.selector.combobox.SelectedOption(new PageTemplateSelectedOptionView(option), this.count());
                };
                return PageTemplateSelectedOptionsView;
            })(api.ui.selector.combobox.BaseSelectedOptionsView);
            page.PageTemplateSelectedOptionsView = PageTemplateSelectedOptionsView;
            var PageTemplateSelectedOptionView = (function (_super) {
                __extends(PageTemplateSelectedOptionView, _super);
                function PageTemplateSelectedOptionView(option) {
                    this.pageTemplate = option.displayValue;
                    _super.call(this, option);
                    this.addClass("page-template-selected-option-view");
                }
                PageTemplateSelectedOptionView.prototype.layout = function () {
                    var _this = this;
                    var namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(2 /* small */).build();
                    namesAndIconView.setIconClass("icon-newspaper icon-large").setMainName(this.pageTemplate.getDisplayName()).setSubName(this.pageTemplate.getController().toString());
                    var removeButtonEl = new api.dom.AEl("remove");
                    removeButtonEl.onClicked(function (event) {
                        _this.notifySelectedOptionRemoveRequested();
                        event.stopPropagation();
                        event.preventDefault();
                        return false;
                    });
                    this.appendChild(removeButtonEl);
                    this.appendChild(namesAndIconView);
                };
                return PageTemplateSelectedOptionView;
            })(api.ui.selector.combobox.BaseSelectedOptionView);
            page.PageTemplateSelectedOptionView = PageTemplateSelectedOptionView;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageTemplateLoader = (function (_super) {
                __extends(PageTemplateLoader, _super);
                function PageTemplateLoader(request) {
                    _super.call(this, request);
                }
                PageTemplateLoader.prototype.filterFn = function (template) {
                    return template.getDisplayName().toString().indexOf(this.getSearchString().toLowerCase()) != -1;
                };
                return PageTemplateLoader;
            })(api.util.loader.BaseLoader);
            page.PageTemplateLoader = PageTemplateLoader;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageTemplateResourceRequest = (function (_super) {
                __extends(PageTemplateResourceRequest, _super);
                function PageTemplateResourceRequest() {
                    _super.call(this);
                    this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "content", "page", "template");
                }
                PageTemplateResourceRequest.prototype.getResourcePath = function () {
                    return this.resourcePath;
                };
                PageTemplateResourceRequest.prototype.fromJsonToContent = function (json) {
                    return api.content.Content.fromJson(json);
                };
                return PageTemplateResourceRequest;
            })(api.rest.ResourceRequest);
            page.PageTemplateResourceRequest = PageTemplateResourceRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var GetPageTemplateByKeyRequest = (function (_super) {
                __extends(GetPageTemplateByKeyRequest, _super);
                function GetPageTemplateByKeyRequest(pageTemplateKey) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.pageTemplateKey = pageTemplateKey;
                }
                GetPageTemplateByKeyRequest.prototype.validate = function () {
                    api.util.assertNotNull(this.pageTemplateKey, "pageTemplateKey cannot be null");
                };
                GetPageTemplateByKeyRequest.prototype.getParams = function () {
                    return {
                        key: this.pageTemplateKey.toString()
                    };
                };
                GetPageTemplateByKeyRequest.prototype.getRequestPath = function () {
                    return _super.prototype.getResourcePath.call(this);
                };
                GetPageTemplateByKeyRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return _this.fromJsonToContent(response.getResult());
                    });
                };
                return GetPageTemplateByKeyRequest;
            })(page.PageTemplateResourceRequest);
            page.GetPageTemplateByKeyRequest = GetPageTemplateByKeyRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var GetPageTemplatesRequest = (function (_super) {
                __extends(GetPageTemplatesRequest, _super);
                function GetPageTemplatesRequest() {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                }
                GetPageTemplatesRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "list");
                };
                GetPageTemplatesRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().contents.map(function (contentJson) {
                            return _this.fromJsonToContent(contentJson);
                        });
                    });
                };
                return GetPageTemplatesRequest;
            })(page.PageTemplateResourceRequest);
            page.GetPageTemplatesRequest = GetPageTemplatesRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var GetPageTemplatesByCanRenderRequest = (function (_super) {
                __extends(GetPageTemplatesByCanRenderRequest, _super);
                function GetPageTemplatesByCanRenderRequest(site, contentTypeName) {
                    _super.call(this);
                    this.setMethod("GET");
                    this.site = site;
                    this.contentTypeName = contentTypeName;
                }
                GetPageTemplatesByCanRenderRequest.prototype.getParams = function () {
                    return {
                        siteId: this.site.toString(),
                        contentTypeName: this.contentTypeName.toString()
                    };
                };
                GetPageTemplatesByCanRenderRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "listByCanRender");
                };
                GetPageTemplatesByCanRenderRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.getResult().contents.map(function (contentJson) {
                            return _this.fromJsonToContent(contentJson);
                        });
                    });
                };
                return GetPageTemplatesByCanRenderRequest;
            })(page.PageTemplateResourceRequest);
            page.GetPageTemplatesByCanRenderRequest = GetPageTemplatesByCanRenderRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var GetDefaultPageTemplateRequest = (function (_super) {
                __extends(GetDefaultPageTemplateRequest, _super);
                function GetDefaultPageTemplateRequest(site, contentName) {
                    _super.call(this);
                    this.setMethod("GET");
                    this.site = site;
                    this.contentTypeName = contentName;
                }
                GetDefaultPageTemplateRequest.prototype.getParams = function () {
                    return {
                        siteId: this.site.toString(),
                        contentTypeName: this.contentTypeName.toString()
                    };
                };
                GetDefaultPageTemplateRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "default");
                };
                GetDefaultPageTemplateRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        if (response.hasResult()) {
                            return _this.fromJsonToContent(response.getResult());
                        }
                        else {
                            return null;
                        }
                    });
                };
                return GetDefaultPageTemplateRequest;
            })(page.PageTemplateResourceRequest);
            page.GetDefaultPageTemplateRequest = GetDefaultPageTemplateRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var IsRenderableRequest = (function (_super) {
                __extends(IsRenderableRequest, _super);
                function IsRenderableRequest(contentId) {
                    _super.call(this);
                    this.setMethod("GET");
                    this.contentId = contentId;
                }
                IsRenderableRequest.prototype.setContentId = function (value) {
                    this.contentId = value;
                    return this;
                };
                IsRenderableRequest.prototype.getParams = function () {
                    return {
                        contentId: this.contentId.toString()
                    };
                };
                IsRenderableRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "isRenderable");
                };
                IsRenderableRequest.prototype.sendAndParse = function () {
                    return this.send().then(function (response) {
                        return response.getResult();
                    });
                };
                return IsRenderableRequest;
            })(page.PageTemplateResourceRequest);
            page.IsRenderableRequest = IsRenderableRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageResourceRequest = (function (_super) {
                __extends(PageResourceRequest, _super);
                function PageResourceRequest() {
                    _super.call(this);
                    this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "content", "page");
                }
                PageResourceRequest.prototype.getResourcePath = function () {
                    return this.resourcePath;
                };
                PageResourceRequest.prototype.fromJsonToContent = function (json) {
                    return api.content.Content.fromJson(json);
                };
                return PageResourceRequest;
            })(api.rest.ResourceRequest);
            page.PageResourceRequest = PageResourceRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var CreatePageRequest = (function (_super) {
                __extends(CreatePageRequest, _super);
                function CreatePageRequest(contentId) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "POST");
                    this.contentId = contentId;
                }
                CreatePageRequest.prototype.setController = function (controller) {
                    this.controller = controller;
                    return this;
                };
                CreatePageRequest.prototype.setPageTemplateKey = function (pageTemplateKey) {
                    this.template = pageTemplateKey;
                    return this;
                };
                CreatePageRequest.prototype.setConfig = function (config) {
                    this.config = config;
                    return this;
                };
                CreatePageRequest.prototype.setRegions = function (value) {
                    this.regions = value;
                    return this;
                };
                CreatePageRequest.prototype.setCustomized = function (value) {
                    this.customized = value;
                    return this;
                };
                CreatePageRequest.prototype.getParams = function () {
                    return {
                        contentId: this.contentId.toString(),
                        controller: this.controller ? this.controller.toString() : null,
                        template: this.template ? this.template.toString() : null,
                        config: this.config ? this.config.toJson() : null,
                        regions: this.regions != null ? this.regions.toJson() : null,
                        customized: this.customized
                    };
                };
                CreatePageRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "create");
                };
                CreatePageRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.isBlank() ? null : _this.fromJsonToContent(response.getResult());
                    });
                };
                return CreatePageRequest;
            })(page.PageResourceRequest);
            page.CreatePageRequest = CreatePageRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var UpdatePageRequest = (function (_super) {
                __extends(UpdatePageRequest, _super);
                function UpdatePageRequest(contentId) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "POST");
                    this.contentId = contentId;
                }
                UpdatePageRequest.prototype.setController = function (controller) {
                    this.controller = controller;
                    return this;
                };
                UpdatePageRequest.prototype.setPageTemplateKey = function (pageTemplateKey) {
                    this.template = pageTemplateKey;
                    return this;
                };
                UpdatePageRequest.prototype.setConfig = function (config) {
                    this.config = config;
                    return this;
                };
                UpdatePageRequest.prototype.setRegions = function (value) {
                    this.regions = value;
                    return this;
                };
                UpdatePageRequest.prototype.setCustomized = function (value) {
                    this.customized = value;
                    return this;
                };
                UpdatePageRequest.prototype.getParams = function () {
                    return {
                        contentId: this.contentId.toString(),
                        controller: this.controller ? this.controller.toString() : null,
                        template: this.template ? this.template.toString() : null,
                        config: this.config ? this.config.toJson() : null,
                        regions: this.regions != null ? this.regions.toJson() : null,
                        customized: this.customized
                    };
                };
                UpdatePageRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "update");
                };
                UpdatePageRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.isBlank() ? null : _this.fromJsonToContent(response.getResult());
                    });
                };
                return UpdatePageRequest;
            })(page.PageResourceRequest);
            page.UpdatePageRequest = UpdatePageRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var DeletePageRequest = (function (_super) {
                __extends(DeletePageRequest, _super);
                function DeletePageRequest(contentId) {
                    _super.call(this);
                    _super.prototype.setMethod.call(this, "GET");
                    this.contentId = contentId;
                }
                DeletePageRequest.prototype.getParams = function () {
                    return {
                        contentId: this.contentId.toString()
                    };
                };
                DeletePageRequest.prototype.getRequestPath = function () {
                    return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "delete");
                };
                DeletePageRequest.prototype.sendAndParse = function () {
                    var _this = this;
                    return this.send().then(function (response) {
                        return response.isBlank() ? null : _this.fromJsonToContent(response.getResult());
                    });
                };
                return DeletePageRequest;
            })(page.PageResourceRequest);
            page.DeletePageRequest = DeletePageRequest;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PropertyTree = api.data.PropertyTree;
            var Page = (function () {
                function Page(builder) {
                    this.controller = builder.controller;
                    this.template = builder.template;
                    this.regions = builder.regions;
                    this.config = builder.config;
                    this.customized = builder.customized;
                }
                Page.prototype.hasController = function () {
                    return !!this.controller;
                };
                Page.prototype.getController = function () {
                    return this.controller;
                };
                Page.prototype.hasTemplate = function () {
                    return !!this.template;
                };
                Page.prototype.getTemplate = function () {
                    return this.template;
                };
                Page.prototype.hasRegions = function () {
                    return this.regions != null;
                };
                Page.prototype.getRegions = function () {
                    return this.regions;
                };
                Page.prototype.hasConfig = function () {
                    return this.config != null;
                };
                Page.prototype.getConfig = function () {
                    return this.config;
                };
                Page.prototype.isCustomized = function () {
                    return this.customized;
                };
                Page.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Page)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.equals(this.controller, other.controller)) {
                        return false;
                    }
                    if (!api.ObjectHelper.equals(this.template, other.template)) {
                        return false;
                    }
                    if (!api.ObjectHelper.equals(this.regions, other.regions)) {
                        return false;
                    }
                    if (!this.config && (!other.config || other.config.isEmpty())) {
                        return true;
                    }
                    if (!other.config && (!this.config || this.config.isEmpty())) {
                        return true;
                    }
                    return api.ObjectHelper.equals(this.config, other.config);
                };
                Page.prototype.clone = function () {
                    return new PageBuilder(this).build();
                };
                return Page;
            })();
            page.Page = Page;
            var PageBuilder = (function () {
                function PageBuilder(source) {
                    if (source) {
                        this.controller = source.getController();
                        this.template = source.getTemplate();
                        this.regions = source.getRegions() ? source.getRegions().clone() : null;
                        this.config = source.getConfig() ? source.getConfig().copy() : null;
                        this.customized = source.isCustomized();
                    }
                }
                PageBuilder.prototype.fromJson = function (json) {
                    this.setController(json.controller ? page.DescriptorKey.fromString(json.controller) : null);
                    this.setTemplate(json.template ? page.PageTemplateKey.fromString(json.template) : null);
                    this.setRegions(json.regions != null ? api.content.page.region.Regions.create().fromJson(json.regions, null).build() : null);
                    this.setConfig(json.config != null ? PropertyTree.fromJson(json.config) : null);
                    this.setCustomized(json.customized);
                    return this;
                };
                PageBuilder.prototype.setController = function (value) {
                    this.controller = value;
                    return this;
                };
                PageBuilder.prototype.setTemplate = function (value) {
                    this.template = value;
                    return this;
                };
                PageBuilder.prototype.setRegions = function (value) {
                    this.regions = value;
                    return this;
                };
                PageBuilder.prototype.setConfig = function (value) {
                    this.config = value;
                    return this;
                };
                PageBuilder.prototype.setCustomized = function (value) {
                    this.customized = value;
                    return this;
                };
                PageBuilder.prototype.build = function () {
                    return new Page(this);
                };
                return PageBuilder;
            })();
            page.PageBuilder = PageBuilder;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            (function (PageMode) {
                PageMode[PageMode["AUTOMATIC"] = 0] = "AUTOMATIC";
                PageMode[PageMode["FORCED_TEMPLATE"] = 1] = "FORCED_TEMPLATE";
                PageMode[PageMode["FORCED_CONTROLLER"] = 2] = "FORCED_CONTROLLER";
                PageMode[PageMode["NO_CONTROLLER"] = 3] = "NO_CONTROLLER";
            })(page.PageMode || (page.PageMode = {}));
            var PageMode = page.PageMode;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageModeChangedEvent = (function () {
                function PageModeChangedEvent(previousMode, newMode) {
                    this.previousMode = previousMode;
                    this.newMode = newMode;
                }
                PageModeChangedEvent.prototype.getPreviousMode = function () {
                    return this.previousMode;
                };
                PageModeChangedEvent.prototype.getNewMode = function () {
                    return this.newMode;
                };
                return PageModeChangedEvent;
            })();
            page.PageModeChangedEvent = PageModeChangedEvent;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PropertyTree = api.data.PropertyTree;
            var SetController = (function () {
                function SetController(eventSource) {
                    this.eventSource = eventSource;
                }
                SetController.prototype.setDescriptor = function (value) {
                    this.descriptor = value;
                    return this;
                };
                SetController.prototype.setRegions = function (value) {
                    this.regions = value;
                    return this;
                };
                SetController.prototype.setConfig = function (value) {
                    this.config = value;
                    return this;
                };
                return SetController;
            })();
            page.SetController = SetController;
            var SetTemplate = (function () {
                function SetTemplate(eventSource) {
                    this.eventSource = eventSource;
                }
                SetTemplate.prototype.setTemplate = function (template, descriptor) {
                    this.template = template;
                    this.descriptor = descriptor;
                    return this;
                };
                SetTemplate.prototype.setRegions = function (value) {
                    this.regions = value;
                    return this;
                };
                SetTemplate.prototype.setConfig = function (value) {
                    this.config = value;
                    return this;
                };
                return SetTemplate;
            })();
            page.SetTemplate = SetTemplate;
            var PageModel = (function () {
                function PageModel(liveEditModel, defaultTemplate, defaultTemplateDescriptor, pageMode) {
                    var _this = this;
                    this.pageModeChangedListeners = [];
                    this.propertyChangedListeners = [];
                    this.componentPropertyChangedListeners = [];
                    this.resetListeners = [];
                    this.ignorePropertyChanges = false;
                    this.liveEditModel = liveEditModel;
                    this.defaultTemplate = defaultTemplate;
                    this.defaultTemplateDescriptor = defaultTemplateDescriptor;
                    this.mode = pageMode;
                    this.customized = liveEditModel.getContent().isPage() && liveEditModel.getContent().getPage().isCustomized();
                    this.configPropertyChangedHandler = function (event) {
                        if (!_this.ignorePropertyChanges) {
                            if (_this.mode == 0 /* AUTOMATIC */) {
                                var setTemplate = new SetTemplate(_this).setTemplate(_this.defaultTemplate, _this.defaultTemplateDescriptor);
                                _this.setTemplate(setTemplate, true);
                            }
                        }
                    };
                    this.regionsChangedEventHandler = function (event) {
                        if (!_this.ignorePropertyChanges) {
                            if (_this.mode == 0 /* AUTOMATIC */) {
                                var setTemplate = new SetTemplate(_this).setTemplate(_this.defaultTemplate, _this.defaultTemplateDescriptor);
                                _this.setTemplate(setTemplate);
                            }
                        }
                    };
                    this.componentPropertyChangedEventHandler = function (event) {
                        if (!_this.isPageTemplate() && _this.getMode() == 0 /* AUTOMATIC */) {
                            _this.initializePageFromDefault(_this);
                        }
                        _this.componentPropertyChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                }
                PageModel.prototype.setIgnorePropertyChanges = function (value) {
                    this.ignorePropertyChanges = value;
                };
                PageModel.prototype.initializePageFromDefault = function (eventSource) {
                    var skip = false;
                    if (this.hasTemplate()) {
                        skip = true;
                    }
                    if (this.hasController()) {
                        skip = true;
                    }
                    if (!skip) {
                        var setTemplate = new SetTemplate(eventSource).setTemplate(this.defaultTemplate, this.defaultTemplateDescriptor);
                        this.setTemplate(setTemplate);
                    }
                };
                PageModel.prototype.setCustomized = function (value) {
                    this.customized = value;
                };
                PageModel.prototype.reset = function (eventSource) {
                    if (this.isPageTemplate() || !this.defaultTemplate) {
                        var setController = new SetController(eventSource).setDescriptor(null).setConfig(new PropertyTree()).setRegions(api.content.page.region.Regions.create().build());
                        this.setController(setController);
                    }
                    else {
                        this.setAutomaticTemplate(eventSource);
                    }
                    this.notifyReset();
                };
                PageModel.prototype.setMode = function (value) {
                    var oldValue = this.mode;
                    this.mode = value;
                    if (this.mode != oldValue) {
                        this.notifyPageModeChanged(oldValue, this.mode);
                    }
                };
                PageModel.prototype.setController = function (setController) {
                    var oldControllerKey = this.controller ? this.controller.getKey() : null;
                    var newControllerKey = setController.descriptor ? setController.descriptor.getKey() : null;
                    var controllerChanged = !api.ObjectHelper.equals(oldControllerKey, newControllerKey);
                    if (setController.descriptor) {
                        this.setMode(2 /* FORCED_CONTROLLER */);
                    }
                    else {
                        this.setMode(3 /* NO_CONTROLLER */);
                    }
                    this.setControllerData(setController);
                    if (!this.isPageTemplate()) {
                        this.setCustomized(true);
                    }
                    if (controllerChanged) {
                        this.setIgnorePropertyChanges(true);
                        this.notifyPropertyChanged(PageModel.PROPERTY_CONTROLLER, oldControllerKey, newControllerKey, setController.eventSource);
                        this.setIgnorePropertyChanges(false);
                    }
                    this.template = null;
                    return this;
                };
                PageModel.prototype.initController = function (setController) {
                    this.setControllerData(setController);
                    return this;
                };
                PageModel.prototype.setControllerData = function (setController) {
                    this.controller = setController.descriptor;
                    if (setController.config) {
                        this.setConfig(setController.config, setController.eventSource);
                    }
                    if (setController.regions) {
                        this.setRegions(setController.regions, setController.eventSource);
                    }
                    if (this.regions && setController.descriptor) {
                        this.regions.changeRegionsTo(setController.descriptor.getRegions());
                    }
                };
                PageModel.prototype.setAutomaticTemplate = function (eventSource, ignoreRegionChanges) {
                    if (ignoreRegionChanges === void 0) { ignoreRegionChanges = false; }
                    var config = this.defaultTemplate.hasConfig() ? this.defaultTemplate.getConfig().copy() : new PropertyTree();
                    var regions = this.defaultTemplate.hasRegions() ? this.defaultTemplate.getRegions().clone() : api.content.page.region.Regions.create().build();
                    var setTemplate = new SetTemplate(eventSource).setTemplate(null, this.defaultTemplateDescriptor).setRegions(regions).setConfig(config);
                    this.setTemplate(setTemplate, ignoreRegionChanges);
                    return this;
                };
                PageModel.prototype.setTemplate = function (setTemplate, ignoreRegionChanges) {
                    if (ignoreRegionChanges === void 0) { ignoreRegionChanges = false; }
                    var oldTemplateKey = this.template ? this.template.getKey() : null;
                    var newTemplateKey = setTemplate.template ? setTemplate.template.getKey() : null;
                    var templateOrModeChanged = !api.ObjectHelper.equals(oldTemplateKey, newTemplateKey) || this.hasController();
                    if (setTemplate.template) {
                        this.setMode(1 /* FORCED_TEMPLATE */);
                    }
                    else {
                        this.setMode(0 /* AUTOMATIC */);
                    }
                    this.setTemplateData(setTemplate, ignoreRegionChanges);
                    if (templateOrModeChanged) {
                        this.setIgnorePropertyChanges(true);
                        this.notifyPropertyChanged(PageModel.PROPERTY_TEMPLATE, oldTemplateKey, newTemplateKey, setTemplate.eventSource);
                        this.setIgnorePropertyChanges(false);
                    }
                    this.controller = null;
                    return this;
                };
                PageModel.prototype.initTemplate = function (setTemplate, ignoreRegionChanges) {
                    if (ignoreRegionChanges === void 0) { ignoreRegionChanges = false; }
                    this.setTemplateData(setTemplate, ignoreRegionChanges);
                    return this;
                };
                PageModel.prototype.setTemplateData = function (setTemplate, ignoreRegionChanges) {
                    if (ignoreRegionChanges === void 0) { ignoreRegionChanges = false; }
                    this.template = setTemplate.template;
                    this.templateDescriptor = setTemplate.descriptor;
                    if (setTemplate.config) {
                        this.setConfig(setTemplate.config, setTemplate.eventSource);
                    }
                    if (setTemplate.regions) {
                        this.setRegions(setTemplate.regions, setTemplate.eventSource, ignoreRegionChanges);
                    }
                    else if (setTemplate.template && setTemplate.template.hasRegions()) {
                        this.setRegions(setTemplate.template.getRegions().clone(), setTemplate.eventSource, ignoreRegionChanges);
                    }
                    if (this.regions) {
                        this.regions.changeRegionsTo(setTemplate.descriptor.getRegions());
                    }
                };
                PageModel.prototype.setRegions = function (value, eventOrigin, ignoreRegionChanges) {
                    if (ignoreRegionChanges === void 0) { ignoreRegionChanges = false; }
                    var oldValue = this.regions;
                    if (oldValue) {
                        this.unregisterRegionsListeners(oldValue);
                    }
                    this.regions = value;
                    this.registerRegionsListeners(this.regions);
                    this.setIgnorePropertyChanges(true);
                    if (!ignoreRegionChanges) {
                        this.notifyPropertyChanged(PageModel.PROPERTY_REGIONS, oldValue, value, eventOrigin);
                    }
                    this.setIgnorePropertyChanges(false);
                    return this;
                };
                PageModel.prototype.setConfig = function (value, eventOrigin) {
                    var oldValue = this.config;
                    if (oldValue) {
                        oldValue.unChanged(this.configPropertyChangedHandler);
                    }
                    this.config = value;
                    this.config.onChanged(this.configPropertyChangedHandler);
                    this.setIgnorePropertyChanges(true);
                    this.notifyPropertyChanged(PageModel.PROPERTY_CONFIG, oldValue, value, eventOrigin);
                    this.setIgnorePropertyChanges(false);
                    return this;
                };
                PageModel.prototype.getPage = function () {
                    if (this.mode == 0 /* AUTOMATIC */) {
                        return null;
                    }
                    else if (this.mode == 1 /* FORCED_TEMPLATE */) {
                        var regionsUnchanged = this.regions.equals(this.defaultTemplate.getRegions());
                        var regions = regionsUnchanged ? null : this.regions;
                        var configUnchanged = this.config.equals(this.defaultTemplate.getConfig());
                        var config = configUnchanged ? null : this.config;
                        return new page.PageBuilder().setTemplate(this.getTemplateKey()).setRegions(regions).setConfig(config).setCustomized(this.isCustomized()).build();
                    }
                    else if (this.mode == 2 /* FORCED_CONTROLLER */) {
                        return new page.PageBuilder().setController(this.controller.getKey()).setRegions(this.regions).setConfig(this.config).setCustomized(this.isCustomized()).build();
                    }
                    else if (this.mode == 3 /* NO_CONTROLLER */) {
                        if (this.contentHasNonRenderableTemplateSet()) {
                            return new page.PageBuilder().setTemplate(this.liveEditModel.getContent().getPage().getTemplate()).build();
                        }
                        else {
                            return null;
                        }
                    }
                    else {
                        throw new Error("Page mode not supported: " + this.mode);
                    }
                };
                PageModel.prototype.getDefaultPageTemplate = function () {
                    return this.defaultTemplate;
                };
                PageModel.prototype.getDefaultPageTemplateController = function () {
                    return this.defaultTemplateDescriptor;
                };
                PageModel.prototype.getMode = function () {
                    return this.mode;
                };
                PageModel.prototype.isPageTemplate = function () {
                    return this.liveEditModel.getContent().isPageTemplate();
                };
                PageModel.prototype.hasController = function () {
                    return !!this.controller;
                };
                PageModel.prototype.getController = function () {
                    return this.controller;
                };
                PageModel.prototype.hasTemplate = function () {
                    return !!this.template;
                };
                PageModel.prototype.hasDefaultPageTemplate = function () {
                    return !!this.defaultTemplate;
                };
                PageModel.prototype.getTemplateKey = function () {
                    return this.template ? this.template.getKey() : null;
                };
                PageModel.prototype.getTemplate = function () {
                    return this.template;
                };
                PageModel.prototype.getTemplateDescriptor = function () {
                    return this.templateDescriptor;
                };
                PageModel.prototype.getRegions = function () {
                    return this.regions;
                };
                PageModel.prototype.getConfig = function () {
                    return this.config;
                };
                PageModel.prototype.isCustomized = function () {
                    return this.customized;
                };
                PageModel.prototype.contentHasNonRenderableTemplateSet = function () {
                    return !this.isPageTemplate() && (this.mode == 3 /* NO_CONTROLLER */) && this.liveEditModel.getContent().getPage() && this.liveEditModel.getContent().getPage().getTemplate();
                };
                PageModel.prototype.registerRegionsListeners = function (regions) {
                    regions.onComponentPropertyChanged(this.componentPropertyChangedEventHandler);
                    regions.onChanged(this.regionsChangedEventHandler);
                };
                PageModel.prototype.unregisterRegionsListeners = function (regions) {
                    regions.unComponentPropertyChanged(this.componentPropertyChangedEventHandler);
                    regions.unChanged(this.regionsChangedEventHandler);
                };
                PageModel.prototype.onPageModeChanged = function (listener) {
                    this.pageModeChangedListeners.push(listener);
                };
                PageModel.prototype.unPageModeChanged = function (listener) {
                    this.pageModeChangedListeners = this.pageModeChangedListeners.filter(function (curr) {
                        return listener != curr;
                    });
                };
                PageModel.prototype.notifyPageModeChanged = function (oldValue, newValue) {
                    var event = new page.PageModeChangedEvent(oldValue, newValue);
                    this.pageModeChangedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                PageModel.prototype.onPropertyChanged = function (listener) {
                    this.propertyChangedListeners.push(listener);
                };
                PageModel.prototype.unPropertyChanged = function (listener) {
                    this.propertyChangedListeners = this.propertyChangedListeners.filter(function (curr) {
                        return listener != curr;
                    });
                };
                PageModel.prototype.notifyPropertyChanged = function (property, oldValue, newValue, origin) {
                    var event = new api.PropertyChangedEvent(property, oldValue, newValue, origin);
                    this.propertyChangedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                PageModel.prototype.onComponentPropertyChangedEvent = function (listener) {
                    this.componentPropertyChangedListeners.push(listener);
                };
                PageModel.prototype.unComponentPropertyChangedEvent = function (listener) {
                    this.componentPropertyChangedListeners = this.componentPropertyChangedListeners.filter(function (curr) {
                        return listener != curr;
                    });
                };
                PageModel.prototype.onReset = function (listener) {
                    this.resetListeners.push(listener);
                };
                PageModel.prototype.notifyReset = function () {
                    this.resetListeners.forEach(function (listener) {
                        listener();
                    });
                };
                PageModel.PROPERTY_REGIONS = 'regions';
                PageModel.PROPERTY_TEMPLATE = 'template';
                PageModel.PROPERTY_CONTROLLER = 'controller';
                PageModel.PROPERTY_CONFIG = 'config';
                return PageModel;
            })();
            page.PageModel = PageModel;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var DescriptorByDisplayNameComparator = (function () {
                function DescriptorByDisplayNameComparator() {
                }
                DescriptorByDisplayNameComparator.prototype.compare = function (a, b) {
                    if (!a) {
                        return 1;
                    }
                    else {
                        var firstName = a.getDisplayName();
                    }
                    if (!b) {
                        return -1;
                    }
                    else {
                        var secondName = b.getDisplayName();
                    }
                    return firstName.localeCompare(secondName);
                };
                return DescriptorByDisplayNameComparator;
            })();
            page.DescriptorByDisplayNameComparator = DescriptorByDisplayNameComparator;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var PageTemplateByDisplayNameComparator = (function () {
                function PageTemplateByDisplayNameComparator() {
                }
                PageTemplateByDisplayNameComparator.prototype.compare = function (a, b) {
                    if (!a) {
                        return 1;
                    }
                    else {
                        var firstName = a.getDisplayName();
                    }
                    if (!b) {
                        return -1;
                    }
                    else {
                        var secondName = b.getDisplayName();
                    }
                    return firstName.localeCompare(secondName);
                };
                return PageTemplateByDisplayNameComparator;
            })();
            page.PageTemplateByDisplayNameComparator = PageTemplateByDisplayNameComparator;
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentPath = (function () {
                    function ComponentPath(regionAndComponentList) {
                        var _this = this;
                        this.regionAndComponentList = regionAndComponentList;
                        this.refString = "";
                        this.regionAndComponentList.forEach(function (regionAndComponent, index) {
                            _this.refString += regionAndComponent.toString();
                            if (index < _this.regionAndComponentList.length - 1) {
                                _this.refString += ComponentPath.DIVIDER;
                            }
                        });
                    }
                    ComponentPath.prototype.numberOfLevels = function () {
                        return this.regionAndComponentList.length;
                    };
                    ComponentPath.prototype.getFirstLevel = function () {
                        return this.regionAndComponentList[0];
                    };
                    ComponentPath.prototype.getLastLevel = function () {
                        return this.regionAndComponentList[this.regionAndComponentList.length - 1];
                    };
                    ComponentPath.prototype.getLevels = function () {
                        return this.regionAndComponentList;
                    };
                    ComponentPath.prototype.getComponentIndex = function () {
                        return this.getLastLevel().getComponentIndex();
                    };
                    ComponentPath.prototype.getRegionPath = function () {
                        var _this = this;
                        var regionPathAsString = "";
                        this.regionAndComponentList.forEach(function (regionAndComponent, index) {
                            if (index == _this.regionAndComponentList.length - 1) {
                                regionPathAsString += regionAndComponent.getRegionName();
                            }
                            else {
                                regionPathAsString += regionAndComponent.toString();
                                regionPathAsString += "/";
                            }
                        });
                        return region.RegionPath.fromString(regionPathAsString);
                    };
                    ComponentPath.prototype.removeFirstLevel = function () {
                        if (this.numberOfLevels() <= 1) {
                            return null;
                        }
                        var newRegionAndComponentList = [];
                        for (var i = 1; i < this.regionAndComponentList.length; i++) {
                            newRegionAndComponentList.push(this.regionAndComponentList[i]);
                        }
                        return new ComponentPath(newRegionAndComponentList);
                    };
                    ComponentPath.prototype.toString = function () {
                        return this.refString;
                    };
                    ComponentPath.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ComponentPath)) {
                            return false;
                        }
                        var other = o;
                        if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                            return false;
                        }
                        return true;
                    };
                    ComponentPath.fromString = function (str) {
                        if (!str) {
                            return null;
                        }
                        var elements = api.util.StringHelper.removeEmptyStrings(str.split(ComponentPath.DIVIDER));
                        var regionAndComponentList = [];
                        for (var i = 0; i < elements.length - 1; i += 2) {
                            var regionName = elements[i];
                            var componentIndexAsString = elements[i + 1];
                            var regionAndComponent = new ComponentPathRegionAndComponent(regionName, parseInt(componentIndexAsString));
                            regionAndComponentList.push(regionAndComponent);
                        }
                        return new ComponentPath(regionAndComponentList);
                    };
                    ComponentPath.fromRegionPathAndComponentIndex = function (regionPath, componentIndex) {
                        api.util.assertNotNull(regionPath, "regionPath cannot be null");
                        api.util.assert(componentIndex >= 0, "componentIndex must be zero or more");
                        var regionAndComponentList = [];
                        if (regionPath.getParentComponentPath()) {
                            regionPath.getParentComponentPath().regionAndComponentList.forEach(function (regionAndComponent) {
                                regionAndComponentList.push(regionAndComponent);
                            });
                        }
                        regionAndComponentList.push(new ComponentPathRegionAndComponent(regionPath.getRegionName(), componentIndex));
                        return new ComponentPath(regionAndComponentList);
                    };
                    ComponentPath.DIVIDER = "/";
                    return ComponentPath;
                })();
                region.ComponentPath = ComponentPath;
                var ComponentPathRegionAndComponent = (function () {
                    function ComponentPathRegionAndComponent(regionName, componentIndex) {
                        this.regionName = regionName;
                        this.componentIndex = componentIndex;
                        this.refString = regionName + ComponentPathRegionAndComponent.DIVIDER + this.componentIndex;
                    }
                    ComponentPathRegionAndComponent.prototype.getRegionName = function () {
                        return this.regionName;
                    };
                    ComponentPathRegionAndComponent.prototype.getComponentIndex = function () {
                        return this.componentIndex;
                    };
                    ComponentPathRegionAndComponent.prototype.toString = function () {
                        return this.refString;
                    };
                    ComponentPathRegionAndComponent.DIVIDER = "/";
                    return ComponentPathRegionAndComponent;
                })();
                region.ComponentPathRegionAndComponent = ComponentPathRegionAndComponent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentName = (function () {
                    function ComponentName(value) {
                        api.util.assertNotNull(value, "ComponentName value cannot be null");
                        this.value = value;
                    }
                    ComponentName.prototype.hasCountPostfix = function () {
                        var countDelimiterIndex = this.value.lastIndexOf(ComponentName.COUNT_DELIMITER);
                        return countDelimiterIndex > 0 && countDelimiterIndex <= this.value.length - 2;
                    };
                    ComponentName.prototype.removeCountPostfix = function () {
                        if (!this.hasCountPostfix()) {
                            return this;
                        }
                        var nameWithoutCountPostfix = this.value.substring(0, this.value.lastIndexOf(ComponentName.COUNT_DELIMITER));
                        return new ComponentName(nameWithoutCountPostfix);
                    };
                    ComponentName.prototype.isDuplicateOf = function (other) {
                        if (this.value == other.value) {
                            return true;
                        }
                        if (!this.hasCountPostfix()) {
                            return false;
                        }
                        var nameWithoutCountPostfix = this.removeCountPostfix();
                        return nameWithoutCountPostfix.equals(other);
                    };
                    ComponentName.prototype.createDuplicate = function (count) {
                        var newValue = this.value + ComponentName.COUNT_DELIMITER + "" + count;
                        return new ComponentName(newValue);
                    };
                    ComponentName.prototype.toString = function () {
                        return this.value;
                    };
                    ComponentName.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ComponentName)) {
                            return false;
                        }
                        var other = o;
                        if (!api.ObjectHelper.stringEquals(this.value, other.value)) {
                            return false;
                        }
                        return true;
                    };
                    ComponentName.COUNT_DELIMITER = "-";
                    return ComponentName;
                })();
                region.ComponentName = ComponentName;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentType = (function () {
                    function ComponentType(shortName) {
                        ComponentType.shortNameToInstance[shortName] = this;
                        this.shortName = shortName;
                    }
                    ComponentType.prototype.getShortName = function () {
                        return this.shortName;
                    };
                    ComponentType.prototype.newComponentBuilder = function () {
                        throw new Error("Must be implemented by inheritors");
                    };
                    ComponentType.byShortName = function (shortName) {
                        return ComponentType.shortNameToInstance[shortName];
                    };
                    ComponentType.shortNameToInstance = {};
                    return ComponentType;
                })();
                region.ComponentType = ComponentType;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentChangedEvent = (function () {
                    function ComponentChangedEvent(path) {
                        this.path = path;
                    }
                    ComponentChangedEvent.prototype.getPath = function () {
                        return this.path;
                    };
                    return ComponentChangedEvent;
                })();
                region.ComponentChangedEvent = ComponentChangedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentPropertyChangedEvent = (function (_super) {
                    __extends(ComponentPropertyChangedEvent, _super);
                    function ComponentPropertyChangedEvent(builder) {
                        _super.call(this, builder.component.getPath());
                        this.component = builder.component;
                        this.propertyName = builder.propertyName;
                    }
                    ComponentPropertyChangedEvent.prototype.getComponent = function () {
                        return this.component;
                    };
                    ComponentPropertyChangedEvent.prototype.getPropertyName = function () {
                        return this.propertyName;
                    };
                    ComponentPropertyChangedEvent.create = function () {
                        return new ComponentPropertyChangedEventBuilder();
                    };
                    return ComponentPropertyChangedEvent;
                })(region.ComponentChangedEvent);
                region.ComponentPropertyChangedEvent = ComponentPropertyChangedEvent;
                var ComponentPropertyChangedEventBuilder = (function () {
                    function ComponentPropertyChangedEventBuilder() {
                    }
                    ComponentPropertyChangedEventBuilder.prototype.setPropertyName = function (value) {
                        this.propertyName = value;
                        return this;
                    };
                    ComponentPropertyChangedEventBuilder.prototype.setComponent = function (value) {
                        this.component = value;
                        return this;
                    };
                    ComponentPropertyChangedEventBuilder.prototype.build = function () {
                        return new ComponentPropertyChangedEvent(this);
                    };
                    return ComponentPropertyChangedEventBuilder;
                })();
                region.ComponentPropertyChangedEventBuilder = ComponentPropertyChangedEventBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentResetEvent = (function (_super) {
                    __extends(ComponentResetEvent, _super);
                    function ComponentResetEvent(componentPath) {
                        _super.call(this, componentPath);
                    }
                    return ComponentResetEvent;
                })(region.ComponentChangedEvent);
                region.ComponentResetEvent = ComponentResetEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentPropertyValueChangedEvent = (function (_super) {
                    __extends(ComponentPropertyValueChangedEvent, _super);
                    function ComponentPropertyValueChangedEvent(path, propertyName) {
                        _super.call(this, path);
                        this.propertyName = propertyName;
                    }
                    ComponentPropertyValueChangedEvent.prototype.getPropertyName = function () {
                        return this.propertyName;
                    };
                    return ComponentPropertyValueChangedEvent;
                })(region.ComponentChangedEvent);
                region.ComponentPropertyValueChangedEvent = ComponentPropertyValueChangedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var Component = (function () {
                    function Component(builder) {
                        this.index = -1;
                        this.changedListeners = [];
                        this.propertyChangedListeners = [];
                        this.propertyValueChangedListeners = [];
                        this.resetListeners = [];
                        this.name = builder.name;
                        this.index = builder.index;
                        this.parent = builder.parent;
                    }
                    Component.prototype.setParent = function (parent) {
                        this.parent = parent;
                    };
                    Component.prototype.setIndex = function (value) {
                        this.index = value;
                    };
                    Component.prototype.getIndex = function () {
                        return this.index;
                    };
                    Component.prototype.hasPath = function () {
                        return this.parent && this.index >= 0;
                    };
                    Component.prototype.getPath = function () {
                        return this.hasPath() ? region.ComponentPath.fromRegionPathAndComponentIndex(this.parent.getPath(), this.index) : null;
                    };
                    Component.prototype.getName = function () {
                        return this.name;
                    };
                    Component.prototype.setName = function (newValue) {
                        var oldValue = this.name;
                        this.name = newValue;
                        if (!api.ObjectHelper.equals(oldValue, newValue)) {
                            this.notifyPropertyChanged(Component.PROPERTY_NAME);
                        }
                    };
                    Component.prototype.doReset = function () {
                        throw new Error("Must be implemented by inheritors");
                    };
                    Component.prototype.reset = function () {
                        this.doReset();
                        this.notifyResetEvent();
                    };
                    Component.prototype.isEmpty = function () {
                        throw new Error("Must be implemented by inheritors");
                    };
                    Component.prototype.getParent = function () {
                        return this.parent;
                    };
                    Component.prototype.duplicate = function () {
                        var duplicateName = this.getName();
                        var duplicatedComponent = this.clone();
                        duplicatedComponent.setName(duplicateName);
                        return duplicatedComponent;
                    };
                    Component.prototype.remove = function () {
                        this.parent.removeComponent(this);
                    };
                    Component.prototype.toJson = function () {
                        throw new Error("Must be implemented by inheritor: " + api.ClassHelper.getClassName(this));
                    };
                    Component.prototype.toString = function () {
                        return "Component[" + (this.name ? this.name.toString() : "") + "]";
                    };
                    Component.prototype.toComponentJson = function () {
                        return {
                            "name": this.name ? this.name.toString() : null
                        };
                    };
                    Component.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Component)) {
                            return false;
                        }
                        var other = o;
                        if (!api.ObjectHelper.equals(this.name, other.name)) {
                            return false;
                        }
                        return true;
                    };
                    Component.prototype.clone = function () {
                        throw new Error("Must be implemented by inheritors");
                    };
                    Component.prototype.onChanged = function (listener) {
                        this.changedListeners.push(listener);
                    };
                    Component.prototype.unChanged = function (listener) {
                        this.changedListeners = this.changedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Component.prototype.notifyChangedEvent = function (event) {
                        this.changedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Component.prototype.onReset = function (listener) {
                        this.resetListeners.push(listener);
                    };
                    Component.prototype.unReset = function (listener) {
                        this.resetListeners = this.resetListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Component.prototype.notifyResetEvent = function () {
                        var event = new region.ComponentResetEvent(this.getPath());
                        this.resetListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Component.prototype.onPropertyChanged = function (listener) {
                        this.propertyChangedListeners.push(listener);
                    };
                    Component.prototype.unPropertyChanged = function (listener) {
                        this.propertyChangedListeners = this.propertyChangedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Component.prototype.notifyPropertyChanged = function (propertyName) {
                        var event = region.ComponentPropertyChangedEvent.create().setComponent(this).setPropertyName(propertyName).build();
                        this.propertyChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChangedEvent(event);
                    };
                    Component.prototype.forwardComponentPropertyChangedEvent = function (event) {
                        this.propertyChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Component.prototype.onPropertyValueChanged = function (listener) {
                        this.propertyValueChangedListeners.push(listener);
                    };
                    Component.prototype.unPropertyValueChanged = function (listener) {
                        this.propertyValueChangedListeners = this.propertyValueChangedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Component.prototype.notifyPropertyValueChanged = function (propertyName) {
                        var event = new region.ComponentPropertyValueChangedEvent(this.getPath(), propertyName);
                        this.propertyValueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChangedEvent(event);
                    };
                    Component.PROPERTY_NAME = 'name';
                    return Component;
                })();
                region.Component = Component;
                var ComponentBuilder = (function () {
                    function ComponentBuilder(source) {
                        if (source) {
                            this.name = source.getName();
                            this.parent = source.getParent();
                            this.index = source.getIndex();
                        }
                    }
                    ComponentBuilder.prototype.setIndex = function (value) {
                        this.index = value;
                        return this;
                    };
                    ComponentBuilder.prototype.setName = function (value) {
                        this.name = value;
                        return this;
                    };
                    ComponentBuilder.prototype.setParent = function (value) {
                        this.parent = value;
                        return this;
                    };
                    ComponentBuilder.prototype.build = function () {
                        throw new Error("Must be implemented by inheritor");
                    };
                    return ComponentBuilder;
                })();
                region.ComponentBuilder = ComponentBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var ComponentFactory = (function () {
                    function ComponentFactory() {
                    }
                    ComponentFactory.createFromJson = function (json, componentIndex, region) {
                        if (json.PartComponent) {
                            return new _region.PartComponentBuilder().fromJson(json.PartComponent, region).build();
                        }
                        else if (json.ImageComponent) {
                            return new _region.ImageComponentBuilder().fromJson(json.ImageComponent, region).build();
                        }
                        else if (json.LayoutComponent) {
                            var layoutComponentBuilder = new _region.LayoutComponentBuilder();
                            layoutComponentBuilder.setIndex(componentIndex);
                            return layoutComponentBuilder.fromJson(json.LayoutComponent, region);
                        }
                        else if (json.TextComponent) {
                            return new _region.TextComponentBuilder().fromJson(json.TextComponent, region).setIndex(componentIndex).build();
                        }
                        else {
                            throw new Error("Not a component that can be placed in a Region: " + json);
                        }
                    };
                    return ComponentFactory;
                })();
                _region.ComponentFactory = ComponentFactory;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var PropertyTree = api.data.PropertyTree;
                var DescriptorBasedComponent = (function (_super) {
                    __extends(DescriptorBasedComponent, _super);
                    function DescriptorBasedComponent(builder) {
                        var _this = this;
                        _super.call(this, builder);
                        this.descriptor = builder.descriptor;
                        this.config = builder.config;
                        this.configChangedHandler = function (event) {
                            if (DescriptorBasedComponent.debug) {
                                console.debug("DescriptorBasedComponent[" + _this.getPath().toString() + "].config.onChanged: ", event);
                            }
                            if (!_this.disableEventForwarding) {
                                _this.notifyPropertyValueChanged(DescriptorBasedComponent.PROPERTY_CONFIG);
                            }
                        };
                        this.config.onChanged(this.configChangedHandler);
                    }
                    DescriptorBasedComponent.prototype.setDisableEventForwarding = function (value) {
                        this.disableEventForwarding = value;
                    };
                    DescriptorBasedComponent.prototype.hasDescriptor = function () {
                        return !!this.descriptor;
                    };
                    DescriptorBasedComponent.prototype.getDescriptor = function () {
                        return this.descriptor;
                    };
                    DescriptorBasedComponent.prototype.setDescriptor = function (descriptorKey, descriptor) {
                        var oldValue = this.descriptor;
                        this.descriptor = descriptorKey;
                        this.alignNameWithDescriptor(descriptor);
                        if (!api.ObjectHelper.equals(oldValue, descriptorKey)) {
                            this.notifyPropertyChanged(DescriptorBasedComponent.PROPERTY_DESCRIPTOR);
                        }
                        this.setConfig(new PropertyTree());
                    };
                    DescriptorBasedComponent.prototype.setConfig = function (config) {
                        var oldValue = this.config;
                        if (oldValue) {
                            this.config.unChanged(this.configChangedHandler);
                        }
                        this.config = config;
                        this.config.onChanged(this.configChangedHandler);
                        if (!api.ObjectHelper.equals(oldValue, config)) {
                            this.notifyPropertyChanged(DescriptorBasedComponent.PROPERTY_CONFIG);
                        }
                    };
                    DescriptorBasedComponent.prototype.alignNameWithDescriptor = function (descriptor) {
                        var newName = descriptor ? new region.ComponentName(descriptor.getDisplayName()) : null;
                        this.setName(newName);
                    };
                    DescriptorBasedComponent.prototype.getConfig = function () {
                        return this.config;
                    };
                    DescriptorBasedComponent.prototype.doReset = function () {
                        this.setDescriptor(null, null);
                    };
                    DescriptorBasedComponent.prototype.toComponentJson = function () {
                        return {
                            "name": this.getName() ? this.getName().toString() : null,
                            "descriptor": this.descriptor != null ? this.descriptor.toString() : null,
                            "config": this.config != null ? this.config.toJson() : null
                        };
                    };
                    DescriptorBasedComponent.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, DescriptorBasedComponent)) {
                            return false;
                        }
                        if (!_super.prototype.equals.call(this, o)) {
                            return false;
                        }
                        var other = o;
                        if (!api.ObjectHelper.equals(this.descriptor, other.descriptor)) {
                            return false;
                        }
                        if (!api.ObjectHelper.equals(this.config, other.config)) {
                            return false;
                        }
                        return true;
                    };
                    DescriptorBasedComponent.prototype.clone = function () {
                        throw new Error("Must be implemented by inheritors");
                    };
                    DescriptorBasedComponent.debug = false;
                    DescriptorBasedComponent.PROPERTY_DESCRIPTOR = 'descriptor';
                    DescriptorBasedComponent.PROPERTY_CONFIG = 'config';
                    return DescriptorBasedComponent;
                })(region.Component);
                region.DescriptorBasedComponent = DescriptorBasedComponent;
                var DescriptorBasedComponentBuilder = (function (_super) {
                    __extends(DescriptorBasedComponentBuilder, _super);
                    function DescriptorBasedComponentBuilder(source) {
                        _super.call(this, source);
                        if (source) {
                            this.descriptor = source.getDescriptor();
                            this.config = source.getConfig() ? source.getConfig().copy() : null;
                        }
                        else {
                            this.config = new PropertyTree();
                        }
                    }
                    DescriptorBasedComponentBuilder.prototype.setDescriptor = function (value) {
                        this.descriptor = value;
                        return this;
                    };
                    DescriptorBasedComponentBuilder.prototype.setConfig = function (value) {
                        this.config = value;
                        return this;
                    };
                    DescriptorBasedComponentBuilder.prototype.build = function () {
                        throw new Error("Must be implemented by inheritor");
                    };
                    return DescriptorBasedComponentBuilder;
                })(region.ComponentBuilder);
                region.DescriptorBasedComponentBuilder = DescriptorBasedComponentBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionsChangedEvent = (function () {
                    function RegionsChangedEvent() {
                    }
                    return RegionsChangedEvent;
                })();
                region.RegionsChangedEvent = RegionsChangedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionAddedEvent = (function (_super) {
                    __extends(RegionAddedEvent, _super);
                    function RegionAddedEvent(regionPath) {
                        _super.call(this);
                        this.regionPath = regionPath;
                    }
                    RegionAddedEvent.prototype.getRegionPath = function () {
                        return this.regionPath;
                    };
                    return RegionAddedEvent;
                })(region.RegionsChangedEvent);
                region.RegionAddedEvent = RegionAddedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionRemovedEvent = (function (_super) {
                    __extends(RegionRemovedEvent, _super);
                    function RegionRemovedEvent(regionPath) {
                        _super.call(this);
                        this.regionPath = regionPath;
                    }
                    RegionRemovedEvent.prototype.getRegionPath = function () {
                        return this.regionPath;
                    };
                    return RegionRemovedEvent;
                })(region.RegionsChangedEvent);
                region.RegionRemovedEvent = RegionRemovedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionChangedEvent = (function (_super) {
                    __extends(RegionChangedEvent, _super);
                    function RegionChangedEvent(regionPath) {
                        _super.call(this);
                        this.regionPath = regionPath;
                    }
                    RegionChangedEvent.prototype.getRegionPath = function () {
                        return this.regionPath;
                    };
                    return RegionChangedEvent;
                })(region.RegionsChangedEvent);
                region.RegionChangedEvent = RegionChangedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var Regions = (function () {
                    function Regions(builder) {
                        var _this = this;
                        this.regionByName = {};
                        this.changedListeners = [];
                        this.componentPropertyChangedListeners = [];
                        this.regionChangedListeners = [];
                        this.regionAddedListeners = [];
                        this.regionRemovedListeners = [];
                        this.regionChangedEventHandler = function (event) { return _this.notifyRegionChanged(event.getPath()); };
                        this.componentPropertyChangedEventHandler = function (event) { return _this.forwardComponentPropertyChangedEvent(event); };
                        builder.regions.forEach(function (region) {
                            if (_this.regionByName[region.getName()] != undefined) {
                                throw new Error("Regions must be unique by name, duplicate found: " + region.getName());
                            }
                            _this.addRegion(region);
                        });
                    }
                    Regions.prototype.mergeRegions = function (descriptorRegions, parent) {
                        return new _region.LayoutRegionsMerger().merge(this, descriptorRegions, parent);
                    };
                    Regions.prototype.addRegion = function (region) {
                        this.regionByName[region.getName()] = region;
                        this.notifyRegionAdded(region.getPath());
                        this.registerRegionListeners(region);
                    };
                    Regions.prototype.registerRegionListeners = function (region) {
                        region.onChanged(this.regionChangedEventHandler);
                        region.onComponentPropertyChangedEvent(this.componentPropertyChangedEventHandler);
                    };
                    Regions.prototype.unregisterRegionListeners = function (region) {
                        region.unChanged(this.regionChangedEventHandler);
                        region.unComponentPropertyChangedEvent(this.componentPropertyChangedEventHandler);
                    };
                    Regions.prototype.removeRegions = function (regions) {
                        var _this = this;
                        regions.forEach(function (region) {
                            delete _this.regionByName[region.getName()];
                            _this.notifyRegionRemoved(region.getPath());
                            _this.unregisterRegionListeners(region);
                        });
                    };
                    Regions.prototype.getRegions = function () {
                        var regions = [];
                        for (var i in this.regionByName) {
                            var region = this.regionByName[i];
                            regions.push(region);
                        }
                        return regions;
                    };
                    Regions.prototype.getRegionByName = function (name) {
                        return this.regionByName[name];
                    };
                    Regions.prototype.getComponent = function (path) {
                        var first = path.getFirstLevel();
                        var region = this.getRegionByName(first.getRegionName());
                        var component = region.getComponentByIndex(first.getComponentIndex());
                        if (path.numberOfLevels() == 1) {
                            return component;
                        }
                        else {
                            if (!api.ObjectHelper.iFrameSafeInstanceOf(component, _region.LayoutComponent)) {
                                throw new Error("Expected component to be a LayoutComponent: " + api.ClassHelper.getClassName(component));
                            }
                            var layoutComponent = component;
                            return layoutComponent.getComponent(path.removeFirstLevel());
                        }
                    };
                    Regions.prototype.changeRegionsTo = function (regionDescriptors) {
                        var _this = this;
                        var regionsToRemove = this.getRegions().filter(function (region, index) {
                            return !regionDescriptors.some(function (regionDescriptor) {
                                return regionDescriptor.getName() == region.getName();
                            });
                        });
                        this.removeRegions(regionsToRemove);
                        regionDescriptors.forEach(function (regionDescriptor) {
                            var region = _this.getRegionByName(regionDescriptor.getName());
                            if (!region) {
                                region = _region.Region.create().setName(regionDescriptor.getName()).build();
                                _this.addRegion(region);
                            }
                        });
                    };
                    Regions.prototype.toJson = function () {
                        var regionJsons = [];
                        this.getRegions().forEach(function (region) {
                            regionJsons.push(region.toJson());
                        });
                        return regionJsons;
                    };
                    Regions.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Regions)) {
                            return false;
                        }
                        var other = o;
                        var thisRegions = this.getRegions();
                        var otherRegions = other.getRegions();
                        if (!api.ObjectHelper.arrayEquals(thisRegions, otherRegions)) {
                            return false;
                        }
                        return true;
                    };
                    Regions.prototype.clone = function () {
                        return new RegionsBuilder(this).build();
                    };
                    Regions.prototype.onChanged = function (listener) {
                        this.changedListeners.push(listener);
                    };
                    Regions.prototype.unChanged = function (listener) {
                        this.changedListeners = this.changedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Regions.prototype.notifyChanged = function (event) {
                        if (Regions.debug) {
                            console.debug("Regions.notifyChanged");
                        }
                        this.changedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Regions.prototype.onComponentPropertyChanged = function (listener) {
                        this.componentPropertyChangedListeners.push(listener);
                    };
                    Regions.prototype.unComponentPropertyChanged = function (listener) {
                        this.componentPropertyChangedListeners = this.componentPropertyChangedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Regions.prototype.forwardComponentPropertyChangedEvent = function (event) {
                        this.componentPropertyChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Regions.prototype.onRegionChanged = function (listener) {
                        this.regionChangedListeners.push(listener);
                    };
                    Regions.prototype.unRegionChanged = function (listener) {
                        this.regionChangedListeners = this.regionChangedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Regions.prototype.notifyRegionChanged = function (regionPath) {
                        var event = new _region.RegionChangedEvent(regionPath);
                        if (Regions.debug) {
                            console.debug("Regions.notifyRegionChanged: " + event.getRegionPath().toString());
                        }
                        this.regionChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChanged(event);
                    };
                    Regions.prototype.onRegionAdded = function (listener) {
                        this.regionAddedListeners.push(listener);
                    };
                    Regions.prototype.unRegionAdded = function (listener) {
                        this.regionAddedListeners = this.regionAddedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Regions.prototype.notifyRegionAdded = function (regionPath) {
                        var event = new _region.RegionAddedEvent(regionPath);
                        if (Regions.debug) {
                            console.debug("Regions.notifyRegionAdded: " + event.getRegionPath().toString());
                        }
                        this.regionAddedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChanged(event);
                    };
                    Regions.prototype.onRegionRemoved = function (listener) {
                        this.regionRemovedListeners.push(listener);
                    };
                    Regions.prototype.unRegionRemoved = function (listener) {
                        this.regionRemovedListeners = this.regionRemovedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Regions.prototype.notifyRegionRemoved = function (regionPath) {
                        var event = new _region.RegionRemovedEvent(regionPath);
                        if (Regions.debug) {
                            console.debug("Regions.notifyRegionRemoved: " + event.getRegionPath().toString());
                        }
                        this.regionRemovedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChanged(event);
                    };
                    Regions.create = function () {
                        return new RegionsBuilder();
                    };
                    Regions.debug = false;
                    return Regions;
                })();
                _region.Regions = Regions;
                var RegionsBuilder = (function () {
                    function RegionsBuilder(source) {
                        var _this = this;
                        this.regions = [];
                        if (source) {
                            source.getRegions().forEach(function (region) {
                                _this.regions.push(region.clone());
                            });
                        }
                    }
                    RegionsBuilder.prototype.fromJson = function (regionsJson, parent) {
                        var _this = this;
                        regionsJson.forEach(function (regionJson) {
                            var region = _region.Region.create().setName(regionJson.name).setParent(parent).build();
                            regionJson.components.forEach(function (componentJson, componentIndex) {
                                var component = _region.ComponentFactory.createFromJson(componentJson, componentIndex, region);
                                region.addComponent(component);
                            });
                            _this.addRegion(region);
                        });
                        return this;
                    };
                    RegionsBuilder.prototype.addRegion = function (value) {
                        this.regions.push(value);
                        return this;
                    };
                    RegionsBuilder.prototype.build = function () {
                        return new Regions(this);
                    };
                    return RegionsBuilder;
                })();
                _region.RegionsBuilder = RegionsBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var FormBuilder = api.form.FormBuilder;
                var OccurrencesBuilder = api.form.OccurrencesBuilder;
                var TextArea = api.form.inputtype.text.TextArea;
                var PropertyTree = api.data.PropertyTree;
                var ImageComponent = (function (_super) {
                    __extends(ImageComponent, _super);
                    function ImageComponent(builder) {
                        var _this = this;
                        _super.call(this, builder);
                        this.image = builder.image;
                        this.config = builder.config;
                        this.configChangedHandler = function (event) {
                            if (ImageComponent.debug) {
                                console.debug("ImageComponent[" + _this.getPath().toString() + "].config.onChanged: ", event);
                            }
                            if (!_this.disableEventForwarding) {
                                _this.notifyPropertyValueChanged(ImageComponent.PROPERTY_CONFIG);
                            }
                        };
                        this.config.onChanged(this.configChangedHandler);
                        var formBuilder = new FormBuilder();
                        formBuilder.addFormItem(new api.form.InputBuilder().setName("caption").setInputType(TextArea.getName()).setLabel("Caption").setOccurrences(new OccurrencesBuilder().setMinimum(0).setMaximum(1).build()).build());
                        this.form = formBuilder.build();
                    }
                    ImageComponent.prototype.setDisableEventForwarding = function (value) {
                        this.disableEventForwarding = value;
                    };
                    ImageComponent.prototype.getImage = function () {
                        return this.image;
                    };
                    ImageComponent.prototype.getForm = function () {
                        return this.form;
                    };
                    ImageComponent.prototype.getConfig = function () {
                        return this.config;
                    };
                    ImageComponent.prototype.setImage = function (contentId, name) {
                        var oldValue = this.image;
                        this.image = contentId;
                        this.setName(name ? new _region.ComponentName(name) : null);
                        if (!api.ObjectHelper.equals(oldValue, contentId)) {
                            this.notifyPropertyChanged(ImageComponent.PROPERTY_IMAGE);
                        }
                    };
                    ImageComponent.prototype.hasImage = function () {
                        return !!this.image;
                    };
                    ImageComponent.prototype.doReset = function () {
                        this.setImage(null, null);
                    };
                    ImageComponent.prototype.isEmpty = function () {
                        return !this.image;
                    };
                    ImageComponent.prototype.toJson = function () {
                        var json = _super.prototype.toComponentJson.call(this);
                        json.image = this.image != null ? this.image.toString() : null;
                        json.config = this.config != null ? this.config.toJson() : null;
                        return {
                            ImageComponent: json
                        };
                    };
                    ImageComponent.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ImageComponent)) {
                            return false;
                        }
                        var other = o;
                        if (!_super.prototype.equals.call(this, o)) {
                            return false;
                        }
                        if (!api.ObjectHelper.equals(this.image, other.image)) {
                            return false;
                        }
                        if (!api.ObjectHelper.equals(this.config, other.config)) {
                            return false;
                        }
                        return true;
                    };
                    ImageComponent.prototype.clone = function () {
                        return new ImageComponentBuilder(this).build();
                    };
                    ImageComponent.PROPERTY_IMAGE = 'image';
                    ImageComponent.PROPERTY_CONFIG = 'config';
                    ImageComponent.debug = false;
                    return ImageComponent;
                })(_region.Component);
                _region.ImageComponent = ImageComponent;
                var ImageComponentBuilder = (function (_super) {
                    __extends(ImageComponentBuilder, _super);
                    function ImageComponentBuilder(source) {
                        _super.call(this, source);
                        if (source) {
                            this.image = source.getImage();
                            this.config = source.getConfig() ? source.getConfig().copy() : null;
                        }
                        else {
                            this.config = new PropertyTree();
                        }
                    }
                    ImageComponentBuilder.prototype.setImage = function (value) {
                        this.image = value;
                        return this;
                    };
                    ImageComponentBuilder.prototype.setConfig = function (value) {
                        this.config = value;
                        return this;
                    };
                    ImageComponentBuilder.prototype.fromJson = function (json, region) {
                        if (json.image) {
                            this.setImage(new api.content.ContentId(json.image));
                        }
                        this.setName(json.name ? new _region.ComponentName(json.name) : null);
                        if (json.config) {
                            this.setConfig(PropertyTree.fromJson(json.config));
                        }
                        this.setParent(region);
                        return this;
                    };
                    ImageComponentBuilder.prototype.build = function () {
                        return new ImageComponent(this);
                    };
                    return ImageComponentBuilder;
                })(_region.ComponentBuilder);
                _region.ImageComponentBuilder = ImageComponentBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ImageComponentType = (function (_super) {
                    __extends(ImageComponentType, _super);
                    function ImageComponentType() {
                        _super.call(this, "image");
                    }
                    ImageComponentType.prototype.newComponentBuilder = function () {
                        return new region.ImageComponentBuilder();
                    };
                    return ImageComponentType;
                })(region.ComponentType);
                region.ImageComponentType = ImageComponentType;
                new ImageComponentType();
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var TextComponent = (function (_super) {
                    __extends(TextComponent, _super);
                    function TextComponent(builder) {
                        _super.call(this, builder);
                        if (builder) {
                            this.setText(builder.text, true);
                        }
                    }
                    TextComponent.prototype.getText = function () {
                        return this.text;
                    };
                    TextComponent.prototype.setText = function (value, silent) {
                        this.text = api.util.StringHelper.isBlank(value) ? undefined : value;
                        if (!silent) {
                            this.notifyPropertyChanged(TextComponent.PROPERTY_TEXT);
                        }
                    };
                    TextComponent.prototype.doReset = function () {
                        this.setText();
                    };
                    TextComponent.prototype.isEmpty = function () {
                        return api.util.StringHelper.isBlank(this.text);
                    };
                    TextComponent.prototype.toJson = function () {
                        var json = _super.prototype.toComponentJson.call(this);
                        json.text = this.text != null ? this.text : null;
                        return {
                            TextComponent: json
                        };
                    };
                    TextComponent.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, TextComponent)) {
                            return false;
                        }
                        var other = o;
                        if (!_super.prototype.equals.call(this, o)) {
                            return false;
                        }
                        if (!api.ObjectHelper.stringEquals(this.text, other.text)) {
                            return false;
                        }
                        return true;
                    };
                    TextComponent.prototype.clone = function () {
                        return new TextComponentBuilder(this).build();
                    };
                    TextComponent.PROPERTY_TEXT = "text";
                    return TextComponent;
                })(_region.Component);
                _region.TextComponent = TextComponent;
                var TextComponentBuilder = (function (_super) {
                    __extends(TextComponentBuilder, _super);
                    function TextComponentBuilder(source) {
                        _super.call(this, source);
                        if (source) {
                            this.text = source.getText();
                        }
                    }
                    TextComponentBuilder.prototype.fromJson = function (json, region) {
                        if (json.text) {
                            this.setText(json.text);
                        }
                        this.setName(json.name ? new _region.ComponentName(json.name) : null);
                        this.setParent(region);
                        return this;
                    };
                    TextComponentBuilder.prototype.setText = function (value) {
                        this.text = value;
                        return this;
                    };
                    TextComponentBuilder.prototype.build = function () {
                        return new TextComponent(this);
                    };
                    return TextComponentBuilder;
                })(_region.ComponentBuilder);
                _region.TextComponentBuilder = TextComponentBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var TextComponentType = (function (_super) {
                    __extends(TextComponentType, _super);
                    function TextComponentType() {
                        _super.call(this, "text");
                    }
                    TextComponentType.prototype.newComponentBuilder = function () {
                        return new region.TextComponentBuilder();
                    };
                    return TextComponentType;
                })(region.ComponentType);
                region.TextComponentType = TextComponentType;
                new TextComponentType();
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var PartDescriptor = (function (_super) {
                    __extends(PartDescriptor, _super);
                    function PartDescriptor() {
                        _super.apply(this, arguments);
                    }
                    PartDescriptor.prototype.clone = function () {
                        return new PartDescriptorBuilder(this).build();
                    };
                    return PartDescriptor;
                })(api.content.page.Descriptor);
                region.PartDescriptor = PartDescriptor;
                var PartDescriptorBuilder = (function (_super) {
                    __extends(PartDescriptorBuilder, _super);
                    function PartDescriptorBuilder(source) {
                        _super.call(this, source);
                    }
                    PartDescriptorBuilder.prototype.fromJson = function (json) {
                        this.setKey(api.content.page.DescriptorKey.fromString(json.key));
                        this.setName(new api.content.page.DescriptorName(json.name));
                        this.setDisplayName(json.displayName);
                        this.setConfig(json.config != null ? api.form.Form.fromJson(json.config) : null);
                        return this;
                    };
                    PartDescriptorBuilder.prototype.setKey = function (value) {
                        this.key = value;
                        return this;
                    };
                    PartDescriptorBuilder.prototype.setName = function (value) {
                        this.name = value;
                        return this;
                    };
                    PartDescriptorBuilder.prototype.setDisplayName = function (value) {
                        this.displayName = value;
                        return this;
                    };
                    PartDescriptorBuilder.prototype.setConfig = function (value) {
                        this.config = value;
                        return this;
                    };
                    PartDescriptorBuilder.prototype.build = function () {
                        return new PartDescriptor(this);
                    };
                    return PartDescriptorBuilder;
                })(api.content.page.DescriptorBuilder);
                region.PartDescriptorBuilder = PartDescriptorBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var Dropdown = api.ui.selector.dropdown.Dropdown;
                var PartDescriptorDropdown = (function (_super) {
                    __extends(PartDescriptorDropdown, _super);
                    function PartDescriptorDropdown(name, config) {
                        var _this = this;
                        _super.call(this, name, {
                            optionDisplayValueViewer: new region.PartDescriptorViewer(),
                            dataIdProperty: "value"
                        });
                        config.loader.onLoadedData(function (event) {
                            var descriptors = event.getData();
                            descriptors.forEach(function (descriptor) {
                                var indices = [];
                                indices.push(descriptor.getDisplayName());
                                indices.push(descriptor.getName().toString());
                                var option = {
                                    value: descriptor.getKey().toString(),
                                    displayValue: descriptor,
                                    indices: indices
                                };
                                _this.addOption(option);
                            });
                        });
                    }
                    PartDescriptorDropdown.prototype.setDescriptor = function (descriptor) {
                        if (descriptor) {
                            var option = this.getOptionByValue(descriptor.getKey().toString());
                            if (option) {
                                this.selectOption(option, true);
                            }
                        }
                        else {
                            this.reset();
                        }
                    };
                    PartDescriptorDropdown.prototype.getDescriptor = function (descriptorKey) {
                        if (descriptorKey) {
                            var option = this.getOptionByValue(descriptorKey.toString());
                            if (option) {
                                return option.displayValue;
                            }
                        }
                        return null;
                    };
                    return PartDescriptorDropdown;
                })(Dropdown);
                region.PartDescriptorDropdown = PartDescriptorDropdown;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var PropertyTree = api.data.PropertyTree;
                var PartComponent = (function (_super) {
                    __extends(PartComponent, _super);
                    function PartComponent(builder) {
                        _super.call(this, builder);
                    }
                    PartComponent.prototype.toJson = function () {
                        var json = _super.prototype.toComponentJson.call(this);
                        return {
                            PartComponent: json
                        };
                    };
                    PartComponent.prototype.isEmpty = function () {
                        return !this.hasDescriptor();
                    };
                    PartComponent.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, PartComponent)) {
                            return false;
                        }
                        return _super.prototype.equals.call(this, o);
                    };
                    PartComponent.prototype.clone = function () {
                        return new PartComponentBuilder(this).build();
                    };
                    return PartComponent;
                })(_region.DescriptorBasedComponent);
                _region.PartComponent = PartComponent;
                var PartComponentBuilder = (function (_super) {
                    __extends(PartComponentBuilder, _super);
                    function PartComponentBuilder(source) {
                        _super.call(this, source);
                    }
                    PartComponentBuilder.prototype.fromJson = function (json, region) {
                        if (json.descriptor) {
                            this.setDescriptor(api.content.page.DescriptorKey.fromString(json.descriptor));
                        }
                        this.setName(json.name ? new _region.ComponentName(json.name) : null);
                        if (json.config) {
                            this.setConfig(PropertyTree.fromJson(json.config));
                        }
                        this.setParent(region);
                        return this;
                    };
                    PartComponentBuilder.prototype.build = function () {
                        return new PartComponent(this);
                    };
                    return PartComponentBuilder;
                })(_region.DescriptorBasedComponentBuilder);
                _region.PartComponentBuilder = PartComponentBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var PartComponentType = (function (_super) {
                    __extends(PartComponentType, _super);
                    function PartComponentType() {
                        _super.call(this, "part");
                    }
                    PartComponentType.prototype.newComponentBuilder = function () {
                        return new region.PartComponentBuilder();
                    };
                    return PartComponentType;
                })(region.ComponentType);
                region.PartComponentType = PartComponentType;
                new PartComponentType();
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var PartDescriptorViewer = (function (_super) {
                    __extends(PartDescriptorViewer, _super);
                    function PartDescriptorViewer() {
                        _super.call(this);
                    }
                    PartDescriptorViewer.prototype.resolveDisplayName = function (object) {
                        return object.getDisplayName();
                    };
                    PartDescriptorViewer.prototype.resolveSubName = function (object, relativePath) {
                        if (relativePath === void 0) { relativePath = false; }
                        return object.getKey().toString();
                    };
                    PartDescriptorViewer.prototype.resolveIconClass = function (object) {
                        return "icon-puzzle icon-large";
                    };
                    return PartDescriptorViewer;
                })(api.ui.NamesAndIconViewer);
                region.PartDescriptorViewer = PartDescriptorViewer;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RichComboBox = api.ui.selector.combobox.RichComboBox;
                var RichComboBoxBuilder = api.ui.selector.combobox.RichComboBoxBuilder;
                var SelectedOption = api.ui.selector.combobox.SelectedOption;
                var BaseSelectedOptionView = api.ui.selector.combobox.BaseSelectedOptionView;
                var BaseSelectedOptionsView = api.ui.selector.combobox.BaseSelectedOptionsView;
                var PartDescriptorComboBox = (function (_super) {
                    __extends(PartDescriptorComboBox, _super);
                    function PartDescriptorComboBox(loader) {
                        _super.call(this, new RichComboBoxBuilder().setIdentifierMethod("getKey").setOptionDisplayValueViewer(new region.PartDescriptorViewer()).setSelectedOptionsView(new PartDescriptorSelectedOptionsView()).setLoader(loader).setMaximumOccurrences(1).setNextInputFocusWhenMaxReached(false));
                    }
                    PartDescriptorComboBox.prototype.getDescriptor = function (descriptorKey) {
                        var option = this.getOptionByValue(descriptorKey.toString());
                        if (option) {
                            return option.displayValue;
                        }
                        return null;
                    };
                    PartDescriptorComboBox.prototype.setDescriptor = function (descriptor) {
                        this.clearSelection();
                        if (descriptor) {
                            var optionToSelect = this.getOptionByValue(descriptor.getKey().toString());
                            if (!optionToSelect) {
                                optionToSelect = {
                                    value: descriptor.getKey().toString(),
                                    displayValue: descriptor
                                };
                                this.addOption(optionToSelect);
                            }
                            this.selectOption(optionToSelect);
                        }
                    };
                    return PartDescriptorComboBox;
                })(RichComboBox);
                region.PartDescriptorComboBox = PartDescriptorComboBox;
                var PartDescriptorSelectedOptionsView = (function (_super) {
                    __extends(PartDescriptorSelectedOptionsView, _super);
                    function PartDescriptorSelectedOptionsView() {
                        _super.apply(this, arguments);
                    }
                    PartDescriptorSelectedOptionsView.prototype.createSelectedOption = function (option) {
                        return new SelectedOption(new PartDescriptorSelectedOptionView(option), this.count());
                    };
                    return PartDescriptorSelectedOptionsView;
                })(BaseSelectedOptionsView);
                region.PartDescriptorSelectedOptionsView = PartDescriptorSelectedOptionsView;
                var PartDescriptorSelectedOptionView = (function (_super) {
                    __extends(PartDescriptorSelectedOptionView, _super);
                    function PartDescriptorSelectedOptionView(option) {
                        this.descriptor = option.displayValue;
                        _super.call(this, option);
                        this.addClass("part-descriptor-selected-option-view");
                    }
                    PartDescriptorSelectedOptionView.prototype.layout = function () {
                        var _this = this;
                        var namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(2 /* small */).build();
                        namesAndIconView.setIconClass("icon-puzzle icon-medium").setMainName(this.descriptor.getDisplayName()).setSubName(this.descriptor.getKey().toString());
                        var removeButtonEl = new api.dom.AEl("remove");
                        removeButtonEl.onClicked(function (event) {
                            _this.notifySelectedOptionRemoveRequested();
                            event.stopPropagation();
                            event.preventDefault();
                            return false;
                        });
                        this.appendChild(removeButtonEl);
                        this.appendChild(namesAndIconView);
                    };
                    return PartDescriptorSelectedOptionView;
                })(BaseSelectedOptionView);
                region.PartDescriptorSelectedOptionView = PartDescriptorSelectedOptionView;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var PartDescriptorLoader = (function (_super) {
                    __extends(PartDescriptorLoader, _super);
                    function PartDescriptorLoader(request) {
                        _super.call(this, request);
                    }
                    PartDescriptorLoader.prototype.filterFn = function (descriptor) {
                        return descriptor.getDisplayName().toString().toLowerCase().indexOf(this.getSearchString().toLowerCase()) != -1;
                    };
                    return PartDescriptorLoader;
                })(api.util.loader.BaseLoader);
                region.PartDescriptorLoader = PartDescriptorLoader;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var PartDescriptorResourceRequest = (function (_super) {
                    __extends(PartDescriptorResourceRequest, _super);
                    function PartDescriptorResourceRequest() {
                        _super.call(this);
                        this.cache = region.PartDescriptorCache.get();
                        this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "content", "page", "part", "descriptor");
                    }
                    PartDescriptorResourceRequest.prototype.getResourcePath = function () {
                        return this.resourcePath;
                    };
                    PartDescriptorResourceRequest.prototype.fromJsonToPartDescriptor = function (json) {
                        var partDescriptor = new region.PartDescriptorBuilder().fromJson(json).build();
                        this.cache.put(partDescriptor);
                        return partDescriptor;
                    };
                    return PartDescriptorResourceRequest;
                })(api.rest.ResourceRequest);
                region.PartDescriptorResourceRequest = PartDescriptorResourceRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var PartDescriptorsResourceRequest = (function (_super) {
                    __extends(PartDescriptorsResourceRequest, _super);
                    function PartDescriptorsResourceRequest() {
                        _super.apply(this, arguments);
                    }
                    PartDescriptorsResourceRequest.prototype.fromJsonToPartDescriptors = function (json) {
                        var _this = this;
                        var array = [];
                        json.descriptors.forEach(function (descriptorJson) {
                            array.push(_this.fromJsonToPartDescriptor(descriptorJson));
                        });
                        return array;
                    };
                    return PartDescriptorsResourceRequest;
                })(region.PartDescriptorResourceRequest);
                region.PartDescriptorsResourceRequest = PartDescriptorsResourceRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var GetPartDescriptorsByApplicationsRequest = (function (_super) {
                    __extends(GetPartDescriptorsByApplicationsRequest, _super);
                    function GetPartDescriptorsByApplicationsRequest(applicationKeys) {
                        _super.call(this);
                        this.applicationKeys = applicationKeys;
                    }
                    GetPartDescriptorsByApplicationsRequest.prototype.setApplicationKeys = function (applicationKeys) {
                        this.applicationKeys = applicationKeys;
                    };
                    GetPartDescriptorsByApplicationsRequest.prototype.getParams = function () {
                        throw new Error("Unexpected call");
                    };
                    GetPartDescriptorsByApplicationsRequest.prototype.getRequestPath = function () {
                        throw new Error("Unexpected call");
                    };
                    GetPartDescriptorsByApplicationsRequest.prototype.sendAndParse = function () {
                        var promises = this.applicationKeys.map(function (applicationKey) { return new region.GetPartDescriptorsByApplicationRequest(applicationKey).sendAndParse(); });
                        return wemQ.all(promises).then(function (results) {
                            var all = [];
                            results.forEach(function (result) {
                                Array.prototype.push.apply(all, result);
                            });
                            return all;
                        });
                    };
                    return GetPartDescriptorsByApplicationsRequest;
                })(region.PartDescriptorsResourceRequest);
                region.GetPartDescriptorsByApplicationsRequest = GetPartDescriptorsByApplicationsRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var GetPartDescriptorsByApplicationRequest = (function (_super) {
                    __extends(GetPartDescriptorsByApplicationRequest, _super);
                    function GetPartDescriptorsByApplicationRequest(applicationKey) {
                        _super.call(this);
                        _super.prototype.setMethod.call(this, "GET");
                        this.applicationKey = applicationKey;
                    }
                    GetPartDescriptorsByApplicationRequest.prototype.getParams = function () {
                        return {
                            applicationKey: this.applicationKey.toString()
                        };
                    };
                    GetPartDescriptorsByApplicationRequest.prototype.getRequestPath = function () {
                        return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "list", "by_application");
                    };
                    GetPartDescriptorsByApplicationRequest.prototype.sendAndParse = function () {
                        var _this = this;
                        var cached = this.cache.getByApplication(this.applicationKey);
                        if (cached) {
                            return wemQ(cached);
                        }
                        else {
                            return this.send().then(function (response) {
                                return _this.fromJsonToPartDescriptors(response.getResult());
                            });
                        }
                    };
                    return GetPartDescriptorsByApplicationRequest;
                })(region.PartDescriptorsResourceRequest);
                region.GetPartDescriptorsByApplicationRequest = GetPartDescriptorsByApplicationRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var GetPartDescriptorByKeyRequest = (function (_super) {
                    __extends(GetPartDescriptorByKeyRequest, _super);
                    function GetPartDescriptorByKeyRequest(key) {
                        _super.call(this);
                        this.key = key;
                    }
                    GetPartDescriptorByKeyRequest.prototype.setKey = function (key) {
                        this.key = key;
                    };
                    GetPartDescriptorByKeyRequest.prototype.getParams = function () {
                        throw new Error("Unexpected call");
                    };
                    GetPartDescriptorByKeyRequest.prototype.getRequestPath = function () {
                        throw new Error("Unexpected call");
                    };
                    GetPartDescriptorByKeyRequest.prototype.sendAndParse = function () {
                        var _this = this;
                        var deferred = wemQ.defer();
                        new region.GetPartDescriptorsByApplicationRequest(this.key.getApplicationKey()).sendAndParse().then(function (descriptors) {
                            descriptors.forEach(function (descriptor) {
                                if (_this.key.equals(descriptor.getKey())) {
                                    deferred.resolve(descriptor);
                                }
                            });
                        }).catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).done();
                        return deferred.promise;
                    };
                    return GetPartDescriptorByKeyRequest;
                })(region.PartDescriptorResourceRequest);
                region.GetPartDescriptorByKeyRequest = GetPartDescriptorByKeyRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ApplicationBasedCache = api.application.ApplicationBasedCache;
                var PartDescriptorCache = (function (_super) {
                    __extends(PartDescriptorCache, _super);
                    function PartDescriptorCache() {
                        if (PartDescriptorCache.instance) {
                            throw new Error("Instantiation failed: Use PartDescriptorCache.get() instead!");
                        }
                        _super.call(this);
                    }
                    PartDescriptorCache.get = function () {
                        var w = api.dom.WindowDOM.get();
                        var topWindow = w.getTopParent().asWindow();
                        if (!topWindow.api.content.page.region.PartDescriptorCache.instance) {
                            topWindow.api.content.page.region.PartDescriptorCache.instance = new PartDescriptorCache();
                        }
                        return topWindow.api.content.page.region.PartDescriptorCache.instance;
                    };
                    PartDescriptorCache.prototype.loadByApplication = function (applicationKey) {
                        new region.GetPartDescriptorsByApplicationRequest(applicationKey).sendAndParse().catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).done();
                    };
                    PartDescriptorCache.prototype.put = function (descriptor) {
                        api.util.assertNotNull(descriptor, "a PartDescriptor must be given");
                        _super.prototype.put.call(this, descriptor, descriptor.getKey().getApplicationKey());
                    };
                    PartDescriptorCache.prototype.getByKey = function (key) {
                        return _super.prototype.getByKey.call(this, key, key.getApplicationKey());
                    };
                    PartDescriptorCache.prototype.createApplicationCache = function () {
                        return new PartDescriptorApplicationCache();
                    };
                    return PartDescriptorCache;
                })(ApplicationBasedCache);
                region.PartDescriptorCache = PartDescriptorCache;
                var PartDescriptorApplicationCache = (function (_super) {
                    __extends(PartDescriptorApplicationCache, _super);
                    function PartDescriptorApplicationCache() {
                        _super.apply(this, arguments);
                    }
                    PartDescriptorApplicationCache.prototype.copy = function (object) {
                        return object.clone();
                    };
                    PartDescriptorApplicationCache.prototype.getKeyFromObject = function (object) {
                        return object.getKey();
                    };
                    PartDescriptorApplicationCache.prototype.getKeyAsString = function (key) {
                        return key.toString();
                    };
                    return PartDescriptorApplicationCache;
                })(api.cache.Cache);
                region.PartDescriptorApplicationCache = PartDescriptorApplicationCache;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var LayoutDescriptor = (function (_super) {
                    __extends(LayoutDescriptor, _super);
                    function LayoutDescriptor(builder) {
                        _super.call(this, builder);
                        this.regions = builder.regions;
                    }
                    LayoutDescriptor.prototype.getRegions = function () {
                        return this.regions;
                    };
                    LayoutDescriptor.prototype.clone = function () {
                        return new LayoutDescriptorBuilder(this).build();
                    };
                    return LayoutDescriptor;
                })(api.content.page.Descriptor);
                _region.LayoutDescriptor = LayoutDescriptor;
                var LayoutDescriptorBuilder = (function (_super) {
                    __extends(LayoutDescriptorBuilder, _super);
                    function LayoutDescriptorBuilder(source) {
                        _super.call(this, source);
                        this.regions = [];
                        if (source) {
                            this.regions = source.getRegions();
                        }
                    }
                    LayoutDescriptorBuilder.prototype.fromJson = function (json) {
                        this.setKey(api.content.page.DescriptorKey.fromString(json.key));
                        this.setName(new api.content.page.DescriptorName(json.name));
                        this.setDisplayName(json.displayName);
                        this.setConfig(json.config != null ? api.form.Form.fromJson(json.config) : null);
                        for (var i = 0; i < json.regions.length; i++) {
                            var region = new _region.RegionDescriptorBuilder().fromJson(json.regions[i]).build();
                            this.regions.push(region);
                        }
                        return this;
                    };
                    LayoutDescriptorBuilder.prototype.setKey = function (value) {
                        this.key = value;
                        return this;
                    };
                    LayoutDescriptorBuilder.prototype.setName = function (value) {
                        this.name = value;
                        return this;
                    };
                    LayoutDescriptorBuilder.prototype.setDisplayName = function (value) {
                        this.displayName = value;
                        return this;
                    };
                    LayoutDescriptorBuilder.prototype.setConfig = function (value) {
                        this.config = value;
                        return this;
                    };
                    LayoutDescriptorBuilder.prototype.addRegion = function (value) {
                        this.regions.push(value);
                        return this;
                    };
                    LayoutDescriptorBuilder.prototype.build = function () {
                        return new LayoutDescriptor(this);
                    };
                    return LayoutDescriptorBuilder;
                })(api.content.page.DescriptorBuilder);
                _region.LayoutDescriptorBuilder = LayoutDescriptorBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var LayoutComponentType = (function (_super) {
                    __extends(LayoutComponentType, _super);
                    function LayoutComponentType() {
                        _super.call(this, "layout");
                    }
                    LayoutComponentType.prototype.newComponentBuilder = function () {
                        return new region.LayoutComponentBuilder();
                    };
                    return LayoutComponentType;
                })(region.ComponentType);
                region.LayoutComponentType = LayoutComponentType;
                new LayoutComponentType();
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var PropertyTree = api.data.PropertyTree;
                var LayoutComponent = (function (_super) {
                    __extends(LayoutComponent, _super);
                    function LayoutComponent(builder) {
                        var _this = this;
                        _super.call(this, builder);
                        this.componentPropertyChangedListeners = [];
                        if (builder.regions) {
                            this.regions = builder.regions;
                            this.regions.getRegions().forEach(function (region) {
                                region.setParent(_this);
                            });
                        }
                        else {
                            this.regions = _region.Regions.create().build();
                        }
                        this.componentPropertyChangedEventHandler = function (event) { return _this.forwardComponentPropertyChangedEvent(event); };
                        this.regionsChangedEventHandler = function (event) {
                            if (LayoutComponent.debug) {
                                console.debug("LayoutComponent[" + _this.getPath().toString() + "].onChanged: ", event);
                            }
                            _this.notifyPropertyValueChanged("regions");
                        };
                        this.registerRegionsListeners(this.regions);
                    }
                    LayoutComponent.prototype.getComponent = function (path) {
                        return this.regions.getComponent(path);
                    };
                    LayoutComponent.prototype.getRegions = function () {
                        return this.regions;
                    };
                    LayoutComponent.prototype.setRegions = function (value) {
                        var oldValue = this.regions;
                        if (oldValue) {
                            this.unregisterRegionsListeners(oldValue);
                        }
                        this.regions = value;
                        this.registerRegionsListeners(this.regions);
                        if (!api.ObjectHelper.equals(oldValue, value)) {
                            if (LayoutComponent.debug) {
                                console.debug("LayoutComponent[" + this.getPath().toString() + "].regions reassigned: ", api.event);
                            }
                            this.notifyPropertyChanged("regions");
                        }
                    };
                    LayoutComponent.prototype.setDescriptor = function (descriptorKey, descriptor) {
                        _super.prototype.setDescriptor.call(this, descriptorKey, descriptor);
                        if (descriptor) {
                            this.addRegions(descriptor);
                        }
                    };
                    LayoutComponent.prototype.addRegions = function (layoutDescriptor) {
                        var sourceRegions = this.getRegions();
                        var mergedRegions = sourceRegions.mergeRegions(layoutDescriptor.getRegions(), this);
                        this.setRegions(mergedRegions);
                    };
                    LayoutComponent.prototype.isEmpty = function () {
                        return !this.hasDescriptor();
                    };
                    LayoutComponent.prototype.toJson = function () {
                        var json = _super.prototype.toComponentJson.call(this);
                        json.regions = this.regions.toJson();
                        return {
                            LayoutComponent: json
                        };
                    };
                    LayoutComponent.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, LayoutComponent)) {
                            return false;
                        }
                        var other = o;
                        if (!_super.prototype.equals.call(this, o)) {
                            return false;
                        }
                        if (!api.ObjectHelper.equals(this.regions, other.regions)) {
                            return false;
                        }
                        return true;
                    };
                    LayoutComponent.prototype.clone = function () {
                        return new LayoutComponentBuilder(this).build();
                    };
                    LayoutComponent.prototype.registerRegionsListeners = function (regions) {
                        regions.onChanged(this.regionsChangedEventHandler);
                        regions.onComponentPropertyChanged(this.componentPropertyChangedEventHandler);
                    };
                    LayoutComponent.prototype.unregisterRegionsListeners = function (regions) {
                        regions.unChanged(this.regionsChangedEventHandler);
                        regions.unComponentPropertyChanged(this.componentPropertyChangedEventHandler);
                    };
                    LayoutComponent.prototype.onComponentPropertyChanged = function (listener) {
                        this.componentPropertyChangedListeners.push(listener);
                    };
                    LayoutComponent.prototype.unComponentPropertyChanged = function (listener) {
                        this.componentPropertyChangedListeners = this.componentPropertyChangedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    LayoutComponent.debug = false;
                    return LayoutComponent;
                })(_region.DescriptorBasedComponent);
                _region.LayoutComponent = LayoutComponent;
                var LayoutComponentBuilder = (function (_super) {
                    __extends(LayoutComponentBuilder, _super);
                    function LayoutComponentBuilder(source) {
                        _super.call(this, source);
                        if (source) {
                            this.regions = source.getRegions().clone();
                        }
                    }
                    LayoutComponentBuilder.prototype.fromJson = function (json, region) {
                        if (json.descriptor) {
                            this.setDescriptor(api.content.page.DescriptorKey.fromString(json.descriptor));
                        }
                        this.setName(json.name ? new _region.ComponentName(json.name) : null);
                        if (json.config) {
                            this.setConfig(PropertyTree.fromJson(json.config));
                        }
                        this.setParent(region);
                        var layoutComponent = this.build();
                        var layoutRegions = _region.Regions.create().fromJson(json.regions, layoutComponent).build();
                        layoutComponent.setRegions(layoutRegions);
                        return layoutComponent;
                    };
                    LayoutComponentBuilder.prototype.setRegions = function (value) {
                        this.regions = value;
                        return this;
                    };
                    LayoutComponentBuilder.prototype.build = function () {
                        return new LayoutComponent(this);
                    };
                    return LayoutComponentBuilder;
                })(_region.DescriptorBasedComponentBuilder);
                _region.LayoutComponentBuilder = LayoutComponentBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var LayoutDescriptorViewer = (function (_super) {
                    __extends(LayoutDescriptorViewer, _super);
                    function LayoutDescriptorViewer() {
                        _super.call(this);
                    }
                    LayoutDescriptorViewer.prototype.resolveDisplayName = function (object) {
                        return object.getDisplayName();
                    };
                    LayoutDescriptorViewer.prototype.resolveSubName = function (object, relativePath) {
                        if (relativePath === void 0) { relativePath = false; }
                        return object.getKey().toString();
                    };
                    LayoutDescriptorViewer.prototype.resolveIconClass = function (object) {
                        return "icon-insert-template icon-large";
                    };
                    return LayoutDescriptorViewer;
                })(api.ui.NamesAndIconViewer);
                region.LayoutDescriptorViewer = LayoutDescriptorViewer;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RichComboBox = api.ui.selector.combobox.RichComboBox;
                var RichComboBoxBuilder = api.ui.selector.combobox.RichComboBoxBuilder;
                var SelectedOption = api.ui.selector.combobox.SelectedOption;
                var BaseSelectedOptionView = api.ui.selector.combobox.BaseSelectedOptionView;
                var BaseSelectedOptionsView = api.ui.selector.combobox.BaseSelectedOptionsView;
                var LayoutDescriptorComboBox = (function (_super) {
                    __extends(LayoutDescriptorComboBox, _super);
                    function LayoutDescriptorComboBox(loader) {
                        _super.call(this, new RichComboBoxBuilder().setIdentifierMethod("getKey").setOptionDisplayValueViewer(new region.LayoutDescriptorViewer()).setSelectedOptionsView(new LayoutDescriptorSelectedOptionsView()).setLoader(loader).setMaximumOccurrences(1).setNextInputFocusWhenMaxReached(false));
                    }
                    LayoutDescriptorComboBox.prototype.getDescriptor = function (descriptorKey) {
                        var option = this.getOptionByValue(descriptorKey.toString());
                        if (option) {
                            return option.displayValue;
                        }
                        return null;
                    };
                    LayoutDescriptorComboBox.prototype.setDescriptor = function (descriptor) {
                        this.clearSelection();
                        if (descriptor) {
                            var optionToSelect = this.getOptionByValue(descriptor.getKey().toString());
                            if (!optionToSelect) {
                                optionToSelect = {
                                    value: descriptor.getKey().toString(),
                                    displayValue: descriptor
                                };
                                this.addOption(optionToSelect);
                            }
                            this.selectOption(optionToSelect);
                        }
                    };
                    return LayoutDescriptorComboBox;
                })(RichComboBox);
                region.LayoutDescriptorComboBox = LayoutDescriptorComboBox;
                var LayoutDescriptorSelectedOptionsView = (function (_super) {
                    __extends(LayoutDescriptorSelectedOptionsView, _super);
                    function LayoutDescriptorSelectedOptionsView() {
                        _super.apply(this, arguments);
                    }
                    LayoutDescriptorSelectedOptionsView.prototype.createSelectedOption = function (option) {
                        return new SelectedOption(new LayoutDescriptorSelectedOptionView(option), this.count());
                    };
                    return LayoutDescriptorSelectedOptionsView;
                })(BaseSelectedOptionsView);
                region.LayoutDescriptorSelectedOptionsView = LayoutDescriptorSelectedOptionsView;
                var LayoutDescriptorSelectedOptionView = (function (_super) {
                    __extends(LayoutDescriptorSelectedOptionView, _super);
                    function LayoutDescriptorSelectedOptionView(option) {
                        this.descriptor = option.displayValue;
                        _super.call(this, option);
                        this.addClass("layout-descriptor-selected-option-view");
                    }
                    LayoutDescriptorSelectedOptionView.prototype.layout = function () {
                        var _this = this;
                        var namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(2 /* small */).build();
                        namesAndIconView.setIconClass("icon-earth icon-medium").setMainName(this.descriptor.getDisplayName()).setSubName(this.descriptor.getKey().toString());
                        var removeButtonEl = new api.dom.AEl("remove");
                        removeButtonEl.onClicked(function (event) {
                            _this.notifySelectedOptionRemoveRequested();
                            event.stopPropagation();
                            event.preventDefault();
                            return false;
                        });
                        this.appendChild(removeButtonEl);
                        this.appendChild(namesAndIconView);
                    };
                    return LayoutDescriptorSelectedOptionView;
                })(BaseSelectedOptionView);
                region.LayoutDescriptorSelectedOptionView = LayoutDescriptorSelectedOptionView;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var LayoutDescriptorLoader = (function (_super) {
                    __extends(LayoutDescriptorLoader, _super);
                    function LayoutDescriptorLoader(request) {
                        _super.call(this, request);
                    }
                    LayoutDescriptorLoader.prototype.filterFn = function (descriptor) {
                        return descriptor.getDisplayName().toString().toLowerCase().indexOf(this.getSearchString().toLowerCase()) != -1;
                    };
                    return LayoutDescriptorLoader;
                })(api.util.loader.BaseLoader);
                region.LayoutDescriptorLoader = LayoutDescriptorLoader;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var LayoutDescriptorResourceRequest = (function (_super) {
                    __extends(LayoutDescriptorResourceRequest, _super);
                    function LayoutDescriptorResourceRequest() {
                        _super.call(this);
                        this.cache = region.LayoutDescriptorCache.get();
                        this.resourcePath = api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "content", "page", "layout", "descriptor");
                    }
                    LayoutDescriptorResourceRequest.prototype.getResourcePath = function () {
                        return this.resourcePath;
                    };
                    LayoutDescriptorResourceRequest.prototype.fromJsonToLayoutDescriptor = function (json) {
                        var descriptor = new region.LayoutDescriptorBuilder().fromJson(json).build();
                        this.cache.put(descriptor);
                        return descriptor;
                    };
                    return LayoutDescriptorResourceRequest;
                })(api.rest.ResourceRequest);
                region.LayoutDescriptorResourceRequest = LayoutDescriptorResourceRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var LayoutDescriptorsResourceRequest = (function (_super) {
                    __extends(LayoutDescriptorsResourceRequest, _super);
                    function LayoutDescriptorsResourceRequest() {
                        _super.apply(this, arguments);
                    }
                    LayoutDescriptorsResourceRequest.prototype.fromJsonToLayoutDescriptors = function (json) {
                        var _this = this;
                        var array = [];
                        json.descriptors.forEach(function (descriptorJson) {
                            array.push(_this.fromJsonToLayoutDescriptor(descriptorJson));
                        });
                        return array;
                    };
                    return LayoutDescriptorsResourceRequest;
                })(region.LayoutDescriptorResourceRequest);
                region.LayoutDescriptorsResourceRequest = LayoutDescriptorsResourceRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var GetLayoutDescriptorsByApplicationRequest = (function (_super) {
                    __extends(GetLayoutDescriptorsByApplicationRequest, _super);
                    function GetLayoutDescriptorsByApplicationRequest(applicationKey) {
                        _super.call(this);
                        _super.prototype.setMethod.call(this, "GET");
                        this.applicationKey = applicationKey;
                    }
                    GetLayoutDescriptorsByApplicationRequest.prototype.getParams = function () {
                        return {
                            applicationKey: this.applicationKey.toString()
                        };
                    };
                    GetLayoutDescriptorsByApplicationRequest.prototype.getRequestPath = function () {
                        return api.rest.Path.fromParent(_super.prototype.getResourcePath.call(this), "list", "by_application");
                    };
                    GetLayoutDescriptorsByApplicationRequest.prototype.sendAndParse = function () {
                        var _this = this;
                        var cached = this.cache.getByApplication(this.applicationKey);
                        if (cached) {
                            return wemQ(cached);
                        }
                        else {
                            return this.send().then(function (response) {
                                return _this.fromJsonToLayoutDescriptors(response.getResult());
                            });
                        }
                    };
                    return GetLayoutDescriptorsByApplicationRequest;
                })(region.LayoutDescriptorsResourceRequest);
                region.GetLayoutDescriptorsByApplicationRequest = GetLayoutDescriptorsByApplicationRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var GetLayoutDescriptorsByApplicationsRequest = (function (_super) {
                    __extends(GetLayoutDescriptorsByApplicationsRequest, _super);
                    function GetLayoutDescriptorsByApplicationsRequest(applicationKey) {
                        _super.call(this);
                        this.applicationKeys = applicationKey;
                    }
                    GetLayoutDescriptorsByApplicationsRequest.prototype.setApplicationKeys = function (applicationKeys) {
                        this.applicationKeys = applicationKeys;
                    };
                    GetLayoutDescriptorsByApplicationsRequest.prototype.getParams = function () {
                        throw new Error("Unexpected call");
                    };
                    GetLayoutDescriptorsByApplicationsRequest.prototype.getRequestPath = function () {
                        throw new Error("Unexpected call");
                    };
                    GetLayoutDescriptorsByApplicationsRequest.prototype.sendAndParse = function () {
                        var promises = this.applicationKeys.map(function (applicationKey) { return new region.GetLayoutDescriptorsByApplicationRequest(applicationKey).sendAndParse(); });
                        return wemQ.all(promises).then(function (results) {
                            var all = [];
                            results.forEach(function (result) {
                                Array.prototype.push.apply(all, result);
                            });
                            return all;
                        });
                    };
                    return GetLayoutDescriptorsByApplicationsRequest;
                })(region.LayoutDescriptorsResourceRequest);
                region.GetLayoutDescriptorsByApplicationsRequest = GetLayoutDescriptorsByApplicationsRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var GetLayoutDescriptorByKeyRequest = (function (_super) {
                    __extends(GetLayoutDescriptorByKeyRequest, _super);
                    function GetLayoutDescriptorByKeyRequest(key) {
                        _super.call(this);
                        this.key = key;
                    }
                    GetLayoutDescriptorByKeyRequest.prototype.setKey = function (key) {
                        this.key = key;
                    };
                    GetLayoutDescriptorByKeyRequest.prototype.getParams = function () {
                        throw new Error("Unexpected call");
                    };
                    GetLayoutDescriptorByKeyRequest.prototype.getRequestPath = function () {
                        throw new Error("Unexpected call");
                    };
                    GetLayoutDescriptorByKeyRequest.prototype.sendAndParse = function () {
                        var _this = this;
                        var deferred = wemQ.defer();
                        new region.GetLayoutDescriptorsByApplicationRequest(this.key.getApplicationKey()).sendAndParse().then(function (descriptors) {
                            descriptors.forEach(function (descriptor) {
                                if (_this.key.equals(descriptor.getKey())) {
                                    deferred.resolve(descriptor);
                                }
                            });
                        }).catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).done();
                        return deferred.promise;
                    };
                    return GetLayoutDescriptorByKeyRequest;
                })(region.LayoutDescriptorResourceRequest);
                region.GetLayoutDescriptorByKeyRequest = GetLayoutDescriptorByKeyRequest;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var Dropdown = api.ui.selector.dropdown.Dropdown;
                var LayoutDescriptorDropdown = (function (_super) {
                    __extends(LayoutDescriptorDropdown, _super);
                    function LayoutDescriptorDropdown(name, config) {
                        var _this = this;
                        _super.call(this, name, {
                            optionDisplayValueViewer: new region.LayoutDescriptorViewer(),
                            dataIdProperty: "value"
                        });
                        config.loader.onLoadedData(function (event) {
                            var descriptors = event.getData();
                            descriptors.forEach(function (descriptor) {
                                var indices = [];
                                indices.push(descriptor.getDisplayName());
                                indices.push(descriptor.getName().toString());
                                var option = {
                                    value: descriptor.getKey().toString(),
                                    displayValue: descriptor,
                                    indices: indices
                                };
                                _this.addOption(option);
                            });
                        });
                    }
                    LayoutDescriptorDropdown.prototype.setDescriptor = function (descriptor) {
                        if (descriptor) {
                            var option = this.getOptionByValue(descriptor.getKey().toString());
                            if (option) {
                                this.selectOption(option, true);
                            }
                        }
                        else {
                            this.reset();
                        }
                    };
                    LayoutDescriptorDropdown.prototype.getDescriptor = function (descriptorKey) {
                        if (descriptorKey) {
                            var option = this.getOptionByValue(descriptorKey.toString());
                            if (option) {
                                return option.displayValue;
                            }
                        }
                        return null;
                    };
                    return LayoutDescriptorDropdown;
                })(Dropdown);
                region.LayoutDescriptorDropdown = LayoutDescriptorDropdown;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (_region) {
                var LayoutRegionsMerger = (function () {
                    function LayoutRegionsMerger() {
                    }
                    LayoutRegionsMerger.prototype.merge = function (regions, layoutDescriptorRegions, parent) {
                        var _this = this;
                        this.layoutComponentRegions = regions.getRegions();
                        this.initLookupTables(layoutDescriptorRegions, parent);
                        this.mergeExistingRegions();
                        this.mergeMissingRegions(layoutDescriptorRegions);
                        var layoutRegionsBuilder = _region.Regions.create();
                        layoutDescriptorRegions.forEach(function (regionDescriptor) {
                            var layoutRegion = _this.targetRegionsByName[regionDescriptor.getName()];
                            layoutRegionsBuilder.addRegion(layoutRegion);
                        });
                        return layoutRegionsBuilder.build();
                    };
                    LayoutRegionsMerger.prototype.mergeExistingRegions = function () {
                        var _this = this;
                        this.layoutComponentRegions.forEach(function (region) {
                            var regionName = region.getName();
                            if (_this.targetRegionsByName[regionName]) {
                                var targetRegion = _this.targetRegionsByName[regionName];
                                var updatedRegion = _this.addComponents(region, targetRegion);
                                _this.targetRegionsByName[regionName] = updatedRegion;
                            }
                        });
                    };
                    LayoutRegionsMerger.prototype.mergeMissingRegions = function (layoutDescriptorRegions) {
                        var _this = this;
                        var lastRegionName = layoutDescriptorRegions[layoutDescriptorRegions.length - 1].getName();
                        this.layoutComponentRegions.forEach(function (region) {
                            var regionName = region.getName();
                            if (!_this.targetRegionsByName[regionName]) {
                                var sourceRegionPos = _this.sourceRegionsPositionByName[regionName];
                                var targetRegionName = _this.targetRegionsNameByPosition[sourceRegionPos] || lastRegionName;
                                var targetRegion = _this.targetRegionsByName[targetRegionName];
                                var updatedRegion = _this.addComponents(region, targetRegion);
                                _this.targetRegionsByName[targetRegionName] = updatedRegion;
                            }
                        });
                    };
                    LayoutRegionsMerger.prototype.initLookupTables = function (layoutDescriptorRegions, parent) {
                        var _this = this;
                        this.targetRegionsByName = {};
                        this.targetRegionsNameByPosition = {};
                        this.sourceRegionsPositionByName = {};
                        layoutDescriptorRegions.forEach(function (regionDescriptor, idx) {
                            var regionName = regionDescriptor.getName();
                            var layoutRegion = _region.Region.create().setName(regionName).setParent(parent).build();
                            _this.targetRegionsByName[regionName] = layoutRegion;
                            _this.targetRegionsNameByPosition[idx] = regionName;
                        });
                        this.layoutComponentRegions.forEach(function (region, idx) {
                            _this.sourceRegionsPositionByName[region.getName()] = idx;
                        });
                    };
                    LayoutRegionsMerger.prototype.addComponents = function (fromRegion, toRegion) {
                        if (fromRegion.getComponents().length === 0) {
                            return toRegion;
                        }
                        var result = _region.Region.create(toRegion);
                        fromRegion.getComponents().forEach(function (component) {
                            result.addComponent(component);
                        });
                        return result.build();
                    };
                    return LayoutRegionsMerger;
                })();
                _region.LayoutRegionsMerger = LayoutRegionsMerger;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ApplicationBasedCache = api.application.ApplicationBasedCache;
                var LayoutDescriptorCache = (function (_super) {
                    __extends(LayoutDescriptorCache, _super);
                    function LayoutDescriptorCache() {
                        if (LayoutDescriptorCache.instance) {
                            throw new Error("Instantiation failed: Use LayoutDescriptorCache.get() instead!");
                        }
                        _super.call(this);
                    }
                    LayoutDescriptorCache.get = function () {
                        var w = api.dom.WindowDOM.get();
                        var topWindow = w.getTopParent().asWindow();
                        if (!topWindow.api.content.page.region.LayoutDescriptorCache.instance) {
                            topWindow.api.content.page.region.LayoutDescriptorCache.instance = new LayoutDescriptorCache();
                        }
                        return topWindow.api.content.page.region.LayoutDescriptorCache.instance;
                    };
                    LayoutDescriptorCache.prototype.loadByApplication = function (applicationKey) {
                        new region.GetLayoutDescriptorsByApplicationRequest(applicationKey).sendAndParse().catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).done();
                    };
                    LayoutDescriptorCache.prototype.put = function (descriptor) {
                        api.util.assertNotNull(descriptor, "a LayoutDescriptor must be given");
                        _super.prototype.put.call(this, descriptor, descriptor.getKey().getApplicationKey());
                    };
                    LayoutDescriptorCache.prototype.getByKey = function (key) {
                        return _super.prototype.getByKey.call(this, key, key.getApplicationKey());
                    };
                    LayoutDescriptorCache.prototype.createApplicationCache = function () {
                        return new LayoutDescriptorApplicationCache();
                    };
                    return LayoutDescriptorCache;
                })(ApplicationBasedCache);
                region.LayoutDescriptorCache = LayoutDescriptorCache;
                var LayoutDescriptorApplicationCache = (function (_super) {
                    __extends(LayoutDescriptorApplicationCache, _super);
                    function LayoutDescriptorApplicationCache() {
                        _super.apply(this, arguments);
                    }
                    LayoutDescriptorApplicationCache.prototype.copy = function (object) {
                        return object.clone();
                    };
                    LayoutDescriptorApplicationCache.prototype.getKeyFromObject = function (object) {
                        return object.getKey();
                    };
                    LayoutDescriptorApplicationCache.prototype.getKeyAsString = function (key) {
                        return key.toString();
                    };
                    return LayoutDescriptorApplicationCache;
                })(api.cache.Cache);
                region.LayoutDescriptorApplicationCache = LayoutDescriptorApplicationCache;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionPath = (function () {
                    function RegionPath(parentComponentPath, regionName) {
                        this.parentComponentPath = parentComponentPath;
                        this.regionName = regionName;
                        if (parentComponentPath != null) {
                            this.refString = parentComponentPath + "/" + regionName;
                        }
                        else {
                            this.refString = regionName;
                        }
                    }
                    RegionPath.prototype.hasParentComponentPath = function () {
                        return this.parentComponentPath != null;
                    };
                    RegionPath.prototype.getParentComponentPath = function () {
                        return this.parentComponentPath;
                    };
                    RegionPath.prototype.getRegionName = function () {
                        return this.regionName;
                    };
                    RegionPath.prototype.toString = function () {
                        return this.refString;
                    };
                    RegionPath.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, RegionPath)) {
                            return false;
                        }
                        var other = o;
                        if (!api.ObjectHelper.stringEquals(this.refString, other.refString)) {
                            return false;
                        }
                        return true;
                    };
                    RegionPath.fromString = function (str) {
                        var lastDivider = str.lastIndexOf(RegionPath.DIVIDER);
                        if (lastDivider == -1) {
                            return new RegionPath(null, str);
                        }
                        var regionNameStart = lastDivider + 1;
                        var regionName = str.substring(regionNameStart, str.length);
                        var componentPathAsString = str.substring(0, regionNameStart);
                        var parentPath = region.ComponentPath.fromString(componentPathAsString);
                        return new RegionPath(parentPath, regionName);
                    };
                    RegionPath.DIVIDER = "/";
                    return RegionPath;
                })();
                region.RegionPath = RegionPath;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionDescriptor = (function () {
                    function RegionDescriptor(builder) {
                        this.name = builder.name;
                    }
                    RegionDescriptor.prototype.getName = function () {
                        return this.name;
                    };
                    return RegionDescriptor;
                })();
                region.RegionDescriptor = RegionDescriptor;
                var RegionDescriptorBuilder = (function () {
                    function RegionDescriptorBuilder() {
                    }
                    RegionDescriptorBuilder.prototype.fromJson = function (json) {
                        this.name = json.name;
                        return this;
                    };
                    RegionDescriptorBuilder.prototype.setName = function (value) {
                        this.name = value;
                        return this;
                    };
                    RegionDescriptorBuilder.prototype.build = function () {
                        return new RegionDescriptor(this);
                    };
                    return RegionDescriptorBuilder;
                })();
                region.RegionDescriptorBuilder = RegionDescriptorBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var BaseRegionChangedEvent = (function () {
                    function BaseRegionChangedEvent(path) {
                        this.path = path;
                    }
                    BaseRegionChangedEvent.prototype.getPath = function () {
                        return this.path;
                    };
                    return BaseRegionChangedEvent;
                })();
                region.BaseRegionChangedEvent = BaseRegionChangedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionPropertyChangedEvent = (function (_super) {
                    __extends(RegionPropertyChangedEvent, _super);
                    function RegionPropertyChangedEvent(path, propertyName) {
                        _super.call(this, path);
                        this.propertyName = propertyName;
                    }
                    RegionPropertyChangedEvent.prototype.getPropertyName = function () {
                        return this.propertyName;
                    };
                    return RegionPropertyChangedEvent;
                })(region.BaseRegionChangedEvent);
                region.RegionPropertyChangedEvent = RegionPropertyChangedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var RegionPropertyValueChangedEvent = (function (_super) {
                    __extends(RegionPropertyValueChangedEvent, _super);
                    function RegionPropertyValueChangedEvent(path, propertyName) {
                        _super.call(this, path);
                        this.propertyName = propertyName;
                    }
                    RegionPropertyValueChangedEvent.prototype.getPropertyName = function () {
                        return this.propertyName;
                    };
                    return RegionPropertyValueChangedEvent;
                })(region.BaseRegionChangedEvent);
                region.RegionPropertyValueChangedEvent = RegionPropertyValueChangedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentAddedEvent = (function (_super) {
                    __extends(ComponentAddedEvent, _super);
                    function ComponentAddedEvent(regionPath, componentPath) {
                        _super.call(this, regionPath);
                        this.componentPath = componentPath;
                    }
                    ComponentAddedEvent.prototype.getComponentPath = function () {
                        return this.componentPath;
                    };
                    return ComponentAddedEvent;
                })(region.BaseRegionChangedEvent);
                region.ComponentAddedEvent = ComponentAddedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var ComponentRemovedEvent = (function (_super) {
                    __extends(ComponentRemovedEvent, _super);
                    function ComponentRemovedEvent(regionPath, componentPath) {
                        _super.call(this, regionPath);
                        this.componentPath = componentPath;
                    }
                    ComponentRemovedEvent.prototype.getComponentPath = function () {
                        return this.componentPath;
                    };
                    return ComponentRemovedEvent;
                })(region.BaseRegionChangedEvent);
                region.ComponentRemovedEvent = ComponentRemovedEvent;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var page;
        (function (page) {
            var region;
            (function (region) {
                var Region = (function () {
                    function Region(builder) {
                        var _this = this;
                        this.components = [];
                        this.changedListeners = [];
                        this.componentAddedListeners = [];
                        this.componentRemovedListeners = [];
                        this.componentPropertyChangedListeners = [];
                        this.propertyValueChangedListeners = [];
                        this.name = builder.name;
                        this.parent = builder.parent;
                        this.componentChangedEventHandler = function (event) {
                            if (Region.debug) {
                                console.debug(_this.toString() + ".handleComponentChanged: ", event);
                            }
                            _this.notifyRegionPropertyValueChanged("components");
                        };
                        this.componentPropertyChangedEventHandler = function (event) { return _this.forwardComponentPropertyChangedEvent(event); };
                        builder.components.forEach(function (component) {
                            _this.registerComponent(component);
                        });
                    }
                    Region.prototype.getName = function () {
                        return this.name;
                    };
                    Region.prototype.setParent = function (value) {
                        this.parent = value;
                    };
                    Region.prototype.getParent = function () {
                        return this.parent;
                    };
                    Region.prototype.getPath = function () {
                        var parentPath = null;
                        if (this.parent) {
                            parentPath = this.parent.getPath();
                        }
                        return new region.RegionPath(parentPath, this.name);
                    };
                    Region.prototype.isEmpty = function () {
                        return !this.components || this.components.length == 0;
                    };
                    Region.prototype.empty = function () {
                        if (Region.debug) {
                            console.debug(this.toString() + ".empty()", this.components);
                        }
                        while (this.components.length > 0) {
                            this.removeComponent(this.components[0]);
                        }
                    };
                    Region.prototype.addComponent = function (component, index) {
                        if (Region.debug) {
                            console.debug(this.toString() + ".addComponent: " + component.toString());
                        }
                        this.registerComponent(component, index);
                        this.notifyComponentAdded(component.getPath());
                        return component;
                    };
                    Region.prototype.removeComponent = function (component) {
                        if (Region.debug) {
                            console.debug(this.toString() + ".removeComponent: " + component.toString(), this.components);
                        }
                        var path = component.getPath();
                        this.unregisterComponent(component);
                        this.notifyComponentRemoved(path);
                        return component;
                    };
                    Region.prototype.getComponents = function () {
                        return this.components;
                    };
                    Region.prototype.getComponentByIndex = function (index) {
                        var component = this.components[index];
                        if (!component) {
                            var message = "The rendered page is not consistent with the page components structure. Expected component with index " + index + " was not found in region '" + this.getName() + "'.";
                            console.error(message);
                            throw new api.Exception(message, 1 /* ERROR */);
                        }
                        api.util.assertState(component.getIndex() == index, "Index of Component is not as expected. Expected [" + index + "], was: " + component.getIndex());
                        return component;
                    };
                    Region.prototype.toJson = function () {
                        var componentJsons = [];
                        this.components.forEach(function (component) {
                            componentJsons.push(component.toJson());
                        });
                        return {
                            name: this.name,
                            components: componentJsons
                        };
                    };
                    Region.prototype.toString = function () {
                        return "Region[" + this.getPath().toString() + "]";
                    };
                    Region.prototype.equals = function (o) {
                        if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Region)) {
                            return false;
                        }
                        var other = o;
                        if (!api.ObjectHelper.stringEquals(this.name, other.name)) {
                            return false;
                        }
                        if (!api.ObjectHelper.arrayEquals(this.components, other.components)) {
                            return false;
                        }
                        return true;
                    };
                    Region.prototype.clone = function () {
                        return new RegionBuilder(this).build();
                    };
                    Region.prototype.checkIllegalLayoutComponentWithinLayoutComponent = function (component, parent) {
                        if (!!parent && api.ObjectHelper.iFrameSafeInstanceOf(component, region.LayoutComponent)) {
                            throw new Error("Not allowed to have a LayoutComponent within a LayoutComponent: " + component.getPath().toString());
                        }
                    };
                    Region.prototype.refreshIndexes = function (start) {
                        for (var i = Math.min(0, start); i < this.components.length; i++) {
                            this.components[i].setIndex(i);
                        }
                    };
                    Region.prototype.registerComponent = function (component, index) {
                        if (Region.debug) {
                            console.debug(this.toString() + ".registerComponent: " + component.toString() + " at " + component.getIndex());
                        }
                        this.checkIllegalLayoutComponentWithinLayoutComponent(component, this.parent);
                        if (index >= 0 && index < this.components.length) {
                            this.components.splice(index, 0, component);
                            this.refreshIndexes(index);
                        }
                        else {
                            this.components.push(component);
                            component.setIndex(this.components.length - 1);
                        }
                        component.setParent(this);
                        component.onChanged(this.componentChangedEventHandler);
                        component.onPropertyChanged(this.componentPropertyChangedEventHandler);
                    };
                    Region.prototype.unregisterComponent = function (component) {
                        if (Region.debug) {
                            console.debug(this.toString() + ".unregisterComponent: " + component.toString(), this.components);
                        }
                        component.unChanged(this.componentChangedEventHandler);
                        component.unPropertyChanged(this.componentPropertyChangedEventHandler);
                        var index = component.getIndex();
                        if (index == -1) {
                            throw new Error(component.toString() + " to remove does not exist in " + this.toString());
                        }
                        this.components.splice(index, 1);
                        this.refreshIndexes(index);
                        component.setParent(null);
                        component.setIndex(-1);
                    };
                    Region.prototype.onChanged = function (listener) {
                        this.changedListeners.push(listener);
                    };
                    Region.prototype.unChanged = function (listener) {
                        this.changedListeners = this.changedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Region.prototype.notifyChangedEvent = function (event) {
                        this.changedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Region.prototype.onComponentAdded = function (listener) {
                        this.componentAddedListeners.push(listener);
                    };
                    Region.prototype.unComponentAdded = function (listener) {
                        this.componentAddedListeners = this.componentAddedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Region.prototype.notifyComponentAdded = function (componentPath) {
                        var event = new region.ComponentAddedEvent(this.getPath(), componentPath);
                        this.componentAddedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChangedEvent(event);
                    };
                    Region.prototype.onComponentRemoved = function (listener) {
                        this.componentRemovedListeners.push(listener);
                    };
                    Region.prototype.unComponentRemoved = function (listener) {
                        this.componentRemovedListeners = this.componentRemovedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Region.prototype.notifyComponentRemoved = function (componentPath) {
                        var event = new region.ComponentRemovedEvent(this.getPath(), componentPath);
                        this.componentRemovedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChangedEvent(event);
                    };
                    Region.prototype.onComponentPropertyChangedEvent = function (listener) {
                        this.componentPropertyChangedListeners.push(listener);
                    };
                    Region.prototype.unComponentPropertyChangedEvent = function (listener) {
                        this.componentPropertyChangedListeners = this.componentPropertyChangedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Region.prototype.forwardComponentPropertyChangedEvent = function (event) {
                        this.componentPropertyChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                    };
                    Region.prototype.onRegionPropertyValueChanged = function (listener) {
                        this.propertyValueChangedListeners.push(listener);
                    };
                    Region.prototype.unRegionPropertyValueChanged = function (listener) {
                        this.propertyValueChangedListeners = this.propertyValueChangedListeners.filter(function (curr) {
                            return listener != curr;
                        });
                    };
                    Region.prototype.notifyRegionPropertyValueChanged = function (propertyName) {
                        var event = new region.RegionPropertyValueChangedEvent(this.getPath(), propertyName);
                        this.propertyValueChangedListeners.forEach(function (listener) {
                            listener(event);
                        });
                        this.notifyChangedEvent(event);
                    };
                    Region.create = function (source) {
                        return new RegionBuilder(source);
                    };
                    Region.debug = false;
                    return Region;
                })();
                region.Region = Region;
                var RegionBuilder = (function () {
                    function RegionBuilder(source) {
                        var _this = this;
                        this.components = [];
                        if (source) {
                            this.name = source.getName();
                            this.parent = source.getParent();
                            source.getComponents().forEach(function (component) {
                                _this.components.push(component.clone());
                            });
                        }
                    }
                    RegionBuilder.prototype.setName = function (value) {
                        this.name = value;
                        return this;
                    };
                    RegionBuilder.prototype.setParent = function (value) {
                        this.parent = value;
                        return this;
                    };
                    RegionBuilder.prototype.addComponent = function (value) {
                        this.components.push(value);
                        return this;
                    };
                    RegionBuilder.prototype.build = function () {
                        return new Region(this);
                    };
                    return RegionBuilder;
                })();
                region.RegionBuilder = RegionBuilder;
            })(region = page.region || (page.region = {}));
        })(page = content.page || (content.page = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var PropertyTree = api.data.PropertyTree;
            var ApplicationKey = api.application.ApplicationKey;
            var SiteConfig = (function () {
                function SiteConfig(builder) {
                    this.applicationKey = builder.applicationKey;
                    this.config = builder.config;
                }
                SiteConfig.prototype.getApplicationKey = function () {
                    return this.applicationKey;
                };
                SiteConfig.prototype.getConfig = function () {
                    return this.config;
                };
                SiteConfig.prototype.toJson = function () {
                    return {
                        applicationKey: this.applicationKey.toString(),
                        config: this.config.toJson()
                    };
                };
                SiteConfig.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, SiteConfig)) {
                        return false;
                    }
                    var other = o;
                    if (!api.ObjectHelper.equals(this.applicationKey, other.applicationKey)) {
                        return false;
                    }
                    if (!api.ObjectHelper.equals(this.config, other.config)) {
                        return false;
                    }
                    return true;
                };
                SiteConfig.prototype.clone = function () {
                    return new SiteConfigBuilder(this).build();
                };
                SiteConfig.create = function () {
                    return new SiteConfigBuilder();
                };
                return SiteConfig;
            })();
            site.SiteConfig = SiteConfig;
            var SiteConfigBuilder = (function () {
                function SiteConfigBuilder(source) {
                    if (source) {
                        this.applicationKey = source.getApplicationKey();
                        if (source.getConfig()) {
                            var newTree = new PropertyTree(source.getConfig());
                            this.config = newTree.getRoot();
                        }
                    }
                }
                SiteConfigBuilder.prototype.fromData = function (propertySet) {
                    api.util.assertNotNull(propertySet, "data cannot be null");
                    var applicationKey = ApplicationKey.fromString(propertySet.getString("applicationKey"));
                    var siteConfig = propertySet.getPropertySet("config");
                    this.setApplicationKey(applicationKey);
                    this.setConfig(siteConfig);
                    return this;
                };
                SiteConfigBuilder.prototype.setApplicationKey = function (value) {
                    this.applicationKey = value;
                    return this;
                };
                SiteConfigBuilder.prototype.setConfig = function (value) {
                    this.config = value;
                    return this;
                };
                SiteConfigBuilder.prototype.build = function () {
                    return new SiteConfig(this);
                };
                return SiteConfigBuilder;
            })();
            site.SiteConfigBuilder = SiteConfigBuilder;
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var Site = (function (_super) {
                __extends(Site, _super);
                function Site(builder) {
                    _super.call(this, builder);
                }
                Site.prototype.isSite = function () {
                    return true;
                };
                Site.prototype.getDescription = function () {
                    return this.getContentData().getString("description");
                };
                Site.prototype.getSiteConfigs = function () {
                    var siteConfigs = [];
                    this.getContentData().forEachProperty("siteConfig", function (applicationProperty) {
                        var siteConfigData = applicationProperty.getPropertySet();
                        if (siteConfigData) {
                            var siteConfig = site.SiteConfig.create().fromData(siteConfigData).build();
                            siteConfigs.push(siteConfig);
                        }
                    });
                    return siteConfigs;
                };
                Site.prototype.getApplicationKeys = function () {
                    return this.getSiteConfigs().map(function (config) { return config.getApplicationKey(); });
                };
                Site.prototype.equals = function (o, ignoreEmptyValues) {
                    if (ignoreEmptyValues === void 0) { ignoreEmptyValues = false; }
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, Site)) {
                        return false;
                    }
                    return _super.prototype.equals.call(this, o, ignoreEmptyValues);
                };
                Site.prototype.clone = function () {
                    return this.newBuilder().build();
                };
                Site.prototype.newBuilder = function () {
                    return new SiteBuilder(this);
                };
                return Site;
            })(api.content.Content);
            site.Site = Site;
            var SiteBuilder = (function (_super) {
                __extends(SiteBuilder, _super);
                function SiteBuilder(source) {
                    _super.call(this, source);
                }
                SiteBuilder.prototype.fromContentJson = function (contentJson) {
                    _super.prototype.fromContentJson.call(this, contentJson);
                    return this;
                };
                SiteBuilder.prototype.build = function () {
                    return new Site(this);
                };
                return SiteBuilder;
            })(api.content.ContentBuilder);
            site.SiteBuilder = SiteBuilder;
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (_site) {
            var ApplicationKey = api.application.ApplicationKey;
            var SiteModel = (function () {
                function SiteModel(site) {
                    var _this = this;
                    this.applicationAddedListeners = [];
                    this.applicationRemovedListeners = [];
                    this.propertyChangedListeners = [];
                    this.site = site;
                    this.siteConfigs = site.getSiteConfigs();
                    this.site.getContentData().onPropertyAdded(function (event) {
                        var property = event.getProperty();
                        if (property.getPath().toString().indexOf(".siteConfig") == 0 && property.getName() == "config") {
                            var siteConfig = api.content.site.SiteConfig.create().fromData(property.getParent()).build();
                            if (!_this.siteConfigs) {
                                _this.siteConfigs = [];
                            }
                            _this.siteConfigs.push(siteConfig);
                            _this.notifyApplicationAdded(siteConfig);
                        }
                    });
                    this.site.getContentData().onPropertyRemoved(function (event) {
                        var property = event.getProperty();
                        if (property.getName() == "siteConfig") {
                            var applicationKey = ApplicationKey.fromString(property.getPropertySet().getString("applicationKey"));
                            _this.siteConfigs = _this.siteConfigs.filter(function (siteConfig) { return !siteConfig.getApplicationKey().equals(applicationKey); });
                            _this.notifyApplicationRemoved(applicationKey);
                        }
                    });
                }
                SiteModel.prototype.getSite = function () {
                    return this.site;
                };
                SiteModel.prototype.getSiteId = function () {
                    return this.site.getContentId();
                };
                SiteModel.prototype.getApplicationKeys = function () {
                    return this.siteConfigs.map(function (sc) { return sc.getApplicationKey(); });
                };
                SiteModel.prototype.onPropertyChanged = function (listener) {
                    this.propertyChangedListeners.push(listener);
                };
                SiteModel.prototype.unPropertyChanged = function (listener) {
                    this.propertyChangedListeners = this.propertyChangedListeners.filter(function (curr) {
                        return listener != curr;
                    });
                };
                SiteModel.prototype.notifyPropertyChanged = function (property, oldValue, newValue, source) {
                    var event = new api.PropertyChangedEvent(property, oldValue, newValue, source);
                    this.propertyChangedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                SiteModel.prototype.onApplicationAdded = function (listener) {
                    this.applicationAddedListeners.push(listener);
                };
                SiteModel.prototype.unApplicationAdded = function (listener) {
                    this.applicationAddedListeners = this.applicationAddedListeners.filter(function (curr) {
                        return listener != curr;
                    });
                };
                SiteModel.prototype.notifyApplicationAdded = function (siteConfig) {
                    var event = new _site.ApplicationAddedEvent(siteConfig);
                    this.applicationAddedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                SiteModel.prototype.onApplicationRemoved = function (listener) {
                    this.applicationRemovedListeners.push(listener);
                };
                SiteModel.prototype.unApplicationRemoved = function (listener) {
                    this.applicationRemovedListeners = this.applicationRemovedListeners.filter(function (curr) {
                        return listener != curr;
                    });
                };
                SiteModel.prototype.notifyApplicationRemoved = function (applicationKey) {
                    var event = new _site.ApplicationRemovedEvent(applicationKey);
                    this.applicationRemovedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                SiteModel.PROPERTY_NAME_SITE_CONFIGS = "siteConfigs";
                return SiteModel;
            })();
            _site.SiteModel = SiteModel;
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var ApplicationAddedEvent = (function () {
                function ApplicationAddedEvent(siteConfig) {
                    this.siteConfig = siteConfig;
                }
                ApplicationAddedEvent.prototype.getApplicationKey = function () {
                    return this.siteConfig.getApplicationKey();
                };
                ApplicationAddedEvent.prototype.getSiteConfig = function () {
                    return this.siteConfig;
                };
                return ApplicationAddedEvent;
            })();
            site.ApplicationAddedEvent = ApplicationAddedEvent;
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var ApplicationRemovedEvent = (function () {
                function ApplicationRemovedEvent(applicationKey) {
                    this.applicationKey = applicationKey;
                }
                ApplicationRemovedEvent.prototype.getApplicationKey = function () {
                    return this.applicationKey;
                };
                return ApplicationRemovedEvent;
            })();
            site.ApplicationRemovedEvent = ApplicationRemovedEvent;
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var site;
        (function (site) {
            var inputtype;
            (function (inputtype) {
                var siteconfigurator;
                (function (siteconfigurator) {
                    var PropertyTree = api.data.PropertyTree;
                    var FormView = api.form.FormView;
                    var SiteConfig = api.content.site.SiteConfig;
                    var ContentRequiresSaveEvent = api.content.ContentRequiresSaveEvent;
                    var SiteView = (function (_super) {
                        __extends(SiteView, _super);
                        function SiteView(application, siteConfig, formContext) {
                            var _this = this;
                            _super.call(this, "site-view");
                            this.siteConfigFormDisplayedListeners = [];
                            this.removeClickedListeners = [];
                            this.collapseClickedListeners = [];
                            this.application = application;
                            this.siteConfig = siteConfig;
                            this.formContext = formContext;
                            var header = new api.dom.DivEl('header');
                            var namesAndIconView = new api.app.NamesAndIconView(new api.app.NamesAndIconViewBuilder().setSize(0 /* large */)).setMainName(this.application.getDisplayName()).setSubName(this.application.getName() + "-" + this.application.getVersion()).setIconClass("icon-xlarge icon-puzzle");
                            header.appendChild(namesAndIconView);
                            var removeButton = new api.dom.AEl("remove-button icon-close");
                            removeButton.onClicked(function (event) {
                                _this.notifyRemoveClicked(event);
                            });
                            header.appendChild(removeButton);
                            this.appendChild(header);
                            this.initFormView();
                            if (this.application.getForm().getFormItems().length > 0) {
                                header.appendChild(this.createEditButton());
                            }
                        }
                        SiteView.prototype.initFormView = function () {
                            var _this = this;
                            this.formValidityChangedHandler = function (event) {
                                _this.toggleClass("invalid", !event.isValid());
                            };
                            this.formView = this.createFormView(this.formContext, this.siteConfig);
                        };
                        SiteView.prototype.createEditButton = function () {
                            var _this = this;
                            var editButton = new api.dom.AEl('edit-button');
                            editButton.onClicked(function (event) {
                                _this.notifyCollapseClicked(event);
                                _this.initAndOpenConfigureDialog();
                            });
                            return editButton;
                        };
                        SiteView.prototype.initAndOpenConfigureDialog = function (comboBoxToUndoSelectionOnCancel) {
                            var _this = this;
                            if (this.application.getForm().getFormItems().length > 0) {
                                var tempSiteConfig = this.makeTemporarySiteConfig();
                                var formViewStateOnDialogOpen = this.formView;
                                this.unbindValidationEvent(formViewStateOnDialogOpen);
                                this.formView = this.createFormView(this.formContext, tempSiteConfig);
                                this.bindValidationEvent(this.formView);
                                var okCallback = function () {
                                    if (!tempSiteConfig.equals(_this.siteConfig)) {
                                        _this.applyTemporaryConfig(tempSiteConfig);
                                        new ContentRequiresSaveEvent(_this.formContext.getPersistedContent()).fire();
                                    }
                                };
                                var cancelCallback = function () {
                                    _this.revertFormViewToGivenState(formViewStateOnDialogOpen);
                                    if (comboBoxToUndoSelectionOnCancel) {
                                        _this.undoSelectionOnCancel(comboBoxToUndoSelectionOnCancel);
                                    }
                                };
                                var siteConfiguratorDialog = new siteconfigurator.SiteConfiguratorDialog(this.application.getDisplayName(), this.application.getName() + "-" + this.application.getVersion(), this.formView, okCallback, cancelCallback);
                                siteConfiguratorDialog.open();
                            }
                        };
                        SiteView.prototype.revertFormViewToGivenState = function (formViewStateToRevertTo) {
                            this.unbindValidationEvent(this.formView);
                            this.formView = formViewStateToRevertTo;
                            this.formView.validate(false, true);
                            this.toggleClass("invalid", !this.formView.isValid());
                        };
                        SiteView.prototype.undoSelectionOnCancel = function (comboBoxToUndoSelectionOnCancel) {
                            comboBoxToUndoSelectionOnCancel.deselect(this.application);
                        };
                        SiteView.prototype.applyTemporaryConfig = function (tempSiteConfig) {
                            var _this = this;
                            tempSiteConfig.getConfig().forEach(function (property) {
                                _this.siteConfig.getConfig().setProperty(property.getName(), property.getIndex(), property.getValue());
                            });
                            this.siteConfig.getConfig().forEach(function (property) {
                                var prop = tempSiteConfig.getConfig().getProperty(property.getName(), property.getIndex());
                                if (!prop) {
                                    _this.siteConfig.getConfig().removeProperty(property.getName(), property.getIndex());
                                }
                            });
                        };
                        SiteView.prototype.makeTemporarySiteConfig = function () {
                            var propSet = (new PropertyTree(this.siteConfig.getConfig())).getRoot();
                            propSet.setContainerProperty(this.siteConfig.getConfig().getProperty());
                            return SiteConfig.create().setConfig(propSet).setApplicationKey(this.siteConfig.getApplicationKey()).build();
                        };
                        SiteView.prototype.createFormView = function (formContext, siteConfig) {
                            var _this = this;
                            var formView = new FormView(formContext, this.application.getForm(), siteConfig.getConfig());
                            formView.addClass("site-form");
                            formView.layout().then(function () {
                                _this.formView.validate(false, true);
                                _this.toggleClass("invalid", !_this.formView.isValid());
                                _this.notifySiteConfigFormDisplayed(_this.application.getApplicationKey());
                                formView.onEditContentRequest(function (content) {
                                    new api.content.EditContentEvent([content]).fire();
                                });
                            }).catch(function (reason) {
                                api.DefaultErrorHandler.handle(reason);
                            }).done();
                            return formView;
                        };
                        SiteView.prototype.bindValidationEvent = function (formView) {
                            if (formView) {
                                formView.onValidityChanged(this.formValidityChangedHandler);
                            }
                        };
                        SiteView.prototype.unbindValidationEvent = function (formView) {
                            if (formView) {
                                formView.unValidityChanged(this.formValidityChangedHandler);
                            }
                        };
                        SiteView.prototype.getApplication = function () {
                            return this.application;
                        };
                        SiteView.prototype.getSiteConfig = function () {
                            return this.siteConfig;
                        };
                        SiteView.prototype.getFormView = function () {
                            return this.formView;
                        };
                        SiteView.prototype.onRemoveClicked = function (listener) {
                            this.removeClickedListeners.push(listener);
                        };
                        SiteView.prototype.unRemoveClicked = function (listener) {
                            this.removeClickedListeners = this.removeClickedListeners.filter(function (curr) {
                                return listener != curr;
                            });
                        };
                        SiteView.prototype.notifyRemoveClicked = function (event) {
                            this.removeClickedListeners.forEach(function (listener) {
                                listener(event);
                            });
                        };
                        SiteView.prototype.onCollapseClicked = function (listener) {
                            this.collapseClickedListeners.push(listener);
                        };
                        SiteView.prototype.unCollapseClicked = function (listener) {
                            this.collapseClickedListeners = this.collapseClickedListeners.filter(function (curr) {
                                return listener != curr;
                            });
                        };
                        SiteView.prototype.notifyCollapseClicked = function (event) {
                            this.collapseClickedListeners.forEach(function (listener) {
                                listener(event);
                            });
                        };
                        SiteView.prototype.onSiteConfigFormDisplayed = function (listener) {
                            this.siteConfigFormDisplayedListeners.push(listener);
                        };
                        SiteView.prototype.unSiteConfigFormDisplayed = function (listener) {
                            this.siteConfigFormDisplayedListeners = this.siteConfigFormDisplayedListeners.filter(function (curr) { return (curr != listener); });
                        };
                        SiteView.prototype.notifySiteConfigFormDisplayed = function (applicationKey) {
                            this.siteConfigFormDisplayedListeners.forEach(function (listener) { return listener(applicationKey); });
                        };
                        return SiteView;
                    })(api.dom.DivEl);
                    siteconfigurator.SiteView = SiteView;
                })(siteconfigurator = inputtype.siteconfigurator || (inputtype.siteconfigurator = {}));
            })(inputtype = site.inputtype || (site.inputtype = {}));
        })(site = _content.site || (_content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var inputtype;
            (function (inputtype) {
                var siteconfigurator;
                (function (siteconfigurator) {
                    var ModalDialogHeader = api.ui.dialog.ModalDialogHeader;
                    var SiteConfiguratorDialog = (function (_super) {
                        __extends(SiteConfiguratorDialog, _super);
                        function SiteConfiguratorDialog(name, subName, formView, okCallback, cancelCallback) {
                            _super.call(this, {
                                title: this.initHeader(name, subName)
                            });
                            this.getEl().addClass("app-configurator-dialog");
                            this.appendChildToContentPanel(formView);
                            this.addOkButton(okCallback);
                            this.addCancelButton(cancelCallback);
                        }
                        SiteConfiguratorDialog.prototype.addOkButton = function (okCallback) {
                            var _this = this;
                            var okAction = new api.ui.Action("Apply", "enter");
                            this.addAction(okAction, true, true);
                            okAction.onExecuted(function () {
                                if (okCallback) {
                                    okCallback();
                                }
                                _this.close();
                            });
                        };
                        SiteConfiguratorDialog.prototype.addCancelButton = function (cancelCallback) {
                            var _this = this;
                            var cancelAction = new api.ui.Action("Cancel", "esc");
                            cancelAction.setIconClass("cancel-button-bottom");
                            cancelAction.onExecuted(function () {
                                if (cancelCallback) {
                                    cancelCallback();
                                }
                                _this.close();
                            });
                            this.addAction(cancelAction);
                        };
                        SiteConfiguratorDialog.prototype.initHeader = function (name, subName) {
                            var dialogHeader = new ModalDialogHeader("");
                            var namesAndIconView = new api.app.NamesAndIconView(new api.app.NamesAndIconViewBuilder().setSize(0 /* large */)).setMainName(name).setSubName(subName).setIconClass("icon-xlarge icon-puzzle");
                            dialogHeader.appendChild(namesAndIconView);
                            return dialogHeader;
                        };
                        SiteConfiguratorDialog.prototype.subscribeOnDropdownExpand = function () {
                            var _this = this;
                            wemjq.each(wemjq(this.getHTMLElement()).find('.dropdown-handle'), function (key, value) {
                                wemjq(value).click(function () {
                                    if (wemjq(value).hasClass("down")) {
                                        var optionsContainer = wemjq(value).nextAll(".options-container").get(0);
                                        _this.scrollToExpandedDropdownIfNotVisible(optionsContainer);
                                    }
                                });
                            });
                        };
                        SiteConfiguratorDialog.prototype.scrollToExpandedDropdownIfNotVisible = function (dropdown) {
                            var _this = this;
                            if (dropdown) {
                                var optionsContainerElement = api.dom.Element.fromHtmlElement(dropdown);
                                setTimeout(function () {
                                    var expandedDropdownBottomPosition = _this.getAbsoluteBottomPosition(optionsContainerElement), siteDialogBottomPosition = _this.getAbsoluteBottomPosition(_this), dialogBottomAreaHeight = _this.getButtonRow().getEl().getHeightWithMargin() + _this.getEl().getMarginBottom();
                                    if (expandedDropdownBottomPosition > (siteDialogBottomPosition - dialogBottomAreaHeight)) {
                                        _this.getContentPanel().getEl().setScrollTop(_this.getContentPanel().getEl().getScrollTop() + optionsContainerElement.getEl().getHeightWithBorder());
                                    }
                                }, 100);
                            }
                        };
                        SiteConfiguratorDialog.prototype.getAbsoluteBottomPosition = function (element) {
                            var el = element.getEl(), bottomPosition = (el.getOffset().top + el.getHeightWithBorder());
                            return bottomPosition;
                        };
                        SiteConfiguratorDialog.prototype.show = function () {
                            var _this = this;
                            api.dom.Body.get().appendChild(this);
                            _super.prototype.show.call(this);
                            setTimeout(function () {
                                _this.addClass("animated");
                                _this.centerMyself();
                                wemjq(_this.getHTMLElement()).find('input[type=text],textarea,select').first().focus();
                            }, 100);
                            setTimeout(function () {
                                _this.subscribeOnDropdownExpand();
                            }, 1500);
                        };
                        SiteConfiguratorDialog.prototype.close = function () {
                            _super.prototype.close.call(this);
                            this.remove();
                        };
                        return SiteConfiguratorDialog;
                    })(api.ui.dialog.ModalDialog);
                    siteconfigurator.SiteConfiguratorDialog = SiteConfiguratorDialog;
                })(siteconfigurator = inputtype.siteconfigurator || (inputtype.siteconfigurator = {}));
            })(inputtype = site.inputtype || (site.inputtype = {}));
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var inputtype;
            (function (inputtype) {
                var siteconfigurator;
                (function (siteconfigurator) {
                    var SiteConfig = api.content.site.SiteConfig;
                    var SiteConfigProvider = (function () {
                        function SiteConfigProvider(propertyArray) {
                            this.propertyArray = propertyArray;
                        }
                        SiteConfigProvider.prototype.getConfig = function (applicationKey) {
                            var match = null;
                            this.propertyArray.forEach(function (property) {
                                if (property.hasNonNullValue()) {
                                    var siteConfigAsSet = property.getPropertySet();
                                    var siteConfig = SiteConfig.create().fromData(siteConfigAsSet).build();
                                    if (siteConfig.getApplicationKey().equals(applicationKey)) {
                                        match = siteConfig;
                                    }
                                }
                            });
                            if (!match) {
                                var siteConfigAsSet = this.propertyArray.addSet();
                                siteConfigAsSet.addString("applicationKey", applicationKey.toString());
                                siteConfigAsSet.addPropertySet("config");
                                var newSiteConfig = SiteConfig.create().fromData(siteConfigAsSet).build();
                                return newSiteConfig;
                            }
                            else {
                                return match;
                            }
                        };
                        return SiteConfigProvider;
                    })();
                    siteconfigurator.SiteConfigProvider = SiteConfigProvider;
                })(siteconfigurator = inputtype.siteconfigurator || (inputtype.siteconfigurator = {}));
            })(inputtype = site.inputtype || (site.inputtype = {}));
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var inputtype;
            (function (inputtype) {
                var siteconfigurator;
                (function (siteconfigurator) {
                    var ValueTypes = api.data.ValueTypes;
                    var SiteConfig = api.content.site.SiteConfig;
                    var GetApplicationRequest = api.application.GetApplicationRequest;
                    var SiteConfigurator = (function (_super) {
                        __extends(SiteConfigurator, _super);
                        function SiteConfigurator(config) {
                            _super.call(this, "site-configurator");
                            this._displayValidationErrors = false;
                            this.context = config;
                            this.formContext = config.formContext;
                        }
                        SiteConfigurator.prototype.getValueType = function () {
                            return ValueTypes.DATA;
                        };
                        SiteConfigurator.prototype.newInitialValue = function () {
                            return null;
                        };
                        SiteConfigurator.prototype.layout = function (input, propertyArray) {
                            var _this = this;
                            _super.prototype.layout.call(this, input, propertyArray);
                            var siteConfigProvider = new siteconfigurator.SiteConfigProvider(propertyArray);
                            this.comboBox = this.createComboBox(input, siteConfigProvider);
                            this.appendChild(this.comboBox);
                            return this.doLoadApplications(propertyArray).then(function () {
                                _this.setLayoutInProgress(false);
                                _this.comboBox.onOptionDeselected(function (removed) {
                                    _this.getPropertyArray().remove(removed.getIndex());
                                    _this.validate(false);
                                });
                                _this.comboBox.onOptionSelected(function (event) {
                                    var key = event.getOption().displayValue.getApplicationKey();
                                    if (!key) {
                                        return;
                                    }
                                    var selectedOption = _this.comboBox.getSelectedOption(event.getOption());
                                    var selectedOptionView = selectedOption.getOptionView();
                                    var siteConfig = selectedOptionView.getSiteConfig();
                                    _this.validate(false);
                                });
                            });
                        };
                        SiteConfigurator.prototype.doLoadApplications = function (propertyArray) {
                            var _this = this;
                            var deferred = wemQ.defer();
                            var siteConfigFormsToDisplay = [];
                            if (propertyArray.getSize() == 0) {
                                deferred.resolve(null);
                            }
                            else {
                                propertyArray.forEach(function (property) {
                                    if (property.hasNonNullValue()) {
                                        var siteConfig = SiteConfig.create().fromData(property.getPropertySet()).build();
                                        siteConfigFormsToDisplay.push(siteConfig.getApplicationKey().toString());
                                        new GetApplicationRequest(siteConfig.getApplicationKey()).sendAndParse().then(function (requestedApplication) {
                                            _this.comboBox.onSiteConfigFormDisplayed(function (applicationKey, formView) {
                                                var indexToRemove = siteConfigFormsToDisplay.indexOf(applicationKey.toString());
                                                if (indexToRemove != -1) {
                                                    siteConfigFormsToDisplay.splice(indexToRemove, 1);
                                                }
                                                if (siteConfigFormsToDisplay.length == 0) {
                                                    deferred.resolve(null);
                                                }
                                                formView.onValidityChanged(function (event) {
                                                    _this.validate(false);
                                                });
                                                _this.validate(false);
                                            });
                                            _this.comboBox.select(requestedApplication);
                                        });
                                    }
                                });
                            }
                            return deferred.promise;
                        };
                        SiteConfigurator.prototype.createComboBox = function (input, siteConfigProvider) {
                            return new siteconfigurator.SiteConfiguratorComboBox(input.getOccurrences().getMaximum() || 0, siteConfigProvider, this.formContext);
                        };
                        SiteConfigurator.prototype.availableSizeChanged = function () {
                        };
                        SiteConfigurator.prototype.displayValidationErrors = function (value) {
                            this._displayValidationErrors = value;
                            this.comboBox.getSelectedOptionViews().forEach(function (view) {
                                view.getFormView().displayValidationErrors(value);
                            });
                        };
                        SiteConfigurator.prototype.getNumberOfValids = function () {
                            return this.comboBox.countSelected();
                        };
                        SiteConfigurator.prototype.validate = function (silent) {
                            if (silent === void 0) { silent = true; }
                            var recording = new api.form.inputtype.InputValidationRecording();
                            this.comboBox.getSelectedOptionViews().forEach(function (view) {
                                var validationRecording = view.getFormView().validate(true);
                                if (!validationRecording.isMinimumOccurrencesValid()) {
                                    recording.setBreaksMinimumOccurrences(true);
                                }
                                if (!validationRecording.isMaximumOccurrencesValid()) {
                                    recording.setBreaksMaximumOccurrences(true);
                                }
                            });
                            return _super.prototype.validate.call(this, silent, recording);
                        };
                        SiteConfigurator.prototype.giveFocus = function () {
                            if (this.comboBox.maximumOccurrencesReached()) {
                                return false;
                            }
                            return this.comboBox.giveFocus();
                        };
                        return SiteConfigurator;
                    })(api.form.inputtype.support.BaseInputTypeManagingAdd);
                    siteconfigurator.SiteConfigurator = SiteConfigurator;
                    api.form.inputtype.InputTypeManager.register(new api.Class("SiteConfigurator", SiteConfigurator));
                })(siteconfigurator = inputtype.siteconfigurator || (inputtype.siteconfigurator = {}));
            })(inputtype = site.inputtype || (site.inputtype = {}));
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var site;
        (function (site) {
            var inputtype;
            (function (inputtype) {
                var siteconfigurator;
                (function (siteconfigurator) {
                    var Application = api.application.Application;
                    var ApplicationViewer = api.application.ApplicationViewer;
                    var ApplicationLoader = api.application.ApplicationLoader;
                    var SelectedOption = api.ui.selector.combobox.SelectedOption;
                    var BaseSelectedOptionsView = api.ui.selector.combobox.BaseSelectedOptionsView;
                    var SiteConfiguratorComboBox = (function (_super) {
                        __extends(SiteConfiguratorComboBox, _super);
                        function SiteConfiguratorComboBox(maxOccurrences, siteConfigProvider, formContext) {
                            var filterObject = {
                                state: Application.STATE_STARTED
                            };
                            this.siteConfiguratorSelectedOptionsView = new SiteConfiguratorSelectedOptionsView(siteConfigProvider, formContext);
                            var builder = new api.ui.selector.combobox.RichComboBoxBuilder();
                            builder.setMaximumOccurrences(maxOccurrences).setIdentifierMethod('getApplicationKey').setComboBoxName("applicationSelector").setLoader(new ApplicationLoader(500, filterObject)).setSelectedOptionsView(this.siteConfiguratorSelectedOptionsView).setOptionDisplayValueViewer(new ApplicationViewer()).setDelayedInputValueChangedHandling(500);
                            _super.call(this, builder);
                        }
                        SiteConfiguratorComboBox.prototype.getSelectedOptionViews = function () {
                            var views = [];
                            this.getSelectedOptions().forEach(function (selectedOption) {
                                views.push(selectedOption.getOptionView());
                            });
                            return views;
                        };
                        SiteConfiguratorComboBox.prototype.onSiteConfigFormDisplayed = function (listener) {
                            this.siteConfiguratorSelectedOptionsView.onSiteConfigFormDisplayed(listener);
                        };
                        SiteConfiguratorComboBox.prototype.unSiteConfigFormDisplayed = function (listener) {
                            this.siteConfiguratorSelectedOptionsView.unSiteConfigFormDisplayed(listener);
                        };
                        return SiteConfiguratorComboBox;
                    })(api.ui.selector.combobox.RichComboBox);
                    siteconfigurator.SiteConfiguratorComboBox = SiteConfiguratorComboBox;
                    var SiteConfiguratorSelectedOptionsView = (function (_super) {
                        __extends(SiteConfiguratorSelectedOptionsView, _super);
                        function SiteConfiguratorSelectedOptionsView(siteConfigProvider, formContext) {
                            _super.call(this);
                            this.siteConfigFormDisplayedListeners = [];
                            this.siteConfigProvider = siteConfigProvider;
                            this.formContext = formContext;
                        }
                        SiteConfiguratorSelectedOptionsView.prototype.createSelectedOption = function (option) {
                            var _this = this;
                            var siteConfig = this.siteConfigProvider.getConfig(option.displayValue.getApplicationKey());
                            var optionView = new SiteConfiguratorSelectedOptionView(option, siteConfig, this.formContext);
                            optionView.onSiteConfigFormDisplayed(function (applicationKey) {
                                _this.notifySiteConfigFormDisplayed(applicationKey, optionView.getFormView());
                            });
                            return new SelectedOption(optionView, this.count());
                        };
                        SiteConfiguratorSelectedOptionsView.prototype.onSiteConfigFormDisplayed = function (listener) {
                            this.siteConfigFormDisplayedListeners.push(listener);
                        };
                        SiteConfiguratorSelectedOptionsView.prototype.unSiteConfigFormDisplayed = function (listener) {
                            this.siteConfigFormDisplayedListeners = this.siteConfigFormDisplayedListeners.filter(function (curr) { return (curr != listener); });
                        };
                        SiteConfiguratorSelectedOptionsView.prototype.notifySiteConfigFormDisplayed = function (applicationKey, formView) {
                            this.siteConfigFormDisplayedListeners.forEach(function (listener) { return listener(applicationKey, formView); });
                        };
                        return SiteConfiguratorSelectedOptionsView;
                    })(BaseSelectedOptionsView);
                    siteconfigurator.SiteConfiguratorSelectedOptionsView = SiteConfiguratorSelectedOptionsView;
                    var SiteConfiguratorSelectedOptionView = (function (_super) {
                        __extends(SiteConfiguratorSelectedOptionView, _super);
                        function SiteConfiguratorSelectedOptionView(option, siteConfig, formContext) {
                            var _this = this;
                            this.selectedOptionToBeRemovedListeners = [];
                            this.option = option;
                            _super.call(this, option.displayValue, siteConfig, formContext);
                            this.onRemoveClicked(function (event) {
                                _this.notifySelectedOptionRemoveRequested();
                            });
                        }
                        SiteConfiguratorSelectedOptionView.prototype.setOption = function (option) {
                            this.option = option;
                        };
                        SiteConfiguratorSelectedOptionView.prototype.getOption = function () {
                            return this.option;
                        };
                        SiteConfiguratorSelectedOptionView.prototype.notifySelectedOptionRemoveRequested = function () {
                            this.selectedOptionToBeRemovedListeners.forEach(function (listener) {
                                listener();
                            });
                        };
                        SiteConfiguratorSelectedOptionView.prototype.onSelectedOptionRemoveRequest = function (listener) {
                            this.selectedOptionToBeRemovedListeners.push(listener);
                        };
                        SiteConfiguratorSelectedOptionView.prototype.unSelectedOptionRemoveRequest = function (listener) {
                            this.selectedOptionToBeRemovedListeners = this.selectedOptionToBeRemovedListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        return SiteConfiguratorSelectedOptionView;
                    })(siteconfigurator.SiteView);
                    siteconfigurator.SiteConfiguratorSelectedOptionView = SiteConfiguratorSelectedOptionView;
                })(siteconfigurator = inputtype.siteconfigurator || (inputtype.siteconfigurator = {}));
            })(inputtype = site.inputtype || (site.inputtype = {}));
        })(site = content.site || (content.site = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var ContentFormContext = (function (_super) {
                __extends(ContentFormContext, _super);
                function ContentFormContext(builder) {
                    _super.call(this, builder);
                    this.site = builder.site;
                    this.parentContent = builder.parentContent;
                    this.persistedContent = builder.persistedContent;
                    if (builder.contentTypeName) {
                        this.contentTypeName = builder.contentTypeName;
                    }
                    else if (builder.persistedContent) {
                        this.contentTypeName = builder.persistedContent.getType();
                    }
                }
                ContentFormContext.prototype.getSite = function () {
                    return this.site;
                };
                ContentFormContext.prototype.getContentId = function () {
                    return this.persistedContent != null ? this.persistedContent.getContentId() : null;
                };
                ContentFormContext.prototype.getContentPath = function () {
                    return this.persistedContent != null ? this.persistedContent.getPath() : null;
                };
                ContentFormContext.prototype.getParentContentPath = function () {
                    if (this.parentContent == null) {
                        return api.content.ContentPath.ROOT;
                    }
                    return this.parentContent.getPath();
                };
                ContentFormContext.prototype.getPersistedContent = function () {
                    return this.persistedContent;
                };
                ContentFormContext.prototype.getContentTypeName = function () {
                    return this.contentTypeName;
                };
                ContentFormContext.prototype.createInputTypeViewContext = function (inputTypeConfig, parentPropertyPath, input) {
                    return {
                        formContext: this,
                        input: input,
                        inputConfig: inputTypeConfig,
                        parentDataPath: parentPropertyPath,
                        site: this.getSite(),
                        contentId: this.getContentId(),
                        contentPath: this.getContentPath(),
                        parentContentPath: this.getParentContentPath()
                    };
                };
                ContentFormContext.create = function () {
                    return new ContentFormContextBuilder();
                };
                return ContentFormContext;
            })(api.form.FormContext);
            form.ContentFormContext = ContentFormContext;
            var ContentFormContextBuilder = (function (_super) {
                __extends(ContentFormContextBuilder, _super);
                function ContentFormContextBuilder() {
                    _super.apply(this, arguments);
                }
                ContentFormContextBuilder.prototype.setSite = function (value) {
                    this.site = value;
                    return this;
                };
                ContentFormContextBuilder.prototype.setParentContent = function (value) {
                    this.parentContent = value;
                    return this;
                };
                ContentFormContextBuilder.prototype.setPersistedContent = function (value) {
                    this.persistedContent = value;
                    return this;
                };
                ContentFormContextBuilder.prototype.setContentTypeName = function (value) {
                    this.contentTypeName = value;
                    return this;
                };
                ContentFormContextBuilder.prototype.build = function () {
                    return new ContentFormContext(this);
                };
                return ContentFormContextBuilder;
            })(api.form.FormContextBuilder);
            form.ContentFormContextBuilder = ContentFormContextBuilder;
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var upload;
                (function (upload) {
                    var ValueTypes = api.data.ValueTypes;
                    var ImageUploader = (function (_super) {
                        __extends(ImageUploader, _super);
                        function ImageUploader(config) {
                            _super.call(this, config, "image");
                            var input = config.input;
                            this.imageUploader = new api.content.ImageUploader({
                                params: {
                                    content: config.contentId.toString()
                                },
                                operation: 1 /* update */,
                                name: input.getName(),
                                skipWizardEvents: false,
                                maximumOccurrences: 1,
                                scaleWidth: true,
                                hideDropZone: true,
                                showReset: false
                            });
                            this.appendChild(this.imageUploader);
                        }
                        ImageUploader.prototype.getContext = function () {
                            return _super.prototype.getContext.call(this);
                        };
                        ImageUploader.prototype.getValueType = function () {
                            return ValueTypes.STRING;
                        };
                        ImageUploader.prototype.newInitialValue = function () {
                            return ValueTypes.STRING.newNullValue();
                        };
                        ImageUploader.prototype.layoutProperty = function (input, property) {
                            var _this = this;
                            this.input = input;
                            this.property = property;
                            if (property.hasNonNullValue()) {
                                new api.content.GetContentByIdRequest(this.getContext().contentId).sendAndParse().then(function (content) {
                                    _this.imageUploader.setOriginalDimensions(content);
                                    _this.imageUploader.setValue(content.getId());
                                    _this.configEditorsProperties(content);
                                }).catch(function (reason) {
                                    api.DefaultErrorHandler.handle(reason);
                                }).done();
                            }
                            this.imageUploader.onFileUploaded(function (event) {
                                var content = event.getUploadItem().getModel(), value = _this.imageUploader.getMediaValue(content);
                                _this.imageUploader.setOriginalDimensions(content);
                                switch (_this.property.getType()) {
                                    case ValueTypes.DATA:
                                        var set = _this.property.getPropertySet();
                                        set.setProperty('attachment', 0, value);
                                        set.removeProperty('focalPoint', 0);
                                        set.removeProperty('cropPosition', 0);
                                        set.removeProperty('zoomPosition', 0);
                                        break;
                                    case ValueTypes.STRING:
                                        _this.property.setValue(value);
                                        break;
                                }
                            });
                            this.imageUploader.onUploadReset(function () {
                                switch (_this.property.getType()) {
                                    case ValueTypes.DATA:
                                        var set = _this.property.getPropertySet();
                                        set.setProperty('attachment', 0, ValueTypes.STRING.newNullValue());
                                        set.removeProperty('focalPoint', 0);
                                        break;
                                    case ValueTypes.STRING:
                                        _this.property.setValue(ValueTypes.STRING.newNullValue());
                                        break;
                                }
                            });
                            this.imageUploader.onEditModeChanged(function (edit, crop, zoom, focus) {
                                _this.validate(false);
                                if (!edit && crop) {
                                    _this.saveToProperty(crop, zoom, focus);
                                }
                            });
                            this.imageUploader.onCropAutoPositionedChanged(function (auto) {
                                if (auto) {
                                    _this.saveToProperty({ x: 0, y: 0, x2: 1, y2: 1 }, { x: 0, y: 0, x2: 1, y2: 1 }, null);
                                }
                            });
                            this.imageUploader.onFocusAutoPositionedChanged(function (auto) {
                                if (auto) {
                                    _this.saveToProperty(null, null, { x: 0.5, y: 0.5 });
                                }
                            });
                            return wemQ(null);
                        };
                        ImageUploader.prototype.saveToProperty = function (crop, zoom, focus) {
                            var container = this.getPropertyContainer(this.property);
                            if (container) {
                                if (crop) {
                                    container.setDoubleByPath('cropPosition.left', crop.x);
                                    container.setDoubleByPath('cropPosition.top', crop.y);
                                    container.setDoubleByPath('cropPosition.right', crop.x2);
                                    container.setDoubleByPath('cropPosition.bottom', crop.y2);
                                    container.setDoubleByPath('cropPosition.zoom', zoom.x2 - zoom.x);
                                }
                                if (zoom) {
                                    container.setDoubleByPath('zoomPosition.left', zoom.x);
                                    container.setDoubleByPath('zoomPosition.top', zoom.y);
                                    container.setDoubleByPath('zoomPosition.right', zoom.x2);
                                    container.setDoubleByPath('zoomPosition.bottom', zoom.y2);
                                }
                                if (focus) {
                                    container.setDoubleByPath('focalPoint.x', focus.x);
                                    container.setDoubleByPath('focalPoint.y', focus.y);
                                }
                            }
                        };
                        ImageUploader.prototype.getPropertyContainer = function (property) {
                            var container;
                            switch (property.getType()) {
                                case ValueTypes.DATA:
                                    container = property.getPropertySet();
                                    break;
                                case ValueTypes.STRING:
                                    container = new api.data.PropertyTree();
                                    container.setString('attachment', 0, property.getString());
                                    var propertyParent = property.getParent();
                                    var propertyName = property.getName();
                                    propertyParent.removeProperty(propertyName, 0);
                                    propertyParent.setPropertySet(propertyName, 0, container.getRoot());
                                    this.property = propertyParent.getProperty(propertyName);
                                    break;
                            }
                            return container;
                        };
                        ImageUploader.prototype.getFocalPoint = function (content) {
                            var focalProperty = this.getMediaProperty(content, 'focalPoint');
                            if (!focalProperty) {
                                return null;
                            }
                            var focalSet = focalProperty.getPropertySet(), x = focalSet.getDouble('x'), y = focalSet.getDouble('y');
                            if (!x || !y) {
                                return null;
                            }
                            return {
                                x: x,
                                y: y
                            };
                        };
                        ImageUploader.prototype.getRectFromProperty = function (content, propertyName) {
                            var property = this.getMediaProperty(content, propertyName);
                            if (!property) {
                                return null;
                            }
                            var cropPositionSet = property.getPropertySet(), x = cropPositionSet.getDouble('left'), y = cropPositionSet.getDouble('top'), x2 = cropPositionSet.getDouble('right'), y2 = cropPositionSet.getDouble('bottom');
                            return {
                                x: x,
                                y: y,
                                x2: x2,
                                y2: y2
                            };
                        };
                        ImageUploader.prototype.getMediaProperty = function (content, propertyName) {
                            var mediaProperty = content.getProperty('media');
                            if (!mediaProperty || !ValueTypes.DATA.equals(mediaProperty.getType())) {
                                return null;
                            }
                            var resultProperty = mediaProperty.getPropertySet().getProperty(propertyName);
                            if (!resultProperty || !ValueTypes.DATA.equals(resultProperty.getType())) {
                                return null;
                            }
                            return resultProperty;
                        };
                        ImageUploader.prototype.configEditorsProperties = function (content) {
                            var focalPoint = this.getFocalPoint(content);
                            if (focalPoint) {
                                this.imageUploader.setFocalPoint(focalPoint.x, focalPoint.y);
                            }
                            var cropPosition = this.getRectFromProperty(content, 'cropPosition');
                            if (cropPosition) {
                                this.imageUploader.setCrop(cropPosition);
                            }
                            var zoomPosition = this.getRectFromProperty(content, 'zoomPosition');
                            if (zoomPosition) {
                                this.imageUploader.setZoom(zoomPosition);
                            }
                        };
                        ImageUploader.prototype.validate = function (silent) {
                            if (silent === void 0) { silent = true; }
                            var recording = new api.form.inputtype.InputValidationRecording();
                            var propertyValue = this.property.getValue();
                            if (this.imageUploader.isFocalPointEditMode() || this.imageUploader.isCropEditMode()) {
                                recording.setBreaksMinimumOccurrences(true);
                            }
                            if (propertyValue.isNull() && this.input.getOccurrences().getMinimum() > 0) {
                                recording.setBreaksMinimumOccurrences(true);
                            }
                            if (!silent) {
                                if (recording.validityChanged(this.previousValidationRecording)) {
                                    this.notifyValidityChanged(new api.form.inputtype.InputValidityChangedEvent(recording, this.input.getName()));
                                }
                            }
                            this.previousValidationRecording = recording;
                            return recording;
                        };
                        ImageUploader.prototype.onFocus = function (listener) {
                            this.imageUploader.onFocus(listener);
                        };
                        ImageUploader.prototype.unFocus = function (listener) {
                            this.imageUploader.unFocus(listener);
                        };
                        ImageUploader.prototype.onBlur = function (listener) {
                            this.imageUploader.onBlur(listener);
                        };
                        ImageUploader.prototype.unBlur = function (listener) {
                            this.imageUploader.unBlur(listener);
                        };
                        return ImageUploader;
                    })(api.form.inputtype.support.BaseInputTypeSingleOccurrence);
                    upload.ImageUploader = ImageUploader;
                    api.form.inputtype.InputTypeManager.register(new api.Class("ImageUploader", ImageUploader));
                })(upload = inputtype.upload || (inputtype.upload = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var upload;
                (function (upload) {
                    var ValueTypes = api.data.ValueTypes;
                    var ContentRequiresSaveEvent = api.content.ContentRequiresSaveEvent;
                    var FileUploader = (function (_super) {
                        __extends(FileUploader, _super);
                        function FileUploader(config) {
                            _super.call(this, config, "file-uploader");
                            this.config = config;
                        }
                        FileUploader.prototype.getContext = function () {
                            return _super.prototype.getContext.call(this);
                        };
                        FileUploader.prototype.getValueType = function () {
                            return ValueTypes.STRING;
                        };
                        FileUploader.prototype.newInitialValue = function () {
                            return ValueTypes.STRING.newNullValue();
                        };
                        FileUploader.prototype.layoutProperty = function (input, property) {
                            var _this = this;
                            if (this.propertyAlreadyHasAttachment(property)) {
                                this.uploader = this.createUploader(this.getAllowTypeFromFileName(this.getFileNameFromProperty(property)));
                            }
                            else {
                                this.uploader = this.createUploader();
                            }
                            this.uploaderWrapper = this.createUploaderWrapper(property);
                            if (this.getContext().contentId) {
                                this.uploader.setValue(this.getContext().contentId.toString());
                                if (property.getValue() != null) {
                                    this.uploader.setFileName(this.getFileNameFromProperty(property));
                                }
                            }
                            this.uploader.onUploadStarted(function () {
                                _this.uploaderWrapper.removeClass("empty");
                            });
                            this.uploader.onFileUploaded(function (event) {
                                var content = event.getUploadItem().getModel(), value = _this.uploader.getMediaValue(content), fileName = value.getString();
                                _this.uploader.setFileName(fileName);
                                switch (property.getType()) {
                                    case ValueTypes.DATA:
                                        property.getPropertySet().setProperty('attachment', 0, value);
                                        break;
                                    case ValueTypes.STRING:
                                        property.setValue(ValueTypes.STRING.newValue(fileName));
                                        break;
                                }
                                api.notify.showFeedback('\"' + fileName + '\" uploaded');
                                new ContentRequiresSaveEvent(content).fire();
                            });
                            this.uploader.onUploadFailed(function () {
                                _this.uploader.setProgressVisible(false);
                                _this.uploaderWrapper.addClass("empty");
                            });
                            this.uploader.onUploadReset(function () {
                                _this.uploader.setFileName('');
                                switch (property.getType()) {
                                    case ValueTypes.DATA:
                                        property.getPropertySet().setProperty('attachment', 0, ValueTypes.STRING.newNullValue());
                                        break;
                                    case ValueTypes.STRING:
                                        property.setValue(ValueTypes.STRING.newNullValue());
                                        break;
                                }
                            });
                            this.appendChild(this.uploaderWrapper);
                            return wemQ(null);
                        };
                        FileUploader.prototype.validate = function (silent) {
                            if (silent === void 0) { silent = true; }
                            return new api.form.inputtype.InputValidationRecording();
                        };
                        FileUploader.prototype.getFileNameFromProperty = function (property) {
                            if (property.getValue() != null) {
                                switch (property.getType()) {
                                    case ValueTypes.DATA:
                                        return property.getPropertySet().getString('attachment');
                                    case ValueTypes.STRING:
                                        return property.getValue().getString();
                                }
                            }
                            return "";
                        };
                        FileUploader.prototype.getFileExtensionFromFileName = function (fileName) {
                            return fileName.split('.').pop();
                        };
                        FileUploader.prototype.propertyAlreadyHasAttachment = function (property) {
                            return (property.getValue() != null && property.getType() == ValueTypes.DATA && !api.util.StringHelper.isEmpty(property.getPropertySet().getString('attachment')));
                        };
                        FileUploader.prototype.getAllowTypeFromFileName = function (fileName) {
                            return [{ name: "Media", extensions: this.getFileExtensionFromFileName(fileName) }];
                        };
                        FileUploader.prototype.createUploaderWrapper = function (property) {
                            var _this = this;
                            var wrapper = new api.dom.DivEl("uploader-wrapper");
                            var uploadButton = new api.ui.button.Button();
                            uploadButton.addClass('upload-button');
                            uploadButton.onClicked(function (event) {
                                if (property.hasNullValue()) {
                                    return;
                                }
                                wemjq(_this.uploader.getDropzone().getEl().getHTMLElement()).simulate("click");
                            });
                            wrapper.appendChild(this.uploader);
                            wrapper.appendChild(uploadButton);
                            return wrapper;
                        };
                        FileUploader.prototype.createUploader = function (predefinedAllowTypes) {
                            var allowTypesConfig = predefinedAllowTypes || (this.config.inputConfig).allowTypes || [];
                            var allowTypes = allowTypesConfig.map(function (allowType) {
                                return { title: allowType.name, extensions: allowType.extensions };
                            });
                            return new api.content.MediaUploader({
                                params: {
                                    content: this.getContext().contentId.toString()
                                },
                                operation: 1 /* update */,
                                allowTypes: allowTypes,
                                name: this.getContext().input.getName(),
                                showReset: false,
                                showCancel: false,
                                maximumOccurrences: 1,
                                allowMultiSelection: false,
                                hideDropZone: !!(this.config.inputConfig).hideDropZone,
                                deferred: true
                            });
                        };
                        FileUploader.prototype.onFocus = function (listener) {
                            this.uploader.onFocus(listener);
                        };
                        FileUploader.prototype.unFocus = function (listener) {
                            this.uploader.unFocus(listener);
                        };
                        FileUploader.prototype.onBlur = function (listener) {
                            this.uploader.onBlur(listener);
                        };
                        FileUploader.prototype.unBlur = function (listener) {
                            this.uploader.unBlur(listener);
                        };
                        return FileUploader;
                    })(api.form.inputtype.support.BaseInputTypeSingleOccurrence);
                    upload.FileUploader = FileUploader;
                    api.form.inputtype.InputTypeManager.register(new api.Class("FileUploader", FileUploader));
                })(upload = inputtype.upload || (inputtype.upload = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var relationship;
                (function (relationship) {
                    var ContentSelectorLoader = (function (_super) {
                        __extends(ContentSelectorLoader, _super);
                        function ContentSelectorLoader() {
                            _super.apply(this, arguments);
                            this.postponedSearchString = null;
                        }
                        ContentSelectorLoader.prototype.setAllowedContentTypes = function (contentTypes) {
                            this.allowedContentTypes = contentTypes;
                            _super.prototype.setAllowedContentTypes.call(this, contentTypes);
                            if (this.postponedSearchString != null) {
                                this.search(this.postponedSearchString);
                            }
                        };
                        ContentSelectorLoader.prototype.search = function (searchString) {
                            return _super.prototype.search.call(this, searchString);
                        };
                        return ContentSelectorLoader;
                    })(api.content.ContentSummaryLoader);
                    relationship.ContentSelectorLoader = ContentSelectorLoader;
                })(relationship = inputtype.relationship || (inputtype.relationship = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var relationship;
                (function (relationship) {
                    var Value = api.data.Value;
                    var ValueTypes = api.data.ValueTypes;
                    var GetRelationshipTypeByNameRequest = api.schema.relationshiptype.GetRelationshipTypeByNameRequest;
                    var RelationshipTypeName = api.schema.relationshiptype.RelationshipTypeName;
                    var ContentSelector = (function (_super) {
                        __extends(ContentSelector, _super);
                        function ContentSelector(config) {
                            _super.call(this, "relationship");
                            this.addClass("input-type-view");
                            this.config = config;
                            this.readConfig(config.inputConfig);
                        }
                        ContentSelector.prototype.readConfig = function (inputConfig) {
                            var relationshipTypeConfig = inputConfig['relationshipType'] ? inputConfig['relationshipType'][0] : {};
                            var relationshipType = relationshipTypeConfig['value'];
                            if (relationshipType) {
                                this.relationshipTypeName = new RelationshipTypeName(relationshipType);
                            }
                            else {
                                this.relationshipTypeName = RelationshipTypeName.REFERENCE;
                            }
                            var allowContentTypeConfig = inputConfig['allowContentType'] || [];
                            this.allowedContentTypes = allowContentTypeConfig.map(function (cfg) { return cfg['value']; }).filter(function (val) { return !!val; });
                        };
                        ContentSelector.prototype.availableSizeChanged = function () {
                            console.log("Relationship.availableSizeChanged(" + this.getEl().getWidth() + "x" + this.getEl().getWidth() + ")");
                        };
                        ContentSelector.prototype.getValueType = function () {
                            return ValueTypes.REFERENCE;
                        };
                        ContentSelector.prototype.newInitialValue = function () {
                            return null;
                        };
                        ContentSelector.prototype.layout = function (input, propertyArray) {
                            var _this = this;
                            _super.prototype.layout.call(this, input, propertyArray);
                            var relationshipLoader = new relationship.ContentSelectorLoader();
                            this.contentComboBox = api.content.ContentComboBox.create().setName(input.getName()).setMaximumOccurrences(input.getOccurrences().getMaximum()).setLoader(relationshipLoader).build();
                            return new GetRelationshipTypeByNameRequest(this.relationshipTypeName).sendAndParse().then(function (relationshipType) {
                                _this.contentComboBox.setInputIconUrl(relationshipType.getIconUrl());
                                var inputAllowedContentTypes = _this.allowedContentTypes || [];
                                var relationshipAllowedContentTypes = relationshipType.getAllowedToTypes() || [];
                                var allowedContentTypes = inputAllowedContentTypes.length ? inputAllowedContentTypes : relationshipAllowedContentTypes;
                                relationshipLoader.setAllowedContentTypes(allowedContentTypes);
                                _this.appendChild(_this.contentComboBox);
                                return _this.doLoadContent(propertyArray).then(function (contents) {
                                    contents.forEach(function (content) {
                                        _this.contentComboBox.select(content);
                                    });
                                    _this.contentComboBox.onOptionSelected(function (event) {
                                        var reference = api.util.Reference.from(event.getOption().displayValue.getContentId());
                                        var value = new Value(reference, ValueTypes.REFERENCE);
                                        if (_this.contentComboBox.countSelected() == 1) {
                                            _this.getPropertyArray().set(0, value);
                                        }
                                        else if (!_this.getPropertyArray().containsValue(value)) {
                                            _this.getPropertyArray().add(value);
                                        }
                                        _this.refreshSortable();
                                        _this.updateSelectedOptionStyle();
                                        _this.validate(false);
                                    });
                                    _this.contentComboBox.onOptionDeselected(function (removed) {
                                        _this.getPropertyArray().remove(removed.getIndex());
                                        _this.updateSelectedOptionStyle();
                                        _this.validate(false);
                                    });
                                    _this.setupSortable();
                                    _this.setLayoutInProgress(false);
                                });
                            });
                        };
                        ContentSelector.prototype.doLoadContent = function (propertyArray) {
                            var contentIds = [];
                            propertyArray.forEach(function (property) {
                                if (property.hasNonNullValue()) {
                                    var referenceValue = property.getReference();
                                    if (referenceValue instanceof api.util.Reference) {
                                        contentIds.push(_content.ContentId.fromReference(referenceValue));
                                    }
                                }
                            });
                            return new api.content.GetContentSummaryByIds(contentIds).get().then(function (result) {
                                return result;
                            });
                        };
                        ContentSelector.prototype.setupSortable = function () {
                            var _this = this;
                            wemjq(this.getHTMLElement()).find(".selected-options").sortable({
                                axis: "y",
                                containment: 'parent',
                                handle: '.drag-control',
                                tolerance: 'pointer',
                                start: function (event, ui) { return _this.handleDnDStart(event, ui); },
                                update: function (event, ui) { return _this.handleDnDUpdate(event, ui); }
                            });
                            this.updateSelectedOptionStyle();
                        };
                        ContentSelector.prototype.handleDnDStart = function (event, ui) {
                            var draggedElement = api.dom.Element.fromHtmlElement(ui.item.context);
                            this.draggingIndex = draggedElement.getSiblingIndex();
                            ui.placeholder.html("Drop form item set here");
                        };
                        ContentSelector.prototype.handleDnDUpdate = function (event, ui) {
                            if (this.draggingIndex >= 0) {
                                var draggedElement = api.dom.Element.fromHtmlElement(ui.item.context);
                                var draggedToIndex = draggedElement.getSiblingIndex();
                                this.getPropertyArray().move(this.draggingIndex, draggedToIndex);
                            }
                            this.draggingIndex = -1;
                        };
                        ContentSelector.prototype.updateSelectedOptionStyle = function () {
                            if (this.getPropertyArray().getSize() > 1) {
                                this.addClass("multiple-occurrence").removeClass("single-occurrence");
                            }
                            else {
                                this.addClass("single-occurrence").removeClass("multiple-occurrence");
                            }
                        };
                        ContentSelector.prototype.refreshSortable = function () {
                            wemjq(this.getHTMLElement()).find(".selected-options").sortable("refresh");
                        };
                        ContentSelector.prototype.getNumberOfValids = function () {
                            return this.contentComboBox.countSelected();
                        };
                        ContentSelector.prototype.giveFocus = function () {
                            if (this.contentComboBox.maximumOccurrencesReached()) {
                                return false;
                            }
                            return this.contentComboBox.giveFocus();
                        };
                        ContentSelector.prototype.onFocus = function (listener) {
                            this.contentComboBox.onFocus(listener);
                        };
                        ContentSelector.prototype.unFocus = function (listener) {
                            this.contentComboBox.unFocus(listener);
                        };
                        ContentSelector.prototype.onBlur = function (listener) {
                            this.contentComboBox.onBlur(listener);
                        };
                        ContentSelector.prototype.unBlur = function (listener) {
                            this.contentComboBox.unBlur(listener);
                        };
                        return ContentSelector;
                    })(api.form.inputtype.support.BaseInputTypeManagingAdd);
                    relationship.ContentSelector = ContentSelector;
                    api.form.inputtype.InputTypeManager.register(new api.Class("ContentSelector", ContentSelector));
                })(relationship = inputtype.relationship || (inputtype.relationship = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var LoadMask = api.ui.mask.LoadMask;
                    var ImageSelectorSelectedOptionView = (function (_super) {
                        __extends(ImageSelectorSelectedOptionView, _super);
                        function ImageSelectorSelectedOptionView(option) {
                            _super.call(this, option);
                            this.selectionChangeListeners = [];
                        }
                        ImageSelectorSelectedOptionView.prototype.setOption = function (option) {
                            _super.prototype.setOption.call(this, option);
                            var content = this.getOption().displayValue;
                            if (content.getContentSummary()) {
                                this.updateIconSrc(content);
                                this.label.getEl().setInnerHtml(content.getLabel());
                            }
                            else {
                                this.showProgress();
                            }
                        };
                        ImageSelectorSelectedOptionView.prototype.updateIconSrc = function (content) {
                            var newIconSrc = content.getImageUrl() + "?thumbnail=false&size=" + ImageSelectorSelectedOptionView.IMAGE_SIZE;
                            if (this.icon.getSrc().indexOf(newIconSrc) == -1) {
                                if (this.isVisible()) {
                                    this.showSpinner();
                                }
                                this.icon.setSrc(newIconSrc);
                            }
                        };
                        ImageSelectorSelectedOptionView.prototype.setProgress = function (value) {
                            this.progress.setValue(value);
                            if (value == 100) {
                                this.showSpinner();
                            }
                        };
                        ImageSelectorSelectedOptionView.prototype.layout = function () {
                            var _this = this;
                            this.icon = new api.dom.ImgEl();
                            this.appendChild(this.icon);
                            this.label = new api.dom.DivEl("label");
                            this.appendChild(this.label);
                            this.check = new api.ui.Checkbox();
                            this.appendChild(this.check);
                            this.progress = new api.ui.ProgressBar();
                            this.appendChild(this.progress);
                            this.error = new api.dom.DivEl("error");
                            this.appendChild(this.error);
                            this.loadMask = new LoadMask(this);
                            this.appendChild(this.loadMask);
                            this.check.onClicked(function (event) {
                                _this.check.toggleChecked();
                                event.preventDefault();
                                event.stopPropagation();
                            });
                            this.check.onMouseDown(function (event) {
                                event.stopPropagation();
                                event.preventDefault();
                            });
                            this.check.onValueChanged(function (event) {
                                _this.notifyChecked(event.getNewValue() == 'true');
                            });
                            this.onShown(function () {
                                if (_this.getOption().displayValue.getContentSummary()) {
                                    if (!_this.icon.isLoaded()) {
                                        _this.showSpinner();
                                    }
                                }
                            });
                            this.icon.onLoaded(function (event) {
                                if (_this.getOption().displayValue.getContentSummary()) {
                                    _this.showResult();
                                }
                            });
                        };
                        ImageSelectorSelectedOptionView.prototype.showProgress = function () {
                            this.check.hide();
                            this.icon.getEl().setVisibility('hidden');
                            this.loadMask.hide();
                            this.progress.show();
                        };
                        ImageSelectorSelectedOptionView.prototype.showSpinner = function () {
                            this.progress.hide();
                            this.check.hide();
                            this.icon.getEl().setVisibility('hidden');
                            this.loadMask.show();
                        };
                        ImageSelectorSelectedOptionView.prototype.showResult = function () {
                            this.loadMask.hide();
                            this.icon.getEl().setVisibility('visible');
                            this.check.show();
                            this.progress.hide();
                        };
                        ImageSelectorSelectedOptionView.prototype.showError = function (text) {
                            this.progress.hide();
                            this.error.setHtml(text).show();
                            this.check.show();
                        };
                        ImageSelectorSelectedOptionView.prototype.updateProportions = function (optionHeight) {
                            this.getEl().setHeightPx(optionHeight);
                            var iconHeight = this.icon.getEl().getHeightWithBorder();
                            var contentHeight = optionHeight - this.getEl().getBorderTopWidth() - this.getEl().getBorderBottomWidth();
                            if (iconHeight <= contentHeight && iconHeight !== 0) {
                                this.icon.getEl().setMarginTop((contentHeight - iconHeight) / 2 + 'px');
                            }
                            else {
                                this.icon.getEl().setMarginTop('0px');
                            }
                            this.centerVertically(this.progress, contentHeight);
                            this.centerVertically(this.error, contentHeight);
                        };
                        ImageSelectorSelectedOptionView.prototype.centerVertically = function (el, contentHeight) {
                            el.getEl().setMarginTop((contentHeight - this.progress.getEl().getHeight()) / 2 + 'px');
                        };
                        ImageSelectorSelectedOptionView.prototype.getIcon = function () {
                            return this.icon;
                        };
                        ImageSelectorSelectedOptionView.prototype.getCheckbox = function () {
                            return this.check;
                        };
                        ImageSelectorSelectedOptionView.prototype.toggleChecked = function () {
                            this.check.toggleChecked();
                        };
                        ImageSelectorSelectedOptionView.prototype.notifyChecked = function (checked) {
                            var _this = this;
                            this.selectionChangeListeners.forEach(function (listener) {
                                listener(_this, checked);
                            });
                        };
                        ImageSelectorSelectedOptionView.prototype.onChecked = function (listener) {
                            this.selectionChangeListeners.push(listener);
                        };
                        ImageSelectorSelectedOptionView.prototype.unChecked = function (listener) {
                            this.selectionChangeListeners = this.selectionChangeListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        ImageSelectorSelectedOptionView.IMAGE_SIZE = 270;
                        return ImageSelectorSelectedOptionView;
                    })(api.ui.selector.combobox.BaseSelectedOptionView);
                    image.ImageSelectorSelectedOptionView = ImageSelectorSelectedOptionView;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var SelectedOption = api.ui.selector.combobox.SelectedOption;
                    var ValueTypes = api.data.ValueTypes;
                    var ValueChangedEvent = api.form.inputtype.ValueChangedEvent;
                    var ImageSelectorSelectedOptionsView = (function (_super) {
                        __extends(ImageSelectorSelectedOptionsView, _super);
                        function ImageSelectorSelectedOptionsView() {
                            var _this = this;
                            _super.call(this);
                            this.numberOfOptionsPerRow = 3;
                            this.selection = [];
                            this.editSelectedOptionsListeners = [];
                            this.removeSelectedOptionsListeners = [];
                            this.valueChangedListeners = [];
                            this.dialog = new image.ImageSelectorDialog();
                            wemjq(this.getHTMLElement()).sortable({
                                containment: this.getHTMLElement(),
                                cursor: 'move',
                                tolerance: 'pointer',
                                start: function (event, ui) { return _this.handleDnDStart(event, ui); },
                                update: function (event, ui) { return _this.handleDnDUpdate(event, ui); }
                            });
                            this.toolbar = new image.SelectionToolbar();
                            this.toolbar.hide();
                            this.toolbar.onEditClicked(function () {
                                _this.notifyEditSelectedOptions(_this.selection);
                            });
                            this.toolbar.onRemoveClicked(function () {
                                _this.removeSelectedOptions(_this.selection);
                            });
                            this.appendChild(this.toolbar);
                            this.onShown(function (event) {
                                _this.updateLayout();
                            });
                        }
                        ImageSelectorSelectedOptionsView.prototype.removeSelectedOptions = function (options) {
                            this.notifyRemoveSelectedOptions(options);
                            this.selection.length = 0;
                            this.updateSelectionToolbarLayout();
                            this.hideImageSelectorDialog();
                        };
                        ImageSelectorSelectedOptionsView.prototype.createSelectedOption = function (option) {
                            return new SelectedOption(new image.ImageSelectorSelectedOptionView(option), this.count());
                        };
                        ImageSelectorSelectedOptionsView.prototype.addOption = function (option) {
                            this.dialog.remove();
                            var selectedOption = this.getByOption(option);
                            if (!selectedOption && !this.maximumOccurrencesReached()) {
                                this.addNewOption(option);
                                return true;
                            }
                            else if (selectedOption) {
                                var displayValue = selectedOption.getOption().displayValue;
                                if (displayValue.getContentSummary() == null && option.displayValue.getContentSummary() != null) {
                                    this.updateUploadedOption(option);
                                    return true;
                                }
                            }
                            return false;
                        };
                        ImageSelectorSelectedOptionsView.prototype.addNewOption = function (option) {
                            var _this = this;
                            var selectedOption = this.createSelectedOption(option);
                            this.getSelectedOptions().push(selectedOption);
                            var optionView = selectedOption.getOptionView();
                            optionView.updateProportions(this.calculateOptionHeight());
                            optionView.onClicked(function (event) {
                                if (_this.dialog.isVisible()) {
                                    if (_this.activeOption != selectedOption) {
                                        _this.hideImageSelectorDialog();
                                        _this.showImageSelectorDialog(selectedOption);
                                    }
                                }
                                else {
                                    _this.showImageSelectorDialog(selectedOption);
                                }
                                _this.uncheckOthers(selectedOption);
                                if (document.activeElement == optionView.getEl().getHTMLElement()) {
                                    optionView.getCheckbox().toggleChecked();
                                }
                                else {
                                    optionView.getCheckbox().setChecked(true);
                                }
                                optionView.getCheckbox().giveFocus();
                            });
                            optionView.getCheckbox().onKeyDown(function (event) {
                                var checkbox = optionView.getCheckbox();
                                switch (event.which) {
                                    case 32:
                                        var isChecked = !checkbox.isChecked();
                                        checkbox.setChecked(isChecked, isChecked);
                                        break;
                                    case 8:
                                        checkbox.setChecked(false);
                                        _this.removeOptionViewAndRefocus(selectedOption);
                                        event.preventDefault();
                                        break;
                                    case 46:
                                        checkbox.setChecked(false);
                                        _this.removeOptionViewAndRefocus(selectedOption);
                                        break;
                                    case 13:
                                        _this.notifyEditSelectedOptions([selectedOption]);
                                        break;
                                    case 9:
                                        _this.hideImageSelectorDialog();
                                        break;
                                }
                                event.stopPropagation();
                            });
                            optionView.getCheckbox().onFocus(function (event) { return _this.showImageSelectorDialog(selectedOption); });
                            optionView.onChecked(function (view, checked) {
                                if (checked) {
                                    if (_this.selection.indexOf(selectedOption) < 0)
                                        _this.selection.push(selectedOption);
                                }
                                else {
                                    var index = _this.selection.indexOf(selectedOption);
                                    if (index > -1) {
                                        _this.selection.splice(index, 1);
                                    }
                                }
                                _this.updateSelectionToolbarLayout();
                            });
                            optionView.getIcon().onLoaded(function (event) {
                                optionView.updateProportions(_this.calculateOptionHeight());
                                wemjq(_this.getHTMLElement()).sortable("refresh");
                            });
                            optionView.insertBeforeEl(this.toolbar);
                        };
                        ImageSelectorSelectedOptionsView.prototype.updateUploadedOption = function (option) {
                            var selectedOption = this.getByOption(option);
                            var content = option.displayValue.getContentSummary();
                            var newOption = {
                                value: content.getId(),
                                displayValue: image.ImageSelectorDisplayValue.fromContentSummary(content)
                            };
                            selectedOption.getOptionView().setOption(newOption);
                        };
                        ImageSelectorSelectedOptionsView.prototype.uncheckOthers = function (option) {
                            var selectedOptions = this.getSelectedOptions();
                            for (var i = 0; i < selectedOptions.length; i++) {
                                var view = selectedOptions[i].getOptionView();
                                if (i != option.getIndex()) {
                                    view.getCheckbox().setChecked(false);
                                }
                            }
                        };
                        ImageSelectorSelectedOptionsView.prototype.removeOptionViewAndRefocus = function (option) {
                            var index = this.isLast(option.getIndex()) ? (this.isFirst(option.getIndex()) ? -1 : option.getIndex() - 1) : option.getIndex();
                            this.notifyRemoveSelectedOptions([option]);
                            this.hideImageSelectorDialog();
                            if (index > -1) {
                                this.getByIndex(index).getOptionView().getCheckbox().giveFocus();
                            }
                        };
                        ImageSelectorSelectedOptionsView.prototype.showImageSelectorDialog = function (option) {
                            if (this.dialog) {
                                if (this.activeOption == option) {
                                    return;
                                }
                                this.dialog.remove();
                            }
                            var selectedOptions = this.getSelectedOptions();
                            for (var i = 0; i < selectedOptions.length; i++) {
                                var view = selectedOptions[i].getOptionView();
                                var passedSelectedOption = i >= option.getIndex();
                                if ((this.isLastInRow(i) || this.isLast(i)) && passedSelectedOption) {
                                    this.dialog.insertAfterEl(view);
                                    break;
                                }
                            }
                            if (this.activeOption) {
                                this.activeOption.getOptionView().removeClass("editing");
                            }
                            this.activeOption = option;
                            option.getOptionView().addClass("editing");
                            this.dialog.setContent(option.getOption().displayValue);
                            this.setOutsideClickListener();
                            this.updateDialogLayout(this.calculateOptionHeight());
                            wemjq(this.getHTMLElement()).sortable("disable");
                        };
                        ImageSelectorSelectedOptionsView.prototype.updateLayout = function () {
                            var optionHeight = this.calculateOptionHeight();
                            this.getSelectedOptions().forEach(function (selectedOption) {
                                selectedOption.getOptionView().updateProportions(optionHeight);
                            });
                            if (this.dialog.isVisible()) {
                                this.updateDialogLayout(optionHeight);
                            }
                        };
                        ImageSelectorSelectedOptionsView.prototype.updateDialogLayout = function (optionHeight) {
                            this.dialog.getEl().setMarginTop(((optionHeight / 3) - 20) + 'px');
                        };
                        ImageSelectorSelectedOptionsView.prototype.updateSelectionToolbarLayout = function () {
                            var showToolbar = this.selection.length > 0;
                            this.toolbar.setVisible(showToolbar);
                            if (showToolbar) {
                                this.toolbar.setSelectionCount(this.selection.length);
                            }
                        };
                        ImageSelectorSelectedOptionsView.prototype.calculateOptionHeight = function () {
                            var availableWidth = this.getEl().getWidthWithMargin();
                            return Math.floor(0.3 * availableWidth);
                        };
                        ImageSelectorSelectedOptionsView.prototype.hideImageSelectorDialog = function () {
                            this.dialog.remove();
                            if (this.activeOption) {
                                this.activeOption.getOptionView().removeClass('editing first-in-row last-in-row');
                                this.activeOption = null;
                            }
                            wemjq(this.getHTMLElement()).sortable("enable");
                            api.dom.Body.get().unClicked(this.mouseClickListener);
                        };
                        ImageSelectorSelectedOptionsView.prototype.setOutsideClickListener = function () {
                            var _this = this;
                            this.mouseClickListener = function (event) {
                                for (var element = event.target; element; element = element.parentNode) {
                                    if (element == _this.getHTMLElement()) {
                                        return;
                                    }
                                }
                                _this.hideImageSelectorDialog();
                            };
                            api.dom.Body.get().onClicked(this.mouseClickListener);
                        };
                        ImageSelectorSelectedOptionsView.prototype.isFirstInRow = function (index) {
                            return index % this.numberOfOptionsPerRow == 0;
                        };
                        ImageSelectorSelectedOptionsView.prototype.isLastInRow = function (index) {
                            return index % this.numberOfOptionsPerRow == 2;
                        };
                        ImageSelectorSelectedOptionsView.prototype.isFirst = function (index) {
                            return index == 0;
                        };
                        ImageSelectorSelectedOptionsView.prototype.isLast = function (index) {
                            return index == this.getSelectedOptions().length - 1;
                        };
                        ImageSelectorSelectedOptionsView.prototype.handleDnDStart = function (event, ui) {
                            var draggedElement = api.dom.Element.fromHtmlElement(ui.item.context);
                            this.draggingIndex = draggedElement.getSiblingIndex();
                        };
                        ImageSelectorSelectedOptionsView.prototype.handleDnDUpdate = function (event, ui) {
                            if (this.draggingIndex >= 0) {
                                var draggedElement = api.dom.Element.fromHtmlElement(ui.item.context);
                                var draggedToIndex = draggedElement.getSiblingIndex();
                                this.handleMovedOccurrence(this.draggingIndex, draggedToIndex);
                            }
                            this.draggingIndex = -1;
                        };
                        ImageSelectorSelectedOptionsView.prototype.handleMovedOccurrence = function (fromIndex, toIndex) {
                            var _this = this;
                            this.moveOccurrence(fromIndex, toIndex);
                            this.getValues().forEach(function (value, index) {
                                if (Math.min(fromIndex, toIndex) <= index && index <= Math.max(fromIndex, toIndex)) {
                                    _this.notifyValueChanged(new ValueChangedEvent(value, index));
                                }
                            });
                        };
                        ImageSelectorSelectedOptionsView.prototype.getValues = function () {
                            return this.getSelectedOptions().map(function (selectedOption) {
                                return ValueTypes.REFERENCE.newValue(selectedOption.getOption().value);
                            });
                        };
                        ImageSelectorSelectedOptionsView.prototype.notifyRemoveSelectedOptions = function (option) {
                            this.removeSelectedOptionsListeners.forEach(function (listener) {
                                listener(option);
                            });
                        };
                        ImageSelectorSelectedOptionsView.prototype.onRemoveSelectedOptions = function (listener) {
                            this.removeSelectedOptionsListeners.push(listener);
                        };
                        ImageSelectorSelectedOptionsView.prototype.unRemoveSelectedOptions = function (listener) {
                            this.removeSelectedOptionsListeners = this.removeSelectedOptionsListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        ImageSelectorSelectedOptionsView.prototype.notifyEditSelectedOptions = function (option) {
                            this.editSelectedOptionsListeners.forEach(function (listener) {
                                listener(option);
                            });
                        };
                        ImageSelectorSelectedOptionsView.prototype.onEditSelectedOptions = function (listener) {
                            this.editSelectedOptionsListeners.push(listener);
                        };
                        ImageSelectorSelectedOptionsView.prototype.unEditSelectedOptions = function (listener) {
                            this.editSelectedOptionsListeners = this.editSelectedOptionsListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        ImageSelectorSelectedOptionsView.prototype.onValueChanged = function (listener) {
                            this.valueChangedListeners.push(listener);
                        };
                        ImageSelectorSelectedOptionsView.prototype.unValueChanged = function (listener) {
                            this.valueChangedListeners.filter(function (currentListener) {
                                return listener == currentListener;
                            });
                        };
                        ImageSelectorSelectedOptionsView.prototype.notifyValueChanged = function (event) {
                            this.valueChangedListeners.forEach(function (listener) {
                                listener(event);
                            });
                        };
                        return ImageSelectorSelectedOptionsView;
                    })(api.ui.selector.combobox.BaseSelectedOptionsView);
                    image.ImageSelectorSelectedOptionsView = ImageSelectorSelectedOptionsView;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var ImageSelectorDialog = (function (_super) {
                        __extends(ImageSelectorDialog, _super);
                        function ImageSelectorDialog() {
                            _super.call(this, "dialog");
                            this.nameEl = new api.dom.H1El();
                            this.appendChild(this.nameEl);
                            this.pathEl = new api.dom.PEl();
                            this.appendChild(this.pathEl);
                        }
                        ImageSelectorDialog.prototype.setContent = function (value) {
                            this.content = value;
                            this.refreshUI();
                        };
                        ImageSelectorDialog.prototype.getContent = function () {
                            return this.content;
                        };
                        ImageSelectorDialog.prototype.refreshUI = function () {
                            this.nameEl.getEl().setInnerHtml(this.content.getLabel());
                            this.pathEl.getEl().setInnerHtml(this.content.getPath());
                        };
                        return ImageSelectorDialog;
                    })(api.dom.DivEl);
                    image.ImageSelectorDialog = ImageSelectorDialog;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var Value = api.data.Value;
                    var ValueTypes = api.data.ValueTypes;
                    var ContentId = api.content.ContentId;
                    var ContentSummaryLoader = api.content.ContentSummaryLoader;
                    var ContentTypeName = api.schema.content.ContentTypeName;
                    var ResponsiveManager = api.ui.responsive.ResponsiveManager;
                    var RelationshipTypeName = api.schema.relationshiptype.RelationshipTypeName;
                    var ImageSelector = (function (_super) {
                        __extends(ImageSelector, _super);
                        function ImageSelector(config) {
                            var _this = this;
                            _super.call(this, "image-selector");
                            this.editContentRequestListeners = [];
                            this.addClass("input-type-view");
                            this.config = config;
                            this.contentRequestsAllowed = false;
                            this.readConfig(config.inputConfig);
                            ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                                _this.availableSizeChanged();
                            });
                            this.onRemoved(function (event) {
                                ResponsiveManager.unAvailableSizeChanged(_this);
                            });
                            this.onShown(function () {
                                _this.updateSelectedItemsIcons();
                            });
                            api.content.ContentDeletedEvent.on(function (event) {
                                var deleted = event.getContentId();
                                var option = _this.selectedOptionsView.getById(deleted.toString());
                                if (option != null) {
                                    _this.selectedOptionsView.removeSelectedOptions([option]);
                                }
                            });
                        }
                        ImageSelector.prototype.readConfig = function (inputConfig) {
                            var relationshipTypeConfig = inputConfig['relationshipType'] ? inputConfig['relationshipType'][0] : {};
                            var relationshipType = relationshipTypeConfig['value'];
                            if (relationshipType) {
                                this.relationshipTypeName = new RelationshipTypeName(relationshipType);
                            }
                            else {
                                this.relationshipTypeName = RelationshipTypeName.REFERENCE;
                            }
                        };
                        ImageSelector.prototype.updateSelectedItemsIcons = function () {
                            var _this = this;
                            if (this.contentComboBox.getSelectedOptions().length > 0) {
                                this.doLoadContent(this.getPropertyArray()).then(function (contents) {
                                    contents.forEach(function (content) {
                                        _this.selectedOptionsView.updateUploadedOption({
                                            value: content.getId(),
                                            displayValue: image.ImageSelectorDisplayValue.fromContentSummary(content)
                                        });
                                    });
                                    _this.setLayoutInProgress(false);
                                });
                            }
                        };
                        ImageSelector.prototype.availableSizeChanged = function () {
                            if (this.selectedOptionsView) {
                                this.selectedOptionsView.updateLayout();
                            }
                        };
                        ImageSelector.prototype.getValueType = function () {
                            return ValueTypes.REFERENCE;
                        };
                        ImageSelector.prototype.newInitialValue = function () {
                            return null;
                        };
                        ImageSelector.prototype.countSelectedOptions = function () {
                            return this.selectedOptionsView.count();
                        };
                        ImageSelector.prototype.getRemainingOccurrences = function () {
                            var inputMaximum = this.getInput().getOccurrences().getMaximum();
                            var countSelected = this.countSelectedOptions();
                            var rest = -1;
                            if (inputMaximum == 0) {
                                rest = 0;
                            }
                            else {
                                rest = inputMaximum - countSelected;
                                rest = (rest == 0) ? -1 : rest;
                            }
                            return rest;
                        };
                        ImageSelector.prototype.createSelectedOptionsView = function () {
                            var _this = this;
                            var selectedOptionsView = new image.ImageSelectorSelectedOptionsView();
                            selectedOptionsView.onEditSelectedOptions(function (options) {
                                options.forEach(function (option) {
                                    _this.notifyEditContentRequested(option.getOption().displayValue.getContentSummary());
                                });
                            });
                            selectedOptionsView.onRemoveSelectedOptions(function (options) {
                                options.forEach(function (option) {
                                    _this.contentComboBox.deselect(option.getOption().displayValue);
                                });
                                _this.validate(false);
                            });
                            selectedOptionsView.onValueChanged(function (event) {
                                _this.getPropertyArray().set(event.getArrayIndex(), event.getNewValue());
                                _this.validate(false);
                            });
                            return selectedOptionsView;
                        };
                        ImageSelector.prototype.createContentComboBox = function (maximumOccurrences, inputIconUrl, allowedContentTypes) {
                            var _this = this;
                            var contentComboBox = image.ImageContentComboBox.create().setMaximumOccurrences(maximumOccurrences).setAllowedContentTypes(allowedContentTypes.length ? allowedContentTypes : [ContentTypeName.IMAGE.toString()]).setLoader(this.contentSummaryLoader = new ContentSummaryLoader()).setSelectedOptionsView(this.selectedOptionsView = this.createSelectedOptionsView()).build(), comboBox = contentComboBox.getComboBox();
                            comboBox.onHidden(function (event) {
                                if (_this.uploader) {
                                    _this.uploader.hide();
                                }
                            });
                            comboBox.onShown(function (event) {
                                if (_this.uploader) {
                                    _this.uploader.show();
                                }
                            });
                            comboBox.setInputIconUrl(inputIconUrl);
                            comboBox.onOptionDeselected(function (removed) {
                                if (!!removed.getOption().displayValue.getContentSummary()) {
                                    _this.getPropertyArray().remove(removed.getIndex());
                                }
                                _this.validate(false);
                            });
                            comboBox.onOptionSelected(function (event) {
                                if (!_this.isLayoutInProgress()) {
                                    var contentId = event.getOption().displayValue.getContentId();
                                    if (!contentId) {
                                        return;
                                    }
                                    _this.setContentIdProperty(contentId);
                                }
                                _this.validate(false);
                            });
                            return contentComboBox;
                        };
                        ImageSelector.prototype.layout = function (input, propertyArray) {
                            var _this = this;
                            _super.prototype.layout.call(this, input, propertyArray);
                            return new api.schema.relationshiptype.GetRelationshipTypeByNameRequest(this.relationshipTypeName).sendAndParse().then(function (relationshipType) {
                                _this.contentComboBox = _this.createContentComboBox(input.getOccurrences().getMaximum(), relationshipType.getIconUrl(), relationshipType.getAllowedToTypes() || []);
                                var comboBoxWrapper = new api.dom.DivEl("combobox-wrapper");
                                comboBoxWrapper.appendChild(_this.contentComboBox);
                                _this.contentRequestsAllowed = true;
                                if (_this.config.contentId) {
                                    comboBoxWrapper.appendChild(_this.createUploader());
                                }
                                _this.appendChild(comboBoxWrapper);
                                _this.appendChild(_this.selectedOptionsView);
                                return _this.doLoadContent(_this.getPropertyArray()).then(function (contents) {
                                    contents.forEach(function (content) {
                                        _this.contentComboBox.selectOption({
                                            value: content.getId(),
                                            displayValue: image.ImageSelectorDisplayValue.fromContentSummary(content)
                                        });
                                    });
                                    _this.setLayoutInProgress(false);
                                });
                            });
                        };
                        ImageSelector.prototype.createUploader = function () {
                            var _this = this;
                            var multiSelection = (this.getInput().getOccurrences().getMaximum() != 1);
                            this.uploader = new api.content.ImageUploader({
                                params: {
                                    parent: this.config.contentId.toString()
                                },
                                operation: 0 /* create */,
                                name: 'image-selector-upload-dialog',
                                showCancel: false,
                                showReset: false,
                                showResult: false,
                                maximumOccurrences: this.getRemainingOccurrences(),
                                allowMultiSelection: multiSelection,
                                scaleWidth: false,
                                deferred: true
                            });
                            this.uploader.onUploadStarted(function (event) {
                                event.getUploadItems().forEach(function (uploadItem) {
                                    var value = image.ImageSelectorDisplayValue.fromUploadItem(uploadItem);
                                    var option = {
                                        value: value.getId(),
                                        displayValue: value
                                    };
                                    _this.contentComboBox.selectOption(option);
                                });
                            });
                            this.uploader.onUploadProgress(function (event) {
                                var item = event.getUploadItem();
                                var selectedOption = _this.selectedOptionsView.getById(item.getId());
                                if (!!selectedOption) {
                                    selectedOption.getOptionView().setProgress(item.getProgress());
                                }
                                _this.uploader.setMaximumOccurrences(_this.getRemainingOccurrences());
                            });
                            this.uploader.onFileUploaded(function (event) {
                                var item = event.getUploadItem();
                                var createdContent = item.getModel();
                                new api.content.ContentUpdatedEvent(_this.config.contentId).fire();
                                var selectedOption = _this.selectedOptionsView.getById(item.getId());
                                var option = selectedOption.getOption();
                                option.displayValue.setContentSummary(createdContent);
                                option.value = createdContent.getContentId().toString();
                                selectedOption.getOptionView().setOption(option);
                                var optionView = selectedOption.getOptionView();
                                optionView.getCheckbox().setChecked(true);
                                _this.setContentIdProperty(createdContent.getContentId());
                                _this.validate(false);
                                _this.uploader.setMaximumOccurrences(_this.getRemainingOccurrences());
                            });
                            this.uploader.onUploadFailed(function (event) {
                                var item = event.getUploadItem();
                                var selectedOption = _this.selectedOptionsView.getById(item.getId());
                                if (!!selectedOption) {
                                    selectedOption.getOptionView().showError("Upload failed");
                                }
                                _this.uploader.setMaximumOccurrences(_this.getRemainingOccurrences());
                            });
                            this.uploader.onClicked(function () {
                                _this.uploader.setMaximumOccurrences(_this.getRemainingOccurrences());
                            });
                            var iFrame = api.app.Application.getApplication().getAppFrame();
                            var body = api.dom.Body.get();
                            this.uploader.addClass("minimized");
                            var dragOverEl;
                            this.onDragEnter(function (event) {
                                if (iFrame.isVisible()) {
                                    var target = event.target;
                                    _this.uploader.giveFocus();
                                    _this.uploader.toggleClass("minimized", false);
                                    dragOverEl = target;
                                }
                            });
                            body.onDragLeave(function (event) {
                                if (iFrame.isVisible()) {
                                    var targetEl = event.target;
                                    if (dragOverEl == targetEl) {
                                        _this.uploader.giveBlur();
                                        _this.uploader.toggleClass("minimized", true);
                                        dragOverEl = null;
                                    }
                                }
                            });
                            body.onDrop(function (event) {
                                if (iFrame.isVisible()) {
                                    _this.uploader.setMaximumOccurrences(_this.getRemainingOccurrences());
                                    _this.uploader.toggleClass("minimized", true);
                                }
                            });
                            return this.uploader;
                        };
                        ImageSelector.prototype.doLoadContent = function (propertyArray) {
                            var contentIds = [];
                            propertyArray.forEach(function (property) {
                                if (property.hasNonNullValue()) {
                                    contentIds.push(ContentId.fromReference(property.getReference()));
                                }
                            });
                            return new api.content.GetContentSummaryByIds(contentIds).get();
                        };
                        ImageSelector.prototype.getNumberOfValids = function () {
                            return this.contentComboBox.countSelected();
                        };
                        ImageSelector.prototype.giveFocus = function () {
                            if (this.contentComboBox.maximumOccurrencesReached()) {
                                return false;
                            }
                            return this.contentComboBox.giveFocus();
                        };
                        ImageSelector.prototype.setContentIdProperty = function (contentId) {
                            var reference = api.util.Reference.from(contentId);
                            var value = new Value(reference, ValueTypes.REFERENCE);
                            if (this.contentComboBox.countSelected() == 1) {
                                this.getPropertyArray().set(0, value);
                            }
                            else {
                                this.getPropertyArray().add(value);
                            }
                        };
                        ImageSelector.prototype.onFocus = function (listener) {
                            this.contentComboBox.onFocus(listener);
                        };
                        ImageSelector.prototype.unFocus = function (listener) {
                            this.contentComboBox.unFocus(listener);
                        };
                        ImageSelector.prototype.onBlur = function (listener) {
                            this.contentComboBox.onBlur(listener);
                        };
                        ImageSelector.prototype.unBlur = function (listener) {
                            this.contentComboBox.unBlur(listener);
                        };
                        ImageSelector.prototype.onEditContentRequest = function (listener) {
                            this.editContentRequestListeners.push(listener);
                        };
                        ImageSelector.prototype.unEditContentRequest = function (listener) {
                            this.editContentRequestListeners = this.editContentRequestListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        ImageSelector.prototype.notifyEditContentRequested = function (content) {
                            this.editContentRequestListeners.forEach(function (listener) {
                                listener(content);
                            });
                        };
                        return ImageSelector;
                    })(api.form.inputtype.support.BaseInputTypeManagingAdd);
                    image.ImageSelector = ImageSelector;
                    api.form.inputtype.InputTypeManager.register(new api.Class("ImageSelector", ImageSelector));
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var UploadDialogCancelAction = (function (_super) {
                        __extends(UploadDialogCancelAction, _super);
                        function UploadDialogCancelAction() {
                            _super.call(this, "Cancel", "esc");
                        }
                        return UploadDialogCancelAction;
                    })(api.ui.Action);
                    image.UploadDialogCancelAction = UploadDialogCancelAction;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var ImageUploadDialog = (function (_super) {
                        __extends(ImageUploadDialog, _super);
                        function ImageUploadDialog(parentContent, allowMultiSelection) {
                            var _this = this;
                            if (allowMultiSelection === void 0) { allowMultiSelection = false; }
                            api.util.assertNotNull(parentContent, "parentContent required");
                            _super.call(this, {
                                title: new api.ui.dialog.ModalDialogHeader("Image uploader")
                            });
                            this.getEl().addClass("upload-dialog");
                            var description = new api.dom.PEl();
                            description.getEl().setInnerHtml("Images uploaded will be added as children to this content, you may move them later if desired");
                            this.appendChildToContentPanel(description);
                            this.imageUploader = new api.content.ImageUploader({
                                params: {
                                    parent: parentContent.toString()
                                },
                                operation: 0 /* create */,
                                name: 'image-selector-upload-dialog',
                                showCancel: false,
                                showReset: false,
                                showResult: false,
                                allowMultiSelection: allowMultiSelection,
                                scaleWidth: false
                            });
                            this.imageUploader.onUploadCompleted(function () {
                                _this.close();
                            });
                            this.appendChildToContentPanel(this.imageUploader);
                            this.getCancelAction().onExecuted(function (action) {
                                _this.imageUploader.stop();
                                _this.imageUploader.reset();
                                _this.close();
                            });
                            api.dom.Body.get().appendChild(this);
                            this.addCancelButtonToBottom();
                        }
                        ImageUploadDialog.prototype.setMaximumOccurrences = function (value) {
                            this.imageUploader.setMaximumOccurrences(value);
                        };
                        ImageUploadDialog.prototype.open = function () {
                            this.imageUploader.reset();
                            _super.prototype.open.call(this);
                        };
                        ImageUploadDialog.prototype.onUploadStarted = function (listener) {
                            this.imageUploader.onUploadStarted(listener);
                        };
                        ImageUploadDialog.prototype.unUploadStarted = function (listener) {
                            this.imageUploader.unUploadStarted(listener);
                        };
                        ImageUploadDialog.prototype.onUploadProgress = function (listener) {
                            this.imageUploader.onUploadProgress(listener);
                        };
                        ImageUploadDialog.prototype.unUploadProgress = function (listener) {
                            this.imageUploader.unUploadProgress(listener);
                        };
                        ImageUploadDialog.prototype.onImageUploaded = function (listener) {
                            this.imageUploader.onFileUploaded(listener);
                        };
                        ImageUploadDialog.prototype.unImageUploaded = function (listener) {
                            this.imageUploader.unFileUploaded(listener);
                        };
                        ImageUploadDialog.prototype.onUploadCompleted = function (listener) {
                            this.imageUploader.onUploadCompleted(listener);
                        };
                        ImageUploadDialog.prototype.unUploadCompleted = function (listener) {
                            this.imageUploader.unUploadCompleted(listener);
                        };
                        ImageUploadDialog.prototype.onUploadReset = function (listener) {
                            this.imageUploader.onUploadReset(listener);
                        };
                        ImageUploadDialog.prototype.unUploadReset = function (listener) {
                            this.imageUploader.unUploadReset(listener);
                        };
                        ImageUploadDialog.prototype.onUploadFailed = function (listener) {
                            this.imageUploader.onUploadFailed(listener);
                        };
                        ImageUploadDialog.prototype.unUploadFailed = function (listener) {
                            this.imageUploader.unUploadFailed(listener);
                        };
                        return ImageUploadDialog;
                    })(api.ui.dialog.ModalDialog);
                    image.ImageUploadDialog = ImageUploadDialog;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var Button = api.ui.button.Button;
                    var SelectionToolbar = (function (_super) {
                        __extends(SelectionToolbar, _super);
                        function SelectionToolbar() {
                            var _this = this;
                            _super.call(this, "selection-toolbar");
                            this.editClickListeners = [];
                            this.removeClickListeners = [];
                            this.editButton = new Button("Edit");
                            this.editButton.addClass("large");
                            this.editButton.onClicked(function (event) {
                                _this.notifyEditClicked();
                            });
                            this.appendChild(this.editButton);
                            this.removeButton = new Button("Remove");
                            this.removeButton.addClass("large red");
                            this.removeButton.onClicked(function (event) {
                                _this.notifyRemoveClicked();
                            });
                            this.appendChild(this.removeButton);
                        }
                        SelectionToolbar.prototype.setSelectionCount = function (count) {
                            this.selectionCount = count;
                            this.refreshUI();
                        };
                        SelectionToolbar.prototype.refreshUI = function () {
                            this.editButton.setLabel("Edit" + (this.selectionCount > 0 ? " (" + this.selectionCount + ")" : ""));
                            this.removeButton.setLabel("Remove " + (this.selectionCount > 0 ? " (" + this.selectionCount + ")" : ""));
                        };
                        SelectionToolbar.prototype.notifyEditClicked = function () {
                            this.editClickListeners.forEach(function (listener) {
                                listener();
                            });
                        };
                        SelectionToolbar.prototype.onEditClicked = function (listener) {
                            this.editClickListeners.push(listener);
                        };
                        SelectionToolbar.prototype.unEditClicked = function (listener) {
                            this.editClickListeners = this.editClickListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        SelectionToolbar.prototype.notifyRemoveClicked = function () {
                            this.removeClickListeners.forEach(function (listener) {
                                listener();
                            });
                        };
                        SelectionToolbar.prototype.onRemoveClicked = function (listener) {
                            this.removeClickListeners.push(listener);
                        };
                        SelectionToolbar.prototype.unRemoveClicked = function (listener) {
                            this.removeClickListeners = this.removeClickListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        return SelectionToolbar;
                    })(api.dom.DivEl);
                    image.SelectionToolbar = SelectionToolbar;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var FocusChangedEvent = (function () {
                        function FocusChangedEvent(focused) {
                            this.focused = focused;
                        }
                        FocusChangedEvent.prototype.isFocused = function () {
                            return this.focused;
                        };
                        return FocusChangedEvent;
                    })();
                    image.FocusChangedEvent = FocusChangedEvent;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var ContentIconUrlResolver = api.content.ContentIconUrlResolver;
                    var ImageSelectorDisplayValue = (function () {
                        function ImageSelectorDisplayValue() {
                        }
                        ImageSelectorDisplayValue.fromUploadItem = function (item) {
                            return new ImageSelectorDisplayValue().setUploadItem(item);
                        };
                        ImageSelectorDisplayValue.fromContentSummary = function (content) {
                            return new ImageSelectorDisplayValue().setContentSummary(content);
                        };
                        ImageSelectorDisplayValue.prototype.setUploadItem = function (item) {
                            this.uploadItem = item;
                            return this;
                        };
                        ImageSelectorDisplayValue.prototype.setContentSummary = function (contentSummary) {
                            this.content = contentSummary;
                            return this;
                        };
                        ImageSelectorDisplayValue.prototype.getUploadItem = function () {
                            return this.uploadItem;
                        };
                        ImageSelectorDisplayValue.prototype.getContentSummary = function () {
                            return this.content;
                        };
                        ImageSelectorDisplayValue.prototype.getId = function () {
                            return this.content ? this.content.getId() : this.uploadItem.getId();
                        };
                        ImageSelectorDisplayValue.prototype.getContentId = function () {
                            return this.content ? this.content.getContentId() : null;
                        };
                        ImageSelectorDisplayValue.prototype.getImageUrl = function () {
                            return this.content ? new ContentIconUrlResolver().setContent(this.content).resolve() : null;
                        };
                        ImageSelectorDisplayValue.prototype.getLabel = function () {
                            return this.content ? this.content.getName().toString() : this.uploadItem.getName();
                        };
                        ImageSelectorDisplayValue.prototype.getDisplayName = function () {
                            return this.content ? this.content.getDisplayName() : null;
                        };
                        ImageSelectorDisplayValue.prototype.getTypeLocaleName = function () {
                            return (this.content && this.content.getType()) ? this.content.getType().getLocalName() : null;
                        };
                        ImageSelectorDisplayValue.prototype.getPath = function () {
                            return this.content ? this.content.getPath().toString() : null;
                        };
                        return ImageSelectorDisplayValue;
                    })();
                    image.ImageSelectorDisplayValue = ImageSelectorDisplayValue;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var ImageSelectorViewer = (function (_super) {
                        __extends(ImageSelectorViewer, _super);
                        function ImageSelectorViewer() {
                            _super.call(this);
                        }
                        ImageSelectorViewer.prototype.resolveDisplayName = function (object) {
                            return object.getDisplayName();
                        };
                        ImageSelectorViewer.prototype.resolveUnnamedDisplayName = function (object) {
                            return object.getTypeLocaleName();
                        };
                        ImageSelectorViewer.prototype.resolveSubName = function (object, relativePath) {
                            if (relativePath === void 0) { relativePath = false; }
                            return object.getPath();
                        };
                        ImageSelectorViewer.prototype.resolveIconUrl = function (object) {
                            return object.getImageUrl() + "?crop=false";
                        };
                        return ImageSelectorViewer;
                    })(api.ui.NamesAndIconViewer);
                    image.ImageSelectorViewer = ImageSelectorViewer;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var image;
                (function (image) {
                    var ContentSummaryLoader = api.content.ContentSummaryLoader;
                    var RichComboBox = api.ui.selector.combobox.RichComboBox;
                    var RichComboBoxBuilder = api.ui.selector.combobox.RichComboBoxBuilder;
                    var ImageSelectorDisplayValue = api.content.form.inputtype.image.ImageSelectorDisplayValue;
                    var ImageContentComboBox = (function (_super) {
                        __extends(ImageContentComboBox, _super);
                        function ImageContentComboBox(builder) {
                            var loader = builder.loader ? builder.loader : new ContentSummaryLoader();
                            loader.setAllowedContentTypes(builder.allowedContentTypes);
                            var richComboBoxBuilder = new RichComboBoxBuilder().setComboBoxName(builder.name ? builder.name : 'imageContentSelector').setLoader(loader).setSelectedOptionsView(builder.selectedOptionsView || new api.content.form.inputtype.image.ImageSelectorSelectedOptionsView()).setMaximumOccurrences(builder.maximumOccurrences).setOptionDisplayValueViewer(new api.content.form.inputtype.image.ImageSelectorViewer()).setDelayedInputValueChangedHandling(750).setMinWidth(builder.minWidth);
                            _super.call(this, richComboBoxBuilder);
                        }
                        ImageContentComboBox.prototype.createOption = function (value) {
                            return {
                                value: this.getDisplayValueId(value),
                                displayValue: ImageSelectorDisplayValue.fromContentSummary(value)
                            };
                        };
                        ImageContentComboBox.create = function () {
                            return new ImageContentComboBoxBuilder();
                        };
                        return ImageContentComboBox;
                    })(RichComboBox);
                    image.ImageContentComboBox = ImageContentComboBox;
                    var ImageContentComboBoxBuilder = (function () {
                        function ImageContentComboBoxBuilder() {
                            this.maximumOccurrences = 0;
                        }
                        ImageContentComboBoxBuilder.prototype.setName = function (value) {
                            this.name = value;
                            return this;
                        };
                        ImageContentComboBoxBuilder.prototype.setMaximumOccurrences = function (maximumOccurrences) {
                            this.maximumOccurrences = maximumOccurrences;
                            return this;
                        };
                        ImageContentComboBoxBuilder.prototype.setLoader = function (loader) {
                            this.loader = loader;
                            return this;
                        };
                        ImageContentComboBoxBuilder.prototype.setAllowedContentTypes = function (allowedTypes) {
                            this.allowedContentTypes = allowedTypes;
                            return this;
                        };
                        ImageContentComboBoxBuilder.prototype.setMinWidth = function (value) {
                            this.minWidth = value;
                            return this;
                        };
                        ImageContentComboBoxBuilder.prototype.setSelectedOptionsView = function (value) {
                            this.selectedOptionsView = value;
                            return this;
                        };
                        ImageContentComboBoxBuilder.prototype.setOptionDisplayValueViewer = function (value) {
                            this.optionDisplayValueViewer = value;
                            return this;
                        };
                        ImageContentComboBoxBuilder.prototype.build = function () {
                            return new ImageContentComboBox(this);
                        };
                        return ImageContentComboBoxBuilder;
                    })();
                    image.ImageContentComboBoxBuilder = ImageContentComboBoxBuilder;
                })(image = inputtype.image || (inputtype.image = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (_content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var tag;
                (function (tag) {
                    var ContentQuery = api.content.query.ContentQuery;
                    var ContentQueryRequest = api.content.ContentQueryRequest;
                    var QueryExpr = api.query.expr.QueryExpr;
                    var ContentTagSuggesterBuilder = (function () {
                        function ContentTagSuggesterBuilder() {
                        }
                        ContentTagSuggesterBuilder.prototype.setDataPath = function (value) {
                            this.dataPath = value;
                            return this;
                        };
                        ContentTagSuggesterBuilder.prototype.build = function () {
                            return new ContentTagSuggester(this);
                        };
                        return ContentTagSuggesterBuilder;
                    })();
                    tag.ContentTagSuggesterBuilder = ContentTagSuggesterBuilder;
                    var ContentTagSuggester = (function () {
                        function ContentTagSuggester(builder) {
                            this.propertyPath = builder.dataPath;
                        }
                        ContentTagSuggester.prototype.suggest = function (value) {
                            var _this = this;
                            var fieldName = "data" + this.propertyPath.getParentPath().toString() + this.propertyPath.getLastElement().getName();
                            var fulltextExpression = new api.query.FulltextSearchExpressionBuilder().setSearchString(value).addField(new api.query.QueryField(fieldName)).build();
                            var queryExpr = new QueryExpr(fulltextExpression);
                            var query = new ContentQuery();
                            query.setSize(10);
                            query.setQueryExpr(queryExpr);
                            var queryRequest = new ContentQueryRequest(query);
                            queryRequest.setExpand(2 /* FULL */);
                            return queryRequest.sendAndParse().then(function (contentQueryResult) {
                                var suggestedTags = [];
                                contentQueryResult.getContents().forEach(function (content) {
                                    var propertySet = _this.propertyPath.getParentPath().isRoot() ? content.getContentData().getRoot() : content.getContentData().getPropertySet(_this.propertyPath);
                                    propertySet.forEachProperty(_this.propertyPath.getLastElement().getName(), function (property) {
                                        if (property.hasNonNullValue()) {
                                            var suggestedTag = property.getString();
                                            if (suggestedTag.search(new RegExp(value, "i")) == 0 && suggestedTags.indexOf(suggestedTag) < 0) {
                                                suggestedTags.push(suggestedTag);
                                            }
                                        }
                                    });
                                });
                                return suggestedTags;
                            });
                        };
                        return ContentTagSuggester;
                    })();
                    tag.ContentTagSuggester = ContentTagSuggester;
                })(tag = inputtype.tag || (inputtype.tag = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = _content.form || (_content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var tag;
                (function (tag) {
                    var PropertyPath = api.data.PropertyPath;
                    var PropertyPathElement = api.data.PropertyPathElement;
                    var Value = api.data.Value;
                    var ValueTypes = api.data.ValueTypes;
                    var Tag = (function (_super) {
                        __extends(Tag, _super);
                        function Tag(context) {
                            _super.call(this, "tag");
                            this.addClass("input-type-view");
                            var dataPath = this.resolveDataPath(context);
                            this.tagSuggester = new tag.ContentTagSuggesterBuilder().setDataPath(dataPath).build();
                            var tagsBuilder = new api.ui.tags.TagsBuilder().setTagSuggester(this.tagSuggester).setMaxTags(context.input.getOccurrences().getMaximum());
                            this.tags = tagsBuilder.build();
                            this.appendChild(this.tags);
                        }
                        Tag.prototype.resolveDataPath = function (context) {
                            if (context.parentDataPath) {
                                return PropertyPath.fromParent(context.parentDataPath, PropertyPathElement.fromString(context.input.getName()));
                            }
                            else {
                                return new PropertyPath([PropertyPathElement.fromString(context.input.getName())], false);
                            }
                        };
                        Tag.prototype.availableSizeChanged = function () {
                        };
                        Tag.prototype.getValueType = function () {
                            return ValueTypes.STRING;
                        };
                        Tag.prototype.newInitialValue = function () {
                            return null;
                        };
                        Tag.prototype.layout = function (input, propertyArray) {
                            var _this = this;
                            _super.prototype.layout.call(this, input, propertyArray);
                            this.tags.clearTags();
                            propertyArray.forEach(function (property) {
                                if (property.hasNonNullValue()) {
                                    _this.tags.addTag(property.getString());
                                }
                            });
                            this.tags.onTagAdded(function (event) {
                                var value = new Value(event.getValue(), ValueTypes.STRING);
                                if (_this.tags.countTags() == 1) {
                                    _this.getPropertyArray().set(0, value);
                                }
                                else {
                                    _this.getPropertyArray().add(value);
                                }
                                _this.validate(false);
                            });
                            this.tags.onTagRemoved(function (event) {
                                _this.getPropertyArray().remove(event.getIndex());
                                _this.validate(false);
                            });
                            this.setLayoutInProgress(false);
                            return wemQ(null);
                        };
                        Tag.prototype.getNumberOfValids = function () {
                            return this.tags.countTags();
                        };
                        Tag.prototype.giveFocus = function () {
                            return this.tags.giveFocus();
                        };
                        Tag.prototype.onFocus = function (listener) {
                            this.tags.onFocus(listener);
                        };
                        Tag.prototype.unFocus = function (listener) {
                            this.tags.unFocus(listener);
                        };
                        Tag.prototype.onBlur = function (listener) {
                            this.tags.onBlur(listener);
                        };
                        Tag.prototype.unBlur = function (listener) {
                            this.tags.unBlur(listener);
                        };
                        return Tag;
                    })(api.form.inputtype.support.BaseInputTypeManagingAdd);
                    tag.Tag = Tag;
                    api.form.inputtype.InputTypeManager.register(new api.Class("Tag", Tag));
                })(tag = inputtype.tag || (inputtype.tag = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var content;
    (function (content) {
        var form;
        (function (form) {
            var inputtype;
            (function (inputtype) {
                var geo;
                (function (geo) {
                    var support = api.form.inputtype.support;
                    var ValueTypes = api.data.ValueTypes;
                    var GeoPoint = (function (_super) {
                        __extends(GeoPoint, _super);
                        function GeoPoint(config) {
                            _super.call(this, config);
                        }
                        GeoPoint.prototype.getValueType = function () {
                            return ValueTypes.GEO_POINT;
                        };
                        GeoPoint.prototype.newInitialValue = function () {
                            return ValueTypes.GEO_POINT.newNullValue();
                        };
                        GeoPoint.prototype.createInputOccurrenceElement = function (index, property) {
                            var _this = this;
                            var geoPoint = new api.ui.geo.GeoPoint();
                            if (property.hasNonNullValue()) {
                                var geoPointValue = property.getGeoPoint();
                                if (geoPointValue) {
                                    geoPoint.setGeoPoint(geoPointValue);
                                }
                            }
                            geoPoint.onValueChanged(function (event) {
                                var value = api.util.GeoPoint.isValidString(event.getNewValue()) ? api.util.GeoPoint.fromString(event.getNewValue()) : null;
                                _this.onValueChanged(property, value, ValueTypes.GEO_POINT);
                            });
                            return geoPoint;
                        };
                        GeoPoint.prototype.availableSizeChanged = function () {
                        };
                        GeoPoint.prototype.valueBreaksRequiredContract = function (value) {
                            return value.isNull() || !value.getType().equals(ValueTypes.GEO_POINT);
                        };
                        GeoPoint.prototype.hasInputElementValidUserInput = function (inputElement) {
                            var geoPoint = inputElement;
                            return geoPoint.hasValidUserInput();
                        };
                        return GeoPoint;
                    })(support.BaseInputTypeNotManagingAdd);
                    geo.GeoPoint = GeoPoint;
                    api.form.inputtype.InputTypeManager.register(new api.Class("GeoPoint", GeoPoint));
                })(geo = inputtype.geo || (inputtype.geo = {}));
            })(inputtype = form.inputtype || (form.inputtype = {}));
        })(form = content.form || (content.form = {}));
    })(content = api.content || (api.content = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var Aggregation = (function () {
            function Aggregation(name) {
                this.name = name;
            }
            Aggregation.prototype.getName = function () {
                return this.name;
            };
            Aggregation.fromJsonArray = function (aggregationWrapperJsons) {
                var aggregations = [];
                aggregationWrapperJsons.forEach(function (aggregationJson) {
                    aggregations.push(aggregation.AggregationFactory.createFromJson(aggregationJson));
                });
                return aggregations;
            };
            return Aggregation;
        })();
        aggregation.Aggregation = Aggregation;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (_aggregation) {
        var AggregationView = (function (_super) {
            __extends(AggregationView, _super);
            function AggregationView(aggregation, parentGroupView) {
                _super.call(this, 'aggregation-view');
                this.bucketSelectionChangedListeners = [];
                this.displayNameMap = [];
                this.aggregation = aggregation;
                this.parentGroupView = parentGroupView;
            }
            AggregationView.prototype.setDisplayNamesMap = function (displayNameMap) {
                this.displayNameMap = displayNameMap;
                this.setDisplayNames();
            };
            AggregationView.prototype.setDisplayNames = function () {
                throw new Error("Must be implemented by inheritor");
            };
            AggregationView.prototype.getDisplayNameForName = function (name) {
                return this.displayNameMap[name];
            };
            AggregationView.prototype.getAggregation = function () {
                return this.aggregation;
            };
            AggregationView.prototype.getParentGroupView = function () {
                return this.parentGroupView;
            };
            AggregationView.prototype.getName = function () {
                return this.aggregation.getName();
            };
            AggregationView.prototype.deselectFacet = function (supressEvent) {
                throw new Error("Must be implemented by inheritor");
            };
            AggregationView.prototype.hasSelectedEntry = function () {
                throw new Error("Must be implemented by inheritor");
            };
            AggregationView.prototype.getSelectedValues = function () {
                throw new Error("Must be implemented by inheritor");
            };
            AggregationView.prototype.update = function (aggregation) {
                throw new Error("Must be implemented by inheritor");
            };
            AggregationView.prototype.onBucketViewSelectionChanged = function (listener) {
                this.bucketSelectionChangedListeners.push(listener);
            };
            AggregationView.prototype.unBucketViewSelectionChanged = function (listener) {
                this.bucketSelectionChangedListeners = this.bucketSelectionChangedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            AggregationView.prototype.notifyBucketViewSelectionChanged = function (event) {
                this.bucketSelectionChangedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            AggregationView.createAggregationView = function (aggregation, parentGroupView) {
                if (api.ObjectHelper.iFrameSafeInstanceOf(aggregation, api.aggregation.BucketAggregation)) {
                    return new api.aggregation.BucketAggregationView(aggregation, parentGroupView);
                }
                else {
                    throw Error("Creating AggregationView of this type of Aggregation is not supported: " + aggregation);
                }
            };
            return AggregationView;
        })(api.dom.DivEl);
        _aggregation.AggregationView = AggregationView;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var BucketAggregation = (function (_super) {
            __extends(BucketAggregation, _super);
            function BucketAggregation(name) {
                _super.call(this, name);
                this.buckets = [];
            }
            BucketAggregation.prototype.getBucketByName = function (name) {
                for (var i = 0; i < this.buckets.length; i++) {
                    var bucket = this.buckets[i];
                    if (bucket.getKey() == name) {
                        return bucket;
                    }
                }
                return null;
            };
            BucketAggregation.prototype.getBuckets = function () {
                return this.buckets;
            };
            BucketAggregation.prototype.addBucket = function (bucket) {
                this.buckets.push(bucket);
            };
            BucketAggregation.fromJson = function (json) {
                var bucketAggregation = new BucketAggregation(json.name);
                json.buckets.forEach(function (bucketWrapper) {
                    bucketAggregation.addBucket(api.aggregation.BucketFactory.createFromJson(bucketWrapper));
                });
                return bucketAggregation;
            };
            return BucketAggregation;
        })(aggregation.Aggregation);
        aggregation.BucketAggregation = BucketAggregation;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (_aggregation) {
        var BucketAggregationView = (function (_super) {
            __extends(BucketAggregationView, _super);
            function BucketAggregationView(bucketAggregation, parentGroupView) {
                var _this = this;
                _super.call(this, bucketAggregation, parentGroupView);
                this.bucketViews = [];
                this.bucketAggregation = bucketAggregation;
                this.showBucketView = false;
                this.bucketAggregation.getBuckets().forEach(function (bucket) {
                    _this.addBucket(new api.aggregation.BucketView(bucket, _this, false, _this.getDisplayNameForName(bucket.getKey())));
                    if (bucket.getDocCount() > 0) {
                        _this.showBucketView = true;
                    }
                });
                if (!this.showBucketView) {
                    this.hide();
                }
            }
            BucketAggregationView.prototype.setDisplayNames = function () {
                var _this = this;
                this.bucketViews.forEach(function (bucketView) {
                    bucketView.setDisplayName(_this.getDisplayNameForName(bucketView.getName()));
                });
            };
            BucketAggregationView.prototype.hasSelectedEntry = function () {
                var isSelected = false;
                this.bucketViews.forEach(function (bucketView) {
                    if (bucketView.isSelected()) {
                        isSelected = true;
                    }
                });
                return isSelected;
            };
            BucketAggregationView.prototype.addBucket = function (bucketView) {
                var _this = this;
                this.appendChild(bucketView);
                bucketView.onSelectionChanged(function (event) {
                    _this.notifyBucketViewSelectionChanged(event);
                });
                this.bucketViews.push(bucketView);
            };
            BucketAggregationView.prototype.getSelectedValues = function () {
                var selectedBuckets = [];
                this.bucketViews.forEach(function (bucketView) {
                    if (bucketView.isSelected()) {
                        selectedBuckets.push(bucketView.getBucket());
                    }
                });
                return selectedBuckets;
            };
            BucketAggregationView.prototype.deselectFacet = function (supressEvent) {
                this.bucketViews.forEach(function (bucketView) {
                    bucketView.deselect(supressEvent);
                });
            };
            BucketAggregationView.prototype.update = function (aggregation) {
                var _this = this;
                var selectedBucketNames = this.getSelectedBucketNames();
                this.bucketAggregation = aggregation;
                this.bucketViews = [];
                this.removeChildren();
                var anyBucketVisible = false;
                this.bucketAggregation.getBuckets().forEach(function (bucket) {
                    var wasSelected = (wemjq.inArray(bucket.getKey(), selectedBucketNames)) > -1;
                    var bucketView = new api.aggregation.BucketView(bucket, _this, wasSelected, _this.getDisplayNameForName(bucket.getKey()));
                    _this.addBucket(bucketView);
                    if (bucket.getDocCount() > 0 || wasSelected) {
                        anyBucketVisible = true;
                    }
                });
                this.showBucketView = anyBucketVisible;
                if (!this.showBucketView) {
                    this.hide();
                }
                else if (!this.isVisible()) {
                    this.show();
                }
            };
            BucketAggregationView.prototype.getSelectedBucketNames = function () {
                var selectedBucketNames = [];
                var selectedBuckets = this.getSelectedValues();
                selectedBuckets.forEach(function (bucket) {
                    selectedBucketNames.push(bucket.getKey());
                });
                return selectedBucketNames;
            };
            return BucketAggregationView;
        })(api.aggregation.AggregationView);
        _aggregation.BucketAggregationView = BucketAggregationView;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var Bucket = (function () {
            function Bucket(key, docCount) {
                this.key = key;
                this.docCount = docCount;
            }
            Bucket.prototype.getKey = function () {
                return this.key;
            };
            Bucket.prototype.getDocCount = function () {
                return this.docCount;
            };
            Bucket.prototype.setKey = function (key) {
                this.key = key;
            };
            Bucket.prototype.setDocCount = function (docCount) {
                this.docCount = docCount;
            };
            Bucket.fromJson = function (json) {
                return new Bucket(json.key, json.docCount);
            };
            Bucket.prototype.getSelectionValue = function () {
                return "test";
            };
            return Bucket;
        })();
        aggregation.Bucket = Bucket;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var BucketFactory = (function () {
            function BucketFactory() {
            }
            BucketFactory.createFromJson = function (json) {
                if (json.DateRangeBucket) {
                    return aggregation.DateRangeBucket.fromDateRangeJson(json.DateRangeBucket);
                }
                else if (json.BucketJson) {
                    return aggregation.Bucket.fromJson(json.BucketJson);
                }
                else {
                    throw new Error("Bucket-type not recognized: " + json);
                }
            };
            return BucketFactory;
        })();
        aggregation.BucketFactory = BucketFactory;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var BucketView = (function (_super) {
            __extends(BucketView, _super);
            function BucketView(bucket, parentAggregationView, select, displayName) {
                var _this = this;
                _super.call(this, 'aggregation-bucket-view');
                this.selectionChangedListeners = [];
                this.bucket = bucket;
                this.parentAggregationView = parentAggregationView;
                this.displayName = displayName;
                this.checkbox = new api.ui.Checkbox(this.resolveLabelValue());
                if (select) {
                    this.checkbox.setChecked(true, true);
                }
                this.checkbox.onValueChanged(function (event) {
                    _this.notifySelectionChanged(eval(event.getOldValue()), eval(event.getNewValue()));
                });
                this.appendChild(this.checkbox);
                this.updateUI();
            }
            BucketView.prototype.resolveLabelValue = function () {
                if (this.displayName != null) {
                    return this.displayName + ' (' + this.bucket.getDocCount() + ')';
                }
                return this.bucket.getKey() + ' (' + this.bucket.getDocCount() + ')';
            };
            BucketView.prototype.setDisplayName = function (displayName) {
                this.displayName = displayName;
                this.updateLabel();
            };
            BucketView.prototype.updateLabel = function () {
                this.checkbox.setLabel(this.resolveLabelValue());
            };
            BucketView.prototype.getBucket = function () {
                return this.bucket;
            };
            BucketView.prototype.getName = function () {
                return this.bucket.getKey();
            };
            BucketView.prototype.update = function (bucket) {
                this.bucket = bucket;
                this.updateUI();
            };
            BucketView.prototype.isSelected = function () {
                return this.checkbox.isChecked();
            };
            BucketView.prototype.deselect = function (supressEvent) {
                this.checkbox.setChecked(false, supressEvent);
            };
            BucketView.prototype.updateUI = function () {
                this.updateLabel();
                if (this.bucket.getDocCount() > 0 || this.isSelected()) {
                    this.show();
                }
                else {
                    this.hide();
                }
            };
            BucketView.prototype.getParentAggregationView = function () {
                return this.parentAggregationView;
            };
            BucketView.prototype.notifySelectionChanged = function (oldValue, newValue) {
                var _this = this;
                this.selectionChangedListeners.forEach(function (listener) {
                    listener(new api.aggregation.BucketViewSelectionChangedEvent(oldValue, newValue, _this));
                });
            };
            BucketView.prototype.unSelectionChanged = function (listener) {
                this.selectionChangedListeners = this.selectionChangedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            BucketView.prototype.onSelectionChanged = function (listener) {
                this.selectionChangedListeners.push(listener);
            };
            return BucketView;
        })(api.dom.DivEl);
        aggregation.BucketView = BucketView;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var DateRangeBucket = (function (_super) {
            __extends(DateRangeBucket, _super);
            function DateRangeBucket(key, docCount) {
                _super.call(this, key, docCount);
            }
            DateRangeBucket.prototype.getFrom = function () {
                return this.from;
            };
            DateRangeBucket.prototype.getTo = function () {
                return this.to;
            };
            DateRangeBucket.fromDateRangeJson = function (json) {
                var dateRangeBucket = new api.aggregation.DateRangeBucket(json.key, json.docCount);
                dateRangeBucket.from = json.from;
                dateRangeBucket.to = json.to;
                return dateRangeBucket;
            };
            return DateRangeBucket;
        })(aggregation.Bucket);
        aggregation.DateRangeBucket = DateRangeBucket;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var AggregationContainer = (function (_super) {
            __extends(AggregationContainer, _super);
            function AggregationContainer() {
                _super.call(this);
                this.aggregationGroupViews = [];
            }
            AggregationContainer.prototype.addAggregationGroupView = function (aggregationGroupView) {
                var _this = this;
                this.appendChild(aggregationGroupView);
                aggregationGroupView.onBucketViewSelectionChanged(function (event) {
                    if (event.getNewValue()) {
                        _this.lastSelectedGroupView = event.getBucketView().getParentAggregationView().getParentGroupView();
                    }
                });
                this.aggregationGroupViews.push(aggregationGroupView);
            };
            AggregationContainer.prototype.deselectAll = function (supressEvent) {
                this.aggregationGroupViews.forEach(function (aggregationGroupView) {
                    aggregationGroupView.deselectGroup(supressEvent);
                });
                this.lastSelectedGroupView = null;
            };
            AggregationContainer.prototype.hasSelectedBuckets = function () {
                var hasSelected = false;
                this.aggregationGroupViews.forEach(function (aggregationGroupView) {
                    if (aggregationGroupView.hasSelections()) {
                        hasSelected = true;
                    }
                });
                return hasSelected;
            };
            AggregationContainer.prototype.updateAggregations = function (aggregations, doUpdateAll) {
                var _this = this;
                this.aggregationGroupViews.forEach(function (aggregationGroupView) {
                    var matchingAggregations = aggregations.filter(function (current) {
                        return aggregationGroupView.handlesAggregation(current);
                    });
                    if (doUpdateAll || _this.isGroupUpdatable(aggregationGroupView)) {
                        aggregationGroupView.update(matchingAggregations);
                    }
                });
            };
            AggregationContainer.prototype.isGroupUpdatable = function (aggregationGroupView) {
                return aggregationGroupView != this.lastSelectedGroupView;
            };
            AggregationContainer.prototype.getSelectedValuesByAggregationName = function () {
                var aggregationSelections = [];
                this.aggregationGroupViews.forEach(function (aggregationGroupView) {
                    var selectedValuesByAggregationName = aggregationGroupView.getSelectedValuesByAggregationName();
                    aggregationSelections = aggregationSelections.concat(selectedValuesByAggregationName);
                });
                return aggregationSelections;
            };
            return AggregationContainer;
        })(api.dom.DivEl);
        aggregation.AggregationContainer = AggregationContainer;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var AggregationFactory = (function () {
            function AggregationFactory() {
            }
            AggregationFactory.createFromJson = function (json) {
                if (json.BucketAggregation) {
                    return api.aggregation.BucketAggregation.fromJson(json.BucketAggregation);
                }
                else {
                    throw new Error("Aggregation type not recognized: " + json);
                }
            };
            return AggregationFactory;
        })();
        aggregation.AggregationFactory = AggregationFactory;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (_aggregation) {
        var AggregationGroupView = (function (_super) {
            __extends(AggregationGroupView, _super);
            function AggregationGroupView(name, displayName) {
                _super.call(this, "aggregation-group-view");
                this.aggregationViews = [];
                this.titleEl = new api.dom.H2El();
                this.bucketSelectionChangedListeners = [];
                this.name = name;
                this.displayName = displayName;
                this.titleEl.getEl().setInnerHtml(this.displayName);
                this.appendChild(this.titleEl);
            }
            AggregationGroupView.prototype.addAggregationView = function (aggregationView) {
                var _this = this;
                this.appendChild(aggregationView);
                aggregationView.onBucketViewSelectionChanged(function (event) {
                    _this.notifyBucketViewSelectionChanged(event);
                });
                this.aggregationViews.push(aggregationView);
            };
            AggregationGroupView.prototype.initialize = function () {
            };
            AggregationGroupView.prototype.getAggregationViews = function () {
                return this.aggregationViews;
            };
            AggregationGroupView.prototype.getName = function () {
                return this.name;
            };
            AggregationGroupView.prototype.handlesAggregation = function (aggregation) {
                return aggregation.getName() == this.name;
            };
            AggregationGroupView.prototype.getSelectedValuesByAggregationName = function () {
                var aggregationSelections = [];
                this.aggregationViews.forEach(function (bucketAggregationView) {
                    var selectedBuckets = bucketAggregationView.getSelectedValues();
                    if (selectedBuckets != null) {
                        var aggregationSelection = new api.aggregation.AggregationSelection(bucketAggregationView.getName());
                        aggregationSelection.setValues(selectedBuckets);
                        aggregationSelections.push(aggregationSelection);
                    }
                });
                return aggregationSelections;
            };
            AggregationGroupView.prototype.hasSelections = function () {
                var hasSelections = false;
                for (var i = 0; i < this.aggregationViews.length; i++) {
                    if (this.aggregationViews[i].hasSelectedEntry()) {
                        hasSelections = true;
                        break;
                    }
                }
                return hasSelections;
            };
            AggregationGroupView.prototype.deselectGroup = function (supressEvent) {
                this.aggregationViews.forEach(function (aggregationView) {
                    aggregationView.deselectFacet(supressEvent);
                });
            };
            AggregationGroupView.prototype.onBucketViewSelectionChanged = function (listener) {
                this.bucketSelectionChangedListeners.push(listener);
            };
            AggregationGroupView.prototype.unBucketViewSelectionChanged = function (listener) {
                this.bucketSelectionChangedListeners = this.bucketSelectionChangedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            AggregationGroupView.prototype.notifyBucketViewSelectionChanged = function (event) {
                this.bucketSelectionChangedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            AggregationGroupView.prototype.update = function (aggregations) {
                var _this = this;
                aggregations.forEach(function (aggregation) {
                    var existingAggregationView = _this.getAggregationView(aggregation.getName());
                    if (existingAggregationView == null) {
                        _this.addAggregationView(api.aggregation.AggregationView.createAggregationView(aggregation, _this));
                    }
                    else {
                        if (api.ObjectHelper.iFrameSafeInstanceOf(existingAggregationView, api.aggregation.BucketAggregationView)) {
                            var bucketAggregationView = existingAggregationView;
                            bucketAggregationView.update(aggregation);
                        }
                    }
                });
            };
            AggregationGroupView.prototype.getAggregationView = function (name) {
                for (var i = 0; i < this.aggregationViews.length; i++) {
                    var aggregationView = this.aggregationViews[i];
                    if (aggregationView.getName() == name) {
                        return aggregationView;
                    }
                }
                return null;
            };
            return AggregationGroupView;
        })(api.dom.DivEl);
        _aggregation.AggregationGroupView = AggregationGroupView;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var ContentTypeAggregationGroupView = (function (_super) {
            __extends(ContentTypeAggregationGroupView, _super);
            function ContentTypeAggregationGroupView() {
                _super.apply(this, arguments);
            }
            ContentTypeAggregationGroupView.prototype.initialize = function () {
                var _this = this;
                var displayNameMap = [];
                var mask = new api.ui.mask.LoadMask(this);
                this.appendChild(mask);
                this.onRendered(function (event) {
                    mask.show();
                });
                var request = new api.schema.content.GetAllContentTypesRequest();
                request.sendAndParse().done(function (contentTypes) {
                    contentTypes.forEach(function (contentType) {
                        displayNameMap[contentType.getName()] = contentType.getDisplayName();
                    });
                    _this.getAggregationViews().forEach(function (aggregationView) {
                        aggregationView.setDisplayNamesMap(displayNameMap);
                    });
                    mask.remove();
                });
            };
            return ContentTypeAggregationGroupView;
        })(aggregation.AggregationGroupView);
        aggregation.ContentTypeAggregationGroupView = ContentTypeAggregationGroupView;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var AggregationEntryViewSelectionChangedEvent = (function () {
            function AggregationEntryViewSelectionChangedEvent(oldValue, newValue, bucketView) {
                this.oldValue = oldValue;
                this.newValue = newValue;
                this.bucketView = bucketView;
            }
            AggregationEntryViewSelectionChangedEvent.prototype.getOldValue = function () {
                return this.oldValue;
            };
            AggregationEntryViewSelectionChangedEvent.prototype.getNewValue = function () {
                return this.newValue;
            };
            AggregationEntryViewSelectionChangedEvent.prototype.getBucketView = function () {
                return this.bucketView;
            };
            return AggregationEntryViewSelectionChangedEvent;
        })();
        aggregation.AggregationEntryViewSelectionChangedEvent = AggregationEntryViewSelectionChangedEvent;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var BucketViewSelectionChangedEvent = (function () {
            function BucketViewSelectionChangedEvent(oldValue, newValue, bucketView) {
                this.oldValue = oldValue;
                this.newValue = newValue;
                this.bucketView = bucketView;
            }
            BucketViewSelectionChangedEvent.prototype.getOldValue = function () {
                return this.oldValue;
            };
            BucketViewSelectionChangedEvent.prototype.getNewValue = function () {
                return this.newValue;
            };
            BucketViewSelectionChangedEvent.prototype.getBucketView = function () {
                return this.bucketView;
            };
            return BucketViewSelectionChangedEvent;
        })();
        aggregation.BucketViewSelectionChangedEvent = BucketViewSelectionChangedEvent;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var AggregationSelection = (function () {
            function AggregationSelection(name) {
                this.name = name;
            }
            AggregationSelection.prototype.setValues = function (selectedBuckets) {
                this.selectedBuckets = selectedBuckets;
            };
            AggregationSelection.prototype.getName = function () {
                return this.name;
            };
            AggregationSelection.prototype.getSelectedBuckets = function () {
                return this.selectedBuckets;
            };
            return AggregationSelection;
        })();
        aggregation.AggregationSelection = AggregationSelection;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var aggregation;
    (function (aggregation) {
        var PrincipalAggregationGroupView = (function (_super) {
            __extends(PrincipalAggregationGroupView, _super);
            function PrincipalAggregationGroupView() {
                _super.apply(this, arguments);
            }
            PrincipalAggregationGroupView.prototype.initialize = function () {
                var displayNameMap = [];
                var mask = new api.ui.mask.LoadMask(this);
                this.appendChild(mask);
                this.onRendered(function (event) {
                    mask.show();
                });
                mask.remove();
            };
            return PrincipalAggregationGroupView;
        })(aggregation.AggregationGroupView);
        aggregation.PrincipalAggregationGroupView = PrincipalAggregationGroupView;
    })(aggregation = api.aggregation || (api.aggregation = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rendering;
    (function (rendering) {
        (function (RenderingMode) {
            RenderingMode[RenderingMode["EDIT"] = 0] = "EDIT";
            RenderingMode[RenderingMode["PREVIEW"] = 1] = "PREVIEW";
            RenderingMode[RenderingMode["LIVE"] = 2] = "LIVE";
        })(rendering.RenderingMode || (rendering.RenderingMode = {}));
        var RenderingMode = rendering.RenderingMode;
    })(rendering = api.rendering || (api.rendering = {}));
})(api || (api = {}));
var api;
(function (api) {
    var rendering;
    (function (rendering) {
        var UriHelper = (function () {
            function UriHelper() {
            }
            UriHelper.getPortalUri = function (path, renderingMode, workspace) {
                path = api.util.UriHelper.relativePath(path);
                var workspaceName;
                switch (workspace) {
                    case 0 /* DRAFT */:
                        workspaceName = "draft";
                        break;
                    case 1 /* MASTER */:
                        workspaceName = "master";
                        break;
                    default:
                        workspaceName = "draft";
                }
                switch (renderingMode) {
                    case 0 /* EDIT */:
                        return api.util.UriHelper.getPortalUri('edit/' + workspaceName + '/' + path);
                    case 2 /* LIVE */:
                        return api.util.UriHelper.getPortalUri('live/' + workspaceName + '/' + path);
                    case 1 /* PREVIEW */:
                        return api.util.UriHelper.getPortalUri('preview/' + workspaceName + '/' + path);
                }
            };
            UriHelper.getComponentUri = function (contentId, componentPath, renderingMode, workspace) {
                return UriHelper.getPortalUri(contentId + "/_/component/" + componentPath, renderingMode, workspace);
            };
            return UriHelper;
        })();
        rendering.UriHelper = UriHelper;
    })(rendering = api.rendering || (api.rendering = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var ShowAppLauncherEvent = (function (_super) {
            __extends(ShowAppLauncherEvent, _super);
            function ShowAppLauncherEvent(application, sessionExpired) {
                _super.call(this);
                this.application = application;
                this.sessionExpired = !!sessionExpired;
            }
            ShowAppLauncherEvent.prototype.getApplication = function () {
                return this.application;
            };
            ShowAppLauncherEvent.prototype.isSessionExpired = function () {
                return this.sessionExpired;
            };
            ShowAppLauncherEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowAppLauncherEvent;
        })(api.event.Event);
        app.ShowAppLauncherEvent = ShowAppLauncherEvent;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var ShowBrowsePanelEvent = (function (_super) {
            __extends(ShowBrowsePanelEvent, _super);
            function ShowBrowsePanelEvent() {
                _super.apply(this, arguments);
            }
            ShowBrowsePanelEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowBrowsePanelEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowBrowsePanelEvent;
        })(api.event.Event);
        app.ShowBrowsePanelEvent = ShowBrowsePanelEvent;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var NamesView = (function (_super) {
            __extends(NamesView, _super);
            function NamesView(addTitleAttribute) {
                if (addTitleAttribute === void 0) { addTitleAttribute = true; }
                _super.call(this, "names-view");
                this.addTitleAttribute = addTitleAttribute;
                this.mainNameEl = new api.dom.H6El("main-name");
                this.appendChild(this.mainNameEl);
                this.subNameEl = new api.dom.PEl("sub-name");
                this.appendChild(this.subNameEl);
            }
            NamesView.prototype.setMainName = function (value) {
                this.mainNameEl.setHtml(value, true);
                if (this.addTitleAttribute) {
                    this.mainNameEl.getEl().setAttribute("title", value);
                }
                return this;
            };
            NamesView.prototype.setSubName = function (value, title) {
                this.subNameEl.setHtml(value, true);
                if (this.addTitleAttribute) {
                    this.subNameEl.getEl().setAttribute("title", title || value);
                }
                return this;
            };
            NamesView.prototype.setSubNameElements = function (elements) {
                var _this = this;
                elements.forEach(function (element) {
                    _this.subNameEl.appendChild(element);
                });
                return this;
            };
            return NamesView;
        })(api.dom.DivEl);
        app.NamesView = NamesView;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        (function (NamesAndIconViewSize) {
            NamesAndIconViewSize[NamesAndIconViewSize["large"] = 0] = "large";
            NamesAndIconViewSize[NamesAndIconViewSize["medium"] = 1] = "medium";
            NamesAndIconViewSize[NamesAndIconViewSize["small"] = 2] = "small";
        })(app.NamesAndIconViewSize || (app.NamesAndIconViewSize = {}));
        var NamesAndIconViewSize = app.NamesAndIconViewSize;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var NamesAndIconViewBuilder = (function () {
            function NamesAndIconViewBuilder() {
                this.addTitleAttribute = false;
                this.appendIcon = true;
            }
            NamesAndIconViewBuilder.prototype.setSize = function (size) {
                this.size = size;
                return this;
            };
            NamesAndIconViewBuilder.prototype.setAddTitleAttribute = function (addTitleAttribute) {
                this.addTitleAttribute = addTitleAttribute;
                return this;
            };
            NamesAndIconViewBuilder.prototype.setAppendIcon = function (appendIcon) {
                this.appendIcon = appendIcon;
                return this;
            };
            NamesAndIconViewBuilder.prototype.build = function () {
                return new NamesAndIconView(this);
            };
            return NamesAndIconViewBuilder;
        })();
        app.NamesAndIconViewBuilder = NamesAndIconViewBuilder;
        var NamesAndIconView = (function (_super) {
            __extends(NamesAndIconView, _super);
            function NamesAndIconView(builder) {
                _super.call(this, "names-and-icon-view");
                var sizeClassName = app.NamesAndIconViewSize[builder.size];
                if (builder.size) {
                    this.addClass(sizeClassName);
                }
                if (builder.appendIcon) {
                    this.wrapperDivEl = new api.dom.DivEl("wrapper");
                    this.appendChild(this.wrapperDivEl);
                    this.iconImageEl = new api.dom.ImgEl(null, "icon");
                    this.wrapperDivEl.appendChild(this.iconImageEl);
                    this.iconDivEl = new api.dom.DivEl("icon");
                    this.wrapperDivEl.appendChild(this.iconDivEl);
                    this.iconDivEl.hide();
                }
                this.namesView = new api.app.NamesView(builder.addTitleAttribute);
                this.appendChild(this.namesView);
                this.iconLabelEl = new api.dom.SpanEl("icon-label");
                this.iconLabelEl.hide();
                this.appendChild(this.iconLabelEl);
            }
            NamesAndIconView.prototype.setMainName = function (value) {
                this.namesView.setMainName(value);
                return this;
            };
            NamesAndIconView.prototype.setSubName = function (value, title) {
                this.namesView.setSubName(value, title);
                return this;
            };
            NamesAndIconView.prototype.setSubNameElements = function (elements) {
                this.namesView.setSubNameElements(elements);
                return this;
            };
            NamesAndIconView.prototype.setIconClass = function (value) {
                this.iconDivEl.setClass("icon " + value);
                this.iconDivEl.getEl().setDisplay('inline-block');
                this.iconImageEl.hide();
                return this;
            };
            NamesAndIconView.prototype.setIconUrl = function (value) {
                this.iconImageEl.setSrc(value);
                this.iconDivEl.hide();
                this.iconImageEl.show();
                return this;
            };
            NamesAndIconView.prototype.setDisplayIconLabel = function (display) {
                if (display) {
                    this.iconLabelEl.show();
                }
                else {
                    this.iconLabelEl.hide();
                }
                return this;
            };
            NamesAndIconView.prototype.getIconImageEl = function () {
                return this.iconImageEl;
            };
            NamesAndIconView.prototype.getWrapperDivEl = function () {
                return this.wrapperDivEl;
            };
            return NamesAndIconView;
        })(api.dom.DivEl);
        app.NamesAndIconView = NamesAndIconView;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        (function (ApplicationShowStatus) {
            ApplicationShowStatus[ApplicationShowStatus["NOT_DISPLAYED"] = 0] = "NOT_DISPLAYED";
            ApplicationShowStatus[ApplicationShowStatus["PREPARING"] = 1] = "PREPARING";
            ApplicationShowStatus[ApplicationShowStatus["DISPLAYED"] = 2] = "DISPLAYED";
        })(app.ApplicationShowStatus || (app.ApplicationShowStatus = {}));
        var ApplicationShowStatus = app.ApplicationShowStatus;
        var Application = (function () {
            function Application(id, name, shortName, icon, appFrame, iconImage) {
                if (appFrame === void 0) { appFrame = null; }
                if (iconImage === void 0) { iconImage = false; }
                this.loadedListeners = [];
                this.id = id;
                this.name = name;
                this.shortName = shortName;
                this.iconUrl = icon;
                this.fullSizeIcon = iconImage;
                this.openTabs = 0;
                this.status = 0 /* NOT_DISPLAYED */;
            }
            Application.getApplication = function () {
                return window.parent['getApplication'] ? window.parent['getApplication'](Application.getAppId()) : null;
            };
            Application.getAppId = function () {
                return window.frameElement ? new api.dom.ElementHelper(window.frameElement).getAttribute("data-wem-app-id") : null;
            };
            Application.prototype.isLoaded = function () {
                return this.loaded && !!this.appFrame && !!this.appFrame.getParentElement();
            };
            Application.prototype.getId = function () {
                return this.id;
            };
            Application.prototype.getName = function () {
                return this.name;
            };
            Application.prototype.getShortName = function () {
                return this.shortName;
            };
            Application.prototype.getIconUrl = function () {
                return this.iconUrl;
            };
            Application.prototype.getAppUrl = function () {
                return api.util.UriHelper.getUri('admin?app=' + this.id);
            };
            Application.prototype.getOpenTabs = function () {
                return this.openTabs;
            };
            Application.prototype.getAppFrame = function () {
                if (!this.appFrame) {
                    this.appFrame = new api.dom.IFrameEl();
                    this.appFrame.getEl().setHeight('100%').setWidth('100%').getHTMLElement().style.border = '0';
                    this.appFrame.setSrc(this.getAppUrl());
                    this.appFrame.getEl().setAttribute('data-wem-app-id', this.id);
                }
                return this.appFrame;
            };
            Application.prototype.getWindow = function () {
                return this.getAppFrame().getHTMLElement()["contentWindow"];
            };
            Application.prototype.isFullSizeIcon = function () {
                return this.fullSizeIcon;
            };
            Application.prototype.hide = function () {
                if (this.appFrame) {
                    this.appFrame.hide();
                }
                this.status = 0 /* NOT_DISPLAYED */;
            };
            Application.prototype.show = function () {
                if (this.appFrame) {
                    this.appFrame.show();
                }
                this.status = 2 /* DISPLAYED */;
            };
            Application.prototype.setOpenTabs = function (value) {
                this.openTabs = value;
                return this;
            };
            Application.prototype.setLoaded = function (value) {
                this.loaded = value;
                this.notifyLoaded();
                return this;
            };
            Application.prototype.setDisplayingStatus = function (status) {
                this.status = status;
            };
            Application.prototype.setFullSizeIcon = function (value) {
                this.fullSizeIcon = value;
                return this;
            };
            Application.prototype.setPath = function (path) {
                this.path = path;
                return this;
            };
            Application.prototype.getPath = function () {
                return this.path;
            };
            Application.prototype.isDisplayed = function () {
                return 2 /* DISPLAYED */ == this.status;
            };
            Application.prototype.isPreparing = function () {
                return this.status == 1 /* PREPARING */;
            };
            Application.prototype.isNotDisplayed = function () {
                return this.status == 0 /* NOT_DISPLAYED */;
            };
            Application.prototype.onLoaded = function (listener) {
                this.loadedListeners.push(listener);
            };
            Application.prototype.unLoaded = function (listener) {
                this.loadedListeners = this.loadedListeners.filter(function (curr) {
                    return curr !== listener;
                });
            };
            Application.prototype.notifyLoaded = function () {
                this.loadedListeners.forEach(function (listener) {
                    listener();
                });
            };
            return Application;
        })();
        app.Application = Application;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        (function (AppLauncherEventType) {
            AppLauncherEventType[AppLauncherEventType["Show"] = 0] = "Show";
        })(app.AppLauncherEventType || (app.AppLauncherEventType = {}));
        var AppLauncherEventType = app.AppLauncherEventType;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var AppPanel = (function (_super) {
            __extends(AppPanel, _super);
            function AppPanel(tabNavigator) {
                _super.call(this, tabNavigator);
            }
            AppPanel.prototype.addBrowsePanel = function (browsePanel) {
                if (!this.browsePanel) {
                    var browseMenuItem = new api.app.bar.AppBarTabMenuItemBuilder().setLabel("<Select>").setTabId(new api.app.bar.AppBarTabId("hidden", "____home")).build();
                    browseMenuItem.setVisibleInMenu(false);
                    this.addNavigablePanel(browseMenuItem, browsePanel, true);
                    this.browsePanel = browsePanel;
                }
            };
            AppPanel.prototype.getBrowsePanel = function () {
                return this.browsePanel;
            };
            AppPanel.prototype.removeNavigablePanel = function (panel, checkCanRemovePanel) {
                if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                var index = _super.prototype.removeNavigablePanel.call(this, panel, checkCanRemovePanel);
                this.checkBrowsePanelNeedsToBeShown(index, panel);
                return index;
            };
            AppPanel.prototype.checkBrowsePanelNeedsToBeShown = function (index, panel) {
                if (panel == this.browsePanel && index > -1) {
                    this.browsePanel = undefined;
                }
                else if (this.getSize() == 0) {
                    new app.ShowBrowsePanelEvent().fire();
                }
            };
            AppPanel.prototype.getNavigator = function () {
                return _super.prototype.getNavigator.call(this);
            };
            return AppPanel;
        })(api.ui.panel.NavigatedDeckPanel);
        app.AppPanel = AppPanel;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var BrowseAndWizardBasedAppPanel = (function (_super) {
            __extends(BrowseAndWizardBasedAppPanel, _super);
            function BrowseAndWizardBasedAppPanel(config) {
                var _this = this;
                _super.call(this, config.appBar.getTabMenu());
                this.appBar = config.appBar;
                this.appBarTabMenu = config.appBar.getTabMenu();
                this.onPanelShown(function (event) {
                    if (event.getPanel() === _this.getBrowsePanel()) {
                        _this.getBrowsePanel().refreshFilter();
                    }
                    var previousActions = _this.resolveActions(event.getPreviousPanel());
                    api.ui.KeyBindings.get().unbindKeys(api.ui.Action.getKeyBindings(previousActions));
                    var nextActions = _this.resolveActions(event.getPanel());
                    _this.currentKeyBindings = api.ui.Action.getKeyBindings(nextActions);
                    api.ui.KeyBindings.get().bindKeys(_this.currentKeyBindings);
                });
            }
            BrowseAndWizardBasedAppPanel.prototype.addBrowsePanel = function (browsePanel) {
                _super.prototype.addBrowsePanel.call(this, browsePanel);
                this.currentKeyBindings = api.ui.Action.getKeyBindings(this.resolveActions(browsePanel));
                this.activateCurrentKeyBindings();
            };
            BrowseAndWizardBasedAppPanel.prototype.activateCurrentKeyBindings = function () {
                if (this.currentKeyBindings) {
                    api.ui.KeyBindings.get().bindKeys(this.currentKeyBindings);
                }
            };
            BrowseAndWizardBasedAppPanel.prototype.getAppBarTabMenu = function () {
                return this.appBarTabMenu;
            };
            BrowseAndWizardBasedAppPanel.prototype.addViewPanel = function (tabMenuItem, viewPanel) {
                var _this = this;
                _super.prototype.addNavigablePanel.call(this, tabMenuItem, viewPanel, true);
                viewPanel.onClosed(function (event) {
                    _this.removeNavigablePanel(event.getView(), false);
                });
            };
            BrowseAndWizardBasedAppPanel.prototype.addWizardPanel = function (tabMenuItem, wizardPanel) {
                var _this = this;
                _super.prototype.addNavigablePanel.call(this, tabMenuItem, wizardPanel, true);
                wizardPanel.onClosed(function (event) {
                    _this.removeNavigablePanel(event.getWizard(), false);
                });
            };
            BrowseAndWizardBasedAppPanel.prototype.canRemovePanel = function (panel) {
                if (api.ObjectHelper.iFrameSafeInstanceOf(panel, api.app.wizard.WizardPanel)) {
                    var wizardPanel = panel;
                    return wizardPanel.canClose();
                }
                return true;
            };
            BrowseAndWizardBasedAppPanel.prototype.resolveActions = function (panel) {
                var actions = [];
                actions = actions.concat(this.appBar.getActions());
                if (api.ObjectHelper.iFrameSafeInstanceOf(panel, api.app.wizard.WizardPanel) || api.ObjectHelper.iFrameSafeInstanceOf(panel, api.app.browse.BrowsePanel) || api.ObjectHelper.iFrameSafeInstanceOf(panel, api.app.view.ItemViewPanel)) {
                    var actionContainer = panel;
                    actions = actions.concat(actionContainer.getActions());
                }
                return actions;
            };
            return BrowseAndWizardBasedAppPanel;
        })(app.AppPanel);
        app.BrowseAndWizardBasedAppPanel = BrowseAndWizardBasedAppPanel;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var AppManager = (function () {
            function AppManager() {
                this.connectionLostListeners = [];
                this.connectionRestoredListeners = [];
                this.showLauncherListeners = [];
                api.app.AppManager._instance = this;
            }
            AppManager.prototype.onConnectionLost = function (listener) {
                this.connectionLostListeners.push(listener);
            };
            AppManager.prototype.onConnectionRestored = function (listener) {
                this.connectionRestoredListeners.push(listener);
            };
            AppManager.prototype.unConnectionLost = function (listener) {
                this.connectionLostListeners = this.connectionLostListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            AppManager.prototype.unConnectionRestored = function (listener) {
                this.connectionRestoredListeners = this.connectionRestoredListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            AppManager.prototype.notifyConnectionLost = function () {
                var _this = this;
                this.connectionLostListeners.forEach(function (listener) {
                    listener.call(_this);
                });
            };
            AppManager.prototype.notifyConnectionRestored = function () {
                var _this = this;
                this.connectionRestoredListeners.forEach(function (listener) {
                    listener.call(_this);
                });
            };
            AppManager.instance = function () {
                if (api.app.AppManager._instance) {
                    return api.app.AppManager._instance;
                }
                else if (window !== window.parent) {
                    var apiAppModule = window.parent.api.app;
                    if (apiAppModule && apiAppModule.AppManager) {
                        var parentAppManager = apiAppModule.AppManager._instance;
                        if (parentAppManager) {
                            api.app.AppManager._instance = parentAppManager;
                        }
                    }
                }
                return api.app.AppManager._instance;
            };
            AppManager._instance = null;
            return AppManager;
        })();
        app.AppManager = AppManager;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var ServerEventsConnection = (function () {
            function ServerEventsConnection(reconnectIntervalSeconds) {
                if (reconnectIntervalSeconds === void 0) { reconnectIntervalSeconds = 5; }
                this.serverEventReceivedListeners = [];
                this.connectionLostListeners = [];
                this.connectionRestoredListeners = [];
                this.connected = false;
                this.keepConnected = false;
                this.ws = null;
                this.reconnectInterval = reconnectIntervalSeconds * 1000;
            }
            ServerEventsConnection.prototype.connect = function () {
                if (!WebSocket) {
                    console.warn('ServerEventsConnection: WebSockets not supported. Server events disabled.');
                    return;
                }
                var wsUrl = api.util.UriHelper.joinPath(this.getWebSocketUriPrefix(), 'admin', 'event');
                this.keepConnected = true;
                this.doConnect(wsUrl);
            };
            ServerEventsConnection.prototype.doConnect = function (wsUrl) {
                var _this = this;
                this.ws = new WebSocket(wsUrl, 'text');
                this.ws.addEventListener('close', function (ev) {
                    clearInterval(_this.keepAliveIntervalId);
                    if (ServerEventsConnection.debug) {
                        var m = 'ServerEventsConnection: connection closed to ' + wsUrl;
                        if (_this.downTime > 0) {
                            m += '\nUptime: ' + (new Date().getTime() - _this.downTime);
                        }
                        console.warn(m);
                        _this.downTime = new Date().getTime();
                    }
                    _this.disconnectTimeoutHandle = setTimeout(function () {
                        if (_this.connected) {
                            if (_this.keepConnected) {
                                _this.notifyConnectionLost();
                            }
                            _this.connected = !_this.connected;
                        }
                    }, _this.reconnectInterval + 1000);
                    if (_this.keepConnected) {
                        setTimeout(function () {
                            if (_this.keepConnected) {
                                _this.doConnect(wsUrl);
                            }
                        }, _this.reconnectInterval);
                    }
                });
                this.ws.addEventListener('error', function (ev) {
                    if (ServerEventsConnection.debug) {
                        console.error('ServerEventsConnection: Unable to connect to server web socket on ' + wsUrl, ev);
                    }
                });
                this.ws.addEventListener('message', function (remoteEvent) {
                    var jsonEvent = JSON.parse(remoteEvent.data);
                    if (ServerEventsConnection.debug) {
                        console.debug('ServerEventsConnection: Server event [' + jsonEvent.type + ']', jsonEvent.event);
                    }
                    _this.handleServerEvent(jsonEvent);
                });
                this.ws.addEventListener('open', function (event) {
                    if (ServerEventsConnection.debug) {
                        var m = 'ServerEventsConnection: connection opened to ' + wsUrl;
                        if (_this.downTime > 0) {
                            m += '\nDowntime: ' + (new Date().getTime() - _this.downTime);
                        }
                        console.log(m);
                        _this.downTime = new Date().getTime();
                    }
                    clearTimeout(_this.disconnectTimeoutHandle);
                    _this.keepAliveIntervalId = setInterval(function () {
                        if (_this.connected) {
                            _this.ws.send("KeepAlive");
                            if (ServerEventsConnection.debug) {
                                console.log('ServerEventsConnection: Sending Keep Alive message');
                            }
                        }
                    }, ServerEventsConnection.KEEP_ALIVE_TIME);
                    if (!_this.connected) {
                        _this.notifyConnectionRestored();
                        _this.connected = !_this.connected;
                    }
                });
            };
            ServerEventsConnection.prototype.disconnect = function () {
                this.keepConnected = false;
                if (this.ws) {
                    this.ws.close();
                }
            };
            ServerEventsConnection.prototype.isConnected = function () {
                return this.ws.readyState === WebSocket.OPEN;
            };
            ServerEventsConnection.prototype.handleServerEvent = function (serverEventJson) {
                var clientEvent = this.translateServerEvent(serverEventJson);
                if (clientEvent) {
                    this.notifyServerEvent(clientEvent);
                }
            };
            ServerEventsConnection.prototype.translateServerEvent = function (serverEventJson) {
                var eventType = serverEventJson.type;
                if (eventType === 'ContentChangeEvent') {
                    return api.content.ContentServerEvent.fromJson(serverEventJson.event);
                }
                if (eventType === 'ApplicationUpdatedEvent') {
                    return api.application.ApplicationUpdatedEvent.fromJson(serverEventJson.event);
                }
                return null;
            };
            ServerEventsConnection.prototype.getWebSocketUriPrefix = function () {
                var loc = window.location, newUri;
                if (loc.protocol === "https:") {
                    newUri = "wss:";
                }
                else {
                    newUri = "ws:";
                }
                newUri += "//" + loc.host;
                return newUri;
            };
            ServerEventsConnection.prototype.notifyServerEvent = function (serverEvent) {
                var _this = this;
                this.serverEventReceivedListeners.forEach(function (listener) {
                    listener.call(_this, serverEvent);
                });
            };
            ServerEventsConnection.prototype.onServerEvent = function (listener) {
                this.serverEventReceivedListeners.push(listener);
            };
            ServerEventsConnection.prototype.unServerEvent = function (listener) {
                this.serverEventReceivedListeners = this.serverEventReceivedListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            ServerEventsConnection.prototype.notifyConnectionLost = function () {
                var _this = this;
                this.connectionLostListeners.forEach(function (listener) {
                    listener.call(_this);
                });
            };
            ServerEventsConnection.prototype.onConnectionLost = function (listener) {
                this.connectionLostListeners.push(listener);
            };
            ServerEventsConnection.prototype.unConnectionLost = function (listener) {
                this.connectionLostListeners = this.connectionLostListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            ServerEventsConnection.prototype.notifyConnectionRestored = function () {
                var _this = this;
                this.connectionRestoredListeners.forEach(function (listener) {
                    listener.call(_this);
                });
            };
            ServerEventsConnection.prototype.onConnectionRestored = function (listener) {
                this.connectionRestoredListeners.push(listener);
            };
            ServerEventsConnection.prototype.unConnectionRestored = function (listener) {
                this.connectionRestoredListeners = this.connectionRestoredListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            ServerEventsConnection.KEEP_ALIVE_TIME = 30 * 1000;
            ServerEventsConnection.debug = false;
            return ServerEventsConnection;
        })();
        app.ServerEventsConnection = ServerEventsConnection;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var ServerEventsListener = (function () {
            function ServerEventsListener(applications) {
                var _this = this;
                this.applications = applications;
                this.serverEventsConnection = new api.app.ServerEventsConnection();
                this.serverEventsConnection.onServerEvent(function (event) { return _this.onServerEvent(event); });
            }
            ServerEventsListener.prototype.start = function () {
                this.serverEventsConnection.connect();
            };
            ServerEventsListener.prototype.stop = function () {
                this.serverEventsConnection.disconnect();
            };
            ServerEventsListener.prototype.onConnectionLost = function (listener) {
                this.serverEventsConnection.onConnectionLost(listener);
            };
            ServerEventsListener.prototype.unConnectionLost = function (listener) {
                this.serverEventsConnection.unConnectionLost(listener);
            };
            ServerEventsListener.prototype.onConnectionRestored = function (listener) {
                this.serverEventsConnection.onConnectionRestored(listener);
            };
            ServerEventsListener.prototype.unConnectionRestored = function (listener) {
                this.serverEventsConnection.unConnectionRestored(listener);
            };
            ServerEventsListener.prototype.onServerEvent = function (event) {
                this.applications.forEach(function (application) {
                    var appWindow = application.getWindow();
                    if (appWindow) {
                        event.fire(appWindow);
                    }
                });
            };
            return ServerEventsListener;
        })();
        app.ServerEventsListener = ServerEventsListener;
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var bar;
        (function (bar) {
            var ShowAppLauncherAction = (function (_super) {
                __extends(ShowAppLauncherAction, _super);
                function ShowAppLauncherAction(application) {
                    _super.call(this, 'Start', 'mod+esc', true);
                    this.onExecuted(function () {
                        new app.ShowAppLauncherEvent(application).fire(window.parent);
                        new app.ShowAppLauncherEvent(application).fire();
                    });
                }
                return ShowAppLauncherAction;
            })(api.ui.Action);
            bar.ShowAppLauncherAction = ShowAppLauncherAction;
        })(bar = app.bar || (app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var bar;
        (function (bar) {
            var ShowBrowsePanelAction = (function (_super) {
                __extends(ShowBrowsePanelAction, _super);
                function ShowBrowsePanelAction() {
                    _super.call(this, 'Browse');
                    this.onExecuted(function () {
                        new api.app.ShowBrowsePanelEvent().fire();
                    });
                }
                return ShowBrowsePanelAction;
            })(api.ui.Action);
            bar.ShowBrowsePanelAction = ShowBrowsePanelAction;
        })(bar = app.bar || (app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var bar;
        (function (bar) {
            var AppBarActions = (function () {
                function AppBarActions() {
                }
                AppBarActions.SHOW_BROWSE_PANEL = new bar.ShowBrowsePanelAction();
                return AppBarActions;
            })();
            bar.AppBarActions = AppBarActions;
        })(bar = app.bar || (app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (_app) {
        var bar;
        (function (bar) {
            var AppBar = (function (_super) {
                __extends(AppBar, _super);
                function AppBar(application) {
                    var _this = this;
                    _super.call(this, "appbar");
                    this.application = application;
                    this.tabMenu = new bar.AppBarTabMenu();
                    this.showAppLauncherAction = new bar.ShowAppLauncherAction(this.application);
                    this.launcherButton = new LauncherButton(this.showAppLauncherAction);
                    this.appendChild(this.launcherButton);
                    this.homeButton = new HomeButton(this.application, bar.AppBarActions.SHOW_BROWSE_PANEL);
                    this.appendChild(this.homeButton);
                    this.appendChild(this.tabMenu);
                    this.tabMenu.onNavigationItemAdded(function (event) {
                        _this.updateAppOpenTabs();
                    });
                    this.tabMenu.onNavigationItemRemoved(function (event) {
                        _this.updateAppOpenTabs();
                    });
                    api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                        if (_this.tabMenu.countVisible() > 0) {
                            _this.addClass("tabs-present");
                        }
                        else {
                            _this.removeClass("tabs-present");
                        }
                    });
                    this.onRendered(function () {
                        api.ui.responsive.ResponsiveManager.fireResizeEvent();
                    });
                }
                AppBar.prototype.getActions = function () {
                    return [this.showAppLauncherAction];
                };
                AppBar.prototype.getTabMenu = function () {
                    return this.tabMenu;
                };
                AppBar.prototype.updateAppOpenTabs = function () {
                    this.application.setOpenTabs(this.tabMenu.countVisible());
                };
                return AppBar;
            })(api.dom.DivEl);
            bar.AppBar = AppBar;
            var LauncherButton = (function (_super) {
                __extends(LauncherButton, _super);
                function LauncherButton(action) {
                    _super.call(this, action, false);
                    this.addClass('launcher-button');
                }
                return LauncherButton;
            })(api.ui.button.ActionButton);
            bar.LauncherButton = LauncherButton;
            var HomeButton = (function (_super) {
                __extends(HomeButton, _super);
                function HomeButton(app, action) {
                    _super.call(this, app.getName());
                    this.addClass("home-button app-icon icon-" + app.getIconUrl());
                    this.onClicked(function (event) {
                        action.execute();
                    });
                }
                return HomeButton;
            })(api.ui.button.Button);
            bar.HomeButton = HomeButton;
        })(bar = _app.bar || (_app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var bar;
        (function (bar) {
            var AppBarTabId = (function () {
                function AppBarTabId(mode, id) {
                    this.mode = mode;
                    this.id = id;
                }
                AppBarTabId.forNew = function (id) {
                    return new AppBarTabId("new", id);
                };
                AppBarTabId.forEdit = function (id) {
                    return new AppBarTabId("edit", id);
                };
                AppBarTabId.forView = function (id) {
                    return new AppBarTabId("view", id);
                };
                AppBarTabId.prototype.getId = function () {
                    return this.id;
                };
                AppBarTabId.prototype.changeToEditMode = function (id) {
                    this.mode = "edit";
                    this.id = id;
                };
                AppBarTabId.prototype.equals = function (other) {
                    return other.id == this.id && other.mode == this.mode;
                };
                AppBarTabId.prototype.toString = function () {
                    return this.mode + ":" + this.id;
                };
                return AppBarTabId;
            })();
            bar.AppBarTabId = AppBarTabId;
        })(bar = app.bar || (app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var bar;
        (function (bar) {
            var ResponsiveManager = api.ui.responsive.ResponsiveManager;
            var AppBarTabMenu = (function (_super) {
                __extends(AppBarTabMenu, _super);
                function AppBarTabMenu() {
                    var _this = this;
                    _super.call(this, "appbar-tab-menu");
                    this.buttonLabelChangedListeners = [];
                    this.barEl = new api.dom.UlEl("bar");
                    this.prependChild(this.barEl);
                    this.onRendered(function () {
                        _this.updateTabMenuButtonVisibility();
                    });
                    ResponsiveManager.onAvailableSizeChanged(this, this.moveTabs.bind(this, 50));
                }
                AppBarTabMenu.prototype.updateTabMenuButtonVisibility = function () {
                    var menuTabsCount = this.getMenuEl().getChildren().length;
                    if (menuTabsCount === 0) {
                        this.appBarTabMenuButton.hide();
                        this.getMenuEl().hide();
                    }
                    else {
                        this.appBarTabMenuButton.show();
                    }
                    this.appBarTabMenuButton.setTabCount(menuTabsCount);
                };
                AppBarTabMenu.prototype.moveTabs = function (timeout) {
                    if (timeout === void 0) { timeout = 0; }
                    clearInterval(this.timeoutHandler);
                    if (timeout > 0) {
                        this.timeoutHandler = setTimeout(this.moveTabsHandler.bind(this), timeout);
                    }
                    else {
                        this.moveTabsHandler();
                    }
                };
                AppBarTabMenu.prototype.moveTabsHandler = function () {
                    var width = this.getEl().getWidth(), barWidth = this.barEl.getEl().getWidth(), exactTabs = AppBarTabMenu.MAX_WIDTH < width ? Math.ceil(barWidth / AppBarTabMenu.TAB_WIDTH) || 1 : 1, barTabs = this.barEl.getChildren(), menuTabs = this.getMenuEl().getChildren(), tabsInBar = barTabs.length, tabsInMenu = menuTabs.length;
                    while (!(exactTabs === tabsInBar || (exactTabs > tabsInBar && tabsInMenu === 0))) {
                        if (exactTabs > tabsInBar) {
                            if (tabsInMenu > 0) {
                                var tabEl = this.getMenuEl().getFirstChild();
                                this.getMenuEl().removeChild(tabEl);
                                this.barEl.appendChild(tabEl);
                                tabsInBar++;
                                tabsInMenu--;
                            }
                        }
                        else if (exactTabs < tabsInBar) {
                            if (tabsInBar > 0) {
                                var tabEl = this.barEl.getLastChild();
                                this.barEl.removeChild(tabEl);
                                this.getMenuEl().prependChild(tabEl);
                                tabsInBar--;
                                tabsInMenu++;
                            }
                        }
                    }
                    this.updateTabMenuButtonVisibility();
                };
                AppBarTabMenu.prototype.createTabMenuButton = function () {
                    this.appBarTabMenuButton = new bar.AppBarTabMenuButton();
                    return this.appBarTabMenuButton;
                };
                AppBarTabMenu.prototype.setButtonLabel = function (value) {
                    _super.prototype.setButtonLabel.call(this, value);
                    this.notifyButtonLabelChanged();
                    return this;
                };
                AppBarTabMenu.prototype.addNavigationItem = function (tab) {
                    _super.prototype.addNavigationItem.call(this, tab);
                    this.appBarTabMenuButton.setTabCount(this.countVisible());
                    this.appBarTabMenuButton.setEditing(tab.isEditing());
                    this.moveTabs(0);
                    this.makeTabFirst(tab);
                    ResponsiveManager.fireResizeEvent();
                };
                AppBarTabMenu.prototype.removeNavigationItem = function (tab) {
                    _super.prototype.removeNavigationItem.call(this, tab);
                    this.appBarTabMenuButton.setTabCount(this.countVisible());
                    var newSelectedTab = this.getSelectedNavigationItem();
                    if (newSelectedTab) {
                        this.appBarTabMenuButton.setEditing(newSelectedTab.isEditing());
                    }
                    this.moveTabs(0);
                    ResponsiveManager.fireResizeEvent();
                };
                AppBarTabMenu.prototype.getNavigationItemById = function (tabId) {
                    var items = this.getNavigationItems();
                    var item;
                    for (var i = 0; i < items.length; i++) {
                        item = items[i];
                        if (item.getTabId().equals(tabId)) {
                            return item;
                        }
                    }
                    return null;
                };
                AppBarTabMenu.prototype.getNavigationItemByIdValue = function (tabIdValue) {
                    var items = this.getNavigationItems();
                    var item;
                    for (var i = 0; i < items.length; i++) {
                        item = items[i];
                        if (item.getTabId().getId() === tabIdValue) {
                            return item;
                        }
                    }
                    return null;
                };
                AppBarTabMenu.prototype.selectNavigationItem = function (tabIndex) {
                    _super.prototype.selectNavigationItem.call(this, tabIndex);
                    var tab = this.getNavigationItem(tabIndex);
                    this.appBarTabMenuButton.setEditing(tab.isEditing());
                    this.hideMenu();
                    this.moveTabs();
                    this.makeSelectedTabFirst(tab);
                };
                AppBarTabMenu.prototype.makeSelectedTabFirst = function (tab) {
                    if (this.getMenuEl().hasChild(tab)) {
                        this.getMenuEl().removeChild(tab);
                        this.barEl.prependChild(tab);
                        this.moveTabs();
                    }
                };
                AppBarTabMenu.prototype.makeTabFirst = function (tab) {
                    var canBeMoved;
                    if (canBeMoved = this.getMenuEl().hasChild(tab)) {
                        this.getMenuEl().removeChild(tab);
                    }
                    else if (canBeMoved = this.barEl.hasChild(tab)) {
                        this.barEl.removeChild(tab);
                    }
                    if (canBeMoved) {
                        this.barEl.prependChild(tab);
                        this.moveTabs();
                    }
                };
                AppBarTabMenu.prototype.deselectNavigationItem = function () {
                    _super.prototype.deselectNavigationItem.call(this);
                    this.appBarTabMenuButton.setEditing(false);
                };
                AppBarTabMenu.prototype.onButtonLabelChanged = function (listener) {
                    this.buttonLabelChangedListeners.push(listener);
                };
                AppBarTabMenu.prototype.unButtonLabelChanged = function (listener) {
                    this.buttonLabelChangedListeners = this.buttonLabelChangedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                AppBarTabMenu.prototype.notifyButtonLabelChanged = function () {
                    var _this = this;
                    this.buttonLabelChangedListeners.forEach(function (listener) {
                        listener.call(_this);
                    });
                };
                AppBarTabMenu.TAB_WIDTH = 190;
                AppBarTabMenu.MAX_WIDTH = 248;
                return AppBarTabMenu;
            })(api.ui.tab.TabMenu);
            bar.AppBarTabMenu = AppBarTabMenu;
        })(bar = app.bar || (app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var bar;
        (function (bar) {
            var AppBarTabMenuButton = (function (_super) {
                __extends(AppBarTabMenuButton, _super);
                function AppBarTabMenuButton() {
                    _super.call(this);
                    this.tabCountEl = new AppBarTabCount();
                    this.prependChild(this.tabCountEl);
                }
                AppBarTabMenuButton.prototype.setTabCount = function (value) {
                    this.tabCountEl.setCount(value);
                };
                AppBarTabMenuButton.prototype.setEditing = function (editing) {
                    if (editing && !this.hasClass('editing')) {
                        this.addClass('editing');
                    }
                    else if (!editing && this.hasClass('editing')) {
                        this.removeClass('editing');
                    }
                };
                return AppBarTabMenuButton;
            })(api.ui.tab.TabMenuButton);
            bar.AppBarTabMenuButton = AppBarTabMenuButton;
            var AppBarTabCount = (function (_super) {
                __extends(AppBarTabCount, _super);
                function AppBarTabCount() {
                    _super.call(this, "tab-count");
                }
                AppBarTabCount.prototype.setCount = function (value) {
                    this.getEl().setInnerHtml(value > 0 ? "" + value : "");
                };
                return AppBarTabCount;
            })(api.dom.SpanEl);
            bar.AppBarTabCount = AppBarTabCount;
        })(bar = app.bar || (app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var bar;
        (function (bar) {
            var AppBarTabMenuItem = (function (_super) {
                __extends(AppBarTabMenuItem, _super);
                function AppBarTabMenuItem(builder) {
                    builder.setAddLabelTitleAttribute(false);
                    _super.call(this, builder);
                    this.addClass("appbar-tab-menu-item");
                    this.editing = builder.editing;
                    this.tabId = builder.tabId;
                }
                AppBarTabMenuItem.prototype.isEditing = function () {
                    return this.editing;
                };
                AppBarTabMenuItem.prototype.getTabId = function () {
                    return this.tabId;
                };
                AppBarTabMenuItem.prototype.setTabId = function (tabId) {
                    this.tabId = tabId;
                };
                return AppBarTabMenuItem;
            })(api.ui.tab.TabMenuItem);
            bar.AppBarTabMenuItem = AppBarTabMenuItem;
            var AppBarTabMenuItemBuilder = (function (_super) {
                __extends(AppBarTabMenuItemBuilder, _super);
                function AppBarTabMenuItemBuilder() {
                    _super.call(this);
                    this.closeButtonEnabled = true;
                }
                AppBarTabMenuItemBuilder.prototype.setLabel = function (label) {
                    _super.prototype.setLabel.call(this, label);
                    return this;
                };
                AppBarTabMenuItemBuilder.prototype.setTabId = function (tabId) {
                    this.tabId = tabId;
                    return this;
                };
                AppBarTabMenuItemBuilder.prototype.setEditing = function (editing) {
                    this.editing = editing;
                    return this;
                };
                AppBarTabMenuItemBuilder.prototype.setCloseAction = function (closeAction) {
                    _super.prototype.setCloseAction.call(this, closeAction);
                    return this;
                };
                AppBarTabMenuItemBuilder.prototype.setMarkUnnamed = function (markUnnamed) {
                    _super.prototype.setMarkUnnamed.call(this, markUnnamed);
                    return this;
                };
                AppBarTabMenuItemBuilder.prototype.setMarkInvalid = function (markInvalid) {
                    _super.prototype.setMarkInvalid.call(this, markInvalid);
                    return this;
                };
                AppBarTabMenuItemBuilder.prototype.build = function () {
                    return new AppBarTabMenuItem(this);
                };
                return AppBarTabMenuItemBuilder;
            })(api.ui.tab.TabMenuItemBuilder);
            bar.AppBarTabMenuItemBuilder = AppBarTabMenuItemBuilder;
        })(bar = app.bar || (app.bar = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var ResponsiveManager = api.ui.responsive.ResponsiveManager;
            var ResponsiveRanges = api.ui.responsive.ResponsiveRanges;
            var BrowsePanel = (function (_super) {
                __extends(BrowsePanel, _super);
                function BrowsePanel(params) {
                    var _this = this;
                    _super.call(this);
                    this.filterPanelRefreshNeeded = false;
                    this.filterPanelForcedShown = false;
                    this.filterPanelForcedHidden = false;
                    this.filterPanelToBeShownFullScreen = false;
                    this.filterPanelIsHiddenByDefault = true;
                    this.browseToolbar = params.browseToolbar;
                    this.treeGrid = params.treeGrid;
                    this.browseItemPanel = params.browseItemPanel;
                    this.filterPanel = params.filterPanel;
                    this.gridAndToolbarContainer = new api.ui.panel.Panel();
                    var gridPanel = new api.ui.panel.Panel();
                    gridPanel.appendChild(this.treeGrid);
                    this.gridAndToolbarContainer.appendChild(gridPanel);
                    this.contentGridAndBrowseSplitPanel = new api.ui.panel.SplitPanelBuilder(this.gridAndToolbarContainer, this.browseItemPanel).setAlignmentTreshold(BrowsePanel.SPLIT_PANEL_ALIGNMENT_TRESHOLD).build();
                    this.contentGridAndBrowseSplitPanel.prependChild(this.browseToolbar);
                    this.contentGridAndBrowseSplitPanel.setFirstPanelSize(38, 1 /* PERCENT */);
                    this.browseToolbar.addClass("browse-toolbar");
                    this.contentGridAndBrowseSplitPanel.addClass("content-grid-and-browse-split-panel");
                    if (this.filterPanel) {
                        this.setupFilterPanel();
                        if (this.filterPanelIsHiddenByDefault) {
                            this.hideFilterPanel();
                        }
                    }
                    else {
                        this.filterAndContentGridAndBrowseSplitPanel = this.contentGridAndBrowseSplitPanel;
                    }
                    this.treeGrid.onSelectionChanged(function (currentSelection, fullSelection) {
                        var browseItems = _this.treeNodesToBrowseItems(fullSelection);
                        _this.browseItemPanel.setItems(browseItems);
                        _this.treeGrid.getContextMenu().getActions().updateActionsEnabledState(_this.browseItemPanel.getItems()).then(function () {
                            _this.browseItemPanel.updateDisplayedPanel();
                        });
                        _this.treeGrid.getToolbar().refresh(browseItems.length);
                    });
                    this.onRendered(function (event) {
                        if (params.hasDetailsPanel) {
                            _this.initNonMobileDetailsPanels();
                        }
                        else {
                            _this.appendChild(_this.filterAndContentGridAndBrowseSplitPanel);
                        }
                    });
                    ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                        _this.checkFilterPanelToBeShownFullScreen(item);
                        if (!_this.filterPanelIsHiddenByDefault) {
                            _this.toggleFilterPanelDependingOnScreenSize(item);
                        }
                        _this.togglePreviewPanelDependingOnScreenSize(item);
                    });
                }
                BrowsePanel.prototype.initNonMobileDetailsPanels = function () {
                };
                BrowsePanel.prototype.getFilterAndContentGridAndBrowseSplitPanel = function () {
                    return this.filterAndContentGridAndBrowseSplitPanel;
                };
                BrowsePanel.prototype.getTreeGrid = function () {
                    return this.treeGrid;
                };
                BrowsePanel.prototype.getBrowseItemPanel = function () {
                    return this.browseItemPanel;
                };
                BrowsePanel.prototype.getActions = function () {
                    return this.browseToolbar.getActions();
                };
                BrowsePanel.prototype.treeNodesToBrowseItems = function (nodes) {
                    return [];
                };
                BrowsePanel.prototype.refreshFilter = function () {
                    if (this.isFilterPanelRefreshNeeded()) {
                        if (this.filterPanel) {
                            this.filterPanel.refresh();
                        }
                        this.filterPanelRefreshNeeded = false;
                    }
                };
                BrowsePanel.prototype.isFilterPanelRefreshNeeded = function () {
                    return this.filterPanelRefreshNeeded;
                };
                BrowsePanel.prototype.setFilterPanelRefreshNeeded = function (refreshNeeded) {
                    this.filterPanelRefreshNeeded = refreshNeeded;
                };
                BrowsePanel.prototype.toggleFilterPanel = function () {
                    this.filterAndContentGridAndBrowseSplitPanel.setFirstPanelIsFullScreen(this.filterPanelToBeShownFullScreen);
                    if (this.filterPanelIsHidden()) {
                        this.showFilterPanel();
                    }
                    else {
                        this.hideFilterPanel();
                    }
                };
                BrowsePanel.prototype.filterPanelIsHidden = function () {
                    return this.filterAndContentGridAndBrowseSplitPanel.isFirstPanelHidden();
                };
                BrowsePanel.prototype.showFilterPanel = function () {
                    this.filterPanelForcedShown = true;
                    this.filterPanelForcedHidden = false;
                    if (this.filterPanelToBeShownFullScreen) {
                        this.filterAndContentGridAndBrowseSplitPanel.hideSecondPanel();
                    }
                    this.filterAndContentGridAndBrowseSplitPanel.showFirstPanel();
                    this.filterPanel.giveFocusToSearch();
                    this.toggleFilterPanelAction.setVisible(false);
                };
                BrowsePanel.prototype.hideFilterPanel = function () {
                    this.filterPanelForcedShown = false;
                    this.filterPanelForcedHidden = true;
                    this.filterAndContentGridAndBrowseSplitPanel.showSecondPanel();
                    this.filterAndContentGridAndBrowseSplitPanel.hideFirstPanel();
                    this.toggleFilterPanelAction.setVisible(true);
                };
                BrowsePanel.prototype.setupFilterPanel = function () {
                    var _this = this;
                    this.filterAndContentGridAndBrowseSplitPanel = new api.ui.panel.SplitPanelBuilder(this.filterPanel, this.contentGridAndBrowseSplitPanel).setFirstPanelSize(200, 0 /* PIXEL */).setAlignment(1 /* VERTICAL */).build();
                    this.filterPanel.onHideFilterPanelButtonClicked(function () {
                        _this.toggleFilterPanel();
                    });
                    this.filterPanel.onShowResultsButtonClicked(function () {
                        _this.toggleFilterPanel();
                    });
                    this.addToggleFilterPanelButtonInToolbar();
                };
                BrowsePanel.prototype.addToggleFilterPanelButtonInToolbar = function () {
                    this.toggleFilterPanelAction = new api.app.browse.action.ToggleFilterPanelAction(this);
                    var existingActions = this.browseToolbar.getActions();
                    this.browseToolbar.removeActions();
                    this.browseToolbar.addAction(this.toggleFilterPanelAction);
                    this.browseToolbar.addActions(existingActions);
                    this.toggleFilterPanelAction.setVisible(false);
                };
                BrowsePanel.prototype.checkFilterPanelToBeShownFullScreen = function (item) {
                    if (item.isInRangeOrSmaller(ResponsiveRanges._360_540)) {
                        this.filterPanelToBeShownFullScreen = true;
                    }
                    else {
                        this.filterPanelToBeShownFullScreen = false;
                    }
                };
                BrowsePanel.prototype.toggleFilterPanelDependingOnScreenSize = function (item) {
                    if (item.isInRangeOrSmaller(ResponsiveRanges._1380_1620)) {
                        if (this.filterPanel && !this.filterAndContentGridAndBrowseSplitPanel.isFirstPanelHidden() && !this.filterPanelForcedShown) {
                            this.filterAndContentGridAndBrowseSplitPanel.hideFirstPanel();
                            this.toggleFilterPanelAction.setVisible(true);
                        }
                    }
                    else if (item.isInRangeOrBigger(ResponsiveRanges._1620_1920)) {
                        if (this.filterPanel && this.filterAndContentGridAndBrowseSplitPanel.isFirstPanelHidden() && !this.filterPanelForcedHidden) {
                            this.filterAndContentGridAndBrowseSplitPanel.showFirstPanel();
                            this.toggleFilterPanelAction.setVisible(false);
                        }
                    }
                };
                BrowsePanel.prototype.togglePreviewPanelDependingOnScreenSize = function (item) {
                    if (item.isInRangeOrSmaller(ResponsiveRanges._360_540)) {
                        if (!this.contentGridAndBrowseSplitPanel.isSecondPanelHidden()) {
                            this.contentGridAndBrowseSplitPanel.hideSecondPanel();
                        }
                    }
                    else if (item.isInRangeOrBigger(ResponsiveRanges._540_720)) {
                        if (this.contentGridAndBrowseSplitPanel.isSecondPanelHidden()) {
                            this.contentGridAndBrowseSplitPanel.showSecondPanel();
                        }
                    }
                };
                BrowsePanel.SPLIT_PANEL_ALIGNMENT_TRESHOLD = 720;
                return BrowsePanel;
            })(api.ui.panel.Panel);
            browse.BrowsePanel = BrowsePanel;
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var BrowseItem = (function () {
                function BrowseItem(model) {
                    this.model = model;
                }
                BrowseItem.prototype.setId = function (value) {
                    this.id = value;
                    return this;
                };
                BrowseItem.prototype.setDisplayName = function (value) {
                    this.displayName = value;
                    return this;
                };
                BrowseItem.prototype.setPath = function (value) {
                    this.path = value;
                    return this;
                };
                BrowseItem.prototype.setIconUrl = function (value) {
                    this.iconUrl = value;
                    return this;
                };
                BrowseItem.prototype.setIconClass = function (iconClass) {
                    this.iconClass = iconClass;
                    return this;
                };
                BrowseItem.prototype.setRenderable = function (value) {
                    this.renderable = value;
                    return this;
                };
                BrowseItem.prototype.getIconClass = function () {
                    return this.iconClass;
                };
                BrowseItem.prototype.getModel = function () {
                    return this.model;
                };
                BrowseItem.prototype.getId = function () {
                    return this.id;
                };
                BrowseItem.prototype.getDisplayName = function () {
                    return this.displayName;
                };
                BrowseItem.prototype.getPath = function () {
                    return this.path;
                };
                BrowseItem.prototype.getIconUrl = function () {
                    return this.iconUrl;
                };
                BrowseItem.prototype.isRenderable = function () {
                    return this.renderable;
                };
                BrowseItem.prototype.toViewItem = function () {
                    return new api.app.view.ViewItem(this.model).setIconUrl(this.iconUrl).setIconClass(this.iconClass).setDisplayName(this.displayName).setPath(this.path).setRenderable(this.isRenderable());
                };
                BrowseItem.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, BrowseItem)) {
                        return false;
                    }
                    var other = o;
                    return this.model.equals(other.model) && this.displayName == other.displayName && this.path == other.path && this.iconUrl == other.iconUrl && this.iconClass == other.iconClass;
                };
                return BrowseItem;
            })();
            browse.BrowseItem = BrowseItem;
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var SelectionItem = (function (_super) {
                __extends(SelectionItem, _super);
                function SelectionItem(viewer, item, removeCallback) {
                    _super.call(this, "browse-selection-item");
                    this.viewer = viewer;
                    this.item = item;
                    this.initRemoveButton(removeCallback);
                }
                SelectionItem.prototype.initRemoveButton = function (callback) {
                    this.removeEl = new api.dom.DivEl("icon remove");
                    this.removeEl.onClicked(function (event) {
                        if (callback) {
                            callback();
                        }
                    });
                };
                SelectionItem.prototype.getBrowseItem = function () {
                    return this.item;
                };
                SelectionItem.prototype.updateViewer = function (viewer) {
                    this.viewer = viewer;
                    this.render();
                };
                SelectionItem.prototype.doRender = function () {
                    this.removeChildren();
                    this.appendChild(this.removeEl);
                    this.appendChild(this.viewer);
                    return true;
                };
                return SelectionItem;
            })(api.dom.DivEl);
            browse.SelectionItem = SelectionItem;
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var BrowseItemsSelectionPanel = (function (_super) {
                __extends(BrowseItemsSelectionPanel, _super);
                function BrowseItemsSelectionPanel() {
                    _super.call(this, "items-selection-panel");
                    this.deselectedListeners = [];
                    this.items = [];
                    this.selectionItems = [];
                    this.messageForNoSelection = "You are wasting this space - select something!";
                    this.getEl().addClass('no-selection').setInnerHtml(this.messageForNoSelection);
                }
                BrowseItemsSelectionPanel.prototype.addItem = function (item) {
                    var _this = this;
                    var index = this.indexOf(item);
                    if (index >= 0) {
                        var currentItem = this.items[index];
                        if (!currentItem.equals(item)) {
                            this.items[index] = item;
                        }
                        return;
                    }
                    if (this.items.length === 0) {
                        this.removeClass('no-selection');
                        this.removeChildren();
                    }
                    var removeCallback = function () {
                        _this.removeItem(item);
                    };
                    var selectionItem = new browse.SelectionItem(this.createItemViewer(item), item, removeCallback);
                    this.appendChild(selectionItem);
                    this.selectionItems.push(selectionItem);
                    this.items.push(item);
                };
                BrowseItemsSelectionPanel.prototype.removeItem = function (item) {
                    var index = this.indexOf(item);
                    if (index < 0) {
                        return;
                    }
                    this.selectionItems[index].remove();
                    this.selectionItems.splice(index, 1);
                    this.items.splice(index, 1);
                    if (this.items.length === 0) {
                        this.getEl().addClass('no-selection').setInnerHtml(this.messageForNoSelection);
                    }
                };
                BrowseItemsSelectionPanel.prototype.getItems = function () {
                    return this.items;
                };
                BrowseItemsSelectionPanel.prototype.setItems = function (items) {
                    var _this = this;
                    var itemsToRemove = this.items.filter(function (item) {
                        for (var i = 0; i < items.length; i++) {
                            if (item.getPath()) {
                                if (item.getPath() == items[i].getPath()) {
                                    return false;
                                }
                            }
                            else {
                                if (item.getId() == items[i].getId()) {
                                    return false;
                                }
                            }
                        }
                        return true;
                    });
                    itemsToRemove.forEach(function (item) {
                        _this.removeItem(item);
                    });
                    items.forEach(function (item) {
                        _this.addItem(item);
                    });
                };
                BrowseItemsSelectionPanel.prototype.createItemViewer = function (item) {
                    var viewer = new api.ui.Viewer();
                    viewer.setObject(item.getModel());
                    return viewer;
                };
                BrowseItemsSelectionPanel.prototype.updateItemViewers = function (items) {
                    var _this = this;
                    items.forEach(function (item) {
                        for (var i = 0; i < _this.selectionItems.length; i++) {
                            if (_this.selectionItems[i].getBrowseItem().getPath() === item.getPath()) {
                                _this.selectionItems[i].updateViewer(_this.createItemViewer(item));
                                break;
                            }
                        }
                    });
                };
                BrowseItemsSelectionPanel.prototype.indexOf = function (item) {
                    for (var i = 0; i < this.items.length; i++) {
                        if (item.getPath()) {
                            if (item.getPath() == this.items[i].getPath()) {
                                return i;
                            }
                        }
                        else {
                            if (item.getId() == this.items[i].getId()) {
                                return i;
                            }
                        }
                    }
                    return -1;
                };
                return BrowseItemsSelectionPanel;
            })(api.ui.panel.Panel);
            browse.BrowseItemsSelectionPanel = BrowseItemsSelectionPanel;
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var BrowseItemPanel = (function (_super) {
                __extends(BrowseItemPanel, _super);
                function BrowseItemPanel() {
                    _super.call(this, "browse-item-panel");
                    this.itemsSelectionPanel = this.createItemSelectionPanel();
                    this.itemStatisticsPanel = this.createItemStatisticsPanel();
                    this.addPanel(this.itemsSelectionPanel);
                    this.addPanel(this.itemStatisticsPanel);
                    this.showPanelByIndex(0);
                }
                BrowseItemPanel.prototype.createItemSelectionPanel = function () {
                    return new browse.BrowseItemsSelectionPanel();
                };
                BrowseItemPanel.prototype.createItemStatisticsPanel = function () {
                    return new api.app.view.ItemStatisticsPanel();
                };
                BrowseItemPanel.prototype.setItems = function (items) {
                    this.itemsSelectionPanel.setItems(items);
                    this.updateDisplayedPanel();
                };
                BrowseItemPanel.prototype.getItems = function () {
                    return this.itemsSelectionPanel.getItems();
                };
                BrowseItemPanel.prototype.updateItemViewers = function (items) {
                    this.itemsSelectionPanel.updateItemViewers(items);
                };
                BrowseItemPanel.prototype.updateDisplayedPanel = function () {
                    var selectedItems = this.getItems();
                    if (selectedItems.length == 1) {
                        this.showPanelByIndex(1);
                        this.itemStatisticsPanel.setItem(selectedItems[0].toViewItem());
                    }
                    else {
                        this.showPanelByIndex(0);
                    }
                };
                BrowseItemPanel.prototype.setStatisticsItem = function (item) {
                    this.itemStatisticsPanel.setItem(item.toViewItem());
                };
                BrowseItemPanel.prototype.getStatisticsItem = function () {
                    return this.itemStatisticsPanel.getItem();
                };
                return BrowseItemPanel;
            })(api.ui.panel.DeckPanel);
            browse.BrowseItemPanel = BrowseItemPanel;
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var ItemDeselectedEvent = (function () {
                function ItemDeselectedEvent(browseItem) {
                    this.browseItem = browseItem;
                }
                ItemDeselectedEvent.prototype.getBrowseItem = function () {
                    return this.browseItem;
                };
                return ItemDeselectedEvent;
            })();
            browse.ItemDeselectedEvent = ItemDeselectedEvent;
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var filter;
            (function (filter) {
                var BrowseFilterPanel = (function (_super) {
                    __extends(BrowseFilterPanel, _super);
                    function BrowseFilterPanel(aggregations, groupViews) {
                        var _this = this;
                        _super.call(this);
                        this.searchListeners = [];
                        this.refreshListeners = [];
                        this.resetListeners = [];
                        this.hideFilterPanelButtonClickedListeners = [];
                        this.showResultsButtonClickedListeners = [];
                        this.addClass('filter-panel');
                        this.hideFilterPanelButton = new api.dom.SpanEl("hide-filter-panel-button icon-search");
                        this.hideFilterPanelButton.onClicked(function () { return _this.notifyHidePanelButtonPressed(); });
                        var showResultsButtonWrapper = new api.dom.DivEl("show-filter-results");
                        this.showResultsButton = new api.dom.SpanEl("show-filter-results-button");
                        this.showResultsButton.setHtml("Show results");
                        this.showResultsButton.onClicked(function () { return _this.notifyShowResultsButtonPressed(); });
                        showResultsButtonWrapper.appendChild(this.showResultsButton);
                        this.searchField = new filter.TextSearchField('Search');
                        this.searchField.onValueChanged(function () {
                            _this.search(_this.searchField);
                        });
                        this.clearFilter = new filter.ClearFilterButton();
                        this.clearFilter.onClicked(function (event) {
                            _this.reset();
                        });
                        this.hitsCounterEl = new api.dom.SpanEl("hits-counter");
                        var hitsCounterAndClearButtonWrapper = new api.dom.DivEl("hits-and-clear");
                        hitsCounterAndClearButtonWrapper.appendChildren(this.hitsCounterEl, this.clearFilter);
                        this.aggregationContainer = new api.aggregation.AggregationContainer();
                        this.appendChild(this.aggregationContainer);
                        if (groupViews != null) {
                            groupViews.forEach(function (aggregationGroupView) {
                                aggregationGroupView.onBucketViewSelectionChanged(function (event) {
                                    _this.search(event.getBucketView());
                                });
                                _this.aggregationContainer.addAggregationGroupView(aggregationGroupView);
                            });
                        }
                        this.onRendered(function (event) {
                            _this.appendChild(_this.hideFilterPanelButton);
                            _this.appendChild(_this.searchField);
                            _this.appendChild(hitsCounterAndClearButtonWrapper);
                            _this.appendChild(_this.aggregationContainer);
                            _this.appendChild(showResultsButtonWrapper);
                            _this.showResultsButton.hide();
                            api.ui.KeyBindings.get().bindKey(new api.ui.KeyBinding("/", function (e) {
                                setTimeout(_this.giveFocusToSearch.bind(_this), 100);
                            }).setGlobal(true));
                        });
                    }
                    BrowseFilterPanel.prototype.giveFocusToSearch = function () {
                        this.searchField.giveFocus();
                    };
                    BrowseFilterPanel.prototype.updateAggregations = function (aggregations, doUpdateAll) {
                        this.aggregationContainer.updateAggregations(aggregations, doUpdateAll);
                    };
                    BrowseFilterPanel.prototype.getSearchInputValues = function () {
                        var searchInputValues = new api.query.SearchInputValues();
                        searchInputValues.setAggregationSelections(this.aggregationContainer.getSelectedValuesByAggregationName());
                        searchInputValues.setTextSearchFieldValue(this.searchField.getEl().getValue());
                        return searchInputValues;
                    };
                    BrowseFilterPanel.prototype.hasFilterSet = function () {
                        return this.aggregationContainer.hasSelectedBuckets() || this.searchField.getHTMLElement()['value'].trim() != '';
                    };
                    BrowseFilterPanel.prototype.search = function (elementChanged) {
                        if (this.hasFilterSet()) {
                            this.clearFilter.show();
                        }
                        else {
                            this.clearFilter.hide();
                        }
                        var values = this.getSearchInputValues();
                        this.notifySearch(values, elementChanged);
                    };
                    BrowseFilterPanel.prototype.refresh = function () {
                        this.notifyRefresh();
                    };
                    BrowseFilterPanel.prototype.reset = function () {
                        this.searchField.clear(true);
                        this.aggregationContainer.deselectAll(true);
                        this.clearFilter.hide();
                        this.notifyReset();
                    };
                    BrowseFilterPanel.prototype.onSearch = function (listener) {
                        this.searchListeners.push(listener);
                    };
                    BrowseFilterPanel.prototype.onReset = function (listener) {
                        this.resetListeners.push(listener);
                    };
                    BrowseFilterPanel.prototype.onRefresh = function (listener) {
                        this.refreshListeners.push(listener);
                    };
                    BrowseFilterPanel.prototype.unSearch = function (listener) {
                        this.searchListeners = this.searchListeners.filter(function (currentListener) {
                            return currentListener != listener;
                        });
                    };
                    BrowseFilterPanel.prototype.unReset = function (listener) {
                        this.resetListeners = this.resetListeners.filter(function (currentListener) {
                            return currentListener != listener;
                        });
                    };
                    BrowseFilterPanel.prototype.onHideFilterPanelButtonClicked = function (listener) {
                        this.hideFilterPanelButtonClickedListeners.push(listener);
                    };
                    BrowseFilterPanel.prototype.onShowResultsButtonClicked = function (listener) {
                        this.showResultsButtonClickedListeners.push(listener);
                    };
                    BrowseFilterPanel.prototype.notifySearch = function (searchInputValues, elementChanged) {
                        var _this = this;
                        this.searchListeners.forEach(function (listener) {
                            listener.call(_this, new filter.SearchEvent(searchInputValues, elementChanged));
                        });
                    };
                    BrowseFilterPanel.prototype.notifyRefresh = function () {
                        var _this = this;
                        this.refreshListeners.forEach(function (listener) {
                            listener.call(_this, new filter.RefreshEvent(_this.getSearchInputValues()));
                        });
                    };
                    BrowseFilterPanel.prototype.notifyReset = function () {
                        var _this = this;
                        this.resetListeners.forEach(function (listener) {
                            listener.call(_this);
                        });
                    };
                    BrowseFilterPanel.prototype.notifyHidePanelButtonPressed = function () {
                        var _this = this;
                        this.hideFilterPanelButtonClickedListeners.forEach(function (listener) {
                            listener.call(_this);
                        });
                    };
                    BrowseFilterPanel.prototype.notifyShowResultsButtonPressed = function () {
                        var _this = this;
                        this.showResultsButtonClickedListeners.forEach(function (listener) {
                            listener.call(_this);
                        });
                    };
                    BrowseFilterPanel.prototype.updateHitsCounter = function (hits, emptyFilterValue) {
                        if (emptyFilterValue === void 0) { emptyFilterValue = false; }
                        if (!emptyFilterValue) {
                            if (hits != 1) {
                                this.hitsCounterEl.setHtml(hits + " hits");
                            }
                            else {
                                this.hitsCounterEl.setHtml(hits + " hit");
                            }
                        }
                        else {
                            this.hitsCounterEl.setHtml(hits + " total");
                        }
                        if (hits != 0) {
                            this.showResultsButton.show();
                        }
                        else {
                            this.showResultsButton.hide();
                        }
                    };
                    return BrowseFilterPanel;
                })(api.ui.panel.Panel);
                filter.BrowseFilterPanel = BrowseFilterPanel;
            })(filter = browse.filter || (browse.filter = {}));
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var filter;
            (function (filter) {
                var TextSearchField = (function (_super) {
                    __extends(TextSearchField, _super);
                    function TextSearchField(placeholder) {
                        var _this = this;
                        _super.call(this, 'text-search-field');
                        this.valueChangedListeners = [];
                        this.setPlaceholder(placeholder);
                        this.onKeyDown(function (event) {
                            if (event.which == 9) {
                            }
                            else if (event.which == 13) {
                                _this.notifyValueChanged();
                            }
                            else {
                                if (_this.timerId !== null) {
                                    window.clearTimeout(_this.timerId);
                                    _this.timerId = null;
                                }
                                _this.timerId = window.setTimeout(function () {
                                    _this.notifyValueChanged();
                                }, 500);
                            }
                        });
                    }
                    TextSearchField.prototype.clear = function (supressEvent) {
                        window.clearTimeout(this.timerId);
                        this.getHTMLElement()['value'] = '';
                        this.previousValue = '';
                        if (!supressEvent) {
                            this.notifyValueChanged();
                        }
                    };
                    TextSearchField.prototype.onValueChanged = function (listener) {
                        this.valueChangedListeners.push(listener);
                    };
                    TextSearchField.prototype.notifyValueChanged = function () {
                        var currentValue = this.getHTMLElement()['value'];
                        if (currentValue == this.previousValue) {
                            return;
                        }
                        this.valueChangedListeners.forEach(function (listener) {
                            listener();
                        });
                        this.previousValue = currentValue;
                    };
                    return TextSearchField;
                })(api.dom.InputEl);
                filter.TextSearchField = TextSearchField;
            })(filter = browse.filter || (browse.filter = {}));
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var filter;
            (function (filter) {
                var ClearFilterButton = (function (_super) {
                    __extends(ClearFilterButton, _super);
                    function ClearFilterButton() {
                        _super.call(this, 'clear-filter-button');
                        this.getEl().setInnerHtml('Clear');
                        this.getHTMLElement().setAttribute('href', 'javascript:;');
                        this.hide();
                    }
                    return ClearFilterButton;
                })(api.dom.AEl);
                filter.ClearFilterButton = ClearFilterButton;
            })(filter = browse.filter || (browse.filter = {}));
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var filter;
            (function (filter) {
                var SearchEvent = (function () {
                    function SearchEvent(searchInputValues, elementChanged) {
                        this.searchInputValues = searchInputValues;
                        this.elementChanged = elementChanged;
                    }
                    SearchEvent.prototype.getSearchInputValues = function () {
                        return this.searchInputValues;
                    };
                    SearchEvent.prototype.getElementChanged = function () {
                        return this.elementChanged;
                    };
                    return SearchEvent;
                })();
                filter.SearchEvent = SearchEvent;
            })(filter = browse.filter || (browse.filter = {}));
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var filter;
            (function (filter) {
                var RefreshEvent = (function (_super) {
                    __extends(RefreshEvent, _super);
                    function RefreshEvent() {
                        _super.apply(this, arguments);
                    }
                    return RefreshEvent;
                })(filter.SearchEvent);
                filter.RefreshEvent = RefreshEvent;
            })(filter = browse.filter || (browse.filter = {}));
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var browse;
        (function (browse) {
            var action;
            (function (action) {
                var Action = api.ui.Action;
                var ToggleFilterPanelAction = (function (_super) {
                    __extends(ToggleFilterPanelAction, _super);
                    function ToggleFilterPanelAction(browsePanel) {
                        _super.call(this, "");
                        this.setIconClass("icon-search");
                        this.setEnabled(true);
                        this.onExecuted(function () {
                            browsePanel.toggleFilterPanel();
                        });
                    }
                    return ToggleFilterPanelAction;
                })(Action);
                action.ToggleFilterPanelAction = ToggleFilterPanelAction;
            })(action = browse.action || (browse.action = {}));
        })(browse = app.browse || (app.browse = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var remove;
        (function (remove) {
            var DeleteItem = (function () {
                function DeleteItem(iconUrl, displayName) {
                    this.iconUrl = iconUrl;
                    this.displayName = displayName;
                }
                DeleteItem.prototype.getDisplayName = function () {
                    return this.displayName;
                };
                DeleteItem.prototype.getIconUrl = function () {
                    return this.iconUrl;
                };
                return DeleteItem;
            })();
            remove.DeleteItem = DeleteItem;
        })(remove = app.remove || (app.remove = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var remove;
        (function (remove) {
            var DeleteDialog = (function (_super) {
                __extends(DeleteDialog, _super);
                function DeleteDialog(modelName) {
                    _super.call(this, {
                        title: new api.ui.dialog.ModalDialogHeader("Delete " + modelName)
                    });
                    this.itemList = new DeleteDialogItemList();
                    this.modelName = modelName;
                    this.getEl().addClass("delete-dialog");
                    this.appendChildToContentPanel(this.itemList);
                    var descMessage = new api.dom.H6El().addClass("desc-message").setHtml("Delete selected items and their children");
                    this.appendChildToTitle(descMessage);
                }
                DeleteDialog.prototype.show = function () {
                    api.dom.Body.get().appendChild(this);
                    _super.prototype.show.call(this);
                };
                DeleteDialog.prototype.close = function () {
                    _super.prototype.close.call(this);
                    this.remove();
                };
                DeleteDialog.prototype.setDeleteAction = function (action) {
                    this.deleteAction = action;
                    return this.addAction(action, true, true);
                };
                DeleteDialog.prototype.getDeleteAction = function () {
                    return this.deleteAction;
                };
                DeleteDialog.prototype.renderSelectedItems = function (selectedItems) {
                    this.itemList.clear();
                    if (selectedItems.length > 1) {
                        this.setTitle("Delete " + this.modelName + "s");
                    }
                    else {
                        this.setTitle("Delete " + this.modelName);
                    }
                    for (var i in selectedItems) {
                        var selectionItem = selectedItems[i];
                        this.itemList.appendChild(selectionItem);
                    }
                };
                return DeleteDialog;
            })(api.ui.dialog.ModalDialog);
            remove.DeleteDialog = DeleteDialog;
            var DeleteDialogItemList = (function (_super) {
                __extends(DeleteDialogItemList, _super);
                function DeleteDialogItemList() {
                    _super.call(this);
                    this.getEl().addClass("item-list");
                }
                DeleteDialogItemList.prototype.clear = function () {
                    this.removeChildren();
                };
                return DeleteDialogItemList;
            })(api.dom.DivEl);
            remove.DeleteDialogItemList = DeleteDialogItemList;
        })(remove = app.remove || (app.remove = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var remove;
        (function (remove) {
            var CancelDeleteDialogAction = (function (_super) {
                __extends(CancelDeleteDialogAction, _super);
                function CancelDeleteDialogAction() {
                    _super.call(this, "Cancel", "esc");
                }
                return CancelDeleteDialogAction;
            })(api.ui.Action);
            remove.CancelDeleteDialogAction = CancelDeleteDialogAction;
        })(remove = app.remove || (app.remove = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (view) {
            var ViewItem = (function () {
                function ViewItem(model) {
                    this.model = model;
                }
                ViewItem.prototype.setDisplayName = function (value) {
                    this.displayName = value;
                    return this;
                };
                ViewItem.prototype.setPath = function (value) {
                    this.path = value;
                    return this;
                };
                ViewItem.prototype.setPathName = function (value) {
                    this.pathName = value;
                    return this;
                };
                ViewItem.prototype.setIconUrl = function (value) {
                    this.iconUrl = value;
                    return this;
                };
                ViewItem.prototype.setIconClass = function (iconClass) {
                    this.iconClass = iconClass;
                    return this;
                };
                ViewItem.prototype.setIconSize = function (value) {
                    this.iconSize = value;
                    return this;
                };
                ViewItem.prototype.setRenderable = function (value) {
                    this.renderable = value;
                    return this;
                };
                ViewItem.prototype.getModel = function () {
                    return this.model;
                };
                ViewItem.prototype.getDisplayName = function () {
                    return this.displayName;
                };
                ViewItem.prototype.getPath = function () {
                    return this.path;
                };
                ViewItem.prototype.getPathName = function () {
                    return this.pathName;
                };
                ViewItem.prototype.getIconUrl = function () {
                    return this.iconUrl;
                };
                ViewItem.prototype.getIconClass = function () {
                    return this.iconClass;
                };
                ViewItem.prototype.getIconSize = function () {
                    return this.iconSize;
                };
                ViewItem.prototype.isRenderable = function () {
                    return this.renderable;
                };
                ViewItem.prototype.equals = function (o) {
                    if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ViewItem)) {
                        return false;
                    }
                    var other = o;
                    return this.model.equals(other.getModel()) && this.displayName === other.getDisplayName() && this.path === other.getPath() && this.pathName === other.getPathName() && this.iconUrl === other.getIconUrl() && this.iconClass === other.getIconClass() && this.iconSize === other.getIconSize();
                };
                return ViewItem;
            })();
            view.ViewItem = ViewItem;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (view) {
            var ItemStatisticsPanel = (function (_super) {
                __extends(ItemStatisticsPanel, _super);
                function ItemStatisticsPanel(className) {
                    _super.call(this, "item-statistics-panel" + (className ? " " + className : ""));
                    this.header = new view.ItemStatisticsHeader();
                    this.appendChild(this.header);
                }
                ItemStatisticsPanel.prototype.getHeader = function () {
                    return this.header;
                };
                ItemStatisticsPanel.prototype.setItem = function (item) {
                    this.browseItem = item;
                    this.header.setItem(item);
                };
                ItemStatisticsPanel.prototype.getItem = function () {
                    return this.browseItem;
                };
                return ItemStatisticsPanel;
            })(api.ui.panel.Panel);
            view.ItemStatisticsPanel = ItemStatisticsPanel;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (view) {
            var MultiItemStatisticsPanel = (function (_super) {
                __extends(MultiItemStatisticsPanel, _super);
                function MultiItemStatisticsPanel(className) {
                    _super.call(this, className);
                    this.tabMenu = new api.ui.tab.TabMenu();
                    this.tabMenu.hide();
                    this.appendChild(this.tabMenu);
                    this.deckPanel = new api.ui.panel.NavigatedDeckPanel(this.tabMenu);
                    this.deckPanel.setDoOffset(false);
                    this.appendChild(this.deckPanel);
                }
                MultiItemStatisticsPanel.prototype.getTabMenu = function () {
                    return this.tabMenu;
                };
                MultiItemStatisticsPanel.prototype.getDeckPanel = function () {
                    return this.deckPanel;
                };
                MultiItemStatisticsPanel.prototype.addNavigablePanel = function (tab, panel, select) {
                    this.tabMenu.show();
                    this.deckPanel.addNavigablePanel(tab, panel, select);
                };
                MultiItemStatisticsPanel.prototype.showPanel = function (index) {
                    this.tabMenu.selectNavigationItem(index);
                };
                return MultiItemStatisticsPanel;
            })(view.ItemStatisticsPanel);
            view.MultiItemStatisticsPanel = MultiItemStatisticsPanel;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (view) {
            var ItemStatisticsHeader = (function (_super) {
                __extends(ItemStatisticsHeader, _super);
                function ItemStatisticsHeader() {
                    _super.call(this, "header");
                    this.headerTitleEl = new api.dom.H1El("title");
                    this.headerPathEl = new api.dom.H4El("path");
                    this.appendChild(this.headerTitleEl);
                    this.appendChild(this.headerPathEl);
                }
                ItemStatisticsHeader.prototype.setItem = function (item) {
                    if (this.iconEl) {
                        this.iconEl.remove();
                    }
                    if (this.iconDivEl) {
                        this.iconDivEl.remove();
                    }
                    if (item) {
                        if (item.getIconUrl()) {
                            var size = item.getIconSize() || 64, icon = api.util.loader.ImageLoader.get(item.getIconUrl() + "?size=size", size, size);
                            this.iconEl = new api.dom.Element(new api.dom.NewElementBuilder().setTagName("img").setHelper(new api.dom.ImgHelper(icon)));
                            this.iconEl.addClass("icon");
                            this.prependChild(this.iconEl);
                        }
                        else {
                            this.iconDivEl = new api.dom.DivEl(item.getIconClass());
                            this.iconDivEl.addClass("icon");
                            this.prependChild(this.iconDivEl);
                        }
                        var displayName = item.getDisplayName() || '';
                        this.headerTitleEl.getEl().setInnerHtml(displayName, true).setAttribute("title", displayName);
                        this.headerPathEl.removeChildren();
                        if (item.getPath()) {
                            this.appendToHeaderPath(item.getPath(), 'parent-path');
                            this.appendToHeaderPath(item.getPathName(), 'path-name');
                        }
                    }
                    this.browseItem = item;
                };
                ItemStatisticsHeader.prototype.appendToHeaderPath = function (value, className) {
                    var pathName = new api.dom.SpanEl(className);
                    pathName.getEl().setInnerHtml(value, true);
                    this.headerPathEl.appendChild(pathName);
                };
                return ItemStatisticsHeader;
            })(api.dom.DivEl);
            view.ItemStatisticsHeader = ItemStatisticsHeader;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (view) {
            var ItemViewPanel = (function (_super) {
                __extends(ItemViewPanel, _super);
                function ItemViewPanel(toolbar, panel) {
                    _super.call(this, "item-view-panel");
                    this.closedListeners = [];
                    this.toolbar = toolbar;
                    this.panel = panel;
                    this.appendChild(this.toolbar);
                    this.appendChild(this.panel);
                }
                ItemViewPanel.prototype.getActions = function () {
                    return [];
                };
                ItemViewPanel.prototype.setItem = function (item) {
                    this.browseItem = item;
                };
                ItemViewPanel.prototype.getItem = function () {
                    return this.browseItem;
                };
                ItemViewPanel.prototype.close = function (checkCanClose) {
                    if (checkCanClose === void 0) { checkCanClose = false; }
                    if (!checkCanClose || this.canClose()) {
                        this.notifyClosed();
                    }
                };
                ItemViewPanel.prototype.canClose = function () {
                    return true;
                };
                ItemViewPanel.prototype.onClosed = function (listener) {
                    this.closedListeners.push(listener);
                };
                ItemViewPanel.prototype.unClosed = function (listener) {
                    this.closedListeners = this.closedListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                ItemViewPanel.prototype.notifyClosed = function () {
                    var _this = this;
                    this.closedListeners.forEach(function (listener) {
                        listener.call(_this, new view.ItemViewClosedEvent(_this));
                    });
                };
                return ItemViewPanel;
            })(api.ui.panel.Panel);
            view.ItemViewPanel = ItemViewPanel;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (_view) {
            var ItemViewClosedEvent = (function () {
                function ItemViewClosedEvent(view) {
                    this.view = view;
                }
                ItemViewClosedEvent.prototype.getView = function () {
                    return this.view;
                };
                return ItemViewClosedEvent;
            })();
            _view.ItemViewClosedEvent = ItemViewClosedEvent;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (view) {
            var ItemPreviewPanel = (function (_super) {
                __extends(ItemPreviewPanel, _super);
                function ItemPreviewPanel(className) {
                    var _this = this;
                    _super.call(this, "item-preview-panel" + (className ? " " + className : ""));
                    this.mask = new api.ui.mask.LoadMask(this);
                    this.frame = new api.dom.IFrameEl();
                    this.frame.onLoaded(function (event) { return _this.mask.hide(); });
                    this.appendChild(this.frame);
                }
                return ItemPreviewPanel;
            })(api.ui.panel.Panel);
            view.ItemPreviewPanel = ItemPreviewPanel;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var view;
        (function (view) {
            var ItemDataGroup = (function (_super) {
                __extends(ItemDataGroup, _super);
                function ItemDataGroup(title, className) {
                    _super.call(this, !!className ? className + " item-data-group" : "item-data-group");
                    this.header = new api.dom.H2El();
                    this.header.getEl().setInnerHtml(title);
                    this.appendChild(this.header);
                }
                ItemDataGroup.prototype.addDataList = function (header) {
                    var datas = [];
                    for (var _i = 1; _i < arguments.length; _i++) {
                        datas[_i - 1] = arguments[_i];
                    }
                    this.addDataArray(header, datas);
                };
                ItemDataGroup.prototype.addDataArray = function (header, datas) {
                    var dataList = new api.dom.UlEl("data-list");
                    if (header) {
                        var headerElement = new api.dom.LiEl();
                        headerElement.addClass("list-header");
                        headerElement.getEl().setInnerHtml(header);
                        dataList.appendChild(headerElement);
                    }
                    datas.forEach(function (data) {
                        var dataElement = new api.dom.LiEl();
                        dataElement.getEl().setInnerHtml(data);
                        dataList.appendChild(dataElement);
                    });
                    this.appendChild(dataList);
                };
                return ItemDataGroup;
            })(api.dom.DivEl);
            view.ItemDataGroup = ItemDataGroup;
        })(view = app.view || (app.view = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var SaveAction = (function (_super) {
                __extends(SaveAction, _super);
                function SaveAction(wizardPanel, label) {
                    var _this = this;
                    if (label === void 0) { label = "Save"; }
                    _super.call(this, label, "mod+s", true);
                    this.onExecuted(function () {
                        _this.setEnabled(false);
                        wizardPanel.saveChanges().catch(function (reason) { return api.DefaultErrorHandler.handle(reason); }).finally(function () { return _this.setEnabled(true); }).done();
                    });
                }
                return SaveAction;
            })(api.ui.Action);
            wizard.SaveAction = SaveAction;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var CloseAction = (function (_super) {
                __extends(CloseAction, _super);
                function CloseAction(wizardPanel, checkCanClose) {
                    var _this = this;
                    if (checkCanClose === void 0) { checkCanClose = true; }
                    _super.call(this, "Close", "alt+w", true);
                    this.onExecuted(function () {
                        if (_this.forceExecute) {
                            wizardPanel.close(false);
                        }
                        else {
                            wizardPanel.close(checkCanClose);
                        }
                    });
                }
                return CloseAction;
            })(api.ui.Action);
            wizard.CloseAction = CloseAction;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var SaveAndCloseAction = (function (_super) {
                __extends(SaveAndCloseAction, _super);
                function SaveAndCloseAction(wizardPanel) {
                    _super.call(this, "SaveAndClose", "mod+enter", true);
                    this.onExecuted(function () {
                        new wizard.SaveAction(wizardPanel).execute();
                        new wizard.CloseAction(wizardPanel).execute();
                    });
                }
                return SaveAndCloseAction;
            })(api.ui.Action);
            wizard.SaveAndCloseAction = SaveAndCloseAction;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardActions = (function () {
                function WizardActions() {
                    var actions = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        actions[_i - 0] = arguments[_i];
                    }
                    this.suspendedActions = [];
                    this.actions = actions;
                }
                WizardActions.prototype.enableActionsForNew = function () {
                    throw new Error('Must be overridden by inheritors');
                };
                WizardActions.prototype.enableActionsForExisting = function (existing) {
                    throw new Error('Must be overridden by inheritors');
                };
                WizardActions.prototype.getActions = function () {
                    return this.actions;
                };
                WizardActions.prototype.suspendActions = function (suspend) {
                    var _this = this;
                    if (suspend === void 0) { suspend = true; }
                    if (suspend) {
                        this.actions.forEach(function (action) {
                            if (action.isEnabled()) {
                                _this.suspendedActions.push(action);
                                action.setEnabled(false);
                            }
                        });
                    }
                    else {
                        this.suspendedActions.forEach(function (action) { return action.setEnabled(true); });
                        this.suspendedActions.length = 0;
                    }
                };
                return WizardActions;
            })();
            wizard.WizardActions = WizardActions;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var FormIcon = (function (_super) {
                __extends(FormIcon, _super);
                function FormIcon(iconUrl, iconTitle, uploadUrl) {
                    var _this = this;
                    _super.call(this, "form-icon");
                    this.iconUrl = iconUrl;
                    this.iconTitle = iconTitle;
                    this.uploadUrl = uploadUrl;
                    this.uploadStartedListeners = [];
                    this.uploadFinishedListeners = [];
                    var el = this.getEl();
                    this.img = new api.dom.ImgEl(this.iconUrl);
                    el.appendChild(this.img.getHTMLElement());
                    if (this.uploadUrl) {
                        this.progress = new api.ui.ProgressBar();
                        el.appendChild(this.progress.getHTMLElement());
                    }
                    this.onRendered(function (event) {
                        if (!_this.uploader && _this.uploadUrl) {
                            _this.uploader = _this.initUploader(_this.getId());
                        }
                    });
                    this.onRemoved(function (event) {
                        if (_this.uploader) {
                            _this.uploader.destroy();
                        }
                    });
                }
                FormIcon.prototype.setSrc = function (src) {
                    this.img.setSrc(src);
                };
                FormIcon.prototype.initUploader = function (elId) {
                    var _this = this;
                    var uploader = new plupload.Uploader({
                        runtimes: 'gears,html5,flash,silverlight,browserplus',
                        multi_selection: false,
                        browse_button: elId,
                        url: this.uploadUrl,
                        multipart: true,
                        drop_element: elId,
                        flash_swf_url: api.util.UriHelper.getUri('common/js/fileupload/plupload/js/plupload.flash.swf'),
                        silverlight_xap_url: api.util.UriHelper.getUri('common/js/fileupload/plupload/js/plupload.silverlight.xap'),
                        filters: [
                            { title: 'Image files', extensions: 'jpg,gif,png' }
                        ]
                    });
                    uploader.bind('Init', function (up, params) {
                    });
                    uploader.bind('FilesAdded', function (up, files) {
                    });
                    uploader.bind('QueueChanged', function (up) {
                        if (up.files.length > 0) {
                            up.start();
                            _this.notifyUploadStarted();
                        }
                    });
                    uploader.bind('UploadFile', function (up, file) {
                        _this.progress.show();
                    });
                    uploader.bind('UploadProgress', function (up, file) {
                        _this.progress.setValue(file.percent);
                    });
                    uploader.bind('FileUploaded', function (up, file, response) {
                        console.log('uploader file uploaded', up, file, response);
                        var responseObj;
                        if (response && response.status === 200) {
                            responseObj = JSON.parse(response.response);
                            if (responseObj.items && responseObj.items.length > 0) {
                                var uploadedFile = responseObj.items[0];
                                var uploadItem = new api.ui.uploader.UploadItem(file);
                                _this.notifyUploadFinished(uploadItem);
                            }
                        }
                        _this.progress.hide();
                    });
                    uploader.bind('UploadComplete', function (up, files) {
                        up.total.reset();
                        var uploadedFiles = up.splice();
                    });
                    uploader.init();
                    return uploader;
                };
                FormIcon.prototype.onUploadStarted = function (listener) {
                    this.uploadStartedListeners.push(listener);
                };
                FormIcon.prototype.onUploadFinished = function (listener) {
                    this.uploadFinishedListeners.push(listener);
                };
                FormIcon.prototype.unUploadStarted = function (listener) {
                    this.uploadStartedListeners = this.uploadStartedListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                FormIcon.prototype.unUploadFinished = function (listener) {
                    this.uploadFinishedListeners = this.uploadFinishedListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                FormIcon.prototype.notifyUploadStarted = function () {
                    var _this = this;
                    this.uploadStartedListeners.forEach(function (listener) {
                        listener.call(_this);
                    });
                };
                FormIcon.prototype.notifyUploadFinished = function (uploadItem) {
                    var _this = this;
                    this.uploadFinishedListeners.forEach(function (listener) {
                        listener.call(_this, new wizard.UploadFinishedEvent(uploadItem));
                    });
                };
                return FormIcon;
            })(api.dom.ButtonEl);
            wizard.FormIcon = FormIcon;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardHeader = (function (_super) {
                __extends(WizardHeader, _super);
                function WizardHeader() {
                    _super.call(this, "wizard-header");
                    this.propertyChangedListeners = [];
                }
                WizardHeader.prototype.onPropertyChanged = function (listener) {
                    this.propertyChangedListeners.push(listener);
                };
                WizardHeader.prototype.unPropertyChanged = function (listener) {
                    this.propertyChangedListeners = this.propertyChangedListeners.filter(function (currentListener) {
                        return listener != currentListener;
                    });
                };
                WizardHeader.prototype.notifyPropertyChanged = function (property, oldValue, newValue) {
                    var _this = this;
                    var event = new api.PropertyChangedEvent(property, oldValue, newValue);
                    this.propertyChangedListeners.forEach(function (listener) {
                        listener.call(_this, event);
                    });
                };
                WizardHeader.prototype.isValid = function () {
                    return true;
                };
                WizardHeader.prototype.giveFocus = function () {
                    return false;
                };
                return WizardHeader;
            })(api.dom.DivEl);
            wizard.WizardHeader = WizardHeader;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardHeaderWithDisplayNameAndNameBuilder = (function () {
                function WizardHeaderWithDisplayNameAndNameBuilder() {
                }
                WizardHeaderWithDisplayNameAndNameBuilder.prototype.setDisplayNameGenerator = function (value) {
                    this.displayNameGenerator = value;
                    return this;
                };
                WizardHeaderWithDisplayNameAndNameBuilder.prototype.build = function () {
                    return new WizardHeaderWithDisplayNameAndName(this);
                };
                return WizardHeaderWithDisplayNameAndNameBuilder;
            })();
            wizard.WizardHeaderWithDisplayNameAndNameBuilder = WizardHeaderWithDisplayNameAndNameBuilder;
            var WizardHeaderWithDisplayNameAndName = (function (_super) {
                __extends(WizardHeaderWithDisplayNameAndName, _super);
                function WizardHeaderWithDisplayNameAndName(builder) {
                    var _this = this;
                    _super.call(this);
                    this.forbiddenChars = /[\/\\]+/ig;
                    this.autoGenerateName = false;
                    this.autoGenerationEnabled = true;
                    this.ignoreGenerateStatusForName = true;
                    this.simplifiedNameGeneration = false;
                    this.addClass("wizard-header-with-display-name-and-name");
                    this.displayNameGenerator = builder.displayNameGenerator;
                    this.displayNameProgrammaticallySet = this.displayNameGenerator != null;
                    this.displayNameEl = api.ui.text.AutosizeTextInput.large().setName('displayName');
                    this.displayNameEl.setPlaceholder("<Display Name>");
                    this.displayNameEl.onValueChanged(function (event) {
                        _this.notifyPropertyChanged("displayName", event.getOldValue(), event.getNewValue());
                    });
                    this.appendChild(this.displayNameEl);
                    this.pathEl = new api.dom.SpanEl('path');
                    this.pathEl.hide();
                    this.appendChild(this.pathEl);
                    this.nameElWrapper = new api.dom.DivEl("name-wrapper");
                    this.nameEl = api.ui.text.AutosizeTextInput.middle().setName('name').setForbiddenCharsRe(this.forbiddenChars);
                    this.nameEl.setPlaceholder("<name>");
                    this.nameEl.onValueChanged(function (event) {
                        _this.notifyPropertyChanged("name", event.getOldValue(), event.getNewValue());
                    });
                    this.nameElWrapper.appendChild(this.nameEl);
                    this.appendChild(this.nameElWrapper);
                    this.displayNameEl.onValueChanged(function (event) {
                        _this.displayNameEl.removeClass("generated");
                        var currentDisplayName = event.getNewValue() || "";
                        if (_this.displayNameGenerator && _this.displayNameGenerator.hasScript()) {
                            var generatedDisplayName = _this.displayNameGenerator.execute() || "";
                            _this.displayNameProgrammaticallySet = generatedDisplayName.toLowerCase() === currentDisplayName.toLowerCase() || generatedDisplayName.trim().toLowerCase() === currentDisplayName.toLowerCase() || api.util.StringHelper.isEmpty(currentDisplayName);
                            if (_this.displayNameProgrammaticallySet) {
                                _this.displayNameEl.addClass("generated");
                            }
                        }
                        _this.doAutoGenerateName(currentDisplayName);
                    });
                    this.nameEl.onValueChanged(function (event) {
                        var currentName = event.getNewValue() || "";
                        var displayName = _this.getDisplayName() || "";
                        _this.autoGenerateName = _this.checkAutoGenerateName(currentName, displayName);
                        _this.updateNameGeneratedStatus();
                    });
                    this.onShown(function (event) { return _this.updatePathAndNameWidth(); });
                    api.dom.WindowDOM.get().onResized(function (event) { return _this.updatePathAndNameWidth(); }, this);
                }
                WizardHeaderWithDisplayNameAndName.prototype.checkAutoGenerateName = function (name, displayName) {
                    return api.util.StringHelper.isEmpty(name) || displayName.toLowerCase() === name.toLowerCase() || name.toLowerCase() === this.generateName(displayName).toLowerCase();
                };
                WizardHeaderWithDisplayNameAndName.prototype.initNames = function (displayName, name, forceDisplayNameProgrammaticallySet) {
                    this.autoGenerateName = this.checkAutoGenerateName(name, displayName);
                    this.displayNameEl.setValue(displayName);
                    if (name != null) {
                        this.nameEl.setValue(name);
                    }
                    else {
                        this.nameEl.setValue(this.generateName(displayName));
                    }
                    if (this.displayNameGenerator && this.displayNameGenerator.hasScript()) {
                        if (!forceDisplayNameProgrammaticallySet) {
                            var generatedDisplayName = this.displayNameGenerator.execute();
                            this.displayNameProgrammaticallySet = generatedDisplayName == displayName;
                        }
                        else {
                            this.displayNameProgrammaticallySet = true;
                        }
                    }
                };
                WizardHeaderWithDisplayNameAndName.prototype.doAutoGenerateName = function (value) {
                    if (this.autoGenerateName && this.autoGenerationEnabled) {
                        this.nameEl.setValue(this.generateName(value));
                    }
                };
                WizardHeaderWithDisplayNameAndName.prototype.isAutoGenerationEnabled = function () {
                    return this.autoGenerationEnabled;
                };
                WizardHeaderWithDisplayNameAndName.prototype.setAutoGenerationEnabled = function (value) {
                    this.autoGenerationEnabled = value;
                };
                WizardHeaderWithDisplayNameAndName.prototype.setDisplayName = function (value) {
                    if (this.displayNameProgrammaticallySet) {
                        value = value.trim();
                        this.displayNameEl.setValue(value);
                        this.doAutoGenerateName(value);
                    }
                };
                WizardHeaderWithDisplayNameAndName.prototype.setPath = function (value) {
                    this.pathEl.getEl().setText(value);
                    if (value) {
                        this.pathEl.show();
                    }
                    else {
                        this.pathEl.hide();
                    }
                };
                WizardHeaderWithDisplayNameAndName.prototype.setSimplifiedNameGeneration = function (value) {
                    this.simplifiedNameGeneration = value;
                };
                WizardHeaderWithDisplayNameAndName.prototype.disableNameInput = function () {
                    this.nameEl.getEl().setAttribute("disabled", "disabled");
                };
                WizardHeaderWithDisplayNameAndName.prototype.getName = function () {
                    return this.nameEl.getValue();
                };
                WizardHeaderWithDisplayNameAndName.prototype.getDisplayName = function () {
                    return this.displayNameEl.getValue();
                };
                WizardHeaderWithDisplayNameAndName.prototype.giveFocus = function () {
                    return this.displayNameEl.giveFocus();
                };
                WizardHeaderWithDisplayNameAndName.prototype.generateName = function (value) {
                    return this.ensureValidName(value);
                };
                WizardHeaderWithDisplayNameAndName.prototype.ensureValidName = function (possibleInvalidName) {
                    if (!possibleInvalidName) {
                        return "";
                    }
                    var generated;
                    if (this.simplifiedNameGeneration) {
                        generated = possibleInvalidName.replace(api.Name.SIMPLIFIED_FORBIDDEN_CHARS, '').toLowerCase();
                    }
                    else {
                        generated = possibleInvalidName.replace(/[\s+\.\/]/ig, '-').replace(/-{2,}/g, '-').replace(/^-|-$/g, '').replace(api.Name.FORBIDDEN_CHARS, '').toLowerCase();
                    }
                    return (generated || '');
                };
                WizardHeaderWithDisplayNameAndName.prototype.setIgnoreGenerateStatusForName = function (value) {
                    this.ignoreGenerateStatusForName = value;
                    this.updateNameGeneratedStatus();
                };
                WizardHeaderWithDisplayNameAndName.prototype.disableNameGeneration = function (value) {
                    var _this = this;
                    if (value) {
                        this.setAutoGenerationEnabled(false);
                        this.nameEl.getEl().setDisabled(true);
                        this.nameElWrapper.onClicked(function () {
                            _this.nameEl.getEl().setDisabled(false);
                            _this.nameEl.giveFocus();
                        });
                        this.nameEl.onBlur(function () {
                            _this.nameEl.getEl().setDisabled(true);
                        });
                    }
                    this.setIgnoreGenerateStatusForName(value);
                };
                WizardHeaderWithDisplayNameAndName.prototype.updateNameGeneratedStatus = function () {
                    if (this.autoGenerateName && !this.ignoreGenerateStatusForName) {
                        this.nameEl.addClass("generated");
                    }
                    else {
                        this.nameEl.removeClass("generated");
                    }
                };
                WizardHeaderWithDisplayNameAndName.prototype.updatePathAndNameWidth = function () {
                    var pathEl = this.pathEl.getEl(), nameEl = this.nameEl.getEl(), headerWidth = this.getEl().getWidth(), pathWidth = pathEl.getWidthWithMargin(), nameWidth = nameEl.getWidthWithMargin(), nameMinWidth = nameEl.getMinWidth();
                    if (pathWidth + nameWidth > headerWidth) {
                        if (nameWidth > nameMinWidth) {
                            nameEl.setWidthPx(Math.max(nameMinWidth, headerWidth - pathWidth));
                        }
                        if (pathWidth + nameMinWidth > headerWidth) {
                            pathEl.setWidthPx(headerWidth - nameMinWidth - pathEl.getMarginLeft() - pathEl.getMarginRight());
                        }
                    }
                    else {
                        var pathClone = new api.dom.SpanEl('path');
                        pathClone.setHtml(pathEl.getInnerHtml());
                        pathClone.getEl().setHeight('0px');
                        pathClone.insertAfterEl(this.pathEl);
                        var pathCloneWidth = pathClone.getEl().getWidth();
                        if (pathEl.getWidth() < pathCloneWidth) {
                            pathEl.setWidthPx(Math.min(pathCloneWidth, headerWidth - nameWidth - pathEl.getMarginLeft() - pathEl.getMarginRight()));
                        }
                        pathClone.remove();
                    }
                };
                WizardHeaderWithDisplayNameAndName.prototype.isValid = function () {
                    return !!this.displayNameEl.getValue() && !!this.nameEl.getValue();
                };
                return WizardHeaderWithDisplayNameAndName;
            })(wizard.WizardHeader);
            wizard.WizardHeaderWithDisplayNameAndName = WizardHeaderWithDisplayNameAndName;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardHeaderWithName = (function (_super) {
                __extends(WizardHeaderWithName, _super);
                function WizardHeaderWithName() {
                    var _this = this;
                    _super.call(this);
                    this.nameEl = api.ui.text.AutosizeTextInput.large().setName('name').setForbiddenCharsRe(/[^_a-z0-9\-]+/ig);
                    this.nameEl.onValueChanged(function (event) {
                        _this.notifyPropertyChanged("name", event.getOldValue(), event.getNewValue());
                    });
                    this.appendChild(this.nameEl);
                }
                WizardHeaderWithName.prototype.getName = function () {
                    return this.nameEl.getValue();
                };
                WizardHeaderWithName.prototype.setName = function (value) {
                    this.nameEl.setValue(value);
                };
                WizardHeaderWithName.prototype.giveFocus = function () {
                    return this.nameEl.giveFocus();
                };
                return WizardHeaderWithName;
            })(wizard.WizardHeader);
            wizard.WizardHeaderWithName = WizardHeaderWithName;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardStepsPanel = (function (_super) {
                __extends(WizardStepsPanel, _super);
                function WizardStepsPanel(navigator, scrollable) {
                    _super.call(this, navigator, scrollable, "wizard-steps-panel");
                }
                return WizardStepsPanel;
            })(api.ui.panel.NavigatedPanelStrip);
            wizard.WizardStepsPanel = WizardStepsPanel;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardStepNavigator = (function (_super) {
                __extends(WizardStepNavigator, _super);
                function WizardStepNavigator() {
                    _super.call(this, "wizard-step-navigator");
                }
                WizardStepNavigator.prototype.addNavigationItem = function (step) {
                    _super.prototype.addNavigationItem.call(this, step);
                    if (this.getSize() == 1) {
                        step.addClass("first");
                    }
                };
                WizardStepNavigator.prototype.nextStep = function () {
                    var nextIndex = Math.min(this.getSelectedIndex() + 1, this.getSize() - 1);
                    this.selectNavigationItem(nextIndex);
                };
                WizardStepNavigator.prototype.previousStep = function () {
                    var previousIndex = Math.max(this.getSelectedIndex() - 1, 0);
                    this.selectNavigationItem(previousIndex);
                };
                WizardStepNavigator.prototype.hasNext = function () {
                    return this.getSelectedIndex() < this.getSize() - 1;
                };
                WizardStepNavigator.prototype.hasPrevious = function () {
                    return this.getSelectedIndex() > 0;
                };
                return WizardStepNavigator;
            })(api.ui.tab.TabBar);
            wizard.WizardStepNavigator = WizardStepNavigator;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardStep = (function () {
                function WizardStep(label, stepForm) {
                    this.tabBarItem = new api.ui.tab.TabBarItemBuilder().setAddLabelTitleAttribute(false).setLabel(label).build();
                    this.stepForm = stepForm;
                }
                WizardStep.prototype.getTabBarItem = function () {
                    return this.tabBarItem;
                };
                WizardStep.prototype.getStepForm = function () {
                    return this.stepForm;
                };
                return WizardStep;
            })();
            wizard.WizardStep = WizardStep;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardStepValidityChangedEvent = (function () {
                function WizardStepValidityChangedEvent(isValid) {
                    this.valid = isValid;
                }
                WizardStepValidityChangedEvent.prototype.isValid = function () {
                    return this.valid;
                };
                return WizardStepValidityChangedEvent;
            })();
            wizard.WizardStepValidityChangedEvent = WizardStepValidityChangedEvent;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var ValidationRecording = api.form.ValidationRecording;
            var WizardStepForm = (function (_super) {
                __extends(WizardStepForm, _super);
                function WizardStepForm(className) {
                    _super.call(this, className);
                    this.validityChangedListeners = [];
                    this.focusListeners = [];
                    this.blurListeners = [];
                    this.addClass("wizard-step-form");
                }
                WizardStepForm.prototype.validate = function (silent) {
                    return new ValidationRecording();
                };
                WizardStepForm.prototype.isValid = function () {
                    if (!this.previousValidation) {
                        this.previousValidation = this.validate(true);
                    }
                    return this.previousValidation.isValid();
                };
                WizardStepForm.prototype.onValidityChanged = function (listener) {
                    this.validityChangedListeners.push(listener);
                };
                WizardStepForm.prototype.unValidityChanged = function (listener) {
                    this.validityChangedListeners = this.validityChangedListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                WizardStepForm.prototype.notifyValidityChanged = function (event) {
                    this.validityChangedListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                WizardStepForm.prototype.onFocus = function (listener) {
                    this.focusListeners.push(listener);
                };
                WizardStepForm.prototype.unFocus = function (listener) {
                    this.focusListeners = this.focusListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                WizardStepForm.prototype.onBlur = function (listener) {
                    this.blurListeners.push(listener);
                };
                WizardStepForm.prototype.unBlur = function (listener) {
                    this.blurListeners = this.blurListeners.filter(function (curr) {
                        return curr !== listener;
                    });
                };
                WizardStepForm.prototype.notifyFocused = function (event) {
                    this.focusListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                WizardStepForm.prototype.notifyBlurred = function (event) {
                    this.blurListeners.forEach(function (listener) {
                        listener(event);
                    });
                };
                return WizardStepForm;
            })(api.ui.panel.Panel);
            wizard.WizardStepForm = WizardStepForm;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var ResponsiveManager = api.ui.responsive.ResponsiveManager;
            var WizardPanel = (function (_super) {
                __extends(WizardPanel, _super);
                function WizardPanel(params, onSuccess, onError) {
                    var _this = this;
                    _super.call(this, "wizard-panel");
                    this.isChanged = true;
                    this.closedListeners = [];
                    this.splitPanelThreshold = 960;
                    this.minimized = false;
                    this.scrollPosition = 0;
                    this.validityManager = new wizard.WizardValidityManager();
                    this.tabId = params.tabId;
                    this.persistedItem = params.persistedItem;
                    this.header = params.header;
                    this.mainToolbar = params.mainToolbar;
                    this.stepToolbar = params.stepToolbar;
                    this.actions = params.actions;
                    this.validityManager.setHeader(this.header);
                    this.formPanel = new api.ui.panel.Panel("form-panel");
                    this.formPanel.onScroll(function () { return _this.updateStickyToolbar(); });
                    this.appendChild(this.mainToolbar);
                    if (params.split && params.livePanel) {
                        this.toggleMinimizeListener = function (event) {
                            _this.toggleMinimize(event.getIndex());
                        };
                        this.minimizeEditButton = new api.dom.DivEl("minimize-edit icon icon-arrow-right");
                        this.formPanel.prependChild(this.minimizeEditButton);
                        this.splitPanel = this.createSplitPanel(this.formPanel, params.livePanel);
                        this.appendChild(this.splitPanel);
                        api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this.minimizeEditButton, function () {
                            _this.minimizeEditButton.getEl().setLeftPx(_this.stepsPanel.getEl().getWidth());
                        });
                        this.minimizeEditButton.onClicked(this.toggleMinimize.bind(this, -1));
                    }
                    else {
                        this.appendChild(this.formPanel);
                    }
                    var headerAndNavigatorContainer = new api.dom.DivEl("header-and-navigator-container");
                    headerAndNavigatorContainer.appendChild(params.formIcon);
                    headerAndNavigatorContainer.appendChild(this.header);
                    var container = new api.dom.DivEl("test-container");
                    this.stepNavigatorAndToolbarContainer = new api.dom.DivEl("wizard-step-navigator-and-toolbar");
                    this.stepNavigator = new wizard.WizardStepNavigator();
                    if (this.stepToolbar) {
                        this.stepNavigatorAndToolbarContainer.appendChild(this.stepToolbar);
                    }
                    this.stepNavigatorAndToolbarContainer.appendChild(this.stepNavigator);
                    headerAndNavigatorContainer.appendChild(this.stepNavigatorAndToolbarContainer);
                    this.stepsPanel = new wizard.WizardStepsPanel(this.stepNavigator, this.formPanel);
                    this.stepNavigatorAndToolbarContainer.onShown(function (event) {
                        _this.stepsPanel.setScrollOffset(event.getElement().getEl().getHeight());
                    });
                    ResponsiveManager.onAvailableSizeChanged(this.stepNavigatorAndToolbarContainer, function (item) {
                        if (_this.isVisible()) {
                            _this.updateStickyToolbar();
                            _this.stepsPanel.setScrollOffset(item.getElement().getEl().getHeight());
                        }
                    });
                    this.formPanel.appendChildren(headerAndNavigatorContainer, this.stepsPanel);
                    this.layingOutNew = this.persistedItem == null;
                    if (this.layingOutNew) {
                        this.preLayoutNew().then(function () { return _this.layoutNew(); }).then(function () {
                            _this.postLayoutNew();
                            onSuccess();
                        }).catch(function (reason) {
                            if (onError) {
                                onError(reason);
                            }
                            else {
                                api.DefaultErrorHandler.handle(reason);
                            }
                        }).done(function () { return _this.validityManager.notifyValidityChanged(_this.isValid()); });
                    }
                    else {
                        this.startLayoutPersistedItem(this.persistedItem).then(function () {
                            _this.postLayoutPersisted(_this.persistedItem);
                            onSuccess();
                        }).catch(function (reason) {
                            if (onError) {
                                onError(reason);
                            }
                            else {
                                api.DefaultErrorHandler.handle(reason);
                            }
                        }).done(function () { return _this.validityManager.notifyValidityChanged(_this.isValid()); });
                    }
                    this.onRendered(function (event) {
                        _this.firstShow = true;
                    });
                    this.onShown(function (event) {
                        if (_this.firstShow) {
                            _this.firstShow = false;
                            _this.giveInitialFocus();
                        }
                        if (_this.lastFocusedElement) {
                            _this.lastFocusedElement.focus();
                        }
                    });
                }
                WizardPanel.prototype.getWizardStepsPanel = function () {
                    return this.stepsPanel;
                };
                WizardPanel.prototype.getMainToolbar = function () {
                    return this.mainToolbar;
                };
                WizardPanel.prototype.updateStickyToolbar = function () {
                    var scrollTop = this.formPanel.getHTMLElement().scrollTop;
                    var wizardHeaderHeight = this.header.getEl().getHeightWithMargin();
                    var navigationWidth;
                    if (scrollTop > wizardHeaderHeight) {
                        this.mainToolbar.removeClass("scroll-shadow");
                        var stepNavigatorEl = this.stepNavigatorAndToolbarContainer.getEl().addClass("scroll-stick");
                        if (!this.stepNavigatorPlaceholder) {
                            this.stepNavigatorPlaceholder = new api.dom.DivEl('toolbar-placeholder');
                            this.stepNavigatorPlaceholder.insertAfterEl(this.stepNavigatorAndToolbarContainer);
                            this.stepNavigatorPlaceholder.getEl().setWidthPx(stepNavigatorEl.getWidth()).setHeightPx(stepNavigatorEl.getHeight());
                        }
                    }
                    else if (scrollTop < wizardHeaderHeight) {
                        this.mainToolbar.addClass("scroll-shadow");
                        this.stepNavigatorAndToolbarContainer.removeClass("scroll-stick");
                        if (this.stepNavigatorPlaceholder) {
                            this.stepNavigatorPlaceholder.remove();
                            this.stepNavigatorPlaceholder = undefined;
                        }
                    }
                    if (scrollTop == 0) {
                        this.mainToolbar.removeClass("scroll-shadow");
                    }
                    if (this.minimized) {
                        navigationWidth = this.splitPanel.getEl().getHeight();
                    }
                    else {
                        navigationWidth = this.stepsPanel.getEl().getWidth();
                    }
                    this.stepNavigatorAndToolbarContainer.getEl().setWidthPx(navigationWidth);
                };
                WizardPanel.prototype.toggleMinimize = function (navigationIndex) {
                    if (navigationIndex === void 0) { navigationIndex = -1; }
                    this.stepsPanel.setListenToScroll(false);
                    var scroll = this.stepsPanel.getScroll();
                    this.minimized = !this.minimized;
                    this.stepNavigator.unNavigationItemActivated(this.toggleMinimizeListener);
                    this.formPanel.toggleClass("minimized");
                    if (this.minimized) {
                        this.stepNavigator.setScrollEnabled(false);
                        this.scrollPosition = scroll;
                        this.splitPanel.saveFirstPanelSizeAndDistribute(40, 0, 0 /* PIXEL */);
                        this.splitPanel.hideSplitter();
                        this.minimizeEditButton.getEl().setLeftPx(this.stepsPanel.getEl().getWidth());
                        this.stepNavigator.onNavigationItemActivated(this.toggleMinimizeListener);
                    }
                    else {
                        this.splitPanel.loadFirstPanelSizeAndDistribute();
                        this.splitPanel.showSplitter();
                        this.stepsPanel.setScroll(this.scrollPosition);
                        this.stepsPanel.setListenToScroll(true);
                        this.stepNavigator.setScrollEnabled(true);
                        this.stepNavigator.selectNavigationItem(navigationIndex, false, true);
                    }
                };
                WizardPanel.prototype.isMinimized = function () {
                    return this.minimized;
                };
                WizardPanel.prototype.giveInitialFocus = function () {
                    this.header.giveFocus();
                    this.startRememberFocus();
                };
                WizardPanel.prototype.startRememberFocus = function () {
                    var _this = this;
                    wemjq(this.getHTMLElement()).on("focus", "*", function (e) {
                        e.stopPropagation();
                        _this.lastFocusedElement = wemjq(e.target);
                    });
                };
                WizardPanel.prototype.getTabId = function () {
                    return this.tabId;
                };
                WizardPanel.prototype.setTabId = function (tabId) {
                    this.tabId = tabId;
                };
                WizardPanel.prototype.getHeader = function () {
                    return this.header;
                };
                WizardPanel.prototype.getIconUrl = function () {
                    return null;
                };
                WizardPanel.prototype.getActions = function () {
                    return this.actions.getActions();
                };
                WizardPanel.prototype.getSteps = function () {
                    return this.steps;
                };
                WizardPanel.prototype.setSteps = function (steps) {
                    var _this = this;
                    steps.forEach(function (step, index) {
                        _this.addStep(step, index === 0);
                    });
                    this.steps = steps;
                };
                WizardPanel.prototype.addStep = function (step, select) {
                    this.stepsPanel.addNavigablePanel(step.getTabBarItem(), step.getStepForm(), step.getTabBarItem().getLabel(), select);
                    this.validityManager.addItem(step);
                };
                WizardPanel.prototype.insertStepBefore = function (stepToInsert, beforeStep) {
                    var indexOfBeforeStep = this.steps.indexOf(beforeStep);
                    this.steps.splice(indexOfBeforeStep, 0, stepToInsert);
                    this.stepsPanel.insertNavigablePanel(stepToInsert.getTabBarItem(), stepToInsert.getStepForm(), stepToInsert.getTabBarItem().getLabel(), indexOfBeforeStep);
                    this.validityManager.addItem(stepToInsert);
                };
                WizardPanel.prototype.removeStepWithForm = function (form) {
                    var _this = this;
                    this.steps = this.steps.filter(function (step) {
                        var remove = (step.getStepForm() == form);
                        if (remove) {
                            _this.validityManager.removeItem(step);
                        }
                        return !remove;
                    });
                    return this.stepsPanel.removeNavigablePanel(form);
                };
                WizardPanel.prototype.preLayoutNew = function () {
                    var deferred = wemQ.defer();
                    deferred.resolve(null);
                    return deferred.promise;
                };
                WizardPanel.prototype.isLayingOutNew = function () {
                    return this.layingOutNew;
                };
                WizardPanel.prototype.layoutNew = function () {
                    var deferred = wemQ.defer();
                    this.actions.enableActionsForNew();
                    deferred.resolve(null);
                    return deferred.promise;
                };
                WizardPanel.prototype.postLayoutNew = function () {
                    var deferred = wemQ.defer();
                    deferred.resolve(null);
                    return deferred.promise;
                };
                WizardPanel.prototype.startLayoutPersistedItem = function (persistedItem) {
                    this.persistedItem = persistedItem;
                    this.actions.enableActionsForExisting(persistedItem);
                    return this.layoutPersistedItem(persistedItem);
                };
                WizardPanel.prototype.layoutPersistedItem = function (persistedItem) {
                    var deferred = wemQ.defer();
                    deferred.resolve(null);
                    return deferred.promise;
                };
                WizardPanel.prototype.postLayoutPersisted = function (existing) {
                    var deferred = wemQ.defer();
                    deferred.resolve(null);
                    return deferred.promise;
                };
                WizardPanel.prototype.getPersistedItem = function () {
                    return this.persistedItem;
                };
                WizardPanel.prototype.isItemPersisted = function () {
                    return this.persistedItem != null;
                };
                WizardPanel.prototype.setPersistedItem = function (newPersistedItem) {
                    this.persistedItem = newPersistedItem;
                };
                WizardPanel.prototype.hasUnsavedChanges = function () {
                    return this.isChanged;
                };
                WizardPanel.prototype.askUserForSaveChangesBeforeClosing = function () {
                    new wizard.SaveBeforeCloseDialog(this).open();
                };
                WizardPanel.prototype.saveChanges = function () {
                    var _this = this;
                    if (this.isItemPersisted()) {
                        return this.updatePersistedItem().then(function (persistedItem) {
                            _this.isChanged = false;
                            return _this.startLayoutPersistedItem(persistedItem).then(function () { return persistedItem; });
                        });
                    }
                    else {
                        return this.persistNewItem().then(function (persistedItem) {
                            _this.isChanged = false;
                            return _this.postPersistNewItem(persistedItem).then(function () { return _this.startLayoutPersistedItem(persistedItem); }).then(function () { return persistedItem; });
                        });
                    }
                };
                WizardPanel.prototype.persistNewItem = function () {
                    throw new Error("Must be overriden by inheritor");
                };
                WizardPanel.prototype.postPersistNewItem = function (persistedItem) {
                    var deferred = wemQ.defer();
                    deferred.resolve(null);
                    return deferred.promise;
                };
                WizardPanel.prototype.updatePersistedItem = function () {
                    throw new Error("Must be overriden by inheritor");
                };
                WizardPanel.prototype.close = function (checkCanClose) {
                    if (checkCanClose === void 0) { checkCanClose = false; }
                    if (!checkCanClose || this.canClose()) {
                        this.notifyClosed();
                    }
                };
                WizardPanel.prototype.canClose = function () {
                    if (this.hasUnsavedChanges()) {
                        this.askUserForSaveChangesBeforeClosing();
                        return false;
                    }
                    else {
                        return true;
                    }
                };
                WizardPanel.prototype.onClosed = function (listener) {
                    this.closedListeners.push(listener);
                };
                WizardPanel.prototype.unClosed = function (listener) {
                    this.closedListeners = this.closedListeners.filter(function (currentListener) {
                        return currentListener != listener;
                    });
                };
                WizardPanel.prototype.getSplitPanel = function () {
                    return this.splitPanel;
                };
                WizardPanel.prototype.showMinimizeEditButton = function () {
                    this.minimizeEditButton.show();
                };
                WizardPanel.prototype.hideMinimizeEditButton = function () {
                    this.minimizeEditButton.hide();
                };
                WizardPanel.prototype.createSplitPanel = function (firstPanel, secondPanel) {
                    var splitPanel = new api.ui.panel.SplitPanelBuilder(firstPanel, secondPanel).setFirstPanelMinSize(280, 0 /* PIXEL */).setAlignment(1 /* VERTICAL */).build();
                    this.updateSplitPanel(splitPanel);
                    return splitPanel;
                };
                WizardPanel.prototype.updateSplitPanel = function (splitPanel) {
                    if (wemjq(window).width() > this.splitPanelThreshold) {
                        splitPanel.setFirstPanelSize(38, 1 /* PERCENT */);
                    }
                    splitPanel.distribute();
                };
                WizardPanel.prototype.notifyClosed = function () {
                    var _this = this;
                    this.closedListeners.forEach(function (listener) {
                        listener(new wizard.WizardClosedEvent(_this));
                    });
                };
                WizardPanel.prototype.onValidityChanged = function (listener) {
                    this.validityManager.onValidityChanged(listener);
                };
                WizardPanel.prototype.unValidityChanged = function (listener) {
                    this.validityManager.onValidityChanged(listener);
                };
                WizardPanel.prototype.notifyValidityChanged = function (valid) {
                    this.validityManager.notifyValidityChanged(valid);
                };
                WizardPanel.prototype.isValid = function () {
                    return this.validityManager.isAllValid();
                };
                return WizardPanel;
            })(api.ui.panel.Panel);
            wizard.WizardPanel = WizardPanel;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var SaveBeforeCloseDialog = (function (_super) {
                __extends(SaveBeforeCloseDialog, _super);
                function SaveBeforeCloseDialog(wizardPanel) {
                    var _this = this;
                    _super.call(this, {
                        idPrefix: "SaveBeforeCloseDialog",
                        title: new api.ui.dialog.ModalDialogHeader("Close wizard")
                    });
                    this.yesAction = new api.ui.Action('Yes');
                    this.noAction = new api.ui.Action('No');
                    this.wizardPanel = wizardPanel;
                    var message = new api.dom.H6El();
                    message.getEl().setInnerHtml("There are unsaved changes, do you want to save them before closing?");
                    this.appendChildToContentPanel(message);
                    this.yesAction.setMnemonic("y");
                    this.yesAction.onExecuted(function () {
                        _this.doSaveAndClose();
                    });
                    this.addAction(this.yesAction, true);
                    this.noAction.setMnemonic("n");
                    this.noAction.onExecuted(function () {
                        _this.doCloseWithoutSaveCheck();
                    });
                    this.addAction(this.noAction);
                    this.getCancelAction().setMnemonic("c");
                    this.getCancelAction().onExecuted(function () {
                        _this.close();
                    });
                }
                SaveBeforeCloseDialog.prototype.show = function () {
                    api.dom.Body.get().appendChild(this);
                    _super.prototype.show.call(this);
                };
                SaveBeforeCloseDialog.prototype.close = function () {
                    this.remove();
                    _super.prototype.close.call(this);
                };
                SaveBeforeCloseDialog.prototype.doSaveAndClose = function () {
                    var _this = this;
                    this.close();
                    this.wizardPanel.saveChanges().then(function () { return _this.wizardPanel.close(true); }).catch(function (reason) { return api.DefaultErrorHandler.handle(reason); }).done();
                };
                SaveBeforeCloseDialog.prototype.doCloseWithoutSaveCheck = function () {
                    this.close();
                    this.wizardPanel.close();
                };
                return SaveBeforeCloseDialog;
            })(api.ui.dialog.ModalDialog);
            wizard.SaveBeforeCloseDialog = SaveBeforeCloseDialog;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var UploadFinishedEvent = (function () {
                function UploadFinishedEvent(uploadItem) {
                    this.uploadItem = uploadItem;
                }
                UploadFinishedEvent.prototype.getUploadItem = function () {
                    return this.uploadItem;
                };
                return UploadFinishedEvent;
            })();
            wizard.UploadFinishedEvent = UploadFinishedEvent;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (_wizard) {
            var WizardClosedEvent = (function () {
                function WizardClosedEvent(wizard) {
                    this.wizard = wizard;
                }
                WizardClosedEvent.prototype.getWizard = function () {
                    return this.wizard;
                };
                return WizardClosedEvent;
            })();
            _wizard.WizardClosedEvent = WizardClosedEvent;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var WizardValidityManager = (function () {
                function WizardValidityManager() {
                    this.validityChangedListeners = [];
                    this.steps = [];
                    this.validityChangedListeners = [];
                }
                WizardValidityManager.prototype.clearItems = function () {
                    this.steps = [];
                };
                WizardValidityManager.prototype.getSteps = function () {
                    return this.steps;
                };
                WizardValidityManager.prototype.addItem = function (step) {
                    var _this = this;
                    this.steps.push(step);
                    step.getStepForm().onValidityChanged(function (event) {
                        _this.notifyValidityChanged(event.isValid());
                    });
                };
                WizardValidityManager.prototype.removeItem = function (step) {
                    var index = this.steps.indexOf(step);
                    if (index >= 0) {
                        this.steps.splice(index, 1);
                    }
                };
                WizardValidityManager.prototype.setHeader = function (header) {
                    var _this = this;
                    this.header = header;
                    this.header.onPropertyChanged(function (event) {
                        _this.notifyValidityChanged(_this.header.isValid());
                    });
                };
                WizardValidityManager.prototype.isAllValid = function () {
                    if (this.header && !this.header.isValid()) {
                        return false;
                    }
                    for (var i = 0; i < this.steps.length; i++) {
                        if (!this.steps[i].getStepForm().isValid()) {
                            return false;
                        }
                    }
                    return true;
                };
                WizardValidityManager.prototype.onValidityChanged = function (listener) {
                    this.validityChangedListeners.push(listener);
                };
                WizardValidityManager.prototype.unValidityChanged = function (listener) {
                    this.validityChangedListeners = this.validityChangedListeners.filter(function (curr) {
                        return curr != listener;
                    });
                };
                WizardValidityManager.prototype.notifyValidityChanged = function (valid) {
                    var _this = this;
                    this.validityChangedListeners.forEach(function (listener) {
                        listener.call(_this, new api.ValidityChangedEvent(valid));
                    });
                };
                return WizardValidityManager;
            })();
            wizard.WizardValidityManager = WizardValidityManager;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var app;
    (function (app) {
        var wizard;
        (function (wizard) {
            var MaskContentWizardPanelEvent = (function (_super) {
                __extends(MaskContentWizardPanelEvent, _super);
                function MaskContentWizardPanelEvent(contentId, mask) {
                    if (mask === void 0) { mask = true; }
                    _super.call(this);
                    this.contentId = contentId;
                    this.mask = mask;
                }
                MaskContentWizardPanelEvent.prototype.isMask = function () {
                    return this.mask;
                };
                MaskContentWizardPanelEvent.prototype.getContentId = function () {
                    return this.contentId;
                };
                MaskContentWizardPanelEvent.on = function (handler) {
                    api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
                };
                MaskContentWizardPanelEvent.un = function (handler) {
                    api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
                };
                return MaskContentWizardPanelEvent;
            })(api.event.Event);
            wizard.MaskContentWizardPanelEvent = MaskContentWizardPanelEvent;
        })(wizard = app.wizard || (app.wizard = {}));
    })(app = api.app || (api.app = {}));
})(api || (api = {}));
var api;
(function (api) {
    var system;
    (function (system) {
        var StatusRequest = (function (_super) {
            __extends(StatusRequest, _super);
            function StatusRequest() {
                _super.apply(this, arguments);
            }
            StatusRequest.prototype.getRequestPath = function () {
                return api.rest.Path.fromParent(_super.prototype.getRestPath.call(this), "status");
            };
            StatusRequest.prototype.getParams = function () {
                return {};
            };
            StatusRequest.prototype.sendAndParse = function () {
                return this.send().then(function (response) {
                    return new system.StatusResult(response.getResult());
                });
            };
            return StatusRequest;
        })(api.rest.ResourceRequest);
        system.StatusRequest = StatusRequest;
    })(system = api.system || (api.system = {}));
})(api || (api = {}));
var api;
(function (api) {
    var system;
    (function (system) {
        var StatusResult = (function () {
            function StatusResult(json) {
                this.version = json.version;
                this.installation = json.installation;
                if (json.context) {
                    this.authenticated = json.context.authenticated;
                    this.principals = json.context.principals ? json.context.principals.map(function (principal) { return api.security.PrincipalKey.fromString(principal); }) : [];
                }
                else {
                    this.authenticated = false;
                    this.principals = [];
                }
            }
            StatusResult.prototype.isAuthenticated = function () {
                return this.authenticated;
            };
            StatusResult.prototype.getInstallation = function () {
                return this.installation;
            };
            StatusResult.prototype.getVersion = function () {
                return this.version;
            };
            StatusResult.prototype.getPrincipals = function () {
                return this.principals;
            };
            return StatusResult;
        })();
        system.StatusResult = StatusResult;
    })(system = api.system || (api.system = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PropertyTree = api.data.PropertyTree;
        var GetPageDescriptorByKeyRequest = api.content.page.GetPageDescriptorByKeyRequest;
        var SetTemplate = api.content.page.SetTemplate;
        var PageModel = api.content.page.PageModel;
        var SetController = api.content.page.SetController;
        var Regions = api.content.page.region.Regions;
        var PageMode = api.content.page.PageMode;
        var GetPageTemplateByKeyRequest = api.content.page.GetPageTemplateByKeyRequest;
        var LiveEditModel = (function () {
            function LiveEditModel(builder) {
                this.siteModel = builder.siteModel;
                this.parentContent = builder.parentContent;
                this.content = builder.content;
                this.formContext = builder.formContext;
            }
            LiveEditModel.prototype.init = function (defaultTemplate, defaultTemplateDescriptor) {
                var _this = this;
                return this.initPageModel(defaultTemplate, defaultTemplateDescriptor).then(function (pageModel) {
                    _this.pageModel = pageModel;
                });
            };
            LiveEditModel.prototype.initPageModel = function (defaultPageTemplate, defaultTemplateDescriptor) {
                var _this = this;
                var deferred = wemQ.defer();
                var pageMode = this.getPageMode(this.content, !!defaultPageTemplate);
                var pageModel = new PageModel(this, defaultPageTemplate, defaultTemplateDescriptor, pageMode);
                var pageDescriptorPromise = null;
                var pageTemplatePromise = null;
                if (this.content.isPageTemplate()) {
                    var pageTemplate = this.content;
                    if (pageMode == 2 /* FORCED_CONTROLLER */) {
                        var pageDescriptorKey = pageTemplate.getController();
                        pageDescriptorPromise = this.loadPageDescriptor(pageDescriptorKey);
                        pageDescriptorPromise.then(function (pageDescriptor) {
                            var config = pageTemplate.hasConfig() ? pageTemplate.getPage().getConfig().copy() : new PropertyTree();
                            var regions = pageTemplate.hasRegions() ? pageTemplate.getRegions().clone() : Regions.create().build();
                            var setController = new SetController(_this).setDescriptor(pageDescriptor).setConfig(config).setRegions(regions);
                            pageModel.initController(setController);
                        });
                    }
                    else if (pageMode == 3 /* NO_CONTROLLER */) {
                        var config = pageTemplate.hasConfig() ? pageTemplate.getConfig().copy() : new PropertyTree();
                        var regions = pageTemplate.hasRegions() ? pageTemplate.getRegions().clone() : Regions.create().build();
                        var setController = new SetController(this).setDescriptor(null).setConfig(config).setRegions(regions);
                        pageModel.initController(setController);
                    }
                    else {
                        throw new Error("Unsupported PageMode for a PageTemplate: " + pageMode);
                    }
                }
                else {
                    var page = this.content.getPage();
                    if (pageMode == 1 /* FORCED_TEMPLATE */) {
                        var pageTemplateKey = page.getTemplate();
                        pageTemplatePromise = this.loadPageTemplate(pageTemplateKey);
                        pageTemplatePromise.then(function (pageTemplate) {
                            var pageDescriptorKey = pageTemplate.getController();
                            pageDescriptorPromise = _this.loadPageDescriptor(pageDescriptorKey);
                            pageDescriptorPromise.then(function (pageDescriptor) {
                                var config = _this.content.getPage().hasConfig() ? _this.content.getPage().getConfig().copy() : pageTemplate.getConfig().copy();
                                var regions = _this.content.getPage().hasRegions() ? _this.content.getPage().getRegions().clone() : pageTemplate.getRegions().clone();
                                var setTemplate = new SetTemplate(_this).setTemplate(pageTemplate, pageDescriptor).setRegions(regions).setConfig(config);
                                pageModel.initTemplate(setTemplate);
                            });
                        });
                    }
                    else if (pageMode == 2 /* FORCED_CONTROLLER */) {
                        var pageDescriptorKey = page.getController();
                        pageDescriptorPromise = this.loadPageDescriptor(pageDescriptorKey);
                        pageDescriptorPromise.then(function (pageDescriptor) {
                            var config = page.hasConfig() ? page.getConfig().copy() : new PropertyTree();
                            var regions = page.hasRegions() ? page.getRegions().clone() : Regions.create().build();
                            var setController = new SetController(_this).setDescriptor(pageDescriptor).setConfig(config).setRegions(regions);
                            pageModel.initController(setController);
                        });
                    }
                    else if (pageMode == 0 /* AUTOMATIC */) {
                        pageModel.setAutomaticTemplate(this);
                    }
                    else if (pageMode == 3 /* NO_CONTROLLER */) {
                        var config = new PropertyTree();
                        var regions = Regions.create().build();
                        var setController = new SetController(this).setDescriptor(null).setConfig(config).setRegions(regions);
                        pageModel.initController(setController);
                    }
                    else {
                        throw new Error("Unsupported PageMode for a Content: " + PageMode[pageMode]);
                    }
                }
                var promises = [];
                if (pageDescriptorPromise) {
                    promises.push(pageDescriptorPromise);
                }
                if (pageTemplatePromise) {
                    promises.push(pageTemplatePromise);
                }
                if (promises.length > 0) {
                    wemQ.all(promises).then(function () {
                        deferred.resolve(pageModel);
                    }).catch(function (reason) {
                        deferred.reject(reason);
                    }).done();
                }
                else {
                    deferred.resolve(pageModel);
                }
                return deferred.promise;
            };
            LiveEditModel.prototype.getPageMode = function (content, defaultTemplatePresents) {
                if (content.isPage()) {
                    if (content.getPage().hasTemplate()) {
                        if (defaultTemplatePresents) {
                            return 1 /* FORCED_TEMPLATE */;
                        }
                        else {
                            return 3 /* NO_CONTROLLER */;
                        }
                    }
                    else {
                        return 2 /* FORCED_CONTROLLER */;
                    }
                }
                else if (defaultTemplatePresents) {
                    return 0 /* AUTOMATIC */;
                }
                else {
                    return 3 /* NO_CONTROLLER */;
                }
            };
            LiveEditModel.prototype.loadPageTemplate = function (key) {
                var deferred = wemQ.defer();
                new GetPageTemplateByKeyRequest(key).sendAndParse().then(function (pageTemplate) {
                    deferred.resolve(pageTemplate);
                }).catch(function (reason) {
                    deferred.reject(new api.Exception("Page template '" + key + "' not found.", 2 /* WARNING */));
                }).done();
                return deferred.promise;
            };
            LiveEditModel.prototype.loadPageDescriptor = function (key) {
                var deferred = wemQ.defer();
                new GetPageDescriptorByKeyRequest(key).sendAndParse().then(function (pageDescriptor) {
                    deferred.resolve(pageDescriptor);
                }).catch(function (reason) {
                    deferred.reject(new api.Exception("Page descriptor '" + key + "' not found.", 2 /* WARNING */));
                }).done();
                return deferred.promise;
            };
            LiveEditModel.prototype.setContent = function (value) {
                this.content = value;
            };
            LiveEditModel.prototype.getParentContent = function () {
                return this.parentContent;
            };
            LiveEditModel.prototype.getFormContext = function () {
                return this.formContext;
            };
            LiveEditModel.prototype.getContent = function () {
                return this.content;
            };
            LiveEditModel.prototype.getSiteModel = function () {
                return this.siteModel;
            };
            LiveEditModel.prototype.getPageModel = function () {
                return this.pageModel;
            };
            LiveEditModel.create = function () {
                return new LiveEditModelBuilder();
            };
            return LiveEditModel;
        })();
        liveedit.LiveEditModel = LiveEditModel;
        var LiveEditModelBuilder = (function () {
            function LiveEditModelBuilder() {
            }
            LiveEditModelBuilder.prototype.setSiteModel = function (value) {
                this.siteModel = value;
                return this;
            };
            LiveEditModelBuilder.prototype.setParentContent = function (value) {
                this.parentContent = value;
                return this;
            };
            LiveEditModelBuilder.prototype.setContent = function (value) {
                this.content = value;
                return this;
            };
            LiveEditModelBuilder.prototype.setContentFormContext = function (value) {
                this.formContext = value;
                return this;
            };
            LiveEditModelBuilder.prototype.build = function () {
                return new LiveEditModel(this);
            };
            return LiveEditModelBuilder;
        })();
        liveedit.LiveEditModelBuilder = LiveEditModelBuilder;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var StringHelper = api.util.StringHelper;
        var ItemType = (function () {
            function ItemType(shortName, config) {
                ItemType.shortNameToInstance[shortName] = this;
                this.shortName = shortName;
                this.config = new liveedit.ItemTypeConfig(config);
            }
            ItemType.prototype.getShortName = function () {
                return this.shortName;
            };
            ItemType.prototype.getConfig = function () {
                return this.config;
            };
            ItemType.prototype.isComponentType = function () {
                return false;
            };
            ItemType.prototype.toComponentType = function () {
                api.util.assert(this.isComponentType(), "Not support when ItemType is not a ComponentType");
                return api.content.page.region.ComponentType.byShortName(this.shortName);
            };
            ItemType.prototype.createView = function (config) {
                throw new Error("Must be implemented by inheritors");
            };
            ItemType.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ItemType)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.shortName, other.shortName)) {
                    return false;
                }
                return true;
            };
            ItemType.getDraggables = function () {
                var draggables = [];
                for (var shortName in ItemType.shortNameToInstance) {
                    var itemType = ItemType.shortNameToInstance[shortName];
                    if (itemType.getConfig().isDraggable()) {
                        draggables.push(itemType);
                    }
                }
                return draggables;
            };
            ItemType.byShortName = function (shortName) {
                return ItemType.shortNameToInstance[shortName];
            };
            ItemType.fromHTMLElement = function (element) {
                var typeAsString = element.getAttribute("data-" + ItemType.ATTRIBUTE_TYPE);
                if (StringHelper.isBlank(typeAsString)) {
                    var regionName = element.getAttribute("data-" + ItemType.ATTRIBUTE_REGION_NAME);
                    if (!StringHelper.isBlank(regionName)) {
                        typeAsString = "region";
                    }
                }
                return ItemType.byShortName(typeAsString);
            };
            ItemType.fromElement = function (element) {
                return ItemType.fromHTMLElement(element.getHTMLElement());
            };
            ItemType.ATTRIBUTE_TYPE = "portal-component-type";
            ItemType.ATTRIBUTE_REGION_NAME = "portal-region";
            ItemType.shortNameToInstance = {};
            return ItemType;
        })();
        liveedit.ItemType = ItemType;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var CreateItemViewConfig = (function () {
            function CreateItemViewConfig() {
                this.positionIndex = -1;
            }
            CreateItemViewConfig.prototype.setItemViewProducer = function (value) {
                this.itemViewProducer = value;
                return this;
            };
            CreateItemViewConfig.prototype.setParentView = function (value) {
                this.parentView = value;
                return this;
            };
            CreateItemViewConfig.prototype.setParentElement = function (value) {
                this.parentElement = value;
                return this;
            };
            CreateItemViewConfig.prototype.setData = function (value) {
                this.data = value;
                return this;
            };
            CreateItemViewConfig.prototype.setElement = function (value) {
                this.element = value;
                return this;
            };
            CreateItemViewConfig.prototype.setPositionIndex = function (value) {
                this.positionIndex = value;
                return this;
            };
            return CreateItemViewConfig;
        })();
        liveedit.CreateItemViewConfig = CreateItemViewConfig;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemTypeConfig = (function () {
            function ItemTypeConfig(json) {
                this.typeName = json.typeName;
                this.draggable = json.draggable;
                this.cssSelector = json.cssSelector;
                this.iconCls = json.iconCls;
                this.cursor = json.cursor;
                this.highlighterStyle = json.highlighterStyle;
                this.contextMenuConfig = json.contextMenuConfig;
            }
            ItemTypeConfig.prototype.getName = function () {
                return this.typeName;
            };
            ItemTypeConfig.prototype.isDraggable = function () {
                return this.draggable;
            };
            ItemTypeConfig.prototype.getCssSelector = function () {
                return this.cssSelector;
            };
            ItemTypeConfig.prototype.getIconCls = function () {
                return this.iconCls;
            };
            ItemTypeConfig.prototype.getCursor = function () {
                return this.cursor;
            };
            ItemTypeConfig.prototype.getHighlighterStyle = function () {
                return this.highlighterStyle;
            };
            ItemTypeConfig.prototype.getContextMenuConfig = function () {
                return this.contextMenuConfig;
            };
            return ItemTypeConfig;
        })();
        liveedit.ItemTypeConfig = ItemTypeConfig;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var RegionItemType = (function (_super) {
            __extends(RegionItemType, _super);
            function RegionItemType() {
                _super.call(this, "region", {
                    cssSelector: '[data-portal-region]',
                    draggable: false,
                    cursor: 'pointer',
                    iconCls: 'live-edit-font-icon-region',
                    highlighterStyle: {
                        stroke: 'rgba(20, 20, 20, 1)',
                        strokeDasharray: '',
                        fill: 'rgba(255, 255, 255, 0)'
                    },
                    contextMenuConfig: ['parent', 'clearRegion']
                });
            }
            RegionItemType.get = function () {
                return RegionItemType.INSTANCE;
            };
            RegionItemType.getRegionName = function (element) {
                return element.getEl().getAttribute('data-' + liveedit.ItemType.ATTRIBUTE_REGION_NAME);
            };
            RegionItemType.prototype.createView = function (config) {
                return new liveedit.RegionView(new liveedit.RegionViewBuilder().setParentView(config.parentView).setParentElement(config.parentElement).setRegion(config.data).setElement(config.element));
            };
            RegionItemType.INSTANCE = new RegionItemType();
            return RegionItemType;
        })(liveedit.ItemType);
        liveedit.RegionItemType = RegionItemType;
        RegionItemType.get();
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var InitializeLiveEditEvent = (function (_super) {
            __extends(InitializeLiveEditEvent, _super);
            function InitializeLiveEditEvent(liveEditModel) {
                _super.call(this);
                this.liveEditModel = liveEditModel;
            }
            InitializeLiveEditEvent.prototype.getLiveEditModel = function () {
                return this.liveEditModel;
            };
            InitializeLiveEditEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            InitializeLiveEditEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return InitializeLiveEditEvent;
        })(api.event.Event);
        liveedit.InitializeLiveEditEvent = InitializeLiveEditEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var SkipLiveEditReloadConfirmationEvent = (function (_super) {
            __extends(SkipLiveEditReloadConfirmationEvent, _super);
            function SkipLiveEditReloadConfirmationEvent(skip) {
                _super.call(this);
                this.skip = skip;
            }
            SkipLiveEditReloadConfirmationEvent.prototype.isSkip = function () {
                return this.skip;
            };
            SkipLiveEditReloadConfirmationEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            SkipLiveEditReloadConfirmationEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return SkipLiveEditReloadConfirmationEvent;
        })(api.event.Event);
        liveedit.SkipLiveEditReloadConfirmationEvent = SkipLiveEditReloadConfirmationEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ComponentItemType = (function (_super) {
            __extends(ComponentItemType, _super);
            function ComponentItemType(shortName, config) {
                _super.call(this, shortName, config);
            }
            ComponentItemType.prototype.createView = function (config) {
                throw new Error("Must be implemented by inheritors");
            };
            return ComponentItemType;
        })(liveedit.ItemType);
        liveedit.ComponentItemType = ComponentItemType;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageItemType = (function (_super) {
            __extends(PageItemType, _super);
            function PageItemType() {
                _super.call(this, "page", {
                    cssSelector: '[data-portal-component-type=page]',
                    draggable: false,
                    cursor: 'pointer',
                    iconCls: 'live-edit-font-icon-page',
                    highlighterStyle: {
                        stroke: 'rgba(20, 20, 20, 1)',
                        strokeDasharray: '',
                        fill: 'rgba(255, 255, 255, 0)'
                    },
                    contextMenuConfig: ['reset']
                });
            }
            PageItemType.get = function () {
                return PageItemType.INSTANCE;
            };
            PageItemType.prototype.createView = function (config) {
                throw new Error("Not supported");
            };
            PageItemType.INSTANCE = new PageItemType();
            return PageItemType;
        })(liveedit.ItemType);
        liveedit.PageItemType = PageItemType;
        PageItemType.get();
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ContentItemType = (function (_super) {
            __extends(ContentItemType, _super);
            function ContentItemType() {
                _super.call(this, "content", {
                    cssSelector: '[data-portal-component-type=content]',
                    draggable: false,
                    cursor: 'pointer',
                    iconCls: 'live-edit-font-icon-content',
                    highlighterStyle: {
                        stroke: '',
                        strokeDasharray: '',
                        fill: 'rgba(0, 108, 255, .25)'
                    },
                    contextMenuConfig: ['parent', 'opencontent', 'view']
                });
            }
            ContentItemType.get = function () {
                return ContentItemType.INSTANCE;
            };
            ContentItemType.prototype.createView = function (config) {
                return new liveedit.ContentView(new liveedit.ContentViewBuilder().setParentPartComponentView(config.parentView).setParentElement(config.parentElement).setElement(config.element));
            };
            ContentItemType.INSTANCE = new ContentItemType();
            return ContentItemType;
        })(liveedit.ItemType);
        liveedit.ContentItemType = ContentItemType;
        ContentItemType.get();
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var DivEl = api.dom.DivEl;
        var Body = api.dom.Body;
        var Shader = (function () {
            function Shader() {
                var _this = this;
                this.scrollEnabled = true;
                this.clickListeners = [];
                this.unlockClickedListeners = [];
                this.mouseEnterListeners = [];
                this.mouseLeaveListeners = [];
                this.pageShader = new DivEl(Shader.CLS_NAME + " page");
                this.northShader = new DivEl(Shader.CLS_NAME + " north");
                this.eastShader = new DivEl(Shader.CLS_NAME + " east");
                this.southShader = new DivEl(Shader.CLS_NAME + " south");
                this.westShader = new DivEl(Shader.CLS_NAME + " west");
                this.shaders = [this.pageShader, this.northShader, this.eastShader, this.southShader, this.westShader];
                var body = Body.get();
                body.appendChildren.apply(body, this.shaders);
                body.onMouseWheel(function (event) {
                    if (_this.target && _this.isVisible()) {
                        if (Shader.debug) {
                            console.log('Shader.onMouseWheel, scroll enabled = ' + _this.scrollEnabled);
                        }
                        if (!_this.scrollEnabled) {
                            event.preventDefault();
                            event.stopPropagation();
                        }
                        else {
                            setTimeout(function () { return _this.resizeToElement(_this.target); }, 5);
                        }
                    }
                });
                this.shaders.forEach(function (shader) {
                    shader.onClicked(function (event) { return _this.handleClick(event); });
                    shader.onContextMenu(function (event) { return _this.handleClick(event); });
                    shader.onMouseEnter(function (event) { return _this.notifyMouseEntered(event); });
                    shader.onMouseLeave(function (event) { return _this.notifyMouseLeft(event); });
                });
            }
            Shader.get = function () {
                if (!Shader.INSTANCE) {
                    Shader.INSTANCE = new Shader();
                }
                return Shader.INSTANCE;
            };
            Shader.prototype.setScrollEnabled = function (enabled) {
                this.scrollEnabled = enabled;
                return this;
            };
            Shader.prototype.shade = function (element) {
                if (!element) {
                    this.hide();
                    return;
                }
                if (api.ObjectHelper.iFrameSafeInstanceOf(element, liveedit.PageView)) {
                    this.resizeToPage();
                }
                else {
                    this.resizeToElement(element);
                }
            };
            Shader.prototype.hide = function () {
                this.target = undefined;
                this.shaders.forEach(function (shader) { return shader.hide(); });
            };
            Shader.prototype.isVisible = function () {
                return this.shaders.some(function (shader) { return shader.isVisible(); });
            };
            Shader.prototype.onUnlockClicked = function (listener) {
                this.unlockClickedListeners.push(listener);
            };
            Shader.prototype.unUnlockClicked = function (listener) {
                this.unlockClickedListeners = this.unlockClickedListeners.filter(function (curr) {
                    return listener != curr;
                });
            };
            Shader.prototype.notifyUnlockClicked = function (event) {
                this.unlockClickedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            Shader.prototype.onMouseEnter = function (listener) {
                this.mouseEnterListeners.push(listener);
            };
            Shader.prototype.unMouseEnter = function (listener) {
                this.mouseEnterListeners = this.mouseEnterListeners.filter(function (curr) {
                    return listener != curr;
                });
            };
            Shader.prototype.notifyMouseEntered = function (event) {
                this.mouseEnterListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            Shader.prototype.onMouseLeave = function (listener) {
                this.mouseLeaveListeners.push(listener);
            };
            Shader.prototype.unMouseLeave = function (listener) {
                this.mouseLeaveListeners = this.mouseLeaveListeners.filter(function (curr) {
                    return listener != curr;
                });
            };
            Shader.prototype.notifyMouseLeft = function (event) {
                this.mouseLeaveListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            Shader.prototype.onClicked = function (listener) {
                this.clickListeners.push(listener);
            };
            Shader.prototype.unClicked = function (listener) {
                this.clickListeners = this.clickListeners.filter(function (curr) {
                    return listener != curr;
                });
            };
            Shader.prototype.notifyClicked = function (event) {
                this.clickListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            Shader.prototype.handleUnlockClick = function (event) {
                event.stopPropagation();
                event.preventDefault();
                this.notifyUnlockClicked(event);
            };
            Shader.prototype.handleClick = function (event) {
                event.stopPropagation();
                event.preventDefault();
                this.notifyClicked(event);
            };
            Shader.prototype.showShaderIfNecessary = function (shader, x, y, width, height) {
                var shaderEl = shader.getEl();
                shaderEl.setTopPx(y).setLeftPx(x).setWidthPx(width).setHeightPx(height);
                shader.setVisible(width > 0 && height > 0);
            };
            Shader.prototype.resizeToPage = function () {
                if (Shader.debug) {
                    console.log('Shader.resizeToPage');
                }
                this.target = undefined;
                this.pageShader.getEl().setTopPx(0).setRightPx(0).setBottomPx(0).setLeftPx(0);
                this.pageShader.show();
            };
            Shader.prototype.resizeToElement = function (element) {
                this.target = element;
                var win = api.dom.WindowDOM.get(), bodyEl = api.dom.Body.get().getEl(), documentWidth = Math.max(win.getWidth(), bodyEl.getWidth()), documentHeight = Math.max(win.getHeight(), bodyEl.getHeight());
                var dimensions = element.getEl().getDimensions(), x1 = Math.max(0, dimensions.left), y1 = Math.max(0, dimensions.top), x2 = Math.min(documentWidth, dimensions.left + dimensions.width), y2 = Math.min(documentHeight, dimensions.top + dimensions.height);
                if (Shader.debug) {
                    console.log('Shader.resizeToElement(' + x1 + ', ' + y1 + ', ' + x2 + ', ' + y2 + ')', element);
                }
                this.showShaderIfNecessary(this.northShader, 0, 0, documentWidth, y1);
                this.showShaderIfNecessary(this.eastShader, x2, y1, documentWidth - x2, y2 - y1);
                this.showShaderIfNecessary(this.southShader, 0, y2, documentWidth, documentHeight - y2);
                this.showShaderIfNecessary(this.westShader, 0, y1, x1, y2 - y1);
            };
            Shader.CLS_NAME = 'shader';
            Shader.debug = false;
            return Shader;
        })();
        liveedit.Shader = Shader;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Highlighter = (function (_super) {
            __extends(Highlighter, _super);
            function Highlighter() {
                var html = '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" class="live-edit-highlight-border" style="top:-5000px;left:-5000px">' + '    <rect width="150" height="150"/>' + '</svg>';
                _super.call(this, new api.dom.ElementFromHelperBuilder().setHelper(api.dom.Element.fromString(html).getEl()).setLoadExistingChildren(true));
                this.rectangle = this.getChildren()[0];
                this.appendChild(this.rectangle);
                api.dom.Body.get().appendChild(this);
            }
            Highlighter.get = function () {
                if (!Highlighter.INSTANCE) {
                    Highlighter.INSTANCE = new Highlighter();
                }
                return Highlighter.INSTANCE;
            };
            Highlighter.prototype.highlightItemView = function (itemView) {
                if (!itemView) {
                    this.hide();
                    return;
                }
                this.resizeToComponent(itemView);
                this.show();
            };
            Highlighter.prototype.resizeToComponent = function (itemView) {
                var itemDimensions = itemView.getElementDimensions();
                var w = Math.round(itemDimensions.width), h = Math.round(itemDimensions.height), top = Math.round(itemDimensions.top), left = Math.round(itemDimensions.left);
                this.getEl().setWidthPx(w).setHeightPx(h).setTopPx(top).setLeftPx(left);
                this.rectangle.getEl().setAttribute('width', w + '').setAttribute('height', h + '');
                var style = itemView.getType().getConfig().getHighlighterStyle();
                this.getEl().setStroke(style.stroke).setStrokeDasharray(style.strokeDasharray).setFill(style.fill);
            };
            return Highlighter;
        })(api.dom.Element);
        liveedit.Highlighter = Highlighter;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Body = api.dom.Body;
        var Cursor = (function () {
            function Cursor() {
                this.defaultBodyCursor = Body.get().getEl().getCursor();
            }
            Cursor.get = function () {
                if (!Cursor.INSTANCE) {
                    Cursor.INSTANCE = new Cursor();
                }
                return Cursor.INSTANCE;
            };
            Cursor.prototype.displayItemViewCursor = function (itemView) {
                if (!itemView) {
                    return;
                }
                Body.get().getEl().setCursor(itemView.getType().getConfig().getCursor());
            };
            Cursor.prototype.hide = function () {
                Body.get().getEl().setCursor('none');
            };
            Cursor.prototype.reset = function () {
                Body.get().getEl().setCursor(this.defaultBodyCursor || '');
            };
            return Cursor;
        })();
        liveedit.Cursor = Cursor;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewId = (function () {
            function ItemViewId(value) {
                api.util.assert(value >= 1, "An ItemViewId must be 1 or larger");
                this.value = value;
                this.refString = "" + value;
            }
            ItemViewId.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ItemViewId)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.numberEquals(this.value, other.value)) {
                    return false;
                }
                return true;
            };
            ItemViewId.prototype.toNumber = function () {
                return this.value;
            };
            ItemViewId.prototype.toString = function () {
                return this.refString;
            };
            ItemViewId.fromString = function (s) {
                return new ItemViewId(+s);
            };
            ItemViewId.DATA_ATTRIBUTE = "live-edit-id";
            return ItemViewId;
        })();
        liveedit.ItemViewId = ItemViewId;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewIdProducer = (function () {
            function ItemViewIdProducer() {
                this.itemViewCounter = 0;
            }
            ItemViewIdProducer.prototype.next = function () {
                return new liveedit.ItemViewId(++this.itemViewCounter);
            };
            return ItemViewIdProducer;
        })();
        liveedit.ItemViewIdProducer = ItemViewIdProducer;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var RepeatNextItemViewIdProducer = (function (_super) {
            __extends(RepeatNextItemViewIdProducer, _super);
            function RepeatNextItemViewIdProducer(idToRepeatNext, itemViewProducer) {
                _super.call(this);
                this.idToRepeatNext = idToRepeatNext;
                this.itemViewProducer = itemViewProducer;
                this.repeated = false;
            }
            RepeatNextItemViewIdProducer.prototype.next = function () {
                if (!this.repeated) {
                    this.repeated = true;
                    return this.idToRepeatNext;
                }
                else {
                    return this.itemViewProducer.next();
                }
            };
            return RepeatNextItemViewIdProducer;
        })(liveedit.ItemViewIdProducer);
        liveedit.RepeatNextItemViewIdProducer = RepeatNextItemViewIdProducer;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewBuilder = (function () {
            function ItemViewBuilder() {
            }
            ItemViewBuilder.prototype.setLiveEditModel = function (value) {
                this.liveEditModel = value;
                return this;
            };
            ItemViewBuilder.prototype.setItemViewIdProducer = function (value) {
                this.itemViewIdProducer = value;
                return this;
            };
            ItemViewBuilder.prototype.setType = function (value) {
                this.type = value;
                return this;
            };
            ItemViewBuilder.prototype.setElement = function (value) {
                this.element = value;
                return this;
            };
            ItemViewBuilder.prototype.setPlaceholder = function (value) {
                this.placeholder = value;
                return this;
            };
            ItemViewBuilder.prototype.setTooltipViewer = function (value) {
                this.tooltipViewer = value;
                return this;
            };
            ItemViewBuilder.prototype.setParentView = function (value) {
                this.parentView = value;
                return this;
            };
            ItemViewBuilder.prototype.setParentElement = function (value) {
                this.parentElement = value;
                return this;
            };
            ItemViewBuilder.prototype.setContextMenuActions = function (actions) {
                this.contextMenuActions = actions;
                return this;
            };
            ItemViewBuilder.prototype.setContextMenuTitle = function (title) {
                this.contextMenuTitle = title;
                return this;
            };
            return ItemViewBuilder;
        })();
        liveedit.ItemViewBuilder = ItemViewBuilder;
        var ItemView = (function (_super) {
            __extends(ItemView, _super);
            function ItemView(builder) {
                api.util.assertNotNull(builder.type, "type cannot be null");
                this.type = builder.type;
                this.parentItemView = builder.parentView;
                this.liveEditModel = builder.liveEditModel ? builder.liveEditModel : builder.parentView.liveEditModel;
                this.itemViewIdProducer = builder.itemViewIdProducer;
                this.contextMenuActions = builder.contextMenuActions;
                this.contextMenuTitle = builder.contextMenuTitle;
                var props = null;
                if (builder.element) {
                    var elementFromElementBuilder = new api.dom.ElementFromElementBuilder();
                    elementFromElementBuilder.setElement(builder.element);
                    elementFromElementBuilder.setParentElement(builder.parentElement);
                    elementFromElementBuilder.setGenerateId(false);
                    props = elementFromElementBuilder;
                }
                else {
                    var newElementBuilder = new api.dom.NewElementBuilder();
                    newElementBuilder.setTagName("div");
                    newElementBuilder.setParentElement(builder.parentElement);
                    newElementBuilder.setGenerateId(false);
                    props = newElementBuilder;
                }
                _super.call(this, props);
                this.addClass("item-view");
                this.setDraggable(true);
                this.mouseOver = false;
                this.mouseOverViewListeners = [];
                this.mouseOutViewListeners = [];
                this.setItemId(builder.itemViewIdProducer.next());
                if (!builder.element) {
                    this.getEl().setData(liveedit.ItemType.ATTRIBUTE_TYPE, builder.type.getShortName());
                }
                for (var i = 0; i < this.getChildren().length; i++) {
                    var child = this.getChildren()[i];
                    if (api.ObjectHelper.iFrameSafeInstanceOf(child, liveedit.ItemViewPlaceholder)) {
                        this.removeChild(child);
                        break;
                    }
                }
                if (builder.placeholder) {
                    this.placeholder = builder.placeholder;
                    this.appendChild(this.placeholder);
                }
                this.onRemoved(this.invalidateContextMenu.bind(this));
                this.bindMouseListeners();
            }
            ItemView.prototype.bindMouseListeners = function () {
                var _this = this;
                var pageView = this.getPageView();
                this.mouseEnterListener = this.handleMouseEnter.bind(this);
                this.onMouseEnter(this.mouseEnterListener);
                this.mouseLeaveListener = this.handleMouseLeave.bind(this);
                this.onMouseLeave(this.mouseLeaveListener);
                this.mouseClickedListener = this.handleClick.bind(this);
                this.onClicked(this.mouseClickedListener);
                this.onTouchStart(this.mouseClickedListener);
                this.contextMenuListener = this.handleClick.bind(this);
                this.onContextMenu(this.contextMenuListener);
                api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                    if (_this.isSelected()) {
                        _this.highlight();
                        _this.shade();
                    }
                });
                this.shaderClickedListener = this.handleShaderClick.bind(this);
                liveedit.Shader.get().onClicked(this.shaderClickedListener);
                this.mouseOverViewListener = function () {
                    var isRegistered = !!_this.getParentItemView();
                    if (ItemView.debug) {
                        console.log('ItemView[' + _this.toString() + '].mouseOverViewListener registered: ' + isRegistered);
                    }
                    if (!isRegistered) {
                        return;
                    }
                    var hasSelectedView = pageView.hasSelectedView();
                    var isDragging = liveedit.DragAndDrop.get().isDragging();
                    if (!hasSelectedView && !isDragging) {
                        _this.showTooltip();
                        _this.showCursor();
                        _this.highlight();
                    }
                };
                this.onMouseOverView(this.mouseOverViewListener);
                this.mouseLeaveViewListener = function () {
                    var isRegistered = !!_this.getParentItemView();
                    if (ItemView.debug) {
                        console.log('ItemView[' + _this.toString() + '].mouseLeaveViewListener registered: ' + isRegistered);
                    }
                    if (!isRegistered) {
                        return;
                    }
                    var hasSelectedView = pageView.hasSelectedView();
                    var isDragging = liveedit.DragAndDrop.get().isDragging();
                    if (!hasSelectedView && !isDragging) {
                        _this.hideTooltip();
                        _this.resetCursor();
                        _this.unhighlight();
                    }
                };
                this.onMouseLeaveView(this.mouseLeaveViewListener);
                pageView.onItemViewAdded(this.shaderClickedListener);
                this.onRemoved(function () {
                    pageView.unItemViewAdded(_this.shaderClickedListener);
                });
            };
            ItemView.prototype.unbindMouseListeners = function () {
                this.unMouseEnter(this.mouseEnterListener);
                this.unMouseLeave(this.mouseLeaveListener);
                this.unClicked(this.mouseClickedListener);
                this.unTouchStart(this.mouseClickedListener);
                this.unContextMenu(this.contextMenuListener);
                api.ui.responsive.ResponsiveManager.unAvailableSizeChanged(this);
                liveedit.Shader.get().unClicked(this.shaderClickedListener);
                this.unMouseOverView(this.mouseOverViewListener);
                this.unMouseLeaveView(this.mouseLeaveViewListener);
            };
            ItemView.prototype.highlight = function () {
                liveedit.Highlighter.get().highlightItemView(this);
            };
            ItemView.prototype.unhighlight = function () {
                liveedit.Highlighter.get().hide();
            };
            ItemView.prototype.shade = function () {
                liveedit.Shader.get().shade(this);
            };
            ItemView.prototype.unshade = function () {
                liveedit.Shader.get().hide();
            };
            ItemView.prototype.showCursor = function () {
                liveedit.Cursor.get().displayItemViewCursor(this);
            };
            ItemView.prototype.resetCursor = function () {
                liveedit.Cursor.get().reset();
            };
            ItemView.prototype.getPageView = function () {
                var pageView = this;
                while (!liveedit.PageItemType.get().equals(pageView.getType())) {
                    pageView = pageView.parentItemView;
                }
                return pageView;
            };
            ItemView.prototype.remove = function () {
                if (ItemView.debug) {
                    console.log('ItemView.remove [' + this.toString() + ']');
                }
                if (this.contextMenu) {
                    this.contextMenu.remove();
                }
                if (this.loadMask) {
                    this.loadMask.remove();
                }
                this.unhighlight();
                this.unshade();
                this.unbindMouseListeners();
                _super.prototype.remove.call(this);
                return this;
            };
            ItemView.prototype.setDraggable = function (value) {
                this.toggleClass('not-draggable', !value);
            };
            ItemView.prototype.scrollComponentIntoView = function () {
                if (!this.visibleInViewport()) {
                    wemjq("html,body").animate({ scrollTop: this.getElementDimensions().top - 10 }, 200);
                }
            };
            ItemView.prototype.handleMouseEnter = function (event) {
                if (ItemView.debug) {
                    console.group("mouse enter [" + this.getId() + "]");
                }
                if (this.mouseOver) {
                    if (ItemView.debug) {
                        console.log('mouseOver = true, returning.');
                        console.groupEnd();
                    }
                    return;
                }
                this.manageParentsMouseOver();
                this.mouseOver = true;
                this.notifyMouseOverView();
                if (ItemView.debug) {
                    console.groupEnd();
                }
            };
            ItemView.prototype.manageParentsMouseOver = function () {
                var parentsStack = [];
                for (var parent = this.parentItemView; parent; parent = parent.parentItemView) {
                    parentsStack.push(parent);
                    if (parent.mouseOver) {
                        break;
                    }
                }
                parentsStack.reverse().forEach(function (view) {
                    if (view.mouseOver) {
                        if (ItemView.debug) {
                            console.debug('parent.mouseOver = true, notifying mouse out [' + view.getId() + "]");
                        }
                        view.notifyMouseLeaveView();
                    }
                    else {
                        view.mouseOver = true;
                        if (ItemView.debug) {
                            console.debug('parent.mouseOver = false, setting to true [' + view.getId() + "]");
                        }
                        view.notifyMouseOverView();
                        view.notifyMouseLeaveView();
                    }
                });
            };
            ItemView.prototype.handleMouseLeave = function (event) {
                if (ItemView.debug) {
                    console.group("mouse leave [" + this.getId() + "]");
                }
                this.mouseOver = false;
                this.notifyMouseLeaveView();
                if (this.tooltip) {
                    this.tooltip.hide();
                }
                if (this.parentItemView) {
                    this.parentItemView.notifyMouseOverView();
                }
                if (ItemView.debug) {
                    console.groupEnd();
                }
            };
            ItemView.prototype.isEmpty = function () {
                throw new Error("Must be implemented by inheritors");
            };
            ItemView.prototype.refreshEmptyState = function () {
                this.toggleClass('empty', this.isEmpty());
                return this;
            };
            ItemView.prototype.handleClick = function (event) {
                event.stopPropagation();
                if (!this.isSelected()) {
                    this.deselectParent();
                    this.getPageView().deselectChildViews();
                    this.select(!this.isEmpty() ? { x: event.pageX, y: event.pageY } : null);
                }
                else {
                    this.deselect();
                }
            };
            ItemView.prototype.handleShaderClick = function (event) {
                if (this.isSelected()) {
                    this.deselect();
                }
            };
            ItemView.prototype.deselectParent = function () {
                for (var parent = this.parentItemView; parent; parent = parent.parentItemView) {
                    if (parent.isSelected()) {
                        parent.deselect();
                        return;
                    }
                }
            };
            ItemView.prototype.getItemViewIdProducer = function () {
                return this.itemViewIdProducer;
            };
            ItemView.prototype.setTooltipObject = function (object) {
                if (this.tooltipViewer) {
                    this.tooltipViewer.setObject(object);
                }
            };
            ItemView.prototype.showTooltip = function () {
                if (ItemView.debug) {
                    console.log('showing tooltip [' + this.getId() + "]");
                }
                if (this.tooltip) {
                    this.tooltip.show();
                }
            };
            ItemView.prototype.hideTooltip = function (hideParentTooltip) {
                if (hideParentTooltip === void 0) { hideParentTooltip = true; }
                if (ItemView.debug) {
                    console.log('hiding tooltip [' + this.getId() + "]");
                }
                if (this.tooltip) {
                    this.tooltip.hide();
                }
                if (hideParentTooltip && this.parentItemView) {
                    this.parentItemView.hideTooltip();
                }
            };
            ItemView.prototype.showContextMenu = function (clickPosition, menuPosition) {
                if (this.getPageView().isDisabledContextMenu()) {
                    return;
                }
                if (!this.contextMenu) {
                    this.contextMenu = new api.liveedit.ItemViewContextMenu(this.contextMenuTitle, this.contextMenuActions);
                }
                var dimensions = this.getElementDimensions();
                var x, y;
                if (clickPosition) {
                    x = clickPosition.x;
                    y = clickPosition.y;
                }
                else {
                    x = dimensions.left + dimensions.width / 2;
                    y = dimensions.top + (0 /* TOP */ == menuPosition ? 0 : dimensions.height);
                }
                this.contextMenu.showAt(x, y, !clickPosition);
            };
            ItemView.prototype.hideContextMenu = function () {
                if (this.contextMenu) {
                    this.contextMenu.hide();
                }
            };
            ItemView.prototype.invalidateContextMenu = function () {
                if (this.contextMenu) {
                    this.contextMenu.remove();
                    this.contextMenu = null;
                }
            };
            ItemView.prototype.setItemId = function (value) {
                this.getEl().setAttribute("data-" + liveedit.ItemViewId.DATA_ATTRIBUTE, value.toString());
            };
            ItemView.prototype.getItemId = function () {
                var asString = this.getEl().getAttribute("data-" + liveedit.ItemViewId.DATA_ATTRIBUTE);
                if (!asString) {
                    return null;
                }
                return liveedit.ItemViewId.fromString(asString);
            };
            ItemView.parseItemId = function (element) {
                var attribute = element.getAttribute("data-" + liveedit.ItemViewId.DATA_ATTRIBUTE);
                if (api.util.StringHelper.isEmpty(attribute)) {
                    return null;
                }
                return liveedit.ItemViewId.fromString(attribute);
            };
            ItemView.prototype.getType = function () {
                return this.type;
            };
            ItemView.prototype.getParentItemView = function () {
                return this.parentItemView;
            };
            ItemView.prototype.setParentItemView = function (itemView) {
                this.parentItemView = itemView;
            };
            ItemView.prototype.isSelected = function () {
                return this.getEl().hasAttribute('data-live-edit-selected');
            };
            ItemView.prototype.select = function (clickPosition, menuPosition, isNew) {
                if (isNew === void 0) { isNew = false; }
                this.selectItem();
                this.showContextMenu(clickPosition, menuPosition);
                new liveedit.ItemViewSelectedEvent(this, clickPosition, isNew).fire();
            };
            ItemView.prototype.selectWithoutMenu = function (isNew) {
                if (isNew === void 0) { isNew = false; }
                this.selectItem();
                new liveedit.ItemViewSelectedEvent(this, null, isNew).fire();
            };
            ItemView.prototype.selectItem = function () {
                var selectedView = this.getPageView().getSelectedView();
                if (selectedView == this) {
                    return;
                }
                else if (selectedView) {
                    selectedView.deselect();
                }
                this.getEl().setData("live-edit-selected", "true");
                this.hideTooltip();
                this.highlight();
                this.shade();
                this.showCursor();
                this.stopTextEditMode();
                if (this.isEmpty()) {
                    this.selectPlaceholder();
                }
            };
            ItemView.prototype.deselect = function (silent) {
                this.getEl().removeAttribute("data-live-edit-selected");
                this.hideContextMenu();
                this.unhighlight();
                this.unshade();
                if (this.isEmpty()) {
                    this.deselectPlaceholder();
                }
                if (!silent) {
                    new liveedit.ItemViewDeselectedEvent(this).fire();
                }
            };
            ItemView.prototype.isDraggableView = function () {
                return !(api.ObjectHelper.iFrameSafeInstanceOf(this, liveedit.RegionView) || api.ObjectHelper.iFrameSafeInstanceOf(this, liveedit.PageView));
            };
            ItemView.prototype.stopTextEditMode = function () {
                var pageView = this.getPageView();
                if (pageView.isTextEditMode()) {
                    pageView.setTextEditMode(false);
                }
            };
            ItemView.prototype.selectPlaceholder = function () {
                if (this.placeholder) {
                    this.placeholder.select();
                }
            };
            ItemView.prototype.deselectPlaceholder = function () {
                if (this.placeholder) {
                    this.placeholder.deselect();
                }
            };
            ItemView.prototype.showRenderingError = function (url, errorMessage) {
                if (this.placeholder) {
                    this.addClass("error");
                    this.placeholder.showRenderingError(url, errorMessage);
                }
            };
            ItemView.prototype.getName = function () {
                return '[No Name]';
            };
            ItemView.prototype.showLoadingSpinner = function () {
                if (!this.loadMask) {
                    this.loadMask = new api.ui.mask.LoadMask(this);
                    this.appendChild(this.loadMask);
                }
                this.loadMask.show();
            };
            ItemView.prototype.hideLoadingSpinner = function () {
                if (this.loadMask) {
                    this.loadMask.hide();
                }
            };
            ItemView.prototype.getElementDimensions = function () {
                var el = this.getEl(), offset = el.getOffset();
                return {
                    top: offset.top,
                    left: offset.left,
                    width: el.getWidthWithBorder(),
                    height: el.getHeightWithBorder()
                };
            };
            ItemView.prototype.getContextMenuActions = function () {
                return this.contextMenuActions;
            };
            ItemView.prototype.toItemViewArray = function () {
                return [this];
            };
            ItemView.prototype.toString = function () {
                return this.getItemId().toNumber() + " : " + this.getType().getShortName();
            };
            ItemView.findParentItemViewAsHTMLElement = function (htmlElement) {
                var parentHTMLElement = htmlElement.parentElement;
                var parseItemId = ItemView.parseItemId(parentHTMLElement);
                while (parseItemId == null) {
                    parentHTMLElement = parentHTMLElement.parentElement;
                    parseItemId = ItemView.parseItemId(parentHTMLElement);
                }
                return parentHTMLElement;
            };
            ItemView.prototype.onMouseOverView = function (listener) {
                this.mouseOverViewListeners.push(listener);
            };
            ItemView.prototype.unMouseOverView = function (listener) {
                this.mouseOverViewListeners = this.mouseOverViewListeners.filter(function (current) { return (current != listener); });
            };
            ItemView.prototype.notifyMouseOverView = function () {
                if (ItemView.debug) {
                    console.log("notifying mouse over [" + this.getId() + "]");
                }
                this.mouseOverViewListeners.forEach(function (listener) { return listener(); });
            };
            ItemView.prototype.onMouseLeaveView = function (listener) {
                this.mouseOutViewListeners.push(listener);
            };
            ItemView.prototype.unMouseLeaveView = function (listener) {
                this.mouseOutViewListeners = this.mouseOutViewListeners.filter(function (current) { return (current != listener); });
            };
            ItemView.prototype.notifyMouseLeaveView = function () {
                if (ItemView.debug) {
                    console.log("notifying mouse out [" + this.getId() + "]");
                }
                this.mouseOutViewListeners.forEach(function (listener) { return listener(); });
            };
            ItemView.prototype.getContextMenuTitle = function () {
                return this.contextMenuTitle;
            };
            ItemView.prototype.visibleInViewport = function () {
                var dimensions = this.getElementDimensions();
                var screenTopPosition = document.body.scrollTop != 0 ? document.body.scrollTop : document.documentElement.scrollTop;
                return !(dimensions.top != undefined && ((dimensions.top - 10 < screenTopPosition) || (dimensions.top + dimensions.height > screenTopPosition + window.innerHeight)));
            };
            ItemView.prototype.addComponentView = function (componentView, index, isNew) {
                if (isNew === void 0) { isNew = false; }
                throw new Error("Must be implemented by inheritors");
            };
            ItemView.prototype.getNewItemIndex = function () {
                throw new Error("Must be implemented by inheritors");
            };
            ItemView.prototype.createComponentView = function (typeAsString) {
                var componentItemType = liveedit.ItemType.byShortName(typeAsString), regionView = this.getRegionView(), newComponent = regionView.createComponent(componentItemType.toComponentType());
                return componentItemType.createView(new liveedit.CreateItemViewConfig().setParentView(regionView).setParentElement(regionView).setData(newComponent));
            };
            ItemView.prototype.getInsertActions = function () {
                var actions = [this.createInsertSubAction("image", "Image", "live-edit-font-icon-image icon"), this.createInsertSubAction("part", "Part", "live-edit-font-icon-part icon")];
                if (!this.getRegionView().hasParentLayoutComponentView()) {
                    actions.push(this.createInsertSubAction("layout", "Layout", "live-edit-font-icon-layout icon"));
                }
                actions.push(this.createInsertSubAction("text", "Text", "live-edit-font-icon-text icon"));
                return actions;
            };
            ItemView.prototype.getRegionView = function () {
                throw new Error("Must be implemented by inheritors");
            };
            ItemView.prototype.createInsertAction = function () {
                return new api.ui.Action('Insert').setChildActions(this.getInsertActions());
            };
            ItemView.prototype.createSelectParentAction = function () {
                var _this = this;
                var action = new api.ui.Action("Select parent");
                action.setSortOrder(0);
                action.onExecuted(function () {
                    var parentView = _this.getParentItemView();
                    if (parentView) {
                        _this.deselect();
                        parentView.select(null, 0 /* TOP */);
                        parentView.scrollComponentIntoView();
                    }
                });
                return action;
            };
            ItemView.prototype.createInsertSubAction = function (typeAsString, label, cls) {
                var _this = this;
                var action = new api.ui.Action(label).onExecuted(function () {
                    var componentView = _this.createComponentView(typeAsString);
                    _this.addComponentView(componentView, _this.getNewItemIndex(), true);
                });
                action.setIconClass(cls);
                return action;
            };
            ItemView.debug = false;
            return ItemView;
        })(api.dom.Element);
        liveedit.ItemView = ItemView;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewAddedEvent = (function () {
            function ItemViewAddedEvent(view, isNew) {
                if (isNew === void 0) { isNew = false; }
                this.view = view;
                this.newlyCreated = isNew;
            }
            ItemViewAddedEvent.prototype.getView = function () {
                return this.view;
            };
            ItemViewAddedEvent.prototype.isNew = function () {
                return this.newlyCreated;
            };
            return ItemViewAddedEvent;
        })();
        liveedit.ItemViewAddedEvent = ItemViewAddedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewRemovedEvent = (function () {
            function ItemViewRemovedEvent(view) {
                this.view = view;
            }
            ItemViewRemovedEvent.prototype.getView = function () {
                return this.view;
            };
            return ItemViewRemovedEvent;
        })();
        liveedit.ItemViewRemovedEvent = ItemViewRemovedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewSelectedEvent = (function (_super) {
            __extends(ItemViewSelectedEvent, _super);
            function ItemViewSelectedEvent(itemView, position, isNew) {
                if (isNew === void 0) { isNew = false; }
                _super.call(this);
                this.pageItemView = itemView;
                this.position = position;
                this.newlyCreated = isNew;
            }
            ItemViewSelectedEvent.prototype.getItemView = function () {
                return this.pageItemView;
            };
            ItemViewSelectedEvent.prototype.getPosition = function () {
                return this.position;
            };
            ItemViewSelectedEvent.prototype.isNew = function () {
                return this.newlyCreated;
            };
            ItemViewSelectedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ItemViewSelectedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ItemViewSelectedEvent;
        })(api.event.Event);
        liveedit.ItemViewSelectedEvent = ItemViewSelectedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewDeselectedEvent = (function (_super) {
            __extends(ItemViewDeselectedEvent, _super);
            function ItemViewDeselectedEvent(itemView) {
                _super.call(this);
                this.itemView = itemView;
            }
            ItemViewDeselectedEvent.prototype.getItemView = function () {
                return this.itemView;
            };
            ItemViewDeselectedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ItemViewDeselectedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ItemViewDeselectedEvent;
        })(api.event.Event);
        liveedit.ItemViewDeselectedEvent = ItemViewDeselectedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewContextMenuTitle = (function (_super) {
            __extends(ItemViewContextMenuTitle, _super);
            function ItemViewContextMenuTitle(name, icon) {
                _super.call(this, new api.app.NamesAndIconViewBuilder().setAddTitleAttribute(false));
                this.setMainName(name);
                this.setIconClass(icon);
            }
            return ItemViewContextMenuTitle;
        })(api.app.NamesAndIconView);
        liveedit.ItemViewContextMenuTitle = ItemViewContextMenuTitle;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Component = api.content.page.region.Component;
        var ComponentViewContextMenuTitle = (function (_super) {
            __extends(ComponentViewContextMenuTitle, _super);
            function ComponentViewContextMenuTitle(component, type) {
                var _this = this;
                var handler = function (event) {
                    if (event.getPropertyName() == Component.PROPERTY_NAME) {
                        _this.setMainName(component.getName() ? component.getName().toString() : "");
                    }
                };
                _super.call(this, component.getName() ? component.getName().toString() : "", type.getConfig().getIconCls());
                component.onPropertyChanged(handler);
                this.onRemoved(function () { return component.unPropertyChanged(handler); });
            }
            return ComponentViewContextMenuTitle;
        })(liveedit.ItemViewContextMenuTitle);
        liveedit.ComponentViewContextMenuTitle = ComponentViewContextMenuTitle;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ContentViewContextMenuTitle = (function (_super) {
            __extends(ContentViewContextMenuTitle, _super);
            function ContentViewContextMenuTitle(contentView) {
                _super.call(this, contentView.getName(), liveedit.ContentItemType.get().getConfig().getIconCls());
            }
            return ContentViewContextMenuTitle;
        })(liveedit.ItemViewContextMenuTitle);
        liveedit.ContentViewContextMenuTitle = ContentViewContextMenuTitle;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageViewContextMenuTitle = (function (_super) {
            __extends(PageViewContextMenuTitle, _super);
            function PageViewContextMenuTitle(content) {
                _super.call(this, content.getDisplayName(), liveedit.PageItemType.get().getConfig().getIconCls());
            }
            return PageViewContextMenuTitle;
        })(liveedit.ItemViewContextMenuTitle);
        liveedit.PageViewContextMenuTitle = PageViewContextMenuTitle;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var RegionViewContextMenuTitle = (function (_super) {
            __extends(RegionViewContextMenuTitle, _super);
            function RegionViewContextMenuTitle(region) {
                _super.call(this, region.getName(), liveedit.RegionItemType.get().getConfig().getIconCls());
            }
            return RegionViewContextMenuTitle;
        })(liveedit.ItemViewContextMenuTitle);
        liveedit.RegionViewContextMenuTitle = RegionViewContextMenuTitle;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewContextMenu = (function (_super) {
            __extends(ItemViewContextMenu, _super);
            function ItemViewContextMenu(menuTitle, actions) {
                var _this = this;
                _super.call(this, 'item-view-context-menu bottom');
                this.arrow = new ItemViewContextMenuArrow(this);
                this.appendChild(this.arrow);
                this.title = menuTitle;
                if (this.title) {
                    var lastPosition;
                    var dragListener = function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        var x = e.pageX, y = e.pageY;
                        _this.moveBy(x - lastPosition.x, y - lastPosition.y);
                        lastPosition = {
                            x: x,
                            y: y
                        };
                    };
                    var upListener = function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        _this.stopDrag(dragListener, upListener);
                    };
                    this.title.onMouseDown(function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                        lastPosition = {
                            x: e.pageX,
                            y: e.pageY
                        };
                        _this.startDrag(dragListener, upListener);
                    });
                    this.appendChild(this.title);
                }
                this.menu = new api.ui.menu.TreeContextMenu(actions, false);
                this.menu.onItemClicked(function () {
                    _this.hide();
                });
                this.appendChild(this.menu);
                this.onClicked(function (e) {
                    e.preventDefault();
                    e.stopPropagation();
                });
                this.onHidden(function (e) {
                    _this.stopDrag(dragListener, upListener);
                });
                api.dom.Body.get().appendChild(this);
            }
            ItemViewContextMenu.prototype.showAt = function (x, y, notClicked) {
                if (notClicked === void 0) { notClicked = false; }
                this.menu.showAt.call(this, this.getXPosition(x), this.getYPosition(y, notClicked));
            };
            ItemViewContextMenu.prototype.moveBy = function (dx, dy) {
                this.menu.moveBy.call(this, dx, dy);
            };
            ItemViewContextMenu.prototype.setActions = function (actions) {
                this.menu.setActions(actions);
            };
            ItemViewContextMenu.prototype.getMenu = function () {
                return this.menu;
            };
            ItemViewContextMenu.prototype.startDrag = function (dragListener, upListener) {
                api.dom.Body.get().onMouseMove(dragListener);
                api.dom.Body.get().onMouseUp(upListener);
            };
            ItemViewContextMenu.prototype.stopDrag = function (dragListener, upListener) {
                api.dom.Body.get().unMouseMove(dragListener);
                api.dom.Body.get().unMouseUp(upListener);
            };
            ItemViewContextMenu.prototype.getXPosition = function (x) {
                var pageView = wemjq(this.getHTMLElement()).closest(".page-view");
                var minDistFromFrameBorder = (pageView.outerWidth(true) - pageView.innerWidth()) / 2;
                if (this.overflowsLeftFrameBorder(x)) {
                    this.arrow.shiftXPositionLeft(x, pageView);
                    return minDistFromFrameBorder;
                }
                else if (this.overflowsRightFrameBorder(x)) {
                    this.arrow.shiftXPositionRight(x, pageView);
                    return pageView.outerWidth(true) - this.getEl().getWidth() - minDistFromFrameBorder;
                }
                else {
                    this.arrow.resetXPosition();
                    return x - this.getEl().getWidth() / 2;
                }
            };
            ItemViewContextMenu.prototype.getYPosition = function (y, notClicked) {
                if (this.overflowsBottom(y, notClicked)) {
                    this.arrow.toggleVerticalPosition(false);
                    return y - this.getEl().getHeight() - this.arrow.getHeight();
                }
                else {
                    this.arrow.toggleVerticalPosition(true);
                    return y + this.arrow.getHeight();
                }
            };
            ItemViewContextMenu.prototype.overflowsLeftFrameBorder = function (x) {
                return (x - this.getEl().getWidth() / 2) < 0;
            };
            ItemViewContextMenu.prototype.overflowsRightFrameBorder = function (x) {
                return (x + this.getEl().getWidth() / 2) > window.innerWidth;
            };
            ItemViewContextMenu.prototype.overflowsBottom = function (y, notClicked) {
                var yPos = y + this.getEl().getHeight() + this.arrow.getHeight() + 1;
                return yPos > (notClicked ? Math.max(document.body.scrollHeight, document.documentElement.scrollHeight) : (wemjq(window).scrollTop() + window.innerHeight));
            };
            return ItemViewContextMenu;
        })(api.dom.DivEl);
        liveedit.ItemViewContextMenu = ItemViewContextMenu;
        var ItemViewContextMenuArrow = (function (_super) {
            __extends(ItemViewContextMenuArrow, _super);
            function ItemViewContextMenuArrow(contextMenu) {
                _super.call(this, "item-view-context-menu-arrow bottom");
                this.height = 7;
                this.width = 7;
                this.contextMenu = contextMenu;
            }
            ItemViewContextMenuArrow.prototype.toggleVerticalPosition = function (bottom) {
                this.toggleClass("bottom", bottom);
                this.toggleClass("top", !bottom);
            };
            ItemViewContextMenuArrow.prototype.updateArrowXPosition = function (x) {
                this.getEl().setLeftPx(x);
            };
            ItemViewContextMenuArrow.prototype.resetXPosition = function () {
                this.getEl().setLeft("");
            };
            ItemViewContextMenuArrow.prototype.shiftXPositionLeft = function (x, pageView) {
                var minDistFromFrameBorder = (pageView.outerWidth(true) - pageView.innerWidth()) / 2;
                if (this.overflowsLeftFrameBorder(x, minDistFromFrameBorder)) {
                    this.updateArrowXPosition(this.width);
                }
                else {
                    this.updateArrowXPosition(x - minDistFromFrameBorder);
                }
            };
            ItemViewContextMenuArrow.prototype.shiftXPositionRight = function (x, pageView) {
                var minDistFromFrameBorder = (pageView.outerWidth(true) - pageView.innerWidth()) / 2;
                var arrowPos = this.contextMenu.getEl().getWidth() - (pageView.outerWidth(true) - x) + minDistFromFrameBorder;
                if (this.overflowsRightFrameBorder(arrowPos)) {
                    this.updateArrowXPosition(this.contextMenu.getEl().getWidth() - this.width);
                }
                else {
                    this.updateArrowXPosition(arrowPos);
                }
            };
            ItemViewContextMenuArrow.prototype.getHeight = function () {
                return this.height;
            };
            ItemViewContextMenuArrow.prototype.overflowsLeftFrameBorder = function (x, minimalDistFromFrameBorder) {
                return (minimalDistFromFrameBorder + this.width) > x;
            };
            ItemViewContextMenuArrow.prototype.overflowsRightFrameBorder = function (arrowPos) {
                return (this.contextMenu.getEl().getWidth() - this.width) < arrowPos;
            };
            return ItemViewContextMenuArrow;
        })(api.dom.DivEl);
        liveedit.ItemViewContextMenuArrow = ItemViewContextMenuArrow;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        (function (ItemViewContextMenuPosition) {
            ItemViewContextMenuPosition[ItemViewContextMenuPosition["TOP"] = 0] = "TOP";
            ItemViewContextMenuPosition[ItemViewContextMenuPosition["BOTTOM"] = 1] = "BOTTOM";
        })(liveedit.ItemViewContextMenuPosition || (liveedit.ItemViewContextMenuPosition = {}));
        var ItemViewContextMenuPosition = liveedit.ItemViewContextMenuPosition;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PropertyTree = api.data.PropertyTree;
        var ComponentName = api.content.page.region.ComponentName;
        var DescriptorBasedComponentBuilder = api.content.page.region.DescriptorBasedComponentBuilder;
        var RegionViewBuilder = (function () {
            function RegionViewBuilder() {
            }
            RegionViewBuilder.prototype.setLiveEditModel = function (value) {
                this.liveEditModel = value;
                return this;
            };
            RegionViewBuilder.prototype.setParentElement = function (value) {
                this.parentElement = value;
                return this;
            };
            RegionViewBuilder.prototype.setParentView = function (value) {
                this.parentView = value;
                return this;
            };
            RegionViewBuilder.prototype.setRegion = function (value) {
                this.region = value;
                return this;
            };
            RegionViewBuilder.prototype.setElement = function (value) {
                this.element = value;
                return this;
            };
            return RegionViewBuilder;
        })();
        liveedit.RegionViewBuilder = RegionViewBuilder;
        var RegionView = (function (_super) {
            __extends(RegionView, _super);
            function RegionView(builder) {
                var _this = this;
                this.componentViews = [];
                this.componentIndex = 0;
                this.itemViewAddedListeners = [];
                this.itemViewRemovedListeners = [];
                this.parentView = builder.parentView;
                RegionView.debug = false;
                this.itemViewAddedListener = function (event) { return _this.notifyItemViewAdded(event.getView(), event.isNew()); };
                this.itemViewRemovedListener = function (event) {
                    if (api.ObjectHelper.iFrameSafeInstanceOf(event.getView(), liveedit.ComponentView)) {
                        var removedComponentView = event.getView();
                        var childIndex = _this.getComponentViewIndex(removedComponentView);
                        if (childIndex > -1) {
                            _this.componentViews.splice(childIndex, 1);
                        }
                    }
                    _this.notifyItemViewRemoved(event.getView());
                };
                _super.call(this, new liveedit.ItemViewBuilder().setItemViewIdProducer(builder.parentView.getItemViewIdProducer()).setType(liveedit.RegionItemType.get()).setElement(builder.element).setPlaceholder(new liveedit.RegionPlaceholder(builder.region)).setTooltipViewer(new liveedit.RegionComponentViewer()).setParentElement(builder.parentElement).setParentView(builder.parentView).setContextMenuActions(this.createRegionContextMenuActions()).setContextMenuTitle(new liveedit.RegionViewContextMenuTitle(builder.region)));
                this.addClass('region-view');
                this.componentAddedListener = function (event) {
                    if (RegionView.debug) {
                        console.log('RegionView.handleComponentAdded: ' + event.getComponentPath().toString());
                    }
                    _this.refreshEmptyState();
                };
                this.componentRemovedListener = function (event) {
                    if (RegionView.debug) {
                        console.log('RegionView.handleComponentRemoved: ' + event.getComponentPath().toString());
                    }
                    _this.refreshEmptyState();
                };
                this.setRegion(builder.region);
                this.parseComponentViews();
                this.onMouseDown(this.memorizeLastMouseDownTarget.bind(this));
            }
            RegionView.prototype.memorizeLastMouseDownTarget = function (event) {
                this.mouseDownLastTarget = event.target;
            };
            RegionView.prototype.createRegionContextMenuActions = function () {
                var _this = this;
                var actions = [];
                actions.push(this.createSelectParentAction());
                actions.push(this.createInsertAction());
                actions.push(new api.ui.Action('Reset').onExecuted(function () {
                    _this.deselect();
                    _this.empty();
                }));
                return actions;
            };
            RegionView.prototype.getParentItemView = function () {
                return this.parentView;
            };
            RegionView.prototype.setParentItemView = function (itemView) {
                _super.prototype.setParentItemView.call(this, itemView);
                this.parentView = itemView;
            };
            RegionView.prototype.setRegion = function (region) {
                if (region) {
                    if (this.region) {
                        this.region.unComponentAdded(this.componentAddedListener);
                        this.region.unComponentRemoved(this.componentRemovedListener);
                    }
                    this.region = region;
                    this.setTooltipObject(region);
                    this.region.onComponentAdded(this.componentAddedListener);
                    this.region.onComponentRemoved(this.componentRemovedListener);
                    var components = region.getComponents();
                    var componentViews = this.getComponentViews();
                    componentViews.forEach(function (view, index) {
                        view.setComponent(components[index]);
                    });
                }
                this.refreshEmptyState();
            };
            RegionView.prototype.getRegion = function () {
                return this.region;
            };
            RegionView.prototype.getRegionName = function () {
                return this.getRegionPath() ? this.getRegionPath().getRegionName() : null;
            };
            RegionView.prototype.getRegionPath = function () {
                return this.region ? this.region.getPath() : null;
            };
            RegionView.prototype.getName = function () {
                return this.getRegionName() ? this.getRegionName().toString() : "[No Name]";
            };
            RegionView.prototype.highlight = function () {
                var isDragging = liveedit.DragAndDrop.get().isDragging();
                if (!this.getPageView().isTextEditMode() && !isDragging) {
                    _super.prototype.highlight.call(this);
                }
            };
            RegionView.prototype.showCursor = function () {
                if (!this.getPageView().isTextEditMode()) {
                    _super.prototype.showCursor.call(this);
                }
            };
            RegionView.prototype.handleClick = function (event) {
                event.stopPropagation();
                var pageView = this.getPageView();
                if (pageView.isTextEditMode()) {
                    if (!pageView.hasTargetWithinTextComponent(this.mouseDownLastTarget)) {
                        pageView.setTextEditMode(false);
                    }
                }
                else {
                    _super.prototype.handleClick.call(this, event);
                }
            };
            RegionView.prototype.select = function (clickPosition, menuPosition) {
                _super.prototype.select.call(this, clickPosition, menuPosition);
                new liveedit.RegionSelectedEvent(this).fire();
            };
            RegionView.prototype.selectWithoutMenu = function (isNew) {
                if (isNew === void 0) { isNew = false; }
                _super.prototype.selectWithoutMenu.call(this, isNew);
                new liveedit.RegionSelectedEvent(this).fire();
            };
            RegionView.prototype.toString = function () {
                var extra = "";
                if (this.getRegionPath()) {
                    extra = " : " + this.getRegionPath().toString();
                }
                return _super.prototype.toString.call(this) + extra;
            };
            RegionView.prototype.registerComponentView = function (componentView, index, isNew) {
                if (isNew === void 0) { isNew = false; }
                if (RegionView.debug) {
                    console.log('RegionView[' + this.toString() + '].registerComponentView: ' + componentView.toString() + " at " + index);
                }
                if (index >= 0) {
                    this.componentViews.splice(index, 0, componentView);
                }
                else {
                    this.componentViews.push(componentView);
                }
                componentView.setParentItemView(this);
                componentView.onItemViewAdded(this.itemViewAddedListener);
                componentView.onItemViewRemoved(this.itemViewRemovedListener);
                this.notifyItemViewAdded(componentView, isNew);
            };
            RegionView.prototype.unregisterComponentView = function (componentView) {
                if (RegionView.debug) {
                    console.log('RegionView[' + this.toString() + '].unregisterComponentView: ' + componentView.toString());
                }
                var indexToRemove = this.getComponentViewIndex(componentView);
                if (indexToRemove >= 0) {
                    componentView.unItemViewAdded(this.itemViewAddedListener);
                    componentView.unItemViewRemoved(this.itemViewRemovedListener);
                    this.componentViews.splice(indexToRemove, 1);
                    componentView.setParentItemView(null);
                    this.notifyItemViewRemoved(componentView);
                }
                else {
                    throw new Error("Did not find ComponentView to remove: " + componentView.getItemId().toString());
                }
            };
            RegionView.prototype.getNewItemIndex = function () {
                return 0;
            };
            RegionView.prototype.addComponentView = function (componentView, index, isNew) {
                if (isNew === void 0) { isNew = false; }
                if (RegionView.debug) {
                    console.log('RegionView[' + this.toString() + ']addComponentView: ' + componentView.toString() + " at " + index);
                }
                if (componentView.getComponent()) {
                    this.region.addComponent(componentView.getComponent(), index);
                }
                this.insertChild(componentView, index);
                this.registerComponentView(componentView, index, isNew);
                new liveedit.ComponentAddedEvent(componentView, this).fire();
            };
            RegionView.prototype.removeComponentView = function (componentView) {
                if (RegionView.debug) {
                    console.log('RegionView[' + this.toString() + '].removeComponentView: ' + componentView.toString());
                }
                this.unregisterComponentView(componentView);
                this.removeChild(componentView);
                if (componentView.getComponent()) {
                    componentView.getComponent().remove();
                }
                new liveedit.ComponentRemovedEvent(componentView, this).fire();
            };
            RegionView.prototype.getComponentViews = function () {
                return this.componentViews;
            };
            RegionView.prototype.getComponentViewIndex = function (view) {
                return this.componentViews.indexOf(view);
            };
            RegionView.prototype.getComponentViewByIndex = function (index) {
                return this.componentViews[index];
            };
            RegionView.prototype.getComponentViewByPath = function (path) {
                var firstLevelOfPath = path.getFirstLevel();
                if (path.numberOfLevels() == 1) {
                    return this.componentViews[firstLevelOfPath.getComponentIndex()];
                }
                for (var i = 0; i < this.componentViews.length; i++) {
                    var componentView = this.componentViews[i];
                    if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, api.liveedit.layout.LayoutComponentView)) {
                        var layoutView = componentView;
                        var match = layoutView.getComponentViewByPath(path.removeFirstLevel());
                        if (match) {
                            return match;
                        }
                    }
                }
                return null;
            };
            RegionView.prototype.hasParentLayoutComponentView = function () {
                return api.ObjectHelper.iFrameSafeInstanceOf(this.parentView, api.liveedit.layout.LayoutComponentView);
            };
            RegionView.prototype.hasOnlyMovingComponentViews = function () {
                return this.componentViews.length > 0 && this.componentViews.every(function (view) {
                    return view.isMoving();
                });
            };
            RegionView.prototype.isEmpty = function () {
                var onlyMoving = this.hasOnlyMovingComponentViews();
                var empty = !this.region || this.region.isEmpty();
                return empty || onlyMoving;
            };
            RegionView.prototype.empty = function () {
                if (RegionView.debug) {
                    console.debug("RegionView[" + this.toString() + "].empty()", this.componentViews);
                }
                while (this.componentViews.length > 0) {
                    this.removeComponentView(this.componentViews[0]);
                }
            };
            RegionView.prototype.toItemViewArray = function () {
                var array = [];
                array.push(this);
                this.componentViews.forEach(function (componentView) {
                    var itemViews = componentView.toItemViewArray();
                    array = array.concat(itemViews);
                });
                return array;
            };
            RegionView.prototype.onItemViewAdded = function (listener) {
                this.itemViewAddedListeners.push(listener);
            };
            RegionView.prototype.unItemViewAdded = function (listener) {
                this.itemViewAddedListeners = this.itemViewAddedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            RegionView.prototype.notifyItemViewAddedForAll = function (itemViews) {
                var _this = this;
                itemViews.forEach(function (itemView) {
                    _this.notifyItemViewAdded(itemView);
                });
            };
            RegionView.prototype.notifyItemViewAdded = function (itemView, isNew) {
                if (isNew === void 0) { isNew = false; }
                var event = new liveedit.ItemViewAddedEvent(itemView, isNew);
                this.itemViewAddedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            RegionView.prototype.onItemViewRemoved = function (listener) {
                this.itemViewRemovedListeners.push(listener);
            };
            RegionView.prototype.unItemViewRemoved = function (listener) {
                this.itemViewRemovedListeners = this.itemViewRemovedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            RegionView.prototype.getRegionView = function () {
                return this;
            };
            RegionView.prototype.createComponent = function (componentType) {
                var shortName = api.util.StringHelper.capitalize(api.util.StringHelper.removeWhitespaces(componentType.getShortName()));
                var componentName = new ComponentName(shortName);
                var builder = componentType.newComponentBuilder().setName(componentName);
                if (api.ObjectHelper.iFrameSafeInstanceOf(builder, DescriptorBasedComponentBuilder)) {
                    var descriptorBuilder = builder;
                    descriptorBuilder.setConfig(new PropertyTree());
                }
                return builder.build();
            };
            RegionView.prototype.notifyItemViewRemovedForAll = function (itemViews) {
                var _this = this;
                itemViews.forEach(function (itemView) {
                    _this.notifyItemViewRemoved(itemView);
                });
            };
            RegionView.prototype.notifyItemViewRemoved = function (itemView) {
                var event = new liveedit.ItemViewRemovedEvent(itemView);
                this.itemViewRemovedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            RegionView.isRegionViewFromHTMLElement = function (htmlElement) {
                var name = htmlElement.getAttribute("data-" + liveedit.ItemType.ATTRIBUTE_REGION_NAME);
                return !api.util.StringHelper.isBlank(name);
            };
            RegionView.prototype.parseComponentViews = function () {
                var _this = this;
                this.componentViews.forEach(function (componentView) {
                    _this.unregisterComponentView(componentView);
                });
                this.componentViews = [];
                this.componentIndex = 0;
                this.doParseComponentViews();
            };
            RegionView.prototype.doParseComponentViews = function (parentElement) {
                var _this = this;
                var children = parentElement ? parentElement.getChildren() : this.getChildren();
                var region = this.getRegion();
                children.forEach(function (childElement) {
                    var itemType = liveedit.ItemType.fromElement(childElement);
                    var isComponentView = api.ObjectHelper.iFrameSafeInstanceOf(childElement, liveedit.ComponentView);
                    var component, componentView;
                    if (isComponentView) {
                        component = region.getComponentByIndex(_this.componentIndex++);
                        if (component) {
                            componentView = childElement;
                            componentView.setComponent(component);
                            _this.registerComponentView(componentView, _this.componentIndex);
                        }
                    }
                    else if (itemType) {
                        api.util.assert(itemType.isComponentType(), "Expected ItemView beneath a Region to be a Component: " + itemType.getShortName());
                        component = region.getComponentByIndex(_this.componentIndex++);
                        if (component) {
                            componentView = itemType.createView(new liveedit.CreateItemViewConfig().setParentView(_this).setData(component).setElement(childElement).setParentElement(parentElement ? parentElement : _this));
                            _this.registerComponentView(componentView, _this.componentIndex);
                        }
                    }
                    else {
                        _this.doParseComponentViews(childElement);
                    }
                });
            };
            RegionView.prototype.handleDragEnter = function (event) {
                if (event.target === this.getHTMLElement()) {
                    console.log("ItemView.handleDragEnter", event, this.getHTMLElement());
                }
            };
            RegionView.prototype.handleDragLeave = function (event) {
                if (event.target === this.getHTMLElement()) {
                    console.log("ItemView.handleDragLeave", event, this.getHTMLElement());
                }
            };
            RegionView.prototype.handleDragOver = function (event) {
                if (event.target === this.getHTMLElement()) {
                    console.log("RegionView[" + this.toString() + "].handleDragOver: ", event.target, event.target);
                    event.preventDefault();
                }
            };
            RegionView.prototype.handleDrop = function (event) {
                if (event.target === this.getHTMLElement()) {
                    console.log("RegionView[" + this.toString() + "].handleDrop: ", event.target, this.getHTMLElement());
                    event.preventDefault();
                    var data = event.dataTransfer.getData("Text");
                }
            };
            return RegionView;
        })(liveedit.ItemView);
        liveedit.RegionView = RegionView;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var RegionComponentViewer = (function (_super) {
            __extends(RegionComponentViewer, _super);
            function RegionComponentViewer() {
                _super.call(this);
            }
            RegionComponentViewer.prototype.resolveDisplayName = function (object) {
                return object.getName().toString();
            };
            RegionComponentViewer.prototype.resolveSubName = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                return object.getPath().toString();
            };
            RegionComponentViewer.prototype.resolveIconClass = function (object) {
                return "live-edit-font-icon-region";
            };
            return RegionComponentViewer;
        })(api.ui.NamesAndIconViewer);
        liveedit.RegionComponentViewer = RegionComponentViewer;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ItemViewPlaceholder = (function (_super) {
            __extends(ItemViewPlaceholder, _super);
            function ItemViewPlaceholder() {
                _super.call(this, "item-placeholder");
            }
            ItemViewPlaceholder.prototype.showRenderingError = function (url, errorMessage) {
                if (errorMessage === void 0) { errorMessage = "Error rendering component"; }
                this.removeChildren();
                this.addClass("rendering-error");
                var errorTitle = new api.dom.PEl().setHtml(errorMessage);
                var urlAnchor = new api.dom.AEl().setUrl(url, "_blank").setHtml("Show more...");
                this.appendChildren(errorTitle, urlAnchor);
            };
            ItemViewPlaceholder.prototype.select = function () {
            };
            ItemViewPlaceholder.prototype.deselect = function () {
            };
            return ItemViewPlaceholder;
        })(api.dom.DivEl);
        liveedit.ItemViewPlaceholder = ItemViewPlaceholder;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var DragPlaceholder = (function (_super) {
            __extends(DragPlaceholder, _super);
            function DragPlaceholder() {
                _super.call(this);
                this.pattern = 'Drop {0} here';
                this.setId('drag-placeholder').addClass('drag-placeholder');
                this.messageEl = new api.dom.DivEl("message");
                this.appendChild(this.messageEl);
            }
            DragPlaceholder.get = function () {
                if (!DragPlaceholder.instance) {
                    if (DragPlaceholder.debug) {
                        console.log('DragPlaceholder.get() creating new instance');
                    }
                    DragPlaceholder.instance = new DragPlaceholder();
                }
                return DragPlaceholder.instance;
            };
            DragPlaceholder.prototype.setItemType = function (type) {
                if (DragPlaceholder.debug) {
                    console.log('DragPlaceholder.setItemType', type);
                }
                if (this.itemType) {
                    this.removeClass(this.itemType.getShortName() + '-placeholder');
                }
                this.itemType = type;
                if (type) {
                    this.setText(this.getDefaultText());
                    this.addClass(type.getShortName() + '-placeholder');
                }
                else {
                    this.setText('');
                }
                return this;
            };
            DragPlaceholder.prototype.getDefaultText = function () {
                return api.util.StringHelper.format(this.pattern, api.util.StringHelper.capitalize(this.itemType.getShortName()));
            };
            DragPlaceholder.prototype.setDropAllowed = function (allowed) {
                if (DragPlaceholder.debug) {
                    console.log('DragPlaceholder.seDropAllowed: ' + allowed);
                }
                if (allowed && this.itemType) {
                    this.setText(this.getDefaultText());
                }
                this.toggleClass('drop-allowed', allowed);
                return this;
            };
            DragPlaceholder.prototype.setText = function (text) {
                if (DragPlaceholder.debug) {
                    console.log('DragPlaceholder.setText: ' + text);
                }
                this.messageEl.setHtml(text);
                return this;
            };
            DragPlaceholder.prototype.setRegionView = function (regionView) {
                if (DragPlaceholder.debug) {
                    console.log('DragPlaceholder.setRegionView: ' + (regionView ? regionView.toString() : ''));
                }
                this.regionView = regionView;
                this.setDropAllowed(!!regionView);
                return this;
            };
            DragPlaceholder.prototype.reset = function () {
                this.setItemType(null);
                this.setRegionView(null);
                return this;
            };
            DragPlaceholder.debug = false;
            return DragPlaceholder;
        })(liveedit.ItemViewPlaceholder);
        liveedit.DragPlaceholder = DragPlaceholder;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var RegionPlaceholder = (function (_super) {
            __extends(RegionPlaceholder, _super);
            function RegionPlaceholder(region) {
                _super.call(this);
                this.addClass("region-placeholder");
                this.region = region;
                var dragComponentsHereEl = new api.dom.PEl();
                dragComponentsHereEl.setHtml("Drop here");
                this.appendChild(dragComponentsHereEl);
            }
            return RegionPlaceholder;
        })(liveedit.ItemViewPlaceholder);
        liveedit.RegionPlaceholder = RegionPlaceholder;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ComponentPlaceholder = (function (_super) {
            __extends(ComponentPlaceholder, _super);
            function ComponentPlaceholder() {
                _super.apply(this, arguments);
            }
            return ComponentPlaceholder;
        })(liveedit.ItemViewPlaceholder);
        liveedit.ComponentPlaceholder = ComponentPlaceholder;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ComponentViewBuilder = (function () {
            function ComponentViewBuilder() {
            }
            ComponentViewBuilder.prototype.setItemViewProducer = function (value) {
                this.itemViewProducer = value;
                return this;
            };
            ComponentViewBuilder.prototype.setType = function (value) {
                this.type = value;
                return this;
            };
            ComponentViewBuilder.prototype.setParentRegionView = function (value) {
                this.parentRegionView = value;
                return this;
            };
            ComponentViewBuilder.prototype.setParentElement = function (value) {
                this.parentElement = value;
                return this;
            };
            ComponentViewBuilder.prototype.setComponent = function (value) {
                this.component = value;
                return this;
            };
            ComponentViewBuilder.prototype.setElement = function (value) {
                this.element = value;
                return this;
            };
            ComponentViewBuilder.prototype.setPositionIndex = function (value) {
                this.positionIndex = value;
                return this;
            };
            ComponentViewBuilder.prototype.setContextMenuActions = function (actions) {
                this.contextMenuActions = actions;
                return this;
            };
            ComponentViewBuilder.prototype.setPlaceholder = function (value) {
                this.placeholder = value;
                return this;
            };
            ComponentViewBuilder.prototype.setTooltipViewer = function (value) {
                this.tooltipViewer = value;
                return this;
            };
            return ComponentViewBuilder;
        })();
        liveedit.ComponentViewBuilder = ComponentViewBuilder;
        var ComponentView = (function (_super) {
            __extends(ComponentView, _super);
            function ComponentView(builder) {
                var _this = this;
                this.itemViewAddedListeners = [];
                this.itemViewRemovedListeners = [];
                this.moving = false;
                this.parentRegionView = builder.parentRegionView;
                _super.call(this, new liveedit.ItemViewBuilder().setItemViewIdProducer(builder.itemViewProducer ? builder.itemViewProducer : builder.parentRegionView.getItemViewIdProducer()).setPlaceholder(builder.placeholder).setTooltipViewer(builder.tooltipViewer).setType(builder.type).setElement(builder.element).setParentView(builder.parentRegionView).setParentElement(builder.parentElement).setContextMenuActions(this.createComponentContextMenuActions(builder.contextMenuActions)).setContextMenuTitle(new liveedit.ComponentViewContextMenuTitle(builder.component, builder.type)));
                this.propertyChangedListener = function () { return _this.refreshEmptyState(); };
                this.resetListener = function () {
                    _this.deselect();
                    var clone = _this.clone();
                    _this.replaceWith(clone);
                    clone.select();
                    new api.liveedit.ComponentResetEvent(clone, _this).fire();
                };
                this.setComponent(builder.component);
                this.onRemoved(function (event) { return _this.unregisterComponentListeners(_this.component); });
            }
            ComponentView.prototype.registerComponentListeners = function (component) {
                component.onReset(this.resetListener);
                component.onPropertyChanged(this.propertyChangedListener);
            };
            ComponentView.prototype.unregisterComponentListeners = function (component) {
                component.unPropertyChanged(this.propertyChangedListener);
                component.unReset(this.resetListener);
            };
            ComponentView.prototype.createComponentContextMenuActions = function (actions) {
                var _this = this;
                var actions = actions || [];
                actions.push(this.createSelectParentAction());
                actions.push(this.createInsertAction());
                actions.push(new api.ui.Action("Reset").onExecuted(function () {
                    _this.component.reset();
                }));
                actions.push(new api.ui.Action("Remove").onExecuted(function () {
                    _this.deselect();
                    _this.remove();
                }));
                actions.push(new api.ui.Action("Duplicate").onExecuted(function () {
                    _this.deselect();
                    var duplicatedComponent = _this.getComponent().duplicate();
                    var duplicatedView = _this.duplicate(duplicatedComponent);
                    duplicatedView.showLoadingSpinner();
                    new liveedit.ComponentDuplicatedEvent(_this, duplicatedView).fire();
                }));
                return actions;
            };
            ComponentView.prototype.remove = function () {
                this.unregisterComponentListeners(this.component);
                var parentView = this.getParentItemView();
                if (parentView) {
                    parentView.removeComponentView(this);
                }
                _super.prototype.remove.call(this);
                return this;
            };
            ComponentView.prototype.getType = function () {
                return _super.prototype.getType.call(this);
            };
            ComponentView.prototype.setComponent = function (component) {
                if (component) {
                    if (this.component) {
                        this.unregisterComponentListeners(this.component);
                    }
                    this.setTooltipObject(component);
                    this.registerComponentListeners(component);
                }
                this.component = component;
                this.refreshEmptyState();
            };
            ComponentView.prototype.getComponent = function () {
                return this.component;
            };
            ComponentView.prototype.hasComponentPath = function () {
                return this.component && this.component.hasPath();
            };
            ComponentView.prototype.getComponentPath = function () {
                return this.hasComponentPath() ? this.component.getPath() : null;
            };
            ComponentView.prototype.getName = function () {
                return this.component && this.component.getName() ? this.component.getName().toString() : null;
            };
            ComponentView.prototype.getParentItemView = function () {
                return this.parentRegionView;
            };
            ComponentView.prototype.setParentItemView = function (regionView) {
                _super.prototype.setParentItemView.call(this, regionView);
                this.parentRegionView = regionView;
            };
            ComponentView.prototype.setMoving = function (value) {
                this.moving = value;
            };
            ComponentView.prototype.isMoving = function () {
                return this.moving;
            };
            ComponentView.prototype.clone = function () {
                var index = this.getParentItemView().getComponentViewIndex(this);
                var clone = this.getType().createView(new liveedit.CreateItemViewConfig().setParentView(this.getParentItemView()).setParentElement(this.getParentElement()).setData(this.getComponent()).setPositionIndex(index));
                return clone;
            };
            ComponentView.prototype.duplicate = function (duplicate) {
                var parentView = this.getParentItemView();
                var index = parentView.getComponentViewIndex(this);
                var duplicateView = this.getType().createView(new liveedit.CreateItemViewConfig().setParentView(this.getParentItemView()).setParentElement(this.getParentElement()).setData(duplicate).setPositionIndex(index + 1));
                parentView.addComponentView(duplicateView, index + 1);
                return duplicateView;
            };
            ComponentView.prototype.toString = function () {
                var extra = "";
                if (this.hasComponentPath()) {
                    extra = " : " + this.getComponentPath().toString();
                }
                return _super.prototype.toString.call(this) + extra;
            };
            ComponentView.prototype.replaceWith = function (replacement) {
                if (ComponentView.debug) {
                    console.log('ComponentView[' + this.toString() + '].replaceWith', this, replacement);
                }
                _super.prototype.replaceWith.call(this, replacement);
                var index = this.getParentItemView().getComponentViewIndex(this);
                this.unregisterComponentListeners(this.component);
                var parentRegionView = this.parentRegionView;
                this.parentRegionView.unregisterComponentView(this);
                parentRegionView.registerComponentView(replacement, index);
            };
            ComponentView.prototype.moveToRegion = function (toRegionView, toIndex) {
                if (ComponentView.debug) {
                    console.log('ComponentView[' + this.toString() + '].moveToRegion', this, this.parentRegionView, toRegionView);
                }
                this.moving = false;
                if (this.parentRegionView.getRegionPath().equals(toRegionView.getRegionPath()) && toIndex == this.parentRegionView.getComponentViewIndex(this)) {
                    if (ComponentView.debug) {
                        console.debug('Dropped in the same region at the same index, no need to move', this.parentRegionView, toRegionView);
                    }
                    return;
                }
                var parentView = this.getParentItemView();
                if (parentView) {
                    parentView.removeComponentView(this);
                }
                toRegionView.addComponentView(this, toIndex);
            };
            ComponentView.prototype.onItemViewAdded = function (listener) {
                this.itemViewAddedListeners.push(listener);
            };
            ComponentView.prototype.unItemViewAdded = function (listener) {
                this.itemViewAddedListeners = this.itemViewAddedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            ComponentView.prototype.notifyItemViewAdded = function (view, isNew) {
                if (isNew === void 0) { isNew = false; }
                var event = new liveedit.ItemViewAddedEvent(view, isNew);
                this.itemViewAddedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            ComponentView.prototype.onItemViewRemoved = function (listener) {
                this.itemViewRemovedListeners.push(listener);
            };
            ComponentView.prototype.unItemViewRemoved = function (listener) {
                this.itemViewRemovedListeners = this.itemViewRemovedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            ComponentView.prototype.notifyItemViewRemoved = function (view) {
                var event = new liveedit.ItemViewRemovedEvent(view);
                this.itemViewRemovedListeners.forEach(function (listener) {
                    listener(event);
                });
            };
            ComponentView.prototype.getNewItemIndex = function () {
                return this.getParentItemView().getComponentViewIndex(this) + 1;
            };
            ComponentView.prototype.addComponentView = function (componentView, index) {
                this.getParentItemView().addComponentView(componentView, index, true);
            };
            ComponentView.prototype.getRegionView = function () {
                return this.getParentItemView();
            };
            ComponentView.findParentRegionViewHTMLElement = function (htmlElement) {
                var parentItemView = liveedit.ItemView.findParentItemViewAsHTMLElement(htmlElement);
                while (!liveedit.RegionView.isRegionViewFromHTMLElement(parentItemView)) {
                    parentItemView = liveedit.ItemView.findParentItemViewAsHTMLElement(parentItemView);
                }
                return parentItemView;
            };
            ComponentView.prototype.handleDragStart2 = function (event) {
                if (event.target === this.getHTMLElement()) {
                    event.dataTransfer.effectAllowed = "move";
                    console.log("ComponentView[" + this.getItemId().toNumber() + "].handleDragStart", event, this.getHTMLElement());
                    this.hideTooltip();
                }
            };
            ComponentView.prototype.handleDrag = function (event) {
                if (event.target === this.getHTMLElement()) {
                    console.log("ComponentView[" + this.getItemId().toNumber() + "].handleDrag", event, this.getHTMLElement());
                }
            };
            ComponentView.prototype.handleDragEnd = function (event) {
                if (event.target === this.getHTMLElement()) {
                    console.log("ComponentView[" + this.getItemId().toNumber() + "].handleDragEnd", event, this.getHTMLElement());
                }
            };
            ComponentView.debug = false;
            return ComponentView;
        })(liveedit.ItemView);
        liveedit.ComponentView = ComponentView;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageDescriptorDropdown = api.content.page.PageDescriptorDropdown;
        var GetContentTypeByNameRequest = api.schema.content.GetContentTypeByNameRequest;
        var PagePlaceholder = (function (_super) {
            __extends(PagePlaceholder, _super);
            function PagePlaceholder(pageView) {
                var _this = this;
                _super.call(this);
                this.addClass("page-placeholder");
                this.pageView = pageView;
                this.controllerDropdown = new PageDescriptorDropdown(pageView.liveEditModel);
                this.controllerDropdown.load();
                this.appendChild(this.controllerDropdown);
                this.infoBlock = new liveedit.PagePlaceholderInfoBlock();
                this.appendChild(this.infoBlock);
                this.controllerDropdown.onLoadedData(function (event) {
                    if (event.getData().length > 0) {
                        _this.controllerDropdown.show();
                        var content = _this.pageView.liveEditModel.getContent();
                        if (!content.isPageTemplate()) {
                            new GetContentTypeByNameRequest(content.getType()).sendAndParse().then(function (contentType) {
                                _this.infoBlock.setTextForContent(contentType.getDisplayName());
                            }).catch(function (reason) {
                                _this.infoBlock.setTextForContent(content.getType().toString());
                                api.DefaultErrorHandler.handle(reason);
                            }).done();
                        }
                    }
                    else {
                        _this.controllerDropdown.hide();
                        _this.infoBlock.setNoControllersAvailableText();
                    }
                });
                this.controllerDropdown.onClicked(function (event) {
                    _this.select();
                    event.stopPropagation();
                });
            }
            PagePlaceholder.prototype.select = function () {
                this.controllerDropdown.giveFocus();
            };
            PagePlaceholder.prototype.deselect = function () {
            };
            return PagePlaceholder;
        })(liveedit.ItemViewPlaceholder);
        liveedit.PagePlaceholder = PagePlaceholder;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PagePlaceholderInfoBlock = (function (_super) {
            __extends(PagePlaceholderInfoBlock, _super);
            function PagePlaceholderInfoBlock() {
                _super.call(this, "page-placeholder-info");
                this.line1 = new api.dom.DivEl("page-placeholder-info-line1");
                this.line2 = new api.dom.DivEl("page-placeholder-info-line2");
                this.appendChildren(this.line1, this.line2);
            }
            PagePlaceholderInfoBlock.prototype.setTextForContent = function (contentTypeDisplayName) {
                this.line1.setHtml('No page template supporting content type "' + contentTypeDisplayName + '" found');
                this.line2.setHtml("Select a controller below to setup a customized page");
            };
            PagePlaceholderInfoBlock.prototype.setNoControllersAvailableText = function () {
                this.line1.setHtml("No page controllers found");
                this.line2.setHtml("Please add an application to your site to support rendering of this item");
            };
            return PagePlaceholderInfoBlock;
        })(api.dom.DivEl);
        liveedit.PagePlaceholderInfoBlock = PagePlaceholderInfoBlock;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageModel = api.content.page.PageModel;
        var PageMode = api.content.page.PageMode;
        var PageViewBuilder = (function () {
            function PageViewBuilder() {
            }
            PageViewBuilder.prototype.setLiveEditModel = function (value) {
                this.liveEditModel = value;
                return this;
            };
            PageViewBuilder.prototype.setItemViewProducer = function (value) {
                this.itemViewProducer = value;
                return this;
            };
            PageViewBuilder.prototype.setElement = function (value) {
                this.element = value;
                return this;
            };
            PageViewBuilder.prototype.build = function () {
                return new PageView(this);
            };
            return PageViewBuilder;
        })();
        liveedit.PageViewBuilder = PageViewBuilder;
        var PageView = (function (_super) {
            __extends(PageView, _super);
            function PageView(builder) {
                var _this = this;
                this.liveEditModel = builder.liveEditModel;
                this.pageModel = builder.liveEditModel.getPageModel();
                this.regionViews = [];
                this.viewsById = {};
                this.itemViewAddedListeners = [];
                this.itemViewRemovedListeners = [];
                this.ignorePropertyChanges = false;
                this.disableContextMenu = false;
                var resetAction = new api.ui.Action('Reset');
                resetAction.onExecuted(function () {
                    if (PageView.debug) {
                        console.log('PageView.reset');
                    }
                    _this.setIgnorePropertyChanges(true);
                    _this.pageModel.reset(_this);
                    _this.setIgnorePropertyChanges(false);
                });
                this.unlockedScreenActions = [resetAction];
                if (this.pageModel.getMode() == 0 /* AUTOMATIC */ || this.pageModel.getMode() == 3 /* NO_CONTROLLER */) {
                    resetAction.setEnabled(false);
                }
                this.registerPageModel(this.pageModel, resetAction);
                this.itemViewAddedListener = function (event) {
                    var itemView = event.getView();
                    itemView.toItemViewArray().forEach(function (itemView) {
                        _this.registerItemView(itemView);
                    });
                    if (itemView.getType().equals(api.liveedit.text.TextItemType.get())) {
                        if (!_this.isTextEditMode()) {
                            _this.setTextEditMode(true);
                        }
                        itemView.giveFocus();
                    }
                    else {
                        if (_this.isTextEditMode()) {
                            _this.setTextEditMode(false);
                        }
                        itemView.select(null, null, event.isNew());
                    }
                };
                this.itemViewRemovedListener = function (event) {
                    event.getView().toItemViewArray().forEach(function (itemView) {
                        _this.unregisterItemView(itemView);
                    });
                };
                _super.call(this, new liveedit.ItemViewBuilder().setLiveEditModel(builder.liveEditModel).setItemViewIdProducer(builder.itemViewProducer).setPlaceholder(new liveedit.PagePlaceholder(this)).setTooltipViewer(new api.content.ContentSummaryViewer()).setType(liveedit.PageItemType.get()).setElement(builder.element).setParentElement(builder.element.getParentElement()).setContextMenuActions(this.unlockedScreenActions).setContextMenuTitle(new liveedit.PageViewContextMenuTitle(builder.liveEditModel.getContent())));
                this.addClass('page-view');
                this.setTooltipObject(builder.liveEditModel.getContent());
                this.parseItemViews();
                this.appendChild(this.createTextModeToolbar());
                this.refreshEmptyState();
                var isCustomized = this.liveEditModel.getPageModel().isCustomized();
                if (!this.liveEditModel.getContent().isPageTemplate() && !this.isPageModified(this.pageModel) && !isCustomized) {
                    this.setLocked(true);
                }
                this.listenToMouseEvents();
                this.onRemoved(function (event) { return _this.unregisterPageModel(_this.pageModel); });
            }
            PageView.prototype.registerPageModel = function (pageModel, resetAction) {
                var _this = this;
                if (PageView.debug) {
                    console.log('PageView.registerPageModel', pageModel);
                }
                this.propertyChangedListener = function (event) {
                    if (event.getPropertyName() === PageModel.PROPERTY_REGIONS && !_this.ignorePropertyChanges) {
                        _this.parseItemViews();
                    }
                    _this.refreshEmptyState();
                };
                pageModel.onPropertyChanged(this.propertyChangedListener);
                this.pageModeChangedListener = function (event) {
                    var resetEnabled = event.getNewMode() != 0 /* AUTOMATIC */ && event.getNewMode() != 3 /* NO_CONTROLLER */;
                    if (PageView.debug) {
                        console.log('PageView.pageModeChangedListener setting reset enabled', resetEnabled);
                    }
                    resetAction.setEnabled(resetEnabled);
                };
                pageModel.onPageModeChanged(this.pageModeChangedListener);
            };
            PageView.prototype.unregisterPageModel = function (pageModel) {
                if (PageView.debug) {
                    console.log('PageView.unregisterPageModel', pageModel);
                }
                pageModel.unPropertyChanged(this.propertyChangedListener);
                pageModel.unPageModeChanged(this.pageModeChangedListener);
            };
            PageView.prototype.setIgnorePropertyChanges = function (value) {
                this.ignorePropertyChanges = value;
            };
            PageView.prototype.isPageModified = function (pageModel) {
                return !!pageModel.getDefaultPageTemplate() && pageModel.getDefaultPageTemplate().isPage() && !pageModel.getDefaultPageTemplate().getRegions().equals(pageModel.getRegions());
            };
            PageView.prototype.showTooltip = function () {
                if (!this.isTextEditMode() && !this.isLocked()) {
                    _super.prototype.showTooltip.call(this);
                }
            };
            PageView.prototype.highlight = function () {
                var isDragging = liveedit.DragAndDrop.get().isDragging();
                if (!this.isTextEditMode() && !this.isLocked() && !isDragging) {
                    _super.prototype.highlight.call(this);
                }
            };
            PageView.prototype.showCursor = function () {
                if (!this.isTextEditMode() && !this.isLocked()) {
                    _super.prototype.showCursor.call(this);
                }
            };
            PageView.prototype.shade = function () {
                if (!this.isEmpty()) {
                    _super.prototype.shade.call(this);
                }
            };
            PageView.prototype.unshade = function () {
                if (!this.isLocked()) {
                    _super.prototype.unshade.call(this);
                }
            };
            PageView.prototype.listenToMouseEvents = function () {
                var _this = this;
                liveedit.Shader.get().onUnlockClicked(function (event) {
                    if (_this.isLocked()) {
                        _this.setLocked(false);
                    }
                });
                this.onMouseOverView(function () {
                    var isDragging = liveedit.DragAndDrop.get().isDragging();
                    if (isDragging && _this.lockedContextMenu) {
                        if (_this.lockedContextMenu.isVisible()) {
                            _this.lockedContextMenu.hide();
                        }
                    }
                });
            };
            PageView.prototype.select = function (clickPosition, menuPosition) {
                _super.prototype.select.call(this, clickPosition, menuPosition);
                new liveedit.PageSelectedEvent(this).fire();
            };
            PageView.prototype.selectWithoutMenu = function (isNew) {
                if (isNew === void 0) { isNew = false; }
                _super.prototype.selectWithoutMenu.call(this, isNew);
                new liveedit.PageSelectedEvent(this).fire();
            };
            PageView.prototype.showContextMenu = function (clickPosition, menuPosition) {
                if (!this.isLocked()) {
                    _super.prototype.showContextMenu.call(this, clickPosition, menuPosition);
                }
            };
            PageView.prototype.createLockedContextMenu = function () {
                return new api.liveedit.ItemViewContextMenu(this.getContextMenuTitle(), this.getLockedMenuActions());
            };
            PageView.prototype.getLockedMenuActions = function () {
                var _this = this;
                var unlockAction = new api.ui.Action("Customize");
                unlockAction.onExecuted(function () {
                    _this.setLocked(false);
                });
                return [unlockAction];
            };
            PageView.prototype.selectLocked = function (pos) {
                this.setLockVisible(true);
                this.lockedContextMenu.showAt(pos.x, pos.y);
                new liveedit.ItemViewSelectedEvent(this, pos).fire();
                new liveedit.PageSelectedEvent(this).fire();
            };
            PageView.prototype.deselectLocked = function () {
                this.setLockVisible(false);
                this.lockedContextMenu.hide();
                new liveedit.ItemViewDeselectedEvent(this).fire();
            };
            PageView.prototype.handleShaderClick = function (event) {
                if (this.isLocked()) {
                    if (!this.lockedContextMenu) {
                        this.lockedContextMenu = this.createLockedContextMenu();
                    }
                    if (this.lockedContextMenu.isVisible()) {
                        this.deselectLocked();
                    }
                    else {
                        this.selectLocked({ x: event.pageX, y: event.pageY });
                    }
                }
                else if (this.isSelected()) {
                    this.deselect();
                }
            };
            PageView.prototype.handleClick = function (event) {
                event.stopPropagation();
                if (this.isTextEditMode()) {
                    this.setTextEditMode(false);
                }
                else {
                    _super.prototype.handleClick.call(this, event);
                }
            };
            PageView.prototype.hideContextMenu = function () {
                if (this.lockedContextMenu) {
                    this.lockedContextMenu.hide();
                }
                return _super.prototype.hideContextMenu.call(this);
            };
            PageView.prototype.isLocked = function () {
                return this.hasClass('locked');
            };
            PageView.prototype.setLockVisible = function (visible) {
                this.toggleClass('force-locked', visible);
            };
            PageView.prototype.setLocked = function (locked) {
                this.toggleClass('locked', locked);
                this.hideContextMenu();
                if (locked) {
                    this.shade();
                    new liveedit.PageLockedEvent(this).fire();
                }
                else {
                    this.unshade();
                    if (!this.pageModel.isPageTemplate() || this.pageModel.getMode() == 0 /* AUTOMATIC */) {
                        this.pageModel.initializePageFromDefault(this);
                    }
                    new liveedit.PageUnlockedEvent(this).fire();
                }
            };
            PageView.prototype.createTextModeToolbar = function () {
                var _this = this;
                var toolbar = new api.dom.DivEl('text-edit-toolbar');
                var wrapper = new api.dom.DivEl('wrapper');
                wrapper.setHtml('Text Edit Mode');
                var closeButton = new api.ui.button.CloseButton('transparent');
                closeButton.onClicked(function (event) {
                    event.stopPropagation();
                    event.preventDefault();
                    _this.setTextEditMode(false);
                });
                wrapper.appendChild(closeButton);
                toolbar.appendChild(wrapper);
                return toolbar;
            };
            PageView.prototype.isTextEditMode = function () {
                return this.hasClass('text-edit-mode');
            };
            PageView.prototype.setTextEditMode = function (flag) {
                this.toggleClass('text-edit-mode', flag);
                var textItemViews = this.getItemViewsByType(api.liveedit.text.TextItemType.get());
                var textView;
                textItemViews.forEach(function (view) {
                    textView = view;
                    if (textView.isEditMode() != flag) {
                        textView.setEditMode(flag);
                    }
                });
                if (flag) {
                    new liveedit.PageTextModeStartedEvent(this).fire();
                }
            };
            PageView.prototype.hasTargetWithinTextComponent = function (target) {
                var textItemViews = this.getItemViewsByType(api.liveedit.text.TextItemType.get());
                var result = false;
                var textView;
                textItemViews.forEach(function (view) {
                    textView = view;
                    if (textView.getEl().contains(target)) {
                        result = true;
                        return;
                    }
                });
                return result;
            };
            PageView.prototype.isEmpty = function () {
                return !this.pageModel || this.pageModel.getMode() == 3 /* NO_CONTROLLER */;
            };
            PageView.prototype.getName = function () {
                var content = this.liveEditModel.getContent();
                if (!content || api.util.StringHelper.isEmpty(content.getDisplayName())) {
                    return "[No name]";
                }
                else {
                    return content.getDisplayName();
                }
            };
            PageView.prototype.getParentItemView = function () {
                return null;
            };
            PageView.prototype.setParentItemView = function (itemView) {
                throw new Error("PageView is the topmost item view and cannot have a parent");
            };
            PageView.prototype.registerRegionView = function (regionView) {
                this.regionViews.push(regionView);
                regionView.onItemViewAdded(this.itemViewAddedListener);
                regionView.onItemViewRemoved(this.itemViewRemovedListener);
            };
            PageView.prototype.unregisterRegionView = function (regionView) {
                var index = this.regionViews.indexOf(regionView);
                if (index > -1) {
                    this.regionViews.splice(index, 1);
                    regionView.unItemViewAdded(this.itemViewAddedListener);
                    regionView.unItemViewRemoved(this.itemViewRemovedListener);
                }
            };
            PageView.prototype.getRegions = function () {
                return this.regionViews;
            };
            PageView.prototype.toItemViewArray = function () {
                var array = [];
                array.push(this);
                this.regionViews.forEach(function (regionView) {
                    var itemViews = regionView.toItemViewArray();
                    array = array.concat(itemViews);
                });
                return array;
            };
            PageView.prototype.hasSelectedView = function () {
                return !!this.getSelectedView();
            };
            PageView.prototype.getSelectedView = function () {
                for (var id in this.viewsById) {
                    if (this.viewsById.hasOwnProperty(id) && this.viewsById[id].isSelected()) {
                        return this.viewsById[id];
                    }
                }
                return null;
            };
            PageView.prototype.getItemViewById = function (id) {
                api.util.assertNotNull(id, "value cannot be null");
                return this.viewsById[id.toNumber()];
            };
            PageView.prototype.getItemViewsByType = function (type) {
                var views = [];
                for (var key in this.viewsById) {
                    if (this.viewsById.hasOwnProperty(key)) {
                        var view = this.viewsById[key];
                        if (type.equals(view.getType())) {
                            views.push(view);
                        }
                    }
                }
                return views;
            };
            PageView.prototype.getItemViewByElement = function (element) {
                api.util.assertNotNull(element, "element cannot be null");
                var itemId = liveedit.ItemView.parseItemId(element);
                if (!itemId) {
                    return null;
                }
                var itemView = this.getItemViewById(itemId);
                api.util.assertNotNull(itemView, "ItemView not found: " + itemId.toString());
                return itemView;
            };
            PageView.prototype.getRegionViewByElement = function (element) {
                var itemView = this.getItemViewByElement(element);
                if (api.ObjectHelper.iFrameSafeInstanceOf(itemView, liveedit.RegionView)) {
                    return itemView;
                }
                return null;
            };
            PageView.prototype.getComponentViewByElement = function (element) {
                var itemView = this.getItemViewByElement(element);
                if (api.ObjectHelper.iFrameSafeInstanceOf(itemView, liveedit.ComponentView)) {
                    return itemView;
                }
                return null;
            };
            PageView.prototype.getRegionViewByPath = function (path) {
                for (var i = 0; i < this.regionViews.length; i++) {
                    var regionView = this.regionViews[i];
                    if (path.hasParentComponentPath()) {
                        var componentView = this.getComponentViewByPath(path.getParentComponentPath());
                        if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, api.liveedit.layout.LayoutComponentView)) {
                            var layoutView = componentView;
                            layoutView.getRegionViewByName(path.getRegionName());
                        }
                    }
                    else {
                        if (path.getRegionName() == regionView.getRegionName()) {
                            return regionView;
                        }
                    }
                }
                return null;
            };
            PageView.prototype.getComponentViewByPath = function (path) {
                var firstLevelOfPath = path.getFirstLevel();
                for (var i = 0; i < this.regionViews.length; i++) {
                    var regionView = this.regionViews[i];
                    if (firstLevelOfPath.getRegionName() == regionView.getRegionName()) {
                        if (path.numberOfLevels() == 1) {
                            return regionView.getComponentViewByIndex(firstLevelOfPath.getComponentIndex());
                        }
                        else {
                            var layoutView = regionView.getComponentViewByIndex(firstLevelOfPath.getComponentIndex());
                            return layoutView.getComponentViewByPath(path.removeFirstLevel());
                        }
                    }
                }
                return null;
            };
            PageView.prototype.registerItemView = function (view) {
                if (PageView.debug) {
                    console.debug("PageView.registerItemView: " + view.toString());
                }
                this.viewsById[view.getItemId().toNumber()] = view;
                this.notifyItemViewAdded(view);
            };
            PageView.prototype.unregisterItemView = function (view) {
                if (PageView.debug) {
                    console.debug("PageView.unregisterItemView: " + view.toString());
                }
                delete this.viewsById[view.getItemId().toNumber()];
                this.notifyItemViewRemoved(view);
            };
            PageView.prototype.parseItemViews = function () {
                var _this = this;
                for (var itemView in this.viewsById) {
                    if (this.viewsById.hasOwnProperty(itemView)) {
                        this.unregisterItemView(this.viewsById[itemView]);
                    }
                }
                this.regionViews.forEach(function (regionView) {
                    _this.unregisterRegionView(regionView);
                });
                this.regionViews = [];
                this.viewsById = {};
                this.doParseItemViews();
                this.toItemViewArray().forEach(function (itemView) {
                    _this.registerItemView(itemView);
                });
            };
            PageView.prototype.deselectChildViews = function () {
                for (var itemView in this.viewsById) {
                    var view = this.viewsById[itemView];
                    if (api.ObjectHelper.iFrameSafeInstanceOf(view, liveedit.ItemView) && view.isSelected()) {
                        view.deselect();
                    }
                }
            };
            PageView.prototype.doParseItemViews = function (parentElement) {
                var _this = this;
                var pageRegions = this.liveEditModel.getPageModel().getRegions();
                if (!pageRegions) {
                    return;
                }
                var children = parentElement ? parentElement.getChildren() : this.getChildren();
                children.forEach(function (childElement) {
                    var itemType = liveedit.ItemType.fromElement(childElement);
                    var isRegionView = api.ObjectHelper.iFrameSafeInstanceOf(childElement, liveedit.RegionView);
                    var region, regionName, regionView;
                    if (isRegionView) {
                        regionName = liveedit.RegionItemType.getRegionName(childElement);
                        region = pageRegions.getRegionByName(regionName);
                        if (region) {
                            regionView = childElement;
                            regionView.setRegion(region);
                            _this.registerRegionView(regionView);
                        }
                    }
                    else if (itemType && liveedit.RegionItemType.get().equals(itemType)) {
                        regionName = liveedit.RegionItemType.getRegionName(childElement);
                        region = pageRegions.getRegionByName(regionName);
                        if (region) {
                            regionView = new liveedit.RegionView(new liveedit.RegionViewBuilder().setLiveEditModel(_this.liveEditModel).setParentView(_this).setRegion(region).setElement(childElement));
                            _this.registerRegionView(regionView);
                        }
                    }
                    else {
                        _this.doParseItemViews(childElement);
                    }
                });
            };
            PageView.prototype.onItemViewAdded = function (listener) {
                this.itemViewAddedListeners.push(listener);
            };
            PageView.prototype.unItemViewAdded = function (listener) {
                this.itemViewAddedListeners = this.itemViewAddedListeners.filter(function (current) { return (current != listener); });
            };
            PageView.prototype.notifyItemViewAdded = function (itemView) {
                var event = new liveedit.ItemViewAddedEvent(itemView);
                this.itemViewAddedListeners.forEach(function (listener) { return listener(event); });
            };
            PageView.prototype.onItemViewRemoved = function (listener) {
                this.itemViewRemovedListeners.push(listener);
            };
            PageView.prototype.unItemViewRemoved = function (listener) {
                this.itemViewRemovedListeners = this.itemViewRemovedListeners.filter(function (current) { return (current != listener); });
            };
            PageView.prototype.notifyItemViewRemoved = function (itemView) {
                var event = new liveedit.ItemViewRemovedEvent(itemView);
                this.itemViewRemovedListeners.forEach(function (listener) { return listener(event); });
            };
            PageView.prototype.setDisabledContextMenu = function (value) {
                this.disableContextMenu = value;
            };
            PageView.prototype.isDisabledContextMenu = function () {
                return this.disableContextMenu;
            };
            return PageView;
        })(liveedit.ItemView);
        liveedit.PageView = PageView;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ContentViewBuilder = (function () {
            function ContentViewBuilder() {
            }
            ContentViewBuilder.prototype.setParentPartComponentView = function (value) {
                this.parentPartComponentView = value;
                return this;
            };
            ContentViewBuilder.prototype.setParentElement = function (value) {
                this.parentElement = value;
                return this;
            };
            ContentViewBuilder.prototype.setElement = function (value) {
                this.element = value;
                return this;
            };
            return ContentViewBuilder;
        })();
        liveedit.ContentViewBuilder = ContentViewBuilder;
        var ContentView = (function (_super) {
            __extends(ContentView, _super);
            function ContentView(builder) {
                _super.call(this, new liveedit.ItemViewBuilder().setItemViewIdProducer(builder.parentPartComponentView.getItemViewIdProducer()).setType(liveedit.ContentItemType.get()).setElement(builder.element).setParentElement(builder.parentElement).setParentView(builder.parentPartComponentView).setContextMenuActions(this.createContentContextMenuActions()).setContextMenuTitle(new liveedit.ContentViewContextMenuTitle(this)));
                this.parentPartComponentView = builder.parentPartComponentView;
            }
            ContentView.prototype.createContentContextMenuActions = function () {
                var actions = [];
                actions.push(this.createSelectParentAction());
                actions.push(new api.ui.Action('Insert').onExecuted(function () {
                }));
                actions.push(new api.ui.Action('View').onExecuted(function () {
                }));
                actions.push(new api.ui.Action('Edit').onExecuted(function () {
                }));
                return actions;
            };
            ContentView.prototype.isEmpty = function () {
                return false;
            };
            ContentView.prototype.getParentItemView = function () {
                return this.parentPartComponentView;
            };
            ContentView.prototype.setParentItemView = function (partView) {
                _super.prototype.setParentItemView.call(this, partView);
                this.parentPartComponentView = partView;
            };
            return ContentView;
        })(liveedit.ItemView);
        liveedit.ContentView = ContentView;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageLockedEvent = (function (_super) {
            __extends(PageLockedEvent, _super);
            function PageLockedEvent(pageView) {
                _super.call(this);
                this.pageView = pageView;
            }
            PageLockedEvent.prototype.getPageView = function () {
                return this.pageView;
            };
            PageLockedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            PageLockedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return PageLockedEvent;
        })(api.event.Event);
        liveedit.PageLockedEvent = PageLockedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageUnlockedEvent = (function (_super) {
            __extends(PageUnlockedEvent, _super);
            function PageUnlockedEvent(pageView) {
                _super.call(this);
                this.pageView = pageView;
            }
            PageUnlockedEvent.prototype.getPageView = function () {
                return this.pageView;
            };
            PageUnlockedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            PageUnlockedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return PageUnlockedEvent;
        })(api.event.Event);
        liveedit.PageUnlockedEvent = PageUnlockedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageUnloadedEvent = (function (_super) {
            __extends(PageUnloadedEvent, _super);
            function PageUnloadedEvent(pageView) {
                _super.call(this);
                this.pageView = pageView;
            }
            PageUnloadedEvent.prototype.getPageView = function () {
                return this.pageView;
            };
            PageUnloadedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            PageUnloadedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return PageUnloadedEvent;
        })(api.event.Event);
        liveedit.PageUnloadedEvent = PageUnloadedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageTextModeStartedEvent = (function (_super) {
            __extends(PageTextModeStartedEvent, _super);
            function PageTextModeStartedEvent(pageView) {
                _super.call(this);
                this.pageView = pageView;
            }
            PageTextModeStartedEvent.prototype.getPageView = function () {
                return this.pageView;
            };
            PageTextModeStartedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            PageTextModeStartedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return PageTextModeStartedEvent;
        })(api.event.Event);
        liveedit.PageTextModeStartedEvent = PageTextModeStartedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ImageOpenUploadDialogEvent = (function (_super) {
            __extends(ImageOpenUploadDialogEvent, _super);
            function ImageOpenUploadDialogEvent(target) {
                _super.call(this);
                this.targetImagePlaceholder = target;
            }
            ImageOpenUploadDialogEvent.prototype.getTargetImagePlaceholder = function () {
                return this.targetImagePlaceholder;
            };
            ImageOpenUploadDialogEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ImageOpenUploadDialogEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ImageOpenUploadDialogEvent;
        })(api.event.Event);
        liveedit.ImageOpenUploadDialogEvent = ImageOpenUploadDialogEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ImageUploadedEvent = (function (_super) {
            __extends(ImageUploadedEvent, _super);
            function ImageUploadedEvent(uploadedItem, target) {
                _super.call(this);
                this.uploadedItem = uploadedItem;
                this.targetImagePlaceholder = target;
            }
            ImageUploadedEvent.prototype.getUploadedItem = function () {
                return this.uploadedItem;
            };
            ImageUploadedEvent.prototype.getTargetImagePlaceholder = function () {
                return this.targetImagePlaceholder;
            };
            ImageUploadedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ImageUploadedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ImageUploadedEvent;
        })(api.event.Event);
        liveedit.ImageUploadedEvent = ImageUploadedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ComponentViewDragStartedEvent = (function (_super) {
            __extends(ComponentViewDragStartedEvent, _super);
            function ComponentViewDragStartedEvent(componentView) {
                _super.call(this);
                this.componentView = componentView;
            }
            ComponentViewDragStartedEvent.prototype.getComponentView = function () {
                return this.componentView;
            };
            ComponentViewDragStartedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ComponentViewDragStartedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ComponentViewDragStartedEvent;
        })(Event);
        liveedit.ComponentViewDragStartedEvent = ComponentViewDragStartedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ComponentViewDragStoppedEvent = (function (_super) {
            __extends(ComponentViewDragStoppedEvent, _super);
            function ComponentViewDragStoppedEvent(componentView) {
                _super.call(this);
                this.componentView = componentView;
            }
            ComponentViewDragStoppedEvent.prototype.getComponentView = function () {
                return this.componentView;
            };
            ComponentViewDragStoppedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ComponentViewDragStoppedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ComponentViewDragStoppedEvent;
        })(Event);
        liveedit.ComponentViewDragStoppedEvent = ComponentViewDragStoppedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ComponentViewDragCanceledEvent = (function (_super) {
            __extends(ComponentViewDragCanceledEvent, _super);
            function ComponentViewDragCanceledEvent(componentView) {
                _super.call(this);
                this.componentView = componentView;
            }
            ComponentViewDragCanceledEvent.prototype.getComponentView = function () {
                return this.componentView;
            };
            ComponentViewDragCanceledEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ComponentViewDragCanceledEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ComponentViewDragCanceledEvent;
        })(Event);
        liveedit.ComponentViewDragCanceledEvent = ComponentViewDragCanceledEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ComponentViewDragDroppedEvent = (function (_super) {
            __extends(ComponentViewDragDroppedEvent, _super);
            function ComponentViewDragDroppedEvent(view, region) {
                _super.call(this);
                this.componentView = view;
                this.regionView = region;
            }
            ComponentViewDragDroppedEvent.prototype.getComponentView = function () {
                return this.componentView;
            };
            ComponentViewDragDroppedEvent.prototype.getRegionView = function () {
                return this.regionView;
            };
            ComponentViewDragDroppedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ComponentViewDragDroppedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ComponentViewDragDroppedEvent;
        })(Event);
        liveedit.ComponentViewDragDroppedEvent = ComponentViewDragDroppedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var PageSelectedEvent = (function (_super) {
            __extends(PageSelectedEvent, _super);
            function PageSelectedEvent(pageView) {
                _super.call(this);
                this.pageView = pageView;
            }
            PageSelectedEvent.prototype.getPageView = function () {
                return this.pageView;
            };
            PageSelectedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            PageSelectedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return PageSelectedEvent;
        })(api.event.Event);
        liveedit.PageSelectedEvent = PageSelectedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var RegionSelectedEvent = (function (_super) {
            __extends(RegionSelectedEvent, _super);
            function RegionSelectedEvent(regionView) {
                _super.call(this);
                this.pageItemView = regionView;
            }
            RegionSelectedEvent.prototype.getRegionView = function () {
                return this.pageItemView;
            };
            RegionSelectedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            RegionSelectedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return RegionSelectedEvent;
        })(api.event.Event);
        liveedit.RegionSelectedEvent = RegionSelectedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ComponentAddedEvent = (function (_super) {
            __extends(ComponentAddedEvent, _super);
            function ComponentAddedEvent(componentView, regionView) {
                _super.call(this);
                this.componentView = componentView;
                this.parentRegionView = regionView;
            }
            ComponentAddedEvent.prototype.getComponentView = function () {
                return this.componentView;
            };
            ComponentAddedEvent.prototype.getParentRegionView = function () {
                return this.parentRegionView;
            };
            ComponentAddedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ComponentAddedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ComponentAddedEvent;
        })(api.event.Event);
        liveedit.ComponentAddedEvent = ComponentAddedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var ComponentRemovedEvent = (function (_super) {
            __extends(ComponentRemovedEvent, _super);
            function ComponentRemovedEvent(componentView, regionView) {
                _super.call(this);
                this.componentView = componentView;
                this.parentRegionView = regionView;
            }
            ComponentRemovedEvent.prototype.getComponentView = function () {
                return this.componentView;
            };
            ComponentRemovedEvent.prototype.getParentRegionView = function () {
                return this.parentRegionView;
            };
            ComponentRemovedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ComponentRemovedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ComponentRemovedEvent;
        })(api.event.Event);
        liveedit.ComponentRemovedEvent = ComponentRemovedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ComponentResetEvent = (function (_super) {
            __extends(ComponentResetEvent, _super);
            function ComponentResetEvent(newComponentView, oldComponentView) {
                _super.call(this);
                this.newComponentView = newComponentView;
                this.oldComponentView = oldComponentView;
            }
            ComponentResetEvent.prototype.getNewComponentView = function () {
                return this.newComponentView;
            };
            ComponentResetEvent.prototype.getOldComponentView = function () {
                return this.oldComponentView;
            };
            ComponentResetEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ComponentResetEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ComponentResetEvent;
        })(api.event.Event);
        liveedit.ComponentResetEvent = ComponentResetEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ComponentDuplicatedEvent = (function (_super) {
            __extends(ComponentDuplicatedEvent, _super);
            function ComponentDuplicatedEvent(originalComponentView, duplicatedComponentView) {
                _super.call(this);
                this.originalComponentView = originalComponentView;
                this.duplicatedComponentView = duplicatedComponentView;
            }
            ComponentDuplicatedEvent.prototype.getOriginalComponentView = function () {
                return this.originalComponentView;
            };
            ComponentDuplicatedEvent.prototype.getDuplicatedComponentView = function () {
                return this.duplicatedComponentView;
            };
            ComponentDuplicatedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ComponentDuplicatedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ComponentDuplicatedEvent;
        })(api.event.Event);
        liveedit.ComponentDuplicatedEvent = ComponentDuplicatedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var Event = api.event.Event;
        var ComponentLoadedEvent = (function (_super) {
            __extends(ComponentLoadedEvent, _super);
            function ComponentLoadedEvent(newComponentView, oldComponentView) {
                _super.call(this);
                this.newComponentView = newComponentView;
                this.oldComponentView = oldComponentView;
            }
            ComponentLoadedEvent.prototype.getNewComponentView = function () {
                return this.newComponentView;
            };
            ComponentLoadedEvent.prototype.getOldComponentView = function () {
                return this.oldComponentView;
            };
            ComponentLoadedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ComponentLoadedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ComponentLoadedEvent;
        })(Event);
        liveedit.ComponentLoadedEvent = ComponentLoadedEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var LiveEditPageViewReadyEvent = (function (_super) {
            __extends(LiveEditPageViewReadyEvent, _super);
            function LiveEditPageViewReadyEvent(pageView) {
                _super.call(this);
                this.pageView = pageView;
            }
            LiveEditPageViewReadyEvent.prototype.getPageView = function () {
                return this.pageView;
            };
            LiveEditPageViewReadyEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            LiveEditPageViewReadyEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return LiveEditPageViewReadyEvent;
        })(api.event.Event);
        liveedit.LiveEditPageViewReadyEvent = LiveEditPageViewReadyEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var DragHelper = api.ui.DragHelper;
        var DragAndDrop = (function () {
            function DragAndDrop(pageView) {
                this.REGION_SELECTOR = liveedit.RegionItemType.get().getConfig().getCssSelector();
                this.ITEM_NOT_DRAGGABLE_SELECTOR = '.not-draggable';
                this.DRAGGED_OVER_CLASS = 'dragged-over';
                this.DRAGGING_ACTIVE_CLASS = 'dragging';
                this._isDragging = false;
                this._wasDropped = false;
                this.dragStartedListeners = [];
                this.dragStoppedListeners = [];
                this.droppedListeners = [];
                this.canceledListeners = [];
                this.pageView = pageView;
                this.SORTABLE_ITEMS_SELECTOR = this.createSortableItemsSelector();
                this.createSortable(this.REGION_SELECTOR);
            }
            DragAndDrop.init = function (pageView) {
                DragAndDrop.instance = new DragAndDrop(pageView);
            };
            DragAndDrop.get = function () {
                if (!DragAndDrop.instance) {
                    throw Error('Do DragAndDrop.init(pageView) first');
                }
                return DragAndDrop.instance;
            };
            DragAndDrop.prototype.isDragging = function () {
                return this._isDragging;
            };
            DragAndDrop.prototype.createSortableLayout = function (component) {
                var _this = this;
                wemjq(component.getHTMLElement()).find(this.REGION_SELECTOR).each(function (index, element) {
                    _this.createSortable(wemjq(element));
                });
            };
            DragAndDrop.prototype.refreshSortable = function () {
                wemjq(this.REGION_SELECTOR).sortable('refresh');
            };
            DragAndDrop.prototype.processMouseOverRegionView = function (regionView) {
                var _this = this;
                this.pageView.getItemViewsByType(liveedit.RegionItemType.get()).forEach(function (region) {
                    region.toggleClass(_this.DRAGGED_OVER_CLASS, region.getRegionPath().equals(regionView.getRegionPath()));
                });
                regionView.refreshEmptyState();
            };
            DragAndDrop.prototype.processMouseOutRegionView = function (regionView) {
                regionView.refreshEmptyState().removeClass(this.DRAGGED_OVER_CLASS);
            };
            DragAndDrop.prototype.createSortable = function (selector) {
                var _this = this;
                wemjq(selector).sortable({
                    appendTo: document.body,
                    revert: false,
                    cancel: this.ITEM_NOT_DRAGGABLE_SELECTOR,
                    connectWith: this.REGION_SELECTOR,
                    items: this.SORTABLE_ITEMS_SELECTOR,
                    distance: 20,
                    delay: 50,
                    tolerance: 'intersect',
                    cursor: 'move',
                    cursorAt: DragHelper.CURSOR_AT,
                    scrollSensitivity: this.calculateScrollSensitivity(),
                    placeholder: 'live-edit-drag-placeholder-container',
                    forceHelperSize: true,
                    helper: function (event, ui) { return DragHelper.get().getHTMLElement(); },
                    start: function (event, ui) { return _this.handleSortStart(event, ui); },
                    activate: function (event, ui) { return _this.handleActivate(event, ui); },
                    over: function (event, ui) { return _this.handleDragOver(event, ui); },
                    out: function (event, ui) { return _this.handleDragOut(event, ui); },
                    beforeStop: function (event, ui) { return _this.handleBeforeStop(event, ui); },
                    receive: function (event, ui) { return _this.handleReceive(event, ui); },
                    deactivate: function (event, ui) { return _this.handleDeactivate(event, ui); },
                    stop: function (event, ui) { return _this.handleSortStop(event, ui); },
                    remove: function (event, ui) { return _this.handleRemove(event, ui); }
                });
            };
            DragAndDrop.prototype.createDraggable = function (jq) {
                var _this = this;
                this._newItemItemType = liveedit.ItemType.fromHTMLElement(jq.get(0));
                jq.draggable({
                    connectToSortable: this.REGION_SELECTOR,
                    addClasses: false,
                    cursor: 'move',
                    appendTo: 'body',
                    cursorAt: DragHelper.CURSOR_AT,
                    helper: function (event, ui) { return DragHelper.get().getHTMLElement(); },
                    start: function (event, ui) { return _this.handleDraggableStart(event, ui); },
                    stop: function (event, ui) { return _this.handleDraggableStop(event, ui); }
                });
            };
            DragAndDrop.prototype.destroyDraggable = function (jq) {
                jq.draggable("destroy");
            };
            DragAndDrop.prototype.handleDraggableStart = function (event, ui) {
                if (DragAndDrop.debug) {
                    console.groupCollapsed((DragAndDrop._messageCounter++) + " DragDropSort.handleDraggableStart");
                    console.log("Event", event, "\nUI", ui);
                    console.groupEnd();
                }
                this.notifyDragStarted(this._draggedComponentView);
            };
            DragAndDrop.prototype.handleDraggableStop = function (event, ui) {
                if (DragAndDrop.debug) {
                    console.groupCollapsed((DragAndDrop._messageCounter++) + " DragDropSort.handleDraggableStop");
                    console.log("Event", event, "\nUI", ui);
                    console.groupEnd();
                }
                if (!this._wasDropped) {
                    this.notifyCanceled(this._draggedComponentView);
                }
                this.notifyDragStopped(this._draggedComponentView);
            };
            DragAndDrop.prototype.handleSortStart = function (event, ui) {
                if (DragAndDrop.debug) {
                    console.groupCollapsed((DragAndDrop._messageCounter++) + " DragDropSort.handleSortStart");
                    console.log("Event", event, "\nUI", ui);
                    console.groupEnd();
                }
                var regionView = this.getRegionView(ui.item.parent());
                var itemType;
                var placeholder = liveedit.DragPlaceholder.get().setRegionView(regionView);
                if (this.isDraggingFromContextWindow()) {
                    itemType = this._newItemItemType;
                }
                else {
                    this._draggedComponentView = this.getComponentView(ui.item);
                    this._draggedComponentView.deselect();
                    this._draggedComponentView.setMoving(true);
                    itemType = this._draggedComponentView.getType();
                }
                ui.placeholder.append(placeholder.setItemType(itemType).getHTMLElement());
                ui.helper.show();
                this.processMouseOverRegionView(regionView);
                this.updateHelperAndPlaceholder(regionView);
                this.updateScrollSensitivity(event.target);
                this.notifyDragStarted(this._draggedComponentView);
            };
            DragAndDrop.prototype.handleBeforeStop = function (event, ui) {
                if (DragAndDrop.debug) {
                    console.groupCollapsed((DragAndDrop._messageCounter++) + " DragDropSort.handleBeforeStop");
                    console.log("Event", event, "\nUI", ui);
                    console.groupEnd();
                }
                ui.helper.hide();
            };
            DragAndDrop.prototype.handleSortStop = function (event, ui) {
                if (DragAndDrop.debug) {
                    console.groupCollapsed((DragAndDrop._messageCounter++) + " DragDropSort.handleSortStop");
                    console.log("Event", event, "\nUI", ui);
                    console.groupEnd();
                }
                var regionView = this.getRegionView(ui.item.parent());
                if (!DragHelper.get().isDropAllowed()) {
                    if (DragAndDrop.debug) {
                        console.log("DragAndDrop.handleStop: cancelling drag because it is not allowed to drop here...");
                    }
                    this.cancelDrag(event, ui);
                }
                else {
                    var componentIndex = wemjq('>.drag-helper, >.item-view', regionView.getHTMLElement()).index(ui.item);
                    if (this.isDraggingFromContextWindow()) {
                        if (this.pageView.isLocked()) {
                            this.pageView.setLocked(false);
                        }
                        var componentType = this._newItemItemType;
                        var newComponent = regionView.createComponent(componentType.toComponentType());
                        this._draggedComponentView = componentType.createView(new liveedit.CreateItemViewConfig().setParentView(regionView).setParentElement(regionView).setData(newComponent).setPositionIndex(componentIndex));
                        regionView.addComponentView(this._draggedComponentView, componentIndex, true);
                    }
                    else {
                        if (this._draggedComponentView.hasComponentPath()) {
                            this._draggedComponentView.moveToRegion(regionView, componentIndex);
                        }
                    }
                    this.notifyDropped(this._draggedComponentView, regionView);
                }
                if (!this.isDraggingFromContextWindow()) {
                    this._draggedComponentView.setMoving(false);
                }
                regionView.refreshEmptyState();
                this.notifyDragStopped(this._draggedComponentView);
                if (this.isDraggingFromContextWindow()) {
                    ui.item.remove();
                }
                this._newItemItemType = null;
                this._draggedComponentView = null;
            };
            DragAndDrop.prototype.handleActivate = function (event, ui) {
                if (DragAndDrop.debug) {
                    console.groupCollapsed((DragAndDrop._messageCounter++) + " DragDropSort.handleActivate");
                    console.log("Event", event, "\nUI", ui);
                    console.groupEnd();
                }
                this.getRegionView(wemjq(event.target)).addClass(this.DRAGGING_ACTIVE_CLASS);
            };
            DragAndDrop.prototype.handleDeactivate = function (event, ui) {
                if (DragAndDrop.debug) {
                    console.groupCollapsed((DragAndDrop._messageCounter++) + " DragDropSort.handleDeactivate");
                    console.log("Event", event, "\nUI", ui);
                    console.groupEnd();
                }
                this.getRegionView(wemjq(event.target)).removeClass(this.DRAGGING_ACTIVE_CLASS);
            };
            DragAndDrop.prototype.handleDragOver = function (event, ui) {
                if (DragAndDrop.debug) {
                    console.groupCollapsed((DragAndDrop._messageCounter++) + " DragDropSort.handleDragOver");
                    console.log("Event", event, "\nUI", ui);
                    console.groupEnd();
                }
                var regionView = this.getRegionView(ui.placeholder.parent());
                this.processMouseOverRegionView(regionView);
                this.updateHelperAndPlaceholder(regionView);
            };
            DragAndDrop.prototype.handleDragOut = function (event, ui) {
                if (DragAndDrop.debug) {
                    console.groupCollapsed((DragAndDrop._messageCounter++) + " DragDropSort.handleDragOut");
                    console.log("Event", event, "\nUI", ui);
                    console.groupEnd();
                }
                if (!ui.placeholder || !ui.placeholder.parent()[0]) {
                    if (DragAndDrop.debug) {
                        console.log('DragAndDrop.handleDragOut skipping because there is no placeholder, probably item has been already dropped...');
                    }
                    return;
                }
                var regionView = this.getRegionView(ui.placeholder.parent());
                this.processMouseOutRegionView(regionView);
                this.updateHelperAndPlaceholder(regionView, false);
            };
            DragAndDrop.prototype.handleSortChange = function (event, ui) {
                if (DragAndDrop.debug) {
                    console.groupCollapsed((DragAndDrop._messageCounter++) + " DragDropSort.handleSortChange");
                    console.log("Event", event, "\nUI", ui);
                    console.groupEnd();
                }
            };
            DragAndDrop.prototype.handleSortUpdate = function (event, ui) {
                if (DragAndDrop.debug) {
                    console.groupCollapsed((DragAndDrop._messageCounter++) + " DragDropSort.handleSortUpdate");
                    console.log("Event", event, "\nUI", ui);
                    console.groupEnd();
                }
            };
            DragAndDrop.prototype.handleRemove = function (event, ui) {
                if (DragAndDrop.debug) {
                    console.groupCollapsed((DragAndDrop._messageCounter++) + " DragDropSort.handleRemove");
                    console.log("Event", event, "\nUI", ui);
                    console.groupEnd();
                }
                var fromRegionView = this.getRegionView(wemjq(event.target));
                fromRegionView.refreshEmptyState();
            };
            DragAndDrop.prototype.handleReceive = function (event, ui) {
                if (DragAndDrop.debug) {
                    console.groupCollapsed((DragAndDrop._messageCounter++) + " DragDropSort.handleReceive");
                    console.log("Event", event, "\nUI", ui);
                    console.groupEnd();
                }
                var toRegionView = this.getRegionView(wemjq(event.target));
                toRegionView.refreshEmptyState();
            };
            DragAndDrop.prototype.cancelDrag = function (event, ui) {
                wemjq(event.target).sortable('cancel');
                this.notifyCanceled(this._draggedComponentView);
            };
            DragAndDrop.prototype.onDragStarted = function (listener) {
                this.dragStartedListeners.push(listener);
            };
            DragAndDrop.prototype.unDragStarted = function (listener) {
                this.dragStartedListeners = this.dragStartedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            DragAndDrop.prototype.notifyDragStarted = function (componentView) {
                if (DragAndDrop.debug) {
                    console.log('DragAndDrop.notifyDragStarted', componentView);
                }
                this._isDragging = true;
                this._wasDropped = false;
                this.dragStartedListeners.forEach(function (curr) {
                    curr(componentView);
                });
                new liveedit.ComponentViewDragStartedEvent(componentView).fire();
            };
            DragAndDrop.prototype.onDragStopped = function (listener) {
                this.dragStoppedListeners.push(listener);
            };
            DragAndDrop.prototype.unDragStopped = function (listener) {
                this.dragStoppedListeners = this.dragStoppedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            DragAndDrop.prototype.notifyDragStopped = function (componentView) {
                if (DragAndDrop.debug) {
                    console.log('DragAndDrop.notifyDragStopped', componentView);
                }
                this._isDragging = false;
                DragHelper.get().reset();
                this.dragStoppedListeners.forEach(function (curr) {
                    curr(componentView);
                });
                new liveedit.ComponentViewDragStoppedEvent(componentView).fire();
            };
            DragAndDrop.prototype.onDropped = function (listener) {
                this.droppedListeners.push(listener);
            };
            DragAndDrop.prototype.unDropped = function (listener) {
                this.droppedListeners = this.droppedListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            DragAndDrop.prototype.notifyDropped = function (componentView, regionView) {
                if (DragAndDrop.debug) {
                    console.log('DragAndDrop.notifyDropped', componentView, regionView);
                }
                this.droppedListeners.forEach(function (curr) {
                    curr(componentView, regionView);
                });
                this._wasDropped = true;
                new liveedit.ComponentViewDragDroppedEvent(componentView, regionView).fire();
            };
            DragAndDrop.prototype.onCanceled = function (listener) {
                this.canceledListeners.push(listener);
            };
            DragAndDrop.prototype.unCanceled = function (listener) {
                this.canceledListeners = this.canceledListeners.filter(function (curr) {
                    return curr != listener;
                });
            };
            DragAndDrop.prototype.notifyCanceled = function (componentView) {
                if (DragAndDrop.debug) {
                    console.log('DragAndDrop.notifyCanceled', componentView);
                }
                this.canceledListeners.forEach(function (curr) {
                    curr(componentView);
                });
                new liveedit.ComponentViewDragCanceledEvent(componentView).fire();
            };
            DragAndDrop.prototype.updateHelperAndPlaceholder = function (regionView, enter) {
                if (enter === void 0) { enter = true; }
                var helper = DragHelper.get();
                var placeholder = liveedit.DragPlaceholder.get().setRegionView(enter ? regionView : null);
                if (!enter) {
                    helper.setDropAllowed(false);
                }
                else if (this.isDraggingLayoutOverLayout(regionView, this.getItemType())) {
                    helper.setDropAllowed(false);
                    placeholder.setText("Layout within layout not allowed");
                    placeholder.setDropAllowed(false);
                }
                else {
                    helper.setDropAllowed(true);
                }
            };
            DragAndDrop.prototype.getItemType = function () {
                if (this._draggedComponentView) {
                    return this._draggedComponentView.getType();
                }
                else if (this._newItemItemType) {
                    return this._newItemItemType;
                }
                else {
                    throw new Error('Dragged component and new item type can not be both null');
                }
            };
            DragAndDrop.prototype.isDraggingFromContextWindow = function () {
                return !!this._newItemItemType;
            };
            DragAndDrop.prototype.isDraggingLayoutOverLayout = function (regionView, draggingItemType) {
                var isLayout = regionView.hasParentLayoutComponentView() && draggingItemType.getShortName() == 'layout';
                if (DragAndDrop.debug) {
                    console.log('DragAndDrop.isDraggingLayoutOverLayout = ' + isLayout);
                }
                return isLayout;
            };
            DragAndDrop.prototype.getComponentView = function (jq) {
                var comp = this.pageView.getComponentViewByElement(jq.get(0));
                api.util.assertState(!!comp, "ComponentView is not expected to be null");
                return comp;
            };
            DragAndDrop.prototype.getRegionView = function (jq) {
                var region = this.pageView.getRegionViewByElement(jq.get(0));
                api.util.assertState(!!region, "RegionView is not expected to be null");
                return region;
            };
            DragAndDrop.prototype.createSortableItemsSelector = function () {
                var sortableItemsSelector = [];
                liveedit.ItemType.getDraggables().forEach(function (draggableItemType) {
                    sortableItemsSelector.push(draggableItemType.getConfig().getCssSelector());
                });
                return sortableItemsSelector.toString();
            };
            DragAndDrop.prototype.updateScrollSensitivity = function (selector) {
                var scrollSensitivity = this.calculateScrollSensitivity();
                wemjq(selector).sortable('option', 'scrollSensitivity', scrollSensitivity);
            };
            DragAndDrop.prototype.calculateScrollSensitivity = function () {
                var height = wemjq(window).height();
                var scrollSensitivity = Math.round(height / 8);
                scrollSensitivity = Math.max(20, Math.min(scrollSensitivity, 100));
                return scrollSensitivity;
            };
            DragAndDrop.debug = false;
            DragAndDrop._messageCounter = 0;
            return DragAndDrop;
        })();
        liveedit.DragAndDrop = DragAndDrop;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var LiveEditPageInitializationErrorEvent = (function (_super) {
            __extends(LiveEditPageInitializationErrorEvent, _super);
            function LiveEditPageInitializationErrorEvent(message) {
                _super.call(this);
                this.message = message;
            }
            LiveEditPageInitializationErrorEvent.prototype.getMessage = function () {
                return this.message;
            };
            LiveEditPageInitializationErrorEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            LiveEditPageInitializationErrorEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return LiveEditPageInitializationErrorEvent;
        })(api.event.Event);
        liveedit.LiveEditPageInitializationErrorEvent = LiveEditPageInitializationErrorEvent;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var image;
        (function (image) {
            var ComponentItemType = api.liveedit.ComponentItemType;
            var ImageItemType = (function (_super) {
                __extends(ImageItemType, _super);
                function ImageItemType() {
                    _super.call(this, "image", {
                        cssSelector: '[data-portal-component-type=image]',
                        draggable: true,
                        cursor: 'move',
                        iconCls: 'live-edit-font-icon-image',
                        highlighterStyle: {
                            stroke: 'rgba(68, 68, 68, 1)',
                            strokeDasharray: '5 5',
                            fill: 'rgba(255, 255, 255, 0)'
                        },
                        contextMenuConfig: ['parent', 'remove', 'clear', 'duplicate']
                    });
                }
                ImageItemType.get = function () {
                    return ImageItemType.INSTANCE;
                };
                ImageItemType.prototype.createView = function (config) {
                    return new image.ImageComponentView(new image.ImageComponentViewBuilder().setItemViewProducer(config.itemViewProducer).setParentRegionView(config.parentView).setParentElement(config.parentElement).setElement(config.element).setComponent(config.data).setPositionIndex(config.positionIndex));
                };
                ImageItemType.prototype.isComponentType = function () {
                    return true;
                };
                ImageItemType.INSTANCE = new ImageItemType();
                return ImageItemType;
            })(ComponentItemType);
            image.ImageItemType = ImageItemType;
            ImageItemType.get();
        })(image = liveedit.image || (liveedit.image = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var image;
        (function (_image) {
            var ComponentView = api.liveedit.ComponentView;
            var ImageComponentViewBuilder = (function (_super) {
                __extends(ImageComponentViewBuilder, _super);
                function ImageComponentViewBuilder() {
                    _super.call(this);
                    this.setType(_image.ImageItemType.get());
                }
                return ImageComponentViewBuilder;
            })(liveedit.ComponentViewBuilder);
            _image.ImageComponentViewBuilder = ImageComponentViewBuilder;
            var ImageComponentView = (function (_super) {
                __extends(ImageComponentView, _super);
                function ImageComponentView(builder) {
                    this.liveEditModel = builder.parentRegionView.liveEditModel;
                    this.imageComponent = builder.component;
                    _super.call(this, builder.setPlaceholder(new _image.ImagePlaceholder(this)).setTooltipViewer(new _image.ImageComponentViewer()));
                    this.initializeImage();
                    this.addClass('image-view');
                }
                ImageComponentView.prototype.initializeImage = function () {
                    var _this = this;
                    var figureElChildren = this.getChildren();
                    for (var i = 0; i < figureElChildren.length; i++) {
                        var image = figureElChildren[i];
                        if (image.getHTMLElement().tagName.toUpperCase() == 'IMG') {
                            this.image = image;
                            this.image.getEl().addEventListener("load", function (event) {
                                if (_this.isSelected()) {
                                    _this.highlight();
                                    _this.shade();
                                }
                            });
                        }
                        return;
                    }
                };
                ImageComponentView.prototype.isEmpty = function () {
                    return !this.imageComponent || this.imageComponent.isEmpty();
                };
                return ImageComponentView;
            })(ComponentView);
            _image.ImageComponentView = ImageComponentView;
        })(image = liveedit.image || (liveedit.image = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var image;
        (function (image) {
            var ImageUploadedEvent = api.liveedit.ImageUploadedEvent;
            var ImageOpenUploadDialogEvent = api.liveedit.ImageOpenUploadDialogEvent;
            var ContentTypeName = api.schema.content.ContentTypeName;
            var ImagePlaceholder = (function (_super) {
                __extends(ImagePlaceholder, _super);
                function ImagePlaceholder(imageView) {
                    var _this = this;
                    _super.call(this);
                    this.addClass("image-placeholder");
                    this.imageComponentView = imageView;
                    var imageUploadHandler = function (event) {
                        if (event.getTargetImagePlaceholder() === _this) {
                            var createdImage = event.getUploadedItem();
                            var component = _this.imageComponentView.getComponent();
                            component.setImage(createdImage.getContentId(), createdImage.getDisplayName());
                        }
                    };
                    ImageUploadedEvent.on(imageUploadHandler);
                    this.onRemoved(function (event) { return ImageUploadedEvent.un(imageUploadHandler); });
                    this.comboboxWrapper = new api.dom.DivEl('rich-combobox-wrapper');
                    this.uploadButton = new api.dom.ButtonEl("button upload-button");
                    this.uploadButton.addClass("upload-button");
                    this.uploadButton.onClicked(function (event) {
                        event.preventDefault();
                        event.stopPropagation();
                        new ImageOpenUploadDialogEvent(_this).fire();
                    });
                    this.comboBox = api.content.ContentComboBox.create().setMaximumOccurrences(1).setAllowedContentTypes([ContentTypeName.IMAGE.toString()]).setLoader(new api.content.ContentSummaryLoader()).setMinWidth(270).build();
                    this.comboboxWrapper.appendChildren(this.comboBox, this.uploadButton);
                    this.appendChild(this.comboboxWrapper);
                    this.comboBox.onOptionSelected(function (event) {
                        var component = _this.imageComponentView.getComponent();
                        var imageContent = event.getOption().displayValue;
                        component.setImage(imageContent.getContentId(), imageContent.getDisplayName());
                        _this.uploadButton.hide();
                        _this.imageComponentView.showLoadingSpinner();
                    });
                }
                ImagePlaceholder.prototype.select = function () {
                    this.comboboxWrapper.show();
                    this.comboBox.giveFocus();
                };
                ImagePlaceholder.prototype.deselect = function () {
                    this.comboboxWrapper.hide();
                };
                return ImagePlaceholder;
            })(api.liveedit.ComponentPlaceholder);
            image.ImagePlaceholder = ImagePlaceholder;
        })(image = liveedit.image || (liveedit.image = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var image;
        (function (image) {
            var ImageComponentViewer = (function (_super) {
                __extends(ImageComponentViewer, _super);
                function ImageComponentViewer() {
                    _super.call(this);
                }
                ImageComponentViewer.prototype.resolveDisplayName = function (object) {
                    return !!object.getName() ? object.getName().toString() : "";
                };
                ImageComponentViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getPath().toString();
                };
                ImageComponentViewer.prototype.resolveIconClass = function (object) {
                    return "live-edit-font-icon-image";
                };
                return ImageComponentViewer;
            })(api.ui.NamesAndIconViewer);
            image.ImageComponentViewer = ImageComponentViewer;
        })(image = liveedit.image || (liveedit.image = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var part;
        (function (part) {
            var ComponentItemType = api.liveedit.ComponentItemType;
            var PartItemType = (function (_super) {
                __extends(PartItemType, _super);
                function PartItemType() {
                    _super.call(this, "part", {
                        cssSelector: '[data-portal-component-type=part]',
                        draggable: true,
                        cursor: 'move',
                        iconCls: 'live-edit-font-icon-part',
                        highlighterStyle: {
                            stroke: 'rgba(68, 68, 68, 1)',
                            strokeDasharray: '5 5',
                            fill: 'rgba(255, 255, 255, 0)'
                        },
                        contextMenuConfig: ['parent', 'remove', 'clear', 'duplicate']
                    });
                }
                PartItemType.get = function () {
                    return PartItemType.INSTANCE;
                };
                PartItemType.prototype.isComponentType = function () {
                    return true;
                };
                PartItemType.prototype.createView = function (config) {
                    return new part.PartComponentView(new part.PartComponentViewBuilder().setItemViewProducer(config.itemViewProducer).setParentRegionView(config.parentView).setParentElement(config.parentElement).setComponent(config.data).setElement(config.element).setPositionIndex(config.positionIndex));
                };
                PartItemType.INSTANCE = new PartItemType();
                return PartItemType;
            })(ComponentItemType);
            part.PartItemType = PartItemType;
            PartItemType.get();
        })(part = liveedit.part || (liveedit.part = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var part;
        (function (part) {
            var SiteModel = api.content.site.SiteModel;
            var PartDescriptorLoader = api.content.page.region.PartDescriptorLoader;
            var PartDescriptorComboBox = api.content.page.region.PartDescriptorComboBox;
            var GetPartDescriptorsByApplicationsRequest = api.content.page.region.GetPartDescriptorsByApplicationsRequest;
            var PartPlaceholder = (function (_super) {
                __extends(PartPlaceholder, _super);
                function PartPlaceholder(partView) {
                    var _this = this;
                    _super.call(this);
                    this.addClass("part-placeholder");
                    this.partComponentView = partView;
                    var request = new GetPartDescriptorsByApplicationsRequest(partView.liveEditModel.getSiteModel().getApplicationKeys());
                    var loader = new PartDescriptorLoader(request);
                    loader.setComparator(new api.content.page.DescriptorByDisplayNameComparator());
                    this.comboBox = new PartDescriptorComboBox(loader);
                    loader.load();
                    this.appendChild(this.comboBox);
                    this.comboBox.onOptionSelected(function (event) {
                        _this.partComponentView.showLoadingSpinner();
                        var descriptor = event.getOption().displayValue;
                        var partComponent = _this.partComponentView.getComponent();
                        partComponent.setDescriptor(descriptor.getKey(), descriptor);
                    });
                    partView.liveEditModel.getSiteModel().onPropertyChanged(function (event) {
                        if (event.getPropertyName() == SiteModel.PROPERTY_NAME_SITE_CONFIGS) {
                            request.setApplicationKeys(partView.liveEditModel.getSiteModel().getApplicationKeys());
                            loader.load();
                        }
                    });
                    this.displayName = new api.dom.H3El('display-name');
                    this.appendChild(this.displayName);
                    var partComponent = this.partComponentView.getComponent();
                    if (partComponent && partComponent.getName()) {
                        this.setDisplayName(partComponent.getName().toString());
                    }
                }
                PartPlaceholder.prototype.setDisplayName = function (name) {
                    this.displayName.setHtml(name);
                };
                PartPlaceholder.prototype.select = function () {
                    this.comboBox.show();
                    this.comboBox.giveFocus();
                };
                PartPlaceholder.prototype.deselect = function () {
                    this.comboBox.hide();
                };
                return PartPlaceholder;
            })(liveedit.ComponentPlaceholder);
            part.PartPlaceholder = PartPlaceholder;
        })(part = liveedit.part || (liveedit.part = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var part;
        (function (part) {
            var ComponentView = api.liveedit.ComponentView;
            var ContentView = api.liveedit.ContentView;
            var PartComponentViewBuilder = (function (_super) {
                __extends(PartComponentViewBuilder, _super);
                function PartComponentViewBuilder() {
                    _super.call(this);
                    this.setType(part.PartItemType.get());
                }
                return PartComponentViewBuilder;
            })(liveedit.ComponentViewBuilder);
            part.PartComponentViewBuilder = PartComponentViewBuilder;
            var PartComponentView = (function (_super) {
                __extends(PartComponentView, _super);
                function PartComponentView(builder) {
                    this.contentViews = [];
                    this.liveEditModel = builder.parentRegionView.liveEditModel;
                    this.partComponent = builder.component;
                    this.partPlaceholder = new part.PartPlaceholder(this);
                    _super.call(this, builder.setTooltipViewer(new part.PartComponentViewer()).setPlaceholder(this.partPlaceholder));
                    this.addClass('part-view');
                    this.parseContentViews(this);
                }
                PartComponentView.prototype.setComponent = function (partComponent) {
                    _super.prototype.setComponent.call(this, partComponent);
                    if (partComponent && partComponent.getName()) {
                        this.partPlaceholder.setDisplayName(partComponent.getName().toString());
                    }
                };
                PartComponentView.prototype.addContent = function (view) {
                    this.contentViews.push(view);
                };
                PartComponentView.prototype.getContentViews = function () {
                    return this.contentViews;
                };
                PartComponentView.prototype.isEmpty = function () {
                    return !this.partComponent || this.partComponent.isEmpty();
                };
                PartComponentView.prototype.toItemViewArray = function () {
                    var array = [];
                    array.push(this);
                    this.contentViews.forEach(function (contentView) {
                        array = array.concat(contentView.toItemViewArray());
                    });
                    return array;
                };
                PartComponentView.prototype.parseContentViews = function (parentElement) {
                    var _this = this;
                    var children = parentElement ? parentElement.getChildren() : this.getChildren();
                    children.forEach(function (childElement) {
                        var itemType = liveedit.ItemType.fromElement(childElement);
                        if (itemType) {
                            if (liveedit.ContentItemType.get().equals(itemType)) {
                                var contentView = new ContentView(new liveedit.ContentViewBuilder().setParentPartComponentView(_this).setParentElement(parentElement ? parentElement : _this).setElement(childElement));
                                _this.addContent(contentView);
                            }
                            else {
                                _this.parseContentViews(childElement);
                            }
                        }
                    });
                };
                return PartComponentView;
            })(ComponentView);
            part.PartComponentView = PartComponentView;
        })(part = liveedit.part || (liveedit.part = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var part;
        (function (part) {
            var PartComponentViewer = (function (_super) {
                __extends(PartComponentViewer, _super);
                function PartComponentViewer() {
                    _super.call(this);
                }
                PartComponentViewer.prototype.resolveDisplayName = function (object) {
                    return !!object.getName() ? object.getName().toString() : "";
                };
                PartComponentViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getPath().toString();
                };
                PartComponentViewer.prototype.resolveIconClass = function (object) {
                    return "live-edit-font-icon-part";
                };
                return PartComponentViewer;
            })(api.ui.NamesAndIconViewer);
            part.PartComponentViewer = PartComponentViewer;
        })(part = liveedit.part || (liveedit.part = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var layout;
        (function (layout) {
            var ComponentItemType = api.liveedit.ComponentItemType;
            var LayoutItemType = (function (_super) {
                __extends(LayoutItemType, _super);
                function LayoutItemType() {
                    _super.call(this, "layout", {
                        cssSelector: '[data-portal-component-type=layout]',
                        draggable: true,
                        cursor: 'move',
                        iconCls: 'live-edit-font-icon-layout',
                        highlighterStyle: {
                            stroke: 'rgba(255, 165, 0, 1)',
                            strokeDasharray: '5 5',
                            fill: 'rgba(100, 12, 36, 0)'
                        },
                        contextMenuConfig: ['parent', 'remove', 'clear', 'duplicate']
                    });
                }
                LayoutItemType.get = function () {
                    return LayoutItemType.INSTANCE;
                };
                LayoutItemType.prototype.isComponentType = function () {
                    return true;
                };
                LayoutItemType.prototype.createView = function (config) {
                    return new layout.LayoutComponentView(new layout.LayoutComponentViewBuilder().setItemViewProducer(config.itemViewProducer).setParentRegionView(config.parentView).setParentElement(config.parentElement).setComponent(config.data).setElement(config.element).setPositionIndex(config.positionIndex));
                };
                LayoutItemType.INSTANCE = new LayoutItemType();
                return LayoutItemType;
            })(ComponentItemType);
            layout.LayoutItemType = LayoutItemType;
            LayoutItemType.get();
        })(layout = liveedit.layout || (liveedit.layout = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var layout;
        (function (layout) {
            var ComponentView = api.liveedit.ComponentView;
            var RegionView = api.liveedit.RegionView;
            var LayoutComponentViewBuilder = (function (_super) {
                __extends(LayoutComponentViewBuilder, _super);
                function LayoutComponentViewBuilder() {
                    _super.call(this);
                    this.setType(layout.LayoutItemType.get());
                }
                return LayoutComponentViewBuilder;
            })(liveedit.ComponentViewBuilder);
            layout.LayoutComponentViewBuilder = LayoutComponentViewBuilder;
            var LayoutComponentView = (function (_super) {
                __extends(LayoutComponentView, _super);
                function LayoutComponentView(builder) {
                    var _this = this;
                    this.regionViews = [];
                    this.liveEditModel = builder.parentRegionView.liveEditModel;
                    this.layoutComponent = builder.component;
                    LayoutComponentView.debug = false;
                    this.itemViewAddedListener = function (event) { return _this.notifyItemViewAdded(event.getView(), event.isNew()); };
                    this.itemViewRemovedListener = function (event) { return _this.notifyItemViewRemoved(event.getView()); };
                    _super.call(this, builder.setTooltipViewer(new layout.LayoutComponentViewer()).setPlaceholder(new layout.LayoutPlaceholder(this)));
                    this.addClass('layout-view');
                    this.parseRegions();
                }
                LayoutComponentView.prototype.getRegionViewByName = function (name) {
                    for (var i = 0; i < this.regionViews.length; i++) {
                        var regionView = this.regionViews[i];
                        if (regionView.getRegionName() == name) {
                            return regionView;
                        }
                    }
                    return null;
                };
                LayoutComponentView.prototype.getComponentViewByPath = function (path) {
                    var firstLevelOfPath = path.getFirstLevel();
                    for (var i = 0; i < this.regionViews.length; i++) {
                        var regionView = this.regionViews[i];
                        if (firstLevelOfPath.getRegionName() == regionView.getRegionName()) {
                            if (path.numberOfLevels() == 1) {
                                return regionView.getComponentViewByIndex(firstLevelOfPath.getComponentIndex());
                            }
                            else {
                                var layoutView = regionView.getComponentViewByIndex(firstLevelOfPath.getComponentIndex());
                                return layoutView.getComponentViewByPath(path.removeFirstLevel());
                            }
                        }
                    }
                    return null;
                };
                LayoutComponentView.prototype.setComponent = function (layoutComponent) {
                    _super.prototype.setComponent.call(this, layoutComponent);
                    var regions = layoutComponent.getRegions().getRegions();
                    this.regionViews.forEach(function (regionView, index) {
                        var region = regions[index];
                        regionView.setRegion(region);
                    });
                };
                LayoutComponentView.prototype.getRegions = function () {
                    return this.regionViews;
                };
                LayoutComponentView.prototype.isEmpty = function () {
                    return !this.layoutComponent || this.layoutComponent.isEmpty();
                };
                LayoutComponentView.prototype.toItemViewArray = function () {
                    var array = [];
                    array.push(this);
                    this.regionViews.forEach(function (regionView) {
                        var itemsInRegion = regionView.toItemViewArray();
                        array = array.concat(itemsInRegion);
                    });
                    return array;
                };
                LayoutComponentView.prototype.parseRegions = function () {
                    var _this = this;
                    this.regionViews.forEach(function (regionView) {
                        _this.unregisterRegionView(regionView);
                    });
                    this.regionViews = [];
                    return this.doParseRegions();
                };
                LayoutComponentView.prototype.doParseRegions = function (parentElement) {
                    var _this = this;
                    var layoutComponent = this.getComponent();
                    var layoutRegions = layoutComponent.getRegions();
                    if (!layoutRegions) {
                        return;
                    }
                    var children = parentElement ? parentElement.getChildren() : this.getChildren();
                    children.forEach(function (childElement) {
                        var itemType = liveedit.ItemType.fromElement(childElement);
                        var isRegionView = api.ObjectHelper.iFrameSafeInstanceOf(childElement, RegionView);
                        var region, regionName, regionView;
                        if (isRegionView) {
                            regionName = liveedit.RegionItemType.getRegionName(childElement);
                            region = layoutRegions.getRegionByName(regionName);
                            if (region) {
                                regionView = childElement;
                                regionView.setRegion(region);
                                _this.registerRegionView(regionView);
                            }
                        }
                        else if (itemType && liveedit.RegionItemType.get().equals(itemType)) {
                            regionName = liveedit.RegionItemType.getRegionName(childElement);
                            region = layoutRegions.getRegionByName(regionName);
                            if (region) {
                                regionView = new RegionView(new liveedit.RegionViewBuilder().setParentView(_this).setParentElement(parentElement ? parentElement : _this).setRegion(region).setElement(childElement));
                                _this.registerRegionView(regionView);
                            }
                        }
                        else {
                            _this.doParseRegions(childElement);
                        }
                    });
                };
                LayoutComponentView.prototype.registerRegionView = function (regionView) {
                    if (LayoutComponentView.debug) {
                        console.log('LayoutComponentView.registerRegionView: ' + regionView.toString());
                    }
                    this.regionViews.push(regionView);
                    this.notifyItemViewAdded(regionView);
                    regionView.onItemViewAdded(this.itemViewAddedListener);
                    regionView.onItemViewRemoved(this.itemViewRemovedListener);
                };
                LayoutComponentView.prototype.unregisterRegionView = function (regionView) {
                    if (LayoutComponentView.debug) {
                        console.log('LayoutComponentView.unregisterRegionView: ' + regionView.toString(), this.regionViews);
                    }
                    var index = this.regionViews.indexOf(regionView);
                    if (index > -1) {
                        this.regionViews.splice(index, 1);
                        this.notifyItemViewRemoved(regionView);
                        regionView.unItemViewAdded(this.itemViewAddedListener);
                        regionView.unItemViewRemoved(this.itemViewRemovedListener);
                    }
                };
                return LayoutComponentView;
            })(ComponentView);
            layout.LayoutComponentView = LayoutComponentView;
        })(layout = liveedit.layout || (liveedit.layout = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var layout;
        (function (layout) {
            var SiteModel = api.content.site.SiteModel;
            var GetLayoutDescriptorsByApplicationsRequest = api.content.page.region.GetLayoutDescriptorsByApplicationsRequest;
            var LayoutDescriptorLoader = api.content.page.region.LayoutDescriptorLoader;
            var LayoutDescriptorComboBox = api.content.page.region.LayoutDescriptorComboBox;
            var LayoutPlaceholder = (function (_super) {
                __extends(LayoutPlaceholder, _super);
                function LayoutPlaceholder(layoutView) {
                    var _this = this;
                    _super.call(this);
                    this.addClass("layout-placeholder");
                    this.layoutComponentView = layoutView;
                    var request = new GetLayoutDescriptorsByApplicationsRequest(layoutView.liveEditModel.getSiteModel().getApplicationKeys());
                    var loader = new LayoutDescriptorLoader(request);
                    loader.setComparator(new api.content.page.DescriptorByDisplayNameComparator());
                    this.comboBox = new LayoutDescriptorComboBox(loader);
                    loader.load();
                    this.appendChild(this.comboBox);
                    this.comboBox.onOptionSelected(function (event) {
                        _this.layoutComponentView.showLoadingSpinner();
                        var descriptor = event.getOption().displayValue;
                        var layoutComponent = _this.layoutComponentView.getComponent();
                        layoutComponent.setDescriptor(descriptor.getKey(), descriptor);
                    });
                    layoutView.liveEditModel.getSiteModel().onPropertyChanged(function (event) {
                        if (event.getPropertyName() == SiteModel.PROPERTY_NAME_SITE_CONFIGS) {
                            request.setApplicationKeys(layoutView.liveEditModel.getSiteModel().getApplicationKeys());
                            loader.load();
                        }
                    });
                }
                LayoutPlaceholder.prototype.select = function () {
                    this.comboBox.show();
                    this.comboBox.giveFocus();
                };
                LayoutPlaceholder.prototype.deselect = function () {
                    this.comboBox.hide();
                };
                return LayoutPlaceholder;
            })(liveedit.ComponentPlaceholder);
            layout.LayoutPlaceholder = LayoutPlaceholder;
        })(layout = liveedit.layout || (liveedit.layout = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var layout;
        (function (layout) {
            var LayoutComponentViewer = (function (_super) {
                __extends(LayoutComponentViewer, _super);
                function LayoutComponentViewer() {
                    _super.call(this);
                }
                LayoutComponentViewer.prototype.resolveDisplayName = function (object) {
                    return !!object.getName() ? object.getName().toString() : "";
                };
                LayoutComponentViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getPath().toString();
                };
                LayoutComponentViewer.prototype.resolveIconClass = function (object) {
                    return "live-edit-font-icon-layout";
                };
                return LayoutComponentViewer;
            })(api.ui.NamesAndIconViewer);
            layout.LayoutComponentViewer = LayoutComponentViewer;
        })(layout = liveedit.layout || (liveedit.layout = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var text;
        (function (text) {
            var ComponentItemType = api.liveedit.ComponentItemType;
            var TextItemType = (function (_super) {
                __extends(TextItemType, _super);
                function TextItemType() {
                    _super.call(this, "text", {
                        cssSelector: '[data-portal-component-type=text]',
                        draggable: true,
                        cursor: 'move',
                        iconCls: 'live-edit-font-icon-text',
                        highlighterStyle: {
                            stroke: 'rgba(152, 201, 242, 1)',
                            strokeDasharray: '5 5',
                            fill: 'rgba(255, 255, 255, 0)'
                        },
                        contextMenuConfig: ['parent', 'edit', 'remove', 'clear', 'duplicate']
                    });
                }
                TextItemType.get = function () {
                    return TextItemType.INSTANCE;
                };
                TextItemType.prototype.createView = function (config) {
                    return new text.TextComponentView(new text.TextComponentViewBuilder().setItemViewProducer(config.itemViewProducer).setParentRegionView(config.parentView).setParentElement(config.parentElement).setComponent(config.data).setElement(config.element).setPositionIndex(config.positionIndex));
                };
                TextItemType.prototype.isComponentType = function () {
                    return true;
                };
                TextItemType.INSTANCE = new TextItemType();
                return TextItemType;
            })(ComponentItemType);
            text.TextItemType = TextItemType;
            TextItemType.get();
        })(text = liveedit.text || (liveedit.text = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var text;
        (function (text) {
            var TextPlaceholder = (function (_super) {
                __extends(TextPlaceholder, _super);
                function TextPlaceholder(componentView) {
                    this.textView = componentView;
                    _super.call(this);
                    this.addClass("text-placeholder");
                }
                return TextPlaceholder;
            })(liveedit.ComponentPlaceholder);
            text.TextPlaceholder = TextPlaceholder;
        })(text = liveedit.text || (liveedit.text = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var text;
        (function (text) {
            var MediumHeadersDropdownExtension = (function () {
                function MediumHeadersDropdownExtension() {
                    this.button = new api.dom.ButtonEl('medium-editor-action');
                    this.button.onClicked(this.onClick.bind(this));
                    this.icon = new api.dom.IEl('icon-headers');
                    this.button.appendChild(this.icon);
                }
                MediumHeadersDropdownExtension.prototype.onClick = function (event) {
                };
                MediumHeadersDropdownExtension.prototype.getButton = function () {
                    return this.button.getHTMLElement();
                };
                MediumHeadersDropdownExtension.prototype.beforeCheckState = function () {
                    this.icon.removeClass('icon-headers');
                    this.icon.removeClass('icon-headerNormal');
                    this.icon.removeClass('icon-header1');
                    this.icon.removeClass('icon-header2');
                    this.icon.removeClass('icon-header3');
                    this.icon.removeClass('icon-header4');
                    this.button.removeClass('medium-editor-button-active');
                };
                MediumHeadersDropdownExtension.prototype.checkState = function (node) {
                    if (this.icon.getClass() !== '') {
                        return;
                    }
                    var tagName = node.tagName.toLowerCase();
                    if (tagName === 'p') {
                        this.icon.setClass('icon-headerNormal');
                        this.button.addClass('medium-editor-button-active');
                    }
                    else if (tagName === 'h1') {
                        this.icon.setClass('icon-header1');
                        this.button.addClass('medium-editor-button-active');
                    }
                    else if (tagName === 'h2') {
                        this.icon.setClass('icon-header2');
                        this.button.addClass('medium-editor-button-active');
                    }
                    else if (tagName === 'h3') {
                        this.icon.setClass('icon-header3');
                        this.button.addClass('medium-editor-button-active');
                    }
                    else if (tagName === 'h4') {
                        this.icon.setClass('icon-header4');
                        this.button.addClass('medium-editor-button-active');
                    }
                };
                MediumHeadersDropdownExtension.prototype.afterCheckState = function () {
                    if (this.icon.getClass() === '') {
                        this.icon.setClass('icon-headers');
                    }
                };
                MediumHeadersDropdownExtension.prototype.setEditor = function (editor) {
                    this.editor = editor;
                };
                MediumHeadersDropdownExtension.prototype.onHideToolbar = function () {
                };
                MediumHeadersDropdownExtension.prototype.onShowToolbar = function () {
                    this.initDropdownButtons();
                };
                MediumHeadersDropdownExtension.prototype.initDropdownButtons = function () {
                    var mainButtonParent = this.button.getHTMLElement().parentElement;
                    if (!mainButtonParent) {
                        return;
                    }
                    if (this.dropdownButtons && mainButtonParent.contains(this.dropdownButtons.getHTMLElement())) {
                        return;
                    }
                    this.dropdownButtons = new api.dom.UlEl('medium-editor-toolbar-actions');
                    this.dropdownButtons.addClass('clearfix');
                    var header0Item = this.createHeaderMenuItem('p', 'icon-headerNormal');
                    var header1Item = this.createHeaderMenuItem('h1', 'icon-header1');
                    var header2Item = this.createHeaderMenuItem('h2', 'icon-header2');
                    var header3Item = this.createHeaderMenuItem('h3', 'icon-header3');
                    var header4Item = this.createHeaderMenuItem('h4', 'icon-header4', true);
                    this.dropdownButtons.appendChildren(header0Item, header1Item, header2Item, header3Item, header4Item);
                    this.button.appendChild(this.dropdownButtons);
                    mainButtonParent.appendChild(this.dropdownButtons.getHTMLElement());
                };
                MediumHeadersDropdownExtension.prototype.createHeaderMenuItem = function (elementTag, iconClass, lastButton) {
                    var _this = this;
                    if (lastButton === void 0) { lastButton = false; }
                    var menuItem = new api.dom.LiEl();
                    var button = new api.dom.ButtonEl('medium-editor-action');
                    if (lastButton) {
                        button.addClass('medium-editor-button-bottom');
                    }
                    var icon = new api.dom.IEl(iconClass);
                    button.appendChild(icon);
                    button.onClicked(function (event) {
                        event.preventDefault();
                        event.stopPropagation();
                        _this.editor.execAction('append-' + elementTag, event);
                        _this.icon.setClass(iconClass);
                        _this.button.addClass('medium-editor-button-active');
                    });
                    menuItem.appendChild(button);
                    return menuItem;
                };
                return MediumHeadersDropdownExtension;
            })();
            text.MediumHeadersDropdownExtension = MediumHeadersDropdownExtension;
        })(text = liveedit.text || (liveedit.text = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var text;
        (function (_text) {
            var ComponentView = api.liveedit.ComponentView;
            var TextComponentViewBuilder = (function (_super) {
                __extends(TextComponentViewBuilder, _super);
                function TextComponentViewBuilder() {
                    _super.call(this);
                    this.setType(_text.TextItemType.get());
                }
                return TextComponentViewBuilder;
            })(liveedit.ComponentViewBuilder);
            _text.TextComponentViewBuilder = TextComponentViewBuilder;
            var TextComponentView = (function (_super) {
                __extends(TextComponentView, _super);
                function TextComponentView(builder) {
                    this.lastClicked = 0;
                    this.liveEditModel = builder.parentRegionView.liveEditModel;
                    this.textComponent = builder.component;
                    _super.call(this, builder.setContextMenuActions(this.createTextContextMenuActions()).setPlaceholder(new _text.TextPlaceholder(this)).setTooltipViewer(new _text.TextComponentViewer()));
                    this.addClass('text-view');
                    this.initializeRootElement();
                    this.onKeyDown(this.handleKey.bind(this));
                    this.onKeyUp(this.handleKey.bind(this));
                    this.rootElement.getHTMLElement().onpaste = this.handlePasteEvent.bind(this);
                }
                TextComponentView.prototype.isAllTextSelected = function () {
                    return this.rootElement.getHTMLElement().innerText.trim() == window['getSelection']().toString();
                };
                TextComponentView.prototype.handlePasteEvent = function (event) {
                    if (this.isAllTextSelected()) {
                        this.rootElement.getHTMLElement().innerHTML = "";
                    }
                };
                TextComponentView.prototype.showTooltip = function () {
                    if (!this.isEditMode()) {
                        _super.prototype.showTooltip.call(this);
                    }
                };
                TextComponentView.prototype.hideTooltip = function (hideParentTooltip) {
                    if (hideParentTooltip === void 0) { hideParentTooltip = true; }
                    if (!this.isEditMode()) {
                        _super.prototype.hideTooltip.call(this, hideParentTooltip);
                    }
                };
                TextComponentView.prototype.highlight = function () {
                    var isDragging = liveedit.DragAndDrop.get().isDragging();
                    if (!this.isEditMode() && !isDragging) {
                        _super.prototype.highlight.call(this);
                    }
                };
                TextComponentView.prototype.unhighlight = function () {
                    if (!this.isEditMode()) {
                        _super.prototype.unhighlight.call(this);
                    }
                };
                TextComponentView.prototype.initializeRootElement = function () {
                    for (var i = 0; i < this.getChildren().length; i++) {
                        var child = this.getChildren()[i];
                        if (child.getEl().getTagName().toUpperCase() == 'SECTION') {
                            this.rootElement = child;
                            break;
                        }
                    }
                    if (!this.rootElement) {
                        this.rootElement = new api.dom.SectionEl();
                        this.prependChild(this.rootElement);
                    }
                };
                TextComponentView.prototype.processChanges = function () {
                    var text = this.rootElement.getHtml();
                    if (TextComponentView.debug) {
                        console.log('Processing editor contents: \n', text);
                    }
                    var contentWithoutTags = text.replace(/(<([^>]+)>)/ig, "").trim();
                    this.textComponent.setText(contentWithoutTags.length == 0 ? undefined : text);
                };
                TextComponentView.prototype.isEmpty = function () {
                    return !this.textComponent || this.textComponent.isEmpty();
                };
                TextComponentView.prototype.doHandleDbClick = function (event) {
                    if (this.isEditMode()) {
                        return;
                    }
                    this.startPageTextEditMode();
                    this.selectText();
                };
                TextComponentView.prototype.doHandleClick = function (event) {
                    if (this.isEditMode()) {
                        return;
                    }
                    _super.prototype.handleClick.call(this, event);
                };
                TextComponentView.prototype.handleClick = function (event) {
                    var _this = this;
                    if (TextComponentView.debug) {
                        console.group('Handling click [' + this.getId() + '] at ' + new Date().getTime());
                        console.log(event);
                    }
                    event.stopPropagation();
                    if (this.isEditMode()) {
                        if (TextComponentView.debug) {
                            console.log('Is in text edit mode, not handling click');
                            console.groupEnd();
                        }
                        return;
                    }
                    var timeSinceLastClick = new Date().getTime() - this.lastClicked;
                    if (timeSinceLastClick > TextComponentView.DBL_CLICK_TIMEOUT) {
                        this.singleClickTimer = setTimeout(function () {
                            if (TextComponentView.debug) {
                                console.log('no dblclick occured during ' + TextComponentView.DBL_CLICK_TIMEOUT + 'ms, notifying click', _this);
                                console.groupEnd();
                            }
                            _this.doHandleClick(event);
                        }, TextComponentView.DBL_CLICK_TIMEOUT);
                    }
                    else {
                        if (TextComponentView.debug) {
                            console.log('dblclick occured after ' + timeSinceLastClick + 'ms, notifying dbl click', this);
                            console.groupEnd();
                            console.groupEnd();
                        }
                        clearTimeout(this.singleClickTimer);
                        this.doHandleDbClick(event);
                    }
                    this.lastClicked = new Date().getTime();
                };
                TextComponentView.prototype.handleKey = function () {
                    this.processChanges();
                };
                TextComponentView.prototype.isEditMode = function () {
                    return this.hasClass('edit-mode');
                };
                TextComponentView.prototype.setEditMode = function (flag) {
                    if (!flag && this.editor) {
                        this.deselectText();
                        this.editor.deactivate();
                    }
                    this.toggleClass('edit-mode', flag);
                    this.setDraggable(!flag);
                    if (flag) {
                        this.hideTooltip();
                        if (!this.editor) {
                            this.editor = this.createEditor();
                        }
                        this.editor.activate();
                        if (this.textComponent.isEmpty()) {
                            this.rootElement.setHtml("<h2>Text</h2>");
                        }
                    }
                };
                TextComponentView.prototype.createEditor = function () {
                    var _this = this;
                    var headersExtension = new _text.MediumHeadersDropdownExtension();
                    var editor = new MediumEditor([this.rootElement.getHTMLElement()], {
                        buttons: ['bold', 'italic', 'underline', 'strikethrough', 'justifyLeft', 'justifyCenter', 'justifyRight', 'justifyFull', 'anchor', 'headers', 'orderedlist', 'unorderedlist', 'quote'],
                        buttonLabels: {
                            'bold': '<i class="icon-bold"></i>',
                            'italic': '<i class="icon-italic"></i>',
                            'underline': '<i class="icon-underline"></i>',
                            'strikethrough': '<i class="icon-strikethrough"></i>',
                            'justifyCenter': '<i class="icon-justify-center"></i>',
                            'justifyFull': '<i class="icon-justify-full"></i>',
                            'justifyLeft': '<i class="icon-justify-left"></i>',
                            'justifyRight': '<i class="icon-justify-right"></i>',
                            'anchor': '<i class="icon-anchor"></i>',
                            'header1': '<i class="icon-header1"></i>',
                            'header2': '<i class="icon-header2"></i>',
                            'orderedlist': '<i class="icon-ordered-list"></i>',
                            'unorderedlist': '<i class="icon-unordered-list"></i>',
                            'quote': '<i class="icon-quote"></i>'
                        },
                        cleanPastedHTML: true,
                        targetBlank: true,
                        disablePlaceholders: true,
                        firstHeader: 'h1',
                        secondHeader: 'h2',
                        extensions: {
                            'headers': headersExtension
                        }
                    });
                    var checkActiveButtons = editor.checkActiveButtons.bind(editor);
                    editor.checkActiveButtons = function () {
                        headersExtension.beforeCheckState();
                        checkActiveButtons();
                        headersExtension.afterCheckState();
                    };
                    headersExtension.setEditor(editor);
                    editor.onHideToolbar = function () {
                        _this.processChanges();
                        headersExtension.onHideToolbar();
                    };
                    editor.onShowToolbar = function () {
                        headersExtension.onShowToolbar();
                    };
                    return editor;
                };
                TextComponentView.prototype.selectText = function () {
                    var doc = document;
                    var text = this.rootElement.getHTMLElement();
                    if (window['getSelection']) {
                        var selection = window['getSelection']();
                        var rangeOther = doc.createRange();
                        rangeOther.selectNodeContents(text);
                        selection.removeAllRanges();
                        selection.addRange(rangeOther);
                    }
                    else if (doc.body['createTextRange']) {
                        var rangeIE = doc.body['createTextRange']();
                        rangeIE.moveToElementText(text);
                        rangeIE.select();
                    }
                    text.click();
                };
                TextComponentView.prototype.deselectText = function () {
                    if (window['getSelection']) {
                        window['getSelection']().removeAllRanges();
                    }
                    else if (document.selection) {
                        document.selection.empty();
                    }
                };
                TextComponentView.prototype.startPageTextEditMode = function () {
                    this.deselect();
                    var pageView = this.getPageView();
                    if (!pageView.isTextEditMode()) {
                        pageView.setTextEditMode(true);
                    }
                    this.giveFocus();
                };
                TextComponentView.prototype.giveFocus = function () {
                    if (!this.isEditMode()) {
                        return false;
                    }
                    return this.rootElement.giveFocus();
                };
                TextComponentView.prototype.createTextContextMenuActions = function () {
                    var _this = this;
                    var actions = [];
                    actions.push(new api.ui.Action('Edit').onExecuted(function () {
                        _this.startPageTextEditMode();
                    }));
                    return actions;
                };
                TextComponentView.debug = false;
                TextComponentView.DBL_CLICK_TIMEOUT = 250;
                return TextComponentView;
            })(ComponentView);
            _text.TextComponentView = TextComponentView;
        })(text = liveedit.text || (liveedit.text = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var text;
        (function (text) {
            var TextComponentViewer = (function (_super) {
                __extends(TextComponentViewer, _super);
                function TextComponentViewer() {
                    _super.call(this);
                }
                TextComponentViewer.prototype.resolveDisplayName = function (object) {
                    return object.getText();
                };
                TextComponentViewer.prototype.resolveSubName = function (object, relativePath) {
                    if (relativePath === void 0) { relativePath = false; }
                    return object.getPath().toString();
                };
                TextComponentViewer.prototype.resolveIconClass = function (object) {
                    return "live-edit-font-icon-text";
                };
                return TextComponentViewer;
            })(api.ui.NamesAndIconViewer);
            text.TextComponentViewer = TextComponentViewer;
        })(text = liveedit.text || (liveedit.text = {}));
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
var api;
(function (api) {
    var liveedit;
    (function (liveedit) {
        var LayoutItemType = api.liveedit.layout.LayoutItemType;
        var PartItemType = api.liveedit.part.PartItemType;
        var ImageItemType = api.liveedit.image.ImageItemType;
        var TextItemType = api.liveedit.text.TextItemType;
        var ItemViewIconClassResolver = (function () {
            function ItemViewIconClassResolver() {
            }
            ItemViewIconClassResolver.resolve = function (itemView) {
                if (!itemView) {
                    return '';
                }
                var iconClass = '', itemType = itemView.getType();
                if (liveedit.RegionItemType.get().equals(itemType)) {
                    iconClass = 'live-edit-font-icon-region';
                }
                else if (LayoutItemType.get().equals(itemType)) {
                    iconClass = 'live-edit-font-icon-layout';
                }
                else if (PartItemType.get().equals(itemType)) {
                    iconClass = 'live-edit-font-icon-part';
                }
                else if (ImageItemType.get().equals(itemType)) {
                    iconClass = 'live-edit-font-icon-image';
                }
                else if (TextItemType.get().equals(itemType)) {
                    iconClass = 'live-edit-font-icon-text';
                }
                return iconClass;
            };
            return ItemViewIconClassResolver;
        })();
        liveedit.ItemViewIconClassResolver = ItemViewIconClassResolver;
    })(liveedit = api.liveedit || (api.liveedit = {}));
})(api || (api = {}));
wemQ.longStackSupport = true;
var app;
(function (app) {
    var Router = (function () {
        function Router() {
        }
        Router.setHash = function (path) {
            if (window.parent["setHash"]) {
                window.parent["setHash"](api.app.Application.getAppId() + "/" + path);
            }
        };
        return Router;
    })();
    app.Router = Router;
})(app || (app = {}));
var app;
(function (app) {
    var ContentIconUrlResolver = api.content.ContentIconUrlResolver;
    var ContentId = api.content.ContentId;
    var ContentUpdatedEvent = api.content.ContentUpdatedEvent;
    var AppBarTabId = api.app.bar.AppBarTabId;
    var AppBarTabMenuItemBuilder = api.app.bar.AppBarTabMenuItemBuilder;
    var ShowBrowsePanelEvent = api.app.ShowBrowsePanelEvent;
    var ContentAppPanel = (function (_super) {
        __extends(ContentAppPanel, _super);
        function ContentAppPanel(appBar, path) {
            _super.call(this, {
                appBar: appBar
            });
            this.mask = new api.ui.mask.LoadMask(this);
            this.handleGlobalEvents();
            this.route(path);
        }
        ContentAppPanel.prototype.addWizardPanel = function (tabMenuItem, wizardPanel) {
            var _this = this;
            _super.prototype.addWizardPanel.call(this, tabMenuItem, wizardPanel);
            wizardPanel.getHeader().onPropertyChanged(function (event) {
                if (event.getPropertyName() === "displayName") {
                    var contentType = wizardPanel.getContentType(), name = event.getNewValue() || "<Unnamed " + _this.convertName(contentType.getDisplayName()) + ">";
                    tabMenuItem.setLabel(name, !event.getNewValue(), false);
                }
            });
            var contentWizardPanel = wizardPanel;
            tabMenuItem.markInvalid(!contentWizardPanel.getPersistedItem().isValid());
            contentWizardPanel.onValidityChanged(function (event) {
                tabMenuItem.markInvalid(!contentWizardPanel.isValid());
            });
        };
        ContentAppPanel.prototype.route = function (path) {
            var action = path ? path.getElement(0) : undefined;
            switch (action) {
                case 'edit':
                    var id = path.getElement(1);
                    if (id) {
                        new api.content.GetContentByIdRequest(new ContentId(id)).sendAndParse().done(function (content) {
                            new api.content.EditContentEvent([content]).fire();
                        });
                    }
                    break;
                case 'view':
                    var id = path.getElement(1);
                    if (id) {
                        new api.content.GetContentByIdRequest(new ContentId(id)).sendAndParse().done(function (content) {
                            new app.browse.ViewContentEvent([content]).fire();
                        });
                    }
                    break;
                default:
                    new ShowBrowsePanelEvent().fire();
                    break;
            }
        };
        ContentAppPanel.prototype.handleGlobalEvents = function () {
            var _this = this;
            app.create.NewContentEvent.on(function (event) {
                _this.handleNewContent(event);
            });
            app.browse.ViewContentEvent.on(function (event) {
                _this.handleView(event);
            });
            api.content.EditContentEvent.on(function (event) {
                _this.handleEdit(event);
            });
            ShowBrowsePanelEvent.on(function (event) {
                _this.handleBrowse(event);
            });
            ContentUpdatedEvent.on(function (event) {
                _this.handleUpdated(event);
            });
            app.browse.SortContentEvent.on(function (event) {
                _this.handleSort(event);
            });
            app.browse.MoveContentEvent.on(function (event) {
                _this.handleMove(event);
            });
            api.content.ContentDeletedEvent.on(function (event) {
                var item = _this.getNavigator().getNavigationItemByIdValue(event.getContentId().toString());
                if (item) {
                    item.getCloseAction().execute(true);
                }
            });
        };
        ContentAppPanel.prototype.handleUpdated = function (event) {
        };
        ContentAppPanel.prototype.handleBrowse = function (event) {
            var browsePanel = this.getBrowsePanel();
            if (!browsePanel) {
                this.addBrowsePanel(new app.browse.ContentBrowsePanel());
            }
            else {
                this.selectPanelByIndex(this.getPanelIndex(browsePanel));
            }
        };
        ContentAppPanel.prototype.handleNewContent = function (newContentEvent) {
            var _this = this;
            var contentTypeSummary = newContentEvent.getContentType();
            var tabId = AppBarTabId.forNew(contentTypeSummary.getName());
            var tabMenuItem = this.getAppBarTabMenu().getNavigationItemById(tabId);
            if (tabMenuItem != null) {
                this.selectPanel(tabMenuItem);
            }
            else {
                this.mask.show();
                var contentWizardPanelFactory = new app.wizard.ContentWizardPanelFactory().setAppBarTabId(tabId).setParentContent(newContentEvent.getParentContent()).setContentTypeName(contentTypeSummary.getContentTypeName());
                contentWizardPanelFactory.setCreateSite(newContentEvent.getContentType().isSite());
                contentWizardPanelFactory.createForNew().then(function (wizard) {
                    if (!wizard.getPersistedItem()) {
                        wizard.close();
                        return;
                    }
                    tabMenuItem = new AppBarTabMenuItemBuilder().setLabel("<Unnamed " + _this.convertName(contentTypeSummary.getDisplayName()) + ">").setTabId(tabId).setCloseAction(wizard.getCloseAction()).build();
                    wizard.onContentNamed(function (event) {
                        _this.handleContentNamedEvent(event);
                    });
                    _this.addWizardPanel(tabMenuItem, wizard);
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                }).finally(function () {
                    _this.mask.hide();
                }).done();
            }
        };
        ContentAppPanel.prototype.handleEdit = function (event) {
            var _this = this;
            var contents = event.getModels();
            contents.forEach(function (content) {
                if (!content) {
                    return;
                }
                var closeViewPanelMenuItem = _this.resolveTabMenuItemForContentBeingViewed(content);
                var tabMenuItem = _this.resolveTabMenuItemForContentBeingEdited(content);
                if (tabMenuItem != null) {
                    _this.selectPanel(tabMenuItem);
                }
                else {
                    _this.mask.show();
                    var tabId = AppBarTabId.forEdit(content.getId());
                    new app.wizard.ContentWizardPanelFactory().setAppBarTabId(tabId).setContentIdToEdit(content.getContentId()).createForEdit(content).then(function (wizard) {
                        if (closeViewPanelMenuItem != null) {
                            _this.getAppBarTabMenu().deselectNavigationItem();
                            _this.getAppBarTabMenu().removeNavigationItem(closeViewPanelMenuItem);
                            _this.removePanelByIndex(closeViewPanelMenuItem.getIndex());
                        }
                        var contentType = wizard.getContentType(), name = content.getDisplayName() || "<Unnamed " + _this.convertName(contentType.getDisplayName()) + ">";
                        tabMenuItem = new AppBarTabMenuItemBuilder().setLabel(name).setMarkUnnamed(!content.getDisplayName()).setMarkInvalid(!content.isValid()).setTabId(tabId).setEditing(true).setCloseAction(wizard.getCloseAction()).build();
                        _this.addWizardPanel(tabMenuItem, wizard);
                        var viewTabId = AppBarTabId.forView(content.getId());
                        var viewTabMenuItem = _this.getAppBarTabMenu().getNavigationItemById(viewTabId);
                        if (viewTabMenuItem != null) {
                            _this.removePanelByIndex(viewTabMenuItem.getIndex());
                        }
                    }).catch(function (reason) {
                        if (tabId) {
                            var tabMenuItem = _this.getAppBarTabMenu().getNavigationItemById(tabId);
                            if (tabMenuItem) {
                                _this.removePanelByIndex(tabMenuItem.getIndex());
                            }
                        }
                        api.DefaultErrorHandler.handle(reason);
                    }).finally(function () {
                        _this.mask.hide();
                    }).done();
                }
            });
        };
        ContentAppPanel.prototype.handleView = function (event) {
            var _this = this;
            var contents = event.getModels();
            contents.forEach(function (content) {
                if (!content) {
                    return;
                }
                var tabMenuItem = _this.resolveTabMenuItemForContentBeingEditedOrViewed(content);
                if (tabMenuItem) {
                    _this.selectPanel(tabMenuItem);
                }
                else {
                    var tabId = AppBarTabId.forView(content.getId());
                    var contentItemViewPanel = new app.view.ContentItemViewPanel();
                    tabMenuItem = new AppBarTabMenuItemBuilder().setLabel(content.getDisplayName()).setMarkInvalid(!content.isValid()).setTabId(tabId).setCloseAction(contentItemViewPanel.getCloseAction()).build();
                    if (!content.getDisplayName()) {
                        new api.schema.content.GetContentTypeByNameRequest(content.getType()).sendAndParse().then(function (contentType) {
                            tabMenuItem.setLabel("<Unnamed " + _this.convertName(contentType.getDisplayName()) + ">", true);
                        }).done();
                    }
                    var contentItem = new api.app.view.ViewItem(content).setDisplayName(content.getDisplayName()).setPath(content.getPath().toString()).setIconUrl(new ContentIconUrlResolver().setContent(content).resolve());
                    contentItemViewPanel.setItem(contentItem);
                    _this.addViewPanel(tabMenuItem, contentItemViewPanel);
                }
            });
        };
        ContentAppPanel.prototype.handleSort = function (event) {
            var contents = event.getModels();
            new app.browse.OpenSortDialogEvent(contents[0]).fire();
        };
        ContentAppPanel.prototype.handleMove = function (event) {
            var contents = event.getModels();
            new app.browse.OpenMoveDialogEvent(contents).fire();
        };
        ContentAppPanel.prototype.handleContentNamedEvent = function (event) {
            var wizard = event.getWizard(), tabMenuItem = this.getAppBarTabMenu().getNavigationItemById(wizard.getTabId());
            var newTabId = AppBarTabId.forEdit(event.getContent().getId());
            tabMenuItem.setTabId(newTabId);
            wizard.setTabId(newTabId);
        };
        ContentAppPanel.prototype.resolveTabMenuItemForContentBeingEditedOrViewed = function (content) {
            var result = this.resolveTabMenuItemForContentBeingEdited(content);
            if (!result) {
                result = this.resolveTabMenuItemForContentBeingViewed(content);
            }
            return result;
        };
        ContentAppPanel.prototype.resolveTabMenuItemForContentBeingEdited = function (content) {
            if (!!content) {
                var tabId = this.getAppBarTabMenu().getNavigationItemById(AppBarTabId.forEdit(content.getId()));
                if (tabId) {
                    return tabId;
                }
            }
            return null;
        };
        ContentAppPanel.prototype.resolveTabMenuItemForContentBeingViewed = function (content) {
            if (!!content) {
                var tabId = this.getAppBarTabMenu().getNavigationItemById(AppBarTabId.forView(content.getId()));
                if (tabId) {
                    return tabId;
                }
            }
            return null;
        };
        ContentAppPanel.prototype.convertName = function (name) {
            return api.util.StringHelper.capitalizeAll(name.replace(/-/g, " ").trim());
        };
        return ContentAppPanel;
    })(api.app.BrowseAndWizardBasedAppPanel);
    app.ContentAppPanel = ContentAppPanel;
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (_view) {
        var detail;
        (function (detail) {
            var ResponsiveManager = api.ui.responsive.ResponsiveManager;
            var ResponsiveRanges = api.ui.responsive.ResponsiveRanges;
            var Dropdown = api.ui.selector.dropdown.Dropdown;
            var DetailsPanel = (function (_super) {
                __extends(DetailsPanel, _super);
                function DetailsPanel(builder) {
                    var _this = this;
                    _super.call(this, "details-panel");
                    this.widgetViews = [];
                    this.detailsContainer = new api.dom.DivEl("details-container");
                    this.minWidth = 280;
                    this.parentMinWidth = 15;
                    this.sizeChangedListeners = [];
                    this.contentStatusChangedListeners = [];
                    this.setDoOffset(false);
                    this.initSlideFunctions(builder.getSlideFrom());
                    this.useSplitter = builder.getUseSplitter();
                    this.versionsPanel = new _view.ContentItemVersionsPanel();
                    this.ghostDragger = new api.dom.DivEl("ghost-dragger");
                    if (this.useSplitter) {
                        this.splitter = new api.dom.DivEl("splitter");
                        this.appendChild(this.splitter);
                        this.onRendered(function () { return _this.onRenderedHandler(); });
                    }
                    this.onShown(function (event) {
                        if (_this.item) {
                            _this.resetItem();
                        }
                    });
                    this.onPanelSizeChanged(function () {
                        _this.versionsPanel.reRenderActivePanel();
                    });
                    var delayedReset = api.util.AppHelper.debounce(this.resetItem.bind(this), 300, false);
                    ResponsiveManager.onAvailableSizeChanged(this, delayedReset);
                    api.content.ContentsPublishedEvent.on(function (event) {
                        if (_this.getItem()) {
                            var itemId = _this.getItem().getModel().getId();
                            var idPublished = event.getContentIds().some(function (id, index, array) {
                                return itemId === id.toString();
                            });
                            if (idPublished) {
                                _this.versionsPanel.reloadActivePanel();
                            }
                        }
                    });
                    this.initNameAndIconView(builder.getUseNameAndIconView());
                    this.initDefaultWidget();
                    this.initCommonWidgetsViews();
                    this.initDivForNoSelection();
                    this.initWidgetsSelectionRow();
                    this.getAndInitCustomWidgetsViews().done(function () {
                        _this.initWidgetsDropdownForSelectedItem();
                    });
                    this.appendChild(this.detailsContainer);
                    this.appendChild(this.divForNoSelection);
                    this.layout();
                }
                DetailsPanel.prototype.initDivForNoSelection = function () {
                    this.divForNoSelection = new api.dom.DivEl("no-selection-message");
                    this.divForNoSelection.getEl().setInnerHtml("Select an item - and we'll show you the details!");
                    this.appendChild(this.divForNoSelection);
                };
                DetailsPanel.prototype.initWidgetsSelectionRow = function () {
                    this.widgetsSelectionRow = new WidgetsSelectionRow(this);
                    this.appendChild(this.widgetsSelectionRow);
                };
                DetailsPanel.prototype.setActiveWidget = function (widgetView) {
                    if (this.activeWidget) {
                        this.activeWidget.deactivate();
                    }
                    if (!this.isDefaultWidget(this.activeWidget)) {
                        this.previousActiveWidget = this.activeWidget;
                    }
                    this.activeWidget = widgetView;
                    this.widgetsSelectionRow.updateState(this.activeWidget);
                };
                DetailsPanel.prototype.resetActiveWidget = function () {
                    this.activeWidget = null;
                };
                DetailsPanel.prototype.activateDefaultWidget = function () {
                    var defaultWidget = this.getDefaultWidget();
                    if (defaultWidget) {
                        defaultWidget.setActive();
                    }
                };
                DetailsPanel.prototype.activatePreviousWidget = function () {
                    if (this.previousActiveWidget) {
                        this.previousActiveWidget.setActive();
                    }
                };
                DetailsPanel.prototype.isDefaultWidget = function (widgetView) {
                    return widgetView == this.defaultWidgetView;
                };
                DetailsPanel.prototype.getDefaultWidget = function () {
                    return this.defaultWidgetView;
                };
                DetailsPanel.prototype.initSlideFunctions = function (slideFrom) {
                    switch (slideFrom) {
                        case 1 /* RIGHT */:
                            this.slideInFunction = this.slideInRight;
                            this.slideOutFunction = this.slideOutRight;
                            break;
                        case 0 /* LEFT */:
                            this.slideInFunction = this.slideInLeft;
                            this.slideOutFunction = this.slideOutLeft;
                            break;
                        case 3 /* TOP */:
                            this.slideInFunction = this.slideInTop;
                            this.slideOutFunction = this.slideOutTop;
                            break;
                        case 2 /* BOTTOM */:
                            this.slideInFunction = this.slideInBottom;
                            this.slideOutFunction = this.slideOutBottom;
                            break;
                        default:
                            this.slideInFunction = this.slideInRight;
                            this.slideOutFunction = this.slideOutRight;
                    }
                };
                DetailsPanel.prototype.initNameAndIconView = function (useNameAndIconView) {
                    this.useNameAndIconView = useNameAndIconView;
                    if (useNameAndIconView) {
                        this.nameAndIconView = new api.app.NamesAndIconView(new api.app.NamesAndIconViewBuilder().setSize(2 /* small */));
                        this.nameAndIconView.addClass("details-panel-label");
                        this.appendChild(this.nameAndIconView);
                    }
                };
                DetailsPanel.prototype.setItem = function (item) {
                    if (!this.item || !this.item.equals(item)) {
                        this.item = item;
                        if (item) {
                            this.layout(false);
                            this.updateWidgetsForItem();
                        }
                        else {
                            this.layout();
                        }
                    }
                };
                DetailsPanel.prototype.setContentStatus = function (status) {
                    this.contentStatus = status;
                    this.notifyContentStatusChanged();
                };
                DetailsPanel.prototype.getWidgetsInterfaceName = function () {
                    return "com.enonic.xp.content-manager.context-widget";
                };
                DetailsPanel.prototype.updateWidgetsForItem = function () {
                    var _this = this;
                    this.updateNameAndIconView();
                    this.updateCommonWidgets();
                    this.updateCustomWidgets();
                    setTimeout(function () {
                        _this.updateWidgetsHeights();
                    }, 400);
                };
                DetailsPanel.prototype.updateWidgetsHeights = function () {
                    var _this = this;
                    this.widgetViews.forEach(function (widgetView) {
                        if (widgetView != _this.activeWidget) {
                            widgetView.updateNormalHeightSilently();
                        }
                        else {
                            widgetView.updateNormalHeight();
                        }
                    });
                    if (this.defaultWidgetView) {
                        if (this.defaultWidgetView != this.activeWidget) {
                            this.defaultWidgetView.updateNormalHeightSilently();
                        }
                        else {
                            this.defaultWidgetView.updateNormalHeight();
                        }
                    }
                };
                DetailsPanel.prototype.updateCommonWidgets = function () {
                    this.setDefaultWidget();
                    this.versionsPanel.setItem(this.item);
                };
                DetailsPanel.prototype.updateCustomWidgets = function () {
                };
                DetailsPanel.prototype.initWidgetsDropdownForSelectedItem = function () {
                    this.widgetsSelectionRow.updateWidgetsDropdown(this.widgetViews);
                    this.activateDefaultWidget();
                };
                DetailsPanel.prototype.setStatus = function (statusWidgetItemView) {
                    statusWidgetItemView.setStatus(this.contentStatus);
                    this.versionsPanel.setStatus(this.contentStatus);
                    this.versionsPanel.reRenderActivePanel();
                };
                DetailsPanel.prototype.setDefaultWidget = function () {
                    var _this = this;
                    var widgetItemView = new detail.StatusWidgetItemView();
                    var propWidgetItemView = new detail.PropertiesWidgetItemView();
                    if (this.item) {
                        api.content.ContentSummaryAndCompareStatusFetcher.fetch(this.item.getModel().getContentId()).then(function (contentSummaryAndCompareStatus) {
                            _this.contentStatus = contentSummaryAndCompareStatus.getCompareStatus();
                            if (_this.defaultWidgetView) {
                                _this.detailsContainer.removeChild(_this.defaultWidgetView);
                            }
                            _this.setStatus(widgetItemView);
                            _this.onContentStatusChanged(function () {
                                _this.setStatus(widgetItemView);
                                widgetItemView.layout();
                            });
                            propWidgetItemView.setContent(_this.item.getModel());
                            _this.defaultWidgetView = detail.WidgetView.create().setName(DetailsPanel.DEFAULT_WIDGET_NAME).setDetailsPanel(_this).setUseToggleButton(false).addWidgetItemView(widgetItemView).addWidgetItemView(propWidgetItemView).build();
                            _this.detailsContainer.appendChild(_this.defaultWidgetView);
                            if (DetailsPanel.DEFAULT_WIDGET_NAME == _this.activeWidget.getWidgetName()) {
                                _this.setActiveWidget(_this.defaultWidgetView);
                            }
                            _this.updateWidgetsHeights();
                        }).done();
                    }
                };
                DetailsPanel.prototype.initDefaultWidget = function () {
                    this.defaultWidgetView = detail.WidgetView.create().setName(DetailsPanel.DEFAULT_WIDGET_NAME).setDetailsPanel(this).setUseToggleButton(false).build();
                    this.detailsContainer.appendChild(this.defaultWidgetView);
                };
                DetailsPanel.prototype.initCommonWidgetsViews = function () {
                    this.versionWidgetItemView = new detail.WidgetItemView("version-history");
                    this.versionWidgetItemView.setItem(this.versionsPanel);
                    var versionsWidgetView = detail.WidgetView.create().setName("Version history").setDetailsPanel(this).setUseToggleButton(false).addWidgetItemView(this.versionWidgetItemView).build();
                    this.addWidgets([versionsWidgetView]);
                };
                DetailsPanel.prototype.getAndInitCustomWidgetsViews = function () {
                    var _this = this;
                    var getWidgetsByInterfaceRequest = new api.content.GetWidgetsByInterfaceRequest(this.getWidgetsInterfaceName());
                    return getWidgetsByInterfaceRequest.sendAndParse().then(function (widgets) {
                        widgets.forEach(function (widget) {
                            var widgetView = detail.WidgetView.create().setName(widget.getDisplayName()).setDetailsPanel(_this).setUseToggleButton(false).build();
                            _this.addWidget(widgetView);
                        });
                    }).catch(function (reason) {
                        if (reason && reason.message) {
                        }
                        else {
                        }
                    });
                };
                DetailsPanel.prototype.onRenderedHandler = function () {
                    var _this = this;
                    var initialPos = 0;
                    var splitterPosition = 0;
                    this.actualWidth = this.getEl().getWidth();
                    this.mask = new api.ui.mask.DragMask(this.getParentElement());
                    var dragListener = function (e) {
                        if (_this.splitterWithinBoundaries(initialPos - e.clientX)) {
                            splitterPosition = e.clientX;
                            _this.ghostDragger.getEl().setLeftPx(e.clientX - _this.getEl().getOffsetLeft());
                        }
                    };
                    this.splitter.onMouseDown(function (e) {
                        e.preventDefault();
                        initialPos = e.clientX;
                        splitterPosition = e.clientX;
                        _this.ghostDragger.insertBeforeEl(_this.splitter);
                        _this.startDrag(dragListener);
                    });
                    this.mask.onMouseUp(function (e) {
                        if (_this.ghostDragger.getHTMLElement().parentNode) {
                            _this.actualWidth = _this.getEl().getWidth() + initialPos - splitterPosition;
                            _this.stopDrag(dragListener);
                            _this.removeChild(_this.ghostDragger);
                            ResponsiveManager.fireResizeEvent();
                        }
                    });
                };
                DetailsPanel.prototype.splitterWithinBoundaries = function (offset) {
                    var newWidth = this.actualWidth + offset;
                    return (newWidth >= this.minWidth) && (newWidth <= this.getParentElement().getEl().getWidth() - this.parentMinWidth);
                };
                DetailsPanel.prototype.startDrag = function (dragListener) {
                    this.mask.show();
                    this.addClass("dragging");
                    this.mask.onMouseMove(dragListener);
                    this.ghostDragger.getEl().setLeftPx(this.splitter.getEl().getOffsetLeftRelativeToParent()).setTop(null);
                };
                DetailsPanel.prototype.stopDrag = function (dragListener) {
                    var _this = this;
                    this.getEl().setWidthPx(this.actualWidth);
                    this.removeClass("dragging");
                    setTimeout(function () {
                        _this.notifyPanelSizeChanged();
                        _this.widgetsSelectionRow.render();
                    }, 800);
                    this.mask.hide();
                    this.mask.unMouseMove(dragListener);
                };
                DetailsPanel.prototype.addWidget = function (widget) {
                    this.widgetViews.push(widget);
                    this.detailsContainer.appendChild(widget);
                };
                DetailsPanel.prototype.addWidgets = function (widgetViews) {
                    var _this = this;
                    widgetViews.forEach(function (widget) {
                        _this.addWidget(widget);
                    });
                };
                DetailsPanel.prototype.setWidthPx = function (value) {
                    this.getEl().setWidthPx(value);
                    this.actualWidth = value;
                };
                DetailsPanel.prototype.getActualWidth = function () {
                    return this.actualWidth;
                };
                DetailsPanel.prototype.getItem = function () {
                    return this.item;
                };
                DetailsPanel.prototype.resetItem = function () {
                    if (this.item) {
                        var temp = this.item;
                        this.item = null;
                        this.setItem(temp);
                    }
                };
                DetailsPanel.prototype.updateNameAndIconView = function () {
                    if (this.useNameAndIconView && this.item) {
                        this.nameAndIconView.setMainName(this.item.getDisplayName());
                        this.nameAndIconView.setSubName(this.item.getPath());
                        this.nameAndIconView.setIconUrl(this.item.getIconUrl());
                    }
                };
                DetailsPanel.prototype.slideIn = function () {
                    this.slideInFunction();
                };
                DetailsPanel.prototype.slideOut = function () {
                    this.slideOutFunction();
                };
                DetailsPanel.prototype.layout = function (empty) {
                    if (empty === void 0) { empty = true; }
                    if (this.widgetsSelectionRow) {
                        this.widgetsSelectionRow.setVisible(!empty);
                    }
                    if (this.nameAndIconView) {
                        this.nameAndIconView.setVisible(!empty);
                    }
                    this.detailsContainer.setVisible(!empty);
                    this.toggleClass("no-selection", empty);
                };
                DetailsPanel.prototype.slideInRight = function () {
                    this.getEl().setRightPx(0);
                };
                DetailsPanel.prototype.slideOutRight = function () {
                    this.getEl().setRightPx(-this.getEl().getWidthWithBorder());
                };
                DetailsPanel.prototype.slideInLeft = function () {
                    this.getEl().setLeftPx(0);
                };
                DetailsPanel.prototype.slideOutLeft = function () {
                    this.getEl().setLeftPx(-this.getEl().getWidthWithBorder());
                };
                DetailsPanel.prototype.slideInTop = function () {
                    this.getEl().setTopPx(36);
                };
                DetailsPanel.prototype.slideOutTop = function () {
                    this.getEl().setTopPx(-window.outerHeight);
                };
                DetailsPanel.prototype.slideInBottom = function () {
                    this.getEl().setTopPx(36);
                };
                DetailsPanel.prototype.slideOutBottom = function () {
                    this.getEl().setTopPx(window.outerHeight);
                };
                DetailsPanel.prototype.notifyPanelSizeChanged = function () {
                    this.sizeChangedListeners.forEach(function (listener) { return listener(); });
                };
                DetailsPanel.prototype.onPanelSizeChanged = function (listener) {
                    this.sizeChangedListeners.push(listener);
                };
                DetailsPanel.prototype.unPanelSizeChanged = function (listener) {
                    this.sizeChangedListeners.filter(function (currentListener) {
                        return listener == currentListener;
                    });
                };
                DetailsPanel.prototype.notifyContentStatusChanged = function () {
                    this.contentStatusChangedListeners.forEach(function (listener) { return listener(); });
                };
                DetailsPanel.prototype.onContentStatusChanged = function (listener) {
                    this.contentStatusChangedListeners.push(listener);
                };
                DetailsPanel.prototype.unContentStatusChanged = function (listener) {
                    this.contentStatusChangedListeners.filter(function (currentListener) {
                        return listener == currentListener;
                    });
                };
                DetailsPanel.create = function () {
                    return new Builder();
                };
                DetailsPanel.DEFAULT_WIDGET_NAME = "Info";
                return DetailsPanel;
            })(api.ui.panel.Panel);
            detail.DetailsPanel = DetailsPanel;
            var Builder = (function () {
                function Builder() {
                    this.useNameAndIconView = true;
                    this.slideFrom = 1 /* RIGHT */;
                    this.useSplitter = true;
                }
                Builder.prototype.setUseNameAndIconView = function (value) {
                    this.useNameAndIconView = value;
                    return this;
                };
                Builder.prototype.setSlideFrom = function (value) {
                    this.slideFrom = value;
                    return this;
                };
                Builder.prototype.setName = function (value) {
                    this.name = value;
                    return this;
                };
                Builder.prototype.setUseSplitter = function (value) {
                    this.useSplitter = value;
                    return this;
                };
                Builder.prototype.getUseNameAndIconView = function () {
                    return this.useNameAndIconView;
                };
                Builder.prototype.getSlideFrom = function () {
                    return this.slideFrom;
                };
                Builder.prototype.getName = function () {
                    return this.name;
                };
                Builder.prototype.getUseSplitter = function () {
                    return this.useSplitter;
                };
                Builder.prototype.build = function () {
                    return new DetailsPanel(this);
                };
                return Builder;
            })();
            detail.Builder = Builder;
            var MobileDetailsPanelToggleButton = (function (_super) {
                __extends(MobileDetailsPanelToggleButton, _super);
                function MobileDetailsPanelToggleButton(detailsPanel) {
                    var _this = this;
                    _super.call(this, "mobile-details-panel-toggle-button");
                    this.detailsPanel = detailsPanel;
                    this.onClicked(function (event) {
                        _this.toggleClass("expanded");
                        if (_this.hasClass("expanded")) {
                            _this.detailsPanel.slideIn();
                        }
                        else {
                            _this.detailsPanel.slideOut();
                        }
                    });
                }
                return MobileDetailsPanelToggleButton;
            })(api.dom.DivEl);
            detail.MobileDetailsPanelToggleButton = MobileDetailsPanelToggleButton;
            var NonMobileDetailsPanelsToggleButton = (function (_super) {
                __extends(NonMobileDetailsPanelsToggleButton, _super);
                function NonMobileDetailsPanelsToggleButton(builder) {
                    var _this = this;
                    _super.call(this, "button non-mobile-details-panel-toggle-button");
                    this.resizeEventMonitorLocked = false;
                    this.splitPanelWithGridAndDetails = builder.getSplitPanelWithGridAndDetails();
                    this.defaultDockedDetailsPanel = builder.getDefaultDetailsPanel();
                    this.floatingDetailsPanel = builder.getFloatingDetailsPanel();
                    this.onClicked(function (event) {
                        _this.toggleClass("expanded");
                        if (_this.requiresAnimation()) {
                            _this.doPanelAnimation();
                        }
                    });
                    this.defaultDockedDetailsPanel.onShown(function () {
                        _this.splitPanelWithGridAndDetails.distribute();
                    });
                }
                NonMobileDetailsPanelsToggleButton.prototype.handleResizeEvent = function () {
                    var _this = this;
                    if (!this.resizeEventMonitorLocked) {
                        this.resizeEventMonitorLocked = true;
                        if (this.needsSwitchToFloatingMode() || this.needsSwitchToDockedMode()) {
                            this.doPanelAnimation();
                        }
                        else if (!this.splitPanelWithGridAndDetails.isSecondPanelHidden()) {
                            this.defaultDockedDetailsPanel.notifyPanelSizeChanged();
                        }
                        setTimeout(function () {
                            _this.resizeEventMonitorLocked = false;
                        }, 600);
                    }
                    else {
                        return;
                    }
                };
                NonMobileDetailsPanelsToggleButton.prototype.doPanelAnimation = function () {
                    var _this = this;
                    if (this.requiresFloatingPanelDueToShortWidth()) {
                        if (!this.splitPanelWithGridAndDetails.isSecondPanelHidden()) {
                            this.dockedToFloatingSync();
                        }
                        if (this.hasClass("expanded")) {
                            this.floatingDetailsPanel.slideIn();
                        }
                        else {
                            this.floatingDetailsPanel.slideOut();
                        }
                        this.splitPanelWithGridAndDetails.setActiveWidthPxOfSecondPanel(this.floatingDetailsPanel.getActualWidth());
                    }
                    else {
                        if (this.floatingPanelIsShown()) {
                            this.floatingToDockedSync();
                        }
                        this.defaultDockedDetailsPanel.addClass("left-bordered");
                        if (this.hasClass("expanded")) {
                            this.splitPanelWithGridAndDetails.showSecondPanel(false);
                        }
                        else if (!this.splitPanelWithGridAndDetails.isSecondPanelHidden()) {
                            this.splitPanelWithGridAndDetails.foldSecondPanel();
                        }
                        setTimeout(function () {
                            _this.defaultDockedDetailsPanel.removeClass("left-bordered");
                            if (_this.hasClass("expanded")) {
                                _this.splitPanelWithGridAndDetails.showSplitter();
                                _this.defaultDockedDetailsPanel.notifyPanelSizeChanged();
                            }
                        }, 500);
                    }
                    this.ensureButtonHasCorrectState();
                };
                NonMobileDetailsPanelsToggleButton.prototype.hideActivePanel = function () {
                    this.removeClass("expanded");
                    this.doPanelAnimation();
                };
                NonMobileDetailsPanelsToggleButton.prototype.hideDockedDetailsPanel = function () {
                    this.splitPanelWithGridAndDetails.hideSecondPanel();
                };
                NonMobileDetailsPanelsToggleButton.prototype.dockedToFloatingSync = function () {
                    var activePanelWidth = this.splitPanelWithGridAndDetails.getActiveWidthPxOfSecondPanel();
                    this.hideDockedDetailsPanel();
                    this.floatingDetailsPanel.setWidthPx(activePanelWidth);
                };
                NonMobileDetailsPanelsToggleButton.prototype.floatingToDockedSync = function () {
                    this.floatingDetailsPanel.slideOut();
                    var activePanelWidth = this.floatingDetailsPanel.getActualWidth();
                    this.splitPanelWithGridAndDetails.setActiveWidthPxOfSecondPanel(activePanelWidth);
                };
                NonMobileDetailsPanelsToggleButton.prototype.needsSwitchToFloatingMode = function () {
                    if (this.requiresFloatingPanelDueToShortWidth() && !this.splitPanelWithGridAndDetails.isSecondPanelHidden()) {
                        return true;
                    }
                    return false;
                };
                NonMobileDetailsPanelsToggleButton.prototype.needsSwitchToDockedMode = function () {
                    if (!this.requiresFloatingPanelDueToShortWidth() && this.splitPanelWithGridAndDetails.isSecondPanelHidden() && this.hasClass("expanded")) {
                        return true;
                    }
                    return false;
                };
                NonMobileDetailsPanelsToggleButton.prototype.requiresAnimation = function () {
                    if (this.hasClass("expanded")) {
                        if (this.splitPanelWithGridAndDetails.isSecondPanelHidden() && !this.floatingPanelIsShown()) {
                            return true;
                        }
                    }
                    else {
                        if (!this.splitPanelWithGridAndDetails.isSecondPanelHidden() || this.floatingPanelIsShown()) {
                            return true;
                        }
                    }
                    return false;
                };
                NonMobileDetailsPanelsToggleButton.prototype.floatingPanelIsShown = function () {
                    var right = this.floatingDetailsPanel.getHTMLElement().style.right;
                    if (right && right.indexOf("px") > -1) {
                        right = right.substring(0, right.indexOf("px"));
                        return Number(right) >= 0;
                    }
                    return false;
                };
                NonMobileDetailsPanelsToggleButton.prototype.requiresFloatingPanelDueToShortWidth = function () {
                    var splitPanelWidth = this.splitPanelWithGridAndDetails.getEl().getWidthWithBorder();
                    if (this.floatingPanelIsShown()) {
                        return (splitPanelWidth - this.floatingDetailsPanel.getActualWidth()) < 320;
                    }
                    else {
                        var defaultDetailsPanelWidth = this.splitPanelWithGridAndDetails.getActiveWidthPxOfSecondPanel();
                        return (splitPanelWidth - defaultDetailsPanelWidth) < 320;
                    }
                };
                NonMobileDetailsPanelsToggleButton.prototype.requiresCollapsedDetailsPanel = function () {
                    var splitPanelWidth = this.splitPanelWithGridAndDetails.getEl().getWidthWithBorder();
                    return this.requiresFloatingPanelDueToShortWidth() || ResponsiveRanges._1620_1920.isFitOrSmaller(splitPanelWidth);
                };
                NonMobileDetailsPanelsToggleButton.prototype.ensureButtonHasCorrectState = function () {
                    this.toggleClass("expanded", !this.splitPanelWithGridAndDetails.isSecondPanelHidden() || this.floatingPanelIsShown());
                };
                NonMobileDetailsPanelsToggleButton.create = function () {
                    return new NonMobileDetailsPanelsToggleButtonBuilder();
                };
                return NonMobileDetailsPanelsToggleButton;
            })(api.dom.DivEl);
            detail.NonMobileDetailsPanelsToggleButton = NonMobileDetailsPanelsToggleButton;
            var NonMobileDetailsPanelsToggleButtonBuilder = (function () {
                function NonMobileDetailsPanelsToggleButtonBuilder() {
                }
                NonMobileDetailsPanelsToggleButtonBuilder.prototype.setSplitPanelWithGridAndDetails = function (splitPanelWithGridAndDetails) {
                    this.splitPanelWithGridAndDetails = splitPanelWithGridAndDetails;
                };
                NonMobileDetailsPanelsToggleButtonBuilder.prototype.setDefaultDetailsPanel = function (defaultDockedDetailsPanel) {
                    this.defaultDockedDetailsPanel = defaultDockedDetailsPanel;
                };
                NonMobileDetailsPanelsToggleButtonBuilder.prototype.setFloatingDetailsPanel = function (floatingDetailsPanel) {
                    this.floatingDetailsPanel = floatingDetailsPanel;
                };
                NonMobileDetailsPanelsToggleButtonBuilder.prototype.getSplitPanelWithGridAndDetails = function () {
                    return this.splitPanelWithGridAndDetails;
                };
                NonMobileDetailsPanelsToggleButtonBuilder.prototype.getDefaultDetailsPanel = function () {
                    return this.defaultDockedDetailsPanel;
                };
                NonMobileDetailsPanelsToggleButtonBuilder.prototype.getFloatingDetailsPanel = function () {
                    return this.floatingDetailsPanel;
                };
                NonMobileDetailsPanelsToggleButtonBuilder.prototype.build = function () {
                    return new NonMobileDetailsPanelsToggleButton(this);
                };
                return NonMobileDetailsPanelsToggleButtonBuilder;
            })();
            detail.NonMobileDetailsPanelsToggleButtonBuilder = NonMobileDetailsPanelsToggleButtonBuilder;
            var InfoWidgetToggleButton = (function (_super) {
                __extends(InfoWidgetToggleButton, _super);
                function InfoWidgetToggleButton(detailsPanel) {
                    var _this = this;
                    _super.call(this, "info-widget-toggle-button");
                    this.detailsPanel = detailsPanel;
                    this.onClicked(function (event) {
                        _this.addClass("active");
                        detailsPanel.activateDefaultWidget();
                    });
                }
                InfoWidgetToggleButton.prototype.setActive = function () {
                    this.addClass("active");
                };
                InfoWidgetToggleButton.prototype.setInactive = function () {
                    this.removeClass("active");
                };
                return InfoWidgetToggleButton;
            })(api.dom.DivEl);
            detail.InfoWidgetToggleButton = InfoWidgetToggleButton;
            (function (SLIDE_FROM) {
                SLIDE_FROM[SLIDE_FROM["LEFT"] = 0] = "LEFT";
                SLIDE_FROM[SLIDE_FROM["RIGHT"] = 1] = "RIGHT";
                SLIDE_FROM[SLIDE_FROM["BOTTOM"] = 2] = "BOTTOM";
                SLIDE_FROM[SLIDE_FROM["TOP"] = 3] = "TOP";
            })(detail.SLIDE_FROM || (detail.SLIDE_FROM = {}));
            var SLIDE_FROM = detail.SLIDE_FROM;
            var WidgetViewOption = (function () {
                function WidgetViewOption(widgetView) {
                    this.widgetView = widgetView;
                }
                WidgetViewOption.prototype.getWidgetView = function () {
                    return this.widgetView;
                };
                WidgetViewOption.prototype.toString = function () {
                    return this.widgetView.getWidgetName();
                };
                return WidgetViewOption;
            })();
            detail.WidgetViewOption = WidgetViewOption;
            var WidgetsSelectionRow = (function (_super) {
                __extends(WidgetsSelectionRow, _super);
                function WidgetsSelectionRow(detailsPanel) {
                    _super.call(this, "widgets-selection-row");
                    this.detailsPanel = detailsPanel;
                    this.infoWidgetToggleButton = new InfoWidgetToggleButton(detailsPanel);
                    this.widgetSelectorDropdown = new WidgetSelectorDropdown();
                    this.widgetSelectorDropdown.addClass("widget-selector");
                    this.widgetSelectorDropdown.getInput().getEl().setDisabled(true);
                    this.widgetSelectorDropdown.getInput().setPlaceholder("");
                    this.widgetSelectorDropdown.onOptionSelected(function (event) {
                        var widgetView = event.getOption().displayValue.getWidgetView();
                        widgetView.setActive();
                    });
                    this.appendChild(this.infoWidgetToggleButton);
                    this.appendChild(this.widgetSelectorDropdown);
                }
                WidgetsSelectionRow.prototype.updateState = function (widgetView) {
                    if (this.detailsPanel.isDefaultWidget(widgetView)) {
                        this.infoWidgetToggleButton.setActive();
                        this.widgetSelectorDropdown.removeClass("non-default-selected");
                    }
                    else {
                        this.widgetSelectorDropdown.addClass("non-default-selected");
                        this.infoWidgetToggleButton.setInactive();
                    }
                };
                WidgetsSelectionRow.prototype.updateWidgetsDropdown = function (widgetViews) {
                    var _this = this;
                    this.widgetSelectorDropdown.removeAllOptions();
                    widgetViews.forEach(function (view) {
                        var option = {
                            value: view.getWidgetName(),
                            displayValue: new WidgetViewOption(view)
                        };
                        _this.widgetSelectorDropdown.addOption(option);
                    });
                    if (this.widgetSelectorDropdown.getOptionCount() < 2) {
                        this.widgetSelectorDropdown.addClass("single-optioned");
                    }
                    var visisbleNow = this.isVisible();
                    if (visisbleNow) {
                        this.setVisible(false);
                    }
                    this.widgetSelectorDropdown.selectRow(0, true);
                    if (visisbleNow) {
                        this.setVisible(true);
                    }
                };
                return WidgetsSelectionRow;
            })(api.dom.DivEl);
            detail.WidgetsSelectionRow = WidgetsSelectionRow;
            var WidgetSelectorDropdown = (function (_super) {
                __extends(WidgetSelectorDropdown, _super);
                function WidgetSelectorDropdown() {
                    var _this = this;
                    _super.call(this, "widgetSelector", {});
                    this.onClicked(function (event) {
                        if (!_this.isDropdownHandle(event.target)) {
                            if (_this.getSelectedOption()) {
                                var widgetView = _this.getSelectedOption().displayValue.getWidgetView();
                                widgetView.setActive();
                                _this.hideDropdown();
                            }
                        }
                    });
                }
                WidgetSelectorDropdown.prototype.isDropdownHandle = function (object) {
                    if (object && object["id"] && object["id"].toString().indexOf("DropdownHandle") > 0) {
                        return true;
                    }
                    return false;
                };
                return WidgetSelectorDropdown;
            })(Dropdown);
            detail.WidgetSelectorDropdown = WidgetSelectorDropdown;
        })(detail = _view.detail || (_view.detail = {}));
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var detail;
        (function (detail) {
            var WidgetView = (function (_super) {
                __extends(WidgetView, _super);
                function WidgetView(builder) {
                    _super.call(this, "widget-view");
                    this.detailsPanel = builder.detailsPanel;
                    this.widgetName = builder.name;
                    this.widgetItemViews = builder.widgetItemViews;
                    if (builder.useToggleButton) {
                        this.initWidgetToggleButton();
                    }
                    this.layout();
                }
                WidgetView.prototype.layout = function () {
                    var _this = this;
                    this.slideOut();
                    if (this.widgetItemViews) {
                        this.widgetItemViews.forEach(function (itemView) {
                            _this.appendChild(itemView);
                            itemView.layout();
                        });
                    }
                };
                WidgetView.prototype.initWidgetToggleButton = function () {
                    this.widgetToggleButton = new detail.WidgetViewToggleButton(this);
                    this.widgetToggleButton.setLabel(this.widgetName);
                    this.appendChild(this.widgetToggleButton);
                };
                WidgetView.prototype.updateNormalHeightSilently = function () {
                    this.setVisible(false);
                    var currentHeight = this.getEl().getHeight();
                    this.getEl().setHeight("auto");
                    this.normalHeightOfContent = this.getEl().getHeightWithBorder();
                    this.getEl().setHeightPx(currentHeight);
                    this.setVisible(true);
                };
                WidgetView.prototype.updateNormalHeight = function () {
                    this.getEl().setHeight("auto");
                    this.normalHeightOfContent = this.getEl().getHeightWithBorder();
                    this.getEl().setHeightPx(this.normalHeightOfContent);
                };
                WidgetView.prototype.getWidgetName = function () {
                    return this.widgetName;
                };
                WidgetView.prototype.slideOut = function () {
                    this.getEl().setHeightPx(0);
                };
                WidgetView.prototype.slideIn = function () {
                    this.getEl().setHeightPx(this.normalHeightOfContent);
                };
                WidgetView.prototype.setActive = function () {
                    this.detailsPanel.setActiveWidget(this);
                    this.slideIn();
                };
                WidgetView.prototype.setInactive = function () {
                    this.detailsPanel.resetActiveWidget();
                    this.deactivate();
                };
                WidgetView.prototype.deactivate = function () {
                    this.slideOut();
                    this.removeClass("expanded");
                };
                WidgetView.create = function () {
                    return new WidgetViewBuilder();
                };
                return WidgetView;
            })(api.dom.DivEl);
            detail.WidgetView = WidgetView;
            var WidgetViewBuilder = (function () {
                function WidgetViewBuilder() {
                    this.useToggleButton = true;
                    this.widgetItemViews = [];
                }
                WidgetViewBuilder.prototype.setName = function (name) {
                    this.name = name;
                    return this;
                };
                WidgetViewBuilder.prototype.setDetailsPanel = function (detailsPanel) {
                    this.detailsPanel = detailsPanel;
                    return this;
                };
                WidgetViewBuilder.prototype.setUseToggleButton = function (useToggleButton) {
                    this.useToggleButton = useToggleButton;
                    return this;
                };
                WidgetViewBuilder.prototype.addWidgetItemView = function (widgetItemView) {
                    this.widgetItemViews.push(widgetItemView);
                    return this;
                };
                WidgetViewBuilder.prototype.setWidgetItemViews = function (widgetItemViews) {
                    this.widgetItemViews = widgetItemViews;
                    return this;
                };
                WidgetViewBuilder.prototype.build = function () {
                    return new WidgetView(this);
                };
                return WidgetViewBuilder;
            })();
            detail.WidgetViewBuilder = WidgetViewBuilder;
        })(detail = view.detail || (view.detail = {}));
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var detail;
        (function (detail) {
            var WidgetItemView = (function (_super) {
                __extends(WidgetItemView, _super);
                function WidgetItemView(className) {
                    _super.call(this, "widget-item-view" + (className ? " " + className : ""));
                }
                WidgetItemView.prototype.setItem = function (item) {
                    this.item = item;
                };
                WidgetItemView.prototype.layout = function () {
                    if (this.item) {
                        this.appendChild(this.item);
                    }
                };
                return WidgetItemView;
            })(api.dom.DivEl);
            detail.WidgetItemView = WidgetItemView;
        })(detail = view.detail || (view.detail = {}));
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var detail;
        (function (detail) {
            var CompareStatus = api.content.CompareStatus;
            var CompareStatusFormatter = api.content.CompareStatusFormatter;
            var StatusWidgetItemView = (function (_super) {
                __extends(StatusWidgetItemView, _super);
                function StatusWidgetItemView() {
                    _super.call(this, "status-widget-item-view");
                }
                StatusWidgetItemView.prototype.setStatus = function (status) {
                    this.status = status;
                };
                StatusWidgetItemView.prototype.layout = function () {
                    this.removeChildren();
                    if (this.status != undefined) {
                        var statusEl = new api.dom.SpanEl().setHtml(CompareStatusFormatter.formatStatus(this.status).toLocaleUpperCase());
                        statusEl.addClass(CompareStatus[this.status].toLowerCase().replace("_", "-") || "unknown");
                        this.appendChild(statusEl);
                    }
                    _super.prototype.layout.call(this);
                };
                return StatusWidgetItemView;
            })(detail.WidgetItemView);
            detail.StatusWidgetItemView = StatusWidgetItemView;
        })(detail = view.detail || (view.detail = {}));
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var detail;
        (function (detail) {
            var DateTimeFormatter = api.ui.treegrid.DateTimeFormatter;
            var PropertiesWidgetItemView = (function (_super) {
                __extends(PropertiesWidgetItemView, _super);
                function PropertiesWidgetItemView() {
                    _super.call(this, "properties-widget-item-view");
                }
                PropertiesWidgetItemView.prototype.setContent = function (content) {
                    this.content = content;
                };
                PropertiesWidgetItemView.prototype.layout = function () {
                    var _this = this;
                    this.removeChildren();
                    if (this.content != undefined) {
                        if (!this.content.getType().getApplicationKey().isSystemReserved()) {
                            new api.application.GetApplicationRequest(this.content.getType().getApplicationKey()).sendAndParse().then(function (application) {
                                _this.doLayout(application);
                            }).catch(function () {
                                _this.doLayout();
                            }).done();
                        }
                        else {
                            this.doLayout();
                        }
                    }
                    _super.prototype.layout.call(this);
                };
                PropertiesWidgetItemView.prototype.doLayout = function (application) {
                    var newDl = new api.dom.DlEl();
                    var strings;
                    strings = [
                        new FieldString().setName("Type").setValue(this.content.getType().getLocalName() ? this.content.getType().getLocalName() : this.content.getType().toString()),
                        new FieldString().setName("Application").setValue(application ? application.getDisplayName() : this.content.getType().getApplicationKey().getName()),
                        this.content.getLanguage() ? new FieldString().setName("Language").setValue(this.content.getLanguage()) : null,
                        this.content.getOwner() ? new FieldString().setName("Owner").setValue(this.content.getOwner().getId()) : null,
                        this.content.getModifiedTime() ? new FieldString().setName("Modified").setValue(DateTimeFormatter.createHtmlNoTimestamp(this.content.getModifiedTime())) : null,
                        new FieldString().setName("Created").setValue(DateTimeFormatter.createHtmlNoTimestamp(this.content.getCreatedTime())),
                        new FieldString().setName("Id").setValue(this.content.getId())
                    ];
                    strings.forEach(function (stringItem) {
                        if (stringItem) {
                            stringItem.layout(newDl);
                        }
                    });
                    this.appendChild(newDl);
                };
                return PropertiesWidgetItemView;
            })(detail.WidgetItemView);
            detail.PropertiesWidgetItemView = PropertiesWidgetItemView;
            var FieldString = (function () {
                function FieldString() {
                }
                FieldString.prototype.setName = function (name) {
                    this.fieldName = name;
                    return this;
                };
                FieldString.prototype.setValue = function (value) {
                    this.value = value;
                    return this;
                };
                FieldString.prototype.layout = function (parentEl) {
                    var valueEl = new api.dom.DdDtEl("dt").setHtml(this.value);
                    var spanEl = new api.dom.DdDtEl("dd").setHtml(this.fieldName + ": ");
                    parentEl.appendChildren(spanEl, valueEl);
                };
                return FieldString;
            })();
        })(detail = view.detail || (view.detail = {}));
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var detail;
        (function (detail) {
            var WidgetViewToggleButton = (function (_super) {
                __extends(WidgetViewToggleButton, _super);
                function WidgetViewToggleButton(widget) {
                    var _this = this;
                    _super.call(this, "widget-toggle-button");
                    this.widget = widget;
                    this.labelEl = new api.dom.SpanEl('label');
                    this.appendChild(this.labelEl);
                    this.onClicked(function (event) {
                        _this.widget.toggleClass("expanded");
                        if (_this.widget.hasClass("expanded")) {
                            _this.widget.setActive();
                        }
                        else {
                            _this.widget.setInactive();
                        }
                    });
                }
                WidgetViewToggleButton.prototype.setLabel = function (value, addTitle) {
                    if (addTitle === void 0) { addTitle = true; }
                    this.labelEl.setHtml(value, true);
                    if (addTitle) {
                        this.labelEl.getEl().setAttribute('title', value);
                    }
                };
                return WidgetViewToggleButton;
            })(api.dom.DivEl);
            detail.WidgetViewToggleButton = WidgetViewToggleButton;
        })(detail = view.detail || (view.detail = {}));
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var filter;
        (function (filter) {
            var ContentBrowseResetEvent = (function (_super) {
                __extends(ContentBrowseResetEvent, _super);
                function ContentBrowseResetEvent() {
                    _super.apply(this, arguments);
                }
                ContentBrowseResetEvent.on = function (handler) {
                    api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
                };
                ContentBrowseResetEvent.un = function (handler) {
                    api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
                };
                return ContentBrowseResetEvent;
            })(api.event.Event);
            filter.ContentBrowseResetEvent = ContentBrowseResetEvent;
        })(filter = browse.filter || (browse.filter = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var filter;
        (function (filter) {
            var ContentBrowseSearchEvent = (function (_super) {
                __extends(ContentBrowseSearchEvent, _super);
                function ContentBrowseSearchEvent(contentQueryResult, contentQuery) {
                    _super.call(this);
                    this.contentQueryResult = contentQueryResult;
                    this.contentQuery = contentQuery;
                }
                ContentBrowseSearchEvent.prototype.getContentQueryResult = function () {
                    return this.contentQueryResult;
                };
                ContentBrowseSearchEvent.prototype.getContentQuery = function () {
                    return this.contentQuery;
                };
                ContentBrowseSearchEvent.on = function (handler) {
                    api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
                };
                ContentBrowseSearchEvent.un = function (handler) {
                    api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
                };
                return ContentBrowseSearchEvent;
            })(api.event.Event);
            filter.ContentBrowseSearchEvent = ContentBrowseSearchEvent;
        })(filter = browse.filter || (browse.filter = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var filter;
        (function (filter) {
            var ContentBrowseRefreshEvent = (function (_super) {
                __extends(ContentBrowseRefreshEvent, _super);
                function ContentBrowseRefreshEvent() {
                    _super.apply(this, arguments);
                }
                ContentBrowseRefreshEvent.on = function (handler) {
                    api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
                };
                ContentBrowseRefreshEvent.un = function (handler) {
                    api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
                };
                return ContentBrowseRefreshEvent;
            })(api.event.Event);
            filter.ContentBrowseRefreshEvent = ContentBrowseRefreshEvent;
        })(filter = browse.filter || (browse.filter = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var filter;
        (function (filter) {
            var ContentQueryRequest = api.content.ContentQueryRequest;
            var ContentTypeName = api.schema.content.ContentTypeName;
            var AggregationGroupView = api.aggregation.AggregationGroupView;
            var ContentTypeAggregationGroupView = api.aggregation.ContentTypeAggregationGroupView;
            var ContentQuery = api.content.query.ContentQuery;
            var TermsAggregationQuery = api.query.aggregation.TermsAggregationQuery;
            var DateRangeAggregationQuery = api.query.aggregation.DateRangeAggregationQuery;
            var DateRange = api.query.aggregation.DateRange;
            var QueryExpr = api.query.expr.QueryExpr;
            var ValueExpr = api.query.expr.ValueExpr;
            var RefreshEvent = api.app.browse.filter.RefreshEvent;
            var QueryField = api.query.QueryField;
            var ContentBrowseFilterPanel = (function (_super) {
                __extends(ContentBrowseFilterPanel, _super);
                function ContentBrowseFilterPanel() {
                    var _this = this;
                    this.contentTypeAggregation = new ContentTypeAggregationGroupView(ContentBrowseFilterPanel.CONTENT_TYPE_AGGREGATION_NAME, ContentBrowseFilterPanel.CONTENT_TYPE_AGGREGATION_DISPLAY_NAME);
                    this.lastModifiedAggregation = new AggregationGroupView(ContentBrowseFilterPanel.LAST_MODIFIED_AGGREGATION_NAME, ContentBrowseFilterPanel.LAST_MODIFIED_AGGREGATION_DISPLAY_NAME);
                    _super.call(this, null, [this.contentTypeAggregation, this.lastModifiedAggregation]);
                    this.initAggregationGroupView([this.contentTypeAggregation, this.lastModifiedAggregation]);
                    this.onReset(function () {
                        _this.resetFacets();
                    });
                    this.onRefresh(this.searchFacets);
                    this.onSearch(this.searchFacets);
                }
                ContentBrowseFilterPanel.prototype.searchFacets = function (event) {
                    var isClean = !this.hasFilterSet();
                    if (isClean) {
                        this.handleEmptyFilterInput(event);
                        return;
                    }
                    var contentQuery = this.createContentQuery(event);
                    var searchString = event.getSearchInputValues().getTextSearchFieldValue();
                    var refreshWithoutReload = event instanceof RefreshEvent;
                    this.searchDataAndHandleResponse(contentQuery, searchString, refreshWithoutReload);
                };
                ContentBrowseFilterPanel.prototype.handleEmptyFilterInput = function (event) {
                    if (event instanceof RefreshEvent) {
                        this.resetFacets(true, true).then(function () {
                            new filter.ContentBrowseRefreshEvent().fire();
                        }).catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).done();
                    }
                    else {
                        this.reset();
                    }
                };
                ContentBrowseFilterPanel.prototype.createContentQuery = function (event) {
                    var contentQuery = new ContentQuery();
                    this.appendFulltextSearch(event.getSearchInputValues(), contentQuery);
                    this.appendContentTypeFilter(event.getSearchInputValues(), contentQuery);
                    var lastModifiedFilter = this.appendLastModifiedQuery(event.getSearchInputValues());
                    if (lastModifiedFilter != null) {
                        contentQuery.addQueryFilter(lastModifiedFilter);
                    }
                    contentQuery.setSize(ContentQuery.POSTLOAD_SIZE);
                    this.appendContentTypesAggregationQuery(contentQuery);
                    this.appendLastModifiedAggregationQuery(contentQuery);
                    return contentQuery;
                };
                ContentBrowseFilterPanel.prototype.searchDataAndHandleResponse = function (contentQuery, searchString, refreshWithoutReload) {
                    var _this = this;
                    new ContentQueryRequest(contentQuery).setExpand(1 /* SUMMARY */).sendAndParse().then(function (contentQueryResult) {
                        if (!api.util.StringHelper.isEmpty(searchString)) {
                            _this.updateAggregations(contentQueryResult.getAggregations(), true);
                            _this.updateHitsCounter(contentQueryResult.getMetadata().getTotalHits());
                            _this.toggleAggregationsVisibility(contentQueryResult.getAggregations());
                            new filter.ContentBrowseSearchEvent(contentQueryResult, contentQuery).fire();
                        }
                        else {
                            if (refreshWithoutReload) {
                                _this.resetFacets(true, true);
                                new filter.ContentBrowseRefreshEvent().fire();
                            }
                            else {
                                _this.updateAggregations(contentQueryResult.getAggregations(), false);
                                _this.updateHitsCounter(contentQueryResult.getMetadata().getTotalHits());
                                _this.toggleAggregationsVisibility(contentQueryResult.getAggregations());
                                new filter.ContentBrowseSearchEvent(contentQueryResult, contentQuery).fire();
                            }
                        }
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).done();
                };
                ContentBrowseFilterPanel.prototype.initAggregationGroupView = function (aggregationGroupView) {
                    var _this = this;
                    var contentQuery = this.buildAggregationsQuery(new QueryExpr(null));
                    new ContentQueryRequest(contentQuery).sendAndParse().then(function (contentQueryResult) {
                        _this.updateAggregations(contentQueryResult.getAggregations(), false);
                        _this.updateHitsCounter(contentQueryResult.getMetadata().getTotalHits(), true);
                        _this.toggleAggregationsVisibility(contentQueryResult.getAggregations());
                        aggregationGroupView.forEach(function (aggregationGroupView) {
                            aggregationGroupView.initialize();
                        });
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).done();
                };
                ContentBrowseFilterPanel.prototype.resetFacets = function (suppressEvent, doResetAll) {
                    var _this = this;
                    var contentQuery = this.buildAggregationsQuery(new QueryExpr(null));
                    return new ContentQueryRequest(contentQuery).sendAndParse().then(function (contentQueryResult) {
                        _this.updateAggregations(contentQueryResult.getAggregations(), doResetAll);
                        _this.updateHitsCounter(contentQueryResult.getMetadata().getTotalHits(), true);
                        _this.toggleAggregationsVisibility(contentQueryResult.getAggregations());
                        if (!suppressEvent) {
                            new filter.ContentBrowseResetEvent().fire();
                        }
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    });
                };
                ContentBrowseFilterPanel.prototype.buildAggregationsQuery = function (queryExpr) {
                    var contentQuery = new ContentQuery();
                    contentQuery.setQueryExpr(queryExpr);
                    contentQuery.setSize(0);
                    this.appendContentTypesAggregationQuery(contentQuery);
                    this.appendLastModifiedAggregationQuery(contentQuery);
                    return contentQuery;
                };
                ContentBrowseFilterPanel.prototype.appendFulltextSearch = function (searchInputValues, contentQuery) {
                    var searchString = searchInputValues.getTextSearchFieldValue();
                    var fulltextSearchExpression = new api.query.FulltextSearchExpressionBuilder().setSearchString(searchString).addField(new QueryField(QueryField.DISPLAY_NAME, 5)).addField(new QueryField(QueryField.NAME, 3)).addField(new QueryField(QueryField.ALL)).build();
                    var query = new QueryExpr(fulltextSearchExpression);
                    contentQuery.setQueryExpr(query);
                };
                ContentBrowseFilterPanel.prototype.appendContentTypeFilter = function (searchInputValues, contentQuery) {
                    var selectedBuckets = searchInputValues.getSelectedValuesForAggregationName(ContentBrowseFilterPanel.CONTENT_TYPE_AGGREGATION_NAME);
                    var contentTypeNames = this.parseContentTypeNames(selectedBuckets);
                    contentQuery.setContentTypeNames(contentTypeNames);
                };
                ContentBrowseFilterPanel.prototype.appendLastModifiedQuery = function (searchInputValues) {
                    var lastModifiedSelectedBuckets = searchInputValues.getSelectedValuesForAggregationName(ContentBrowseFilterPanel.LAST_MODIFIED_AGGREGATION_NAME);
                    if (lastModifiedSelectedBuckets == null || lastModifiedSelectedBuckets.length == 0) {
                        return null;
                    }
                    if (lastModifiedSelectedBuckets.length == 1) {
                        var dateRangeBucket = lastModifiedSelectedBuckets.pop();
                        return new api.query.filter.RangeFilter(QueryField.MODIFIED_TIME, ValueExpr.dateTime(dateRangeBucket.getFrom()).getValue(), null);
                    }
                    var booleanFilter = new api.query.filter.BooleanFilter();
                    lastModifiedSelectedBuckets.forEach(function (selectedBucket) {
                        var rangeFilter = new api.query.filter.RangeFilter(QueryField.MODIFIED_TIME, ValueExpr.dateTime(selectedBucket.getFrom()).getValue(), null);
                        booleanFilter.addShould(rangeFilter);
                    });
                    return booleanFilter;
                };
                ContentBrowseFilterPanel.prototype.parseContentTypeNames = function (buckets) {
                    var contentTypeNames = [];
                    if (buckets) {
                        for (var i = 0; i < buckets.length; i++) {
                            var bucket = buckets[i];
                            contentTypeNames.push(new ContentTypeName(bucket.getKey()));
                        }
                    }
                    return contentTypeNames;
                };
                ContentBrowseFilterPanel.prototype.appendContentTypesAggregationQuery = function (contentQuery) {
                    contentQuery.addAggregationQuery(this.createTermsAggregation((ContentBrowseFilterPanel.CONTENT_TYPE_AGGREGATION_NAME), QueryField.CONTENT_TYPE, 30));
                };
                ContentBrowseFilterPanel.prototype.createTermsAggregation = function (name, fieldName, size) {
                    var termsAggregation = new TermsAggregationQuery(name);
                    termsAggregation.setFieldName(fieldName);
                    termsAggregation.setSize(size);
                    termsAggregation.setOrderByType(api.query.aggregation.TermsAggregationOrderType.DOC_COUNT);
                    termsAggregation.setOrderByDirection(api.query.aggregation.TermsAggregationOrderDirection.DESC);
                    return termsAggregation;
                };
                ContentBrowseFilterPanel.prototype.appendLastModifiedAggregationQuery = function (contentQuery) {
                    var dateRangeAgg = new DateRangeAggregationQuery((ContentBrowseFilterPanel.LAST_MODIFIED_AGGREGATION_NAME));
                    dateRangeAgg.setFieldName(QueryField.MODIFIED_TIME);
                    dateRangeAgg.addRange(new DateRange("now-1h", null, "< 1 hour"));
                    dateRangeAgg.addRange(new DateRange("now-1d", null, "< 1 day"));
                    dateRangeAgg.addRange(new DateRange("now-1w", null, "< 1 week"));
                    contentQuery.addAggregationQuery(dateRangeAgg);
                };
                ContentBrowseFilterPanel.prototype.toggleAggregationsVisibility = function (aggregations) {
                    var _this = this;
                    aggregations.forEach(function (aggregation) {
                        var aggregationIsEmpty = !aggregation.getBuckets().some(function (bucket) {
                            if (bucket.docCount > 0) {
                                return true;
                            }
                        });
                        var aggregationGroupView = aggregation.getName() == ContentBrowseFilterPanel.CONTENT_TYPE_AGGREGATION_NAME ? _this.contentTypeAggregation : _this.lastModifiedAggregation;
                        if (aggregationIsEmpty) {
                            aggregationGroupView.hide();
                        }
                        else {
                            aggregationGroupView.show();
                        }
                    });
                };
                ContentBrowseFilterPanel.CONTENT_TYPE_AGGREGATION_NAME = "contentTypes";
                ContentBrowseFilterPanel.LAST_MODIFIED_AGGREGATION_NAME = "lastModified";
                ContentBrowseFilterPanel.CONTENT_TYPE_AGGREGATION_DISPLAY_NAME = "Content Types";
                ContentBrowseFilterPanel.LAST_MODIFIED_AGGREGATION_DISPLAY_NAME = "Last Modified";
                return ContentBrowseFilterPanel;
            })(api.app.browse.filter.BrowseFilterPanel);
            filter.ContentBrowseFilterPanel = ContentBrowseFilterPanel;
        })(filter = browse.filter || (browse.filter = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var ShowNewContentDialogAction = (function (_super) {
                __extends(ShowNewContentDialogAction, _super);
                function ShowNewContentDialogAction(grid) {
                    _super.call(this, "New", "alt+n");
                    this.setEnabled(true);
                    this.onExecuted(function () {
                        var contentSummaries = grid.getSelectedDataList().map(function (elem) {
                            return elem.getContentSummary();
                        });
                        new browse.ShowNewContentDialogEvent(contentSummaries.length > 0 ? contentSummaries[0] : null).fire();
                    });
                }
                return ShowNewContentDialogAction;
            })(Action);
            action.ShowNewContentDialogAction = ShowNewContentDialogAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var RenderingMode = api.rendering.RenderingMode;
            var PreviewContentAction = (function (_super) {
                __extends(PreviewContentAction, _super);
                function PreviewContentAction(grid) {
                    var _this = this;
                    _super.call(this, "Preview", "");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        var contentSummaries = grid.getSelectedDataList().map(function (elem) {
                            return elem.getContentSummary();
                        });
                        _this.showPreviewDialog(contentSummaries[0]);
                    });
                }
                PreviewContentAction.prototype.showPreviewDialog = function (content) {
                    window.open(api.rendering.UriHelper.getPortalUri(content.getPath().toString(), 1 /* PREVIEW */, 0 /* DRAFT */), 'preview').focus();
                };
                return PreviewContentAction;
            })(Action);
            action.PreviewContentAction = PreviewContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var EditContentAction = (function (_super) {
                __extends(EditContentAction, _super);
                function EditContentAction(grid) {
                    _super.call(this, "Edit", "mod+e");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        var contentSummaries = grid.getSelectedDataList().map(function (elem) {
                            return elem.getContentSummary();
                        });
                        new api.content.EditContentEvent(contentSummaries).fire();
                    });
                }
                return EditContentAction;
            })(Action);
            action.EditContentAction = EditContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var DeleteContentAction = (function (_super) {
                __extends(DeleteContentAction, _super);
                function DeleteContentAction(grid) {
                    _super.call(this, "Delete", "mod+del");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        var contentSummaries = grid.getSelectedDataList().map(function (elem) {
                            return elem.getContentSummary();
                        });
                        new browse.ContentDeletePromptEvent(contentSummaries).fire();
                    });
                }
                return DeleteContentAction;
            })(Action);
            action.DeleteContentAction = DeleteContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var DuplicateContentAction = (function (_super) {
                __extends(DuplicateContentAction, _super);
                function DuplicateContentAction(grid) {
                    var _this = this;
                    _super.call(this, "Duplicate");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        grid.getSelectedDataList().forEach(function (elem) {
                            _this.duplicate(elem.getContentSummary());
                        });
                    });
                }
                DuplicateContentAction.prototype.duplicate = function (source) {
                    new api.content.DuplicateContentRequest(source.getContentId()).sendAndParse().then(function (content) {
                        api.notify.showFeedback('\"' + source.getDisplayName() + '\" duplicated');
                    });
                };
                return DuplicateContentAction;
            })(Action);
            action.DuplicateContentAction = DuplicateContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var MoveContentAction = (function (_super) {
                __extends(MoveContentAction, _super);
                function MoveContentAction(grid) {
                    _super.call(this, "Move");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        var contentSummaries = grid.getSelectedDataList().map(function (elem) {
                            return elem.getContentSummary();
                        });
                        new browse.MoveContentEvent(contentSummaries).fire();
                    });
                }
                return MoveContentAction;
            })(Action);
            action.MoveContentAction = MoveContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var SortContentAction = (function (_super) {
                __extends(SortContentAction, _super);
                function SortContentAction(grid) {
                    _super.call(this, "Sort");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        var contentSummaries = grid.getSelectedDataList().map(function (elem) {
                            return elem.getContentSummary();
                        });
                        new app.browse.SortContentEvent(contentSummaries).fire();
                    });
                }
                return SortContentAction;
            })(Action);
            action.SortContentAction = SortContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var PublishContentAction = (function (_super) {
                __extends(PublishContentAction, _super);
                function PublishContentAction(grid) {
                    _super.call(this, "Publish");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        var contentSummaries = grid.getSelectedDataList().map(function (elem) {
                            return elem.getContentSummary();
                        });
                        new browse.ContentPublishPromptEvent(contentSummaries).fire();
                    });
                }
                return PublishContentAction;
            })(Action);
            action.PublishContentAction = PublishContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var SaveSortedContentAction = (function (_super) {
                __extends(SaveSortedContentAction, _super);
                function SaveSortedContentAction(dialog) {
                    _super.call(this, "Save");
                    this.setEnabled(true);
                    this.onExecuted(function () {
                        new browse.SaveSortedContentEvent(dialog.getContent()).fire();
                    });
                }
                return SaveSortedContentAction;
            })(api.ui.Action);
            action.SaveSortedContentAction = SaveSortedContentAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var Action = api.ui.Action;
            var ToggleSearchPanelAction = (function (_super) {
                __extends(ToggleSearchPanelAction, _super);
                function ToggleSearchPanelAction() {
                    _super.call(this, "");
                    this.onExecuted(function () {
                        new browse.ToggleSearchPanelEvent().fire();
                    });
                    this.setIconClass("icon-search3");
                }
                return ToggleSearchPanelAction;
            })(Action);
            action.ToggleSearchPanelAction = ToggleSearchPanelAction;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var PermissionHelper = api.security.acl.PermissionHelper;
            var ContentTreeGridActions = (function () {
                function ContentTreeGridActions(grid) {
                    this.actions = [];
                    this.TOGGLE_SEARCH_PANEL = new action.ToggleSearchPanelAction();
                    this.SHOW_NEW_CONTENT_DIALOG_ACTION = new action.ShowNewContentDialogAction(grid);
                    this.PREVIEW_CONTENT = new action.PreviewContentAction(grid);
                    this.EDIT_CONTENT = new action.EditContentAction(grid);
                    this.DELETE_CONTENT = new action.DeleteContentAction(grid);
                    this.DUPLICATE_CONTENT = new action.DuplicateContentAction(grid);
                    this.MOVE_CONTENT = new action.MoveContentAction(grid);
                    this.SORT_CONTENT = new action.SortContentAction(grid);
                    this.PUBLISH_CONTENT = new action.PublishContentAction(grid);
                    this.actions.push(this.SHOW_NEW_CONTENT_DIALOG_ACTION, this.EDIT_CONTENT, this.DELETE_CONTENT, this.DUPLICATE_CONTENT, this.MOVE_CONTENT, this.SORT_CONTENT, this.PREVIEW_CONTENT, this.PUBLISH_CONTENT);
                }
                ContentTreeGridActions.prototype.getAllActions = function () {
                    return this.actions;
                };
                ContentTreeGridActions.prototype.updateActionsEnabledState = function (contentBrowseItems) {
                    var _this = this;
                    this.TOGGLE_SEARCH_PANEL.setVisible(false);
                    var contentSummaries = contentBrowseItems.map(function (elem) {
                        return elem.getModel();
                    });
                    var deferred = wemQ.defer();
                    switch (contentBrowseItems.length) {
                        case 0:
                            this.SHOW_NEW_CONTENT_DIALOG_ACTION.setEnabled(true);
                            this.EDIT_CONTENT.setEnabled(false);
                            this.DELETE_CONTENT.setEnabled(false);
                            this.DUPLICATE_CONTENT.setEnabled(false);
                            this.MOVE_CONTENT.setEnabled(false);
                            this.SORT_CONTENT.setEnabled(false);
                            this.PREVIEW_CONTENT.setEnabled(false);
                            this.PUBLISH_CONTENT.setEnabled(false);
                            var promise = this.updateActionsEnabledStateByPermissions(contentSummaries);
                            promise.then(function () {
                                deferred.resolve(contentBrowseItems);
                                return wemQ(null);
                            }).done();
                            break;
                        case 1:
                            var contentSummary = contentSummaries[0];
                            this.SHOW_NEW_CONTENT_DIALOG_ACTION.setEnabled(true);
                            this.EDIT_CONTENT.setEnabled(!contentSummary ? false : contentSummary.isEditable());
                            this.DELETE_CONTENT.setEnabled(!contentSummary ? false : contentSummary.isDeletable());
                            this.DUPLICATE_CONTENT.setEnabled(true);
                            this.MOVE_CONTENT.setEnabled(true);
                            this.PUBLISH_CONTENT.setEnabled(true);
                            this.SORT_CONTENT.setEnabled(true);
                            this.PREVIEW_CONTENT.setEnabled(false);
                            var parallelPromises = [
                                new api.content.page.IsRenderableRequest(contentSummary.getContentId()).sendAndParse().then(function (renderable) {
                                    _this.PREVIEW_CONTENT.setEnabled(renderable);
                                    if (contentBrowseItems.length > 0) {
                                        contentBrowseItems[0].setRenderable(renderable);
                                    }
                                }),
                                this.updateActionsEnabledStateByPermissions(contentSummaries)
                            ];
                            wemQ.all(parallelPromises).spread(function () {
                                deferred.resolve(contentBrowseItems);
                                return wemQ(null);
                            }).done();
                            break;
                        default:
                            this.SHOW_NEW_CONTENT_DIALOG_ACTION.setEnabled(false);
                            this.PREVIEW_CONTENT.setEnabled(false);
                            this.EDIT_CONTENT.setEnabled(this.anyEditable(contentSummaries));
                            this.DELETE_CONTENT.setEnabled(this.anyDeletable(contentSummaries));
                            this.DUPLICATE_CONTENT.setEnabled(false);
                            this.MOVE_CONTENT.setEnabled(true);
                            this.SORT_CONTENT.setEnabled(false);
                            this.PUBLISH_CONTENT.setEnabled(true);
                            var promise = this.updateActionsEnabledStateByPermissions(contentSummaries);
                            promise.then(function () {
                                deferred.resolve(contentBrowseItems);
                                return wemQ(null);
                            }).done();
                    }
                    return deferred.promise;
                };
                ContentTreeGridActions.prototype.updateActionsEnabledStateByPermissions = function (contentSummaries) {
                    var _this = this;
                    return new api.security.auth.IsAuthenticatedRequest().sendAndParse().then(function (loginResult) {
                        if (contentSummaries.length == 0) {
                            new api.content.GetContentRootPermissionsRequest().sendAndParse().then(function (accessControlList) {
                                var hasCreatePermission = PermissionHelper.hasPermission(1 /* CREATE */, loginResult, accessControlList);
                                if (!hasCreatePermission) {
                                    _this.SHOW_NEW_CONTENT_DIALOG_ACTION.setEnabled(false);
                                }
                            });
                        }
                        else {
                            var contentTypesAllowChildren = true;
                            var hasCreatePermission = true;
                            var hasDeletePermission = true;
                            var hasPublishPermission = true;
                            var hasParentCreatePermission = true;
                            var parallelPromises = [];
                            var nestedParallelPromises = [];
                            for (var i = 0; i < contentSummaries.length; i++) {
                                var contentSummary = contentSummaries[i];
                                if (contentSummaries.length == 1) {
                                    parallelPromises.push(new api.schema.content.GetContentTypeByNameRequest(contentSummary.getType()).sendAndParse().then(function (contentType) {
                                        contentTypesAllowChildren = contentTypesAllowChildren && (contentType && contentType.isAllowChildContent());
                                    }));
                                }
                                parallelPromises.push(new api.content.GetContentPermissionsByIdRequest(contentSummary.getContentId()).sendAndParse().then(function (accessControlList) {
                                    hasCreatePermission = hasCreatePermission && PermissionHelper.hasPermission(1 /* CREATE */, loginResult, accessControlList);
                                    hasDeletePermission = hasDeletePermission && PermissionHelper.hasPermission(3 /* DELETE */, loginResult, accessControlList);
                                    hasPublishPermission = hasDeletePermission && PermissionHelper.hasPermission(4 /* PUBLISH */, loginResult, accessControlList);
                                }));
                                if (contentSummaries.length == 1) {
                                    if (contentSummary.hasParent()) {
                                        parallelPromises.push(new api.content.GetContentByPathRequest(contentSummary.getPath().getParentPath()).sendAndParse().then(function (parent) {
                                            nestedParallelPromises.push(new api.content.GetContentPermissionsByIdRequest(parent.getContentId()).sendAndParse().then(function (accessControlList) {
                                                hasParentCreatePermission = hasParentCreatePermission && PermissionHelper.hasPermission(1 /* CREATE */, loginResult, accessControlList);
                                            }));
                                        }));
                                    }
                                    else {
                                        parallelPromises.push(new api.content.GetContentRootPermissionsRequest().sendAndParse().then(function (accessControlList) {
                                            hasParentCreatePermission = hasParentCreatePermission && PermissionHelper.hasPermission(1 /* CREATE */, loginResult, accessControlList);
                                        }));
                                    }
                                }
                            }
                            wemQ.all(parallelPromises).spread(function () {
                                if (!contentTypesAllowChildren || !hasCreatePermission) {
                                    _this.SHOW_NEW_CONTENT_DIALOG_ACTION.setEnabled(false);
                                    _this.SORT_CONTENT.setEnabled(false);
                                }
                                if (!hasDeletePermission) {
                                    _this.DELETE_CONTENT.setEnabled(false);
                                    _this.MOVE_CONTENT.setEnabled(false);
                                }
                                if (!hasPublishPermission) {
                                    _this.PUBLISH_CONTENT.setEnabled(false);
                                }
                                wemQ.all(nestedParallelPromises).spread(function () {
                                    if (!hasParentCreatePermission) {
                                        _this.DUPLICATE_CONTENT.setEnabled(false);
                                    }
                                    return wemQ(null);
                                }).done();
                                return wemQ(null);
                            }).done();
                        }
                    });
                };
                ContentTreeGridActions.prototype.anyEditable = function (contentSummaries) {
                    for (var i = 0; i < contentSummaries.length; i++) {
                        var content = contentSummaries[i];
                        if (!!content && content.isEditable()) {
                            return true;
                        }
                    }
                    return false;
                };
                ContentTreeGridActions.prototype.anyDeletable = function (contentSummaries) {
                    for (var i = 0; i < contentSummaries.length; i++) {
                        var content = contentSummaries[i];
                        if (!!content && content.isDeletable()) {
                            return true;
                        }
                    }
                    return false;
                };
                return ContentTreeGridActions;
            })();
            action.ContentTreeGridActions = ContentTreeGridActions;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var action;
        (function (action) {
            var MobileContentTreeGridActions = (function () {
                function MobileContentTreeGridActions(grid) {
                    this.actions = [];
                    this.SHOW_NEW_CONTENT_DIALOG_ACTION = new action.ShowNewContentDialogAction(grid);
                    this.EDIT_CONTENT = new action.EditContentAction(grid);
                    this.DELETE_CONTENT = new action.DeleteContentAction(grid);
                    this.DUPLICATE_CONTENT = new action.DuplicateContentAction(grid);
                    this.MOVE_CONTENT = new action.MoveContentAction(grid);
                    this.SORT_CONTENT = new action.SortContentAction(grid);
                    this.PUBLISH_CONTENT = new action.PublishContentAction(grid);
                    this.actions.push(this.SHOW_NEW_CONTENT_DIALOG_ACTION, this.DELETE_CONTENT, this.DUPLICATE_CONTENT, this.MOVE_CONTENT, this.SORT_CONTENT, this.PUBLISH_CONTENT);
                }
                MobileContentTreeGridActions.prototype.getAllActions = function () {
                    return this.actions;
                };
                MobileContentTreeGridActions.prototype.updateActionsEnabledState = function (contentBrowseItems) {
                    var _this = this;
                    var contentSummaries = contentBrowseItems.map(function (elem) {
                        return elem.getModel();
                    });
                    var deferred = wemQ.defer();
                    switch (contentBrowseItems.length) {
                        case 0:
                            this.EDIT_CONTENT.setEnabled(false);
                            this.DELETE_CONTENT.setEnabled(false);
                            this.DUPLICATE_CONTENT.setEnabled(false);
                            this.MOVE_CONTENT.setEnabled(false);
                            this.SORT_CONTENT.setEnabled(false);
                            this.PUBLISH_CONTENT.setEnabled(false);
                            deferred.resolve(contentBrowseItems);
                            break;
                        case 1:
                            var contentSummary = contentSummaries[0];
                            this.EDIT_CONTENT.setEnabled(!contentSummary ? false : contentSummary.isEditable());
                            this.DELETE_CONTENT.setEnabled(!contentSummary ? false : contentSummary.isDeletable());
                            this.DUPLICATE_CONTENT.setEnabled(true);
                            this.MOVE_CONTENT.setEnabled(true);
                            this.PUBLISH_CONTENT.setEnabled(true);
                            var parallelPromises = [
                                new api.schema.content.GetContentTypeByNameRequest(contentSummary.getType()).sendAndParse().then(function (contentType) {
                                    var allowsChildren = (contentType && contentType.isAllowChildContent());
                                    _this.SORT_CONTENT.setEnabled(allowsChildren);
                                    var hasCreatePermission = false;
                                    new api.security.auth.IsAuthenticatedRequest().sendAndParse().then(function (loginResult) {
                                        new api.content.GetContentPermissionsByIdRequest(contentSummary.getContentId()).sendAndParse().then(function (accessControlList) {
                                            hasCreatePermission = _this.hasPermission(1 /* CREATE */, loginResult, accessControlList);
                                            _this.SHOW_NEW_CONTENT_DIALOG_ACTION.setEnabled(allowsChildren && hasCreatePermission);
                                        });
                                    });
                                })
                            ];
                            wemQ.all(parallelPromises).spread(function () {
                                deferred.resolve(contentBrowseItems);
                                return wemQ(null);
                            }).done();
                            break;
                        default:
                            this.SHOW_NEW_CONTENT_DIALOG_ACTION.setEnabled(false);
                            this.EDIT_CONTENT.setEnabled(this.anyEditable(contentSummaries));
                            this.DELETE_CONTENT.setEnabled(this.anyDeletable(contentSummaries));
                            this.DUPLICATE_CONTENT.setEnabled(false);
                            this.MOVE_CONTENT.setEnabled(true);
                            this.SORT_CONTENT.setEnabled(false);
                            this.PUBLISH_CONTENT.setEnabled(true);
                            deferred.resolve(contentBrowseItems);
                    }
                    return deferred.promise;
                };
                MobileContentTreeGridActions.prototype.anyEditable = function (contentSummaries) {
                    for (var i = 0; i < contentSummaries.length; i++) {
                        var content = contentSummaries[i];
                        if (!!content && content.isEditable()) {
                            return true;
                        }
                    }
                    return false;
                };
                MobileContentTreeGridActions.prototype.anyDeletable = function (contentSummaries) {
                    for (var i = 0; i < contentSummaries.length; i++) {
                        var content = contentSummaries[i];
                        if (!!content && content.isDeletable()) {
                            return true;
                        }
                    }
                    return false;
                };
                MobileContentTreeGridActions.prototype.isPrincipalPresent = function (principalKey, accessEntriesToCheck) {
                    return accessEntriesToCheck.some(function (entry) {
                        return entry.getPrincipalKey().equals(principalKey);
                    });
                };
                MobileContentTreeGridActions.prototype.hasPermission = function (permission, loginResult, accessControlList) {
                    var _this = this;
                    var entries = accessControlList.getEntries();
                    var accessEntriesWithGivenPermissions = entries.filter(function (item) {
                        return item.isAllowed(permission);
                    });
                    return loginResult.getPrincipals().some(function (principalKey) {
                        return api.security.RoleKeys.ADMIN.equals(principalKey) || _this.isPrincipalPresent(principalKey, accessEntriesWithGivenPermissions);
                    });
                };
                return MobileContentTreeGridActions;
            })();
            action.MobileContentTreeGridActions = MobileContentTreeGridActions;
        })(action = browse.action || (browse.action = {}));
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var BaseContentModelEvent = (function (_super) {
            __extends(BaseContentModelEvent, _super);
            function BaseContentModelEvent(model) {
                this.model = model;
                _super.call(this);
            }
            BaseContentModelEvent.prototype.getModels = function () {
                return this.model;
            };
            return BaseContentModelEvent;
        })(api.event.Event);
        browse.BaseContentModelEvent = BaseContentModelEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var Event = api.event.Event;
        var CloseContentEvent = (function (_super) {
            __extends(CloseContentEvent, _super);
            function CloseContentEvent(panel, checkCanRemovePanel) {
                if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                _super.call(this);
                this.panel = panel;
                this.checkCanRemovePanel = checkCanRemovePanel;
            }
            CloseContentEvent.prototype.getPanel = function () {
                return this.panel;
            };
            CloseContentEvent.prototype.isCheckCanRemovePanel = function () {
                return this.checkCanRemovePanel;
            };
            CloseContentEvent.on = function (handler) {
                Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            CloseContentEvent.un = function (handler) {
                Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return CloseContentEvent;
        })(Event);
        browse.CloseContentEvent = CloseContentEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ContentDeletePromptEvent = (function (_super) {
            __extends(ContentDeletePromptEvent, _super);
            function ContentDeletePromptEvent() {
                _super.apply(this, arguments);
            }
            ContentDeletePromptEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentDeletePromptEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ContentDeletePromptEvent;
        })(browse.BaseContentModelEvent);
        browse.ContentDeletePromptEvent = ContentDeletePromptEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var GridSelectionChangeEvent = (function (_super) {
            __extends(GridSelectionChangeEvent, _super);
            function GridSelectionChangeEvent() {
                _super.apply(this, arguments);
            }
            GridSelectionChangeEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            GridSelectionChangeEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return GridSelectionChangeEvent;
        })(browse.BaseContentModelEvent);
        browse.GridSelectionChangeEvent = GridSelectionChangeEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var MoveContentEvent = (function (_super) {
            __extends(MoveContentEvent, _super);
            function MoveContentEvent() {
                _super.apply(this, arguments);
            }
            MoveContentEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            MoveContentEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return MoveContentEvent;
        })(browse.BaseContentModelEvent);
        browse.MoveContentEvent = MoveContentEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ShowContextMenuEvent = (function (_super) {
            __extends(ShowContextMenuEvent, _super);
            function ShowContextMenuEvent(x, y) {
                this.x = x;
                this.y = y;
                _super.call(this);
            }
            ShowContextMenuEvent.prototype.getX = function () {
                return this.x;
            };
            ShowContextMenuEvent.prototype.getY = function () {
                return this.y;
            };
            ShowContextMenuEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowContextMenuEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowContextMenuEvent;
        })(api.event.Event);
        browse.ShowContextMenuEvent = ShowContextMenuEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ShowDetailsEvent = (function (_super) {
            __extends(ShowDetailsEvent, _super);
            function ShowDetailsEvent() {
                _super.apply(this, arguments);
            }
            ShowDetailsEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowDetailsEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowDetailsEvent;
        })(browse.BaseContentModelEvent);
        browse.ShowDetailsEvent = ShowDetailsEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ShowNewContentDialogEvent = (function (_super) {
            __extends(ShowNewContentDialogEvent, _super);
            function ShowNewContentDialogEvent(parentContent) {
                _super.call(this, [parentContent]);
                this.parentContent = parentContent;
            }
            ShowNewContentDialogEvent.prototype.getParentContent = function () {
                return this.parentContent;
            };
            ShowNewContentDialogEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowNewContentDialogEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowNewContentDialogEvent;
        })(browse.BaseContentModelEvent);
        browse.ShowNewContentDialogEvent = ShowNewContentDialogEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ShowNewContentGridEvent = (function (_super) {
            __extends(ShowNewContentGridEvent, _super);
            function ShowNewContentGridEvent() {
                _super.apply(this, arguments);
            }
            ShowNewContentGridEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowNewContentGridEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowNewContentGridEvent;
        })(api.event.Event);
        browse.ShowNewContentGridEvent = ShowNewContentGridEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ShowPreviewEvent = (function (_super) {
            __extends(ShowPreviewEvent, _super);
            function ShowPreviewEvent() {
                _super.apply(this, arguments);
            }
            ShowPreviewEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowPreviewEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowPreviewEvent;
        })(browse.BaseContentModelEvent);
        browse.ShowPreviewEvent = ShowPreviewEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ToggleSearchPanelEvent = (function (_super) {
            __extends(ToggleSearchPanelEvent, _super);
            function ToggleSearchPanelEvent() {
                _super.apply(this, arguments);
            }
            ToggleSearchPanelEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ToggleSearchPanelEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ToggleSearchPanelEvent;
        })(api.event.Event);
        browse.ToggleSearchPanelEvent = ToggleSearchPanelEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ViewContentEvent = (function (_super) {
            __extends(ViewContentEvent, _super);
            function ViewContentEvent() {
                _super.apply(this, arguments);
            }
            ViewContentEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ViewContentEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ViewContentEvent;
        })(browse.BaseContentModelEvent);
        browse.ViewContentEvent = ViewContentEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ContentBrowseToolbar = (function (_super) {
            __extends(ContentBrowseToolbar, _super);
            function ContentBrowseToolbar(actions) {
                _super.call(this);
                this.addClass("content-browse-toolbar");
                this.addActions(actions.getAllActions());
            }
            return ContentBrowseToolbar;
        })(api.ui.toolbar.Toolbar);
        browse.ContentBrowseToolbar = ContentBrowseToolbar;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ContentBrowseItemPanel = (function (_super) {
            __extends(ContentBrowseItemPanel, _super);
            function ContentBrowseItemPanel() {
                _super.apply(this, arguments);
            }
            ContentBrowseItemPanel.prototype.createItemSelectionPanel = function () {
                return new browse.ContentBrowseItemsSelectionPanel();
            };
            ContentBrowseItemPanel.prototype.createItemStatisticsPanel = function () {
                return new app.view.ContentItemStatisticsPanel();
            };
            return ContentBrowseItemPanel;
        })(api.app.browse.BrowseItemPanel);
        browse.ContentBrowseItemPanel = ContentBrowseItemPanel;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ContentSummaryViewer = api.content.ContentSummaryViewer;
        var ContentBrowseItemsSelectionPanel = (function (_super) {
            __extends(ContentBrowseItemsSelectionPanel, _super);
            function ContentBrowseItemsSelectionPanel() {
                _super.apply(this, arguments);
            }
            ContentBrowseItemsSelectionPanel.prototype.createItemViewer = function (item) {
                var viewer = new ContentSummaryViewer();
                viewer.setObject(item.getModel());
                return viewer;
            };
            return ContentBrowseItemsSelectionPanel;
        })(api.app.browse.BrowseItemsSelectionPanel);
        browse.ContentBrowseItemsSelectionPanel = ContentBrowseItemsSelectionPanel;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var BrowseItem = api.app.browse.BrowseItem;
        var ContentSummaryAndCompareStatusFetcher = api.content.ContentSummaryAndCompareStatusFetcher;
        var ResponsiveManager = api.ui.responsive.ResponsiveManager;
        var ResponsiveRanges = api.ui.responsive.ResponsiveRanges;
        var ContentIconUrlResolver = api.content.ContentIconUrlResolver;
        var ContentServerEvent = api.content.ContentServerEvent;
        var ContentServerChangeType = api.content.ContentServerChangeType;
        var ContentChangeResult = api.content.ContentChangeResult;
        var DetailsPanel = app.view.detail.DetailsPanel;
        var NonMobileDetailsPanelsToggleButton = app.view.detail.NonMobileDetailsPanelsToggleButton;
        var ContentBrowsePanel = (function (_super) {
            __extends(ContentBrowsePanel, _super);
            function ContentBrowsePanel() {
                var _this = this;
                this.contentTreeGrid = new app.browse.ContentTreeGrid();
                this.contentBrowseItemPanel = components.detailPanel = new browse.ContentBrowseItemPanel();
                this.contentFilterPanel = new app.browse.filter.ContentBrowseFilterPanel();
                this.browseActions = this.contentTreeGrid.getContextMenu().getActions();
                this.toolbar = new browse.ContentBrowseToolbar(this.browseActions);
                this.defaultDockedDetailsPanel = DetailsPanel.create().setUseSplitter(false).build();
                _super.call(this, {
                    browseToolbar: this.toolbar,
                    treeGrid: this.contentTreeGrid,
                    browseItemPanel: this.contentBrowseItemPanel,
                    filterPanel: this.contentFilterPanel,
                    hasDetailsPanel: true
                });
                var showMask = function () {
                    if (_this.isVisible()) {
                        _this.contentTreeGrid.mask();
                    }
                };
                this.contentFilterPanel.onSearch(showMask);
                this.contentFilterPanel.onReset(showMask);
                this.contentFilterPanel.onRefresh(showMask);
                this.getTreeGrid().onDataChanged(function (event) {
                    if (event.getType() === 'updated') {
                        var browseItems = _this.treeNodesToBrowseItems(event.getTreeNodes());
                        _this.getBrowseItemPanel().updateItemViewers(browseItems);
                    }
                });
                this.onShown(function (event) {
                    app.Router.setHash("browse");
                });
                ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                    if (item.isInRangeOrSmaller(ResponsiveRanges._360_540)) {
                        _this.browseActions.TOGGLE_SEARCH_PANEL.setVisible(true);
                    }
                    else if (item.isInRangeOrBigger(ResponsiveRanges._540_720)) {
                        _this.browseActions.TOGGLE_SEARCH_PANEL.setVisible(false);
                    }
                });
                this.handleGlobalEvents();
                this.onRendered(function (event) {
                    _this.appendChild(_this.floatingDetailsPanel);
                    _this.initItemStatisticsPanelForMobile();
                });
            }
            ContentBrowsePanel.prototype.initNonMobileDetailsPanels = function () {
                var controlButtonBuilder = NonMobileDetailsPanelsToggleButton.create();
                this.initFloatingDetailsPanel(controlButtonBuilder);
                this.initSplitPanelWithDockedDetails(controlButtonBuilder);
                var nonMobileDetailsPanelsToggleButton = controlButtonBuilder.build();
                if (nonMobileDetailsPanelsToggleButton.requiresCollapsedDetailsPanel()) {
                    nonMobileDetailsPanelsToggleButton.hideDockedDetailsPanel();
                }
                nonMobileDetailsPanelsToggleButton.ensureButtonHasCorrectState();
                this.subscribeDetailsPanelsOnEvents(nonMobileDetailsPanelsToggleButton);
                this.toolbar.appendChild(nonMobileDetailsPanelsToggleButton);
            };
            ContentBrowsePanel.prototype.subscribeDetailsPanelsOnEvents = function (nonMobileDetailsPanelsToggleButton) {
                var _this = this;
                this.getTreeGrid().onSelectionChanged(function (currentSelection, fullSelection) {
                    var browseItems = _this.getBrowseItemPanel().getItems();
                    if (browseItems.length == 0) {
                        _this.floatingDetailsPanel.setItem(null);
                        _this.defaultDockedDetailsPanel.setItem(null);
                    }
                    else {
                        var item = browseItems[browseItems.length - 1].toViewItem();
                        _this.floatingDetailsPanel.setItem(item);
                        _this.defaultDockedDetailsPanel.setItem(item);
                    }
                });
                ResponsiveManager.onAvailableSizeChanged(this.getFilterAndContentGridAndBrowseSplitPanel(), function (item) {
                    nonMobileDetailsPanelsToggleButton.handleResizeEvent();
                });
                ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                    if (ResponsiveRanges._540_720.isFitOrBigger(item.getOldRangeValue()) && item.isInRangeOrSmaller(ResponsiveRanges._360_540)) {
                        nonMobileDetailsPanelsToggleButton.hideActivePanel();
                    }
                });
            };
            ContentBrowsePanel.prototype.initSplitPanelWithDockedDetails = function (controlButtonBuilder) {
                var contentPanelsAndDetailPanel = new api.ui.panel.SplitPanelBuilder(this.getFilterAndContentGridAndBrowseSplitPanel(), this.defaultDockedDetailsPanel).setAlignment(1 /* VERTICAL */).setSecondPanelSize(280, 0 /* PIXEL */).setSecondPanelMinSize(280, 0 /* PIXEL */).setAnimationDelay(600).build();
                contentPanelsAndDetailPanel.addClass("split-panel-with-details");
                contentPanelsAndDetailPanel.setSecondPanelSize(280, 0 /* PIXEL */);
                this.appendChild(contentPanelsAndDetailPanel);
                controlButtonBuilder.setSplitPanelWithGridAndDetails(contentPanelsAndDetailPanel);
                controlButtonBuilder.setDefaultDetailsPanel(this.defaultDockedDetailsPanel);
            };
            ContentBrowsePanel.prototype.initFloatingDetailsPanel = function (controlButtonBuilder) {
                this.floatingDetailsPanel = DetailsPanel.create().build();
                controlButtonBuilder.setFloatingDetailsPanel(this.floatingDetailsPanel);
            };
            ContentBrowsePanel.prototype.initItemStatisticsPanelForMobile = function () {
                var _this = this;
                this.mobileBrowseActions = new app.browse.action.MobileContentTreeGridActions(this.contentTreeGrid);
                this.mobileContentItemStatisticsPanel = new app.view.MobileContentItemStatisticsPanel(this.mobileBrowseActions);
                api.content.TreeGridItemClickedEvent.on(function (event) {
                    var browseItems = _this.getBrowseItemPanel().getItems();
                    if (browseItems.length == 1) {
                        new api.content.page.IsRenderableRequest(new api.content.ContentId(browseItems[0].getId())).sendAndParse().then(function (renderable) {
                            var item = browseItems[0].toViewItem();
                            item.setRenderable(renderable);
                            _this.mobileContentItemStatisticsPanel.setItem(item);
                            _this.mobileBrowseActions.updateActionsEnabledState(browseItems);
                        });
                    }
                });
                this.appendChild(this.mobileContentItemStatisticsPanel);
            };
            ContentBrowsePanel.prototype.treeNodesToBrowseItems = function (nodes) {
                var browseItems = [];
                nodes.forEach(function (node, index) {
                    for (var i = 0; i <= index; i++) {
                        if (nodes[i].getData().getId() === node.getData().getId()) {
                            break;
                        }
                    }
                    if (i === index) {
                        var content = node.getData().getContentSummary();
                        if (!!content) {
                            var item = new BrowseItem(content).setId(content.getId()).setDisplayName(content.getDisplayName()).setPath(content.getPath().toString()).setIconUrl(new ContentIconUrlResolver().setContent(content).resolve());
                            browseItems.push(item);
                        }
                    }
                });
                return browseItems;
            };
            ContentBrowsePanel.prototype.handleGlobalEvents = function () {
                var _this = this;
                api.content.ContentChildOrderUpdatedEvent.on(function (event) {
                });
                browse.ToggleSearchPanelEvent.on(function () {
                    _this.toggleFilterPanel();
                });
                app.create.NewMediaUploadEvent.on(function (event) {
                    _this.handleNewMediaUpload(event);
                });
                var handler = this.contentServerEventHandler.bind(this);
                ContentServerEvent.on(handler);
                this.onRemoved(function () {
                    ContentServerEvent.un(handler);
                });
            };
            ContentBrowsePanel.prototype.contentServerEventHandler = function (event) {
                var _this = this;
                var changes = event.getContentChanges();
                var deferred = wemQ.defer(), promise = deferred.promise;
                changes.forEach(function (change) {
                    switch (change.getChangeType()) {
                        case 3 /* CREATE */:
                            promise = _this.handleContentCreated(change, promise, changes);
                            break;
                        case 4 /* UPDATE */:
                            promise = _this.handleContentUpdated(change, promise, changes);
                            break;
                        case 7 /* RENAME */:
                            promise = _this.handleContentRenamed(change, promise, changes);
                            break;
                        case 5 /* DELETE */:
                            promise = _this.handleContentDeleted(change, promise, changes);
                            break;
                        case 6 /* PENDING */:
                            promise = _this.handleContentPending(change, promise, changes);
                            break;
                        case 2 /* DUPLICATE */:
                            promise = _this.handleContentCreated(change, promise, changes);
                            break;
                        case 1 /* PUBLISH */:
                            promise = _this.handleContentPublished(change, promise, changes);
                            break;
                        case 8 /* SORT */:
                            promise = _this.handleContentSorted(change, promise, changes);
                            break;
                        case 0 /* UNKNOWN */:
                            break;
                        default:
                    }
                });
                deferred.resolve(null);
            };
            ContentBrowsePanel.prototype.handleContentCreated = function (change, promise, changes) {
                var _this = this;
                promise = promise.then(function (result) {
                    var createResult = _this.contentTreeGrid.findByPaths(change.getContentPaths(), true);
                    return ContentSummaryAndCompareStatusFetcher.fetchByPaths(createResult.map(function (el) {
                        return el.getAltPath();
                    })).then(function (data) {
                        var isFiltered = _this.contentTreeGrid.getRoot().isFiltered(), nodes = [];
                        data.forEach(function (el) {
                            for (var i = 0; i < createResult.length; i++) {
                                if (el.getContentSummary().getPath().isChildOf(createResult[i].getPath())) {
                                    if (result && result.getChangeType() === 7 /* RENAME */) {
                                        var premerged = result.getResult().map(function (el) {
                                            return el.getNodes();
                                        });
                                        nodes = nodes.concat.apply(nodes, premerged);
                                        nodes.forEach(function (node) {
                                            if (node.getDataId() === el.getId()) {
                                                node.setData(el);
                                                node.clearViewers();
                                                _this.contentTreeGrid.xUpdatePathsInChildren(node);
                                            }
                                        });
                                    }
                                    else {
                                        _this.contentTreeGrid.xAppendContentNodes(createResult[i].getNodes().map(function (node) {
                                            return new api.content.TreeNodeParentOfContent(el, node);
                                        }), !isFiltered).then(function (results) {
                                            nodes = nodes.concat(results);
                                        });
                                    }
                                    break;
                                }
                            }
                        });
                        if (result && result.getChangeType() === 5 /* DELETE */) {
                            var ids = result.getResult().map(function (el) {
                                return el.getId();
                            });
                            nodes.forEach(function (node) {
                                var index = ids.indexOf(node.getDataId());
                                if (index >= 0) {
                                    _this.contentTreeGrid.xPopulateWithChildren(result.getResult()[index].getNodes()[0], node);
                                }
                            });
                        }
                        _this.contentTreeGrid.initAndRender();
                        isFiltered = true;
                        if (isFiltered) {
                            _this.setFilterPanelRefreshNeeded(true);
                            window.setTimeout(function () {
                                _this.refreshFilter();
                            }, 1000);
                        }
                    });
                });
                return promise;
            };
            ContentBrowsePanel.prototype.handleContentUpdated = function (change, promise, changes) {
                var _this = this;
                if (changes.length === 1) {
                    promise = promise.then(function (result) {
                        var updateResult = _this.contentTreeGrid.findByPaths(change.getContentPaths());
                        var ids = [];
                        updateResult.forEach(function (el) {
                            ids = ids.concat(el.getNodes().map(function (node) {
                                return node.getData().getContentId();
                            }));
                        });
                        return ContentSummaryAndCompareStatusFetcher.fetchByIds(ids).then(function (data) {
                            var results = [];
                            data.forEach(function (el) {
                                for (var i = 0; i < updateResult.length; i++) {
                                    if (updateResult[i].getId() === el.getId()) {
                                        updateResult[i].updateNodeData(el);
                                        _this.updateStatisticsPreview(el);
                                        var selectedItems = _this.getBrowseItemPanel().getItems();
                                        var viewItem = selectedItems[selectedItems.length - 1].toViewItem();
                                        _this.updateDetailsPanels(el.getContentId(), el.getCompareStatus(), viewItem);
                                        results.push(updateResult[i]);
                                        break;
                                    }
                                }
                            });
                            _this.browseActions.updateActionsEnabledState(_this.getBrowseItemPanel().getItems());
                            _this.mobileBrowseActions.updateActionsEnabledState(_this.getBrowseItemPanel().getItems());
                            return _this.contentTreeGrid.xPlaceContentNodes(results);
                        });
                    });
                }
                return promise;
            };
            ContentBrowsePanel.prototype.handleContentRenamed = function (change, promise, changes) {
                var _this = this;
                promise = promise.then(function (result) {
                    var renameResult = _this.contentTreeGrid.findByPaths(change.getContentPaths());
                    return new ContentChangeResult(7 /* RENAME */, renameResult);
                });
                return promise;
            };
            ContentBrowsePanel.prototype.handleContentDeleted = function (change, promise, changes) {
                var _this = this;
                promise = promise.then(function (result) {
                    if (result && result.getChangeType() === 7 /* RENAME */) {
                        return result;
                    }
                    var deleteResult = _this.contentTreeGrid.findByPaths(change.getContentPaths());
                    var nodes = deleteResult.map(function (el) {
                        return el.getNodes();
                    });
                    var merged = [];
                    merged = merged.concat.apply(merged, nodes);
                    merged.forEach(function (node) {
                        if (node.getData() && node.getData().getContentSummary()) {
                            _this.updateDetailsPanels(node.getData().getContentId(), node.getData().getCompareStatus(), null);
                            new api.content.ContentDeletedEvent(node.getData().getContentSummary().getContentId()).fire();
                        }
                    });
                    _this.contentTreeGrid.xDeleteContentNodes(merged);
                    var isFiltered = _this.contentTreeGrid.getRoot().isFiltered();
                    isFiltered = true;
                    if (isFiltered) {
                        _this.setFilterPanelRefreshNeeded(true);
                        window.setTimeout(function () {
                            _this.refreshFilter();
                        }, 1000);
                    }
                    return new ContentChangeResult(5 /* DELETE */, deleteResult);
                });
                return promise;
            };
            ContentBrowsePanel.prototype.handleContentPending = function (change, promise, changes) {
                var _this = this;
                promise = promise.then(function () {
                    var pendingResult = _this.contentTreeGrid.findByPaths(change.getContentPaths());
                    return ContentSummaryAndCompareStatusFetcher.fetchByPaths(pendingResult.map(function (el) {
                        return (el.getNodes().length > 0 && el.getNodes()[0].getData()) ? el.getNodes()[0].getData().getContentSummary().getPath() : null;
                    }).filter(function (el) {
                        return el !== null;
                    })).then(function (data) {
                        data.forEach(function (el) {
                            for (var i = 0; i < pendingResult.length; i++) {
                                if (pendingResult[i].getId() === el.getId()) {
                                    pendingResult[i].updateNodeData(el);
                                    _this.updateDetailsPanels(el.getContentId(), el.getCompareStatus());
                                    break;
                                }
                            }
                        });
                        _this.contentTreeGrid.invalidate();
                    });
                });
                return promise;
            };
            ContentBrowsePanel.prototype.handleContentPublished = function (change, promise, changes) {
                var _this = this;
                promise = promise.then(function () {
                    var publishResult = _this.contentTreeGrid.findByPaths(change.getContentPaths());
                    return ContentSummaryAndCompareStatusFetcher.fetchByPaths(publishResult.map(function (el) {
                        return el.getPath();
                    })).then(function (data) {
                        data.forEach(function (el) {
                            for (var i = 0; i < publishResult.length; i++) {
                                if (publishResult[i].getId() === el.getId()) {
                                    new api.content.ContentPublishedEvent(new api.content.ContentId(el.getId())).fire();
                                    publishResult[i].updateNodeData(el);
                                    _this.updateDetailsPanels(el.getContentId(), el.getCompareStatus());
                                    break;
                                }
                            }
                        });
                        _this.contentTreeGrid.invalidate();
                    });
                });
                return promise;
            };
            ContentBrowsePanel.prototype.handleContentSorted = function (change, promise, changes) {
                var _this = this;
                promise = promise.then(function (result) {
                    var sortResult = _this.contentTreeGrid.findByPaths(change.getContentPaths());
                    var nodes = sortResult.map(function (el) {
                        return el.getNodes();
                    });
                    var merged = [];
                    merged = merged.concat.apply(merged, nodes);
                    _this.contentTreeGrid.xSortNodesChildren(merged).then(function () { return _this.contentTreeGrid.invalidate(); });
                });
                return promise;
            };
            ContentBrowsePanel.prototype.handleNewMediaUpload = function (event) {
                var _this = this;
                event.getUploadItems().forEach(function (item) {
                    _this.contentTreeGrid.appendUploadNode(item);
                });
            };
            ContentBrowsePanel.prototype.updateStatisticsPreview = function (el) {
                var _this = this;
                var content = el.getContentSummary();
                var previewItem = this.getBrowseItemPanel().getStatisticsItem();
                var previewItemPath = previewItem.getPath();
                if (!!content && content.getPath().toString() === previewItemPath) {
                    new api.content.page.IsRenderableRequest(el.getContentId()).sendAndParse().then(function (renderable) {
                        var item = new BrowseItem(content).setId(content.getId()).setDisplayName(content.getDisplayName()).setPath(content.getPath().toString()).setIconUrl(new ContentIconUrlResolver().setContent(content).resolve()).setRenderable(renderable);
                        _this.getBrowseItemPanel().setStatisticsItem(item);
                    });
                }
            };
            ContentBrowsePanel.prototype.updateDetailsPanels = function (contentId, status, viewItem) {
                if (viewItem !== undefined) {
                    this.defaultDockedDetailsPanel.setItem(viewItem);
                    this.floatingDetailsPanel.setItem(viewItem);
                    this.mobileContentItemStatisticsPanel.setItem(viewItem);
                }
                this.updateDetailsPanelContentStatus(this.defaultDockedDetailsPanel, contentId, status);
                this.updateDetailsPanelContentStatus(this.floatingDetailsPanel, contentId, status);
                this.updateDetailsPanelContentStatus(this.mobileContentItemStatisticsPanel.getDetailsPanel(), contentId, status);
            };
            ContentBrowsePanel.prototype.updateDetailsPanelContentStatus = function (detailsPanel, contentId, status) {
                if (contentId && detailsPanel.getItem() && contentId.equals(detailsPanel.getItem().getModel().getContentId())) {
                    detailsPanel.setContentStatus(status);
                }
            };
            return ContentBrowsePanel;
        })(api.app.browse.BrowsePanel);
        browse.ContentBrowsePanel = ContentBrowsePanel;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ElementHelper = api.dom.ElementHelper;
        var GridColumnBuilder = api.ui.grid.GridColumnBuilder;
        var TreeGrid = api.ui.treegrid.TreeGrid;
        var TreeGridBuilder = api.ui.treegrid.TreeGridBuilder;
        var DateTimeFormatter = api.ui.treegrid.DateTimeFormatter;
        var TreeGridContextMenu = api.ui.treegrid.TreeGridContextMenu;
        var ContentPath = api.content.ContentPath;
        var ContentSummaryBuilder = api.content.ContentSummaryBuilder;
        var ContentSummaryAndCompareStatusViewer = api.content.ContentSummaryAndCompareStatusViewer;
        var CompareContentRequest = api.content.CompareContentRequest;
        var ContentSummaryAndCompareStatus = api.content.ContentSummaryAndCompareStatus;
        var ContentSummaryAndCompareStatusFetcher = api.content.ContentSummaryAndCompareStatusFetcher;
        var TreeNodesOfContentPath = api.content.TreeNodesOfContentPath;
        var ContentBrowseSearchEvent = app.browse.filter.ContentBrowseSearchEvent;
        var ContentBrowseResetEvent = app.browse.filter.ContentBrowseResetEvent;
        var ContentBrowseRefreshEvent = app.browse.filter.ContentBrowseRefreshEvent;
        var ContentQueryRequest = api.content.ContentQueryRequest;
        var ContentTreeGridActions = app.browse.action.ContentTreeGridActions;
        var CompareStatus = api.content.CompareStatus;
        var ResponsiveRanges = api.ui.responsive.ResponsiveRanges;
        var ContentTreeGrid = (function (_super) {
            __extends(ContentTreeGrid, _super);
            function ContentTreeGrid() {
                var _this = this;
                var nameColumn = new GridColumnBuilder().setName("Name").setId("displayName").setField("contentSummary.displayName").setMinWidth(130).setFormatter(this.nameFormatter).build();
                var compareStatusColumn = new GridColumnBuilder().setName("CompareStatus").setId("compareStatus").setField("compareStatus").setFormatter(this.statusFormatter).setCssClass("status").setMinWidth(75).setMaxWidth(75).build();
                var orderColumn = new GridColumnBuilder().setName("Order").setId("order").setField("contentSummary.order").setCssClass("order").setMinWidth(25).setMaxWidth(40).setFormatter(this.orderFormatter).build();
                var modifiedTimeColumn = new GridColumnBuilder().setName("ModifiedTime").setId("modifiedTime").setField("contentSummary.modifiedTime").setCssClass("modified").setMinWidth(90).setMaxWidth(135).setFormatter(DateTimeFormatter.format).build();
                _super.call(this, new TreeGridBuilder().setColumns([
                    nameColumn,
                    orderColumn,
                    compareStatusColumn,
                    modifiedTimeColumn
                ]).setShowContextMenu(new TreeGridContextMenu(new ContentTreeGridActions(this))).setPartialLoadEnabled(true).setLoadBufferSize(20).prependClasses("content-tree-grid"));
                api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                    if (item.isRangeSizeChanged()) {
                        if (item.isInRangeOrSmaller(ResponsiveRanges._240_360)) {
                            _this.getGrid().setColumns([nameColumn, orderColumn]);
                        }
                        else if (item.isInRangeOrSmaller(ResponsiveRanges._360_540)) {
                            _this.getGrid().setColumns([nameColumn, orderColumn, compareStatusColumn]);
                        }
                        else {
                            if (item.isInRangeOrSmaller(ResponsiveRanges._540_720)) {
                                modifiedTimeColumn.setMaxWidth(90);
                                modifiedTimeColumn.setFormatter(DateTimeFormatter.formatNoTimestamp);
                            }
                            else {
                                modifiedTimeColumn.setMaxWidth(135);
                                modifiedTimeColumn.setFormatter(DateTimeFormatter.format);
                            }
                            _this.getGrid().setColumns([nameColumn, orderColumn, compareStatusColumn, modifiedTimeColumn]);
                        }
                    }
                    else {
                        _this.getGrid().resizeCanvas();
                    }
                });
                this.getGrid().subscribeOnClick(function (event, data) {
                    var elem = new ElementHelper(event.target);
                    if (elem.hasClass("sort-dialog-trigger")) {
                        _this.sortIconClickCallback();
                    }
                });
                this.getGrid().subscribeOnDblClick(function (event, data) {
                    if (_this.isActive()) {
                        var node = _this.getGrid().getDataView().getItem(data.row);
                        if (!!_this.getDataId(node.getData())) {
                            new api.content.EditContentEvent([node.getData().getContentSummary()]).fire();
                        }
                    }
                });
                ContentBrowseSearchEvent.on(function (event) {
                    var contentQueryResult = event.getContentQueryResult();
                    var contentSummaries = contentQueryResult.getContents(), compareRequest = CompareContentRequest.fromContentSummaries(contentSummaries);
                    _this.filterQuery = event.getContentQuery();
                    compareRequest.sendAndParse().then(function (compareResults) {
                        var contents = ContentSummaryAndCompareStatusFetcher.updateCompareStatus(contentSummaries, compareResults);
                        var metadata = contentQueryResult.getMetadata();
                        if (metadata.getTotalHits() > metadata.getHits()) {
                            contents.push(new ContentSummaryAndCompareStatus());
                        }
                        _this.filter(contents);
                        _this.getRoot().getCurrentRoot().setMaxChildren(metadata.getTotalHits());
                        _this.notifyLoaded();
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).done();
                });
                ContentBrowseResetEvent.on(function (event) {
                    _this.resetFilter();
                });
                ContentBrowseRefreshEvent.on(function (event) {
                    _this.notifyLoaded();
                });
            }
            ContentTreeGrid.prototype.orderFormatter = function (row, cell, value, columnDef, node) {
                var wrapper = new api.dom.SpanEl();
                if (!api.util.StringHelper.isBlank(value)) {
                    wrapper.getEl().setTitle(value);
                }
                if (node.getData().getContentSummary()) {
                    var childOrder = node.getData().getContentSummary().getChildOrder();
                    var icon;
                    if (!childOrder.isDefault()) {
                        if (!childOrder.isManual()) {
                            if (childOrder.isDesc()) {
                                icon = new api.dom.DivEl("icon-arrow-up4 sort-dialog-trigger");
                            }
                            else {
                                icon = new api.dom.DivEl("icon-arrow-down4 sort-dialog-trigger");
                            }
                        }
                        else {
                            icon = new api.dom.DivEl("icon-menu3 sort-dialog-trigger");
                        }
                        wrapper.getEl().setInnerHtml(icon.toString());
                    }
                }
                return wrapper.toString();
            };
            ContentTreeGrid.prototype.sortIconClickCallback = function () {
                new app.browse.SortContentEvent(this.getSelectedContentsSummaries()).fire();
            };
            ContentTreeGrid.prototype.getSelectedContentsSummaries = function () {
                return this.getSelectedDataList().map(function (elem) {
                    return elem.getContentSummary();
                });
            };
            ContentTreeGrid.prototype.statusFormatter = function (row, cell, value, columnDef, node) {
                var data = node.getData(), status, statusEl = new api.dom.SpanEl();
                if (!!data.getContentSummary()) {
                    var compareStatus = CompareStatus[CompareStatus[value]];
                    status = api.content.CompareStatusFormatter.formatStatus(compareStatus);
                    if (!!CompareStatus[value]) {
                        statusEl.addClass(CompareStatus[value].toLowerCase().replace("_", "-") || "unknown");
                    }
                    statusEl.getEl().setText(status);
                }
                else if (!!data.getUploadItem()) {
                    status = new api.ui.ProgressBar(data.getUploadItem().getProgress());
                    statusEl.appendChild(status);
                }
                return statusEl.toString();
            };
            ContentTreeGrid.prototype.nameFormatter = function (row, cell, value, columnDef, node) {
                var data = node.getData();
                if (!!data.getContentSummary() || !!data.getUploadItem()) {
                    var viewer = node.getViewer("name");
                    if (!viewer) {
                        viewer = new ContentSummaryAndCompareStatusViewer();
                        node.setViewer("name", viewer);
                    }
                    viewer.setObject(node.getData(), node.calcLevel() > 1);
                    return viewer.toString();
                }
                else {
                    var content = new api.dom.DivEl("children-to-load"), parent = node.getParent();
                    content.setHtml((parent.getMaxChildren() - parent.getChildren().length + 1) + " children left to load.");
                    return content.toString();
                }
            };
            ContentTreeGrid.prototype.fetch = function (node, dataId) {
                return this.fetchById(node.getData().getContentId());
            };
            ContentTreeGrid.prototype.fetchById = function (id) {
                return ContentSummaryAndCompareStatusFetcher.fetch(id);
            };
            ContentTreeGrid.prototype.fetchChildren = function (parentNode) {
                var parentContentId = null;
                if (parentNode) {
                    parentContentId = parentNode.getData() ? parentNode.getData().getContentId() : parentContentId;
                }
                else {
                    parentNode = this.getRoot().getCurrentRoot();
                }
                var from = parentNode.getChildren().length;
                if (from > 0 && !parentNode.getChildren()[from - 1].getData().getContentSummary()) {
                    parentNode.getChildren().pop();
                    from--;
                }
                if (!this.isFiltered() || parentNode != this.getRoot().getCurrentRoot()) {
                    return ContentSummaryAndCompareStatusFetcher.fetchChildren(parentContentId, from, ContentTreeGrid.MAX_FETCH_SIZE).then(function (data) {
                        var contents = parentNode.getChildren().map(function (el) {
                            return el.getData();
                        }).slice(0, from).concat(data.getContents());
                        var meta = data.getMetadata();
                        parentNode.setMaxChildren(meta.getTotalHits());
                        if (from + meta.getHits() < meta.getTotalHits()) {
                            contents.push(new ContentSummaryAndCompareStatus());
                        }
                        return contents;
                    });
                }
                else {
                    this.filterQuery.setFrom(from);
                    this.filterQuery.setSize(ContentTreeGrid.MAX_FETCH_SIZE);
                    return new ContentQueryRequest(this.filterQuery).setExpand(1 /* SUMMARY */).sendAndParse().then(function (contentQueryResult) {
                        var contentSummaries = contentQueryResult.getContents();
                        var compareRequest = CompareContentRequest.fromContentSummaries(contentSummaries);
                        return compareRequest.sendAndParse().then(function (compareResults) {
                            var list = parentNode.getChildren().map(function (el) {
                                return el.getData();
                            }).slice(0, from).concat(ContentSummaryAndCompareStatusFetcher.updateCompareStatus(contentSummaries, compareResults));
                            var meta = contentQueryResult.getMetadata();
                            if (from + meta.getHits() < meta.getTotalHits()) {
                                list.push(new ContentSummaryAndCompareStatus());
                            }
                            parentNode.setMaxChildren(meta.getTotalHits());
                            return list;
                        });
                    });
                }
            };
            ContentTreeGrid.prototype.fetchChildrenIds = function (parentNode) {
                var parentContentId = null;
                if (parentNode) {
                    parentContentId = parentNode.getData() ? parentNode.getData().getContentId() : parentContentId;
                }
                else {
                    parentNode = this.getRoot().getCurrentRoot();
                }
                var size = parentNode.getChildren().length;
                if (size > 0 && !parentNode.getChildren()[size - 1].getData().getContentSummary()) {
                    parentNode.getChildren().pop();
                    size--;
                }
                if (!this.isFiltered() || parentNode != this.getRoot().getCurrentRoot()) {
                    return ContentSummaryAndCompareStatusFetcher.fetchChildrenIds(parentContentId, 0, size + 1).then(function (response) {
                        return response.getContents();
                    });
                }
                else {
                    this.filterQuery.setFrom(0);
                    this.filterQuery.setSize(size + 1);
                    return new ContentQueryRequest(this.filterQuery).setExpand(1 /* SUMMARY */).sendAndParse().then(function (contentQueryResult) {
                        return contentQueryResult.getContents();
                    });
                }
            };
            ContentTreeGrid.prototype.hasChildren = function (data) {
                return data.hasChildren();
            };
            ContentTreeGrid.prototype.getDataId = function (data) {
                return data.getId();
            };
            ContentTreeGrid.prototype.deleteNodes = function (dataList) {
                var _this = this;
                var root = this.getRoot().getCurrentRoot(), node;
                dataList = dataList.filter(function (data) {
                    node = root.findNode(_this.getDataId(data));
                    if (node.getData().getCompareStatus() !== 0 /* NEW */) {
                        node.clearViewers();
                        return false;
                    }
                    return true;
                });
                _super.prototype.deleteNodes.call(this, dataList);
            };
            ContentTreeGrid.prototype.updateContentNode = function (contentId) {
                var root = this.getRoot().getCurrentRoot();
                var treeNode = root.findNode(contentId.toString());
                if (treeNode) {
                    var content = treeNode.getData();
                    this.updateNode(ContentSummaryAndCompareStatus.fromContentSummary(content.getContentSummary()));
                }
            };
            ContentTreeGrid.prototype.appendContentNode = function (contentId, nextToSelection) {
                var _this = this;
                this.fetchById(contentId).then(function (data) {
                    _this.appendNode(data, nextToSelection);
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                });
            };
            ContentTreeGrid.prototype.appendUploadNode = function (item) {
                var _this = this;
                var data = ContentSummaryAndCompareStatus.fromUploadItem(item);
                var parent = this.getRoot().getCurrentSelection()[0];
                this.appendNode(data, false).then(function () {
                    if (parent) {
                        var parentData = parent.getData();
                        var contentSummary = new ContentSummaryBuilder(parentData.getContentSummary()).setHasChildren(true).build();
                        _this.updateNode(parentData.setContentSummary(contentSummary));
                        _this.expandNode(parent);
                    }
                }).done();
                item.onProgress(function (progress) {
                    _this.invalidate();
                });
                item.onUploaded(function (model) {
                    var nodeToRemove = _this.getRoot().getCurrentRoot().findNode(item.getId());
                    if (nodeToRemove) {
                        nodeToRemove.remove();
                        _this.invalidate();
                    }
                    api.notify.showFeedback(data.getContentSummary().getType().toString() + " \"" + item.getName() + "\" created successfully");
                });
                item.onFailed(function () {
                    _this.deleteNode(data);
                });
            };
            ContentTreeGrid.prototype.refreshNodeData = function (parentNode) {
                var _this = this;
                return ContentSummaryAndCompareStatusFetcher.fetch(parentNode.getData().getContentId()).then(function (content) {
                    parentNode.setData(content);
                    _this.refreshNode(parentNode);
                    return parentNode;
                });
            };
            ContentTreeGrid.prototype.sortNodeChildren = function (node) {
                var _this = this;
                var rootNode = this.getRoot().getCurrentRoot();
                if (node != rootNode) {
                    if (node.hasChildren()) {
                        node.setChildren([]);
                        node.setMaxChildren(0);
                        this.fetchChildren(node).then(function (dataList) {
                            var parentNode = _this.getRoot().getCurrentRoot().findNode(node.getDataId());
                            parentNode.setChildren(_this.dataToTreeNodes(dataList, parentNode));
                            var rootList = _this.getRoot().getCurrentRoot().treeToList();
                            _this.initData(rootList);
                        }).catch(function (reason) {
                            api.DefaultErrorHandler.handle(reason);
                        }).done();
                    }
                }
            };
            ContentTreeGrid.prototype.findByPaths = function (paths, useParent) {
                if (useParent === void 0) { useParent = false; }
                var root = this.getRoot().getDefaultRoot().treeToList(false, false), filter = this.getRoot().getFilteredRoot().treeToList(false, false), all = root.concat(filter), result = [];
                for (var i = 0; i < paths.length; i++) {
                    var node = useParent ? new TreeNodesOfContentPath(paths[i].getParentPath(), paths[i]) : new TreeNodesOfContentPath(paths[i]);
                    if (useParent && node.getPath().isRoot()) {
                        node.getNodes().push(this.getRoot().getDefaultRoot());
                        if (this.isFiltered()) {
                            node.getNodes().push(this.getRoot().getFilteredRoot());
                        }
                    }
                    else {
                        for (var j = 0; j < all.length; j++) {
                            var path = (all[j].getData() && all[j].getData().getContentSummary()) ? all[j].getData().getContentSummary().getPath() : null;
                            if (path && path.equals(node.getPath())) {
                                node.getNodes().push(all[j]);
                            }
                        }
                    }
                    if (node.hasNodes()) {
                        result.push(node);
                    }
                }
                return result;
            };
            ContentTreeGrid.prototype.xAppendContentNode = function (relationship, update) {
                var _this = this;
                if (update === void 0) { update = true; }
                var appendedNode = this.dataToTreeNode(relationship.getData(), relationship.getNode()), data = relationship.getNode().getData();
                return this.fetchChildrenIds(relationship.getNode()).then(function (result) {
                    var map = result.map(function (el) {
                        return el.getId();
                    });
                    var index = map.indexOf(appendedNode.getData().getId());
                    if (!relationship.getNode().hasParent() || (data && relationship.getNode().hasChildren()) || (data && !relationship.getNode().hasChildren() && !data.getContentSummary().hasChildren())) {
                        relationship.getNode().insertChild(appendedNode, index);
                    }
                    if (data && !data.getContentSummary().hasChildren()) {
                        data.setContentSummary(new ContentSummaryBuilder(data.getContentSummary()).setHasChildren(true).build());
                    }
                    relationship.getNode().clearViewers();
                    if (update) {
                        _this.initAndRender();
                    }
                    return appendedNode;
                });
            };
            ContentTreeGrid.prototype.xAppendContentNodes = function (relationships, update) {
                var _this = this;
                if (update === void 0) { update = true; }
                var nodes = [];
                var parallelPromises = [];
                this.xUpdateNodesData(relationships.map(function (el) {
                    return el.getNode();
                }));
                relationships.forEach(function (relationship) {
                    parallelPromises.push(_this.xAppendContentNode(relationship, false));
                });
                return wemQ.allSettled(parallelPromises).then(function (results) {
                    var rootList = _this.getRoot().getCurrentRoot().treeToList();
                    _this.initData(rootList);
                    _this.invalidate();
                    return results;
                });
            };
            ContentTreeGrid.prototype.xPlaceContentNode = function (parent, child) {
                return this.fetchChildrenIds(parent).then(function (result) {
                    var map = result.map(function (el) {
                        return el.getId();
                    });
                    var index = map.indexOf(child.getData().getId());
                    if (!parent.hasParent() || (child.getData() && parent.hasChildren()) || (child.getData() && !parent.hasChildren() && !child.getData().getContentSummary().hasChildren())) {
                        parent.moveChild(child, index);
                    }
                    child.clearViewers();
                    return child;
                });
            };
            ContentTreeGrid.prototype.xPlaceContentNodes = function (results) {
                var _this = this;
                var parallelPromises = [];
                var nodes = results.map(function (el) {
                    return el.getNodes();
                });
                var merged = [];
                merged = merged.concat.apply(merged, nodes);
                merged.forEach(function (node) {
                    parallelPromises.push(_this.xPlaceContentNode(node.getParent(), node));
                });
                return wemQ.allSettled(parallelPromises).then(function (results) {
                    var rootList = _this.getRoot().getCurrentRoot().treeToList();
                    _this.initData(rootList);
                    _this.invalidate();
                    return results;
                });
            };
            ContentTreeGrid.prototype.xDeleteContentNode = function (node, update) {
                if (update === void 0) { update = true; }
                var parentNode = node.getParent();
                node.remove();
                var data = !!parentNode ? parentNode.getData() : null;
                if (data && !parentNode.hasChildren() && data.getContentSummary().hasChildren()) {
                    data.setContentSummary(new ContentSummaryBuilder(data.getContentSummary()).setHasChildren(false).build());
                }
                if (update) {
                    this.initAndRender();
                }
                return parentNode;
            };
            ContentTreeGrid.prototype.xDeleteContentNodes = function (nodes, update) {
                var _this = this;
                if (update === void 0) { update = true; }
                nodes.forEach(function (node) {
                    _this.xDeleteContentNode(node, false);
                });
                if (update) {
                    this.initAndRender();
                }
            };
            ContentTreeGrid.prototype.xPopulateWithChildren = function (source, dest) {
                dest.setChildren(source.getChildren());
                dest.setExpanded(source.isExpanded());
                if (dest.getData() && dest.getData().getContentSummary()) {
                    dest.getData().setContentSummary(new ContentSummaryBuilder(dest.getData().getContentSummary()).setHasChildren(dest.hasChildren()).build());
                    this.xUpdatePathsInChildren(dest);
                }
                dest.clearViewers();
            };
            ContentTreeGrid.prototype.xUpdatePathsInChildren = function (node) {
                var _this = this;
                node.getChildren().forEach(function (child) {
                    var nodeSummary = node.getData() ? node.getData().getContentSummary() : null, childSummary = child.getData() ? child.getData().getContentSummary() : null;
                    if (nodeSummary && childSummary) {
                        var path = ContentPath.fromParent(nodeSummary.getPath(), childSummary.getPath().getName());
                        child.getData().setContentSummary(new ContentSummaryBuilder(childSummary).setPath(path).build());
                        child.clearViewers();
                        _this.xUpdatePathsInChildren(child);
                    }
                });
            };
            ContentTreeGrid.prototype.xUpdateNodesData = function (nodes) {
                var _this = this;
                nodes = this.xFilterParentNodes(nodes);
                var parallelPromises = [];
                nodes.forEach(function (node) {
                    if (!node.hasChildren()) {
                        if (!!node.getData()) {
                            parallelPromises.push(new api.content.GetContentByIdRequest(node.getData().getContentSummary().getContentId()).sendAndParse().then(function (content) {
                                node.getData().setContentSummary(content);
                            }));
                        }
                    }
                });
                wemQ.all(parallelPromises).spread(function () {
                    _this.triggerSelectionChangedListeners();
                    return wemQ(null);
                }).catch(function (reason) { return api.DefaultErrorHandler.handle(reason); }).done();
                return nodes;
            };
            ContentTreeGrid.prototype.xFilterParentNodes = function (nodes) {
                return nodes.filter(function (node, index) {
                    var result = true;
                    var path = node.getData() && node.getData().getContentSummary() ? node.getData().getContentSummary().getPath() : null;
                    if (path) {
                        for (var i = 0; i < nodes.length; i++) {
                            if (index === i) {
                                continue;
                            }
                            var nodePath = nodes[i].getData() && nodes[i].getData().getContentSummary() ? nodes[i].getData().getContentSummary().getPath() : null;
                            if (nodePath && (path.isChildOf(nodePath) || path.toString() === nodePath.toString())) {
                                return false;
                            }
                        }
                    }
                    return result;
                });
            };
            ContentTreeGrid.prototype.xSortNodesChildren = function (nodes) {
                var _this = this;
                var parallelPromises = [];
                nodes.sort(function (a, b) {
                    return a.getDataId().localeCompare(b.getDataId());
                });
                var groups = [], group = [];
                groups.push(group);
                for (var i = 0; i < nodes.length; i++) {
                    if (!!group[group.length - 1] && nodes[i].getDataId() !== group[group.length - 1].getDataId()) {
                        group = [];
                        groups.push(group);
                    }
                    group.push(nodes[i]);
                }
                groups.forEach(function (grp) {
                    if (grp.length > 0) {
                        parallelPromises.push(_this.updateNodes(grp[0].getData()).then(function () {
                            var hasChildren = grp[0].hasChildren();
                            grp[0].setChildren([]);
                            return _this.fetchChildren(grp[0]).then(function (dataList) {
                                grp.forEach(function (el) {
                                    if (hasChildren) {
                                        el.setChildren(_this.dataToTreeNodes(dataList, el));
                                    }
                                });
                            }).catch(function (reason) {
                                api.DefaultErrorHandler.handle(reason);
                            });
                        }).then(function () {
                            var rootList = _this.getRoot().getCurrentRoot().treeToList();
                            _this.initData(rootList);
                        }));
                    }
                });
                return wemQ.all(parallelPromises).spread(function () {
                    return wemQ(null);
                }).catch(function (reason) { return api.DefaultErrorHandler.handle(reason); });
            };
            ContentTreeGrid.MAX_FETCH_SIZE = 10;
            return ContentTreeGrid;
        })(TreeGrid);
        browse.ContentTreeGrid = ContentTreeGrid;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var SortContentEvent = (function (_super) {
            __extends(SortContentEvent, _super);
            function SortContentEvent() {
                _super.apply(this, arguments);
            }
            SortContentEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            SortContentEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return SortContentEvent;
        })(browse.BaseContentModelEvent);
        browse.SortContentEvent = SortContentEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var OpenSortDialogEvent = (function (_super) {
            __extends(OpenSortDialogEvent, _super);
            function OpenSortDialogEvent(content) {
                _super.call(this);
                this.content = content;
            }
            OpenSortDialogEvent.prototype.getContent = function () {
                return this.content;
            };
            OpenSortDialogEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            OpenSortDialogEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return OpenSortDialogEvent;
        })(api.event.Event);
        browse.OpenSortDialogEvent = OpenSortDialogEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var OpenMoveDialogEvent = (function (_super) {
            __extends(OpenMoveDialogEvent, _super);
            function OpenMoveDialogEvent(content) {
                _super.call(this);
                this.content = content;
            }
            OpenMoveDialogEvent.prototype.getContentSummaries = function () {
                return this.content;
            };
            OpenMoveDialogEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            OpenMoveDialogEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return OpenMoveDialogEvent;
        })(api.event.Event);
        browse.OpenMoveDialogEvent = OpenMoveDialogEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var SaveSortedContentAction = browse.action.SaveSortedContentAction;
        var TabMenuItemBuilder = api.ui.tab.TabMenuItemBuilder;
        var SortContentDialog = (function (_super) {
            __extends(SortContentDialog, _super);
            function SortContentDialog() {
                var _this = this;
                _super.call(this, {
                    title: new api.ui.dialog.ModalDialogHeader("Sort items")
                });
                this.initTabMenu();
                this.initSortContentMenu();
                this.getEl().addClass("sort-content-dialog");
                this.initSortContentGrid();
                this.initGridDragHandler();
                this.populateContentPanel();
                this.initSaveButtonWithAction();
                browse.OpenSortDialogEvent.on(function (event) {
                    _this.handleOpenSortDialogEvent(event);
                });
                this.addCancelButtonToBottom();
            }
            SortContentDialog.prototype.open = function () {
                if (!this.isOpen) {
                    if (this.contentGrid.getGrid().getDataView().getLength() > 0) {
                        this.contentGrid.getGrid().getEl().setHeightPx(45);
                    }
                    this.contentGrid.getGrid().resizeCanvas();
                    _super.prototype.open.call(this);
                    this.isOpen = true;
                }
            };
            SortContentDialog.prototype.show = function () {
                api.dom.Body.get().appendChild(this);
                _super.prototype.show.call(this);
            };
            SortContentDialog.prototype.close = function () {
                this.remove();
                _super.prototype.close.call(this);
                this.isOpen = false;
                this.contentGrid.setChildOrder(null);
                this.gridDragHandler.clearContentMovements();
            };
            SortContentDialog.prototype.getContent = function () {
                return this.parentContent;
            };
            SortContentDialog.prototype.initSortContentGrid = function () {
                var _this = this;
                this.contentGrid = new app.browse.SortContentTreeGrid();
                this.contentGrid.getEl().addClass("sort-content-grid");
                this.contentGrid.onLoaded(function () {
                    _this.contentGrid.render(true);
                    if (_this.contentGrid.getContentId()) {
                        _this.open();
                    }
                });
            };
            SortContentDialog.prototype.initGridDragHandler = function () {
                var _this = this;
                this.gridDragHandler = new browse.ContentGridDragHandler(this.contentGrid);
                this.gridDragHandler.onPositionChanged(function () {
                    _this.sortContentMenu.selectManualSortingItem();
                });
            };
            SortContentDialog.prototype.initTabMenu = function () {
                var menu = new api.ui.tab.TabMenu();
                var tabMenuItem = new TabMenuItemBuilder().setLabel("(sorting type)").build();
                tabMenuItem.setActive(true);
                menu.addNavigationItem(tabMenuItem);
                menu.selectNavigationItem(0);
                menu.show();
            };
            SortContentDialog.prototype.initSortContentMenu = function () {
                var _this = this;
                this.sortContentMenu = new browse.SortContentTabMenu();
                this.sortContentMenu.show();
                this.appendChildToTitle(this.sortContentMenu);
                this.sortContentMenu.onSortOrderChanged(function () {
                    _this.handleOnSortOrderChangedEvent();
                });
            };
            SortContentDialog.prototype.initSaveButtonWithAction = function () {
                var _this = this;
                this.sortAction = new SaveSortedContentAction(this);
                this.saveButton = this.addAction(this.sortAction);
                this.saveButton.addClass("save-button");
                this.sortAction.onExecuted(function () {
                    _this.handleSortAction();
                });
            };
            SortContentDialog.prototype.populateContentPanel = function () {
                var header = new api.dom.H6El();
                header.setHtml("Sort content by selecting default sort above, or drag and drop for manual sorting");
                this.appendChildToContentPanel(header);
                this.appendChildToContentPanel(this.contentGrid);
            };
            SortContentDialog.prototype.handleSortAction = function () {
                var _this = this;
                if (this.curChildOrder.equals(this.parentContent.getChildOrder()) && !this.curChildOrder.isManual()) {
                    this.close();
                }
                else {
                    this.showLoadingSpinner();
                    if (this.curChildOrder.isManual()) {
                        this.setManualReorder(this.hasChangedPrevChildOrder() ? this.prevChildOrder : null, this.gridDragHandler.getContentMovements()).done(function () { return _this.onAfterSetOrder(); });
                    }
                    else {
                        this.setContentChildOrder(this.curChildOrder).done(function () { return _this.onAfterSetOrder(); });
                    }
                }
            };
            SortContentDialog.prototype.handleOpenSortDialogEvent = function (event) {
                var _this = this;
                this.parentContent = event.getContent();
                this.curChildOrder = this.parentContent.getChildOrder();
                this.prevChildOrder = undefined;
                this.sortContentMenu.selectNavigationItemByOrder(this.curChildOrder);
                api.content.ContentSummaryAndCompareStatusFetcher.fetch(this.parentContent.getContentId()).done(function (response) {
                    _this.contentGrid.reload(response);
                    if (!response.hasChildren()) {
                        _this.contentGrid.getEl().setAttribute("data-content", event.getContent().getPath().toString());
                        _this.contentGrid.addClass("no-content");
                    }
                    else {
                        _this.contentGrid.removeClass("no-content");
                        _this.contentGrid.getEl().removeAttribute("data-content");
                    }
                });
            };
            SortContentDialog.prototype.handleOnSortOrderChangedEvent = function () {
                var _this = this;
                var newOrder = this.sortContentMenu.getSelectedNavigationItem().getChildOrder();
                if (!this.curChildOrder.equals(newOrder)) {
                    if (!newOrder.isManual()) {
                        this.curChildOrder = newOrder;
                        this.contentGrid.setChildOrder(this.curChildOrder);
                        api.content.ContentSummaryAndCompareStatusFetcher.fetch(this.parentContent.getContentId()).done(function (response) {
                            _this.contentGrid.reload(response);
                        });
                        this.gridDragHandler.clearContentMovements();
                    }
                    else {
                        this.prevChildOrder = this.curChildOrder;
                        this.curChildOrder = newOrder;
                        this.contentGrid.setChildOrder(this.curChildOrder);
                    }
                }
            };
            SortContentDialog.prototype.onAfterSetOrder = function () {
                new api.content.ContentChildOrderUpdatedEvent(this.parentContent.getContentId()).fire();
                this.hideLoadingSpinner();
                this.close();
            };
            SortContentDialog.prototype.hasChangedPrevChildOrder = function () {
                return this.prevChildOrder && !this.prevChildOrder.equals(this.parentContent.getChildOrder());
            };
            SortContentDialog.prototype.showLoadingSpinner = function () {
                this.saveButton.addClass("spinner");
            };
            SortContentDialog.prototype.hideLoadingSpinner = function () {
                this.saveButton.removeClass("spinner");
            };
            SortContentDialog.prototype.setContentChildOrder = function (order, silent) {
                if (silent === void 0) { silent = false; }
                return new api.content.OrderContentRequest().setSilent(silent).setContentId(this.parentContent.getContentId()).setChildOrder(order).sendAndParse();
            };
            SortContentDialog.prototype.setManualReorder = function (order, movements, silent) {
                if (silent === void 0) { silent = false; }
                return new api.content.OrderChildContentRequest().setSilent(silent).setManualOrder(true).setContentId(this.parentContent.getContentId()).setChildOrder(order).setContentMovements(movements).sendAndParse();
            };
            return SortContentDialog;
        })(api.ui.dialog.ModalDialog);
        browse.SortContentDialog = SortContentDialog;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ContentPath = api.content.ContentPath;
        var GetContentTypeByNameRequest = api.schema.content.GetContentTypeByNameRequest;
        var ContentIds = api.content.ContentIds;
        var MoveContentDialog = (function (_super) {
            __extends(MoveContentDialog, _super);
            function MoveContentDialog() {
                var _this = this;
                _super.call(this, {
                    title: new api.ui.dialog.ModalDialogHeader("Move item with children")
                });
                this.getEl().addClass("move-content-dialog");
                this.contentPath = new api.dom.H6El().addClass("content-path");
                var descMessage = new api.dom.H6El().addClass("desc-message").setHtml("Moves selected items with all children and current permissions to selected destination");
                this.appendChildToContentPanel(this.contentPath);
                this.appendChildToContentPanel(descMessage);
                this.contentComboBox = new api.content.ContentMoveComboBox();
                this.contentComboBox.addClass("content-selector");
                this.appendChildToContentPanel(this.contentComboBox);
                this.initializeActions();
                this.contentMoveMask = new api.ui.mask.LoadMask(this);
                this.appendChildToContentPanel(this.contentMoveMask);
                browse.OpenMoveDialogEvent.on(function (event) {
                    _this.movedContentSummaries = event.getContentSummaries();
                    _this.contentComboBox.clearCombobox();
                    if (event.getContentSummaries().length == 1) {
                        var contentToMove = event.getContentSummaries()[0];
                        new GetContentTypeByNameRequest(contentToMove.getType()).sendAndParse().then(function (contentType) {
                            _this.contentComboBox.setFilterContentPath(contentToMove.getPath());
                            _this.contentComboBox.setFilterSourceContentType(contentType);
                            _this.contentPath.setHtml(contentToMove.getPath().toString());
                            _this.open();
                        }).catch(function (reason) {
                            api.notify.showError(reason.getMessage());
                        }).done();
                    }
                    else {
                        _this.contentComboBox.setFilterContentPath(null);
                        _this.contentPath.setHtml("");
                        _this.open();
                    }
                });
                this.addCancelButtonToBottom();
            }
            MoveContentDialog.prototype.initializeActions = function () {
                var _this = this;
                this.addAction(new api.ui.Action("Move", "").onExecuted(function () {
                    _this.contentMoveMask.show();
                    var parentContent = _this.getParentContent();
                    _this.moveContent(parentContent);
                }));
            };
            MoveContentDialog.prototype.moveContent = function (parentContent) {
                var _this = this;
                var parentRoot = (!!parentContent) ? parentContent.getPath() : ContentPath.ROOT;
                var contentIds = ContentIds.create().fromContentIds(this.movedContentSummaries.map(function (summary) { return summary.getContentId(); })).build();
                new api.content.MoveContentRequest(contentIds, parentRoot).sendAndParse().then(function (response) {
                    if (parentContent) {
                        _this.contentComboBox.deselect(parentContent);
                    }
                    _this.contentMoveMask.hide();
                    if (response.getMoved().length > 0) {
                        if (response.getMoved().length > 1) {
                            api.notify.showFeedback(response.getMoved().length + ' items moved');
                        }
                        else {
                            api.notify.showFeedback("\"" + response.getMoved()[0] + '\" moved');
                        }
                    }
                    response.getMoveFailures().forEach(function (failure) {
                        api.notify.showWarning(failure.getReason());
                    });
                    _this.close();
                }).catch(function (reason) {
                    api.notify.showWarning(reason.getMessage());
                    _this.close();
                    _this.contentComboBox.deselect(_this.getParentContent());
                }).done();
            };
            MoveContentDialog.prototype.getParentContent = function () {
                return this.contentComboBox.getSelectedDisplayValues()[0];
            };
            MoveContentDialog.prototype.open = function () {
                _super.prototype.open.call(this);
            };
            MoveContentDialog.prototype.show = function () {
                api.dom.Body.get().appendChild(this);
                _super.prototype.show.call(this);
                this.contentComboBox.giveFocus();
            };
            MoveContentDialog.prototype.close = function () {
                _super.prototype.close.call(this);
            };
            return MoveContentDialog;
        })(api.ui.dialog.ModalDialog);
        browse.MoveContentDialog = MoveContentDialog;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var GridColumnBuilder = api.ui.grid.GridColumnBuilder;
        var TreeGrid = api.ui.treegrid.TreeGrid;
        var TreeGridBuilder = api.ui.treegrid.TreeGridBuilder;
        var DateTimeFormatter = api.ui.treegrid.DateTimeFormatter;
        var ContentSummaryViewer = api.content.ContentSummaryViewer;
        var ContentSummaryAndCompareStatus = api.content.ContentSummaryAndCompareStatus;
        var ContentSummaryAndCompareStatusFetcher = api.content.ContentSummaryAndCompareStatusFetcher;
        var CompareStatus = api.content.CompareStatus;
        var SortContentTreeGrid = (function (_super) {
            __extends(SortContentTreeGrid, _super);
            function SortContentTreeGrid() {
                var nameColumn = new GridColumnBuilder().setName("Name").setId("displayName").setField("contentSummary.displayName").setMinWidth(130).setFormatter(this.nameFormatter).setBehavior("selectAndMove").build();
                var modifiedTimeColumn = new GridColumnBuilder().setName("ModifiedTime").setId("modifiedTime").setField("contentSummary.modifiedTime").setCssClass("modified").setMinWidth(150).setMaxWidth(170).setFormatter(DateTimeFormatter.format).setBehavior("selectAndMove").build();
                _super.call(this, new TreeGridBuilder().setColumns([
                    nameColumn,
                    modifiedTimeColumn
                ]).setPartialLoadEnabled(true).setCheckableRows(false).setShowToolbar(false).setDragAndDrop(true).disableMultipleSelection(true).prependClasses("content-tree-grid").setSelectedCellCssClass("selected-sort-row"));
            }
            SortContentTreeGrid.prototype.statusFormatter = function (row, cell, value, columnDef, node) {
                if (!node.getData().getContentSummary()) {
                    return "";
                }
                var compareLabel = api.content.CompareStatus[value];
                var compareStatus = CompareStatus[compareLabel];
                switch (compareStatus) {
                    case 0 /* NEW */:
                        return "Offline";
                        break;
                    case 2 /* NEWER */:
                        return "Modified";
                        break;
                    case 3 /* OLDER */:
                        return "Behind";
                        break;
                    case 10 /* UNKNOWN */:
                        return "Unknown";
                        break;
                    case 4 /* PENDING_DELETE */:
                        return "Deleted";
                        break;
                    case 6 /* EQUAL */:
                        return "Online";
                        break;
                    default:
                        return "Unknown";
                }
            };
            SortContentTreeGrid.prototype.dragFormatter = function (row, cell, value, columnDef, node) {
                var wrapper = new api.dom.SpanEl();
                if (!api.util.StringHelper.isBlank(value)) {
                    wrapper.getEl().setTitle(value);
                }
                var icon = new api.dom.DivEl("icon-menu3 drag-icon");
                wrapper.getEl().setInnerHtml(icon.toString());
                return wrapper.toString();
            };
            SortContentTreeGrid.prototype.nameFormatter = function (row, cell, value, columnDef, node) {
                if (!!node.getData().getContentSummary()) {
                    var viewer = node.getViewer("name");
                    if (!viewer) {
                        viewer = new ContentSummaryViewer();
                        viewer.setObject(node.getData().getContentSummary(), node.calcLevel() > 1);
                        node.setViewer("name", viewer);
                    }
                    return viewer.toString();
                }
                else {
                    var content = new api.dom.DivEl("children-to-load"), parent = node.getParent();
                    content.setHtml((parent.getMaxChildren() - parent.getChildren().length + 1) + " children left to load.");
                    return content.toString();
                }
            };
            SortContentTreeGrid.prototype.sortNodeChildren = function (node) {
                this.initData(this.getRoot().getCurrentRoot().treeToList());
            };
            SortContentTreeGrid.prototype.fetchChildren = function () {
                var parentContentId;
                var parentNode = this.getRoot().getCurrentRoot();
                if (parentNode.getData()) {
                    parentContentId = parentNode.getData().getContentSummary().getContentId();
                    this.contentId = parentContentId;
                    parentNode.setData(null);
                }
                else {
                    parentContentId = this.contentId;
                }
                var from = parentNode.getChildren().length;
                if (from > 0 && !parentNode.getChildren()[from - 1].getData().getContentSummary()) {
                    parentNode.getChildren().pop();
                    from--;
                }
                return ContentSummaryAndCompareStatusFetcher.fetchChildren(parentContentId, from, SortContentTreeGrid.MAX_FETCH_SIZE, this.curChildOrder).then(function (data) {
                    var contents = parentNode.getChildren().map(function (el) {
                        return el.getData();
                    }).slice(0, from).concat(data.getContents());
                    var meta = data.getMetadata();
                    parentNode.setMaxChildren(meta.getTotalHits());
                    if (from + meta.getHits() < meta.getTotalHits()) {
                        contents.push(new ContentSummaryAndCompareStatus());
                    }
                    return contents;
                });
            };
            SortContentTreeGrid.prototype.hasChildren = function (data) {
                return data.hasChildren();
            };
            SortContentTreeGrid.prototype.getDataId = function (data) {
                return data.getId();
            };
            SortContentTreeGrid.prototype.getContentId = function () {
                return this.contentId;
            };
            SortContentTreeGrid.prototype.getChildOrder = function () {
                return this.curChildOrder;
            };
            SortContentTreeGrid.prototype.setChildOrder = function (value) {
                this.curChildOrder = value;
            };
            SortContentTreeGrid.MAX_FETCH_SIZE = 30;
            return SortContentTreeGrid;
        })(TreeGrid);
        browse.SortContentTreeGrid = SortContentTreeGrid;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var DropdownHandle = api.ui.selector.DropdownHandle;
        var SortContentTabMenu = (function (_super) {
            __extends(SortContentTabMenu, _super);
            function SortContentTabMenu() {
                var _this = this;
                _super.call(this, "sort-tab-menu");
                this.sortOrderChangedListeners = [];
                this.navigationItems = new browse.SortContentTabMenuItems();
                this.addNavigationItems(this.navigationItems.getAllItems());
                this.selectNavigationItem(0);
                this.dropdownHandle = new DropdownHandle();
                this.appendChild(this.dropdownHandle);
                this.dropdownHandle.up();
                this.dropdownHandle.onClicked(function (event) {
                    if (_this.isMenuVisible()) {
                        _this.hideMenu();
                    }
                    else {
                        _this.showMenu();
                    }
                });
            }
            SortContentTabMenu.prototype.hideMenu = function () {
                _super.prototype.hideMenu.call(this);
                this.dropdownHandle.up();
            };
            SortContentTabMenu.prototype.showMenu = function () {
                _super.prototype.showMenu.call(this);
                this.dropdownHandle.down();
            };
            SortContentTabMenu.prototype.selectNavigationItem = function (tabIndex) {
                _super.prototype.selectNavigationItem.call(this, tabIndex);
                this.notifySortOrderChanged();
            };
            SortContentTabMenu.prototype.getSelectedNavigationItem = function () {
                return _super.prototype.getSelectedNavigationItem.call(this);
            };
            SortContentTabMenu.prototype.getSortMenuNavigationItems = function () {
                return this.navigationItems;
            };
            SortContentTabMenu.prototype.addNavigationItems = function (items) {
                var _this = this;
                if (items) {
                    items.forEach(function (item) {
                        _this.addNavigationItem(item);
                    });
                }
            };
            SortContentTabMenu.prototype.selectNavigationItemByOrder = function (order) {
                var items = this.navigationItems.getAllItems();
                for (var key in items) {
                    if (items[key].getChildOrder().equals(order)) {
                        var item = items.splice(key, 1)[0];
                        this.removeNavigationItems();
                        this.addNavigationItem(item);
                        this.addNavigationItems(items);
                        this.selectNavigationItem(0);
                        break;
                    }
                }
            };
            SortContentTabMenu.prototype.selectManualSortingItem = function () {
                this.selectNavigationItemByOrder(this.navigationItems.SORT_MANUAL_ITEM.getChildOrder());
            };
            SortContentTabMenu.prototype.onSortOrderChanged = function (listener) {
                this.sortOrderChangedListeners.push(listener);
            };
            SortContentTabMenu.prototype.unSortOrderChanged = function (listener) {
                this.sortOrderChangedListeners = this.sortOrderChangedListeners.filter(function (currentListener) {
                    return listener != currentListener;
                });
            };
            SortContentTabMenu.prototype.notifySortOrderChanged = function () {
                var _this = this;
                this.sortOrderChangedListeners.forEach(function (listener) {
                    listener.call(_this);
                });
            };
            return SortContentTabMenu;
        })(api.ui.tab.TabMenu);
        browse.SortContentTabMenu = SortContentTabMenu;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var TabMenuItemBuilder = api.ui.tab.TabMenuItemBuilder;
        var SortContentTabMenuItem = (function (_super) {
            __extends(SortContentTabMenuItem, _super);
            function SortContentTabMenuItem(builder) {
                _super.call(this, new TabMenuItemBuilder().setLabel(builder.label));
                this.childOrder = builder.childOrder;
            }
            SortContentTabMenuItem.prototype.getChildOrder = function () {
                return this.childOrder;
            };
            return SortContentTabMenuItem;
        })(api.ui.tab.TabMenuItem);
        browse.SortContentTabMenuItem = SortContentTabMenuItem;
        var SortContentTabMenuItemBuilder = (function () {
            function SortContentTabMenuItemBuilder() {
            }
            SortContentTabMenuItemBuilder.prototype.setLabel = function (label) {
                this.label = label;
                return this;
            };
            SortContentTabMenuItemBuilder.prototype.setChildOrder = function (value) {
                this.childOrder = value;
                return this;
            };
            SortContentTabMenuItemBuilder.prototype.build = function () {
                return new SortContentTabMenuItem(this);
            };
            return SortContentTabMenuItemBuilder;
        })();
        browse.SortContentTabMenuItemBuilder = SortContentTabMenuItemBuilder;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ChildOrder = api.content.ChildOrder;
        var FieldOrderExprBuilder = api.content.FieldOrderExprBuilder;
        var SortContentTabMenuItems = (function () {
            function SortContentTabMenuItems() {
                this.items = [];
                var order = new ChildOrder();
                order.addOrderExpr(new FieldOrderExprBuilder().setFieldName(api.query.QueryField.DISPLAY_NAME).setDirection(ChildOrder.ASC_ORDER_DIRECTION_VALUE).build());
                this.SORT_ASC_DISPALAY_NAME_ITEM = new browse.SortContentTabMenuItemBuilder().setLabel("DisplayName - Ascending").setChildOrder(order).build();
                order = new ChildOrder();
                order.addOrderExpr(new FieldOrderExprBuilder().setFieldName(api.query.QueryField.DISPLAY_NAME).setDirection(ChildOrder.DESC_ORDER_DIRECTION_VALUE).build());
                this.SORT_DESC_DISPALAY_NAME_ITEM = new browse.SortContentTabMenuItemBuilder().setLabel("DisplayName - Descending").setChildOrder(order).build();
                order = new ChildOrder();
                order.addOrderExpr(new FieldOrderExprBuilder().setFieldName(api.query.QueryField.MODIFIED_TIME).setDirection(ChildOrder.ASC_ORDER_DIRECTION_VALUE).build());
                this.SORT_ASC_MODIFIED_ITEM = new browse.SortContentTabMenuItemBuilder().setLabel("Modified - Ascending").setChildOrder(order).build();
                order = new ChildOrder();
                order.addOrderExpr(new FieldOrderExprBuilder().setFieldName(api.query.QueryField.MODIFIED_TIME).setDirection(ChildOrder.DESC_ORDER_DIRECTION_VALUE).build());
                this.SORT_DESC_MODIFIED_ITEM = new browse.SortContentTabMenuItemBuilder().setLabel("Modified - Descending (default)").setChildOrder(order).build();
                order = new ChildOrder();
                order.addOrderExpr(new FieldOrderExprBuilder().setFieldName(api.query.QueryField.MANUAL_ORDER_VALUE).setDirection(ChildOrder.DESC_ORDER_DIRECTION_VALUE).build());
                this.SORT_MANUAL_ITEM = new browse.SortContentTabMenuItemBuilder().setLabel("Manually Sorted").setChildOrder(order).build();
                this.items.push(this.SORT_ASC_DISPALAY_NAME_ITEM, this.SORT_DESC_DISPALAY_NAME_ITEM, this.SORT_ASC_MODIFIED_ITEM, this.SORT_DESC_MODIFIED_ITEM, this.SORT_MANUAL_ITEM);
            }
            SortContentTabMenuItems.prototype.getAllItems = function () {
                return this.items.slice(0, this.items.length);
            };
            return SortContentTabMenuItems;
        })();
        browse.SortContentTabMenuItems = SortContentTabMenuItems;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var SaveSortedContentEvent = (function (_super) {
            __extends(SaveSortedContentEvent, _super);
            function SaveSortedContentEvent(content) {
                this.content = content;
                _super.call(this);
            }
            SaveSortedContentEvent.prototype.getContent = function () {
                return this.content;
            };
            SaveSortedContentEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            SaveSortedContentEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return SaveSortedContentEvent;
        })(api.event.Event);
        browse.SaveSortedContentEvent = SaveSortedContentEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var GridDragHandler = api.ui.grid.GridDragHandler;
        var OrderChildMovement = api.content.OrderChildMovement;
        var OrderChildMovements = api.content.OrderChildMovements;
        var ContentGridDragHandler = (function (_super) {
            __extends(ContentGridDragHandler, _super);
            function ContentGridDragHandler(treeGrid) {
                _super.call(this, treeGrid);
                this.movements = new OrderChildMovements();
            }
            ContentGridDragHandler.prototype.getContentMovements = function () {
                return this.movements;
            };
            ContentGridDragHandler.prototype.clearContentMovements = function () {
                this.movements = new OrderChildMovements();
            };
            ContentGridDragHandler.prototype.handleMovements = function (rowDataId, moveBeforeRowDataId) {
                this.movements.addChildMovement(new OrderChildMovement(rowDataId, moveBeforeRowDataId));
            };
            ContentGridDragHandler.prototype.getModelId = function (model) {
                return model ? model.getContentId() : null;
            };
            return ContentGridDragHandler;
        })(GridDragHandler);
        browse.ContentGridDragHandler = ContentGridDragHandler;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var ContentPublishPromptEvent = (function (_super) {
            __extends(ContentPublishPromptEvent, _super);
            function ContentPublishPromptEvent() {
                _super.apply(this, arguments);
            }
            ContentPublishPromptEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ContentPublishPromptEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ContentPublishPromptEvent;
        })(browse.BaseContentModelEvent);
        browse.ContentPublishPromptEvent = ContentPublishPromptEvent;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var browse;
    (function (browse) {
        var FoldButton = api.ui.toolbar.FoldButton;
        var MobileContentBrowseToolbar = (function (_super) {
            __extends(MobileContentBrowseToolbar, _super);
            function MobileContentBrowseToolbar(actions) {
                _super.call(this, "toolbar");
                this.actions = [];
                var foldButton = this.initFoldButton();
                this.addActions(actions.getAllActions(), foldButton);
                this.initEditButton(actions.EDIT_CONTENT);
            }
            MobileContentBrowseToolbar.prototype.initFoldButton = function () {
                var fold = new FoldButton();
                fold.setLabel('More...');
                this.appendChild(fold);
                return fold;
            };
            MobileContentBrowseToolbar.prototype.initEditButton = function (editAction) {
                var editButton = new api.ui.button.ActionButton(editAction);
                editButton.addClass("mobile-edit-action");
                this.appendChild(editButton);
            };
            MobileContentBrowseToolbar.prototype.addActions = function (actions, foldButton) {
                this.actions = this.actions.concat(actions);
                actions.forEach(function (action) {
                    var actionButton = new api.ui.button.ActionButton(action);
                    var buttonWidth = actionButton.getEl().getWidthWithBorder();
                    foldButton.push(actionButton, buttonWidth);
                });
            };
            MobileContentBrowseToolbar.prototype.getActions = function () {
                return this.actions;
            };
            return MobileContentBrowseToolbar;
        })(api.dom.DivEl);
        browse.MobileContentBrowseToolbar = MobileContentBrowseToolbar;
    })(browse = app.browse || (app.browse = {}));
})(app || (app = {}));
var app;
(function (app) {
    var remove;
    (function (remove) {
        var ContentIconUrlResolver = api.content.ContentIconUrlResolver;
        var BrowseItem = api.app.browse.BrowseItem;
        var SelectionItem = api.app.browse.SelectionItem;
        var ContentPath = api.content.ContentPath;
        var ContentDeleteDialog = (function (_super) {
            __extends(ContentDeleteDialog, _super);
            function ContentDeleteDialog() {
                var _this = this;
                _super.call(this, "item");
                this.deleteButton = this.setDeleteAction(new remove.ContentDeleteDialogAction());
                this.getDeleteAction().onExecuted(function () {
                    _this.createDeleteRequest().sendAndParse().then(function (result) {
                        _this.close();
                        app.view.DeleteAction.showDeleteResult(result);
                    }).catch(function (reason) {
                        if (reason && reason.message) {
                            api.notify.showError(reason.message);
                        }
                        else {
                            api.notify.showError('Content could not be deleted.');
                        }
                    }).done();
                });
                this.addCancelButtonToBottom();
            }
            ContentDeleteDialog.prototype.setContentToDelete = function (contents) {
                var _this = this;
                this.selectedItems = [];
                contents.forEach(function (content) {
                    _this.selectedItems.push(_this.createSelectionItemForDelete(content));
                });
                this.renderSelectedItems(this.selectedItems);
                this.countItemsToDeleteAndUpdateButtonCounter();
            };
            ContentDeleteDialog.prototype.indexOf = function (item) {
                for (var i = 0; i < this.selectedItems.length; i++) {
                    if (item.getBrowseItem().getPath() == this.selectedItems[i].getBrowseItem().getPath()) {
                        return i;
                    }
                }
                return -1;
            };
            ContentDeleteDialog.prototype.createSelectionItemForDelete = function (content) {
                var _this = this;
                var deleteItemViewer = new api.content.ContentSummaryViewer();
                deleteItemViewer.setObject(content);
                var browseItem = new BrowseItem(content).setId(content.getId()).setDisplayName(content.getDisplayName()).setPath(content.getPath().toString()).setIconUrl(new ContentIconUrlResolver().setContent(content).resolve());
                var selectionItem = new SelectionItem(deleteItemViewer, browseItem, function () {
                    var index = _this.indexOf(selectionItem);
                    if (index < 0) {
                        return;
                    }
                    _this.selectedItems[index].remove();
                    _this.selectedItems.splice(index, 1);
                    if (_this.selectedItems.length == 0) {
                        _this.close();
                    }
                    else {
                        _this.countItemsToDeleteAndUpdateButtonCounter();
                    }
                });
                return selectionItem;
            };
            ContentDeleteDialog.prototype.countItemsToDeleteAndUpdateButtonCounter = function () {
                var _this = this;
                this.cleanDeleteButtonText();
                this.showLoadingSpinner();
                this.createRequestForCountingItemsToDelete().sendAndParse().then(function (itemsToDeleteCounter) {
                    _this.hideLoadingSpinner();
                    _this.updateDeleteButtonCounter(itemsToDeleteCounter);
                }).finally(function () {
                    _this.hideLoadingSpinner();
                }).done();
            };
            ContentDeleteDialog.prototype.createRequestForCountingItemsToDelete = function () {
                var countContentChildrenRequest = new api.content.CountContentsWithDescendantsRequest();
                for (var j = 0; j < this.selectedItems.length; j++) {
                    countContentChildrenRequest.addContentPath(ContentPath.fromString(this.selectedItems[j].getBrowseItem().getPath()));
                }
                return countContentChildrenRequest;
            };
            ContentDeleteDialog.prototype.createDeleteRequest = function () {
                var deleteRequest = new api.content.DeleteContentRequest();
                for (var i = 0; i < this.selectedItems.length; i++) {
                    deleteRequest.addContentPath(ContentPath.fromString(this.selectedItems[i].getBrowseItem().getPath()));
                }
                return deleteRequest;
            };
            ContentDeleteDialog.prototype.updateDeleteButtonCounter = function (count) {
                this.deleteButton.setLabel("Delete (" + count + ")");
            };
            ContentDeleteDialog.prototype.showLoadingSpinner = function () {
                this.deleteButton.addClass("spinner");
            };
            ContentDeleteDialog.prototype.hideLoadingSpinner = function () {
                this.deleteButton.removeClass("spinner");
            };
            ContentDeleteDialog.prototype.cleanDeleteButtonText = function () {
                this.deleteButton.setLabel("Delete ");
            };
            return ContentDeleteDialog;
        })(api.app.remove.DeleteDialog);
        remove.ContentDeleteDialog = ContentDeleteDialog;
    })(remove = app.remove || (app.remove = {}));
})(app || (app = {}));
var app;
(function (app) {
    var remove;
    (function (remove) {
        var ContentDeleteDialogAction = (function (_super) {
            __extends(ContentDeleteDialogAction, _super);
            function ContentDeleteDialogAction() {
                _super.call(this, "Delete", "enter");
                this.setIconClass("delete-action");
            }
            return ContentDeleteDialogAction;
        })(api.ui.Action);
        remove.ContentDeleteDialogAction = ContentDeleteDialogAction;
    })(remove = app.remove || (app.remove = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var NewContentEvent = (function (_super) {
            __extends(NewContentEvent, _super);
            function NewContentEvent(contentType, parentContent) {
                _super.call(this);
                this.contentType = contentType;
                this.parentContent = parentContent;
            }
            NewContentEvent.prototype.getContentType = function () {
                return this.contentType;
            };
            NewContentEvent.prototype.getParentContent = function () {
                return this.parentContent;
            };
            NewContentEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            NewContentEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return NewContentEvent;
        })(api.event.Event);
        create.NewContentEvent = NewContentEvent;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var NewMediaUploadEvent = (function (_super) {
            __extends(NewMediaUploadEvent, _super);
            function NewMediaUploadEvent(items, parentContent) {
                _super.call(this);
                this.uploadItems = items;
                this.parentContent = parentContent;
            }
            NewMediaUploadEvent.prototype.getUploadItems = function () {
                return this.uploadItems;
            };
            NewMediaUploadEvent.prototype.getParentContent = function () {
                return this.parentContent;
            };
            NewMediaUploadEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            NewMediaUploadEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return NewMediaUploadEvent;
        })(api.event.Event);
        create.NewMediaUploadEvent = NewMediaUploadEvent;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var GetAllContentTypesRequest = api.schema.content.GetAllContentTypesRequest;
        var GetNearestSiteRequest = api.content.GetNearestSiteRequest;
        var ContentPath = api.content.ContentPath;
        var ListContentByPathRequest = api.content.ListContentByPathRequest;
        var NewContentDialog = (function (_super) {
            __extends(NewContentDialog, _super);
            function NewContentDialog() {
                this.contentDialogTitle = new NewContentDialogTitle("Create Content", "");
                _super.call(this, {
                    title: this.contentDialogTitle
                });
                this.uploaderEnabled = true;
                this.listItems = [];
                this.addClass("new-content-dialog hidden");
                this.initContentList();
                this.initMostPopularItemsBlock();
                this.initFileInput();
                this.initAndAppendContentSection();
                this.initAndAppendRecentItemsBlock();
                this.initMediaUploader();
                this.initLoadingMasks();
                api.dom.Body.get().appendChild(this);
            }
            NewContentDialog.prototype.initContentList = function () {
                var _this = this;
                this.contentList = new app.create.NewContentDialogList();
                this.contentList.onSelected(function (event) {
                    _this.closeAndFireEventFromContentType(event.getItem());
                });
            };
            NewContentDialog.prototype.initMostPopularItemsBlock = function () {
                var _this = this;
                this.mostPopularItemsBlock = new create.MostPopularItemsBlock();
                this.mostPopularItemsBlock.hide();
                this.mostPopularItemsBlock.getMostPopularList().onSelected(function (event) {
                    _this.closeAndFireEventFromContentType(event.getItem());
                });
            };
            NewContentDialog.prototype.initFileInput = function () {
                var _this = this;
                this.fileInput = new api.ui.text.FileInput('large').setPlaceholder("Search for content types").setUploaderParams({
                    parent: ContentPath.ROOT.toString()
                });
                this.fileInput.onUploadStarted(function (event) {
                    _this.closeAndFireEventFromMediaUpload(event.getUploadItems());
                });
                this.fileInput.onInput(function (event) {
                    if (api.util.StringHelper.isEmpty(_this.fileInput.getValue()) && _this.mostPopularItems.length > 0) {
                        _this.mostPopularItemsBlock.show();
                    }
                    else {
                        _this.mostPopularItemsBlock.hide();
                    }
                    _this.filterList();
                });
                this.fileInput.onKeyUp(function (event) {
                    if (event.keyCode === 27) {
                        _this.getCancelAction().execute();
                    }
                });
            };
            NewContentDialog.prototype.initAndAppendContentSection = function () {
                var section = new api.dom.SectionEl().setClass("column");
                this.appendChildToContentPanel(section);
                var contentTypesListDiv = new api.dom.DivEl("content-types-content");
                contentTypesListDiv.appendChildren(this.mostPopularItemsBlock, this.contentList);
                section.appendChildren(this.fileInput, contentTypesListDiv);
            };
            NewContentDialog.prototype.initAndAppendRecentItemsBlock = function () {
                var _this = this;
                this.recentItemsBlock = new create.RecentItemsBlock();
                this.appendChildToContentPanel(this.recentItemsBlock);
                this.recentItemsBlock.getRecentItemsList().onSelected(function (event) {
                    _this.closeAndFireEventFromContentType(event.getItem());
                });
            };
            NewContentDialog.prototype.initMediaUploader = function () {
                var _this = this;
                var uploaderContainer = new api.dom.DivEl('uploader-container');
                this.appendChild(uploaderContainer);
                var uploaderMask = new api.dom.DivEl('uploader-mask');
                uploaderContainer.appendChild(uploaderMask);
                this.mediaUploader = new api.content.MediaUploader({
                    operation: 0 /* create */,
                    params: {
                        parent: ContentPath.ROOT.toString()
                    },
                    name: 'new-content-uploader',
                    showButtons: false,
                    showResult: false,
                    allowMultiSelection: true,
                    deferred: true
                });
                uploaderContainer.appendChild(this.mediaUploader);
                this.mediaUploader.onUploadStarted(function (event) {
                    _this.closeAndFireEventFromMediaUpload(event.getUploadItems());
                });
                var dragOverEl;
                this.onDragEnter(function (event) {
                    if (_this.uploaderEnabled) {
                        var target = event.target;
                        if (!!dragOverEl || dragOverEl == _this.getHTMLElement()) {
                            uploaderContainer.show();
                        }
                        dragOverEl = target;
                    }
                });
                this.onDragLeave(function (event) {
                    if (_this.uploaderEnabled) {
                        var targetEl = event.target;
                        if (dragOverEl == targetEl) {
                            uploaderContainer.hide();
                        }
                    }
                });
                this.onDrop(function (event) {
                    if (_this.uploaderEnabled) {
                        uploaderContainer.hide();
                    }
                });
            };
            NewContentDialog.prototype.initLoadingMasks = function () {
                this.contentListMask = new api.ui.mask.LoadMask(this.contentList);
                this.recentListMask = new api.ui.mask.LoadMask(this.recentItemsBlock.getRecentItemsList());
            };
            NewContentDialog.prototype.closeAndFireEventFromMediaUpload = function (items) {
                this.close();
                new create.NewMediaUploadEvent(items, this.parentContent).fire();
            };
            NewContentDialog.prototype.closeAndFireEventFromContentType = function (item) {
                this.close();
                new create.NewContentEvent(item.getContentType(), this.parentContent).fire();
            };
            NewContentDialog.prototype.filterList = function () {
                var inputValue = this.fileInput.getValue();
                var filteredItems = this.listItems.filter(function (item) {
                    return (!inputValue || (item.getDisplayName().indexOf(inputValue) != -1) || (item.getName().indexOf(inputValue) != -1));
                });
                this.contentList.setItems(filteredItems);
            };
            NewContentDialog.prototype.filterByParentContent = function (items, siteApplicationKeys) {
                var _this = this;
                var createContentFilter = new api.content.CreateContentFilter().siteApplicationsFilter(siteApplicationKeys);
                return items.filter(function (item) { return createContentFilter.isCreateContentAllowed(_this.parentContent, item.getContentType()); });
            };
            NewContentDialog.prototype.setParentContent = function (parent) {
                this.parentContent = parent;
                var params = {
                    parent: parent ? parent.getPath().toString() : api.content.ContentPath.ROOT.toString()
                };
                this.mediaUploader.setParams(params);
                this.fileInput.setUploaderParams(params);
            };
            NewContentDialog.prototype.open = function () {
                _super.prototype.open.call(this);
                var keyBindings = [
                    new api.ui.KeyBinding('up', function () {
                        api.dom.FormEl.moveFocusToPrevFocusable(api.dom.Element.fromHtmlElement(document.activeElement), "input,li");
                    }).setGlobal(true),
                    new api.ui.KeyBinding('down', function () {
                        api.dom.FormEl.moveFocusToNextFocusable(api.dom.Element.fromHtmlElement(document.activeElement), "input,li");
                    }).setGlobal(true)
                ];
                api.ui.KeyBindings.get().bindKeys(keyBindings);
            };
            NewContentDialog.prototype.show = function () {
                this.updateDialogTitlePath();
                this.toggleUploaderEnabled();
                this.resetFileInputWithUploader();
                _super.prototype.show.call(this);
                this.fileInput.giveFocus();
                if (this.mockModalDialog == null) {
                    this.createMockDialog();
                }
                this.loadContentTypes();
            };
            NewContentDialog.prototype.hide = function () {
                _super.prototype.hide.call(this);
                this.mediaUploader.stop();
                this.addClass("hidden");
                this.removeClass("animated");
                this.mostPopularItemsBlock.hide();
            };
            NewContentDialog.prototype.close = function () {
                this.fileInput.reset();
                _super.prototype.close.call(this);
            };
            NewContentDialog.prototype.loadContentTypes = function () {
                var _this = this;
                this.showLoadingMasks();
                wemQ.all(this.prepareRequestsToFetchContentData()).spread(function (contentTypes, directChilds, parentSite) {
                    _this.listItems = _this.createListOfContentTypeItems(contentTypes, parentSite);
                    _this.mostPopularItems = _this.createMostPopularItemList(_this.listItems.map(function (el) { return el.getContentType(); }), directChilds.getContents());
                    _this.resetNewContentDialogContent();
                    _this.toggleMostPopularBlockShown();
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                }).finally(function () {
                    _this.filterList();
                    _this.hideLoadingMasks();
                    _this.handleModalDialogAnimation();
                }).done();
            };
            NewContentDialog.prototype.showLoadingMasks = function () {
                this.contentList.insertChild(this.contentListMask, 0);
                this.recentItemsBlock.getRecentItemsList().insertChild(this.recentListMask, 0);
                this.contentListMask.show();
                this.recentListMask.show();
            };
            NewContentDialog.prototype.hideLoadingMasks = function () {
                this.contentListMask.hide();
                this.recentListMask.hide();
            };
            NewContentDialog.prototype.prepareRequestsToFetchContentData = function () {
                var requests = [];
                requests.push(new GetAllContentTypesRequest().sendAndParse());
                if (this.parentContent) {
                    requests.push(new ListContentByPathRequest(this.parentContent.getPath()).sendAndParse());
                    requests.push(new GetNearestSiteRequest(this.parentContent.getContentId()).sendAndParse());
                }
                else {
                    requests.push(new ListContentByPathRequest(ContentPath.ROOT).sendAndParse());
                }
                return requests;
            };
            NewContentDialog.prototype.showMockDialog = function () {
                wemjq(this.getEl().getHTMLElement()).show();
            };
            NewContentDialog.prototype.handleModalDialogAnimation = function () {
                this.mockModalDialog.mostPopularItemsBlock.getMostPopularList().setItems(this.mostPopularItems);
                this.toggleMockDialogMostPopularBlockShown();
                this.mockModalDialog.contentList.setItems(this.listItems);
                this.updateMockDialogTitlePath();
                this.mockModalDialog.showMockDialog();
                this.addClass("animated");
                this.removeClass("hidden");
                this.alignDialogWindowVertically();
            };
            NewContentDialog.prototype.toggleMockDialogMostPopularBlockShown = function () {
                if (this.mostPopularItems.length > 0) {
                    this.mockModalDialog.mostPopularItemsBlock.show();
                }
                else {
                    this.mockModalDialog.mostPopularItemsBlock.hide();
                }
            };
            NewContentDialog.prototype.toggleMostPopularBlockShown = function () {
                if (this.mostPopularItems.length > 0) {
                    this.mostPopularItemsBlock.getMostPopularList().setItems(this.mostPopularItems);
                    this.mostPopularItemsBlock.show();
                }
            };
            NewContentDialog.prototype.updateMockDialogTitlePath = function () {
                if (this.parentContent) {
                    this.mockModalDialog.contentDialogTitle.setPath(this.parentContent.getPath().toString());
                }
                else {
                    this.mockModalDialog.contentDialogTitle.setPath('');
                }
            };
            NewContentDialog.prototype.updateDialogTitlePath = function () {
                if (this.parentContent) {
                    this.contentDialogTitle.setPath(this.parentContent.getPath().toString());
                }
                else {
                    this.contentDialogTitle.setPath('');
                }
            };
            NewContentDialog.prototype.resetNewContentDialogContent = function () {
                if (this.listItems.length > 0) {
                    this.contentList.setItems(this.listItems);
                    this.recentItemsBlock.getRecentItemsList().setItems(this.listItems);
                    this.mostPopularItemsBlock.getMostPopularList().setItems(this.mostPopularItems);
                }
                else {
                    this.mostPopularItemsBlock.getMostPopularList().clearItems();
                    this.contentList.clearItems();
                    this.recentItemsBlock.getRecentItemsList().clearItems();
                }
            };
            NewContentDialog.prototype.toggleUploaderEnabled = function () {
                this.uploaderEnabled = !this.parentContent || !this.parentContent.getType().isTemplateFolder();
                if (this.uploaderEnabled) {
                    this.removeClass("no-uploader");
                }
                else {
                    this.addClass("no-uploader");
                }
            };
            NewContentDialog.prototype.resetFileInputWithUploader = function () {
                this.mediaUploader.reset();
                this.fileInput.reset();
                this.mediaUploader.setEnabled(this.uploaderEnabled);
                this.fileInput.getUploader().setEnabled(this.uploaderEnabled);
            };
            NewContentDialog.prototype.createMockDialog = function () {
                this.mockModalDialog = new NewContentDialog();
                this.mockModalDialog.close = function () {
                    wemjq(this.getEl().getHTMLElement()).hide();
                };
                this.getParentElement().appendChild(this.mockModalDialog);
                this.mockModalDialog.addClass("mock-modal-dialog");
                this.mockModalDialog.removeClass("hidden");
            };
            NewContentDialog.prototype.createListItems = function (contentTypes) {
                var contentTypesByName = {};
                var items = [];
                contentTypes.forEach(function (contentType) {
                    var contentTypeName = contentType.getContentTypeName();
                    if (!contentTypeName.isMedia() && !contentTypeName.isDescendantOfMedia()) {
                        contentTypesByName[contentType.getName()] = contentType;
                        items.push(create.NewContentDialogListItem.fromContentType(contentType));
                    }
                });
                items.sort(this.compareListItems);
                return items;
            };
            NewContentDialog.prototype.createListOfContentTypeItems = function (allContentTypes, parentSite) {
                var allListItems = this.createListItems(allContentTypes);
                var siteApplications = parentSite ? parentSite.getApplicationKeys() : [];
                return this.filterByParentContent(allListItems, siteApplications);
            };
            NewContentDialog.prototype.findElementByFieldValue = function (array, field, value) {
                var result;
                array.every(function (element) {
                    if (element[field] == value) {
                        result = element;
                        return false;
                    }
                    return true;
                });
                return result;
            };
            NewContentDialog.prototype.sortByCountAndDate = function (contentType1, contentType2) {
                if (contentType2.count == contentType1.count) {
                    return contentType2.lastModified > contentType1.lastModified ? 1 : -1;
                }
                return contentType2.count - contentType1.count;
            };
            NewContentDialog.prototype.getAggregatedItemList = function (contentTypes) {
                var _this = this;
                var aggregatedList = [];
                contentTypes.forEach(function (content) {
                    var contentType = content.getType().toString();
                    var existingContent = _this.findElementByFieldValue(aggregatedList, "contentType", contentType);
                    if (existingContent) {
                        existingContent.count++;
                        if (content.getModifiedTime() > existingContent.lastModified) {
                            existingContent.lastModified = content.getModifiedTime();
                        }
                    }
                    else {
                        aggregatedList.push({ contentType: contentType, count: 1, lastModified: content.getModifiedTime() });
                    }
                });
                aggregatedList.sort(this.sortByCountAndDate);
                return aggregatedList;
            };
            NewContentDialog.prototype.createMostPopularItemList = function (allowedContentTypes, directChildContents) {
                var _this = this;
                var mostPopularItems = [], filteredList = directChildContents.filter(function (content) {
                    return _this.isAllowedContentType(allowedContentTypes, content);
                }), aggregatedList = this.getAggregatedItemList(filteredList);
                for (var i = 0; i < aggregatedList.length && i < create.MostPopularItemsBlock.DEFAULT_MAX_ITEMS; i++) {
                    var contentType = this.findElementByFieldValue(allowedContentTypes, "name", aggregatedList[i].contentType);
                    mostPopularItems.push(new create.MostPopularItem(contentType, aggregatedList[i].count));
                }
                return mostPopularItems;
            };
            NewContentDialog.prototype.compareListItems = function (item1, item2) {
                if (item1.getDisplayName().toLowerCase() > item2.getDisplayName().toLowerCase()) {
                    return 1;
                }
                else if (item1.getDisplayName().toLowerCase() < item2.getDisplayName().toLowerCase()) {
                    return -1;
                }
                else if (item1.getName() > item2.getName()) {
                    return 1;
                }
                else if (item1.getName() < item2.getName()) {
                    return -1;
                }
                else {
                    return 0;
                }
            };
            NewContentDialog.prototype.isAllowedContentType = function (allowedContentTypes, content) {
                return !content.getType().isMedia() && !content.getType().isDescendantOfMedia() && Boolean(this.findElementByFieldValue(allowedContentTypes, "id", content.getType().toString()));
            };
            NewContentDialog.prototype.alignDialogWindowVertically = function () {
                this.getEl().setMarginTop("-" + (this.mockModalDialog.getEl().getHeightWithBorder() / 2) + "px");
            };
            return NewContentDialog;
        })(api.ui.dialog.ModalDialog);
        create.NewContentDialog = NewContentDialog;
        var NewContentDialogTitle = (function (_super) {
            __extends(NewContentDialogTitle, _super);
            function NewContentDialogTitle(title, path) {
                _super.call(this, title);
                this.pathEl = new api.dom.PEl('path');
                this.pathEl.setHtml(path);
                this.appendChild(this.pathEl);
            }
            NewContentDialogTitle.prototype.setPath = function (path) {
                this.pathEl.setHtml(path).setVisible(!api.util.StringHelper.isBlank(path));
            };
            return NewContentDialogTitle;
        })(api.ui.dialog.ModalDialogHeader);
        create.NewContentDialogTitle = NewContentDialogTitle;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var NewContentDialogList = (function (_super) {
            __extends(NewContentDialogList, _super);
            function NewContentDialogList(className) {
                if (className === void 0) { className = 'content-types-list'; }
                _super.call(this, className);
                this.selectedListeners = [];
            }
            NewContentDialogList.prototype.onSelected = function (listener) {
                this.selectedListeners.push(listener);
            };
            NewContentDialogList.prototype.unSelected = function (listener) {
                this.selectedListeners = this.selectedListeners.filter(function (currentListener) {
                    return currentListener != listener;
                });
            };
            NewContentDialogList.prototype.notifySelected = function (listItem) {
                var _this = this;
                this.selectedListeners.forEach(function (listener) {
                    listener.call(_this, new create.NewContentDialogItemSelectedEvent(listItem));
                });
            };
            NewContentDialogList.prototype.createItemView = function (item) {
                var _this = this;
                var namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(2 /* small */).build();
                namesAndIconView.setIconUrl(item.getIconUrl()).setMainName(item.getDisplayName()).setSubName(item.getName()).setDisplayIconLabel(item.isSite());
                var itemEl = new api.dom.LiEl('content-types-list-item' + (item.isSite() ? ' site' : ''));
                itemEl.getEl().setTabIndex(0);
                itemEl.appendChild(namesAndIconView);
                itemEl.onClicked(function (event) { return _this.notifySelected(item); });
                itemEl.onKeyPressed(function (event) {
                    if (event.keyCode == 13) {
                        _this.notifySelected(item);
                    }
                });
                return itemEl;
            };
            NewContentDialogList.prototype.getItemId = function (item) {
                return item.getName();
            };
            return NewContentDialogList;
        })(api.ui.selector.list.ListBox);
        create.NewContentDialogList = NewContentDialogList;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var NewContentDialogListItem = (function () {
            function NewContentDialogListItem(contentType) {
                this.contentType = contentType;
                this.name = contentType.getName();
                this.displayName = contentType.getDisplayName();
                this.iconUrl = contentType.getIconUrl();
            }
            NewContentDialogListItem.fromContentType = function (contentType) {
                return new NewContentDialogListItem(contentType);
            };
            NewContentDialogListItem.prototype.getContentType = function () {
                return this.contentType;
            };
            NewContentDialogListItem.prototype.isSite = function () {
                return this.contentType.isSite();
            };
            NewContentDialogListItem.prototype.getName = function () {
                return this.name;
            };
            NewContentDialogListItem.prototype.getDisplayName = function () {
                return this.displayName;
            };
            NewContentDialogListItem.prototype.getIconUrl = function () {
                return this.iconUrl;
            };
            return NewContentDialogListItem;
        })();
        create.NewContentDialogListItem = NewContentDialogListItem;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var NewContentDialogItemSelectedEvent = (function () {
            function NewContentDialogItemSelectedEvent(item) {
                this.item = item;
            }
            NewContentDialogItemSelectedEvent.prototype.getItem = function () {
                return this.item;
            };
            return NewContentDialogItemSelectedEvent;
        })();
        create.NewContentDialogItemSelectedEvent = NewContentDialogItemSelectedEvent;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        create.NewContentEvent.on(function (event) {
            RecentItems.get().addItemName(event.getContentType());
        });
        var RecentItems = (function () {
            function RecentItems() {
                this.maximum = 7;
                this.cookieKey = 'app.browse.RecentItemsList';
                this.cookieExpire = 30;
                this.valueSeparator = '|';
            }
            RecentItems.get = function () {
                return RecentItems.INSTANCE;
            };
            RecentItems.prototype.addItemName = function (contentType) {
                var itemsNames = this.getRecentItemsNames();
                var name = contentType.getName();
                itemsNames = itemsNames.filter(function (storedName) { return storedName != name; });
                itemsNames.unshift(name);
                itemsNames = itemsNames.slice(0, this.maximum);
                api.util.CookieHelper.setCookie(this.cookieKey, itemsNames.join(this.valueSeparator), this.cookieExpire);
            };
            RecentItems.prototype.getRecentItemsNames = function () {
                var cookies = api.util.CookieHelper.getCookie(this.cookieKey);
                return cookies ? cookies.split(this.valueSeparator) : [];
            };
            RecentItems.INSTANCE = new RecentItems();
            return RecentItems;
        })();
        create.RecentItems = RecentItems;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var RecentItemsList = (function (_super) {
            __extends(RecentItemsList, _super);
            function RecentItemsList() {
                _super.call(this, "recent-content-types-list");
            }
            RecentItemsList.prototype.setItems = function (items) {
                var itemsByName = {};
                items.forEach(function (item) {
                    itemsByName[item.getName()] = item;
                });
                var recentItemsNames = create.RecentItems.get().getRecentItemsNames();
                var recentItems = [];
                recentItemsNames.forEach(function (name) {
                    if (itemsByName[name]) {
                        recentItems.push(itemsByName[name]);
                    }
                });
                _super.prototype.setItems.call(this, recentItems);
            };
            return RecentItemsList;
        })(create.NewContentDialogList);
        create.RecentItemsList = RecentItemsList;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var RecentItemsBlock = (function (_super) {
            __extends(RecentItemsBlock, _super);
            function RecentItemsBlock(title) {
                if (title === void 0) { title = "Recently Used"; }
                _super.call(this, "column");
                this.title = new api.dom.H1El();
                this.title.setHtml(title);
                this.recentItemsList = new create.RecentItemsList();
                this.appendChildren(this.title, this.recentItemsList);
            }
            RecentItemsBlock.prototype.getRecentItemsList = function () {
                return this.recentItemsList;
            };
            RecentItemsBlock.prototype.setTitle = function (newTitle) {
                this.title.setHtml(newTitle);
            };
            return RecentItemsBlock;
        })(api.dom.AsideEl);
        create.RecentItemsBlock = RecentItemsBlock;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var MostPopularItemsList = (function (_super) {
            __extends(MostPopularItemsList, _super);
            function MostPopularItemsList() {
                _super.call(this, "most-popular-content-types-list");
            }
            MostPopularItemsList.prototype.createItemView = function (item) {
                var _this = this;
                var namesAndIconView = new api.app.NamesAndIconViewBuilder().setSize(2 /* small */).build();
                namesAndIconView.setIconUrl(item.getIconUrl()).setMainName(item.getDisplayName() + " (" + item.getHits() + ")").setSubName(item.getName()).setDisplayIconLabel(item.isSite());
                var itemEl = new api.dom.LiEl('content-types-list-item' + (item.isSite() ? ' site' : ''));
                itemEl.getEl().setTabIndex(0);
                itemEl.appendChild(namesAndIconView);
                itemEl.onClicked(function (event) { return _this.notifySelected(item); });
                itemEl.onKeyPressed(function (event) {
                    if (event.keyCode == 13) {
                        _this.notifySelected(item);
                    }
                });
                return itemEl;
            };
            return MostPopularItemsList;
        })(create.NewContentDialogList);
        create.MostPopularItemsList = MostPopularItemsList;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var MostPopularItem = (function (_super) {
            __extends(MostPopularItem, _super);
            function MostPopularItem(contentType, hits) {
                _super.call(this, contentType);
                this.hits = hits;
            }
            MostPopularItem.prototype.getHits = function () {
                return this.hits;
            };
            return MostPopularItem;
        })(create.NewContentDialogListItem);
        create.MostPopularItem = MostPopularItem;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var create;
    (function (create) {
        var MostPopularItemsBlock = (function (_super) {
            __extends(MostPopularItemsBlock, _super);
            function MostPopularItemsBlock(title) {
                if (title === void 0) { title = "Most Popular"; }
                _super.call(this, "most-popular-content-types-container");
                this.title = new api.dom.H2El();
                this.title.setHtml(title);
                this.mostPopularItemsList = new create.MostPopularItemsList();
                this.appendChildren(this.title, this.mostPopularItemsList);
            }
            MostPopularItemsBlock.prototype.getMostPopularList = function () {
                return this.mostPopularItemsList;
            };
            MostPopularItemsBlock.prototype.setTitle = function (newTitle) {
                this.title.setHtml(newTitle);
            };
            MostPopularItemsBlock.DEFAULT_MAX_ITEMS = 2;
            return MostPopularItemsBlock;
        })(api.dom.DivEl);
        create.MostPopularItemsBlock = MostPopularItemsBlock;
    })(create = app.create || (app.create = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var CloseAction = (function (_super) {
            __extends(CloseAction, _super);
            function CloseAction(itemViewPanel, checkCanRemovePanel) {
                if (checkCanRemovePanel === void 0) { checkCanRemovePanel = true; }
                _super.call(this, "Close", "mod+alt+f4");
                this.onExecuted(function () {
                    itemViewPanel.close(checkCanRemovePanel);
                });
            }
            return CloseAction;
        })(api.ui.Action);
        view.CloseAction = CloseAction;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var DeleteAction = (function (_super) {
            __extends(DeleteAction, _super);
            function DeleteAction(itemViewPanel) {
                _super.call(this, "Delete", "mod+del");
                this.onExecuted(function () {
                    var contentToDelete = itemViewPanel.getItem().getModel();
                    api.ui.dialog.ConfirmationDialog.get().setQuestion("Are you sure you want to delete this content?").setNoCallback(null).setYesCallback(function () {
                        itemViewPanel.close();
                        new api.content.DeleteContentRequest().addContentPath(contentToDelete.getPath()).sendAndParse().then(function (result) {
                            DeleteAction.showDeleteResult(result);
                        }).catch(function (reason) {
                            if (reason && reason.message) {
                                api.notify.showError(reason.message);
                            }
                            else {
                                api.notify.showError('Content could not be deleted.');
                            }
                        }).done();
                    }).open();
                });
            }
            DeleteAction.showDeleteResult = function (result) {
                if (result.getPendings().length == 1) {
                    api.notify.showFeedback('\"' + result.getPendings()[0] + '\" marked for deletion');
                }
                else if (result.getPendings().length > 1) {
                    api.notify.showFeedback(result.getPendings().length + ' items marked for deletion');
                }
                if (result.getDeleted().length == 1) {
                    api.notify.showFeedback('\"' + result.getDeleted()[0].getName() + '\" deleted');
                }
                else if (result.getDeleted().length > 1) {
                    api.notify.showFeedback(result.getDeleted().length + ' items deleted');
                }
                if (result.getDeleteFailures().length > 0) {
                    api.notify.showWarning('Content could not be deleted. ' + result.getDeleteFailures()[0].getReason());
                }
            };
            return DeleteAction;
        })(api.ui.Action);
        view.DeleteAction = DeleteAction;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var EditAction = (function (_super) {
            __extends(EditAction, _super);
            function EditAction(panel) {
                _super.call(this, "Edit");
                this.onExecuted(function () {
                    new api.content.EditContentEvent([panel.getItem().getModel()]).fire();
                });
            }
            return EditAction;
        })(api.ui.Action);
        view.EditAction = EditAction;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var ContentItemViewToolbar = (function (_super) {
            __extends(ContentItemViewToolbar, _super);
            function ContentItemViewToolbar(params) {
                _super.call(this);
                _super.prototype.addAction.call(this, params.editAction);
                _super.prototype.addAction.call(this, params.deleteAction);
            }
            return ContentItemViewToolbar;
        })(api.ui.toolbar.Toolbar);
        view.ContentItemViewToolbar = ContentItemViewToolbar;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var RenderingMode = api.rendering.RenderingMode;
        var ContentImageUrlResolver = api.content.ContentImageUrlResolver;
        var ContentItemPreviewPanel = (function (_super) {
            __extends(ContentItemPreviewPanel, _super);
            function ContentItemPreviewPanel() {
                var _this = this;
                _super.call(this, "content-item-preview-panel");
                this.image = new api.dom.ImgEl();
                this.image.onLoaded(function (event) {
                    _this.mask.hide();
                    var imgEl = _this.image.getEl();
                    var myEl = _this.getEl();
                    _this.centerImage(imgEl.getWidth(), imgEl.getHeight(), myEl.getWidth(), myEl.getHeight());
                });
                this.appendChild(this.image);
                api.ui.responsive.ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                    if (_this.hasClass("image-preview")) {
                        var imgEl = _this.image.getEl(), el = _this.getEl();
                        _this.centerImage(imgEl.getWidth(), imgEl.getHeight(), el.getWidth(), el.getHeight());
                    }
                });
                this.onShown(function (event) {
                    if (_this.item && _this.hasClass("image-preview")) {
                        _this.addImageSizeToUrl(_this.item);
                    }
                });
            }
            ContentItemPreviewPanel.prototype.centerImage = function (imgWidth, imgHeight, myWidth, myHeight) {
                var imgMarginTop = 0;
                if (imgHeight < myHeight) {
                    imgMarginTop = (myHeight - imgHeight) / 2;
                }
                this.image.getEl().setMarginTop(imgMarginTop + "px");
            };
            ContentItemPreviewPanel.prototype.addImageSizeToUrl = function (item) {
                var imgSize = Math.max(this.getEl().getWidth(), this.getEl().getHeight());
                var imgUrl = new ContentImageUrlResolver().setContentId(item.getModel().getContentId()).setTimestamp(item.getModel().getModifiedTime()).setSize(imgSize).resolve();
                this.image.setSrc(imgUrl);
            };
            ContentItemPreviewPanel.prototype.setItem = function (item) {
                if (item && !item.equals(this.item)) {
                    if (typeof item.isRenderable() === "undefined") {
                        return;
                    }
                    if (item.getModel().getType().isImage()) {
                        this.getEl().removeClass("no-preview page-preview").addClass("image-preview");
                        if (this.isVisible()) {
                            this.addImageSizeToUrl(item);
                        }
                        if (!this.image.isLoaded()) {
                            this.showMask();
                        }
                    }
                    else {
                        this.showMask();
                        if (item.isRenderable()) {
                            this.getEl().removeClass("image-preview no-preview").addClass('page-preview');
                            this.frame.setSrc(api.rendering.UriHelper.getPortalUri(item.getPath(), 1 /* PREVIEW */, 0 /* DRAFT */));
                        }
                        else {
                            this.getEl().removeClass("image-preview page-preview").addClass('no-preview');
                            this.frame.setSrc("about:blank");
                            this.mask.hide();
                        }
                    }
                }
                this.item = item;
            };
            ContentItemPreviewPanel.prototype.getItem = function () {
                return this.item;
            };
            ContentItemPreviewPanel.prototype.showMask = function () {
                if (this.isVisible()) {
                    this.mask.show();
                }
            };
            return ContentItemPreviewPanel;
        })(api.app.view.ItemPreviewPanel);
        view.ContentItemPreviewPanel = ContentItemPreviewPanel;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var ContentVersionViewer = api.content.ContentVersionViewer;
        var GridColumnBuilder = api.ui.grid.GridColumnBuilder;
        var TreeGridBuilder = api.ui.treegrid.TreeGridBuilder;
        var ContentVersionsTreeGrid = (function (_super) {
            __extends(ContentVersionsTreeGrid, _super);
            function ContentVersionsTreeGrid() {
                var descColumn = new GridColumnBuilder().setName("DisplayName").setField("displayName").setCssClass("description").setMinWidth(160).setFormatter(this.descriptionFormatter).build();
                var statusColumn = new GridColumnBuilder().setName("Status").setField("workspaces").setCssClass("status").setFormatter(this.statusFormatter.bind(this)).setMinWidth(80).build();
                _super.call(this, new TreeGridBuilder().setAutoLoad(false).setCheckableRows(false).setHotkeysEnabled(false).setSelectedCellCssClass("").setShowToolbar(false).setRowHeight(70).setColumns([
                    descColumn,
                    statusColumn
                ]).prependClasses("content-versions-tree-grid"));
            }
            ContentVersionsTreeGrid.prototype.setContentId = function (contentId) {
                this.contentId = contentId;
                this.reload();
            };
            ContentVersionsTreeGrid.prototype.getContentId = function () {
                return this.contentId;
            };
            ContentVersionsTreeGrid.prototype.setStatus = function (status) {
                this.status = status;
            };
            ContentVersionsTreeGrid.prototype.getDataId = function (data) {
                return data.id;
            };
            ContentVersionsTreeGrid.prototype.descriptionFormatter = function (row, cell, value, columnDef, node) {
                var description = "";
                if (node.getData()) {
                    var viewer = node.getViewer("description");
                    if (!viewer) {
                        viewer = new ContentVersionViewer();
                        var length = node.getRoot().getChildren().length;
                        viewer.setObject(node.getData(), length - row);
                        node.setViewer("description", viewer);
                    }
                    description = viewer.toString();
                }
                return description;
            };
            ContentVersionsTreeGrid.prototype.getState = function (workspace) {
                if (workspace == ContentVersionsTreeGrid.branchMaster) {
                    return api.content.CompareStatusFormatter.formatStatus(6 /* EQUAL */);
                }
                else {
                    return api.content.CompareStatusFormatter.formatStatus(this.status);
                }
            };
            ContentVersionsTreeGrid.prototype.statusFormatter = function (row, cell, value, columnDef, node) {
                var _this = this;
                if (this.status == undefined) {
                    return "";
                }
                var badges = [];
                var hasMaster = value.some(function (workspace) {
                    return workspace == ContentVersionsTreeGrid.branchMaster;
                });
                value.forEach(function (workspace) {
                    if (!hasMaster || workspace == ContentVersionsTreeGrid.branchMaster) {
                        badges.push(new api.dom.PEl('badge ' + workspace).setHtml(_this.getState(workspace)).toString());
                    }
                });
                return badges.join("");
            };
            ContentVersionsTreeGrid.branchMaster = "master";
            return ContentVersionsTreeGrid;
        })(api.ui.treegrid.TreeGrid);
        view.ContentVersionsTreeGrid = ContentVersionsTreeGrid;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var AllContentVersionsTreeGrid = (function (_super) {
            __extends(AllContentVersionsTreeGrid, _super);
            function AllContentVersionsTreeGrid() {
                _super.call(this);
            }
            AllContentVersionsTreeGrid.prototype.fetchChildren = function (parentNode) {
                var _this = this;
                if (this.contentId) {
                    var activePromise = new api.content.GetActiveContentVersionsRequest(this.contentId);
                    var allPromise = new api.content.GetContentVersionsRequest(this.contentId);
                    return wemQ.all([allPromise.sendAndParse(), activePromise.sendAndParse()]).spread(function (allVersions, activeVersions) { return _this.enrichWithWorkspaces(allVersions, activeVersions); });
                }
                else {
                    throw new Error("Required contentId not set for ActiveContentVersionsTreeGrid");
                }
            };
            AllContentVersionsTreeGrid.prototype.enrichWithWorkspaces = function (allVersions, activeVersions) {
                var filteredVersions = allVersions.length ? [allVersions[0]] : [];
                for (var i = 1; i < allVersions.length; i++) {
                    if (Math.abs(allVersions[i - 1].modified.getTime() - allVersions[i].modified.getTime()) > 500) {
                        filteredVersions.push(allVersions[i]);
                    }
                }
                activeVersions.forEach(function (activeVersion) {
                    filteredVersions.filter(function (version) {
                        return version.id == activeVersion.id;
                    }).forEach(function (version) {
                        version.workspaces = activeVersion.workspaces;
                    });
                });
                return filteredVersions;
            };
            return AllContentVersionsTreeGrid;
        })(view.ContentVersionsTreeGrid);
        view.AllContentVersionsTreeGrid = AllContentVersionsTreeGrid;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var ActiveContentVersionsTreeGrid = (function (_super) {
            __extends(ActiveContentVersionsTreeGrid, _super);
            function ActiveContentVersionsTreeGrid() {
                _super.call(this);
            }
            ActiveContentVersionsTreeGrid.prototype.fetchChildren = function (parentNode) {
                if (this.contentId) {
                    return new api.content.GetActiveContentVersionsRequest(this.contentId).sendAndParse();
                }
                else {
                    throw new Error("Required contentId not set for ActiveContentVersionsTreeGrid");
                }
            };
            return ActiveContentVersionsTreeGrid;
        })(view.ContentVersionsTreeGrid);
        view.ActiveContentVersionsTreeGrid = ActiveContentVersionsTreeGrid;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var ContentItemVersionsPanel = (function (_super) {
            __extends(ContentItemVersionsPanel, _super);
            function ContentItemVersionsPanel() {
                _super.call(this, "content-item-versions-panel");
                this.allGrid = new view.AllContentVersionsTreeGrid();
                this.appendChild(this.allGrid);
            }
            ContentItemVersionsPanel.prototype.setItem = function (item) {
                this.item = item;
                if (this.item) {
                    if (this.allGrid.getContentId() != this.item.getModel().getContentId()) {
                        this.allGrid.setContentId(item.getModel().getContentId());
                    }
                }
            };
            ContentItemVersionsPanel.prototype.setStatus = function (status) {
                this.allGrid.setStatus(status);
            };
            ContentItemVersionsPanel.prototype.reRenderActivePanel = function () {
                if (this.item) {
                    this.allGrid.getGrid().invalidate();
                    this.allGrid.render();
                }
            };
            ContentItemVersionsPanel.prototype.reloadActivePanel = function () {
                this.allGrid.reload();
            };
            ContentItemVersionsPanel.prototype.getItem = function () {
                return this.item;
            };
            return ContentItemVersionsPanel;
        })(api.ui.panel.Panel);
        view.ContentItemVersionsPanel = ContentItemVersionsPanel;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var ContentItemStatisticsPanel = (function (_super) {
            __extends(ContentItemStatisticsPanel, _super);
            function ContentItemStatisticsPanel() {
                _super.call(this, "content-item-statistics-panel");
                this.previewPanel = new view.ContentItemPreviewPanel();
                this.previewPanel.setDoOffset(false);
                this.appendChild(this.previewPanel);
            }
            ContentItemStatisticsPanel.prototype.setItem = function (item) {
                if (this.getItem() != item) {
                    _super.prototype.setItem.call(this, item);
                    this.previewPanel.setItem(item);
                }
            };
            return ContentItemStatisticsPanel;
        })(api.app.view.ItemStatisticsPanel);
        view.ContentItemStatisticsPanel = ContentItemStatisticsPanel;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var ContentItemViewPanel = (function (_super) {
            __extends(ContentItemViewPanel, _super);
            function ContentItemViewPanel() {
                var _this = this;
                this.deckPanel = new api.ui.panel.DeckPanel();
                this.editAction = new view.EditAction(this);
                this.deleteAction = new view.DeleteAction(this);
                this.closeAction = new view.CloseAction(this, true);
                this.actions = [this.editAction, this.deleteAction, this.closeAction];
                var toolbar = new view.ContentItemViewToolbar({
                    editAction: this.editAction,
                    deleteAction: this.deleteAction
                });
                _super.call(this, toolbar, this.deckPanel);
                this.statisticsPanel = new view.ContentItemStatisticsPanel();
                this.previewPanel = new view.ContentItemPreviewPanel();
                this.statisticsPanelIndex = this.deckPanel.addPanel(this.statisticsPanel);
                this.previewPanelIndex = this.deckPanel.addPanel(this.previewPanel);
                this.showPreview(false);
                app.browse.ShowPreviewEvent.on(function (event) {
                    _this.showPreview(true);
                });
                app.browse.ShowDetailsEvent.on(function (event) {
                    _this.showPreview(false);
                });
                this.onShown(function (event) {
                    if (_this.getItem()) {
                        app.Router.setHash("view/" + _this.getItem().getModel().getId());
                    }
                });
            }
            ContentItemViewPanel.prototype.setItem = function (item) {
                _super.prototype.setItem.call(this, item);
                this.statisticsPanel.setItem(item);
                this.previewPanel.setItem(item);
            };
            ContentItemViewPanel.prototype.showPreview = function (enabled) {
                this.previewMode = enabled;
                if (enabled) {
                    this.deckPanel.showPanelByIndex(this.previewPanelIndex);
                }
                else {
                    this.deckPanel.showPanelByIndex(this.statisticsPanelIndex);
                }
            };
            ContentItemViewPanel.prototype.getCloseAction = function () {
                return this.closeAction;
            };
            ContentItemViewPanel.prototype.getActions = function () {
                return this.actions;
            };
            return ContentItemViewPanel;
        })(api.app.view.ItemViewPanel);
        view.ContentItemViewPanel = ContentItemViewPanel;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var view;
    (function (view) {
        var DetailsPanel = app.view.detail.DetailsPanel;
        var StringHelper = api.util.StringHelper;
        var ResponsiveManager = api.ui.responsive.ResponsiveManager;
        var MobileContentBrowseToolbar = app.browse.MobileContentBrowseToolbar;
        var MobileContentItemStatisticsPanel = (function (_super) {
            __extends(MobileContentItemStatisticsPanel, _super);
            function MobileContentItemStatisticsPanel(mobileBrowseActions) {
                var _this = this;
                _super.call(this, "mobile-content-item-statistics-panel");
                this.itemHeader = new api.dom.DivEl("mobile-content-item-statistics-header");
                this.headerLabel = new api.dom.SpanEl();
                this.detailsPanel = DetailsPanel.create().setUseSplitter(false).setUseNameAndIconView(false).setSlideFrom(2 /* BOTTOM */).build();
                this.mobileBrowseActions = mobileBrowseActions;
                this.setDoOffset(false);
                this.initHeader();
                this.initPreviewPanel();
                this.initDetailsPanel();
                this.initToolbar();
                this.onRendered(function () {
                    _this.slideAllOut();
                });
                ResponsiveManager.onAvailableSizeChanged(this, function (item) {
                    _this.slideAllOut();
                });
            }
            MobileContentItemStatisticsPanel.prototype.initToolbar = function () {
                this.toolbar = new MobileContentBrowseToolbar(this.mobileBrowseActions);
                this.appendChild(this.toolbar);
            };
            MobileContentItemStatisticsPanel.prototype.initHeader = function () {
                var _this = this;
                this.itemHeader.appendChild(this.headerLabel);
                this.detailsToggleButton = new app.view.detail.MobileDetailsPanelToggleButton(this.detailsPanel);
                var backButton = new api.dom.DivEl("back-button");
                backButton.onClicked(function (event) {
                    _this.slideAllOut();
                });
                this.itemHeader.appendChild(backButton);
                this.itemHeader.appendChild(this.detailsToggleButton);
                this.appendChild(this.itemHeader);
            };
            MobileContentItemStatisticsPanel.prototype.initDetailsPanel = function () {
                this.appendChild(this.detailsPanel);
            };
            MobileContentItemStatisticsPanel.prototype.initPreviewPanel = function () {
                this.previewPanel = new view.ContentItemPreviewPanel();
                this.previewPanel.setDoOffset(false);
                this.appendChild(this.previewPanel);
            };
            MobileContentItemStatisticsPanel.prototype.setItem = function (item) {
                if (!this.getItem() || !this.getItem().equals(item)) {
                    _super.prototype.setItem.call(this, item);
                    this.previewPanel.setItem(item);
                    this.detailsPanel.setItem(item);
                    if (item) {
                        this.setName(this.makeDisplayName(item));
                    }
                }
                this.slideIn();
            };
            MobileContentItemStatisticsPanel.prototype.makeDisplayName = function (item) {
                return StringHelper.isEmpty(item.getDisplayName()) ? StringHelper.escapeHtml("<Unnamed " + this.convertName(item.getModel().getType().getLocalName() + "") + ">") : item.getDisplayName();
            };
            MobileContentItemStatisticsPanel.prototype.convertName = function (name) {
                return StringHelper.capitalize(name.replace(/-/g, " ").trim());
            };
            MobileContentItemStatisticsPanel.prototype.getDetailsPanel = function () {
                return this.detailsPanel;
            };
            MobileContentItemStatisticsPanel.prototype.setName = function (name) {
                this.headerLabel.setHtml(name);
            };
            MobileContentItemStatisticsPanel.prototype.slideAllOut = function () {
                this.slideOut();
                this.detailsPanel.slideOut();
                this.detailsToggleButton.removeClass("expanded");
            };
            MobileContentItemStatisticsPanel.prototype.slideOut = function () {
                this.getEl().setRightPx(-this.getEl().getWidthWithBorder());
            };
            MobileContentItemStatisticsPanel.prototype.slideIn = function () {
                this.getEl().setRightPx(0);
            };
            return MobileContentItemStatisticsPanel;
        })(api.app.view.ItemStatisticsPanel);
        view.MobileContentItemStatisticsPanel = MobileContentItemStatisticsPanel;
    })(view = app.view || (app.view = {}));
})(app || (app = {}));
var app;
(function (app) {
    var publish;
    (function (publish) {
        var PublishContentRequest = api.content.PublishContentRequest;
        var ContentId = api.content.ContentId;
        var ContentPublishDialog = (function (_super) {
            __extends(ContentPublishDialog, _super);
            function ContentPublishDialog() {
                var _this = this;
                _super.call(this, {
                    title: new api.ui.dialog.ModalDialogHeader("Publishing Wizard")
                });
                this.selectionItems = [];
                this.initialItemsView = new PublishDialogItemList();
                this.dependenciesItemsView = new PublishDialogDependantsItemList();
                this.subheaderMessage = new api.dom.H6El("publish-dialog-subheader");
                this.initialContentsResolvedWithChildren = new ContentsResolved();
                this.initialContentsResolvedWithoutChildren = new ContentsResolved();
                this.dependenciesContentsResolvedWithChildren = new ContentsResolved();
                this.dependenciesContentsResolvedWithoutChildren = new ContentsResolved();
                this.modelName = "item";
                this.initIncludeChildrenCheckbox();
                this.getEl().addClass("publish-dialog");
                this.appendChildToContentPanel(this.initialItemsView);
                this.appendChildToContentPanel(this.dependenciesItemsView);
                this.initSubheaderMessage();
                this.publishButton = this.setPublishAction(new ContentPublishDialogAction());
                this.publishButton.setEnabled(false);
                this.getPublishAction().onExecuted(function () {
                    _this.doPublish();
                });
                this.addCancelButtonToBottom();
                this.appendChildToContentPanel(this.includeChildItemsCheck);
            }
            ContentPublishDialog.prototype.initAndOpen = function () {
                var _this = this;
                this.renderSelectedContentsWhileItemsGettingResolved();
                if (!this.atLeastOneInitialItemHasChild()) {
                    this.includeChildItemsCheck.setVisible(false);
                    this.getButtonRow().addClass("no-checkbox");
                }
                this.runResolveTasks(this.getResolveTasks(), function () {
                    _this.centerMyself();
                });
                this.open();
            };
            ContentPublishDialog.prototype.runResolveTasks = function (resolveTasks, doneCallback) {
                var _this = this;
                this.showLoadingSpinnerAtButton();
                this.publishButton.setEnabled(false);
                wemQ.all(resolveTasks).done(function () {
                    if (doneCallback) {
                        doneCallback();
                    }
                    _this.hideLoadingSpinnerAtButton();
                    _this.countItemsToPublishAndUpdateCounterElements();
                });
            };
            ContentPublishDialog.prototype.getResolveTasks = function () {
                return [this.resolvePublishDependenciesIfNeededAndUpdateView()];
            };
            ContentPublishDialog.prototype.show = function () {
                api.dom.Body.get().appendChild(this);
                _super.prototype.show.call(this);
            };
            ContentPublishDialog.prototype.close = function () {
                _super.prototype.close.call(this);
                this.remove();
            };
            ContentPublishDialog.prototype.setPublishAction = function (action) {
                this.publishAction = action;
                return this.addAction(action, true, true);
            };
            ContentPublishDialog.prototype.getPublishAction = function () {
                return this.publishAction;
            };
            ContentPublishDialog.prototype.setSelectedContents = function (contents) {
                this.selectedContents = contents;
            };
            ContentPublishDialog.prototype.renderSelectedContentsWhileItemsGettingResolved = function () {
                var _this = this;
                var initiallySelectedContents = publish.ContentPublishItem.buildPublishItemsFromContentSummaries(this.sortContentSummariesArrayByPath(this.selectedContents).slice(0, 15));
                initiallySelectedContents.forEach(function (content) {
                    var item = new publish.SelectionPublishItemBuilder().create().setViewer(new app.publish.ResolvedPublishContentViewer()).setContent(content).setIsCheckBoxEnabled(false).setRemovable(true).build();
                    _this.initialItemsView.appendChild(item);
                });
            };
            ContentPublishDialog.prototype.sortContentSummariesArrayByPath = function (arrayToSort) {
                arrayToSort.sort(function (contentA, contentB) {
                    var pathA = contentA.getPath().toString(), pathB = contentB.getPath().toString();
                    return (pathA < pathB) ? -1 : (pathA > pathB) ? 1 : 0;
                });
                return arrayToSort;
            };
            ContentPublishDialog.prototype.renderResolvedPublishItems = function () {
                var _this = this;
                this.initSelectionItems();
                this.initialItemsView.clear();
                this.selectionItems.forEach(function (item) {
                    if (!item.isHidden()) {
                        _this.initialItemsView.appendChild(item);
                    }
                });
            };
            ContentPublishDialog.prototype.initSubheaderMessage = function () {
                this.appendChildToTitle(this.subheaderMessage);
                this.subheaderMessage.setHtml("Resolving items...");
            };
            ContentPublishDialog.prototype.initIncludeChildrenCheckbox = function () {
                var _this = this;
                var resolveTasks = [];
                this.includeChildrenCheckedListener = function () {
                    resolveTasks.push(_this.resolvePublishDependenciesIfNeededAndUpdateView());
                    _this.runResolveTasks(resolveTasks);
                };
                this.includeChildItemsCheck = new api.ui.Checkbox();
                this.includeChildItemsCheck.setChecked(false);
                this.includeChildItemsCheck.addClass('include-child-check');
                this.includeChildItemsCheck.onValueChanged(this.includeChildrenCheckedListener);
                this.includeChildItemsCheck.setLabel('Include child items');
            };
            ContentPublishDialog.prototype.initSelectionItems = function () {
                var _this = this;
                this.selectionItems = [];
                var pushRequestedItems = this.includeChildItemsCheck.isChecked() ? this.initialContentsResolvedWithChildren.getContentsResolved() : this.initialContentsResolvedWithoutChildren.getContentsResolved();
                pushRequestedItems.forEach(function (content) {
                    var item = new publish.SelectionPublishItemBuilder().create().setViewer(new app.publish.ResolvedPublishContentViewer()).setContent(content).setIsCheckBoxEnabled(false).setRemovable(true).setRemoveCallback(function () {
                        if (_this.selectionItems.length == 1) {
                            _this.close();
                            return;
                        }
                        _this.removeFromInitialSelection(item);
                        _this.initialContentsResolvedWithChildren.setAlreadyResolved(false);
                        _this.initialContentsResolvedWithoutChildren.setAlreadyResolved(false);
                        _this.dependenciesContentsResolvedWithChildren.setAlreadyResolved(false);
                        _this.dependenciesContentsResolvedWithoutChildren.setAlreadyResolved(false);
                        _this.runResolveTasks(_this.getResolveTasks());
                    }).build();
                    _this.selectionItems.push(item);
                });
            };
            ContentPublishDialog.prototype.removeSelectionItem = function (item) {
                var index = this.indexOf(item);
                if (index < 0) {
                    return;
                }
                this.selectionItems[index].remove();
                this.selectionItems.splice(index, 1);
                if (this.selectionItems.length == 0) {
                    this.close();
                }
            };
            ContentPublishDialog.prototype.indexOf = function (item) {
                for (var i = 0; i < this.selectionItems.length; i++) {
                    if (item.getBrowseItem().getId() == this.selectionItems[i].getBrowseItem().getId()) {
                        return i;
                    }
                }
                return -1;
            };
            ContentPublishDialog.prototype.removeFromInitialSelection = function (item) {
                for (var i = 0; i < this.selectedContents.length; i++) {
                    if (item.getBrowseItem().getId() == this.selectedContents[i].getId()) {
                        this.selectedContents.splice(i, 1);
                        break;
                    }
                }
            };
            ContentPublishDialog.prototype.getSelectedContentsIds = function () {
                return this.selectedContents.filter(function (el) {
                    return (typeof el !== "undefined");
                }).map(function (el) {
                    return new ContentId(el.getId());
                });
            };
            ContentPublishDialog.prototype.restoreUncheckedState = function (unCheckedRequestedContentsIds, publishRequestedSelectionItems) {
                publishRequestedSelectionItems.forEach(function (item) {
                    if (unCheckedRequestedContentsIds.indexOf(item.getBrowseItem().getId()) > -1) {
                        item.setChecked(false);
                    }
                });
            };
            ContentPublishDialog.prototype.resolvePublishDependenciesIfNeededAndUpdateView = function () {
                var _this = this;
                if ((this.includeChildItemsCheck.isChecked() && !this.dependenciesContentsResolvedWithChildren.isAlreadyResolved()) || (!this.includeChildItemsCheck.isChecked() && !this.dependenciesContentsResolvedWithoutChildren.isAlreadyResolved())) {
                    var resolveDependenciesRequest = new api.content.ResolvePublishDependenciesRequest(this.getSelectedContentsIds(), this.includeChildItemsCheck.isChecked());
                    return resolveDependenciesRequest.send().then(function (jsonResponse) {
                        _this.initResolvedPublishItems(jsonResponse.getResult());
                        _this.renderResolvedPublishItems();
                        _this.initResolvedDependenciesItems(jsonResponse.getResult());
                        _this.renderResolvedDependenciesItems();
                    });
                }
                else {
                    this.renderResolvedDependenciesItems();
                    return wemQ(null);
                }
            };
            ContentPublishDialog.prototype.renderResolvedDependenciesItems = function () {
                var _this = this;
                this.dependenciesItemsView.clear();
                var dependenciesItems = this.includeChildItemsCheck.isChecked() ? this.dependenciesContentsResolvedWithChildren : this.dependenciesContentsResolvedWithoutChildren;
                if (dependenciesItems.getContentsResolved().length > 0) {
                    var dependenciesHeader = new api.dom.H6El("dependencies-header");
                    dependenciesHeader.setHtml("Other items that will be published");
                    this.dependenciesItemsView.appendChild(dependenciesHeader);
                }
                dependenciesItems.getContentsResolved().forEach(function (dependency) {
                    var dependencyView = new publish.SelectionPublishItemBuilder().create().setViewer(new app.publish.ResolvedDependantContentViewer()).setContent(dependency).setIsCheckBoxEnabled(false).setIsCheckboxHidden(true).setShowStatus(false).build();
                    _this.dependenciesItemsView.appendDependency(dependencyView);
                });
                if (this.extendsWindowHeightSize()) {
                    this.centerMyself();
                }
            };
            ContentPublishDialog.prototype.extendsWindowHeightSize = function () {
                if (this.getResponsiveItem().isInRangeOrBigger(api.ui.responsive.ResponsiveRanges._540_720)) {
                    var el = this.getEl(), bottomPosition = (el.getTopPx() || parseFloat(el.getComputedProperty('top')) || 0) + el.getMarginTop() + el.getHeightWithBorder() + el.getMarginBottom();
                    if (window.innerHeight < bottomPosition) {
                        return true;
                    }
                }
                return false;
            };
            ContentPublishDialog.prototype.initResolvedPublishItems = function (json) {
                if (this.includeChildItemsCheck.isChecked()) {
                    this.initialContentsResolvedWithChildren.setContentsResolved(publish.ContentPublishItem.fromNewContentPublishItems(json.requestedContents));
                }
                else {
                    this.initialContentsResolvedWithoutChildren.setContentsResolved(publish.ContentPublishItem.fromNewContentPublishItems(json.requestedContents));
                }
            };
            ContentPublishDialog.prototype.initResolvedDependenciesItems = function (json) {
                if (this.includeChildItemsCheck.isChecked()) {
                    this.dependenciesContentsResolvedWithChildren.setContentsResolved(publish.ContentPublishItem.fromNewContentPublishItems(json.dependentContents));
                }
                else {
                    this.dependenciesContentsResolvedWithoutChildren.setContentsResolved(publish.ContentPublishItem.fromNewContentPublishItems(json.dependentContents));
                }
            };
            ContentPublishDialog.prototype.doPublish = function () {
                var _this = this;
                var selectedIds = this.getSelectedContentsIds();
                new PublishContentRequest().setIncludeChildren(this.includeChildItemsCheck.isChecked()).setIds(selectedIds).send().done(function (jsonResponse) {
                    _this.close();
                    PublishContentRequest.feedback(jsonResponse);
                    new api.content.ContentsPublishedEvent(selectedIds).fire();
                });
            };
            ContentPublishDialog.prototype.countItemsToPublishAndUpdateCounterElements = function () {
                var totalCountToPublish = this.getTotalCountToPublish();
                this.updateSubheaderMessage(totalCountToPublish);
                this.updatePublishButton(totalCountToPublish);
            };
            ContentPublishDialog.prototype.getTotalCountToPublish = function () {
                var result = 0;
                result += this.getCountToPublish(this.includeChildItemsCheck.isChecked() ? this.initialContentsResolvedWithChildren : this.initialContentsResolvedWithoutChildren);
                result += this.getCountToPublish(this.includeChildItemsCheck.isChecked() ? this.dependenciesContentsResolvedWithChildren : this.dependenciesContentsResolvedWithoutChildren);
                return result;
            };
            ContentPublishDialog.prototype.getCountToPublish = function (contentsResolved) {
                var result = 0;
                contentsResolved.getContentsResolved().forEach(function (contentPublishItem) {
                    if (contentPublishItem.getCompareStatus() != 6 /* EQUAL */) {
                        result++;
                    }
                });
                return result;
            };
            ContentPublishDialog.prototype.updateSubheaderMessage = function (count) {
                var allValid = this.allResolvedItemsAreValid();
                this.subheaderMessage.setHtml(count == 0 ? "No items to publish" : allValid ? "Your changes are ready for publishing" : "Invalid content(s) prevent publish");
                this.subheaderMessage.toggleClass("invalid", !allValid);
            };
            ContentPublishDialog.prototype.updatePublishButton = function (count) {
                this.cleanPublishButtonText();
                this.publishButton.setLabel(count > 0 ? "Publish (" + count + ")" : "Publish");
                this.publishButton.setEnabled(count > 0 && this.allResolvedItemsAreValid());
            };
            ContentPublishDialog.prototype.contentItemsAreValid = function (contentPublishItems) {
                var result = true;
                contentPublishItems.getContentsResolved().forEach(function (content) {
                    var contentName = content.getName(), invalid = !content.isValid() || !content.getDisplayName() || contentName.isUnnamed();
                    if (invalid) {
                        result = false;
                        return;
                    }
                });
                return result;
            };
            ContentPublishDialog.prototype.atLeastOneInitialItemHasChild = function () {
                return this.selectedContents.some(function (obj) {
                    return obj.hasChildren();
                });
            };
            ContentPublishDialog.prototype.allResolvedItemsAreValid = function () {
                var includeChildItems = this.includeChildItemsCheck.isChecked();
                if (!this.contentItemsAreValid(includeChildItems ? this.initialContentsResolvedWithChildren : this.initialContentsResolvedWithoutChildren)) {
                    return false;
                }
                if (!this.contentItemsAreValid(includeChildItems ? this.dependenciesContentsResolvedWithChildren : this.dependenciesContentsResolvedWithoutChildren)) {
                    return false;
                }
                return true;
            };
            ContentPublishDialog.prototype.showLoadingSpinnerAtButton = function () {
                this.publishButton.addClass("spinner");
            };
            ContentPublishDialog.prototype.hideLoadingSpinnerAtButton = function () {
                this.publishButton.removeClass("spinner");
            };
            ContentPublishDialog.prototype.cleanPublishButtonText = function () {
                this.publishButton.setLabel("Publish");
            };
            return ContentPublishDialog;
        })(api.ui.dialog.ModalDialog);
        publish.ContentPublishDialog = ContentPublishDialog;
        var PublishDialogItemList = (function (_super) {
            __extends(PublishDialogItemList, _super);
            function PublishDialogItemList() {
                _super.call(this);
                this.getEl().addClass("item-list");
                this.getEl().addClass("initial-items");
            }
            PublishDialogItemList.prototype.clear = function () {
                this.removeChildren();
            };
            return PublishDialogItemList;
        })(api.dom.DivEl);
        publish.PublishDialogItemList = PublishDialogItemList;
        var PublishDialogDependantsItemList = (function (_super) {
            __extends(PublishDialogDependantsItemList, _super);
            function PublishDialogDependantsItemList() {
                _super.call(this);
                this.selectionDependenciesItems = [];
                this.getEl().addClass("item-list");
                this.getEl().addClass("dependant-items");
            }
            PublishDialogDependantsItemList.prototype.appendDependency = function (dependencyView) {
                this.appendChild(dependencyView);
                this.selectionDependenciesItems.push(dependencyView);
            };
            PublishDialogDependantsItemList.prototype.clear = function () {
                this.selectionDependenciesItems = [];
                this.removeChildren();
            };
            return PublishDialogDependantsItemList;
        })(api.dom.DivEl);
        publish.PublishDialogDependantsItemList = PublishDialogDependantsItemList;
        var ContentPublishDialogAction = (function (_super) {
            __extends(ContentPublishDialogAction, _super);
            function ContentPublishDialogAction() {
                _super.call(this, "Publish", "enter");
                this.setIconClass("publish-action");
            }
            return ContentPublishDialogAction;
        })(api.ui.Action);
        publish.ContentPublishDialogAction = ContentPublishDialogAction;
        var ContentsResolved = (function () {
            function ContentsResolved() {
                this.alreadyResolved = false;
                this.contentsResolved = [];
            }
            ContentsResolved.prototype.isAlreadyResolved = function () {
                return this.alreadyResolved;
            };
            ContentsResolved.prototype.setAlreadyResolved = function (value) {
                this.alreadyResolved = value;
            };
            ContentsResolved.prototype.setContentsResolved = function (contentsResolvedWithChildren) {
                this.contentsResolved = contentsResolvedWithChildren;
                this.alreadyResolved = true;
            };
            ContentsResolved.prototype.getContentsResolved = function () {
                return this.contentsResolved;
            };
            ContentsResolved.prototype.removeItemWithId = function (id) {
                if (this.alreadyResolved) {
                    for (var i = 0; i < this.contentsResolved.length; i++) {
                        if (id == this.contentsResolved[i].getId()) {
                            this.contentsResolved.splice(i, 1);
                            break;
                        }
                    }
                }
            };
            return ContentsResolved;
        })();
    })(publish = app.publish || (app.publish = {}));
})(app || (app = {}));
var app;
(function (app) {
    var publish;
    (function (publish) {
        var ContentPath = api.content.ContentPath;
        var CompareStatus = api.content.CompareStatus;
        var ContentName = api.content.ContentName;
        var ContentTypeName = api.schema.content.ContentTypeName;
        var ContentPublishItem = (function () {
            function ContentPublishItem(builder) {
                this.displayName = builder.displayName;
                this.path = builder.path;
                this.iconUrl = builder.iconUrl;
                this.id = builder.id;
                this.name = builder.name;
                this.type = builder.type;
                this.valid = builder.valid;
                this.compareStatus = builder.compareStatus;
            }
            ContentPublishItem.prototype.getDisplayName = function () {
                return this.displayName;
            };
            ContentPublishItem.prototype.getCompareStatus = function () {
                return this.compareStatus;
            };
            ContentPublishItem.prototype.hasParent = function () {
                return this.path.hasParentContent();
            };
            ContentPublishItem.prototype.getPath = function () {
                return this.path;
            };
            ContentPublishItem.prototype.getIconUrl = function () {
                return this.iconUrl;
            };
            ContentPublishItem.prototype.getId = function () {
                return this.id;
            };
            ContentPublishItem.prototype.getName = function () {
                return this.name;
            };
            ContentPublishItem.prototype.getType = function () {
                return this.type;
            };
            ContentPublishItem.prototype.isValid = function () {
                return this.valid;
            };
            ContentPublishItem.prototype.isImage = function () {
                return this.type.isImage();
            };
            ContentPublishItem.prototype.equals = function (o) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(o, ContentPublishItem)) {
                    return false;
                }
                var other = o;
                if (!api.ObjectHelper.stringEquals(this.id, other.id)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.displayName, other.displayName)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.path, other.path)) {
                    return false;
                }
                if (!api.ObjectHelper.stringEquals(this.iconUrl, other.iconUrl)) {
                    return false;
                }
                if (this.compareStatus != other.compareStatus) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.name, other.name)) {
                    return false;
                }
                if (!api.ObjectHelper.equals(this.type, other.type)) {
                    return false;
                }
                if (!api.ObjectHelper.booleanEquals(this.valid, other.valid)) {
                    return false;
                }
                return true;
            };
            ContentPublishItem.fromNewContentPublishItems = function (jsonItems) {
                var array = [];
                jsonItems.forEach(function (obj) {
                    array.push(new ContentPublishItemBuilder().fromJson(obj).build());
                });
                return array;
            };
            ContentPublishItem.buildPublishItemsFromContentSummaries = function (items) {
                var array = [];
                items.forEach(function (obj) {
                    array.push(new ContentPublishItemBuilder().fromContentSummary(obj).build());
                });
                return array;
            };
            ContentPublishItem.buildPublishItemFromContentSummary = function (item) {
                return new ContentPublishItemBuilder().fromContentSummary(item).build();
            };
            return ContentPublishItem;
        })();
        publish.ContentPublishItem = ContentPublishItem;
        var ContentPublishItemBuilder = (function () {
            function ContentPublishItemBuilder(source) {
                if (source) {
                    this.id = source.getId();
                    this.displayName = source.getDisplayName();
                    this.path = source.getPath();
                    this.iconUrl = source.getIconUrl();
                    this.compareStatus = source.getCompareStatus();
                    this.name = source.getName();
                    this.type = source.getType();
                    this.valid = source.isValid();
                }
            }
            ContentPublishItemBuilder.prototype.fromJson = function (json) {
                this.id = json.id;
                this.path = ContentPath.fromString(json.path);
                this.compareStatus = CompareStatus[json.compareStatus];
                this.displayName = json.displayName;
                this.iconUrl = json.iconUrl;
                this.name = ContentName.fromString(json.name);
                this.type = new ContentTypeName(json.type);
                this.valid = json.valid;
                return this;
            };
            ContentPublishItemBuilder.prototype.fromContentSummary = function (contentSummary) {
                this.id = contentSummary.getId();
                this.path = contentSummary.getPath();
                this.compareStatus = contentSummary.getContentState().isDefault() ? 0 /* NEW */ : 4 /* PENDING_DELETE */;
                this.displayName = contentSummary.getDisplayName();
                this.iconUrl = contentSummary.getIconUrl();
                this.name = contentSummary.getName();
                this.type = contentSummary.getType();
                this.valid = contentSummary.isValid();
                return this;
            };
            ContentPublishItemBuilder.prototype.setId = function (value) {
                this.id = value;
                return this;
            };
            ContentPublishItemBuilder.prototype.setCompareStatus = function (value) {
                this.compareStatus = value;
                return this;
            };
            ContentPublishItemBuilder.prototype.setPath = function (path) {
                this.path = path;
                return this;
            };
            ContentPublishItemBuilder.prototype.setDisplayName = function (value) {
                this.displayName = value;
                return this;
            };
            ContentPublishItemBuilder.prototype.build = function () {
                return new ContentPublishItem(this);
            };
            return ContentPublishItemBuilder;
        })();
        publish.ContentPublishItemBuilder = ContentPublishItemBuilder;
    })(publish = app.publish || (app.publish = {}));
})(app || (app = {}));
var app;
(function (app) {
    var publish;
    (function (publish) {
        var ContentPath = api.content.ContentPath;
        var ContentIconUrlResolver = api.content.ContentIconUrlResolver;
        var CompareStatus = api.content.CompareStatus;
        var ResolvedPublishContentViewer = (function (_super) {
            __extends(ResolvedPublishContentViewer, _super);
            function ResolvedPublishContentViewer() {
                _super.call(this, "content-resolved-publish-viewer");
            }
            ResolvedPublishContentViewer.prototype.resolveDisplayName = function (object) {
                var contentName = object.getName(), invalid = !object.isValid() || !object.getDisplayName() || contentName.isUnnamed(), pendingDelete = 4 /* PENDING_DELETE */ == object.getCompareStatus() ? true : false;
                this.toggleClass("invalid", invalid);
                this.toggleClass("pending-delete", pendingDelete);
                return object.getDisplayName();
            };
            ResolvedPublishContentViewer.prototype.resolveUnnamedDisplayName = function (object) {
                return object.getType() ? object.getType().getLocalName() : "";
            };
            ResolvedPublishContentViewer.prototype.resolveSubName = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                var contentName = object.getName();
                if (relativePath) {
                    return !contentName.isUnnamed() ? object.getName().toString() : api.ui.NamesAndIconViewer.EMPTY_SUB_NAME;
                }
                else {
                    return !contentName.isUnnamed() ? object.getPath().toString() : ContentPath.fromParent(object.getPath().getParentPath(), api.ui.NamesAndIconViewer.EMPTY_SUB_NAME).toString();
                }
            };
            ResolvedPublishContentViewer.prototype.resolveSubTitle = function (object) {
                return object.getPath().toString();
            };
            ResolvedPublishContentViewer.prototype.resolveIconUrl = function (object) {
                return new ContentIconUrlResolver().setContent(object).resolve();
            };
            return ResolvedPublishContentViewer;
        })(api.ui.NamesAndIconViewer);
        publish.ResolvedPublishContentViewer = ResolvedPublishContentViewer;
        var ResolvedDependantContentViewer = (function (_super) {
            __extends(ResolvedDependantContentViewer, _super);
            function ResolvedDependantContentViewer(className, size) {
                if (size === void 0) { size = 2 /* small */; }
                _super.call(this, className);
                this.namesAndIconView = new DependantNamesAndIconView(size);
                this.addClass("content-resolved-dependant-viewer");
            }
            ResolvedDependantContentViewer.prototype.setObject = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                _super.prototype.setObject.call(this, object);
                var displayName = this.resolveDisplayName(object), iconUrl = this.resolveIconUrl(object);
                this.namesAndIconView.setMainName(displayName);
                if (!!iconUrl) {
                    this.namesAndIconView.setIconUrl(iconUrl);
                }
                this.render();
            };
            ResolvedDependantContentViewer.prototype.resolveDisplayName = function (object) {
                var contentName = object.getName(), invalid = !object.isValid() || !object.getDisplayName() || contentName.isUnnamed(), pendingDelete = 4 /* PENDING_DELETE */ == object.getCompareStatus() ? true : false;
                this.toggleClass("invalid", invalid);
                this.toggleClass("pending-delete", pendingDelete);
                return object.getPath().toString();
            };
            ResolvedDependantContentViewer.prototype.resolveIconUrl = function (object) {
                return new ContentIconUrlResolver().setContent(object).resolve();
            };
            ResolvedDependantContentViewer.prototype.getPreferredHeight = function () {
                return 50;
            };
            ResolvedDependantContentViewer.prototype.doRender = function () {
                this.removeChildren();
                this.appendChild(this.namesAndIconView);
                return true;
            };
            return ResolvedDependantContentViewer;
        })(api.ui.Viewer);
        publish.ResolvedDependantContentViewer = ResolvedDependantContentViewer;
        var DependantNamesAndIconView = (function (_super) {
            __extends(DependantNamesAndIconView, _super);
            function DependantNamesAndIconView(size) {
                _super.call(this, "names-and-icon-view");
                if (size) {
                    this.size = size;
                    this.addClass(api.app.NamesAndIconViewSize[size]);
                }
                this.wrapperDivEl = new api.dom.DivEl("wrapper");
                this.appendChild(this.wrapperDivEl);
                this.iconImageEl = new api.dom.ImgEl(null, "icon");
                this.wrapperDivEl.appendChild(this.iconImageEl);
                this.iconDivEl = new api.dom.DivEl("icon");
                this.wrapperDivEl.appendChild(this.iconDivEl);
                this.iconDivEl.hide();
                this.namesView = new api.dom.SpanEl();
                this.namesView.addClass("name-span");
                this.appendChild(this.namesView);
                this.iconLabelEl = new api.dom.SpanEl("icon-label");
                this.iconLabelEl.hide();
                this.appendChild(this.iconLabelEl);
            }
            DependantNamesAndIconView.prototype.setMainName = function (value) {
                this.namesView.setHtml(value, true);
                return this;
            };
            DependantNamesAndIconView.prototype.setIconClass = function (value) {
                this.iconDivEl.setClass("icon " + value);
                this.iconDivEl.getEl().setDisplay('inline-block');
                this.iconImageEl.hide();
                return this;
            };
            DependantNamesAndIconView.prototype.setIconUrl = function (value) {
                this.iconImageEl.setSrc(value);
                this.iconDivEl.hide();
                this.iconImageEl.show();
                return this;
            };
            return DependantNamesAndIconView;
        })(api.dom.DivEl);
        publish.DependantNamesAndIconView = DependantNamesAndIconView;
    })(publish = app.publish || (app.publish = {}));
})(app || (app = {}));
var app;
(function (app) {
    var publish;
    (function (publish) {
        var BrowseItem = api.app.browse.BrowseItem;
        var CompareStatus = api.content.CompareStatus;
        var SelectionPublishItem = (function (_super) {
            __extends(SelectionPublishItem, _super);
            function SelectionPublishItem(viewer, item, status, isCheckboxHidden, showStatus, changeCallback, toggleCallback, isCheckBoxEnabled, isHidden, isChecked, expandable, removable, removeCallback) {
                if (isCheckBoxEnabled === void 0) { isCheckBoxEnabled = true; }
                if (isHidden === void 0) { isHidden = false; }
                if (isChecked === void 0) { isChecked = true; }
                if (expandable === void 0) { expandable = false; }
                if (removable === void 0) { removable = false; }
                _super.call(this, "browse-selection-publish-item");
                this.childrenDiv = new DependantsItemList();
                this.dependants = [];
                this.viewer = viewer;
                this.item = item;
                this.hidden = isHidden;
                this.isCheckboxHidden = isCheckboxHidden;
                this.expandable = expandable;
                this.removable = removable;
                this.showStatus = showStatus;
                this.initStatusDiv(status);
                this.initCheckBox(isCheckBoxEnabled, isChecked, changeCallback);
                this.initChildrenSwitch(expandable, toggleCallback);
                this.initRemoveButton(removeCallback);
            }
            SelectionPublishItem.prototype.initStatusDiv = function (status) {
                if (this.showStatus) {
                    this.statusDiv = new api.dom.DivEl("status");
                    this.statusDiv.setHtml(api.content.CompareStatusFormatter.formatStatus(status));
                    var statusClass = "" + CompareStatus[status];
                    this.statusDiv.addClass(statusClass.toLowerCase());
                }
            };
            SelectionPublishItem.prototype.initChildrenSwitch = function (expandable, toggleCallback) {
                var _this = this;
                if (expandable) {
                    this.childrenSwitch = new api.dom.SpanEl("toggle icon expand");
                    this.childrenSwitch.onClicked(function (event) {
                        var isExpand = _this.childrenSwitch.hasClass("expand");
                        _this.childrenSwitch.toggleClass("expand");
                        _this.childrenSwitch.toggleClass("collapse");
                        toggleCallback(isExpand);
                    });
                }
            };
            SelectionPublishItem.prototype.initCheckBox = function (isCheckBoxEnabled, isChecked, callback) {
                if (!this.isCheckboxHidden) {
                    this.checkBox = new api.ui.Checkbox();
                    this.checkBox.addClass("checkbox");
                    this.checkBox.setChecked(isChecked);
                    if (!isCheckBoxEnabled) {
                        this.checkBox.setDisabled(true);
                        this.checkBox.addClass("disabled");
                    }
                    else {
                        this.checkBox.onValueChanged(function (event) {
                            if (callback) {
                                callback();
                            }
                        });
                    }
                }
            };
            SelectionPublishItem.prototype.initRemoveButton = function (callback) {
                if (this.removable) {
                    this.removeEl = new api.dom.DivEl("icon remove");
                    this.removeEl.onClicked(function (event) {
                        if (callback) {
                            callback();
                        }
                    });
                }
            };
            SelectionPublishItem.prototype.getBrowseItem = function () {
                return this.item;
            };
            SelectionPublishItem.prototype.isChecked = function () {
                return this.checkBox.isChecked();
            };
            SelectionPublishItem.prototype.isHidden = function () {
                return this.hidden;
            };
            SelectionPublishItem.prototype.isExpandable = function () {
                return this.expandable;
            };
            SelectionPublishItem.prototype.setChecked = function (value) {
                this.checkBox.setChecked(value);
            };
            SelectionPublishItem.prototype.updateViewer = function (viewer) {
                this.viewer = viewer;
                this.render();
            };
            SelectionPublishItem.prototype.appendDependant = function (dependant) {
                this.dependants.push(dependant);
                this.childrenDiv.appendChild(dependant);
                dependant.doRender();
            };
            SelectionPublishItem.prototype.showDependants = function () {
                this.dependants.forEach(function (item) {
                    item.show();
                });
                ;
            };
            SelectionPublishItem.prototype.hideDependants = function () {
                this.dependants.forEach(function (item) {
                    item.hide();
                });
            };
            SelectionPublishItem.prototype.hasDependants = function () {
                return this.dependants.length > 0;
            };
            SelectionPublishItem.prototype.doRender = function () {
                this.removeChildren();
                if (this.expandable) {
                    this.appendChild(this.childrenSwitch);
                }
                if (this.removable) {
                    this.appendChild(this.removeEl);
                }
                if (!this.isCheckboxHidden) {
                    this.appendChild(this.checkBox);
                }
                this.appendChild(this.viewer);
                if (this.showStatus) {
                    this.appendChild(this.statusDiv);
                }
                if (this.expandable) {
                    this.appendChild(this.childrenDiv);
                }
                return true;
            };
            return SelectionPublishItem;
        })(api.dom.DivEl);
        publish.SelectionPublishItem = SelectionPublishItem;
        var DependantsItemList = (function (_super) {
            __extends(DependantsItemList, _super);
            function DependantsItemList() {
                _super.call(this);
                this.getEl().addClass("item-list");
                this.getEl().addClass("dependant-items");
            }
            DependantsItemList.prototype.clear = function () {
                this.removeChildren();
            };
            return DependantsItemList;
        })(api.dom.DivEl);
        publish.DependantsItemList = DependantsItemList;
        var SelectionPublishItemBuilder = (function () {
            function SelectionPublishItemBuilder() {
                this.isCheckBoxEnabled = true;
                this.isHidden = false;
                this.isCheckboxHidden = false;
                this.isChecked = true;
                this.expandable = false;
                this.removable = false;
                this.showStatus = true;
            }
            SelectionPublishItemBuilder.prototype.setViewer = function (viewer) {
                this.viewer = viewer;
                return this;
            };
            SelectionPublishItemBuilder.prototype.setItem = function (item) {
                this.browseItem = item;
                return this;
            };
            SelectionPublishItemBuilder.prototype.setContent = function (content) {
                this.content = content;
                return this;
            };
            SelectionPublishItemBuilder.prototype.setChangeCallback = function (changeCallback) {
                this.changeCallback = changeCallback;
                return this;
            };
            SelectionPublishItemBuilder.prototype.setRemoveCallback = function (removeCallback) {
                this.removeCallback = removeCallback;
                return this;
            };
            SelectionPublishItemBuilder.prototype.setToggleCallback = function (toggleCallback) {
                this.toggleCallback = toggleCallback;
                return this;
            };
            SelectionPublishItemBuilder.prototype.setIsCheckBoxEnabled = function (isCheckBoxEnabled) {
                this.isCheckBoxEnabled = isCheckBoxEnabled;
                return this;
            };
            SelectionPublishItemBuilder.prototype.setIsHidden = function (isHidden) {
                this.isHidden = isHidden;
                return this;
            };
            SelectionPublishItemBuilder.prototype.setShowStatus = function (showStatus) {
                this.showStatus = showStatus;
                return this;
            };
            SelectionPublishItemBuilder.prototype.setIsCheckboxHidden = function (isCheckboxHidden) {
                this.isCheckboxHidden = isCheckboxHidden;
                return this;
            };
            SelectionPublishItemBuilder.prototype.setIsChecked = function (isChecked) {
                this.isChecked = isChecked;
                return this;
            };
            SelectionPublishItemBuilder.prototype.setExpandable = function (expandable) {
                this.expandable = expandable;
                return this;
            };
            SelectionPublishItemBuilder.prototype.setRemovable = function (removable) {
                this.removable = removable;
                return this;
            };
            SelectionPublishItemBuilder.prototype.create = function () {
                return new SelectionPublishItemBuilder();
            };
            SelectionPublishItemBuilder.prototype.build = function () {
                this.viewer.setObject(this.content);
                this.browseItem = new BrowseItem(this.content).setId(this.content.getId()).setDisplayName(this.content.getDisplayName()).setPath(this.content.getPath().toString()).setIconUrl(this.content.getIconUrl());
                return new SelectionPublishItem(this.viewer, this.browseItem, this.content.getCompareStatus(), this.isCheckboxHidden, this.showStatus, this.changeCallback, this.toggleCallback, this.isCheckBoxEnabled, this.isHidden, this.isChecked, this.expandable, this.removable, this.removeCallback);
            };
            return SelectionPublishItemBuilder;
        })();
        publish.SelectionPublishItemBuilder = SelectionPublishItemBuilder;
    })(publish = app.publish || (app.publish = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var action;
        (function (action) {
            var ContentId = api.content.ContentId;
            var DeleteContentAction = (function (_super) {
                __extends(DeleteContentAction, _super);
                function DeleteContentAction(wizardPanel) {
                    _super.call(this, "Delete", "mod+del", true);
                    this.onExecuted(function () {
                        api.ui.dialog.ConfirmationDialog.get().setQuestion("Are you sure you want to delete this content?").setNoCallback(null).setYesCallback(function () {
                            wizardPanel.close();
                            new api.content.DeleteContentRequest().addContentPath(wizardPanel.getPersistedItem().getPath()).sendAndParse().then(function (result) {
                                app.view.DeleteAction.showDeleteResult(result);
                                result.getDeleted().forEach(function (deleted) {
                                    new api.content.ContentDeletedEvent(new ContentId(deleted.getId())).fire();
                                });
                            }).catch(function (reason) {
                                if (reason && reason.message) {
                                    api.notify.showError(reason.message);
                                }
                                else {
                                    api.notify.showError('Content could not be deleted.');
                                }
                            }).done();
                        }).open();
                    });
                }
                return DeleteContentAction;
            })(api.ui.Action);
            action.DeleteContentAction = DeleteContentAction;
        })(action = wizard.action || (wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var action;
        (function (action) {
            var DuplicateContentAction = (function (_super) {
                __extends(DuplicateContentAction, _super);
                function DuplicateContentAction(wizardPanel) {
                    _super.call(this, "Duplicate");
                    this.onExecuted(function () {
                        var source = wizardPanel.getPersistedItem();
                        new api.content.DuplicateContentRequest(source.getContentId()).sendAndParse().then(function (content) {
                            api.notify.showFeedback('\"' + source.getDisplayName() + '\" duplicated');
                        });
                    });
                }
                return DuplicateContentAction;
            })(api.ui.Action);
            action.DuplicateContentAction = DuplicateContentAction;
        })(action = wizard.action || (wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (_wizard) {
        var action;
        (function (action) {
            var PublishAction = (function (_super) {
                __extends(PublishAction, _super);
                function PublishAction(wizard) {
                    var _this = this;
                    _super.call(this, "Publish");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        if (wizard.checkContentCanBePublished(true)) {
                            wizard.setRequireValid(true);
                            _this.setEnabled(false);
                            wizard.saveChanges().then(function (content) {
                                if (content) {
                                    new app.browse.ContentPublishPromptEvent([content]).fire();
                                }
                            }).catch(function (reason) {
                                api.DefaultErrorHandler.handle(reason);
                            }).finally(function () { return _this.setEnabled(true); }).done();
                        }
                        else {
                            api.notify.showWarning('The content cannot be published yet. One or more form values are not valid.');
                        }
                    });
                }
                return PublishAction;
            })(api.ui.Action);
            action.PublishAction = PublishAction;
        })(action = _wizard.action || (_wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (_wizard) {
        var action;
        (function (action) {
            var RenderingMode = api.rendering.RenderingMode;
            var PreviewAction = (function (_super) {
                __extends(PreviewAction, _super);
                function PreviewAction(wizard) {
                    var _this = this;
                    _super.call(this, "Preview");
                    this.onExecuted(function () {
                        if (wizard.hasUnsavedChanges()) {
                            wizard.setRequireValid(true);
                            var previewDialog = window.open('', 'preview');
                            wizard.updatePersistedItem().then(function (content) {
                                previewDialog.location.href = _this.getPreviewDialogURI(content);
                            }).catch(function (reason) { return api.DefaultErrorHandler.handle(reason); }).done();
                        }
                        else {
                            _this.showPreviewDialog(wizard.getPersistedItem());
                        }
                    });
                }
                PreviewAction.prototype.showPreviewDialog = function (content) {
                    window.open(this.getPreviewDialogURI(content), 'preview');
                };
                PreviewAction.prototype.getPreviewDialogURI = function (content) {
                    return api.rendering.UriHelper.getPortalUri(content.getPath().toString(), 1 /* PREVIEW */, 0 /* DRAFT */);
                };
                return PreviewAction;
            })(api.ui.Action);
            action.PreviewAction = PreviewAction;
        })(action = _wizard.action || (_wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (_wizard) {
        var action;
        (function (action) {
            var ShowFormAction = (function (_super) {
                __extends(ShowFormAction, _super);
                function ShowFormAction(wizard) {
                    _super.call(this, "Form");
                    this.setEnabled(true);
                    this.setTitle("Hide Page Editor");
                    this.onExecuted(function () {
                        wizard.showForm();
                        new _wizard.ShowContentFormEvent().fire();
                    });
                }
                return ShowFormAction;
            })(api.ui.Action);
            action.ShowFormAction = ShowFormAction;
        })(action = _wizard.action || (_wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (_wizard) {
        var action;
        (function (action) {
            var ShowLiveEditAction = (function (_super) {
                __extends(ShowLiveEditAction, _super);
                function ShowLiveEditAction(wizard) {
                    _super.call(this, "Live");
                    this.setEnabled(false);
                    this.setTitle("Show Page Editor");
                    this.onExecuted(function () {
                        wizard.showLiveEdit();
                        new _wizard.ShowLiveEditEvent().fire();
                    });
                }
                return ShowLiveEditAction;
            })(api.ui.Action);
            action.ShowLiveEditAction = ShowLiveEditAction;
        })(action = _wizard.action || (_wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (_wizard) {
        var action;
        (function (action) {
            var ShowSplitEditAction = (function (_super) {
                __extends(ShowSplitEditAction, _super);
                function ShowSplitEditAction(wizard) {
                    _super.call(this, "Split");
                    this.setEnabled(false);
                    this.onExecuted(function () {
                        wizard.showSplitEdit();
                        new _wizard.ShowSplitEditEvent().fire();
                    });
                }
                return ShowSplitEditAction;
            })(api.ui.Action);
            action.ShowSplitEditAction = ShowSplitEditAction;
        })(action = _wizard.action || (_wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var action;
        (function (action) {
            var ContentWizardActions = (function (_super) {
                __extends(ContentWizardActions, _super);
                function ContentWizardActions(wizardPanel) {
                    this.save = new api.app.wizard.SaveAction(wizardPanel, "Save draft");
                    this.duplicate = new action.DuplicateContentAction(wizardPanel);
                    this.delete = new action.DeleteContentAction(wizardPanel);
                    this.close = new api.app.wizard.CloseAction(wizardPanel);
                    this.saveAndClose = new api.app.wizard.SaveAndCloseAction(wizardPanel);
                    this.publish = new action.PublishAction(wizardPanel);
                    this.preview = new action.PreviewAction(wizardPanel);
                    this.showLiveEditAction = new action.ShowLiveEditAction(wizardPanel);
                    this.showFormAction = new action.ShowFormAction(wizardPanel);
                    this.showSplitEditAction = new action.ShowSplitEditAction(wizardPanel);
                    _super.call(this, this.save, this.delete, this.duplicate, this.preview, this.publish, this.close, this.showLiveEditAction, this.showFormAction, this.showSplitEditAction, this.saveAndClose);
                }
                ContentWizardActions.prototype.enableActionsForNew = function () {
                    this.save.setEnabled(true);
                    this.duplicate.setEnabled(false);
                    this.delete.setEnabled(true);
                };
                ContentWizardActions.prototype.enableActionsForExisting = function (existing) {
                    this.save.setEnabled(existing.isEditable());
                    this.duplicate.setEnabled(true);
                    this.delete.setEnabled(existing.isDeletable());
                    this.enableActionsForExistingByPermissions(existing);
                };
                ContentWizardActions.prototype.enableActionsForExistingByPermissions = function (existing) {
                    var _this = this;
                    new api.security.auth.IsAuthenticatedRequest().sendAndParse().then(function (loginResult) {
                        var hasModifyPermission = api.security.acl.PermissionHelper.hasPermission(2 /* MODIFY */, loginResult, existing.getPermissions());
                        var hasDeletePermission = api.security.acl.PermissionHelper.hasPermission(3 /* DELETE */, loginResult, existing.getPermissions());
                        var hasPublishPermission = api.security.acl.PermissionHelper.hasPermission(4 /* PUBLISH */, loginResult, existing.getPermissions());
                        if (!hasModifyPermission) {
                            _this.save.setEnabled(false);
                            _this.saveAndClose.setEnabled(false);
                        }
                        if (!hasDeletePermission) {
                            _this.delete.setEnabled(false);
                        }
                        if (!hasPublishPermission) {
                            _this.publish.setEnabled(false);
                        }
                        if (existing.hasParent()) {
                            new api.content.GetContentByPathRequest(existing.getPath().getParentPath()).sendAndParse().then(function (parent) {
                                new api.content.GetContentPermissionsByIdRequest(parent.getContentId()).sendAndParse().then(function (accessControlList) {
                                    var hasParentCreatePermission = api.security.acl.PermissionHelper.hasPermission(1 /* CREATE */, loginResult, accessControlList);
                                    if (!hasParentCreatePermission) {
                                        _this.duplicate.setEnabled(false);
                                    }
                                });
                            });
                        }
                        else {
                            new api.content.GetContentRootPermissionsRequest().sendAndParse().then(function (accessControlList) {
                                var hasParentCreatePermission = api.security.acl.PermissionHelper.hasPermission(1 /* CREATE */, loginResult, accessControlList);
                                if (!hasParentCreatePermission) {
                                    _this.duplicate.setEnabled(false);
                                }
                            });
                        }
                    });
                };
                ContentWizardActions.prototype.getDeleteAction = function () {
                    return this.delete;
                };
                ContentWizardActions.prototype.getSaveAction = function () {
                    return this.save;
                };
                ContentWizardActions.prototype.getDuplicateAction = function () {
                    return this.duplicate;
                };
                ContentWizardActions.prototype.getCloseAction = function () {
                    return this.close;
                };
                ContentWizardActions.prototype.getPublishAction = function () {
                    return this.publish;
                };
                ContentWizardActions.prototype.getPreviewAction = function () {
                    return this.preview;
                };
                ContentWizardActions.prototype.getShowLiveEditAction = function () {
                    return this.showLiveEditAction;
                };
                ContentWizardActions.prototype.getShowFormAction = function () {
                    return this.showFormAction;
                };
                ContentWizardActions.prototype.getShowSplitEditAction = function () {
                    return this.showSplitEditAction;
                };
                return ContentWizardActions;
            })(api.app.wizard.WizardActions);
            action.ContentWizardActions = ContentWizardActions;
        })(action = wizard.action || (wizard.action = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var insert;
                (function (insert) {
                    var Insertable = (function () {
                        function Insertable() {
                        }
                        Insertable.prototype.setName = function (value) {
                            this.name = value;
                            return this;
                        };
                        Insertable.prototype.getName = function () {
                            return this.name;
                        };
                        Insertable.prototype.setDescription = function (value) {
                            this.description = value;
                            return this;
                        };
                        Insertable.prototype.getDescription = function () {
                            return this.description;
                        };
                        Insertable.prototype.setDisplayName = function (value) {
                            this.displayName = value;
                            return this;
                        };
                        Insertable.prototype.getDisplayName = function () {
                            return this.displayName;
                        };
                        Insertable.prototype.setIconCls = function (value) {
                            this.iconCls = value;
                            return this;
                        };
                        Insertable.prototype.getIconCls = function () {
                            return this.iconCls;
                        };
                        return Insertable;
                    })();
                    insert.Insertable = Insertable;
                })(insert = contextwindow.insert || (contextwindow.insert = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var insert;
                (function (insert) {
                    var Insertables = (function () {
                        function Insertables() {
                        }
                        Insertables.IMAGE = new insert.Insertable().setName("image").setDisplayName("Image").setDescription("Upload or use existing images").setIconCls("image");
                        Insertables.PART = new insert.Insertable().setName("part").setDisplayName("Part").setDescription("Advanced components").setIconCls("part");
                        Insertables.LAYOUT = new insert.Insertable().setName("layout").setDisplayName("Layout").setDescription("Customize page layout").setIconCls("layout");
                        Insertables.TEXT = new insert.Insertable().setName("text").setDisplayName("Text").setDescription("Write directly on the page").setIconCls("text");
                        Insertables.ALL = [Insertables.IMAGE, Insertables.PART, Insertables.LAYOUT, Insertables.TEXT];
                        return Insertables;
                    })();
                    insert.Insertables = Insertables;
                })(insert = contextwindow.insert || (contextwindow.insert = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var insert;
                (function (insert) {
                    var DragHelper = api.ui.DragHelper;
                    var InsertablesPanel = (function (_super) {
                        __extends(InsertablesPanel, _super);
                        function InsertablesPanel(config) {
                            var _this = this;
                            _super.call(this, "insertables-panel");
                            this.hideContextWindowRequestListeners = [];
                            this.overIFrame = false;
                            this.liveEditPageProxy = config.liveEditPage;
                            var topDescription = new api.dom.PEl();
                            topDescription.getEl().setInnerHtml('Drag and drop components into the page');
                            this.insertablesDataView = new api.ui.grid.DataView();
                            this.insertablesGrid = new insert.InsertablesGrid(this.insertablesDataView, { draggableRows: true, rowClass: "comp" });
                            this.insertablesDataView.setItems(insert.Insertables.ALL, "name");
                            this.componentsView = new app.wizard.PageComponentsView(config.liveEditPage);
                            this.componentsView.onShown(function () {
                                if (!_this.componentsView.getParentElement()) {
                                    var offset = config.contentWizardPanel.getLiveFormPanel().getEl().getOffsetToParent();
                                    _this.componentsView.getEl().setOffset(offset);
                                    config.contentWizardPanel.appendChild(_this.componentsView);
                                }
                            });
                            this.appendChildren(topDescription, this.insertablesGrid);
                            this.liveEditPageProxy.onLiveEditPageViewReady(function (event) {
                                _this.pageView = event.getPageView();
                            });
                            this.liveEditPageProxy.onComponentViewDragStopped(function () {
                                if (_this.contextWindowDraggable) {
                                    if (InsertablesPanel.debug) {
                                        console.log('Simulating mouse up for', _this.contextWindowDraggable);
                                    }
                                    _this.contextWindowDraggable.simulate('mouseup');
                                }
                            });
                            this.onRendered(this.initializeDraggables.bind(this));
                            this.onRemoved(this.destroyDraggables.bind(this));
                        }
                        InsertablesPanel.prototype.getComponentsView = function () {
                            return this.componentsView;
                        };
                        InsertablesPanel.prototype.setPageView = function (pageView) {
                            this.componentsView.setPageView(pageView);
                        };
                        InsertablesPanel.prototype.setContent = function (content) {
                            this.componentsView.setContent(content);
                        };
                        InsertablesPanel.prototype.initializeDraggables = function () {
                            var _this = this;
                            var components = wemjq('[data-context-window-draggable="true"]:not(.ui-draggable)');
                            if (InsertablesPanel.debug) {
                                console.log('InsertablesPanel.initializeDraggables', components);
                            }
                            components.draggable({
                                cursorAt: DragHelper.CURSOR_AT,
                                appendTo: 'body',
                                cursor: 'move',
                                revert: 'true',
                                distance: 10,
                                scope: 'component',
                                helper: function (event, ui) { return DragHelper.get().getHTMLElement(); },
                                start: function (event, ui) { return _this.handleDragStart(event, ui); },
                                drag: function (event, ui) { return _this.handleDrag(event, ui); },
                                stop: function (event, ui) { return _this.handleDragStop(event, ui); }
                            });
                        };
                        InsertablesPanel.prototype.destroyDraggables = function () {
                            var components = wemjq('[data-context-window-draggable="true"]:not(.ui-draggable)');
                            components.draggable('destroy');
                        };
                        InsertablesPanel.prototype.handleDragStart = function (event, ui) {
                            if (InsertablesPanel.debug) {
                                console.log('InsertablesPanel.handleDragStart', event, ui);
                            }
                            ui.helper.show();
                            this.liveEditPageProxy.getDragMask().show();
                            this.contextWindowDraggable = wemjq(event.target);
                        };
                        InsertablesPanel.prototype.handleDrag = function (event, ui) {
                            var over = this.isOverIFrame(event);
                            if (this.overIFrame != over) {
                                if (over) {
                                    this.onEnterIFrame(event, ui);
                                }
                                else {
                                    this.onLeftIFrame(event, ui);
                                }
                                this.overIFrame = over;
                            }
                        };
                        InsertablesPanel.prototype.handleDragStop = function (event, ui) {
                            if (InsertablesPanel.debug) {
                                console.log('InsertablesPanel.handleDragStop', event, ui);
                            }
                            this.liveEditPageProxy.getDragMask().hide();
                            this.contextWindowDraggable = null;
                            if (this.iFrameDraggable) {
                                this.iFrameDraggable.simulate('mouseup');
                                this.liveEditPageProxy.destroyDraggable(this.iFrameDraggable);
                                this.iFrameDraggable.remove();
                                this.iFrameDraggable = null;
                            }
                        };
                        InsertablesPanel.prototype.isOverIFrame = function (event) {
                            return event.originalEvent.target == this.liveEditPageProxy.getDragMask().getHTMLElement();
                        };
                        InsertablesPanel.prototype.onLeftIFrame = function (event, ui) {
                            if (InsertablesPanel.debug) {
                                console.log('InsertablesPanel.onLeftIFrame');
                            }
                            this.liveEditPageProxy.getDragMask().show();
                            if (this.iFrameDraggable) {
                                var livejq = this.liveEditPageProxy.getJQuery();
                                livejq(this.iFrameDraggable.draggable("option", "helper")()).hide();
                            }
                            ui.helper.show();
                        };
                        InsertablesPanel.prototype.onEnterIFrame = function (event, ui) {
                            if (InsertablesPanel.debug) {
                                console.log('InsertablesPanel.onEnterIFrame');
                            }
                            this.liveEditPageProxy.getDragMask().hide();
                            var livejq = this.liveEditPageProxy.getJQuery();
                            var iFrame = this.liveEditPageProxy.getIFrame().getHTMLElement();
                            var hasBody = iFrame && iFrame.contentDocument && iFrame.contentDocument.body;
                            if (!hasBody) {
                                if (InsertablesPanel.debug) {
                                    console.warn('InsertablesPanel.onEnterIFrame, skip due to missing body in document');
                                }
                                return;
                            }
                            if (!this.iFrameDraggable) {
                                this.iFrameDraggable = livejq(event.target).clone();
                                livejq('body').append(this.iFrameDraggable);
                                this.liveEditPageProxy.createDraggable(this.iFrameDraggable);
                                this.iFrameDraggable.simulate('mousedown').hide();
                            }
                            livejq(this.iFrameDraggable.draggable("option", "helper")()).show();
                            ui.helper.hide();
                            this.notifyHideContextWindowRequest();
                        };
                        InsertablesPanel.prototype.onHideContextWindowRequest = function (listener) {
                            this.hideContextWindowRequestListeners.push(listener);
                        };
                        InsertablesPanel.prototype.unHideContextWindowRequest = function (listener) {
                            this.hideContextWindowRequestListeners = this.hideContextWindowRequestListeners.filter(function (curr) {
                                return curr != listener;
                            });
                        };
                        InsertablesPanel.prototype.notifyHideContextWindowRequest = function () {
                            this.hideContextWindowRequestListeners.forEach(function (listener) {
                                listener();
                            });
                        };
                        InsertablesPanel.debug = false;
                        return InsertablesPanel;
                    })(api.ui.panel.Panel);
                    insert.InsertablesPanel = InsertablesPanel;
                })(insert = contextwindow.insert || (contextwindow.insert = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var insert;
                (function (insert) {
                    var InsertablesGrid = (function (_super) {
                        __extends(InsertablesGrid, _super);
                        function InsertablesGrid(dataView, options) {
                            var _this = this;
                            if (options === void 0) { options = {}; }
                            _super.call(this, dataView, this.createColumns(), this.createOptions());
                            this.componentDataView = dataView;
                            this.componentGridOptions = options;
                            this.onRendered(function (event) {
                                if (_this.componentGridOptions.onClick) {
                                    _this.setOnClick(_this.componentGridOptions.onClick);
                                }
                            });
                        }
                        InsertablesGrid.prototype.createOptions = function () {
                            return new api.ui.grid.GridOptionsBuilder().setHideColumnHeaders(true).setRowHeight(50).setHeight(400).setWidth(320).build();
                        };
                        InsertablesGrid.prototype.createColumns = function () {
                            var _this = this;
                            return [new api.ui.grid.GridColumnBuilder().setName("component").setField("component").setId("component").setWidth(320).setCssClass("grid-row").setFormatter(function (row, cell, value, columnDef, dataContext) {
                                return _this.buildRow(row, cell, value, columnDef, dataContext).toString();
                            }).build()];
                        };
                        InsertablesGrid.prototype.buildRow = function (row, cell, value, columnDef, insertable) {
                            var rowEl = new api.dom.DivEl();
                            rowEl.getEl().setData('portal-component-type', insertable.getName());
                            if (this.componentGridOptions.draggableRows) {
                                rowEl.getEl().setData('context-window-draggable', 'true');
                            }
                            if (this.componentGridOptions.rowClass) {
                                rowEl.addClass(this.componentGridOptions.rowClass);
                            }
                            var icon = new api.dom.DivEl();
                            icon.setClass('live-edit-font-icon-' + insertable.getIconCls());
                            icon.addClass('icon');
                            var title = new api.dom.H5El();
                            title.getEl().setInnerHtml(insertable.getDisplayName());
                            var subtitle = new api.dom.H6El();
                            subtitle.getEl().setInnerHtml(insertable.getDescription());
                            rowEl.appendChild(icon);
                            rowEl.appendChild(title);
                            rowEl.appendChild(subtitle);
                            return rowEl;
                        };
                        return InsertablesGrid;
                    })(api.ui.grid.Grid);
                    insert.InsertablesGrid = InsertablesGrid;
                })(insert = contextwindow.insert || (contextwindow.insert = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var BaseInspectionPanel = (function (_super) {
                        __extends(BaseInspectionPanel, _super);
                        function BaseInspectionPanel() {
                            var _this = this;
                            _super.call(this, "inspection-panel");
                            this.onRendered(function (event) {
                                wemjq(_this.getHTMLElement()).slimScroll({
                                    height: '100%'
                                });
                            });
                        }
                        return BaseInspectionPanel;
                    })(api.ui.panel.Panel);
                    inspect.BaseInspectionPanel = BaseInspectionPanel;
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var NoSelectionInspectionPanel = (function (_super) {
                        __extends(NoSelectionInspectionPanel, _super);
                        function NoSelectionInspectionPanel() {
                            _super.call(this, "inspection-panel");
                            this.header = new api.app.NamesView().setMainName("No item selected");
                            this.appendChild(this.header);
                        }
                        return NoSelectionInspectionPanel;
                    })(api.ui.panel.Panel);
                    inspect.NoSelectionInspectionPanel = NoSelectionInspectionPanel;
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var ContentInspectionPanel = (function (_super) {
                        __extends(ContentInspectionPanel, _super);
                        function ContentInspectionPanel() {
                            _super.call(this);
                            this.namesAndIcon = new api.app.NamesAndIconView(new api.app.NamesAndIconViewBuilder().setSize(1 /* medium */)).setIconClass("live-edit-font-icon-content icon-xlarge");
                            this.appendChild(this.namesAndIcon);
                        }
                        ContentInspectionPanel.prototype.setContent = function (content) {
                            this.content = content;
                            if (content) {
                                this.namesAndIcon.setMainName(content.getDisplayName());
                                this.namesAndIcon.setSubName(content.getPath().toString());
                            }
                            else {
                                this.namesAndIcon.setMainName("[No Content given]");
                                this.namesAndIcon.setSubName("");
                            }
                        };
                        return ContentInspectionPanel;
                    })(inspect.BaseInspectionPanel);
                    inspect.ContentInspectionPanel = ContentInspectionPanel;
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var InspectionsPanel = (function (_super) {
                        __extends(InspectionsPanel, _super);
                        function InspectionsPanel(config) {
                            _super.call(this, 'inspections-panel');
                            this.saveRequestListeners = [];
                            this.deck = new api.ui.panel.DeckPanel();
                            this.noSelectionPanel = new inspect.NoSelectionInspectionPanel();
                            this.imageInspectionPanel = config.imageInspectionPanel;
                            this.partInspectionPanel = config.partInspectionPanel;
                            this.layoutInspectionPanel = config.layoutInspectionPanel;
                            this.contentInspectionPanel = config.contentInspectionPanel;
                            this.pageInspectionPanel = config.pageInspectionPanel;
                            this.regionInspectionPanel = config.regionInspectionPanel;
                            this.deck.addPanel(this.imageInspectionPanel);
                            this.deck.addPanel(this.partInspectionPanel);
                            this.deck.addPanel(this.layoutInspectionPanel);
                            this.deck.addPanel(this.contentInspectionPanel);
                            this.deck.addPanel(this.regionInspectionPanel);
                            this.deck.addPanel(this.pageInspectionPanel);
                            this.deck.addPanel(this.noSelectionPanel);
                            this.deck.showPanel(this.pageInspectionPanel);
                            this.buttons = new api.dom.DivEl('button-bar');
                            var saveButton = new api.ui.button.ActionButton(config.saveAction);
                            this.buttons.appendChild(saveButton);
                            this.appendChildren(this.deck, this.buttons);
                        }
                        InspectionsPanel.prototype.showInspectionPanel = function (panel) {
                            this.deck.showPanel(panel);
                            var showButtons = !(api.ObjectHelper.iFrameSafeInstanceOf(panel, inspect.region.RegionInspectionPanel) || api.ObjectHelper.iFrameSafeInstanceOf(panel, inspect.NoSelectionInspectionPanel));
                            this.buttons.setVisible(showButtons);
                        };
                        InspectionsPanel.prototype.clearInspection = function () {
                            this.showInspectionPanel(this.pageInspectionPanel);
                        };
                        InspectionsPanel.prototype.isInspecting = function () {
                            return this.deck.getPanelShown() != this.noSelectionPanel;
                        };
                        return InspectionsPanel;
                    })(api.ui.panel.Panel);
                    inspect.InspectionsPanel = InspectionsPanel;
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var PageTemplateOption = (function () {
                            function PageTemplateOption(template, pageModel) {
                                this.template = template;
                                this.pageModel = pageModel;
                            }
                            PageTemplateOption.prototype.getPageTemplate = function () {
                                return this.template;
                            };
                            PageTemplateOption.prototype.getPageModel = function () {
                                return this.pageModel;
                            };
                            return PageTemplateOption;
                        })();
                        page.PageTemplateOption = PageTemplateOption;
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var PageTemplateOptionViewer = (function (_super) {
                            __extends(PageTemplateOptionViewer, _super);
                            function PageTemplateOptionViewer() {
                                _super.call(this);
                            }
                            PageTemplateOptionViewer.prototype.resolveDisplayName = function (object) {
                                return !!object.getPageTemplate() ? object.getPageTemplate().getDisplayName() : "Automatic";
                            };
                            PageTemplateOptionViewer.prototype.resolveSubName = function (object, relativePath) {
                                if (relativePath === void 0) { relativePath = false; }
                                if (!!object.getPageTemplate()) {
                                    if (object.getPageTemplate().getDisplayName() == "Customized") {
                                        return "Set up your own page";
                                    }
                                    else {
                                        return object.getPageTemplate().getPath().toString();
                                    }
                                }
                                else if (!!object.getPageModel().getDefaultPageTemplate()) {
                                    return "(" + object.getPageModel().getDefaultPageTemplate().getDisplayName().toString() + ")";
                                }
                                else {
                                    return "( no default template found )";
                                }
                            };
                            PageTemplateOptionViewer.prototype.resolveIconClass = function (object) {
                                return !!object.getPageTemplate() ? object.getPageTemplate().getDisplayName() == "Customized" ? "icon-cog icon-large" : "icon-newspaper icon-large" : "icon-wand icon-large";
                            };
                            PageTemplateOptionViewer.prototype.getPreferredHeight = function () {
                                return 50;
                            };
                            return PageTemplateOptionViewer;
                        })(api.ui.NamesAndIconViewer);
                        page.PageTemplateOptionViewer = PageTemplateOptionViewer;
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var GetPageTemplatesByCanRenderRequest = api.content.page.GetPageTemplatesByCanRenderRequest;
                        var PageTemplateLoader = api.content.page.PageTemplateLoader;
                        var PageTemplateOptions = (function () {
                            function PageTemplateOptions(siteId, contentType, pageModel) {
                                this.siteId = siteId;
                                this.contentType = contentType;
                                this.pageModel = pageModel;
                                this.defaultPageTemplateOption = { value: "__auto__", displayValue: new page.PageTemplateOption(null, pageModel) };
                            }
                            PageTemplateOptions.prototype.getDefault = function () {
                                return this.defaultPageTemplateOption;
                            };
                            PageTemplateOptions.prototype.getOptions = function () {
                                var _this = this;
                                var deferred = wemQ.defer();
                                var options = [];
                                options.push(this.defaultPageTemplateOption);
                                var loader = new PageTemplateLoader(new GetPageTemplatesByCanRenderRequest(this.siteId, this.contentType));
                                loader.setComparator(new api.content.page.PageTemplateByDisplayNameComparator());
                                loader.onLoadedData(function (event) {
                                    event.getData().forEach(function (pageTemplate) {
                                        var indices = [];
                                        indices.push(pageTemplate.getName().toString());
                                        indices.push(pageTemplate.getDisplayName());
                                        indices.push(pageTemplate.getController().toString());
                                        var option = {
                                            value: pageTemplate.getId().toString(),
                                            displayValue: new page.PageTemplateOption(pageTemplate, _this.pageModel),
                                            indices: indices
                                        };
                                        options.push(option);
                                    });
                                    deferred.resolve(options);
                                });
                                loader.load();
                                return deferred.promise;
                            };
                            return PageTemplateOptions;
                        })();
                        page.PageTemplateOptions = PageTemplateOptions;
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var PageModel = api.content.page.PageModel;
                        var PageControllerSelector = (function (_super) {
                            __extends(PageControllerSelector, _super);
                            function PageControllerSelector(model) {
                                var _this = this;
                                _super.call(this, model);
                                this.pageModel = model.getPageModel();
                                this.onLoadedData(function (event) {
                                    if (_this.pageModel.hasController()) {
                                        _this.selectController(_this.pageModel.getController().getKey());
                                    }
                                });
                                this.pageModel.onPropertyChanged(function (event) {
                                    if (event.getPropertyName() == PageModel.PROPERTY_CONTROLLER && _this !== event.getSource()) {
                                        var descriptorKey = event.getNewValue();
                                        if (descriptorKey) {
                                            _this.selectController(descriptorKey);
                                        }
                                    }
                                });
                            }
                            PageControllerSelector.prototype.selectController = function (descriptorKey) {
                                var optionToSelect = this.getOptionByValue(descriptorKey.toString());
                                if (optionToSelect) {
                                    this.selectOption(optionToSelect, true);
                                }
                            };
                            return PageControllerSelector;
                        })(api.content.page.PageDescriptorDropdown);
                        page.PageControllerSelector = PageControllerSelector;
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var PageTemplateBuilder = api.content.page.PageTemplateBuilder;
                        var Dropdown = api.ui.selector.dropdown.Dropdown;
                        var PageModel = api.content.page.PageModel;
                        var PageTemplateSelector = (function (_super) {
                            __extends(PageTemplateSelector, _super);
                            function PageTemplateSelector() {
                                _super.call(this, "pageTemplate", {
                                    optionDisplayValueViewer: new page.PageTemplateOptionViewer()
                                });
                                this.selectionListeners = [];
                                this.customizedSelectedListeners = [];
                            }
                            PageTemplateSelector.prototype.setModel = function (liveEditModel) {
                                var _this = this;
                                this.pageModel = liveEditModel.getPageModel();
                                var pageTemplateOptions = new page.PageTemplateOptions(liveEditModel.getSiteModel().getSiteId(), liveEditModel.getContent().getType(), this.pageModel);
                                pageTemplateOptions.getOptions().then(function (options) {
                                    options.forEach(function (option) {
                                        _this.addOption(option);
                                    });
                                    _this.customizedOption = _this.createCustomizedOption();
                                    _this.addOption(_this.customizedOption);
                                    if (_this.pageModel.isCustomized()) {
                                        _this.selectRow(options.length);
                                    }
                                    else if (_this.pageModel.hasTemplate()) {
                                        _this.selectTemplate(_this.pageModel.getTemplateKey());
                                    }
                                    else {
                                        _this.selectOption(pageTemplateOptions.getDefault(), true);
                                    }
                                    _this.onOptionSelected(function (event) {
                                        var pageTemplate = event.getOption().displayValue.getPageTemplate();
                                        if (pageTemplate && pageTemplate.getDisplayName() == "Customized") {
                                            _this.notifyCustomizedSelected();
                                        }
                                        else {
                                            _this.notifySelection(pageTemplate);
                                        }
                                    });
                                    _this.pageModel.onPropertyChanged(function (event) {
                                        if (event.getPropertyName() == PageModel.PROPERTY_TEMPLATE && _this !== event.getSource()) {
                                            var pageTemplateKey = event.getNewValue();
                                            if (pageTemplateKey) {
                                                _this.selectTemplate(pageTemplateKey);
                                            }
                                            else {
                                                _this.selectOption(pageTemplateOptions.getDefault(), true);
                                            }
                                        }
                                        else if (event.getPropertyName() == PageModel.PROPERTY_CONTROLLER && event.getNewValue()) {
                                            _this.selectCustomized();
                                        }
                                    });
                                    _this.pageModel.onReset(function () {
                                        _this.selectOption(pageTemplateOptions.getDefault(), true);
                                    });
                                }).catch(function (reason) {
                                    api.DefaultErrorHandler.handle(reason);
                                }).done();
                            };
                            PageTemplateSelector.prototype.selectTemplate = function (template) {
                                var optionToSelect = this.getOptionByValue(template.toString());
                                if (optionToSelect) {
                                    this.selectOption(optionToSelect, true);
                                }
                            };
                            PageTemplateSelector.prototype.selectCustomized = function () {
                                this.selectOption(this.customizedOption, true);
                            };
                            PageTemplateSelector.prototype.onSelection = function (listener) {
                                this.selectionListeners.push(listener);
                            };
                            PageTemplateSelector.prototype.unSelection = function (listener) {
                                this.selectionListeners.filter(function (currentListener) {
                                    return listener != currentListener;
                                });
                            };
                            PageTemplateSelector.prototype.notifySelection = function (item) {
                                this.selectionListeners.forEach(function (listener) {
                                    listener(item);
                                });
                            };
                            PageTemplateSelector.prototype.onCustomizedSelected = function (listener) {
                                this.customizedSelectedListeners.push(listener);
                            };
                            PageTemplateSelector.prototype.notifyCustomizedSelected = function () {
                                this.customizedSelectedListeners.forEach(function (listener) {
                                    listener();
                                });
                            };
                            PageTemplateSelector.prototype.createCustomizedOption = function () {
                                var pageTemplate = new PageTemplateBuilder().setData(new api.data.PropertyTree()).setDisplayName("Customized").build();
                                var option = {
                                    value: "Customized",
                                    displayValue: new page.PageTemplateOption(pageTemplate, this.pageModel)
                                };
                                return option;
                            };
                            return PageTemplateSelector;
                        })(Dropdown);
                        page.PageTemplateSelector = PageTemplateSelector;
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var PageControllerForm = (function (_super) {
                            __extends(PageControllerForm, _super);
                            function PageControllerForm(controllerSelector) {
                                _super.call(this, 'page-controller-form');
                                this.controllerSelector = controllerSelector;
                                var fieldSet = new api.ui.form.Fieldset();
                                fieldSet.add(new api.ui.form.FormItemBuilder(controllerSelector).setLabel("Page Controller").build());
                                this.add(fieldSet);
                            }
                            PageControllerForm.prototype.getSelector = function () {
                                return this.controllerSelector;
                            };
                            return PageControllerForm;
                        })(api.ui.form.Form);
                        page.PageControllerForm = PageControllerForm;
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var PageTemplateForm = (function (_super) {
                            __extends(PageTemplateForm, _super);
                            function PageTemplateForm(templateSelector) {
                                _super.call(this, 'page-template-form');
                                this.templateSelector = templateSelector;
                                var fieldSet = new api.ui.form.Fieldset();
                                fieldSet.add(new api.ui.form.FormItemBuilder(templateSelector).setLabel("Renderer").build());
                                this.add(fieldSet);
                            }
                            PageTemplateForm.prototype.getSelector = function () {
                                return this.templateSelector;
                            };
                            return PageTemplateForm;
                        })(api.ui.form.Form);
                        page.PageTemplateForm = PageTemplateForm;
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (_page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var page;
                    (function (page) {
                        var FormContextBuilder = api.form.FormContextBuilder;
                        var FormView = api.form.FormView;
                        var SetTemplate = api.content.page.SetTemplate;
                        var PageModel = api.content.page.PageModel;
                        var PageMode = api.content.page.PageMode;
                        var GetPageDescriptorByKeyRequest = api.content.page.GetPageDescriptorByKeyRequest;
                        var PageInspectionPanel = (function (_super) {
                            __extends(PageInspectionPanel, _super);
                            function PageInspectionPanel() {
                                _super.call(this);
                            }
                            PageInspectionPanel.prototype.setModel = function (liveEditModel) {
                                var _this = this;
                                this.liveEditModel = liveEditModel;
                                this.pageModel = liveEditModel.getPageModel();
                                this.siteModel = liveEditModel.getSiteModel();
                                this.pageTemplateSelector = new page.PageTemplateSelector();
                                this.pageTemplateForm = new page.PageTemplateForm(this.pageTemplateSelector);
                                this.pageTemplateForm.hide();
                                this.appendChild(this.pageTemplateForm);
                                this.pageControllerSelector = new page.PageControllerSelector(liveEditModel);
                                this.pageControllerForm = new page.PageControllerForm(this.pageControllerSelector);
                                this.pageControllerForm.hide();
                                this.appendChild(this.pageControllerForm);
                                this.inspectionHandler = this.pageModel.isPageTemplate() ? new PageTemplateInspectionHandler() : new ContentInspectionHandler();
                                if (!this.pageModel.isPageTemplate()) {
                                    this.pageControllerSelector.load();
                                    if (this.pageModel.isCustomized()) {
                                        this.addClass("customized");
                                    }
                                    if (this.pageModeImpliesPageControllerShown()) {
                                        this.pageControllerForm.show();
                                    }
                                }
                                this.inspectionHandler.setPageModel(this.pageModel).setPageInspectionPanel(this).setPageControllerForm(this.pageControllerForm).setPageTemplateForm(this.pageTemplateForm).setModel(liveEditModel);
                                this.pageTemplateForm.getSelector().onSelection(function (pageTemplate) {
                                    _this.removeClass("customized");
                                    _this.pageModel.setCustomized(false);
                                    if (pageTemplate) {
                                        _this.pageControllerForm.hide();
                                        new GetPageDescriptorByKeyRequest(pageTemplate.getController()).sendAndParse().then(function (pageDescriptor) {
                                            var setTemplate = new SetTemplate(_this).setTemplate(pageTemplate, pageDescriptor);
                                            _this.pageModel.setTemplate(setTemplate, true);
                                        }).catch(function (reason) {
                                            api.DefaultErrorHandler.handle(reason);
                                        }).done();
                                    }
                                    else if (_this.pageModel.hasDefaultPageTemplate()) {
                                        _this.pageControllerForm.hide();
                                        _this.pageModel.setAutomaticTemplate(_this, true);
                                    }
                                    else {
                                        _this.pageModel.reset(_this);
                                    }
                                });
                                this.pageTemplateForm.getSelector().onCustomizedSelected(function () {
                                    _this.addClass("customized");
                                    _this.pageControllerForm.getSelector().reset();
                                    _this.pageControllerForm.show();
                                    _this.pageModel.setCustomized(true);
                                });
                                this.pageModel.onReset(function () {
                                    _this.pageControllerForm.getSelector().reset();
                                    if (!_this.pageModel.isPageTemplate() && !(_this.pageModel.isCustomized() && _this.pageModel.hasController())) {
                                        _this.pageControllerForm.hide();
                                    }
                                });
                            };
                            PageInspectionPanel.prototype.refreshInspectionHandler = function (liveEditModel) {
                                this.inspectionHandler.refreshConfigView(liveEditModel);
                            };
                            PageInspectionPanel.prototype.pageModeImpliesPageControllerShown = function () {
                                return (this.pageModel.isCustomized() && this.pageModel.hasController()) || this.pageModel.getMode() == 2 /* FORCED_CONTROLLER */;
                            };
                            return PageInspectionPanel;
                        })(app.wizard.page.contextwindow.inspect.BaseInspectionPanel);
                        page.PageInspectionPanel = PageInspectionPanel;
                        var BaseInspectionHandler = (function () {
                            function BaseInspectionHandler() {
                            }
                            BaseInspectionHandler.prototype.setPageModel = function (value) {
                                this.pageModel = value;
                                return this;
                            };
                            BaseInspectionHandler.prototype.setPageInspectionPanel = function (value) {
                                this.pageInspectionPanel = value;
                                return this;
                            };
                            BaseInspectionHandler.prototype.setPageControllerForm = function (value) {
                                this.pageControllerForm = value;
                                return this;
                            };
                            BaseInspectionHandler.prototype.setPageTemplateForm = function (value) {
                                this.pageTemplateForm = value;
                                return this;
                            };
                            BaseInspectionHandler.prototype.setModel = function (liveEditModel) {
                                throw new Error("Must be implemented by inheritors");
                            };
                            BaseInspectionHandler.prototype.refreshConfigForm = function (pageDescriptor, config, context) {
                                var _this = this;
                                if (this.configForm) {
                                    this.configForm.remove();
                                    this.configForm = null;
                                }
                                if (!pageDescriptor) {
                                    return;
                                }
                                this.configForm = new FormView(context ? context : new FormContextBuilder().build(), pageDescriptor.getConfig(), config.getRoot());
                                this.pageInspectionPanel.appendChild(this.configForm);
                                this.pageModel.setIgnorePropertyChanges(true);
                                this.configForm.layout().catch(function (reason) {
                                    api.DefaultErrorHandler.handle(reason);
                                }).finally(function () {
                                    _this.pageModel.setIgnorePropertyChanges(false);
                                }).done();
                            };
                            BaseInspectionHandler.prototype.refreshConfigView = function (liveEditModel) {
                            };
                            return BaseInspectionHandler;
                        })();
                        var PageTemplateInspectionHandler = (function (_super) {
                            __extends(PageTemplateInspectionHandler, _super);
                            function PageTemplateInspectionHandler() {
                                _super.apply(this, arguments);
                            }
                            PageTemplateInspectionHandler.prototype.setModel = function (liveEditModel) {
                                var _this = this;
                                var pageModel = liveEditModel.getPageModel();
                                this.pageControllerForm.getSelector().load();
                                this.pageControllerForm.show();
                                this.refreshConfigForm(pageModel.getController(), pageModel.getConfig(), liveEditModel.getFormContext());
                                pageModel.onPropertyChanged(function (event) {
                                    if (event.getPropertyName() == PageModel.PROPERTY_CONTROLLER && _this !== event.getSource()) {
                                        _this.pageControllerForm.show();
                                        _this.refreshConfigForm(pageModel.getController(), pageModel.getConfig(), liveEditModel.getFormContext());
                                    }
                                    else if (event.getPropertyName() == PageModel.PROPERTY_CONFIG && _this !== event.getSource()) {
                                        _this.pageControllerForm.show();
                                        var controller = pageModel.getController();
                                        if (controller) {
                                            _this.refreshConfigForm(controller, pageModel.getConfig(), liveEditModel.getFormContext());
                                        }
                                    }
                                });
                            };
                            return PageTemplateInspectionHandler;
                        })(BaseInspectionHandler);
                        var ContentInspectionHandler = (function (_super) {
                            __extends(ContentInspectionHandler, _super);
                            function ContentInspectionHandler() {
                                _super.apply(this, arguments);
                            }
                            ContentInspectionHandler.prototype.refreshConfigView = function (liveEditModel) {
                                var pageModel = liveEditModel.getPageModel();
                                var pageMode = pageModel.getMode();
                                if (pageMode == 1 /* FORCED_TEMPLATE */) {
                                    this.showPageConfig(pageModel, liveEditModel.getFormContext());
                                }
                                else if (pageMode == 0 /* AUTOMATIC */) {
                                    this.showDefaultPageTemplateConfig(pageModel, liveEditModel.getFormContext());
                                }
                                else {
                                    throw new Error("Unsupported PageMode: " + PageMode[pageMode]);
                                }
                            };
                            ContentInspectionHandler.prototype.setModel = function (liveEditModel) {
                                var pageModel = liveEditModel.getPageModel();
                                if (this.propertyChangedListener) {
                                    pageModel.unPropertyChanged(this.propertyChangedListener);
                                }
                                this.initListener(pageModel, liveEditModel);
                                var pageMode = pageModel.getMode();
                                this.pageTemplateForm.getSelector().setModel(liveEditModel);
                                this.pageTemplateForm.show();
                                if (pageMode == 0 /* AUTOMATIC */) {
                                    this.showDefaultPageTemplateConfig(pageModel, liveEditModel.getFormContext());
                                }
                                else {
                                    this.showPageConfig(pageModel, liveEditModel.getFormContext());
                                }
                                pageModel.onPropertyChanged(this.propertyChangedListener);
                            };
                            ContentInspectionHandler.prototype.initListener = function (pageModel, liveEditModel) {
                                var _this = this;
                                this.propertyChangedListener = function (event) {
                                    if (event.getPropertyName() == "controller" && _this !== event.getSource()) {
                                        _this.pageControllerForm.show();
                                        var controller = pageModel.getController();
                                        if (controller) {
                                            _this.refreshConfigForm(controller, pageModel.getConfig(), liveEditModel.getFormContext());
                                        }
                                    }
                                    else if (event.getPropertyName() == PageModel.PROPERTY_CONFIG && _this !== event.getSource()) {
                                        _this.pageTemplateForm.show();
                                        if (pageModel.getMode() == 0 /* AUTOMATIC */) {
                                            _this.showDefaultPageTemplateConfig(pageModel, liveEditModel.getFormContext());
                                        }
                                        else {
                                            _this.showPageConfig(pageModel, liveEditModel.getFormContext());
                                        }
                                    }
                                };
                            };
                            ContentInspectionHandler.prototype.showPageConfig = function (pageModel, formContext) {
                                this.refreshConfigForm(pageModel.getTemplateDescriptor(), pageModel.getConfig(), formContext);
                            };
                            ContentInspectionHandler.prototype.showDefaultPageTemplateConfig = function (pageModel, formContext) {
                                var controller = pageModel.getDefaultPageTemplateController();
                                var config = pageModel.getConfig();
                                this.refreshConfigForm(controller, config, formContext);
                            };
                            return ContentInspectionHandler;
                        })(BaseInspectionHandler);
                    })(page = inspect.page || (inspect.page = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = _page.contextwindow || (_page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var region;
                    (function (region) {
                        var ComponentInspectionPanel = (function (_super) {
                            __extends(ComponentInspectionPanel, _super);
                            function ComponentInspectionPanel(config) {
                                _super.call(this);
                            }
                            ComponentInspectionPanel.prototype.setModel = function (liveEditModel) {
                                this.liveEditModel = liveEditModel;
                                this.formContext = liveEditModel.getFormContext();
                            };
                            ComponentInspectionPanel.prototype.setComponent = function (component) {
                                this.component = component;
                            };
                            ComponentInspectionPanel.prototype.getComponentView = function () {
                                throw new Error("Must be implemented by inheritors");
                            };
                            ComponentInspectionPanel.prototype.getComponent = function () {
                                return this.component;
                            };
                            return ComponentInspectionPanel;
                        })(app.wizard.page.contextwindow.inspect.BaseInspectionPanel);
                        region.ComponentInspectionPanel = ComponentInspectionPanel;
                    })(region = inspect.region || (inspect.region = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var region;
                    (function (region) {
                        var FormView = api.form.FormView;
                        var DescriptorBasedComponentInspectionPanel = (function (_super) {
                            __extends(DescriptorBasedComponentInspectionPanel, _super);
                            function DescriptorBasedComponentInspectionPanel(config) {
                                _super.call(this, config);
                                this.formView = null;
                            }
                            DescriptorBasedComponentInspectionPanel.prototype.setupComponentForm = function (component, descriptor) {
                                if (this.formView) {
                                    this.removeChild(this.formView);
                                    this.formView = null;
                                }
                                if (!component || !descriptor) {
                                    return;
                                }
                                var form = descriptor.getConfig();
                                var config = component.getConfig();
                                this.formView = new FormView(this.formContext, form, config.getRoot());
                                this.appendChild(this.formView);
                                component.setDisableEventForwarding(true);
                                this.formView.layout().catch(function (reason) {
                                    api.DefaultErrorHandler.handle(reason);
                                }).finally(function () {
                                    component.setDisableEventForwarding(false);
                                }).done();
                            };
                            return DescriptorBasedComponentInspectionPanel;
                        })(region.ComponentInspectionPanel);
                        region.DescriptorBasedComponentInspectionPanel = DescriptorBasedComponentInspectionPanel;
                    })(region = inspect.region || (inspect.region = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var region;
                    (function (region) {
                        var ImageComponent = api.content.page.region.ImageComponent;
                        var GetContentSummaryByIdRequest = api.content.GetContentSummaryByIdRequest;
                        var ContentComboBox = api.content.ContentComboBox;
                        var ContentTypeName = api.schema.content.ContentTypeName;
                        var ImageInspectionPanel = (function (_super) {
                            __extends(ImageInspectionPanel, _super);
                            function ImageInspectionPanel() {
                                var _this = this;
                                _super.call(this, {
                                    iconClass: "live-edit-font-icon-image icon-xlarge"
                                });
                                this.handleSelectorEvents = true;
                                this.imageSelector = ContentComboBox.create().setMaximumOccurrences(1).setAllowedContentTypes([ContentTypeName.IMAGE.toString()]).setLoader(new api.content.ContentSummaryLoader()).build();
                                this.componentPropertyChangedEventHandler = function (event) {
                                    if (event.getPropertyName() == ImageComponent.PROPERTY_IMAGE) {
                                        if (!_this.imageComponent.hasImage()) {
                                            _this.setupComponentForm(_this.imageComponent);
                                            _this.imageSelector.setContent(null);
                                        }
                                    }
                                };
                                this.initSelectorListeners();
                                this.appendChild(this.imageSelector);
                            }
                            ImageInspectionPanel.prototype.setComponent = function (component) {
                                _super.prototype.setComponent.call(this, component);
                            };
                            ImageInspectionPanel.prototype.setImageComponent = function (imageView) {
                                var _this = this;
                                this.imageView = imageView;
                                if (this.imageComponent) {
                                    this.unregisterComponentListeners(this.imageComponent);
                                }
                                this.imageComponent = imageView.getComponent();
                                this.setComponent(this.imageComponent);
                                var contentId = this.imageComponent.getImage();
                                if (contentId) {
                                    var image = this.imageSelector.getContent(contentId);
                                    if (image) {
                                        this.setSelectorValue(image);
                                        this.setupComponentForm(this.imageComponent);
                                    }
                                    else {
                                        new GetContentSummaryByIdRequest(contentId).sendAndParse().then(function (image) {
                                            _this.setSelectorValue(image);
                                            _this.setupComponentForm(_this.imageComponent);
                                        }).catch(function (reason) {
                                            api.DefaultErrorHandler.handle(reason);
                                        }).done();
                                    }
                                }
                                else {
                                    this.setSelectorValue(null);
                                    this.setupComponentForm(this.imageComponent);
                                }
                                this.registerComponentListeners(this.imageComponent);
                            };
                            ImageInspectionPanel.prototype.registerComponentListeners = function (component) {
                                component.onPropertyChanged(this.componentPropertyChangedEventHandler);
                            };
                            ImageInspectionPanel.prototype.unregisterComponentListeners = function (component) {
                                component.unPropertyChanged(this.componentPropertyChangedEventHandler);
                            };
                            ImageInspectionPanel.prototype.setSelectorValue = function (image) {
                                this.handleSelectorEvents = false;
                                this.imageSelector.setContent(image);
                                this.handleSelectorEvents = true;
                            };
                            ImageInspectionPanel.prototype.setupComponentForm = function (imageComponent) {
                                if (this.formView) {
                                    this.removeChild(this.formView);
                                    this.formView = null;
                                }
                                var configData = imageComponent.getConfig();
                                var configForm = imageComponent.getForm();
                                this.formView = new api.form.FormView(this.formContext, configForm, configData.getRoot());
                                this.appendChild(this.formView);
                                imageComponent.setDisableEventForwarding(true);
                                this.formView.layout().catch(function (reason) {
                                    api.DefaultErrorHandler.handle(reason);
                                }).finally(function () {
                                    imageComponent.setDisableEventForwarding(false);
                                }).done();
                            };
                            ImageInspectionPanel.prototype.initSelectorListeners = function () {
                                var _this = this;
                                this.imageSelector.onOptionSelected(function (event) {
                                    if (_this.handleSelectorEvents) {
                                        var option = event.getOption();
                                        var imageContent = option.displayValue;
                                        _this.imageComponent.setImage(imageContent.getContentId(), imageContent.getDisplayName());
                                    }
                                });
                                this.imageSelector.onOptionDeselected(function (option) {
                                    if (_this.handleSelectorEvents) {
                                        _this.imageComponent.setImage(null, null);
                                    }
                                });
                            };
                            ImageInspectionPanel.prototype.getComponentView = function () {
                                return this.imageView;
                            };
                            return ImageInspectionPanel;
                        })(region.ComponentInspectionPanel);
                        region.ImageInspectionPanel = ImageInspectionPanel;
                    })(region = inspect.region || (inspect.region = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var region;
                    (function (region) {
                        var PartDescriptorLoader = api.content.page.region.PartDescriptorLoader;
                        var GetPartDescriptorsByApplicationsRequest = api.content.page.region.GetPartDescriptorsByApplicationsRequest;
                        var GetPartDescriptorByKeyRequest = api.content.page.region.GetPartDescriptorByKeyRequest;
                        var PartDescriptorDropdown = api.content.page.region.PartDescriptorDropdown;
                        var DescriptorBasedComponent = api.content.page.region.DescriptorBasedComponent;
                        var PartInspectionPanel = (function (_super) {
                            __extends(PartInspectionPanel, _super);
                            function PartInspectionPanel() {
                                _super.call(this, {
                                    iconClass: "live-edit-font-icon-part icon-xlarge"
                                });
                                this.handleSelectorEvents = true;
                            }
                            PartInspectionPanel.prototype.setModel = function (liveEditModel) {
                                var _this = this;
                                _super.prototype.setModel.call(this, liveEditModel);
                                var descriptorsRequest = new GetPartDescriptorsByApplicationsRequest(liveEditModel.getSiteModel().getApplicationKeys());
                                var loader = new PartDescriptorLoader(descriptorsRequest);
                                loader.setComparator(new api.content.page.DescriptorByDisplayNameComparator());
                                this.partSelector = new PartDescriptorDropdown("", { loader: loader });
                                this.partForm = new region.DescriptorBasedDropdownForm(this.partSelector, "Part");
                                loader.load();
                                this.componentPropertyChangedEventHandler = function (event) {
                                    if (event.getPropertyName() == DescriptorBasedComponent.PROPERTY_DESCRIPTOR) {
                                        if (!_this.partComponent.hasDescriptor()) {
                                            _this.setupComponentForm(_this.partComponent, null);
                                            _this.partSelector.setDescriptor(null);
                                        }
                                    }
                                };
                                this.initSelectorListeners();
                                this.appendChild(this.partForm);
                            };
                            PartInspectionPanel.prototype.setComponent = function (component, descriptor) {
                                _super.prototype.setComponent.call(this, component);
                                this.partSelector.setDescriptor(descriptor);
                            };
                            PartInspectionPanel.prototype.setSelectorValue = function (descriptor) {
                                this.handleSelectorEvents = false;
                                this.partSelector.setDescriptor(descriptor);
                                this.handleSelectorEvents = true;
                            };
                            PartInspectionPanel.prototype.registerComponentListeners = function (component) {
                                component.onPropertyChanged(this.componentPropertyChangedEventHandler);
                            };
                            PartInspectionPanel.prototype.unregisterComponentListeners = function (component) {
                                component.unPropertyChanged(this.componentPropertyChangedEventHandler);
                            };
                            PartInspectionPanel.prototype.setPartComponent = function (partView) {
                                var _this = this;
                                if (this.partComponent) {
                                    this.unregisterComponentListeners(this.partComponent);
                                }
                                this.partView = partView;
                                this.partComponent = partView.getComponent();
                                this.setComponent(this.partComponent);
                                var key = this.partComponent.getDescriptor();
                                if (key) {
                                    var descriptor = this.partSelector.getDescriptor(key);
                                    if (descriptor) {
                                        this.setSelectorValue(descriptor);
                                        this.setupComponentForm(this.partComponent, descriptor);
                                    }
                                    else {
                                        new GetPartDescriptorByKeyRequest(key).sendAndParse().then(function (descriptor) {
                                            _this.setSelectorValue(descriptor);
                                            _this.setupComponentForm(_this.partComponent, descriptor);
                                        }).catch(function (reason) {
                                            api.DefaultErrorHandler.handle(reason);
                                        }).done();
                                    }
                                }
                                else {
                                    this.setSelectorValue(null);
                                    this.setupComponentForm(this.partComponent, null);
                                }
                                this.registerComponentListeners(this.partComponent);
                            };
                            PartInspectionPanel.prototype.initSelectorListeners = function () {
                                var _this = this;
                                this.partSelector.onOptionSelected(function (event) {
                                    if (_this.handleSelectorEvents) {
                                        var option = event.getOption();
                                        var selectedDescriptorKey = option.displayValue.getKey();
                                        _this.partComponent.setDescriptor(selectedDescriptorKey, option.displayValue);
                                    }
                                });
                            };
                            return PartInspectionPanel;
                        })(region.DescriptorBasedComponentInspectionPanel);
                        region.PartInspectionPanel = PartInspectionPanel;
                    })(region = inspect.region || (inspect.region = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var region;
                    (function (region) {
                        var SiteModel = api.content.site.SiteModel;
                        var DescriptorByDisplayNameComparator = api.content.page.DescriptorByDisplayNameComparator;
                        var DescriptorBasedComponent = api.content.page.region.DescriptorBasedComponent;
                        var GetLayoutDescriptorByKeyRequest = api.content.page.region.GetLayoutDescriptorByKeyRequest;
                        var GetLayoutDescriptorsByApplicationsRequest = api.content.page.region.GetLayoutDescriptorsByApplicationsRequest;
                        var LayoutDescriptorLoader = api.content.page.region.LayoutDescriptorLoader;
                        var LayoutDescriptorDropdown = api.content.page.region.LayoutDescriptorDropdown;
                        var LayoutInspectionPanel = (function (_super) {
                            __extends(LayoutInspectionPanel, _super);
                            function LayoutInspectionPanel() {
                                _super.call(this, {
                                    iconClass: "live-edit-font-icon-layout icon-xlarge"
                                });
                                this.handleSelectorEvents = true;
                            }
                            LayoutInspectionPanel.prototype.setModel = function (liveEditModel) {
                                var _this = this;
                                _super.prototype.setModel.call(this, liveEditModel);
                                var descriptorsRequest = new GetLayoutDescriptorsByApplicationsRequest(liveEditModel.getSiteModel().getApplicationKeys());
                                var loader = new LayoutDescriptorLoader(descriptorsRequest);
                                loader.setComparator(new DescriptorByDisplayNameComparator());
                                this.layoutSelector = new LayoutDescriptorDropdown("", { loader: loader });
                                this.layoutForm = new region.DescriptorBasedDropdownForm(this.layoutSelector, "Layout");
                                loader.load();
                                this.componentPropertyChangedEventHandler = function (event) {
                                    if (event.getPropertyName() == DescriptorBasedComponent.PROPERTY_DESCRIPTOR) {
                                        if (!_this.layoutComponent.hasDescriptor()) {
                                            _this.setupComponentForm(_this.layoutComponent, null);
                                            _this.layoutSelector.setDescriptor(null);
                                        }
                                    }
                                };
                                this.initSelectorListeners();
                                this.appendChild(this.layoutForm);
                                liveEditModel.getSiteModel().onPropertyChanged(function (event) {
                                    if (event.getPropertyName() == SiteModel.PROPERTY_NAME_SITE_CONFIGS) {
                                        descriptorsRequest.setApplicationKeys(liveEditModel.getSiteModel().getApplicationKeys());
                                        loader.load();
                                    }
                                });
                            };
                            LayoutInspectionPanel.prototype.registerComponentListeners = function (component) {
                                component.onPropertyChanged(this.componentPropertyChangedEventHandler);
                            };
                            LayoutInspectionPanel.prototype.unregisterComponentListeners = function (component) {
                                component.unPropertyChanged(this.componentPropertyChangedEventHandler);
                            };
                            LayoutInspectionPanel.prototype.setComponent = function (component, descriptor) {
                                _super.prototype.setComponent.call(this, component);
                                this.layoutSelector.setDescriptor(descriptor);
                            };
                            LayoutInspectionPanel.prototype.setLayoutComponent = function (layoutView) {
                                var _this = this;
                                if (this.layoutComponent) {
                                    this.unregisterComponentListeners(this.layoutComponent);
                                }
                                this.layoutView = layoutView;
                                this.layoutComponent = layoutView.getComponent();
                                this.setComponent(this.layoutComponent);
                                var key = this.layoutComponent.getDescriptor();
                                if (key) {
                                    var descriptor = this.layoutSelector.getDescriptor(key);
                                    if (descriptor) {
                                        this.setSelectorValue(descriptor);
                                        this.setupComponentForm(this.layoutComponent, descriptor);
                                    }
                                    else {
                                        new GetLayoutDescriptorByKeyRequest(key).sendAndParse().then(function (descriptor) {
                                            _this.setSelectorValue(descriptor);
                                            _this.setupComponentForm(_this.layoutComponent, descriptor);
                                        }).catch(function (reason) {
                                            api.DefaultErrorHandler.handle(reason);
                                        }).done();
                                    }
                                }
                                else {
                                    this.setSelectorValue(null);
                                    this.setupComponentForm(this.layoutComponent, null);
                                }
                                this.registerComponentListeners(this.layoutComponent);
                            };
                            LayoutInspectionPanel.prototype.setSelectorValue = function (descriptor) {
                                this.handleSelectorEvents = false;
                                this.layoutSelector.setDescriptor(descriptor);
                                this.handleSelectorEvents = true;
                            };
                            LayoutInspectionPanel.prototype.initSelectorListeners = function () {
                                var _this = this;
                                this.layoutSelector.onOptionSelected(function (event) {
                                    if (_this.handleSelectorEvents) {
                                        var option = event.getOption();
                                        var selectedDescriptorKey = option.displayValue.getKey();
                                        _this.layoutComponent.setDescriptor(selectedDescriptorKey, option.displayValue);
                                    }
                                });
                            };
                            return LayoutInspectionPanel;
                        })(region.DescriptorBasedComponentInspectionPanel);
                        region.LayoutInspectionPanel = LayoutInspectionPanel;
                    })(region = inspect.region || (inspect.region = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var region;
                    (function (_region) {
                        var RegionInspectionPanel = (function (_super) {
                            __extends(RegionInspectionPanel, _super);
                            function RegionInspectionPanel() {
                                _super.call(this);
                                this.namesAndIcon = new api.app.NamesAndIconView(new api.app.NamesAndIconViewBuilder().setSize(1 /* medium */)).setIconClass("live-edit-font-icon-region icon-xlarge");
                                this.appendChild(this.namesAndIcon);
                            }
                            RegionInspectionPanel.prototype.setRegion = function (region) {
                                this.region = region;
                                if (region) {
                                    this.namesAndIcon.setMainName(region.getName());
                                    this.namesAndIcon.setSubName(region.getPath().toString());
                                }
                                else {
                                    this.namesAndIcon.setMainName("[No  Region given]");
                                    this.namesAndIcon.setSubName("");
                                }
                            };
                            return RegionInspectionPanel;
                        })(inspect.BaseInspectionPanel);
                        _region.RegionInspectionPanel = RegionInspectionPanel;
                    })(region = inspect.region || (inspect.region = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var inspect;
                (function (inspect) {
                    var region;
                    (function (region) {
                        var DescriptorBasedDropdownForm = (function (_super) {
                            __extends(DescriptorBasedDropdownForm, _super);
                            function DescriptorBasedDropdownForm(templateSelector, title) {
                                _super.call(this, 'descriptor-based-dropdown-form');
                                this.templateSelector = templateSelector;
                                var fieldSet = new api.ui.form.Fieldset();
                                if (!api.util.StringHelper.isBlank(title)) {
                                    fieldSet.add(new api.ui.form.FormItemBuilder(templateSelector).setLabel(title).build());
                                }
                                else {
                                    fieldSet.add(new api.ui.form.FormItemBuilder(templateSelector).build());
                                }
                                this.add(fieldSet);
                            }
                            DescriptorBasedDropdownForm.prototype.getSelector = function () {
                                return this.templateSelector;
                            };
                            return DescriptorBasedDropdownForm;
                        })(api.ui.form.Form);
                        region.DescriptorBasedDropdownForm = DescriptorBasedDropdownForm;
                    })(region = inspect.region || (inspect.region = {}));
                })(inspect = contextwindow.inspect || (contextwindow.inspect = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var EmulatorGrid = (function (_super) {
                    __extends(EmulatorGrid, _super);
                    function EmulatorGrid(dataView) {
                        _super.call(this, dataView, this.createColumns(), this.createOptions());
                    }
                    EmulatorGrid.prototype.createOptions = function () {
                        return new api.ui.grid.GridOptionsBuilder().setHideColumnHeaders(true).setRowHeight(50).setHeight(450).setWidth(320).build();
                    };
                    EmulatorGrid.prototype.createColumns = function () {
                        var _this = this;
                        return [new api.ui.grid.GridColumnBuilder().setName("device").setField("device").setId("device").setWidth(320).setCssClass("grid-row").setFormatter(function (row, cell, value, columnDef, dataContext) {
                            return _this.buildRow(row, cell, value).toString();
                        }).build()];
                    };
                    EmulatorGrid.prototype.buildRow = function (row, cell, data) {
                        var rowEl = new api.dom.DivEl();
                        rowEl.getEl().setData('width', data.width);
                        rowEl.getEl().setData('height', data.height);
                        rowEl.getEl().setData('units', data.units);
                        var icon = new api.dom.DivEl();
                        icon.setClass('icon-' + data.device_type);
                        icon.addClass('icon');
                        var title = new api.dom.H5El();
                        title.getEl().setInnerHtml(data.name);
                        var subtitle = new api.dom.H6El();
                        var units = data.display_units ? data.units : "";
                        subtitle.getEl().setInnerHtml(data.width + units + " &times; " + data.height + units);
                        rowEl.appendChild(icon);
                        rowEl.appendChild(title);
                        rowEl.appendChild(subtitle);
                        if (data.rotatable == true) {
                            var rotator = new api.dom.DivEl();
                            rotator.addClass('rotate');
                            rotator.addClass('icon-loop');
                            rowEl.appendChild(rotator);
                        }
                        return rowEl;
                    };
                    EmulatorGrid.toSlickData = function (data) {
                        var result = [];
                        var i = 1;
                        data["devices"].forEach(function (item, index) {
                            var tmp = {
                                "id": i,
                                "device": item
                            };
                            result.push(tmp);
                            i++;
                        });
                        return result;
                    };
                    return EmulatorGrid;
                })(api.ui.grid.Grid);
                contextwindow.EmulatorGrid = EmulatorGrid;
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var EmulatorPanel = (function (_super) {
                    __extends(EmulatorPanel, _super);
                    function EmulatorPanel(config) {
                        var _this = this;
                        _super.call(this, "emulator-panel");
                        this.liveEditPage = config.liveEditPage;
                        var text = new api.dom.PEl();
                        text.getEl().setInnerHtml("Emulate different client's physical sizes");
                        this.appendChild(text);
                        this.dataView = new api.ui.grid.DataView();
                        this.grid = new contextwindow.EmulatorGrid(this.dataView);
                        this.appendChild(this.grid);
                        this.getData();
                        wemjq(this.getHTMLElement()).on("click", ".grid-row > div", function (event) {
                            var el = wemjq(event.currentTarget), width = el.data("width"), height = el.data("height"), units = el.data("units");
                            _this.liveEditPage.setWidth(width + units);
                            _this.liveEditPage.setHeight(height + units);
                        });
                        wemjq(this.getHTMLElement()).on("click", ".rotate", function (event) {
                            event.stopPropagation();
                            _this.liveEditPage.setHeightPx(_this.liveEditPage.getHeight());
                            _this.liveEditPage.setWidthPx(_this.liveEditPage.getWidth());
                        });
                    }
                    EmulatorPanel.prototype.getData = function () {
                        var _this = this;
                        wemjq.ajax({
                            url: api.util.UriHelper.getAdminUri("apps/content-manager/js/data/context-window/devices.json"),
                            success: function (data, textStatus, jqXHR) {
                                _this.dataView.setItems(contextwindow.EmulatorGrid.toSlickData(data));
                                _this.grid.setActiveCell(0, 0);
                            }
                        });
                    };
                    return EmulatorPanel;
                })(api.ui.panel.Panel);
                contextwindow.EmulatorPanel = EmulatorPanel;
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var ResponsiveManager = api.ui.responsive.ResponsiveManager;
                var ContextWindow = (function (_super) {
                    __extends(ContextWindow, _super);
                    function ContextWindow(config) {
                        var _this = this;
                        _super.call(this);
                        this.contextWindowState = 1 /* HIDDEN */;
                        this.minWidth = 280;
                        this.parentMinWidth = 15;
                        this.displayModeChangedListeners = [];
                        this.liveFormPanel = config.liveFormPanel;
                        this.inspectionsPanel = config.inspectionPanel;
                        this.emulatorPanel = config.emulatorPanel;
                        this.insertablesPanel = config.insertablesPanel;
                        this.addClass("context-window");
                        this.ghostDragger = new api.dom.DivEl("ghost-dragger");
                        this.splitter = new api.dom.DivEl("splitter");
                        ResponsiveManager.onAvailableSizeChanged(this.liveFormPanel, function (item) {
                            _this.updateFrameSize();
                        });
                        this.appendChild(this.splitter);
                        this.addItem("Insert", false, this.insertablesPanel);
                        this.addItem("Inspect", false, this.inspectionsPanel);
                        this.addItem("Emulator", false, this.emulatorPanel);
                        this.onRendered(function () { return _this.onRenderedHandler(); });
                        this.onRemoved(function (event) {
                            ResponsiveManager.unAvailableSizeChanged(_this);
                            ResponsiveManager.unAvailableSizeChanged(_this.liveFormPanel);
                        });
                    }
                    ContextWindow.prototype.getComponentsView = function () {
                        return this.insertablesPanel.getComponentsView();
                    };
                    ContextWindow.prototype.onRenderedHandler = function () {
                        var _this = this;
                        var initialPos = 0;
                        var splitterPosition = 0;
                        var parent = this.getParentElement();
                        this.actualWidth = this.getEl().getWidth();
                        this.mask = new api.ui.mask.DragMask(parent);
                        var dragListener = function (e) {
                            if (_this.splitterWithinBoundaries(initialPos - e.clientX)) {
                                splitterPosition = e.clientX;
                                _this.ghostDragger.getEl().setLeftPx(e.clientX - _this.getEl().getOffsetLeft());
                            }
                        };
                        this.splitter.onMouseDown(function (e) {
                            e.preventDefault();
                            initialPos = e.clientX;
                            splitterPosition = e.clientX;
                            _this.startDrag(dragListener);
                        });
                        this.mask.onMouseUp(function (e) {
                            _this.actualWidth = _this.getEl().getWidth() + initialPos - splitterPosition;
                            _this.stopDrag(dragListener);
                            ResponsiveManager.fireResizeEvent();
                        });
                        this.addClass('hidden');
                    };
                    ContextWindow.prototype.splitterWithinBoundaries = function (offset) {
                        var newWidth = this.actualWidth + offset;
                        return (newWidth >= this.minWidth) && (newWidth <= this.getParentElement().getEl().getWidth() - this.parentMinWidth);
                    };
                    ContextWindow.prototype.startDrag = function (dragListener) {
                        this.mask.show();
                        this.mask.onMouseMove(dragListener);
                        this.ghostDragger.insertBeforeEl(this.splitter);
                        this.ghostDragger.getEl().setLeftPx(this.splitter.getEl().getOffsetLeftRelativeToParent()).setTop(null);
                    };
                    ContextWindow.prototype.stopDrag = function (dragListener) {
                        this.getEl().setWidthPx(this.actualWidth);
                        this.mask.unMouseMove(dragListener);
                        this.mask.hide();
                        this.removeChild(this.ghostDragger);
                    };
                    ContextWindow.prototype.isShown = function () {
                        return this.contextWindowState == 0 /* SHOWN */;
                    };
                    ContextWindow.prototype.isShownOrAboutToBeShown = function () {
                        return this.contextWindowState == 0 /* SHOWN */ || this.contextWindowState == 2 /* SLIDING_IN */;
                    };
                    ContextWindow.prototype.slideOut = function () {
                        var _this = this;
                        this.getEl().setRightPx(-this.getEl().getWidthWithBorder());
                        if (this.animationTimer) {
                            clearTimeout(this.animationTimer);
                        }
                        this.contextWindowState = 3 /* SLIDING_OUT */;
                        this.animationTimer = setTimeout(function () {
                            _this.getEl().addClass('hidden');
                            _this.contextWindowState = 1 /* HIDDEN */;
                            _this.updateFrameSize();
                            _this.animationTimer = null;
                        }, 100);
                    };
                    ContextWindow.prototype.slideIn = function () {
                        var _this = this;
                        this.getEl().removeClass('hidden').setRightPx(0);
                        if (this.animationTimer) {
                            clearTimeout(this.animationTimer);
                        }
                        this.contextWindowState = 2 /* SLIDING_IN */;
                        this.animationTimer = setTimeout(function () {
                            _this.contextWindowState = 0 /* SHOWN */;
                            _this.updateFrameSize();
                            _this.animationTimer = null;
                        }, 100);
                    };
                    ContextWindow.prototype.showInspectionPanel = function (panel) {
                        this.inspectionsPanel.showInspectionPanel(panel);
                        this.selectPanel(this.inspectionsPanel);
                    };
                    ContextWindow.prototype.clearSelection = function () {
                        this.inspectionsPanel.clearInspection();
                        this.selectPanel(this.insertablesPanel);
                    };
                    ContextWindow.prototype.isInspecting = function () {
                        return this.inspectionsPanel.isInspecting();
                    };
                    ContextWindow.prototype.updateFrameSize = function () {
                        var isFloating = this.isFloating(), displayModeChanged = this.hasClass('floating') && !isFloating, contextWindowWidth = this.actualWidth || this.getEl().getWidth();
                        this.liveFormPanel.updateFrameContainerSize(!isFloating && this.isShown(), contextWindowWidth);
                        this.toggleClass("floating", isFloating);
                        if (displayModeChanged) {
                            this.notifyDisplayModeChanged();
                        }
                    };
                    ContextWindow.prototype.isLiveFormShown = function () {
                        return this.liveFormPanel.isVisible();
                    };
                    ContextWindow.prototype.isFloating = function () {
                        var contextWindowWidth = this.actualWidth || this.getEl().getWidth();
                        var liveFormPanelWidth = this.liveFormPanel.getEl().getWidth();
                        return (liveFormPanelWidth < 1200) || ((liveFormPanelWidth - contextWindowWidth) < 920);
                    };
                    ContextWindow.prototype.notifyDisplayModeChanged = function () {
                        this.displayModeChangedListeners.forEach(function (listener) { return listener(); });
                    };
                    ContextWindow.prototype.onDisplayModeChanged = function (listener) {
                        this.displayModeChangedListeners.push(listener);
                    };
                    ContextWindow.prototype.unDisplayModeChanged = function (listener) {
                        this.displayModeChangedListeners.filter(function (currentListener) {
                            return listener == currentListener;
                        });
                    };
                    return ContextWindow;
                })(api.ui.panel.DockedPanel);
                contextwindow.ContextWindow = ContextWindow;
                var ContextWindowState;
                (function (ContextWindowState) {
                    ContextWindowState[ContextWindowState["SHOWN"] = 0] = "SHOWN";
                    ContextWindowState[ContextWindowState["HIDDEN"] = 1] = "HIDDEN";
                    ContextWindowState[ContextWindowState["SLIDING_IN"] = 2] = "SLIDING_IN";
                    ContextWindowState[ContextWindowState["SLIDING_OUT"] = 3] = "SLIDING_OUT";
                })(ContextWindowState || (ContextWindowState = {}));
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var contextwindow;
            (function (contextwindow) {
                var ContextWindowController = (function () {
                    function ContextWindowController(contextWindow, contextWindowToggler, componentsViewToggler) {
                        var _this = this;
                        this.togglerOverriden = false;
                        this.contextWindow = contextWindow;
                        this.contextWindowToggler = contextWindowToggler;
                        this.componentsViewToggler = componentsViewToggler;
                        var componentsView = this.contextWindow.getComponentsView();
                        this.contextWindowToggler.onClicked(function (event) {
                            if (_this.contextWindowToggler.isEnabled()) {
                                _this.togglerOverriden = true;
                            }
                        });
                        this.contextWindowToggler.onActiveChanged(function (isActive) {
                            if (isActive) {
                                _this.contextWindow.slideIn();
                            }
                            else {
                                _this.contextWindow.slideOut();
                            }
                        });
                        this.componentsViewToggler.onActiveChanged(function (isActive) {
                            componentsView.setVisible(isActive);
                        });
                        componentsView.onHidden(function (event) {
                            _this.componentsViewToggler.setActive(false, true);
                        });
                        var liveEditShownHandler = function () {
                            if (_this.contextWindow.isLiveFormShown()) {
                                _this.contextWindowToggler.setEnabled(true);
                                _this.componentsViewToggler.setEnabled(true);
                            }
                        };
                        var liveEditHiddenHandler = function () {
                            _this.contextWindowToggler.setEnabled(false);
                            _this.componentsViewToggler.setEnabled(false);
                        };
                        app.wizard.ShowLiveEditEvent.on(liveEditShownHandler);
                        app.wizard.ShowSplitEditEvent.on(liveEditShownHandler);
                        app.wizard.ShowContentFormEvent.on(liveEditHiddenHandler);
                    }
                    return ContextWindowController;
                })();
                contextwindow.ContextWindowController = ContextWindowController;
            })(contextwindow = page.contextwindow || (page.contextwindow = {}));
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var DefaultModels = (function () {
                function DefaultModels(pageTemplate, pageDescriptor) {
                    this.pageTemplate = pageTemplate;
                    this.pageDescriptor = pageDescriptor;
                }
                DefaultModels.prototype.hasPageTemplate = function () {
                    return !!this.pageTemplate;
                };
                DefaultModels.prototype.getPageTemplate = function () {
                    return this.pageTemplate ? this.pageTemplate.clone() : null;
                };
                DefaultModels.prototype.getPageDescriptor = function () {
                    return this.pageDescriptor;
                };
                return DefaultModels;
            })();
            page.DefaultModels = DefaultModels;
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var GetPageDescriptorByKeyRequest = api.content.page.GetPageDescriptorByKeyRequest;
            var GetDefaultPageTemplateRequest = api.content.page.GetDefaultPageTemplateRequest;
            var DefaultModelsFactory = (function () {
                function DefaultModelsFactory() {
                }
                DefaultModelsFactory.create = function (config) {
                    return new GetDefaultPageTemplateRequest(config.siteId, config.contentType).sendAndParse().then(function (defaultPageTemplate) {
                        var defaultPageTemplateDescriptorPromise = null;
                        if (defaultPageTemplate && defaultPageTemplate.isPage()) {
                            defaultPageTemplateDescriptorPromise = new GetPageDescriptorByKeyRequest(defaultPageTemplate.getController()).sendAndParse();
                        }
                        else if (defaultPageTemplate && !defaultPageTemplate.isPage()) {
                            defaultPageTemplate = null;
                        }
                        var deferred = wemQ.defer();
                        if (defaultPageTemplateDescriptorPromise) {
                            defaultPageTemplateDescriptorPromise.then(function (defaultPageTemplateDescriptor) {
                                deferred.resolve(new page.DefaultModels(defaultPageTemplate, defaultPageTemplateDescriptor));
                            }).catch(function (reason) {
                                deferred.reject(new api.Exception("Page descriptor '" + defaultPageTemplate.getController() + "' not found.", 2 /* WARNING */));
                            }).done();
                        }
                        else {
                            deferred.resolve(new page.DefaultModels(defaultPageTemplate, null));
                        }
                        return deferred.promise;
                    });
                };
                return DefaultModelsFactory;
            })();
            page.DefaultModelsFactory = DefaultModelsFactory;
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var ImageUploadDialog = api.content.form.inputtype.image.ImageUploadDialog;
            var RenderingMode = api.rendering.RenderingMode;
            var Workspace = api.content.Branch;
            var ImageOpenUploadDialogEvent = api.liveedit.ImageOpenUploadDialogEvent;
            var ImageUploadedEvent = api.liveedit.ImageUploadedEvent;
            var LiveEditPageViewReadyEvent = api.liveedit.LiveEditPageViewReadyEvent;
            var ComponentViewDragStartedEvent = api.liveedit.ComponentViewDragStartedEvent;
            var ComponentViewDragStoppedEvent = api.liveedit.ComponentViewDragStoppedEvent;
            var ComponentViewDragCanceledEvent = api.liveedit.ComponentViewDragCanceledEvent;
            var ComponentViewDragDroppedEvent = api.liveedit.ComponentViewDragDroppedEvent;
            var PageSelectedEvent = api.liveedit.PageSelectedEvent;
            var PageLockedEvent = api.liveedit.PageLockedEvent;
            var PageUnlockedEvent = api.liveedit.PageUnlockedEvent;
            var PageUnloadedEvent = api.liveedit.PageUnloadedEvent;
            var PageTextModeStartedEvent = api.liveedit.PageTextModeStartedEvent;
            var RegionSelectedEvent = api.liveedit.RegionSelectedEvent;
            var ItemViewSelectedEvent = api.liveedit.ItemViewSelectedEvent;
            var ItemViewDeselectedEvent = api.liveedit.ItemViewDeselectedEvent;
            var ComponentAddedEvent = api.liveedit.ComponentAddedEvent;
            var ComponentRemovedEvent = api.liveedit.ComponentRemovedEvent;
            var ComponentDuplicatedEvent = api.liveedit.ComponentDuplicatedEvent;
            var ComponentLoadedEvent = api.liveedit.ComponentLoadedEvent;
            var ComponentResetEvent = api.liveedit.ComponentResetEvent;
            var LiveEditPageInitializationErrorEvent = api.liveedit.LiveEditPageInitializationErrorEvent;
            var CreateItemViewConfig = api.liveedit.CreateItemViewConfig;
            var LiveEditPageProxy = (function () {
                function LiveEditPageProxy() {
                    var _this = this;
                    this.loadMaskCanBeShown = false;
                    this.loadedListeners = [];
                    this.componentViewDragStartedListeners = [];
                    this.componentViewDragStoppedListeners = [];
                    this.componentViewDragCanceledListeners = [];
                    this.componentViewDragDroppedListeners = [];
                    this.pageSelectedListeners = [];
                    this.pageLockedListeners = [];
                    this.pageUnlockedListeners = [];
                    this.pageUnloadedListeners = [];
                    this.pageTextModeStartedListeners = [];
                    this.regionSelectedListeners = [];
                    this.itemViewSelectedListeners = [];
                    this.itemViewDeselectedListeners = [];
                    this.componentAddedListeners = [];
                    this.componentRemovedListeners = [];
                    this.componentDuplicatedListeners = [];
                    this.componentLoadedListeners = [];
                    this.componentResetListeners = [];
                    this.liveEditPageViewReadyListeners = [];
                    this.liveEditPageInitErrorListeners = [];
                    this.LIVE_EDIT_ERROR_PAGE_BODY_ID = "wem-error-page";
                    this.liveEditIFrame = new api.dom.IFrameEl("live-edit-frame");
                    this.liveEditIFrame.onLoaded(function () { return _this.handleIFrameLoadedEvent(); });
                    this.liveEditIFrame.onShown(function () {
                        if (!_this.loadMaskCanBeShown && _this.liveEditFrameIsVisible()) {
                            _this.loadMask.show();
                            _this.loadMaskCanBeShown = true;
                        }
                    });
                    this.loadMask = new api.ui.mask.LoadMask(this.liveEditIFrame);
                    this.dragMask = new api.ui.mask.DragMask(this.liveEditIFrame);
                    wizard.ShowContentFormEvent.on(function () {
                        if (_this.loadMask.isVisible()) {
                            _this.loadMask.hide();
                        }
                    });
                }
                LiveEditPageProxy.prototype.setModel = function (liveEditModel) {
                    this.liveEditModel = liveEditModel;
                };
                LiveEditPageProxy.prototype.setWidth = function (value) {
                    this.liveEditIFrame.getEl().setWidth(value);
                };
                LiveEditPageProxy.prototype.setWidthPx = function (value) {
                    this.liveEditIFrame.getEl().setWidthPx(value);
                };
                LiveEditPageProxy.prototype.setHeight = function (value) {
                    this.liveEditIFrame.getEl().setHeight(value);
                };
                LiveEditPageProxy.prototype.setHeightPx = function (value) {
                    this.liveEditIFrame.getEl().setHeightPx(value);
                };
                LiveEditPageProxy.prototype.getWidth = function () {
                    return this.liveEditIFrame.getEl().getWidth();
                };
                LiveEditPageProxy.prototype.getHeight = function () {
                    return this.liveEditIFrame.getEl().getHeight();
                };
                LiveEditPageProxy.prototype.getIFrame = function () {
                    return this.liveEditIFrame;
                };
                LiveEditPageProxy.prototype.getLoadMask = function () {
                    return this.loadMask;
                };
                LiveEditPageProxy.prototype.getJQuery = function () {
                    return this.livejq;
                };
                LiveEditPageProxy.prototype.createDraggable = function (item) {
                    this.liveEditWindow.api.liveedit.DragAndDrop.get().createDraggable(item);
                };
                LiveEditPageProxy.prototype.destroyDraggable = function (item) {
                    this.liveEditWindow.api.liveedit.DragAndDrop.get().destroyDraggable(item);
                };
                LiveEditPageProxy.prototype.getDragMask = function () {
                    return this.dragMask;
                };
                LiveEditPageProxy.prototype.remove = function () {
                    this.dragMask.remove();
                    this.loadMask.remove();
                };
                LiveEditPageProxy.prototype.load = function () {
                    if (this.loadMaskCanBeShown) {
                        this.loadMask.show();
                    }
                    var contentId = this.liveEditModel.getContent().getContentId().toString();
                    var pageUrl = api.rendering.UriHelper.getPortalUri(contentId, 0 /* EDIT */, 0 /* DRAFT */);
                    if (LiveEditPageProxy.debug) {
                        console.log("LiveEditPageProxy.load pageUrl: " + pageUrl);
                    }
                    this.liveEditIFrame.setSrc(pageUrl);
                };
                LiveEditPageProxy.prototype.skipNextReloadConfirmation = function (skip) {
                    new api.liveedit.SkipLiveEditReloadConfirmationEvent(skip).fire(this.liveEditWindow);
                };
                LiveEditPageProxy.prototype.handleIFrameLoadedEvent = function () {
                    var liveEditWindow = this.liveEditIFrame.getHTMLElement()["contentWindow"];
                    if (liveEditWindow && liveEditWindow.wemjq) {
                        liveEditWindow.CONFIG = { baseUri: CONFIG.baseUri };
                        this.livejq = liveEditWindow.wemjq;
                        if (this.liveEditWindow) {
                            this.stopListening(this.liveEditWindow);
                        }
                        this.liveEditWindow = liveEditWindow;
                        this.listenToPage(this.liveEditWindow);
                        this.loadMask.hide();
                        new api.liveedit.InitializeLiveEditEvent(this.liveEditModel).fire(this.liveEditWindow);
                    }
                    else if (!liveEditWindow.document.body || (liveEditWindow.document.body.id == this.LIVE_EDIT_ERROR_PAGE_BODY_ID)) {
                        this.loadMask.hide();
                    }
                    this.loadMaskCanBeShown = true;
                    this.notifyLoaded();
                };
                LiveEditPageProxy.prototype.loadComponent = function (componentView, componentUrl) {
                    var _this = this;
                    var deferred = wemQ.defer();
                    api.util.assertNotNull(componentView, "componentView cannot be null");
                    api.util.assertNotNull(componentUrl, "componentUrl cannot be null");
                    wemjq.ajax({
                        url: componentUrl,
                        type: 'GET',
                        success: function (htmlAsString) {
                            var newElement = api.dom.Element.fromString(htmlAsString);
                            var itemViewIdProducer = componentView.getItemViewIdProducer();
                            var createViewConfig = new CreateItemViewConfig().setItemViewProducer(itemViewIdProducer).setParentView(componentView.getParentItemView()).setData(componentView.getComponent()).setElement(newElement);
                            var newComponentView = componentView.getType().createView(createViewConfig);
                            componentView.replaceWith(newComponentView);
                            var event = new ComponentLoadedEvent(newComponentView, componentView);
                            event.fire(_this.liveEditWindow);
                            _this.notifyComponentLoaded(event);
                            newComponentView.select();
                            deferred.resolve("");
                        },
                        error: function (jqXHR, textStatus, errorThrow) {
                            var responseHtml = wemjq.parseHTML(jqXHR.responseText);
                            var errorMessage = "";
                            responseHtml.forEach(function (el, i) {
                                if (el.tagName && el.tagName.toLowerCase() == "title") {
                                    errorMessage = el.innerHTML;
                                }
                            });
                            deferred.reject(errorMessage);
                        }
                    });
                    return deferred.promise;
                };
                LiveEditPageProxy.prototype.stopListening = function (contextWindow) {
                    ImageOpenUploadDialogEvent.un(null, contextWindow);
                    ComponentViewDragStartedEvent.un(null, contextWindow);
                    ComponentViewDragStoppedEvent.un(null, contextWindow);
                    ComponentViewDragCanceledEvent.un(null, contextWindow);
                    ComponentViewDragDroppedEvent.un(null, contextWindow);
                    PageSelectedEvent.un(null, contextWindow);
                    PageLockedEvent.un(null, contextWindow);
                    PageUnlockedEvent.un(null, contextWindow);
                    PageUnloadedEvent.un(null, contextWindow);
                    PageTextModeStartedEvent.un(null, contextWindow);
                    RegionSelectedEvent.un(null, contextWindow);
                    ItemViewSelectedEvent.un(null, contextWindow);
                    ItemViewDeselectedEvent.un(null, contextWindow);
                    ComponentAddedEvent.un(null, contextWindow);
                    ComponentRemovedEvent.un(null, contextWindow);
                    ComponentDuplicatedEvent.un(null, contextWindow);
                    ComponentLoadedEvent.un(null, contextWindow);
                    ComponentResetEvent.un(null, contextWindow);
                    LiveEditPageViewReadyEvent.un(null, contextWindow);
                    LiveEditPageInitializationErrorEvent.un(null, contextWindow);
                };
                LiveEditPageProxy.prototype.listenToPage = function (contextWindow) {
                    var _this = this;
                    ImageOpenUploadDialogEvent.on(function (openDialogEvent) {
                        var imageUploadDialog = new ImageUploadDialog(_this.liveEditModel.getContent().getContentId());
                        imageUploadDialog.onImageUploaded(function (event) {
                            new ImageUploadedEvent(event.getUploadItem().getModel(), openDialogEvent.getTargetImagePlaceholder()).fire(contextWindow);
                            imageUploadDialog.close();
                            imageUploadDialog.remove();
                        });
                        imageUploadDialog.open();
                    }, contextWindow);
                    ComponentViewDragStartedEvent.on(this.notifyComponentViewDragStarted.bind(this), contextWindow);
                    ComponentViewDragStoppedEvent.on(this.notifyComponentViewDragStopped.bind(this), contextWindow);
                    ComponentViewDragCanceledEvent.on(this.notifyComponentViewDragCanceled.bind(this), contextWindow);
                    ComponentViewDragDroppedEvent.on(this.notifyComponentViewDragDropped.bind(this), contextWindow);
                    PageSelectedEvent.on(this.notifyPageSelected.bind(this), contextWindow);
                    PageLockedEvent.on(this.notifyPageLocked.bind(this), contextWindow);
                    PageUnlockedEvent.on(this.notifyPageUnlocked.bind(this), contextWindow);
                    PageUnloadedEvent.on(this.notifyPageUnloaded.bind(this), contextWindow);
                    PageTextModeStartedEvent.on(this.notifyPageTextModeStarted.bind(this), contextWindow);
                    RegionSelectedEvent.on(this.notifyRegionSelected.bind(this), contextWindow);
                    ItemViewSelectedEvent.on(this.notifyItemViewSelected.bind(this), contextWindow);
                    ItemViewDeselectedEvent.on(this.notifyItemViewDeselected.bind(this), contextWindow);
                    ComponentAddedEvent.on(this.notifyComponentAdded.bind(this), contextWindow);
                    ComponentRemovedEvent.on(this.notifyComponentRemoved.bind(this), contextWindow);
                    ComponentDuplicatedEvent.on(this.notifyComponentDuplicated.bind(this), contextWindow);
                    ComponentLoadedEvent.on(this.notifyComponentLoaded.bind(this), contextWindow);
                    ComponentResetEvent.on(this.notifyComponentReset.bind(this), contextWindow);
                    LiveEditPageViewReadyEvent.on(this.notifyLiveEditPageViewReady.bind(this), contextWindow);
                    LiveEditPageInitializationErrorEvent.on(this.notifyLiveEditPageInitializationError.bind(this), contextWindow);
                };
                LiveEditPageProxy.prototype.onLoaded = function (listener) {
                    this.loadedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unLoaded = function (listener) {
                    this.loadedListeners = this.loadedListeners.filter(function (curr) {
                        return curr != listener;
                    });
                };
                LiveEditPageProxy.prototype.notifyLoaded = function () {
                    this.loadedListeners.forEach(function (listener) {
                        listener();
                    });
                };
                LiveEditPageProxy.prototype.onComponentViewDragStarted = function (listener) {
                    this.componentViewDragStartedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unComponentViewDragStarted = function (listener) {
                    this.componentViewDragStartedListeners = this.componentViewDragStartedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyComponentViewDragStarted = function (event) {
                    this.componentViewDragStartedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onComponentViewDragStopped = function (listener) {
                    this.componentViewDragStoppedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unComponentViewDragStopped = function (listener) {
                    this.componentViewDragStoppedListeners = this.componentViewDragStoppedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyComponentViewDragStopped = function (event) {
                    this.componentViewDragStoppedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onComponentViewDragCanceled = function (listener) {
                    this.componentViewDragCanceledListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unComponentViewDragCanceled = function (listener) {
                    this.componentViewDragCanceledListeners = this.componentViewDragCanceledListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyComponentViewDragCanceled = function (event) {
                    this.componentViewDragCanceledListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onComponentViewDragDropped = function (listener) {
                    this.componentViewDragDroppedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unComponentViewDragDropped = function (listener) {
                    this.componentViewDragDroppedListeners = this.componentViewDragDroppedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyComponentViewDragDropped = function (event) {
                    this.componentViewDragDroppedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onPageSelected = function (listener) {
                    this.pageSelectedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unPageSelected = function (listener) {
                    this.pageSelectedListeners = this.pageSelectedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyPageSelected = function (event) {
                    this.pageSelectedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onPageLocked = function (listener) {
                    this.pageLockedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unPageLocked = function (listener) {
                    this.pageLockedListeners = this.pageLockedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyPageLocked = function (event) {
                    this.pageLockedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onPageUnlocked = function (listener) {
                    this.pageUnlockedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unPageUnlocked = function (listener) {
                    this.pageUnlockedListeners = this.pageUnlockedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyPageUnlocked = function (event) {
                    this.pageUnlockedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onPageUnloaded = function (listener) {
                    this.pageUnloadedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unPageUnloaded = function (listener) {
                    this.pageUnloadedListeners = this.pageUnloadedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyPageUnloaded = function (event) {
                    this.pageUnloadedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onPageTextModeStarted = function (listener) {
                    this.pageTextModeStartedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unPageTextModeStarted = function (listener) {
                    this.pageTextModeStartedListeners = this.pageTextModeStartedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyPageTextModeStarted = function (event) {
                    this.pageTextModeStartedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onRegionSelected = function (listener) {
                    this.regionSelectedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unRegionSelected = function (listener) {
                    this.regionSelectedListeners = this.regionSelectedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyRegionSelected = function (event) {
                    this.regionSelectedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onItemViewSelected = function (listener) {
                    this.itemViewSelectedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unItemViewSelected = function (listener) {
                    this.itemViewSelectedListeners = this.itemViewSelectedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyItemViewSelected = function (event) {
                    this.itemViewSelectedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onItemViewDeselected = function (listener) {
                    this.itemViewDeselectedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unItemViewDeselected = function (listener) {
                    this.itemViewDeselectedListeners = this.itemViewDeselectedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyItemViewDeselected = function (event) {
                    this.itemViewDeselectedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onComponentAdded = function (listener) {
                    this.componentAddedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unComponentAdded = function (listener) {
                    this.componentAddedListeners = this.componentAddedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyComponentAdded = function (event) {
                    this.componentAddedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onComponentRemoved = function (listener) {
                    this.componentRemovedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unComponentRemoved = function (listener) {
                    this.componentRemovedListeners = this.componentRemovedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyComponentRemoved = function (event) {
                    this.componentRemovedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onComponentDuplicated = function (listener) {
                    this.componentDuplicatedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unComponentDuplicated = function (listener) {
                    this.componentDuplicatedListeners = this.componentDuplicatedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyComponentDuplicated = function (event) {
                    this.componentDuplicatedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onComponentLoaded = function (listener) {
                    this.componentLoadedListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unComponentLoaded = function (listener) {
                    this.componentLoadedListeners = this.componentLoadedListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyComponentLoaded = function (event) {
                    this.componentLoadedListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onComponentReset = function (listener) {
                    this.componentResetListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unComponentReset = function (listener) {
                    this.componentResetListeners = this.componentResetListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyComponentReset = function (event) {
                    this.componentResetListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onLiveEditPageViewReady = function (listener) {
                    this.liveEditPageViewReadyListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unLiveEditPageViewReady = function (listener) {
                    this.liveEditPageViewReadyListeners = this.liveEditPageViewReadyListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyLiveEditPageViewReady = function (event) {
                    this.liveEditPageViewReadyListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.onLiveEditPageInitializationError = function (listener) {
                    this.liveEditPageInitErrorListeners.push(listener);
                };
                LiveEditPageProxy.prototype.unLiveEditPageInitializationError = function (listener) {
                    this.liveEditPageInitErrorListeners = this.liveEditPageInitErrorListeners.filter(function (curr) { return (curr != listener); });
                };
                LiveEditPageProxy.prototype.notifyLiveEditPageInitializationError = function (event) {
                    this.liveEditPageInitErrorListeners.forEach(function (listener) { return listener(event); });
                };
                LiveEditPageProxy.prototype.liveEditFrameIsVisible = function () {
                    return this.liveEditIFrame.getEl().getWidthWithBorder() > 0 && this.liveEditIFrame.getEl().getHeightWithBorder() > 0;
                };
                LiveEditPageProxy.debug = false;
                return LiveEditPageProxy;
            })();
            page.LiveEditPageProxy = LiveEditPageProxy;
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var page;
        (function (page) {
            var PageMode = api.content.page.PageMode;
            var PageModel = api.content.page.PageModel;
            var ImageComponent = api.content.page.region.ImageComponent;
            var DescriptorBasedComponent = api.content.page.region.DescriptorBasedComponent;
            var InspectionsPanel = app.wizard.page.contextwindow.inspect.InspectionsPanel;
            var ContentInspectionPanel = app.wizard.page.contextwindow.inspect.ContentInspectionPanel;
            var PageInspectionPanel = app.wizard.page.contextwindow.inspect.page.PageInspectionPanel;
            var RegionInspectionPanel = app.wizard.page.contextwindow.inspect.region.RegionInspectionPanel;
            var ImageInspectionPanel = app.wizard.page.contextwindow.inspect.region.ImageInspectionPanel;
            var PartInspectionPanel = app.wizard.page.contextwindow.inspect.region.PartInspectionPanel;
            var LayoutInspectionPanel = app.wizard.page.contextwindow.inspect.region.LayoutInspectionPanel;
            var ContextWindow = app.wizard.page.contextwindow.ContextWindow;
            var EmulatorPanel = app.wizard.page.contextwindow.EmulatorPanel;
            var InsertablesPanel = app.wizard.page.contextwindow.insert.InsertablesPanel;
            var RenderingMode = api.rendering.RenderingMode;
            var ComponentView = api.liveedit.ComponentView;
            var PageView = api.liveedit.PageView;
            var ImageComponentView = api.liveedit.image.ImageComponentView;
            var PartComponentView = api.liveedit.part.PartComponentView;
            var LayoutComponentView = api.liveedit.layout.LayoutComponentView;
            var TextComponentView = api.liveedit.text.TextComponentView;
            var Panel = api.ui.panel.Panel;
            var LiveFormPanel = (function (_super) {
                __extends(LiveFormPanel, _super);
                function LiveFormPanel(config) {
                    var _this = this;
                    _super.call(this, "live-form-panel");
                    this.contentWizardPanel = config.contentWizardPanel;
                    this.defaultModels = config.defaultModels;
                    this.pageLoading = false;
                    this.pageSkipReload = false;
                    this.lockPageAfterProxyLoad = false;
                    this.liveEditPageProxy = new page.LiveEditPageProxy();
                    this.contentInspectionPanel = new ContentInspectionPanel();
                    this.pageInspectionPanel = new PageInspectionPanel();
                    this.regionInspectionPanel = new RegionInspectionPanel();
                    this.imageInspectionPanel = new ImageInspectionPanel();
                    this.partInspectionPanel = new PartInspectionPanel();
                    this.layoutInspectionPanel = new LayoutInspectionPanel();
                    api.dom.WindowDOM.get().onBeforeUnload(function (event) {
                        console.log("onbeforeunload " + _this.liveEditModel.getContent().getDisplayName());
                        _this.liveEditPageProxy.skipNextReloadConfirmation(true);
                    });
                    var saveAction = new api.ui.Action('Apply');
                    saveAction.onExecuted(function () {
                        if (!_this.pageView) {
                            _this.saveAndReloadPage();
                            return;
                        }
                        var itemView = _this.pageView.getSelectedView();
                        if (api.ObjectHelper.iFrameSafeInstanceOf(itemView, ComponentView)) {
                            _this.saveAndReloadOnlyComponent(itemView);
                        }
                        else if (_this.pageView.isLocked() || api.ObjectHelper.iFrameSafeInstanceOf(itemView, PageView)) {
                            _this.saveAndReloadPage();
                        }
                    });
                    this.inspectionsPanel = new InspectionsPanel({
                        contentInspectionPanel: this.contentInspectionPanel,
                        pageInspectionPanel: this.pageInspectionPanel,
                        regionInspectionPanel: this.regionInspectionPanel,
                        imageInspectionPanel: this.imageInspectionPanel,
                        partInspectionPanel: this.partInspectionPanel,
                        layoutInspectionPanel: this.layoutInspectionPanel,
                        saveAction: saveAction
                    });
                    this.emulatorPanel = new EmulatorPanel({
                        liveEditPage: this.liveEditPageProxy
                    });
                    this.insertablesPanel = new InsertablesPanel({
                        liveEditPage: this.liveEditPageProxy,
                        contentWizardPanel: config.contentWizardPanel,
                    });
                    this.frameContainer = new Panel("frame-container");
                    this.appendChild(this.frameContainer);
                    this.frameContainer.appendChild(this.liveEditPageProxy.getIFrame());
                    this.appendChild(this.liveEditPageProxy.getLoadMask());
                    this.contextWindow = new ContextWindow({
                        liveEditPage: this.liveEditPageProxy,
                        liveFormPanel: this,
                        inspectionPanel: this.inspectionsPanel,
                        emulatorPanel: this.emulatorPanel,
                        insertablesPanel: this.insertablesPanel
                    });
                    this.appendChild(this.contextWindow);
                    this.contextWindowController = new app.wizard.page.contextwindow.ContextWindowController(this.contextWindow, this.contentWizardPanel.getContextWindowToggler(), this.contentWizardPanel.getComponentsViewToggler());
                    this.liveEditListen();
                }
                LiveFormPanel.prototype.remove = function () {
                    this.liveEditPageProxy.remove();
                    _super.prototype.remove.call(this);
                    return this;
                };
                LiveFormPanel.prototype.getPage = function () {
                    return this.pageModel.getPage();
                };
                LiveFormPanel.prototype.setModel = function (liveEditModel) {
                    var _this = this;
                    this.liveEditModel = liveEditModel;
                    this.content = liveEditModel.getContent();
                    this.insertablesPanel.setContent(this.content);
                    this.pageModel = liveEditModel.getPageModel();
                    this.pageModel.setIgnorePropertyChanges(true);
                    this.liveEditPageProxy.setModel(liveEditModel);
                    this.imageInspectionPanel.setModel(liveEditModel);
                    this.pageInspectionPanel.setModel(liveEditModel);
                    this.partInspectionPanel.setModel(liveEditModel);
                    this.layoutInspectionPanel.setModel(liveEditModel);
                    this.imageInspectionPanel.setModel(liveEditModel);
                    this.pageModel.setIgnorePropertyChanges(false);
                    this.pageModel.onPropertyChanged(function (event) {
                        if (event.getPropertyName() == PageModel.PROPERTY_CONTROLLER && _this !== event.getSource()) {
                            _this.saveAndReloadPage(false);
                        }
                        else if (event.getPropertyName() == PageModel.PROPERTY_TEMPLATE && _this !== event.getSource()) {
                            if (!(_this.pageModel.getDefaultPageTemplate().equals(_this.pageModel.getTemplate()) && !event.getOldValue() && !_this.pageModel.hasController())) {
                                _this.pageInspectionPanel.refreshInspectionHandler(liveEditModel);
                                _this.lockPageAfterProxyLoad = true;
                                _this.saveAndReloadPage(false);
                            }
                        }
                    });
                    this.pageModel.onComponentPropertyChangedEvent(function (event) {
                        if (api.ObjectHelper.iFrameSafeInstanceOf(event.getComponent(), DescriptorBasedComponent)) {
                            if (event.getPropertyName() == DescriptorBasedComponent.PROPERTY_DESCRIPTOR) {
                                var componentView = _this.pageView.getComponentViewByPath(event.getPath());
                                if (componentView) {
                                    if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, PartComponentView)) {
                                        var partView = componentView;
                                        var partComponent = partView.getComponent();
                                        if (partComponent.hasDescriptor()) {
                                            _this.saveAndReloadOnlyComponent(componentView);
                                        }
                                    }
                                    else if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, LayoutComponentView)) {
                                        var layoutView = componentView;
                                        var layoutComponent = layoutView.getComponent();
                                        if (layoutComponent.hasDescriptor()) {
                                            _this.saveAndReloadOnlyComponent(componentView);
                                        }
                                    }
                                }
                                else {
                                    console.debug("ComponentView by path not found: " + event.getPath().toString());
                                }
                            }
                        }
                        else if (api.ObjectHelper.iFrameSafeInstanceOf(event.getComponent(), ImageComponent)) {
                            if (event.getPropertyName() == ImageComponent.PROPERTY_IMAGE && !event.getComponent().isEmpty()) {
                                var componentView = _this.pageView.getComponentViewByPath(event.getPath());
                                if (componentView) {
                                    _this.saveAndReloadOnlyComponent(componentView);
                                }
                            }
                        }
                    });
                };
                LiveFormPanel.prototype.skipNextReloadConfirmation = function (skip) {
                    this.liveEditPageProxy.skipNextReloadConfirmation(skip);
                };
                LiveFormPanel.prototype.loadPage = function () {
                    var _this = this;
                    if (this.pageSkipReload == false && !this.pageLoading) {
                        this.clearSelection();
                        this.pageLoading = true;
                        this.liveEditPageProxy.load();
                        this.liveEditPageProxy.onLoaded(function () {
                            _this.pageLoading = false;
                            if (_this.lockPageAfterProxyLoad) {
                                _this.pageView.setLocked(true);
                                _this.lockPageAfterProxyLoad = false;
                            }
                        });
                    }
                };
                LiveFormPanel.prototype.saveAndReloadPage = function (clearInspection) {
                    var _this = this;
                    if (clearInspection === void 0) { clearInspection = false; }
                    this.pageSkipReload = true;
                    this.contentWizardPanel.saveChanges().then(function () {
                        _this.pageSkipReload = false;
                        if (clearInspection) {
                            _this.contextWindow.clearSelection();
                        }
                        _this.liveEditPageProxy.load();
                    }).catch(function (reason) { return api.DefaultErrorHandler.handle(reason); }).done();
                };
                LiveFormPanel.prototype.saveAndReloadOnlyComponent = function (componentView) {
                    var _this = this;
                    api.util.assertNotNull(componentView, "componentView cannot be null");
                    this.pageSkipReload = true;
                    var componentUrl = api.rendering.UriHelper.getComponentUri(this.content.getContentId().toString(), componentView.getComponentPath().toString(), 0 /* EDIT */, 0 /* DRAFT */);
                    this.contentWizardPanel.saveChanges().then(function () {
                        _this.pageSkipReload = false;
                        componentView.showLoadingSpinner();
                        return _this.liveEditPageProxy.loadComponent(componentView, componentUrl);
                    }).catch(function (errorMessage) {
                        api.DefaultErrorHandler.handle(errorMessage);
                        componentView.hideLoadingSpinner();
                        componentView.showRenderingError(componentUrl, errorMessage);
                    }).done();
                };
                LiveFormPanel.prototype.updateFrameContainerSize = function (contextWindowShown, contextWindowWidth) {
                    if (contextWindowShown && contextWindowWidth) {
                        this.frameContainer.getEl().setWidth("calc(100% - " + (contextWindowWidth - 1) + "px)");
                    }
                    else {
                        this.frameContainer.getEl().setWidth("100%");
                    }
                };
                LiveFormPanel.prototype.liveEditListen = function () {
                    var _this = this;
                    this.liveEditPageProxy.onPageLocked(function (event) {
                        _this.inspectPage();
                    });
                    this.liveEditPageProxy.onPageUnlocked(function (event) {
                        _this.contextWindow.clearSelection();
                    });
                    this.liveEditPageProxy.onLiveEditPageViewReady(function (event) {
                        _this.pageView = event.getPageView();
                        _this.insertablesPanel.setPageView(_this.pageView);
                    });
                    this.liveEditPageProxy.onPageSelected(function (event) {
                        _this.inspectPage();
                    });
                    this.liveEditPageProxy.onRegionSelected(function (event) {
                        _this.inspectRegion(event.getRegionView());
                    });
                    this.liveEditPageProxy.onItemViewSelected(function (event) {
                        var itemView = event.getItemView();
                        var toggler = _this.contentWizardPanel.getContextWindowToggler();
                        if (api.ObjectHelper.iFrameSafeInstanceOf(itemView, ComponentView)) {
                            if (event.isNew() && !toggler.isActive()) {
                                toggler.setActive(true);
                            }
                            _this.inspectComponent(itemView);
                        }
                    });
                    this.liveEditPageProxy.onItemViewDeselected(function (event) {
                        var toggler = _this.contentWizardPanel.getContextWindowToggler();
                        if (!toggler.isActive() && _this.contextWindow.isShownOrAboutToBeShown()) {
                            _this.contextWindow.slideOut();
                        }
                        else if (toggler.isActive() && !_this.contextWindow.isShownOrAboutToBeShown()) {
                            _this.contextWindow.slideIn();
                        }
                        _this.clearSelection();
                    });
                    this.liveEditPageProxy.onComponentAdded(function (event) {
                    });
                    this.liveEditPageProxy.onComponentRemoved(function (event) {
                        if (!_this.pageModel.isPageTemplate() && _this.pageModel.getMode() == 0 /* AUTOMATIC */) {
                            _this.pageModel.initializePageFromDefault(_this);
                        }
                        _this.clearSelection();
                    });
                    this.liveEditPageProxy.onComponentViewDragDropped(function (event) {
                        var componentView = event.getComponentView();
                        if (!componentView.isEmpty()) {
                            _this.inspectComponent(componentView);
                        }
                    });
                    this.liveEditPageProxy.onComponentDuplicated(function (event) {
                        _this.saveAndReloadOnlyComponent(event.getDuplicatedComponentView());
                    });
                    this.liveEditPageProxy.onLiveEditPageInitializationError(function (event) {
                        api.notify.showError(event.getMessage(), false);
                        new app.wizard.ShowContentFormEvent().fire();
                        _this.contentWizardPanel.showForm();
                    });
                };
                LiveFormPanel.prototype.inspectPage = function () {
                    this.contextWindow.showInspectionPanel(this.pageInspectionPanel);
                };
                LiveFormPanel.prototype.clearSelection = function () {
                    var pageModel = this.liveEditModel.getPageModel();
                    var customizedWithController = pageModel.isCustomized() && pageModel.hasController();
                    if (pageModel.hasDefaultPageTemplate() || customizedWithController) {
                        this.contextWindow.clearSelection();
                    }
                    else {
                        this.inspectPage();
                    }
                };
                LiveFormPanel.prototype.clearPageViewSelectionAndOpenInspectPage = function () {
                    if (this.pageView && this.pageView.hasSelectedView()) {
                        this.pageView.getSelectedView().deselect();
                    }
                    this.contextWindow.showInspectionPanel(this.pageInspectionPanel);
                };
                LiveFormPanel.prototype.inspectRegion = function (regionView) {
                    var region = regionView.getRegion();
                    this.regionInspectionPanel.setRegion(region);
                    this.contextWindow.showInspectionPanel(this.regionInspectionPanel);
                };
                LiveFormPanel.prototype.inspectComponent = function (componentView) {
                    api.util.assertNotNull(componentView, "componentView cannot be null");
                    if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, ImageComponentView)) {
                        this.imageInspectionPanel.setImageComponent(componentView);
                        this.contextWindow.showInspectionPanel(this.imageInspectionPanel);
                    }
                    else if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, PartComponentView)) {
                        this.partInspectionPanel.setPartComponent(componentView);
                        this.contextWindow.showInspectionPanel(this.partInspectionPanel);
                    }
                    else if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, LayoutComponentView)) {
                        this.layoutInspectionPanel.setLayoutComponent(componentView);
                        this.contextWindow.showInspectionPanel(this.layoutInspectionPanel);
                    }
                    else if (api.ObjectHelper.iFrameSafeInstanceOf(componentView, TextComponentView)) {
                    }
                    else {
                        throw new Error("ComponentView cannot be selected: " + api.ClassHelper.getClassName(componentView));
                    }
                };
                return LiveFormPanel;
            })(api.ui.panel.Panel);
            page.LiveFormPanel = LiveFormPanel;
        })(page = wizard.page || (wizard.page = {}));
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ContentSettingsModel = (function () {
            function ContentSettingsModel(content) {
                this.propertyChangedListeners = [];
                this.language = content.getLanguage();
                this.owner = content.getOwner();
            }
            ContentSettingsModel.prototype.getOwner = function () {
                return this.owner;
            };
            ContentSettingsModel.prototype.setOwner = function (owner, silent) {
                if (!silent) {
                    this.notifyPropertyChanged(new api.PropertyChangedEvent(ContentSettingsModel.PROPERTY_OWNER, this.owner, owner));
                }
                this.owner = owner;
                return this;
            };
            ContentSettingsModel.prototype.getLanguage = function () {
                return this.language;
            };
            ContentSettingsModel.prototype.setLanguage = function (lang, silent) {
                if (!silent) {
                    this.notifyPropertyChanged(new api.PropertyChangedEvent(ContentSettingsModel.PROPERTY_LANG, this.language, lang));
                }
                this.language = lang;
                return this;
            };
            ContentSettingsModel.prototype.onPropertyChanged = function (listener) {
                this.propertyChangedListeners.push(listener);
            };
            ContentSettingsModel.prototype.unPropertyChanged = function (listener) {
                this.propertyChangedListeners = this.propertyChangedListeners.filter(function (curr) { return (curr != listener); });
            };
            ContentSettingsModel.prototype.notifyPropertyChanged = function (event) {
                this.propertyChangedListeners.forEach(function (listener) { return listener(event); });
            };
            ContentSettingsModel.prototype.equals = function (other) {
                if (!api.ObjectHelper.iFrameSafeInstanceOf(other, ContentSettingsModel)) {
                    return false;
                }
                else {
                    var otherModel = other;
                    return otherModel.owner == this.owner && otherModel.language == this.language;
                }
            };
            ContentSettingsModel.prototype.apply = function (builder) {
                builder.owner = this.owner;
                builder.language = this.language;
            };
            ContentSettingsModel.PROPERTY_OWNER = 'owner';
            ContentSettingsModel.PROPERTY_LANG = 'language';
            return ContentSettingsModel;
        })();
        wizard.ContentSettingsModel = ContentSettingsModel;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var FormView = api.form.FormView;
        var WizardStepValidityChangedEvent = api.app.wizard.WizardStepValidityChangedEvent;
        var ContentWizardStepForm = (function (_super) {
            __extends(ContentWizardStepForm, _super);
            function ContentWizardStepForm() {
                _super.call(this);
            }
            ContentWizardStepForm.prototype.layout = function (formContext, data, form) {
                var _this = this;
                this.formContext = formContext;
                this.form = form;
                this.data = data;
                return this.doLayout(form, data).then(function () {
                    if (form.getFormItems().length === 0) {
                        _this.hide();
                    }
                });
            };
            ContentWizardStepForm.prototype.doLayout = function (form, data) {
                var _this = this;
                this.formView = new FormView(this.formContext, form, data.getRoot());
                return this.formView.layout().then(function () {
                    _this.formView.onFocus(function (event) {
                        _this.notifyFocused(event);
                    });
                    _this.formView.onBlur(function (event) {
                        _this.notifyBlurred(event);
                    });
                    _this.formView.onEditContentRequest(function (content) {
                        new api.content.EditContentEvent([content]).fire();
                    });
                    _this.appendChild(_this.formView);
                    _this.formView.onValidityChanged(function (event) {
                        _this.previousValidation = event.getRecording();
                        _this.notifyValidityChanged(new WizardStepValidityChangedEvent(event.isValid()));
                    });
                    var formViewValid = _this.formView.isValid();
                    _this.notifyValidityChanged(new WizardStepValidityChangedEvent(formViewValid));
                });
            };
            ContentWizardStepForm.prototype.validate = function (silent) {
                return this.formView.validate(silent);
            };
            ContentWizardStepForm.prototype.displayValidationErrors = function (display) {
                this.formView.displayValidationErrors(display);
            };
            ContentWizardStepForm.prototype.getForm = function () {
                return this.form;
            };
            ContentWizardStepForm.prototype.getFormView = function () {
                return this.formView;
            };
            ContentWizardStepForm.prototype.getData = function () {
                return this.data;
            };
            ContentWizardStepForm.prototype.giveFocus = function () {
                return this.formView.giveFocus();
            };
            return ContentWizardStepForm;
        })(api.app.wizard.WizardStepForm);
        wizard.ContentWizardStepForm = ContentWizardStepForm;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var PrincipalType = api.security.PrincipalType;
        var PrincipalLoader = api.security.PrincipalLoader;
        var FormItemBuilder = api.ui.form.FormItemBuilder;
        var PrincipalComboBox = api.ui.security.PrincipalComboBox;
        var LocaleComboBox = api.ui.locale.LocaleComboBox;
        var SettingsWizardStepForm = (function (_super) {
            __extends(SettingsWizardStepForm, _super);
            function SettingsWizardStepForm() {
                var _this = this;
                _super.call(this, "settings-wizard-step-form");
                this.localeCombo = new LocaleComboBox(1);
                var localeFormItem = new FormItemBuilder(this.localeCombo).setLabel('Language').build();
                var loader = new PrincipalLoader().setAllowedTypes([0 /* USER */]);
                this.ownerCombo = new PrincipalComboBox(loader, 1);
                var ownerFormItem = new FormItemBuilder(this.ownerCombo).setLabel('Owner').build();
                var fieldSet = new api.ui.form.Fieldset();
                fieldSet.add(localeFormItem);
                fieldSet.add(ownerFormItem);
                var form = new api.ui.form.Form().add(fieldSet);
                this.appendChild(form);
                form.onFocus(function (event) {
                    _this.notifyFocused(event);
                });
                form.onBlur(function (event) {
                    _this.notifyBlurred(event);
                });
            }
            SettingsWizardStepForm.prototype.layout = function (content) {
                this.content = content;
            };
            SettingsWizardStepForm.prototype.setModel = function (model) {
                var _this = this;
                api.util.assertNotNull(model, "Model can't be null");
                if (model.getOwner()) {
                    this.ownerCombo.setIgnoreNextFocus().setValue(model.getOwner().toString());
                }
                if (model.getLanguage()) {
                    this.localeCombo.setIgnoreNextFocus().setValue(model.getLanguage());
                }
                var ownerListener = function () {
                    var principals = _this.ownerCombo.getSelectedDisplayValues();
                    model.setOwner(principals.length > 0 ? principals[0].getKey() : null, true);
                };
                this.ownerCombo.onOptionSelected(function (event) { return ownerListener(); });
                this.ownerCombo.onOptionDeselected(function (option) { return ownerListener(); });
                var localeListener = function () {
                    model.setLanguage(_this.localeCombo.getValue(), true);
                };
                this.localeCombo.onOptionSelected(function (event) { return localeListener(); });
                this.localeCombo.onOptionDeselected(function (option) { return localeListener(); });
                model.onPropertyChanged(function (event) {
                    switch (event.getPropertyName()) {
                        case wizard.ContentSettingsModel.PROPERTY_LANG:
                            _this.localeCombo.setValue(event.getNewValue());
                            break;
                        case wizard.ContentSettingsModel.PROPERTY_OWNER:
                            _this.ownerCombo.setValue(event.getNewValue());
                            break;
                    }
                });
                this.model = model;
            };
            SettingsWizardStepForm.prototype.getModel = function () {
                return this.model;
            };
            SettingsWizardStepForm.prototype.giveFocus = function () {
                return this.ownerCombo.giveFocus();
            };
            return SettingsWizardStepForm;
        })(api.app.wizard.WizardStepForm);
        wizard.SettingsWizardStepForm = SettingsWizardStepForm;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var AccessControlListView = api.ui.security.acl.AccessControlListView;
        var DivEl = api.dom.DivEl;
        var LabelEl = api.dom.LabelEl;
        var Button = api.ui.button.Button;
        var SecurityWizardStepForm = (function (_super) {
            __extends(SecurityWizardStepForm, _super);
            function SecurityWizardStepForm() {
                var _this = this;
                _super.call(this, "security-wizard-step-form");
                var label = new DivEl("input-label"), wrapper = new DivEl("wrapper required");
                this.label = new LabelEl("Permissions");
                wrapper.appendChild(this.label);
                label.appendChild(wrapper);
                this.inheritance = new DivEl();
                this.accessListView = new AccessControlListView();
                this.accessListView.setItemsEditable(false);
                this.editLink = new Button("Edit Permissions");
                this.editLink.addClass("edit-permissions");
                this.editLink.onFocus(function (event) {
                    _this.notifyFocused(event);
                });
                this.editLink.onBlur(function (event) {
                    _this.notifyBlurred(event);
                });
                var formView = new DivEl("form-view"), inputView = new DivEl("input-view valid"), inputTypeView = new DivEl("input-type-view"), inputOccurrenceView = new DivEl("input-occurrence-view single-occurrence"), inputWrapper = new DivEl("input-wrapper");
                inputWrapper.appendChildren(this.inheritance, this.accessListView, this.editLink);
                inputOccurrenceView.appendChild(inputWrapper);
                inputTypeView.appendChild(inputOccurrenceView);
                inputView.appendChildren(label, inputTypeView);
                formView.appendChild(inputView);
                this.appendChild(formView);
                this.editLink.onClicked(function () {
                    if (!!_this.content) {
                        new wizard.OpenEditPermissionsDialogEvent(_this.content).fire();
                    }
                });
                wizard.ContentPermissionsAppliedEvent.on(function (event) {
                    var content = event.getContent();
                    if (content.getId() === _this.content.getId()) {
                        _this.layout(content);
                    }
                });
            }
            SecurityWizardStepForm.prototype.layout = function (content) {
                var _this = this;
                this.accessListView.clearItems();
                content.getPermissions().getEntries().sort().forEach(function (entry) {
                    _this.accessListView.addItem(entry);
                    var entryView = _this.accessListView.getItemView(entry), selector = entryView.getPermissionSelector();
                    entryView.getValueChangedListeners().splice(0);
                    entryView.getPermissionSelector().hide();
                    entryView.onClicked(function () {
                        var isDisplayed = selector.getEl().getDisplay() !== "block";
                        _this.accessListView.getItemViews().forEach(function (itemView) {
                            itemView.getPermissionSelector().hide();
                        });
                        if (isDisplayed) {
                            selector.show();
                        }
                    });
                });
                var inheritsText = "";
                if (content.isInheritPermissionsEnabled() && content.isRoot() == false) {
                    inheritsText = "Inherits permissions from parent";
                    this.inheritance.addClass("inheritance");
                }
                else {
                    this.inheritance.removeClass("inheritance");
                }
                this.inheritance.setHtml(inheritsText);
                this.content = content;
            };
            SecurityWizardStepForm.prototype.giveFocus = function () {
                return this.accessListView.giveFocus();
            };
            return SecurityWizardStepForm;
        })(api.app.wizard.WizardStepForm);
        wizard.SecurityWizardStepForm = SecurityWizardStepForm;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var CycleButton = api.ui.button.CycleButton;
        var TogglerButton = api.ui.button.TogglerButton;
        var ContentWizardToolbar = (function (_super) {
            __extends(ContentWizardToolbar, _super);
            function ContentWizardToolbar(params) {
                _super.call(this, "content-wizard-toolbar");
                _super.prototype.addAction.call(this, params.saveAction);
                _super.prototype.addAction.call(this, params.deleteAction);
                _super.prototype.addAction.call(this, params.duplicateAction);
                _super.prototype.addAction.call(this, params.previewAction);
                _super.prototype.addGreedySpacer.call(this);
                this.cycleViewModeButton = new CycleButton([params.showLiveEditAction, params.showFormAction]);
                this.componentsViewToggler = new TogglerButton("icon-clipboard", "Show Component View");
                this.contextWindowToggler = new TogglerButton("icon-cog", "Show Inspection Panel");
                this.contentWizardToolbarPublishControls = new wizard.ContentWizardToolbarPublishControls(params.publishAction);
                _super.prototype.addElement.call(this, this.contentWizardToolbarPublishControls);
                _super.prototype.addElement.call(this, this.componentsViewToggler);
                _super.prototype.addElement.call(this, this.contextWindowToggler);
                _super.prototype.addElement.call(this, this.cycleViewModeButton);
            }
            ContentWizardToolbar.prototype.getCycleViewModeButton = function () {
                return this.cycleViewModeButton;
            };
            ContentWizardToolbar.prototype.getContextWindowToggler = function () {
                return this.contextWindowToggler;
            };
            ContentWizardToolbar.prototype.getComponentsViewToggler = function () {
                return this.componentsViewToggler;
            };
            ContentWizardToolbar.prototype.getContentWizardToolbarPublishControls = function () {
                return this.contentWizardToolbarPublishControls;
            };
            return ContentWizardToolbar;
        })(api.ui.toolbar.Toolbar);
        wizard.ContentWizardToolbar = ContentWizardToolbar;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var DisplayNameScriptExecutor = (function () {
            function DisplayNameScriptExecutor() {
            }
            DisplayNameScriptExecutor.prototype.setFormView = function (value) {
                this.formView = value;
                return this;
            };
            DisplayNameScriptExecutor.prototype.setScript = function (value) {
                this.script = value;
                return this;
            };
            DisplayNameScriptExecutor.prototype.hasScript = function () {
                return !api.util.StringHelper.isBlank(this.script);
            };
            DisplayNameScriptExecutor.prototype.execute = function () {
                api.util.assertNotNull(this.formView, "formView not set");
                api.util.assertNotNull(this.script, "script not set");
                return this.safeEval(this.script, this.formView);
            };
            DisplayNameScriptExecutor.prototype.safeEval = function (script, formView) {
                function $() {
                    var paths = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        paths[_i - 0] = arguments[_i];
                    }
                    var strValues = [];
                    paths.forEach(function (path) {
                        var strValue = formView.getData().getString(path);
                        if (!api.util.StringHelper.isBlank(strValue)) {
                            strValues.push(strValue);
                        }
                    });
                    var strValue = "";
                    strValues.forEach(function (s, index) {
                        strValue += s;
                        if (index < strValues.length - 1) {
                            strValue += " ";
                        }
                    });
                    return strValue;
                }
                var result = '';
                try {
                    result = eval('var eval; var Function; var document; var location; ' + 'var window; var parent; var self; var top; ' + script);
                }
                catch (e) {
                    console.error('Cannot evaluate script [' + script + '].', e);
                }
                return result;
            };
            return DisplayNameScriptExecutor;
        })();
        wizard.DisplayNameScriptExecutor = DisplayNameScriptExecutor;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (_app) {
    var wizard;
    (function (_wizard) {
        var PropertyTree = api.data.PropertyTree;
        var ContentFormContext = api.content.form.ContentFormContext;
        var CompareStatus = api.content.CompareStatus;
        var ContentBuilder = api.content.ContentBuilder;
        var ContentName = api.content.ContentName;
        var ContentUnnamed = api.content.ContentUnnamed;
        var CreateContentRequest = api.content.CreateContentRequest;
        var UpdateContentRequest = api.content.UpdateContentRequest;
        var ContentIconUrlResolver = api.content.ContentIconUrlResolver;
        var ExtraData = api.content.ExtraData;
        var SiteModel = api.content.site.SiteModel;
        var LiveEditModel = api.liveedit.LiveEditModel;
        var GetNearestSiteRequest = api.content.GetNearestSiteRequest;
        var ConfirmationDialog = api.ui.dialog.ConfirmationDialog;
        var ResponsiveManager = api.ui.responsive.ResponsiveManager;
        var ResponsiveRanges = api.ui.responsive.ResponsiveRanges;
        var ThumbnailUploader = api.content.ThumbnailUploader;
        var WizardHeaderWithDisplayNameAndNameBuilder = api.app.wizard.WizardHeaderWithDisplayNameAndNameBuilder;
        var WizardStep = api.app.wizard.WizardStep;
        var ContentRequiresSaveEvent = api.content.ContentRequiresSaveEvent;
        var MixinName = api.schema.mixin.MixinName;
        var MixinNames = api.schema.mixin.MixinNames;
        var GetMixinByQualifiedNameRequest = api.schema.mixin.GetMixinByQualifiedNameRequest;
        var DialogButton = api.ui.dialog.DialogButton;
        var ContentWizardPanel = (function (_super) {
            __extends(ContentWizardPanel, _super);
            function ContentWizardPanel(params, onSuccess, onError) {
                var _this = this;
                this.constructing = true;
                this.isContentFormValid = false;
                this.isSecurityWizardStepFormAllowed = false;
                this.requireValid = false;
                this.contentNamedListeners = [];
                this.parentContent = params.parentContent;
                this.defaultModels = params.defaultModels;
                this.site = params.site;
                this.contentType = params.contentType;
                this.displayNameScriptExecutor = new _wizard.DisplayNameScriptExecutor();
                this.contentWizardHeader = new WizardHeaderWithDisplayNameAndNameBuilder().setDisplayNameGenerator(this.displayNameScriptExecutor).build();
                this.thumbnailUploader = new ThumbnailUploader({
                    name: 'thumbnail-uploader',
                    disabled: params.contentType.isImage(),
                    deferred: true
                });
                this.wizardActions = new app.wizard.action.ContentWizardActions(this);
                this.previewAction = this.wizardActions.getPreviewAction();
                this.publishAction = this.wizardActions.getPublishAction();
                var mainToolbar = new _wizard.ContentWizardToolbar({
                    saveAction: this.wizardActions.getSaveAction(),
                    deleteAction: this.wizardActions.getDeleteAction(),
                    duplicateAction: this.wizardActions.getDuplicateAction(),
                    previewAction: this.wizardActions.getPreviewAction(),
                    publishAction: this.wizardActions.getPublishAction(),
                    showLiveEditAction: this.wizardActions.getShowLiveEditAction(),
                    showFormAction: this.wizardActions.getShowFormAction(),
                    showSplitEditAction: this.wizardActions.getShowSplitEditAction()
                });
                this.contextWindowToggler = mainToolbar.getContextWindowToggler();
                this.componentsViewToggler = mainToolbar.getComponentsViewToggler();
                this.cycleViewModeButton = mainToolbar.getCycleViewModeButton();
                this.contentWizardToolbarPublishControls = mainToolbar.getContentWizardToolbarPublishControls();
                this.showLiveEditAction = this.wizardActions.getShowLiveEditAction();
                this.showSplitEditAction = this.wizardActions.getShowSplitEditAction();
                this.showLiveEditAction.setEnabled(false);
                if (this.parentContent) {
                    this.contentWizardHeader.setPath(this.parentContent.getPath().prettifyUnnamedPathElements().toString() + "/");
                }
                else {
                    this.contentWizardHeader.setPath("/");
                }
                this.createSite = params.createSite;
                if (this.createSite || (params.persistedContent && params.persistedContent.isSite())) {
                    this.thumbnailUploader.addClass("site");
                }
                this.contentWizardStepForm = new _wizard.ContentWizardStepForm();
                this.metadataStepFormByName = {};
                this.settingsWizardStepForm = new _wizard.SettingsWizardStepForm();
                this.securityWizardStepForm = new _wizard.SecurityWizardStepForm();
                _wizard.ContentPermissionsAppliedEvent.on(function (event) { return _this.contentPermissionsUpdated(event.getContent()); });
                var isSiteOrWithinSite = this.site || this.createSite;
                var isPageTemplate = this.contentType.getContentTypeName().isPageTemplate();
                if (isSiteOrWithinSite || isPageTemplate) {
                    this.liveFormPanel = new _wizard.page.LiveFormPanel({
                        contentWizardPanel: this,
                        contentType: this.contentType.getContentTypeName(),
                        defaultModels: params.defaultModels
                    });
                }
                if (this.contentType.hasContentDisplayNameScript()) {
                    this.displayNameScriptExecutor.setScript(this.contentType.getContentDisplayNameScript());
                }
                _super.call(this, {
                    tabId: params.tabId,
                    persistedItem: params.persistedContent,
                    formIcon: this.thumbnailUploader,
                    mainToolbar: mainToolbar,
                    header: this.contentWizardHeader,
                    actions: this.wizardActions,
                    livePanel: this.liveFormPanel,
                    split: !!this.liveFormPanel
                }, function () {
                    _this.onValidityChanged(function (event) {
                        _this.isContentFormValid = _this.isValid();
                        _this.thumbnailUploader.toggleClass("invalid", !_this.isValid());
                        _this.contentWizardToolbarPublishControls.setContentCanBePublished(_this.checkContentCanBePublished(false));
                    });
                    _this.addClass("content-wizard-panel");
                    if (_this.getSplitPanel()) {
                        _this.getSplitPanel().addClass("prerendered");
                    }
                    _this.inMobileViewMode = false;
                    var responsiveItem = ResponsiveManager.onAvailableSizeChanged(_this, function (item) {
                        if (_this.isVisible()) {
                            _this.updateStickyToolbar();
                            if (item.isInRangeOrSmaller(ResponsiveRanges._720_960)) {
                                _this.inMobileViewMode = true;
                                if (_this.isSplitView()) {
                                    if (_this.isMinimized()) {
                                        _this.toggleMinimize();
                                    }
                                    _this.cycleViewModeButton.executePrevAction();
                                }
                            }
                            else {
                                if (_this.inMobileViewMode && _this.isLiveView()) {
                                    _this.inMobileViewMode = false;
                                    _this.showSplitEdit();
                                }
                                _this.inMobileViewMode = false;
                            }
                        }
                    });
                    _this.onRemoved(function (event) {
                        ResponsiveManager.unAvailableSizeChanged(_this);
                    });
                    _this.onShown(function (event) {
                        if (_this.getPersistedItem()) {
                            app.Router.setHash("edit/" + _this.getPersistedItem().getId());
                        }
                        else {
                            app.Router.setHash("new/" + _this.contentType.getName());
                        }
                        _this.wizardActions.getShowSplitEditAction().onExecuted(function () {
                            if (!_this.inMobileViewMode) {
                                if (_this.contentNotRenderable()) {
                                    _this.closeLiveEdit();
                                    _this.contextWindowToggler.setEnabled(false);
                                }
                                else {
                                    _this.cycleViewModeButton.selectActiveAction(_this.showLiveEditAction);
                                }
                            }
                        });
                        _this.wizardActions.getShowSplitEditAction().execute();
                        responsiveItem.update();
                    });
                    if (_this.thumbnailUploader) {
                        _this.thumbnailUploader.onFileUploaded(function (event) {
                            var newPersistedContent = event.getUploadItem().getModel();
                            _this.setPersistedItem(newPersistedContent);
                            _this.updateMetadataAndMetadataForms(newPersistedContent);
                            _this.updateThumbnailWithContent(newPersistedContent);
                            var contentToDisplay = (newPersistedContent.getDisplayName() && newPersistedContent.getDisplayName().length > 0) ? '\"' + newPersistedContent.getDisplayName() + '\"' : "Content";
                            api.notify.showFeedback(contentToDisplay + ' saved');
                        });
                    }
                    _this.constructing = false;
                    onSuccess(_this);
                }, onError);
                this.initPublishButtonForMobile();
                this.handleSiteConfigApply();
                api.app.wizard.MaskContentWizardPanelEvent.on(function (event) {
                    if (_this.getPersistedItem().getContentId().equals(event.getContentId())) {
                        _this.actions.suspendActions(event.isMask());
                    }
                });
            }
            ContentWizardPanel.prototype.handleSiteConfigApply = function () {
                var _this = this;
                var siteConfigApplyHandler = function (event) {
                    if (_this.getPersistedItem().getId() == event.getContent().getId()) {
                        _this.saveChanges();
                    }
                };
                ContentRequiresSaveEvent.on(siteConfigApplyHandler);
                this.onClosed(function () {
                    ContentRequiresSaveEvent.un(siteConfigApplyHandler);
                });
            };
            ContentWizardPanel.prototype.getContentType = function () {
                return this.contentType;
            };
            ContentWizardPanel.prototype.giveInitialFocus = function () {
                console.log("ContentWizardPanel.giveInitialFocus");
                if (this.contentType.hasContentDisplayNameScript()) {
                    if (!this.contentWizardStepForm.giveFocus()) {
                        console.log("ContentWizardPanel.giveInitialFocus() WARNING: Failed to give focus to contentWizardStepForm");
                        this.contentWizardHeader.giveFocus();
                    }
                }
                else {
                    this.contentWizardHeader.giveFocus();
                }
                this.startRememberFocus();
            };
            ContentWizardPanel.prototype.createSteps = function () {
                var _this = this;
                var applicationKeys = this.site ? this.site.getApplicationKeys() : [];
                var applicationPromises = applicationKeys.map(function (key) { return _this.fetchApplication(key); });
                return new api.security.auth.IsAuthenticatedRequest().sendAndParse().then(function (loginResult) {
                    _this.checkSecurityWizardStepFormAllowed(loginResult);
                    _this.enablePublishIfAllowed(loginResult);
                    return wemQ.all(applicationPromises);
                }).then(function (applications) {
                    for (var i = 0; i < applications.length; i++) {
                        var app = applications[i];
                        if (!app.isStarted()) {
                            var deferred = wemQ.defer();
                            deferred.reject(new api.Exception("Content cannot be opened. Required application '" + app.getDisplayName() + "' is not started.", 2 /* WARNING */));
                            return deferred.promise;
                        }
                    }
                    var metadataMixinPromises = [];
                    metadataMixinPromises = metadataMixinPromises.concat(_this.contentType.getMetadata().map(function (name) {
                        return _this.fetchMixin(name);
                    }));
                    applications.forEach(function (app) {
                        metadataMixinPromises = metadataMixinPromises.concat(app.getMetaSteps().map(function (name) {
                            return _this.fetchMixin(name);
                        }));
                    });
                    return wemQ.all(metadataMixinPromises);
                }).then(function (mixins) {
                    var steps = [];
                    _this.contentWizardStep = new WizardStep(_this.contentType.getDisplayName(), _this.contentWizardStepForm);
                    steps.push(_this.contentWizardStep);
                    mixins.forEach(function (mixin, index) {
                        if (!_this.metadataStepFormByName[mixin.getMixinName().toString()]) {
                            var stepForm = new _wizard.ContentWizardStepForm();
                            _this.metadataStepFormByName[mixin.getMixinName().toString()] = stepForm;
                            steps.splice(index + 1, 0, new WizardStep(mixin.getDisplayName(), stepForm));
                        }
                    });
                    _this.settingsWizardStep = new WizardStep("Settings", _this.settingsWizardStepForm);
                    steps.push(_this.settingsWizardStep);
                    if (_this.isSecurityWizardStepFormAllowed) {
                        steps.push(new WizardStep("Security", _this.securityWizardStepForm));
                    }
                    _this.setSteps(steps);
                    return mixins;
                });
            };
            ContentWizardPanel.prototype.fetchMixin = function (name) {
                var deferred = wemQ.defer();
                new GetMixinByQualifiedNameRequest(name).sendAndParse().then(function (mixin) {
                    deferred.resolve(mixin);
                }).catch(function (reason) {
                    deferred.reject(new api.Exception("Content cannot be opened. Required mixin '" + name.toString() + "' not found.", 2 /* WARNING */));
                }).done();
                return deferred.promise;
            };
            ContentWizardPanel.prototype.fetchApplication = function (key) {
                var deferred = wemQ.defer();
                new api.application.GetApplicationRequest(key).sendAndParse().then(function (mod) {
                    deferred.resolve(mod);
                }).catch(function (reason) {
                    deferred.reject(new api.Exception("Content cannot be opened. Required application '" + key.toString() + "' not found.", 2 /* WARNING */));
                }).done();
                return deferred.promise;
            };
            ContentWizardPanel.prototype.saveChanges = function () {
                if (this.liveFormPanel) {
                    this.liveFormPanel.skipNextReloadConfirmation(true);
                }
                this.setRequireValid(false);
                return _super.prototype.saveChanges.call(this);
            };
            ContentWizardPanel.prototype.preLayoutNew = function () {
                var deferred = wemQ.defer();
                this.saveChanges().then(function () {
                    deferred.resolve(null);
                }).catch(function (reason) {
                    deferred.reject(reason);
                }).done();
                return deferred.promise;
            };
            ContentWizardPanel.prototype.postLayoutNew = function () {
                var deferred = wemQ.defer();
                this.enableDisplayNameScriptExecution(this.contentWizardStepForm.getFormView());
                this.contentWizardHeader.initNames(this.getPersistedItem().getDisplayName(), this.getPersistedItem().getName().toString(), true);
                deferred.resolve(null);
                return deferred.promise;
            };
            ContentWizardPanel.prototype.layoutPersistedItem = function (persistedContent) {
                var _this = this;
                this.updateThumbnailWithContent(persistedContent);
                this.notifyValidityChanged(persistedContent.isValid());
                api.content.ContentSummaryAndCompareStatusFetcher.fetchByContent(persistedContent).then(function (contentSummaryAndCompareStatus) {
                    var ignore = contentSummaryAndCompareStatus.getCompareStatus() !== 0 /* NEW */;
                    _this.contentWizardHeader.disableNameGeneration(ignore);
                    var publishHandler = function (event) {
                        if (_this.getPersistedItem() && event.getContentId() && (_this.getPersistedItem().getId() === event.getContentId().toString())) {
                            _this.contentWizardToolbarPublishControls.setCompareStatus(6 /* EQUAL */);
                            if (!ignore) {
                                _this.contentWizardHeader.disableNameGeneration(true);
                                api.content.ContentPublishedEvent.un(publishHandler);
                            }
                        }
                    };
                    api.content.ContentPublishedEvent.on(publishHandler);
                    _this.onClosed(function () {
                        api.content.ContentPublishedEvent.un(publishHandler);
                    });
                    _this.contentWizardToolbarPublishControls.setCompareStatus(contentSummaryAndCompareStatus.getCompareStatus());
                    _this.managePublishButtonStateForMobile(contentSummaryAndCompareStatus.getCompareStatus());
                }).done();
                var viewedContent;
                var deferred = wemQ.defer();
                if (!this.constructing) {
                    viewedContent = this.assembleViewedContent(persistedContent.newBuilder()).build();
                    if (viewedContent.equals(persistedContent)) {
                        if (this.liveFormPanel) {
                            this.liveFormPanel.loadPage();
                        }
                    }
                    else {
                        console.warn("Received Content from server differs from what's viewed:");
                        if (!viewedContent.getContentData().equals(persistedContent.getContentData())) {
                            console.warn(" inequality found in Content.data");
                            if (persistedContent.getContentData() && viewedContent.getContentData()) {
                                console.warn(" comparing persistedContent.data against viewedContent.data:");
                                new api.data.PropertyTreeComparator().compareTree(persistedContent.getContentData(), viewedContent.getContentData());
                            }
                        }
                        if (!api.ObjectHelper.equals(viewedContent.getPage(), persistedContent.getPage())) {
                            console.warn(" inequality found in Content.page");
                            if (persistedContent.getPage() && viewedContent.getPage()) {
                                console.warn(" comparing persistedContent.page.config against viewedContent.page.config:");
                                new api.data.PropertyTreeComparator().compareTree(persistedContent.getPage().getConfig(), viewedContent.getPage().getConfig());
                            }
                        }
                        if (!api.ObjectHelper.arrayEquals(viewedContent.getAllExtraData(), persistedContent.getAllExtraData())) {
                            console.warn(" inequality found in Content.meta");
                        }
                        if (!api.ObjectHelper.equals(viewedContent.getAttachments(), persistedContent.getAttachments())) {
                            console.warn(" inequality found in Content.attachments");
                        }
                        if (!api.ObjectHelper.equals(viewedContent.getPermissions(), persistedContent.getPermissions())) {
                            console.warn(" inequality found in Content.permissions");
                        }
                        console.warn(" viewedContent: ", viewedContent);
                        console.warn(" persistedContent: ", persistedContent);
                        if (persistedContent.getType().isDescendantOfMedia()) {
                            this.updateMetadataAndMetadataForms(persistedContent.clone());
                        }
                        else {
                            ConfirmationDialog.get().setQuestion("Received Content from server differs from what you have. Would you like to load changes from server?").setYesCallback(function () { return _this.doLayoutPersistedItem(persistedContent.clone()); }).setNoCallback(function () {
                            }).show();
                        }
                    }
                    deferred.resolve(null);
                }
                else {
                    this.doLayoutPersistedItem(persistedContent.clone()).then(function () {
                        deferred.resolve(null);
                    }).catch(function (reason) {
                        deferred.reject(reason);
                    }).done();
                }
                this.contentWizardHeader.setSimplifiedNameGeneration(persistedContent.getType().isDescendantOfMedia());
                this.contentWizardToolbarPublishControls.enableActionsForExisting(persistedContent);
                return deferred.promise;
            };
            ContentWizardPanel.prototype.updateThumbnailWithContent = function (content) {
                this.thumbnailUploader.setValue(new ContentIconUrlResolver().setContent(content).resolve()).setEnabled(!content.isImage()).setParams({
                    id: content.getContentId().toString()
                });
                this.thumbnailUploader.toggleClass("invalid", !content.isValid());
            };
            ContentWizardPanel.prototype.doLayoutPersistedItem = function (content) {
                var _this = this;
                this.toggleClass("rendered", false);
                this.showLiveEditAction.setEnabled(false);
                this.previewAction.setVisible(false);
                this.previewAction.setEnabled(false);
                new GetNearestSiteRequest(content.getContentId()).sendAndParse().then(function (parentSite) {
                    if (parentSite || content.isSite()) {
                        _this.setupWizardLiveEdit(true);
                    }
                    else {
                        _this.setupWizardLiveEdit(false);
                    }
                }).catch(function (reason) {
                    _this.setupWizardLiveEdit(false);
                    api.DefaultErrorHandler.handle(reason);
                }).done();
                var parallelPromises = [this.createSteps()];
                return wemQ.all(parallelPromises).spread(function (schemas) {
                    var formContext = _this.createFormContext(content);
                    var contentData = content.getContentData();
                    contentData.onChanged(function () {
                        if (_this.isContentFormValid && _this.contentWizardToolbarPublishControls.isOnline()) {
                            _this.contentWizardToolbarPublishControls.setCompareStatus(2 /* NEWER */);
                        }
                    });
                    var formViewLayoutPromises = [];
                    formViewLayoutPromises.push(_this.contentWizardStepForm.layout(formContext, contentData, _this.contentType.getForm()));
                    _this.displayNameScriptExecutor.setFormView(_this.contentWizardStepForm.getFormView());
                    _this.settingsWizardStepForm.layout(content);
                    _this.settingsWizardStepForm.setModel(new _wizard.ContentSettingsModel(content));
                    if (_this.isSecurityWizardStepFormAllowed) {
                        _this.securityWizardStepForm.layout(content);
                    }
                    schemas.forEach(function (schema, index) {
                        var extraData = content.getExtraData(schema.getMixinName());
                        if (!extraData) {
                            extraData = new ExtraData(schema.getMixinName(), new PropertyTree());
                            content.getAllExtraData().push(extraData);
                        }
                        var metadataFormView = _this.metadataStepFormByName[schema.getMixinName().toString()];
                        var metadataForm = new api.form.FormBuilder().addFormItems(schema.getFormItems()).build();
                        formViewLayoutPromises.push(metadataFormView.layout(formContext, extraData.getData(), metadataForm));
                    });
                    return wemQ.all(formViewLayoutPromises).spread(function () {
                        console.log("ContentWizardPanel.doLayoutPersistedItem: all FormView-s layed out");
                        if (_this.liveFormPanel) {
                            if (!_this.liveEditModel) {
                                var site = content.isSite() ? content : _this.site;
                                _this.siteModel = new SiteModel(site);
                                return _this.initLiveEditModel(content, _this.siteModel, formContext).then(function () {
                                    _this.liveFormPanel.setModel(_this.liveEditModel);
                                    _this.liveFormPanel.loadPage();
                                    _this.updatePreviewActionVisibility();
                                    return wemQ(null);
                                });
                            }
                            else {
                                _this.liveFormPanel.loadPage();
                            }
                        }
                        if (!_this.siteModel && content.isSite()) {
                            _this.siteModel = new SiteModel(content);
                        }
                        if (_this.siteModel) {
                            _this.initSiteModelListeners();
                        }
                        return wemQ(null);
                    });
                });
            };
            ContentWizardPanel.prototype.setupWizardLiveEdit = function (renderable) {
                this.toggleClass("rendered", renderable);
                this.showLiveEditAction.setEnabled(renderable);
                this.showSplitEditAction.setEnabled(renderable);
                this.previewAction.setVisible(renderable);
                if (this.getEl().getWidth() > ResponsiveRanges._720_960.getMaximumRange() && renderable) {
                    this.wizardActions.getShowSplitEditAction().execute();
                }
            };
            ContentWizardPanel.prototype.initSiteModelListeners = function () {
                var _this = this;
                this.siteModel.onApplicationAdded(function (event) {
                    _this.addMetadataStepForms(event.getApplicationKey());
                });
                this.siteModel.onApplicationRemoved(function (event) {
                    _this.removeMetadataStepForms();
                });
            };
            ContentWizardPanel.prototype.removeMetadataStepForms = function () {
                var _this = this;
                var applicationKeys = this.siteModel.getApplicationKeys();
                var applicationPromises = applicationKeys.map(function (key) { return new api.application.GetApplicationRequest(key).sendAndParse(); });
                return wemQ.all(applicationPromises).then(function (applications) {
                    var metadataMixinPromises = [];
                    applications.forEach(function (app) {
                        metadataMixinPromises = metadataMixinPromises.concat(app.getMetaSteps().map(function (name) {
                            return new GetMixinByQualifiedNameRequest(name).sendAndParse();
                        }));
                    });
                    return wemQ.all(metadataMixinPromises);
                }).then(function (mixins) {
                    var activeMixinsNames = api.schema.mixin.MixinNames.create().fromMixins(mixins).build();
                    var panelNamesToRemoveBuilder = MixinNames.create();
                    for (var key in _this.metadataStepFormByName) {
                        var mixinName = new MixinName(key);
                        if (!activeMixinsNames.contains(mixinName)) {
                            panelNamesToRemoveBuilder.addMixinName(mixinName);
                        }
                    }
                    var panelNamesToRemove = panelNamesToRemoveBuilder.build();
                    panelNamesToRemove.forEach(function (panelName) {
                        _this.removeStepWithForm(_this.metadataStepFormByName[panelName.toString()]);
                        delete _this.metadataStepFormByName[panelName.toString()];
                    });
                    return mixins;
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                }).done();
            };
            ContentWizardPanel.prototype.addMetadataStepForms = function (applicationKey) {
                var _this = this;
                new api.application.GetApplicationRequest(applicationKey).sendAndParse().then(function (currentApplication) {
                    var mixinNames = currentApplication.getMetaSteps();
                    var mixinNamesToAdd = mixinNames.filter(function (mixinName) {
                        return !_this.metadataStepFormByName[mixinName.toString()];
                    });
                    var getMixinPromises = mixinNamesToAdd.map(function (name) {
                        return new GetMixinByQualifiedNameRequest(name).sendAndParse();
                    });
                    return wemQ.all(getMixinPromises);
                }).then(function (mixins) {
                    mixins.forEach(function (mixin) {
                        if (!_this.metadataStepFormByName[mixin.getMixinName().toString()]) {
                            var stepForm = new _wizard.ContentWizardStepForm();
                            _this.metadataStepFormByName[mixin.getMixinName().toString()] = stepForm;
                            var wizardStep = new WizardStep(mixin.getDisplayName(), stepForm);
                            _this.insertStepBefore(wizardStep, _this.settingsWizardStep);
                            var extraData = new ExtraData(mixin.getMixinName(), new PropertyTree());
                            stepForm.layout(_this.createFormContext(_this.getPersistedItem()), extraData.getData(), mixin.toForm());
                        }
                    });
                    return mixins;
                }).catch(function (reason) {
                    api.DefaultErrorHandler.handle(reason);
                }).done();
            };
            ContentWizardPanel.prototype.initLiveEditModel = function (content, siteModel, formContext) {
                this.initSiteModelListeners();
                this.liveEditModel = LiveEditModel.create().setParentContent(this.parentContent).setContent(content).setContentFormContext(formContext).setSiteModel(siteModel).build();
                return this.liveEditModel.init(this.defaultModels.getPageTemplate(), this.defaultModels.getPageDescriptor());
            };
            ContentWizardPanel.prototype.postLayoutPersisted = function (existing) {
                var deferred = wemQ.defer();
                this.contentWizardHeader.initNames(existing.getDisplayName(), existing.getName().toString(), false);
                this.enableDisplayNameScriptExecution(this.contentWizardStepForm.getFormView());
                deferred.resolve(null);
                return deferred.promise;
            };
            ContentWizardPanel.prototype.persistNewItem = function () {
                return new _wizard.PersistNewContentRoutine(this).setCreateContentRequestProducer(this.produceCreateContentRequest).execute().then(function (content) {
                    api.notify.showFeedback('Content created');
                    return content;
                });
            };
            ContentWizardPanel.prototype.postPersistNewItem = function (persistedContent) {
                var deferred = wemQ.defer();
                if (persistedContent.isSite()) {
                    this.site = persistedContent;
                }
                deferred.resolve(null);
                return deferred.promise;
            };
            ContentWizardPanel.prototype.produceCreateContentRequest = function () {
                var deferred = wemQ.defer();
                var parentPath = this.parentContent != null ? this.parentContent.getPath() : api.content.ContentPath.ROOT;
                if (this.contentType.getContentTypeName().isMedia()) {
                    deferred.resolve(null);
                }
                else {
                    deferred.resolve(new CreateContentRequest().setRequireValid(this.requireValid).setName(api.content.ContentUnnamed.newUnnamed()).setParent(parentPath).setContentType(this.contentType.getContentTypeName()).setDisplayName(this.contentWizardHeader.getDisplayName()).setData(new PropertyTree()).setExtraData([]));
                }
                return deferred.promise;
            };
            ContentWizardPanel.prototype.updatePersistedItem = function () {
                var _this = this;
                var persistedContent = this.getPersistedItem();
                var viewedContent = this.assembleViewedContent(persistedContent.newBuilder()).build();
                var updatePersistedContentRoutine = new _wizard.UpdatePersistedContentRoutine(this, persistedContent, viewedContent).setUpdateContentRequestProducer(this.produceUpdateContentRequest);
                return updatePersistedContentRoutine.execute().then(function (content) {
                    if (persistedContent.getName().isUnnamed() && !content.getName().isUnnamed()) {
                        _this.notifyContentNamed(content);
                    }
                    var contentToDisplay = (content.getDisplayName() && content.getDisplayName().length > 0) ? '\"' + content.getDisplayName() + '\"' : "Content";
                    api.notify.showFeedback(contentToDisplay + ' saved');
                    new api.content.ContentUpdatedEvent(content.getContentId()).fire();
                    return content;
                });
            };
            ContentWizardPanel.prototype.produceUpdateContentRequest = function (persistedContent, viewedContent) {
                var persistedContent = this.getPersistedItem();
                var updateContentRequest = new UpdateContentRequest(persistedContent.getId()).setRequireValid(this.requireValid).setContentName(viewedContent.getName()).setDisplayName(viewedContent.getDisplayName()).setData(viewedContent.getContentData()).setExtraData(viewedContent.getAllExtraData()).setOwner(viewedContent.getOwner()).setLanguage(viewedContent.getLanguage());
                return updateContentRequest;
            };
            ContentWizardPanel.prototype.hasUnsavedChanges = function () {
                var persistedContent = this.getPersistedItem();
                if (persistedContent == undefined) {
                    return true;
                }
                else {
                    var viewedContent = this.assembleViewedContent(new ContentBuilder(persistedContent)).build();
                    return !viewedContent.equals(persistedContent, true);
                }
            };
            ContentWizardPanel.prototype.enableDisplayNameScriptExecution = function (formView) {
                var _this = this;
                if (this.displayNameScriptExecutor.hasScript()) {
                    formView.onKeyUp(function (event) {
                        if (_this.displayNameScriptExecutor.hasScript()) {
                            _this.contentWizardHeader.setDisplayName(_this.displayNameScriptExecutor.execute());
                        }
                    });
                }
            };
            ContentWizardPanel.prototype.assembleViewedContent = function (viewedContentBuilder) {
                viewedContentBuilder.setName(this.resolveContentNameForUpdateRequest());
                viewedContentBuilder.setDisplayName(this.contentWizardHeader.getDisplayName());
                if (this.contentWizardStepForm) {
                    viewedContentBuilder.setData(this.contentWizardStepForm.getData());
                }
                var extraData = [];
                for (var key in this.metadataStepFormByName) {
                    if (this.metadataStepFormByName.hasOwnProperty(key)) {
                        extraData.push(new ExtraData(new MixinName(key), this.metadataStepFormByName[key].getData()));
                    }
                }
                viewedContentBuilder.setExtraData(extraData);
                this.settingsWizardStepForm.getModel().apply(viewedContentBuilder);
                viewedContentBuilder.setPage(this.assembleViewedPage());
                return viewedContentBuilder;
            };
            ContentWizardPanel.prototype.assembleViewedPage = function () {
                if (!this.liveFormPanel) {
                    return null;
                }
                return this.liveFormPanel.getPage();
            };
            ContentWizardPanel.prototype.resolveContentNameForUpdateRequest = function () {
                if (api.util.StringHelper.isEmpty(this.contentWizardHeader.getName())) {
                    if (this.getPersistedItem().getName().isUnnamed()) {
                        return this.getPersistedItem().getName();
                    }
                    else {
                        return ContentUnnamed.newUnnamed();
                    }
                }
                return ContentName.fromString(this.contentWizardHeader.getName());
            };
            ContentWizardPanel.prototype.setRequireValid = function (requireValid) {
                this.requireValid = requireValid;
            };
            ContentWizardPanel.prototype.showLiveEdit = function () {
                if (!this.inMobileViewMode) {
                    this.showSplitEdit();
                    return;
                }
                this.getSplitPanel().addClass("toggle-live");
                this.getSplitPanel().removeClass("toggle-form toggle-split prerendered");
                this.getMainToolbar().toggleClass("live", true);
                this.toggleClass("form", false);
                this.openLiveEdit();
                ResponsiveManager.fireResizeEvent();
            };
            ContentWizardPanel.prototype.showSplitEdit = function () {
                this.getSplitPanel().addClass("toggle-split");
                this.getSplitPanel().removeClass("toggle-live toggle-form prerendered");
                this.getMainToolbar().toggleClass("live", true);
                this.toggleClass("form", false);
                this.openLiveEdit();
                ResponsiveManager.fireResizeEvent();
            };
            ContentWizardPanel.prototype.showForm = function () {
                this.getSplitPanel().addClass("toggle-form");
                this.getSplitPanel().removeClass("toggle-live toggle-split prerendered");
                this.getMainToolbar().toggleClass("live", false);
                this.toggleClass("form", true);
                this.closeLiveEdit();
                ResponsiveManager.fireResizeEvent();
            };
            ContentWizardPanel.prototype.isSplitView = function () {
                return this.getSplitPanel() && this.getSplitPanel().hasClass("toggle-split");
            };
            ContentWizardPanel.prototype.isLiveView = function () {
                return this.getSplitPanel() && this.getSplitPanel().hasClass("toggle-live");
            };
            ContentWizardPanel.prototype.checkContentCanBePublished = function (displayValidationErrors) {
                if (!this.isContentFormValid) {
                    this.contentWizardStepForm.displayValidationErrors(displayValidationErrors);
                }
                var contentFormHasValidUserInput = this.contentWizardStepForm.getFormView().hasValidUserInput();
                var allMetadataFormsValid = true;
                var allMetadataFormsHasValidUserInput = true;
                for (var key in this.metadataStepFormByName) {
                    if (this.metadataStepFormByName.hasOwnProperty(key)) {
                        var form = this.metadataStepFormByName[key];
                        if (!form.isValid()) {
                            form.displayValidationErrors(displayValidationErrors);
                            allMetadataFormsValid = false;
                        }
                        var formHasValidUserInput = form.getFormView().hasValidUserInput();
                        if (!formHasValidUserInput) {
                            allMetadataFormsHasValidUserInput = false;
                        }
                    }
                }
                return this.isContentFormValid && allMetadataFormsValid && contentFormHasValidUserInput && allMetadataFormsHasValidUserInput;
            };
            ContentWizardPanel.prototype.getLiveFormPanel = function () {
                return this.liveFormPanel;
            };
            ContentWizardPanel.prototype.getContextWindowToggler = function () {
                return this.contextWindowToggler;
            };
            ContentWizardPanel.prototype.getComponentsViewToggler = function () {
                return this.componentsViewToggler;
            };
            ContentWizardPanel.prototype.getCloseAction = function () {
                return this.wizardActions.getCloseAction();
            };
            ContentWizardPanel.prototype.onContentNamed = function (listener) {
                this.contentNamedListeners.push(listener);
            };
            ContentWizardPanel.prototype.unContentNamed = function (listener) {
                this.contentNamedListeners = this.contentNamedListeners.filter(function (curr) {
                    return curr != listener;
                });
                return this;
            };
            ContentWizardPanel.prototype.notifyContentNamed = function (content) {
                var _this = this;
                this.contentNamedListeners.forEach(function (listener) {
                    listener.call(_this, new api.content.ContentNamedEvent(_this, content));
                });
            };
            ContentWizardPanel.prototype.contentPermissionsUpdated = function (content) {
                var persistedContent = this.getPersistedItem();
                if (persistedContent && (content.getId() === persistedContent.getId())) {
                    var updatedContent = persistedContent.newBuilder().setInheritPermissionsEnabled(content.isInheritPermissionsEnabled()).setPermissions(content.getPermissions().clone()).build();
                    this.setPersistedItem(updatedContent);
                }
            };
            ContentWizardPanel.prototype.createFormContext = function (content) {
                var formContext = ContentFormContext.create().setSite(this.site).setParentContent(this.parentContent).setPersistedContent(content).setContentTypeName(this.contentType ? this.contentType.getContentTypeName() : undefined).setShowEmptyFormItemSetOccurrences(this.isItemPersisted()).build();
                return formContext;
            };
            ContentWizardPanel.prototype.checkSecurityWizardStepFormAllowed = function (loginResult) {
                var _this = this;
                var entries = this.getPersistedItem().getPermissions().getEntries();
                var accessEntriesWithEditPermissions = entries.filter(function (item) {
                    return item.isAllowed(6 /* WRITE_PERMISSIONS */);
                });
                loginResult.getPrincipals().some(function (principalKey) {
                    if (api.security.RoleKeys.ADMIN.equals(principalKey) || _this.isPrincipalPresent(principalKey, accessEntriesWithEditPermissions)) {
                        _this.isSecurityWizardStepFormAllowed = true;
                        return true;
                    }
                });
            };
            ContentWizardPanel.prototype.isPrincipalPresent = function (principalKey, accessEntriesToCheck) {
                var result = false;
                accessEntriesToCheck.some(function (entry) {
                    if (entry.getPrincipalKey().equals(principalKey)) {
                        result = true;
                        return true;
                    }
                });
                return result;
            };
            ContentWizardPanel.prototype.enablePublishIfAllowed = function (loginResult) {
                var _this = this;
                var entries = this.getPersistedItem().getPermissions().getEntries();
                var accessEntriesWithPublishPermissions = entries.filter(function (item) {
                    return item.isAllowed(4 /* PUBLISH */);
                });
                loginResult.getPrincipals().some(function (principalKey) {
                    if (api.security.RoleKeys.ADMIN.equals(principalKey) || _this.isPrincipalPresent(principalKey, accessEntriesWithPublishPermissions)) {
                        _this.publishAction.setEnabled(true);
                        return true;
                    }
                });
            };
            ContentWizardPanel.prototype.updateMetadataAndMetadataForms = function (content) {
                var formContext = this.createFormContext(content);
                for (var key in this.metadataStepFormByName) {
                    if (this.metadataStepFormByName.hasOwnProperty(key)) {
                        this.metadataStepFormByName[key].removeChildren();
                        var mixinName = new MixinName(key);
                        var extraData = content.getExtraData(mixinName);
                        if (!extraData) {
                            extraData = new ExtraData(mixinName, new PropertyTree());
                            content.getAllExtraData().push(extraData);
                        }
                        this.metadataStepFormByName[key].layout(formContext, extraData.getData(), this.metadataStepFormByName[key].getForm());
                    }
                }
            };
            ContentWizardPanel.prototype.initPublishButtonForMobile = function () {
                var _this = this;
                var action = new api.ui.Action("Publish", "enter");
                action.setIconClass("publish-action");
                action.onExecuted(function () {
                    _this.publishAction.execute();
                });
                this.publishButtonForMobile = new DialogButton(action);
                this.publishButtonForMobile.addClass("mobile-edit-publish-button");
                this.subscribePublishButtonForMobileToPublishEvents();
                var publishButtonContainer = this.getSplitPanel();
                if (!publishButtonContainer) {
                    publishButtonContainer = this.getWizardStepsPanel();
                }
                publishButtonContainer.appendChild(this.publishButtonForMobile);
            };
            ContentWizardPanel.prototype.managePublishButtonStateForMobile = function (compareStatus) {
                var canBeShown = compareStatus !== 6 /* EQUAL */;
                this.publishButtonForMobile.toggleClass("visible", canBeShown);
                this.publishButtonForMobile.setLabel("Publish " + api.content.CompareStatusFormatter.formatStatus(compareStatus) + " item");
            };
            ContentWizardPanel.prototype.subscribePublishButtonForMobileToPublishEvents = function () {
                var _this = this;
                var publishHandler = function (event) {
                    if (_this.getPersistedItem() && event.getContentIds()) {
                        var isPublished = (event.getContentIds().some(function (obj) {
                            return obj.toString() == _this.getPersistedItem().getId();
                        }));
                        if (isPublished) {
                            _this.managePublishButtonStateForMobile(6 /* EQUAL */);
                        }
                    }
                };
                var publishHandlerOfServerEvent = function (event) {
                    if (_this.getPersistedItem() && event.getContentId() && (_this.getPersistedItem().getId() === event.getContentId().toString())) {
                        _this.managePublishButtonStateForMobile(6 /* EQUAL */);
                    }
                };
                api.content.ContentsPublishedEvent.on(publishHandler);
                api.content.ContentPublishedEvent.on(publishHandlerOfServerEvent);
                this.onClosed(function () {
                    api.content.ContentPublishedEvent.un(publishHandlerOfServerEvent);
                    api.content.ContentsPublishedEvent.un(publishHandler);
                });
            };
            ContentWizardPanel.prototype.openLiveEdit = function () {
                this.getSplitPanel().showSecondPanel();
                this.liveFormPanel.clearPageViewSelectionAndOpenInspectPage();
                this.showMinimizeEditButton();
            };
            ContentWizardPanel.prototype.closeLiveEdit = function () {
                this.getSplitPanel().hideSecondPanel();
                this.hideMinimizeEditButton();
                if (this.isMinimized()) {
                    this.toggleMinimize();
                }
            };
            ContentWizardPanel.prototype.contentNotRenderable = function () {
                return this.liveEditModel.getPageModel().getMode() == 3 /* NO_CONTROLLER */;
            };
            ContentWizardPanel.prototype.updatePreviewActionVisibility = function () {
                var _this = this;
                this.previewAction.setEnabled(!this.contentNotRenderable());
                this.liveEditModel.getPageModel().onPageModeChanged(function () {
                    _this.previewAction.setEnabled(!_this.contentNotRenderable());
                });
            };
            return ContentWizardPanel;
        })(api.app.wizard.WizardPanel);
        _wizard.ContentWizardPanel = ContentWizardPanel;
    })(wizard = _app.wizard || (_app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var PersistedNewContentRoutineContext = (function () {
            function PersistedNewContentRoutineContext() {
                this.content = null;
            }
            return PersistedNewContentRoutineContext;
        })();
        wizard.PersistedNewContentRoutineContext = PersistedNewContentRoutineContext;
        var PersistNewContentRoutine = (function (_super) {
            __extends(PersistNewContentRoutine, _super);
            function PersistNewContentRoutine(thisOfProducer) {
                _super.call(this, thisOfProducer);
                this.doneHandledContent = false;
            }
            PersistNewContentRoutine.prototype.setCreateContentRequestProducer = function (producer) {
                this.createContentRequestProducer = producer;
                return this;
            };
            PersistNewContentRoutine.prototype.execute = function () {
                var context = new PersistedNewContentRoutineContext();
                return this.doExecute(context);
            };
            PersistNewContentRoutine.prototype.doExecuteNext = function (context) {
                var _this = this;
                if (!this.doneHandledContent) {
                    return this.doHandleCreateContent(context).then(function () {
                        _this.doneHandledContent = true;
                        return _this.doExecuteNext(context);
                    });
                }
                else {
                    return wemQ(context.content);
                }
            };
            PersistNewContentRoutine.prototype.doHandleCreateContent = function (context) {
                if (this.createContentRequestProducer != undefined) {
                    return this.createContentRequestProducer.call(this.getThisOfProducer()).then(function (createContentRequest) {
                        return createContentRequest.sendAndParse().then(function (content) {
                            context.content = content;
                        });
                    });
                }
                else {
                    return api.util.PromiseHelper.newResolvedVoidPromise();
                }
            };
            return PersistNewContentRoutine;
        })(api.util.Flow);
        wizard.PersistNewContentRoutine = PersistNewContentRoutine;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var CreatePageRequest = api.content.page.CreatePageRequest;
        var UpdatePageRequest = api.content.page.UpdatePageRequest;
        var DeletePageRequest = api.content.page.DeletePageRequest;
        var UpdatePersistedContentRoutineContext = (function () {
            function UpdatePersistedContentRoutineContext() {
                this.content = null;
            }
            return UpdatePersistedContentRoutineContext;
        })();
        wizard.UpdatePersistedContentRoutineContext = UpdatePersistedContentRoutineContext;
        var UpdatePersistedContentRoutine = (function (_super) {
            __extends(UpdatePersistedContentRoutine, _super);
            function UpdatePersistedContentRoutine(thisOfProducer, persistedContent, viewedContent) {
                _super.call(this, thisOfProducer);
                this.doneHandledContent = false;
                this.doneHandledPage = false;
                this.persistedContent = persistedContent;
                this.viewedContent = viewedContent;
            }
            UpdatePersistedContentRoutine.prototype.setUpdateContentRequestProducer = function (producer) {
                this.updateContentRequestProducer = producer;
                return this;
            };
            UpdatePersistedContentRoutine.prototype.execute = function () {
                var context = new UpdatePersistedContentRoutineContext();
                context.content = this.persistedContent;
                return this.doExecute(context);
            };
            UpdatePersistedContentRoutine.prototype.doExecuteNext = function (context) {
                var _this = this;
                if (!this.doneHandledContent) {
                    return this.doHandleUpdateContent(context).then(function () {
                        _this.doneHandledContent = true;
                        return _this.doExecuteNext(context);
                    });
                }
                else if (!this.doneHandledPage) {
                    return this.doHandlePage(context).then(function () {
                        _this.doneHandledPage = true;
                        return _this.doExecuteNext(context);
                    });
                }
                else {
                    return wemQ(context.content);
                }
            };
            UpdatePersistedContentRoutine.prototype.doHandleUpdateContent = function (context) {
                return this.updateContentRequestProducer.call(this.getThisOfProducer(), context.content, this.viewedContent).sendAndParse().then(function (content) {
                    context.content = content;
                });
            };
            UpdatePersistedContentRoutine.prototype.doHandlePage = function (context) {
                var pageCUDRequest = this.producePageCUDRequest(context.content, this.viewedContent);
                if (pageCUDRequest != null) {
                    return pageCUDRequest.sendAndParse().then(function (content) {
                        context.content = content;
                    });
                }
                else {
                    var deferred = wemQ.defer();
                    deferred.resolve(null);
                    return deferred.promise;
                }
            };
            UpdatePersistedContentRoutine.prototype.producePageCUDRequest = function (persistedContent, viewedContent) {
                if (persistedContent.isPage() && !viewedContent.isPage()) {
                    return new DeletePageRequest(persistedContent.getContentId());
                }
                else if (!persistedContent.isPage() && viewedContent.isPage()) {
                    var viewedPage = viewedContent.getPage();
                    return new CreatePageRequest(persistedContent.getContentId()).setController(viewedPage.getController()).setPageTemplateKey(viewedPage.getTemplate()).setConfig(viewedPage.getConfig()).setRegions(viewedPage.getRegions()).setCustomized(viewedPage.isCustomized());
                }
                else if (persistedContent.isPage() && viewedContent.isPage()) {
                    var viewedPage = viewedContent.getPage();
                    return new UpdatePageRequest(persistedContent.getContentId()).setController((viewedPage.getController())).setPageTemplateKey((viewedPage.getTemplate())).setConfig(viewedPage.getConfig()).setRegions(viewedPage.getRegions()).setCustomized(viewedPage.isCustomized());
                }
            };
            return UpdatePersistedContentRoutine;
        })(api.util.Flow);
        wizard.UpdatePersistedContentRoutine = UpdatePersistedContentRoutine;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (_wizard) {
        var Content = api.content.Content;
        var DefaultModels = app.wizard.page.DefaultModels;
        var DefaultModelsFactory = app.wizard.page.DefaultModelsFactory;
        var ContentWizardPanelFactory = (function () {
            function ContentWizardPanelFactory() {
                this.createSite = false;
            }
            ContentWizardPanelFactory.prototype.setContentIdToEdit = function (value) {
                this.contentId = value;
                return this;
            };
            ContentWizardPanelFactory.prototype.setContentTypeName = function (value) {
                this.contentTypeName = value;
                return this;
            };
            ContentWizardPanelFactory.prototype.setParentContent = function (value) {
                this.parentContent = value;
                return this;
            };
            ContentWizardPanelFactory.prototype.setCreateSite = function (value) {
                this.createSite = value;
                return this;
            };
            ContentWizardPanelFactory.prototype.setAppBarTabId = function (value) {
                this.appBarTabId = value;
                return this;
            };
            ContentWizardPanelFactory.prototype.createForNew = function () {
                var _this = this;
                this.creatingForNew = true;
                return this.loadContentType(this.contentTypeName).then(function (loadedContentType) {
                    _this.contentType = loadedContentType;
                    return _this.loadParentContent();
                }).then(function (loadedParentContent) {
                    _this.parentContent = loadedParentContent;
                    return _this.loadSite(loadedParentContent ? loadedParentContent.getContentId() : null);
                }).then(function (loadedSite) {
                    _this.siteContent = loadedSite;
                    return _this.loadDefaultModels(_this.siteContent, _this.contentTypeName);
                }).then(function (defaultModels) {
                    _this.defaultModels = defaultModels;
                    return _this.newContentWizardPanelForNew();
                });
            };
            ContentWizardPanelFactory.prototype.createForEdit = function (content) {
                var _this = this;
                this.creatingForNew = false;
                return this.getContentToEdit(content).then(function (contentType) {
                    _this.contentType = contentType;
                    return _this.loadParentContent();
                }).then(function (loadedParentContent) {
                    _this.parentContent = loadedParentContent;
                    return _this.loadSite(_this.contentId);
                }).then(function (loadedSite) {
                    if (!!loadedSite) {
                        _this.siteContent = loadedSite;
                    }
                    return _this.loadDefaultModels(_this.siteContent, _this.contentToEdit.getType());
                }).then(function (defaultModels) {
                    _this.defaultModels = defaultModels;
                    return _this.newContentWizardPanelForEdit();
                });
            };
            ContentWizardPanelFactory.prototype.loadContentToEdit = function () {
                return new api.content.GetContentByIdRequest(this.contentId).sendAndParse();
            };
            ContentWizardPanelFactory.prototype.getContentToEdit = function (content) {
                var _this = this;
                if (api.ObjectHelper.iFrameSafeInstanceOf(content, Content)) {
                    this.contentToEdit = content;
                    return this.loadContentType(this.contentToEdit.getType());
                }
                else {
                    return this.loadContentToEdit().then(function (loadedContentToEdit) {
                        _this.contentToEdit = loadedContentToEdit;
                        return _this.loadContentType(_this.contentToEdit.getType());
                    });
                }
            };
            ContentWizardPanelFactory.prototype.loadContentType = function (name) {
                var deferred = wemQ.defer();
                new api.schema.content.GetContentTypeByNameRequest(name).sendAndParse().then(function (contentType) {
                    deferred.resolve(contentType);
                }).catch(function (reason) {
                    deferred.reject(new api.Exception("Content cannot be opened. Required content type '" + name.toString() + "' not found.", 2 /* WARNING */));
                }).done();
                return deferred.promise;
            };
            ContentWizardPanelFactory.prototype.loadSite = function (contentId) {
                return contentId ? new api.content.GetNearestSiteRequest(contentId).sendAndParse() : wemQ(null);
            };
            ContentWizardPanelFactory.prototype.loadDefaultModels = function (site, contentType) {
                if (site) {
                    return DefaultModelsFactory.create({
                        siteId: site.getContentId(),
                        contentType: contentType,
                        applications: site.getApplicationKeys()
                    });
                }
                else if (contentType.isSite()) {
                    return wemQ(new DefaultModels(null, null));
                }
                else {
                    return wemQ(null);
                }
            };
            ContentWizardPanelFactory.prototype.loadParentContent = function () {
                if (this.parentContent != null) {
                    return wemQ(this.parentContent);
                }
                else if (!this.creatingForNew && !this.contentToEdit.hasParent()) {
                    return wemQ(null);
                }
                else if (this.creatingForNew && this.parentContent == null) {
                    return wemQ(null);
                }
                return new api.content.GetContentByPathRequest(this.contentToEdit.getPath().getParentPath()).sendAndParse();
            };
            ContentWizardPanelFactory.prototype.newContentWizardPanelForNew = function () {
                var deferred = wemQ.defer();
                var wizardParams = new app.wizard.ContentWizardPanelParams().setAppBarTabId(this.appBarTabId).setContentType(this.contentType).setParentContent(this.parentContent).setSite(this.siteContent).setDefaultModels(this.defaultModels);
                wizardParams.setCreateSite(this.createSite);
                new app.wizard.ContentWizardPanel(wizardParams, function (wizard) {
                    deferred.resolve(wizard);
                }, function (reason) {
                    deferred.reject(reason);
                });
                return deferred.promise;
            };
            ContentWizardPanelFactory.prototype.newContentWizardPanelForEdit = function () {
                var deferred = wemQ.defer();
                var wizardParams = new _wizard.ContentWizardPanelParams().setAppBarTabId(this.appBarTabId).setContentType(this.contentType).setParentContent(this.parentContent).setPersistedContent(this.contentToEdit).setSite(this.siteContent).setDefaultModels(this.defaultModels);
                new _wizard.ContentWizardPanel(wizardParams, function (wizard) {
                    deferred.resolve(wizard);
                }, function (reason) {
                    deferred.reject(reason);
                });
                return deferred.promise;
            };
            return ContentWizardPanelFactory;
        })();
        _wizard.ContentWizardPanelFactory = ContentWizardPanelFactory;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ContentWizardPanelParams = (function () {
            function ContentWizardPanelParams() {
                this.createSite = false;
            }
            ContentWizardPanelParams.prototype.setAppBarTabId = function (value) {
                this.tabId = value;
                return this;
            };
            ContentWizardPanelParams.prototype.setContentType = function (value) {
                this.contentType = value;
                return this;
            };
            ContentWizardPanelParams.prototype.setParentContent = function (value) {
                this.parentContent = value;
                return this;
            };
            ContentWizardPanelParams.prototype.setPersistedContent = function (value) {
                this.persistedContent = value;
                return this;
            };
            ContentWizardPanelParams.prototype.setSite = function (value) {
                this.site = value;
                return this;
            };
            ContentWizardPanelParams.prototype.setDefaultModels = function (value) {
                this.defaultModels = value;
                return this;
            };
            ContentWizardPanelParams.prototype.setCreateSite = function (value) {
                this.createSite = value;
                return this;
            };
            return ContentWizardPanelParams;
        })();
        wizard.ContentWizardPanelParams = ContentWizardPanelParams;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ShowContentFormEvent = (function (_super) {
            __extends(ShowContentFormEvent, _super);
            function ShowContentFormEvent() {
                _super.apply(this, arguments);
            }
            ShowContentFormEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowContentFormEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowContentFormEvent;
        })(api.event.Event);
        wizard.ShowContentFormEvent = ShowContentFormEvent;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ShowLiveEditEvent = (function (_super) {
            __extends(ShowLiveEditEvent, _super);
            function ShowLiveEditEvent() {
                _super.apply(this, arguments);
            }
            ShowLiveEditEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowLiveEditEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowLiveEditEvent;
        })(api.event.Event);
        wizard.ShowLiveEditEvent = ShowLiveEditEvent;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ShowSplitEditEvent = (function (_super) {
            __extends(ShowSplitEditEvent, _super);
            function ShowSplitEditEvent() {
                _super.apply(this, arguments);
            }
            ShowSplitEditEvent.on = function (handler) {
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler);
            };
            ShowSplitEditEvent.un = function (handler) {
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler);
            };
            return ShowSplitEditEvent;
        })(api.event.Event);
        wizard.ShowSplitEditEvent = ShowSplitEditEvent;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var GridColumnBuilder = api.ui.grid.GridColumnBuilder;
        var ContentSummaryViewer = api.content.ContentSummaryViewer;
        var TreeGrid = api.ui.treegrid.TreeGrid;
        var TreeGridBuilder = api.ui.treegrid.TreeGridBuilder;
        var ContentSummaryAndCompareStatusFetcher = api.content.ContentSummaryAndCompareStatusFetcher;
        var CompareContentGrid = (function (_super) {
            __extends(CompareContentGrid, _super);
            function CompareContentGrid(content) {
                var _this = this;
                _super.call(this, new TreeGridBuilder().setColumns([
                    new GridColumnBuilder().setName("Name").setId("displayName").setField("displayName").setFormatter(this.nameFormatter).build()
                ]).setPartialLoadEnabled(true).setLoadBufferSize(20).prependClasses("compare-content-grid"));
                this.content = content;
                this.onLoaded(function () {
                    _this.selectAll();
                });
            }
            CompareContentGrid.prototype.nameFormatter = function (row, cell, value, columnDef, node) {
                var viewer = node.getViewer("name");
                if (!viewer) {
                    viewer = new ContentSummaryViewer();
                    viewer.setObject(node.getData().getContentSummary());
                    node.setViewer("name", viewer);
                }
                return viewer.toString();
            };
            CompareContentGrid.prototype.fetchChildren = function (parentNode) {
                var parentContentId = parentNode && parentNode.getData() ? parentNode.getData().getContentId() : null;
                return api.content.ContentSummaryAndCompareStatusFetcher.fetchChildren(parentContentId).then(function (data) {
                    return data.getContents();
                });
            };
            CompareContentGrid.prototype.hasChildren = function (elem) {
                return elem.hasChildren();
            };
            CompareContentGrid.prototype.getDataId = function (data) {
                return data.getId();
            };
            CompareContentGrid.prototype.refreshNodeData = function (parentNode) {
                var _this = this;
                return ContentSummaryAndCompareStatusFetcher.fetch(parentNode.getData().getContentId()).then(function (content) {
                    parentNode.setData(content);
                    _this.refreshNode(parentNode);
                    return parentNode;
                });
            };
            CompareContentGrid.prototype.sortNodeChildren = function (node) {
                var comparator;
                if (this.getRoot().getCurrentRoot() == node) {
                    comparator = new api.content.ContentNodeByDisplayNameComparator();
                }
                else {
                    comparator = new api.content.ContentNodeByModifiedTimeComparator();
                }
                var children = node.getChildren().sort(comparator.compare);
                node.setChildren(children);
                this.initData(this.getRoot().getCurrentRoot().treeToList());
            };
            return CompareContentGrid;
        })(TreeGrid);
        wizard.CompareContentGrid = CompareContentGrid;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var PublishAction = (function (_super) {
            __extends(PublishAction, _super);
            function PublishAction() {
                _super.call(this, PublishAction.BASE_STRING);
            }
            PublishAction.prototype.setToBePublishedAmount = function (amount) {
                this.setEnabled(amount >= 1);
                this.setLabel(PublishAction.BASE_STRING + " (" + amount + ")");
            };
            PublishAction.BASE_STRING = "Publish now";
            return PublishAction;
        })(api.ui.Action);
        wizard.PublishAction = PublishAction;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ScheduleAction = (function (_super) {
            __extends(ScheduleAction, _super);
            function ScheduleAction() {
                _super.call(this, "Schedule");
            }
            return ScheduleAction;
        })(api.ui.Action);
        wizard.ScheduleAction = ScheduleAction;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var OpenEditPermissionsDialogEvent = (function (_super) {
            __extends(OpenEditPermissionsDialogEvent, _super);
            function OpenEditPermissionsDialogEvent(content) {
                _super.call(this);
                this.content = content;
            }
            OpenEditPermissionsDialogEvent.prototype.getContent = function () {
                return this.content;
            };
            OpenEditPermissionsDialogEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            OpenEditPermissionsDialogEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return OpenEditPermissionsDialogEvent;
        })(api.event.Event);
        wizard.OpenEditPermissionsDialogEvent = OpenEditPermissionsDialogEvent;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ContentPermissionsAppliedEvent = (function (_super) {
            __extends(ContentPermissionsAppliedEvent, _super);
            function ContentPermissionsAppliedEvent(content) {
                _super.call(this);
                this.content = content;
            }
            ContentPermissionsAppliedEvent.prototype.getContent = function () {
                return this.content;
            };
            ContentPermissionsAppliedEvent.on = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.bind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            ContentPermissionsAppliedEvent.un = function (handler, contextWindow) {
                if (contextWindow === void 0) { contextWindow = window; }
                api.event.Event.unbind(api.ClassHelper.getFullName(this), handler, contextWindow);
            };
            return ContentPermissionsAppliedEvent;
        })(api.event.Event);
        wizard.ContentPermissionsAppliedEvent = ContentPermissionsAppliedEvent;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var AccessControlComboBox = api.ui.security.acl.AccessControlComboBox;
        var AccessControlList = api.security.acl.AccessControlList;
        var EditPermissionsDialog = (function (_super) {
            __extends(EditPermissionsDialog, _super);
            function EditPermissionsDialog() {
                var _this = this;
                this.dialogTitle = new EditPermissionsDialogTitle('Edit Permissions', '');
                _super.call(this, {
                    title: this.dialogTitle
                });
                this.addClass('edit-permissions-dialog');
                this.inheritPermissionsCheck = new api.ui.Checkbox().setLabel('Inherit permissions');
                this.inheritPermissionsCheck.addClass('inherit-perm-check');
                this.appendChildToContentPanel(this.inheritPermissionsCheck);
                var section = new api.dom.SectionEl();
                this.appendChildToContentPanel(section);
                var form = new api.ui.form.Form();
                section.appendChild(form);
                this.comboBox = new AccessControlComboBox();
                this.comboBox.addClass('principal-combobox');
                form.appendChild(this.comboBox);
                var comboBoxChangeListener = function () {
                    var currentEntries = _this.getEntries().sort();
                    var permissionsModified = !api.ObjectHelper.arrayEquals(currentEntries, _this.originalValues);
                    var inheritCheckModified = _this.inheritPermissionsCheck.isChecked() !== _this.originalInherit;
                    _this.applyAction.setEnabled(permissionsModified || inheritCheckModified);
                };
                var changeListener = function () {
                    var inheritPermissions = _this.inheritPermissionsCheck.isChecked();
                    _this.comboBox.toggleClass('disabled', inheritPermissions);
                    if (inheritPermissions) {
                        _this.layoutInheritedPermissions();
                    }
                    _this.comboBox.getComboBox().setVisible(!inheritPermissions);
                    _this.comboBox.setEditable(!inheritPermissions);
                    comboBoxChangeListener();
                };
                this.inheritPermissionsCheck.onValueChanged(changeListener);
                this.comboBox.onOptionValueChanged(comboBoxChangeListener);
                this.comboBox.onOptionSelected(comboBoxChangeListener);
                this.comboBox.onOptionDeselected(comboBoxChangeListener);
                this.overwriteChildPermissionsCheck = new api.ui.Checkbox().setLabel('Overwrite child permissions');
                this.overwriteChildPermissionsCheck.addClass('overwrite-child-check');
                this.appendChildToContentPanel(this.overwriteChildPermissionsCheck);
                this.applyAction = new api.ui.Action('Apply');
                this.applyAction.onExecuted(function () {
                    _this.applyPermissions();
                });
                this.addAction(this.applyAction, true);
                api.dom.Body.get().appendChild(this);
                this.parentPermissions = [];
                wizard.OpenEditPermissionsDialogEvent.on(function (event) {
                    _this.content = event.getContent();
                    _this.getParentPermissions().then(function (parentPermissions) {
                        _this.parentPermissions = parentPermissions.getEntries();
                        _this.setUpDialog();
                        _this.open();
                    }).catch(function (reason) {
                        api.notify.showWarning('Could not read inherit permissions for content ' + _this.content.getPath().toString());
                    }).done();
                });
                this.addCancelButtonToBottom();
            }
            EditPermissionsDialog.prototype.applyPermissions = function () {
                var _this = this;
                var permissions = new AccessControlList(this.getEntries());
                var req = new api.content.ApplyContentPermissionsRequest().setId(this.content.getId()).setInheritPermissions(this.inheritPermissionsCheck.isChecked()).setPermissions(permissions).setOverwriteChildPermissions(this.overwriteChildPermissionsCheck.isChecked());
                var res = req.sendAndParse();
                res.done(function (updatedContent) {
                    new wizard.ContentPermissionsAppliedEvent(updatedContent).fire();
                    api.notify.showFeedback('Permissions applied to content ' + updatedContent.getPath().toString());
                    _this.close();
                });
            };
            EditPermissionsDialog.prototype.setUpDialog = function () {
                var _this = this;
                this.comboBox.clearSelection(true);
                this.overwriteChildPermissionsCheck.setChecked(false);
                var contentPermissions = this.content.getPermissions();
                var contentPermissionsEntries = contentPermissions.getEntries();
                this.originalValues = contentPermissionsEntries.sort();
                this.originalInherit = this.content.isInheritPermissionsEnabled();
                this.originalValues.forEach(function (item) {
                    if (!_this.comboBox.isSelected(item)) {
                        _this.comboBox.select(item);
                    }
                });
                this.inheritPermissionsCheck.setChecked(this.content.isInheritPermissionsEnabled());
                this.comboBox.giveFocus();
            };
            EditPermissionsDialog.prototype.layoutInheritedPermissions = function () {
                var _this = this;
                this.comboBox.clearSelection(true);
                this.parentPermissions.forEach(function (item) {
                    if (!_this.comboBox.isSelected(item)) {
                        _this.comboBox.select(item);
                    }
                });
            };
            EditPermissionsDialog.prototype.getEntries = function () {
                return this.comboBox.getSelectedDisplayValues();
            };
            EditPermissionsDialog.prototype.getParentPermissions = function () {
                var _this = this;
                var deferred = wemQ.defer();
                var parentPath = this.content.getPath().getParentPath();
                if (parentPath && parentPath.isNotRoot()) {
                    new api.content.GetContentByPathRequest(parentPath).sendAndParse().then(function (content) {
                        deferred.resolve(content.getPermissions());
                    }).catch(function (reason) {
                        deferred.reject(new Error("Inherit permissions for [" + _this.content.getPath().toString() + "] could not be retrieved"));
                    }).done();
                }
                else {
                    new api.content.GetContentRootPermissionsRequest().sendAndParse().then(function (rootPermissions) {
                        deferred.resolve(rootPermissions);
                    }).catch(function (reason) {
                        deferred.reject(new Error("Inherit permissions for [" + _this.content.getPath().toString() + "] could not be retrieved"));
                    }).done();
                }
                return deferred.promise;
            };
            EditPermissionsDialog.prototype.show = function () {
                if (this.content) {
                    this.dialogTitle.setPath(this.content.getPath().toString());
                }
                else {
                    this.dialogTitle.setPath('');
                }
                _super.prototype.show.call(this);
            };
            return EditPermissionsDialog;
        })(api.ui.dialog.ModalDialog);
        wizard.EditPermissionsDialog = EditPermissionsDialog;
        var EditPermissionsDialogTitle = (function (_super) {
            __extends(EditPermissionsDialogTitle, _super);
            function EditPermissionsDialogTitle(title, path) {
                _super.call(this, title);
                this.pathEl = new api.dom.PEl('path');
                this.pathEl.setHtml(path);
                this.appendChild(this.pathEl);
            }
            EditPermissionsDialogTitle.prototype.setPath = function (path) {
                this.pathEl.setHtml(path);
            };
            return EditPermissionsDialogTitle;
        })(api.ui.dialog.ModalDialogHeader);
        wizard.EditPermissionsDialogTitle = EditPermissionsDialogTitle;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var DialogButton = api.ui.dialog.DialogButton;
        var SpanEl = api.dom.SpanEl;
        var CompareStatus = api.content.CompareStatus;
        var ContentWizardToolbarPublishControls = (function (_super) {
            __extends(ContentWizardToolbarPublishControls, _super);
            function ContentWizardToolbarPublishControls(action) {
                _super.call(this, "toolbar-publish-controls");
                this.contentCanBePublished = false;
                this.userCanPublish = true;
                this.publishAction = action;
                this.publishAction.setIconClass("publish-action");
                this.publishButton = new DialogButton(action);
                this.publishButton.addClass("content-wizard-toolbar-publish-button");
                this.contentStateSpan = new SpanEl("content-status");
                this.appendChildren(this.contentStateSpan, this.publishButton);
            }
            ContentWizardToolbarPublishControls.prototype.setCompareStatus = function (compareStatus, refresh) {
                if (refresh === void 0) { refresh = true; }
                this.contentCompareStatus = compareStatus;
                if (refresh) {
                    this.refreshState();
                }
            };
            ContentWizardToolbarPublishControls.prototype.setContentCanBePublished = function (value, refresh) {
                if (refresh === void 0) { refresh = true; }
                this.contentCanBePublished = value;
                if (refresh) {
                    this.refreshState();
                }
            };
            ContentWizardToolbarPublishControls.prototype.setUserCanPublish = function (value, refresh) {
                if (refresh === void 0) { refresh = true; }
                this.userCanPublish = value;
                if (refresh) {
                    this.refreshState();
                }
            };
            ContentWizardToolbarPublishControls.prototype.refreshState = function () {
                var canBeEnabled = this.contentCompareStatus !== 6 /* EQUAL */ && this.contentCanBePublished && this.userCanPublish;
                this.publishAction.setEnabled(canBeEnabled);
                this.contentStateSpan.setHtml(this.getContentStateValueForSpan(this.contentCompareStatus));
            };
            ContentWizardToolbarPublishControls.prototype.isOnline = function () {
                return this.contentCompareStatus === 6 /* EQUAL */;
            };
            ContentWizardToolbarPublishControls.prototype.enableActionsForExisting = function (existing) {
                var _this = this;
                new api.security.auth.IsAuthenticatedRequest().sendAndParse().then(function (loginResult) {
                    var hasPublishPermission = api.security.acl.PermissionHelper.hasPermission(4 /* PUBLISH */, loginResult, existing.getPermissions());
                    _this.setUserCanPublish(hasPublishPermission);
                });
            };
            ContentWizardToolbarPublishControls.prototype.getContentStateValueForSpan = function (compareStatus) {
                var status = new api.dom.SpanEl();
                if (compareStatus === 6 /* EQUAL */) {
                    status.addClass("online");
                }
                status.setHtml(api.content.CompareStatusFormatter.formatStatus(compareStatus));
                return "Item is " + status.toString();
            };
            return ContentWizardToolbarPublishControls;
        })(api.dom.DivEl);
        wizard.ContentWizardToolbarPublishControls = ContentWizardToolbarPublishControls;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var PageItemType = api.liveedit.PageItemType;
        var PageComponentsItemViewer = (function (_super) {
            __extends(PageComponentsItemViewer, _super);
            function PageComponentsItemViewer(content) {
                this.content = content;
                _super.call(this, 'page-components-item-viewer');
            }
            PageComponentsItemViewer.prototype.resolveDisplayName = function (object) {
                return object.getName();
            };
            PageComponentsItemViewer.prototype.resolveSubName = function (object, relativePath) {
                if (relativePath === void 0) { relativePath = false; }
                return object.getType() ? object.getType().getShortName() : "";
            };
            PageComponentsItemViewer.prototype.resolveIconUrl = function (object) {
                if (PageItemType.get().equals(object.getType())) {
                    return new api.content.ContentIconUrlResolver().setContent(this.content).resolve();
                }
                return null;
            };
            PageComponentsItemViewer.prototype.resolveIconClass = function (object) {
                return api.liveedit.ItemViewIconClassResolver.resolve(object);
            };
            return PageComponentsItemViewer;
        })(api.ui.NamesAndIconViewer);
        wizard.PageComponentsItemViewer = PageComponentsItemViewer;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var GridColumnBuilder = api.ui.grid.GridColumnBuilder;
        var GridOptionsBuilder = api.ui.grid.GridOptionsBuilder;
        var TreeGrid = api.ui.treegrid.TreeGrid;
        var TreeGridBuilder = api.ui.treegrid.TreeGridBuilder;
        var PageView = api.liveedit.PageView;
        var PageItemType = api.liveedit.PageItemType;
        var RegionItemType = api.liveedit.RegionItemType;
        var RegionView = api.liveedit.RegionView;
        var LayoutItemType = api.liveedit.layout.LayoutItemType;
        var PageComponentsTreeGrid = (function (_super) {
            __extends(PageComponentsTreeGrid, _super);
            function PageComponentsTreeGrid(content, pageView) {
                this.content = content;
                this.pageView = pageView;
                _super.call(this, new TreeGridBuilder().setColumns([
                    new GridColumnBuilder().setName("Name").setId("displayName").setField("displayName").setFormatter(this.nameFormatter.bind(this)).setMinWidth(250).setBehavior("selectAndMove").build(),
                    new GridColumnBuilder().setName("Menu").setId("menu").setMinWidth(45).setMaxWidth(45).setField("menu").setCssClass("menu-cell").setFormatter(this.menuFormatter).build()
                ]).setOptions(new GridOptionsBuilder().setAutoHeight(true).setShowHeaderRow(false).setHideColumnHeaders(true).setForceFitColumns(true).setFullWidthRows(true).setEnableCellNavigation(false).setSelectedCellCssClass("selected cell").setCheckableRows(false).disableMultipleSelection(true).setMultiSelect(false).setRowHeight(45).setDragAndDrop(true).build()).setShowToolbar(false).setAutoLoad(true).setExpandAll(true).prependClasses("components-grid"));
                this.gridDragHandler = new wizard.PageComponentsGridDragHandler(this);
            }
            PageComponentsTreeGrid.prototype.queryScrollable = function () {
                return this;
            };
            PageComponentsTreeGrid.prototype.setPageView = function (pageView) {
                this.pageView = pageView;
                this.reload();
            };
            PageComponentsTreeGrid.prototype.nameFormatter = function (row, cell, value, columnDef, node) {
                var viewer = node.getViewer("name");
                if (!viewer) {
                    var viewer = new wizard.PageComponentsItemViewer(this.content);
                    var data = node.getData();
                    viewer.setObject(data);
                    node.setViewer("name", viewer);
                    if (!(api.ObjectHelper.iFrameSafeInstanceOf(data, RegionView) || api.ObjectHelper.iFrameSafeInstanceOf(data, PageView))) {
                        viewer.addClass("draggable");
                    }
                }
                return viewer.toString();
            };
            PageComponentsTreeGrid.prototype.getDataId = function (data) {
                return data.getItemId().toString();
            };
            PageComponentsTreeGrid.prototype.hasChildren = function (data) {
                return this.getDataChildren(data).length > 0;
            };
            PageComponentsTreeGrid.prototype.fetch = function (node, dataId) {
                var deferred = wemQ.defer();
                var itemViewId = dataId ? new api.liveedit.ItemViewId(parseInt(dataId)) : node.getData().getItemId();
                deferred.resolve(this.pageView.getItemViewById(itemViewId));
                return deferred.promise;
            };
            PageComponentsTreeGrid.prototype.fetchRoot = function () {
                var deferred = wemQ.defer();
                deferred.resolve([this.pageView]);
                return deferred.promise;
            };
            PageComponentsTreeGrid.prototype.fetchChildren = function (parentNode) {
                var deferred = wemQ.defer();
                deferred.resolve(this.getDataChildren(parentNode.getData()));
                return deferred.promise;
            };
            PageComponentsTreeGrid.prototype.getDataChildren = function (data) {
                var children = [];
                var dataType = data.getType();
                if (PageItemType.get().equals(dataType)) {
                    var pageView = data;
                    children = pageView.getRegions();
                }
                else if (RegionItemType.get().equals(dataType)) {
                    var regionView = data;
                    children = regionView.getComponentViews();
                }
                else if (LayoutItemType.get().equals(dataType)) {
                    var layoutView = data;
                    children = layoutView.getRegions();
                }
                return children;
            };
            PageComponentsTreeGrid.prototype.menuFormatter = function (row, cell, value, columnDef, node) {
                var wrapper = new api.dom.SpanEl();
                var icon = new api.dom.DivEl("menu-icon");
                wrapper.getEl().setInnerHtml(icon.toString());
                return wrapper.toString();
            };
            return PageComponentsTreeGrid;
        })(TreeGrid);
        wizard.PageComponentsTreeGrid = PageComponentsTreeGrid;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var ResponsiveManager = api.ui.responsive.ResponsiveManager;
        var ResponsiveRanges = api.ui.responsive.ResponsiveRanges;
        var PageComponentsView = (function (_super) {
            __extends(PageComponentsView, _super);
            function PageComponentsView(liveEditPage) {
                var _this = this;
                _super.call(this, 'page-components-view');
                this.mouseDown = false;
                this.liveEditPage = liveEditPage;
                var closeButton = new api.ui.button.CloseButton();
                closeButton.onClicked(function (event) { return _this.hide(); });
                this.header = new api.dom.H2El('header');
                this.header.setHtml('Page Components');
                this.appendChildren(closeButton, this.header);
                this.setModal(false).setFloating(true).setDraggable(true);
                this.onShown(function (event) {
                    _this.constrainToParent();
                });
                ResponsiveManager.onAvailableSizeChanged(api.dom.Body.get(), function (item) {
                    var smallSize = item.isInRangeOrSmaller(ResponsiveRanges._360_540);
                    if (!smallSize && _this.isVisible()) {
                        _this.constrainToParent();
                    }
                    if (item.isRangeSizeChanged()) {
                        _this.setModal(smallSize).setDraggable(!smallSize);
                    }
                });
            }
            PageComponentsView.prototype.setPageView = function (pageView) {
                this.pageView = pageView;
                if (!this.tree && this.content && this.pageView) {
                    this.createTree(this.content, this.pageView);
                }
                else if (this.tree) {
                    this.tree.setPageView(pageView);
                }
            };
            PageComponentsView.prototype.setContent = function (content) {
                this.content = content;
                if (!this.tree && this.content && this.pageView) {
                    this.createTree(this.content, this.pageView);
                }
            };
            PageComponentsView.prototype.createTree = function (content, pageView) {
                var _this = this;
                this.tree = new wizard.PageComponentsTreeGrid(content, pageView);
                this.liveEditPage.onItemViewSelected(function (event) {
                    if (!event.isNew()) {
                        var selectedItemId = _this.tree.getDataId(event.getItemView());
                        _this.tree.selectNode(selectedItemId);
                        if (!event.getPosition()) {
                            _this.scrollToItem(selectedItemId);
                        }
                    }
                });
                this.liveEditPage.onItemViewDeselected(function (event) {
                    _this.tree.deselectNode(_this.tree.getDataId(event.getItemView()));
                });
                this.liveEditPage.onComponentAdded(function (event) {
                    var parentNode = _this.tree.getRoot().getCurrentRoot().findNode(_this.tree.getDataId(event.getParentRegionView()));
                    if (parentNode) {
                        _this.tree.deselectAll();
                        var index = event.getParentRegionView().getComponentViews().indexOf(event.getComponentView());
                        if (index >= 0) {
                            _this.tree.insertNode(event.getComponentView(), false, index, parentNode).then(function () {
                                _this.tree.expandNode(parentNode);
                                if (event.getComponentView().isSelected()) {
                                    _this.tree.selectNode(_this.tree.getDataId(event.getComponentView()));
                                }
                                if (_this.tree.hasChildren(event.getComponentView())) {
                                    var componentNode = _this.tree.getRoot().getCurrentRoot().findNode(_this.tree.getDataId(event.getComponentView()));
                                    _this.tree.expandNode(componentNode, true);
                                }
                                _this.constrainToParent();
                            });
                        }
                    }
                });
                this.liveEditPage.onComponentRemoved(function (event) {
                    _this.tree.deleteNode(event.getComponentView());
                    _this.tree.updateNode(event.getParentRegionView());
                });
                this.liveEditPage.onComponentLoaded(function (event) {
                    var oldDataId = _this.tree.getDataId(event.getOldComponentView());
                    _this.tree.updateNode(event.getNewComponentView(), oldDataId).then(function () {
                        var newDataId = _this.tree.getDataId(event.getNewComponentView());
                        if (_this.tree.hasChildren(event.getNewComponentView())) {
                            var newNode = _this.tree.getRoot().getCurrentRoot().findNode(newDataId);
                            _this.tree.expandNode(newNode);
                        }
                        if (event.getNewComponentView().isSelected()) {
                            _this.tree.selectNode(newDataId);
                            _this.scrollToItem(newDataId);
                        }
                    });
                });
                this.liveEditPage.onComponentReset(function (event) {
                    var oldDataId = _this.tree.getDataId(event.getOldComponentView());
                    if (_this.tree.hasChildren(event.getOldComponentView())) {
                        var oldNode = _this.tree.getRoot().getCurrentRoot().findNode(oldDataId);
                        oldNode.removeChildren();
                        _this.tree.refreshNode(oldNode);
                    }
                    _this.tree.updateNode(event.getNewComponentView(), oldDataId).then(function () {
                        if (event.getNewComponentView().isSelected()) {
                            var newDataId = _this.tree.getDataId(event.getNewComponentView());
                            _this.tree.selectNode(newDataId);
                        }
                    });
                });
                this.clickListener = function (event, data) {
                    var elem = new api.dom.ElementHelper(event.target);
                    if (_this.sameRowClicked(data.row)) {
                        _this.hideContextMenu();
                    }
                    if (elem.hasClass('toggle')) {
                        return;
                    }
                    var treeNode = _this.tree.getGrid().getDataView().getItem(data.row);
                    if (treeNode) {
                        treeNode.getData().selectWithoutMenu();
                    }
                    _this.tree.getGrid().selectRow(data.row);
                    if (_this.isMenuIconClicked(data.cell)) {
                        _this.showContextMenu(data.row, { x: event.pageX, y: event.pageY });
                    }
                    if (_this.isModal()) {
                        _this.hide();
                    }
                };
                this.tree.getGrid().subscribeOnClick(this.clickListener);
                this.tree.onSelectionChanged(function (data, nodes) {
                    if (nodes.length > 0) {
                        if (_this.isModal()) {
                            _this.hide();
                        }
                    }
                    _this.hideContextMenu();
                });
                this.tree.getGrid().subscribeOnContextMenu(function (event) {
                    event.stopPropagation();
                    event.preventDefault();
                    var cell = _this.tree.getGrid().getCellFromEvent(event);
                    _this.showContextMenu(cell.row, { x: event.pageX, y: event.pageY });
                });
                this.appendChild(this.tree);
                this.tree.onRemoved(function (event) { return _this.tree.getGrid().unsubscribeOnClick(_this.clickListener); });
            };
            PageComponentsView.prototype.isDraggable = function () {
                return this.draggable;
            };
            PageComponentsView.prototype.setDraggable = function (draggable) {
                var _this = this;
                var body = api.dom.Body.get();
                if (!this.draggable && draggable) {
                    var lastPos;
                    if (!this.mouseDownListener) {
                        this.mouseDownListener = function (event) {
                            if (PageComponentsView.debug) {
                                console.log('mouse down', _this.mouseDown, event);
                            }
                            if (!_this.mouseDown && event.buttons == 1) {
                                event.preventDefault();
                                event.stopPropagation();
                                _this.mouseDown = true;
                                lastPos = {
                                    x: event.clientX,
                                    y: event.clientY
                                };
                            }
                        };
                    }
                    if (!this.mouseUpListener) {
                        this.mouseUpListener = function (event) {
                            if (PageComponentsView.debug) {
                                console.log('mouse up', _this.mouseDown, event);
                            }
                            if (_this.mouseDown) {
                                if (event) {
                                    event.preventDefault();
                                    event.stopPropagation();
                                }
                                _this.mouseDown = false;
                            }
                        };
                    }
                    if (!this.mouseMoveListener) {
                        this.mouseMoveListener = function (event) {
                            if (_this.mouseDown) {
                                if (event.buttons != 1) {
                                    _this.mouseUpListener();
                                    return;
                                }
                                event.preventDefault();
                                event.stopPropagation();
                                var el = _this.getEl(), newPos = {
                                    x: event.clientX,
                                    y: event.clientY
                                }, offset = el.getOffset(), newOffset = {
                                    top: offset.top + newPos.y - lastPos.y,
                                    left: offset.left + newPos.x - lastPos.x
                                };
                                _this.constrainToParent(newOffset);
                                lastPos = newPos;
                                _this.hideContextMenu();
                            }
                        };
                    }
                    this.header.onMouseDown(this.mouseDownListener);
                    body.onMouseUp(this.mouseUpListener);
                    body.onMouseMove(this.mouseMoveListener);
                }
                else if (this.draggable && !draggable) {
                    this.header.unMouseDown(this.mouseDownListener);
                    body.unMouseUp(this.mouseUpListener);
                    body.unMouseMove(this.mouseMoveListener);
                }
                this.toggleClass('draggable', draggable);
                this.draggable = draggable;
                return this;
            };
            PageComponentsView.prototype.constrainToParent = function (offset) {
                var parentEl, parentOffset, el = this.getEl(), offset = offset || el.getOffset();
                if (this.getParentElement()) {
                    parentEl = this.getParentElement().getEl();
                    parentOffset = parentEl.getOffset();
                }
                else {
                    parentEl = api.dom.WindowDOM.get();
                    parentOffset = {
                        top: 0,
                        left: 0
                    };
                }
                el.setMaxHeightPx(parentEl.getHeight());
                el.setOffset({
                    top: Math.max(parentOffset.top, Math.min(offset.top, parentOffset.top + parentEl.getHeight() - el.getHeightWithBorder())),
                    left: Math.max(parentOffset.left, Math.min(offset.left, parentOffset.left + parentEl.getWidth() - el.getWidthWithBorder()))
                });
            };
            PageComponentsView.prototype.isFloating = function () {
                return this.floating;
            };
            PageComponentsView.prototype.setFloating = function (floating) {
                this.toggleClass('floating', floating);
                this.floating = floating;
                return this;
            };
            PageComponentsView.prototype.isModal = function () {
                return this.modal;
            };
            PageComponentsView.prototype.setModal = function (modal) {
                this.toggleClass('modal', modal);
                if (this.tree) {
                    this.tree.getGrid().resizeCanvas();
                }
                this.modal = modal;
                return this;
            };
            PageComponentsView.prototype.scrollToItem = function (dataId) {
                var node = this.tree.getRoot().getCurrentRoot().findNode(dataId);
                if (node) {
                    node.getData().scrollComponentIntoView();
                    this.tree.scrollToRow(this.tree.getGrid().getDataView().getRowById(node.getId()));
                }
            };
            PageComponentsView.prototype.isMenuIconClicked = function (cellNumber) {
                return cellNumber == 1;
            };
            PageComponentsView.prototype.showContextMenu = function (row, clickPosition) {
                var _this = this;
                var contextMenuActions = this.tree.getGrid().getDataView().getItem(row).getData().getContextMenuActions();
                if (!this.contextMenu) {
                    this.contextMenu = new api.liveedit.ItemViewContextMenu(null, contextMenuActions);
                }
                else {
                    this.contextMenu.setActions(contextMenuActions);
                }
                this.contextMenu.getMenu().onBeforeAction(function (action) {
                    _this.pageView.setDisabledContextMenu(true);
                });
                this.contextMenu.getMenu().onAfterAction(function (action) {
                    setTimeout(function () {
                        _this.pageView.setDisabledContextMenu(false);
                        _this.contextMenu.getMenu().clearActionListeners();
                    }, 500);
                });
                var x = clickPosition.x;
                var y = clickPosition.y;
                this.contextMenu.showAt(x, y, false);
                this.setMenuOpenStyleOnMenuIcon(row);
            };
            PageComponentsView.prototype.setMenuOpenStyleOnMenuIcon = function (row) {
                var stylesHash = {};
                stylesHash[row] = { menu: "menu-open" };
                this.tree.getGrid().setCellCssStyles("menu-open", stylesHash);
            };
            PageComponentsView.prototype.removeMenuOpenStyleFromMenuIcon = function () {
                this.tree.getGrid().removeCellCssStyles("menu-open");
            };
            PageComponentsView.prototype.hideContextMenu = function () {
                if (this.contextMenu && this.contextMenu.isVisible()) {
                    this.contextMenu.hide();
                    this.removeMenuOpenStyleFromMenuIcon();
                }
            };
            PageComponentsView.prototype.sameRowClicked = function (clickedRow) {
                var currentlySelectedRow = this.tree.getGrid().getSelectedRows()[0];
                return clickedRow == currentlySelectedRow;
            };
            PageComponentsView.debug = false;
            return PageComponentsView;
        })(api.dom.DivEl);
        wizard.PageComponentsView = PageComponentsView;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var app;
(function (app) {
    var wizard;
    (function (wizard) {
        var GridDragHandler = api.ui.grid.GridDragHandler;
        var RegionView = api.liveedit.RegionView;
        var PageView = api.liveedit.PageView;
        var LayoutComponentView = api.liveedit.layout.LayoutComponentView;
        var DragHelper = api.ui.DragHelper;
        var ElementHelper = api.dom.ElementHelper;
        var PageComponentsGridDragHandler = (function (_super) {
            __extends(PageComponentsGridDragHandler, _super);
            function PageComponentsGridDragHandler() {
                _super.apply(this, arguments);
            }
            PageComponentsGridDragHandler.prototype.handleDragInit = function (e, dd) {
                var row = this.getRowByTarget(new ElementHelper(e.target)), nodes = this.contentGrid.getRoot().getCurrentRoot().treeToList(), draggedNode = nodes[row.getSiblingIndex()];
                if (draggedNode.getData().isDraggableView()) {
                    e.stopImmediatePropagation();
                }
            };
            PageComponentsGridDragHandler.prototype.handleDragStart = function () {
                _super.prototype.handleDragStart.call(this);
                this.getDraggableItem().getChildren().forEach(function (childEl) {
                    childEl.removeClass("selected");
                });
                DragHelper.get().setDropAllowed(true);
                api.dom.Body.get().appendChild(DragHelper.get());
                api.dom.Body.get().onMouseMove(this.handleHelperMove);
                this.contentGrid.onMouseLeave(this.handleMouseLeave);
                this.contentGrid.onMouseEnter(this.handleMouseEnter);
            };
            PageComponentsGridDragHandler.prototype.handleDragEnd = function (event, data) {
                api.dom.Body.get().unMouseMove(this.handleHelperMove);
                api.dom.Body.get().removeChild(DragHelper.get());
                this.contentGrid.unMouseLeave(this.handleMouseLeave);
                this.contentGrid.unMouseEnter(this.handleMouseEnter);
                _super.prototype.handleDragEnd.call(this, event, data);
            };
            PageComponentsGridDragHandler.prototype.handleBeforeMoveRows = function (event, data) {
                var dataList = this.contentGrid.getRoot().getCurrentRoot().treeToList();
                var draggableRow = data.rows[0], insertBefore = data.insertBefore;
                var insertPosition = (draggableRow > insertBefore) ? insertBefore : insertBefore + 1;
                this.updateDragHelperStatus(draggableRow, insertPosition, dataList);
                if (DragHelper.get().isDropAllowed()) {
                    _super.prototype.handleBeforeMoveRows.call(this, event, data);
                }
                return true;
            };
            PageComponentsGridDragHandler.prototype.handleMoveRows = function (event, args) {
                if (DragHelper.get().isDropAllowed()) {
                    _super.prototype.handleMoveRows.call(this, event, args);
                }
            };
            PageComponentsGridDragHandler.prototype.makeMovementInNodes = function (draggableRow, insertBefore) {
                var root = this.contentGrid.getRoot().getCurrentRoot();
                var dataList = root.treeToList();
                var item = dataList.slice(draggableRow, draggableRow + 1)[0];
                var insertPosition = (draggableRow > insertBefore) ? insertBefore : insertBefore + 1;
                this.moveIntoNewParent(item, insertPosition, dataList);
                dataList.splice(dataList.indexOf(item), 1);
                dataList.splice(insertBefore, 0, item);
                return dataList.indexOf(item);
            };
            PageComponentsGridDragHandler.prototype.getModelId = function (model) {
                return model ? model.getItemId() : null;
            };
            PageComponentsGridDragHandler.prototype.handleMovements = function (rowDataId, moveBeforeRowDataId) {
                return;
            };
            PageComponentsGridDragHandler.prototype.moveIntoNewParent = function (item, insertBefore, data) {
                var insertData = this.getParentPosition(insertBefore, data), regionPosition = insertData.parentPosition, insertIndex = insertData.insertIndex;
                var newParent = data[regionPosition];
                if (newParent == item.getParent() && data.indexOf(item) < insertBefore) {
                    insertIndex--;
                }
                this.contentGrid.deselectAll();
                item.getData().deselect();
                item.getData().moveToRegion(newParent.getData(), insertIndex);
                item.getData().select();
                this.contentGrid.refresh();
                return data[regionPosition];
            };
            PageComponentsGridDragHandler.prototype.updateDragHelperStatus = function (draggableRow, insertBeforePos, data) {
                var parentPosition = this.getParentPosition(insertBeforePos, data).parentPosition;
                var parentComponentNode = data[parentPosition], parentComponentView = parentComponentNode.getData(), draggableComponentView = data[draggableRow].getData();
                if (parentComponentView) {
                    if (api.ObjectHelper.iFrameSafeInstanceOf(draggableComponentView, LayoutComponentView)) {
                        if (parentComponentView.getName() != "main") {
                            DragHelper.get().setDropAllowed(false);
                            return;
                        }
                    }
                    if (api.ObjectHelper.iFrameSafeInstanceOf(parentComponentView, RegionView)) {
                        DragHelper.get().setDropAllowed(true);
                        var draggableItem = this.getDraggableItem();
                        if (draggableItem) {
                            this.updateDraggableItemPosition(draggableItem, parentComponentNode.calcLevel());
                        }
                        return;
                    }
                }
                DragHelper.get().setDropAllowed(false);
            };
            PageComponentsGridDragHandler.prototype.updateDraggableItemPosition = function (draggableItem, parentLevel) {
                var margin = parentLevel * api.ui.treegrid.TreeGrid.LEVEL_STEP_INDENT;
                var nodes = draggableItem.getEl().getElementsByClassName("toggle icon");
                if (nodes.length == 1) {
                    nodes[0].setMarginLeft(margin + "px");
                }
            };
            PageComponentsGridDragHandler.prototype.getParentPosition = function (insertBeforePos, data) {
                var parentPosition = insertBeforePos, insertIndex = 0;
                if (!data[insertBeforePos - 1]) {
                    return { parentPosition: 0, insertIndex: 0 };
                }
                var calcLevel = data[parentPosition - 1].calcLevel();
                var isFirstChildPosition = (data[insertBeforePos] ? data[insertBeforePos - 1].calcLevel() < data[insertBeforePos].calcLevel() : false) || (api.ObjectHelper.iFrameSafeInstanceOf(data[insertBeforePos - 1].getData(), RegionView));
                do {
                    parentPosition = parentPosition <= 0 ? 0 : parentPosition - 1;
                    var parentComponentNode = data[parentPosition], parentComponentView = parentComponentNode.getData();
                    if (parentComponentNode.calcLevel() == calcLevel && !isFirstChildPosition) {
                        insertIndex++;
                    }
                } while (!(api.ObjectHelper.iFrameSafeInstanceOf(parentComponentView, RegionView) || (api.ObjectHelper.iFrameSafeInstanceOf(parentComponentView, LayoutComponentView) && (parentComponentNode.isExpanded() && parentComponentNode.getChildren().length > 0)) || api.ObjectHelper.iFrameSafeInstanceOf(parentComponentView, PageView)) || (parentComponentNode.calcLevel() >= calcLevel && !isFirstChildPosition));
                return { parentPosition: parentPosition, insertIndex: insertIndex };
            };
            PageComponentsGridDragHandler.prototype.getRowByTarget = function (el) {
                return (el && el.hasClass("slick-row")) ? el : this.getRowByTarget(el.getParent());
            };
            PageComponentsGridDragHandler.prototype.handleMouseLeave = function () {
                DragHelper.get().setVisible(false);
            };
            PageComponentsGridDragHandler.prototype.handleMouseEnter = function () {
                DragHelper.get().setVisible(true);
            };
            PageComponentsGridDragHandler.prototype.handleHelperMove = function (event) {
                DragHelper.get().getEl().setLeftPx(event.pageX);
                DragHelper.get().getEl().setTopPx(event.pageY);
            };
            return PageComponentsGridDragHandler;
        })(GridDragHandler);
        wizard.PageComponentsGridDragHandler = PageComponentsGridDragHandler;
    })(wizard = app.wizard || (app.wizard = {}));
})(app || (app = {}));
var components;
(function (components) {
    components.detailPanel;
})(components || (components = {}));
function initToolTip() {
    var ID = "tooltip", CLS_ON = "tooltip_ON", FOLLOW = true, DATA = "_tooltip", OFFSET_X = 0, OFFSET_Y = 20, pageX = 0, pageY = 0, showAt = function (e) {
        var ntop = pageY + OFFSET_Y, nleft = pageX + OFFSET_X;
        var tooltipText = api.util.StringHelper.escapeHtml(wemjq(e.target).data(DATA));
        var tooltipWidth = tooltipText.length * 7.5;
        var windowWidth = wemjq(window).width();
        if (nleft + tooltipWidth >= windowWidth) {
            nleft = windowWidth - tooltipWidth;
        }
        wemjq("#" + ID).html(tooltipText).css({
            position: "absolute",
            top: ntop,
            left: nleft
        }).show();
    };
    wemjq(document).on("mouseenter", "*[title]", function (e) {
        wemjq(this).data(DATA, wemjq(this).attr("title"));
        wemjq(this).removeAttr("title").addClass(CLS_ON);
        wemjq("<div id='" + ID + "' />").appendTo("body");
        if (e.pageX) {
            pageX = e.pageX;
        }
        if (e.pageY) {
            pageY = e.pageY;
        }
        showAt(e);
    });
    wemjq(document).on("mouseleave", "." + CLS_ON, function (e) {
        if (wemjq(this).data(DATA)) {
            wemjq(this).attr("title", wemjq(this).data(DATA));
        }
        wemjq(this).removeClass(CLS_ON);
        wemjq("#" + ID).remove();
    });
    if (FOLLOW) {
        wemjq(document).on("mousemove", "." + CLS_ON, showAt);
    }
}
function startApplication() {
    var application = api.app.Application.getApplication();
    var body = api.dom.Body.get();
    var appBar = new api.app.bar.AppBar(application);
    var appPanel = new app.ContentAppPanel(appBar, application.getPath());
    body.appendChild(appBar);
    body.appendChild(appPanel);
    var contentDeleteDialog = new app.remove.ContentDeleteDialog();
    app.browse.ContentDeletePromptEvent.on(function (event) {
        contentDeleteDialog.setContentToDelete(event.getModels());
        contentDeleteDialog.open();
    });
    app.browse.ContentPublishPromptEvent.on(function (event) {
        var contentPublishDialog = new app.publish.ContentPublishDialog();
        contentPublishDialog.setSelectedContents(event.getModels());
        contentPublishDialog.initAndOpen();
    });
    var newContentDialog = new app.create.NewContentDialog();
    app.browse.ShowNewContentDialogEvent.on(function (event) {
        var parentContent = event.getParentContent();
        if (parentContent != null) {
            new api.content.GetContentByIdRequest(parentContent.getContentId()).sendAndParse().then(function (newParentContent) {
                if (parentContent.hasParent() && parentContent.getType().isTemplateFolder()) {
                    new api.content.GetContentByPathRequest(parentContent.getPath().getParentPath()).sendAndParse().then(function (grandParent) {
                        newContentDialog.setParentContent(newParentContent);
                        newContentDialog.open();
                    }).catch(function (reason) {
                        api.DefaultErrorHandler.handle(reason);
                    }).done();
                }
                else {
                    newContentDialog.setParentContent(newParentContent);
                    newContentDialog.open();
                }
            }).catch(function (reason) {
                api.DefaultErrorHandler.handle(reason);
            }).done();
        }
        else {
            newContentDialog.setParentContent(null);
            newContentDialog.open();
        }
    });
    initToolTip();
    api.util.AppHelper.preventDragRedirect();
    var sortDialog = new app.browse.SortContentDialog();
    var moveDialog = new app.browse.MoveContentDialog();
    var editPermissionsDialog = new app.wizard.EditPermissionsDialog();
    application.setLoaded(true);
    window.onmessage = function (e) {
        if (e.data.appLauncherEvent) {
            var eventType = api.app.AppLauncherEventType[e.data.appLauncherEvent];
            if (eventType == 0 /* Show */) {
                appPanel.activateCurrentKeyBindings();
            }
        }
    };
}
//# sourceMappingURL=_all.js.map